/**
 * @file cl.c
 *
 * This file contains engine related  connectivity  module functions.
 *                                                                              
 * Copyright (c) 2005 Nokia. All rights reserved.
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#if CL_USE

#include<stdio.h>
#include<glib.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<unistd.h>
#include <log-functions.h>
#include "cl.h"

volatile gint cb_status = FALSE;	/* status updated by ic library */

static email_cl_wa cl_wa;
static gboolean cl_iap_cb(struct iap_event_t *event, void *arg);
static gint cl_validate_iap(const gchar * iap_name);
static IapInfo *iap_info_new(const gchar * iap_name);

/**
    This is initialization function for connectivity layer 
    for email Application
    @param none
    @return CL_OK on success or CL_INIT_FAILURE on fail 
*/
gint cl_init(void *cb_func)
{
	DBusError derror = { 0 };
	email_cl_wa *cl_wap = NULL;

	g_type_init();
	memset(&cl_wa, '\0', sizeof(email_cl_wa));
	cl_wap = &cl_wa;
	/* registers call back */
	if (osso_iap_cb((osso_iap_cb_t) cl_iap_cb) != OSSO_OK) {
		osso_log(LOG_ERR, "Iap Call back Register failed \n");
		return CL_INIT_FAILURE;
	}

	dbus_error_init(&derror);
	cl_wap->gmainctx = g_main_context_default();
	cl_wap->loop = g_main_loop_new(cl_wap->gmainctx, FALSE);
	cl_wap->dbusconn = dbus_bus_get(DBUS_BUS_SYSTEM, &derror);
	cl_wap->conn_type = OSSO_IAP_REQUESTED_CONNECT;
	cl_wap->cancel_mails_cb = cb_func;

	if (cl_wap->dbusconn == NULL) {
		osso_log(LOG_ERR, "dbus connection failure(%s)\n", derror.message);
		dbus_error_free(&derror);
		cl_uninit();
		return CL_INIT_FAILURE;
	}
	dbus_connection_setup_with_g_main(cl_wap->dbusconn, cl_wap->gmainctx);
	osso_log(LOG_INFO, "cl init success\n");
	return CL_OK;
}

/**
    This function is called to uninitialise connectivity context
    @param none
    @return none 
*/

void cl_uninit(void)
{
	if (cl_wa.dbusconn) {
		dbus_connection_disconnect(cl_wa.dbusconn);
		dbus_connection_unref(cl_wa.dbusconn);
	}
	g_main_loop_unref(cl_wa.loop);
	g_main_context_unref(cl_wa.gmainctx);
}

/**
   This function is called by the osso-ic lib for event notification

   @param event iap_event information
   @param arg additional info passed by the osso ic lib
   @return TRUE or FALSE
*/

static gboolean cl_iap_cb(struct iap_event_t *event, void *arg)
{
	email_cl_wa *cl_wap = &cl_wa;
	cb_status = TRUE;
	osso_log(LOG_INFO, "iap_name:%s  status = %d\n", event->iap_name, event->type);
	switch (event->type) {
	case OSSO_IAP_CONNECTED:
		if (cl_wap->cur_iap == NULL) {
			if (event->iap_name != NULL) {
				cl_wap->cur_iap = g_strdup(event->iap_name);
			}
		} else {
			if (event->iap_name != NULL) {
				if (strcmp(cl_wap->cur_iap, event->iap_name) != 0) {
					g_free(cl_wap->cur_iap);
					cl_wap->cur_iap = g_strdup(event->iap_name);
				}
			}
		}
		break;
	case OSSO_IAP_DISCONNECTED:
	case OSSO_IAP_ERROR:
		if (cl_wap->cur_iap != NULL) {
			if (cl_wap->send_recv_cancel_skip == FALSE) {
				if (cl_wap->cancel_mails_cb) {
					cl_wap->cancel_mails_cb();
				}
			}
			g_free(cl_wap->cur_iap);
			cl_wap->cur_iap = NULL;
		}
		break;
	default:
		break;
	}
	cl_wap->status = event->type;
	cl_wap->send_recv_cancel_skip = FALSE;
	return TRUE;
}

/**
   This function is called to connect specific iap 
   @param iap_name iap name
   @return CL_OK on success or returns the cl layer error code 
*/
gint cl_iap_connect(const gchar * iap_name)
{
	gint rv = CL_OK;
	gchar *iap_local = NULL;

	osso_log(LOG_INFO, "cl_iap_connect starts here \n");

	if (iap_name != NULL) {
		/* requested iap is currently connected iap ???  */
		if ((cl_wa.cur_iap) && (strcmp(iap_name, cl_wa.cur_iap) == 0)) {
			cl_iap_set_type(FALSE);
			return CL_OK;
		}
      /** check for iap existance*/
		if (cl_validate_iap(iap_name) != CL_OK) {
			osso_log(LOG_ERR, "Invalid iap name = %s \n", iap_name);
			cl_iap_set_type(FALSE);
			return CL_INVALID_IAP_NAME;
		}
	}
	if (iap_name == NULL) {
		iap_local = g_strdup(OSSO_IAP_ANY);
	} else {
		iap_local = g_strdup(iap_name);
	}

	/* connects to the iap */
	osso_log(LOG_INFO, "osso_iap_connect(%s) Request send \n", iap_local);
	rv = osso_iap_connect(iap_local, cl_iap_get_type(), &cl_wa);
	if (rv != OSSO_OK) {
		osso_log(LOG_ERR, "osso_iap_connect failure \n");
		g_free(cl_wa.cur_iap);
		cl_wa.cur_iap = NULL;
		rv = CL_ERROR_IAP_CONNECT;
		goto endshere;
	}

	cl_wa.status = 0;
	while (cl_wa.status == 0) {
		g_main_context_iteration(NULL, FALSE);
	}
	if (cl_wa.status == OSSO_IAP_CONNECTED) {
		osso_log(LOG_INFO, "%s connected\n", iap_local);
		rv = CL_OK;
	} else {
		osso_log(LOG_INFO, "%s not connected\n", iap_local);
		rv = CL_ERROR_IAP_CONNECT;
	}
      endshere:
	cl_iap_set_type(FALSE);
	g_free(iap_local);
	return rv;
}

/**
   This function is called to disconnect the specific iap
   @param iap_name iap name
   @return success or error code
*/
gint cl_iap_disconnect(const gchar * iap_name)
{
	gint rv = CL_OK;
	gchar *iap_local = NULL;

	if (iap_name != NULL) {
       /** check for iap existance*/
		if (cl_validate_iap(iap_name) != CL_OK) {
			osso_log(LOG_ERR, "Invalid iap name = %s \n", iap_name);
			return CL_INVALID_IAP_NAME;
		}
	}
	if (iap_name == NULL) {
		iap_local = g_strdup(OSSO_IAP_ANY);
	} else {
		iap_local = g_strdup(iap_name);
	}
	cl_wa.send_recv_cancel_skip = TRUE;

	rv = osso_iap_disconnect(iap_local, &cl_wa);
	if (rv != OSSO_OK) {
		osso_log(LOG_ERR, "osso_iap_disconnect failure \n");
		rv = CL_ERROR_IAP_DISCONNECT;
		goto endshere;
	}
	cl_wa.status = 0;
	while (cl_wa.status == 0) {
		g_main_context_iteration(NULL, FALSE);
	}
	if (cl_wa.status == OSSO_IAP_DISCONNECTED) {
		rv = CL_OK;
	} else {
		rv = CL_ERROR_IAP_DISCONNECT;
	}
      endshere:
	g_free(iap_local);
	return rv;
}

/**
   This function creates a new IapInfo instance with the passed values
   @param iap_name iap name
   @param account_name account name
   @param iap_type iap type
   @return pointer IapInfo or NULL
*/

static IapInfo *iap_info_new(const gchar * iap_name)
{
	IapInfo *iap_info = NULL;
	if (iap_name == NULL) {
		osso_log(LOG_INFO, "invalid parameter \n");
		return NULL;
	}
	iap_info = g_new0(IapInfo, 1);
	if (iap_info != NULL) {
		iap_info->iap_name = g_strdup(iap_name);
		iap_info->iap_type = 0;
		iap_info->iap_status = 0;
	}
	return iap_info;
}

/**
   This function is used to free the memory
   @param iap_info iap information
   @return CL_OK on success or return error code 
*/
gint cl_iap_info_free(IapInfo * iap_info)
{
	if (iap_info == NULL) {
		return CL_INVALID_PARAM;
	}
	g_free(iap_info->iap_name);
	g_free(iap_info);
	return CL_OK;
}

/**
   This function is used to free the memory
   @param iap_info iap information
   @return CL_OK on success or return error code 
*/
gint cl_iap_info_list_free(GSList * iap_info_list)
{
	/* GSList *info_list = iap_info_list; */

	if (iap_info_list == NULL) {
		return CL_INVALID_PARAM;
	}
	while (iap_info_list) {
		cl_iap_info_free(iap_info_list->data);
		iap_info_list = g_slist_remove(iap_info_list, iap_info_list->data);
	}
	return CL_OK;
}

/**
   This function return iap_list as an out parameter
   @param iap_list
   @return return status
*/

gint cl_get_iap_info_list(GSList ** iap_list)
{
	GSList *cur = NULL;
	IapInfo *iap_info = NULL;
	GSList *iap_name_list = NULL;
	GSList *iap_info_list = NULL;
	gchar *iap = NULL;
	if (iap_list == NULL) {
		return CL_INVALID_PARAM;
	}
	g_type_init();
    /** get the configurred iaps **/
	iap_name_list = osso_iap_get_configured_iaps();
    /** prepare the list of IapInfo */
	for (cur = iap_name_list; cur != NULL; cur = cur->next) {
		if (cur->data != NULL) {
			iap = (gchar *) cur->data;
			iap_info = iap_info_new(iap);
			if (iap_info != NULL) {
				iap_info_list = g_slist_append(iap_info_list, iap_info);
			}
			g_free(cur->data);
		}
	}
	g_slist_free(iap_name_list);
	*iap_list = iap_info_list;
	return CL_OK;
}

/**
  This function is called to validate the iap name from the iap list
  returned by osso-ic library.
  
  @param iap_name iap name
  @return on success returns CL_OK or returns error code
*/
static gint cl_validate_iap(const gchar * iap_name)
{
	GSList *iap_lt = NULL;
	GSList *cur = NULL;
	gchar *name = NULL;

	if (iap_name == NULL) {
		return CL_INVALID_PARAM;
	}
	if (strcmp(iap_name, OSSO_IAP_ANY) == 0) {
		return CL_OK;
	}
	/* get configured iaps in the device */
	g_type_init();
	iap_lt = osso_iap_get_configured_iaps();
	if (iap_lt == NULL) {
		return CL_ERROR_GET_LIST;
	}

	/* check for the iap */
	for (cur = iap_lt; cur != NULL; cur = cur->next) {
		name = (gchar *) cur->data;
		if (name == NULL) {
			continue;
		}
		if (strcmp((gchar *) iap_name, name) == 0) {
			slist_free_strings(iap_lt);
			g_slist_free(iap_lt);
			return CL_OK;
		}
	}
	slist_free_strings(iap_lt);
	g_slist_free(iap_lt);
	return CL_INVALID_IAP;
}

/**
  This function is called to set the connection type. 
  @param type for timed out connection set to TRUE 
  @return nothing 
*/
void cl_iap_set_type(gboolean type)
{
	if (type == TRUE) {
		cl_wa.conn_type = OSSO_IAP_TIMED_CONNECT;
	} else {
		cl_wa.conn_type = OSSO_IAP_REQUESTED_CONNECT;
	}
}

/**
  This function is called to get the connection type. 
  @param none 
  @return nothing 
*/

gint cl_iap_get_type(void)
{
	return cl_wa.conn_type;
}

/**
  This function is used to get the status of current iap 
  @param option  TRUE means set status and FALSE means get the status
  @param val  to be set  
  @return nothing 
*/

gboolean cl_current_iap_status(gboolean option, gint val)
{
	if (option == FALSE) {	/* get the iap status */
		return (cl_wa.status == OSSO_IAP_CONNECTED ? TRUE : FALSE);
	}
	if (cl_wa.status == 0) {	/* iap status is in connecting mode */
		cl_wa.status = val;
	}
	return cl_wa.status;
}

void cl_discon_cur_iap(void)
{
	if (cl_wa.cur_iap != NULL) {
		osso_iap_disconnect(CL_IAP_ANY, &cl_wa);
		g_free(cl_wa.cur_iap);
	}
}

/**
  This function is used to disconnect at the time of connecting 
  @param void 
  @return nothing 
*/
void cl_cancel_iap_conn(void)
{
	osso_log(LOG_ERR, "%s : Can IAP connection\n", __FUNCTION__);
	if (cl_wa.status == 0) {
		cl_wa.status = CL_CONNECTION_CANCEL_REQUEST;
		cl_wa.send_recv_cancel_skip = FALSE;
		osso_iap_disconnect(CL_IAP_ANY, &cl_wa);
	}
	return;
}

/**
  This function is returns the current activ iap 
  @param void 
  @return the current iap name 
*/
const gchar *cl_active_iap(void)
{
	return (cl_wa.cur_iap);
}

#endif				/* end of CL_USE */
