/**
 @file scheduling.c
 
 This file contains scheduling related functions.
 
 Copyright (c) 2005 Nokia. All rights reserved.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  USA
*/


#include "scheduling.h"
#include "al.h"
#include "dbus_utils_def.h"
#include "account.h"
#include "email_gconf.h"

#if CL_USE
#include "cl.h"
#endif


static guint timer_id = 0;
static guint sch_notify_id = 0;
static void osso_mgr_schdg_remove_timer(void);

/**
This function is called whenever timer for scheduling expires

@return TRUE to rerun the timer
        FALSE to stop the timer.
*/
static gboolean osso_mgr_schdg_send_recv(gpointer data)
{
	GSList *tmp_acc_list = NULL;
	GSList *account_list = NULL;

	osso_log(LOG_INFO, "[%s] starts", __FUNCTION__);

	/*Get all the configured accounts */
	al_get_account_list(&account_list);
	if (!account_list) {
		osso_log(LOG_INFO, " Account list is NULL!\n");
		return TRUE;
	}

	tmp_acc_list = account_list;
	osso_email_gconf_set_sendrecv_status(EMAIL_GCONF_SENDRECV_STATUS, SEND_RECV_PROGRESS);
	/*send and receive mails for each account */
	while (tmp_acc_list != NULL) {
#if CL_USE
		cl_iap_set_type(TRUE);	/* set for timed out connection */
#endif
		al_send_recv_mail(tmp_acc_list->data, TRUE);
#if CL_USE
		cl_iap_set_type(FALSE);
#endif
		tmp_acc_list = tmp_acc_list->next;
	}

	osso_email_gconf_set_sendrecv_status(EMAIL_GCONF_SENDRECV_STATUS, SEND_RECV_DONE);

	/*free the accountlist */
	g_slist_foreach(account_list, (GFunc) g_free, NULL);
	g_slist_free(account_list);

	return TRUE;
}

/**
This function is called to initialize  scheduling process.

@return void.
*/
void osso_mgr_schdg_initializescheduler(void)
{
	gint scheduling_interval = 0;
	gint time_period = 0;

	/*Check if there are any configured account.If not return without
	   starting the  timer
	 */
	if (!is_account_list_valid()) {
		osso_log(LOG_DEBUG, " Account list is NULL!\n");
		return;
	}

	/*Get the timer interval value if scheduling is on */
	if (!osso_email_gconf_get_int(EMAIL_SCHEDULING_INTERVAL, &scheduling_interval)) {
		osso_log(LOG_DEBUG, "error getting the value");
		return;
	}
	if(!scheduling_interval){
		osso_log(LOG_DEBUG, "Auto retrieve is not set");
		return;
	}

	/*Start the timer for scheduling with the specified interval time */
	time_period = scheduling_interval * SECS_IN_MIN * MILISEC;
	timer_id = g_timeout_add(time_period, osso_mgr_schdg_send_recv, NULL);

	return;
}

/**
 This function is used to uninitialise scheduling.
 @return void.
*/
void osso_mgr_schdg_uninit(void)
{
	GConfClient *gc = NULL;

	osso_mgr_schdg_remove_timer();
	gc = gconf_client_get_default();
	gconf_client_notify_remove(gc, sch_notify_id);

	return;
}

/**
This function is called to change scheduling process if any modification 
is made in account management for e.g deletion of all accounts/creation of first new account
or in scheduling settings through UI.

@return void.
*/
void osso_mgr_schdg_update_scheduling()
{
	/*Remove any previous timer */
	osso_mgr_schdg_remove_timer();

	/*Reinitialises the scheduler */
	osso_mgr_schdg_initializescheduler();

	return;
}

/**
 This function is used to check whether scheduling is in progress.
 
 @return void
*/
gboolean osso_mgr_schdg_has_task(void)
{
	if (timer_id < 1) {
		return FALSE;
	}
	return TRUE;

}

/**
 This is a gconf notifier function.Which is called when ever the scheduling gconf
 values change.
 
 @param client is GConfClient pointer.
 @param id is the 

 @return.
*/
static void osso_mgr_schdg_update_scheduling_change(GConfClient * client,
						    guint id, GConfEntry * entry, gpointer data)
{
	/*upadte scheduling for values change */
	osso_mgr_schdg_update_scheduling();

	return;
}

/**
 This function is used to register the gconf client to the 
 scheduling changes.
 
 @return void.
*/
void osso_mgr_schdg_notify_changes(void)
{
	GConfClient *gconf_client = NULL;
	GError *error = NULL;

	gconf_client = gconf_client_get_default();

	/* add gconf dir to watch list */
	gconf_client_add_dir(gconf_client, EMAIL_SCHEDULING_PATH, GCONF_CLIENT_PRELOAD_ONELEVEL,
			     &error);
	if (error != NULL) {
		osso_log(LOG_ERR, "Scheduling status couldn't watch");
		g_error_free(error);
		return;
	}
	/*set the callback function to be called if scheduling value changes */
	sch_notify_id = gconf_client_notify_add(gconf_client, EMAIL_SCHEDULING_PATH,
						osso_mgr_schdg_update_scheduling_change, NULL, NULL,
						NULL);
	return;
}

/**
Remove the scheduling timer in situations where  there are no accounts
@return void
*/
static void osso_mgr_schdg_remove_timer()
{
	if (timer_id > 0) {
		g_source_remove(timer_id);
		timer_id = 0;
	}
	return;
}
