/**
 *  @file email_contacts.c
 *   
 *  This file has functions to create the windows for selecting contacts.
 *  
 *  Copyright (c) 2005 Nokia. All rights reserved.
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *   USA
 *  
 */


#include <libosso-abook/osso-abook.h>
#include <gtk/gtk.h>
#include <string.h>

#include <osso-log.h>

#include "ui_common.h"

#include "contact_utils.h"
#include "check_names.h"


static EBook *book = NULL;
static OssoABookContactModel *contact_model = NULL;
static gboolean addressbook_started = FALSE;

/**
 * This function returns the contact view for already created model
 *
 * @return contact_view of type #GtkWidget
 */


static GtkWidget *get_contact_view_from_model()
{
	GtkWidget *contact_view = NULL;
	if (contact_model)
		contact_view = osso_abook_contact_view_new_basic(contact_model);
	return contact_view;
}

/**
 * This function starts the book view the given search string 
 *
 * @param searchstring of type #gchar
 *
 * @return void 
 */


static void start_book_view_with_search_string(gchar * searchstring)
{
	EBookQuery *full_name_query = NULL;
	EBookView *bookview = NULL;
	GError *error = NULL;

	if (!searchstring)
		return;



	full_name_query =
	    e_book_query_field_test(E_CONTACT_FULL_NAME, E_BOOK_QUERY_BEGINS_WITH, searchstring);

	if (!e_book_get_book_view(book, full_name_query, NULL, -1, &bookview, &error)) {
		if (error) {
			osso_log(LOG_ERR, "%s", error->message);
			g_error_free(error);
			error = NULL;
		}
	}

	e_book_query_unref(full_name_query);
	if (bookview) {
		osso_abook_tree_model_set_book_view(OSSO_ABOOK_TREE_MODEL(contact_model), bookview);
		e_book_view_start(bookview);
	}

}

/**
 *  This function returns contacts list from contact view 
 *  @param contact_view
 *  
 *  @return contacts_list of type #EContact and this list should be freed by calling function
 */


static GList *get_contacts_list_from_contact_view(GtkWidget * contact_view)
{
	GtkWidget *contact_dialog = NULL;
	GList *contacts_list = NULL;
	EBookView *bookview = NULL;
	gint response = -1;

	contact_dialog = osso_abook_select_dialog_new(OSSO_ABOOK_TREE_VIEW(contact_view));
	gtk_window_set_title(GTK_WINDOW(contact_dialog), _("mcen_ti_select_recipients"));
	response = gtk_dialog_run(GTK_DIALOG(contact_dialog));
	if (response == GTK_RESPONSE_OK) {
		contacts_list =
		    osso_abook_contact_view_get_selection(OSSO_ABOOK_CONTACT_VIEW(contact_view));
	}
	bookview = osso_abook_tree_model_get_book_view(OSSO_ABOOK_TREE_MODEL(contact_model));
	e_book_view_stop(bookview);
	g_object_unref(bookview);
	gtk_widget_destroy(contact_dialog);
	return contacts_list;

}

/**
 *  This function returns contacts list from the given search string 
 *  @param contact_view
 *   
 *  @return contacts_list of type #EContact and should be freed by calling function    
 */


static GList *get_matching_contacts_for_string(gchar * searchstring)
{
	GList *contacts_list = NULL;
	GError *error = NULL;
	EBookQuery *full_name_query = NULL;

	if (!searchstring)
		return NULL;

	full_name_query =
	    e_book_query_field_test(E_CONTACT_FULL_NAME, E_BOOK_QUERY_BEGINS_WITH, searchstring);

	if (!e_book_get_contacts(book, full_name_query, &contacts_list, &error)) {
		if (error) {
			osso_log(LOG_ERR, "%s", error->message);
			g_error_free(error);
			error = NULL;
		}
	}
	e_book_query_unref(full_name_query);
	return contacts_list;
}


/**
 *  This function stops the address book 
 */

void stop_addressbook(void)
{
	if (contact_model) {
		g_object_unref(contact_model);
		contact_model = NULL;
	}
	if (book) {
		/*g_object_unref(book); */
		book = NULL;
	}

	addressbook_started = FALSE;
}


/**
 *  This function opens the address book 
 *  @return gboolean
 */



gboolean open_addressbook(void)
{
	GError *error = NULL;


	if (addressbook_started)
		return TRUE;
	if (!book) {
		book = e_book_new_system_addressbook(&error);
		if (!book) {
			if (error) {
				osso_log(LOG_ERR, "Error from addressbook %s\n", error->message);
				g_error_free(error);
				error = NULL;
			}
			return FALSE;
		}
	}
	if (!e_book_open(book, FALSE, &error)) {
		if (error) {
			g_object_unref(book);
			book = NULL;
			osso_log(LOG_ERR, "Error from addressbook %s\n", error->message);
			g_error_free(error);
			error = NULL;
			return FALSE;
		}
	}
	addressbook_started = TRUE;
	contact_model = osso_abook_contact_model_new();

	return TRUE;

}


/**
 *  This public function returns the resolved contacts to the editor for the given string
 *
 *  @param  searchstring  string to be searched for
 *  @param  econtact_id   Out param for storing UID of searched EContact
 *  @return comma separated string containing one or more resolved contacts
 */


gchar *get_resolved_contacts(gchar * searchstring, gchar ** econtact_id)
{
	GList *contacts_list = NULL;
	GtkWidget *contact_view = NULL;
	EContact *contact = NULL;
	GString *recipients = NULL;
	GSList *recp_node = NULL, *recipient_list = NULL;

	if (!addressbook_started)
		return NULL;

	contacts_list = get_matching_contacts_for_string(searchstring);
	if (!contacts_list) {
		return NULL;
	}

	if (g_list_length(contacts_list) > 1) {
		g_list_free(contacts_list);
		contacts_list = NULL;

		start_book_view_with_search_string(searchstring);
		contact_view = get_contact_view_from_model();
		contacts_list = get_contacts_list_from_contact_view(contact_view);
	}
	if (contacts_list) {
		recipients = g_string_new(NULL);
		contact = (EContact *) contacts_list->data;
		if (contact) {
			recipient_list = get_recipients_for_given_contact(contact);
			for (recp_node = recipient_list; recp_node != NULL;
			     recp_node = recp_node->next) {
				recipients = g_string_append(recipients, recp_node->data);
				if (recp_node->next)
					recipients =
					    g_string_append(recipients, ADDR_SEPERATOR_CHAR);
			}
			if (recipient_list) {
				g_slist_foreach(recipient_list, (GFunc) g_free, NULL);
				g_slist_free(recipient_list);
			}
			/*Pass the E_CONTACT_UID value */
			*econtact_id = (gchar *) e_contact_get(contact, E_CONTACT_UID);
		}
		g_list_free(contacts_list);
	}

	return g_string_free(recipients, FALSE);
}
