/**
 *  @file contact_utils.c
 *   
 *  This file has utilities for contact related operations.
 *  
 *  Copyright (c) 2005 Nokia. All rights reserved.
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *   USA
 *  
 */

#include <glib.h>
#include <libosso-abook/osso-abook.h>

#include "contact_utils.h"
#include "ui_common.h"
#include "ui_interface.h"
#include "email_contacts.h"
#include "accountMgmt_callback.h"

extern MainViewInfo *email_main_view;	/* declared in ui_interface.c */


/**
This function is used to get the formated email id with given name and sur name
in the format "GIVENNAME SURNAME <EMAIL ADDRESS>".
@param current_given_name    to hold the given name
@param current_sur_name      to hold the sur name
@param current_email_id      to hold the email id. 
@return gchar* string to be freed by calling function
*/
static gchar *ui_get_formatted_email_id(gchar * current_given_name,
					gchar * current_sur_name, gchar * current_email_id)
{
	GString *email_id_str = NULL;
	gchar *email_id = NULL;

	email_id_str = g_string_new(NULL);

	if ((current_given_name != NULL) && ((strlen(current_given_name) != 0))
	    && (current_sur_name != NULL) && ((strlen(current_sur_name) != 0))) {
		g_string_printf(email_id_str, "%s %s %c%s%c", current_given_name,
				current_sur_name, L_THAN, current_email_id, G_THAN);
		email_id = g_strdup(email_id_str->str);
	} else if ((current_given_name != NULL) && (strlen(current_given_name) != 0)) {
		g_string_printf(email_id_str, "%s %c%s%c", current_given_name, L_THAN,
				current_email_id, G_THAN);
		email_id = g_strdup(email_id_str->str);
	} else if ((current_sur_name != NULL) && (strlen(current_sur_name) != 0)) {
		g_string_printf(email_id_str, "%s %c%s%c", current_sur_name, L_THAN,
				current_email_id, G_THAN);
		email_id = g_strdup(email_id_str->str);
	}
	g_string_free(email_id_str, TRUE);
	email_id_str = NULL;
	return (email_id);
}

/**
 * This is a helper function used to create & run 'Add e-mail address to contact' dialog.
 * It allows user to enter an e-mail address, and shows appropriate infonote if the
 * entered string is not a valid e-mail address.
 *
 * @param  contact_name  Full name of the contact
 * @return E-mail address string entered by user, to be freed by calling function.
 */
static gchar *run_add_email_addr_to_contact_dlg(const gchar * contact_name)
{
	GtkWidget *add_email_addr_to_contact_dlg = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *cptn_cntrl = NULL;
	GtkWidget *name_label = NULL;
	GtkWidget *email_entry = NULL;
	GtkWidget *parent = NULL;
	gint result = -1;
	gchar *new_email_addr = NULL;
	gboolean run_dialog = TRUE;

	parent = ui_get_top_level_window();

	add_email_addr_to_contact_dlg =
	    gtk_dialog_new_with_buttons(_("mcen_ti_add_email_title"), GTK_WINDOW(parent),
					GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
					_("mcen_bd_dialog_ok"), GTK_RESPONSE_ACCEPT,
					_("mcen_bd_dialog_cancel"), GTK_RESPONSE_REJECT, NULL);
	gtk_dialog_set_has_separator(GTK_DIALOG(add_email_addr_to_contact_dlg), FALSE);
	/*Set app_name & state_save related tags to the window */
	g_object_set_data_full(G_OBJECT(add_email_addr_to_contact_dlg), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(add_email_addr_to_contact_dlg), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);

	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	name_label = gtk_label_new(contact_name);
	gtk_misc_set_alignment(GTK_MISC(name_label), 0, 0);
	cptn_cntrl =
	    hildon_caption_new(size_group, _("mcen_ia_add_email_name"), name_label, NULL,
			       HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(add_email_addr_to_contact_dlg)->vbox), cptn_cntrl,
			   FALSE, FALSE, 0);

	email_entry = gtk_entry_new();
	cptn_cntrl =
	    hildon_caption_new(size_group, _("mcen_fi_add_email_name"), email_entry, NULL,
			       HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(add_email_addr_to_contact_dlg)->vbox), cptn_cntrl,
			   TRUE, TRUE, 0);
	gtk_widget_show_all(add_email_addr_to_contact_dlg);

	while (run_dialog) {
		run_dialog = FALSE;
		gtk_widget_grab_focus(email_entry);
		result = gtk_dialog_run(GTK_DIALOG(add_email_addr_to_contact_dlg));

		if (result == GTK_RESPONSE_ACCEPT) {
			new_email_addr = g_strdup(gtk_entry_get_text(GTK_ENTRY(email_entry)));
			new_email_addr = g_strstrip(new_email_addr);
			if (!ui_accmgmt_email_isvalid(new_email_addr)) {
				gtk_infoprint(GTK_WINDOW(add_email_addr_to_contact_dlg),
					      _("mcen_ib_not_valid_email"));
				run_dialog = TRUE;
				g_free(new_email_addr);
				new_email_addr = NULL;
			}
		}
	}

	gtk_widget_destroy(add_email_addr_to_contact_dlg);

	return new_email_addr;
}

/**
 * This is a helper function used to launch 'Add e-mail address to contact' dialog
 * after showing appropriate notification, when there is no e-mail address defined
 * for a selected contact.
 *
 * @param  given_name  Given name of the contact
 * @param  family_name  Family name of the contact
 * @return E-mail address string entered by user, to be freed by calling function.
 */
static gchar *get_email_addr_from_user(const gchar * given_name, const gchar * family_name)
{
	gboolean add_email_addr = FALSE;
	gchar *notification = NULL;
	gchar *full_name = NULL;
	gchar *email_addr = NULL;

	if (given_name && !family_name)
		full_name = g_strdup(given_name);
	else if (!given_name && family_name)
		full_name = g_strdup(family_name);
	else if (given_name && family_name)
		full_name = g_strdup_printf("%s %s", given_name, family_name);
	full_name = g_strstrip(full_name);

	notification = g_strdup_printf(_("mcen_nc_email_address_not_defined"), full_name);
	add_email_addr = get_user_confirmation(notification, NULL, NULL, CONFIRM_DIALOG);

	if (add_email_addr) {
		email_addr = run_add_email_addr_to_contact_dlg(full_name);
	}

	g_free(full_name);
	g_free(notification);
	return email_addr;
}

/**
 * This is helper function to create & run 'Select e-mail address' dialog, used when
 * multiple e-mail addresses are found for a selected contact. It allows user to select
 * one or more e-mail addresses for that contact.
 *
 * @param  email_addr_list  List of e-mail addresses for that contact
 * @return List of user selected e-mail addresses, to be freed by calling function.
 */
static GSList *select_email_addrs_for_contact(GList * email_addr_list)
{
	GtkWidget *select_email_addr_dlg = NULL;
	GtkWidget *label = NULL, *view = NULL, *scrolledwindow = NULL;
	GtkTreeSelection *selection = NULL;
	GtkCellRenderer *renderer = NULL;
	GtkTreeViewColumn *col = NULL;
	GtkListStore *list_store = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	GList *pathslist = NULL, *node = NULL;
	gint result = -1;
	gchar *email_addr = NULL;
	GSList *selected_email_addr_list = NULL;

	if (!email_addr_list)
		return NULL;

	select_email_addr_dlg =
	    gtk_dialog_new_with_buttons(_("mcen_ti_select_email_title"),
					GTK_WINDOW(ui_get_top_level_window()),
					GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
					_("mcen_bd_dialog_ok"), GTK_RESPONSE_ACCEPT,
					_("mcen_bd_dialog_cancel"), GTK_RESPONSE_REJECT, NULL);
	gtk_dialog_set_has_separator(GTK_DIALOG(select_email_addr_dlg), FALSE);
	/*Set app_name & state_save related tags to the window */
	g_object_set_data_full(G_OBJECT(select_email_addr_dlg), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(select_email_addr_dlg), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);

	label = gtk_label_new(_("mcen_ia_select_email_match_addr"));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(select_email_addr_dlg)->vbox), label, FALSE, FALSE,
			   0);

	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(select_email_addr_dlg)->vbox), scrolledwindow, TRUE,
			   TRUE, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow), GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);

	view = gtk_tree_view_new();
	col = gtk_tree_view_column_new();
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", 0);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), view);

	list_store = gtk_list_store_new(1, G_TYPE_STRING);
	model = GTK_TREE_MODEL(list_store);
	gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);

	for (node = email_addr_list; node != NULL && node->data != NULL; node = node->next) {
		email_addr = g_strstrip(g_strdup(node->data));
		gtk_list_store_append(list_store, &iter);
		gtk_list_store_set(list_store, &iter, 0, email_addr, -1);
		g_free(email_addr);
	}

	gtk_widget_show_all(select_email_addr_dlg);
	result = gtk_dialog_run(GTK_DIALOG(select_email_addr_dlg));

	if (result == GTK_RESPONSE_ACCEPT) {
		pathslist = gtk_tree_selection_get_selected_rows(selection, NULL);
		for (node = pathslist; node != NULL; node = node->next) {
			if (gtk_tree_model_get_iter(model, &iter, (GtkTreePath *) node->data)) {
				gtk_tree_model_get(model, &iter, 0, &email_addr, -1);
				selected_email_addr_list =
				    g_slist_append(selected_email_addr_list, g_strdup(email_addr));
				g_free(email_addr);
			}
		}
	}

	gtk_list_store_clear(list_store);
	gtk_widget_destroy(select_email_addr_dlg);
	return selected_email_addr_list;
}


/**
 * This function returns the resolved recipients for a given EContact.
 * If no e-mail address is defined, it launches 'Add e-mail address to contact'
 * dialog to obtain one. If multiple e-mail addresses are found, it launches
 * 'Select e-mail address' dialog to allow user to select one or more e-mail
 * addresses for that contact.
 *
 * @param  Contact of type #EContact
 * @return List of resolved recipient strings, to be freed by calling function.
 */
GSList *get_recipients_for_given_contact(EContact * contact)
{
	gchar *givenname = NULL;
	gchar *familyname = NULL;
	gchar *nickname = NULL;
	gchar *emailid = NULL;
	GList *list = NULL;
	gchar *formatted_string = NULL;
	gboolean email_not_present = FALSE;
	GSList *formattedlist = NULL, *selected_email_addr_list = NULL, *node = NULL;

	if (!contact) {
		return NULL;
	}

	givenname = (gchar *) e_contact_get_const(contact, E_CONTACT_GIVEN_NAME);
	familyname = (gchar *) e_contact_get_const(contact, E_CONTACT_FAMILY_NAME);
	nickname = (gchar *) e_contact_get_const(contact, E_CONTACT_NICKNAME);
	if (!nickname)
		nickname = DUMMY_VAL;
	list = (GList *) e_contact_get(contact, E_CONTACT_EMAIL);

	if (!list) {
		email_not_present = TRUE;
	}

	if (g_list_length(list) == 1) {
		if (list->data == NULL || g_utf8_strlen(list->data, -1) == 0) {
			email_not_present = TRUE;
		} else {
			emailid = g_strstrip(g_strdup(list->data));
			if (g_utf8_strlen(emailid, -1) == 0) {
				g_free(emailid);
				email_not_present = TRUE;
			}
		}
	}

	/*Launch the 'Add e-mail addr to contact' dialog if required */
	if (email_not_present) {
		if (givenname || familyname)
			emailid = get_email_addr_from_user(givenname, familyname);
		else
			emailid = get_email_addr_from_user(nickname, NULL);

		if (emailid) {
			e_contact_set(contact, E_CONTACT_EMAIL_1, emailid);
			commit_contact(contact);
		}
	}

	if (emailid) {
		if (givenname || familyname)
			formatted_string =
			    ui_get_formatted_email_id(givenname, familyname, emailid);
		else
			formatted_string = g_strdup(emailid);
		formattedlist = g_slist_append(formattedlist, formatted_string);
		g_free(emailid);
	}

	/*Launch the 'Select e-mail address' dialog if required */
	if (g_list_length(list) > 1) {
		selected_email_addr_list = select_email_addrs_for_contact(list);
		for (node = selected_email_addr_list; node != NULL; node = node->next) {
			if (givenname || familyname)
				formatted_string =
				    ui_get_formatted_email_id(givenname, familyname, node->data);
			else
				formatted_string = g_strdup(node->data);
			formattedlist = g_slist_append(formattedlist, formatted_string);
		}
		if (selected_email_addr_list) {
			g_slist_foreach(selected_email_addr_list, (GFunc) g_free, NULL);
			g_slist_free(selected_email_addr_list);
		}
	}

	if (list) {
		g_list_foreach(list, (GFunc) g_free, NULL);
		g_list_free(list);
	}

	return formattedlist;
}

/**
 * This old function takes the contact and fills the parsed strings
 * in the form of familyname, firstname and email address. 
 *
 * @param contact_fields of type CheckNameFields.
 *
 * @return void
 */


static void ui_get_add_to_contact_fields(CheckNameFields * contact_fields)
{
	gchar *address = NULL;
	gchar *char_location1 = NULL;
	gchar *char_location2 = NULL;
	gchar *names = NULL;
	gchar *given_names = NULL;
	gchar *surname = NULL;
	gchar *space_location1 = NULL;
	gchar *space_location2 = NULL;


	CHECK_PARAM((contact_fields == NULL), "Input param check failed", return);
	CHECK_PARAM((contact_fields->result_string == NULL), "Input param check failed", return);

	address = g_strdup(contact_fields->result_string);
	g_strstrip(address);
	if (strlen(address) != 0) {
		char_location1 = strchr(address, LESSER_THAN);
		char_location2 = strchr(address, GREATER_THAN);
		if (char_location1 == NULL || char_location2 == NULL
		    || strlen(char_location1) - strlen(char_location2) - 1 <= 0) {
			contact_fields->email = g_strdup(address);
			g_free(address);
			return;
		} else {
			contact_fields->email = g_strndup(char_location1 + 1,
							  (guint) (strlen(char_location1) -
								   strlen(char_location2) - 1));
		}
	}
	gint tempLen = (gint) ((strlen(address) - strlen(char_location1)) - 1);
	if (tempLen > 0) {
		names = g_strndup(address, (guint) (strlen(address) - strlen(char_location1) - 1));
		g_strstrip(names);
		space_location1 = strchr(names, '"');
		space_location2 = strrchr(names, '"');

		if (space_location1 != NULL && space_location2 != NULL &&
		    strlen(space_location1) - strlen(space_location2) - 1 > 0) {
			given_names = g_strndup(space_location1 + 1,
						(guint) (strlen(space_location1) -
							 strlen(space_location2) - 1));
		} else {
			given_names = g_strdup(names);
		}
		space_location1 = strchr(given_names, EMAIL_SPACE);
		if (space_location1 == NULL) {
			contact_fields->givenname = g_strdup(given_names);
		} else {
			contact_fields->givenname =
			    g_strndup(given_names,
				      (guint) (strlen(given_names) - strlen(space_location1)));
			surname = g_strdup(space_location1);
			g_strstrip(surname);
			contact_fields->surname = g_strdup(surname);
		}
	}
	g_free(address);
	g_free(names);
	g_free(given_names);
	g_free(surname);

}


/**
This function is used to free the check name field structure along with its 
internall fields.

@param name_fields Pointer to check name fields structure.
@return void
*/
static void ui_free_check_name_fields_structure(CheckNameFields * name_fields)
{
	CHECK_PARAM((name_fields == NULL), "Input param check failed:", return);

	g_free(name_fields->givenname);
	name_fields->givenname = NULL;

	g_free(name_fields->surname);
	name_fields->surname = NULL;

	g_free(name_fields->email);
	name_fields->email = NULL;

	g_free(name_fields->emailhome);
	name_fields->emailhome = NULL;

	g_free(name_fields->emailbusiness);
	name_fields->emailbusiness = NULL;

	g_free(name_fields->result_string);
	name_fields->result_string = NULL;

	g_free(name_fields);

	return;
}




/**
 * This function is used to get econtact from a given contact in string format.
 * @param vieweraddress The formatted Email-address of the recepient.
 * @return EContact 
 */

EContact *get_e_contact_from_string(gchar * vieweraddress)
{
	EContact *contact = NULL;

	CheckNameFields *contact_fields = NULL;

	contact_fields = g_new0(CheckNameFields, 1);

	contact_fields->result_string = g_strdup(vieweraddress);
	ui_get_add_to_contact_fields(contact_fields);

	contact = e_contact_new();
	e_contact_set(contact, E_CONTACT_FAMILY_NAME, contact_fields->givenname);
	e_contact_set(contact, E_CONTACT_GIVEN_NAME, contact_fields->surname);
	e_contact_set(contact, E_CONTACT_EMAIL_1, contact_fields->email);

	ui_free_check_name_fields_structure(contact_fields);

	return contact;
}
