/**
 * @file editcallback.c
 *
 * This file contains all callbacks to the editor window such
 * as sending/saving of mails.attaching/removing files & other
 * miscelleanous functionality.
 *
 *  Copyright (c) 2005 Nokia. All rights reserved
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/

/*Editor-Viewer header files*/
#include "utils.h"
#include "editcallback.h"
#include "callback.h"
#include "accountMgmt_callback.h"
#include "accountMgmt.h"
#include "ui_account_store.h"
#include "security.h"

#include "html_parser.h"
#include "htcallback.h"
#include "email_clipboard.h"
#include "undo.h"

#include "contact_utils.h"
#include "email_contacts.h"
#include "check_names.h"


/* Global variables */
GtkWidget *currenteditor = NULL;
extern GSList *passwd_list;	/*Declared in security.c for caching the passwords */

void osso_email_format_to_html_text(EditorInfo * Editor, gchar ** buf_text);
gchar *gnome_vfs_mime_type_from_name(const char *);

static void ui_free_imagelist(InlineImageInfo * imageinfo, gpointer data);
extern gint no_of_tags;		/*declared in htcallback.c */
extern MainViewInfo *email_main_view;	/*Declared in ui_interface.c */
extern osso_context_t *ui_context;	/*declared in main.c */



/**
 This function is called by higher layers to find the next  mimeInfo Structure
 for a given mimeinfo.
 
 @param mimeinfo is a pointer to the mimeInfo structure.
 	This param is not freed in this function.
	
 @return MimeInfo structure pointer.
*/
/*@null@*/
MimeInfo *ui_mimeinfo_next(MimeInfo * mimeinfo)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (mimeinfo == NULL) {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return NULL;
	}
	if (mimeinfo->children != NULL) {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return mimeinfo->children;
	}
	if (mimeinfo->next != NULL) {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return mimeinfo->next;
	}

	for (mimeinfo = mimeinfo->parent; mimeinfo != NULL; mimeinfo = mimeinfo->parent) {
		if (mimeinfo->next != NULL) {
			return mimeinfo->next;
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return NULL;
}

/**
 This function is called by higher layers to free the mimeInfo Structure
	During the recv_msg_body call.
	
 @param mimeinfo is a pointer to the mimeInfo structure.
	This param is  freed in this function.
	
 @return void
*/
void ui_mimeinfo_free(MimeInfo * mimeInfo)
{
	MimeInfo *next = NULL;
	MimeInfo *mimeinfo = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	mimeinfo = mimeInfo;
	while (mimeinfo != NULL) {
		g_free(mimeinfo->encoding);
		g_free(mimeinfo->content_type);
		g_free(mimeinfo->charset);
		g_free(mimeinfo->name);
		g_free(mimeinfo->boundary);
		g_free(mimeinfo->content_disposition);
		g_free(mimeinfo->filename);
		g_free(mimeinfo->section_number);

		g_slist_foreach(mimeinfo->inline_images, (GFunc) ui_free_imagelist, NULL);
		g_slist_free(mimeinfo->inline_images);
		g_slist_free(mimeinfo->file_pos_list);
		g_free(mimeinfo->cert_path);

		ui_mimeinfo_free(mimeinfo->sub);
		ui_mimeinfo_free(mimeinfo->children);
		next = mimeinfo->next;
		g_free(mimeinfo);
		mimeinfo = next;
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}

/**
 This function is called higher layers to free the HeaderInfo contents
	for compose window.
	
 @param mimeinfo is a pointer to the mimeInfo structure.
	This param is  freed in this function.
	
 @return void.
*/
void ui_freeHeaderInfo_mimeinfo(MimeInfo * mimeInfo)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	ui_mimeinfo_free(mimeInfo);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called higher layers to free the HeaderInfo contents
	for compose window.
	
 @param headerInfo is a pointer to the HeaderInfo structure.
	This param is  freed in this function.
	
 @return void.
*/
void ui_freeHeaderInfo(HeaderInfo * headerInfo)
{

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (headerInfo == NULL) {
		return;
	}
	g_free(headerInfo->to);
	g_free(headerInfo->cc);
	g_free(headerInfo->bcc);
	g_free(headerInfo->from);
	g_free(headerInfo->reply_to);
	g_free(headerInfo->subject);
	g_free(headerInfo->msgid);
	g_free(headerInfo->filename);
	g_free(headerInfo->email_filename);
	if (headerInfo->msgInfo != NULL) {
		ui_freeMem(headerInfo->msgInfo, NULL);
	}
	g_free(headerInfo);
	ULOG_DEBUG("[%s][%s] : endS here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called higher layers to free the nodes of attachment list
 
 @param attInfo is a pointer to the AttachInfo structure.
	This param is  freed in this function.
 @param data is the gpointer to user data.

 @return void.
*/
void ui_freelist(AttachInfo * attInfo, gpointer data)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (attInfo == NULL) {
		return;
	}
	g_free(attInfo->filepath);
	g_free(attInfo->attachmentname);
	g_free(attInfo->msgid);
	g_free(attInfo->email_path);
	g_free(attInfo);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called higher layers to free the nodes of image list

 @param imagenfo is a pointer to the InlineImageInfo structure.
 	This param is  freed in this function.
 @param data is the gpointer to user data.

 @return void.
*/
static void ui_free_imagelist(InlineImageInfo * imageinfo, gpointer data)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (imageinfo == NULL) {
		return;
	}
	g_free(imageinfo->uri_path);
	g_free(imageinfo->content_id);
	g_free(imageinfo->image_name);
	g_free(imageinfo->image_path);
	g_free(imageinfo);
	imageinfo = NULL;

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 This function is called higher layers to free the message header contents
	for compose window.
 @param composeMsg is a pointer to the msg_contents structure.
	This param is  freed in this function.
 @result void
*/
static void ui_free_msgheader(MsgHeader * header)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (header == NULL) {
		return;
	}
	g_free(header->fromname);
	g_free(header->date);
	g_free(header->from);
	g_free(header->to);
	g_free(header->cc);
	g_free(header->bcc);
	g_free(header->subject);
	g_free(header->msgid);
	g_free(header->inreplyto);
	g_free(header->replyto);
	g_free(header->account_name);
	g_free(header->username);
	g_free(header->password);
	g_free(header);
	header = NULL;
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}


/**
 This function is called by the higher layers to free the message header contents
	for compose window.
 @param composeMsg is a pointer to the msg_contents structure.
	This param is  freed in this function.
 @param msgbody is a pointer to the buffer containing the body.
	This param is  freed in this function.
 @return void.
*/
void ui_freeMem(MsgInfo * composeMsg, gchar * body)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	g_free(body);
	if (composeMsg == NULL) {
		return;
	}
	/*Freeing the header of MsgInfo structure. */
	ui_free_msgheader(composeMsg->header);

	/*Freeing the setting of MsgInfo structure. */
	if (composeMsg->settings != NULL) {
		g_free(composeMsg->settings->account);
		g_free(composeMsg->settings);
		ULOG_DEBUG("freed sending options structure\n");
	}
	/*Freeing the attach_list of MsgInfo structure. */
	g_slist_foreach(composeMsg->attach_list, (GFunc) ui_freelist, NULL);
	g_slist_free(composeMsg->attach_list);

	/*freeing the recip_certs_uid of MsgInfo structure
	   The node elements are intergers so no need to free the nodes. */
	g_slist_free(composeMsg->recip_certs_uid);

	/*Freeing the image_list of MsgInfo structure. */
	g_slist_foreach(composeMsg->image_list, (GFunc) ui_free_imagelist, NULL);
	g_slist_free(composeMsg->image_list);

	g_free(composeMsg);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is used to free the temporary stored address.
 
 @param toAddr is a gchar pointer to To addresses.
 	This param is  freed in this function.
 @param ccAddr is a gchar pointer to Cc addresses.
	This param is  freed in this function.
 @param BccAddr is a gchar pointer to Bcc addresses.
	This param is  freed in this function.
	
 @return void.
*/
static void ui_free_temp_address(gchar * toAddr, gchar * ccAddr, gchar * bccAddr)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	g_free(toAddr);
	g_free(ccAddr);
	g_free(bccAddr);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This is a helper function to fill the valid email address to the header.
 
 @param field is the enum representing the header feild(TO/CC/BCC).
 @param recpFields is the pointer to the MsgHeader structure.
 	This param is not  freed in this function.
 @param Recepients is the gchar having the valid email ids.
 
 @return void.
*/
static void
ui_fill_address_after_validation(gint field, MsgHeader * recpFields, const gchar * Recepients)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (recpFields == NULL) {
		return;
	}
	if (field == TO) {	/*Represents the To field */
		g_free(recpFields->to);
		recpFields->to = NULL;
		if (Recepients != NULL) {
			recpFields->to = g_strdup(Recepients);
		}
	} else if (field == CC) {	/*Represents the Cc field */
		g_free(recpFields->cc);
		recpFields->cc = NULL;
		if (Recepients != NULL) {
			recpFields->cc = g_strdup(Recepients);
		}
	} else if (field == BCC) {	/*Represents the Bcc field */
		g_free(recpFields->bcc);
		recpFields->bcc = NULL;
		if (Recepients != NULL) {
			recpFields->bcc = g_strdup(Recepients);
		}
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}

/**
 This function is used to set the focus for invalid email address.
 
 @param emailid is the gchar pointer to the invalid email adderss.
 @param addressBuffer is the GtkTextBuffer for the recipient field.
 @param offset is the offset representing from where the search for emailid to begin
 
 @return gboolean FALSE if the id is part of a resolved id.
 		  TRUE else.
*/
static gboolean
set_focus_for_invalid_email(const gchar * emailid, GtkTextBuffer * addressBuffer, gint offset)
{
	GtkTextIter iter, start, end;
	GSList *tags = NULL;

	if (!emailid)
		return FALSE;
	gtk_text_buffer_get_iter_at_offset(addressBuffer, &iter, offset);
	if (gtk_text_iter_forward_search(&iter, emailid, GTK_TEXT_SEARCH_VISIBLE_ONLY, &start, &end,
					 NULL)) {
		tags = gtk_text_iter_get_tags(&start);
		if (tags) {
			g_slist_free(tags);
			return FALSE;
		}
		gtk_text_buffer_select_range(addressBuffer, &start, &end);
	}


	return TRUE;
}

/**
 This is a helper function to validate the address.
 
 @param field is the address field entry.
 @param Recepients is the OUT param with valid address.
 @param addressBuffer is the GtkTextBuffer for the recipient field.
 
 @return gboolean TRUE ->valid emailid
 		  FALSE ->else
 	
*/
gboolean
ui_editor_validate_addresses(const gchar * Field, gchar ** Recepients,
			     GtkTextBuffer * addressBuffer)
{
	gint j = 0;
	gchar **parseString = NULL;
	gchar *emailid = NULL;
	GString *str = NULL;
	str = g_string_new(NULL);
	gint offset = 0;

	*Recepients = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (Field == NULL) {
		*Recepients = NULL;
		return TRUE;
	}
	/*tokenize the string with ',' seperator */
	parseString = g_strsplit_set(Field, ",;", -1);
	while (parseString && parseString[j]) {
		g_strstrip(parseString[j]);
		/*tokensize the string with emailid enclosed in "<" ">" */
		emailid = ui_parse_from_address(parseString[j]);
		/*Checks if the mail address has a missing domain or '@' symbol */
		if (!emailid || ((strlen(emailid) > 0) && !ui_accmgmt_email_isvalid(emailid))){
			if (set_focus_for_invalid_email(parseString[j], addressBuffer, offset)) {
				g_strfreev(parseString);
				g_free(emailid);
				g_free(*Recepients);
				return FALSE;
			} else {
				g_free(emailid);
				j++;
				continue;
			}
		} else if (ui_accmgmt_email_isvalid(emailid)) {
			g_string_append_printf(str, "%s, ", emailid);
		}
		offset = offset + strlen(parseString[j]);
		g_free(emailid);
		j++;
	}
	g_strfreev(parseString);
	*Recepients = str->str;
	g_string_free(str, FALSE);
	return TRUE;
}

/*
Support function for email address validation.
@param recpFields is a pointer to the MsgHeader struct containing the receipient fields.
This parameter is not freed.
@param addressBuffer is the GSlist containing the pointers to the GtkTextBufer of the 
address fields
This parameter is not freed.
@returns gboolean TRUE-valid address
                  FALSE-invalid address

*/
static gboolean ui_editor_do_validation(MsgHeader * recpFields, GSList * addressBuffers)
{
	gchar *Field[3] = { NULL, NULL, NULL };
	gchar *Recepients = NULL;
	gint i = 0;
	gboolean addressStatus = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (recpFields == NULL) {
		return FALSE;
	}
	if (recpFields->to != NULL) {
		Field[0] = g_strdup(g_strstrip(recpFields->to));
		if (strlen(Field[0]) < 1) {
			Field[0] = NULL;
		}
	}
	if (recpFields->cc != NULL) {
		Field[1] = g_strdup(g_strstrip(recpFields->cc));
		if (strlen(Field[1]) < 1) {
			Field[1] = NULL;
		}
	}
	if (recpFields->bcc != NULL) {
		Field[2] = g_strdup(g_strstrip(recpFields->bcc));
		if (strlen(Field[2]) < 1) {
			Field[2] = NULL;
		}

	}
	if (Field[0] == NULL && Field[1] == NULL && Field[2] == NULL) {
		ui_free_temp_address(Field[0], Field[1], Field[2]);
		return FALSE;
	}
	for (i = 0; i < 3; i++) {
		addressStatus =
		    ui_editor_validate_addresses(Field[i], &Recepients, (GtkTextBuffer *)
						 g_slist_nth_data(addressBuffers, i));
		if (addressStatus) {
			ui_fill_address_after_validation(i, recpFields, Recepients);
			g_free(Recepients);
			Recepients = NULL;
		} else {
			ui_free_temp_address(Field[0], Field[1], Field[2]);
			return FALSE;
		}
	}
	ui_free_temp_address(Field[0], Field[1], Field[2]);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);

	if (recpFields->to == NULL && recpFields->cc == NULL && recpFields->bcc == NULL) {
		return FALSE;
	}
	return TRUE;
}


/**
 This is helper function for knowing the attachment info.
 
 @param attach_list is the GSList for attachment info.
	This parameter is not freed.
 @return void.
*/
static void ui_attachments_info_contents(const GSList * attach_list)
{
	GSList *node = NULL;
	node = (GSList *) attach_list;
	if (node == NULL) {
		ULOG_DEBUG("[%s]:[%d] : no attachments!!!\n", __FUNCTION__, __LINE__);
		return;
	}
	while (node) {
		AttachInfo *list = (AttachInfo *) node->data;
		if (list != NULL) {
			ULOG_DEBUG("[%s]:[%d] : stored name: %s\n", __FUNCTION__,
				   __LINE__, list->attachmentname);
			ULOG_DEBUG("[%s]:[%d] : stored id: %s\n", __FUNCTION__, __LINE__,
				   list->msgid);
			ULOG_DEBUG("[%s]:[%d] : stored type: %d\n", __FUNCTION__, __LINE__,
				   list->type);
			ULOG_DEBUG("[%s]:[%d] : stored path: %s\n", __FUNCTION__, __LINE__,
				   list->filepath);
		}
		node = node->next;
	}
	return;
}

/**
 Function is used to launch the account setup wizard upon invalid
	username/password/server

 @param account is the gchar representing the account name.
 @param server is the gchar representing the server name.
 @param error is the error representing whether password is NULL/user name is NULL.
 
 @return void.
*/
static void launch_account_setup(const gchar * account, const gchar * server, gint error)
{
	GString *infonote = NULL;

	CHECK_PARAM(account == NULL, "Input param failed", return);
	CHECK_PARAM(server == NULL, "Input param failed", return);

	infonote = g_string_new(NULL);
	if (infonote == NULL) {
		ULOG_CRIT_F("unable to allocate memory");
	}
	if (error == NULL_USERNAME) {
		g_string_printf(infonote, _("emev_ni_ui_smtp_userid_invalid"), account, server);

	} else if (error == NULL_PASSWD) {
		g_string_printf(infonote, _("emev_ni_ui_smtp_passwd_invalid"), account, server);
	}
	get_user_confirmation(infonote->str, NULL, NULL, INFO_BANNER);
	g_string_free(infonote, TRUE);
	email_main_view->account_manager->action_after_acct_creation = SEND_EMAIL;
	ui_accmgmt_display_dialog_for_feilds((gchar *) account, SMTPSERVER_DIALOG);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;

}

/**
 This Function si used to call the account setup dialog
 if no accounts are defined.
 
 @return void.
*/
static void ui_call_account_setup_wizard(GtkWidget * editor)
{
	gboolean confirmation = FALSE;
	GtkWidget *accountSetupWin = NULL;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	confirmation =
	    get_user_confirmation(_("mcen_nc_no_email_acnts_defined"),
				  _("mcen_bd_define"), NULL, CONFIRM_DIALOG);
	if (confirmation == TRUE) {
		ULOG_DEBUG("No Accouts defined\n");
		email_main_view->account_manager->action_after_acct_creation = SEND_EMAIL;
		currenteditor = editor;
		accountSetupWin = (GtkWidget *) ui_accmgmt_createAccountsetupDialog(FALSE);
		if (accountSetupWin != NULL) {
			gtk_widget_show(accountSetupWin);
		}
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	}
	return;
}

/**
 Validate the msgFields in the editor

 @param msgFields is the MsgFields structure pointer of the current editor
	This parameter is not freed in this function.
	
 @return TRUE-if valid fields.
         FALSE-else.
*/

static gboolean ui_editor_validate_msgfields(const MsgFields * msgFields)
{
	CHECK_PARAM(msgFields == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(msgFields->to == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(msgFields->cc == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(msgFields->bcc == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(msgFields->subject == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(msgFields->body == NULL, "Input param check failed\n", return FALSE);
	return TRUE;
}

/**
 Validate the editorinfo structure of  the editor
 
 @param editor_info is the EditorInfo structure pointer of the current editor
 
 @return TRUE-if valid fields.
         FALSE-else.
*/
static gboolean ui_validate_editorinfo(const EditorInfo * editor_info)
{
	gboolean valid = TRUE;
	CHECK_PARAM(editor_info == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(editor_info->msgInfo == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(editor_info->msgFields == NULL, "Input param check failed\n", return FALSE);
	valid = ui_editor_validate_msgfields(editor_info->msgFields);
	return valid;
}

/**
 Fills the msgHeader sructure with the fields in the editor
 
 @param to is the string representing to  field contents.
 @param cc is the string representing cc  field contents.
 @param bcc is the string representing bcc field contents.
 @param subject is the string representing subject  field contents.
 @param header is the pointer to the Msgheader structure.

 @return void.
*/
static void
ui_editor_fill_msgheader(const gchar * to, const gchar * cc,
			 const gchar * bcc, const gchar * subject, MsgHeader * header)
{
	if (header != NULL) {
		if (to != NULL && strlen(to) > 0) {
			header->to = g_strdup(to);
		} else {
			header->to = NULL;
		}
		if (cc != NULL && strlen(cc) > 0) {
			header->cc = g_strdup(cc);
		} else {
			header->cc = NULL;
		}
		if (bcc != NULL && strlen(bcc) > 0) {
			header->bcc = g_strdup(bcc);
		} else {
			header->bcc = NULL;
		}
		if (subject != NULL && strlen(subject) > 0) {
			header->subject = g_strdup(subject);
		} else {
			header->subject = NULL;
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 Extracts the contents of the editor so that they are filled in the msgHeader
 structure later.
 
 @param editor_info is the pointer to the EditorInfo structure of the current editor.
 	This parameter is not freed in this function.
 @param body is the OUT param containing the body content.
 
 @return FALSE if input parm check fails.
         TRUE-else.
*/
static gboolean ui_editor_get_msginfo(EditorInfo * editor_info, gchar ** body)
{
	MsgInfo *msgInfo = NULL;
	MsgFields *msgFields = NULL;
	MsgHeader *msgHeader = NULL;
	GtkTextBuffer *bufferTo = NULL;
	GtkTextBuffer *bufferCc = NULL;
	GtkTextBuffer *bufferBcc = NULL;
	GtkTextIter start, end;
	GtkTextBuffer *bufferBody = NULL;
	gchar *to = NULL;
	gchar *cc = NULL;
	gchar *bcc = NULL;
	const gchar *subject = NULL;
	gchar *msg = NULL;

	CHECK_PARAM(editor_info == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(editor_info->msgInfo == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(editor_info->msgFields == NULL, "Input param check failed\n", return FALSE);

	msgHeader = g_new0(MsgHeader, 1);
	msgInfo = editor_info->msgInfo;
	msgInfo->header = msgHeader;
	msgFields = editor_info->msgFields;

	bufferTo = gtk_text_view_get_buffer((GtkTextView *) msgFields->to);
	bufferCc = gtk_text_view_get_buffer((GtkTextView *) msgFields->cc);
	bufferBcc = gtk_text_view_get_buffer((GtkTextView *) msgFields->bcc);

	gtk_text_buffer_get_bounds(bufferTo, &start, &end);
	to = gtk_text_buffer_get_text(bufferTo, &start, &end, FALSE);
	gtk_text_buffer_get_bounds(bufferCc, &start, &end);
	cc = gtk_text_buffer_get_text(bufferCc, &start, &end, FALSE);
	gtk_text_buffer_get_bounds(bufferBcc, &start, &end);
	bcc = gtk_text_buffer_get_text(bufferBcc, &start, &end, FALSE);

	subject = gtk_entry_get_text((GtkEntry *) msgFields->subject);
	ui_editor_fill_msgheader(to, cc, bcc, subject, msgHeader);
	if (editor_info->accountSettings != NULL) {
		ui_free_prefs_account_structure(editor_info->accountSettings);
		editor_info->accountSettings = NULL;
	}
	if ((editor_info->msgInfo->settings->account == NULL || g_ascii_strncasecmp
	     (editor_info->msgInfo->settings->account, DEFAULT,
	      MAX_STR_LEN(editor_info->msgInfo->settings->account, DEFAULT)) == 0)
	    && email_main_view->account_manager->configured_accounts != NULL) {
		editor_info->msgInfo->settings->account =
		    g_strdup(email_main_view->account_manager->default_account);

	}
	editor_info->accountSettings =
	    ui_get_account_detail(editor_info->msgInfo->settings->account);

	if (editor_info->accountSettings != NULL) {
		if (editor_info->accountSettings->send_settings.username != NULL) {
			msgHeader->username =
			    g_strdup(editor_info->accountSettings->send_settings.username);
		}
		if (editor_info->accountSettings->send_settings.password != NULL) {
			msgHeader->password =
			    g_strdup(editor_info->accountSettings->send_settings.password);
		}
	}
	msgHeader->smime_type = editor_info->msgInfo->settings->smime_type;
	msgHeader->msgnum = email_main_view->tags->window_count;
	g_free(to);
	g_free(cc);
	g_free(bcc);
	if (body != NULL) {
		bufferBody = gtk_text_view_get_buffer((GtkTextView *) msgFields->body);
		gtk_text_buffer_get_bounds(bufferBody, &start, &end);
		msg = gtk_text_buffer_get_text(bufferBody, &start, &end, FALSE);
		*body = g_strdup(msg);
		g_free(msg);
	}

	if ((editor_info->composeType == DRAFT_MAIL || editor_info->composeType == OUTBOX_MAIL)
	    && editor_info->msgid != NULL) {
		msgHeader->msgid = g_strdup(editor_info->msgid);
	} else {
		msgHeader->msgid = NULL;
	}
	return TRUE;
}

static void update_msgid_for_openwindows(EditorInfo * editorInfo, gchar * msgid, gint msgnum)
{
	GSList *tempNode = NULL;
	OpenedWindowData *open_windows = NULL;

	for (tempNode = email_main_view->open_windows; tempNode != NULL; tempNode = tempNode->next) {
		open_windows = (OpenedWindowData *) (tempNode->data);
		if (msgnum == open_windows->msgNum) {
			g_free(open_windows->msgid);
			open_windows->msgid = NULL;
			open_windows->msgid = g_strdup(msgid);
			g_object_set_data_full(G_OBJECT(editorInfo->editor_appview),
					       VIEW_MSG_ID,
					       g_strdup(open_windows->msgid),
					       (GDestroyNotify) g_free);
			ui_update_row_tag_in_folder(editorInfo->folderNum, open_windows->msgid,
						    IN_USE_STATUS, TRUE);
			break;
		}
	}
	return;
}

void stop_editor_from_recieving_events(EditorInfo * editor_info, gboolean enable)
{
	GtkWidget *menu = NULL;
	EditorToolbar *toolbar = NULL;
	MsgFields *fields = NULL;
	gboolean disable = !enable;

	menu = (GtkWidget *) hildon_appview_get_menu((HildonAppView *) editor_info->editor_appview);
	toolbar = editor_info->toolbar;
	fields = editor_info->msgFields;

	gtk_widget_set_sensitive(fields->to, disable);
	gtk_widget_set_sensitive(fields->cc, disable);
	gtk_widget_set_sensitive(fields->bcc, disable);
	gtk_widget_set_sensitive(fields->subject, disable);
	gtk_widget_set_sensitive(fields->body, disable);


	gtk_widget_set_sensitive(menu, disable);
	gtk_widget_set_sensitive(toolbar->send, disable);
	gtk_widget_set_sensitive(toolbar->attachment, disable);
	gtk_widget_set_sensitive(toolbar->recipients, disable);
	gtk_widget_set_sensitive(toolbar->fontname, disable);
	gtk_widget_set_sensitive(toolbar->fontsize, disable);
	gtk_widget_set_sensitive(toolbar->bold, disable);
	gtk_widget_set_sensitive(toolbar->italic, disable);
	gtk_widget_set_sensitive(toolbar->fgcolor, disable);
	gtk_widget_set_sensitive(toolbar->bullets, disable);

	return;
}

/**
 save the editor into the drafts
 
 @param editor_info is the pointer to the EditorInfo structure of the current editor.
 	This parameter is not freed in this function.
 @param body is the gchar containing the body content.
	 This parameter is not freed in this function.
 @param  gbooean TRUE-if enginge is busy sending we save in outbox.
                  FALSE- saving a message.	 
 @return gbooean TRUE-if saving was successful
                  FALSE-else.	 
*/
static gboolean ui_editor_save_msg(EditorInfo * editor_info, const gchar * body, gboolean sending)
{
	gchar *ret_msgid = NULL;
	gchar *old_msgid = NULL;

	CHECK_PARAM(editor_info == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(editor_info->msgInfo == NULL, "Input param check failed\n", return FALSE);

	if (sending == TRUE) {
		editor_info->msgInfo->header->msgid = g_strdup(editor_info->msgid);
		old_msgid = NULL;
		ui_unset_oom_signal_status();
	}

	ui_dbus_send_get_msg_id(editor_info->msgInfo, &ret_msgid, sending);

	g_print("******the MSGID RETURNED %s\n", ret_msgid);

	if (editor_info->msgInfo->header->msgid != NULL) {
		old_msgid = editor_info->msgInfo->header->msgid;
		editor_info->msgInfo->header->msgid = NULL;
	}
	if (sending == FALSE)
		old_msgid = g_strdup(editor_info->msgid);
	else
		old_msgid = NULL;

	/*freeing the previous msgid */
	g_free(editor_info->msgid);
	editor_info->msgid = NULL;

	editor_info->msgInfo->header->msgid = g_strdup(ret_msgid);
	editor_info->msgid = g_strdup(ret_msgid);
	if (ret_msgid == NULL || strlen(ret_msgid) < 1) {
		g_free(old_msgid);
		return FALSE;
	}
	stop_editor_from_recieving_events(editor_info, TRUE);


	if (sending == FALSE) {
		gtk_banner_show_animation((GtkWindow *) app, _("mail_va_saving"));
		ui_flush_gtk_events();
	}
	ui_dbus_send_save_draft_mail(editor_info->msgInfo, body, FALSE, old_msgid);

	editor_info->editorModified = FALSE;
/*update the global open window data*/
	update_msgid_for_openwindows(editor_info, editor_info->msgid, editor_info->msgnum);
	g_free(ret_msgid);
	g_free(old_msgid);
	return FALSE;

}

/**
 Helper function to get the count of number of open messages.
 
 @return a gint representing the count of the open windows.
*/
static gint ui_get_count_open_editors()
{
	GSList *tempNode = NULL;
	OpenedWindowData *open_windows = NULL;
	gint count = 0;

	for (tempNode = email_main_view->open_windows; tempNode != NULL; tempNode = tempNode->next) {
		open_windows = (OpenedWindowData *) (tempNode->data);
		if (open_windows->iseditor == TRUE) {
			count++;
		}
	}
	return count;
}

/*
  This function is used to get the currently selected foldername.
  
  @param foldernum is the gint representing the folder number from which
  	the editor is opened.
  
  @return gchar a newly allocated string containing the folder name.
*/
static gchar *get_folder_name(gint foldernum)
{
	gchar *current_folder_name = NULL;
	gchar *parent_folder = NULL, *sub_folder = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	current_folder_name = ui_get_folder_name(foldernum);
	if (ui_get_folder_names_from_path(current_folder_name, &parent_folder, &sub_folder) ==
	    FALSE) {
		ULOG_DEBUG("[%s][%s] : failed to folder name:\n", __FILE__, __FUNCTION__);
	}
	g_free(current_folder_name);
	g_free(parent_folder);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return sub_folder;
}


/**
 this function is called when activating the close button for closing editor.
 
 @param win is a pointer to the menu item
 @param editor_info is the EditorInfo structure pointer of the current editor
	This param is freed here by calling the function ui_free_editorinfo().
	
 @return void
*/

void ui_editor_close_cb(GtkWidget * win, EditorInfo * editor_info)
{
	MsgInfo *msgInfo = NULL;
	gint mail = 0, foldernum = 0;
	gchar *folder_name = NULL;
	gchar *infonote = NULL;
	gchar *body = NULL;
	GtkTextBuffer *bufferBody = NULL;
	gchar *buf_text = NULL;
	gboolean valid = FALSE;
	gboolean success = FALSE;
	gboolean mem_allocated = TRUE;
	GSList *tempNode = NULL;
	OpenedWindowData *open_windows = NULL;
	const gchar *folder_display_name = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);


	/*validation of the input param editor_info */
	valid = ui_validate_editorinfo(editor_info);
	if (valid == FALSE) {
		return;
	}
	msgInfo = editor_info->msgInfo;
	mail = editor_info->composeType;
	foldernum = editor_info->folderNum;

	bufferBody = gtk_text_view_get_buffer((GtkTextView *) editor_info->msgFields->body);
	g_signal_handlers_block_by_func(editor_info->editor_appview,
					ui_editor_save_all_in_drafts, editor_info);
	if (editor_info->msgInfo->header != NULL) {
		ui_free_msgheader(editor_info->msgInfo->header);
		editor_info->msgInfo->header = NULL;
	}

	mem_allocated = ui_editor_get_msginfo(editor_info, &body);
	if (mem_allocated == FALSE) {
		g_signal_handlers_unblock_by_func(editor_info->editor_appview,
						  ui_editor_save_all_in_drafts, editor_info);
		return;
	}
	if (mail == DRAFT_MAIL || mail == OUTBOX_MAIL) {
		ui_update_row_tag_in_folder(editor_info->folderNum,
					    editor_info->msgid, IN_USE_STATUS, FALSE);
	}
	ui_unset_oom_signal_status();

	/*get the foldername for the message */
	if (mail == NEW_MAIL || foldernum != OUTBOX_MODEL_NUM) {
		foldernum = DRAFT_MODEL_NUM;
	}
	folder_name = get_folder_name(foldernum);
	folder_display_name = ui_get_folder_display_name(foldernum, folder_name);

	/*Checking whether the message is modified & asking for confirmation to save */
	if (editor_info->editorModified == TRUE) {
		if (email_main_view->tags->application_closure == FALSE) {
			/*confirmation for saving in draft folder */
			if (editor_info->folderNum == DRAFT_MODEL_NUM) {
				success =
				    get_user_confirmation(_
							  ("mcen_nc_no_email_message_modified_save_changes"),
							  _
							  ("mcen_nc_no_email_message_modified_save_changes_yes"),
							  _
							  ("mcen_nc_no_email_message_modified_save_changes_no"),
							  CONFIRM_DIALOG);
			}
			/*confirmation for saving in other then draft folder */
			else {
				infonote =
				    g_strdup_printf(_
						    ("mcen_nc_no_email_message_modified_save_changes_to_folder"),
						    folder_display_name);
				success =
				    get_user_confirmation(infonote,
							  _
							  ("mcen_nc_no_email_message_modified_save_changes_to_folder_yes"),
							  _
							  ("mcen_nc_no_email_message_modified_save_changes_to_folder_no"),
							  CONFIRM_DIALOG);
				g_free(infonote);
				infonote = NULL;
			}
		}
		if (success == TRUE || email_main_view->tags->application_closure == TRUE) {
			if (editor_info->msgInfo->header->smime_type != SMIME_NONE &&
			    email_main_view->tags->application_closure == FALSE)
				get_user_confirmation(_("mcen_nc_security_settings_not_saved"),
						      NULL, NULL, INFO_BANNER);

			editor_info->msgInfo->header->smime_type = SMIME_NONE;

			if (editor_info->emailType == HTML_EDITOR) {
				/*This is to convert the formatted textin gtk to html */
				osso_email_format_to_html_text(editor_info, &buf_text);
				if (buf_text != NULL) {
					body = buf_text;
				}
				/*This is to convert back the html text to gtk */
				osso_email_convert_html_to_gtk(editor_info, body);
			}
			if (email_main_view->tags->oom_signal_received == FALSE) {
				success = ui_editor_save_msg(editor_info, body, FALSE);
				if (success == FALSE) {	/*saving in progress */
					g_free(body);
					ui_free_msgheader(editor_info->msgInfo->header);
					editor_info->msgInfo->header = NULL;
					editor_info->editorModified = FALSE;
					return;
				}
			}
		} else {
			if (editor_info->autosaved == TRUE) {
				ui_dbus_folder_remove_msg(folder_name, editor_info->msgid, FALSE);
				g_free(folder_name);
				folder_name = NULL;
				for (tempNode = email_main_view->open_windows;
				     tempNode != NULL; tempNode = tempNode->next) {
					open_windows = (OpenedWindowData *) (tempNode->data);
					if (editor_info->msgnum == open_windows->msgNum) {
						if (open_windows->oldmsgid != NULL) {
							ui_dbus_autosave_mail
							    (open_windows->oldmsgid, 2);
						}
						break;
					}
				}
				ui_get_refresh_folder(editor_info->folderNum);
			}
		}
	}
	if (email_main_view->tags->application_closure == TRUE && HILDON_IS_APPVIEW(win)) {
		/*When appliction goes down we simply destroy the appview.
		   memory is released once the process is killed. */
		gtk_widget_destroy(editor_info->editor_appview);
		return;
	}
	g_free(body);
	ui_update_window_pointers(editor_info->msgid);

	/* Refreashing unread and unsent message count to folders in folder list view */
	if (editor_info->folderNum >= 0)
		ui_add_folder_name_with_unread_unsent_mails(editor_info->folderNum);
	ui_free_editorinfo(editor_info);
	ui_show_next_window();

	if (success == TRUE) {
		if (foldernum == DRAFT_MODEL_NUM) {
			/* Add message count with folder display name to display in folder view */
			ui_add_unread_count_to_folder(DRAFT_MODEL_NUM);
			ui_add_folder_name_with_unread_unsent_mails(DRAFT_MODEL_NUM);
		}
		g_free(folder_name);
		folder_name = get_folder_name(foldernum);
		folder_display_name = ui_get_folder_display_name(foldernum, folder_name);
		infonote = g_strdup_printf(_("mail_va_saved_to_drafts"), folder_display_name, NULL);
		gtk_infoprint((GtkWindow *) app, infonote);
		g_free(infonote);
	}
	g_free(folder_name);
	return;
}

/**
 This Function is used to send draft mail.
 
 @param comMsg is the MsgInfo structure of the current message.
 	This paramter is not freed in this structure.
 @param body is the gchar representing the body of the message.
 @param msgid is the gchar rep the msgid of the mail.
 @param selection is the GtkTreeSelection of the current attchament selected.
 @param folderNum is the foldernum of the current folder.
 
 @return void
*/
static void
ui_send_mail_from_draft(MsgInfo * comMsg, const gchar * body,
			GtkTreeSelection * selection, EditorInfo * editor_info, gboolean busy)
{
	gchar *folder_name = NULL;
	const gchar *msgid = NULL;
	gint folderNum = 0;
	gint outbox_status_bit = -1;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(comMsg == NULL, "Input param check failed\n", return);
	CHECK_PARAM(comMsg->header == NULL, "Input param check failed\n", return);

	msgid = (const gchar *) editor_info->msgid;
	folderNum = editor_info->folderNum;

	comMsg->header->msgid = g_strdup(msgid);
	/*Checking if previously the status of outbox mail is on_request.
	 * & if so, when send is clicked we change the option to immediate*/
	if (comMsg->settings->options == SEND_ON_REQUEST && folderNum == OUTBOX_MODEL_NUM) {
		outbox_status_bit = g_ascii_digit_value(msgid[strlen(msgid) - 1]);
		if (outbox_status_bit == OUTBOX_STATUS_ON_REQUEST)
			comMsg->settings->options = SEND_IMMEDIATE;
	}

	if (busy == TRUE) {
		/*engine is busy send the mail to outbox */
		ui_editor_save_msg(editor_info, body, TRUE);

		/*unsetting the oom signal status */
		if (folderNum == DRAFT_MODEL_NUM) {
			folder_name = ui_get_folder_name(folderNum);
			if (folder_name != NULL)
				ui_dbus_folder_remove_msg(folder_name, msgid, FALSE);
		}
	} else
		ui_dbus_send_mail(comMsg, body);

	/*Refresh the current view */
	ui_folderview_selection_changed_cb(selection, NULL);
	g_free(folder_name);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}

/**
 This function is used to check for empty message body & throw appropriate message
 @param body is the gchar representing the body of the message.
 @param header is the MsgHeader structure pointer.
	 This paramter is not freed in this structure.
 @param editor_info is pointer to EditorInfo structure.	 
 @return TRUE-if message is empty.
         FALSE-else.
*/
static gboolean
ui_check_for_empty_message_body(const gchar * body, MsgHeader * header, EditorInfo * editor_info)
{
	GString *infonote = NULL;
	gchar *subject = NULL;
	gchar *ellipsized_subject = NULL;
	gboolean response = FALSE;
	GtkTextIter start;
	gunichar image_char;


	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	infonote = g_string_new(NULL);
	if (infonote == NULL) {
		ULOG_CRIT_F("Unable to allocate memory");
		return FALSE;
	}
	gtk_text_buffer_get_start_iter(gtk_text_view_get_buffer((GtkTextView *)
								editor_info->msgFields->body),
				       &start);
	image_char = gtk_text_iter_get_char(&start);
	if (((body != NULL && (strlen(body) == 0)) || body == NULL)
	    && (image_char != 0xFFFC) && (backgroundcolor_is_white(editor_info) == TRUE)) {
		if (header != NULL && header->subject == NULL) {
			subject = g_strdup(_("mail_va_no_subject"));
		} else if (header != NULL) {
			if (strlen(header->subject) > 100) {
				ellipsized_subject = g_strndup(header->subject, 100);
				subject = g_strconcat(ellipsized_subject, ELLIPSE, NULL);
				g_free(ellipsized_subject);
			} else
				subject = g_strdup(header->subject);
		}
		g_string_printf(infonote, _("emev_ni_ui_smtp_message_null"), subject);
		response = get_user_confirmation(infonote->str, NULL, NULL, CONFIRM_DIALOG);
		g_string_free(infonote, TRUE);
		g_free(subject);
		if (response == TRUE) {
			ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
			return TRUE;
		} else {
			ui_free_msgheader(header);
			ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
			return FALSE;
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return TRUE;
}

/**
 Checking whether the recipient fields are empty.
	 if so open the address book
 @param editor_info is the pointer to the editorInfo structure of the current editor.
 @param body is the gchar for the message body.
 @return gboolean TRUE-if recipients are empty.& editor_info->msgInfo->header & body are freed.
                  FALSE-else no freeing is done.
*/
static gboolean ui_editor_check_for_receipients(EditorInfo * editor_info, gchar * body)
{
	MsgInfo *msginfo = NULL;
	MsgHeader *msgheader = NULL;

	CHECK_PARAM(editor_info == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(editor_info->msgInfo == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(editor_info->msgInfo->header == NULL, "Input param check failed\n",
		    return FALSE);

	msginfo = editor_info->msgInfo;
	msgheader = editor_info->msgInfo->header;
	gboolean recipientEmpty = FALSE;

	if ((msgheader->to != NULL && strlen(msgheader->to)) != 0 ||
	    (msgheader->cc != NULL && strlen(msgheader->cc)) != 0 ||
	    (msgheader->bcc != NULL && strlen(msgheader->bcc)) != 0) {
		recipientEmpty = FALSE;
	} else {
		recipientEmpty = TRUE;
		ui_free_msgheader(msgheader);
		g_free(body);
	}
	return recipientEmpty;
}

/**
 checks for the address status
	whether VALID/INVALID/DOMAIN_MISSING
	if the address is not valid then the memory for the header &
	body are freed in this call.
	
 @param address is the enum representing the address staus valid/invalid/domain missing.
 @param header is the Msgheader of the email editor.
 @param body is the gchar rep the body content.
 @return TRUE-if valid email address.
        FALSE-else. 
*/
static gboolean
ui_editor_check_address_status(gboolean valid_address, MsgHeader * header, gchar * body)
{

	if (!valid_address) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_not_valid_email"));
		ui_free_msgheader(header);
		g_free(body);
		return FALSE;
	}
	return TRUE;
}

/**
 this function is called when activating the send button for sending the message.
 @param win is a pointer to the menu item
 @param editor_info is the EditorInfo structure pointer of the current editor
	This parameter is freed in side this function by call the function. ui_free_editorinfo().
 if sending operation is successful.
 
 @return void
*/
void ui_editor_send_msg_cb(GtkWidget * win, EditorInfo * editor_info)
{
	MsgInfo *msgInfo = NULL;
	gchar *body = NULL;
	GtkTextBuffer *bufferBody = NULL;
	GtkTextBuffer *bufferTo = NULL;
	GtkTextIter start;
	gboolean valid = FALSE, busy = FALSE;
	gboolean mem_allocated = TRUE;
	gboolean recipientEmpty = FALSE;
	GtkTreeSelection *selection = NULL;
	gboolean response = FALSE;
	gboolean validAddress;
	gint mail = 0;
	GString *body_text = NULL;
	GSList *used_econtact_uids = NULL, *new_emailids = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	g_signal_handlers_block_by_func(win, ui_editor_send_msg_cb, editor_info);

	selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	valid = ui_validate_editorinfo(editor_info);
	if (valid == FALSE) {
		g_signal_handlers_unblock_by_func(win, ui_editor_send_msg_cb, editor_info);
		return;
	}
	/*Checking for configured accounts ,if none Defined Account Setup dialog is launched. */
	if (email_main_view->account_manager->configured_accounts == NULL) {
		ui_call_account_setup_wizard(editor_info->editor_appview);
		g_signal_handlers_unblock_by_func(win, ui_editor_send_msg_cb, editor_info);
		return;
	}
	stop_editor_from_recieving_events(editor_info, TRUE);
	ui_editor_do_check_names(win, editor_info->addressBuffers);
	g_signal_handlers_block_by_func(editor_info->editor_appview,
					ui_editor_save_all_in_drafts, editor_info);
	mem_allocated = ui_editor_get_msginfo(editor_info, &body);
	if (mem_allocated == FALSE) {
		g_signal_handlers_unblock_by_func(editor_info->editor_appview,
						  ui_editor_save_all_in_drafts, editor_info);
		stop_editor_from_recieving_events(editor_info, FALSE);
		g_signal_handlers_unblock_by_func(win, ui_editor_send_msg_cb, editor_info);
		return;
	}

	mail = editor_info->composeType;
	msgInfo = editor_info->msgInfo;

	/*Check if username is NULL */
	if (msgInfo->header->username == NULL) {
		if (editor_info->accountSettings != NULL &&
		    editor_info->accountSettings->send_settings.smtp_auth_type !=
		    SMTPAUTH_NO_AUTH) {
			launch_account_setup(msgInfo->settings->account,
					     editor_info->accountSettings->send_settings.
					     smtp_server, NULL_USERNAME);
			g_signal_handlers_unblock_by_func(editor_info->editor_appview,
							  ui_editor_save_all_in_drafts,
							  editor_info);
			ui_free_msgheader(msgInfo->header);
			editor_info->msgInfo->header = NULL;
			g_free(body);
			stop_editor_from_recieving_events(editor_info, FALSE);
			g_signal_handlers_unblock_by_func(win, ui_editor_send_msg_cb, editor_info);
			return;
		}
	}
	/*Check if the the recipient fields are empty opened the addressbook based on
	 * user action or save to drafts*/
	recipientEmpty = ui_editor_check_for_receipients(editor_info, body);
	if (recipientEmpty == TRUE) {
		editor_info->msgInfo->header = NULL;
		response =
		    get_user_confirmation(_("mcen_nc_recipient_missing"), NULL, NULL,
					  CONFIRM_DIALOG);
		if (response == TRUE) {
			bufferTo =
			    gtk_text_view_get_buffer((GtkTextView *) editor_info->msgFields->to);
			gtk_widget_grab_focus(editor_info->msgFields->to);
			ui_editor_openAddrBook_cb(NULL, NULL, bufferTo);
			g_signal_handlers_unblock_by_func(editor_info->editor_appview,
							  ui_editor_save_all_in_drafts,
							  editor_info);
			g_signal_handlers_unblock_by_func(win, ui_editor_send_msg_cb, editor_info);
			stop_editor_from_recieving_events(editor_info, FALSE);
		} else {
			email_main_view->tags->application_closure = TRUE;
			ui_editor_close_cb(editor_info->menu->close_all_windows, editor_info);
			email_main_view->tags->application_closure = FALSE;
		}
		return;
	}
	/*Check if the the recipient addresses are valid emailids */
	validAddress = ui_editor_do_validation(msgInfo->header, editor_info->addressBuffers);
	valid = ui_editor_check_address_status(validAddress, msgInfo->header, body);
	if (valid == FALSE) {
		editor_info->msgInfo->header = NULL;
		g_signal_handlers_unblock_by_func(editor_info->editor_appview,
						  ui_editor_save_all_in_drafts, editor_info);
		stop_editor_from_recieving_events(editor_info, FALSE);
		g_signal_handlers_unblock_by_func(win, ui_editor_send_msg_cb, editor_info);
		return;
	}
	/*Check if the the subject of the message is NULL */
	if (editor_info->msgInfo->header->subject == NULL) {
		response =
		    get_user_confirmation(_("mcen_nc_subject_is_empty_send"), NULL, NULL,
					  CONFIRM_DIALOG);
		if (response == FALSE) {
			ui_free_msgheader(msgInfo->header);
			editor_info->msgInfo->header = NULL;
			g_free(body);
			g_signal_handlers_unblock_by_func(editor_info->editor_appview,
							  ui_editor_save_all_in_drafts,
							  editor_info);
			stop_editor_from_recieving_events(editor_info, FALSE);
			g_signal_handlers_unblock_by_func(win, ui_editor_send_msg_cb, editor_info);
			return;
		}
	}
	/*Check if the body  of the message is NULL */
	response = ui_check_for_empty_message_body(body, msgInfo->header, editor_info);
	if (response == FALSE) {
		editor_info->msgInfo->header = NULL;
		g_signal_handlers_unblock_by_func(editor_info->editor_appview,
						  ui_editor_save_all_in_drafts, editor_info);
		stop_editor_from_recieving_events(editor_info, FALSE);
		g_signal_handlers_unblock_by_func(win, ui_editor_send_msg_cb, editor_info);
		return;
	}
/*Do smime validation*/
#if SMIME
	if (ui_editor_smime_validations(msgInfo) == FALSE) {
		ui_free_msgheader(msgInfo->header);
		editor_info->msgInfo->header = NULL;
		g_free(body);
		g_signal_handlers_unblock_by_func(editor_info->editor_appview,
						  ui_editor_save_all_in_drafts, editor_info);
		stop_editor_from_recieving_events(editor_info, FALSE);
		g_signal_handlers_unblock_by_func(win, ui_editor_send_msg_cb, editor_info);
		return;
	}
#endif

	bufferBody = gtk_text_view_get_buffer((GtkTextView *) editor_info->msgFields->body);
	/*fix for crashing if selection is present in the body */
	gtk_text_buffer_get_start_iter(bufferBody, &start);
	gtk_text_buffer_move_mark_by_name(bufferBody, "insert", &start);
	gtk_text_buffer_move_mark_by_name(bufferBody, "selection_bound", &start);

	/*Call AL function for sending mail */
	if (editor_info->emailType == HTML_EDITOR) {
		body_text = g_string_new(NULL);

		/*This is to convert the formatted textin gtk to html */
		osso_email_format_to_html_text(editor_info, &(body_text->str));

		/*Adds the <href> tag to any urls/email present in body */
		body_text->len = strlen(body_text->str);
		hyperlinkify_plain_text(body_text);

		/*to avoid showing of tags in buffer we set the plain text */
		gtk_text_buffer_set_text(bufferBody, body, -1);
	} else {
		body_text = g_string_new(body);
	}
	if (email_main_view->tags->progress_bar_available == TRUE &&
	    (email_main_view->tags->current_mode == SENDING ||
	     email_main_view->tags->current_mode == CONNECTING))
		busy = TRUE;

	if (mail != DRAFT_MAIL && mail != OUTBOX_MAIL) {
		if (busy == TRUE)
			ui_editor_save_msg(editor_info, body_text->str, TRUE);
		else
			ui_dbus_send_mail(editor_info->msgInfo, body_text->str);

		/*Refresh SENT folder if it is the current view */
		if (email_main_view->folder_manager->currently_displayed_model == SENT_MODEL_NUM) {
			ui_folderview_selection_changed_cb(selection, NULL);
		}
	} else if (mail == DRAFT_MAIL || mail == OUTBOX_MAIL)
		ui_send_mail_from_draft(msgInfo, body_text->str, selection, editor_info, busy);

	g_free(body);
	g_string_free(body_text, TRUE);


	if (busy == TRUE) {
		/*msgInfo->header will be freed while closing after saving in outbox */
		editor_info->editorModified = FALSE;
		return;
	}

	ui_update_window_pointers(editor_info->msgid);
	/* Refreashing unread and unsent message count to folders in folder list view */
	if (editor_info->folderNum >= 0)
		ui_add_folder_name_with_unread_unsent_mails(editor_info->folderNum);

	/*Update Address Book - adding new addrs as new contacts and updating recently used list */
	open_addressbook();
	get_recipients_list_to_add_to_addbook(editor_info->addressBuffers, &used_econtact_uids, &new_emailids);
	add_contacts_to_recent_list(used_econtact_uids);
	add_contacts_to_addressbook(new_emailids);
	stop_addressbook();

	/*Freeing all the memory allocated.Before destroying window. */
	g_slist_foreach(used_econtact_uids, (GFunc) g_free, NULL);
	g_slist_free(used_econtact_uids);
	g_slist_foreach(new_emailids, (GFunc) g_free, NULL);
	g_slist_free(new_emailids);
	ui_free_editorinfo(editor_info);

	ui_show_next_window();

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by higher layers for registering  the callback for
	enabling/disabling the  cc field.
 @param win is a pointer to the menu item
 @param editor_info is the EditorInfo structure pointer of the current editor
	This parameter is not freed in this structure.
 @return void
*/

void ui_msg_showCc_cb(GtkWidget * win, const EditorInfo * editorInfo)
{
	MsgFields *fields = NULL;
	EmailSpecificSettings *settings = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (editorInfo == NULL) {
		return;
	}
	fields = editorInfo->msgFields;
	settings = editorInfo->msgInfo->settings;
	if (ui_get_cc_field_status() == FALSE) {
		show_cc_field(fields, TRUE);
		ui_set_cc_field_status(TRUE);
		set_security_icon(settings->smime_type, fields);
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return;
	} else {
		show_cc_field(fields, FALSE);
		ui_set_cc_field_status(FALSE);
		set_security_icon(settings->smime_type, fields);
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return;
	}
	return;
}


/**
 This function is called by higher layers for registering  the callbacks.
 @param win is a pointer to the menu item
 @param editor_info is the EditorInfo structure pointer of the current editor
	This parameter is not freed in this structure.
 @return void
*/

void ui_msg_showBcc_cb(GtkWidget * win, const EditorInfo * editorInfo)
{
	MsgFields *fields = NULL;
	EmailSpecificSettings *settings = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (editorInfo == NULL) {
		return;
	}
	fields = editorInfo->msgFields;
	settings = editorInfo->msgInfo->settings;
	if (ui_get_bcc_field_status() == FALSE) {
		show_bcc_field(fields, TRUE);
		ui_set_bcc_field_status(TRUE);
		set_security_icon(settings->smime_type, fields);
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return;
	} else {
		show_bcc_field(fields, FALSE);
		ui_set_bcc_field_status(FALSE);
		set_security_icon(settings->smime_type, fields);
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return;
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 This function is called by higher layers for registering  the callbacks \
	for implementing cut operation.
	
 @param win is a pointer to the menu item
 @param window is a pointer to the editor window.
 
 @return void
*/

void ui_msg_cut_cb(GtkWidget * win, MsgFields * fields)
{
	GtkWidget *view = NULL;
	GtkTextBuffer *clipbuffer = NULL;
	GtkClipboard *clipboard = NULL;	/*clipboard pointer for implementing cut/copy/paste */
	GtkTextIter start, end;
	gchar *text = NULL;

	clipboard = gtk_clipboard_get(GDK_NONE);

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (win == NULL) {
		return;
	}
	view = gtk_window_get_focus((GtkWindow *) app);
	if (GTK_IS_TEXT_VIEW(view) == TRUE) {
		clipbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
		if (view == fields->body)
			gtk_text_buffer_cut_clipboard(clipbuffer, clipboard, TRUE);
		else if (gtk_text_buffer_get_selection_bounds(clipbuffer, &start, &end)) {
			/*fix for bug 19673, when copying to clipboard ,from address field,
			 * tags are removed.*/
			text = gtk_text_buffer_get_text(clipbuffer, &start, &end, -1);
			gtk_text_buffer_delete(clipbuffer, &start, &end);
			gtk_clipboard_set_text(clipboard, text, strlen(text));
			g_free(text);
		}
	} else if (GTK_IS_EDITABLE(view) == TRUE) {
		gtk_editable_cut_clipboard(GTK_EDITABLE(view));
	}

	/*Show 'copied' banner */
	gtk_confirmation_banner_with_icon_name((GtkWindow *) app,
					       _("mail_ib_copied_to_clipboard"),
					       "qgn_note_infoprint");

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called by higher layers for registering  the callbacks
for implementing copy function.

@param win is a pointer to the menu item
@param window is a pointer to the editor window.

@return void
*/

void ui_msg_copy_cb(GtkWidget * win, MsgFields * fields)
{
	GtkWidget *view = NULL;
	GtkTextBuffer *clipbuffer = NULL;
	GtkClipboard *clipboard = NULL;	/*clipboard pointer for implementing cut/copy/paste */
	GtkTextIter start, end;
	gchar *text = NULL;
	gboolean text_copied = FALSE;

	clipboard = gtk_clipboard_get(GDK_NONE);
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (win == NULL || fields == NULL) {
		return;
	}
	view = gtk_window_get_focus((GtkWindow *) app);
	if (GTK_IS_TEXT_VIEW(view) == TRUE) {
		clipbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
		if(!gtk_text_buffer_get_selection_bounds(clipbuffer, &start, &end)) {
			gtk_infoprint(GTK_WINDOW(app),_("mcen_ib_nothing_to_copy"));
			return;
		}
		if (view == fields->body) {
			gtk_text_buffer_copy_clipboard(clipbuffer, clipboard);
			text_copied = TRUE;
		} else if (gtk_text_buffer_get_selection_bounds(clipbuffer, &start, &end)) {
			/*fix for bug 19673, when copying to clipboard ,from address field,
			 * tags are removed.*/
			text = gtk_text_buffer_get_text(clipbuffer, &start, &end, -1);
			gtk_clipboard_set_text(clipboard, text, strlen(text));
			g_free(text);
			text_copied = TRUE;
		}
	}

	else if (GTK_IS_EDITABLE(view) == TRUE) {
		if(!gtk_editable_get_selection_bounds(GTK_EDITABLE(view),NULL,NULL)) {
			gtk_infoprint(GTK_WINDOW(app),_("mcen_ib_nothing_to_copy"));
			return;
		}

		gtk_editable_copy_clipboard(GTK_EDITABLE(view));
		text_copied = TRUE;
	} else if (GTK_IS_TREE_VIEW(view)==TRUE) {
		gtk_infoprint(GTK_WINDOW(app),_("mcen_ib_unable_to_copy"));
		return;
	} else {
		/*Copy contents from the viewer body */
		if (mail_viewer_copy_selection(MAIL_VIEWER(fields->mail_viewer)))
			text_copied = TRUE;

	}

	/*Show 'copied' banner */
	if (text_copied) {
		gtk_confirmation_banner_with_icon_name((GtkWindow *) app,
						       _("mail_ib_copied_to_clipboard"),
						       "qgn_note_infoprint");
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by higher layers .
 
 @param clipboard is a pointer to the GtkClipboard.
 @param selection_data is a pointer to the selected buffer data.
 @param *clipbuffer the editable widget
 
 @return void
 */
static void
paste_in_body_cb(GtkClipboard * clipboard, gchar * selection_data, EditorInfo * editorinfo)
{
	GtkTextIter start;
	GtkTextIter end;
	GtkTextBuffer *clipbuffer = NULL;
	GtkWidget *view = NULL;

	view = gtk_window_get_focus((GtkWindow *) app);
	if (GTK_IS_TEXT_VIEW(view) == FALSE) {
		return;
	}

	clipbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
	/*signal is blocked so that paste of text having both
	   formatted & non formated constitute one operation */
	block_signals_upon_paste(clipbuffer, editorinfo);

	gtk_text_buffer_get_selection_bounds(clipbuffer, &start, &end);
	if (editorinfo->insert_mark == NULL) {
		editorinfo->insert_mark =
		    gtk_text_buffer_create_mark(clipbuffer, NULL, &start, TRUE);
	} else {
		gtk_text_buffer_move_mark(clipbuffer, editorinfo->insert_mark, &start);
	}
	gtk_text_buffer_paste_clipboard(clipbuffer, clipboard, NULL, TRUE);
}

/**
 It deletes the pasted text from the target if the paste operation is done
 through VKB.
 
 @param selection_data is a pointer to the clipboard data.
 @param widget is textview/GtkEntry if paste is done through VKB, or the
 'Edit->Paste' menu item if paste is done from email menu.
 
 @return void
*/
static void delete_im_pasted_text(gchar * selection_data, GtkWidget * widget)
{
	GtkTextIter start, end;
	GtkTextBuffer *buffer = NULL;
	gint cur_pos = 0;

	/*the widget is a gtktextview. */
	if (GTK_IS_TEXT_VIEW(widget)) {
		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
		/*Start iter = (End iter) - (strlen of clipboard data) */
		gtk_text_buffer_get_iter_at_mark(buffer, &end, gtk_text_buffer_get_insert(buffer));
		start = end;
		gtk_text_iter_backward_chars(&start, g_utf8_strlen(selection_data, -1));
		/*Delete the pasted data */
		gtk_text_buffer_delete(buffer, &start, &end);
	}

	/*the widget is a gtkentry. */
	if (GTK_IS_EDITABLE(widget)) {
		cur_pos = gtk_editable_get_position(GTK_EDITABLE(widget));
		gtk_editable_delete_text(GTK_EDITABLE(widget),
					 cur_pos - g_utf8_strlen(selection_data, -1), cur_pos);
	}

}


/**
 This is called when the paste_target is other than rich_text_editor.
 The pasted text can contain rich_text data like bullets,numbers, etc.
 To counter this, these extra characters are removed from the copied text,
 and then inserted to the target. Target can be any of TO, CC, BCC, SUBJECT
 fields or plain_text_editor_body.

 If the paste operation is done via VKB, the data is already pre-inserted by
 the IM-Handler. So, we remove this data first and proceed as above.
 
 @param clipboard is a pointer to the GtkClipboard.
 @param selection_data is a pointer to the selected buffer data.
 @param widget is textview/GtkEntry if paste is done through VKB, or the
 'Edit->Paste' menu item if paste is done from email menu.
 
 @return void
*/
static void paste_function_cb(GtkClipboard * clipboard, gchar * selection_data, GtkWidget * widget)
{
	GtkTextIter sel_start, sel_end, insertiter;
	GtkTextBuffer *clipbuffer = NULL, *tempbuffer = NULL;
	gchar *temptext = NULL;
	gint insertpos = 0;
	GtkWidget *view = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*probably image is selected.It canot be pasted in header field */
	if (!selection_data) {
		gtk_infoprint(GTK_WINDOW(app), _("mcen_ib_unable_to_paste"));
		return;
	}

	delete_im_pasted_text(selection_data, widget);

	view = gtk_window_get_focus(GTK_WINDOW(app));
	tempbuffer = gtk_text_buffer_new(NULL);
	gtk_text_buffer_set_text(tempbuffer, selection_data, -1);

	remove_list_chars_for_paste(tempbuffer);
	gtk_text_buffer_get_bounds(tempbuffer, &sel_start, &sel_end);
	temptext = gtk_text_buffer_get_text(tempbuffer, &sel_start, &sel_end, FALSE);

	if (GTK_IS_TEXT_VIEW(view)) {
		clipbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
		/*delete selected text if present and insert copied text. */
		gtk_text_buffer_delete_selection(clipbuffer, FALSE, TRUE);
		gtk_text_buffer_get_iter_at_mark(clipbuffer, &insertiter,
						 gtk_text_buffer_get_insert(clipbuffer));
		gtk_text_buffer_insert(clipbuffer, &insertiter, temptext, -1);
	} else if (GTK_IS_EDITABLE(view)) {
		/*delete selected text if present and insert copied text. */
		gtk_editable_delete_selection(GTK_EDITABLE(view));
		insertpos = gtk_editable_get_position(GTK_EDITABLE(view));
		gtk_editable_insert_text(GTK_EDITABLE(view), temptext, strlen(temptext),
					 &insertpos);
		gtk_editable_set_position(GTK_EDITABLE(view), insertpos);
	}
	g_free(temptext);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 It is called as callback for paste operation by 'Edit->Paste' menu item
 or through VKB.
 
 @param win is textview/GtkEntry if paste is done through VKB, or the
 'Edit->Paste' menu item if paste is done from email menu.
 @param editorinfo is a pointer to the EditorInfo.
 
 @return void
*/

void ui_msg_paste_cb(GtkWidget * win, EditorInfo * editorinfo)
{
	GtkWidget *view = NULL;
	GtkClipboard *clipboard = NULL;	//clipboard pointer for implementing cut/copy/paste

	clipboard = gtk_clipboard_get(GDK_NONE);

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (win == NULL || editorinfo == NULL) {
		return;
	}

	if (osso_email_clipboard_has_emails_for_attachment(editorinfo->editor_appview)) {
		ULOG_DEBUG("email in clipboard");
		ui_paste_messages_cb(win, editorinfo);
		return;
	}
	view = gtk_window_get_focus((GtkWindow *) app);
	if ((view == editorinfo->msgFields->body)
	    && (editorinfo->msgInfo->settings->type == EMAIL_HTML)) {
		gtk_clipboard_request_text(clipboard, (GtkClipboardTextReceivedFunc)
					   paste_in_body_cb, editorinfo);
		/*carry out paste completion */
		gtk_clipboard_request_text(clipboard, (GtkClipboardTextReceivedFunc)
					   look_for_paste_completion, editorinfo);
	} else {
		gtk_clipboard_request_text(clipboard, (GtkClipboardTextReceivedFunc)
					   paste_function_cb, win);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}



/**
 This function is called by higher layers for getting the word wrap.
 
 @param iter is a pointer to the GtkTextIter with in where is the selected word.
 @param edstart is a pointer to the GtkTextIter with in where is the selected word started
     (OUT param).
 @param edend is a pointer to the GtkTextIter with in where is the selected word ended
     (OUT param).
     
 @return void
*/
static void ui_getWord_wrap(GtkTextIter iter, GtkTextIter * edstart, GtkTextIter * edend)
{
	GtkTextIter matchstart;
	GtkTextIter matchend;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (gtk_text_iter_forward_search
	    (&iter, ADDR_SEPERATOR_CHAR, GTK_TEXT_SEARCH_VISIBLE_ONLY,
	     &matchstart, &matchend, NULL) == TRUE) {
		*edend = matchstart;
	} else if (gtk_text_iter_forward_search
		   (&iter, ATTACHMENT_SEPERATOR_CHAR, GTK_TEXT_SEARCH_VISIBLE_ONLY,
		    &matchstart, &matchend, NULL) == TRUE) {
		*edend = matchstart;
	} else {
		gtk_text_iter_forward_to_end(&iter);
		*edend = iter;
	}

	if (gtk_text_iter_backward_search
	    (&iter, ADDR_SEPERATOR_CHAR, GTK_TEXT_SEARCH_VISIBLE_ONLY,
	     &matchstart, &matchend, NULL) == TRUE) {
		*edstart = matchend;
	} else if (gtk_text_iter_backward_search
		   (&iter, ATTACHMENT_SEPERATOR_CHAR, GTK_TEXT_SEARCH_VISIBLE_ONLY,
		    &matchstart, &matchend, NULL) == TRUE) {
		*edstart = matchend;
	} else {
		gtk_text_iter_set_offset(&iter, 0);
		*edstart = iter;
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 This is a helper function for enabling & disabling of widgets based on
   particular selection.
 @param viewer_info is  pointer the ViewerInfo structure of the current viewer
 	This parameter is not freed.
 @returns void.
*/
static void ui_manage_widgets_upon_selection(const ViewerInfo * viewer_info)
{
	GtkWidget *menuSave = NULL;
	GtkWidget *toolBarSave = NULL;
	GtkWidget *menuView = NULL;
	GtkWidget *menuFind = NULL;
	GtkWidget *copy_menuitem = NULL;
	ViewerToolbar *toolbar = NULL;
	ViewerMenu *menu = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(viewer_info == NULL, "Input param failed", return);
	CHECK_PARAM(viewer_info->menu == NULL, "Input param failed", return);
	CHECK_PARAM(viewer_info->toolbar == NULL, "Input param failed", return);

	toolbar = viewer_info->toolbar;
	menu = viewer_info->menu;

	menuSave = menu->save_attachment;
	toolBarSave = toolbar->saveAttachment;
	menuView = menu->view;
	menuFind = menu->find;
	copy_menuitem = menu->copy;

	gtk_widget_set_sensitive(menuSave, FALSE);
	gtk_widget_set_sensitive(menuView, FALSE);
	gtk_widget_set_sensitive(viewer_info->menu->purge_attachment, FALSE);
	gtk_widget_set_sensitive(toolBarSave, FALSE);
	gtk_widget_set_sensitive(copy_menuitem, TRUE);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}

/**
 This is a helper function to remove the already present selection in textBuffer.
 
 @param msgfields is the pointer to the MsgFields structure containing all the fields of the mail.
  This parameter is not freed here.
  
 @returns void.
*/
static void ui_unselect_text_buffer(const MsgFields * msgfields)
{
	GtkTextBuffer *fromBuffer = NULL;
	GtkTextBuffer *toBuffer = NULL;
	GtkTextBuffer *ccBuffer = NULL;
	GtkTextIter sel_start, sel_end;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(msgfields == NULL, "Input param check failed\n", return);

	CHECK_PARAM(msgfields->from == NULL, "Input param check failed\n", return);
	CHECK_PARAM(msgfields->to == NULL, "Input param check failed\n", return);

	CHECK_PARAM(msgfields->cc == NULL, "Input param check failed\n", return);
	fromBuffer = gtk_text_view_get_buffer((GtkTextView *) msgfields->from);
	toBuffer = gtk_text_view_get_buffer((GtkTextView *) msgfields->to);
	ccBuffer = gtk_text_view_get_buffer((GtkTextView *) msgfields->cc);

	if (gtk_text_buffer_get_selection_bounds(fromBuffer, &sel_start, &sel_end)) {
		gtk_text_buffer_move_mark_by_name(fromBuffer, "insert", &sel_start);
		gtk_text_buffer_move_mark_by_name(fromBuffer, "selection_bound", &sel_start);
	}
	if (gtk_text_buffer_get_selection_bounds(toBuffer, &sel_start, &sel_end)) {
		gtk_text_buffer_move_mark_by_name(toBuffer, "insert", &sel_start);
		gtk_text_buffer_move_mark_by_name(toBuffer, "selection_bound", &sel_start);
	}
	if (gtk_text_buffer_get_selection_bounds(ccBuffer, &sel_start, &sel_end)) {
		gtk_text_buffer_move_mark_by_name(ccBuffer, "insert", &sel_start);
		gtk_text_buffer_move_mark_by_name(ccBuffer, "selection_bound", &sel_start);
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by higher layers for makeing the selection.
	& registering the callback
 @param view is a pointer to the textview of address field.
 @param event is a pointer to the GdkEvent that generated button release.
 @param viewer_info is the viewerInfo structure pointer.
	This parameter is not freed here.
 @return gboolean TRUE-to stop the event from propagating
                 FALSE-else.
*/
gboolean make_selection(GtkTextView * view, GdkEvent * event, ViewerInfo * viewer_info)
{
	GtkTextIter start, end, iter;
	GtkTextBuffer *current_buffer = NULL;
	gdouble xwin = 0;
	gdouble ywin = 0;
	gint xbuf = 0;
	gint ybuf = 0;

	current_buffer = gtk_text_view_get_buffer(view);
	gtk_widget_grab_focus((GtkWidget *) view);
	if (gtk_text_buffer_get_selection_bounds(current_buffer, &start, &end) == FALSE) {
		return FALSE;
	}
	CHECK_PARAM(view == NULL, "Input param validation failed", return FALSE);
	gtk_text_buffer_move_mark_by_name(current_buffer, "insert", &start);
	gtk_text_buffer_move_mark_by_name(current_buffer, "selection_bound", &end);

	gdk_event_get_coords(event, &xwin, &ywin);
	gtk_text_view_window_to_buffer_coords(view, GTK_TEXT_WINDOW_TEXT, xwin, ywin, &xbuf, &ybuf);
	gtk_text_view_get_iter_at_location(view, &iter, xbuf, ybuf);

	ui_getWord_wrap(iter, &start, &end);
	gtk_text_buffer_select_range(current_buffer, &start, &end);
	return TRUE;

}



/**
 This function is called by higher layers for makeing the selection.
	& registering the callback
 @param win is a pointer to the menu item
 @param event is a pointer to the GdkEvent that generated.
 @param viewer_info is the viewerInfo structure pointer.
	This parameter is not freed here.
 @return gboolean TRUE-to stop the event from propagating
                 FALSE-else.
*/
gboolean ui_make_selection_cb(GtkTextView * view, GdkEvent * event, ViewerInfo * viewer_info)
{
	GtkTextIter iter, edstart, edend;
	gdouble xwin = 0;
	gdouble ywin = 0;
	gint xbuf = 0;
	gint ybuf = 0;
	GtkTextBuffer *current_buffer = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM(view == NULL, "Input param validation failed", return FALSE);
	CHECK_PARAM(viewer_info == NULL, "Input param validation failed", return FALSE);

	gtk_widget_grab_focus((GtkWidget *) view);
	gdk_event_get_coords(event, &xwin, &ywin);
	gtk_text_view_window_to_buffer_coords(view, GTK_TEXT_WINDOW_TEXT, xwin, ywin, &xbuf, &ybuf);
	gtk_text_view_get_iter_at_location(view, &iter, xbuf, ybuf);
	if (gtk_text_iter_inside_word(&iter) == FALSE) {

		ui_unselect_text_buffer(viewer_info->msg_fields);
		ui_manage_widgets_upon_selection(viewer_info);

	} else {
		ui_unselect_text_buffer(viewer_info->msg_fields);
		ui_getWord_wrap(iter, &edstart, &edend);
		current_buffer = gtk_text_view_get_buffer(view);
		ui_flush_gtk_events();
		if (current_buffer != NULL) {
			gtk_text_buffer_move_mark_by_name(current_buffer, "insert", &edstart);
			gtk_text_buffer_move_mark_by_name(current_buffer, "selection_bound",
							  &edend);
		}

		ui_manage_widgets_upon_selection(viewer_info);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return TRUE;


}

/**
 This function is called by higher layers for makeing the selection.
	& registering the callback
 @param view is GtkTextView
 @param event is a pointer to the GdkEvent that generated.
 @param data is dummy param
 @return gboolean FALSE-to propagate the event.
                  TRUE-else.
*/
gboolean ui_editor_select_emailid(GtkTextView * view, GdkEvent * event, gpointer data)
{
	GtkTextIter iter, start, end;
	gdouble xwin = 0;
	gdouble ywin = 0;
	gint xbuf = 0;
	gint ybuf = 0;
	GtkTextBuffer *current_buffer = NULL;
	GtkTextTag *contact_tag = NULL;
	gint type = 0;
	GSList *tags = NULL, *tagp = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(view == NULL, "Input param validation failed", return FALSE);

	gdk_event_get_coords(event, &xwin, &ywin);
	gtk_text_view_window_to_buffer_coords(view, GTK_TEXT_WINDOW_TEXT, xwin, ywin, &xbuf, &ybuf);
	gtk_text_view_get_iter_at_location(view, &iter, xbuf, ybuf);
	current_buffer = gtk_text_view_get_buffer(view);
	start = iter;
	end = iter;
	tags = gtk_text_iter_get_tags(&iter);
	for (tagp = tags; tagp != NULL; tagp = tagp->next) {
		contact_tag = tagp->data;
		type = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(contact_tag), "emailid"));
		if (type == EMAIL_ID) {
			if (gtk_text_iter_begins_tag(&start, contact_tag)) {
				gtk_text_iter_forward_to_tag_toggle(&end, contact_tag);
			} else if (gtk_text_iter_ends_tag(&end, contact_tag)) {
				gtk_text_iter_backward_to_tag_toggle(&start, contact_tag);
			} else if (gtk_text_iter_has_tag(&iter, contact_tag)) {
				gtk_text_iter_forward_to_tag_toggle(&end, NULL);
				gtk_text_iter_backward_to_tag_toggle(&start, NULL);
				gtk_text_buffer_place_cursor(current_buffer, &end);
			} else
				return FALSE;

			gtk_text_buffer_move_mark_by_name(current_buffer, "insert", &start);
			gtk_text_buffer_move_mark_by_name(current_buffer, "selection_bound", &end);
		}
	}
	/*Node contents are intergers need not free them */
	g_slist_free(tags);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return FALSE;
}

/**
 This function is called move the cursor over the resolved email-id.
	& registering the callback
	
 @param view is a pointer to the GtkTextView
 @param event is a pointer to the GdkEvent that generated.
 @param data is dummy param
 
 @return gboolean FALSE-propagate the event.
 	          TRUE-else.
*/
gboolean skip_resolved_emailid(GtkWidget * view, GdkEventKey * event, gpointer data)
{
	GtkTextIter iter, sel_start, sel_end;
	GtkTextBuffer *current_buffer = NULL;
	GtkTextMark *insert_mark = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (view == NULL || event == NULL) {
		return FALSE;
	}

	if (event->keyval != GDK_Right && event->keyval != GDK_Left
	    && event->keyval != GDK_Up && event->keyval != GDK_Down) {
		return FALSE;
	}

	current_buffer = gtk_text_view_get_buffer((GtkTextView *) view);
	insert_mark = gtk_text_buffer_get_insert(current_buffer);
	gtk_text_buffer_get_iter_at_mark(current_buffer, &iter, insert_mark);
	if (gtk_text_buffer_get_selection_bounds(current_buffer, &sel_start, &sel_end)) {
		if (event->keyval == GDK_Right) {
			gtk_text_iter_backward_char(&sel_end);
			gtk_text_buffer_place_cursor(current_buffer, &iter);
			gtk_text_buffer_move_mark_by_name(current_buffer, "insert", &sel_end);
		} else if (event->keyval == GDK_Left) {
			//gtk_text_iter_forward_char(&sel_end);
			gtk_text_iter_forward_char(&sel_start);
			gtk_text_buffer_place_cursor(current_buffer, &iter);
			gtk_text_buffer_move_mark_by_name(current_buffer, "insert", &sel_start);
		}
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return FALSE;
}

/**
 This function is called by higher layers for getting the emailid wrap.
 
 @param iter is a pointer to the GtkTextIter with in where is the selected word.
 @param edstart is a pointer to the GtkTextIter with in where is the selected word started
     (OUT param).
 @param edend is a pointer to the GtkTextIter with in where is the selected word ended
     (OUT param).
 @param buffer is the GtkTextBuffer.
 @return void
*/
static void
ui_get_emailid_wrap(GtkTextIter iter, GtkTextIter * start, GtkTextIter * end,
		    GtkTextBuffer * buffer)
{
	gchar *text = NULL;

	ui_getWord_wrap(iter, start, end);
	text = gtk_text_buffer_get_text(buffer, start, end, -1);
	g_strstrip(text);
	gtk_text_iter_forward_search(start, text, GTK_TEXT_SEARCH_VISIBLE_ONLY, start, end, NULL);
	g_free(text);
	return;
}


/**
 This function is called move the cursor over the resolved email-id.
    & registering the callback
    
 @param view is a pointer to the GtkTextView
 @param event is a pointer to the GdkEvent that generated.
 @param data is dummy param
 
 @return gboolean
*/
gboolean skip_emailid_on_key_release(GtkWidget * view, GdkEventKey * event, gpointer data)
{
	GtkTextIter iter, start, end;
	GtkTextBuffer *current_buffer = NULL;
	GtkTextMark *insert_mark = NULL;
	GtkTextTag *contact_tag = NULL;
	gint type = 0;
	GSList *tags = NULL, *tagp = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (view == NULL || event == NULL) {
		return FALSE;
	}

	if (event->keyval != GDK_Up && event->keyval != GDK_Down
	    && event->keyval != GDK_Left && event->keyval != GDK_Right) {
		return FALSE;
	}

	current_buffer = gtk_text_view_get_buffer((GtkTextView *) view);
	insert_mark = gtk_text_buffer_get_insert(current_buffer);
	gtk_text_buffer_get_iter_at_mark(current_buffer, &iter, insert_mark);
	tags = gtk_text_iter_get_tags(&iter);
	if (gtk_text_buffer_get_selection_bounds(current_buffer, &start, &end) == TRUE)
		return FALSE;
	for (tagp = tags; tagp != NULL; tagp = tagp->next) {
		contact_tag = tagp->data;
		type = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(contact_tag), "emailid"));
		if (type == EMAIL_ID) {
			if (event->keyval == GDK_Left) {
				if (gtk_text_iter_is_start(&iter) == FALSE
				    && gtk_text_iter_has_tag(&iter, contact_tag)) {
					gtk_text_iter_backward_char(&iter);
					if (gtk_text_iter_has_tag(&iter, contact_tag)) {
						gtk_text_iter_forward_char(&iter);
						gtk_text_iter_backward_to_tag_toggle(&iter, NULL);
						ui_get_emailid_wrap(iter, &start, &end,
								    current_buffer);
						gtk_text_buffer_select_range
						    (current_buffer, &start, &end);
						break;
					} else {
						gtk_text_iter_forward_char(&iter);
					}
				}
				gtk_text_buffer_move_mark_by_name(current_buffer, "insert", &iter);
				gtk_text_buffer_move_mark_by_name(current_buffer,
								  "selection_bound", &iter);
				break;
			}
			if (event->keyval == GDK_Up || event->keyval == GDK_Down
			    || event->keyval == GDK_Right) {
				if (gtk_text_iter_has_tag(&iter, contact_tag)) {
					gtk_text_iter_backward_char(&iter);
					if (gtk_text_iter_has_tag(&iter, contact_tag)) {
						gtk_text_iter_forward_char(&iter);
						gtk_text_iter_forward_to_tag_toggle(&iter, NULL);
						ui_get_emailid_wrap(iter, &start, &end,
								    current_buffer);
						gtk_text_buffer_select_range
						    (current_buffer, &start, &end);
					} else {
						gtk_text_iter_forward_char(&iter);
						gtk_text_buffer_move_mark_by_name
						    (current_buffer, "insert", &iter);
						gtk_text_buffer_move_mark_by_name
						    (current_buffer, "selection_bound", &iter);
					}
					break;
				}
			}
		}
	}
	/*Node contents are intergers need not free them */
	g_slist_free(tags);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return FALSE;
}

/**
 This function is called by higher layers that registering the callback
	for implementing the select all functionality .
 @param win is a pointer to the menu item
 @param fields is a pointer to the MsgFields structure.

 @return void
*/
void ui_msg_select_all_cb(GtkWidget * win, MsgFields * fields)
{
	GtkWidget *view = NULL;
	GtkTextBuffer *sel_buffer = NULL;
	GtkTextIter sel_start, sel_end;

	if (win == NULL) {
		return;
	}
	/*Check for focussed widget */
	view = gtk_window_get_focus(GTK_WINDOW(app));
	if (GTK_IS_EDITABLE(view) == TRUE) {
		gtk_editable_select_region(GTK_EDITABLE(view), 0, -1);
	} else if (GTK_IS_TEXT_VIEW(view)) {
		sel_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
		gtk_text_buffer_get_bounds(sel_buffer, &sel_start, &sel_end);
		gtk_text_buffer_select_range(sel_buffer, &sel_start, &sel_end);
	} else if (GTK_IS_TREE_VIEW(view)) {
		/*do nothing if focus is on attachment field */
	} else {
		mail_viewer_select_all(MAIL_VIEWER(fields->mail_viewer));
	}
	return;
}

/**
 This function is called by higher layers for  registering the callback
	to add a contact to addressbook.
	
 @param win is a pointer to the menu item
 @param recpbuffer is a pointer to the GtkTextBuffer.
 
 @return void
*/
void ui_editor_openAddrBook_cb(GtkWidget * win, GdkEvent * event, GtkTextBuffer * recpBuffer)
{
	CHECK_PARAM(recpBuffer == NULL, "Input param failed", return)
	    start_email_recipients_dialog(recpBuffer);

	return;
}

/**
  This function is called by higher layers for  registering the callback
  to add a contact to addressbook.
  
  @param win is a pointer to the menu item
  @param fields is a pointer to the MsgFields structure.
  
  @return void
*/
void ui_editor_select_recipients_cb(GtkWidget * win, MsgFields * fields)
{
	GtkWidget *addressfield = NULL;
	GtkTextBuffer *addrbuffer = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(fields == NULL, "Input param check failed\n", return);
	addressfield = gtk_window_get_focus((GtkWindow *) app);
	if (GTK_IS_TEXT_VIEW(addressfield) == TRUE) {
		if (addressfield != fields->body) {
			addrbuffer = gtk_text_view_get_buffer((GtkTextView *) addressfield);

		} else {
			gtk_widget_grab_focus(fields->to);
			addrbuffer = gtk_text_view_get_buffer((GtkTextView *) fields->to);
		}
	} else {
		gtk_widget_grab_focus(fields->to);
		addrbuffer = gtk_text_view_get_buffer((GtkTextView *) fields->to);
	}
	ui_editor_openAddrBook_cb(win, NULL, addrbuffer);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/*
 This function helps in finding the corresponding icon for given mime type

 @param mime_string is the gchar representing the mimetype of the attacheed fle.
 @param icon_string is the  OUT param that specifies the icon name.

 @return void.
*/
void
prepare_the_icon_string(const gchar * mime_string, gchar ** icon_string,
			gboolean is_downloaded, gboolean is_purged)
{
	OssoMimeCategory mime_category = OSSO_MIME_CATEGORY_BOOKMARKS;
	osso_log(LOG_INFO, "[%s],[%s]:starts here\n", __FILE__, __FUNCTION__);
	if (mime_string == NULL) {
		osso_log(LOG_INFO, "[%s],[%s]:the mime type is null:-((\n", __FILE__, __FUNCTION__);
		return;
	}

	/*checking the icon for Vcard */
	if (g_ascii_strncasecmp(mime_string, VCARD, strlen(VCARD)) == 0
	    || g_ascii_strncasecmp(mime_string, VCALENDAR, strlen(VCALENDAR)) == 0) {
		if (is_downloaded == TRUE)
			*icon_string = "qgn_indi_gene_vCard_attachment";
		else if (is_purged == TRUE)
			*icon_string = "qgn_indi_gene_vCard_attachment_purged";
		else
			*icon_string = "qgn_indi_gene_vCard_attachment_notretrieved";

		return;
	}
	if (g_ascii_strncasecmp(mime_string, PDF, strlen(PDF)) == 0) {
		if (is_downloaded == TRUE)
			*icon_string = "qgn_indi_gene_pdf_attachment";
		else if (is_purged == TRUE)
			*icon_string = "qgn_indi_gene_pdf_attachment_purget";
		else
			*icon_string = "qgn_indi_gene_pdf_attachment_notretrieved";
		return;
	}
	if (g_ascii_strncasecmp(mime_string, SKETCH, strlen(SKETCH)) == 0) {
		if (is_downloaded == TRUE)
			*icon_string = "qgn_indi_gene_sketch_attachment";
		else if (is_purged == TRUE)
			*icon_string = "qgn_indi_gene_sketch_attachment_purget";
		else
			*icon_string = "qgn_indi_gene_sketch_attachment_notretrieved";
		return;
	}

	/*We get the category in which the mime-type belong, & get the
	 * corresponding icon for it */
	mime_category = osso_mime_get_category_for_mime_type(mime_string);
	switch (mime_category) {
	case OSSO_MIME_CATEGORY_IMAGES:

		if (is_downloaded == TRUE)
			*icon_string = "qgn_indi_gene_images_attachment";
		else if (is_purged == TRUE)
			*icon_string = "qgn_indi_gene_images_attachment_purged";
		else
			*icon_string = "qgn_indi_gene_images_attachment_notretrieved";
		break;

	case OSSO_MIME_CATEGORY_VIDEO:
		if (is_downloaded == TRUE)
			*icon_string = "qgn_indi_gene_videopl_attachment";
		else if (is_purged == TRUE)
			*icon_string = "qgn_indi_gene_videopl_attachment_purged";
		else
			*icon_string = "qgn_indi_gene_videopl_attachment_notretrieved";
		break;
	case OSSO_MIME_CATEGORY_AUDIO:
		if (is_downloaded == TRUE)
			*icon_string = "qgn_indi_gene_sound_attachment";
		else if (is_purged == TRUE)
			*icon_string = "qgn_indi_gene_sound_attachment_purged";
		else
			*icon_string = "qgn_indi_gene_sound_attachment_notretrieved";

		break;
	case OSSO_MIME_CATEGORY_DOCUMENTS:
		if (is_downloaded == TRUE)
			*icon_string = "qgn_indi_gene_note_attachment";
		else if (is_purged == TRUE)
			*icon_string = "qgn_indi_gene_note_attachment_purged";
		else
			*icon_string = "qgn_indi_gene_note_attachment_notretrieved";
		break;
	default:
		if (is_downloaded == TRUE)
			*icon_string = "qgn_indi_gene_default_file_attachment";
		else if (is_purged == TRUE)
			*icon_string = "qgn_indi_gene_default_file_attachment_purged";
		else
			*icon_string = "qgn_indi_gene_default_file_attachment_notretrieved";
		break;
	}

	osso_log(LOG_INFO, "[%s],[%s]:ends here\n", __FILE__, __FUNCTION__);
	return;
}


/**
 This function is called by higher layers for  registering the callback
 
 @param buffer is a pointer to the attachment text buffer .
 @param attachmentpath is a gchar pointer attachmentpath.
 @param attachmentSize is a gchar poiner to the size string .
 
 @return void.
*/
void
ui_show_attachments_in_buffer(GtkListStore * liststore,
			      const gchar * attachmentname, const gchar * attachmentpath,
			      const gchar * attachmentSize, gboolean mycard_attached)
{
	gchar *mime_string = NULL;
	gchar *icon_string = NULL;
	GtkTreeIter iter;
	GtkIconTheme *icontheme = NULL;
	GdkPixbuf *pixbuf = NULL;
	GError *error = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (attachmentname == NULL) {
		return;
	}
	/*for email attachments the path may be NULL */
	if (attachmentpath == NULL)
		attachmentpath = attachmentname;

	icontheme = gtk_icon_theme_get_default();
	mime_string = (gchar *) gnome_vfs_mime_type_from_name((const char *) attachmentpath);

	if (check_for_vcard_attachment((const char *) attachmentname) == TRUE) {
		mime_string = VCARD;
	}

	prepare_the_icon_string(mime_string, &icon_string, TRUE, TRUE);
	if (icon_string == NULL) {
		icon_string = "qgn_indi_gene_default_file_attachment";
	}

	pixbuf = gtk_icon_theme_load_icon(icontheme, icon_string, 20, 0, &error);
	gtk_list_store_append(liststore, &iter);
	if (attachmentname != NULL && attachmentSize != NULL) {

		/*inserting the icon,filename &size in the liststore columns 0,1,2 */
		gtk_list_store_set(liststore, &iter, COL_ICON, pixbuf, COL_ATT_NAME,
				   attachmentname, COL_ATT_SIZE, attachmentSize, -1);
	} else if (attachmentname != NULL) {
		gtk_list_store_set(liststore, &iter, COL_ICON, pixbuf, COL_ATT_NAME,
				   attachmentname, -1);
	} else if (attachmentSize != NULL) {
		gtk_list_store_set(liststore, &iter, COL_ICON, pixbuf, COL_ATT_SIZE,
				   attachmentSize, -1);
	}
	if (mycard_attached == TRUE)
		gtk_list_store_set(liststore, &iter, COL_ATT_NUM, TRUE, -1);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This is a helper function for freeing the selected file paths from the
 file_chooser_widget.
 
 @param selectionList is a GSList of file paths.
 
 @return void.
*/
void ui_file_paths_free(GSList * selections)
{
	GSList *selectionList = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	selectionList = selections;

	g_slist_foreach(selectionList, (GFunc) g_free, NULL);
	g_slist_free(selections);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}

/**
 This function is called by higher layers for  registering the callback for closing
 	the fileselector dialog.
	
 @param win is a pointer to the widget that registered the callback.
 @param window is a pointer to the fileselection dialog(user data) this is destroyed here.
 
 @return void
*/

static void ui_editor_close_fileselector_cb(GtkWidget * win, GtkWidget * window)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (window != NULL) {
		gtk_widget_destroy(GTK_WIDGET(window));
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called to run the fileselction dialog
 
 @param fileselection is the GtkWidget is the file selction widget.
 	this widget is destroyed either by calling ui_editor_insert_attachment_cb()/
	 ui_editor_close_fileselector_cb().
 @param editor_info is the EditorInfo structure pointer for the current editor.
 
 @result void.
*/
static void ui_fileselector_run(GtkWidget * fileselection, EditorInfo * editor_info)
{
	gint result = 0;

	result = gtk_dialog_run((GtkDialog *) fileselection);
	if (result == GTK_RESPONSE_OK) {
		ui_editor_insert_attachment_cb(fileselection, editor_info);
	} else if (result != GTK_RESPONSE_OK) {
		ui_editor_close_fileselector_cb(fileselection, fileselection);
	}
}

/**
  This gnome_vfs_xfer callback function  .
  
  @param info is a pointer to the GnomeVFSXferProgressInfo structure.
  @param data is a pointer to the user data.
  
  @return static int, No specfic fuctionality.just checking the status of operation
 */
gint progress_info(GnomeVFSXferProgressInfo * info, gpointer * data)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(info == NULL, "Input param check failed\n", return 0);
	ULOG_DEBUG("the status is %d\n", info->status);
	ULOG_DEBUG("the phase is %d\n", info->phase);

	/*check if error while coping is in progress */
	if (info->status == GNOME_VFS_XFER_PROGRESS_STATUS_VFSERROR)
		return 0;

	if (info->phase == GNOME_VFS_XFER_PHASE_COMPLETED) {
		ULOG_DEBUG("done with trasfering the file \n");
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return 1;

}

/**
 Creates a temporary file for attachments
 
 @param attachedpath is the gchar representing the attachment path .
 @param attachedpath is the gchar representing the attachment name .
 
 @return a newly allocated attachment path .
*/
gchar *ui_create_filepath_link(const gchar * attachedpath,
			       gchar ** attachmentName, gboolean move, gboolean encode_path)
{

	gchar *destination_dir = NULL;
	gchar *destination_path = NULL;
	gchar *encodedpath = NULL;
	gchar *tmp_attach_path = NULL;
	GnomeVFSResult result = 0;
	GnomeVFSURI *srcUri = NULL;
	GnomeVFSURI *targetUri = NULL;
	static gint i = 0;


	i++;
	if (move == FALSE) {
		ui_make_temp_dir();
		tmp_attach_path = ui_get_tmp_attach_path();
		destination_dir = g_strdup_printf("%s%d", tmp_attach_path, i);
		g_free(tmp_attach_path);
	} else {
		/*i here is added to create a unique attachment directory with 
		   suffix time + i even if instance of time is same */
		destination_dir =
		    g_strdup_printf("%s%s%u", "/var/tmp/email/", ATTACHM, (guint) time(NULL) + i);
	}

	/*create the tempoarary attachment directory */
	if (mkdir(destination_dir, S_IRWXU | S_IRWXG | S_IRWXO) == -1) {
		ULOG_CRIT("directory creation failed");
		g_free(destination_dir);
		return NULL;
	}
	destination_path =
	    g_strconcat(destination_dir, GNOME_VFS_URI_PATH_STR, *attachmentName, NULL);

	g_free(destination_dir);

	if (move == TRUE) {
		/*moving the file rather then copying */
		if (move_file(attachedpath, destination_path, TRUE) < 0) {
			osso_log(LOG_DEBUG, "unable to copy to /var/tmp");
			g_free(destination_path);
			destination_path = NULL;
		}
		return destination_path;
	}


	srcUri = gnome_vfs_uri_new(attachedpath);
	targetUri = gnome_vfs_uri_new(destination_path);


	result = gnome_vfs_xfer_uri(srcUri, targetUri, GNOME_VFS_XFER_DEFAULT,
				    GNOME_VFS_XFER_ERROR_MODE_QUERY,
				    GNOME_VFS_XFER_OVERWRITE_MODE_QUERY,
				    (GnomeVFSXferProgressCallback) progress_info, NULL);
	osso_log(LOG_INFO, "RESULT is %d\n", result);
	gnome_vfs_uri_unref(srcUri);
	gnome_vfs_uri_unref(targetUri);

	if (result == GNOME_VFS_OK || result == GNOME_VFS_ERROR_EOF) {
		if (encode_path) {
			gchar *tmp_path = NULL;
			g_free(*attachmentName);
			*attachmentName = NULL;
			tmp_path = g_strconcat("file://", destination_path, NULL);
			*attachmentName = get_short_filename((gchar *) tmp_path, &encodedpath);
			g_free(destination_path);
			destination_path = NULL;
			g_free(tmp_path);
			destination_path = encodedpath;
		}
		return destination_path;

	} else {
		g_free(destination_path);
		gtk_banner_close((GtkWindow *) app);
		if (result == GNOME_VFS_ERROR_NO_SPACE)
			get_user_confirmation(DGETTEXT_HCS("sfil_ni_not_enough_memory"), NULL, NULL,
					      INFO_BANNER);
		else
			get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
		return NULL;
	}


	return destination_path;
}

/**
 Function to copy image file to temp location

 @param filepath is the gchar representing the file path .

 @return a newly allocated temp path .
*/
static gchar *ui_create_image_file_link(const gchar * filepath)
{
	GnomeVFSResult result = 0;
	GnomeVFSURI *srcUri = NULL;
	GnomeVFSURI *targetUri = NULL;
	GnomeVFSHandle *write_handle = NULL;
	gchar *destination = NULL;
	gchar *temp_path = NULL;
	static gint i = 0;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (filepath == NULL) {
		return NULL;
	}
	temp_path = ui_get_tmp_path();
	destination = g_strdup_printf("%s%s%s%d", temp_path, GNOME_VFS_URI_PATH_STR, IMAGE_, i++);;
	g_free(temp_path);
	srcUri = gnome_vfs_uri_new(filepath);
	targetUri = gnome_vfs_uri_new(destination);

	if (gnome_vfs_uri_exists(targetUri))
		gnome_vfs_unlink(destination);

	result = gnome_vfs_xfer_uri(srcUri, targetUri, GNOME_VFS_XFER_DEFAULT,
				    GNOME_VFS_XFER_ERROR_MODE_QUERY,
				    GNOME_VFS_XFER_OVERWRITE_MODE_QUERY,
				    (GnomeVFSXferProgressCallback) progress_info, NULL);
	gnome_vfs_uri_unref(srcUri);
	gnome_vfs_uri_unref(targetUri);
	if (result == GNOME_VFS_OK)
		gnome_vfs_close(write_handle);
	if (result == GNOME_VFS_OK || result == GNOME_VFS_ERROR_FILE_EXISTS) {
		return destination;
	}
	if (result != GNOME_VFS_ERROR_INTERRUPTED)
		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	g_free(destination);
	return NULL;
}

/**
 Function to increase the size of attachment field dynamically
 based on number of attachments.
 
 @param attach_list is the GSList of attachments.
 @param fields is the pointer to the MsgFields structure.
 
 @return void.
*/
void set_attachment_view_size(GSList * attach_list, MsgFields * fields)
{
	gint num_attachments = 0;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(attach_list == NULL, "Input param failed", return);
	CHECK_PARAM(fields == NULL, "Input param failed", return);

	num_attachments = g_slist_length(attach_list);
	g_print("no of attachments %d\n", num_attachments);
	osso_log(LOG_DEBUG, "no of attachments %d\n", num_attachments);
	if (num_attachments <= 3) {
		gtk_widget_set_size_request(fields->sw_attach, DEFAULT_WIDTH,
					    ATTA_FIELDS_SIZE * num_attachments);
	} else {
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(fields->sw_attach),
					       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
		gtk_widget_set_size_request(fields->sw_attach, DEFAULT_WIDTH, MAX_ATTA_FIELDS_SIZE);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This funtion is used to get the filename from the gnomevfs file uri path.
 It also convertes the file path to proper encoding which can be used by c FILE operations.

 @param path is teh gchar representing the gnomevfs file uri path.
 @param filepath is the OUT param of for the modified path.
 @return the gchar pointer to the dispaly name of the file path.
*/
gchar *get_short_filename(const gchar * path, gchar ** filepath)
{
	gchar *file_name = NULL;
	gchar *hostname = NULL;
	GError *error = NULL;

	if (path == NULL) {
		ULOG_DEBUG_F("NULL path returned from file chooser wideget");
		return NULL;
	}

	/*gets the proper encoded file path from the path given by file chooser widget */
	*filepath = g_filename_from_uri(path, &hostname, &error);
	if (*filepath == NULL) {
		*filepath = g_strdup(path);
		g_error_free(error);
	}

	/*gets the diaplay name of the file from the path */
	file_name = g_filename_display_basename(*filepath);

	g_free(hostname);
	return file_name;
}

/**
 This function is used to fill the attachinfo structure & append it to the attachment list.
 
 @param selectionlist is the GSList of uris selected.
 @param liststore is the GtkListStore where the attachemnts are show.
 @param fields is the pointer to the structure MsgFields containing various fields of the view.
 	This parameter is not freed here.
 @param comMsg is the MsgInfo structure of the editor.
	 This parameter is not freed here.
	 
 @return FALSE-if memory allocation fails
         TRUe-else.
*/
gboolean
ui_fill_and_append_attachinfo(GSList * selectionlist,
			      GtkListStore * liststore, MsgFields * fields, MsgInfo * comMsg)
{
	AttachInfo *attachSend = NULL;
	gchar *temppath = NULL;
	gchar *attachmentSize = NULL;
	gchar *name = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*this function gets the escaped display name  with proper encoding */
	name = g_filename_display_basename((gchar *) selectionlist->data);

	/*will return a temporarly created file path for the attachment */
	temppath = ui_create_filepath_link((gchar *) selectionlist->data, &name, FALSE, TRUE);
	if (temppath == NULL) {
		g_free(name);
		return FALSE;
	}

	attachSend = g_new0(AttachInfo, 1);
	attachSend->filepath = g_strdup(temppath);
	if (name != NULL) {
		if (fields->mycard_attached)
			attachSend->type = ATT_VCARD;
		else
			attachSend->type = DIRECT;

		attachSend->attachmentname = g_strdup(name);
		attachSend->msgid = NULL;
		attachmentSize = (gchar *) ui_file_size(selectionlist->data);
		ui_show_attachments_in_buffer(liststore, name, attachSend->filepath,
					      attachmentSize, fields->mycard_attached);
		show_attachment_field(fields, TRUE);
		gtk_widget_grab_focus(fields->view);
		comMsg->attach_list = g_slist_append(comMsg->attach_list, attachSend);
		set_attachment_view_size(comMsg->attach_list, fields);
		g_free(attachmentSize);
	} else {
		ui_freelist(attachSend, NULL);
	}
	g_free(temppath);
	g_free(name);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return TRUE;
}

/**
 This function is called by higher layers for  registering the callback
	for attaching the file.

 @param fileselect is a pointer to the widget that registered the callback.
 @param editorInfo is the EditorInfo structure pointer of the current editor.
	 This parameter is not freed here.
 
 @return void
*/
void ui_editor_insert_attachment_cb(GtkWidget * fileSelect, EditorInfo * editorInfo)
{
	GSList *selectionlist = NULL;
	GSList *file_paths = NULL;
	MsgInfo *msgInfo = NULL;
	gboolean memallocated = FALSE;
	gboolean valid = FALSE;
	MsgFields *msgFields = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	/*validation for editorInfo */
	valid = ui_validate_editorinfo(editorInfo);
	CHECK_PARAM(valid == FALSE, "Input param check failed\n", return);
	msgFields = editorInfo->msgFields;
	msgInfo = editorInfo->msgInfo;

	selectionlist = gtk_file_chooser_get_uris((GtkFileChooser *) fileSelect);
	file_paths = selectionlist;
	if (selectionlist == NULL) {
		get_user_confirmation(_("mcen_ib_no_attachment_selected"), NULL, NULL, INFO_BANNER);
		ui_fileselector_run((GtkWidget *) fileSelect, editorInfo);
		return;
	}
	while (selectionlist != NULL) {
		memallocated =
		    ui_fill_and_append_attachinfo(selectionlist,
						  msgFields->liststore, msgFields, msgInfo);
		if (memallocated == FALSE) {
			ui_fileselector_run((GtkWidget *) fileSelect, editorInfo);
			ui_file_paths_free(file_paths);
			return;
		}
		gtk_widget_show_all(msgFields->view);
		editorInfo->editorModified = TRUE;
		selectionlist = selectionlist->next;
	}

	ui_file_paths_free(file_paths);
	ui_attachments_info_contents(msgInfo->attach_list);
	gtk_widget_destroy((GtkWidget *) fileSelect);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by higher layers
 
 @param attstr is a pointer to the attachment filename string.
 @param composeInfo is a pointer to the MsgInfo structure .
 	This parameter is not freed here.
 @param attachment_num is the gint representing the attachment.
 
 @return TRUE if attachmnet is found
        False else
*/

static gboolean ui_editor_findnode(const gchar * attStr, MsgInfo * composeInfo, gint attachment_num)
{
	gint length = 0;
	AttachInfo *attdata = NULL;
	GSList *node = NULL;
	gboolean foundAttachment = FALSE;
	gint nth_loop = -1;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	node = composeInfo->attach_list;
	length = g_slist_length(node);
	while (node) {
		attdata = (AttachInfo *) node->data;
		nth_loop++;
		/*removing the attachment node & making it null */
		if (nth_loop == attachment_num) {
			ui_freelist(attdata, NULL);
			node->data = NULL;
			foundAttachment = TRUE;
			break;
		}
		node = node->next;
	}
	node = composeInfo->attach_list;
	ui_attachments_info_contents(node);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);

	return foundAttachment;
}

/**
 This function is called by higher layers for registering remove attach callback.
 @param editorInfo is the EditorInfo structure pointer of the current editor.
 @param selected_names is the list of attachments selected.
 	This parameter is  freed here.
 @return void
*/
static void ui_remove_attachment_cb(EditorInfo * editorInfo, GSList * selected_names)
{
	MsgInfo *composeInfo = NULL;
	gboolean found = FALSE;
	gint nth_attachment = -1;
	MsgFields *msgFields = NULL;
	EditorMenu *menu = NULL;
	GSList *attachment_names = NULL, *node = NULL;
	gboolean mycard_removed = FALSE;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(editorInfo == NULL, "Input param failed", return);

	msgFields = editorInfo->msgFields;
	menu = editorInfo->menu;
	composeInfo = editorInfo->msgInfo;
	gint nth = 0;

	attachment_names = selected_names;
	while (attachment_names != NULL && attachment_names->data != NULL) {
		nth_attachment = ui_get_attachment_num(msgFields, nth);
		nth++;
		found = ui_editor_findnode((gchar *) attachment_names->data, composeInfo,
					   nth_attachment);
		attachment_names = attachment_names->next;
	}
	/*on successfull deletion removes the rows from ui */
	if (found != FALSE) {
		node = composeInfo->attach_list;
		/*removing the NULL data nodes */
		while (node != NULL) {
			if (node->data == NULL) {
				composeInfo->attach_list =
				    g_slist_delete_link((composeInfo->attach_list), node);
				node = composeInfo->attach_list;
				g_print("deleting link\nlength of list is %d\n",
					g_slist_length(node));
			} else if (node != NULL)
				node = node->next;
		}
		if (g_slist_length(composeInfo->attach_list) == 0) {
			g_slist_free(composeInfo->attach_list);
			composeInfo->attach_list = NULL;
		}

		/*check if my card is deleted */
		mycard_removed = ui_remove_attachment_from_list(msgFields, menu->remove_attachment);
	}

	g_slist_foreach(selected_names, (GFunc) g_free, NULL);
	g_slist_free(selected_names);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by higher layers for  registering the callback
	while removing the attachment.
 @param win is a pointer to the widget that registered the callback.
 @param editorInfo is the EditorInfo structure pointer of the current editor.
 	This parameter is not freed here.
 @return void
*/
void ui_editor_remove_attachment_cb(GtkWidget * win, EditorInfo * editorInfo)
{
	gboolean success = FALSE;
	MsgInfo *msgInfo = NULL;
	GtkWidget *remove_Att = NULL;
	GtkWidget *view = NULL;
	GSList *attachmentList = NULL;
	GSList *selected_names = NULL;
	/*gchar *attach_name = NULL; */
	gboolean valid = FALSE;
	MsgFields *msgFields = NULL;
	gchar *infonote = NULL;
	GtkTreeSelection *attachment_selection = NULL;
	GtkTreeIter iter;
	gint numof_attachments = 0;
	GtkTextBuffer *buffer = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	valid = ui_validate_editorinfo(editorInfo);
	CHECK_PARAM(valid == FALSE, "Input param check failed\n", return);

	msgInfo = editorInfo->msgInfo;
	msgFields = editorInfo->msgFields;

	view = gtk_window_get_focus((GtkWindow *) app);
	if (view == msgFields->body) {
		buffer = gtk_text_view_get_buffer((GtkTextView *) msgFields->body);
		gtk_text_buffer_delete_selection(buffer, FALSE, TRUE);
		return;
	}

	view = msgFields->view;
	remove_Att = editorInfo->menu->remove_attachment;

	attachmentList = msgInfo->attach_list;

	selected_names = ui_get_selected_attachment_name(msgFields);
	if (selected_names == NULL) {
		attachment_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgFields->view));
		gtk_tree_model_get_iter_first(gtk_tree_view_get_model
					      ((GtkTreeView *) msgFields->view), &iter);
		gtk_tree_selection_select_iter(attachment_selection, &iter);
		selected_names = ui_get_selected_attachment_name(msgFields);
	}
	numof_attachments = g_slist_length(selected_names);
	if (numof_attachments == 1) {
		infonote =
		    g_strdup_printf(_("emev_nc_delete_attachment"), (gchar *) selected_names->data);
	} else if (numof_attachments > 1) {
		infonote = g_strdup(_("emev_nc_delete_attachments"));
	}
	success = get_user_confirmation(infonote,
					_("emev_bd_delete_attachement"), NULL, CONFIRM_DIALOG);
	g_free(infonote);
	if (success == TRUE) {
		ui_remove_attachment_cb(editorInfo, selected_names);
		editorInfo->editorModified = TRUE;
	}
	return;
}


/**
 this function is called by higher layers a user defined function
 for invoking the fileselection for attaching the files.
 
 @return a pointer to the fileselection dialog.
*/

static GtkWidget *ui_create_fileselection()
{
	GtkWidget *fileselection = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	fileselection =
	    (GtkWidget *) hildon_file_chooser_dialog_new((GtkWindow *) app,
							 GTK_FILE_CHOOSER_ACTION_OPEN);
	g_object_set((GObject *) fileselection, "open-button-text",
		     _("sfil_bd_select_object_ok_select"), NULL);
	if (ossohelp_exists(HELP_ID_SELECTFILE) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) fileselection);
		g_signal_connect(G_OBJECT(fileselection), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_SELECTFILE);
	}
	ULOG_DEBUG("[%s][%s] : selection created :\n", __FILE__, __FUNCTION__);
	gtk_file_chooser_set_select_multiple((GtkFileChooser *) (fileselection), TRUE);
	gtk_window_set_modal(GTK_WINDOW(fileselection), TRUE);
	gtk_window_set_title(GTK_WINDOW(fileselection), _("mcen_ti_select_attachment_title"));
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return fileselection;
}

/**
 This function is called by higher layers for  registering the callback for attaching the
 file.
 
 @param win is a pointer to the menu item that registered the callback.
 @param editor_info is the pointer to the EditorInfo structure of the current editor.
 This parameter is not freed here.
 
 @return void
*/

void ui_editor_attach_file_cb(GtkWidget * win, EditorInfo * editor_info)
{
	GtkWidget *fileselection_dialog = NULL;
	gint result = 0;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(editor_info == NULL, "Input param failed", return);
	fileselection_dialog = GTK_WIDGET(ui_create_fileselection());
	if (fileselection_dialog != NULL) {
		result = gtk_dialog_run((GtkDialog *) fileselection_dialog);
	}
	if (result == GTK_RESPONSE_OK) {
		ui_editor_insert_attachment_cb(fileselection_dialog, editor_info);
	} else if (result != GTK_RESPONSE_OK) {
		ui_editor_close_fileselector_cb(fileselection_dialog, fileselection_dialog);
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by higher layers for registering the callback for
	attaching the inline image file.
 @param win is a pointer to the menu item that registered the callback.
 @param editor_info is the pointer to the EditorInfo structure of the current editor.
 	This parameter is not freed here.
	
 @return void
*/
void ui_editor_attach_image_cb(GtkWidget * win, EditorInfo * editorinfo)
{
	GSList *selectionlist = NULL;
	GSList *file_paths = NULL;
	GtkWidget *fileselection_dialog = NULL;
	gint result = 0;
	gchar *tempath = NULL;
	gchar *image_dir = NULL;
	const gchar *home_dir = NULL;
	GtkFileFilter *filter = NULL;
	GtkTextBuffer *buffer = NULL;
	gint images_inserted = 0;
	GList *image_mimetypes_list = NULL;
	GList *mimetypes_list = NULL;
	GtkTextIter iter;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (editorinfo == NULL) {
		return;
	}

	fileselection_dialog = GTK_WIDGET(ui_create_fileselection());
	if (fileselection_dialog == NULL) {
		return;
	}
	gtk_window_set_title((GtkWindow *) fileselection_dialog,
			     _("mcen_ia_select_inline_image_title"));
	home_dir = g_get_home_dir();
	image_dir = g_strdup_printf("%s%s", home_dir, MYDOCS_IMAGE);
	gtk_file_chooser_set_current_folder((GtkFileChooser *) fileselection_dialog, image_dir);
	g_free(image_dir);
	filter = gtk_file_filter_new();

	image_mimetypes_list = osso_mime_get_mime_types_for_category(OSSO_MIME_CATEGORY_IMAGES);
	mimetypes_list = image_mimetypes_list;
	while (mimetypes_list != NULL) {
		gtk_file_filter_add_mime_type(filter, (gchar *) mimetypes_list->data);
		mimetypes_list = mimetypes_list->next;
	}
	gtk_file_chooser_set_filter((GtkFileChooser *) fileselection_dialog, filter);
	osso_mime_types_list_free(image_mimetypes_list);


	result = gtk_dialog_run((GtkDialog *) fileselection_dialog);
	if (result == GTK_RESPONSE_OK) {
		selectionlist = gtk_file_chooser_get_uris((GtkFileChooser *) fileselection_dialog);
		if (selectionlist == NULL) {
			get_user_confirmation(_("mcen_ib_no_attachment_selected"), NULL, NULL,
					      INFO_BANNER);
			return;
		}
		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editorinfo->msgFields->body));
		file_paths = selectionlist;

		while (selectionlist != NULL) {
			tempath = ui_create_image_file_link(selectionlist->data);
			if (tempath != NULL) {
				text_buffer_insert_image(tempath, editorinfo);
				gnome_vfs_unlink(tempath);
				selectionlist = selectionlist->next;
				g_free(tempath);
				images_inserted++;
			} else {
				break;
			}
		}
		refresh_current_attributes(GTK_TEXT_VIEW(editorinfo->msgFields->body), editorinfo);
		editorinfo->formating_preselected = TRUE;
		undo_image_insertion(buffer, images_inserted, editorinfo);
		ui_file_paths_free(file_paths);
		/*a new line char is inserted after the insertion of images. */
		gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
		/*signals blocked for undo handling */
		g_signal_handlers_block_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editorinfo);
		gtk_text_buffer_insert(buffer, &iter, HTML_NL, -1);
		g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(undoins),
						  editorinfo);
	}
	gtk_widget_destroy((GtkWidget *) fileselection_dialog);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called by higher layers (user defined call).Checking for additional 'Re:`.
 @param substr is the pointer to the subject text field.
 	In this function a newly allocated subject string is returned.
              with substr being freed. 
	      
 @return void
*/
gchar *ui_editor_reply_check(gchar * substr)
{
	gchar *modifiedSubject = NULL;
	gchar *tempsubject = NULL;
	gchar *single_reply = NULL;
	gchar *single_fwd = NULL;
	gchar *multiple_reply = NULL;
	gchar *multiple_fwd = NULL;

	tempsubject = substr;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (substr == NULL) {
		return NULL;
	}
	single_reply = g_strconcat(_("mail_va_re"), SUBJECT_SEPERATOR, NULL);
	single_fwd = g_strconcat(_("mail_va_fw"), SUBJECT_SEPERATOR, NULL);
	multiple_reply = g_strconcat(_("mail_va_re"), SQUARE_OPEN_BRACKET, NULL);
	multiple_fwd = g_strconcat(_("mail_va_fw"), SQUARE_OPEN_BRACKET, NULL);

	if ((g_ascii_strncasecmp((const char *) substr, single_fwd, strlen(single_fwd)) == 0)
	    || (g_ascii_strncasecmp((const char *) substr, multiple_fwd, strlen(multiple_fwd)) ==
		0)) {
		if (strchr(substr, SUBJECT_SEPERATOR_CHAR) != NULL) {
			substr = ui_editor_forward_check(substr);
			modifiedSubject = g_strconcat(single_fwd, SPACE, substr, NULL);
			g_free(substr);
			g_free(single_reply);
			g_free(single_fwd);
			g_free(multiple_reply);
			g_free(multiple_fwd);
			substr = NULL;
			return (modifiedSubject);
		}
	}

	while (substr != NULL && strlen(substr)) {
		if ((g_ascii_strncasecmp
		     ((const char *) substr, single_reply,
		      strlen(single_reply))) == 0
		    ||
		    (g_ascii_strncasecmp
		     ((const char *) substr, multiple_reply, strlen(multiple_reply))) == 0) {
			if (strchr(substr, SUBJECT_SEPERATOR_CHAR) == NULL) {
				break;
			}
			substr = strchr(substr, SUBJECT_SEPERATOR_CHAR);
			substr++;
			g_strstrip(substr);
			continue;
		} else {
			break;
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	modifiedSubject = g_strdup(substr);
	g_free(tempsubject);
	tempsubject = NULL;
	g_free(single_reply);
	g_free(single_fwd);
	g_free(multiple_reply);
	g_free(multiple_fwd);
	return (modifiedSubject);
}


/**
 This function is called by higher layers (user defined call).Checking for
 additional 'Fwd:'
 In this function a newly allocated subject string is returned.
             with substr being freed.
	     
 @param substr is the pointer to the subject text field.

 @return void
*/

gchar *ui_editor_forward_check(gchar * substr)
{
	gchar *tempsubjectAddr = NULL;
	gchar *modifiedSubject = NULL;
	gchar *single_reply = NULL;
	gchar *single_fwd = NULL;
	gchar *multiple_reply = NULL;
	gchar *multiple_fwd = NULL;
	tempsubjectAddr = substr;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (substr == NULL) {
		return NULL;
	}

	single_reply = g_strconcat(_("mail_va_re"), SUBJECT_SEPERATOR, NULL);
	single_fwd = g_strconcat(_("mail_va_fw"), SUBJECT_SEPERATOR, NULL);
	multiple_reply = g_strconcat(_("mail_va_re"), SQUARE_OPEN_BRACKET, NULL);
	multiple_fwd = g_strconcat(_("mail_va_fw"), SQUARE_OPEN_BRACKET, NULL);

	if ((g_ascii_strncasecmp
	     ((const char *) substr, single_reply,
	      strlen(single_reply))) == 0
	    || (g_ascii_strncasecmp((const char *) substr, multiple_reply, strlen(multiple_reply)))
	    == 0) {
		if (strchr(substr, SUBJECT_SEPERATOR_CHAR) != NULL) {
			substr = ui_editor_reply_check(substr);
			modifiedSubject = g_strconcat(single_reply, SPACE, substr, NULL);
			g_free(substr);
			g_free(single_reply);
			g_free(single_fwd);
			g_free(multiple_reply);
			g_free(multiple_fwd);
			substr = NULL;
			return (modifiedSubject);
		}
	}
	while (substr != NULL && strlen(substr)) {
		if ((g_ascii_strncasecmp
		     ((const char *) substr, single_fwd,
		      strlen(single_fwd))) == 0
		    ||
		    (g_ascii_strncasecmp
		     ((const char *) substr, multiple_fwd, strlen(multiple_fwd))) == 0) {
			if (strchr(substr, SUBJECT_SEPERATOR_CHAR) == NULL) {
				break;
			}
			substr = strchr(substr, SUBJECT_SEPERATOR_CHAR);
			substr++;
			g_strstrip(substr);
			continue;
		} else {
			break;
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	modifiedSubject = g_strdup(substr);
	g_free(tempsubjectAddr);
	tempsubjectAddr = NULL;
	g_free(single_reply);
	g_free(single_fwd);
	g_free(multiple_reply);
	g_free(multiple_fwd);
	return (modifiedSubject);

}

/**
 This function is called to update remove attachment sensitivity.

 @param win is pointer to attachment menu item.
 @param editorinfo is the pointer to the EditorInfo structure of the current editor.
 @return void.
*/
void update_attach_menu_status(GtkWidget * win, EditorInfo * editorinfo)
{
	GtkWidget *view = NULL;
	GtkTextBuffer *buffer = NULL;
	GtkTextIter start, end;
	gunichar ch;
	gint offset = 0, end_offset = 0;
	gboolean valid = FALSE;
	gboolean image_present = FALSE;

	valid = ui_validate_editorinfo(editorinfo);
	CHECK_PARAM(valid == FALSE, "Input param check failed\n", return);

	view = gtk_window_get_focus((GtkWindow *) app);
	if (view != editorinfo->msgFields->body) {
		return;
	}
	buffer = gtk_text_view_get_buffer((GtkTextView *) editorinfo->msgFields->body);
	if (gtk_text_buffer_get_selection_bounds(buffer, &start, &end) == FALSE) {
		return;
	}
	end_offset = gtk_text_iter_get_offset(&end);
	offset = gtk_text_iter_get_offset(&start);
	while (offset < end_offset) {
		ch = gtk_text_iter_get_char(&start);
		if (ch == 0xFFFC) {
			if (gtk_text_iter_get_pixbuf(&start) == NULL) {
				return;
			}
			image_present = TRUE;
		} else if (image_present && (ch != '\n')) {
			return;
		} else if (!image_present)
			return;

		gtk_text_iter_forward_char(&start);
		offset = gtk_text_iter_get_offset(&start);
	}
	gtk_widget_set_sensitive(editorinfo->menu->remove_attachment, TRUE);
	return;
}

/**
 This function is called by higher layers  to make the toolbar
 sensitive.
 
 @param win is pointer to object where an event occured.
 @param event is  pointer to event that took place.
 @param editor_info is the pointer to the EditorInfo structure of the current editor.
 	This parameter is not freed here.
	
 @return void.
*/
void ch_toolbar(GtkWidget * win, GdkEventFocus * event, EditorInfo * editorInfo)
{
	EditorMenu *menu = NULL;
	EditorToolbar *toolbar = NULL;
	gboolean valid = FALSE;
	valid = ui_validate_editorinfo(editorInfo);
	CHECK_PARAM(valid == FALSE, "Input param check failed\n", return);
	menu = editorInfo->menu;
	toolbar = editorInfo->toolbar;

	if (editorInfo->emailType == HTML_EDITOR) {
		if ((ui_get_editor_toolbar_hidden_status() == FALSE
		     && email_main_view->tags->full_screen_mode == FALSE)
		    || (ui_get_editor_fullscreen_toolbar_hidden_status() == FALSE
			&& email_main_view->tags->full_screen_mode == TRUE)) {
			gtk_widget_set_sensitive(toolbar->bold, TRUE);
			gtk_widget_set_sensitive(toolbar->italic, TRUE);
			gtk_widget_set_sensitive(toolbar->fontname, TRUE);
			gtk_widget_set_sensitive(toolbar->fontsize, TRUE);
			gtk_widget_set_sensitive(toolbar->fontsize, TRUE);
			gtk_widget_set_sensitive(toolbar->fgcolor, TRUE);
			gtk_widget_set_sensitive(toolbar->bullets, TRUE);
		}
		gtk_widget_set_sensitive(menu->format_menuitem, TRUE);
		gtk_widget_set_sensitive(menu->attach_image, TRUE);
	}
	gtk_widget_set_sensitive(menu->remove_attachment, FALSE);
	return;
}


/**
 This function is called by higher layers  to make the toolbar insensitive.

 @param win is pointer to object where an event occured.
 @param event is  pointer to event that took place.
 @param editor_info is the pointer to the EditorInfo structure of the current editor.
 	This parameter is not freed here.
	
 @return gboolean TRUE-to stop the event from propagating
                 FALSE-else.
*/
gboolean ch_toolbar_insens(GtkWidget * win, GdkEventFocus * event, EditorInfo * editorInfo)
{
	EditorMenu *menu = NULL;
	EditorToolbar *toolbar = NULL;
	menu = editorInfo->menu;
	if (editorInfo != NULL && editorInfo->toolbar != NULL) {
		toolbar = editorInfo->toolbar;
		if ((ui_get_editor_toolbar_hidden_status() == FALSE
		     && email_main_view->tags->full_screen_mode == FALSE)
		    || (ui_get_editor_fullscreen_toolbar_hidden_status() == FALSE
			&& email_main_view->tags->full_screen_mode == TRUE)) {
			gtk_widget_set_sensitive(toolbar->bold, FALSE);
			gtk_widget_set_sensitive(toolbar->italic, FALSE);
			gtk_widget_set_sensitive(toolbar->fontname, FALSE);
			gtk_widget_set_sensitive(toolbar->fontsize, FALSE);
			gtk_widget_set_sensitive(toolbar->fontsize, FALSE);
			gtk_widget_set_sensitive(toolbar->fgcolor, FALSE);
			gtk_widget_set_sensitive(toolbar->bullets, FALSE);
		}
		gtk_widget_set_sensitive(menu->remove_attachment, FALSE);
		gtk_widget_set_sensitive(menu->format_menuitem, FALSE);
		gtk_widget_set_sensitive(menu->attach_image, FALSE);
	}
	/*if focus is in attachment view dim copy menu */
	if (GTK_IS_TREE_VIEW(win) == TRUE) {
		gtk_widget_set_sensitive(menu->remove_attachment, TRUE);
		gtk_widget_set_sensitive(menu->copy, FALSE);
		gtk_widget_set_sensitive(menu->cut, FALSE);
		gtk_widget_set_sensitive(menu->paste, FALSE);
	}

	return FALSE;

}


/**
 This function is called by higher layers for  registering the callback
 	to add a contact to addressbook.
	
 @param widget is a pointer to the menu item
 @param msgfields is the pointer to the MsgFields structure containing all
	the fields of the mail.
	This parameter is not freed here.
	
 @return void
*/

void ui_viewer_add_to_contacts_cb(GtkWidget * widget, MsgFields * msgfields)
{
	gchar *addrstr = NULL;
	GtkTextBuffer *fromBuffer = NULL;
	GtkTextBuffer *toBuffer = NULL;
	GtkTextBuffer *ccBuffer = NULL;
	GtkTextIter sel_start, sel_end;
	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM(msgfields == NULL, "input param failed", return);
	fromBuffer = gtk_text_view_get_buffer((GtkTextView *) msgfields->from);
	toBuffer = gtk_text_view_get_buffer((GtkTextView *) msgfields->to);
	ccBuffer = gtk_text_view_get_buffer((GtkTextView *) msgfields->cc);
	if (gtk_text_buffer_get_selection_bounds(fromBuffer, &sel_start, &sel_end)) {
		addrstr =
		    gtk_text_buffer_get_text(GTK_TEXT_BUFFER(fromBuffer), &sel_start,
					     &sel_end, FALSE);
	} else if (gtk_text_buffer_get_selection_bounds(toBuffer, &sel_start, &sel_end)) {
		addrstr =
		    gtk_text_buffer_get_text(GTK_TEXT_BUFFER(toBuffer), &sel_start,
					     &sel_end, FALSE);
	} else if (gtk_text_buffer_get_selection_bounds(ccBuffer, &sel_start, &sel_end)) {
		addrstr =
		    gtk_text_buffer_get_text(GTK_TEXT_BUFFER(ccBuffer), &sel_start,
					     &sel_end, FALSE);
	} else {
		gtk_text_buffer_get_bounds(fromBuffer, &sel_start, &sel_end);
		addrstr = gtk_text_buffer_get_text(fromBuffer, &sel_start, &sel_end, FALSE);
	}
	if (addrstr != NULL) {
		create_contact_dialog(addrstr);
		g_free(addrstr);
	} else {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_add"));
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 this is a support function to check for the senders address in the to address field
 
 @param fromAddr is the gchar* for senders address.
 @param toAddr is the gchar* for To address feild.
 
 return gboolean TRUE-if senders address is present.
 	else FALSE.
*/

gboolean ui_editor_do_senderCheck(const gchar * fromAddr, const gchar * toAddr)
{
	gchar **parseString = NULL;
	gint i = 0;
	gchar *toaddr = NULL, *fromaddr = NULL;
	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM(toAddr == NULL, "Input param failed", return FALSE);
	CHECK_PARAM(fromAddr == NULL, "Input param failed", return FALSE);
	fromaddr = ui_parse_from_address(fromAddr);
	parseString = g_strsplit(toAddr, ADDR_SEPERATOR_CHAR, -1);
	while (parseString[i] != NULL) {
		toaddr = ui_parse_from_address(parseString[i]);
		if (strncmp(toaddr, fromaddr, strlen(fromaddr)) == 0) {
			ULOG_DEBUG("[%s]:[%d]: the to address is %s\n", __FUNCTION__, __LINE__,
				   toAddr);
			ULOG_DEBUG("[%s]:[%d]: the from address is %s\n", __FUNCTION__,
				   __LINE__, fromAddr);
			ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
			g_strfreev(parseString);
			g_free(toaddr);
			g_free(fromaddr);
			return TRUE;
		}
		g_free(toaddr);
		toaddr = NULL;
		i++;
	}

	g_strfreev(parseString);
	ULOG_DEBUG("[%s]:[%d]: the to address is %s\n", __FUNCTION__, __LINE__, toAddr);
	ULOG_DEBUG("[%s]:[%d]: the from address is %s\n", __FUNCTION__, __LINE__, fromAddr);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	g_free(fromaddr);
	return FALSE;
}

#if 0
/**
 This call is used to check whether there are any charecters other then 
 " ","/n","/t" in the mail.
 
 @param text is the gchar pointer representing the content of the buffer.
 
 @return TRUE-if charecters other then  " ","/n","/t" are in the mail.
 	 FALSE-else.
*/
static gboolean check_for_empty_chars(const gchar * text)
{
	gchar *content = NULL;
	gint i = 0;
	content = (gchar *) text;

	if (content == NULL || content[i] == NULLCHAR) {
		return TRUE;
	}

	while (content[i] != NULLCHAR) {
		switch (content[i]) {
		case EMAIL_SPACE:
		case NEW_LINE_CHAR:
		case TAB_CHAR:
			break;
		default:
			return FALSE;
		}
		i++;
	}
	return TRUE;
}
#endif

/**
 this is a callback function for checking if the subject field is changed
	after the editor is opened.
	
 @param subjectEntry is the pointer to GtkWidget subject field .
 @param text is the inserted text.
 @param len is the length of inserted text.
 @param position is the position of inserted text.
 @param modifiedFalg is a pointer gboolean  which is set if modified.
 
 @return void.
*/

void
ui_editor_subject_insert_cb(GtkEditable * subjectEntry, gchar * text,
			    gint len, gint * position, gboolean * modifiedFalg)
{

	gchar *text_sub = NULL;
	gchar *insert_sub = NULL;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(subjectEntry == NULL, "Input param check failed\n", return);
	CHECK_PARAM(modifiedFalg == NULL, "Input param check failed\n", return);

	insert_sub = (gchar *) gtk_entry_get_text((GtkEntry *) subjectEntry);
	if (text != NULL) {
		text_sub = strchr(insert_sub, NEW_LINE_CHAR);
		if (text_sub != NULL) {
			insert_sub = g_strndup(insert_sub, strlen(insert_sub) - strlen(text_sub));
		}
	}
	if (insert_sub != NULL) {
		gtk_entry_set_text((GtkEntry *) subjectEntry, insert_sub);
	}

	if (*modifiedFalg == FALSE) {
		*modifiedFalg = TRUE;
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 this is a callback function for checking if the subject field is changed
 after the editor is opened.
 
 @param subjectEntry is the pointer to GtkWidget subject field .
 @param start is start insert position.
 @param end is end insert position.
 @param modifiedFalg is a pointer gboolean  which is set if modified.
 
 @return void.
*/

void
ui_editor_subjectChanged_cb(GtkWidget * subjectEntry, gint start, gint end, gboolean * modifiedFalg)
{

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(subjectEntry == NULL, "Input param check failed\n", return);
	CHECK_PARAM(modifiedFalg == NULL, "Input param check failed\n", return);


	*modifiedFalg = TRUE;

	ULOG_DEBUG("[%s][%s] : the flag is %d :\n", __FILE__, __FUNCTION__, *modifiedFalg);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 this is a callback function for checking if spaces or tabs entered
 	after the editor is opened.
	
 @param editor_buffer is the pointer to textbuffer fields of editor .
 @param iter is the position of insertion.
 @param inserttext is the text inserted.
 @param modifiedFalg is a pointer gboolean  which is set if modified.
 
 @return void.
*/

void
ui_editor_check_modified_on_insert(GtkTextBuffer * editor_buffer,
				   GtkTextIter * iter, gchar * inserttext,
				   gint len, gboolean * modifiedFalg)
{

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(editor_buffer == NULL, "Input param check failed\n", return);
	CHECK_PARAM(modifiedFalg == NULL, "Input param check failed\n", return);
	CHECK_PARAM(inserttext == NULL, "Input param check failed\n", return);


	*modifiedFalg = TRUE;
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 this is a callback function for checking if spaces or tabs entered
	after the editor is opened.
	
 @param editor_buffer is the pointer to textbuffer fields of editor .
 @param startiter is the position of insertion.
 @param enditer is the position of insertion.
 @param modifiedFalg is a pointer gboolean  which is set if modified.
 
 @return void.
*/
void
ui_editor_check_modified_on_delete(GtkTextBuffer * editor_buffer,
				   GtkTextIter * startiter,
				   GtkTextIter * enditer, gboolean * modifiedFalg)
{

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(editor_buffer == NULL, "Input param check failed\n", return);
	CHECK_PARAM(modifiedFalg == NULL, "Input param check failed\n", return);

	*modifiedFalg = TRUE;
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Helper function to determine waht is the zoom level selected
based on the menu clicked.
Previouly zoom value was take from label text of menu.
This was causing bugs with other languages.fix is done to take the value from
the menu being clicked

@param zoom_menuitem is the pointer GtkWidget that is cactivated.
@param zoominfo is the Zoom structure pointer.
@return gint -->the current zoom selected
*/
static gint get_current_clicked_zoom_level(GtkWidget * zoom_menuitem, Zoom * zoominfo)
{
	gint current_level = DEFAULT_ZOOM;

	if (zoom_menuitem == zoominfo->zoom_menu200)
		current_level = ZOOM_OUT;
	else if (zoom_menuitem == zoominfo->zoom_menu150)
		current_level = ZOOM_MEDIUM;
	else if (zoom_menuitem == zoominfo->zoom_menu100)
		current_level = DEFAULT_ZOOM;
	else if (zoom_menuitem == zoominfo->zoom_menu50)
		current_level = ZOOM_IN;
	else if (zoom_menuitem == zoominfo->zoom_menu80)
		current_level = ZOOM_80;
	else if (zoom_menuitem == zoominfo->zoom_menu120)
		current_level = ZOOM_120;

	return current_level;
}

/**
 This function is called by when user clicks one of the sub-level ofzoom
 in editor.
 @param  zoom_menuitem is a pointer to the menu item which is activated.
 @param  editor_info  Pointer to EditorInfo object with editor data
 @return void.
*/
void set_format_presel_tag(GtkWidget * zoom_menuitem, EditorInfo * editor_info)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (editor_info == NULL) {
		return;
	}

	editor_info->formating_preselected = TRUE;

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by when user clicks one of the sub-level ofzoom
 in both editor and viewer.
 @param  zoom_menuitem is a pointer to the menu item which is activated.
 @param  zoominfo  Pointer to Zoom object with zoom related data.
 @return void.
*/
void on_zoom_activate(GtkWidget * zoom_menuitem, Zoom * zoominfo)
{
	gint current_level = -1;
	gboolean zoom_unavailable = FALSE;
	static GtkWidget *previouszoom = NULL;
	GtkWidget *focused_wid = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (zoominfo == NULL) {
		return;
	}
	if (gtk_check_menu_item_get_active((GtkCheckMenuItem *) zoom_menuitem) == FALSE) {
		previouszoom = zoom_menuitem;
		return;
	}

	focused_wid = gtk_window_get_focus(GTK_WINDOW(app));
	if (zoominfo->msgbody && gtk_widget_is_focus((GtkWidget *) (zoominfo->msgbody)) == FALSE) {
		zoom_unavailable = TRUE;
	} else if (!zoominfo->msgbody && (GTK_IS_TEXT_VIEW(focused_wid) ||
					  GTK_IS_TREE_VIEW(focused_wid))) {
		/*This is a viewer & the focus is not in the body */
		zoom_unavailable = TRUE;
	}

	if (zoom_unavailable) {
		if (zoominfo->zoom_off == TRUE)
			return;
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_zoom"));
		/*fix for setting the zoom menuitem back to the previous if unable to zoom */
		zoominfo->zoom_off = TRUE;
		gtk_menu_item_activate((GtkMenuItem *) previouszoom);
		zoominfo->zoom_off = FALSE;
		previouszoom = NULL;
		return;
	}
	current_level = get_current_clicked_zoom_level(zoom_menuitem, zoominfo);
	ui_set_zoom(zoominfo, current_level);

	/*call for state saving the zoom value */
	ui_openmsg_set_zoom_value(zoominfo->msgid, current_level);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This is a helper function to free all window pointer list
 
 @param open_windows is a GSList containing active pointers to windows.
 
 @return void
*/
static void ui_free_all_window_pointers(GSList * open_windowslist)
{
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;

	for (node = open_windowslist; node != NULL && node->data != NULL; node = node->next) {
		open_window = (OpenedWindowData *) node->data;
		g_free(open_window->msgid);
		g_free(open_window->oldmsgid);
		g_free(open_window);
	}
	g_slist_free(open_windowslist);
	return;
}

/**
 This is a callback function that is used for closing all the open windows
 
 @returns void
 */
void ui_close_all_windows_cb(void)
{
	GSList *node = NULL;
	OpenedWindowData *open_windows = NULL;
	GtkWidget *open_appview = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	for (node = g_slist_copy(email_main_view->open_windows);
	     node != NULL && node->data != NULL; node = node->next) {
		open_windows = (OpenedWindowData *) node->data;
		open_appview = GTK_WIDGET(open_windows->window_ptr);

		if (strcmp(open_windows->msgid, MAIN_VIEW_ID) == 0) {
			continue;
		}
		if (open_appview != NULL) {
			g_signal_emit_by_name(open_appview, "close");
		}
	}
	node = g_slist_nth(node, 0);
	ui_free_all_window_pointers(node);
	/*all the contents of openwindows are freed by the node .
	 * The list is freed here*/
	g_slist_free(email_main_view->open_windows);

	g_slist_foreach(passwd_list, (GFunc) g_free, NULL);
	g_slist_free(passwd_list);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This is callback routine made purposefully for auto save. This will be invoked
 for each and every open editor for saving the current contents.
 
 @param appview  pointer to the GtkWidget  Holds address of the called widget.
 @param editorInfo is the EditorInfo structure pointer of the current editor.
	This paramter is not freed here.
	
 @return Nothing.
*/
void ui_editor_save_all_in_drafts(GtkWidget * appview, EditorInfo * editorInfo)
{
	MsgInfo *comMsg = NULL;
	gchar *body = NULL;
	gchar *body_text = NULL;
	gchar *buf_text = NULL;
	gint msg_type = -1;
	GtkTextBuffer *bufferBody = NULL;
	GtkTextIter start, end;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (editorInfo == NULL) {
		ULOG_DEBUG("[%s][%s] : Input Parameter Validation :\n", __FILE__, __FUNCTION__);
		return;
	}
	msg_type = editorInfo->composeType;
	comMsg = editorInfo->msgInfo;

	/*Check if message is modified. Otherwise return */
	if (editorInfo->editorModified == FALSE) {
		return;
	}


	if (editorInfo->emailType == HTML_EDITOR) {
		osso_email_format_to_html_text(editorInfo, &buf_text);

	}
	if ((msg_type != DRAFT_MAIL && msg_type != OUTBOX_MAIL)) {
		editorInfo->composeType = DRAFT_MAIL;
		editorInfo->folderNum = DRAFT_MODEL_NUM;
	}

	comMsg->header->smime_type = SMIME_NONE;


	/*get the body contents */
	bufferBody = gtk_text_view_get_buffer((GtkTextView *) editorInfo->msgFields->body);
	gtk_text_buffer_get_bounds(bufferBody, &start, &end);
	body = gtk_text_buffer_get_text(bufferBody, &start, &end, FALSE);


	if (buf_text == NULL)
		ui_dbus_send_save_draft_mail(comMsg, body, FALSE, editorInfo->old_msgid);
	else {
		ui_dbus_send_save_draft_mail(comMsg, buf_text, FALSE, editorInfo->old_msgid);
		osso_email_convert_html_to_gtk(editorInfo, buf_text);
	}

	ui_dbus_editor_open_mail(editorInfo->old_msgid, EDITOR_OPEN_MSG_DEL);

	editorInfo->editorModified = FALSE;
	g_free(body_text);
	g_free(body);
	editorInfo->autosaved = TRUE;
	update_msgid_for_openwindows(editorInfo, editorInfo->msgid, editorInfo->msgnum);

	ui_free_msgheader(comMsg->header);
	editorInfo->msgInfo->header = NULL;
	/* Add message count with folder display name to display in folder view */
	ui_add_unread_count_to_folder(DRAFT_MODEL_NUM);
	ui_add_folder_name_with_unread_unsent_mails(DRAFT_MODEL_NUM);

	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;

}


/**
 This function is used to refresh the folder based on the current folder selction
 	after the saving/sending.
 
 @param foldernum is the gint rep the folder.
 
 @return void.
*/
void ui_get_refresh_folder(gint foldernum)
{
	GtkTreeSelection *selection = NULL;
	gchar *folder_path = NULL;

	if (foldernum == INBOX_MODEL_NUM) {
	} else if (foldernum == OUTBOX_MODEL_NUM ||
		   foldernum != email_main_view->folder_manager->currently_displayed_model) {
		gtk_list_store_clear(GTK_LIST_STORE(ui_get_unsorted_folder_model(foldernum)));
		folder_path = ui_get_folder_name(foldernum);
		ui_populate_folder(folder_path, foldernum);
		ui_add_folder_name_with_unread_unsent_mails(foldernum);
		g_free(folder_path);
	} else {
		/*Refresh the current view */
		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW
							(email_main_view->widgets->folderView));
		ui_folderview_selection_changed_cb(selection, NULL);
	}
	return;
}


/**
 Restoring the saved state for the message
 
 @param editorInfo is the EditorInfo structure pointer of the current editor.
	This paramter is not freed here.
	
 @return void.
*/
void restore_editor_saved_state(EditorInfo * editor_info)
{
	gint zoomvalue = DEFAULT_ZOOM;

	CHECK_PARAM(editor_info == NULL, "INput param check failed", return);
	/*set the scrollpostion & the cursor position */
	ui_set_saved_msg_values(editor_info->msgFields, editor_info->msgid);


	/*setting the zoom state */
	zoomvalue = ui_openmsg_get_zoom_value(editor_info->msgid);

	/*default zoom value */
	if (zoomvalue == DEFAULT_ZOOM)
		return;

	g_free(editor_info->zoominfo);
	editor_info->zoominfo = NULL;
	editor_info->zoominfo =
	    set_initial_zoom_level((GtkTextView *) editor_info->msgFields->body,
				   editor_info->msgFields->mail_viewer,
				   editor_info->msgid, zoomvalue);
	gtk_widget_grab_focus(editor_info->msgFields->body);

	if (zoomvalue == ZOOM_OUT) {	/*zoom out value */
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor_info->
					       menu->zoom_out_max_level, TRUE);
	} else if (zoomvalue == ZOOM_MEDIUM) {	/*zoom medium value */
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor_info->
					       menu->zoom_out_level, TRUE);
	} else if (zoomvalue == ZOOM_IN) {	/*zoom in value */
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor_info->menu->
					       zoom_in_level, TRUE);
	}
	ui_set_zoom(editor_info->zoominfo, zoomvalue);
	return;
}

/*function to get new msgids when state saving happens
 @param editorappview is the appview which registered this callback.
 @param editorInfo is the EditorInfo structure pointer of the current editor.
	This paramter is not freed here.

 @return void.
*/
void ui_editor_get_new_msgid_cb(GtkWidget * editor, EditorInfo * editor_info)
{
	gchar *ret_msgid = NULL;

        if(!editor_info->editorModified)
		return;
	
	ui_editor_get_msginfo(editor_info, NULL);
	ui_get_curr_msg_settings(editor_info->msgFields, editor_info->msgid);

	ui_dbus_send_get_msg_id(editor_info->msgInfo, &ret_msgid, FALSE);
	if (ret_msgid != NULL) {
		editor_info->old_msgid = editor_info->msgid;
		editor_info->msgid = g_strdup(ret_msgid);
		g_free(editor_info->msgInfo->header->msgid);
		editor_info->msgInfo->header->msgid = NULL;
		editor_info->msgInfo->header->msgid = g_strdup(ret_msgid);
	}
	g_free(ret_msgid);
	return;
}

/**
 This function is called when a request for the application is made to
       go down,then all the values of the open viewers is obtained &
        the values are stored.
	
 @param editorappview is the appview which registered this callback.
 @param editorInfo is the EditorInfo structure pointer of the current editor.
	This paramter is not freed here.

 @return void.
*/
void ui_editor_get_state_values_cb(GtkWidget * editorappview, EditorInfo * editorInfo)
{
	if (editorInfo == NULL && editorInfo->msgFields == NULL &&
			!editorInfo->editorModified) {
		return;
	}
	ui_editor_save_all_in_drafts(editorInfo->editor_appview, editorInfo);
	ui_set_zoom(editorInfo->zoominfo, editorInfo->zoominfo->previous_level);
	return;
}

/*
 This function is used to show/hide the cc
 fields.
 
 @param fields is the MsgFields structure pointer of the current editor.
	This paramter is not freed here.
 @param show is the gboolean representing the on/off status of cc
 
 @return void.
*/
void show_cc_field(MsgFields * fields, gboolean show)
{
	/*check if cc field is created */
	if (!fields->cc_lable)
		return;

	if (show == TRUE) {
		if (fields->cc_addrimage != NULL)
			gtk_widget_show(fields->cc_addrimage);
		if (fields->cc_lable_hbox != NULL)
			gtk_widget_show(fields->cc_lable_hbox);

		gtk_widget_show(fields->cc_lable);
		gtk_widget_show(fields->cc_hbox);

	} else {
		if (fields->cc_addrimage != NULL)
			gtk_widget_hide(fields->cc_addrimage);
		if (fields->cc_lable_hbox != NULL)
			gtk_widget_hide(fields->cc_lable_hbox);
		gtk_widget_hide(fields->cc_lable);
		gtk_widget_hide(fields->cc_hbox);
	}
	return;
}

/*
 This function is used to show/hide the bcc
 fields.
 @param fields is the MsgFields structure pointer of the current editor.
	This paramter is not freed here.
 @param show is the gboolean representing the on/off status of bcc
 
 @return void.
*/
void show_bcc_field(MsgFields * fields, gboolean show)
{
	if (show == TRUE) {
		gtk_widget_show(fields->bcc_addrimage);
		if (fields->bcc_lable_hbox != NULL)
			gtk_widget_show(fields->bcc_lable_hbox);
		gtk_widget_show(fields->bcc_lable);
		gtk_widget_show(fields->bcc_hbox);
	} else {
		gtk_widget_hide(fields->bcc_addrimage);
		if (fields->bcc_lable_hbox != NULL)
			gtk_widget_hide(fields->bcc_lable_hbox);
		gtk_widget_hide(fields->bcc_lable);
		gtk_widget_hide(fields->bcc_hbox);
	}
	return;
}

/*
 This function is used to show/hide the attachmnets
 fields.
 
 @param fields is the MsgFields structure pointer of the current editor.
	This paramter is not freed here.
 @param show is the gboolean representing the on/off status of attachments 

 @return void.
*/
void show_attachment_field(MsgFields * fields, gboolean show)
{
	if (show == TRUE) {
		gtk_widget_show(fields->attachment_lable);
		gtk_widget_show_all(fields->sw_attach);
	}
	if (show == FALSE) {
		gtk_widget_hide(fields->attachment_lable);
		gtk_widget_hide(fields->sw_attach);
	}
	return;
}

/**
 Set the security option of mail
 
 @param menu is the gtkwidget that registered the callback. 
 @param editorInfo is the EditorInfo structure pointer of the current editor.
	This paramter is not freed here.
	
 @return void.
*/
void ui_editor_set_encryptmail_cb(GtkWidget * menu, EditorInfo * editorInfo)
{
	EmailSpecificSettings *settings = NULL;
	SMimeType smime_type = SMIME_NONE;

	settings = editorInfo->msgInfo->settings;
	smime_type = settings->smime_type;
	if (menu == editorInfo->menu->encrypt) {
		settings->smime_type = SMIME_ENCRYPT;
	} else if (menu == editorInfo->menu->sign) {
		settings->smime_type = SMIME_SIGN;
	} else if (menu == editorInfo->menu->sign_encrypt) {
		settings->smime_type = SMIME_SIGN_ENCRYPT;
	} else {
		settings->smime_type = SMIME_NONE;
	}
	set_security_icon(settings->smime_type, editorInfo->msgFields);

	/*if settings are changed update the modified flag */
	if (smime_type != settings->smime_type) {
		editorInfo->settings_changed = TRUE;
		editorInfo->editorModified = TRUE;
	}
	return;
}

/**
 This function is used to show different infonote for save button
 is clicked when insensitive.
 
 @param insensitive_widget is the GtkWidget for which the insensitive_press
 	signal is fired.
 @param fields is the pointer to the MsgFileds structure.
 
 @return void.
*/
void format_dimming_note(GtkWidget * insensitive_widget, EditorInfo * editor)
{
	CHECK_PARAM(editor == NULL, "input param check failed", return);

	if (editor->emailType == HTML_EDITOR) {
		show_dimming_note(insensitive_widget, _("mcen_ib_move_cursor_to_message"));
	} else {
		show_dimming_note(insensitive_widget, _("mcen_ib_item_unavailable"));
	}
	return;
}


/**
 This function is used for enabling/disabling of cut/copy when there is
 no selection made.

 @param editmenu is the GtkWidget pointer of the editmenu.
 @param editor thsi is the pointer to the EditorInfo *.
 
 @return void.
*/
void handle_ccp_menus(GtkWidget * editmenu, EditorInfo * editor)
{
	GtkWidget *view = NULL;
	GtkTextBuffer *textbuffer_to = NULL;
	GtkTextBuffer *textbuffer_cc = NULL;
	GtkTextBuffer *textbuffer_bcc = NULL;
	GtkTextBuffer *textbuffer_body = NULL;
	GtkTextIter start, end;
	gint st, ed;
	MsgFields *fields = NULL;
	EditorMenu *menu = NULL;
	GtkClipboard *clipboard = NULL;	/*clipboard pointer for implementing cut/copy/paste */
	GdkAtom atom;
	gchar *orig_folder = NULL;
	GSList *msg_id_list = NULL;
	gboolean cut_operation_done = FALSE;


	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM(editor == NULL, "input param check failed", return);
	CHECK_PARAM(editor->msgFields == NULL, "input param check failed", return);

	fields = editor->msgFields;
	menu = editor->menu;

	view = gtk_window_get_focus((GtkWindow *) app);
	if (GTK_IS_TREE_VIEW(view)) {
		gtk_widget_set_sensitive(menu->paste, FALSE);
		gtk_widget_set_sensitive(menu->cut, FALSE);
		gtk_widget_set_sensitive(menu->copy, FALSE);
		return;
	}

	clipboard = gtk_clipboard_get(GDK_NONE);
	atom = gdk_atom_intern("GTK_TEXT_BUFFER_CONTENTS", FALSE);

	if (gtk_clipboard_wait_is_text_available(clipboard)) {
		gtk_widget_set_sensitive(menu->paste, TRUE);
	} else {
		gtk_widget_set_sensitive(menu->paste, FALSE);
	}

	if (view == fields->body) {
		/*Checks if clipboard has any mails to paste. */
		if (osso_email_clipboard_has_emails_for_attachment
		    (editor->editor_appview) &&
		    ui_check_clipboard_has_valid_emails
		    (&orig_folder, &msg_id_list, &cut_operation_done) == TRUE) {
			gtk_widget_set_sensitive(menu->paste, TRUE);
		}
		if (msg_id_list != NULL) {
			g_slist_foreach(msg_id_list, (GFunc) g_free, NULL);
			g_slist_free(msg_id_list);
		}
		g_free(orig_folder);
	}

	textbuffer_to = gtk_text_view_get_buffer((GtkTextView *) fields->to);
	textbuffer_cc = gtk_text_view_get_buffer((GtkTextView *) fields->cc);
	textbuffer_bcc = gtk_text_view_get_buffer((GtkTextView *) fields->bcc);
	textbuffer_body = gtk_text_view_get_buffer((GtkTextView *) fields->body);

	/*check if there is any selection */
	if (gtk_text_buffer_get_selection_bounds(textbuffer_to, &start, &end) ||
	    gtk_text_buffer_get_selection_bounds(textbuffer_cc, &start, &end) ||
	    gtk_text_buffer_get_selection_bounds(textbuffer_bcc, &start, &end) ||
	    gtk_text_buffer_get_selection_bounds(textbuffer_body, &start, &end) ||
	    gtk_editable_get_selection_bounds((GtkEditable *) fields->subject, &st, &ed)) {
		/*enable the cut/copy menus. */
		gtk_widget_set_sensitive(menu->cut, TRUE);
		gtk_widget_set_sensitive(menu->copy, TRUE);
	} else {
		gtk_widget_set_sensitive(menu->cut, FALSE);
		gtk_widget_set_sensitive(menu->copy, FALSE);
	}
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function call is used to retrieve the mail contents of the list of non_local_msg
 
 @param foldernum is the folder number from which the messages are copied.
 @param non_local_msgs is the GSList of no local msgids(OUT param).
 @param local_msgs is the GSList of  local msgids(OUT param).
 @return void.
*/
static void
check_if_headers(gint foldernum, const GSList * msg_list, GSList ** local_msgs,
		 GSList ** non_local_msgs)
{
	GSList *msgid_list = NULL;
	gint msg_row_num = 0;
	gboolean full_downloaded = FALSE;
	gboolean download = FALSE;
	GSList *node = NULL;
	GtkTreeModel *model = NULL;
	GtkTreePath *path = NULL;
	GtkTreeIter iter;

	msgid_list = (GSList *) msg_list;
	node = g_slist_nth(email_main_view->sorted_model_list, foldernum);
	if (node == NULL || (model = GTK_TREE_MODEL(node->data)) == NULL) {
		return;
	}
	/*separate the local/nonlocal mails */
	while (msgid_list != NULL) {
		msg_row_num = ui_get_row_number_of_msg((gchar *) msgid_list->data, foldernum);
		path = gtk_tree_path_new_from_indices(msg_row_num, -1);
		if (gtk_tree_model_get_iter(model, &iter, path)) {
			gtk_tree_model_get(model, &iter, COL_FULLY_DOWNLOADED,
					   &full_downloaded, -1);
			if (full_downloaded == FALSE) {
				*non_local_msgs =
				    g_slist_append(*non_local_msgs, ((gchar *) msgid_list->data));
			} else {
				*local_msgs =
				    g_slist_append(*local_msgs, ((gchar *) msgid_list->data));
			}
		}
		msgid_list = msgid_list->next;
	}
	/*ask confirmation for downloading non local mails */
	if (g_slist_length(*non_local_msgs) > 0) {
		download =
		    get_user_confirmation(_("mcen_nc_get_msg_txt"), _("mcen_bd_retrieve"),
					  NULL, CONFIRM_DIALOG);
		if (download == FALSE) {
			g_slist_free(*non_local_msgs);
			*non_local_msgs = NULL;
			return;

		}
		retrieve_mail_contents(*non_local_msgs, foldernum);
	}
	return;
}

/**
 Thsi function is used to attach the email messages as attachments
 	when copied from the listview.
 
 @param msg_list is the GSList of msdid list.
 @param editor is the EditorInfo structure pointer.
 
 @return void.
 */
static void paste_email_attachments(GSList * msg_list, EditorInfo * editor)
{
	GSList *msgid_list = NULL, *local_msgs = NULL, *non_local_msgs = NULL, *tmp_list = NULL;
	gchar *subject = NULL;
	AttachInfo *attachinfo = NULL;
	MsgInfo *msginfo = NULL;
	gchar *parent_name = NULL;
	gchar *folder_name = NULL;
	gchar *email_path = NULL;
	gchar *size = NULL;
	gint foldernum = INBOX_MODEL_NUM;

	msginfo = editor->msgInfo;
	msgid_list = msg_list;

	if (msg_list == NULL || (msg_list != NULL && msg_list->data == NULL)) {
		return;
	}

	/*get the folder num from the folder name */
	if (ui_get_msg_folder_names((gchar *) msgid_list->data, &parent_name, &folder_name) ==
	    FALSE) {
		ULOG_DEBUG_F("Unable to get folder name for msgid \n");
	}
	foldernum = ui_get_folder_model_number(folder_name, parent_name);
	g_free(folder_name);
	g_free(parent_name);


	check_if_headers(foldernum, msg_list, &local_msgs, &non_local_msgs);
	if (local_msgs != NULL)
		msgid_list = g_slist_concat(local_msgs, non_local_msgs);
	else
		msgid_list = non_local_msgs;

	tmp_list = msgid_list;

	while (msgid_list != NULL && msgid_list->data != NULL) {
		if (ui_get_row_number_of_msg(msgid_list->data, foldernum) == -1) {
			ULOG_DEBUG_F
			    ("Message with id '%s' not found during paste operation\n",
			     (gchar *) msgid_list->data);
			msgid_list = msgid_list->next;
			continue;
		}

		subject = ui_get_msg_subject((gchar *) msgid_list->data, foldernum);
		attachinfo = g_new0(AttachInfo, 1);
		attachinfo->type = EMAIL;
		attachinfo->msgid = g_strdup((gchar *) msgid_list->data);
		attachinfo->attachmentname = subject;

		/*get the path of the email */
		email_path = ui_get_message_file_path(attachinfo->msgid, foldernum);
		attachinfo->email_path = ui_create_filepath_link(email_path,
								 &(attachinfo->attachmentname),
								 FALSE, FALSE);

		/*Check if file copying failed */
		if (attachinfo->email_path == NULL)
			break;

		/*get the size of attachment */
		size = (gchar *) ui_file_size(attachinfo->email_path);
		ui_show_attachments_in_buffer(editor->msgFields->liststore, subject,
					      attachinfo->email_path, size, FALSE);

		msginfo->attach_list = g_slist_append(msginfo->attach_list, attachinfo);
		set_attachment_view_size(msginfo->attach_list, editor->msgFields);
		show_attachment_field(editor->msgFields, TRUE);
		msgid_list = msgid_list->next;
		editor->editorModified = TRUE;

		g_free(size);
		g_free(email_path);
		email_path = NULL;
		size = NULL;
	}
	g_slist_free(local_msgs);
	g_slist_free(non_local_msgs);
	return;
}

void ui_paste_messages_cb(GtkWidget * wid, EditorInfo * editor)
{
	GSList *msgid_list = NULL;
	gchar *foldername = NULL;
	gboolean cut = FALSE;

	osso_email_get_list_from_clipboard(editor->editor_appview, &foldername, &msgid_list, &cut);
	if (msgid_list != NULL) {
		paste_email_attachments(msgid_list, editor);
		g_slist_foreach(msgid_list, (GFunc) g_free, NULL);
		g_slist_free(msgid_list);
		g_free(foldername);
	}
	return;
}

/**
 This function is called to invoke the font settings dialog.

 @param wid is the widget that registerd the call back.
 @param editor is the editorInfo structure pointer.

 @return void.
*/
void ui_editor_fontsettings_cb(GtkWidget * wid, EditorInfo * editor)
{
	GtkWidget *font_dialog = NULL;
	gint response = GTK_RESPONSE_OK;
	font_dialog = launch_font_setting_dialog();
	response = gtk_dialog_run((GtkDialog *) font_dialog);
	if (response == GTK_RESPONSE_OK) {
		set_font_values(font_dialog);

		if (editor->emailType == HTML_EDITOR) {
			gtk_combo_box_set_active(GTK_COMBO_BOX(editor->toolbar->fontname),
						 ui_get_editor_html_fontindex());
		} else
			gtk_combo_box_set_active(GTK_COMBO_BOX(editor->toolbar->fontname),
						 ui_get_editor_plain_fontindex());

	}
	gtk_widget_destroy(font_dialog);
	return;

}

/* end of file */
