/**
 * @file editor_sendoption.c
 *
 * This file contains all functionality of sending option dailog of editor.
 *
 *  Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/


/*Editor-Viewer header files*/
#include "editcallback.h"
#include "callback.h"
#include "security.h"

#include "html_parser.h"
#include "htcallback.h"
#include "undo.h"

extern MainViewInfo *email_main_view;	/*Declared in ui_interface.c */

static void toolbar_set(SettingsInfo * settings_info);
static void ui_cancel_sendoption_dialog_cb(GtkWidget * win, SettingsInfo * settings_info);
gchar *gtk_combo_box_get_active_text(GtkComboBox *);

/**
 function to set the modified when the sending option is opened.

 @param editor_info is a pointer to the  editor_info stucture of the current editor.
   This param is not freed in this function.
   
 @return void
*/
static void set_editor_modified(EditorInfo * editor_info)
{
	CHECK_PARAM(editor_info == NULL, "Input param check failed", return);
	editor_info->editorModified = TRUE;
	editor_info->settings_changed = TRUE;
	return;
}

/**
 This Fuction is used to fill the email specific settings other then priority.

 @param settings is the pointer to the email specific settings structure.
   This param is not freed in this function.
 @param read_receipt_req is a gboolean representing the flag set for
	 read_receipt_req is true/false.
 @param copy_to_own is a gboolean representing the flag set for
	 copy_to_own  is true/false.
 @param eaccount is the gchar pointer to the account.
   This param is not freed in this function.
 @param send_option is the enum value representing the sending option of the mail 
	 i.e-immedialtely/on request/next connection.
	 
 @return void.
*/
static void
ui_editor_fill_esettings(EmailSpecificSettings * settings,
			 gboolean read_receipt_req, gboolean copy_to_own,
			 const gchar * eaccount, gint send_option)
{
	ULOG_DEBUG("[%s][%s] :starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(settings == NULL, "Input param check failed", return);
	CHECK_PARAM(eaccount == NULL, "Input param check failed", return);

	if (settings->account != NULL) {
		g_free(settings->account);
	}
	if (eaccount != NULL) {
		settings->account = g_strdup(eaccount);
	}
	settings->read_receipt_request = read_receipt_req;
	settings->copy_to_own_address = copy_to_own;
	if (send_option == NEXT_CONN) {	/*for the sending option durning next connection */
		settings->options = SEND_NEXT_CONNECTION;
	} else if (send_option == IMMEDIATELY) {	/*for the sending option Immediately */
		settings->options = SEND_IMMEDIATE;
	} else {		/*for the sending option ON REQUEST */

		settings->options = SEND_ON_REQUEST;
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is used to fill the email sepecific settings email type.
 either plain/html.

  @param editor_info is the pointer to the editor_info structure.
   This param is not freed in this function.
  @param email_type is the enum representing the html/plain email type.
  @param email_type_entry is the combobox in the dialog that specifies the mail type
  @return FALSE if selections are discarded.
*/
static gboolean
ui_editor_fill_esettings_type(EditorInfo * editor_info,
			      gint email_type, GtkWidget * email_type_entry)
{
	GtkTextIter start, end;
	gboolean user_confirmation = FALSE;
	gchar *text = NULL;
	EmailSpecificSettings *settings = NULL;
	GtkWidget *view = NULL;

	ULOG_DEBUG("[%s][%s] :starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(editor_info == NULL, "Input param check failed", return FALSE);
	CHECK_PARAM(editor_info->msgInfo == NULL, "Input param check failed", return FALSE);

	settings = editor_info->msgInfo->settings;
	view = editor_info->msgFields->body;

	if (settings->type == EMAIL_HTML) {
		if (email_type == PLAIN_MAIL) {	/*for html mail */
			gtk_text_buffer_get_bounds(gtk_text_view_get_buffer
						   ((GtkTextView *) view), &start, &end);
			text =
			    gtk_text_buffer_get_slice(gtk_text_view_get_buffer
						      ((GtkTextView *) view), &start, &end, TRUE);
			if (text != NULL && strlen(text) != 0) {
				user_confirmation =
				    get_user_confirmation(_("emev_nc_formatting_lost"), NULL,
							  NULL, CONFIRM_DIALOG);
				if (user_confirmation == FALSE) {
					gtk_combo_box_set_active((GtkComboBox *)
								 email_type_entry, 1);
					gtk_widget_grab_focus(email_type_entry);
					g_free(text);
					return FALSE;
				}
			}
			settings->type = EMAIL_TEXT;
			set_default_font(editor_info, settings);
			g_free(text);
		}
	} else {
		if (email_type == HTML_MAIL) {
			settings->type = EMAIL_HTML;
			set_default_font(editor_info, settings);
		}
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return TRUE;
}


/**
 This function is used to change the default font setting for email editor
	depending upon the type email type.

 @param editor_info is pointer to EditorInfo structure.
 @param settings is the pointer to the email specific settings structure.
 
 @return void.
*/
void set_default_font(EditorInfo * editor_info, EmailSpecificSettings * settings)
{
	PangoFontDescription *font_desc;
	const gchar *fontstring = NULL;
	GtkTextBuffer *buffer = NULL;
	GtkTextIter start, end;
	gint default_html_fontindex = 0;
	gint default_plain_fontindex = 0;
	PangoAttribute *attribute = NULL;
	GdkColor *color = NULL;

	ULOG_DEBUG("[%s][%s] :starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(settings == NULL, "Input param check failed", return);
	CHECK_PARAM(editor_info == NULL, "Input param check failed", return);

	default_html_fontindex = ui_get_editor_html_fontindex();
	default_plain_fontindex = ui_get_editor_plain_fontindex();

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editor_info->msgFields->body));
	if (settings->type == EMAIL_HTML) {
		gtk_combo_box_set_active(GTK_COMBO_BOX(editor_info->toolbar->fontname),
					 default_html_fontindex);
		fontstring = font_families[default_html_fontindex];
		font_desc = pango_font_description_from_string(fontstring);
		pango_font_description_set_size(font_desc,
						(12 * PANGO_SCALE * (font_scaling_factor)));
		gtk_widget_modify_font(editor_info->msgFields->body, font_desc);
		color = gdk_color_copy((GdkColor *) & (editor_info->body_style->fg));
		attribute = pango_attr_foreground_new(
							     /*editor_info->body_style->fg.red,
							        editor_info->body_style->fg.green,
							        editor_info->body_style->fg.blue); */
							     color->red, color->green, color->blue);
		attribute->start_index = 0;
		attribute->end_index = -1;
		if (editor_info->attr_list == NULL) {
			pango_attr_list_insert(editor_info->attr_list, attribute);
		} else {
			pango_attr_list_change(editor_info->attr_list, attribute);
		}
		gdk_color_free(color);
		pango_font_description_free(font_desc);
		gtk_text_buffer_get_bounds(buffer, &start, &end);
		gtk_text_buffer_set_can_paste_rich_text(buffer, TRUE);
	} else {
		gtk_combo_box_set_active(GTK_COMBO_BOX(editor_info->toolbar->fontname),
					 default_plain_fontindex);
		gtk_combo_box_set_active(GTK_COMBO_BOX(editor_info->toolbar->fontsize),
					 DEFAULT_FONT_POS);
		fontstring = font_families[default_plain_fontindex];
		font_desc = pango_font_description_from_string(fontstring);
		pango_font_description_set_size(font_desc,
						(12 * PANGO_SCALE * (font_scaling_factor)));
		gtk_widget_modify_font(editor_info->msgFields->body, font_desc);
		gtk_widget_modify_fg(editor_info->msgFields->body, GTK_STATE_NORMAL,
				     (GdkColor *) & (editor_info->body_style->fg));
		pango_font_description_free(font_desc);
		gtk_text_buffer_set_can_paste_rich_text(buffer, FALSE);
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}



/**
 This function is used to fill the selcted entried into the structure.

  @param settings is the pointer to the email specific settings structure.
   This param is not freed in this function.
  @param priority is the enum value representing the priority of the mail.
  @param priority_image is the image to be show for the priority of mail.
  
  @return void
*/
static void
ui_editor_fill_esettings_priority(EmailSpecificSettings * settings,
				  gint priority, GtkWidget * priority_image)
{
	gchar *image = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(settings == NULL, "Input param check failed", return);

	if (priority == PRIORITY_NORMAL) {	/*for NORMAL priority mail */
		gtk_widget_hide (priority_image);
		settings->priority = NORMAL;
		return;
	} else if (priority == PRIORITY_LOW) {	/*for LOW priority mail */
		image = "qgn_list_messaging_low";
		settings->priority = LOW;
	} else {		/*for HIGH priority mail */
		settings->priority = HIGH;
		image = "qgn_list_messaging_high";
	}
	 gtk_image_set_from_icon_name ((GtkImage *) priority_image,image, GTK_ICON_SIZE_BUTTON);
	 gtk_widget_show (priority_image);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
  Sets the mime type of the mail.
  
  @param settings is the pointer to the email specific settings structure.
   This param is not freed in this function.
  @param smime is the enum representing the mime type of the mail.
  @param fields is the pointer to the MsgFields structure.
   This param is not freed in this function.
  @param menu is the pointer to the Editormenu structure.
   This param is not freed in this function.
   
  @return void.
*/
void
ui_editor_fill_smime_type(EmailSpecificSettings * settings, gint smime,
			  MsgFields * fields, EditorMenu * menu)
{
	CHECK_PARAM(settings == NULL, "Input param check failed", return);

	if (smime == SMIME_NONE) {
		settings->smime_type = SMIME_NONE;
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) menu->no_encrypt, TRUE);
	} else if (smime == SMIME_SIGN) {
		/*Set the option menu item in editor Active */
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) menu->sign, TRUE);
		settings->smime_type = SMIME_SIGN;
	} else if (smime == SMIME_ENCRYPT) {
/*Set the option menu item in editor Active*/
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) menu->encrypt, TRUE);
		settings->smime_type = SMIME_ENCRYPT;
	} else {
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) menu->sign_encrypt, TRUE);
		settings->smime_type = SMIME_SIGN_ENCRYPT;
	}
	set_security_icon(settings->smime_type, fields);

}

/**
 This function is called by higher layers for message priority settings.

 @param win is a pointer to the widget that registered the callback.
 @param settings_info is a pointer to the SettingsInfo contains the settings window
 pointer & the editor_info stucture. 
   This param is not freed in this function.
   
 @return void
*/
static void ui_ok_sendoption_dialog_cb(GtkWidget * win, SettingsInfo * settings_info)
{
	GtkWidget *priority_entry = NULL;
	GtkWidget *email_type_entry = NULL;
	GtkWidget *smime_entry = NULL;
	GtkWidget *req_read_receipt = NULL;
	GtkWidget *copy_to_own = NULL;
	GtkWidget *account_entry = NULL;
#if RETAINED_EMAIL
	GtkWidget *send_mail_entry = NULL;
#endif
	GtkWidget *priority_image = NULL;
	MsgInfo *msgInfo = NULL;

	gint priority = NORMAL;
	gint email_type = EMAIL_HTML;
	gint send_option = SEND_IMMEDIATE;
	gint smime = SMIME_NONE;
	gint present_zoom = DEFAULT_ZOOM;
	gchar *eaccount = NULL;
	gboolean r_r_req = FALSE;
	gboolean copy_to_own_flag = FALSE;
	EmailSpecificSettings *settings = NULL;
	GtkWidget *view = NULL;
	GtkWidget *settings_dialog = NULL;
	EditorInfo *editor_info = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(settings_info == NULL, "Input param check failed", return);
	CHECK_PARAM(settings_info->editor_info == NULL, "Input param check failed", return);
	CHECK_PARAM(settings_info->settings_dialog == NULL, "Input param check failed", return);

	settings_dialog = settings_info->settings_dialog;
	editor_info = settings_info->editor_info;
	priority_entry = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "priority");
	email_type_entry = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "email_type");
#if SMIME
	smime_entry = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "smime_type");
#endif
	req_read_receipt = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "read_receipt");
	copy_to_own = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "copy_to_own");
	account_entry = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "account");
#if RETAINED_EMAIL
	send_mail_entry = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "send_mail");
#endif

	view = editor_info->msgFields->body;
	priority_image = editor_info->msgFields->priority_image;
	msgInfo = editor_info->msgInfo;
	settings = msgInfo->settings;

	priority = gtk_combo_box_get_active((GtkComboBox *) priority_entry);
	email_type = gtk_combo_box_get_active((GtkComboBox *) email_type_entry);
	smime = gtk_combo_box_get_active((GtkComboBox *) smime_entry);
	r_r_req = gtk_toggle_button_get_active((GtkToggleButton *) req_read_receipt);
	copy_to_own_flag = gtk_toggle_button_get_active((GtkToggleButton *) copy_to_own);
	eaccount = gtk_combo_box_get_active_text((GtkComboBox *) account_entry);
#if RETAINED_EMAIL
	send_option = gtk_combo_box_get_active((GtkComboBox *) send_mail_entry);
#endif

	ui_editor_fill_esettings_priority(settings, priority, priority_image);
	if (ui_editor_fill_esettings_type(editor_info, email_type, email_type_entry) == FALSE) {
		g_free(eaccount);
		return;
	}
	if (settings->type != email_type) {
		present_zoom = editor_info->zoominfo->previous_level;
		editor_info->zoominfo->previous_level = 100;
		ui_set_zoom(editor_info->zoominfo, present_zoom);
	}
#if SMIME
	if (smime == SIGN)
		smime = SMIME_SIGN;
	else if (smime == ENCRY)
		smime = SMIME_ENCRYPT;
	ui_editor_fill_smime_type(settings, smime, editor_info->msgFields, editor_info->menu);
#endif
	ui_editor_fill_esettings(settings, r_r_req, copy_to_own_flag, eaccount, send_option);
	g_free(eaccount);
	toolbar_set(settings_info);
	set_editor_modified(editor_info);
	ui_cancel_sendoption_dialog_cb(NULL, settings_info);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by higher layers for  registering the callback

 @param win is a pointer to the widget that registered the callback.
 @param settings_info is a pointer to the SettingsInfo contains the settings window
 	pointer & the editor_info stucture.  This param is  freed in this function.
 @return void
*/
static void ui_cancel_sendoption_dialog_cb(GtkWidget * win, SettingsInfo * settings_info)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (settings_info != NULL && settings_info->settings_dialog != NULL) {
		gtk_widget_destroy(GTK_WIDGET(settings_info->settings_dialog));
		g_free(settings_info);
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This function is called to remove all the tags from body field when msg type
is changed from HTML to Plain.

@param buffer is the message body field
@return void.
*/
static void remove_tags_on_mail_type_change(GtkTextBuffer * buffer)
{
	GtkTextIter start;
	GtkTextIter end;
	GtkTextIter iter;
	GSList *taglist = NULL;
	gint offset = 0;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (buffer == NULL) {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return;
	}

	gtk_text_buffer_get_bounds(buffer, &start, &end);
	iter = start;
	do {
		taglist = gtk_text_iter_get_tags(&iter);
		if (taglist == NULL) {
			gtk_text_iter_forward_char(&iter);
			offset++;
		} else {
			offset = gtk_text_iter_get_offset(&iter);
			gtk_text_buffer_get_bounds(buffer, &start, &end);
			gtk_text_buffer_remove_all_tags(buffer, &start, &end);
			gtk_text_buffer_get_iter_at_offset(buffer, &iter, offset);
			g_slist_free(taglist);
		}
	} while (offset < gtk_text_buffer_get_char_count(buffer));

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 This function is called by higher layers to make toolbar sensitive or
 	insensitive depending upon the sending option dialog settings.

 @param settings_info is a pointer to the SettingsInfo contains the settings window
 	pointer & the editor_info stucture. 
   This param is not freed in this function.
 @return void.
*/
static void toolbar_set(SettingsInfo * settings_info)
{
	EditorMenu *menu = NULL;
	EditorToolbar *toolbar = NULL;

	gint msgtype = EMAIL_HTML;
	GtkWidget *email_type_entry = NULL;
	GtkTextBuffer *buffer = NULL;
	EditorInfo *editor_info = NULL;
	GdkColor color;
	UndoOpInfo *undo_op = NULL;
	color.pixel = 0;
	color.red = WHITE;
	color.green = WHITE;
	color.blue = WHITE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM(settings_info == NULL, "Input param check failed", return);
	CHECK_PARAM(settings_info->editor_info == NULL, "Input param check failed", return);
	CHECK_PARAM(settings_info->editor_info->menu == NULL, "Input param check failed", return);
	CHECK_PARAM(settings_info->editor_info->toolbar == NULL,
		    "Input param check failed", return);
	CHECK_PARAM(settings_info->editor_info->msgInfo == NULL,
		    "Input param check failed", return);
	CHECK_PARAM(settings_info->editor_info->msgInfo->settings == NULL,
		    "Input param check failed", return);

	menu = settings_info->editor_info->menu;
	toolbar = settings_info->editor_info->toolbar;

	editor_info = settings_info->editor_info;

	if (editor_info->msgInfo->settings->type == EMAIL_TEXT) {
		editor_info->emailType = PLAIN_EDITOR;
	} else {
		editor_info->emailType = HTML_EDITOR;
	}

	buffer = gtk_text_view_get_buffer((GtkTextView *) (editor_info->msgFields->body));

	email_type_entry = EMAIL_UI_LOOKUP_OBJECT(settings_info->settings_dialog, "email_type");
	msgtype = gtk_combo_box_get_active((GtkComboBox *) email_type_entry);
	if (msgtype == PLAIN_MAIL) {	/*if plain mail */
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *)
						  toolbar->bold, FALSE);
		gtk_widget_set_sensitive(toolbar->bold, FALSE);
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *)
						  toolbar->italic, FALSE);
		gtk_widget_set_sensitive(toolbar->italic, FALSE);
		gtk_widget_set_sensitive(toolbar->fontname, FALSE);
		gtk_widget_set_sensitive(toolbar->fontsize, FALSE);
		gtk_widget_set_sensitive(toolbar->fgcolor, FALSE);
		g_signal_handlers_block_by_func(G_OBJECT(toolbar->bullets),
						(gpointer) on_format_button_click, editor_info);
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *)
						  toolbar->bullets, FALSE);
		g_signal_handlers_unblock_by_func(G_OBJECT(toolbar->bullets),
						  (gpointer) on_format_button_click, editor_info);
		gtk_widget_set_sensitive(toolbar->bullets, FALSE);
		gtk_widget_set_sensitive(menu->format_menuitem, FALSE);
		gtk_widget_set_sensitive(menu->attach_image, FALSE);

		if (editor_info->emailType != HTML_EDITOR) {
			editor_info->emailType = PLAIN_EDITOR;
		}

		undo_op = update_undo_on_msg_type_change(editor_info);
		//undodel(buffer, &start_it, &end_it, editor_info);

		g_signal_handlers_block_by_func(G_OBJECT(buffer), (gpointer) undodel, editor_info);
		gtk_widget_modify_base((GtkWidget *) editor_info->msgFields->body,
				       GTK_STATE_NORMAL, &color);
		remove_tags_on_mail_type_change(buffer);
		remove_inline_images(buffer);
		g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
						  (gpointer) undodel, editor_info);
		editor_info->emailType = PLAIN_EDITOR;
	} else {
		editor_info->emailType = HTML_EDITOR;
		if (gtk_widget_is_focus(GTK_WIDGET(editor_info->msgFields->body)) == TRUE) {
			gtk_widget_set_sensitive(toolbar->bold, TRUE);
			gtk_widget_set_sensitive(toolbar->italic, TRUE);
			gtk_widget_set_sensitive(toolbar->fontname, TRUE);
			gtk_widget_set_sensitive(toolbar->fontsize, TRUE);
			gtk_widget_set_sensitive(toolbar->fgcolor, TRUE);
			gtk_widget_set_sensitive(toolbar->bullets, TRUE);
			gtk_widget_set_sensitive(menu->format_menuitem, TRUE);
			gtk_widget_set_sensitive(menu->attach_image, TRUE);
		}
		undo_op = update_undo_on_msg_type_change(editor_info);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}

/**
 This is a helper function to fill the sending option dialog.
 @param settings is a MsgInfo structure pointer.
   This param is not freed in this function.
 @param settings_dialog is a pointer to the sending option window
 
 @return void.
*/
static void
ui_editor_fill_sendoption_dialog(const EmailSpecificSettings * settings,
				 GtkWidget * settings_dialog)
{
	GtkWidget *priority_entry = NULL;
	GtkWidget *email_type_entry = NULL;
	GtkWidget *req_read_receipt = NULL;
	GtkWidget *copy_to_own = NULL;
	GtkWidget *send_mail_entry = NULL;
#if SMIME
	GtkWidget *smime_entry = NULL;

	smime_entry = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "smime_type");
#endif
	priority_entry = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "priority");
	email_type_entry = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "email_type");
	req_read_receipt = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "read_receipt");
	copy_to_own = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "copy_to_own");
	send_mail_entry = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "send_mail");

	if (settings == NULL) {
		return;
	}
	if (settings->priority == HIGH) {
		gtk_combo_box_set_active((GtkComboBox *) priority_entry, PRIORITY_HIGH);
	} else if (settings->priority == LOW) {
		gtk_combo_box_set_active((GtkComboBox *) priority_entry, PRIORITY_LOW);
	} else {
		gtk_combo_box_set_active((GtkComboBox *) priority_entry, PRIORITY_NORMAL);
	}
	if (settings->type == EMAIL_HTML) {
		gtk_combo_box_set_active((GtkComboBox *) email_type_entry, HTML_MAIL);
	} else {
		gtk_combo_box_set_active((GtkComboBox *) email_type_entry, PLAIN_MAIL);
	}
	/*setting security entry */
#if SMIME
	if (settings->smime_type == SMIME_SIGN) {
		gtk_combo_box_set_active((GtkComboBox *) smime_entry, SIGN);
	} else if (settings->smime_type == SMIME_ENCRYPT) {
		gtk_combo_box_set_active((GtkComboBox *) smime_entry, ENCRY);
	} else if (settings->smime_type == SMIME_SIGN_ENCRYPT) {
		gtk_combo_box_set_active((GtkComboBox *) smime_entry, SIGN_ENCRY);
	} else {
		gtk_combo_box_set_active((GtkComboBox *) smime_entry, NONE);
	}
#endif
	if (settings->read_receipt_request == 1) {
		gtk_toggle_button_set_active((GtkToggleButton *) req_read_receipt, TRUE);
	} else {
		gtk_toggle_button_set_active((GtkToggleButton *) req_read_receipt, FALSE);
	}
	if (settings->copy_to_own_address == 1) {
		gtk_toggle_button_set_active((GtkToggleButton *) copy_to_own, TRUE);
	} else {
		gtk_toggle_button_set_active((GtkToggleButton *) copy_to_own, FALSE);
	}
	if (settings->options == SEND_NEXT_CONNECTION) {
		gtk_combo_box_set_active((GtkComboBox *) send_mail_entry, NEXT_CONN);
	} else if (settings->options == SEND_ON_REQUEST) {
		gtk_combo_box_set_active((GtkComboBox *) send_mail_entry, ON_REQUEST);
	} else {
		gtk_combo_box_set_active((GtkComboBox *) send_mail_entry, IMMEDIATELY);
	}
	return;
}


/**
 This function is called by higher layers 
 @param  editor_info is th current editor's EditorInfo structure pointer.
   This param is not freed in this function.
   
 @return the pointer to the sendoption dialog.
*/
static GtkWidget *ui_editor_sendoption_window(EditorInfo * editor_info)
{
	GtkWidget *dialog_sending_opt = NULL;
	GtkWidget *dialog_action_area1 = NULL;
	GtkWidget *dialog_vbox = NULL;
	GtkWidget *vbox_send_opt = NULL;

	GtkSizeGroup *size_group = NULL;
	GtkWidget *caption_control = NULL;
	GtkWidget *combo_priority = NULL;
	GtkWidget *combo_emailtype = NULL;

	/*smime related */
#if SMIME
	GtkWidget *combo_smimetype = NULL;
#endif
	GtkWidget *checkbutton_read_receipt = NULL;
	GtkWidget *combo_email_acc = NULL;
#if RETAINED_EMAIL
	GtkWidget *combo_send_email = NULL;
#endif
	GtkWidget *button_ok = NULL;
	GtkWidget *button_cancel = NULL;
	GList *acc_list = NULL;
	gint index = 0, accindex = 0;

	GtkWidget *checkbutton_copy_to_own = NULL;
	GSList *node = NULL;
	EmailSpecificSettings *settings = NULL;
	SettingsInfo *settings_info = NULL;
	gboolean noAccounts = FALSE;
	gchar *str = NULL;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(editor_info == NULL, "Input param check failed", return NULL);
	CHECK_PARAM(editor_info->msgInfo == NULL, "Input param check failed", return NULL);

	node = email_main_view->account_manager->configured_accounts;
	if (node != NULL && node->data != NULL) {
		if (g_ascii_strcasecmp((gchar *) node->data, DEFAULT) == 0) {
			noAccounts = TRUE;
		}
	}
	if (node != NULL) {
		while (node != NULL) {
			acc_list = g_list_append(acc_list, node->data);
			node = node->next;
		}
	} else {
		noAccounts = TRUE;
	}
	if (editor_info->msgInfo->settings == NULL) {
		return NULL;
	}
	settings_info = g_new0(SettingsInfo, 1);
	if (settings_info == NULL) {
		return NULL;
	}

	settings = editor_info->msgInfo->settings;
	dialog_sending_opt = gtk_dialog_new();
	gtk_dialog_set_has_separator(GTK_DIALOG(dialog_sending_opt), FALSE);
	gtk_window_set_title(GTK_WINDOW(dialog_sending_opt), _("emev_ti_edi_smtp_sendoptions"));
	dialog_vbox = GTK_DIALOG(dialog_sending_opt)->vbox;
	gtk_window_set_transient_for(GTK_WINDOW(dialog_sending_opt), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(dialog_sending_opt), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(dialog_sending_opt), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(dialog_sending_opt), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_widget_show(dialog_vbox);

	vbox_send_opt = gtk_vbox_new(TRUE, ZERO_SPACING);
	gtk_container_add(GTK_CONTAINER(dialog_vbox), vbox_send_opt);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	combo_priority = gtk_combo_box_new_text();
	gtk_combo_box_append_text((GtkComboBox *) combo_priority,
				  (const gchar *) (_("emev_va_smtp_sendopt_prireg")));
	gtk_combo_box_append_text((GtkComboBox *) combo_priority,
				  (const gchar *) (_("emev_va_smtp_sendopt_prilow")));
	gtk_combo_box_append_text((GtkComboBox *) combo_priority,
				  (const gchar *) (_("emev_va_smtp_sendopt_prihigh")));



	caption_control =
	    hildon_caption_new(size_group, _("emev_fi_edi_smtp_sendopt_priority"),
			       combo_priority, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox_send_opt), caption_control, FALSE, FALSE, ZERO_PADDING);

	combo_emailtype = gtk_combo_box_new_text();
	gtk_combo_box_append_text((GtkComboBox *) combo_emailtype,
				  (const gchar *) (_("emev_va_smtp_sendopt_mtype_plain")));
	gtk_combo_box_append_text((GtkComboBox *) combo_emailtype,
				  (const gchar *) (_("emev_va_smtp_sendopt_mtype_html")));

/*smime realted*/
#if SMIME
	combo_smimetype = gtk_combo_box_new_text();
	gtk_combo_box_append_text((GtkComboBox *) combo_smimetype,
				  (const gchar *) (_("emev_va_edi_smtp_sendopt_smime_none")));
	gtk_combo_box_append_text((GtkComboBox *) combo_smimetype,
				  (const gchar *) (_("emev_va_edi_smtp_sendopt_smime_crypt")));
	gtk_combo_box_append_text((GtkComboBox *) combo_smimetype,
				  (const gchar *) (_("emev_va_edi_smtp_sendopt_smime_sign")));
	gtk_combo_box_append_text((GtkComboBox *) combo_smimetype,
				  (const gchar *) (_("emev_va_edi_smtp_sendopt_smime_cryptsign")));

	caption_control =
	    hildon_caption_new(size_group, _("emev_fi_edi_smtp_sendopt_smime"),
			       combo_smimetype, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox_send_opt), caption_control, FALSE, FALSE, ZERO_PADDING);
#endif
	caption_control =
	    hildon_caption_new(size_group, _("emev_fi_edi_smtp_sendopt_msgtype"),
			       combo_emailtype, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox_send_opt), caption_control, FALSE, FALSE, ZERO_PADDING);

	checkbutton_read_receipt = gtk_check_button_new_with_mnemonic("");
	caption_control =
	    hildon_caption_new(size_group, _("emev_fi_edi_smtp_sendopt_reqrcpt"),
			       checkbutton_read_receipt, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox_send_opt), caption_control, FALSE, FALSE, ZERO_PADDING);

	checkbutton_copy_to_own = gtk_check_button_new_with_mnemonic("");
	caption_control =
	    hildon_caption_new(size_group, _("emev_va_smtp_sendopt_copyto_mailbox"),
			       checkbutton_copy_to_own, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox_send_opt), caption_control, FALSE, FALSE, ZERO_PADDING);

	combo_email_acc = gtk_combo_box_new_text();
	caption_control =
	    hildon_caption_new(size_group, _("emev_fi_edi_smtp_sendopt_service"),
			       combo_email_acc, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox_send_opt), caption_control, FALSE, FALSE, ZERO_PADDING);

	for (acc_list = g_list_nth(acc_list, 0); acc_list != NULL; acc_list = acc_list->next) {
		index++;
		if (acc_list->data != NULL) {
			gtk_combo_box_append_text((GtkComboBox *) combo_email_acc,
						  (const gchar *) acc_list->data);
			if (g_ascii_strncasecmp((gchar *) acc_list->data, settings->account,
						MAX_STR_LEN((gchar *) acc_list->data,
							    settings->account)) == 0) {
				accindex = index - 1;
			}

		}
	}
	acc_list = g_list_nth(acc_list, 0);
	if (settings != NULL && settings->account != NULL) {
		gtk_combo_box_set_active((GtkComboBox *) combo_email_acc, accindex);
	}
	if (noAccounts == TRUE) {
		gtk_combo_box_append_text((GtkComboBox *) combo_email_acc,
					  _("emev_va_smtp_sendopt_none_defined"));
		gtk_combo_box_set_active((GtkComboBox *) combo_email_acc, 0);
	}

#if RETAINED_EMAIL
	combo_send_email = gtk_combo_box_new_text();
	caption_control =
	    hildon_caption_new(size_group, _("emev_fi_edi_smtp_sendopt_whensend"),
			       combo_send_email, NULL, HILDON_CAPTION_OPTIONAL);
	/*ugly hack for removing ellipsing:( */
	str = g_strdup_printf("%s  ", _("emev_va_smtp_sendopt_send_nextcon"));
	gtk_box_pack_start(GTK_BOX(vbox_send_opt), caption_control, FALSE, FALSE, 0);
	gtk_combo_box_append_text((GtkComboBox *) combo_send_email,
				  (const gchar *) (_("emev_va_smtp_sendopt_send_now")));
	gtk_combo_box_append_text((GtkComboBox *) combo_send_email, str);
	gtk_combo_box_append_text((GtkComboBox *) combo_send_email,
				  (const gchar *) (_("emev_va_smtp_sendopt_send_onrequest")));
	g_free(str);
	gtk_combo_box_set_active((GtkComboBox *) combo_send_email, NEXT_CONN);
#endif

	gtk_widget_show_all(vbox_send_opt);
	gtk_widget_show(dialog_sending_opt);

	dialog_action_area1 = GTK_DIALOG(dialog_sending_opt)->action_area;
	gtk_widget_show(dialog_action_area1);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1), GTK_BUTTONBOX_SPREAD);

	button_ok = gtk_button_new_with_label(_("emev_bd_send_options_ok"));
	gtk_widget_show(button_ok);
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_sending_opt), button_ok, 0);
	GTK_WIDGET_SET_FLAGS(button_ok, GTK_CAN_DEFAULT);

	button_cancel = gtk_button_new_with_label(_("emev_bd_send_options_cancel"));
	gtk_widget_show(button_cancel);
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_sending_opt), button_cancel, 0);
	GTK_WIDGET_SET_FLAGS(button_cancel, GTK_CAN_DEFAULT);

	settings_info->settings_dialog = dialog_sending_opt;
	settings_info->editor_info = editor_info;

	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_sending_opt, dialog_sending_opt, "settingsDialog");
	EMAIL_UI_HOOKUP_OBJECT(dialog_sending_opt, combo_priority, "priority");
	EMAIL_UI_HOOKUP_OBJECT(dialog_sending_opt, combo_emailtype, "email_type");
#if SMIME
	EMAIL_UI_HOOKUP_OBJECT(dialog_sending_opt, combo_smimetype, "smime_type");
#endif
	EMAIL_UI_HOOKUP_OBJECT(dialog_sending_opt, checkbutton_read_receipt, "read_receipt");
	EMAIL_UI_HOOKUP_OBJECT(dialog_sending_opt, checkbutton_copy_to_own, "copy_to_own");
	EMAIL_UI_HOOKUP_OBJECT(dialog_sending_opt, combo_email_acc, "account");
#if RETAINED_EMAIL
	EMAIL_UI_HOOKUP_OBJECT(dialog_sending_opt, combo_send_email, "send_mail");
#endif

	ui_editor_fill_sendoption_dialog(settings, dialog_sending_opt);
	g_signal_connect(G_OBJECT(button_ok), "clicked",
			 G_CALLBACK(ui_ok_sendoption_dialog_cb), settings_info);
	g_signal_connect(G_OBJECT(button_cancel), "clicked",
			 G_CALLBACK(ui_cancel_sendoption_dialog_cb), (gpointer) settings_info);
	if (ossohelp_exists(HELP_ID_SENDINGOPTIONS) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) dialog_sending_opt);
		g_signal_connect(G_OBJECT(dialog_sending_opt), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_SENDINGOPTIONS);
	}
	g_signal_connect(G_OBJECT(dialog_sending_opt), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), button_cancel);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return dialog_sending_opt;
}

/**
 This function is called by higher layers for  registering the callback

 @param win is a pointer to the menu item that registered the callback.
 @param  editor_info is th current editor's EditorInfo structure pointer.
   This param is not freed in this function.
   
 @return void
*/
void ui_editor_sendingOption_cb(GtkWidget * win, EditorInfo * editor_info)
{
	GtkWidget *sendopt_window = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (editor_info != NULL) {
		sendopt_window = ui_editor_sendoption_window(editor_info);
		if (sendopt_window != NULL) {
			gtk_widget_show(sendopt_window);
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}
