/**
 * @file email_clipboard.c
 *
 * This file contains Clipboard functions for Email application. 
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

//Common header files
#include "common.h"
#include "ui_common.h"

//Module specific Header files
#include "email_clipboard.h"


/*Global variables*/
static const GtkTargetEntry clipboard_targets[] = {
	{CLIPBOARD_EMAILS, 0, 0},
};

static const GtkTargetEntry clipboard_targets_and_attach[] = {
	{CLIPBOARD_EMAILS, 0, 0},
	{CLIPBOARD_EMAILS_AND_ATTACH, 0, 0},
};

static GdkAtom copied_emails_atom;
static GdkAtom copied_emails_and_attach_atom;



/**
This is a helper function to initilize the new clipboard target types

@param	void
@return void
*/
void osso_email_clipboard_init(void)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	copied_emails_atom = gdk_atom_intern(CLIPBOARD_EMAILS, FALSE);
	copied_emails_and_attach_atom = gdk_atom_intern(CLIPBOARD_EMAILS_AND_ATTACH, FALSE);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}



/**
This is a helper function to convert a list of message ID to a string 
each ID seperated by a newline which includes folder name and cut/copy
flag

@param selection   A GSList containing the list of message IDs	
@param cut         A boolean flag representing cut or copy	
@param foldername  Sorce folder name of message list.
@return gchar *    Newly created string having message list. 
*/
static gchar *osso_email_selection_to_string(GSList ** selection, gboolean cut, gchar * foldername)
{
	GString *selectionpaths = NULL;
	gchar *result = NULL;
	GSList *cursel = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (selection == NULL || foldername == NULL)
		return NULL;
	selectionpaths = g_string_new(cut ? "cut" : "copy");
	g_string_append_c(selectionpaths, '\n');
	g_string_append(selectionpaths, foldername);

	for (cursel = *selection; cursel != NULL; cursel = g_slist_next(cursel)) {
		if (cursel->data != NULL) {
			g_string_append_c(selectionpaths, '\n');
			g_string_append(selectionpaths, (gchar *) cursel->data);
		}
	}
	result = selectionpaths->str;
	g_string_free(selectionpaths, FALSE);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return result;
}




/**
This is a helper function to convert a string having list of message ID 
,cut/copy and foldername sepereated by newline to a  GSList.

@param string      gchar* containing the list of message IDs	
@param cut         An out param which will be stored a cut/copy flag
@param foldername  An out param of sorce folder name of message list.
@return GSList *   Newly created GSList having message list. 
*/

static GSList *osso_email_get_selection_from_string(gchar * string,
						    gboolean * cut, char **foldername)
{
	GSList *selection = NULL;
	gchar **paths = NULL;
	gint i = 0;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (string == NULL) {
		ULOG_ERR("Error validating input params\n");
		return NULL;
	}

	paths = g_strsplit(string, "\n", 0);
	if (paths[0] == NULL) {
		return NULL;
	}
	if (strcmp(paths[0], "cut") == 0) {
		*cut = TRUE;
	} else if (strcmp(paths[0], "copy") == 0) {
		*cut = FALSE;
	} else {
		return NULL;
	}
	*foldername = g_strdup(paths[1]);
	for (i = 2; paths[i] != NULL; i++) {
		selection = g_slist_append(selection, g_strdup(paths[i]));
	}

	g_strfreev(paths);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return selection;
}


/**
This is a callback function called by the clipboard when data is required for
pasting. this function is while setting the clipboard with data.

@param	clipboard	GtkClipboard reference which is calling this finction
@param	selectiondata	GtkSelectionData reference which should be filled
@param	info		This provides information about the requested target
@param	user_data	Pointer that is provided while registering this callback
@return void
*/

static void
osso_email_get_clipboard_cb(GtkClipboard * clipboard,
			    GtkSelectionData * selectiondata, guint info, gpointer user_data)
{
	gchar *data = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if ((selectiondata == NULL) || (copied_emails_atom == NULL) || (user_data == NULL)) {
		ULOG_ERR("Error validation input parameters\n");
		return;
	}
	data = (gchar *) user_data;
	gtk_selection_data_set(selectiondata, copied_emails_atom, 8,	//TODO : no of bits per unit had to be checked.
			       data, strlen(data));
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}



/**
This is a callback function called by the clipboard when the clipboard is 
cleared or the owner of the clipboard changed. The memory allocated for
the user_data while coping the contents to the clipboard are cleared here.

@param	clipboard	GtkClipboard reference which is calling this finction
@param	user_data	Pointer that is provided while registering this callback
@return void
*/
static void osso_email_clear_clipboard_cb(GtkClipboard * clipboard, gpointer user_data)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	g_free((gchar *) user_data);
	ui_undim_cut_messages();
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}



/**
This is a helper function to get the clipboard reference for a given widget

@param	widget		GtkWidget whose clipboard reference is required
@return GtkClipboard *	Pointer to the clipboard
*/
static GtkClipboard *osso_email_get_clipboard(GtkWidget * widget)
{
	ULOG_INFO("DISPLAY : %s\n", gdk_display_get_name(gdk_display_get_default()));
	return gtk_clipboard_get_for_display(gdk_display_get_default(), GDK_SELECTION_CLIPBOARD);
}



/**
This is a helper function to copy the email message ID list to clipboard
including the foldername and the cut/copy flag. If the messages are copied
then the target which will enable to paste these messages as email attachments
is also exported.

@param  widget		GtkWidget to whose clipboard the list has to be copied
@param  foldername	Sorce folder name of message list.
@param  selection	A GSList containing the list of message IDs	
@param  cut		A boolean flag representing cut or copy	
@return void
*/
void
osso_email_copy_to_clipboard(GtkWidget * fs, gchar * foldername, GSList * selection, gboolean cut)
{
	gchar *selectionlist = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (fs == NULL) {
		ULOG_ERR("Error Validating input parameters\n");
		return;
	}
	selectionlist = osso_email_selection_to_string(&selection, cut, foldername);
	if (selectionlist != NULL) {
		if (cut) {
			gtk_clipboard_set_with_data(osso_email_get_clipboard
						    (GTK_WIDGET(fs)), clipboard_targets,
						    G_N_ELEMENTS(clipboard_targets),
						    osso_email_get_clipboard_cb,
						    osso_email_clear_clipboard_cb, selectionlist);
		} else {
			gtk_clipboard_set_with_data(osso_email_get_clipboard
						    (GTK_WIDGET(fs)),
						    clipboard_targets_and_attach,
						    G_N_ELEMENTS
						    (clipboard_targets_and_attach),
						    osso_email_get_clipboard_cb,
						    osso_email_clear_clipboard_cb, selectionlist);
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}





/**
This is a helper function to get the email message ID list from clipboard
including the foldername and the cut/copy flag.
call osso_email_clipboard_has_emails first to know weather emails are
present in clipboard and osso_email_clipboard_has_emails_for_attachment 
to know if they can be pasted as attachements.

@param  widget		GtkWidget to get the clipboard.
@param  foldername	An out param of sorce folder name of message list.
@param  selection	A GSList out param contains the list of messages.
@param  cut		A boolean out param flag representing cut or copy	
@return void
*/
void
osso_email_get_list_from_clipboard(GtkWidget * fs,
				   gchar ** foldername, GSList ** selection, gboolean * cut)
{
	GtkSelectionData *selectiondata = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (fs == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	selectiondata =
	    gtk_clipboard_wait_for_contents(osso_email_get_clipboard
					    (GTK_WIDGET(fs)), copied_emails_atom);
	if (selectiondata == NULL) {
		ULOG_INFO("selectiondata is null\n");
		*selection = NULL;
		return;
	}
	if ((selectiondata->type != copied_emails_atom) || (selectiondata->length <= 0)) {
		*selection = NULL;
	} else {
		selectiondata->data[selectiondata->length] = '\0';
		*selection =
		    osso_email_get_selection_from_string(selectiondata->data, cut, foldername);
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}

/**
This is a helper function to clear content from clipboard after
cut-paste operation is completed.

@param  widget		GtkWidget to get the clipboard.
@return void
*/
void osso_email_cut_paste_from_clipboard_done(GtkWidget * widget)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	osso_email_copy_to_clipboard(widget, "", NULL, TRUE);
	gtk_clipboard_clear(osso_email_get_clipboard(widget));

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This is a helper function to know if the clipboard has got emails.

@param  widget		GtkWidget to get the clipboard.
@return gboolean	TRUE --> has emails, FALSE --> no emails present.
*/
gboolean osso_email_clipboard_has_emails(GtkWidget * fs)
{
	gboolean res = FALSE;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (fs == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return FALSE;
	}
	res =
	    gtk_clipboard_wait_is_target_available(osso_email_get_clipboard
						   (GTK_WIDGET(fs)), copied_emails_atom);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return res;
}

/**
This is a helper function to know if the clipboard has got emails and
they can be attached.

@param  widget		GtkWidget to get the clipboard.
@return gboolean	TRUE --> has emails for attachements 
			FALSE --> no emails present/or cannot be attached.
*/
gboolean osso_email_clipboard_has_emails_for_attachment(GtkWidget * fs)
{
	gboolean res = FALSE;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (fs == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return FALSE;
	}
	res =
	    gtk_clipboard_wait_is_target_available(osso_email_get_clipboard
						   (GTK_WIDGET(fs)), copied_emails_and_attach_atom);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return res;
}
