/**
 *  @file email_contacts.c
 *   
 *  This file has functions to create the windows for selecting contacts.
 *  
 *  Copyright (c) 2005 Nokia. All rights reserved.
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *   USA
 *  
 */


#include <libosso-abook/osso-abook.h>
#include <gtk/gtk.h>
#include <string.h>

#include <osso-log.h>

#include "ui_common.h"

#include "contact_utils.h"
#include "email_contacts.h"


static OssoABookContactModel *contact_model = NULL;
static EBook *book = NULL;
static EBookView *bookview = NULL;

/**
 *  This function stops the address book and unrefs the objects created and stops the bookview
 *  @param void 
 *     
 *  @return #void 
 */


void shutdown_addressbook(void)
{
	if (bookview) {
		e_book_view_stop(bookview);
		g_object_unref(bookview);
		bookview = NULL;
	}
	if (book) {
		g_object_unref(book);
		book = NULL;
	}
	osso_log(LOG_INFO, "Address book stopped\n");
	return;
}

/**
 *  This function sets final contacts list to the recepients buffer of the editor
 *  @param contacts_list of type #GList
 *  @param recp_buffer of type #GtkTextBuffer
 *     
 *  @return #void 
 */


static void set_contacts_to_recepients_buffer(GList * contacts_list, GtkTextBuffer * recp_buffer)
{
	EContact *contact = NULL;
	GSList *email_addrs_per_contact = NULL;
	GList *contact_node = NULL;
	gchar *econtact_id = NULL;

	if (!contacts_list || !recp_buffer)
		return;

	contact_node = contacts_list;
	while (contact_node) {
		contact = (EContact *) contact_node->data;
		email_addrs_per_contact = get_recipients_for_given_contact(contact);
		if (email_addrs_per_contact) {
			econtact_id = (gchar *) e_contact_get_const(contact, E_CONTACT_UID);
			ui_update_recipient_buffer(recp_buffer, email_addrs_per_contact,
						   econtact_id);
			g_slist_foreach(email_addrs_per_contact, (GFunc) g_free, NULL);
			g_slist_free(email_addrs_per_contact);
			email_addrs_per_contact = NULL;
		}
		contact_node = contact_node->next;
	}

	return;
}

/**
 *  This function takes GtkTextBuffer as parameter and creates and runs the contact_dialog  
 *  @param recp_buffer	of type #GtkTextBuffer 
 *     
 *  @return #void 
 */


static void run_email_recipients_dialog(GtkTextBuffer * recp_buffer)
{
	GtkWidget *contact_view = NULL;
	GtkWidget *contact_dialog = NULL;
	GList *contacts_list = NULL;
	gint response = -1;

	if (!recp_buffer || !contact_model)
		return;


	contact_view = osso_abook_contact_selector_new_basic(contact_model);
	osso_abook_contact_selector_set_minimum_selection(OSSO_ABOOK_CONTACT_SELECTOR(contact_view),
							  1);

	contact_dialog = osso_abook_select_dialog_new(OSSO_ABOOK_TREE_VIEW(contact_view));
	gtk_window_set_title(GTK_WINDOW(contact_dialog), _("mcen_ti_select_recipients"));
	gtk_window_set_transient_for(GTK_WINDOW(contact_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(contact_dialog), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(contact_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	response = gtk_dialog_run(GTK_DIALOG(contact_dialog));
	if (response == GTK_RESPONSE_OK) {
		contacts_list =
		    osso_abook_contact_selector_get_extended_selection(OSSO_ABOOK_CONTACT_SELECTOR
								       (contact_view));
	}

	if (contacts_list) {
		set_contacts_to_recepients_buffer(contacts_list, recp_buffer);
		g_list_free(contacts_list);
	}
	if (contact_view) {
		g_object_unref(contact_view);
		contact_view = NULL;
	}
	if (contact_model) {
		g_object_unref(contact_model);
		contact_model = NULL;
	}

	gtk_widget_destroy(contact_dialog);

	return;
}

/**
 *  This function is a call back function which starts the bookview
 *  @param book of type #EBook 
 *  @param status of type #EBookStatus 
 *  @param book_view of type #EBookView
 *  @param data of type #gpointer 
 *     
 *  @return #void 
 */


static void get_book_view_cb(EBook * book, EBookStatus status, EBookView * book_view, gpointer data)
{


	if (status != E_BOOK_ERROR_OK) {
		ULOG_CRIT("Unable to get bookview\n");
		gtk_infoprint(GTK_WINDOW(app), _("mail_ib_contacts_file_access_permission"));
		g_object_unref(book);
		book = NULL;
		return;
	}

	bookview = book_view;

	if (contact_model)
		osso_abook_tree_model_set_book_view(OSSO_ABOOK_TREE_MODEL(contact_model), bookview);

	e_book_view_start(bookview);
}

/**
 *  This function is a call back to open the book.
 *  @param book of type #EBook 
 *  @param status of type #EBookStatus 
 *  @param data of type #gpointer 
 *
 *  @return #void 
 */


static void book_open_cb(EBook * book, EBookStatus status, gpointer data)
{
	EBookQuery *query = NULL;


	if (status != E_BOOK_ERROR_OK) {
		ULOG_CRIT("Unable to Open addressbook\n");
		osso_log(LOG_CRIT, "Unable to open addressbook\n");
		gtk_infoprint(GTK_WINDOW(app), _("mail_ib_contacts_file_access_permission"));
		g_object_unref(book);
		book = NULL;
		return;
	}
	query = e_book_query_any_field_contains("");
	e_book_async_get_book_view(book, query, NULL, -1, get_book_view_cb, NULL);
	e_book_query_unref(query);
}

/**
 * This function opens the address book. It does not take any parameters.
 * @param void 
 *  
 * @return #gboolean 
 */


static gboolean open_addressbook(void)
{
	GError *error;


	error = NULL;
	book = e_book_new_system_addressbook(&error);
	if (!book) {
		if (error) {
			ULOG_CRIT("Unable to create/refer to the system addressbook %s\n",
				  error->message);
			osso_log(LOG_CRIT, "Unable to create/refer to system addressbook %s\n",
				 error->message);
			g_error_free(error);
			error = NULL;
		}
		return FALSE;
	}

	e_book_async_open(book, FALSE, book_open_cb, NULL);

	return TRUE;
}

/**
 * This public function starts the email recipients dialog and sets the contacts list to the given text buffer
 * @param recp_buffer of type #GtkTextBuffer 
 *
 * @return #void 
 */


void start_email_recipients_dialog(GtkTextBuffer * recp_buffer)
{
	contact_model = osso_abook_contact_model_new();
	if (!open_addressbook()) {
		gtk_infoprint(GTK_WINDOW(app), _("mail_ib_contacts_file_access_permission"));
		osso_log(LOG_ERR, "Unable to launch address book\n");
		if (contact_model) {
			g_object_unref(contact_model);
		}
		return;
	}
	run_email_recipients_dialog(GTK_TEXT_BUFFER(recp_buffer));

	return;
}

/**
 * This is a callback handler called when user is done with 'Add to contacts' dialog.
 * It destroys the contact_model and the dialog.
 *
 * @param  widget Pointer to the dialog
 * @param  data   Pointer to user data (not used here)
 * @return void
 */

static void finished_adding_to_contacts(GtkWidget * widget, gpointer data)
{
	if (contact_model) {
		g_object_unref(contact_model);
		contact_model = NULL;
	}

	gtk_widget_destroy(widget);
}

/**
 * This public function creates the 'Add to contacts' dialog.
 * User can choose to add the email_address to an existing contact, or as a new contact.
 * 
 * @param contact of type #gchar
 *    
 * @return void 
 */

void create_contact_dialog(const gchar * contact)
{
	OssoABookAccount *account = NULL;
	GtkWidget *add_to_contacts_dlg = NULL;
	gchar *email_addr = NULL;

	if (!contact)
		return;

	contact_model = osso_abook_contact_model_new();
	if (!open_addressbook()) {
		gtk_infoprint(GTK_WINDOW(app), _("mail_ib_contacts_file_access_permission"));
		osso_log(LOG_ERR, "Unable to launch address book\n");
		if (contact_model) {
			g_object_unref(contact_model);
			contact_model = NULL;
		}
		return;
	}

	email_addr = ui_parse_from_address(contact);

	account = osso_abook_account_get(EVC_EMAIL, NULL, email_addr);
	add_to_contacts_dlg = osso_abook_add_to_contacts_dialog_new(contact_model, account);
	gtk_window_set_transient_for(GTK_WINDOW(add_to_contacts_dlg), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(add_to_contacts_dlg), TRUE);
	g_object_unref(account);
	g_free(email_addr);

	g_signal_connect(add_to_contacts_dlg, "response", G_CALLBACK(finished_adding_to_contacts),
			 NULL);
	gtk_widget_show(add_to_contacts_dlg);
}

/**
 *  This is a helper function to add email addresses to the address book. It queries
 *  EBook to find if each email addr is already present in any of the contacts.
 *  If any contacts are matching, it uses the first one to add to recent list.
 *  Otherwise, it adds this email addr as a new contact.
 *
 *  @param  resolved_contacts  List of email addresses
 *  @return void 
 */

void add_contacts_to_addressbook(GSList * resolved_contacts)
{
	EContact *e_contact = NULL;
	GError *error = NULL;
	GSList *node = NULL;
	gchar *contact = NULL, *email_addr = NULL;
	GList *contacts_list = NULL;
	EBookQuery *query = NULL;

	if (!book) {
		if (!open_addressbook()) {
			osso_log(LOG_ERR, "Unable to open address book\n");
			return;
		}
	}

	for (node = resolved_contacts; node != NULL; node = node->next) {
		contact = (gchar *) node->data;
		email_addr = ui_parse_from_address(contact);
		query = e_book_query_field_test(E_CONTACT_EMAIL, E_BOOK_QUERY_IS, email_addr);

		if (!e_book_get_contacts(book, query, &contacts_list, &error)) {
			if (error) {
				osso_log(LOG_ERR, "%s\n", error->message);
				g_error_free(error);
				error = NULL;
			}
		} else if (contacts_list) {
			osso_abook_contact_commit((EContact *) contacts_list->data, FALSE, book);
			g_list_free(contacts_list);
		} else {
			osso_log(LOG_INFO,
				 "[%s]: No matching contacts for emailaddr '%s'. Adding as a new EContact!\n",
				 __FUNCTION__, contact);
			e_contact = get_e_contact_from_string(contact);
			if (e_contact) {
				osso_abook_contact_commit(e_contact, TRUE, book);
			}
		}

		e_book_query_unref(query);
		g_free(email_addr);
	}
}

/**
 * This is a helper function to commit a EContact to Address_Book application.
 *
 * @param  contact  Contact of type #EContact
 * @return void
 */
void commit_contact(EContact * contact)
{
	if (!contact || !book)
		return;

	osso_abook_contact_commit(contact, FALSE, book);
}

/**
 * This is helper function to add recently used EContacts to recent_group of the
 * Address Book application.
 *
 * @param  used_econtact_uids  List of UIDs of used EContacts
 * @return void
 */
void add_contacts_to_recent_list(GSList * used_econtact_uids)
{
	GError *error = NULL;
	GSList *node = NULL;
	GList *contacts_list = NULL;
	EBookQuery *query = NULL;

	if (!used_econtact_uids) {
		return;
	}

	if (!book) {
		if (!open_addressbook()) {
			osso_log(LOG_ERR, "Unable to open address book\n");
			return;
		}
	}

	for (node = used_econtact_uids; node != NULL; node = node->next) {
		query = e_book_query_field_test(E_CONTACT_UID, E_BOOK_QUERY_IS, node->data);

		if (!e_book_get_contacts(book, query, &contacts_list, &error)) {
			if (error) {
				osso_log(LOG_ERR, "%s\n", error->message);
				g_error_free(error);
				error = NULL;
			}
		} else {
			osso_abook_contact_commit((EContact *) contacts_list->data, FALSE, book);
			g_list_free(contacts_list);
		}
		e_book_query_unref(query);
	}
}
