/**
 * @file enablesignatures_callbacks.c
 *
 * This file contains the callback functions to implement the backend 
 * functionality
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */


/* Inclusion Header file containing all standerd paths.*/
#include "ui_common.h"

/*Inclusion of utilities header file.*/
#include "ui_utils.h"
#include "utils.h"
#include "osso-log.h"

/*UI related user defined header files.*/
#include "enablesignatures_callbacks.h"
#include "signatures_callbacks.h"
#include "email_mgr.h"
#include "callback.h"
#include "ui_account_store.h"


gchar *get_signature_filepath(gchar * account)
{
	gchar *config_path = NULL;
	gchar *signature_file_name = NULL;

	config_path = (gchar *) ui_get_config_dir();
	signature_file_name = g_strconcat((gchar *) g_get_home_dir(),
					  G_DIR_SEPARATOR_S, config_path,
					  G_DIR_SEPARATOR_S, account,
					  UNDERSCORE, SIGNATURENAME, NULL);
	g_free(config_path);
	return signature_file_name;
}

/**
 * This function is used to compare the two strings
 *
 * @param tree model which stores the data
 * @param data in the first row of the tree view
 * @param data in the second row of the tree view
 * @param userdata
 * @ return gint
 * */
static gint
sort_iter_compare_func_signature(GtkTreeModel * model,
				 GtkTreeIter * iter_a, GtkTreeIter * iter_b, gpointer userdata)
{
	gchar *name1 = NULL, *name2 = NULL, *compstr1 = NULL, *compstr2 = NULL;
	gint ret = 0;

	gtk_tree_model_get(model, iter_a, 1, &name1, -1);
	gtk_tree_model_get(model, iter_b, 1, &name2, -1);
	if (name1 == NULL || name2 == NULL) {
		if (name1 == NULL && name2 == NULL) {
			ULOG_DEBUG_L("[%s][%s]:leaves from here as both names are null:\n",
				     __FILE__, __FUNCTION__);
			return 0;
		}
		ret = (name1 == NULL) ? -1 : 1;
	} else {
		compstr1 = g_utf8_strdown(name1, -1);
		compstr2 = g_utf8_strdown(name2, -1);
		ret = g_utf8_collate(compstr1, compstr2);
		g_free(compstr1);
		g_free(compstr2);
	}
	g_free(name1);
	g_free(name2);


	return ret;
}


static void get_signature_info(gchar * account, gchar ** signature, gboolean * enablesignature)
{
	PrefsAccount *update_prefsAccount = NULL;
	gchar *signature_file = NULL;

	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);

	update_prefsAccount = ui_get_account_detail(account);
	if (update_prefsAccount == NULL) {
		ULOG_CRIT("[%s][%s]:ui_get_account_detail failed\n", __FILE__, __FUNCTION__);
		return;
	}
	signature_file = get_signature_filepath(account);
	read_file(signature_file, signature);
	*enablesignature = (gboolean) update_prefsAccount->advanced_settings.include_signature;
	ui_free_prefs_account_structure(update_prefsAccount);
	g_free(signature_file);

}

/**
This function is used to create the list store
@param void
@return pointer to the tree model.
*/
GtkTreeModel *enablesignatures_create_and_fill_email_accounts(GSList * accounts)
{
	GtkListStore *listStore = NULL;
	GtkTreeIter iter1;
	GSList *account_list_local = NULL;
	gboolean no_accounts_defined = FALSE;
	GtkTreeSortable *sortable = NULL;
	gchar *signaturetext = NULL;
	gboolean enablesignature = FALSE;

	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);

	if (accounts == NULL) {
		ULOG_ERR("Error : Input parameter validation\n");
		return NULL;
	}
	account_list_local = accounts;
	listStore =
	    gtk_list_store_new(4, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
	sortable = GTK_TREE_SORTABLE(listStore);
	gtk_tree_sortable_set_sort_func(sortable, 1,
					sort_iter_compare_func_signature,
					GINT_TO_POINTER(FALSE), NULL);
	gtk_tree_sortable_set_sort_column_id(sortable, 1, GTK_SORT_ASCENDING);

	while (account_list_local != NULL) {
		get_signature_info(account_list_local->data, &signaturetext, &enablesignature);
		gtk_list_store_append(listStore, &iter1);
		gtk_list_store_set(listStore, &iter1, 0, enablesignature,
				   1, account_list_local->data, -1);
		if ((signaturetext == NULL) ||
		    ((signaturetext != NULL) && (strlen(signaturetext) == 0))) {
			gtk_list_store_set(listStore, &iter1,
					   2, _("mcen_ia_email_signatures_notdefined"),
					   3, FALSE, -1);
		} else {
			gtk_list_store_set(listStore, &iter1, 2, signaturetext, 3, TRUE, -1);
		}
		account_list_local = g_slist_next(account_list_local);
		no_accounts_defined = TRUE;
		g_free(signaturetext);
		signaturetext = NULL;
	}
	if (no_accounts_defined == FALSE) {
		gtk_list_store_clear(listStore);
		g_free(listStore);
		listStore = NULL;
		return NULL;
	}

	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return GTK_TREE_MODEL(listStore);
}



/**
This function is used to define the list store columns
@param void
@returns GtkWidget pointer to the newly created widget.
*/
GtkWidget *enablesignatures_create_view_email_accounts(GSList * accounts)
{
	GtkTreeModel *model = NULL;
	GtkCellRenderer *renderer1 = NULL;
	GtkCellRenderer *renderer2 = NULL;
	GtkCellRenderer *renderer3 = NULL;
	GtkTreeViewColumn *col = NULL;
	GtkWidget *view = NULL;

	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);

	if (accounts == NULL) {
		ULOG_ERR("Error : Input parameter validation\n");
		return NULL;
	}
	model = enablesignatures_create_and_fill_email_accounts(accounts);
	if (model == NULL) {
		return NULL;
	}

	view = gtk_tree_view_new();
	g_object_set(G_OBJECT(view), "allow_checkbox_mode", FALSE, NULL);
	col = gtk_tree_view_column_new();
	renderer1 = gtk_cell_renderer_toggle_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
	gtk_tree_view_column_pack_start(col, renderer1, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer1, "active", 0);
	gtk_tree_view_column_add_attribute(col, renderer1, "sensitive", 3);

	col = gtk_tree_view_column_new();
	renderer2 = gtk_cell_renderer_text_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
	gtk_tree_view_column_pack_start(col, renderer2, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer2, "text", 1);
	gtk_cell_renderer_set_fixed_size(renderer2, SIGN_TREEVIEW_COL_WIDTH, -1);
	g_object_set(renderer2, ATTRIBUTE_ELLIPSIZE, PANGO_ELLIPSIZE_END,
		     ATTRIBUTE_ELLIPSIZE_SET, TRUE, NULL);

	col = gtk_tree_view_column_new();
	renderer3 = gtk_cell_renderer_text_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
	gtk_tree_view_column_pack_start(col, renderer3, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer3, "text", 2);
	gtk_cell_renderer_set_fixed_size(renderer3, SIGN_TREEVIEW_COL_WIDTH, -1);
	g_object_set(renderer3, ATTRIBUTE_ELLIPSIZE, PANGO_ELLIPSIZE_END,
		     ATTRIBUTE_ELLIPSIZE_SET, TRUE, NULL);
	gtk_cell_renderer_text_set_fixed_height_from_font((GtkCellRendererText *) renderer3, 1);

	gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);

	g_signal_connect(G_OBJECT(renderer1), "toggled",
			 G_CALLBACK(enablesignatures_updatevalue_email_accounts), view);

	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return view;
}

/**
This function is used to update the values after getting toggled
@param cell -- the cell which is toggled
@param strpath -- string path 
@param data -- tree view
@ returns void
*/
void
enablesignatures_updatevalue_email_accounts(GtkCellRendererToggle * cell,
					    const gchar * strpath, gpointer data)
{
	GtkTreeIter iter;
	GtkTreeModel *model = NULL;
	gboolean enabled = FALSE, sign_set = FALSE;

	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);

	if (data == NULL || strpath == NULL) {
		ULOG_WARN("[%s][%s]:the input data is NULL :\n", __FILE__, __FUNCTION__);
		return;
	}

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(data));
	if (gtk_tree_model_get_iter_from_string(model, &iter, strpath) == FALSE) {
		return;
	}
	gtk_tree_model_get(model, &iter, 0, &enabled, 3, &sign_set, -1);

	if (sign_set == FALSE && enabled == FALSE) {
		/*fix for bug #25162 when signature was not defined a confirmation note appeared
		   info note is used instead */
		gtk_infoprint(GTK_WINDOW(app), _("mcen_ib_no_sig"));
		return;
	}
	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, !enabled, -1);

	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is used to call the signatures window on clicking the edit window
@param editbutton -- the button which is clicked
@param enablesignature_account_view -- the list view that holds the list of accounts
@ returns void
*/
void
on_click_enablesignature_edit_button(GtkWidget * editbutton,
				     GtkWidget * enablesignature_account_view)
{
	GtkWidget *signature_window = NULL;
	GtkTreeModel *model = NULL;
	gchar *username_select = NULL;
	GtkTreeIter iter_selected;
	gboolean enabled_select;
	gboolean enabled_2_select;
	GtkTreeSelection *selection = NULL;

	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);

	if (enablesignature_account_view == NULL) {
		ULOG_WARN
		    ("[%s][%s]:the input enablesignature_account_view is NULL :\n",
		     __FILE__, __FUNCTION__);
		return;
	}
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(enablesignature_account_view));
	if (model == NULL) {
		ULOG_DEBUG_L("[%s][%s]:Failed to get the model :\n", __FILE__, __FUNCTION__);
		return;
	}
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(enablesignature_account_view));


	if (selection == NULL) {
		ULOG_DEBUG_L
		    ("[%s][%s]:leaves from here as Failed to get the selection :\n",
		     __FILE__, __FUNCTION__);
		return;
	}

	if (gtk_tree_selection_get_selected(selection, &model, &iter_selected)
	    == FALSE) {
		ULOG_DEBUG_L("[%s][%s]: Failed to get the selection :\n", __FILE__, __FUNCTION__);
		return;
	}

	gtk_tree_model_get(model, &iter_selected,
			   0, &enabled_select, 1, &username_select, 2, &enabled_2_select, -1);

	signature_window = create_signature_window(username_select, enablesignature_account_view);
	if (signature_window != NULL)
		gtk_widget_show(signature_window);

	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
}

/**
This function is used to call the signatures window on clicking the edit window
@param editbutton -- the button which is clicked
@param enablesignaturedata -- the data that holds signature specific information
@ returns void
*/
void on_click_enablesignature_ok_button(GtkWidget * button, gpointer enablesignaturedata)
{
	GSList *enablesignature_data_list_local = NULL;
	GtkWidget *enablesignature_window = NULL;
	GtkWidget *enablesignature_accounts_treeview = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	gboolean result = FALSE;
	gboolean enabled = FALSE;
	gboolean enabled_2 = FALSE;
	gchar *username = NULL;
	PrefsAccount *update_prefsAccount = NULL;

	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);

	if (enablesignaturedata == NULL) {
		ULOG_WARN("[%s][%s]:the input enablesignaturedata is NULL :\n",
			  __FILE__, __FUNCTION__);
		return;
	}

	enablesignature_data_list_local = (GSList *) enablesignaturedata;
	enablesignature_window = GTK_WIDGET(enablesignature_data_list_local->data);
	enablesignature_data_list_local = g_slist_next(enablesignature_data_list_local);
	if (enablesignature_data_list_local != NULL) {
		enablesignature_accounts_treeview =
		    GTK_WIDGET(enablesignature_data_list_local->data);
	}

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(enablesignature_accounts_treeview));
	result = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

	while (result == TRUE) {
		gtk_tree_model_get(model, &iter, 0, &enabled, 1, &username, 2, &enabled_2, -1);
		update_prefsAccount = ui_get_account_detail(username);
		if (update_prefsAccount == NULL) {
			g_free(username);
			gtk_widget_destroy(GTK_WIDGET(enablesignature_window));
			return;
		}
		update_prefsAccount->advanced_settings.include_signature = enabled;

		ui_dbus_set_account_settings(ADVANCED_SETTING, update_prefsAccount);
		ui_init_account_store();
		ui_free_prefs_account_structure(update_prefsAccount);
		update_prefsAccount = NULL;

		result = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);
		g_free(username);
		username = NULL;
	}
	gtk_widget_destroy(GTK_WIDGET(enablesignature_window));
	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
}

/**
This function is used to call the signatures window on clicking the edit window
@param button -- the button which is clicked
@param enablesignaturedata -- the enablesignature window 
@return void
*/
void on_click_enablesignature_cancel_button(GtkWidget * button, gpointer enablesignaturedata)
{
	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);
	if (enablesignaturedata == NULL) {
		ULOG_WARN("[%s][%s]:the input enablesignaturedata is NULL :\n",
			  __FILE__, __FUNCTION__);
		return;
	}

	gtk_widget_destroy(GTK_WIDGET(enablesignaturedata));
	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
}


/**
This function is used to delete the signature specific information on deletion of the account
@param fileptr  Holds pointer to the file.
@return pointer to the filecontent   Holds the content of the file.
*/
gchar *read_stream_from_file_to_string(FILE * fileptr)
{
	GByteArray *array = NULL;
	gchar buf[1021] = "\0";
	gint n_read = 0;
	gchar *filecontent = NULL;

	ULOG_DEBUG_L("[%s][%s]:entered the function :\n", __FILE__, __FUNCTION__);

	if (fileptr == NULL) {
		ULOG_WARN("[%s][%s]:the input parameter is null :\n", __FILE__, __FUNCTION__);
		return NULL;
	}

	array = g_byte_array_new();

	while ((n_read = (gint) fread(buf, sizeof(gchar), sizeof(buf), fileptr)) > 0) {
		if (n_read < (gint) sizeof(buf) && ferror(fileptr)) {
			break;
		}
		g_byte_array_append(array, buf, (guint) n_read);
		buf[0] = '\0';
	}

	if (ferror(fileptr) != 0) {
		ULOG_ERR("[%s][%s]:File operation error :\n", __FILE__, __FUNCTION__);
		g_byte_array_free(array, TRUE);
		return NULL;
	}

	buf[0] = '\0';
	g_byte_array_append(array, buf, 1);
	filecontent = (gchar *) array->data;
	g_byte_array_free(array, FALSE);

	ULOG_DEBUG_L("[%s][%s]:the leaves the function :\n", __FILE__, __FUNCTION__);

	return filecontent;
}


/**
This routine is used to check the file permissions of the given file.
@param  signPath  Holds the file path to be checked.
@return  0 for sucess, -1 for file permission denied, 
      -2 on file not found. 
*/
gint check_sign_file_permission(const gchar * signPath)
{
	gint retVal = 0;
	struct stat buf;

	if (signPath == NULL) {
		retVal = -2;
		return retVal;
	}

	if (stat(signPath, &buf) < 0) {
		retVal = -2;
	} else {
		if ((buf.st_mode & S_IRUSR) && (buf.st_mode & S_IWUSR)) {
			retVal = 0;
		} else {
			retVal = -1;
		}
	}
	return retVal;
}

void rename_signature_filepath(gchar * account, gchar * old_account)
{
	gchar *old_path = NULL;
	gchar *signature_file_name = NULL;

	signature_file_name = get_signature_filepath(account);
	old_path = get_signature_filepath(old_account);
	move_file(old_path, signature_file_name, TRUE);
	g_print("new path is %s \n", signature_file_name);

}

void delete_signature_file(gchar * account)
{
	gchar *signature_file_name = NULL;

	signature_file_name = get_signature_filepath(account);
	gnome_vfs_unlink(signature_file_name);
	g_free(signature_file_name);

}

/* end of file */
