/**
 * @file findtxt.c
 *
 * This file contains all callbacks to the Find Option.
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/

#include "ui_common.h"
#include "editcallback.h"
#include "callback.h"

static GtkTextIter iter;
static GtkTextIter start;
static GtkTextIter end;
static gboolean tagflag = FALSE;

extern osso_context_t *ui_context;	/*declared in main.c */

/**
This function is called by higher layers 
@param win is a pointer to the widget that registered the callback.
@param entry is a pointer GtkEntry structure.
@return TRUE if text is found else FALSE.
*/
static void ui_find_string_cb(gchar * required_str, GtkTextBuffer * body_buffer, MsgFields * fields,
			      GtkWidget * find_toolbar)
{
	gboolean success = FALSE;
	GtkTextIter sel_start, sel_end;
	GtkTextMark *mark = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);


	if (tagflag == FALSE && body_buffer != NULL) {
		gtk_text_buffer_get_iter_at_offset(body_buffer, &iter, 0);
	} else if (gtk_text_buffer_get_selection_bounds(body_buffer, &sel_start, &sel_end) == TRUE) {
		gtk_text_buffer_move_mark_by_name(body_buffer, "insert", &start);
		gtk_text_buffer_move_mark_by_name(body_buffer, "selection_bound", &start);
		tagflag = FALSE;
		success = TRUE;
	} else {
		gtk_text_buffer_get_iter_at_offset(body_buffer, &iter, 0);
	}
	if (gtk_text_iter_forward_search
	    (&iter, required_str, GTK_TEXT_SEARCH_VISIBLE_ONLY, &start, &end, NULL) == TRUE) {
		gtk_text_buffer_select_range(body_buffer, &start, &end);
		mark = gtk_text_buffer_get_mark(body_buffer, "insert");
		gtk_text_view_scroll_mark_onscreen((GtkTextView *) fields->body, mark);
		tagflag = TRUE;
		iter = end;
	} else {
		if (success == FALSE) {
			gtk_infoprint((GtkWindow *) app, _("mail_find_ib_not_found"));
			hildon_find_toolbar_highlight_entry(HILDON_FIND_TOOLBAR(find_toolbar),
							    TRUE);
		} else {
			gtk_infoprint((GtkWindow *) app, _("mail_ib_search_complete"));
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function closes the find toolbar.

 @param find_toolbar is the pointer to the find toolbar widget.

 @return void
*/
static void find_toolbar_close_cb(GtkWidget * find_toolbar, GtkWidget * find_menu)
{
	gtk_widget_hide_all(find_toolbar);
	gtk_check_menu_item_set_active((GtkCheckMenuItem *) find_menu, FALSE);
	return;
}

/**
 Function to enable the hide headers button after.
 find on page is closed.
 
 @param find_toolbar is the pointer to the find toolbar widget.
 @param fields is the pointer to the Msgfields structure pointer.

 @return void.
*/
static void enable_header_button(GtkWidget * find_toolbar, MsgFields * fields)
{
	if (fields->hide_headers_button != NULL)
		gtk_widget_set_sensitive(fields->hide_headers_button, TRUE);
	return;
}

void show_find_toolbar_cb(GtkWidget * find_toolbar, MsgFields * fields)
{

	if (gtk_check_menu_item_get_active((GtkCheckMenuItem *) find_toolbar) == TRUE) {
		gtk_widget_show_all(fields->find_toolbar);
		if (fields->hide_headers_button != NULL)
			gtk_widget_set_sensitive(fields->hide_headers_button, FALSE);
	} else {
		gtk_widget_hide(fields->find_toolbar);
		if (fields->hide_headers_button != NULL)
			gtk_widget_set_sensitive(fields->hide_headers_button, TRUE);
	}
	return;
}

/**
 This function starts the find process on the text entered.

 @param find_toolbar is the pointer to the find toolbar widget.
 @param fields is the pointer to the Msgfields structure pointer.

 @return void.
*/
static void find_toolbar_search_cb(GtkWidget * find_toolbar, MsgFields * fields)
{
	gchar *string = NULL;
	GtkTextBuffer *body_buffer = NULL;
	GtkTextIter buffer_start, buffer_end;
	gchar *previous_search_data = NULL;

	g_object_get(G_OBJECT(find_toolbar), "prefix", &string, NULL);
	if ((string == NULL) || (strcmp(string, "") == 0)) {
		gtk_infoprint(GTK_WINDOW(app), _("mail_find_ib_enter_the_text_to_be_found"));
		g_free(string);
		return;
	}
	if (fields != NULL && fields->body == NULL && fields->mail_viewer != NULL) {

		previous_search_data =
		    g_object_get_data(G_OBJECT(fields->mail_viewer), "search_text");
		/*Check if previously selected text & present search text is same.If so
		 * search next else search new*/
		if (previous_search_data && !ui_compare_utf8_strings(previous_search_data, string)) {
			if (!mail_viewer_search_next(MAIL_VIEWER(fields->mail_viewer)))
				gtk_infoprint((GtkWindow *) app, _("mail_ib_search_complete"));
		} else {
			g_object_set_data_full(G_OBJECT(fields->mail_viewer), "search_text",
					       g_strdup(string), (GDestroyNotify) g_free);
			if (!mail_viewer_search(MAIL_VIEWER(fields->mail_viewer), string, TRUE)) {
				gtk_infoprint((GtkWindow *) app, _("mail_find_ib_not_found"));
				hildon_find_toolbar_highlight_entry(HILDON_FIND_TOOLBAR
								    (find_toolbar), TRUE);
			}

		}
	} else {
		body_buffer = gtk_text_view_get_buffer((GtkTextView *) fields->body);
		gtk_text_buffer_get_bounds(body_buffer, &buffer_start, &buffer_end);
		if (!gtk_text_buffer_get_slice(body_buffer, &buffer_start, &buffer_end, FALSE)) {
			gtk_infoprint((GtkWindow *) app, _("mail_ib_nothing_to_find"));
		} else {
			ui_find_string_cb(string, body_buffer, fields, find_toolbar);
		}
	}
	g_free(string);
	return;
}

void create_find_toolbar(MsgFields * fields, GtkWidget * vbox_editor, GtkWidget * find_menu)
{

	fields->find_toolbar = hildon_find_toolbar_new(NULL);
	gtk_widget_show_all(fields->find_toolbar);
	gtk_box_pack_start(GTK_BOX(vbox_editor), fields->find_toolbar, FALSE, FALSE, ZERO_PADDING);
	gtk_widget_hide(fields->find_toolbar);
	g_signal_connect(G_OBJECT(fields->find_toolbar), "search",
			 G_CALLBACK(find_toolbar_search_cb), fields);
	g_signal_connect(G_OBJECT(fields->find_toolbar), "close",
			 G_CALLBACK(find_toolbar_close_cb), find_menu);
	g_signal_connect(G_OBJECT(fields->find_toolbar), "close",
			 G_CALLBACK(enable_header_button), fields);
	return;
}
