/**
 * @file htcallback.c
 *
 * This file contains all callbacks to the editor for handling html 
 * editing features
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */

#include "ui_common.h"
#include "email_hildon.h"
#include "htcallback.h"
#include "html_parser.h"
#include "callbacks.h"
#include "undo.h"
#include "ui_defs.h"
#include "folder_mgmt.h"
#include "message_handler.h"
#include "ui_al_callbacks.h"
#include "ui_interface.h"
#include "ui_state_mgmt.h"

gboolean stop_format = FALSE;
gboolean key_release = TRUE;
static const guint16 font_sizes[] = { 6, 8, 10, 12, 16, 24, 32 };
GSList *color_list = NULL;

const gchar *tagnames[] = {
	"bold",
	"italic",
	"underline",
	"strike",
	"sup-srpt",
	"sub-srpt",
	"left",
	"center",
	"right",
	"bullet",
	"number",
	"image",
	"font",
	"forecolor",
	"backcolor",
	"\0"
};

static void
bullet_num_change_status(GtkWidget * button, EditorInfo * editor, gboolean active_state);

static GSList *get_taglist_for_tag_removal(GSList * taglist, gint tagid);
static void remove_tags_without_name(GtkTextBuffer * buffer, GtkTextIter * start, GtkTextIter * end,
				     gint tagid);
static gboolean consider_iter_for_linetag(GtkTextIter start_iter);
static gboolean get_listchar_bounds(GtkTextIter cur_iter, GtkTextIter * char_start,
				    GtkTextIter * char_end);
static void check_tags_on_listchars(GtkTextBuffer * buffer, GtkTextIter * start, GtkTextIter * end);
static void update_linetag_properties(EditorInfo * editor, GtkTextIter * iter,
				      gint * underline, gint * strikethrough);
static void check_apply_line_tags(GtkTextBuffer * buffer, GtkTextIter end_iter,
				  EditorInfo * editor);

/**
This will give a console print of attributes variables.

@param attribute is a pointer to the GtkTextAttributes structure.

@return void. 
*/
void print_text_attributes(GtkTextAttributes * attribute)
{
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	ULOG_DEBUG_L("\n*************** TEXT ATTRIBUTES ****************\n");
	ULOG_DEBUG_L("rise               = %d\n", attribute->appearance.rise);
	ULOG_DEBUG_L("underline          = %d\n", attribute->appearance.underline);
	ULOG_DEBUG_L("strikethrough      = %d\n", attribute->appearance.strikethrough);
	ULOG_DEBUG_L("justification      = %d\n", attribute->justification);
	if (pango_font_description_get_family(attribute->font) != NULL) {
		ULOG_DEBUG_L("font style         = %d\n",
			     pango_font_description_get_style(attribute->font));
		ULOG_DEBUG_L("font weight        = %d\n",
			     pango_font_description_get_weight(attribute->font));
		ULOG_DEBUG_L("font description   = %s\n",
			     pango_font_description_to_string(attribute->font));
	}
	ULOG_DEBUG_L("*************** TEXT ATTRIBUTES ****************\n");
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
  Helper function to font index given its name.

  @param  fontname  Name of the font
  @return gint      Font index
*/
static gint getindex_fontname(gchar * fontname)
{
	gint count = 0;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	for (count = 0; count < sizeof(font_families) / sizeof(gchar *); count++) {
		ULOG_DEBUG_F("[%s]-[%s]\n", font_families[count], fontname);

		if (!g_ascii_strcasecmp(font_families[count], fontname)) {
			return count;
		}
	}
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return 0;
}

/**
This is called to refresh alignment menu items upon change of cursor point
 or insertion point.

@param textview Is a pointer to the Textarea in html editor.
@param editor Pointer to EditorInfo structure.

@return void. 
*/
void update_alignment_menu(EditorInfo * editor)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextIter start_iter, end_iter;
	GtkTextAttributes *attribute = NULL;
	GSList *taglist = NULL, *templist = NULL;
	gchar *temp_char = NULL;
	GtkTextTag *tag = NULL;
	gboolean align_tag_found = FALSE;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (editor == NULL) {
		ULOG_ERR("Error validation input\n");
		return;
	}

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editor->msgFields->body));
	gtk_text_buffer_get_iter_at_mark(buffer, &start_iter, gtk_text_buffer_get_insert(buffer));
	end_iter = start_iter;
	gtk_text_iter_forward_char(&end_iter);
	temp_char = (gchar *) gtk_text_iter_get_text(&start_iter, &end_iter);
	if (g_ascii_strcasecmp(temp_char, HTML_NL) != 0) {
		gtk_text_iter_forward_char(&start_iter);
	}
	g_free(temp_char);
	if (gtk_text_iter_is_end(&start_iter) == TRUE) {
		gtk_text_iter_backward_char(&start_iter);
	}
	taglist = gtk_text_iter_get_tags(&start_iter);
	if (taglist != NULL) {
		for (templist = taglist; templist != NULL; templist = g_slist_next(templist)) {
			tag = (GtkTextTag *) templist->data;
			attribute = tag->values;
			if (attribute->justification == GTK_JUSTIFY_CENTER) {
				gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->
							       menu->centred, TRUE);
				align_tag_found = TRUE;
				editor->justification = OSSO_JUSTIFY_CENTER;
				break;
			} else if (attribute->justification == GTK_JUSTIFY_RIGHT) {
				gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->
							       menu->right, TRUE);
				align_tag_found = TRUE;
				editor->justification = OSSO_JUSTIFY_RIGHT;
				break;
			}
		}
		g_slist_free(taglist);
	}
	if (align_tag_found == FALSE) {
		editor->justification = OSSO_JUSTIFY_LEFT;
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->menu->left, TRUE);
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Call the function when ever the current text attribute change.
This is mainly called to refresh toolbar and menu items accordingly.

@param textview Is a pointer to the Textarea in html editor.
@param temp_attribute Is the temporary attribute that needs to be reflected on to the UI such as Bold, Italic etc.
@param editor Pointer to EditorInfo structure.

@return void. 
*/
void
set_attribute_to_current_attribute(GtkTextView * textview,
				   GtkTextAttributes * temp_attribute, EditorInfo * editor)
{
	FontInfo *fontinfo = NULL;
	gint fontindex = 0, font_size_index = 0;
	PangoAttribute *attr = NULL;
	PangoFontDescription *desc = NULL;
	gboolean newlist = FALSE;
	gint zoomedsize = 0;
	const gchar *fontname = NULL;
	gint default_fontindex = 0;
	GtkTextIter start, end;
	gint font_size = 0;


	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((temp_attribute == NULL) || (editor == NULL) || (textview == NULL)) {
		ULOG_ERR("Error validation input\n");
		return;
	}
	if (gtk_text_buffer_get_selection_bounds
	    (gtk_text_view_get_buffer(GTK_TEXT_VIEW(editor->msgFields->body)),
	     &start, &end) == FALSE) {
		update_alignment_menu(editor);
	} else {
		editor->justification = temp_attribute->justification;
		switch (editor->justification) {
		case OSSO_JUSTIFY_LEFT:
			gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->
						       menu->left, TRUE);
			break;
		case OSSO_JUSTIFY_RIGHT:
			gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->
						       menu->right, TRUE);
			break;
		case OSSO_JUSTIFY_CENTER:
			gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->
						       menu->centred, TRUE);
			break;
		default:
			gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->
						       menu->none, TRUE);
			break;
		}
	}
	/*set attribute font descrition to zoomed size. */
	if (temp_attribute->font != NULL) {
		zoomedsize = pango_font_description_get_size(temp_attribute->font);
		zoomedsize = (gint) (zoomedsize * (100.0 / editor->zoominfo->previous_level));
		pango_font_description_set_size(temp_attribute->font, zoomedsize);
	}
	fontinfo = getfontinfo(temp_attribute->font);
	default_fontindex = ui_get_editor_html_fontindex();
	fontname = font_families[default_fontindex];
	if (fontinfo == NULL) {
		return;
	}

	if (editor->attr_list == NULL) {
		newlist = TRUE;
		editor->attr_list = pango_attr_list_new();
	} else {
		newlist = FALSE;
	}

	if (fontinfo->name != NULL) {
		fontindex = getindex_fontname(fontinfo->name);
		gtk_combo_box_set_active((GtkComboBox *) editor->toolbar->fontname, fontindex);
		font_size = (gint) (pango_font_description_get_size(temp_attribute->font) /
				    (PANGO_SCALE * font_scaling_factor));
		font_size_index = get_index_font_size(font_size);
		gtk_combo_box_set_active(GTK_COMBO_BOX(editor->toolbar->fontsize), font_size_index);
	} else {
		gtk_combo_box_set_active((GtkComboBox *) editor->toolbar->fontname,
					 default_fontindex);
		gtk_combo_box_set_active(GTK_COMBO_BOX(editor->toolbar->fontsize),
					 DEFAULT_FONT_POS);
	}

	if (fontinfo->name != NULL) {
		attr = pango_attr_font_desc_new(temp_attribute->font);
		attr->start_index = 0;
		attr->end_index = -1;
	} else {
		desc = pango_font_description_new();
		pango_font_description_set_family(desc, fontname);
		pango_font_description_set_size(desc, DEFAULT_FONT_SIZE * PANGO_SCALE
						* font_scaling_factor);
		attr = pango_attr_font_desc_new(desc);
		attr->start_index = 0;
		attr->end_index = -1;
	}
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	if (fontinfo->bold == TRUE) {
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) editor->
						  toolbar->bold, TRUE);
	} else {
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) editor->
						  toolbar->bold, FALSE);
	}

	if (fontinfo->italic == TRUE) {
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) editor->
						  toolbar->italic, TRUE);
	} else {
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) editor->
						  toolbar->italic, FALSE);
	}

	if ((temp_attribute->font != NULL)
	    && (pango_font_description_get_family(temp_attribute->font) != NULL)) {
		attr =
		    pango_attr_family_new(pango_font_description_get_family(temp_attribute->font));
	} else {
		attr = pango_attr_family_new(fontname);
	}

	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	if ((temp_attribute->font != NULL)
	    && (pango_font_description_get_size(temp_attribute->font) != 0)) {
		attr = pango_attr_size_new(pango_font_description_get_size(temp_attribute->font));
	} else {
		attr = pango_attr_size_new(DEFAULT_FONT_SIZE * PANGO_SCALE * font_scaling_factor);
	}
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	if (temp_attribute->font != NULL) {
		attr = pango_attr_style_new(pango_font_description_get_style(temp_attribute->font));
		attr->start_index = 0;
		attr->end_index = -1;
		if (newlist == TRUE) {
			pango_attr_list_insert(editor->attr_list, attr);
		} else {
			pango_attr_list_change(editor->attr_list, attr);
		}

		attr =
		    pango_attr_weight_new(pango_font_description_get_weight(temp_attribute->font));
		attr->start_index = 0;
		attr->end_index = -1;
		if (newlist == TRUE) {
			pango_attr_list_insert(editor->attr_list, attr);
		} else {
			pango_attr_list_change(editor->attr_list, attr);
		}
	}

	attr =
	    pango_attr_foreground_new(temp_attribute->appearance.fg_color.red,
				      temp_attribute->appearance.fg_color.green,
				      temp_attribute->appearance.fg_color.blue);
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	/*Underline */
	attr = pango_attr_underline_new(temp_attribute->appearance.underline);
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	/*strike through */
	attr = pango_attr_strikethrough_new(temp_attribute->appearance.strikethrough);
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	/*rise */
	attr = pango_attr_rise_new(temp_attribute->appearance.rise);
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	g_free(fontinfo->size);
	g_free(fontinfo);
	if (desc != NULL) {
		pango_font_description_free(desc);
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

static void apply_attributes_to_font(GtkTextAttributes * temp_attribute, EditorInfo * editor)
{
	FontInfo *fontinfo = NULL;
	gint fontindex = 0, font_size_index = 0;
	PangoAttribute *attr = NULL;
	PangoFontDescription *desc = NULL;
	gboolean newlist = FALSE;
	gint zoomedsize = 0;
	const gchar *fontname = NULL;
	gint default_fontindex = 0;
	gint font_size = 0;


	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((temp_attribute == NULL) || (editor == NULL)) {
		ULOG_ERR("Error validation input\n");
		return;
	}
	/*set attribute font descrition to zoomed size. */
	zoomedsize = pango_font_description_get_size(temp_attribute->font);
	zoomedsize =
	    (gint) (zoomedsize * (100.0 / editor->zoominfo->previous_level) * font_scaling_factor);
	pango_font_description_set_size(temp_attribute->font, zoomedsize);
	fontinfo = getfontinfo(temp_attribute->font);
	default_fontindex = ui_get_editor_html_fontindex();
	fontname = font_families[default_fontindex];
	if (fontinfo == NULL) {
		return;
	}

	if (editor->attr_list == NULL) {
		newlist = TRUE;
		editor->attr_list = pango_attr_list_new();
	} else {
		newlist = FALSE;
	}

	if (fontinfo->name != NULL) {	/*family field in font dialog is not empty */
		fontindex = getindex_fontname(fontinfo->name);
		gtk_combo_box_set_active((GtkComboBox *) editor->toolbar->fontname, fontindex);
	} else {
		gtk_combo_box_set_active((GtkComboBox *) editor->toolbar->fontname,
					 default_fontindex);
	}
	if (zoomedsize != 0) {	/*size field in font dialog is not empty
				 */
		font_size =
		    (gint) (pango_font_description_get_size(temp_attribute->font) /
			    (PANGO_SCALE * font_scaling_factor));
		font_size_index = get_index_font_size(font_size);
		gtk_combo_box_set_active(GTK_COMBO_BOX(editor->toolbar->fontsize), font_size_index);
	} else {
		gtk_combo_box_set_active(GTK_COMBO_BOX(editor->toolbar->fontsize),
					 DEFAULT_FONT_POS);
	}

	attr = pango_attr_font_desc_new(temp_attribute->font);
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	if (fontinfo->bold == TRUE) {
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) editor->
						  toolbar->bold, TRUE);
	} else {
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) editor->
						  toolbar->bold, FALSE);
	}

	if (fontinfo->italic == TRUE) {
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) editor->
						  toolbar->italic, TRUE);
	} else {
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) editor->
						  toolbar->italic, FALSE);
	}

	if (pango_font_description_get_family(temp_attribute->font) != NULL) {
		attr =
		    pango_attr_family_new(pango_font_description_get_family(temp_attribute->font));
	} else {
		attr = pango_attr_family_new(fontname);
	}
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	attr = pango_attr_size_new(pango_font_description_get_size(temp_attribute->font));
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	attr = pango_attr_style_new(pango_font_description_get_style(temp_attribute->font));
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	attr = pango_attr_weight_new(pango_font_description_get_weight(temp_attribute->font));
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	attr =
	    pango_attr_foreground_new(temp_attribute->appearance.fg_color.red,
				      temp_attribute->appearance.fg_color.green,
				      temp_attribute->appearance.fg_color.blue);
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	/*Underline */
	attr = pango_attr_underline_new(temp_attribute->appearance.underline);
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	/*strike through */
	attr = pango_attr_strikethrough_new(temp_attribute->appearance.strikethrough);
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	/*rise */
	attr = pango_attr_rise_new(temp_attribute->appearance.rise);
	attr->start_index = 0;
	attr->end_index = -1;
	if (newlist == TRUE) {
		pango_attr_list_insert(editor->attr_list, attr);
	} else {
		pango_attr_list_change(editor->attr_list, attr);
	}

	g_free(fontinfo->size);
	g_free(fontinfo);
	if (desc != NULL) {
		pango_font_description_free(desc);
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
set the toolbar sensitive/insensitive based on the format selected for bullets
 & numbering.
 
@param tag bullet or number tag to be checked
@param editor Editor info representing the editor
 */
static void set_bullet_num_toolbuttons(GtkTextTag * tag, EditorInfo * editor)
{

	if (tag == NULL) {
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) editor->
						  toolbar->bullets, FALSE);
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->menu->bullets, FALSE);
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->menu->numbering, FALSE);
	} else if ((tag->name != NULL)
		   && (g_ascii_strcasecmp(tag->name, tagnames[TAG_BULLET]) == 0)) {
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) editor->
						  toolbar->bullets, TRUE);
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->menu->bullets, TRUE);
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->menu->numbering, FALSE);
	} else if ((tag->name != NULL)
		   && (g_ascii_strcasecmp(tag->name, tagnames[TAG_NUMBER]) == 0)) {
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->menu->numbering, TRUE);
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) editor->
						  toolbar->bullets, FALSE);
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->menu->bullets, FALSE);
	} else {
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) editor->
						  toolbar->bullets, FALSE);
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->menu->bullets, FALSE);
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) editor->menu->numbering, FALSE);
	}
	return;
}

/**
This is called on move-cursor event on textview. 
The text attributes at the current cursor position are updated 
and the toolbar information is updated accordingly.

@param textview Is a pointer to the Textarea in html editor.
@param editor Pointer to EditorInfo structure.

@return void. 
*/
void refresh_current_attributes(GtkTextView * textview, EditorInfo * editor)
{
	GtkTextAttributes *temp_attributes = NULL;
	GtkTextAttributes *attr2 = NULL;
	GtkTextBuffer *buffer = NULL;
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	gboolean selection = FALSE;
	gint startline = 0;
	gint endline = 0;
	gint templine = 0;
	GtkTextTag *tag = NULL;
	gunichar ch;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (textview == NULL) {
		ULOG_ERR("Invalid input parameter\n");
		return;
	}
	if (editor->emailType != HTML_EDITOR) {
		return;
	}
	stop_format = TRUE;

	buffer = gtk_text_view_get_buffer(textview);
	temp_attributes = gtk_text_attributes_new();
	if (temp_attributes == NULL) {
		ULOG_ERR("Error allocating memory\n");
		return;
	}

	selection = gtk_text_buffer_get_selection_bounds(buffer, &start_iter, &end_iter);
	tag = iter_has_bullet_or_number_tag(&start_iter);

	if (selection == FALSE) {
		gtk_text_buffer_get_iter_at_mark(buffer, &start_iter,
						 gtk_text_buffer_get_insert(buffer));
		gtk_text_iter_backward_char(&start_iter);
		/*Check if any inline images there , if so we move the iter
		   back ward to get the previous attributes of character */
		ch = gtk_text_iter_get_char(&start_iter);
		while (ch == 0xFFFC && !gtk_text_iter_is_start(&start_iter)) {
			gtk_text_iter_backward_char(&start_iter);
			ch = gtk_text_iter_get_char(&start_iter);
		}
		gtk_text_iter_get_attributes(&start_iter, temp_attributes);
		tag = iter_has_bullet_or_number_tag(&start_iter);
		set_bullet_num_toolbuttons(tag, editor);
	} else {
		startline = gtk_text_iter_get_line(&start_iter);
		endline = gtk_text_iter_get_line(&end_iter);
		gtk_text_iter_get_attributes(&start_iter, temp_attributes);
		/*Checking alignment attribute for each line */
		for (templine = startline; templine <= endline; templine++) {
			if (attr2 != NULL) {
				gtk_text_attributes_unref(attr2);
				attr2 = NULL;
			}
			attr2 = gtk_text_attributes_new();
			if (attr2 == NULL) {
				ULOG_ERR("Error allocating memory\n");
				break;
			}
			gtk_text_iter_get_attributes(&start_iter, attr2);
			if (temp_attributes->justification != attr2->justification) {
				temp_attributes->justification = GTK_JUSTIFY_FILL;
				break;
			}
			gtk_text_iter_forward_line(&start_iter);
		}
		if (attr2 != NULL) {
			gtk_text_attributes_unref(attr2);
			attr2 = NULL;
		}
		//check the attributes are same as starting one...      
	}

	set_attribute_to_current_attribute(textview, temp_attributes, editor);
	set_bullet_num_toolbuttons(tag, editor);
	gtk_text_attributes_unref(temp_attributes);
	stop_format = FALSE;

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Pre creating all supported tags. Except font and color tags. 
These are to be created runtime depending the selection.

@param textview Is a pointer to the Textarea in html editor.

@return void.
*/
void create_supported_tags(GtkTextView * textview)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextTag *boldtag = NULL;
	GtkTextTag *italictag = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (textview == NULL) {
		ULOG_ERR("Error input parameter\n");
		return;
	}

	buffer = gtk_text_view_get_buffer(textview);

	boldtag =
	    gtk_text_buffer_create_tag(buffer, tagnames[TAG_BOLD], "weight",
				       PANGO_WEIGHT_BOLD, NULL);
	g_object_set_data(G_OBJECT(buffer), tagnames[TAG_BOLD], boldtag);

	italictag =
	    gtk_text_buffer_create_tag(buffer, tagnames[TAG_ITALIC], "style",
				       PANGO_STYLE_ITALIC, NULL);
	g_object_set_data(G_OBJECT(buffer), tagnames[TAG_ITALIC], italictag);

	gtk_text_buffer_create_tag(buffer, tagnames[TAG_UNDERLINE], "underline",
				   PANGO_UNDERLINE_SINGLE, NULL);

	gtk_text_buffer_create_tag(buffer, tagnames[TAG_STRIKE_THROUGH],
				   "strikethrough", TRUE, NULL);

	gtk_text_buffer_create_tag(buffer, tagnames[TAG_SUPERSCRIPT],
				   "rise-set", TRUE, "rise", 10 * PANGO_SCALE,
				   "size", 8 * PANGO_SCALE, NULL);

	gtk_text_buffer_create_tag(buffer, tagnames[TAG_SUBSCRIPT], "rise_set",
				   TRUE, "rise", -10 * PANGO_SCALE, "size", 8 * PANGO_SCALE, NULL);

	gtk_text_buffer_create_tag(buffer, tagnames[TAG_LEFT_ALIGN],
				   "justification", GTK_JUSTIFY_LEFT, NULL);

	gtk_text_buffer_create_tag(buffer, tagnames[TAG_CENTER_ALIGN],
				   "justification", GTK_JUSTIFY_CENTER, NULL);

	gtk_text_buffer_create_tag(buffer, tagnames[TAG_RIGHT_ALIGN],
				   "justification", GTK_JUSTIFY_RIGHT, NULL);

	gtk_text_buffer_create_tag(buffer, tagnames[TAG_BULLET], "indent", 8, NULL);
	gtk_text_buffer_create_tag(buffer, tagnames[TAG_NUMBER], "indent", 8, NULL);
	gtk_text_buffer_create_tag(buffer, tagnames[TAG_IMAGE], NULL);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
* This function is used to get the font name.

* @param fontname_entry represents font name selection entry.

* @return gchar
*/
const gchar *get_edittoolbar_fontname(GtkWidget * fontname_entry, EditorInfo * editor)
{
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	gint idvalue = 0;

	if (fontname_entry == NULL) {
		ULOG_ERR("Input param check failed!\n");
		return NULL;
	}
	idvalue = gtk_combo_box_get_active((GtkComboBox *) fontname_entry);
	if (idvalue == -1 || idvalue == sizeof(font_families) / sizeof(gchar *)) {
		ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return NULL;
	} else {
		ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return font_families[idvalue];
	}
}

/**
* This function is used to get the font size.

* @param fontsize_entry represents font name selection entry.

* @return gchar
*/
const gchar *get_edittoolbar_fontsize(GtkWidget * fontsize_entry)
{
	gint idvalue = 0;
	const gchar *font_sizes[] = { "6", "8", "10", "12", "16", "24", "32" };

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (fontsize_entry == NULL) {
		ULOG_ERR("Input param check failed!\n");
		return NULL;
	}
	if (GTK_IS_COMBO_BOX(fontsize_entry)) {
		ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		idvalue = gtk_combo_box_get_active((GtkComboBox *) fontsize_entry);
		return (font_sizes[idvalue]);

	} else {
		ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return NULL;
	}
}

/**
This is called to insert list item in the body if bullet/no. is clicked
and the cursor is on the last and empty line of body.

@param buffer GtkTextBuffer pointer for textarea in html editor
@param start_iter  start iterator for list item insertion.
@param tagid value from supported TAGID enum.

@return void. 
*/
static void list_last_empty_line(GtkTextBuffer * buffer, GtkTextIter * start_iter, gint tagid)
{
	gchar *list_char = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (start_iter == NULL) || (tagid < 0)
	    || (tagid >= TAG_COUNT)) {
		ULOG_ERR("Invalid input parameter\n");
		return;
	}

	if ((gtk_text_buffer_get_line_count(buffer) - 1) != gtk_text_iter_get_line(start_iter)) {
		ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return;
	}
	if (gtk_text_iter_get_chars_in_line(start_iter) != 0) {
		ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return;
	}
	if (tagid == TAG_BULLET) {
		list_char = g_strdup(BULLET_CHAR);
		gtk_text_buffer_insert_with_tags_by_name(buffer, start_iter,
							 list_char, -1, tagnames[TAG_BULLET], NULL);
	} else if (tagid == TAG_NUMBER) {
		list_char = g_strdup(NUMBER_START);
		gtk_text_buffer_insert_with_tags_by_name(buffer, start_iter,
							 list_char, -1, tagnames[TAG_NUMBER], NULL);
	} else {
		ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return;
	}
	g_free(list_char);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is called to apply a particular tag on the selection.

@param buffer GtkTextBuffer pointer for textarea in html editor
@param start_iter selection start iterator.
@param end_iter selection end iterator.
@param tagid value from supported TAGID enum.

@return void. 
*/
void
apply_gtk_tag(GtkTextBuffer * buffer, GtkTextIter * start_iter, GtkTextIter * end_iter, gint tagid)
{
	gint offset = 0;
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (start_iter == NULL) || (end_iter == NULL)
	    || (tagid < 0) || (tagid >= TAG_COUNT)) {
		ULOG_ERR("Invalid input parameter\n");
		return;
	}
	offset = gtk_text_iter_get_offset(start_iter);
	gtk_text_buffer_apply_tag_by_name(buffer, tagnames[tagid], start_iter, end_iter);
	gtk_text_buffer_get_iter_at_offset(buffer, start_iter, offset);
	list_last_empty_line(buffer, start_iter, tagid);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This function recognizes onle those tags that we want to remove and prepares a list of tags.

@param taglist is list of all the tags present in a given selection.
@param tagid value from supported TAGID enum. It corresponds to the format selected for removal.

@return GSList of tags to be removed. 
*/
static GSList *get_taglist_for_tag_removal(GSList * taglist, gint tagid)
{
	GtkTextAttributes *attribute = NULL;
	GSList *tags = NULL;
	GSList *temptags = NULL;
	GtkTextTag *tag = NULL;

	if (taglist == NULL) {
		ULOG_ERR("Invalid input parameter\n");
		return NULL;
	}

	for (temptags = taglist; temptags != NULL; temptags = g_slist_next(temptags)) {
		tag = GTK_TEXT_TAG(temptags->data);
		attribute = tag->values;
		switch (tagid) {
		case TAG_BOLD:
			if ((attribute->font != NULL) &&
			    (pango_font_description_get_weight(attribute->font) ==
			     PANGO_WEIGHT_BOLD)) {
				tags = g_slist_append(tags, temptags->data);
			}
			break;
		case TAG_ITALIC:
			if ((attribute->font != NULL) &&
			    (pango_font_description_get_style(attribute->font) ==
			     PANGO_STYLE_ITALIC)) {
				tags = g_slist_append(tags, temptags->data);
			}
			break;
		case TAG_UNDERLINE:
			if (attribute->appearance.underline == TRUE) {
				tags = g_slist_append(tags, temptags->data);
			}
			break;
		case TAG_STRIKE_THROUGH:
			if (attribute->appearance.strikethrough == TRUE) {
				tags = g_slist_append(tags, temptags->data);
			}
			break;
		case TAG_SUPERSCRIPT:
			if (attribute->appearance.rise > 0) {
				tags = g_slist_append(tags, temptags->data);
			}
			break;
		case TAG_SUBSCRIPT:
			if (attribute->appearance.rise < 0) {
				tags = g_slist_append(tags, temptags->data);
			}
			break;
		case TAG_CENTER_ALIGN:
			if (attribute->justification == GTK_JUSTIFY_CENTER) {
				tags = g_slist_append(tags, temptags->data);
			}
			break;
		case TAG_RIGHT_ALIGN:
			if (attribute->justification == GTK_JUSTIFY_RIGHT) {
				tags = g_slist_append(tags, temptags->data);
			}
			break;
		case TAG_FONT:
		case TAG_FGCOLOR:
		case TAG_BGCOLOR:
		case TAG_BULLET:
		case TAG_NUMBER:
		default:
			break;
		}
	}
	return tags;
}

/**
This function removes tags from selected text without taking the tag name into account. Rather it considers text attributes to remove tag of required type.

@param buffer GtkTextBuffer pointer for textarea in html editor
@param start_iter selection start iterator for format removal.
@param end_iter selection end iterator for format removal.
@param tagid value from supported TAGID enum. It corresponds to the format selected for removal.

@return void. 
*/
static void remove_tags_without_name(GtkTextBuffer * buffer, GtkTextIter * start, GtkTextIter * end,
				     gint tagid)
{
	GSList *taglist = NULL;
	GSList *tags = NULL;
	GSList *toggled_tags = NULL;
	GSList *tmp_list = NULL;
	GtkTextTag *tag = NULL;
	GtkTextIter tmp;

	if (buffer == NULL) {
		ULOG_ERR("Invalid input parameter\n");
		return;
	}
	taglist = gtk_text_iter_get_tags(start);
	tmp = *start;

	while (gtk_text_iter_forward_to_tag_toggle(&tmp, NULL)) {
		if (gtk_text_iter_compare(&tmp, end) >= 0) {
			break;	/* past the end of the range */
		}
		toggled_tags = gtk_text_iter_get_toggled_tags(&tmp, TRUE);
		if (toggled_tags != NULL) {
			taglist = g_slist_concat(taglist, toggled_tags);
		}
	}

	/*taglist is list of all the tags present in selection. So we need to have only  those tags that we want to remove. */
	tags = get_taglist_for_tag_removal(taglist, tagid);

	tmp_list = tags;
	while (tmp_list != NULL) {
		tag = GTK_TEXT_TAG(tmp_list->data);
		gtk_text_buffer_remove_tag(buffer, tag, start, end);
		tmp_list = tmp_list->next;
	}

	g_slist_free(tags);
	g_slist_free(taglist);
}


/**
This is called to remove a particular tag on the selection.

@param buffer GtkTextBuffer pointer for textarea in html editor
@param start_iter selection start iterator.
@param end_iter selection end iterator.
@param tagid value from supported TAGID enum.

@return void. 
*/
void
remove_gtk_tag(GtkTextBuffer * buffer, GtkTextIter * start_iter, GtkTextIter * end_iter, gint tagid)
{

	if ((buffer == NULL) || (start_iter == NULL) || (end_iter == NULL)
	    || (tagid < 0) || (tagid >= TAG_COUNT)) {
		ULOG_ERR("Invalid input parameter\n");
		return;
	}
	gtk_text_buffer_remove_tag_by_name(buffer, tagnames[tagid], start_iter, end_iter);

	if ((tagid == TAG_FONT) || (tagid == TAG_LEFT_ALIGN) || (tagid == TAG_BULLET)
	    || (tagid == TAG_NUMBER) || (tagid == TAG_FGCOLOR) || (tagid == TAG_BGCOLOR)) {
		/* Since removal of these tags is not required through the 
		 * attribute value at the character */
		return;
	}

	/*If some formated text is copied from other application
	 * then _remove_tag_by_name () will not remove the tag of reqd. property
	 * since it recognizes the tag by name*/
	remove_tags_without_name(buffer, start_iter, end_iter, tagid);
}

/**
This function will remove the existing font tags between teh given offsets

@param buffer GtkTextBuffer in which the tag needs to be removed
@param start_offset starting offset
@param end_offset ending offset
*/
void remove_font_tags_at_offset(GtkTextBuffer * buffer, gint start_offset, gint end_offset)
{
	GSList *tags = NULL;
	GSList *temptags = NULL;
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	GtkTextIter tag_end;
	gint fontsize = 0;
	GtkTextTag *tag = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("Invalid input parameter\n");
		return;
	}

	gtk_text_buffer_get_iter_at_offset(buffer, &start_iter, start_offset);
	gtk_text_buffer_get_iter_at_offset(buffer, &end_iter, end_offset);

	while (gtk_text_iter_compare(&start_iter, &end_iter) < 0) {
		tags = gtk_text_iter_get_tags(&start_iter);
		tag_end = start_iter;
		for (temptags = tags; temptags != NULL; temptags = g_slist_next(temptags)) {
			fontsize = 0;
			g_object_get(G_OBJECT(temptags->data), SIZE, &fontsize, NULL);
			if (fontsize > 0) {
				tag = temptags->data;
				gtk_text_iter_forward_to_tag_toggle(&tag_end, tag);
				if (gtk_text_iter_compare(&tag_end, &end_iter) < 1) {
					gtk_text_buffer_remove_tag(buffer, tag, &start_iter,
								   &tag_end);
				} else {
					gtk_text_buffer_remove_tag(buffer, tag, &start_iter,
								   &end_iter);
				}
				/*create tag */
				/*apply rtag */
			}
		}
		if (gtk_text_iter_compare(&start_iter, &tag_end) == 0) {
			gtk_text_iter_forward_char(&start_iter);
		} else {
			start_iter = tag_end;
		}
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Prepares appropriate font tag to be applied on selected text. 
The tag will have name and size property depending upon name/size chosen from toolbar or font dialog.
If a name is selected then tag with new name will be prepared which will retain
size of selected text. same valid for Size.
@param buffer GtkTextBuffer pointer for textarea in html editor
@param fontname Name of the font. If NULL fontnames of selected text are retained.
@param fontsize Size of font to be applied. If 0 font size of selected text are retained.
@param attribute is set of font attributes
@param zoom_level is the zoom level of the text.

@return void. 
*/
static GtkTextTag *get_font_tag(GtkTextBuffer * buffer, gchar * fontname, gint fontsize,
				GtkTextAttributes * attribute, gint zoom_level)
{
	gchar *name_tag = NULL;
	gint size_tag = 0;
	PangoFontDescription *font = NULL;

	if (buffer == NULL) {
		ULOG_ERR("Invalid input parameter\n");
		return NULL;
	}

	font = attribute->font;
	/*Both name and size of text is to be retained. case when any these fields are empty in font selection dialog */
	if (fontname == NULL && fontsize == 0) {
		name_tag = (gchar *) pango_font_description_get_family(font);
		size_tag =
		    (gint) (pango_font_description_get_size(font) /
			    (PANGO_SCALE * font_scaling_factor));
	} else if (fontname == NULL) {
		/*font name on text to be retained. */
		name_tag = (gchar *) pango_font_description_get_family(font);
		size_tag = fontsize;
	} else if (fontsize == 0) {
		/*font size on text to be retained. */
		name_tag = fontname;
		size_tag =
		    (gint) (pango_font_description_get_size(font) /
			    (PANGO_SCALE * font_scaling_factor));
	} else {
		/*font name and size on text needn't be retained. */
		name_tag = fontname;
		size_tag = fontsize;
	}
	/*create tag  and return */
	return (gtk_text_buffer_create_tag(buffer, NULL, "font", name_tag,
					   "size",
					   (gint) (size_tag * PANGO_SCALE * font_scaling_factor *
						   (zoom_level / 100.0)), NULL));
}

/**
This function is invoked to apply selected fontname and size to the 
selected portion of text.

@param view pointer for textarea in html editor
@param fontname Name of the font
@param fontsize Size of font to be applied.
@param zoom_level is the zoom level of the text.
@param start_iter is start of selected text.
@param end_iter is end of selected text.

@return void. 
*/
void
apply_font_name_and_size(GtkTextView * textview, gchar * fontname,
			 gint fontsize, gint zoom_level,
			 GtkTextIter start_iter, GtkTextIter end_iter)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextTag *tag = NULL;
	GtkTextTag *font_tag = NULL;
	GSList *tags = NULL;
	GSList *temptags = NULL;
	GtkTextIter tag_end;
	gint font_size = 0;
	GtkTextAttributes *attribute = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (textview == NULL) {
		ULOG_ERR("Invalid input parameter\n");
		return;
	}

	buffer = gtk_text_view_get_buffer(textview);
	/*Browse from start to end of selected text to remove previously applied
	   font tag and apply the new one. */
	while (gtk_text_iter_compare(&start_iter, &end_iter) < 0) {
		tags = gtk_text_iter_get_tags(&start_iter);
		tag_end = start_iter;
		for (temptags = tags; temptags != NULL; temptags = g_slist_next(temptags)) {
			font_size = 0;
			g_object_get(G_OBJECT(temptags->data), SIZE, &font_size, NULL);
			if (font_size < 1) {
				continue;
			}
			/*text at the iter has some font tag. */
			tag = temptags->data;
			font_tag =
			    get_font_tag(buffer, fontname, fontsize, tag->values, zoom_level);
			gtk_text_iter_forward_to_tag_toggle(&tag_end, tag);
			if (gtk_text_iter_compare(&tag_end, &end_iter) < 1) {
				gtk_text_buffer_remove_tag(buffer, tag, &start_iter, &tag_end);
				gtk_text_buffer_apply_tag(buffer, font_tag, &start_iter, &tag_end);
			} else {
				gtk_text_buffer_remove_tag(buffer, tag, &start_iter, &end_iter);
				gtk_text_buffer_apply_tag(buffer, font_tag, &start_iter, &end_iter);
			}
		}
		if (gtk_text_iter_compare(&start_iter, &tag_end) == 0) {
			/*text at the iter doesn't have any font tag. */
			gtk_text_iter_forward_char(&tag_end);
			attribute = gtk_text_view_get_default_attributes(textview);
			font_tag = get_font_tag(buffer, fontname, fontsize, attribute, zoom_level);
			gtk_text_buffer_apply_tag(buffer, font_tag, &start_iter, &tag_end);
			gtk_text_attributes_unref(attribute);
			gtk_text_iter_forward_char(&start_iter);
		} else {
			start_iter = tag_end;
		}
	}

	raise_bold_italic_priority(buffer);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Checks whether an iterator is just before or after a bullet/number char.

@param start_iter is  iterator whose proximity to list char is checked.

@return TRUE if start_iter is just before or after a bullet/number char.. 
*/
static gboolean consider_iter_for_linetag(GtkTextIter start_iter)
{
	GtkTextIter start;
	GtkTextIter char_start;
	GtkTextIter char_end;

	start = start_iter;
	gtk_text_iter_set_line_offset(&start, 0);
	if (iter_has_bullet_or_number_tag(&start) == NULL) {
		return FALSE;
	}
	/*get the boundary values of list char */
	if (get_listchar_bounds(start, &char_start, &char_end) == FALSE) {
		return FALSE;
	}
	if ((gtk_text_iter_compare(&char_end, &start_iter) == 0)
	    || (gtk_text_iter_ends_line(&start_iter))) {
		return TRUE;
	}

	return FALSE;
}

/**
Applies formating on selected text from properties in the font dialog

@param editor Pointer to EditorInfo structure.
@param iterator PangoAttrIterator to apply font details on the selection.
@param temp_attribute is attribute set selected from font dialog.
@param start_iter is start of selection.
@param end_iter is end of selection.

@return void. 
*/
void
format_font_iterator(EditorInfo * editor, PangoAttrIterator * iterator,
		     GtkTextAttributes * temp_attribute, GtkTextIter start_iter,
		     GtkTextIter end_iter)
{

	PangoFontDescription *desc = NULL;
	PangoAttribute *pattr = NULL;
	PangoAttrColor *color = NULL;
	GdkColor gdkcolor;
	gint fontunderline = 0;
	gint fontstrikethrough = 0;
	gint fontrise = 0;
	gint size = 0;
	GtkTextBuffer *buffer = NULL;
	FontInfo *fontinfo = NULL;
	GSList *attrs = NULL;
	GSList *curattr = NULL;
	GtkTextIter start;
	GtkTextIter end;
	gint start_line = 0;
	gint end_line = 0;
	gboolean iter_near_listchar = FALSE;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((editor == NULL) || (iterator == NULL)) {
		ULOG_ERR("Invalid input parameter\n");
		return;
	}

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editor->msgFields->body));
	iter_near_listchar = consider_iter_for_linetag(start_iter);
	attrs = pango_attr_iterator_get_attrs(iterator);
	if (attrs == NULL) {
		ULOG_ERR("Error populating attributes\n");
		return;
	}

	for (curattr = attrs; curattr != NULL; curattr = curattr->next) {
		pattr = (PangoAttribute *) curattr->data;
		switch (pattr->klass->type) {
		case PANGO_ATTR_FONT_DESC:
			desc = pango_font_description_new();
			if (desc == NULL) {
				ULOG_ERR("Memory allocation error\n");
				return;
			}
			pango_attr_iterator_get_font(iterator, desc, NULL, NULL);
			break;
		case PANGO_ATTR_FOREGROUND:
			color = ((PangoAttrColor *)
				 pango_attr_iterator_get(iterator, PANGO_ATTR_FOREGROUND));
			gdkcolor.red = color->color.red;
			gdkcolor.blue = color->color.blue;
			gdkcolor.green = color->color.green;
			format_font_color(&gdkcolor, editor, start_iter, end_iter);

			break;
		case PANGO_ATTR_BACKGROUND:
			ULOG_ERR("COLOR VALUE FOUND\n");
			break;
		case PANGO_ATTR_FAMILY:
		case PANGO_ATTR_STYLE:
		case PANGO_ATTR_WEIGHT:
			break;
		case PANGO_ATTR_UNDERLINE:
			fontunderline = ((PangoAttrInt *)
					 pango_attr_iterator_get(iterator,
								 PANGO_ATTR_UNDERLINE))->value;
			if ((!fontunderline) && (iter_near_listchar)) {
				/*if underline attribute is unset near a listchar, ignore_strikethrough is set. When a character is inserted at the end of a list line or start of a list line just after unsetting the strikethrough, this variable is checked to see whether to apply underline on it */
				editor->ignore_underline = TRUE;
			}
			break;
		case PANGO_ATTR_STRIKETHROUGH:
			fontstrikethrough = ((PangoAttrInt *)
					     pango_attr_iterator_get(iterator,
								     PANGO_ATTR_STRIKETHROUGH))->
			    value;
			if ((!fontstrikethrough) && (iter_near_listchar)) {
				/*if strikethrough attribute is unset near a listchar, ignore_strikethrough is set. When a character is inserted at the end of a list line or start of a list line just after unsetting the strikethrough, this variable is checked to see whether to apply strikethrough on it */
				editor->ignore_strikethrough = TRUE;
			}
			break;
		case PANGO_ATTR_RISE:
			fontrise = ((PangoAttrInt *)
				    pango_attr_iterator_get(iterator, PANGO_ATTR_RISE))->value;
			break;
		case PANGO_ATTR_SHAPE:
		case PANGO_ATTR_SCALE:
		case PANGO_ATTR_FALLBACK:
		case PANGO_ATTR_LETTER_SPACING:
		case PANGO_ATTR_INVALID:
		case PANGO_ATTR_LANGUAGE:
		case PANGO_ATTR_VARIANT:
		case PANGO_ATTR_STRETCH:
		case PANGO_ATTR_SIZE:
		case PANGO_ATTR_UNDERLINE_COLOR:
		case PANGO_ATTR_STRIKETHROUGH_COLOR:
		case PANGO_ATTR_ABSOLUTE_SIZE:
			break;
		}
	}
	g_slist_foreach(attrs, (GFunc) pango_attribute_destroy, NULL);
	g_slist_free(attrs);
	attrs = NULL;

	/*font scaling factor is already multiplied */
	size = pango_font_description_get_size(desc) / font_scaling_factor;
	pango_font_description_set_size(desc, size);
	fontinfo = getfontinfo(desc);
	remove_tags_without_name(buffer, &start_iter, &end_iter, TAG_BOLD);
	remove_tags_without_name(buffer, &start_iter, &end_iter, TAG_ITALIC);
	remove_tags_without_name(buffer, &start_iter, &end_iter, TAG_UNDERLINE);
	remove_tags_without_name(buffer, &start_iter, &end_iter, TAG_STRIKE_THROUGH);
	if (fontinfo != NULL) {
		if (fontinfo->bold == TRUE) {
			apply_gtk_tag(buffer, &start_iter, &end_iter, TAG_BOLD);
		}
		if (fontinfo->italic == TRUE) {
			apply_gtk_tag(buffer, &start_iter, &end_iter, TAG_ITALIC);
		}
		if (fontinfo->name) {
			apply_font_name_and_size(GTK_TEXT_VIEW(editor->msgFields->body),
						 fontinfo->name, (gint) atoi(fontinfo->size),
						 editor->zoominfo->previous_level, start_iter,
						 end_iter);
		} else {
			apply_font_name_and_size(GTK_TEXT_VIEW(editor->msgFields->body), NULL,
						 (gint) atoi(fontinfo->size),
						 editor->zoominfo->previous_level, start_iter,
						 end_iter);
		}

		g_free(fontinfo->size);
		g_free(fontinfo);
	}
	if (fontunderline == TRUE) {
		apply_gtk_tag(buffer, &start_iter, &end_iter, TAG_UNDERLINE);
	}
	if (fontstrikethrough == TRUE) {
		apply_gtk_tag(buffer, &start_iter, &end_iter, TAG_STRIKE_THROUGH);
	}
	if (fontrise > 0) {
		remove_tags_without_name(buffer, &start_iter, &end_iter, TAG_SUBSCRIPT);
		apply_gtk_tag(buffer, &start_iter, &end_iter, TAG_SUPERSCRIPT);
	}
	if (fontrise < 0) {
		remove_tags_without_name(buffer, &start_iter, &end_iter, TAG_SUPERSCRIPT);
		apply_gtk_tag(buffer, &start_iter, &end_iter, TAG_SUBSCRIPT);
	}

	start_line = gtk_text_iter_get_line(&start_iter);
	gtk_text_buffer_get_iter_at_line(buffer, &start, start_line);
	end_line = gtk_text_iter_get_line(&end_iter);
	gtk_text_buffer_get_iter_at_line(buffer, &end, end_line);
	gtk_text_iter_forward_to_line_end(&end);
	check_tags_on_listchars(buffer, &start, &end);

	if (desc != NULL) {
		pango_font_description_free(desc);
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
reset underline/strikethrough values if cursor is present after a bullet/number. Reseting is done depending upon whether tag is present on first char of bulleted /numbered line or last char of previous line if the line doesn't have any other char.

@param editor Pointer to EditorInfo structure having editor related info.
@param iter is position of cursor.
@param underline is pointer to value for presence of underline at the cursor position. 
@param strikethrough is pointer to value for presence of strikethrough at the cursor position. 

@return void. 
*/
static void update_linetag_properties(EditorInfo * editor, GtkTextIter * iter,
				      gint * underline, gint * strikethrough)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextTagTable *tagtable = NULL;
	GtkTextTag *tag_underline = NULL;
	GtkTextTag *tag_strikethrough = NULL;
	GtkTextIter start_iter;
	GtkTextIter test_iter;
	GtkTextIter char_start;
	GtkTextIter char_end;

	start_iter = *iter;
	gtk_text_iter_set_line_offset(&start_iter, 0);
	if (iter_has_bullet_or_number_tag(&start_iter) == NULL) {
		return;
	}
	if (get_listchar_bounds(start_iter, &char_start, &char_end) == FALSE) {
		return;
	}
	if (gtk_text_iter_compare(&char_end, iter) != 0) {
		return;
	}
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editor->msgFields->body));
	tagtable = gtk_text_buffer_get_tag_table(buffer);
	tag_underline = gtk_text_tag_table_lookup(tagtable, tagnames[TAG_UNDERLINE]);
	tag_strikethrough = gtk_text_tag_table_lookup(tagtable, tagnames[TAG_STRIKE_THROUGH]);
	test_iter = *iter;
	/*conditions checked to see whether a bullet/numbered line doesn't have any other char */
	if (gtk_text_iter_ends_line(&test_iter)) {
		if (gtk_text_iter_backward_line(&test_iter)) {
			if (!gtk_text_iter_ends_line(&test_iter)) {
				gtk_text_iter_forward_to_line_end(&test_iter);
			}
		}
	}
	if ((tag_underline != NULL)
	    && (gtk_text_iter_has_tag(&test_iter, tag_underline))) {
		*underline = 1;
	}
	if ((tag_strikethrough != NULL)
	    && (gtk_text_iter_has_tag(&test_iter, tag_strikethrough))) {
		*strikethrough = 1;
	}
}

static void simple_compare(setting_flag * flag, gint * s_value, gint value)
{
	if (*flag == UNSET) {
		*s_value = value;
		*flag = SET;
	} else if (*flag == SET && *s_value != value)
		*flag = INCONSISTENT;
}

static void
check_attributes(EditorInfo * editor, settings * settings,
		 GtkTextIter * iter, gboolean text_selected)
{
	const gchar *family = NULL;
	gint size, weight, style, underline, rise;
	gboolean strikethrough;
	GdkColor color;
	PangoAttrColor *pangocolor = NULL;
	PangoAttrIterator *iterator = NULL;
	PangoFontDescription *desc = NULL;
	GtkTextAttributes *gta = NULL;

	if (!text_selected) {
		iterator = pango_attr_list_get_iterator(editor->attr_list);
		desc = pango_font_description_new();
		if (desc == NULL) {
			ULOG_ERR("Memory allocation error\n");
			return;
		}
		pango_attr_iterator_get_font(iterator, desc, NULL, NULL);
		family = pango_font_description_get_family(desc);
		size = (gint) (pango_font_description_get_size(desc) / (PANGO_SCALE *
									(font_scaling_factor)));
		pangocolor = ((PangoAttrColor *)
			      pango_attr_iterator_get(iterator, PANGO_ATTR_FOREGROUND));
		color.red = pangocolor->color.red;
		color.blue = pangocolor->color.blue;
		color.green = pangocolor->color.green;
		weight = pango_font_description_get_weight(desc);
		style = pango_font_description_get_style(desc);
		underline = ((PangoAttrInt *)
			     pango_attr_iterator_get(iterator, PANGO_ATTR_UNDERLINE))->value;
		strikethrough = ((PangoAttrInt *)
				 pango_attr_iterator_get(iterator,
							 PANGO_ATTR_STRIKETHROUGH))->value;
		/*reset underline/strikethrough values if cursor is present after a bullet/number */
		update_linetag_properties(editor, iter, &underline, &strikethrough);

		rise = ((PangoAttrInt *)
			pango_attr_iterator_get(iterator, PANGO_ATTR_RISE))->value;
	} else {
		gta = gtk_text_view_get_default_attributes(GTK_TEXT_VIEW(editor->msgFields->body));
		gtk_text_iter_get_attributes(iter, gta);
		family = pango_font_description_get_family(gta->font);
		size = (gint) (pango_font_description_get_size(gta->font) / (PANGO_SCALE *
									     (font_scaling_factor)));
		color = gta->appearance.fg_color;
		weight = pango_font_description_get_weight(gta->font);
		style = pango_font_description_get_style(gta->font);
		underline = (gint) gta->appearance.underline;
		strikethrough = gta->appearance.strikethrough;
		rise = gta->appearance.rise;
	}
	if (settings->family_flag == UNSET) {
		settings->family = strdup(family);
		settings->family_flag = SET;
	} else if (settings->family_flag == SET && settings->family != NULL
		   && strcmp(family, settings->family) != 0)
		settings->family_flag = INCONSISTENT;

	simple_compare(&(settings->size_flag), &(settings->size), size);
	if (settings->color_flag == UNSET) {
		settings->color = color;
		settings->color_flag = SET;
	} else if (settings->color_flag == SET && !gdk_color_equal(&(settings->color), &color))
		settings->color_flag = INCONSISTENT;

	simple_compare(&(settings->weight_flag), &(settings->weight), weight);

	simple_compare(&(settings->style_flag), &(settings->style), style);

	simple_compare(&(settings->underline_flag), &(settings->underline), underline);

	simple_compare(&(settings->strikethrough_flag),
		       (gint *) & (settings->strikethrough), (gint) strikethrough);

	simple_compare(&(settings->rise_flag), &(settings->rise), rise);

	if (desc != NULL) {
		pango_font_description_free(desc);
	}
	if (iterator != NULL) {
		pango_attr_iterator_destroy(iterator);
	}
	if (gta != NULL) {
		gtk_text_attributes_unref(gta);
	}
}

static void s_init(settings * settings)
{
	settings->family = "NULL";
	settings->size = 12;
	settings->color.blue = settings->color.green = settings->color.red = 0;
	settings->weight = PANGO_WEIGHT_NORMAL;
	settings->style = PANGO_STYLE_NORMAL;
	settings->underline = PANGO_UNDERLINE_NONE;
	settings->strikethrough = FALSE;
	settings->rise = 0;

	settings->family_flag = settings->size_flag = settings->color_flag =
	    settings->weight_flag = settings->style_flag =
	    settings->underline_flag = settings->strikethrough_flag = settings->rise_flag = UNSET;
}

static void s_set(settings * settings, GObject * object)
{
	g_object_set(object, "family", settings->family, "family-set",
		     settings->family_flag == SET, "size", settings->size,
		     "size-set", settings->size_flag == SET, "color",
		     &(settings->color), "color-set",
		     settings->color_flag == SET, "bold",
		     settings->weight != PANGO_WEIGHT_NORMAL, "bold-set",
		     settings->weight_flag == SET, "italic",
		     settings->style != PANGO_STYLE_NORMAL, "italic-set",
		     settings->style_flag == SET, "underline",
		     settings->underline != PANGO_UNDERLINE_NONE,
		     "underline-set", settings->underline_flag == SET,
		     "strikethrough", settings->strikethrough,
		     "strikethrough-set", settings->strikethrough_flag == SET,
		     "position",
		     settings->rise ==
		     0 ? 0 : settings->rise / abs(settings->rise),
		     "position-set", settings->rise_flag == SET, NULL);
}


static void set_state(GtkWidget * fsd, EditorInfo * editorinfo)
{
	GtkTextIter begin, end, iter;
	gboolean text_selected = FALSE;
	GtkTextBuffer *buffer =
	    gtk_text_view_get_buffer(GTK_TEXT_VIEW(editorinfo->msgFields->body));
	settings settings;

	text_selected = gtk_text_buffer_get_selection_bounds(buffer, &begin, &end);

	iter = begin;
	s_init(&settings);

	if (gtk_text_iter_compare(&iter, &end) == 0) {
		check_attributes(editorinfo, &settings, &iter, text_selected);
	}
	while (gtk_text_iter_compare(&iter, &end) < 0) {
		check_attributes(editorinfo, &settings, &iter, text_selected);
		if (!gtk_text_iter_forward_cursor_position(&iter))
			break;
	}
	s_set(&settings, G_OBJECT(fsd));

	g_free(settings.family);
}

/**
This funtion blocks signal handlers for toolbutton changes.
@param editor Pointer to EditorInfo structure.
@return void. 
*/
static void block_signals_from_toolbuttons(EditorInfo * editor)
{
	g_signal_handlers_block_by_func(G_OBJECT(editor->toolbar->fontname),
					G_CALLBACK(on_format_button_click), editor);
	g_signal_handlers_block_by_func(G_OBJECT(editor->toolbar->fontsize),
					G_CALLBACK(on_format_button_click), editor);
	g_signal_handlers_block_by_func(G_OBJECT(editor->toolbar->bold),
					G_CALLBACK(on_format_button_click), editor);
	g_signal_handlers_block_by_func(G_OBJECT(editor->toolbar->italic),
					G_CALLBACK(on_format_button_click), editor);

}

/**
This funtion unblocks signal handlers for toolbutton changes.
@param editor Pointer to EditorInfo structure.
@return void. 
*/
static void unblock_signals_from_toolbuttons(EditorInfo * editor)
{
	g_signal_handlers_unblock_by_func(G_OBJECT(editor->toolbar->fontname),
					  G_CALLBACK(on_format_button_click), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(editor->toolbar->fontsize),
					  G_CALLBACK(on_format_button_click), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(editor->toolbar->bold),
					  G_CALLBACK(on_format_button_click), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(editor->toolbar->italic),
					  G_CALLBACK(on_format_button_click), editor);

}

/**
This funtion creates a font tag and apply to the iteraters specified.

@param editor Pointer to EditorInfo structure.
@param start_iter selection start iterater.
@param end_iter selection end iterater.
@param tagid value from supported TAGID enum.

@return void. 
*/
void
apply_font_tag(EditorInfo * editor, GtkTextIter * start_iter, GtkTextIter * end_iter, gint tagid)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextIter start, end;
	GtkWidget *fontDialog = NULL;
	PangoAttrIterator *iterator = NULL;
	gint response = 0;
	GtkTextTag *tag = NULL;
	UndoOpInfo *op = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((editor == NULL) || (start_iter == NULL) || (end_iter == NULL)
	    || (tagid < 0) || (tagid >= TAG_COUNT)) {
		ULOG_ERR("Invalid input parameter\n");
		return;
	}
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editor->msgFields->body));
	/*attributes list at insertion mark is updated to properly populate the fields of the Font Dialog. */
	refresh_current_attributes(GTK_TEXT_VIEW(editor->msgFields->body), editor);
	op = update_undoinfo_on_format_change(editor, NULL);
	block_signals_from_toolbuttons(editor);

	fontDialog = hildon_font_selection_dialog_new(GTK_WINDOW(app), "Font");
	gtk_window_set_modal(GTK_WINDOW(fontDialog), TRUE);
	if (ossohelp_exists(HELP_ID_FONTEDITOR) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) fontDialog);
		g_signal_connect(G_OBJECT(fontDialog), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_FONTEDITOR);
	}
	set_state(fontDialog, editor);
	gtk_text_buffer_select_range(buffer, start_iter, end_iter);
	response = gtk_dialog_run(GTK_DIALOG(fontDialog));
	gtk_text_buffer_select_range(buffer, start_iter, end_iter);

	if (response == GTK_RESPONSE_OK) {
		if (gtk_text_buffer_get_selection_bounds(buffer, &start, &end)
		    == FALSE) {
			if (editor->events_blocked == FALSE) {
				/*signal handlers to change the attribute list upon tapping or cursor movement are blocked for the first instance after pressing OK in the dialog since this is the case of formatting preselection. */
				g_signal_handlers_block_by_func(G_OBJECT(editor->msgFields->body),
								G_CALLBACK
								(on_body_button_release_event),
								editor);
				g_signal_handlers_block_by_func(G_OBJECT(editor->msgFields->body),
								G_CALLBACK
								(on_body_key_release_event),
								editor);
				editor->events_blocked = TRUE;
			}
			editor->formating_preselected = TRUE;
		} else {
			editor->formating_preselected = FALSE;
		}
		tag =
		    hildon_font_selection_dialog_get_text_tag
		    (HILDON_FONT_SELECTION_DIALOG(fontDialog));
		apply_attributes_to_font(tag->values, editor);
		iterator = pango_attr_list_get_iterator(editor->attr_list);
		if (iterator != NULL) {
			format_font_iterator(editor, iterator, tag->values, start, end);
			pango_attr_iterator_destroy(iterator);
		}
	}
	gtk_widget_destroy(fontDialog);
	unblock_signals_from_toolbuttons(editor);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This is called to apply foreground color to the selection.

@param color GdkColor value to be applied to the selection.
@param editor Pointer to EditorInfo structure.

@return void. 
*/
void
format_font_color(GdkColor * color, EditorInfo * editor,
		  GtkTextIter start_iter, GtkTextIter end_iter)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextTag *tag = NULL, *curtag = NULL;
	GtkTextIter test_iter;
	GtkTextAttributes *attribute = NULL;
	GdkColor fgcolor;
	GSList *taglist = NULL, *templist = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((color == NULL) || editor == NULL) {
		ULOG_ERR("Input param check failed!\n");
		return;
	}
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editor->msgFields->body));
	tag = gtk_text_buffer_create_tag(buffer, NULL, "foreground_gdk", color, NULL);
	test_iter = start_iter;
	while (gtk_text_iter_get_offset(&test_iter) < gtk_text_iter_get_offset(&end_iter)) {
		taglist = gtk_text_iter_get_tags(&test_iter);
		for (templist = taglist; templist != NULL; templist = g_slist_next(templist)) {
			curtag = templist->data;
			attribute = curtag->values;
			fgcolor = attribute->appearance.fg_color;
			if (fgcolor.red != 0 || fgcolor.green != 0 || fgcolor.blue != 0) {
				gtk_text_buffer_remove_tag(buffer, curtag, &test_iter, &end_iter);
				break;
			}
		}
		g_slist_free(taglist);
		gtk_text_iter_forward_char(&test_iter);
	}
	gtk_text_buffer_apply_tag(buffer, tag, &start_iter, &end_iter);
	raise_bold_italic_priority(buffer);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This funtion creates a color tag and apply to the iteratios specified.

@param editor Pointer to EditorInfo structure.
@param start_iter selection start iterator.
@param end_iter selection end iterator.
@param tagid value from supported TAGID enum.

@return void. 
*/
GdkColor *get_selected_color(EditorInfo * editor, GtkTextIter * start_iter,
			     GtkTextIter * end_iter, gint tagid)
{
	GtkWidget *colorDialog = NULL;
	GdkColor *color = NULL;
	gint response = 0;
	GtkTextBuffer *buffer = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((editor == NULL) || (start_iter == NULL) || (end_iter == NULL)
	    || (tagid < 0) || (tagid >= TAG_COUNT)) {
		ULOG_ERR("Invalid input parameter\n");
		return NULL;
	}
	g_signal_handlers_block_by_func(G_OBJECT(app), G_CALLBACK(ui_topmost_status_lost), app);
	g_signal_handlers_block_by_func(G_OBJECT(app), G_CALLBACK(ui_topmost_status_acquired), app);
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editor->msgFields->body));

	colorDialog = hildon_color_selector_new(GTK_WINDOW(app));
	if (colorDialog == NULL) {
		osso_log(LOG_WARNING, "Error while creating widget!\n");
		return NULL;
	}
	gtk_window_set_modal(GTK_WINDOW(colorDialog), TRUE);
	gtk_widget_show(colorDialog);

	response = gtk_dialog_run(GTK_DIALOG(colorDialog));
	gtk_text_buffer_select_range(buffer, start_iter, end_iter);
	if (response == GTK_RESPONSE_OK) {
		color = gdk_color_copy(hildon_color_selector_get_color
				       ((HildonColorSelector *) colorDialog));
	}
	gtk_widget_destroy((GtkWidget *) colorDialog);
	g_signal_handlers_unblock_by_func(G_OBJECT(app), G_CALLBACK(ui_topmost_status_lost), app);
	g_signal_handlers_unblock_by_func(G_OBJECT(app),
					  G_CALLBACK(ui_topmost_status_acquired), app);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return color;
}


/**
This function will set the iterators for applying the tag. For example 
the left_align should be applied to a line where the cursor is present.

@param textview Is a pointer to the Textarea in html editor.
@param editor Pointer to EditorInfo structure.

@return void. 
*/
void
set_iterators_for_tag(GtkTextBuffer * buffer, GtkTextIter * start_iter,
		      GtkTextIter * end_iter, gint tagid, gboolean apply_tag)
{

	gint linenumber = 0;
	GtkTextTag *tag = NULL;
	GtkTextIter iter;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (start_iter == NULL) || (end_iter == NULL)
	    || (tagid < 0) || (tagid >= TAG_COUNT)) {
		ULOG_ERR("Invalid input parameter\n");
		return;
	}
	switch (tagid) {
	case TAG_BOLD:
	case TAG_ITALIC:
	case TAG_UNDERLINE:
	case TAG_STRIKE_THROUGH:
	case TAG_SUPERSCRIPT:
	case TAG_SUBSCRIPT:
	case TAG_FONT:
	case TAG_FGCOLOR:
	case TAG_BGCOLOR:
		break;
	case TAG_LEFT_ALIGN:
	case TAG_CENTER_ALIGN:
	case TAG_RIGHT_ALIGN:
		linenumber = gtk_text_iter_get_line(start_iter);
		gtk_text_buffer_get_iter_at_line(buffer, start_iter, linenumber);
		if (!gtk_text_iter_forward_line(end_iter)) {
			gtk_text_iter_forward_sentence_end(end_iter);
		}
		break;
	case TAG_BULLET:
	case TAG_NUMBER:
		linenumber = gtk_text_iter_get_line(start_iter);
		gtk_text_buffer_get_iter_at_line(buffer, start_iter, linenumber);
		if (!gtk_text_iter_ends_line(end_iter)) {
			gtk_text_iter_forward_to_line_end(end_iter);
			// gtk_text_iter_forward_sentence_end (end_iter);
		}
		if (apply_tag == TRUE) {
			if (!gtk_text_iter_is_end(end_iter)) {
				iter = *end_iter;
				gtk_text_iter_forward_char(&iter);
				tag = iter_has_bullet_or_number_tag(&iter);
				if ((tag != NULL) &&
				    (g_ascii_strcasecmp(tag->name, tagnames[tagid]) == 0)) {
					if (!gtk_text_iter_ends_tag(&iter, tag)) {
						gtk_text_iter_forward_to_tag_toggle(&iter, tag);
						*end_iter = iter;
					}
				}
			}
			if (linenumber > 0) {
				iter = *start_iter;
				gtk_text_iter_backward_line(&iter);
				tag = iter_has_bullet_or_number_tag(&iter);
				if ((tag != NULL) &&
				    (g_ascii_strcasecmp(tag->name, tagnames[tagid]) == 0)) {
					*start_iter = iter;
					if (!gtk_text_iter_begins_tag(start_iter, tag)) {
						gtk_text_iter_backward_to_tag_toggle(start_iter,
										     tag);
					}
				}
			}
		} else {
			gtk_text_iter_backward_char(start_iter);
			gtk_text_iter_forward_char(end_iter);
		}
		break;
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Apply tag corresponding to tagid to the text selection.

@param buffer Is a pointer to buffer of the textarea in html editor.
@param editor Pointer to EditorInfo structure.
@param tagid value from supported TAGID enum.

@return void. 
*/
void apply_gtk_tag_for_current_selection(GtkTextBuffer * buffer, EditorInfo * editor, gint tagid)
{
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	GdkColor *color = NULL;
	PangoAttribute *attribute = NULL;
	gboolean apply_tag = TRUE;
	gboolean blocked = FALSE;


	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	ULOG_DEBUG_L("Apply Tag : %d\n", tagid);

	if ((buffer == NULL) || (editor == NULL) || (tagid < 0)
	    || (tagid >= TAG_COUNT)) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	if (!gtk_text_buffer_get_selection_bounds(buffer, &start_iter, &end_iter)) {
		g_signal_handlers_block_by_func(G_OBJECT(buffer),
						G_CALLBACK(updatemodified),
						&(editor->editorModified));
		blocked = TRUE;
	}
	gtk_text_buffer_get_selection_bounds(buffer, &start_iter, &end_iter);
	set_iterators_for_tag(buffer, &start_iter, &end_iter, tagid, apply_tag);
	if (tagid == TAG_FONT) {
		apply_font_tag(editor, &start_iter, &end_iter, tagid);
	} else if (tagid == TAG_FGCOLOR) {
		color = gdk_color_copy(hildon_color_button_get_color(editor->toolbar->color_btn));
		if (color != NULL) {
			format_font_color(color, editor, start_iter, end_iter);
			attribute =
			    pango_attr_foreground_new(color->red, color->green, color->blue);
			attribute->start_index = 0;
			attribute->end_index = -1;
			if (editor->attr_list == NULL) {
				pango_attr_list_insert(editor->attr_list, attribute);
			} else {
				pango_attr_list_change(editor->attr_list, attribute);
			}
			gdk_color_free(color);
		}
	} else if (tagid == TAG_BGCOLOR) {
		color = get_selected_color(editor, &start_iter, &end_iter, tagid);
		if (color != NULL) {
			/*TODO:color needs to be copied and free color variable */
			editor->menu->bgcolorvalue = *(GdkColor *) color;
			/*data updation for undo */
			editor->prevcolor->pixel = editor->bgcolor->pixel;
			editor->prevcolor->red = editor->bgcolor->red;
			editor->prevcolor->green = editor->bgcolor->green;
			editor->prevcolor->blue = editor->bgcolor->blue;
			editor->bgcolor->pixel = color->pixel;
			editor->bgcolor->red = color->red;
			editor->bgcolor->green = color->green;
			editor->bgcolor->blue = color->blue;

			update_undoinfo_on_bg_change(editor, start_iter, end_iter);
			gtk_widget_modify_base((GtkWidget *) editor->msgFields->body,
					       GTK_STATE_NORMAL, &(editor->menu->bgcolorvalue));
			editor->editorModified = TRUE;
			gdk_color_free(color);
		}

	} else {
		apply_gtk_tag(buffer, &start_iter, &end_iter, tagid);
	}

	if (blocked) {
		g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
						  G_CALLBACK(updatemodified),
						  &(editor->editorModified));
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Remove tag corresponding to tagid to the text selection.

@param buffer Is a pointer to buffer of the textarea in html editor.
@param editor Pointer to EditorInfo structure.
@param tagid value from supported TAGID enum.

@return void. 
*/
void remove_gtk_tag_for_current_selection(GtkTextBuffer * buffer, EditorInfo * editor, gint tagid)
{
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	gboolean apply_tag = FALSE;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	ULOG_DEBUG_L("Remove Tag : %d\n", tagid);
	if ((buffer == NULL) || (tagid < 0) || (tagid >= TAG_COUNT)) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	gtk_text_buffer_get_selection_bounds(buffer, &start_iter, &end_iter);
	set_iterators_for_tag(buffer, &start_iter, &end_iter, tagid, apply_tag);
	/*if (tagid == TAG_BULLET || tagid == TAG_NUMBER)
	   {
	   gtk_text_iter_backward_char (&start_iter);
	   gtk_text_iter_forward_char (&end_iter);
	   } */
	remove_gtk_tag(buffer, &start_iter, &end_iter, tagid);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Apply or remove tag corresponding to tagid to the text selection.
Toggle is done depending upon the toggle button state on/off.

@param buffer Is a pointer to buffer of the textarea in html editor.
@param editor Pointer to EditorInfo structure.
@param tagid value from supported TAGID enum.

@return void. 
*/
void add_remove_gtk_tags(GtkToggleToolButton * button, EditorInfo * editor, gint tagid)
{
	GtkTextBuffer *buffer = NULL;
	PangoAttribute *attribute = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((editor == NULL) || (tagid < 0) || (tagid >= TAG_COUNT)) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editor->msgFields->body));

	if (gtk_toggle_tool_button_get_active(button)) {
		ULOG_DEBUG_L("apply tag : %d\n", tagid);
		apply_gtk_tag_for_current_selection(buffer, editor, tagid);

		switch (tagid) {
		case TAG_BOLD:
			attribute = pango_attr_weight_new(PANGO_WEIGHT_BOLD);
			attribute->start_index = 0;
			attribute->end_index = -1;
			pango_attr_list_change(editor->attr_list, attribute);
			break;
		case TAG_ITALIC:
			attribute = pango_attr_style_new(PANGO_STYLE_ITALIC);
			attribute->start_index = 0;
			attribute->end_index = -1;
			pango_attr_list_change(editor->attr_list, attribute);
			break;
		case TAG_BULLET:
			bullet_num_change_status(editor->menu->bullets, editor, TRUE);
			break;
		}
	} else {
		ULOG_DEBUG_L("remove tag : %d\n", tagid);
		switch (tagid) {
		case TAG_BOLD:
			attribute = pango_attr_weight_new(PANGO_WEIGHT_NORMAL);
			attribute->start_index = 0;
			attribute->end_index = -1;
			pango_attr_list_change(editor->attr_list, attribute);
			break;
		case TAG_ITALIC:
			attribute = pango_attr_style_new(PANGO_STYLE_NORMAL);
			attribute->start_index = 0;
			attribute->end_index = -1;
			pango_attr_list_change(editor->attr_list, attribute);
			break;
		case TAG_BULLET:
			bullet_num_change_status(editor->menu->bullets, editor, FALSE);
			break;
		}
		remove_gtk_tag_for_current_selection(buffer, editor, tagid);
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Refreshes menu items upon click of select-all 
@param widget is select all menu item
@param editor Pointer to EditorInfo structure.

@return void. 
*/
void call_attribute_refresh_func(GtkWidget * widget, EditorInfo * editor)
{
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (editor == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}

	refresh_current_attributes(GTK_TEXT_VIEW(editor->msgFields->body), editor);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Refresh the current text attributes to the toolbar and menu.

@param widget Is a pointer to the textarea in html editor.
@param event event key.
@param editor EditorInfo representing html editor .

@return FALSE If other signals handlers to be executed. 
*/
gboolean
on_body_button_release_event(GtkWidget * widget, GdkEventButton * event, EditorInfo * editor)
{
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (editor == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return FALSE;
	}

	refresh_current_attributes(GTK_TEXT_VIEW(editor->msgFields->body), editor);
	move_cursor_to_listitem_end(widget);
	editor->formating_preselected = TRUE;

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return FALSE;
}

/**
Refresh the current text attributes to the toolbar and menu.

@param widget Is a pointer to the textarea in html editor.
@param event event key.
@param editor EditorInfo representing html editor .

@return FALSE If other signals handlers to be executed. 
*/
gboolean on_body_key_release_event(GtkWidget * widget, GdkEventKey * event, EditorInfo * editor)
{
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (editor == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return FALSE;
	}
	if (event->keyval != GDK_Right
	    && event->keyval != GDK_Left && event->keyval != GDK_Up && event->keyval != GDK_Down) {
		return FALSE;
	}

	refresh_current_attributes(GTK_TEXT_VIEW(editor->msgFields->body), editor);
	move_cursor_to_listitem_end(widget);
	editor->formating_preselected = TRUE;

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return FALSE;
}

/**
Applies underline/strikethrough tag on a char. It checks for previous line end char for the same tag if the inserted char is only char in a bullet/numbered line. It checks for second char in the line if the line has more chars except the latest inserted one.
@param buffer is msg body GtkTextBuffer.
@param end_iter is end position of latest inserted char on which underline/strikethrough tag has to be applied.
@param editor Pointer to EditorInfo structure having editor related info.

@return void. 
*/
static void check_apply_line_tags(GtkTextBuffer * buffer, GtkTextIter end_iter, EditorInfo * editor)
{
	GtkTextTagTable *tagtable = NULL;
	GtkTextTag *tag_underline = NULL;
	GtkTextTag *tag_strikethrough = NULL;
	GtkTextIter start_iter;
	GtkTextIter iter;

	if ((buffer == NULL) || (editor) == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	tagtable = gtk_text_buffer_get_tag_table(buffer);
	tag_underline = gtk_text_tag_table_lookup(tagtable, tagnames[TAG_UNDERLINE]);
	tag_strikethrough = gtk_text_tag_table_lookup(tagtable, tagnames[TAG_STRIKE_THROUGH]);
	iter = end_iter;

	/*iter to consider for presence of line tag depends upon whether 
	   the target char is the only char in a listed line */
	if (gtk_text_iter_ends_line(&end_iter)) {
		if (!gtk_text_iter_backward_line(&iter)) {
			return;
		}
		if (!gtk_text_iter_ends_line(&iter)) {
			gtk_text_iter_forward_to_line_end(&iter);
		}
		if (!gtk_text_iter_starts_line(&iter)) {
			gtk_text_iter_backward_char(&iter);
		}
	}

	start_iter = end_iter;
	gtk_text_iter_backward_char(&start_iter);
	if ((tag_underline != NULL)
	    && (gtk_text_iter_has_tag(&iter, tag_underline))
	    && (!editor->ignore_underline)) {
		gtk_text_buffer_apply_tag(buffer, tag_underline, &start_iter, &end_iter);
	}
	if ((tag_strikethrough != NULL)
	    && (gtk_text_iter_has_tag(&iter, tag_strikethrough))
	    && (!editor->ignore_strikethrough)) {
		gtk_text_buffer_apply_tag(buffer, tag_strikethrough, &start_iter, &end_iter);
	}
}

/**
This is called on text insertion on text area of html editor.
@param buffer is msg body GtkTextBuffer.
@param text_iter is insertion pos of text.
@param length is length of text inserted.
@param editor Pointer to EditorInfo structure.

@return void. 
*/
void
on_body_after_text_insert(GtkTextBuffer * buffer, GtkTextIter * text_iter,
			  gchar * text, gint length, EditorInfo * editor)
{
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	GtkTextIter char_start;
	GtkTextIter char_end;
	GtkTextTag *list_tag = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (editor == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	if (editor->emailType != HTML_EDITOR) {
		return;
	}
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_apply_tag), editor);
	populate_character_input(editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_apply_tag), editor);
	gtk_text_buffer_get_iter_at_mark(buffer, &start_iter, gtk_text_buffer_get_insert(buffer));
	end_iter = start_iter;
	gtk_text_iter_set_line_offset(&start_iter, 0);
	list_tag = iter_has_bullet_or_number_tag(&start_iter);
	if (list_tag != NULL) {
		if (get_listchar_bounds(start_iter, &char_start, &char_end)) {
			gtk_text_iter_forward_char(&char_end);
			/*checks whether the latest char inserted is just after a bullet/number char. 
			   If yes, looks for applying underline/strikethrough tags on the char. */
			if (gtk_text_iter_compare(&char_end, &end_iter) == 0) {
				check_apply_line_tags(buffer, end_iter, editor);
				editor->ignore_underline = FALSE;
				editor->ignore_strikethrough = FALSE;
			}
		}
		/*look for removal of underline/strikethrough on bullet/number char */
		gtk_text_iter_forward_char(&end_iter);
		check_tags_on_listchars(buffer, &start_iter, &end_iter);
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is called on text deletion on text area of html editor to initialize 
the text attribute list.
@param buffer is msg body GtkTextBuffer.
@param start_iter Text iterator representing the starting position
@param end_iter Text iterator representing the ending position
@param editor Pointer to EditorInfo structure.
@return void. 
*/
void
on_body_after_text_delete(GtkTextBuffer * buffer,
			  GtkTextIter * start_iter, GtkTextIter * end_iter, EditorInfo * editor)
{
	GtkTextIter end;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (editor == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	if (editor->emailType != HTML_EDITOR) {
		return;
	}
	editor->formating_preselected = TRUE;
	gtk_text_buffer_get_end_iter(buffer, &end);
	if (gtk_text_iter_get_offset(&end) != 0) {
		/*if the textview has some characters */
		refresh_current_attributes(GTK_TEXT_VIEW(editor->msgFields->body), editor);
	} else if (editor->events_blocked == FALSE) {
		/*if the textview doesn't have any character we will not 
		   change the attribute list by any way. */
		g_signal_handlers_block_by_func(G_OBJECT(editor->msgFields->body),
						G_CALLBACK(on_body_button_release_event), editor);
		g_signal_handlers_block_by_func(G_OBJECT(editor->msgFields->body),
						G_CALLBACK(on_body_key_release_event), editor);
		editor->events_blocked = TRUE;
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This is called on text insertion in text area of html editor.
@param buffer is msg body area.
@param text_iter is text insertion pos.
@param text is text inserted.
@param length is length of text.
@param editor Pointer to EditorInfo structure.
@return void. 
*/
void
on_body_before_text_insert(GtkTextBuffer * buffer,
			   GtkTextIter * text_iter, gchar * text,
			   gint length, EditorInfo * editorinfo)
{

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (editorinfo == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	if (editorinfo->emailType != HTML_EDITOR) {
		return;
	}
	if (editorinfo->insert_mark == NULL) {
		editorinfo->insert_mark =
		    gtk_text_buffer_create_mark(buffer, NULL, text_iter, TRUE);
	}
	if (key_release == TRUE) {
		gtk_text_buffer_move_mark(buffer, editorinfo->insert_mark, text_iter);
		key_release = FALSE;
	}
	if (editorinfo->events_blocked == TRUE) {
		g_signal_handlers_unblock_by_func(G_OBJECT(editorinfo->msgFields->body),
						  G_CALLBACK(on_body_button_release_event),
						  editorinfo);
		g_signal_handlers_unblock_by_func(G_OBJECT(editorinfo->msgFields->body),
						  G_CALLBACK(on_body_key_release_event),
						  editorinfo);
		editorinfo->events_blocked = FALSE;
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 This function will raise the priority of bold and italic tags. 
 This is made for fix that the bold and italic properties are not reflected over font tag.

 @param buffer GtkTextBuffer representing the buffer of textarea.
 */
void raise_bold_italic_priority(GtkTextBuffer * buffer)
{

	GtkTextTagTable *table = NULL;
	GtkTextTag *boldtag = NULL;
	GtkTextTag *italictag = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (buffer == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}

	table = gtk_text_buffer_get_tag_table(buffer);

	if (table != NULL) {
		boldtag = g_object_get_data(G_OBJECT(buffer), tagnames[TAG_BOLD]);
		if (boldtag != NULL) {
			gtk_text_tag_set_priority(boldtag, gtk_text_tag_table_get_size(table) - 1);
		}
		italictag = g_object_get_data(G_OBJECT(buffer), tagnames[TAG_ITALIC]);
		if (italictag != NULL) {
			gtk_text_tag_set_priority(italictag,
						  gtk_text_tag_table_get_size(table) - 1);
		}
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This function is invoked to change the active status of toolbutton.

@param button widget for which status has to changed. 
@param editor Pointer to EditorInfo structure.
@param active_state is boolean to specify the required state of button

@return void. 
*/
static void bullet_num_change_status(GtkWidget * button, EditorInfo * editor, gboolean active_state)
{
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if ((editor == NULL) || (button == NULL)) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	g_signal_handlers_block_by_func(G_OBJECT(button),
					G_CALLBACK(on_format_button_click), editor);
	if (GTK_IS_TOOL_BUTTON(button)) {
		gtk_toggle_tool_button_set_active((GtkToggleToolButton *) button, active_state);
	} else {
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) button, active_state);
	}

	g_signal_handlers_unblock_by_func(G_OBJECT(button),
					  G_CALLBACK(on_format_button_click), editor);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a common function invoked on each of text property change from the user.

@param formatbutton source widget from which the event is generated. 
Depending upon the source, the corresponding operation is invoked.
@param editor Pointer to EditorInfo structure.

@return void. 
*/
void on_format_button_click(GtkWidget * formatbutton, EditorInfo * editor)
{
	gchar *fontname = NULL;
	gint fontsize = 0;
	GtkTextBuffer *buffer = NULL;
	PangoAttribute *nameattribute = NULL;
	PangoAttribute *sizeattribute = NULL;
	GtkTextIter start, end;
	UndoOpInfo *op = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (stop_format == TRUE) {
		return;
	}

	if ((editor == NULL) || (formatbutton == NULL)) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}


	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editor->msgFields->body));
	gtk_text_buffer_get_selection_bounds(buffer, &start, &end);
	if ((formatbutton == editor->menu->left)
	    || (formatbutton == editor->menu->centred)
	    || (formatbutton == editor->menu->right)) {
		if (gtk_check_menu_item_get_active((GtkCheckMenuItem *) formatbutton) == FALSE) {
			return;
		}
	}

	op = update_undoinfo_on_format_change(editor, formatbutton);

	/*signals blocked for undo handling of bullets & numbering */
	g_signal_handlers_block_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer), G_CALLBACK(undodel), editor);
	/*signal blocked which has its affect while doing a transition from 
	   bullet/numbering to numbering/bullet and there is no other text 
	   except the list item (the editor becomes empty for a while). */
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_after_text_delete), editor);

	if ((formatbutton != editor->menu->bullets)
	    && (formatbutton != editor->menu->numbering)
	    && (formatbutton != editor->toolbar->bullets)
	    && (formatbutton != editor->menu->font)) {
		if (gtk_text_buffer_get_selection_bounds(buffer, &start, &end)
		    == FALSE) {
			editor->formating_preselected = TRUE;
			if (editor->events_blocked == FALSE) {
				g_signal_handlers_block_by_func(G_OBJECT(editor->msgFields->body),
								G_CALLBACK
								(on_body_button_release_event),
								editor);
				g_signal_handlers_block_by_func(G_OBJECT(editor->msgFields->body),
								G_CALLBACK
								(on_body_key_release_event),
								editor);
				editor->events_blocked = TRUE;
			}
		} else {
			editor->formating_preselected = FALSE;
		}
	}
	if (formatbutton == (GtkWidget *) editor->toolbar->bold) {
		add_remove_gtk_tags((GtkToggleToolButton *) formatbutton, editor, TAG_BOLD);
		raise_bold_italic_priority(buffer);
	} else if (formatbutton == (GtkWidget *) editor->toolbar->italic) {
		add_remove_gtk_tags((GtkToggleToolButton *) formatbutton, editor, TAG_ITALIC);
		raise_bold_italic_priority(buffer);
	} else if (formatbutton == (GtkWidget *) editor->toolbar->color_btn) {
		apply_gtk_tag_for_current_selection(buffer, editor, TAG_FGCOLOR);
	} else if (formatbutton == editor->menu->left) {
		remove_gtk_tag_for_current_selection(buffer, editor, TAG_RIGHT_ALIGN);
		remove_gtk_tag_for_current_selection(buffer, editor, TAG_CENTER_ALIGN);
		apply_gtk_tag_for_current_selection(buffer, editor, TAG_LEFT_ALIGN);
		editor->justification = OSSO_JUSTIFY_LEFT;
	} else if (formatbutton == editor->menu->centred) {
		remove_gtk_tag_for_current_selection(buffer, editor, TAG_RIGHT_ALIGN);
		remove_gtk_tag_for_current_selection(buffer, editor, TAG_LEFT_ALIGN);
		apply_gtk_tag_for_current_selection(buffer, editor, TAG_CENTER_ALIGN);
		editor->justification = OSSO_JUSTIFY_CENTER;
	} else if (formatbutton == editor->menu->right) {
		remove_gtk_tag_for_current_selection(buffer, editor, TAG_LEFT_ALIGN);
		remove_gtk_tag_for_current_selection(buffer, editor, TAG_CENTER_ALIGN);
		apply_gtk_tag_for_current_selection(buffer, editor, TAG_RIGHT_ALIGN);
		editor->justification = OSSO_JUSTIFY_RIGHT;
	} else if (formatbutton == editor->toolbar->bullets) {
		remove_gtk_tag_for_current_selection(buffer, editor, TAG_NUMBER);
		bullet_num_change_status(editor->menu->numbering, editor, FALSE);
		add_remove_gtk_tags((GtkToggleToolButton *) formatbutton, editor, TAG_BULLET);
		undo_set_delete_range(op, editor);

	} else if (formatbutton == editor->menu->font) {
		apply_gtk_tag_for_current_selection(buffer, editor, TAG_FONT);
	} else if (formatbutton == editor->menu->bgcolor) {
		apply_gtk_tag_for_current_selection(buffer, editor, TAG_BGCOLOR);
	} else if (formatbutton == editor->toolbar->fontname) {
		fontname = (gchar *) get_edittoolbar_fontname(editor->toolbar->fontname, editor);
		apply_font_name_and_size(GTK_TEXT_VIEW(editor->msgFields->body), fontname, 0,
					 editor->zoominfo->previous_level, start, end);

		nameattribute = pango_attr_family_new(fontname);
		nameattribute->start_index = 0;
		nameattribute->end_index = -1;
		pango_attr_list_change(editor->attr_list, nameattribute);
	} else if (formatbutton == editor->toolbar->fontsize) {
		fontsize =
		    (gint) atoi((gchar *) get_edittoolbar_fontsize(editor->toolbar->fontsize));
		apply_font_name_and_size(GTK_TEXT_VIEW(editor->msgFields->body), NULL, fontsize,
					 editor->zoominfo->previous_level, start, end);

		sizeattribute = pango_attr_size_new(fontsize * PANGO_SCALE * (font_scaling_factor));
		sizeattribute->start_index = 0;
		sizeattribute->end_index = -1;
		pango_attr_list_change(editor->attr_list, sizeattribute);
	} else if ((formatbutton == editor->menu->bullets)) {
		remove_gtk_tag_for_current_selection(buffer, editor, TAG_NUMBER);
		bullet_num_change_status(editor->menu->numbering, editor, FALSE);
		if (gtk_check_menu_item_get_active
		    ((GtkCheckMenuItem *) editor->menu->bullets) == TRUE) {
			//remove_gtk_tag_for_current_selection (buffer, editor, TAG_NUMBER);
			apply_gtk_tag_for_current_selection(buffer, editor, TAG_BULLET);
			bullet_num_change_status(editor->toolbar->bullets, editor, TRUE);
		} else {
			remove_gtk_tag_for_current_selection(buffer, editor, TAG_BULLET);
			bullet_num_change_status(editor->toolbar->bullets, editor, FALSE);
		}
		undo_set_delete_range(op, editor);
	} else if ((formatbutton == editor->menu->numbering)) {
		remove_gtk_tag_for_current_selection(buffer, editor, TAG_BULLET);
		bullet_num_change_status(editor->toolbar->bullets, editor, FALSE);
		if (gtk_check_menu_item_get_active
		    ((GtkCheckMenuItem *) editor->menu->numbering) == TRUE) {
			apply_gtk_tag_for_current_selection(buffer, editor, TAG_NUMBER);
		} else {
			remove_gtk_tag_for_current_selection(buffer, editor, TAG_NUMBER);
		}
		undo_set_delete_range(op, editor);

		if (gtk_check_menu_item_get_active((GtkCheckMenuItem *) editor->menu->bullets))
			bullet_num_change_status(editor->menu->bullets, editor, FALSE);
	}
	/*signals unblocked for undo handling */
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(undodel), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_after_text_delete), editor);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is invoked upon the font set operation from user.

@param font_type source widget from which the event is generated. 
@param editor Pointer to EditorInfo structure.

@return void. 
*/
void format_font_dialog(GtkWidget * font_type, EditorInfo * editor)
{

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((editor == NULL) || (font_type == NULL)) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}

	add_remove_gtk_tags((GtkToggleToolButton *) font_type, editor, TAG_FONT);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This is invoked to populate the key input as per the preselected format.

@param editor Pointer to EditorInfo structure.

@return void. 
*/
void populate_character_input(EditorInfo * editor)
{

	GtkTextIter cur_iter;
	GtkTextIter start_iter;
	GSList *taglist = NULL;
	GSList *templist = NULL;
	gint start_offset = 0;
	gint current_offset = 0;

	GtkTextBuffer *buffer = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (key_release == TRUE) {
		return;
	}

	key_release = TRUE;

	if (editor == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editor->msgFields->body));
	gtk_text_buffer_get_iter_at_mark(buffer, &start_iter, editor->insert_mark);
	//gtk_text_buffer_get_iter_at_offset (buffer, &start_iter, prev_offset);
	gtk_text_iter_backward_char(&start_iter);

	gtk_text_buffer_get_iter_at_mark(buffer, &cur_iter, gtk_text_buffer_get_insert(buffer));
	start_offset = gtk_text_iter_get_offset(&start_iter);
	current_offset = gtk_text_iter_get_offset(&cur_iter);
	if (!gtk_text_iter_equal(&start_iter, &cur_iter)) {
		taglist = gtk_text_iter_get_tags(&start_iter);
		/*list is reversed so that color tag will come early in
		   position and application of color will be visible 
		   upon text insertion */
		taglist = g_slist_reverse(taglist);
		for (templist = taglist; templist != NULL; templist = g_slist_next(templist)) {
			if (!gtk_text_iter_has_tag(&cur_iter, (GtkTextTag *) templist->data)) {
				gtk_text_buffer_apply_tag(buffer,
							  (GtkTextTag *) templist->data,
							  &start_iter, &cur_iter);
				gtk_text_buffer_get_iter_at_offset(buffer,
								   &start_iter, start_offset);
				gtk_text_buffer_get_iter_at_offset(buffer,
								   &cur_iter, current_offset);
			}
		}
	}
	if (editor->formating_preselected == TRUE) {
		apply_attribute_difference(editor, &start_iter, &cur_iter);
		refresh_current_attributes(GTK_TEXT_VIEW(editor->msgFields->body), editor);
		editor->formating_preselected = FALSE;
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function will apply any new tag that is selected by the user

@param editor Pointer to EditorInfo structure.
@param start_iter starting iterator to check for existing properties
@param end_iter iterator at the ending point
*/
void
apply_attribute_difference(EditorInfo * editor, GtkTextIter * start_iter, GtkTextIter * end_iter)
{

	GtkTextAttributes *gtkattr = NULL;
	GtkTextTag *tag = NULL;
	PangoAttrIterator *iterator = NULL;
	PangoAttribute *pattr = NULL;
	PangoFontDescription *desc = NULL;
	GdkColor tempcolor;
	PangoAttrColor *color = NULL;
	GSList *attrs = NULL;
	GSList *curattr = NULL;
	GtkTextBuffer *buffer = NULL;
	gint fontrise = 0;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((editor == NULL) || (start_iter == NULL) || (end_iter == NULL)) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editor->msgFields->body));

	/*Taking existing attributes at teh iterator */
	gtkattr = gtk_text_attributes_new();
	gtk_text_iter_get_attributes(start_iter, gtkattr);
	/*forward the iterator by a character for applying tags. */
	if (gtk_text_iter_get_offset(end_iter) > 1) {
		gtk_text_iter_forward_char(start_iter);
	}

	switch (editor->justification) {
	case OSSO_JUSTIFY_LEFT:
		remove_gtk_tag(buffer, start_iter, end_iter, TAG_RIGHT_ALIGN);
		remove_gtk_tag(buffer, start_iter, end_iter, TAG_CENTER_ALIGN);
		apply_gtk_tag(buffer, start_iter, end_iter, TAG_LEFT_ALIGN);
		break;
	case OSSO_JUSTIFY_RIGHT:
		remove_gtk_tag(buffer, start_iter, end_iter, TAG_LEFT_ALIGN);
		remove_gtk_tag(buffer, start_iter, end_iter, TAG_CENTER_ALIGN);
		apply_gtk_tag(buffer, start_iter, end_iter, TAG_RIGHT_ALIGN);
		break;
	case OSSO_JUSTIFY_CENTER:
		remove_gtk_tag(buffer, start_iter, end_iter, TAG_RIGHT_ALIGN);
		remove_gtk_tag(buffer, start_iter, end_iter, TAG_LEFT_ALIGN);
		apply_gtk_tag(buffer, start_iter, end_iter, TAG_CENTER_ALIGN);
		break;
	default:
		remove_gtk_tag(buffer, start_iter, end_iter, TAG_RIGHT_ALIGN);
		remove_gtk_tag(buffer, start_iter, end_iter, TAG_LEFT_ALIGN);
		remove_gtk_tag(buffer, start_iter, end_iter, TAG_CENTER_ALIGN);
		break;
	}
	iterator = pango_attr_list_get_iterator(editor->attr_list);
	if (iterator == NULL) {
		return;
	}
	attrs = pango_attr_iterator_get_attrs(iterator);
	if (attrs == NULL) {
		ULOG_ERR("Error populating attributes\n");
		return;
	}

	for (curattr = attrs; curattr != NULL; curattr = curattr->next) {
		pattr = (PangoAttribute *) curattr->data;

		switch (pattr->klass->type) {
			/*setting font name, size, bold and Italic. */
		case PANGO_ATTR_FONT_DESC:
			desc = pango_font_description_new();
			if (desc == NULL) {
				ULOG_ERR("Memory allocation error\n");
				return;
			}
			pango_attr_iterator_get_font(iterator, desc, NULL, NULL);
			verify_font_attributes(buffer, gtkattr, desc, start_iter,
					       end_iter, editor->zoominfo->previous_level);
			break;

		case PANGO_ATTR_FOREGROUND:
			color = ((PangoAttrColor *)
				 pango_attr_iterator_get(iterator, PANGO_ATTR_FOREGROUND));
			if ((gtkattr->appearance.fg_color.red != color->color.red)
			    || (gtkattr->appearance.fg_color.blue != color->color.blue)
			    || (gtkattr->appearance.fg_color.green != color->color.green)) {
				tempcolor.red = color->color.red;
				tempcolor.blue = color->color.blue;
				tempcolor.green = color->color.green;
				tag =
				    gtk_text_buffer_create_tag(buffer, NULL, "foreground_gdk",
							       &tempcolor, NULL);
				gtk_text_buffer_apply_tag(buffer, tag, start_iter, end_iter);
			}
			break;
		case PANGO_ATTR_BACKGROUND:
			break;
		case PANGO_ATTR_FAMILY:
		case PANGO_ATTR_STYLE:
		case PANGO_ATTR_WEIGHT:
			break;
		case PANGO_ATTR_UNDERLINE:
			if (((PangoAttrInt *)
			     pango_attr_iterator_get(iterator,
						     PANGO_ATTR_UNDERLINE))->value == TRUE) {
				apply_gtk_tag(buffer, start_iter, end_iter, TAG_UNDERLINE);
			} else {
				remove_gtk_tag(buffer, start_iter, end_iter, TAG_UNDERLINE);
			}
			break;

		case PANGO_ATTR_STRIKETHROUGH:
			if (((PangoAttrInt *)
			     pango_attr_iterator_get(iterator,
						     PANGO_ATTR_STRIKETHROUGH))->value == TRUE) {
				apply_gtk_tag(buffer, start_iter, end_iter, TAG_STRIKE_THROUGH);
			} else {
				remove_gtk_tag(buffer, start_iter, end_iter, TAG_STRIKE_THROUGH);
			}
			break;
		case PANGO_ATTR_RISE:
			fontrise = ((PangoAttrInt *)
				    pango_attr_iterator_get(iterator, PANGO_ATTR_RISE))->value;
			if (fontrise > 0) {
				apply_gtk_tag(buffer, start_iter, end_iter, TAG_SUPERSCRIPT);
			} else {
				remove_gtk_tag(buffer, start_iter, end_iter, TAG_SUPERSCRIPT);
			}
			if (fontrise < 0) {
				apply_gtk_tag(buffer, start_iter, end_iter, TAG_SUBSCRIPT);
			} else {
				remove_gtk_tag(buffer, start_iter, end_iter, TAG_SUBSCRIPT);
			}
			break;
		case PANGO_ATTR_SHAPE:
		case PANGO_ATTR_SCALE:
		case PANGO_ATTR_FALLBACK:
		case PANGO_ATTR_LETTER_SPACING:
		case PANGO_ATTR_INVALID:
		case PANGO_ATTR_LANGUAGE:
		case PANGO_ATTR_VARIANT:
		case PANGO_ATTR_STRETCH:
		case PANGO_ATTR_SIZE:
		case PANGO_ATTR_UNDERLINE_COLOR:
		case PANGO_ATTR_STRIKETHROUGH_COLOR:
		case PANGO_ATTR_ABSOLUTE_SIZE:
			break;
		}
	}
	g_slist_foreach(attrs, (GFunc) pango_attribute_destroy, NULL);
	g_slist_free(attrs);
	attrs = NULL;

	raise_bold_italic_priority(buffer);

	if (desc != NULL) {
		pango_font_description_free(desc);
	}
	if (iterator != NULL) {
		pango_attr_iterator_destroy(iterator);
	}
	if (gtkattr != NULL) {
		gtk_text_attributes_unref(gtkattr);
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 This function verifies the differences in font name selected to the existing font property.
 Any differnce is applied to the text

 @param buffer text of text area
 @param gtkattr GtkTextAttributes ofthe text appearing
 @param desc PangoFontDescription representing user preselection
 @param start_iter starting iterator to apply any changes in font
 @param end_iter ending iterator to apply any changes in font
 */
void
verify_font_attributes(GtkTextBuffer * buffer,
		       GtkTextAttributes * gtkattr,
		       PangoFontDescription * desc,
		       GtkTextIter * start_iter, GtkTextIter * end_iter, gint zoom_level)
{
	FontInfo *fontinfo = NULL;
	GtkTextTag *tag = NULL;
	gint start_offset = 0;
	gint end_offset = 0;
	gint size = 0;
	const gchar *font_name = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (gtkattr == NULL) || (desc == NULL)
	    || (start_iter == NULL) || (end_iter == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	fontinfo = getfontinfo(desc);
	if (fontinfo != NULL) {
		if (fontinfo->bold == TRUE) {
			apply_gtk_tag(buffer, start_iter, end_iter, TAG_BOLD);
		} else {
			remove_gtk_tag(buffer, start_iter, end_iter, TAG_BOLD);
		}
		if (fontinfo->italic == TRUE) {
			apply_gtk_tag(buffer, start_iter, end_iter, TAG_ITALIC);
		} else {
			remove_gtk_tag(buffer, start_iter, end_iter, TAG_ITALIC);
		}
		if ((zoom_level != ZOOM_IN_LEVEL) && (zoom_level != ZOOM_OUT_LEVEL)
		    && (zoom_level != ZOOM_IN_MID_LEVEL)) {
			zoom_level = ZOOM_NORMAL_LEVEL;
		}
		if (gtkattr->font != NULL) {
			size = pango_font_description_get_size(gtkattr->font);
			font_name = pango_font_description_get_family(gtkattr->font);
		}
		if (((font_name != NULL)
		     && (g_ascii_strcasecmp(fontinfo->name, font_name) != 0))
		    || ((gint) ((atoi(fontinfo->size) * (zoom_level / 100.0))
				/ font_scaling_factor) !=
			size / (PANGO_SCALE * (font_scaling_factor)))) {
			start_offset = gtk_text_iter_get_offset(start_iter);
			end_offset = gtk_text_iter_get_offset(end_iter);
			remove_font_tags_at_offset(buffer, start_offset, end_offset);
			tag = gtk_text_buffer_create_tag(buffer, NULL, "font",
							 fontinfo->name, "size",
							 (gint) (atoi(fontinfo->size) *
								 PANGO_SCALE * (zoom_level /
										100.0)), NULL);

			gtk_text_buffer_get_iter_at_offset(buffer, start_iter, start_offset);
			gtk_text_buffer_get_iter_at_offset(buffer, end_iter, end_offset);

			gtk_text_buffer_apply_tag(buffer, tag, start_iter, end_iter);
		}

		g_free(fontinfo->size);
		g_free(fontinfo);
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 * Checks and returns if the given iterator has a bullet or number tag

 @param iter text iterator representing the position in text buffer
 @return Text tag representing the bullet or number tag.
 */
GtkTextTag *iter_has_bullet_or_number_tag(GtkTextIter * iter)
{
	GSList *tags = NULL;
	GSList *temp_tag = NULL;
	GtkTextTag *tag = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (iter == NULL) {
		ULOG_ERR("Error: Input parameter validation\n");
		return NULL;
	}

	tags = gtk_text_iter_get_tags(iter);

	for (temp_tag = tags; temp_tag != NULL; temp_tag = g_slist_next(temp_tag)) {
		tag = (GtkTextTag *) temp_tag->data;
		if ((tag->name != NULL)
		    && (g_ascii_strcasecmp(tag->name, tagnames[TAG_BULLET]) == 0)) {
			break;
		} else if ((tag->name != NULL)
			   && (g_ascii_strcasecmp(tag->name, tagnames[TAG_NUMBER]) == 0)) {
			break;
		}
		tag = NULL;
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return tag;
}


/**
 * Removes bullet or number character between the given iterators

 * @param buffer GtkTextBuffer representing the text
 * @param cur_iter current iterator or starting iterator for the bullet or number character to be searched.
 *  @param end_iter text iterator representing the limiting position to search for the bullet or number character in text buffer
 */
void remove_bullet_or_number(GtkTextBuffer * buffer, GtkTextIter * cur_iter, GtkTextIter * end_iter)
{

	gboolean matchfound = FALSE;
	GtkTextIter match_start;
	GtkTextIter match_end;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (cur_iter == NULL) || (end_iter == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}
	if (gtk_text_buffer_get_line_count(buffer) - 1 == gtk_text_iter_get_line(end_iter)) {
		end_iter = NULL;
	} else {
		gtk_text_iter_forward_char(end_iter);
	}
	matchfound =
	    gtk_text_iter_forward_search(cur_iter, BULLET_CHAR,
					 GTK_TEXT_SEARCH_VISIBLE_ONLY,
					 &match_start, &match_end, end_iter);

	if (matchfound) {
		gtk_text_buffer_delete(buffer, &match_start, &match_end);
	} else {
		matchfound =
		    gtk_text_iter_forward_search(cur_iter, NUMBER_APPEND,
						 GTK_TEXT_SEARCH_VISIBLE_ONLY,
						 &match_start, &match_end, end_iter);
		if (matchfound) {
			gtk_text_buffer_delete(buffer, cur_iter, &match_end);
		}
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 * Removes list item at the line number specified.

 * @param buffer GtkTextBuffer representing the text
 * @param lineno line number at which the list item as to be removed
 */
void remove_list_item_tag(GtkTextBuffer * buffer, gint lineno)
{
	GtkTextIter line_start;
	GtkTextIter line_end;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (lineno < 0)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	gtk_text_buffer_get_iter_at_line(buffer, &line_start, lineno);
	gtk_text_buffer_get_iter_at_line(buffer, &line_end, lineno);
	if (gtk_text_iter_forward_line(&line_end)) {
		gtk_text_iter_backward_char(&line_end);
	} else {
		gtk_text_iter_forward_sentence_end(&line_end);
	}

	remove_bullet_or_number(buffer, &line_start, &line_end);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 * Removes the list items between the iterators specified

 * @param buffer GtkTextBuffer representing the text
 * @param start_iter starting text iterator representing the position in text buffer
 * @param end_iter ending text iterator representing the position in text buffer
 */
void remove_list_items(GtkTextBuffer * buffer, GtkTextIter * start_iter, GtkTextIter * end_iter)
{
	gint start_line = 0;
	gint end_line = 0;
	gint cur_line = 0;
	gint item_index = 0;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (start_iter == NULL) || (end_iter == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	start_line = gtk_text_iter_get_line(start_iter);
	end_line = gtk_text_iter_get_line(end_iter);

	for (cur_line = start_line; cur_line <= end_line; cur_line++) {
		item_index++;
		remove_list_item_tag(buffer, cur_line);
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 * Get the index for numbering at a given line number.

 * @param buffer GtkTextBuffer representing the text
 * @param  lineno line number at which the number list index to be claculated

 * @return gint representing the line index for the number list
 */
gint get_list_number_value(GtkTextBuffer * buffer, gint lineno)
{

	GtkTextIter match_start;
	GtkTextIter match_end;
	GtkTextIter cur_iter;
	GtkTextIter lineend;
	gchar *text = NULL;
	gint retval = 0;
	gboolean matchfound = FALSE;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (lineno < 0)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return retval;
	}

	gtk_text_buffer_get_iter_at_line(buffer, &cur_iter, lineno);

	gtk_text_buffer_get_iter_at_offset(buffer, &lineend, gtk_text_iter_get_offset(&cur_iter));

	if (!gtk_text_iter_forward_line(&lineend)) {
		gtk_text_iter_forward_sentence_end(&lineend);
	}

	matchfound =
	    gtk_text_iter_forward_search(&cur_iter, NUMBER_APPEND,
					 GTK_TEXT_SEARCH_VISIBLE_ONLY,
					 &match_start, &match_end, &lineend);

	if (matchfound == TRUE) {
		text = gtk_text_buffer_get_text(buffer, &cur_iter, &match_start, FALSE);
		retval = atoi(text);
	}
	g_free(text);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return retval;
}


/**
 * Prepares the bullet or number character depending upon the listing type 
 * at the given iterator.

 * @param buffer GtkTextBuffer representing the text
 * @param start_iter starting text iterator representing the position in text buffer

 * @return gchar representing the bullet or number text.
 */
gchar *get_bullet_or_number_char(GtkTextBuffer * buffer, GtkTextIter * start_iter, GtkTextTag * tag)
{
	gchar *list_char = NULL;
	gint cur_offset = 0;
	gint start_lineno = 0;
	gint cur_lineno = 0;
	gint number = 0;
	GtkTextIter tag_start;
	GtkTextTag *line_tag = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (start_iter == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return NULL;
	}
	/*hack for problem of removal of list tag which considers 
	 * start of line rather than end of previous line*/
	if (!gtk_text_iter_ends_line(start_iter)) {
		line_tag = iter_has_bullet_or_number_tag(start_iter);
		if (line_tag == NULL) {
			return NULL;
		}
		if (g_ascii_strcasecmp(tag->name, line_tag->name) != 0) {
			return NULL;
		}
	}

	cur_offset = gtk_text_iter_get_offset(start_iter);
	if ((tag->name != NULL)
	    && (g_ascii_strcasecmp(tag->name, tagnames[TAG_BULLET]) == 0)) {
		list_char = g_strdup(BULLET_CHAR);
	} else if ((tag->name != NULL)
		   && (g_ascii_strcasecmp(tag->name, tagnames[TAG_NUMBER]) == 0)) {
		gtk_text_buffer_get_iter_at_offset(buffer, &tag_start, cur_offset);
		gtk_text_iter_backward_to_tag_toggle(&tag_start, tag);
		start_lineno = gtk_text_iter_get_line(&tag_start);
		cur_lineno = gtk_text_iter_get_line(start_iter);

		if (cur_lineno == start_lineno) {
			list_char = g_strdup(NUMBER_START);
		} else {
			number = get_list_number_value(buffer, cur_lineno - 1);
			number++;
			list_char = g_strdup_printf("%d%s", number, NUMBER_APPEND);
		}
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return list_char;
}


/**
This function will apply the tags on bullet/number if that particular line is an empty line.

@param start is start of list item
@param end is end of list item 
@return void
*/
static void apply_format_on_listitem(GtkTextIter start, GtkTextIter end, EditorInfo * editorinfo)
{
	PangoAttrIterator *iterator = NULL;

	iterator = pango_attr_list_get_iterator(editorinfo->attr_list);
	if (iterator != NULL) {
		format_font_iterator(editorinfo, iterator, NULL, start, end);
		pango_attr_iterator_destroy(iterator);
	}
}

/**
 * This function will apply the tags applied at the source iter for the range of startiter and enditer.

 * @param buffer GtkTextBuffer representing the text
 * @param sourceiter text iterator representing the source position in text buffer
 * @param startiter text iterator representing the starting position in text buffer
 * @param enditer text iterator representing the ending position in text buffer
 */
void
text_buffer_update_tags(GtkTextBuffer * buffer, GtkTextIter * sourceiter,
			GtkTextIter * startiter, GtkTextIter * enditer, EditorInfo * editorinfo)
{
	GSList *sourcetags = NULL;
	GSList *tempsourcetag = NULL;

	GSList *targettags = NULL;
	GSList *temptargettag = NULL;

	gboolean tagpresent = FALSE;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (sourceiter == NULL) || (startiter == NULL)
	    || (enditer == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	if (gtk_text_iter_ends_line(sourceiter)) {
		apply_format_on_listitem(*startiter, *sourceiter, editorinfo);
	}

	sourcetags = gtk_text_iter_get_tags(sourceiter);
	if (sourcetags == NULL) {
		return;
	}
	targettags = gtk_text_iter_get_tags(startiter);
	/*comparing if any of the tags are missing from the source tag list. */
	for (tempsourcetag = sourcetags; tempsourcetag != NULL;
	     tempsourcetag = g_slist_next(tempsourcetag)) {
		tagpresent = FALSE;
		for (temptargettag = targettags; temptargettag != NULL;
		     temptargettag = g_slist_next(temptargettag)) {
			if ((tempsourcetag->data != NULL)
			    && (temptargettag->data != NULL)) {
				if (tempsourcetag->data == temptargettag->data) {
					tagpresent = TRUE;
					break;
				}
			}
		}

		if (tagpresent == FALSE) {
			if (tempsourcetag->data != NULL) {
				gtk_text_buffer_apply_tag(buffer,
							  (GtkTextTag *) tempsourcetag->
							  data, startiter, enditer);
			}
		}
	}
	g_slist_free(sourcetags);
	g_slist_free(targettags);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
gets the iterators boundaries of bullet or number chars if present in a line.

@param cur is the start iterator in buffer to look for bullet/number char
@param char_start gets populated with the start of bullet/number char
@param char_end gets populated with the end of bullet/number char

@return TRUE if bullet/number char is found in a line
*/
static gboolean get_listchar_bounds(GtkTextIter cur_iter, GtkTextIter * char_start,
				    GtkTextIter * char_end)
{
	gboolean matchfound = FALSE;
	GtkTextIter *line_end = NULL;

	if ((char_start == NULL) || (char_end == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return FALSE;
	}

	line_end = gtk_text_iter_copy(&cur_iter);
	gtk_text_iter_forward_to_line_end(line_end);
	gtk_text_iter_forward_char(line_end);
	if (gtk_text_iter_is_end(line_end)) {
		gtk_text_iter_free(line_end);
		line_end = NULL;
	}

	/*search for bullet char */
	matchfound =
	    gtk_text_iter_forward_search(&cur_iter, BULLET_CHAR,
					 GTK_TEXT_SEARCH_VISIBLE_ONLY,
					 char_start, char_end, line_end);
	if (!matchfound) {
		/*search for number char */
		matchfound =
		    gtk_text_iter_forward_search(&cur_iter, NUMBER_APPEND,
						 GTK_TEXT_SEARCH_VISIBLE_ONLY,
						 char_start, char_end, line_end);
	}

	if (line_end != NULL) {
		gtk_text_iter_free(line_end);
	}
	return matchfound;
}

/**
Checks whether underline/strikethrough tags are present on  bullet/number.
If present it removes these tags on those chars.

@param buffer GtkTextBuffer of message body.
@param start is the start iterator in buffer to look for underline/strikethrough tags on bullet/number
@param end is the end iterator in buffer to look for underline/strikethrough tags on bullet/number

@return void
*/
static void check_tags_on_listchars(GtkTextBuffer * buffer, GtkTextIter * start, GtkTextIter * end)
{
	GtkTextTagTable *tagtable = NULL;
	GtkTextTag *tag_underline = NULL;
	GtkTextTag *tag_strikethrough = NULL;
	GtkTextIter cur;
	GtkTextIter char_start, char_end;

	if ((buffer == NULL) || (start == NULL) || (end == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	tagtable = gtk_text_buffer_get_tag_table(buffer);
	tag_underline = gtk_text_tag_table_lookup(tagtable, tagnames[TAG_UNDERLINE]);
	tag_strikethrough = gtk_text_tag_table_lookup(tagtable, tagnames[TAG_STRIKE_THROUGH]);
	for (cur = *start; gtk_text_iter_compare(&cur, end) < 0; gtk_text_iter_forward_line(&cur)) {
		if (get_listchar_bounds(cur, &char_start, &char_end)) {
			/*remove line tags on bullet/number chars */
			gtk_text_buffer_remove_tag(buffer, tag_underline, &cur, &char_end);
			gtk_text_buffer_remove_tag(buffer, tag_strikethrough, &cur, &char_end);
		}
	}
}

/**
 * Insert bullet or number at the given iterator

 * @param buffer GtkTextBuffer representing the text
 * @param start_iter text iterator representing the position in text buffer
 */
void
insert_bullet_or_number(GtkTextBuffer * buffer, GtkTextIter * start_iter,
			GtkTextTag * tag, EditorInfo * editorinfo)
{

	gchar *list_char = NULL;
	GtkTextIter char_iter;
	gint offset = 0, list_char_len = 0;
	gchar *character = NULL;
	gchar *input_char = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (start_iter == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	list_char = get_bullet_or_number_char(buffer, start_iter, tag);
	if (list_char != NULL) {
		offset = gtk_text_iter_get_offset(start_iter);

		g_signal_handlers_block_by_func(G_OBJECT(buffer),
						G_CALLBACK(on_text_buffer_apply_tag), editorinfo);
		g_signal_handlers_block_by_func(G_OBJECT(buffer),
						G_CALLBACK(on_text_buffer_insert_text), editorinfo);
		g_signal_handlers_block_by_func(G_OBJECT(buffer),
						G_CALLBACK
						(on_text_buffer_delete_range), editorinfo);

		/*remove existing bullet or number character */
		gtk_text_buffer_get_iter_at_offset(buffer, &char_iter, offset);
		if (!gtk_text_iter_ends_line(&char_iter)) {
			gtk_text_iter_forward_to_line_end(&char_iter);
		}
		remove_bullet_or_number(buffer, start_iter, &char_iter);

		gtk_text_buffer_get_iter_at_offset(buffer, start_iter, offset);
		gtk_text_buffer_get_iter_at_offset(buffer, &char_iter, offset);
		gtk_text_iter_forward_char(&char_iter);

		/*insert with applying the tag bullet or number. */
		if (g_ascii_strcasecmp(list_char, BULLET_CHAR) == 0) {
			gtk_text_buffer_insert_with_tags_by_name(buffer, start_iter,
								 list_char, -1,
								 tagnames[TAG_BULLET], NULL);
			list_char_len = BULLET_LENGTH;
		} else {
			gtk_text_buffer_insert_with_tags_by_name(buffer, start_iter,
								 list_char, -1,
								 tagnames[TAG_NUMBER], NULL);
			list_char_len = strlen(list_char);
		}

		gtk_text_buffer_get_iter_at_offset(buffer, start_iter, offset);
		gtk_text_buffer_get_iter_at_offset(buffer, &char_iter, offset);
		gtk_text_iter_forward_chars(&char_iter, list_char_len);

		/*Update the tags at the line starting position on to 
		   the bullet or number character inserted. */
		text_buffer_update_tags(buffer, &char_iter, start_iter, &char_iter, editorinfo);

		g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
						  G_CALLBACK(on_text_buffer_apply_tag), editorinfo);
		g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
						  G_CALLBACK
						  (on_text_buffer_insert_text), editorinfo);
		g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
						  G_CALLBACK
						  (on_text_buffer_delete_range), editorinfo);

		g_free(list_char);
		g_free(input_char);
		g_free(character);
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 * Insert list item at the line number specified

 * @param buffer GtkTextBuffer representing the text
 * @param lineno line number in text buffer
 */
void
insert_list_item_tag(GtkTextBuffer * buffer, gint lineno, GtkTextTag * tag, EditorInfo * editorinfo)
{

	GtkTextIter line_start;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (lineno < 0)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	gtk_text_buffer_get_iter_at_line(buffer, &line_start, lineno);
	insert_bullet_or_number(buffer, &line_start, tag, editorinfo);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**


 * @param buffer GtkTextBuffer representing the text
 * @param start_iter Text iterator representing the starting position
 * @param end_iter Text iterator representing the ending position
 */
void
set_list_items(GtkTextBuffer * buffer, GtkTextIter * start_iter,
	       GtkTextIter * end_iter, GtkTextTag * tag, EditorInfo * editorinfo)
{
	gint start_line = 0;
	gint end_line = 0;
	gint cur_line = 0;
	gint item_index = 0;
	GtkTextIter test_iter, match_start, match_end;
	gboolean matchfound = FALSE;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (start_iter == NULL) || (end_iter == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_after_text_delete), editorinfo);
	start_line = gtk_text_iter_get_line(start_iter);
	end_line = gtk_text_iter_get_line(end_iter);
	gtk_text_buffer_get_iter_at_line(buffer, start_iter, start_line);

	for (cur_line = start_line; cur_line <= end_line; cur_line++) {
		item_index++;
		if ((cur_line == end_line)
		    && (gtk_text_buffer_get_line_count(buffer) - 1 == end_line)) {
			gtk_text_buffer_get_iter_at_line(buffer, &test_iter, end_line);
			matchfound =
			    gtk_text_iter_forward_search(&test_iter, BULLET_CHAR,
							 GTK_TEXT_SEARCH_VISIBLE_ONLY,
							 &match_start, &match_end, NULL);

			if (matchfound == FALSE) {
				matchfound =
				    gtk_text_iter_forward_search(&test_iter, NUMBER_APPEND,
								 GTK_TEXT_SEARCH_VISIBLE_ONLY,
								 &match_start, &match_end, NULL);
			}
			if (matchfound == TRUE) {
				//break;
			}
		}
		insert_list_item_tag(buffer, cur_line, tag, editorinfo);
	}

	gtk_text_buffer_get_iter_at_line(buffer, start_iter, start_line);
	gtk_text_buffer_get_iter_at_line(buffer, end_iter, end_line);
	gtk_text_iter_forward_to_line_end(end_iter);
	/* remove underline and strikethrough tags from bullet/number */
	check_tags_on_listchars(buffer, start_iter, end_iter);

	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_after_text_delete), editorinfo);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 * This is invoked when a tag is applied in the text buffer. The bullet and number tags are handled.

 * @param buffer GtkTextBuffer representing the text
 * @param start_iter Text iterator representing the starting position
 * @param end_iter Text iterator representing the ending position
 * @param user_data : not used
 */
void
on_text_buffer_apply_tag(GtkTextBuffer * buffer, GtkTextTag * tag,
			 GtkTextIter * start_iter, GtkTextIter * end_iter, EditorInfo * editorinfo)
{
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (tag == NULL) || (start_iter == NULL)
	    || (end_iter == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_after_text_insert), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_before_text_insert), editorinfo);
	if ((tag->name != NULL)
	    && (g_ascii_strcasecmp(tag->name, tagnames[TAG_BULLET]) == 0)) {
		set_list_items(buffer, start_iter, end_iter, tag, editorinfo);
	} else if ((tag->name != NULL)
		   && (g_ascii_strcasecmp(tag->name, tagnames[TAG_NUMBER]) == 0)) {
		set_list_items(buffer, start_iter, end_iter, tag, editorinfo);
	}
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_after_text_insert), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_before_text_insert), editorinfo);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is called to update following lines in listed lines upon removal of 
   of some number in between. 

@param buffer GtkTextBuffer representing the text
@param offset is end offset of removed number tag
@param tag is  number tag.
@param editorinfo is struct with all editor related info

@return void
*/
static void
update_following_line_numbers(GtkTextBuffer * buffer,
			      GtkTextIter * tag_end, GtkTextTag * tag, EditorInfo * editorinfo)
{
	GtkTextIter tag_start;
	GtkTextTag *listtag = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (tag == NULL) || (editorinfo == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	if (!gtk_text_iter_forward_line(tag_end)) {
		return;
	}

	listtag = iter_has_bullet_or_number_tag(tag_end);
	if (listtag == NULL) {
		return;
	}
	if (g_ascii_strcasecmp(tag->name, listtag->name) != 0) {
		return;
	}
	tag_start = *tag_end;
	gtk_text_iter_forward_to_tag_toggle(tag_end, tag);
	gtk_text_iter_backward_char(tag_end);
	set_list_items(buffer, &tag_start, tag_end, tag, editorinfo);

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 * This is invoked when a tag is removed in the text buffer. 
 * The bullet and number tags are handled.

 * @param buffer GtkTextBuffer representing the text
 * @param start_iter Text iterator representing the starting position
 * @param end_iter Text iterator representing the ending position
 * @param editorinfo is struct with all editor related info
 */
void
on_text_buffer_remove_tag(GtkTextBuffer * buffer, GtkTextTag * tag,
			  GtkTextIter * start_iter, GtkTextIter * end_iter, EditorInfo * editorinfo)
{
	GtkTextIter line_end;
	gint offset = 0;
	GtkTextMark *end_mark = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (tag == NULL) || (start_iter == NULL)
	    || (end_iter == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}
	offset = gtk_text_iter_get_offset(end_iter);
	if (gtk_text_iter_ends_line(start_iter)
	    && !gtk_text_iter_is_end(start_iter)) {
		gtk_text_iter_forward_char(start_iter);
	}
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_after_text_insert), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_before_text_insert), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_delete_range), editorinfo);

	if ((tag->name != NULL)
	    && (g_ascii_strcasecmp(tag->name, tagnames[TAG_BULLET]) == 0)) {
		gtk_text_iter_backward_char(end_iter);
		remove_list_items(buffer, start_iter, end_iter);
	} else if ((tag->name != NULL)
		   && (g_ascii_strcasecmp(tag->name, tagnames[TAG_NUMBER]) == 0)) {
		gtk_text_iter_backward_char(end_iter);
		end_mark = gtk_text_buffer_create_mark(buffer, NULL, end_iter, FALSE);
		remove_list_items(buffer, start_iter, end_iter);
		gtk_text_buffer_get_iter_at_mark(buffer, &line_end, end_mark);
		update_following_line_numbers(buffer, &line_end, tag, editorinfo);
		gtk_text_buffer_delete_mark(buffer, end_mark);
	}
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_after_text_insert), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_before_text_insert), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_delete_range), editorinfo);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 * Removes the preceeding bullet or number character at the current cursor position.

 * @param buffer GtkTextBuffer representing the text
 */
void remove_preceding_bullet_or_number(GtkTextBuffer * buffer)
{
	GtkTextIter iter;
	GtkTextIter iter2;
	gint line = 0;
	gchar *text = NULL;
	gint start_off = 0, end_off = 0;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
	line = gtk_text_iter_get_line(&iter);
	gtk_text_buffer_get_iter_at_line(buffer, &iter, line);
	gtk_text_buffer_get_iter_at_line(buffer, &iter2, line);

	if (text != NULL) {
		gtk_text_iter_forward_chars(&iter2, strlen(text));
	}
	gtk_text_iter_forward_word_end(&iter2);
	gtk_text_iter_backward_word_start(&iter2);
	start_off = gtk_text_iter_get_line_offset(&iter);
	end_off = gtk_text_iter_get_line_offset(&iter2);
	gtk_text_buffer_delete(buffer, &iter, &iter2);

	g_free(text);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Moves the cursor if it happens to be somewhere inside the bullet or number item to a position after it.

@param textview is body field of editor.
*/
void move_cursor_to_listitem_end(GtkWidget * textview)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextIter iter;
	GtkTextIter iter2;
	gint line = 0, text_len = 0;
	gchar *text = NULL;
	gint cursor_off = 0, list_end_off = 0;
	GtkTextTag *tag = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (textview == NULL) {
		return;
	}

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
	gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
	tag = iter_has_bullet_or_number_tag(&iter);
	if (tag == NULL) {
		return;
	}
	text = get_bullet_or_number_char(buffer, &iter, tag);
	if (g_ascii_strcasecmp(text, BULLET_CHAR) == 0) {
		text_len = BULLET_LENGTH;
	} else {
		text_len = strlen(text);
	}
	line = gtk_text_iter_get_line(&iter);
	gtk_text_buffer_get_iter_at_line(buffer, &iter2, line);
	if (text != NULL) {
		gtk_text_iter_forward_chars(&iter2, text_len);
	}
	cursor_off = gtk_text_iter_get_line_offset(&iter);
	list_end_off = gtk_text_iter_get_line_offset(&iter2);
	if (cursor_off < list_end_off) {
		gtk_text_buffer_move_mark_by_name(buffer, "insert", &iter2);
		gtk_text_buffer_move_mark_by_name(buffer, "selection_bound", &iter2);
	}

	g_free(text);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 * Invoked when text is deleted in the text buffer. Handles bullets &
 * no. upon delete/backspace key-press.

 * @param buffer GtkTextBuffer representing the text
 * @param start_iter Text iterator representing the starting position
 * @param end_iter Text iterator representing the ending position
 * @param editorinfo is pointer to EditorInfo struct.
 */
void
bullet_num_deletion(GtkTextBuffer * buffer, GtkTextIter * start_iter,
		    GtkTextIter * end_iter, EditorInfo * editorinfo)
{
	GtkTextTag *tag = NULL;
	gboolean matchfound = FALSE;
	GtkTextIter match_start, match_end, remove_start, remove_end;
	gint lineno = 0;
	UndoOpInfo *op = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (start_iter == NULL) || (end_iter == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}
	remove_start = *start_iter;
	remove_end = *end_iter;
	if (!gtk_text_iter_ends_line(end_iter)) {
		gtk_text_iter_forward_to_line_end(&remove_end);
	}
	lineno = gtk_text_iter_get_line(&remove_start);
	gtk_text_buffer_get_iter_at_line(buffer, &remove_start, lineno);

	if ((gtk_text_iter_get_offset(end_iter) - gtk_text_iter_get_offset(start_iter)) != 1) {
		return;
	}
	tag = iter_has_bullet_or_number_tag(start_iter);
	if (tag == NULL) {
		return;
	}

	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(bullet_num_deletion), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_delete_range), editorinfo);
	matchfound =
	    gtk_text_iter_backward_search(end_iter, BULLET_CHAR,
					  GTK_TEXT_SEARCH_VISIBLE_ONLY,
					  &match_start, &match_end, &remove_start);
	if (!matchfound) {
		matchfound =
		    gtk_text_iter_backward_search(end_iter, NUMBER_APPEND,
						  GTK_TEXT_SEARCH_VISIBLE_ONLY,
						  &match_start, &match_end, &remove_start);
	}
	if (matchfound) {
		if (gtk_text_iter_equal(&match_end, end_iter)) {
			op = g_queue_pop_tail(editorinfo->undo_queue);
			undo_free_op(op);
			op = NULL;
			g_signal_handlers_block_by_func(G_OBJECT(buffer),
							G_CALLBACK(undoins), editorinfo);
			g_signal_handlers_block_by_func(G_OBJECT(buffer),
							G_CALLBACK(undodel), editorinfo);
			op = update_undoinfo_on_format_change(editorinfo,
							      editorinfo->toolbar->bullets);
			gtk_text_iter_backward_char(&remove_start);
			gtk_text_iter_forward_char(&remove_end);
			if (g_ascii_strcasecmp(tag->name, tagnames[TAG_BULLET]) == 0) {
				remove_gtk_tag(buffer, &remove_start, &remove_end, TAG_BULLET);
			} else {
				remove_gtk_tag(buffer, &remove_start, &remove_end, TAG_NUMBER);
			}
			undo_set_delete_range(op, editorinfo);
			refresh_current_attributes(GTK_TEXT_VIEW(editorinfo->msgFields->body),
						   editorinfo);
			/*undodel callbak is unblocked in on_text_buffer_delete_range() */
			g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
							  G_CALLBACK(undoins), editorinfo);
			g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
							  G_CALLBACK(undodel), editorinfo);
			g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
							  G_CALLBACK
							  (bullet_num_deletion), editorinfo);
			g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
							  G_CALLBACK
							  (on_text_buffer_delete_range),
							  editorinfo);
			return;
		}
	}

	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(bullet_num_deletion), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_delete_range), editorinfo);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

static GtkTextTag *check_apply_list_tag_on_line(GtkTextBuffer * buffer,
						GtkTextIter * end_iter, EditorInfo * editorinfo)
{
	gint lineno = 0;
	GtkTextIter start, end;
	GtkTextTag *tag = NULL;

	lineno = gtk_text_iter_get_line(end_iter);
	gtk_text_buffer_get_iter_at_line(buffer, &start, lineno);
	tag = iter_has_bullet_or_number_tag(&start);
	if (tag == NULL) {
		tag = iter_has_bullet_or_number_tag(end_iter);
		if (tag == NULL) {
			return NULL;
		}
		end = start;
		if (g_ascii_strcasecmp(tag->name, tagnames[TAG_BULLET]) == 0) {
			set_iterators_for_tag(buffer, &start, &end, TAG_BULLET, FALSE);
			remove_gtk_tag(buffer, &start, &end, TAG_BULLET);
		} else {
			set_iterators_for_tag(buffer, &start, &end, TAG_NUMBER, FALSE);
			remove_gtk_tag(buffer, &start, &end, TAG_NUMBER);
		}
		return NULL;
	}
	end = start;
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_apply_tag), editorinfo);

	if (g_ascii_strcasecmp(tag->name, tagnames[TAG_BULLET]) == 0) {
		set_iterators_for_tag(buffer, &start, &end, TAG_BULLET, TRUE);
		apply_gtk_tag(buffer, &start, &end, TAG_BULLET);
	} else {
		set_iterators_for_tag(buffer, &start, &end, TAG_NUMBER, TRUE);
		apply_gtk_tag(buffer, &start, &end, TAG_NUMBER);
	}

	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_apply_tag), editorinfo);
	return tag;
}

/**
 * Invoked when text is deleted in the text buffer

 * @param buffer GtkTextBuffer representing the text
 * @param start_iter Text iterator representing the starting position
 * @param end_iter Text iterator representing the ending position
 * @param user_data : not used in the implementation
 */
void
on_text_buffer_delete_range(GtkTextBuffer * buffer,
			    GtkTextIter * start_iter,
			    GtkTextIter * end_iter, EditorInfo * editorinfo)
{
	GtkTextTag *tag = NULL;
	gint cur_offset = 0;
	GtkTextMark *end_mark = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (start_iter == NULL) || (end_iter == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	g_signal_handlers_block_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer), G_CALLBACK(undodel), editorinfo);

	end_mark = gtk_text_buffer_create_mark(buffer, "del_end", end_iter, FALSE);
	cur_offset = gtk_text_iter_get_offset(end_iter);

	//tag = iter_has_bullet_or_number_tag (end_iter);
	//if (tag == NULL)
	{
		tag = check_apply_list_tag_on_line(buffer, end_iter, editorinfo);
	}

	if (tag != NULL) {
		g_signal_handlers_block_by_func(G_OBJECT(buffer),
						G_CALLBACK
						(on_text_buffer_delete_range), editorinfo);
		g_signal_handlers_block_by_func(G_OBJECT(buffer),
						G_CALLBACK(on_text_buffer_insert_text), editorinfo);
		g_signal_handlers_block_by_func(G_OBJECT(buffer),
						G_CALLBACK(bullet_num_deletion), editorinfo);
		g_signal_handlers_block_by_func(G_OBJECT(buffer),
						G_CALLBACK(on_body_after_text_insert), editorinfo);
		g_signal_handlers_block_by_func(G_OBJECT(buffer),
						G_CALLBACK(on_body_before_text_insert), editorinfo);

		cur_offset = gtk_text_iter_get_offset(end_iter);
		gtk_text_buffer_get_iter_at_offset(buffer, start_iter, cur_offset);
		gtk_text_buffer_get_iter_at_offset(buffer, end_iter, cur_offset);

		/*removing leading bullet or number after delete line. */
		if (!gtk_text_iter_ends_line(end_iter)) {
			gtk_text_iter_forward_to_line_end(end_iter);
		}
		remove_bullet_or_number(buffer, start_iter, end_iter);
		gtk_text_buffer_get_iter_at_mark(buffer, start_iter, end_mark);
		gtk_text_buffer_get_iter_at_mark(buffer, end_iter, end_mark);
		gtk_text_iter_backward_to_tag_toggle(start_iter, tag);
		if (!gtk_text_iter_ends_tag(end_iter, tag)) {
			gtk_text_iter_forward_to_tag_toggle(end_iter, tag);
		}
		set_list_items(buffer, start_iter, end_iter, tag, editorinfo);

		g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
						  G_CALLBACK
						  (on_text_buffer_delete_range), editorinfo);
		g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
						  G_CALLBACK
						  (on_text_buffer_insert_text), editorinfo);
		g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
						  G_CALLBACK(bullet_num_deletion), editorinfo);
		g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
						  G_CALLBACK(on_body_after_text_insert),
						  editorinfo);
		g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
						  G_CALLBACK(on_body_before_text_insert),
						  editorinfo);
	}

	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(undodel), editorinfo);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 * Moves the cursor away from the bullet or number 

 * @param buffer GtkTextBuffer representing the text
 */
void text_buffer_set_cursor(GtkTextBuffer * buffer)
{

	GtkTextIter iter;
	GtkTextTag *tag = NULL;
	gchar *text = NULL;
	gint offset = 0;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
	offset = gtk_text_iter_get_offset(&iter);
	if (gtk_text_iter_starts_line(&iter)) {
		tag = iter_has_bullet_or_number_tag(&iter);
		if (tag != NULL) {
			text = get_bullet_or_number_char(buffer, &iter, tag);
			if (text != NULL) {
				gtk_text_buffer_get_iter_at_offset(buffer, &iter,
								   offset + strlen(text));
			}
			gtk_text_buffer_place_cursor(buffer, &iter);
			g_free(text);
		}
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Invoked to block signal handlers.

@param buffer GtkTextBuffer representing the text.
@param editorinfo is pointer to EditorInfo struct.
@return void
*/
static void block_signals_on_text_insertion(GtkTextBuffer * buffer, EditorInfo * editorinfo)
{
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (editorinfo == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_insert_text), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_delete_range), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_apply_tag), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer), G_CALLBACK(undodel), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_after_text_insert), editorinfo);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Invoked to unblock signal handlers.

@param buffer GtkTextBuffer representing the text.
@param editorinfo is pointer to EditorInfo struct.
@return void
*/
static void unblock_signals_on_text_insertion(GtkTextBuffer * buffer, EditorInfo * editorinfo)
{
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (editorinfo == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_delete_range), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_insert_text), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_apply_tag), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(undodel), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_after_text_insert), editorinfo);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 * This is invoked when text is inserted in the text buffer. 

 * @param buffer GtkTextBuffer representing the text
 * @param start_iter Text iterator representing the starting position
 * @param end_iter Text iterator representing the ending position
 * @param user_data : not used in the implementation
 */
void
on_text_buffer_insert_text(GtkTextBuffer * buffer,
			   GtkTextIter * text_iter, gchar * text,
			   gint length, EditorInfo * editorinfo)
{
	GtkTextIter tag_start;
	GtkTextIter tag_end;
	GtkTextIter prevline_iter, end_iter, match_start, match_end, buf_end;
	GtkTextTag *tag = NULL;
	gint cur_offset = 0, offset, test_offset = 0;
	gboolean matchfound = FALSE;
	gchar *prevline_text = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (text_iter == NULL) || (text == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}
	if (strchr(text, NEW_LINE_CHAR) == NULL) {
		ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return;
	}
	/*update for bullting last line */
	buf_end = *text_iter;
	if (!gtk_text_iter_is_start(&buf_end)) {
		gtk_text_iter_backward_chars(&buf_end, 2);
	}
	cur_offset = gtk_text_iter_get_offset(&buf_end);
	test_offset = gtk_text_iter_get_offset(text_iter);
	tag = iter_has_bullet_or_number_tag(&buf_end);
	if (tag == NULL) {
		ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return;
	}
	block_signals_on_text_insertion(buffer, editorinfo);
	/*block deletion signal to retain the attr when bullet is removed */
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(on_body_after_text_delete),
					  editorinfo);
	prevline_iter = *text_iter;
	end_iter = prevline_iter;
	if (gtk_text_iter_ends_line(&end_iter)) {
		gtk_text_iter_backward_line(&prevline_iter);
		end_iter = prevline_iter;
		gtk_text_iter_forward_to_line_end(&end_iter);
		matchfound =
		    gtk_text_iter_forward_search(&prevline_iter, BULLET_CHAR,
						 GTK_TEXT_SEARCH_VISIBLE_ONLY,
						 &match_start, &match_end, NULL);

		if (matchfound == FALSE) {
			matchfound =
			    gtk_text_iter_forward_search(&prevline_iter, NUMBER_APPEND,
							 GTK_TEXT_SEARCH_VISIBLE_ONLY,
							 &match_start, &match_end, NULL);
		}
		if (matchfound == TRUE) {
			prevline_text =
			    gtk_text_buffer_get_text(buffer, &match_end, &end_iter, FALSE);

			if (strlen(prevline_text) == 0) {
				gtk_text_iter_forward_char(&end_iter);
				gtk_text_iter_forward_char(&end_iter);
				offset = gtk_text_iter_get_line(&prevline_iter);
				gtk_text_buffer_get_iter_at_line(buffer, &prevline_iter, offset);
				gtk_text_iter_backward_char(&prevline_iter);
				if (g_ascii_strcasecmp(tag->name, tagnames[TAG_BULLET]) == 0) {
					//gtk_text_iter_forward_char (&prevline_iter);
					//op = update_undoinfo_on_format_change (editorinfo, 
					//                editorinfo->menu->bullets);
					populate_character_input(editorinfo);
					remove_gtk_tag(buffer, &prevline_iter, &end_iter,
						       TAG_BULLET);
				} else {
					//gtk_text_iter_forward_char (&prevline_iter);
					//op = update_undoinfo_on_format_change (editorinfo, 
					//                editorinfo->menu->numbering);
					populate_character_input(editorinfo);
					remove_gtk_tag(buffer, &prevline_iter, &end_iter,
						       TAG_NUMBER);
				}
				refresh_current_attributes(GTK_TEXT_VIEW
							   (editorinfo->msgFields->body),
							   editorinfo);
				//undo_set_delete_range (op, editorinfo);
				g_free(prevline_text);
				unblock_signals_on_text_insertion(buffer, editorinfo);
				g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
								  G_CALLBACK
								  (on_body_after_text_delete),
								  editorinfo);
				return;
			}
		}
	}
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(on_body_after_text_delete),
					  editorinfo);
	gtk_text_buffer_get_iter_at_offset(buffer, &tag_start, cur_offset);
	gtk_text_buffer_get_iter_at_offset(buffer, &tag_end, cur_offset);
	gtk_text_iter_forward_char(&tag_end);
	if (gtk_text_iter_ends_tag(&tag_end, tag)) {
		gtk_text_iter_forward_char(&tag_end);
	} else {
		gtk_text_iter_forward_to_tag_toggle(&tag_end, tag);
	}
	gtk_text_iter_backward_to_tag_toggle(&tag_start, tag);
	set_list_items(buffer, &tag_start, &tag_end, tag, editorinfo);

	unblock_signals_on_text_insertion(buffer, editorinfo);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 * This is invoked to removed bullets/no. in the intermediate buffer. 

 * @param buffer GtkTextBuffer represents the 
 *   intermediate buffer before pasting any text.
 * @return void
 */
void remove_list_chars_for_paste(GtkTextBuffer * buffer)
{
	GtkTextIter start, end;
	GtkTextAttributes *attributes = NULL;
	GtkTextMark *mark = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}
	gtk_text_buffer_get_start_iter(buffer, &start);
	mark = gtk_text_buffer_create_mark(buffer, NULL, &start, FALSE);
	end = start;

	while (!gtk_text_iter_is_end(&end)) {
		gtk_text_iter_get_attributes(&start, attributes);
		gtk_text_iter_forward_to_line_end(&end);
		remove_bullet_or_number(buffer, &start, &end);
		gtk_text_buffer_get_iter_at_mark(buffer, &start, mark);
		gtk_text_iter_forward_line(&start);
		gtk_text_buffer_move_mark(buffer, mark, &start);
		end = start;
	}
	gtk_text_buffer_delete_mark(buffer, mark);

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Invoked to block signal handlers upon start of paste operation.
  
@param buffer is message body TextBuffer.
@param editorinfo is pointer to Editorinfo structure representing the editor related info.

@return void.
*/
void block_signals_upon_paste(GtkTextBuffer * buffer, EditorInfo * editorinfo)
{
	if ((buffer == NULL) || (editorinfo == NULL)) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_after_text_insert), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_before_text_insert), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_apply_tag), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_insert_text), editorinfo);

}

/**
Invoked to unblock signal handlers upon completion of paste operation.
  
@param buffer is message body TextBuffer.
@param editorinfo is pointer to Editorinfo structure representing the editor related info.

@return void.
*/
void unblock_signals_upon_paste(GtkTextBuffer * buffer, EditorInfo * editorinfo)
{
	if ((buffer == NULL) || (editorinfo == NULL)) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_after_text_insert), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_before_text_insert), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_apply_tag), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_insert_text), editorinfo);

}

/**
Callback function called upon selecting paste from VKM IM. Invoked to block signal handlers upon text insertion.
  
@param view is message body TextView.
@param editorinfo is pointer to Editorinfo structure representing the editor related info.

@return void.
*/
void handle_paste_with_im(GtkTextView * view, EditorInfo * editorinfo)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextIter start;
	GtkTextIter end;

	if ((view == NULL) || (editorinfo == NULL)) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	buffer = gtk_text_view_get_buffer(view);
	gtk_text_buffer_get_selection_bounds(buffer, &start, &end);
	if (editorinfo->insert_mark == NULL) {
		editorinfo->insert_mark = gtk_text_buffer_create_mark(buffer, NULL, &start, TRUE);
	} else {
		gtk_text_buffer_move_mark(buffer, editorinfo->insert_mark, &start);
	}
	/*signals are blocked so that paste of text having both
	   formatted & non formated constitute one operation */
	block_signals_upon_paste(buffer, editorinfo);
}

/**
Looks for the completion of paste operation. Upon completion carries out post paste processes.
  
@param buffer is message body TextBuffer.
@param paste_start is start position of paste.
@param paste_end is end position of paste.
@param editorinfo is pointer to Editorinfo structure representing the editor related info.

@return void.
*/
void process_paste_completion(GtkTextBuffer * buffer, gint paste_start, gint paste_end,
			      EditorInfo * editorinfo)
{
	GtkTextIter insertiter;
	gint offset = 0;
	gint i = 0;

	if ((buffer == NULL) || (editorinfo == NULL)) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	/*loop is run to ensure control doesn't go to next line before 
	   completion of paste operation. */
	while (1) {
		gtk_text_buffer_get_iter_at_mark(buffer, &insertiter,
						 gtk_text_buffer_get_insert(buffer));
		offset = gtk_text_iter_get_offset(&insertiter);
		if (offset >= paste_end) {
			/*pasting is completed */
			break;
		}
		/*flush pending gtk events after every 20 iteration of loop. */
		i++;
		if (i % 20 == 0)
			ui_flush_gtk_events();

	}
	update_list_item_paste(buffer, paste_start, editorinfo);
	unblock_signals_upon_paste(buffer, editorinfo);
	undo_paste(buffer, paste_start, editorinfo);
	editorinfo->formating_preselected = TRUE;
	refresh_current_attributes(GTK_TEXT_VIEW(editorinfo->msgFields->body), editorinfo);
}

/**
Called to obtain start and the expexted end position of pasted text and proceed further to carry out post paste processes.
  
@param clipboard is a pointer to the GtkClipboard.
@param selection_data is a pointer to the copied text.
@param editorinfo is pointer to Editorinfo structure representing the editor related info.

@return void.
*/
void look_for_paste_completion(GtkClipboard * clipboard, gchar * selection_data,
			       EditorInfo * editorinfo)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextIter start;
	gint paste_start = 0;
	gint paste_end = 0;

	if (editorinfo == NULL) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editorinfo->msgFields->body));
	gtk_text_buffer_get_iter_at_mark(buffer, &start, editorinfo->insert_mark);
	paste_start = gtk_text_iter_get_offset(&start);
	paste_end = g_utf8_strlen(selection_data, -1) + paste_start;
	/*see whether copied text is completely pasted. does post paste requirements thereafter. */
	process_paste_completion(buffer, paste_start, paste_end, editorinfo);
}

/**
Callback function called upon selecting paste from VKM IM. Invoked to carry out post paste processes.
  
@param view is message body TextView.
@param editorinfo is pointer to Editorinfo structure representing the editor related info.

@return void.
*/
void handle_paste_completion_with_im(GtkTextView * view, EditorInfo * editorinfo)
{
	GtkClipboard *clipboard = NULL;

	if ((view == NULL) || (editorinfo == NULL)) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	clipboard = gtk_clipboard_get(GDK_NONE);
	gtk_clipboard_request_text(clipboard, (GtkClipboardTextReceivedFunc)
				   look_for_paste_completion, editorinfo);
}

/**
Handles pasting of bullet/number text.
  
@param buffer is message body TextBuffer.
@param offset is start position of paste.
@param editorinfo is pointer to Editorinfo structure representing the editor related info.

@return void.
*/
void update_list_item_paste(GtkTextBuffer * buffer, gint offset, EditorInfo * editorinfo)
{
	GtkTextIter start_iter, end_iter;
	GtkTextTag *tag = NULL;
	GtkTextMark *start_mark = NULL, *end_mark = NULL;

	gtk_text_buffer_get_iter_at_offset(buffer, &start_iter, offset);
	tag = iter_has_bullet_or_number_tag(&start_iter);
	if (tag != NULL) {
		if (!gtk_text_iter_starts_line(&start_iter)) {
			gtk_text_buffer_insert(buffer, &start_iter, HTML_NL, -1);
			gtk_text_buffer_get_iter_at_offset(buffer, &start_iter, offset);
		}
		gtk_text_iter_forward_char(&start_iter);
		end_iter = start_iter;
		gtk_text_iter_forward_to_tag_toggle(&end_iter, tag);
		if (!gtk_text_iter_begins_tag(&start_iter, tag)) {
			gtk_text_iter_backward_to_tag_toggle(&start_iter, tag);
		}
		start_mark = gtk_text_buffer_create_mark(buffer, NULL, &start_iter, FALSE);
		end_mark = gtk_text_buffer_create_mark(buffer, NULL, &end_iter, FALSE);
		if (g_ascii_strcasecmp(tag->name, tagnames[TAG_BULLET]) == 0) {
			if (gtk_text_iter_ends_line(&end_iter)) {
				gtk_text_iter_forward_char(&end_iter);
			}
			remove_gtk_tag(buffer, &start_iter, &end_iter, TAG_NUMBER);
		} else {
			remove_gtk_tag(buffer, &start_iter, &end_iter, TAG_BULLET);
		}
		gtk_text_buffer_get_iter_at_mark(buffer, &start_iter, start_mark);
		gtk_text_buffer_get_iter_at_mark(buffer, &end_iter, end_mark);
		set_list_items(buffer, &start_iter, &end_iter, tag, editorinfo);
		gtk_text_buffer_delete_mark(buffer, start_mark);
		gtk_text_buffer_delete_mark(buffer, end_mark);
	}
}


/**
 * Invoked when a insert image option is selected from menu.

 * @param filepath file path representing the image file
 * @param editorinfo Editor info representing the editor.
 */
void text_buffer_insert_image(const gchar * filepath, EditorInfo * editorinfo)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextIter startiter;
	GtkTextIter enditer;
	gint offset = 0;
	GdkPixbuf *image = NULL;
	gchar *temppath = NULL;
	GError *error = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((editorinfo == NULL) || (filepath == NULL)) {
		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
		return;
	}

	image = gdk_pixbuf_new_from_file(filepath, &error);
	if (image == NULL) {
		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
		ULOG_ERR("Unable to load image from file path\n");
		return;
	}
	temppath = g_strdup(filepath);
	g_object_set_data(G_OBJECT(image), "icon-name", temppath);

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editorinfo->msgFields->body));

	gtk_text_buffer_get_iter_at_mark(buffer, &startiter, gtk_text_buffer_get_insert(buffer));
	offset = gtk_text_iter_get_offset(&startiter);

	gtk_text_buffer_get_iter_at_offset(buffer, &startiter, offset);
	gtk_text_buffer_get_iter_at_offset(buffer, &enditer, offset + 1);
	editorinfo->editorModified = TRUE;
	gtk_text_buffer_insert_pixbuf(buffer, &startiter, image);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Invoked to delete inline image from body field

@param buffer is msg body field.
@param start is start iterator of image
@return TRUE if image is present and removed from the start iterator.
*/
static gboolean delete_inline_image(GtkTextBuffer * buffer, GtkTextIter start)
{
	GdkPixbuf *pixbuf = NULL;
	GtkTextIter end;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("error validating input parameters\n");
		return FALSE;
	}
	pixbuf = gtk_text_iter_get_pixbuf(&start);
	if (pixbuf != NULL) {
		end = start;
		gtk_text_iter_forward_char(&end);
		gtk_text_buffer_delete(buffer, &start, &end);
		pixbuf = NULL;
		return TRUE;
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return FALSE;
}

/**
Invoked when msg type is changed from html to plain to remove 
 inline images.

@param buffer is msg body field.
@return void
*/
void remove_inline_images(GtkTextBuffer * buffer)
{
	GdkPixbuf *pixbuf = NULL;
	GtkTextIter start;
	GtkTextIter iter;
	gint offset = 0;
	gunichar ch;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	gtk_text_buffer_get_start_iter(buffer, &start);
	ch = gtk_text_iter_get_char(&start);
	if (ch == 0xFFFC) {
		offset = gtk_text_iter_get_offset(&start);
		while (delete_inline_image(buffer, start)) {
			gtk_text_buffer_get_iter_at_offset(buffer, &start, offset);
		}
	}
	gtk_text_buffer_get_bounds(buffer, &start, &iter);
	while (gtk_text_iter_forward_find_char(&start, predicate, NULL, NULL)) {
		offset = gtk_text_iter_get_offset(&start);
		pixbuf = gtk_text_iter_get_pixbuf(&start);
		if (pixbuf == NULL) {
			gtk_text_iter_forward_char(&start);
			continue;
		} else {
			while (delete_inline_image(buffer, start)) {
				gtk_text_buffer_get_iter_at_offset(buffer, &start, offset);
			}
		}
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}
