/**
 * @file html_parser.c
 *
 * This file contains implementation for gtk and html conversion
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */

#include "ui_common.h"
#include "html_parser.h"
#include "htcallback.h"
#include "ui_defs.h"
#include "undo.h"

/*tagnames are defined in htcallbacks.c*/
extern const gchar *tagnames[];

static void set_inline_img_list(const gchar * filepath, MsgInfo * msginfo, guint32 num);
static gchar *prepare_html_colortag(GdkColor * color, gchar * color_tag, EditorInfo * editorinfo);
static void insert_inline_image(gchar * img_tag, GtkTextIter iter, EditorInfo * editorinfo);


/**
Prepares html tag for foreground color

@param view is msg body area
@param color GdkColor value to be converted into html tag
@param color_tag describes whether the tag to be created is for 
       foreground or background 

@return html foreground tag. The pointer to the tag returned should be freed
  by the calling function.
*/
static gchar *prepare_html_colortag(GdkColor * color, gchar * color_tag, EditorInfo * editorinfo)
{
	gchar formatname[COLOR_BUF];
	gchar *colortag = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (color == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return NULL;
	}
	memset(formatname, 0, COLOR_BUF);
	snprintf(formatname, COLOR_BUF, "%.2x%.2x%.2x", color->red >> 8,
		 color->green >> 8, color->blue >> 8);
	/*if color is default one, return */
	if (g_ascii_strcasecmp(color_tag, HTML_TAG_FG) == 0) {
		if (gdk_color_equal(color, (GdkColor *)
				    & (editorinfo->body_style->fg))) {
			return NULL;
		}
	}
	colortag = g_strconcat(color_tag, formatname, HTML_CLOSE_GT, NULL);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return colortag;
}

/**
Prepare html font tag for the font description

@param font PangoFontDescription to be converted into html tag
@param zoom_level describes the zoom level of text
@param tag_start is start of the tag
@return html font tag
*/
gchar *get_html_font_tag_for_gtk_font_tag(PangoFontDescription * font,
					  gint zoom_level, GtkTextIter tag_start)
{
	gchar *font_str = NULL;
	const gchar *font_family = NULL;
	gint font_size = 0, font_size_index = 0, size = 0;
	GtkTextAttributes *temp_attributes = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (font == NULL) {
		ULOG_ERR("input param font is NULL\n");
		return NULL;
	}
	font_family = (gchar *) pango_font_description_get_family(font);
	if (font_family == NULL) {
		return NULL;
	}
	font_size = pango_font_description_get_size(font);
	/*size may be shown as zero as the tag may be created separately for
	 * font name and size. So below code provides creation of html tag for 
	 * name and size at the same time.*/
	if (font_size == 0) {
		temp_attributes = gtk_text_attributes_new();
		gtk_text_iter_get_attributes(&tag_start, temp_attributes);
		if (temp_attributes->font != NULL) {
			font_size = pango_font_description_get_size(temp_attributes->font);
		}
		gtk_text_attributes_unref(temp_attributes);
	}

	if ((zoom_level != ZOOM_IN_LEVEL) && (zoom_level != ZOOM_OUT_LEVEL)
	    && (zoom_level != ZOOM_IN_MID_LEVEL)) {
		zoom_level = ZOOM_NORMAL_LEVEL;
	}
	size = (gint) ((font_size / (PANGO_SCALE * font_scaling_factor)) * (100.0 / zoom_level));
	font_size_index = get_index_font_size(size);
	font_str = g_strdup_printf("<font face=\"%s\" size=%d>", font_family, font_size_index);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return font_str;
}


/**
This function creates the html tag corresponding to the gtk tag attributes.

@param view GtkTextView, body area of msg
@param tag GtkTextTag for which corresponding html tag to be generated
@param htmltag variable to hold starting html tag
@param htmltagend variable to hold ending html tag
@param zoom_level describes the zoom level of text
@param tag_start is start of the tag
@param editorinfo is pointer to EditorInfo object having editor related info.

@return TRUE if success
*/
gboolean
get_html_tag_for_gtk_tag(GtkTextTag * tag, gchar * htmltag,
			 gchar * htmltagend, gint zoom_level,
			 GtkTextIter tag_start, EditorInfo * editorinfo)
{
	GtkTextAttributes *attribute = NULL;
	gchar *fontdesc = NULL;
	gchar *fg_color_tag = NULL;
	GdkColor fgcolor;

	if (tag == NULL) {
		ULOG_ERR("Error input validation\n");
		return FALSE;
	}
	attribute = tag->values;
	strcpy(htmltag, "");

	fontdesc = get_html_font_tag_for_gtk_font_tag(attribute->font, zoom_level, tag_start);
	if (fontdesc != NULL) {
		strcat(htmltag, fontdesc);
		strcat(htmltagend, HTML_CLOSE_TAG_FONT);
	}
	if ((attribute->font != NULL) &&
	    (pango_font_description_get_weight(attribute->font) == PANGO_WEIGHT_BOLD)) {
		strcat(htmltag, HTML_TAG_BOLD);
		strcat(htmltagend, HTML_CLOSE_TAG_BOLD);
	}
	if ((attribute->font != NULL) &&
	    (pango_font_description_get_style(attribute->font) == PANGO_STYLE_ITALIC)) {
		strcat(htmltag, HTML_TAG_ITALIC);
		strcat(htmltagend, HTML_CLOSE_TAG_ITALIC);
	}
	if (attribute->appearance.underline == TRUE) {
		strcat(htmltag, HTML_TAG_UNDERLINE);
		strcat(htmltagend, HTML_CLOSE_TAG_UNDERLINE);
	}
	if (attribute->appearance.strikethrough == TRUE) {
		strcat(htmltag, HTML_TAG_STRIKE_THROUGH);
		strcat(htmltagend, HTML_CLOSE_TAG_STRIKE_THROUGH);
	}
	if (attribute->appearance.rise > 0) {
		strcat(htmltag, HTML_TAG_SUPERSCRIPT);
		strcat(htmltagend, HTML_CLOSE_TAG_SUPERSCRIPT);
	}
	if (attribute->appearance.rise < 0) {
		strcat(htmltag, HTML_TAG_SUBSCRIPT);
		strcat(htmltagend, HTML_CLOSE_TAG_SUBSCRIPT);
	}
	if (attribute->justification == GTK_JUSTIFY_CENTER) {
		strcat(htmltag, HTML_TAG_CENTER_ALIGN);
		strcat(htmltagend, HTML_CLOSE_TAG_CENTER_ALIGN);
	}
	if (attribute->justification == GTK_JUSTIFY_RIGHT) {
		strcat(htmltag, HTML_TAG_RIGHT_ALIGN);
		strcat(htmltagend, HTML_CLOSE_TAG_RIGHT_ALIGN);
	}
	fgcolor = attribute->appearance.fg_color;
	fg_color_tag = prepare_html_colortag(&fgcolor, HTML_TAG_FG, editorinfo);
	if (fg_color_tag != NULL) {
		strcat(htmltag, fg_color_tag);
		strcat(htmltagend, HTML_CLOSE_TAG_FG);
	}
	if ((tag->name != NULL)
	    && (g_ascii_strcasecmp(tag->name, tagnames[TAG_BULLET]) == 0)) {
		strcat(htmltag, HTML_TAG_BULLET);
		strcat(htmltagend, HTML_CLOSE_TAG_BULLET);
	}
	if ((tag->name != NULL)
	    && (g_ascii_strcasecmp(tag->name, tagnames[TAG_NUMBER]) == 0)) {
		strcat(htmltag, HTML_TAG_NUMBER);
		strcat(htmltagend, HTML_CLOSE_TAG_NUMBER);
	}

	strcat(htmltag, "");
	strcat(htmltagend, "");

	g_free(fontdesc);
	g_free(fg_color_tag);

	return TRUE;
}


/**
replace GtkTextTag with corresponding html tag at the offset specified.

@param view GtkTextView of the text area in html editor
@param offset position where the tag needs to be replaced
@param tag GtkTextTag representing text property
@param zoom_level is the zoom level of text
 
@return void
*/
void
replace_tag_at_offset(EditorInfo * editorinfo, gint * offset, GtkTextTag * tag, gint zoom_level)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextIter tag_start;
	GtkTextIter tag_end, temp_iter;
	gint start_offset = 0;
	gint end_offset = 0;
	gchar htmltag[100] = "";
	gchar htmltagend[100] = "";
	gint endline = 0, temp_off = 0;

	//TODO The hardcoded value 100 needs to be changed

	if ((tag == NULL)) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}

	buffer = gtk_text_view_get_buffer((GtkTextView *) editorinfo->msgFields->body);
	/*retriving the iterations to starting and ending of the tag specified. */
	gtk_text_buffer_get_iter_at_offset(buffer, &tag_end, *offset);
	gtk_text_iter_forward_to_tag_toggle(&tag_end, tag);
	endline = gtk_text_iter_get_line(&tag_end);
	temp_iter = tag_end;
	if (!gtk_text_iter_ends_line(&temp_iter)) {
		gtk_text_iter_forward_to_line_end(&temp_iter);
		temp_off = gtk_text_iter_get_line_offset(&temp_iter);
	}
	temp_off = gtk_text_iter_get_line_offset(&tag_end);
	gtk_text_buffer_get_iter_at_offset(buffer, &tag_start, *offset);
	start_offset = gtk_text_iter_get_offset(&tag_start);
	end_offset = gtk_text_iter_get_offset(&tag_end);

	/*removing gtk tag and recreating the iterators from the offsets */
	gtk_text_buffer_remove_tag(buffer, tag, &tag_start, &tag_end);

	/*First insert html tag at the end offset and then at the start offset. 
	   This will ensure that no shift in offsets due to html tag addition. */
	if (get_html_tag_for_gtk_tag(tag, htmltag, htmltagend, zoom_level, tag_start, editorinfo)) {
		gtk_text_buffer_get_iter_at_offset(buffer, &tag_end, end_offset);
		gtk_text_buffer_insert(buffer, &tag_end, htmltagend, -1);

		gtk_text_buffer_get_iter_at_offset(buffer, &tag_start, start_offset);
		gtk_text_buffer_insert(buffer, &tag_start, htmltag, -1);
	}
	*offset = *offset + strlen(htmltag);

	return;
}


/**
Check for availability of gtk tags at the given offset. 
If a gtk tag is found the same is replaced with corresponing html tag.

@param view is the text area in html editor
@param offset position where the tag needs to be replaced

@return TRUE if function executes successfully
*/
gboolean check_gtk_tags_at_offset(EditorInfo * editorinfo, gint * offset, gint zoom_level)
{
	GtkTextBuffer *buffer = NULL;
	GSList *taglist = NULL;
	GSList *templist = NULL;
	GtkTextTag *currenttag = NULL;
	GtkTextIter iter;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (editorinfo == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return FALSE;
	}

	buffer = gtk_text_view_get_buffer((GtkTextView *) editorinfo->msgFields->body);
	gtk_text_buffer_get_iter_at_offset(buffer, &iter, *offset);
	taglist = gtk_text_iter_get_tags(&iter);
	if (taglist == NULL) {
		return FALSE;
	}

	for (templist = taglist; templist != NULL; templist = g_slist_next(templist)) {
		currenttag = templist->data;
		replace_tag_at_offset(editorinfo, offset, currenttag, zoom_level);
	}

	g_slist_free(taglist);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return TRUE;
}


/**
Replaces the given string at the specified iterator offset.

@param buffer GtkTextBuffer of the text area in html editor
@param start_iter iterator at selection start
@param end_iter iterator at selection end
@param replacestr string to be replaced with
*/
void
text_buffer_replace_string(GtkTextBuffer * buffer,
			   GtkTextIter * start_iter, GtkTextIter * end_iter, gchar * replacestr)
{

	gint offset = 0, end_offset = 0;
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	offset = gtk_text_iter_get_offset(start_iter);
	end_offset = gtk_text_iter_get_offset(end_iter);
	gtk_text_buffer_insert(buffer, end_iter, replacestr, -1);
	gtk_text_buffer_get_iter_at_offset(buffer, start_iter, offset);
	gtk_text_buffer_get_iter_at_offset(buffer, end_iter, end_offset);
	gtk_text_buffer_delete(buffer, start_iter, end_iter);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 Replaces the oldstr with newstr in the buffer

 @param buffer GtkTextBuffer on which the replacement needs to be done
 @param oldstr string to be searched for
 @param newstr string to be replaced with
*/
static void gtk_text_buffer_replace_char(GtkTextBuffer * buffer, gchar * oldstr, gchar * newstr)
{
	GtkTextIter start_iter;
	GtkTextIter match_start;
	GtkTextIter match_end;
	GtkTextIter iter;
	gint offset = 0;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (oldstr == NULL) || (newstr == NULL)) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}

	gtk_text_buffer_get_start_iter(buffer, &start_iter);
	while (gtk_text_iter_forward_search
	       (&start_iter, oldstr, GTK_TEXT_SEARCH_VISIBLE_ONLY, &match_start,
		&match_end, NULL)) {
		/*replace the consequetive space chars with "&nbsp;" */
		if (g_ascii_strcasecmp(oldstr, HTML_SPC) == 0) {
			iter = match_start;
			gtk_text_iter_forward_char(&iter);
			if (gtk_text_iter_get_char(&iter) != HTML_SPC_CHAR) {
				offset = offset + strlen(oldstr);
				gtk_text_buffer_get_iter_at_offset(buffer, &start_iter, offset);
				continue;
			}
		}
		offset = gtk_text_iter_get_offset(&match_start);
		text_buffer_replace_string(buffer, &match_start, &match_end, newstr);
		offset = offset + strlen(newstr);
		gtk_text_buffer_get_iter_at_offset(buffer, &start_iter, offset);
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}


/**
Converts specific gtk delimiters to corresponding html delimiters.

@param buffer GtkTextBuffer of the text area in html editor
*/
void replace_gtk_to_html_delimiters(GtkTextBuffer * buffer)
{

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	gtk_text_buffer_replace_char(buffer, HTML_AMP, HTML_TAG_AMP);
	gtk_text_buffer_replace_char(buffer, HTML_SPC, HTML_TAG_SPC);
	gtk_text_buffer_replace_char(buffer, HTML_LT, HTML_TAG_LT);
	gtk_text_buffer_replace_char(buffer, HTML_GT, HTML_TAG_GT);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Checks whether the background color is white or not.

@param editor EditorInfo representing editor window
@return TRUE if the background color is white.
*/
gboolean backgroundcolor_is_white(EditorInfo * editor)
{
	GtkStyle *style = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (editor == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return TRUE;
	}
	style = gtk_widget_get_style(editor->msgFields->body);
	if ((editor->menu->bgcolorvalue.red != (style->white).red)
	    || (editor->menu->bgcolorvalue.blue != (style->white).blue)
	    || (editor->menu->bgcolorvalue.green != (style->white).green)) {
		return FALSE;
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return TRUE;
}

static void insert_style_tag(GtkTextBuffer * buffer, EditorInfo * editor)
{
	gint default_fontindex = 0;
	GtkTextIter start, end, start_it;
	const gchar *fontname = NULL;

	if (buffer == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	gtk_text_buffer_get_start_iter(buffer, &start_it);
	if (!gtk_text_iter_forward_search(&start_it, "font-family:",
					  GTK_TEXT_SEARCH_VISIBLE_ONLY, &start, &end, NULL)) {
		return;
	}
	default_fontindex = ui_get_editor_html_fontindex();
	fontname = font_families[default_fontindex];
	gtk_text_iter_forward_char(&end);
	gtk_text_buffer_insert(buffer, &end, fontname, -1);

	return;
}

/**
Ends and restarts the color or font tag at the end of tag of other type
@param buffer GtkTextBuffer of the text area in html editor
@param start_iter is the start of end tag
@param tag_start indicates the start of tag to be reinserted.
@param end_tag represents string of end tag of the tag restarted
@return void
*/
static void
restart_font_color_html_tag(GtkTextBuffer * buffer, GtkTextIter start,
			    GtkTextIter tag_start, gchar * end_tag)
{
	GtkTextIter gt_start, gt_end;
	gchar *tag_text = NULL;
	gint offset = 0;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (buffer == NULL || end_tag == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	/*get the html tag string to be inserted again */
	if (!gtk_text_iter_forward_search(&tag_start, HTML_GT,
					  GTK_TEXT_SEARCH_VISIBLE_ONLY, &gt_start, &gt_end, NULL)) {
		return;
	}
	tag_text = gtk_text_buffer_get_text(buffer, &tag_start, &gt_end, FALSE);

	if (!gtk_text_iter_forward_search(&start, HTML_GT,
					  GTK_TEXT_SEARCH_VISIBLE_ONLY, &gt_start, &gt_end, NULL)) {
		g_free(tag_text);
		return;
	}
	offset = gtk_text_iter_get_offset(&start);
	/*put html tag string at end of > */
	gtk_text_buffer_insert(buffer, &gt_end, tag_text, -1);
	gtk_text_buffer_get_iter_at_offset(buffer, &start, offset);
	/*put closing tag at start of the other color/font tag */
	gtk_text_buffer_insert(buffer, &start, end_tag, -1);

	g_free(tag_text);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Looks for the just previous font name/size or color html start tag before the
end tag of the other type.
@param buffer GtkTextBuffer of the text area in html editor
@param start_iter is the start of end tag
@param search_limit indicates limit for backward search.
@param end_tag represents string of end tag
@return void
*/
static void
lookup_font_start_tag(GtkTextBuffer * buffer, GtkTextIter start_iter,
		      GtkTextIter search_limit, gchar * end_tag)
{
	gchar *tag = NULL, *tagend = NULL;
	GtkTextIter tag_start, tag_end, tagend_start, tagend_end;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (buffer == NULL || end_tag == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	if (strcmp(end_tag, HTML_CLOSE_TAG_FONT) == 0) {
		tag = g_strdup(HTML_TAG_FG);
		tagend = g_strdup(HTML_CLOSE_TAG_FG);
	} else {
		tag = g_strdup(HTML_TAG_FONT);
		tagend = g_strdup(HTML_CLOSE_TAG_FONT);
	}
	if (gtk_text_iter_backward_search(&start_iter, tag,
					  GTK_TEXT_SEARCH_VISIBLE_ONLY, &tag_start,
					  &tag_end, &search_limit)) {
		if (gtk_text_iter_backward_search(&start_iter, tagend,
						  GTK_TEXT_SEARCH_VISIBLE_ONLY, &tagend_start,
						  &tagend_end, &tag_end)) {
			/*the previous start tag is same type as that of end.
			 * No need of putting the tags once again*/
			g_free(tag);
			g_free(tagend);
			return;
		}
		restart_font_color_html_tag(buffer, start_iter, tag_start, tagend);
	}

	g_free(tag);
	g_free(tagend);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Insert HTML tag starts and ends for second time for font name/size
and color at html tag ends.
@param buffer GtkTextBuffer of the text area in html editor
@return void
*/
static void reset_font_color_html_tags(GtkTextBuffer * buffer)
{
	GtkTextIter start_iter, col_start, col_end, font_start, font_end;
	GtkTextMark *mark = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	gtk_text_buffer_get_start_iter(buffer, &start_iter);
	mark = gtk_text_buffer_create_mark(buffer, NULL, &start_iter, FALSE);
	while (gtk_text_iter_forward_search(&start_iter, HTML_CLOSE_TAG_FG,
					    GTK_TEXT_SEARCH_VISIBLE_ONLY, &col_start,
					    &col_end, NULL)) {
		/*this search decides which tag end first */
		if (gtk_text_iter_forward_search(&start_iter, HTML_CLOSE_TAG_FONT,
						 GTK_TEXT_SEARCH_VISIBLE_ONLY, &font_start,
						 &font_end, &col_start)) {
			gtk_text_buffer_move_mark(buffer, mark, &font_end);
			lookup_font_start_tag(buffer, font_start, start_iter, HTML_CLOSE_TAG_FONT);
			gtk_text_buffer_get_iter_at_mark(buffer, &start_iter, mark);
		} else {
			gtk_text_buffer_move_mark(buffer, mark, &col_end);
			lookup_font_start_tag(buffer, col_start, start_iter, HTML_CLOSE_TAG_FG);
			gtk_text_buffer_get_iter_at_mark(buffer, &start_iter, mark);
		}
	}
	gtk_text_buffer_delete_mark(buffer, mark);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}



/**
Converts the gtk formated text to html data.

@param edotor EditorInfo representing editor window
@param buffer GtkTextBuffer of the text area in html editor
*/
void osso_email_format_to_html_text(EditorInfo * editor, gchar ** buftext)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	gchar *bgcolor = NULL;
	gchar *header = NULL;
	gchar *footer = NULL;
	gint offset = 0;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (editor == NULL) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}
	buffer = gtk_text_view_get_buffer((GtkTextView *) editor->msgFields->body);

	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_before_text_insert), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_after_text_insert), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_insert_text), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_delete_range), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_apply_tag), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_remove_tag), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer), G_CALLBACK(undodel), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(ui_editor_check_modified_on_insert),
					&(editor->editorModified));
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(ui_editor_check_modified_on_delete),
					&(editor->editorModified));

	replace_gtk_to_html_delimiters(buffer);
	// to replace inline images
	text_buffer_process_images(buffer, editor);
	text_buffer_replace_list_items(buffer);
	gtk_text_buffer_replace_char(buffer, HTML_NL, HTML_TAG_NL);

	gtk_text_buffer_get_start_iter(buffer, &start_iter);
	offset = gtk_text_iter_get_offset(&start_iter);
	do {
		check_gtk_tags_at_offset(editor, &offset, editor->zoominfo->previous_level);
		gtk_text_buffer_get_iter_at_offset(buffer, &start_iter, offset);
		gtk_text_iter_forward_to_tag_toggle(&start_iter, NULL);
		offset = gtk_text_iter_get_offset(&start_iter);
	}
	while (offset < gtk_text_buffer_get_char_count(buffer));

	/*add background color tag at the begining of the buffer. */
	gtk_text_buffer_get_start_iter(buffer, &start_iter);
	if (backgroundcolor_is_white(editor) == FALSE) {
		bgcolor = prepare_html_colortag(&(editor->menu->bgcolorvalue), HTML_TAG_BG, editor);
	}
	if (bgcolor != NULL) {
		gtk_text_buffer_insert(buffer, &start_iter, bgcolor, -1);
	} else {
		gtk_text_buffer_insert(buffer, &start_iter, TAG_BODY_START, -1);
	}

	/*function called to insert HTML tag starts and ends for second
	   time for font name/size and color. Once html tags are already
	   inserted depending upon the toggling positions of GTK tags.
	   Now its reinsrted depending upon the ending html tags on special
	   situation. Hack to make our cliet compatible with parsing of
	   other clients where ending HTML tag for font name/size and color
	   is case insensitive. */
	reset_font_color_html_tags(buffer);
	gtk_text_buffer_get_start_iter(buffer, &start_iter);
	gtk_text_buffer_insert(buffer, &start_iter, OSSO_EMAIL_HEADER, -1);
	insert_style_tag(buffer, editor);
	gtk_text_buffer_get_end_iter(buffer, &end_iter);
	gtk_text_buffer_insert(buffer, &end_iter, OSSO_EMAIL_FOOTER, -1);
	g_free(header);
	g_free(footer);

	gtk_text_buffer_get_bounds(buffer, &start_iter, &end_iter);
	*buftext = gtk_text_buffer_get_text(buffer, &start_iter, &end_iter, FALSE);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_before_text_insert), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_after_text_insert), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_insert_text), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_delete_range), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_apply_tag), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_remove_tag), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(undodel), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(ui_editor_check_modified_on_insert),
					  &(editor->editorModified));
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(ui_editor_check_modified_on_delete),
					  &(editor->editorModified));

	g_free(bgcolor);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Retrives the font details from the html font tag.

@param tag gchar representing the font tag.
@return font details
*/
gchar *getfontdetails_from_htmltags(gchar * tag)
{
	gchar **tmp = NULL;
	gchar *rv_str = NULL;
	gint font_sizes[] = { 6, 8, 10, 12, 16, 24, 32 };

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (!tag) {
		return NULL;
	}

	tmp = g_strsplit_set(tag, "=\"<>", -1);
	if (*tmp != NULL) {
		/*the tag is of the form <family="Arial" size=3> so temp[3] represents font family and
		   temp[5] represents font size  Basically, 
		   we are specifying how many sizes larger or 
		   how many sizes smaller than the preset font size */
		if (tmp[3] && tmp[5]) {
			rv_str =
			    g_strdup_printf("family:%s#size:%d", tmp[3], font_sizes[atoi(tmp[5])]);
		}
		g_strfreev(tmp);
		ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return rv_str;

	}
	return NULL;
}


/**
Parse the color format into GdkColor value.

@param formatname string representing color

@return GdkColor value corresponfing to format name
*/
GdkColor osso_email_color_parse(gchar * formatname)
{
	GdkColor color;
	gchar **tmp_array = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	tmp_array = g_strsplit_set(formatname, "\"", -1);
	/*This will split the format name into three strings and always 
	   * tmp_array[1] will hold the color value */

	if (*tmp_array != NULL) {
		gdk_color_parse(tmp_array[1], &color);
		g_strfreev(tmp_array);
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return color;
}


/**
Creates GtktextTag for the given html tag specified. 
This is valid for font and color properties.

@param buffer GtkTextBuffer of the text area in html editor
@param start_tag html tag 

@return GtkTextTag for the html tag specified
*/
GtkTextTag *get_gtk_tag_for_html_tag(GtkTextBuffer * buffer, gchar * start_tag)
{

	gchar *formatname = NULL;
	gint property = 0;
	GtkTextTag *gtktag = NULL;
	gchar **tmp_array = NULL;
	GdkColor color;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (start_tag == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return NULL;
	}

	property = find_property_start_htmltag(start_tag);
	switch (property) {
	case TAG_FONT:
		formatname = getfontdetails_from_htmltags(start_tag);
		tmp_array = g_strsplit_set(formatname, ":#", -1);
		if (tmp_array != NULL) {
			gtktag =
			    gtk_text_buffer_create_tag(buffer, NULL, FAMILY, tmp_array[1],
						       SIZE, (gint) (atoi(tmp_array[3]) *
								     PANGO_SCALE *
								     font_scaling_factor), NULL);
			g_strfreev(tmp_array);
		}
		break;
	case TAG_FGCOLOR:
		color = osso_email_color_parse(start_tag);
		gtktag = gtk_text_buffer_create_tag(buffer, NULL, "foreground-gdk", &color, NULL);
		break;
	case TAG_BGCOLOR:
		break;
	}
	g_free(formatname);
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return gtktag;
}


/**
Return html TAG type for the given html tag.

@param htmltag html tag 

@return TAG ID for the html tag. TAG_ID is the enum value declared in htcallbacks.h. The enum represent all the supported tags.
*/
gint find_property_start_htmltag(gchar * htmltag)
{

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (htmltag == NULL) {
		return TAG_ERROR;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_FONT)) {
		return TAG_FONT;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_BOLD)) {
		return TAG_BOLD;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_ITALIC)) {
		return TAG_ITALIC;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_FG)) {
		return TAG_FGCOLOR;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_BG)) {
		return TAG_BGCOLOR;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_UNDERLINE)) {
		return TAG_UNDERLINE;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_SUBSCRIPT)) {
		return TAG_SUBSCRIPT;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_SUPERSCRIPT)) {
		return TAG_SUPERSCRIPT;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_LEFT_ALIGN)) {
		return TAG_LEFT_ALIGN;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_CENTER_ALIGN)) {
		return TAG_CENTER_ALIGN;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_RIGHT_ALIGN)) {
		return TAG_RIGHT_ALIGN;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_STRIKE_THROUGH)) {
		return TAG_STRIKE_THROUGH;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_BULLET)) {
		return TAG_BULLET;
	} else if (g_str_has_prefix(htmltag, HTML_TAG_NUMBER)) {
		return TAG_NUMBER;
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return TAG_ERROR;
}


/**
Returns substring between specified delimiter strings.

@param string source string to be parsed 
@param st_delim starting delimiter string
@param end_delim ending delimiter
@param include_delim specifies if the delimiters to be 
included in the return string

@return text between delimiters
*/
gchar *text_between_delimiters(const gchar * buffer, gchar st_delim,
			       gchar end_delim, gboolean include_delim)
{
	gchar *st_ptr = NULL;
	gchar *en_ptr = NULL;
	gint textlen = 0;
	gchar *text_bt_delim = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("Error : Input parameter validation\n");
		return NULL;
	}

	st_ptr = (gchar *) buffer;
	en_ptr = (gchar *) buffer;

	st_ptr = strchr(buffer, st_delim);
	if (st_ptr == NULL) {
		return NULL;
	}
	en_ptr = strchr(buffer, end_delim);
	if (en_ptr == NULL) {
		return NULL;
	}

	if (include_delim == FALSE) {
		textlen = strlen(st_ptr) - strlen(en_ptr);
		text_bt_delim = g_strndup(st_ptr + 1, textlen - 1);
	} else {
		textlen = strlen(st_ptr) - strlen(en_ptr) + 1;
		text_bt_delim = g_strndup(st_ptr, textlen);
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return text_bt_delim;
}

/**
Apply gtk tag for the corresponding to the html tag specified.

@param editor EditorInfo representing html editor 
@param start_tag html tag 
@param start_tag_st_off starting html tag starting offset in the text buffer
@param end_tag ending html tag 
@param end_tag_st_off ending html tag starting offset

@return void
*/
void
apply_gtk_tags(EditorInfo * editor, gchar * start_tag,
	       gint start_tag_st_off, gchar * end_tag, gint end_tag_st_off)
{
	GtkTextBuffer *mbuffer = NULL;
	GtkTextIter start_it, end_it;
	gint property = 0;
	GtkTextTag *tag = NULL;
	gchar *style_tag = NULL;
	GtkTextMark *start_mark = NULL, *end_mark = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((editor == NULL) || (start_tag == NULL) || (end_tag == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	mbuffer = gtk_text_view_get_buffer((GtkTextView *) editor->msgFields->body);

	gtk_text_buffer_get_iter_at_offset(mbuffer, &start_it, start_tag_st_off);
	gtk_text_buffer_get_iter_at_offset(mbuffer, &end_it, end_tag_st_off);
	start_mark = gtk_text_buffer_create_mark(mbuffer, NULL, &start_it, FALSE);
	end_mark = gtk_text_buffer_create_mark(mbuffer, NULL, &end_it, FALSE);

	property = find_property_start_htmltag(start_tag);
	switch (property) {
	case TAG_FONT:
	case TAG_FGCOLOR:
		tag = get_gtk_tag_for_html_tag(mbuffer, start_tag);
		gtk_text_buffer_apply_tag(mbuffer, tag, &start_it, &end_it);
		break;
	case TAG_BGCOLOR:
		editor->menu->bgcolorvalue = osso_email_color_parse(start_tag);
		gtk_widget_modify_base((GtkWidget *) editor->msgFields->body,
				       GTK_STATE_NORMAL, &editor->menu->bgcolorvalue);
		break;
	case TAG_BOLD:
		gtk_text_buffer_apply_tag_by_name(mbuffer, tagnames[TAG_BOLD], &start_it, &end_it);
		break;
	case TAG_ITALIC:
		gtk_text_buffer_apply_tag_by_name(mbuffer, tagnames[TAG_ITALIC],
						  &start_it, &end_it);
		break;
	case TAG_UNDERLINE:
		gtk_text_buffer_apply_tag_by_name(mbuffer, tagnames[TAG_UNDERLINE],
						  &start_it, &end_it);
		break;
	case TAG_SUBSCRIPT:
		gtk_text_buffer_apply_tag_by_name(mbuffer, tagnames[TAG_SUBSCRIPT],
						  &start_it, &end_it);
		break;
	case TAG_SUPERSCRIPT:
		gtk_text_buffer_apply_tag_by_name(mbuffer,
						  tagnames[TAG_SUPERSCRIPT], &start_it, &end_it);
		break;
	case TAG_LEFT_ALIGN:
		gtk_text_buffer_apply_tag_by_name(mbuffer, tagnames[TAG_LEFT_ALIGN],
						  &start_it, &end_it);
		break;
	case TAG_CENTER_ALIGN:
		gtk_text_buffer_apply_tag_by_name(mbuffer,
						  tagnames[TAG_CENTER_ALIGN], &start_it, &end_it);
		break;
	case TAG_RIGHT_ALIGN:
		gtk_text_buffer_apply_tag_by_name(mbuffer,
						  tagnames[TAG_RIGHT_ALIGN], &start_it, &end_it);
		break;
	case TAG_STRIKE_THROUGH:
		gtk_text_buffer_apply_tag_by_name(mbuffer,
						  tagnames[TAG_STRIKE_THROUGH], &start_it, &end_it);
		break;
	case TAG_BULLET:
		gtk_text_buffer_apply_tag_by_name(mbuffer, tagnames[TAG_BULLET],
						  &start_it, &end_it);
		break;
	case TAG_NUMBER:
		gtk_text_buffer_apply_tag_by_name(mbuffer, tagnames[TAG_NUMBER],
						  &start_it, &end_it);
		break;
	}
	style_tag = start_tag + 1;
	if (g_ascii_strncasecmp(style_tag, STYLE, STYLE_LEN) == 0) {
		gtk_text_buffer_get_iter_at_offset(mbuffer, &start_it, start_tag_st_off);
		gtk_text_buffer_get_iter_at_offset(mbuffer, &end_it,
						   end_tag_st_off + strlen(end_tag));
		gtk_text_buffer_delete(mbuffer, &start_it, &end_it);
		return;
	}

	gtk_text_buffer_get_iter_at_mark(mbuffer, &start_it, end_mark);
	end_it = start_it;
	gtk_text_iter_forward_chars(&end_it, strlen(end_tag));
	gtk_text_buffer_delete(mbuffer, &start_it, &end_it);

	gtk_text_buffer_get_iter_at_mark(mbuffer, &start_it, start_mark);
	end_it = start_it;
	gtk_text_iter_forward_chars(&end_it, strlen(start_tag));
	gtk_text_buffer_delete(mbuffer, &start_it, &end_it);
	gtk_text_buffer_delete_mark(mbuffer, start_mark);
	gtk_text_buffer_delete_mark(mbuffer, end_mark);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Replace <BR> with newline character.

@param buffer GtkTextBuffer of the text area in html editor
@param start_br_it starting iterator for <BR> tag
@param end_br_it ending iterator for <BR> tag.
*/
void replace_br_tag(GtkTextBuffer * buffer, GtkTextIter * start_br_it, GtkTextIter * end_br_it)
{
	gint offset = 0;
	/*remove <BR> and palce "\n" */
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (start_br_it == NULL) || (end_br_it == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	offset = gtk_text_iter_get_offset(start_br_it);
	gtk_text_buffer_delete(buffer, start_br_it, end_br_it);
	gtk_text_buffer_get_iter_at_offset(buffer, start_br_it, offset);
	gtk_text_buffer_insert(buffer, start_br_it, HTML_NL, -1);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Checks for a substring and returns the offset.

@param source source string
@param searchstring sub stringto be searched for

@return offset of substring occurance
*/
gint get_string_occurance(const gchar * source, const gchar * searchstring)
{
	gchar *starttag_occurence = NULL;
	gint retval = -1;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((source == NULL) || (searchstring == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return retval;
	}

	starttag_occurence = strstr(source, searchstring);

	if (starttag_occurence != NULL) {
		retval = (strlen(source) - strlen(starttag_occurence));
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return retval;
}


/**
prepares closing tag from starting html tag.

@param starting_tag starting html tag

@return closing html tag
*/
gchar *get_closing_tag(const gchar * starting_tag)
{
	gchar *texttoken = NULL;
	gchar *endtoken = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (starting_tag == NULL) {
		ULOG_ERR("Error : Input parameter validation\n");
		return NULL;
	}

	texttoken = text_between_delimiters(starting_tag, HTML_LT_CHAR, HTML_SPC_CHAR, FALSE);
	if (texttoken == NULL) {
		texttoken =
		    text_between_delimiters(starting_tag, HTML_LT_CHAR, HTML_GT_CHAR, FALSE);
	}
	endtoken = g_strconcat(HTML_CLOSE_LT, texttoken, HTML_GT, NULL);
	g_free(texttoken);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return endtoken;
}


/**
This function looks for appropriate image to be inserted into the textbuffer
 and inserts it.
@param img_tag is the html tag for image
@param iter is the position of insertion of image
@param editorinfo is pointer to editorInfo structure

@return
*/
static void insert_inline_image(gchar * img_tag, GtkTextIter iter, EditorInfo * editorinfo)
{
	GSList *imagelist = NULL;
	InlineImageInfo *imageinfo = NULL;
	GtkTextBuffer *buffer = NULL;
	GtkTextIter start, end;
	gint offset = 0;
	gchar *cid = NULL, *filepath = NULL;
	GdkPixbuf *image = NULL;
	GError *error = NULL;
	MimeInfo *mimeinfo = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (img_tag == NULL || editorinfo == NULL) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}
	offset = gtk_text_iter_get_offset(&iter);
	buffer = gtk_text_view_get_buffer((GtkTextView *) editorinfo->msgFields->body);

	/*get the filepath of required image */
	cid = strstr(img_tag, "cid:");
	if (cid == NULL) {
		return;
	}
	cid = cid + 4;
	mimeinfo = ui_dbus_recv_msg_body(editorinfo->msgid, TRUE, TRUE,
					 NULL, NULL, SMIME_NONE, FALSE, 0, NULL, TRUE, TRUE);
	if (mimeinfo != NULL && mimeinfo->inline_images != NULL) {
		for (imagelist = mimeinfo->inline_images; imagelist != NULL;
		     imagelist = imagelist->next) {
			imageinfo = (InlineImageInfo *) imagelist->data;
			if ((imageinfo->content_id != NULL)
			    &&
			    (g_ascii_strncasecmp
			     (cid, imageinfo->content_id, strlen(imageinfo->content_id)) == 0)) {
				filepath = g_strdup(imageinfo->image_path);
				break;
			}
		}
	}
	if (filepath == NULL) {
		gtk_text_buffer_get_iter_at_offset(buffer, &start, offset);
		gtk_text_buffer_get_iter_at_offset(buffer, &end, offset + strlen(img_tag));
		gtk_text_buffer_delete(buffer, &start, &end);
		return;
	}
	image = gdk_pixbuf_new_from_file(filepath, &error);
	if (image == NULL) {
		ULOG_ERR("Unable to load image from file path\n");
		gtk_text_buffer_get_iter_at_offset(buffer, &start, offset);
		gtk_text_buffer_get_iter_at_offset(buffer, &end, offset + strlen(img_tag));
		gtk_text_buffer_delete(buffer, &start, &end);
		return;
	}
	gtk_text_buffer_insert_pixbuf(buffer, &iter, image);
	g_free(filepath);
	offset++;
	gtk_text_buffer_get_iter_at_offset(buffer, &start, offset);
	gtk_text_buffer_get_iter_at_offset(buffer, &end, offset + strlen(img_tag));
	gtk_text_buffer_delete(buffer, &start, &end);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Removes <META ...> tag.

@param buffer GtkTextBuffer of the text area in html editor
@param start_br_it starting iterator for <META ...> tag
@param end_br_it ending iterator for <META ...> tag.
*/
static void remove_meta_data(GtkTextBuffer * buffer, GtkTextIter * start_it, GtkTextIter * end_it)
{
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (start_it == NULL) || (end_it == NULL)) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	gtk_text_buffer_delete(buffer, start_it, end_it);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Process html text and comverts to GTK data. 
First occuring html tag is replaced with corresponding GTK tag
@param editor EditorInfo representing html editor 
@return TRUE if html tag is found
*/
gboolean process_html_text(EditorInfo * editor)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextView *textview = NULL;
	GtkTextIter start_it, end_it;
	GtkTextIter lt_start, lt_end;
	GtkTextIter gt_start, gt_end;
	gboolean returnval = FALSE;

	gchar *starttoken = NULL;
	gchar *endtoken = NULL;

	gint starttag_start_offset = 0;
	gint endtag_start_offset = 0;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (editor == NULL) {
		ULOG_ERR("Error : Input parameter validation\n");
		return returnval;
	}

	textview = (GtkTextView *) editor->msgFields->body;
	buffer = gtk_text_view_get_buffer((GtkTextView *) editor->msgFields->body);
	/*Get buffer boundry iterations */
	gtk_text_buffer_get_bounds(buffer, &start_it, &end_it);

	/*Search for the character "<" for the starting tag */
	returnval =
	    gtk_text_iter_forward_search(&start_it, HTML_LT,
					 GTK_TEXT_SEARCH_VISIBLE_ONLY, &lt_start, &lt_end, &end_it);
	if (returnval == FALSE) {
		return FALSE;
	}
	/*Once the "<" is found, search for the ">" character */
	returnval =
	    gtk_text_iter_forward_search(&lt_end, HTML_GT,
					 GTK_TEXT_SEARCH_VISIBLE_ONLY, &gt_start, &gt_end, NULL);
	if (returnval == FALSE) {
		return FALSE;
	}

	/*Get the text between the "<" and ">" which is the starting tag. */
	starttoken = gtk_text_iter_get_text(&lt_start, &gt_end);

	if (starttoken != NULL) {
		/*tag <BR> does not have the endingtag. so replace it with "\n" character
		   This needs to be done for all tags which do not have closing tags. 
		   Another example is <image> tag. */
		if (g_ascii_strncasecmp(starttoken, HTML_TAG_NL, HTML_TAG_NL_LEN) == 0) {
			/*iter forwarded to cover \n after <BR> */
			gtk_text_iter_forward_char(&gt_end);
			if (gtk_text_iter_get_char(&gt_end) == '\n') {
				gtk_text_iter_forward_char(&gt_end);
			}
			replace_br_tag(buffer, &lt_start, &gt_end);
			g_free(starttoken);
			return TRUE;
		}
		if (strstr(starttoken, IMAGE_TAG) != NULL) {
			insert_inline_image(starttoken, lt_start, editor);
			g_free(starttoken);
			return TRUE;
		}
		if (g_ascii_strncasecmp(starttoken, META_TAG, META_LEN) == 0) {
			remove_meta_data(buffer, &lt_start, &gt_end);
			g_free(starttoken);
			return TRUE;
		}
		if (g_ascii_strncasecmp(starttoken, HREF, strlen(HREF)) == 0) {
			remove_meta_data(buffer, &lt_start, &gt_end);
			g_free(starttoken);
			return TRUE;

		}
		if (g_ascii_strncasecmp(starttoken, HREF_END, strlen(HREF_END)) == 0) {
			remove_meta_data(buffer, &lt_start, &gt_end);
			g_free(starttoken);
			return TRUE;
		}
		/*Get the ending tag corresponding to the starting tag */
		endtoken = get_closing_tag(starttoken);

		/*search for the ending tag in the text buffer. */
		returnval =
		    gtk_text_iter_forward_search(&start_it, (const gchar *) endtoken,
						 GTK_TEXT_SEARCH_VISIBLE_ONLY, &gt_start, &gt_end,
						 NULL);
		if (returnval == FALSE) {
			g_free(starttoken);
			g_free(endtoken);
			ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
			return FALSE;
		}
		/*Get the starting offsets for starttag and endtag */
		starttag_start_offset = gtk_text_iter_get_offset(&lt_start);
		endtag_start_offset = gtk_text_iter_get_offset(&gt_start);

		/*If offsets are valid, apply GTK tag */
		if ((starttag_start_offset >= 0) && (endtag_start_offset >= 0)) {
			apply_gtk_tags(editor, starttoken, starttag_start_offset,
				       endtoken, endtag_start_offset);
		}

		g_free(starttoken);
		g_free(endtoken);
		ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return TRUE;
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return FALSE;
}

/**
Removes the new line char present at the end of msg body. 

@param buffer is the message body buffer
@return void
*/
static void remove_newline_at_end(GtkTextBuffer * buffer)
{
	GtkTextIter start, end;
	gchar *text = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}
	gtk_text_buffer_get_end_iter(buffer, &end);
	start = end;
	gtk_text_iter_backward_char(&start);
	text = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
	if (g_ascii_strcasecmp(HTML_NL, text) == 0) {
		gtk_text_buffer_delete(buffer, &start, &end);
	}
	g_free(text);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Process html text and converts to GTK data. 

@param editor EditorInfo representing html editor 
@param textdata html text data to be converted.
*/
void osso_email_convert_html_to_gtk(EditorInfo * editor, gchar * textdata)
{
	GtkTextIter iter;
	GtkTextBuffer *buffer = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (editor == NULL || textdata == NULL) {
		ULOG_ERR("Error : Input parameter validation\n");
		return;
	}

	buffer = gtk_text_view_get_buffer((GtkTextView *) editor->msgFields->body);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_before_text_insert), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_after_text_insert), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_body_after_text_delete), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer), G_CALLBACK(undodel), editor);
	gtk_text_buffer_set_text(buffer, textdata, -1);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_insert_text), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_delete_range), editor);
	g_signal_handlers_block_by_func(G_OBJECT(buffer),
					G_CALLBACK(on_text_buffer_remove_tag), editor);
	remove_newline_at_end(buffer);
	insert_newlines_in_listitem(buffer);
	while (process_html_text(editor));
	remove_html_delimeters(buffer);

	text_buffer_insert_list_items(buffer, editor);

	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_before_text_insert), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_after_text_insert), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_insert_text), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_body_after_text_delete), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_delete_range), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer),
					  G_CALLBACK(on_text_buffer_remove_tag), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(undoins), editor);
	g_signal_handlers_unblock_by_func(G_OBJECT(buffer), G_CALLBACK(undodel), editor);

	gtk_text_buffer_get_end_iter(buffer, &iter);
	gtk_text_buffer_move_mark_by_name(buffer, "insert", &iter);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Remove html delimiters and replace them with corresponding text data.

@param txt_bt_tags html text data

@return html text data with delimiters replaced.
*/
void remove_html_delimeters(GtkTextBuffer * buffer)
{
	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	gtk_text_buffer_replace_char(buffer, HTML_TAG_LT, HTML_LT);
	gtk_text_buffer_replace_char(buffer, HTML_TAG_GT, HTML_GT);
	gtk_text_buffer_replace_char(buffer, HTML_TAG_NL, HTML_NL);
	gtk_text_buffer_replace_char(buffer, HTML_TAG_SPC, HTML_SPC);
	gtk_text_buffer_replace_char(buffer, HTML_TAG_AMP, HTML_AMP);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
replace the bullets and numbers with the list item tags in html

@param buffer text buffer representing the text
@param start_line starting line number in the buffer
@param end_line ending line number in the buffer
@param tagname name of the tag
*/
void
process_list_items(GtkTextBuffer * buffer, gint start_line, gint end_line, const gchar * tagname)
{
	GtkTextIter start;
	GtkTextIter end;
	gint i = 0;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	for (i = start_line; i <= end_line; i++) {
		gtk_text_buffer_get_iter_at_line(buffer, &start, i);
		gtk_text_buffer_get_iter_at_line(buffer, &end, i);
		if (!gtk_text_iter_ends_line(&end)) {
			gtk_text_iter_forward_to_line_end(&end);
		}
		remove_bullet_or_number(buffer, &start, &end);
		gtk_text_buffer_get_iter_at_line(buffer, &end, i);
		if (!gtk_text_iter_ends_line(&end)) {
			gtk_text_iter_forward_to_line_end(&end);
		}
		gtk_text_buffer_insert_with_tags_by_name(buffer, &end,
							 HTML_CLOSE_TAG_LIST_ITEM,
							 -1, tagname, NULL);

		gtk_text_buffer_get_iter_at_line(buffer, &start, i);
		gtk_text_buffer_insert_with_tags_by_name(buffer, &start,
							 HTML_TAG_LIST_ITEM, -1, tagname, NULL);
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


static void remove_newlines_from_list_blocks(GtkTextBuffer * buffer)
{
	GtkTextIter startiter;
	GtkTextIter enditer;
	GtkTextIter match_start;
	GtkTextIter match_end;
	gchar *text = NULL;
	gint offset = 0;

	gtk_text_buffer_get_bounds(buffer, &startiter, &enditer);
	while (gtk_text_iter_forward_search
	       (&startiter, HTML_CLOSE_TAG_LIST_ITEM,
		GTK_TEXT_SEARCH_VISIBLE_ONLY, &match_start, &match_end, NULL)) {
		offset = gtk_text_iter_get_offset(&match_end);
		match_start = match_end;
		gtk_text_iter_forward_char(&match_end);
		text = gtk_text_buffer_get_text(buffer, &match_start, &match_end, FALSE);
		if (g_ascii_strcasecmp(text, HTML_NL) == 0) {
			gtk_text_buffer_delete(buffer, &match_start, &match_end);
		}
		g_free(text);
		gtk_text_buffer_get_iter_at_offset(buffer, &startiter, offset);
	}
}

/**
Replace the bullets and numbers with the list item tags present in the buffer 

@param buffer text buffer representing the text
*/
void text_buffer_replace_list_items(GtkTextBuffer * buffer)
{
	GtkTextTag *tag_bullet = NULL;
	GtkTextTag *tag_number = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	tag_bullet =
	    gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(buffer), tagnames[TAG_BULLET]);
	tag_number =
	    gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(buffer), tagnames[TAG_NUMBER]);

	update_bullet_or_number_list(buffer, tag_bullet);
	update_bullet_or_number_list(buffer, tag_number);
	remove_newlines_from_list_blocks(buffer);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Replace the bullets and numbers with the list item tags present in the buffer 

@param buffer text buffer representing the text
@param tag tag for bullets or numbeting
*/
void update_bullet_or_number_list(GtkTextBuffer * buffer, GtkTextTag * tag)
{
	GtkTextIter startiter;
	GtkTextIter enditer;
	GtkTextIter start;
	GtkTextIter end;
	gboolean tagfound = TRUE;
	gint start_lineno = 0;
	gint end_lineno = 0;
	GtkTextMark *mark = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (tag == NULL)) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	gtk_text_buffer_get_bounds(buffer, &startiter, &enditer);
	gtk_text_iter_backward_char(&enditer);
	mark = gtk_text_buffer_create_mark(buffer, NULL, &startiter, FALSE);

	while (tagfound) {
		tagfound = gtk_text_iter_has_tag(&startiter, tag);
		if (tagfound == FALSE) {
			tagfound = gtk_text_iter_forward_to_tag_toggle(&startiter, tag);
		}
		if (tagfound == FALSE) {
			break;
		}

		enditer = startiter;
		tagfound = gtk_text_iter_forward_to_tag_toggle(&enditer, tag);

		start_lineno = gtk_text_iter_get_line(&startiter);
		gtk_text_iter_backward_char(&enditer);
		end_lineno = gtk_text_iter_get_line(&enditer);
		gtk_text_buffer_move_mark(buffer, mark, &enditer);
		if (tag->name != NULL) {
			process_list_items(buffer, start_lineno, end_lineno, tag->name);
		}
		gtk_text_buffer_get_bounds(buffer, &start, &end);
		gtk_text_buffer_get_iter_at_mark(buffer, &enditer, mark);
		end_lineno = gtk_text_iter_get_line(&enditer);

		if (end_lineno >= gtk_text_iter_get_line(&end)) {
			break;
		}
		gtk_text_buffer_get_iter_at_line(buffer, &startiter, end_lineno + 1);
	}
	gtk_text_buffer_delete_mark(buffer, mark);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Insert the bullets or numbers in the buffer depending upon the tag

@param buffer text buffer representing the text
@param tag tag for bullets or numbeting
*/
void insert_bullet_or_number_list(GtkTextBuffer * buffer, GtkTextTag * tag, EditorInfo * editorinfo)
{

	GtkTextIter startiter;
	GtkTextIter enditer;
	gboolean tagfound = TRUE;
	gint start_lineno = 0;
	gint end_lineno = 0;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL) || (tag == NULL)) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	gtk_text_buffer_get_bounds(buffer, &startiter, &enditer);
	gtk_text_iter_backward_char(&enditer);

	while (tagfound) {
		tagfound = gtk_text_iter_has_tag(&startiter, tag);
		if (tagfound == FALSE) {
			tagfound = gtk_text_iter_forward_to_tag_toggle(&startiter, tag);
		}
		if (tagfound == FALSE) {
			break;
		}

		gtk_text_buffer_get_iter_at_offset(buffer, &enditer,
						   gtk_text_iter_get_offset(&startiter));
		tagfound = gtk_text_iter_forward_to_tag_toggle(&enditer, tag);
		start_lineno = gtk_text_iter_get_line(&startiter);
		end_lineno = gtk_text_iter_get_line(&enditer);
		if (tag->name != NULL) {
			set_list_items(buffer, &startiter, &enditer, tag, editorinfo);
		}
		gtk_text_buffer_get_bounds(buffer, &startiter, &enditer);

		if (end_lineno >= gtk_text_iter_get_line(&enditer)) {
			break;
		}
		gtk_text_buffer_get_iter_at_line(buffer, &startiter, end_lineno + 1);
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}



/**
Insert the bullets and numbers with the list item tags present in the buffer 

@param buffer text buffer representing the text
*/
void text_buffer_insert_list_items(GtkTextBuffer * buffer, EditorInfo * editorinfo)
{
	GtkTextTag *tag_bullet = NULL;
	GtkTextTag *tag_number = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}

	tag_bullet =
	    gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(buffer), tagnames[TAG_BULLET]);
	tag_number =
	    gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(buffer), tagnames[TAG_NUMBER]);

	insert_bullet_or_number_list(buffer, tag_bullet, editorinfo);
	insert_bullet_or_number_list(buffer, tag_number, editorinfo);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/** This function is called to insert new lines in the block of text having 
    list item tag.
@param buffer is the body field buffer.

@return void
*/
void insert_newlines_in_listitem(GtkTextBuffer * buffer)
{
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	GtkTextIter match_start;
	GtkTextIter match_end;
	gint offset = 0;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if ((buffer == NULL)) {
		ULOG_ERR("Error validating input parameters\n");
		return;
	}

	gtk_text_buffer_get_start_iter(buffer, &start_iter);
	while (gtk_text_iter_forward_search
	       (&start_iter, HTML_CLOSE_TAG_LIST_ITEM,
		GTK_TEXT_SEARCH_VISIBLE_ONLY, &match_start, &match_end, NULL)) {
		offset = gtk_text_iter_get_offset(&match_end);
		gtk_text_buffer_insert(buffer, &match_end, HTML_NL, -1);
		gtk_text_buffer_get_iter_at_offset(buffer, &start_iter, offset);
	}

	gtk_text_buffer_get_end_iter(buffer, &end_iter);
	/*remove new line before </ul> and insert after it */
	while (gtk_text_iter_backward_search
	       (&end_iter, HTML_CLOSE_TAG_BULLET,
		GTK_TEXT_SEARCH_VISIBLE_ONLY, &match_start, &match_end, NULL)) {
		offset = gtk_text_iter_get_offset(&match_end);
		gtk_text_buffer_insert(buffer, &match_end, HTML_NL, -1);
		gtk_text_buffer_get_iter_at_offset(buffer, &match_start, offset);
		if (gtk_text_iter_backward_search(&match_start, HTML_NL,
						  GTK_TEXT_SEARCH_VISIBLE_ONLY, &match_start,
						  &match_end, NULL)) {
			offset = gtk_text_iter_get_offset(&match_start);
			gtk_text_buffer_delete(buffer, &match_start, &match_end);
			gtk_text_buffer_get_iter_at_offset(buffer, &end_iter, offset);
		}
	}
	/*remove new line before </ol> and insert after it */
	gtk_text_buffer_get_end_iter(buffer, &end_iter);
	while (gtk_text_iter_backward_search
	       (&end_iter, HTML_CLOSE_TAG_NUMBER,
		GTK_TEXT_SEARCH_VISIBLE_ONLY, &match_start, &match_end, NULL)) {
		offset = gtk_text_iter_get_offset(&match_end);
		gtk_text_buffer_insert(buffer, &match_end, HTML_NL, -1);
		gtk_text_buffer_get_iter_at_offset(buffer, &match_start, offset);
		if (gtk_text_iter_backward_search(&match_start, HTML_NL,
						  GTK_TEXT_SEARCH_VISIBLE_ONLY, &match_start,
						  &match_end, NULL)) {
			offset = gtk_text_iter_get_offset(&match_start);
			gtk_text_buffer_delete(buffer, &match_start, &match_end);
			gtk_text_buffer_get_iter_at_offset(buffer, &end_iter, offset);
		}
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
Callback for searching a image inserted in text buffer 

@param ch character at which the search is
@param user_data 

@return TRUE id the character is found
*/
gboolean predicate(gunichar ch, gpointer user_data)
{
	//character 0xFFFC is for child widgets and pixbufs in the text buffer
	return ch == 0xFFFC;
}

/**
Prepares HTML tag for the image located

@param filepath File path representing the image
@param msginfo is pointer to MsgInfo structure

@return html tag data representing the image
 */
gchar *get_html_tag_for_image_path(GdkPixbuf * pixbuf, const gchar * filepath, MsgInfo * msginfo)
{
	gint height = 0;
	gint width = 0;
	gchar *retval = NULL;
	GRand *num_gen = NULL;
	guint32 num = 0;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (filepath == NULL || msginfo == NULL) {
		ULOG_ERR("error validating input parameters\n");
		return NULL;
	}

	num_gen = g_rand_new();
	num = g_rand_int(num_gen);
	height = gdk_pixbuf_get_height(pixbuf);
	width = gdk_pixbuf_get_width(pixbuf);
	retval = g_strdup_printf("<IMG src=\"cid:%u\" height=%d width=%d>", num, height, width);

	set_inline_img_list(filepath, msginfo, num);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return retval;
}

/**
Prepares HTML tag for the image located
@param filepath File path representing the image
@param msginfo is pointer to MsgInfo structure
@param num random no for cid

@return void
*/
static void set_inline_img_list(const gchar * filepath, MsgInfo * msginfo, guint32 num)
{
	InlineImageInfo *imageinfo = NULL;
	gchar *name = NULL;
	gchar *imagename = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (filepath == NULL || msginfo == NULL) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}
	imageinfo = g_new0(InlineImageInfo, 1);
	imageinfo->content_id = g_strdup_printf("<%u>", num);
	imagename = (gchar *) strrchr(filepath, GNOME_VFS_URI_PATH_CHR);
	if (imagename == NULL) {
		g_free(imageinfo->content_id);
		g_free(imageinfo);
		return;
	}
	name = g_strdup(imagename + 1);
	imageinfo->uri_path = g_strdup(filepath);
	imageinfo->image_path = g_strdup(filepath);
	imageinfo->image_name = g_strdup(name);
	msginfo->image_list = g_slist_append(msginfo->image_list, imageinfo);

	g_free(name);

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function creates image tag if image is present at 
  starting of message

@param buffer text buffer representing the text
@param editinfo is pointer to editorInfo structure
@param imgnum is pointyer to image no.

@return void
*/
static void image_insertion_at_start(GtkTextBuffer * buffer, EditorInfo * editinfo, gint * imgnum)
{
	GdkPixbuf *pixbuf = NULL;
	GtkTextIter iter;
	gchar *filepath = NULL, *temp_path = NULL;
	gchar *htmltag = NULL;
	gunichar ch;
	GError *error = NULL;


	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL || editinfo == NULL) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}
	gtk_text_buffer_get_start_iter(buffer, &iter);
	ch = gtk_text_iter_get_char(&iter);
	if (ch == 0xFFFC) {
		pixbuf = gtk_text_iter_get_pixbuf(&iter);
		if (pixbuf != NULL) {
			temp_path = ui_get_tmp_path();
			filepath =
			    g_strdup_printf("%s%s%s%d", temp_path, GNOME_VFS_URI_PATH_STR, IMAGE_,
					    (*imgnum)++);;
			g_free(temp_path);
			gdk_pixbuf_save(pixbuf, filepath, JPEG_IMAGE, &error, NULL);

			htmltag = get_html_tag_for_image_path(pixbuf, filepath, editinfo->msgInfo);

			if (htmltag != NULL) {
				gtk_text_iter_forward_char(&iter);
				gtk_text_buffer_insert(buffer, &iter, htmltag, -1);
				g_free(htmltag);
				htmltag = NULL;
			}
			g_free(filepath);
		}
		pixbuf = NULL;
	}

	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
Process the text buffer for inserted images

@param buffer text buffer representing the text
@param editinfo is pointer to editorInfo structure

@return void
*/
void text_buffer_process_images(GtkTextBuffer * buffer, EditorInfo * editinfo)
{
	GdkPixbuf *pixbuf = NULL;
	GtkTextIter start;
	GtkTextIter end;
	gint offset = 0;
	gchar *filepath = NULL, *temp_path = NULL;
	gchar *htmltag = NULL;
	gint imgnum = 0;
	GError *error = NULL;

	ULOG_DEBUG_L("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL || editinfo == NULL) {
		ULOG_ERR("error validating input parameters\n");
		return;
	}
	image_insertion_at_start(buffer, editinfo, &imgnum);
	gtk_text_buffer_get_bounds(buffer, &start, &end);
	while (gtk_text_iter_forward_find_char(&start, predicate, NULL, &end)) {
		offset = gtk_text_iter_get_offset(&start);
		pixbuf = gtk_text_iter_get_pixbuf(&start);
		if (pixbuf == NULL) {
			gtk_text_iter_forward_char(&start);
			continue;
		} else {
			temp_path = ui_get_tmp_path();
			filepath =
			    g_strdup_printf("%s%s%s%d", temp_path, GNOME_VFS_URI_PATH_STR, IMAGE_,
					    (imgnum)++);;
			g_free(temp_path);
			gdk_pixbuf_save(pixbuf, filepath, JPEG_IMAGE, &error, NULL);

			htmltag = get_html_tag_for_image_path(pixbuf, filepath, editinfo->msgInfo);
			if (htmltag != NULL) {
				gtk_text_buffer_insert(buffer, &start, htmltag, -1);
				offset = offset + strlen(htmltag);
				g_free(htmltag);
				htmltag = NULL;
				gtk_text_buffer_get_iter_at_offset(buffer, &start, offset);
			}
			g_free(filepath);
		}
		pixbuf = NULL;
	}
	ULOG_DEBUG_L("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}
