/**
 * @file hyperlink_handler.c
 *
 * This file contains all for functions hyper link support and showing
 * plain mails in browser.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/

/*Has the standard header files*/
#include "ui_common.h"

#include "accountMgmt_callback.h"
#include "hyperlink_handler.h"
#include "html_parser.h"
#include <regex.h>


gint getline(gchar ** tmp_ptr, gint * len, FILE * fp1);

/**
 Helper function to check if the filepath send , has some link.
 for this the if the protocol is file:// or obex:// we send the attachment.
   else we treat it as link.

 @param filepath is the const gchar pinter to the filepath.
 @return TRUE--> is it has a link
         FALSE--> else file paths starting with file:// or obex://.
*/
gboolean check_for_links(const gchar * filepath)
{
	GnomeVFSURI *uri = NULL;
	gboolean write_to_file = TRUE;


	uri = gnome_vfs_uri_new(filepath);
	if (uri != NULL) {
		if (gnome_vfs_uri_is_local(uri) == TRUE)
			write_to_file = FALSE;
		gnome_vfs_uri_unref(uri);
	}
	if ((g_ascii_strncasecmp(filepath, OBEX_PROTO, strlen(OBEX_PROTO)) == 0))
		write_to_file = FALSE;

	return write_to_file;
}

/**
 gets the path for the .html file to be created.
 
 @param filename is a gchar representing the body file path.
 	caller has to free this param.
 
 @return a gchar a temporary html file path.caller has to free this param. 
           NULL-on error. 
*/
static gchar *get_file_path(const gchar * filename)
{
	gchar *tmp_path = NULL;
	gchar *filepath = NULL;
	static gint num = 0;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (filename == NULL) {
		ULOG_DEBUG("[%s][%s] : input param check failed :\n", __FILE__, __FUNCTION__);
		return NULL;
	}

	tmp_path = ui_get_tmp_path();
	filepath = g_strdup_printf("%s%s%d", tmp_path, TEMP_PLAIN_FILE, num++);
	g_free(tmp_path);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return filepath;
}

/**
This routine will search for any URL in the given string and
        add hyper link to them.When the formated string is opend
        in the browser the DNL facility will be invoked.
        it adds the <href:/> html tag.

 @param  subject  The unformated source string which contains url strings.
 
 @return gchar   The formated string which contains all hper links to the existing URL's.
                caller has to free this param.
         NULL-on error(input param check fail..etc)
*/
gchar *ui_viewer_html_parser(const gchar * subject)
{
	GString *html_string = NULL;

	html_string = g_string_new(subject);
	hyperlinkify_plain_text(html_string);

	return g_string_free(html_string, FALSE);

}

/**
 This function is used for applying the appropriate html <href>
 tag based on whether it is a email id /link with "www"/normal url.
 
 @param line_txt is the pointer to the GString in the text appened with 
	 the html tag is present.
 @param url is the input in which there is a emailid/url.
 @return void.
*/
static void prepare_hyperlink_tag_text(GString * line_txt, const gchar * url)
{
	/* a www-thing without http:// */
	if (g_ascii_strncasecmp(url, "www", strlen("www")) == 0)
		g_string_append_printf(line_txt, "<a href=\"http://%s\">%s</a>", url, url);

	/* normal url */
	else if (g_strstr_len(url, strlen(url), ":") != NULL)
		g_string_append_printf(line_txt, "<a href=\"%s\">%s</a>", url, url);

	/*  an email address (without mailto:) */
	else if (g_ascii_strncasecmp(url, MAILTO, strlen(MAILTO)) != 0)
		g_string_append_printf(line_txt, "<a href=\"mailto:%s\">%s</a>", url, url);

	return;
}

/**
 This function is used add the charset info for the file containing
 plain text.
 @param filename is gchar pointer to the path of the plain file.
 @param mime_info is the pointer to the MimeInfo structure pointer
  for determining the charset.
 @return gchar a temporary file with the charset info.
*/
gchar *add_charset_info_to_file(const gchar * filename, const MimeInfo * mime_info)
{
	FILE *fp_write = NULL;
	gchar *body_file = NULL;
	gchar *charset = NULL;
	//const gchar *font_name = NULL;
	const MimeInfo *mimeinfo = NULL;
	gchar *str = NULL;
	GString *html_string = NULL;
	GError *err = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Create a new html file */
	body_file = get_file_path(filename);
	fp_write = fopen(body_file, "w+b");
	if (fp_write == NULL) {
		ULOG_CRIT_F("unable to open the file for writing");
		g_free(body_file);
		return NULL;
	}

	/*get the charset of the mail */
	if (mime_info->content_type != NULL &&
	    g_ascii_strcasecmp(mime_info->content_type, MULTIPART_MIXED) == 0) {
		mimeinfo = mime_info->children;
	} else {
		mimeinfo = mime_info;
	}

	if (mimeinfo->charset == NULL) {
		charset = "US-ASCII";
	} else {
		charset = mimeinfo->charset;
	}
	/*TODO: Font info also need to be added to the file.
	   Current MailViewer widget does not support this */
	/*Get the default font for plain mail */
	//font_name = ui_get_default_font(FALSE);

	html_string = g_string_new(NULL);
	/*Getting the file contents */
	g_file_get_contents(filename, &(html_string->str), NULL, &err);
	if (err == NULL) {
		if (html_string->str != NULL) {
			/*append the charset info to the start of the body */
			str = g_strdup_printf("Content-Type: text/plain; charset=\"%s\" \n\n",
					      charset);
			html_string->len = strlen(html_string->str);
			html_string = g_string_insert(html_string, 0, str);
			g_free(str);
			if (fwrite(html_string->str, sizeof(gchar), html_string->len, fp_write)
			    != html_string->len) {
				osso_log(LOG_ERR, "unable writing in file");
				g_free(body_file);
				body_file = NULL;
			}

		}
	} else {
		osso_log(LOG_DEBUG, "Error reding file contents %s\n", err->message);
		g_error_free(err);
		g_free(body_file);
		body_file = NULL;
	}

	g_string_free(html_string, TRUE);
	fclose(fp_write);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return (body_file);
}


/**
 This is a helper function that prepares a html file content
 	for sending the url as attachment by appending the tags
  ex:<html><body>"<a href=\""url"></a></body></html>
 
 @param url is the gchar representing the url.
 	caller has to free this param.
 
 @return gchar which will be the content of the file.caller has to free this param
           NULL-on error
*/
static gchar *prepare_html_file(const gchar * url)
{
	gchar *file_content = NULL;

	/*the html tags are appened to the url */
	file_content =
	    g_strconcat(TAG_HTML_START, TAG_BODY_START, HREF, url, HREF_SEP, url,
			HREF_END, TAG_BODY_END, TAG_HTML_END, NULL);

	return file_content;
}

/**
 This function is used to prepare html files for urls containing
	http:/https: protocols & that are send using SEND AS email.
	
 @param url is the url path send from a application to send as email,
	caller has to free this param.
 
 @return a gchar pointer to the file.caller has to free this param
           NULL-on error
*/
gchar *prepare_files_for_urls(const gchar * url)
{
	gchar *url_name = NULL, *name = NULL;
	gchar *file_path = NULL;
	static gint randnum = 0;
	gchar *file_content = NULL;
	GnomeVFSResult result = GNOME_VFS_OK;
	GnomeVFSHandle *write_handle = NULL;
	GnomeVFSFileSize bytes_written = 0;
	gchar **split_url = NULL, *tmp_attach_dir = NULL, *tmp_attach_name = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);


	url_name = strrchr(url, GNOME_VFS_URI_PATH_CHR);
	if (url_name == NULL) {
		return NULL;
	}
	/*checking if audio file name exists */
	name = g_strndup(url, strlen(url) - strlen(url_name) + 1);
	tmp_attach_dir = ui_get_tmp_attach_dir();
	tmp_attach_name = ui_get_tmp_attach_name();
	if (name != NULL && check_for_links(name) == TRUE) {
		file_path =
		    g_strdup_printf("%s%s %d%s", tmp_attach_dir, tmp_attach_name, randnum++,
				    HTML_EXT);
	}
	/*if file name exits */
	else {
		split_url = g_strsplit(url_name, EXT_SEP, -1);
		if (split_url != NULL && split_url[0] != NULL && split_url[1] != NULL) {
			file_path =
			    g_strdup_printf("%s%s%d%s%s", tmp_attach_dir, split_url[0], randnum++,
					    split_url[1], HTML_EXT);
		} else {
			file_path =
			    g_strdup_printf("%s%d%s%s", tmp_attach_dir, randnum++, url_name,
					    HTML_EXT);
		}
		if (split_url != NULL)
			g_strfreev(split_url);
	}
	g_free(tmp_attach_dir);
	g_free(tmp_attach_name);

	file_content = prepare_html_file(url);
	result =
	    gnome_vfs_create(&write_handle, file_path, GNOME_VFS_OPEN_WRITE, FALSE, READ_WRITE);
	if (result == GNOME_VFS_OK) {
		result =
		    gnome_vfs_write(write_handle, file_content, strlen(file_content),
				    &bytes_written);
		if (result != GNOME_VFS_OK || (bytes_written != strlen(file_content))) {
			ULOG_CRIT_F("error writing to the file");
			g_free(file_path);
			file_path = NULL;
		}
		result = gnome_vfs_close(write_handle);
		if (result != GNOME_VFS_OK)
			ULOG_INFO_F("error closing file handle ");

	} else {
		ULOG_CRIT_F("error creating file");
		g_free(file_path);
		file_path = NULL;
	}
	g_free(file_content);
	g_free(name);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return file_path;
}

/**
 This function scans the entire input string for any url/emailid 
 regular expression matches.It stores the match start offset & the length in 
 structure.
 @param txt is the GString pointer to the text to be scanned for urls.
 @return a GSList containing the pointers to the structure url_match_t
                  which has the match start offset & length of the string as 
		  it elements.
*/
static GSList *get_url_matches(GString * txt)
{

	regex_t rx;
	regmatch_t rm;
	int rv = 0;
	int offset = 0;
	GSList *match_list = NULL;

	if (regcomp(&rx, URL_REGEXP, REG_ICASE | REG_EXTENDED | REG_NEWLINE) != 0) {
		g_warning("error in regexp: %s", URL_REGEXP);
		return NULL;
	}

	/* find all the matches */
	while (1) {
		if ((rv = regexec(&rx, txt->str + offset, 1, &rm, 0)) != 0) {
			if (rv != REG_NOMATCH) {
				g_warning("error executing rx\n");
				regfree(&rx);
				return NULL;
			} else
				break;
		}

		/* make a list of our matches (<offset, len> tupels) */
		if (rm.rm_so != -1 && strlen(txt->str + offset) != 0) {
			url_match_t *match = g_new(url_match_t, 1);
			match->offset = offset + rm.rm_so;
			match->len = rm.rm_eo - rm.rm_so;
			match_list = g_slist_prepend(match_list, match);
			offset += rm.rm_eo;
		} else
			break;
	}
	regfree(&rx);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return match_list;
}

#if 0
void hyperlinkify_plain_text(GString * txt)
{
	GSList *cursor, *match_list = get_url_matches(txt);
	gchar *url = NULL;
	GString *replace_str = NULL;	/* replacement  */

	txt->len = strlen(txt->str);
	/* we will work backwards, so the offsets stay valid */
	for (cursor = match_list; cursor; cursor = cursor->next) {
		replace_str = g_string_new(NULL);

		url_match_t *match = (url_match_t *) cursor->data;
		url = g_strndup(txt->str + match->offset, match->len);

		prepare_hyperlink_tag_text(replace_str, url);

		/* replace the old thing with our hyperlink
		 * replacement thing */
		txt = g_string_erase(txt, match->offset, match->len);
		txt->len = strlen(txt->str);
		txt = g_string_insert(txt, match->offset, replace_str->str);
		txt->len = strlen(txt->str);

		g_free(url);
		g_string_free(replace_str, TRUE);
	}
	g_slist_foreach(match_list, (GFunc) g_free, NULL);
	g_slist_free(match_list);

	return;
}
#endif
/*end of file*/
