/**
 * @file imap_cb.c
 *
 * This file contains IMAP4 module related callback functions for UI.
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#include <stdlib.h>
#include <glib.h>
#include "common.h"
#include "prefs_account.h"
#include "procmsg.h"
#include "al_callbacks.h"
#include "imap.h"

/*UI header files*/
#include "ui_al_callbacks.h"
#include "editview.h"
#include "ui_interface.h"
#include "libosso.h"
#include "accountMgmt_callback.h"
#include "folder_mgmt.h"
#include "message_handler.h"

/* Global variables*/
extern MainViewInfo *email_main_view;	/* Declared in ui_interface.c */

void imap4_param_value_missing_cb(ParamMissing param_number);
void imap4_msg_headers_cb(GSList * new_list);
void imap4_msg_attachment_headers_cb(GSList * new_list);

Imap4Callback *cb;

/**
   This function is called to inform the UI module about error occured
   in imap Module

   @param param_number is the error occured in imap module  
   @return none
*/

void imap4_param_value_missing_cb(ParamMissing param_number)
{

	//Call UI callback function
	ui_param_value_missing_cb(param_number);
	return;
}

/**
   This function is called to update the UI module about the imap4 message 
   headers.

   @param new_list List of imap message headers 
   @return none
*/

void imap4_msg_headers_cb(GSList * new_list)
{
	//Call UI callback function
	ui_imap_msg_headers_cb(new_list);
	return;
}

/**
   This function is called to update the UI module about the imap4 message 
   attachments headers.

   @param new_list List of imap message headers 
   @return none
*/

void imap4_msg_attachment_headers_cb(GSList * new_list)
{
	//Call UI callback function
	ui_recv_msg_attachments(new_list);
	return;
}

/**
   This function is called to update the UI module about the imap4 
   module status/error code and number of mails received
   headers.

   @param msg_number contains the status/error code
   @param index contains the length of GArray arguments array
   @param arguments GArray of osso_rpc_t elements  which contains the 
    additional information
   @return none 
*/

void imap4_set_msg_cb(gint msg_number, gint index, GArray * arguments)
{
	gint i = 0;
	gint recvcount = -1, totalcount = -1, percent_completed = -1;
	gint msg_fully_downloaded = -1, msg_body_downloaded = -1;
	gchar *error_info1 = NULL, *error_info2 = NULL, *account_name = NULL, *msgid = NULL;

	ULOG_DEBUG_F(" imap4_set_msg_cb = %d Index = %d \n", msg_number, index);
	for (i = 0; i < index; i++) {
		switch (g_array_index(arguments, osso_rpc_t, i).type) {
		case DBUS_TYPE_INT32:
			ULOG_DEBUG_F("type %d INT  value = %d \n", DBUS_TYPE_INT32,
				     g_array_index(arguments, osso_rpc_t, i).value.i);
			break;
		case DBUS_TYPE_STRING:
			ULOG_DEBUG_F(" +++ type %d STR  value = %s \n", DBUS_TYPE_STRING,
				     g_array_index(arguments, osso_rpc_t, i).value.s);
			break;
		default:
			ULOG_DEBUG_F(" Type = %d \n", g_array_index(arguments, osso_rpc_t, i).type);
		}
	}

	switch (msg_number) {
	case IMAP_START:
		{
			if (index != 2) {
				ui_update_progressbar_text(CONNECTING, NULL);
			}
			if (index >= 1) {
				g_free(email_main_view->account_manager->current_sendrec_account);
				email_main_view->account_manager->current_sendrec_account =
				    g_strdup(g_array_index(arguments, osso_rpc_t, 0).value.s);
				gtk_widget_set_sensitive(email_main_view->menu->email_send_receive,
							 FALSE);
				gtk_widget_set_sensitive(email_main_view->tools->send_receive,
							 FALSE);
				/*Unset state_save_disabled flag */
				email_main_view->tags->state_save_disabled = TRUE;
			}
			if (index == 2) {
				msgid = g_strdup(g_array_index(arguments, osso_rpc_t, 1).value.s);
				ui_update_progressbar_text(CONNECTING, msgid);
				g_free(msgid);
			}

			break;
		}
	case IMAP_PROGRESS:
		{
			//Update progress bar numbers
			recvcount = g_array_index(arguments, osso_rpc_t, 0).value.i;
			totalcount = g_array_index(arguments, osso_rpc_t, 1).value.i;
			ui_update_progressbar_text(RECEIVING, NULL);
			if (recvcount > 0 && totalcount > 0) {
				ui_update_progressbar_numbers(recvcount, totalcount, RECEIVING);
				percent_completed = (recvcount * 100) / totalcount;
				ui_set_percentage_cb(percent_completed);
			}
			break;
		}
	case IMAP_COMPLETE:
		{
			//Close the progress bar
			ui_send_recv_complete_cb();
			email_main_view->tags->msg_download_in_progress = FALSE;

			/* Edit Account operation stopped Send/Receive */
			if (email_main_view->account_manager->operation_status == EDIT_OP) {
				ui_resume_edit_accout_oper();
			}
			/* Action after deletion of email account  stopped Send/Receive */
			else if (email_main_view->account_manager->operation_status == DELETE_OP) {
				ui_resume_delete_account_oper();
			} else {
				g_free(email_main_view->account_manager->current_sendrec_account);
				email_main_view->account_manager->current_sendrec_account = NULL;
			}
			/*Reset state_save_disabled flag */
			email_main_view->tags->state_save_disabled = FALSE;

			if (email_main_view->account_manager->configured_accounts != NULL) {
				gtk_widget_set_sensitive(email_main_view->menu->email_send_receive,
							 TRUE);
				gtk_widget_set_sensitive(email_main_view->tools->send_receive,
							 TRUE);
			}
			break;
		}
	case IMAP_AUTHFAIL:
	case IMAP_CONNECT_SERVER_ERROR:
		{
			error_info1 = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
			error_info2 = (gchar *) g_array_index(arguments, osso_rpc_t, 1).value.s;

			/*Check if the said account is already being edited/deleted.
			   If so do nothing. */
			if (email_main_view->account_manager->operation_status != NONE_OP) {
				if (error_info2 != NULL
				    && email_main_view->account_manager->current_sendrec_account !=
				    NULL
				    && strcmp(error_info2,
					      email_main_view->account_manager->
					      current_sendrec_account) == 0) {
					ULOG_DEBUG_F("Account is already being edited/deleted\n");
					break;
				}
			}

			ui_set_error_msg_cb(msg_number, error_info1, error_info2);
			break;
		}
	case IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER:
	case IMAP_MSG_FETCH_ERROR:
		{
			error_info1 = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
			ui_set_error_msg_cb(msg_number, error_info1, NULL);
			break;
		}
	case IMAP_MSG_DOWNLOAD:
		{
			msg_fully_downloaded = g_array_index(arguments, osso_rpc_t, 0).value.i;
			msg_body_downloaded = g_array_index(arguments, osso_rpc_t, 1).value.i;
			error_info1 = (gchar *) g_array_index(arguments, osso_rpc_t, 2).value.s;
			ui_update_msg_downloaded_cb(msg_fully_downloaded,
						    msg_body_downloaded, -1, error_info1);
			break;
		}
	case IMAP_SERVER_SYNC_ERROR:
		{
			if (index > 0) {
				error_info1 =
				    (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
			}
			ui_handle_sync_error_cb(error_info1, TRUE);
			break;
		}
	case IMAP_CONNECT_SERVER_SUCCESS:
		{
			if (index == 2) {
				msgid = g_strdup(g_array_index(arguments, osso_rpc_t, 1).value.s);
				ui_update_progressbar_text(RECEIVING, msgid);
				g_free(msgid);
			}
			break;
		}
	case IMAP_PASSWD_FAIL:
		{
			account_name = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
			ui_on_mailbox_protected_dialog(account_name, FALSE, FALSE);
			break;
		}
	case IMAP_PASSWD_INCORRECT:
		{
			account_name = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
			ui_on_mailbox_protected_dialog(account_name, FALSE, TRUE);
			break;
		}
	default:
		{
			//Show engine error
			ui_set_msg_cb(msg_number);
			break;
		}
	}

	return;
}
