/**
 * @file htcnllback.h
 *
 * This file contains the header information for htcallbacks
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */

#ifndef _HTML_CALLBACKS_H_
#define _HTML_CALLBACKS_H_

#include <gtk/gtk.h>
#include "editcallback.h"

#define BULLET_CHAR "\342\200\242	"
#define BULLET_LENGTH 2
#define NUMBER_START "1.	"
#define NUMBER_APPEND ".	"

#define DEFAULT_FONT_SIZE 12
#define FONT_SIZE_STRING "12"
gdouble font_scaling_factor;
#define FONT_SCALING_FACTOR 150/100
#define ZOOM_IN_LEVEL 200
#define ZOOM_IN_MID_LEVEL 150
#define ZOOM_NORMAL_LEVEL 100
#define ZOOM_OUT_LEVEL 50
#define MYDOCS_IMAGE "/MyDocs/.images"
#define IMAGE_JPEG "image/jpeg"
#define IMAGE_PNG "image/png"
#define IMAGE_GIF "image/gif"
#define IMAGE_TIFF "image/tiff"
#define IMAGE_BMP "image/bmp"
#define IMAGE_SKETCH_PNG "sketch/png"


//This is the first part where the IDs are assigned to each supported tags.
//Any adition or removal of supported teags needs to be started from here.
enum TagType {
	TAG_BOLD = 0,
	TAG_ITALIC = 1,
	TAG_UNDERLINE = 2,
	TAG_STRIKE_THROUGH = 3,
	TAG_SUPERSCRIPT = 4,
	TAG_SUBSCRIPT = 5,
	TAG_LEFT_ALIGN = 6,
	TAG_CENTER_ALIGN = 7,
	TAG_RIGHT_ALIGN = 8,
	TAG_BULLET = 9,
	TAG_NUMBER = 10,
	TAG_IMAGE = 12,
	TAG_FONT = 13,
	TAG_FGCOLOR = 14,
	TAG_BGCOLOR = 15,
	TAG_COUNT = 16
};

typedef enum {
	UNSET,
	SET,
	INCONSISTENT
} setting_flag;

typedef struct {
	gchar *family;
	setting_flag family_flag;

	gint size;
	setting_flag size_flag;

	GdkColor color;
	setting_flag color_flag;

	gint weight;
	setting_flag weight_flag;

	gint style;
	setting_flag style_flag;

	gint underline;
	setting_flag underline_flag;

	gboolean strikethrough;
	setting_flag strikethrough_flag;

	gint rise;
	setting_flag rise_flag;
} settings;


/**
This will give a console print of attributes variables.
@param attribute is a pointer to the GtkTextAttributes structure.
@return void. 
*/
void print_text_attributes(GtkTextAttributes * attribute);

void populate_fonts(EditorInfo * editorInfo);

/**
Call the function when ever the current text attribute change.
This is mainly called to refresh toolbar and menu items accordingly.
@param textview Is a pointer to the Textarea in html editor.
@param temp_attribute Is the temporary attribute that needs to be reflected on to the UI such as Bold, Italic etc.
@param editor Pointer to EditorInfo structure.
@return void. 
*/
void set_attribute_to_current_attribute(GtkTextView * textview,
					GtkTextAttributes * temp_attribute, EditorInfo * editor);


/**
This is called on move-cursor event on textview. 
The text attributes at the current cursor position are updated 
and the toolbar information is updated accordingly.
@param textview Is a pointer to the Textarea in html editor.
@param editor Pointer to EditorInfo structure.
@return void. 
*/
void refresh_current_attributes(GtkTextView * textview, EditorInfo * editor);


/**
Pre creating all supported tags. Except font and color tags. 
These are to be created runtime depending the selection.
@param textview Is a pointer to the Textarea in html editor.
@return void.
*/
void create_supported_tags(GtkTextView * textview);


/**
* This function is used to get the font name.
* @param fontname_entry represents font name selection entry.
* @return gchar
* */
const gchar *get_edittoolbar_fontname(GtkWidget * fontname_entry, EditorInfo * editorInfo);


/**
* This function is used to get the font size.
* @param fontsize_entry represents font name selection entry.
* @return gchar
* */
const gchar *get_edittoolbar_fontsize(GtkWidget * fontsize_entry);


/**
This is called to apply a particular tag on the selection.
@param buffer GtkTextBuffer pointer for textarea in html editor
@param start_iter selection start iterator.
@param end_iter selection end iterator.
@param tagid value from supported TAGID enum.
@return void. 
*/
void apply_gtk_tag(GtkTextBuffer * buffer,
		   GtkTextIter * start_iter, GtkTextIter * end_iter, gint tagid);


/**
This is called to remove a particular tag on the selection.
@param buffer GtkTextBuffer pointer for textarea in html editor
@param start_iter selection start iterator.
@param end_iter selection end iterator.
@param tagid value from supported TAGID enum.
@return void. 
*/
void remove_gtk_tag(GtkTextBuffer * buffer,
		    GtkTextIter * start_iter, GtkTextIter * end_iter, gint tagid);


/**
This function is invoked to apply selected fontname and size to the 
selected portion of text.
@param buffer GtkTextBuffer pointer for textarea in html editor
@param fontname Name of the font
@param fontsize Size of font to be applied.
@return void. 
*/
void apply_font_name_and_size(GtkTextView * textview,
			      gchar * fontname, gint fontsize,
			      gint zoom_level, GtkTextIter start_iter, GtkTextIter end_iter);


/**
Formats iterator from PangoFontAttr and applies to text.
@param editor Pointer to EditorInfo structure.
@param iterator Iterator to apply font details on the selection.
@return void. 
*/
void format_font_iterator(EditorInfo * editor,
			  PangoAttrIterator * iterator,
			  GtkTextAttributes * temp_attribute,
			  GtkTextIter start_iter, GtkTextIter end_iter);


/**
This funtion creates a font tag and apply to the iteratios specified.
@param editor Pointer to EditorInfo structure.
@param start_iter selection start iterator.
@param end_iter selection end iterator.
@param tagid value from supported TAGID enum.
@return void. 
*/
void apply_font_tag(EditorInfo * editor,
		    GtkTextIter * start_iter, GtkTextIter * end_iter, gint tagid);


/**
 This function will remove the existing font tags between teh given offsets
 @param buffer GtkTextBuffer in which the tag needs to be removed
 @param start_offset starting offset 
 @param end_offset ending offset
 */
void remove_font_tags_at_offset(GtkTextBuffer * buffer, gint start_offset, gint end_offset);


/**
This is called to apply foreground color to the selection.
@param color GdkColor value to be applied to the selection.
@param editor Pointer to EditorInfo structure.
@return void. 
*/
void format_font_color(GdkColor * color,
		       EditorInfo * editor, GtkTextIter start_iter, GtkTextIter end_iter);


/**
This funtion creates a color tag and apply to the iteratios specified.
@param editor Pointer to EditorInfo structure.
@param start_iter selection start iterator.
@param end_iter selection end iterator.
@param tagid value from supported TAGID enum.
@return void. 
*/
GdkColor *get_selected_color(EditorInfo * editor,
			     GtkTextIter * start_iter, GtkTextIter * end_iter, gint tagid);


/**
This function will set the iterators for applying the tag. For example 
the left_align should be applied to a line where the cursor is present.
@param textview Is a pointer to the Textarea in html editor.
@param editor Pointer to EditorInfo structure.
@return void. 
*/
void set_iterators_for_tag(GtkTextBuffer * buffer,
			   GtkTextIter * start_iter,
			   GtkTextIter * end_iter, gint tagid, gboolean apply_tag);


/**
Apply tag corresponding to tagid to the text selection.
@param buffer Is a pointer to buffer of the textarea in html editor.
@param editor Pointer to EditorInfo structure.
@param tagid value from supported TAGID enum.
@return void. 
*/
void apply_gtk_tag_for_current_selection(GtkTextBuffer * buffer, EditorInfo * editor, gint tagid);
/**
Remove tag corresponding to tagid to the text selection.
@param buffer Is a pointer to buffer of the textarea in html editor.
@param editor Pointer to EditorInfo structure.
@param tagid value from supported TAGID enum.
@return void. 
*/
void remove_gtk_tag_for_current_selection(GtkTextBuffer * buffer, EditorInfo * editor, gint tagid);
/**
Apply or remove tag corresponding to tagid to the text selection.
Toggle is done depending upon the toggle button state on/off.
@param buffer Is a pointer to buffer of the textarea in html editor.
@param editor Pointer to EditorInfo structure.
@param tagid value from supported TAGID enum.
@return void. 
*/
void add_remove_gtk_tags(GtkToggleToolButton * button, EditorInfo * editor, gint tagid);
/**
This is called on text insertion on text area of html editor.
This signal is required to refresh the toolbar and menu items accordingly.
@param buffer is msg body GtkTextBuffer.
@param text_iter is insertion pos of text.
@param length is length of text inserted.
@param editor Pointer to EditorInfo structure.
@return void. 
*/
void
on_body_after_text_insert(GtkTextBuffer * buffer,
			  GtkTextIter * text_iter, gchar * text, gint length, EditorInfo * editor);
/**
This is called on text insertion in text area of html editor.
@param buffer is msg body area.
@param text_iter is text insertion pos.
@param text is text inserted.
@param length is length of text.
@param editor Pointer to EditorInfo structure.
@return void. 
*/
void
on_body_before_text_insert(GtkTextBuffer * buffer,
			   GtkTextIter * text_iter, gchar * text,
			   gint length, EditorInfo * editorinfo);
/**
 This function will raise the priority of bold and italic tags.
 This is made for fix that the bold and italic properties are not reflected over font tag.
 @param buffer GtkTextBuffer representing the buffer of textarea.
*/
void raise_bold_italic_priority(GtkTextBuffer * buffer);
/**
This is called on button release event on text area of html editor.
This signal is required to refresh the toolbar and menu items accordingly.
@param widget Is a pointer to the Textarea in html editor.
@param event GdkEventButton to identify the button pressed.
@param editor Pointer to EditorInfo structure.
@return void. 
*/
gboolean
on_body_button_release_event(GtkWidget * widget, GdkEventButton * event, EditorInfo * editor);
gboolean on_body_key_release_event(GtkWidget * widget, GdkEventKey * event, EditorInfo * editor);
/**
This is a common function invoked on each of text property change from the user.
@param formatbutton source widget from which the event is generated. 
Depending upon the source, the corresponding operation is invoked.
@param editor Pointer to EditorInfo structure.
@return void. 
*/
void on_format_button_click(GtkWidget * formatbutton, EditorInfo * editor);
/**
This is invoked upon the font set operation from user.
@param font_type source widget from which the event is generated. 
@param editor Pointer to EditorInfo structure.
@return void. 
*/
void format_font_dialog(GtkWidget * font_type, EditorInfo * editor);
/**
This is invoked to populate the key inpit as per the preselected format.
@param editor Pointer to EditorInfo structure.
@return void.
*/
void populate_character_input(EditorInfo * editor);
/**
This function will apply any new tag that is selected by the user
@param editor Pointer to EditorInfo structure.
@param start_iter starting iterator to check for existing properties
@param end_iter iterator at the ending point
*/
void apply_attribute_difference(EditorInfo * editor,
				GtkTextIter * start_iter, GtkTextIter * end_iter);
/**
This function verifies the differences in font name selected to the existing font property.
Any differnce is applied to the text
@param buffer text of text area
@param gtkattr GtkTextAttributes ofthe text appearing
@param desc PangoFontDescription representing user preselection
@param start_iter starting iterator to apply any changes in font
@param end_iter ending iterator to apply any changes in font
*/
void verify_font_attributes(GtkTextBuffer * buffer,
			    GtkTextAttributes * gtkattr, PangoFontDescription * desc,
			    GtkTextIter * start_iter, GtkTextIter * end_iter, gint zoom_level);
/**
 * Checks if the given iterator has a bullet or number tag
 * @param iter text iterator representing the position in text buffer
 * @return Text tag representing the bullet or number tag.
 */
GtkTextTag *iter_has_bullet_or_number_tag(GtkTextIter * iter);
/**
 * Removes bullet or number character between the given iterators
 * @param buffer GtkTextBuffer representing the text
 * @param cur_iter current iterator or starting iterator for the bullet or number character to be searched.
 * @param end_iter text iterator representing the limiting position to search for the bullet or number character in text buffer
 */
void remove_bullet_or_number(GtkTextBuffer * buffer,
			     GtkTextIter * cur_iter, GtkTextIter * end_iter);
/**
 * Removes list item at the line number specified.
 * @param buffer GtkTextBuffer representing the text
 * @param lineno line number at which the list item as to be removed
 */
void remove_list_item_tag(GtkTextBuffer * buffer, gint lineno);
/**
 * Removes the list items between the iterators specified
 * @param buffer GtkTextBuffer representing the text
 * @param start_iter starting text iterator representing the position in text buffer
 * @param end_iter ending text iterator representing the position in text buffer
 */
void remove_list_items(GtkTextBuffer * buffer, GtkTextIter * start_iter, GtkTextIter * end_iter);
/**
 * Get the index for numbering at a given line number.
 * @param buffer GtkTextBuffer representing the text
 * @param  lineno line number at which the number list index to be claculated
 * @return gint representing the line index for the number list
 */
gint get_list_number_value(GtkTextBuffer * buffer, gint lineno);
/**
 * Prepares the bullet or number character depending upon the listing type
 * at the given iterator.
 * @param buffer GtkTextBuffer representing the text
 * @param start_iter starting text iterator representing the position in text buffer
 * @return gchar representing the bullet or number text.
 */
gchar *get_bullet_or_number_char(GtkTextBuffer * buffer,
				 GtkTextIter * start_iter, GtkTextTag * tag);
/**
 * This function will apply the tags applied at the source iter for the range of startiter and enditer.
 * @param buffer GtkTextBuffer representing the text
 * @param sourceiter text iterator representing the source position in text buffer
 * @param startiter text iterator representing the starting position in text buffer
 * @param enditer text iterator representing the ending position in text buffer
 */
void text_buffer_update_tags(GtkTextBuffer * buffer, GtkTextIter * sourceiter,
			     GtkTextIter * startiter, GtkTextIter * enditer,
			     EditorInfo * editorinfo);
/**
 * Insert bullet or number at the given iterator
 * @param buffer GtkTextBuffer representing the text
 * @param start_iter text iterator representing the position in text buffer
 */
void insert_bullet_or_number(GtkTextBuffer * buffer,
			     GtkTextIter * start_iter, GtkTextTag * tag, EditorInfo * editorinfo);
/**
 * Insert list item at the line number specified
 * @param buffer GtkTextBuffer representing the text
 * @param lineno line number in text buffer
 */
void insert_list_item_tag(GtkTextBuffer * buffer,
			  gint lineno, GtkTextTag * tag, EditorInfo * editorinfo);
/**
 * Update the list items as per the bullets or number tag applied.
 * @param buffer GtkTextBuffer representing the text
 * @param start_iter Text iterator representing the starting position
 * @param end_iter Text iterator representing the ending position
 */
void set_list_items(GtkTextBuffer * buffer, GtkTextIter * start_iter,
		    GtkTextIter * end_iter, GtkTextTag * tag, EditorInfo * editorinfo);
/**
 * This is invoked when a tag is applied in the text buffer. The bullet and number tags are handled. * @param buffer GtkTextBuffer representing the text
 * @param start_iter Text iterator representing the starting position
 * @param end_iter Text iterator representing the ending position
 * @param user_data : not used in the implementation
 */
void on_text_buffer_apply_tag(GtkTextBuffer * buffer, GtkTextTag * tag,
			      GtkTextIter * start_iter, GtkTextIter * end_iter,
			      EditorInfo * editorinfo);
/**
 * This is invoked when a tag is removed in the text buffer.
 * The bullet and number tags are handled.
 * @param buffer GtkTextBuffer representing the text
 * @param start_iter Text iterator representing the starting position
 * @param end_iter Text iterator representing the ending position
 * @param user_data : not used in the implementation
 */
void on_text_buffer_remove_tag(GtkTextBuffer * buffer, GtkTextTag * tag,
			       GtkTextIter * start_iter, GtkTextIter * end_iter,
			       EditorInfo * editorinfo);
/**
 * Removes the preceeding bullet or number character at the current cursor position.
 * @param buffer GtkTextBuffer representing the text
 */
void remove_preceding_bullet_or_number(GtkTextBuffer * buffer);
/**
 * Invoked when text is deleted in the text buffer
 * @param buffer GtkTextBuffer representing the text
 * @param start_iter Text iterator representing the starting position
 * @param end_iter Text iterator representing the ending position
 * @param user_data : not used in the implementation
 */
void on_text_buffer_delete_range(GtkTextBuffer * buffer,
				 GtkTextIter * start_iter,
				 GtkTextIter * end_iter, EditorInfo * editorinfo);
void on_body_after_text_delete(GtkTextBuffer * buffer,
			       GtkTextIter * start_iter,
			       GtkTextIter * end_iter, EditorInfo * editor);

void bullet_num_deletion(GtkTextBuffer * buffer,
			 GtkTextIter * start_iter, GtkTextIter * end_iter, EditorInfo * editorinfo);
/**
 * Moves teh cursor away from the bullet or number
 * @param buffer GtkTextBuffer representing the text
 */
void text_buffer_set_cursor(GtkTextBuffer * buffer);
/**
 * This is invoked when text is inserted in the text buffer.
 * @param buffer GtkTextBuffer representing the text
 * @param start_iter Text iterator representing the starting position
 * @param end_iter Text iterator representing the ending position
 * @param user_data : not used in the implementation
 */
void on_text_buffer_insert_text(GtkTextBuffer * buffer,
				GtkTextIter * text_iter,
				gchar * text, gint length, EditorInfo * editorinfo);
void check_newline_insert(GtkTextBuffer * buffer,
			  GtkTextIter * text_iter, gchar * text, gint length, gpointer user_data);
/**
 * Invoked when a insert image option is selected from menu.
 * @param filepath file path representing the image file
 * @param editorinfo Editor info representing the editor.
 */
void text_buffer_insert_image(const gchar * filepath, EditorInfo * editorinfo);
void remove_inline_images(GtkTextBuffer * buffer);
void update_alignment_menu(EditorInfo * editor);
gboolean predicate(gunichar ch, gpointer user_data);
void call_attribute_refresh_func(GtkWidget * widget, EditorInfo * editor);
gint get_index_for_default_font(PangoFontFamily ** font_list, gint num_families);
void remove_list_chars_for_paste(GtkTextBuffer * buffer);
void update_list_item_paste(GtkTextBuffer * buffer, gint offset, EditorInfo * editorinfo);
void move_cursor_to_listitem_end(GtkWidget * textview);
void handle_paste_with_im(GtkTextView * view, EditorInfo * editorinfo);
void handle_paste_completion_with_im(GtkTextView * view, EditorInfo * editorinfo);
void look_for_paste_completion(GtkClipboard * clipboard, gchar * selection_data,
			       EditorInfo * editorinfo);
void process_paste_completion(GtkTextBuffer * buffer, gint paste_start, gint paste_end,
			      EditorInfo * editorinfo);
void block_signals_upon_paste(GtkTextBuffer * buffer, EditorInfo * editorinfo);
void unblock_signals_upon_paste(GtkTextBuffer * buffer, EditorInfo * editorinfo);

#endif
