/**
 * @file mailviewer.h
 *
 * This file contains the header file for the mail body display widget
 * 
 *  Copyright (C) 2005 Nokia Corporation 
 */

#ifndef __MAIL_VIEWER_H__
#define __MAIL_VIEWER_H__

#include <gtk/gtk.h>

#ifndef NO_HILDON
#include <osso-uri.h>
#else 
typedef void OssoURIAction;
#endif /*NO_HILDON*/

G_BEGIN_DECLS

/**
 * standard G_ convenience macros
 *  
 */
#define TYPE_MAIL_VIEWER            (mail_viewer_get_type ())
#define MAIL_VIEWER(object)         (G_TYPE_CHECK_INSTANCE_CAST ((object), TYPE_MAIL_VIEWER, Mailviewer))
#define MAIL_VIEWER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAIL_VIEWER, MailviewerClass))
#define IS_MAIL_VIEWER(object)      (G_TYPE_CHECK_INSTANCE_TYPE ((object), TYPE_MAIL_VIEWER))
#define IS_MAIL_VIEWER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAIL_VIEWER))
#define MAIL_VIEWER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAIL_VIEWER, MailviewerClass))


#define MAIL_VIEWER_DEFAULT_CHARSET	"iso-8859-1"
#define MAIL_VIEWER_MAX_FILESIZE	(2*1024*1024)


typedef struct _Mailviewer      Mailviewer;
typedef struct _MailviewerClass MailviewerClass;

struct _Mailviewer {
	GtkScrolledWindow	parent_instance;
};

struct _MailviewerClass {
	GtkScrolledWindowClass parent_class;

	/* signals */

	/* emitted when a link is clicked; 'uri' will contain the uri that was clicked*/
	void (*link_clicked)   (Mailviewer *viewer, const gchar* uri, OssoURIAction *action);
	
	/* emitted when part of the loading has been completed; frac will contain
	 * that has been loaded so far ([0..1]) */
	void (*loaded_part)      (Mailviewer *viewer, gfloat frac);

};


/** 
 * get the g-type of a mail viewer object
 * 
 * 
 * @return the gtype
 */
GType		mail_viewer_get_type                (void)  G_GNUC_CONST;


/** 
 * create a new mail viewer widget
 * 
 * 
 * @return a GtkWidget* for the new mail viewer widget
 */
GtkWidget*      mail_viewer_new                     ();


/** 
 * set the contents for the mail viewer widget. The contents can be
 * either mhtml (MIME) or plain text emails. The widget will take care
 * of the necessary conversions
 *
 * this function will return immediately; when the loading is
 * complete, the "load_done" signal will be emitted
 * 
 * note that you can call the function multiple times, and this will
 * replace the content
 * 
 * @param viewer a ptr to a mail viewer object
 * @param data NULL-terminated buffer with the email content
 * 
 * @return TRUE if it succeeded, FALSE otherwise 
 */
gboolean        mail_viewer_set_content             (Mailviewer *viewer, const gchar *data);


/** 
 * set the contents for the mail viewer widget. The contents can be
 * either mhtml (MIME) or plain text emails. The widget will take care
 * of the necessary conversions
 *  
 * this function will return immediately; when the loading is
 * complete, the "load_done" signal will be emitted
 * 
 * note that you can call the function multiple times, and this will
 * replace the content
 * 
 * @param viewer a ptr to a mail viewer object
 * @param filename filename for a file which contains the email
 * 
 * @return TRUE if it succeeded, FALSE otherwise
 */
gboolean        mail_viewer_set_content_from_file   (Mailviewer *viewer, const gchar *filename);



/** 
 * stop the loading that is currently underway
 * 
 * @param viewer 
 */
void            mail_viewer_stop_loading             (Mailviewer *viewer);


/** 
 * set the zoom factor for the mail viewer
 * 
 * @param viewer a ptr to mail viewer object
 * @param zoomfactor the zoomfactor, 1.0 means no zoom, 2.0 mean
 * double size etc.
 */
void		mail_viewer_set_zoom                (Mailviewer *viewer, gdouble zoomfactor);


/** 
 * get the zoom factor for the mail viewer
 * 
 * @param viewer a ptr to a mail viewer object
 * 
 * @return the zoomfactor; 1.0 means no zoom, 2.0 means double size,
 * etc. 
 */
gdouble		mail_viewer_get_zoom                (Mailviewer *viewer);


/** 
 * search for text in the mail viewer
 * 
 * @param viewer a ptr to a mail viewer object
 * @param txt the text to search for
 * @param search whole document
 * 
 * @return TRUE if txt was found, FALSE otherwise
 */
gboolean	mail_viewer_search      (Mailviewer *viewer, const gchar* txt, gboolean from_start);



/** 
 * search for the next match
 * 
 * @param viewer a ptr to a mail viewer object
 * 
 * @return TRUE if it was found, FALSE otherwise.
 */
gboolean        mail_viewer_search_next (Mailviewer *viewer);


/** 
 * select the whole email message
 * 
 * @param viewer ptr to a mail viewer obj
 */
void		mail_viewer_select_all		    (Mailviewer *viewer);


/** 
 * copy the current selection to the clipboard
 * 
 * @param viewer ptr to a mail viewer obj
 *
 * @return TRUE if something was actually copied, FALSE otherwise
 */
gboolean	mail_viewer_copy_selection          (Mailviewer *viewer);


/** 
 * is there anything selected at the moment?
 * 
 * @param viewer ptr to a mail viewer obj
 * 
 * @return TRUE if there is a selection, FALSE otherwise
 */
gboolean	mail_viewer_has_selection	    (Mailviewer *viewer);
void hyperlinkify_plain_text(GString *);
/* G_END_DECLS */

#endif /*__MAIL_VIEWER_H_ */
