/**
 * @file list_view.c
 *
 * This file contains the implementation for list view handling of main view
 * module of OSSO EMAIL.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

/*Header file containing all standard paths*/
#include "ui_common.h"

/*AL header files*/
#include "al_callbacks.h"
#include "utils.h"
#include "folder.h"
#include "defs.h"

/*UI_DBUS header files*/
#include "email_mgr.h"
#include "dbus_utils.h"

#include "editview.h"
#include "ui_al_callbacks.h"
#include "ui_interface.h"
#include "folder_mgmt.h"
#include "message_handler.h"
#include "list_view.h"
#include "ui_defs.h"
#include "ui_utils.h"


/* Global Variables */
extern MainViewInfo *email_main_view;	/* Declared in ui_interface.c */


/*Helper functions*/

/**
This is a helper function used to free the MsgHeader structure

@param	header		Pointer to MsgHeader structure
@return void
*/
void ui_free_msg_header(MsgHeader * header)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((header == NULL), "Input param check failed", return);

	g_free(header->fromname);
	g_free(header->date);
	g_free(header->from);
	g_free(header->to);
	g_free(header->cc);
	g_free(header->bcc);
	g_free(header->subject);
	g_free(header->msgid);
	g_free(header->replyto);
	g_free(header->inreplyto);
	g_free(header->username);
	g_free(header->password);
	g_free(header->account_name);
	g_free(header);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/*Sorting functions*/

/**
This is a helper function used to determine sorting order based
on date, message_id fields.

@param  date1         First message received time stamp
@param  date2         Second message received time stamp
@param  msg_id1       First message id
@param  msg_id2       Second message id
@return gint          1 if msg1 compares before msg2, 0 if they compare 
                      equal, 1 if msg1 compares after msg2
*/
static gint
ui_sort_on_date_msgid(gint date1, gint date2, const gchar * msg_id_key1, const gchar * msg_id_key2)
{
	gint sort = -1;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	if (date2 == date1) {
		if (msg_id_key1 != NULL && msg_id_key2 != NULL) {
			sort = strcmp(msg_id_key1, msg_id_key2);
		}
	} else {
		sort = (date2 < date1) ? 1 : -1;	//Sort by Date column
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return sort;
}

/**
This is a helper function used to sort messages that have same sorting
criteria. Such messages are sorted by date, message_id fields when the
main sorting criteria is same for both the messages.

@param  date1         First message received time stamp
@param  date2         Second message received time stamp
@param  sort_order    Ascending/Descending order currently selected
@param  sort          sort order for the two messages (out param)
@param  msg_id1       First message id
@param  msg_id2       Second message id
@return void
*/
static void
ui_sort_msgs_with_same_criteria(gint date1, gint date2, gint sort_order,
				gint * sort, const gchar * msg_id_key1, const gchar * msg_id_key2)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	CHECK_PARAM((msg_id_key1 == NULL), "Input param check failed", return);
	CHECK_PARAM((msg_id_key2 == NULL), "Input param check failed", return);
	CHECK_PARAM((sort == NULL), "Input param check failed", return);

	if (*sort == 0) {
		*sort = ui_sort_on_date_msgid(date1, date2, msg_id_key1, msg_id_key2);
		/*If fields are same, most recent msgs should be at top irrespective 
		   of the sort order. Toggle sort order if needed. */
		if (sort_order == GTK_SORT_ASCENDING) {
			*sort = 0 - *sort;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a 'GtkTreeIterCompareFunc' function used to sort folder view.

@param	model		Pointer to the storage model to be sorted
@param	iter_a		Pointer to GtkTreeIter of first row to be compared
@param	iter_b		Pointer to GtkTreeIter of second row to be compared
@param	userdata	Pointer to the user data (not used here)
@return integer		+1 --> b comes before a
			-1 --> a comes before b
			0  --> Both rows are equal
*/
static gint
ui_sort_folder_compare_func(GtkTreeModel * model, GtkTreeIter * iter_a,
			    GtkTreeIter * iter_b, gpointer userdata)
{
	gchar *folder_name1 = NULL, *folder_name2 = NULL;
	gchar *tmp_folder_name1 = NULL, *tmp_folder_name2 = NULL;
	gboolean userCreated1 = FALSE, userCreated2 = FALSE;
	gint model_seq_num1 = 0, model_seq_num2 = 0;
	gint sort = 0;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((model == NULL), "Input param check failed", return 0);
	CHECK_PARAM((iter_a == NULL), "Input param check failed", return 0);
	CHECK_PARAM((iter_b == NULL), "Input param check failed", return 0);

	gtk_tree_model_get(model, iter_a, COL_FOLDER_NAME, &folder_name1,
			   COL_MODEL_SEQ_NUM, &model_seq_num1, COL_USER_CREATED, &userCreated1, -1);
	gtk_tree_model_get(model, iter_b, COL_FOLDER_NAME, &folder_name2, COL_MODEL_SEQ_NUM,
			   &model_seq_num2, COL_USER_CREATED, &userCreated2, -1);

	/*All mandatory folders - INBOX,OUTBOX,SENT,DRAFTS - should be at the top,
	   except for ARCHIVE which should be at the bottom.
	   If 'iter_a' is not user created folder, don't sort */
	if (folder_name1 != NULL && ui_compare_utf8_strings(folder_name1, ARCHIVE) == 0) {
		sort = 1;
	} else if (folder_name2 != NULL && ui_compare_utf8_strings(folder_name2, ARCHIVE) == 0) {
		sort = -1;
	} else if (userCreated1 == FALSE && userCreated2 == FALSE) {	/*Both are mandatory folders */
		sort = (model_seq_num2 < model_seq_num1) ? 1 : -1;
	} else if (userCreated1 == FALSE) {	/*'iter_a' is mandatory folder */
		sort = -1;
	} else if (userCreated2 == FALSE) {	/*'iter_b' is mandatory folder */
		sort = 1;
	} else {
		tmp_folder_name1 = g_utf8_strdown(folder_name1, -1);
		tmp_folder_name2 = g_utf8_strdown(folder_name2, -1);
		sort = g_utf8_collate(tmp_folder_name1, tmp_folder_name2);
		g_free(tmp_folder_name1);
		g_free(tmp_folder_name2);
	}

	ULOG_DEBUG_F("Folder1 = %s, Folder2 = %s, Sort order is %d\n", folder_name1,
		     folder_name2, sort);
	g_free(folder_name1);
	g_free(folder_name2);

	return sort;
}

/**
This is a 'GtkTreeIterCompareFunc' function used to sort current list view.

@param	model		Pointer to the storage model to be sorted
@param	iter_a		Pointer to GtkTreeIter of first row to be compared
@param	iter_b		Pointer to GtkTreeIter of second row to be compared
@param	userdata	Pointer to the user data (not used here)
@return integer		+1 --> b comes before a
			-1 --> a comes before b
			0  --> Both rows are equal
*/
static gint
ui_sort_listview_compare_func(GtkTreeModel * model, GtkTreeIter * iter_a,
			      GtkTreeIter * iter_b, gpointer userdata)
{
	gint date1 = 0, date2 = 0, size_priority_value1 = 0, size_priority_value2 = 0;
        gboolean read_status1 = 0, read_status2 = 0, atta_present1 = 0, atta_present2 = 0,
               body_downloaded2 = 0 , body_downloaded1 = 0, fully_downloaded1 = 0,
               fully_downloaded2 = 0;
	gchar *send_recp_key1 = NULL, *send_recp_key2 = NULL;
	gchar *subject_key1 = NULL, *subject_key2 = NULL;
	gchar *msg_id_key1 = NULL, *msg_id_key2 = NULL;
	gint sort = 0, sort_order = -1;
	gint sortColumn = GPOINTER_TO_INT(userdata);
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((model == NULL), "Input param check failed", return 0);
	CHECK_PARAM((iter_a == NULL), "Input param check failed", return 0);
	CHECK_PARAM((iter_b == NULL), "Input param check failed", return 0);

	/*Get date, message-id fields for each iter */
	gtk_tree_model_get(model, iter_a, COL_DISPLAY_DATE, &date1,
			   COL_MSG_ID_KEY, &msg_id_key1, -1);
	gtk_tree_model_get(model, iter_b, COL_DISPLAY_DATE, &date2, COL_MSG_ID_KEY,
			   &msg_id_key2, -1);
	sort_order = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(model), "sort_order"));
	/*Sorting criteria : sort with required column. If values are same,
	   sort with date. If date fields are also same, sort with message-id.
	   If fields are same, most recent msgs should be at top irrespective 
	   of the sort order. Toggle sort order if needed. */
	switch (sortColumn) {
	case -1:
		{
			sort = -1;
		}
		break;

	case COL_SENDER_RECIPIENT:
		{
			gtk_tree_model_get(model, iter_a, COL_SENDER_RECIPIENT_KEY,
					   &send_recp_key1, -1);
			gtk_tree_model_get(model, iter_b, COL_SENDER_RECIPIENT_KEY,
					   &send_recp_key2, -1);
			if (send_recp_key1 != NULL && send_recp_key2 != NULL) {
				sort = strcmp(send_recp_key1, send_recp_key2);
			}
			ui_sort_msgs_with_same_criteria(date1, date2, sort_order,
							&sort, msg_id_key1, msg_id_key2);
			g_free(send_recp_key1);
			g_free(send_recp_key2);
		}
		break;

	case COL_DISPLAY_DATE:
		{
			sort = ui_sort_on_date_msgid(date1, date2, msg_id_key1, msg_id_key2);
		}
		break;

	case COL_SUBJECT:
		{
			gtk_tree_model_get(model, iter_a, COL_SUBJECT_KEY, &subject_key1, -1);
			gtk_tree_model_get(model, iter_b, COL_SUBJECT_KEY, &subject_key2, -1);
			if (subject_key1 != NULL && subject_key2 != NULL) {
				sort = strcmp(subject_key1, subject_key2);
			}

			ui_sort_msgs_with_same_criteria(date1, date2, sort_order,
							&sort, msg_id_key1, msg_id_key2);
			g_free(subject_key1);
			g_free(subject_key2);
		}
		break;

	case COL_SIZE:
		{
			gtk_tree_model_get(model, iter_a, COL_SIZE, &size_priority_value1, -1);
			gtk_tree_model_get(model, iter_b, COL_SIZE, &size_priority_value2, -1);
			if (size_priority_value1 != size_priority_value2) {
				sort = (size_priority_value2 < size_priority_value1) ? 1 : -1;
			}
			ui_sort_msgs_with_same_criteria(date1, date2, sort_order,
							&sort, msg_id_key1, msg_id_key2);
		}
		break;

	case COL_PRIORITY:
		{
			gtk_tree_model_get(model, iter_a, COL_PRIORITY, &size_priority_value1, -1);
			gtk_tree_model_get(model, iter_b, COL_PRIORITY, &size_priority_value2, -1);
			if (size_priority_value1 != size_priority_value2) {
				sort = (size_priority_value2 > size_priority_value1) ? 1 : -1;
			}
			ui_sort_msgs_with_same_criteria(date1, date2, sort_order,
							&sort, msg_id_key1, msg_id_key2);
		}
		break;
       case COL_READ_STATUS:
                {
                        gtk_tree_model_get(model, iter_a, COL_READ_STATUS, &read_status1,
                                                          COL_PRIORITY, &size_priority_value1,
                                                          COL_BODY_DOWNLOADED, &body_downloaded1,
                                                          COL_FULLY_DOWNLOADED, &fully_downloaded1,
                                                          -1);
                        gtk_tree_model_get(model, iter_b, COL_READ_STATUS, &read_status2,
                                                           COL_PRIORITY,&size_priority_value2,
                                                           COL_BODY_DOWNLOADED, &body_downloaded2,
                                                           COL_FULLY_DOWNLOADED, &fully_downloaded2,
                                                           -1);
                        /*first check for read/unread, then
                         *                priority , then
                         *                body_downloaded, then
                         *                fully_downloaded */
                        if (read_status1 != read_status2) {
                                sort = (read_status1 > read_status2) ? 1 : -1;
                        }else if(size_priority_value1 != size_priority_value2) {
                                sort = (size_priority_value2 > size_priority_value1) ? 1 : -1;
                        }else if(body_downloaded1 != body_downloaded2){
                                sort = (body_downloaded1 > body_downloaded2) ? 1 : -1;
                        }else if(fully_downloaded1 != fully_downloaded2){
                                sort = (fully_downloaded1 > fully_downloaded2) ? 1 : -1;
                        }

                        ui_sort_msgs_with_same_criteria(date1, date2, sort_order,
                                                      &sort, msg_id_key1, msg_id_key2);
                        break;
                }
        case COL_ATTACHMENT_PRESENT:
                {
                        gtk_tree_model_get(model, iter_a, COL_ATTACHMENT_PRESENT, &atta_present1, -1);
                        gtk_tree_model_get(model, iter_b, COL_ATTACHMENT_PRESENT, &atta_present2, -1);
                        if (atta_present1 != atta_present2) {
				 sort = (atta_present1 > atta_present2) ? 1 : -1;
                        }
                        ui_sort_msgs_with_same_criteria(date1, date2, sort_order,
                                                        &sort, msg_id_key1, msg_id_key2);
                        break;
                }
	default:
		break;
	}

	g_free(msg_id_key1);
	g_free(msg_id_key2);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return sort;
}

/*List view Functions*/
/**
This is a helper function used to create a new list view storage model. It is 
called when a new folder is being added to the folder view. The newly created
storage model is added to a global list used for maintaining list view models.
These list stores should be freed when deleting corresponding folders, or
during application exit.

@param	void
@return void
*/
void ui_create_listview_model(void)
{
	GtkListStore *listStore = NULL;
	GtkTreeSortable *sortable = NULL;
	GtkTreeModel *sortModel = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);

	/*Create a new list store object and append it to the global list */
	listStore =
	    gtk_list_store_new(NUM_LIST_VIEW_COLS, G_TYPE_STRING, G_TYPE_STRING,
			       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
			       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
			       G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT,
			       G_TYPE_INT, G_TYPE_STRING, G_TYPE_BOOLEAN,
			       GDK_TYPE_PIXBUF, G_TYPE_BOOLEAN, GDK_TYPE_PIXBUF,
			       G_TYPE_INT, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_INT,
			       G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_BOOLEAN,
			       G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN,
			       G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN,
			       G_TYPE_BOOLEAN, G_TYPE_INT,
			       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
			       G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_UINT);

	sortModel = gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(listStore));
	email_main_view->sorted_model_list =
	    g_slist_append(email_main_view->sorted_model_list, sortModel);

	/*Set all the required sorting functions */
	sortable = GTK_TREE_SORTABLE(sortModel);
	gtk_tree_sortable_set_sort_func(sortable, COL_SENDER_RECIPIENT,
					ui_sort_listview_compare_func,
					GINT_TO_POINTER(COL_SENDER_RECIPIENT), NULL);
	gtk_tree_sortable_set_sort_func(sortable, COL_DISPLAY_DATE,
					ui_sort_listview_compare_func,
					GINT_TO_POINTER(COL_DISPLAY_DATE), NULL);
	gtk_tree_sortable_set_sort_func(sortable, COL_SUBJECT,
					ui_sort_listview_compare_func,
					GINT_TO_POINTER(COL_SUBJECT), NULL);
	gtk_tree_sortable_set_sort_func(sortable, COL_PRIORITY,
					ui_sort_listview_compare_func,
					GINT_TO_POINTER(COL_PRIORITY), NULL);
	gtk_tree_sortable_set_sort_func(sortable, COL_SIZE,
					ui_sort_listview_compare_func,
					GINT_TO_POINTER(COL_SIZE), NULL);
        gtk_tree_sortable_set_sort_func(sortable, COL_ATTACHMENT_PRESENT,
                                        ui_sort_listview_compare_func,
                                        GINT_TO_POINTER(COL_ATTACHMENT_PRESENT), NULL);
        gtk_tree_sortable_set_sort_func(sortable, COL_READ_STATUS,
                                        ui_sort_listview_compare_func,
                                        GINT_TO_POINTER(COL_READ_STATUS), NULL);

	/*Set the default sorting column & sort order */
	gtk_tree_sortable_set_sort_column_id(sortable, COL_DISPLAY_DATE, GTK_SORT_DESCENDING);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to delete a list view storage model. It is 
called when a user folder is being deleted from the folder view. The storage 
model is deleted from a global list used for maintaining list view models.

@param	void
@return void
*/
void ui_delete_listview_model(gint modelNum)
{
	GSList *node = NULL;
	GtkTreeModel *child_model = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->sorted_model_list == NULL),
		    "Input param check failed", return);

	/*Get the listview model from global list and delete it */
	node = g_slist_nth(email_main_view->sorted_model_list, (guint) modelNum);
	child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(node->data));
	gtk_list_store_clear(GTK_LIST_STORE(child_model));
	g_object_unref(node->data);
	node->data = NULL;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
 This function is called when ever the column headers are clicked for sorting.
 
 @param column is the GtkTreeViewColumn.
 @param sort_key is the gpointer to int representing the sort key 
 i.e(from/subject/data/attachment).
 @return void.
*/
static void sort_messages(GtkTreeViewColumn * col, gpointer sort_key)
{
	gint current_sort_key = GPOINTER_TO_INT(sort_key);
	GtkSortType current_sort_order;
	GtkTreeModel *folder_list_store = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeIter iter;

	folder_list_store =
	    gtk_tree_view_get_model(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	if (gtk_tree_selection_get_selected(selection, &folder_list_store, &iter)) {
		gtk_tree_model_get(folder_list_store, &iter,
				   COL_SORT_ORDER, &current_sort_order, -1);
	}
	ui_on_ok_sort_messages_dialog(current_sort_key, !current_sort_order);
	return;
}

/**
This is a helper function used to create a new folder view storage model. It
is called during main UI window creation. The storage model should be freed 
during application exit.

@param	void
@return GtkTreeModel*	Pointer to the newly created Tree Model.
*/
GtkTreeModel *ui_create_folderview_model(void)
{
	GtkTreeStore *folderListStore = NULL;
	GtkTreeSortable *sortable = NULL;
	GtkTreeModel *sortedFolderModel = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return NULL);
	CHECK_PARAM((email_main_view->folder_manager == NULL),
		    "Input param check failed", return NULL);

	/*There are 4 mandatory folders, so folder numbers 0<-->3 are taken up. */
	email_main_view->folder_manager->model_seq_num = NUM_OF_MANDATORY_FOLDERS;

	/*Create a new list store object and store it in the global variable */
	folderListStore =
	    gtk_tree_store_new(NUM_FOLDER_VIEW_COLS, G_TYPE_STRING, G_TYPE_UINT,
			       G_TYPE_BOOLEAN, G_TYPE_UINT, G_TYPE_UINT,
			       GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT,
			       G_TYPE_BOOLEAN);
	email_main_view->folder_manager->folder_view_model = GTK_TREE_MODEL(folderListStore);

	/*Set the default sorting function, sorting column & sort order */
	sortedFolderModel =
	    gtk_tree_model_sort_new_with_model(email_main_view->folder_manager->folder_view_model);
	sortable = GTK_TREE_SORTABLE(sortedFolderModel);
	gtk_tree_sortable_set_default_sort_func(sortable, ui_sort_folder_compare_func, NULL, NULL);
	gtk_tree_sortable_set_sort_column_id(sortable,
					     GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
					     GTK_SORT_ASCENDING);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return sortedFolderModel;
}

/**
This is a helper function used to create a new list view. It is called during 
main UI window creation for creating list views for folder view & message view.
It is also called during Move_To_Dialog creation.

@param	viewtype	Integer variable to indicate type of view to be created.
@param	createModel	Boolean variable to indicate whether to attach a model 
                        to the newly created list view.
@return GtkWidget*	Pointer to the newly created list view.
*/
GtkWidget *ui_create_view_and_model(gint viewtype, gboolean createModel)
{
	GtkTreeViewColumn *tree_view_column = NULL;
	GtkWidget *header_icon = NULL;
	GtkTreeModel *model = NULL;
	GtkCellRenderer *renderer = NULL;
	GtkWidget *view = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return NULL);

	switch (viewtype) {
	case VIEW_FOLDER_LIST:
		{
			view = gtk_tree_view_new();

			tree_view_column = gtk_tree_view_column_new();
			gtk_tree_view_column_set_spacing(tree_view_column, HILDON_MARGIN_DEFAULT);
			gtk_tree_view_append_column(GTK_TREE_VIEW(view), tree_view_column);

			renderer = gtk_cell_renderer_pixbuf_new();
			gtk_tree_view_column_pack_start(tree_view_column, renderer, FALSE);
			gtk_tree_view_column_add_attribute(tree_view_column, renderer,
							   ATTRIBUTE_PIXBUF, COL_FOLDER_ICON);

			renderer = gtk_cell_renderer_text_new();
			gtk_cell_renderer_set_fixed_size(renderer, -1, LISTVIEW_TEXT_HEIGHT);
			gtk_tree_view_column_pack_start(tree_view_column, renderer, TRUE);
			gtk_tree_view_column_add_attribute(tree_view_column, renderer,
							   ATTRIBUTE_TEXT, COL_DISPLAY_NAME);

			if (createModel) {
				model = ui_create_folderview_model();
				gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
			}

			break;
		}

	case VIEW_MSG_LIST:
		{
			view = gtk_tree_view_new();

			/*Msg icon column */
			/*Fix for bug #23882,missing icons for msg and attachment header columns.                          both the icons are displayed */
			tree_view_column = gtk_tree_view_column_new();
			gtk_tree_view_column_set_clickable(tree_view_column, TRUE);
			header_icon = gtk_image_new_from_icon_name("qgn_list_messagin_mail_unread",
								   GTK_ICON_SIZE_LARGE_TOOLBAR);
			gtk_tree_view_column_set_widget(GTK_TREE_VIEW_COLUMN(tree_view_column),
							header_icon);
			g_object_set(header_icon,ATTRIBUTE_YPAD,HILDON_MARGIN_DEFAULT, NULL);
			gtk_widget_show(header_icon);
			gtk_tree_view_append_column(GTK_TREE_VIEW(view), tree_view_column);

			renderer = gtk_cell_renderer_pixbuf_new();
			gtk_tree_view_column_pack_start(tree_view_column, renderer, FALSE);
			gtk_tree_view_column_set_attributes(tree_view_column, renderer,
							    ATTRIBUTE_PIXBUF, COL_MSG_ICON,
							    ATTRIBUTE_SENSITIVE,
							    COL_CUT_STATUS, NULL);
			gtk_tree_view_column_set_sort_indicator(tree_view_column, TRUE);
			g_signal_connect(G_OBJECT(tree_view_column), "clicked",
					 G_CALLBACK(sort_messages),
					 GINT_TO_POINTER(SORT_BY_READ_STATUS));

			gtk_cell_renderer_set_fixed_size(renderer, MSG_ICON_SIZE, MSG_ICON_SIZE);

			/*From column */
			tree_view_column = gtk_tree_view_column_new();
			gtk_tree_view_column_set_clickable(tree_view_column, TRUE);
			gtk_tree_view_column_set_title(tree_view_column, _("mcen_ia_column_from"));
			gtk_tree_view_column_set_resizable(tree_view_column, TRUE);
			gtk_tree_view_append_column(GTK_TREE_VIEW(view), tree_view_column);

			renderer = gtk_cell_renderer_text_new();
			gtk_cell_renderer_set_fixed_size(renderer, TREE_COLUMN_MAX_WIDTH,
							 LISTVIEW_TEXT_HEIGHT);
			gtk_tree_view_column_pack_start(tree_view_column, renderer, TRUE);
			g_object_set(renderer, ATTRIBUTE_WEIGHT, PANGO_WEIGHT_BOLD,
				     ATTRIBUTE_XALIGN, 0.0, ATTRIBUTE_ELLIPSIZE,
				     PANGO_ELLIPSIZE_END, ATTRIBUTE_ELLIPSIZE_SET, TRUE, NULL);
			gtk_tree_view_column_set_attributes(tree_view_column, renderer,
							    ATTRIBUTE_TEXT,
							    COL_SENDER_RECIPIENT,
							    ATTRIBUTE_WEIGHT_SET,
							    COL_READ_STATUS,
							    ATTRIBUTE_SENSITIVE,
							    COL_CUT_STATUS, NULL);
			gtk_tree_view_column_set_sort_indicator(tree_view_column, TRUE);
			g_signal_connect(G_OBJECT(tree_view_column), "clicked",
					 G_CALLBACK(sort_messages),
					 GINT_TO_POINTER(SORT_BY_ADDRESS));

			/*attachment column */
			tree_view_column = gtk_tree_view_column_new();
			gtk_tree_view_column_set_clickable(tree_view_column, TRUE);
			gtk_tree_view_append_column(GTK_TREE_VIEW(view), tree_view_column);

			header_icon = gtk_image_new_from_icon_name("qgn_list_gene_attacpap",
								   GTK_ICON_SIZE_LARGE_TOOLBAR);
			gtk_tree_view_column_set_widget(GTK_TREE_VIEW_COLUMN(tree_view_column),
							header_icon);
			gtk_widget_show(header_icon);

			renderer = gtk_cell_renderer_pixbuf_new();
			gtk_tree_view_column_pack_start(tree_view_column, renderer, FALSE);
			gtk_tree_view_column_set_attributes(tree_view_column, renderer,
							    ATTRIBUTE_PIXBUF,
							    COL_ATTACHMENT_ICON,
							    ATTRIBUTE_SENSITIVE,
							    COL_CUT_STATUS, NULL);
			gtk_cell_renderer_set_fixed_size(renderer, MSG_ICON_SIZE, MSG_ICON_SIZE);
			gtk_tree_view_column_set_sort_indicator(tree_view_column, TRUE);
			g_signal_connect(G_OBJECT(tree_view_column), "clicked",
					 G_CALLBACK(sort_messages),
					 GINT_TO_POINTER(SORT_BY_ATTACHMENT));

			/*subject column */
			tree_view_column = gtk_tree_view_column_new();
			gtk_tree_view_column_set_title(tree_view_column,
						       _("mcen_ia_column_subject"));
			gtk_tree_view_column_set_clickable(tree_view_column, TRUE);
			gtk_tree_view_column_set_resizable(tree_view_column, TRUE);
			gtk_tree_view_append_column(GTK_TREE_VIEW(view), tree_view_column);

			renderer = gtk_cell_renderer_text_new();
			gtk_cell_renderer_set_fixed_size(renderer, TREE_COLUMN_MAX_WIDTH,
							 LISTVIEW_TEXT_HEIGHT);
			gtk_tree_view_column_pack_start(tree_view_column, renderer, TRUE);
			g_object_set(renderer, ATTRIBUTE_WEIGHT, PANGO_WEIGHT_BOLD,
				     ATTRIBUTE_XALIGN, 0.0, ATTRIBUTE_ELLIPSIZE,
				     PANGO_ELLIPSIZE_END, ATTRIBUTE_ELLIPSIZE_SET, TRUE, NULL);
			gtk_tree_view_column_set_attributes(tree_view_column, renderer,
							    ATTRIBUTE_TEXT, COL_SUBJECT,
							    ATTRIBUTE_WEIGHT_SET,
							    COL_READ_STATUS,
							    ATTRIBUTE_SENSITIVE,
							    COL_CUT_STATUS, NULL);
			gtk_tree_view_column_set_sort_indicator(tree_view_column, TRUE);
			g_signal_connect(G_OBJECT(tree_view_column), "clicked",
					 G_CALLBACK(sort_messages),
					 GINT_TO_POINTER(SORT_BY_SUBJECT));

			/*recv time column */
			tree_view_column = gtk_tree_view_column_new();
			gtk_tree_view_column_set_title(tree_view_column, _("mcen_ia_column_date"));
			gtk_tree_view_column_set_clickable(tree_view_column, TRUE);
			gtk_tree_view_column_set_resizable(tree_view_column, TRUE);
			gtk_tree_view_append_column(GTK_TREE_VIEW(view), tree_view_column);

			renderer = gtk_cell_renderer_text_new();
			gtk_cell_renderer_set_fixed_size(renderer, -1, LISTVIEW_TEXT_HEIGHT);
			gtk_tree_view_column_pack_start(tree_view_column, renderer, FALSE);
			g_object_set(renderer, ATTRIBUTE_WEIGHT, PANGO_WEIGHT_BOLD,
				     ATTRIBUTE_XALIGN, 1.0, ATTRIBUTE_XPAD,
				     HILDON_MARGIN_DEFAULT, NULL);
			gtk_tree_view_column_set_attributes(tree_view_column, renderer,
							    ATTRIBUTE_TEXT, COL_DATE_STATUS,
							    ATTRIBUTE_WEIGHT_SET,
							    COL_READ_STATUS,
							    ATTRIBUTE_SENSITIVE,
							    COL_CUT_STATUS, NULL);
			gtk_tree_view_column_set_sort_indicator(tree_view_column, TRUE);
			g_signal_connect(G_OBJECT(tree_view_column), "clicked",
					 G_CALLBACK(sort_messages), GINT_TO_POINTER(SORT_BY_DATE));

			model = ui_get_sorted_folder_model(INBOX_MODEL_NUM);
			gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
			gtk_tree_selection_set_mode(gtk_tree_view_get_selection
						    (GTK_TREE_VIEW(view)), GTK_SELECTION_MULTIPLE);

			break;
		}

	default:
		break;

	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return view;
}

/**
This is a helper function to get the status string of Outbox message -
Waiting, Failed, etc. Last char in the message-id represents this status.

@param  msgid   message-id string
@return gchar*  outbox msg status string, to be freed by calling function.
*/
static gchar *ui_get_outbox_status_string(const gchar * msgid)
{
	gint outbox_status_bit = -1;
	gchar *status_str = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((msgid == NULL), "Input param check failed", return NULL);
	CHECK_PARAM((strlen(msgid) == 0), "Input param check failed", return NULL);

	/*Last character in the message-id represents current mode of outbox message.
	   Extract it and form the status string. */
	outbox_status_bit = g_ascii_digit_value(msgid[strlen(msgid) - 1]);
	ULOG_DEBUG_F("Outbox status bit = %d\n", outbox_status_bit);
	switch (outbox_status_bit) {
	case OUTBOX_STATUS_WAITING:
		{
			status_str = g_strdup(_("mcen_li_outbox_waiting"));
			break;
		}

	case OUTBOX_STATUS_SENDING:
		{
			status_str = g_strdup(_("mcen_li_outbox_sending"));
			break;
		}

	case OUTBOX_STATUS_FAILED:
		{
			status_str = g_strdup(_("mcen_li_outbox_failed"));
			break;
		}

	case OUTBOX_STATUS_ON_REQUEST:
		{
			status_str = g_strdup(_("mcen_li_outbox_onrequest"));
			break;
		}

	case OUTBOX_STATUS_SUSPENDED:
		{
			status_str = g_strdup(_("mcen_li_outbox_suspended"));
			break;
		}

	default:
		{
			status_str = g_strdup(_("mcen_li_outbox_waiting"));
			break;
		}
	}
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return status_str;
}

/**
This is a helper function to get the string representation of date/time
of a message in the list view.

@param  msg_date  'time_t' format of received time stamp
@return gchar*    string representing date/time of the message,
                  to be freed by calling function.
*/
static gchar *ui_get_date_time_display_string(time_t msg_date)
{
	time_t cur_date;
	gchar cur_date_str[MAX_DATE_LEN];
	gchar msg_date_str[MAX_DATE_LEN];
	gchar *display_str = NULL;
	size_t success1 = 0, success2 = 0;

	time(&cur_date);
	memset(cur_date_str, '\0', MAX_DATE_LEN);
	memset(msg_date_str, '\0', MAX_DATE_LEN);

	/*Get the date values as strings for the message date and current date
	   to compare whether the message is received on current date */
	success1 = strftime(cur_date_str, MAX_DATE_LEN, "%x", localtime(&cur_date));
	success2 = strftime(msg_date_str, MAX_DATE_LEN, "%x", localtime(&msg_date));
	if (success1 == 0 || success2 == 0) {
		ULOG_DEBUG_F("Unable to collect date/time values!\n");
		return NULL;
	}

	/*If the dates match, TIME of the message should be displayed.
	   Otherwise DATE should be displayed. */
	display_str = g_new0(gchar, MAX_DATE_LEN);
	if (strncmp(cur_date_str, msg_date_str, MAX_STR_LEN(cur_date_str, msg_date_str)) == 0) {
		ULOG_DEBUG_F("Message is received on same day\n");
		success1 = strftime(display_str, MAX_DATE_LEN, "%X", localtime(&msg_date));
	} else {
		ULOG_DEBUG_F("Message is not received on same day\n");
		success1 = strftime(display_str, MAX_DATE_LEN, "%x", localtime(&msg_date));
	}
	if (success1 == 0) {
		ULOG_DEBUG_F("Unable to collect date/time values!\n");
		g_free(display_str);
		display_str = NULL;
	}

	ULOG_DEBUG_F("Date-time display str is %s\n", display_str);
	return display_str;
}

/**
This is a helper function used to remove a row, corresponding to a message-id,
from a specified folder view.

@param	msgid		Pointer to the message-id string
@param	modelNum	Sequence number of the the folder storage model
@return gboolean	TRUE if success, else FALSE
*/
gboolean ui_remove_listview_header(const gchar * msgid, gint modelNum)
{
	GtkTreeModel *folder_model = NULL;
	GtkTreeIter iter;
	gchar *msgId = NULL;
	gboolean success = FALSE;
	GtkTreeSelection *message_selection = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((msgid == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->folder_manager == NULL),
		    "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->widgets->no_msgs_label == NULL),
		    "Input param check failed", return FALSE);

	/*Get the folder's list store */
	folder_model = ui_get_unsorted_folder_model(modelNum);
	if (folder_model == NULL) {
		ULOG_DEBUG_F("Storage model missing for model number : %d!\n", modelNum);
		return success;
	}

	message_selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->messageListView));

	if (!gtk_tree_model_get_iter_first(folder_model, &iter)) {
		ULOG_DEBUG_F("Storage model empty for model number : %d!\n", modelNum);
		return success;
	}

	/*Traverse the rows until message-id is matched, delete the row if found. */
	do {
		gtk_tree_model_get(folder_model, &iter, COL_MSG_ID, &msgId, -1);

		if (msgId != NULL && ui_compare_utf8_strings(msgid, msgId) == 0) {
			ULOG_DEBUG_F("Row found. Deleting it from the store.\n");
			/*Block tree selection functions, they may cause trouble if
			   the folder is currently on display */
			g_signal_handlers_block_by_func(G_OBJECT(message_selection), (gpointer)
							ui_messageview_selection_changed_cb, NULL);
			gtk_list_store_remove(GTK_LIST_STORE(folder_model), &iter);
			g_signal_handlers_unblock_by_func(G_OBJECT(message_selection), (gpointer)
							  ui_messageview_selection_changed_cb,
							  NULL);
			success = TRUE;
			g_free(msgId);
			break;
		}
		g_free(msgId);
	}
	while (gtk_tree_model_iter_next(folder_model, &iter));

	if (success == FALSE)
		ULOG_DEBUG_F("No row with message-id '%s' found in Outbox!\n", msgid);

	/*If folder view is empty, update the appropriate menus & tool bar buttons */
	if (email_main_view->folder_manager->currently_displayed_model == modelNum) {
		if (gtk_tree_model_iter_n_children(folder_model, NULL) == 0) {
			/*Show 'No messages to show' label */
			gtk_widget_show(email_main_view->widgets->no_msgs_label);
			gtk_widget_hide(email_main_view->widgets->message_scrollwindow);
			gtk_tree_view_set_model(GTK_TREE_VIEW
						(email_main_view->widgets->messageListView), NULL);
			/*Revoke marking mode if necessary */
			if (email_main_view->tags->marking_mode == TRUE) {
				ULOG_DEBUG_F("Revoking marking mode\n");
				email_main_view->tags->marking_mode = FALSE;
				ui_invoke_marking_mode(FALSE);
			}
			/*Update menus and toolbar buttons */
			ui_update_folder_specific_widgets((guint) modelNum);
		}
	}
	return success;
}

/**
This is a helper function to check if a message is already opened.

@param  msgid    Message-id of the current message
@return boolean  TRUE --> Message is opened, FALSE --> Otherwise
*/
static gboolean ui_is_message_already_opened(const gchar * msgid)
{
	GSList *node = NULL;
	gboolean in_use = FALSE;
	OpenedWindowData *open_window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((msgid == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return FALSE);

	for (node = email_main_view->open_windows; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) node->data;
		if (open_window->msgid == NULL || open_window->window_ptr == NULL) {
			ULOG_DEBUG_F("Invalid data in open windows list\n");
		} else if (ui_compare_utf8_strings(open_window->msgid, msgid) == 0) {
			ULOG_DEBUG_F("Message already opened!\n");
			in_use = TRUE;
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return in_use;
}

/**
 This is a helper function to get the listview icon for a smime message.

 @param icon_num is the gint representing a unique number for a icon.
 @param  smime_type   Type of the smime message
 @return GdkPixbuf*  Pointer to the pixbuf icon
 */
static GdkPixbuf *ui_get_smime_read_header_msg_icon(gint icon_num, gint smime_type)
{
	GdkPixbuf *msg_icon = NULL;
	ULOG_DEBUG_L("[%s] : starts here :", __FUNCTION__);

	switch (icon_num) {
	case HEADER_READ_SIGN_LOW:
		if (smime_type == SMIME_SIGN_ENCRYPT) {
			CHECK_PARAM((email_main_view->icons->hdr_encrypted_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_encrypted_icon,
							"qgn_list_messagin_hdr_encrypted",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_encrypted_icon;
		} else {
			CHECK_PARAM((email_main_view->icons->hdr_signed_low_icon ==
				     NULL), "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_signed_low_icon,
							"qgn_list_messagin_hdr_signed_low",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_signed_low_icon;
		}
		break;
	case HEADER_READ_SIGN_HIGH:
		CHECK_PARAM((email_main_view->icons->hdr_signed_high_icon == NULL),
			    "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						hdr_signed_high_icon,
						"qgn_list_messagin_hdr_signed_high",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->hdr_signed_high_icon;
		break;
	case HEADER_READ_SIGN_NORMAL:
		if (smime_type == SMIME_SIGN_ENCRYPT) {
			CHECK_PARAM((email_main_view->icons->hdr_encrypted_high_icon ==
				     NULL), "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_encrypted_high_icon,
							"qgn_list_messagin_hdr_encrypted_high",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_encrypted_high_icon;
		} else {
			CHECK_PARAM((email_main_view->icons->hdr_signed_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_signed_icon,
							"qgn_list_messagin_hdr_signed",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_signed_icon;
		}
		break;
	case HEADER_READ_ENCRYPT_LOW:
	case HEADER_READ_SIGN_ENCRYPT_LOW:
		CHECK_PARAM((email_main_view->icons->hdr_encrypted_low_icon == NULL),
			    "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						hdr_encrypted_low_icon,
						"qgn_list_messagin_hdr_encrypted_low",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->hdr_encrypted_low_icon;
		break;
	case HEADER_READ_ENCRYPT_HIGH:
		CHECK_PARAM((email_main_view->icons->hdr_encrypted_high_icon ==
			     NULL), "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						hdr_encrypted_high_icon,
						"qgn_list_messagin_hdr_encrypted_high",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->hdr_encrypted_high_icon;
		break;
	case HEADER_READ_ENCRYPT_NORMAL:
		CHECK_PARAM((email_main_view->icons->hdr_encrypted_icon == NULL),
			    "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						hdr_encrypted_icon,
						"qgn_list_messagin_hdr_encrypted", MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->hdr_encrypted_icon;
		break;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return msg_icon;
}

/**
 This is a helper function to get the listview icon for a smime message.

 @param icon_num is the gint representing a unique number for a icon.
 @param  smime_type   Type of the smime message
 @return GdkPixbuf*  Pointer to the pixbuf icon
 */
static GdkPixbuf *ui_get_smime_unread_header_msg_icon(gint icon_num, gint smime_type)
{
	GdkPixbuf *msg_icon = NULL;
	ULOG_DEBUG_L("[%s] : starts here :", __FUNCTION__);

	switch (icon_num) {
	case HEADER_UNREAD_SIGN_LOW:
		if (smime_type == SMIME_SIGN_ENCRYPT) {
			CHECK_PARAM((email_main_view->icons->hdr_unread_encrypted_icon ==
				     NULL), "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_unread_encrypted_icon,
							"qgn_list_messagin_hdr_unread_encrypted",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_unread_encrypted_icon;
		} else {
			CHECK_PARAM((email_main_view->icons->
				     hdr_unread_signed_low_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_unread_signed_low_icon,
							"qgn_list_messagin_hdr_unread_signed_low",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_unread_signed_low_icon;
		}
		break;
	case HEADER_UNREAD_SIGN_HIGH:
		CHECK_PARAM((email_main_view->icons->hdr_unread_signed_high_icon ==
			     NULL), "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						hdr_unread_signed_high_icon,
						"qgn_list_messagin_hdr_unread_signed_high",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->hdr_unread_signed_high_icon;
		break;
	case HEADER_UNREAD_SIGN_NORMAL:
		if (smime_type == SMIME_SIGN_ENCRYPT) {
			CHECK_PARAM((email_main_view->icons->
				     hdr_unread_encrypted_high_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_unread_encrypted_high_icon,
							"qgn_list_messagin_hdr_unread_encrypted_high",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_unread_encrypted_high_icon;
		} else {
			CHECK_PARAM((email_main_view->icons->hdr_unread_signed_icon ==
				     NULL), "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_unread_signed_icon,
							"qgn_list_messagin_hdr_unread_signed",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_unread_signed_icon;
		}
		break;
	case HEADER_UNREAD_ENCRYPT_LOW:
	case HEADER_UNREAD_SIGN_ENCRYPT_LOW:
		CHECK_PARAM((email_main_view->icons->hdr_unread_encrypted_low_icon ==
			     NULL), "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						hdr_unread_encrypted_low_icon,
						"qgn_list_messagin_hdr_unread_encrypted_low",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->hdr_unread_encrypted_low_icon;
		break;
	case HEADER_UNREAD_ENCRYPT_HIGH:
		CHECK_PARAM((email_main_view->icons->
			     hdr_unread_encrypted_high_icon == NULL),
			    "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						hdr_unread_encrypted_high_icon,
						"qgn_list_messagin_hdr_unread_encrypted_high",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->hdr_unread_encrypted_high_icon;
		break;
	case HEADER_UNREAD_ENCRYPT_NORMAL:
		CHECK_PARAM((email_main_view->icons->hdr_unread_encrypted_icon ==
			     NULL), "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						hdr_unread_encrypted_icon,
						"qgn_list_messagin_hdr_unread_encrypted",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->hdr_unread_encrypted_icon;
		break;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return msg_icon;
}

/**
 This is a helper function to get the listview icon for a smime message.

 @param icon_num is the gint representing a unique number for a icon.
 @param  smime_type   Type of the smime message
 @return GdkPixbuf*  Pointer to the pixbuf icon
 */
static GdkPixbuf *ui_get_smime_read_nonheader_msg_icon(gint icon_num, gint smime_type)
{
	GdkPixbuf *msg_icon = NULL;
	ULOG_DEBUG_L("[%s] : starts here :", __FUNCTION__);

	switch (icon_num) {
	case BODY_UNREAD_SIGN_LOW:
		if (smime_type == SMIME_SIGN_ENCRYPT) {
			CHECK_PARAM((email_main_view->icons->mail_encrypted_read_icon ==
				     NULL), "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_encrypted_read_icon,
							"qgn_list_messagin_mail_encrypted_read",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_encrypted_read_icon;
		} else {
			CHECK_PARAM((email_main_view->icons->mail_signed_read_low_icon ==
				     NULL), "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_signed_read_low_icon,
							"qgn_list_messagin_mail_signed_read_low",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_signed_read_low_icon;
		}
		break;
	case BODY_UNREAD_SIGN_HIGH:
		CHECK_PARAM((email_main_view->icons->mail_signed_read_high_icon ==
			     NULL), "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						mail_signed_read_high_icon,
						"qgn_list_messagin_mail_signed_read_high",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->mail_signed_read_high_icon;
		break;
	case BODY_UNREAD_SIGN_NORMAL:
		if (smime_type == SMIME_SIGN_ENCRYPT) {
			CHECK_PARAM((email_main_view->icons->
				     mail_encrypted_read_high_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_encrypted_read_high_icon,
							"qgn_list_messagin_mail_encrypted_read_high",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_encrypted_read_high_icon;
		} else {
			CHECK_PARAM((email_main_view->icons->mail_signed_read_icon ==
				     NULL), "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_signed_read_icon,
							"qgn_list_messagin_mail_signed_read",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_signed_read_icon;
		}
		break;
	case BODY_UNREAD_ENCRYPT_LOW:
	case BODY_UNREAD_SIGN_ENCRYPT_LOW:
		CHECK_PARAM((email_main_view->icons->mail_encrypted_read_low_icon ==
			     NULL), "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						mail_encrypted_read_low_icon,
						"qgn_list_messagin_mail_encrypted_read_low",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->mail_encrypted_read_low_icon;
		break;
	case BODY_UNREAD_ENCRYPT_HIGH:
		CHECK_PARAM((email_main_view->icons->mail_encrypted_read_high_icon ==
			     NULL), "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						mail_encrypted_read_high_icon,
						"qgn_list_messagin_mail_encrypted_read_high",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->mail_encrypted_read_high_icon;
		break;
	case BODY_UNREAD_ENCRYPT_NORMAL:
		CHECK_PARAM((email_main_view->icons->mail_encrypted_read_icon ==
			     NULL), "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						mail_encrypted_read_icon,
						"qgn_list_messagin_mail_encrypted_read",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->mail_encrypted_read_icon;
		break;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return msg_icon;
}

/**
 This is a helper function to get the listview icon for a smime message.

 @param icon_num is the gint representing a unique number for a icon.
 @param  smime_type   Type of the smime message
 @return GdkPixbuf*  Pointer to the pixbuf icon
 */
static GdkPixbuf *ui_get_smime_unread_nonheader_msg_icon(gint icon_num, gint smime_type)
{
	GdkPixbuf *msg_icon = NULL;
	ULOG_DEBUG_L("[%s] : starts here :", __FUNCTION__);

	switch (icon_num) {
	case BODY_READ_SIGN_LOW:
		if (smime_type == SMIME_SIGN_ENCRYPT) {
			CHECK_PARAM((email_main_view->icons->
				     mail_encrypted_unread_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_encrypted_unread_icon,
							"qgn_list_messagin_mail_encrypted_unread",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_encrypted_unread_icon;
		} else {
			CHECK_PARAM((email_main_view->icons->
				     mail_signed_unread_low_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_signed_unread_low_icon,
							"qgn_list_messagin_mail_signed_unread_low",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_signed_unread_low_icon;
		}
		break;
	case BODY_READ_SIGN_HIGH:
		CHECK_PARAM((email_main_view->icons->mail_signed_unread_high_icon ==
			     NULL), "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						mail_signed_unread_high_icon,
						"qgn_list_messagin_mail_signed_unread_high",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->mail_signed_unread_high_icon;
		break;
	case BODY_READ_SIGN_NORMAL:
		if (smime_type == SMIME_SIGN_ENCRYPT) {
			CHECK_PARAM((email_main_view->icons->
				     mail_encrypted_unread_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_encrypted_unread_icon,
							"qgn_list_messagin_mail_encrypted_unread",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_encrypted_unread_icon;
		} else {
			CHECK_PARAM((email_main_view->icons->mail_signed_unread_icon ==
				     NULL), "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_signed_unread_icon,
							"qgn_list_messagin_mail_signed_unread",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_signed_unread_icon;
		}
		break;
	case BODY_READ_ENCRYPT_LOW:
	case BODY_READ_SIGN_ENCRYPT_LOW:
		CHECK_PARAM((email_main_view->icons->
			     mail_encrypted_unread_low_icon == NULL),
			    "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						mail_encrypted_unread_low_icon,
						"qgn_list_messagin_mail_encrypted_unread_low",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->mail_encrypted_unread_low_icon;
		break;
	case BODY_READ_ENCRYPT_HIGH:
		CHECK_PARAM((email_main_view->icons->
			     mail_encrypted_unread_high_icon == NULL),
			    "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						mail_encrypted_unread_high_icon,
						"qgn_list_messagin_mail_encrypted_unread_high",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->mail_encrypted_unread_high_icon;
		break;
	case BODY_READ_ENCRYPT_NORMAL:
		CHECK_PARAM((email_main_view->icons->mail_encrypted_unread_icon ==
			     NULL), "Pixbuf not loaded yet",
			    ui_load_icon_pixbuf(&email_main_view->icons->
						mail_encrypted_unread_icon,
						"qgn_list_messagin_mail_encrypted_unread",
						MSG_ICON_SIZE));
		msg_icon = email_main_view->icons->mail_encrypted_unread_icon;
		break;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return msg_icon;
}

/**
 This is a helper function to get the listview icon for a smime message.

 @param icon_num is the gint representing a unique number for a icon.
 @param  body_downloaded  Flag indicating whether message body is downloaded
 @param  read_status  Message read/unread status
 @param  smime_type   Type of the smime message
 @return GdkPixbuf*  Pointer to the pixbuf icon
 */
static GdkPixbuf *ui_get_smime_msg_display_icon(gint icon_num, gboolean body_downloaded,
						gboolean read_status, gint smime_type)
{
	GdkPixbuf *msg_icon = NULL;
	ULOG_DEBUG_L("[%s] : starts here :", __FUNCTION__);

	if (!body_downloaded) {
		if (read_status == FALSE) {
			msg_icon = ui_get_smime_read_header_msg_icon(icon_num, smime_type);
		} else {
			msg_icon = ui_get_smime_unread_header_msg_icon(icon_num, smime_type);
		}
	} else {
		if (read_status == FALSE) {
			msg_icon = ui_get_smime_read_nonheader_msg_icon(icon_num, smime_type);
		} else {
			msg_icon = ui_get_smime_unread_nonheader_msg_icon(icon_num, smime_type);
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :", __FUNCTION__);
	return msg_icon;
}

/**
This is a helper function to obtain list view icon for a header-only message
based on the priority and read/unread status.

@param  priority  Message priority
@param  read_status  Message read/unread status
@return GdkPixbuf  Pointer to the pixbuf icon
*/
static GdkPixbuf *ui_get_header_msg_icon(gint priority, gboolean read_status)
{
	GdkPixbuf *msg_icon = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	if (read_status == TRUE) {
		switch (priority) {
		case LOW:
			CHECK_PARAM((email_main_view->icons->hdr_low_unread_icon ==
				     NULL), "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_low_unread_icon,
							"qgn_list_messagin_hdr_low_unread",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_low_unread_icon;
			break;
		case HIGH:
			CHECK_PARAM((email_main_view->icons->hdr_high_unread_icon ==
				     NULL), "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_high_unread_icon,
							"qgn_list_messagin_hdr_high_unread",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_high_unread_icon;
			break;
		case NORMAL:
		default:
			CHECK_PARAM((email_main_view->icons->hdr_unread_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_unread_icon,
							"qgn_list_messagin_hdr_unread",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_unread_icon;
			break;
		}
	} else {
		switch (priority) {
		case LOW:
			CHECK_PARAM((email_main_view->icons->hdr_low_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_low_icon,
							"qgn_list_messagin_hdr_low",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_low_icon;
			break;
		case HIGH:
			CHECK_PARAM((email_main_view->icons->hdr_high_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_high_icon,
							"qgn_list_messagin_hdr_high",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_high_icon;
			break;
		case NORMAL:
		default:
			CHECK_PARAM((email_main_view->icons->hdr_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							hdr_icon,
							"qgn_list_messagin_hdr", MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->hdr_icon;
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return msg_icon;
}

/**
This is a helper function to obtain list view icon for a non-header-only
message based on the priority and read/unread status.

@param  priority  Message priority
@param  read_status  Message read/unread status
@return GdkPixbuf  Pointer to the pixbuf icon
*/
static GdkPixbuf *ui_get_nonheader_msg_icon(gint priority, gboolean read_status)
{
	GdkPixbuf *msg_icon = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	if (read_status == TRUE) {
		switch (priority) {
		case LOW:
			CHECK_PARAM((email_main_view->icons->mail_low_unread_icon ==
				     NULL), "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_low_unread_icon,
							"qgn_list_messagin_mail_low_unread",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_low_unread_icon;
			break;
		case HIGH:
			CHECK_PARAM((email_main_view->icons->mail_high_unread_icon ==
				     NULL), "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_high_unread_icon,
							"qgn_list_messagin_mail_high_unread",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_high_unread_icon;
			break;
		case NORMAL:
		default:
			CHECK_PARAM((email_main_view->icons->mail_unread_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_unread_icon,
							"qgn_list_messagin_mail_unread",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_unread_icon;
			break;
		}
	} else {
		switch (priority) {
		case LOW:
			CHECK_PARAM((email_main_view->icons->mail_low_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_low_icon,
							"qgn_list_messagin_mail_low",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_low_icon;
			break;
		case HIGH:
			CHECK_PARAM((email_main_view->icons->mail_high_icon == NULL),
				    "Pixbuf not loaded yet",
				    ui_load_icon_pixbuf(&email_main_view->icons->
							mail_high_icon,
							"qgn_list_messagin_mail_high",
							MSG_ICON_SIZE));
			msg_icon = email_main_view->icons->mail_high_icon;
			break;
		case NORMAL:
		default:
			msg_icon = email_main_view->icons->mail_icon;
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return msg_icon;
}

/**
This is a helper function to obtain list view icon based on the
message type, priority and read/unread status.

@param  body_downloaded  Flag indicating whether message body is downloaded
@param  priority  Message priority
@param  read_status  Message read/unread status
@return GdkPixbuf  Pointer to the pixbuf icon
*/
GdkPixbuf *ui_get_message_display_icon(gboolean body_downloaded, gint priority,
				       gboolean read_status, gint smime_type)
{
	GdkPixbuf *msg_icon = NULL;
	gint icon_num = 0;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	if (smime_type != SMIME_NONE) {
		icon_num = priority + smime_type;
	}

	/*Assign default icon */
	CHECK_PARAM((email_main_view->icons->mail_icon == NULL),
		    "Pixbuf not loaded yet",
		    ui_load_icon_pixbuf(&email_main_view->icons->mail_icon,
					"qgn_list_messagin_mail", MSG_ICON_SIZE));
	msg_icon = email_main_view->icons->mail_icon;

	/*Get icon based on required params */
	if (!body_downloaded) {
		msg_icon = ui_get_header_msg_icon(priority, read_status);
	} else {
		msg_icon = ui_get_nonheader_msg_icon(priority, read_status);
	}

	if (smime_type != SMIME_NONE && smime_type != SMIME_INTERNAL) {
		msg_icon =
		    ui_get_smime_msg_display_icon(icon_num, body_downloaded, read_status,
						  smime_type);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return msg_icon;
}

/**
This is a helper function to prepare a collation key from 'Subject'
string. It ignores "Re:" and "Fw:" prefixes while preparing the collation
key.

@param  subject  'Subject' string
@return gchar*   Collation key of 'Subject' string
*/
static gchar *ui_get_subject_collate_key(const gchar * subject)
{
	gchar *subject_key = NULL;
	gint offset = 0;

	if (subject == NULL) {
		return NULL;
	}

	if (has_prefix_insensitive(subject, email_main_view->tags->reply_indicator)) {
		offset = email_main_view->tags->reply_offset;
	} else if (has_prefix_insensitive(subject, email_main_view->tags->fwd_indicator)) {
		offset = email_main_view->tags->fwd_offset;
	}

	subject_key = g_utf8_collate_key(subject + offset, -1);

	return subject_key;
}

/**
This is a helper function used to replace 'double-quote' characters
with spaces in address strings.

@param  address  String to be modified
@return void
*/
static void ui_format_addr_field(gchar ** address)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((address == NULL || *address == NULL), "Input param check failed", return);
	CHECK_PARAM((g_utf8_strlen(*address, -1) == 0), "Input param check failed", return);

	if (g_utf8_strchr(*address, -1, '"') != NULL) {
		*address = g_strdelimit(*address, "\"", ' ');
		*address = g_strstrip(*address);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function to get the 'Subject' field to be displayed in the
list view.

@param  subject       'Subject' field string
@param  field_absent  Boolean flag to update if the field is empty
@return gchar*        Display string
*/
static gchar *ui_get_message_subject_display_string(gchar * subject, gboolean * field_absent)
{
	gchar *display_str = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((field_absent == NULL), "Input param check failed", return NULL);

	/*If subject field is empty, display "(No subject)" string */
	if (subject == NULL) {
		display_str = _("mail_va_no_subject");
		*field_absent = TRUE;
	} else {
		display_str = subject;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return display_str;
}

/**
This is a helper function to get the display time of a message for displaying
in the list view, based on the folder message is in.

@param  header     Pointer to the MsgHeader structure to be used for loading
                   row data
@param  model_num  Sequence number of the the folder storage model
@return gint       Display time in seconds
*/
static gint ui_get_message_display_time(MsgHeader * header, guint model_num)
{
	gint display_time = -1;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((header == NULL), "Input param check failed", return 0);

	/*DATE : For Sent folder items use 'sent time' (date_t).
	   For others, use 'received time' (rx_date) if present
	   otherwise use 'created time' (mtime) */
	if (model_num == SENT_MODEL_NUM) {
		display_time = (gint) header->date_t;
	} else if (header->rx_date != 0) {
		display_time = (gint) header->rx_date;
	} else {
		display_time = (gint) header->mtime;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return display_time;
}

/**
This is a helper function to check if the message has attachments,
and update the display icon and flag accordingly. If message Smime type
is not SMIME_NONE attachment icon is shown always.

@param  msg_tags         Integer representing message flags
@param  smime_type       SMime type of the message
@param  display_icon     Attachment icon to be rendered
@param  attachment_flag  Flag indicating presence of attachments
@return void
*/
static void
ui_get_message_attachment_status(gint msg_tags, SMimeType smime_type,
				 GdkPixbuf ** display_icon, gboolean * attachment_flag)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((display_icon == NULL), "Input param check failed", return);
	CHECK_PARAM((attachment_flag == NULL), "Input param check failed", return);

	if ((msg_tags & ATTACHMENT_STATUS_BIT) == ATTACHMENT_STATUS_BIT || smime_type != SMIME_NONE) {
		if (email_main_view->icons->attachment_icon == NULL) {
			ui_load_icon_pixbuf(&email_main_view->icons->attachment_icon,
					    "qgn_list_gene_attacpap", MSG_ICON_SIZE);
		}
		*display_icon = email_main_view->icons->attachment_icon;
		*attachment_flag = TRUE;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function to check if the message is purgeable.

@param  model_num  Sequence number of the folder storage model
@param  protocol   Receiving protocol
@param  attachment_flag Whether message has any attachments
@param  msg_flags  Internal message flags
@param  smime_type SMIME type of the message
@return boolean    TRUE --> Message is purgeable, FALSE --> Otherwise
*/
static gboolean
ui_get_purge_allowed_status(gint modelNum, gint protocol,
			    gboolean attachment_flag, MsgFlags msg_flags, gint smime_type)
{
	gboolean purge_allowed = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((modelNum == OUTBOX_MODEL_NUM), "Purge not allowed for Outbox", return FALSE);
	CHECK_PARAM((modelNum == DRAFT_MODEL_NUM),
		    "Purge not allowed for Drafts folder", return FALSE);
	CHECK_PARAM((attachment_flag == FALSE), "No attachments to purge", return FALSE);
	CHECK_PARAM((smime_type != SMIME_NONE),
		    "Purge not allowed for signed/encrypted mails", return FALSE);

	if (modelNum == INBOX_MODEL_NUM) {
		if (protocol == PROTO_IMAP4 && MSG_IS_PURGABLE(msg_flags)) {
			purge_allowed = TRUE;
		}
	} else {
		purge_allowed = TRUE;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return purge_allowed;
}

/**
This is a helper function used to add a new row to a specified folder view.

@param  header      Pointer to the MsgHeader structure to be used for loading
                    row data
@param  modelNum    Sequence number of the the folder storage model
@param  set_folder_to_view  Whether to set current folder model to the listview
@return gboolean    TRUE if success, else FALSE
*/
gboolean ui_add_listview_header(MsgHeader * header, guint modelNum, gboolean set_folder_to_view)
{
	GtkListStore *listStore = NULL;
	gchar *date_status = NULL, *subject = NULL, *send_recp = NULL;
	gboolean fully_downloaded = TRUE, remote_copy = FALSE, in_use = FALSE;
	gboolean editable_status = FALSE, body_downloaded = TRUE, read_status = FALSE;
	GtkTreeIter iter1;
	guint msgid_hash = 0;
	gint timeInSecs = 0;
	GdkPixbuf *msg_icon = NULL, *attachment_icon = NULL;
	gchar *msg_id_key = NULL, *send_recp_key = NULL, *subject_key = NULL;
	gint msg_priority = NORMAL;
	gboolean no_subject = FALSE, purge_allowed = FALSE, attachment_flag = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((header == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((header->msgid == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->folder_manager == NULL),
		    "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->widgets->no_msgs_label == NULL),
		    "Input param check failed", return FALSE);

	/*Get the folder's list store */
	listStore = GTK_LIST_STORE(ui_get_unsorted_folder_model(modelNum));
	CHECK_PARAM((listStore == NULL), "Folder list store missing!", return FALSE);

	/* Get subject and time fields for displaying */
	subject = ui_get_message_subject_display_string(header->subject, &no_subject);
	timeInSecs = ui_get_message_display_time(header, modelNum);

	/*For outbox view show 'status' otherwise 'date' */
	if (modelNum == OUTBOX_MODEL_NUM) {
		date_status = ui_get_outbox_status_string(header->msgid);
	} else {
		date_status = ui_get_date_time_display_string(timeInSecs);
	}

	/*Set SENDER_RECIPIENT column depending on folder type
	   For OUTBOX, SENT, DRAFTS it will be TO field,
	   For INBOX and user folders it will be FROM field. */
	ui_format_addr_field(&header->to);
	ui_format_addr_field(&header->cc);
	ui_format_addr_field(&header->from);
	switch (modelNum) {
	case OUTBOX_MODEL_NUM:
	case SENT_MODEL_NUM:
	case DRAFT_MODEL_NUM:
		{
			send_recp = header->to;
			break;
		}
	default:
		{
			if (header->from != NULL) {
				if (strncmp
				    ((header->from), DEFAULT_ADDRESS,
				     MAX_STR_LEN((header->from), DEFAULT_ADDRESS)) == 0
				    && email_main_view->account_manager->configured_accounts ==
				    NULL) {
					g_free(header->from);
					header->from = NULL;
				}
				send_recp = header->from;
			}
			break;
		}
	}

	if (modelNum == INBOX_MODEL_NUM) {
		fully_downloaded = header->msgtag.fullydownloaded;
		body_downloaded = header->msgtag.bodydownloaded;
		/*if the message is fully downloaded set the body bownloaded to true */
		if (fully_downloaded == TRUE)
			body_downloaded = TRUE;
		remote_copy = header->msgtag.remote_copy;
	}

	/*Check if this message has been opened earlier
	   If so update IN_USE_STATUS accordingly */
	in_use = ui_is_message_already_opened(header->msgid);

	/*Check the editable status of the message */
	CHECK_PARAM(((header->flags.perm_flags & EDITABLE_STATUS_BIT) ==
		     EDITABLE_STATUS_BIT), "Message is editable", editable_status = TRUE);

	/*Check the read status of the message */
	CHECK_PARAM(((header->flags.perm_flags & READ_STATUS_BIT) ==
		     READ_STATUS_BIT), "Message read status bit set", read_status = TRUE);

	/*Assign message icon */
	msg_icon =
	    ui_get_message_display_icon(body_downloaded,
					header->recv_priority, read_status, header->smime_type);

	/*Get message attachment status */
	ui_get_message_attachment_status(header->flags.perm_flags,
					 header->smime_type, &attachment_icon, &attachment_flag);

	/*Get current folder's sorting order */
	gtk_tree_model_foreach(email_main_view->folder_manager->folder_view_model,
			       ui_get_folder_sort_order, &modelNum);

	CHECK_PARAM((header->recv_priority != 0), "Setting message priority",
		    msg_priority = header->recv_priority);

	/*Prepare collation key strings used for sorting */
	msg_id_key = g_utf8_collate_key(header->msgid, -1);
	msgid_hash = g_str_hash(header->msgid);

	CHECK_PARAM((send_recp != NULL), "Preparing Subject key",
		    send_recp_key = g_utf8_collate_key(send_recp, -1));
	subject_key = ui_get_subject_collate_key(subject);

	/*Check if purging is allowed for this message
	   Purging is allowed only for -
	   IMAP msgs in Inbox that have local attachments
	   Msgs with attachments in Sent and user defined folders */
	purge_allowed =
	    ui_get_purge_allowed_status(modelNum, header->msgtag.protocol,
					attachment_flag, header->flags, header->smime_type);

	/*Append the new message info into a new row in the folder */
	//gtk_list_store_append(listStore, &iter1);
	//gtk_list_store_set(listStore, &iter1, COL_MSG_ID, header->msgid,
	gtk_list_store_insert_with_values(listStore, &iter1, G_MAXINT, COL_MSG_ID, header->msgid,
					  COL_FROM_ADDR, header->from, COL_TO_ADDR, header->to,
					  COL_CC_ADDR, header->cc, COL_BCC_ADDR, header->bcc,
					  COL_SUBJECT, subject, COL_REPLY_TO, header->replyto,
					  COL_DISPLAY_DATE, timeInSecs,
					  COL_SENT_DATE, header->date_t, COL_RECV_DATE,
					  header->rx_date, COL_CREATED_DATE, header->mtime,
					  COL_DATE_STATUS, date_status, COL_SIZE, header->size,
					  COL_PRIORITY, msg_priority, COL_READ_STATUS, read_status,
					  COL_MARK_STATUS, FALSE, COL_RECV_PROTOCOL,
					  header->msgtag.protocol, COL_MSG_TYPE,
					  header->msgtag.recvtype, COL_FULLY_DOWNLOADED,
					  fully_downloaded, COL_BODY_DOWNLOADED, body_downloaded,
					  COL_REMOTE_COPY, remote_copy, COL_IN_USE_STATUS, in_use,
					  COL_ACTION_AFTER_RETRIEVAL, NO_ACTION_AFTER_RETRIEVAL,
					  COL_READ_RECEIPT, header->read_receipt_request,
					  COL_COPY_TO_ADDR, header->copy_to_own_address,
					  COL_EDITABLE_STATUS, editable_status,
					  COL_ATTACHMENT_PRESENT, FALSE, COL_MSG_ICON, msg_icon,
					  COL_SENDER_RECIPIENT, send_recp, COL_SMIME_TYPE,
					  header->smime_type,
					  COL_MSG_ID_KEY, msg_id_key, COL_SENDER_RECIPIENT_KEY,
					  send_recp_key, COL_SUBJECT_KEY, subject_key,
					  COL_NULL_SUBJECT, no_subject, COL_ATTACHMENT_ICON,
					  attachment_icon, COL_ATTACHMENT_PRESENT, attachment_flag,
					  COL_PURGE_ALLOW, purge_allowed, COL_CUT_STATUS, TRUE,
					  COL_MSGID_HASH, msgid_hash, -1);

	/*Print Header data */
	ULOG_DEBUG_F("Header Info -->\n");
	ULOG_DEBUG_F("MsgId = %s\n", header->msgid);
	ULOG_DEBUG_F("From = %s\n", header->from);
	ULOG_DEBUG_F("To = %s\n", header->to);
	ULOG_DEBUG_F("Cc = %s\n", header->cc);
	ULOG_DEBUG_F("Bcc = %s\n", header->bcc);
	ULOG_DEBUG_F("Subject = %s\n", header->subject);
	ULOG_DEBUG_F("date_status = %s\n", date_status);
	ULOG_DEBUG_F("Size = %d\n", (gint) header->size);
	ULOG_DEBUG_F("Recv Protocol = %d\n", header->msgtag.protocol);
	ULOG_DEBUG_F("Msg Type = %d\n", header->msgtag.recvtype);
	ULOG_DEBUG_F("Fully downloaded = %d\n", header->msgtag.fullydownloaded);
	ULOG_DEBUG_F("Remote copy = %d\n", header->msgtag.remote_copy);
	ULOG_DEBUG_F("Priority = %d\n", header->recv_priority);

	/*Set the model to the list view, if required */
	if (set_folder_to_view) {
		ui_set_folder_model_to_msgview(modelNum, TRUE);
	}

	g_free(date_status);
	g_free(send_recp_key);
	g_free(msg_id_key);
	g_free(subject_key);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return TRUE;
}

/**
This is a helper function to check if a given message is not present
in Inbox list view, if not fetch the corresponding message header from
engine and add it to the list view.

@param  msgid   Message-id string
@return boolean TRUE-->Message not present in Inbox, and successfully added.
                FALSE-->Message already present in Inbox,
                or header fetching failed.
*/
gboolean ui_add_msg_if_not_present(gchar * msgid)
{
	GtkTreeModel *model = NULL;
	GSList *msg_id_list = NULL, *msg_hdr_list = NULL;
	gboolean ret_val = FALSE;
	gchar *parent_folder = NULL, *folder_name = NULL;
	gint model_num = -1;
	guint msgid_hash = 0;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msgid == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->message_handler == NULL),
		    "Input param check failed", return FALSE);

	/*Get the folder name from msgid */
	if (ui_get_msg_folder_names(msgid, &parent_folder, &folder_name) == FALSE) {
		ULOG_DEBUG_F("Unable to resolve folder from msg-id\n");
		return ret_val;
	}

	model_num = ui_get_folder_model_number(folder_name, parent_folder);
	g_free(folder_name);
	g_free(parent_folder);

	/*Simply return if the message is not meant for Inbox */
	if (model_num != INBOX_MODEL_NUM) {
		ULOG_DEBUG_F("Message is not meant for Inbox");
		return ret_val;
	}

	model = ui_get_unsorted_folder_model(INBOX_MODEL_NUM);
	email_main_view->message_handler->is_message_present = FALSE;
	msgid_hash = g_str_hash(msgid);
	gtk_tree_model_foreach(model, ui_is_header_already_present, &msgid_hash);
	if (email_main_view->message_handler->is_message_present == TRUE) {
		email_main_view->message_handler->is_message_present = FALSE;
		ULOG_DEBUG_F("Header already present in Inbox!\n");
		return ret_val;
	}

	msg_id_list = g_slist_append(msg_id_list, g_strdup(msgid));
	msg_hdr_list = ui_dbus_get_msgheader_list(msg_id_list);
	if (msg_hdr_list != NULL) {
		ULOG_DEBUG_F("Fetched msg '%s'\n", msgid);
		ui_update_folder_view_cb((MsgHeader *) msg_hdr_list->data, TRUE, TRUE);
		g_slist_free(msg_hdr_list);
		ret_val = TRUE;
	} else {
		ULOG_DEBUG_F("Unable to fetch msg '%s'\n", msgid);
	}
	g_slist_foreach(msg_id_list, (GFunc) g_free, NULL);
	g_slist_free(msg_id_list);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return ret_val;
}

/* Callback Functions */

/**
This is a callback function called by the engine process whenever a new 
message is received during Send/Receive operation.

@param	header	Pointer to the MsgHeader structure to be used for loading
row data
@return void
*/
void ui_recv_single_cb(MsgHeader * header)
{
	GSList *node = NULL;
	GtkTreeModel *model = NULL;
	guint msgid_hash = 0;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((header == NULL), "Input param check failed", return);
	CHECK_PARAM((header->msgid == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->sorted_model_list == NULL),
		    "Input param check failed", return);
	CHECK_PARAM((email_main_view->message_handler == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets->folderView == NULL),
		    "Input param check failed", return);

	/*Check if header is already present in INBOX. If so, ignore it. */
	node = g_slist_nth(email_main_view->sorted_model_list, INBOX_MODEL_NUM);
	model = GTK_TREE_MODEL(gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(node->data)));
	msgid_hash = g_str_hash(header->msgid);
	gtk_tree_model_foreach(model, ui_is_header_already_present, &msgid_hash);
	if (email_main_view->message_handler->is_message_present == TRUE) {
		email_main_view->message_handler->is_message_present = FALSE;
		ULOG_DEBUG_F("Header already present in Inbox!\n");
		ui_free_msg_header(header);
		return;
	}

	/*Append the new row to Inbox */
	if (ui_add_listview_header(header, INBOX_MODEL_NUM, TRUE) == FALSE) {
		ULOG_DEBUG_F("Header info is not valid!\n");
		ui_free_msg_header(header);
		return;
	}

	/*Set the Inbox model to the list view, if required */
	ui_set_folder_model_to_msgview(INBOX_MODEL_NUM, TRUE);

	/*Display Inbox folder view */
	ui_display_inbox_folder_view();

	ui_free_msg_header(header);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/* end of file */
