/**
 * @file mailviewer.c
 *
 * This file contains the implementation for the mail display widget
 *
 * Copyright (C) 2005 Nokia Corporation
 */
#include "mailviewer.h"

#include <glib-object.h>
#include <glib/gi18n.h>
#include <gtkhtml/gtkhtml.h>
#include <gtkhtml/gtkhtml-stream.h>
#include <gtkhtml/gtkhtml-search.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <regex.h>
#include <ctype.h>
#include <libetpan/libetpan.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libintl.h>

typedef struct _url_match_pattern url_match_pattern;

static void	mail_viewer_class_init           (MailviewerClass *klass);
static void	mail_viewer_init                 (Mailviewer *viewer);
static void	mail_viewer_finalize             (GObject *obj);
static void     mail_viewer_link_clicked         (GtkWidget *widget, const gchar *uri,
						  Mailviewer *viewer);
static void     mail_viewer_url_requested        (GtkWidget *widget, const gchar *uri,
						  GtkHTMLStream *stream, 
						  Mailviewer *viewer);
static GtkWidget *mail_viewer_get_content_widget (Mailviewer *viewer);
static gboolean   set_content                    (Mailviewer* viewer, const gchar *data);
static gchar     *get_file_content               (const gchar* filename, gsize *len);
static gchar     *convert_to_html                (gchar *data);
static GSList    *get_url_matches                (GString *txt);
static gchar     *find_body_part                 (struct mailmime* mime,gboolean want_html);
static gchar     *find_cid_image                 (struct mailmime* mime, const gchar *uri, size_t *len);
static gchar     *get_body_utf8                  (struct mailmime_data *data,
						  struct mailmime_content *ctype);
static gchar     *convert_to_utf8 (const gchar *str, gsize len, const gchar *charset);
static gboolean update_action_menu                (GtkWidget *viewer, GdkEvent *event, gpointer data);
static void     remove_action_menu_item           (GtkWidget *child, GtkWidget *container);    

#define URI_COPY  "copy:"
						     
/*
 * we need these regexps to find URLs in plain text e-mails
 */
typedef struct _UrlMatchPattern UrlMatchPattern;
struct _UrlMatchPattern {
	gchar   *regex;
	regex_t *preg;
	gchar   *prefix;	
};

#define MAIL_VIEWER_URL_MATCH_PATTERNS  {                                                            \
	{ "(file|http|ftp|https)://[-A-Za-z0-9_$.+!*(),;:@%&=?/~#]+[-A-Za-z0-9_$%&=?/~#]",           \
	  NULL, NULL },                                                                              \
	{ "www\\.[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-A-Za-z0-9_$.+!*(),;:@%&=?/~#]*[^]}\\),?!;:\"]?)?",\
	  NULL, "http://" },                                                                         \
	{ "ftp\\.[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-A-Za-z0-9_$.+!*(),;:@%&=?/~#]*[^]}\\),?!;:\"]?)?",\
	  NULL, "ftp://" },                                                                          \
	{ "(voipto|callto|chatto|jabberto|xmpp):[-_a-z@0-9.\\+]+",                                   \
	   NULL, NULL},						                                     \
	{ "mailto:[-_a-z0-9.\\+]+@[-_a-z0-9.]+",		                                     \
	  NULL, NULL},                                                                               \
	{ "[-_a-z0-9.\\+]+@[-_a-z0-9.]+",                                                            \
	  NULL, "mailto:"}                                                                           \
	}



/* signals */
enum {
	LINK_CLICKED,
	LOADED_PART,
	LAST_SIGNAL
};

typedef struct _MailviewerPrivate	 MailviewerPrivate;
struct _MailviewerPrivate {
	GtkHTMLStream     *stream;
	struct mailmime   *mime_message;
	gdouble	           zoomfactor;
	gboolean           stop_now;
	gboolean           is_empty;
	
	gboolean	   show_embedded;
	gboolean	   show_external;
	GtkWidget	   *action_menu; 
	GSList             *actions;
};


#define MAIL_VIEWER_GET_PRIVATE(o)   (G_TYPE_INSTANCE_GET_PRIVATE ((o),	\
				      TYPE_MAIL_VIEWER, MailviewerPrivate))

static GtkWidgetClass *parent_class = NULL;
static guint signals[LAST_SIGNAL]={0};


GType
mail_viewer_get_type (void)
{
	static GType mbv_type = 0;
	
	if (!mbv_type) {
		static const GTypeInfo mbv_info = {
			sizeof (MailviewerClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) mail_viewer_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (Mailviewer),
			16,		/* n_preallocs */
			(GInstanceInitFunc) mail_viewer_init,
		};
		mbv_type = g_type_register_static (GTK_TYPE_SCROLLED_WINDOW,
						   "Mailviewer",
						   &mbv_info, 0);
	}
	return mbv_type;
}


/* semi auto-generated code */
static void
mail_viewer_marshal_VOID__STRING_POINTER (GClosure     *closure,
					  GValue       *return_value,
					  guint         n_param_values,
					  const GValue       *param_values,
					  gpointer      invocation_hint,
					  gpointer      marshal_data)
{
	typedef void (*GMarshalFunc_VOID__STRING_POINTER) (gpointer   data1,
							   const gchar *arg1,
							   gpointer    arg_2,
							   gpointer    data2);
	register GMarshalFunc_VOID__STRING_POINTER callback;
	register GCClosure *cc = (GCClosure*) closure;
	register gpointer data1, data2;
	
	g_return_if_fail (n_param_values == 3);
	callback = (GMarshalFunc_VOID__STRING_POINTER) (marshal_data ? marshal_data : cc->callback);

	data1 = g_value_peek_pointer (param_values + 0);
	data2 = closure->data;
	
	callback (data1,
		  g_value_get_string (param_values + 1),
		  g_value_get_pointer (param_values + 2),
		  data2);
}

static void
mail_viewer_class_init (MailviewerClass *klass)
{
	GObjectClass   *gobject_class;
	GtkObjectClass *object_class;
	GtkWidgetClass *widget_class;
		
	gobject_class = (GObjectClass *)klass;
	object_class  = (GtkObjectClass *)klass;
	widget_class  = (GtkWidgetClass *)klass;

	parent_class		    = g_type_class_peek_parent (klass);
	gobject_class->finalize	    = mail_viewer_finalize;

	g_type_class_add_private (gobject_class, sizeof (MailviewerPrivate));  

	/* signals */
	signals[LINK_CLICKED] = 		
		g_signal_new ("link_clicked", 
			      G_TYPE_FROM_CLASS(object_class),
			      G_SIGNAL_RUN_FIRST,
			      G_STRUCT_OFFSET (MailviewerClass, link_clicked),
			      NULL, NULL,
			      mail_viewer_marshal_VOID__STRING_POINTER,
			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
	signals[LOADED_PART] = 		
		g_signal_new ("loaded_part", 
			      G_TYPE_FROM_CLASS(object_class),
			      G_SIGNAL_RUN_FIRST,
			      G_STRUCT_OFFSET (MailviewerClass, loaded_part),
			      NULL, NULL,
			      g_cclosure_marshal_VOID__FLOAT,
			      G_TYPE_NONE, 1, G_TYPE_FLOAT);	
}


static void
mail_viewer_init (Mailviewer *viewer)
{
	GtkClipboard *clipboard;
	MailviewerPrivate *priv = MAIL_VIEWER_GET_PRIVATE(viewer);
	
	GTK_WIDGET_SET_FLAGS (viewer, GTK_CAN_FOCUS|GTK_RECEIVES_DEFAULT);

	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(viewer),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);	

	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
	if (clipboard)
		gtk_clipboard_set_can_store (clipboard, NULL, 0);
	else
		g_warning ("cannot find the clipboard");
	
	priv->stream          = NULL;
	priv->mime_message    = NULL;
	priv->zoomfactor      = 1;
	priv->is_empty	      = TRUE;
	priv->stop_now	      = FALSE;
	priv->show_embedded   = TRUE;
	priv->show_external   = FALSE;
	priv->action_menu      = gtk_menu_new ();
	priv->actions	       = NULL;
}


static void
mail_viewer_finalize (GObject *obj)
{
	Mailviewer *viewer      = MAIL_VIEWER(obj);
	MailviewerPrivate *priv = MAIL_VIEWER_GET_PRIVATE(viewer);
 
	if (priv->mime_message)
		mailmime_free (priv->mime_message);

	priv->mime_message = NULL;
	priv->stream       = NULL;

	gtk_container_foreach (GTK_CONTAINER(priv->action_menu),
			       (GtkCallback)remove_action_menu_item,
			       priv->action_menu);
	gtk_widget_destroy (priv->action_menu);
	priv->action_menu   = NULL;
	if (priv->actions)
		osso_uri_free_actions (priv->actions);
	priv->actions = NULL;

	G_OBJECT_CLASS (parent_class)->finalize(obj);
}




GtkWidget*
mail_viewer_new (void)
{
	return GTK_WIDGET(g_object_new (TYPE_MAIL_VIEWER,NULL));
}




gboolean
mail_viewer_set_content_from_file (Mailviewer *viewer, const gchar *filename)
{
	gchar *data;
	gboolean retval;
	gsize len;

	g_return_val_if_fail (viewer, FALSE);
	g_return_val_if_fail (filename, FALSE);

	data = get_file_content (filename, &len);
	if (!data) 
		return FALSE;
	
	retval = mail_viewer_set_content (viewer, data);
	g_free (data);
	
	return retval;
}


gboolean
mail_viewer_set_content (Mailviewer* viewer, const gchar *data)
{
	MailviewerPrivate *priv; 

	int res;
	size_t idx;
	gchar *body; 
	gboolean retval;

	g_return_val_if_fail (viewer, FALSE);
	g_return_val_if_fail (data, FALSE);
	
	priv = MAIL_VIEWER_GET_PRIVATE(viewer);
	if (priv->mime_message)
		mailmime_free (priv->mime_message);

	idx = 0;
	res = mailmime_parse (data, strlen(data), &idx, &priv->mime_message);
	if (res != MAILIMF_NO_ERROR) {
		g_warning ("some error occured (%d)", res);
		return FALSE;
	}

	if (priv->mime_message->mm_type != MAILMIME_MESSAGE) {
		g_warning ("unexpected type for data (%d)",priv->mime_message->mm_type);
		retval = FALSE;
	} else {
		body = find_body_part (priv->mime_message, TRUE);
		/* no html body, try txt instead */
		if (!body) {
			gchar *txt_body;
			txt_body = find_body_part (priv->mime_message, FALSE);
			if (!txt_body) {
				g_warning ("body not found");
				return FALSE;
			}
			priv->is_empty = (strlen(txt_body) == 0);			
			body = convert_to_html (txt_body);
			g_free (txt_body);
		} else
			priv->is_empty = (strlen(body) == 0);			
	
		retval = set_content (viewer, body);
		g_free (body);
	}
	return retval;
}



static gboolean
set_content (Mailviewer* viewer, const gchar *data)
{
	GtkWidget *gtkhtml;
	GtkHTMLStream *stream;
	MailviewerPrivate *priv;

	priv = MAIL_VIEWER_GET_PRIVATE(viewer);
	
	/* FIXME: remove the old gtkhtml --> optimize this? */
	if ((gtkhtml = gtk_bin_get_child(GTK_BIN(viewer)))) 
		gtk_container_remove (GTK_CONTAINER(viewer), gtkhtml);

	gtkhtml = gtk_html_new();
	gtk_html_set_editable (GTK_HTML(gtkhtml), FALSE);
	gtk_html_allow_selection (GTK_HTML(gtkhtml), TRUE);
	gtk_html_set_blocking (GTK_HTML(gtkhtml), FALSE);
	gtk_html_set_images_blocking (GTK_HTML(gtkhtml), FALSE);
	
	g_signal_connect (G_OBJECT(gtkhtml),"link_clicked",
			  G_CALLBACK (mail_viewer_link_clicked), viewer);
	g_signal_connect (G_OBJECT(gtkhtml),"url_requested",
			  G_CALLBACK (mail_viewer_url_requested), viewer);
	
	gtk_widget_tap_and_hold_setup (GTK_WIDGET(viewer),
				       priv->action_menu, NULL,0);
	g_signal_connect(viewer, "tap_and_hold_query",
			 G_CALLBACK(update_action_menu), NULL);
	
	stream = gtk_html_begin (GTK_HTML(gtkhtml));

	gtk_container_add (GTK_CONTAINER(viewer), gtkhtml);
	gtk_widget_show_all (GTK_WIDGET(viewer));
	{
		const gchar *ptr     = data;
		const gint  data_len = strlen (data);

		/* load the html in 10Kb blocks, or in 5 parts if it is smaller
		 * than 50K one */
		const gint chunk_size  = data_len < (50*1024) ? (data_len / 5) : (10*1024);  
		priv->stop_now = FALSE;

		while (ptr < data + data_len && !priv->stop_now) {
			
			gint len = MIN(chunk_size, (data + data_len - ptr));
			gtk_html_write (GTK_HTML(gtkhtml), stream, ptr, len);		
			ptr += len;
			
			g_signal_emit (viewer, signals[LOADED_PART], 0,
				       (ptr - data) / (gfloat) data_len);
		}
		priv->stop_now = FALSE;
	}
	
	gtk_html_end (GTK_HTML(gtkhtml), stream, GTK_HTML_STREAM_OK);
	return TRUE;
}


void
mail_viewer_stop_loading (Mailviewer *viewer)
{
	g_return_if_fail (viewer);	

	MAIL_VIEWER_GET_PRIVATE(viewer)->stop_now = TRUE;
}



static gchar *
get_file_content (const gchar* filename, gsize *len)
{
	guint	 file_size = 0;
	gchar	*data	   = NULL;
	GError	*error	   = NULL;

	struct stat statbuf;

	g_return_val_if_fail (filename, FALSE);
	
	if (stat (filename, &statbuf) != 0) {
		g_warning ("cannot read file %s: %s",
			   filename, strerror(errno));
		return NULL;
	}
	
	file_size = statbuf.st_size;
	
	if (file_size > MAIL_VIEWER_MAX_FILESIZE) {
		g_warning ("file is too big (%d)\n", file_size);
		return NULL;
	}
	
	if (!g_file_get_contents (filename, &data, len, &error)) {
		g_warning ("error reading file: %s", 
			   error ? error->message : "?");
		g_error_free (error);
		return NULL;
	}

	return data;
}



static gchar *
convert_to_html (gchar *data)
{
	int		 i;
	gboolean	 first_space = TRUE;
	GString		*html;	    
	gsize           len;

	if (!data)
		return NULL;

	len = strlen (data);
	html = g_string_sized_new (len + 100);	/* just a  guess... */
	
	g_string_append_printf (html,
				"<html>"
				"<meta http-equiv=\"content-type\""
				" content=\"text/html; charset=utf8\">"
				"<body>");
	
	/* replace with special html chars where needed*/
	for (i = 0; i != len; ++i)  {
		char	kar = data[i]; 
		switch (kar) {
			
		case 0:  break; /* ignore embedded \0s */	
		case '<' : g_string_append   (html, "&lt;"); break;
		case '>' : g_string_append   (html, "&gt;"); break;
		case '&' : g_string_append   (html, "&quot;"); break;
		case '\n': g_string_append   (html, "<br>\n"); break;
		default:
			if (kar == ' ') {
				g_string_append (html, first_space ? " " : "&nbsp;");
				first_space = FALSE;
			} else	if (kar == '\t')
				g_string_append (html, "&nbsp; &nbsp;&nbsp;");
			else {
				int charnum = 0;
				first_space = TRUE;
				/* optimization trick: accumulate 'normal' chars, then copy */
				do {
					kar = data [++charnum + i];
					
				} while ((i + charnum < len) &&
					 (kar > '>' || (kar != '<' && kar != '>'
							&& kar != '&' && kar !=  ' '
							&& kar != '\n' && kar != '\t')));
				g_string_append_len (html, &data[i], charnum);
				i += (charnum  - 1);
			}
		}
	}

	g_string_append (html, "</body>");
	hyperlinkify_plain_text (html);

	return g_string_free (html, FALSE);
}

typedef struct  {
	guint offset;
	guint len;
	const gchar* prefix;
} url_match_t;



void
hyperlinkify_plain_text (GString *txt)
{
	GSList *cursor;
	GSList *match_list = get_url_matches (txt);

	g_return_if_fail (txt);

	/* we will work backwards, so the offsets stay valid */
	for (cursor = match_list; cursor; cursor = cursor->next) {

		url_match_t *match = (url_match_t*) cursor->data;
		gchar *url  = g_strndup (txt->str + match->offset, match->len);
		gchar *repl = NULL; /* replacement  */

		/* the prefix is NULL: use the one that is already there */
		repl = g_strdup_printf ("<a href=\"%s%s\">%s</a>",
					match->prefix ? match->prefix : "", url, url);

		/* replace the old thing with our hyperlink
		 * replacement thing */
		g_string_erase  (txt, match->offset, match->len);
		g_string_insert (txt, match->offset, repl);
		
		g_free (url);
		g_free (repl);
		
		g_free (match);
		cursor->data = NULL;
	}
	g_slist_free (match_list);
}



static gint 
cmp_offsets_reverse (const url_match_t *match1, const url_match_t *match2)
{
	return match2->offset - match1->offset;
}



/*
 * check if the match is inside an existing match... */
static void
chk_partial_match (const url_match_t *match, int* offset)
{
	if (*offset >= match->offset && *offset < match->offset + match->len)
		*offset = -1;
}

static GSList*
get_url_matches (GString *txt)
{
	regmatch_t rm;
        int rv, i, offset = 0;
        GSList *match_list = NULL;

	static UrlMatchPattern patterns[] = MAIL_VIEWER_URL_MATCH_PATTERNS;
	const size_t pattern_num = sizeof(patterns)/sizeof(UrlMatchPattern);

	/* initalize the regexps */
	for (i = 0; i != pattern_num; ++i) {
		patterns[i].preg = g_new0 (regex_t,1);
		g_assert(regcomp (patterns[i].preg, patterns[i].regex,
				  REG_ICASE|REG_EXTENDED|REG_NEWLINE) == 0);
	}
        /* find all the matches */
	for (i = 0; i != pattern_num; ++i) {
		offset     = 0;	
		while (1) {
			int test_offset;
			if ((rv = regexec (patterns[i].preg, txt->str + offset, 1, &rm, 0)) != 0) {
				g_assert (rv == REG_NOMATCH); /* this should not happen */
				break; /* try next regexp */ 
			}
			if (rm.rm_so == -1)
				break;

			/* FIXME: optimize this */
			/* to avoid partial matches on something that was already found... */
			/* check_partial_match will put -1 in the data ptr if that is the case */
			test_offset = offset + rm.rm_so;
			g_slist_foreach (match_list, (GFunc)chk_partial_match, &test_offset);
			
			/* make a list of our matches (<offset, len, prefix> tupels)*/
			if (test_offset != -1) {
				url_match_t *match = g_new (url_match_t,1);
				match->offset = offset + rm.rm_so;
				match->len    = rm.rm_eo - rm.rm_so;
				match->prefix = patterns[i].prefix;
				match_list = g_slist_prepend (match_list, match);
			}
			offset += rm.rm_eo;
		}
	}

	for (i = 0; i != pattern_num; ++i) {
		regfree (patterns[i].preg);
		g_free  (patterns[i].preg);
	} /* don't free patterns itself -- it's static */
	
	/* now sort the list, so the matches are in reverse order of occurence.
	 * that way, we can do the replacements starting from the end, so we don't need
	 * to recalculate the offsets
	 */
	match_list = g_slist_sort (match_list,
				   (GCompareFunc)cmp_offsets_reverse); 
	return match_list;	
}



static void
remove_action_menu_item (GtkWidget *child, GtkWidget *container)
{
	g_free (g_object_get_data(G_OBJECT(child), "uri"));
	gtk_container_remove (GTK_CONTAINER(container), child);
}

static void
remove_action_menu_items (MailviewerPrivate *priv)
{
	/* free old items */
	if (priv->action_menu) {
		gtk_container_foreach (GTK_CONTAINER(priv->action_menu),
				       (GtkCallback)(remove_action_menu_item),
				       priv->action_menu);
		if (priv->actions) {
			osso_uri_free_actions (priv->actions);
			priv->actions = NULL;
		}
	}
}



static void
mail_viewer_link_clicked (GtkWidget *widget, const gchar* uri,
			  Mailviewer *viewer)
{
	MailviewerPrivate *priv; 
	
	g_return_if_fail (viewer);
	priv  = MAIL_VIEWER_GET_PRIVATE(viewer);
	
	/* cleanup any action items */
	remove_action_menu_items (priv);
	
	/* ignore anchors ==> if it starts with '#', we don't do anything */
	if (uri) {
		const gchar *cursor = uri;
		while (cursor[0]) {
			if (cursor[0] == ' ' || cursor[0] == '\t')
				++cursor;
			else if (cursor[0] == '#')
				return;
			else
				break;
		}

		g_signal_emit (viewer, signals[LINK_CLICKED], 0, uri, NULL);
	}
}

/* some trickery in on_action_menu_activate, remove_action_menu_item and update_action_menu
 * because the tap_and_hold API is very bad for this
 * more tricks are needed because of OssoURIAction memory management
 *
 * the good news: from the user perspective, this is really easy.
 */
static void
on_action_menu_activate (GtkMenuItem *item, OssoURIAction *action)
{
	const gchar *uri  = (gchar*)g_object_get_data(G_OBJECT(item), "uri");
	GtkWidget *viewer = (GtkWidget*) g_object_get_data(G_OBJECT(item), "viewer");

	/* check if it's the special 'copy:' uri. If so, just add it to the
	 * clipbard, and don't emit any signals
	 */
	if (uri && g_str_has_prefix(uri, URI_COPY)) {
		GtkClipboard *clipboard = gtk_clipboard_get (GDK_NONE);
		gtk_clipboard_set_text (clipboard, uri + strlen(URI_COPY),
					strlen(uri) - strlen(URI_COPY));
	} else 
		g_signal_emit (viewer, signals[LINK_CLICKED], 0, uri, action);
}


static gboolean
update_action_menu (GtkWidget *viewer, GdkEvent *event, gpointer data)
{
	MailviewerPrivate *priv; 
	GtkWidget *gtkhtml;

	g_return_val_if_fail (viewer, TRUE);
			      
	if (!(gtkhtml = gtk_bin_get_child(GTK_BIN(viewer))))
		return TRUE;
			      
	priv  = MAIL_VIEWER_GET_PRIVATE(viewer);
	
	remove_action_menu_items (priv);
	
	/* are we on some url? */
	if (GTK_HTML(gtkhtml)->pointer_url) {

		GSList *cursor  = NULL;
		gchar *url      = GTK_HTML(gtkhtml)->pointer_url;
		gchar  *scheme  = osso_uri_get_scheme_from_uri (url, NULL);
		GtkWidget *menu_item;
		
		if (scheme) 
			cursor = priv->actions = osso_uri_get_actions (scheme, NULL);
		
		while (cursor) {

			OssoURIAction *action = (OssoURIAction*)cursor->data;
			menu_item = gtk_menu_item_new_with_label
				(dgettext(osso_uri_action_get_translation_domain(action),
					  osso_uri_action_get_name (action)));
			
			/* freed in remove_action_menu_item */
			g_object_set_data(G_OBJECT(menu_item), "uri", g_strdup(url)); 
			g_object_set_data(G_OBJECT(menu_item), "viewer", viewer);
			g_signal_connect (G_OBJECT(menu_item), "activate",
					  G_CALLBACK(on_action_menu_activate),
					  action);
			
			gtk_widget_show (menu_item);
			gtk_menu_append (priv->action_menu, menu_item);
			cursor = cursor->next;
		}
		g_free(scheme);
		
		/* always add Copy; stuff is freed in remove_action_menu_item */
		menu_item = gtk_menu_item_new_with_label(_("mcen_me_viewer_copy_url"));
		g_object_set_data(G_OBJECT(menu_item), "uri", g_strconcat(URI_COPY, url, NULL)); /*hack*/ 
		g_object_set_data(G_OBJECT(menu_item), "viewer", viewer);
		g_signal_connect (G_OBJECT(menu_item), "activate",  G_CALLBACK(on_action_menu_activate),NULL);
		gtk_widget_show (menu_item);
		gtk_menu_append (priv->action_menu, menu_item);

		return FALSE;
	} 

	return TRUE;
}


static GtkWidget *
mail_viewer_get_content_widget (Mailviewer *viewer)
{
	return gtk_bin_get_child(GTK_BIN(viewer));
}



void		
mail_viewer_set_zoom (Mailviewer *viewer, gdouble zoomfactor)
{
	GtkWidget *display;
	MailviewerPrivate *priv; 

	g_return_if_fail (viewer);
	g_return_if_fail (zoomfactor > 0);
	
	display = mail_viewer_get_content_widget (viewer);
	if (!display) {
		g_warning ("nothing to zoom");
		return;
	}
	priv    = MAIL_VIEWER_GET_PRIVATE(viewer);
	gtk_html_set_magnification (GTK_HTML(display), zoomfactor);
	priv->zoomfactor = zoomfactor;
}



gdouble
mail_viewer_get_zoom (Mailviewer *viewer)
{
	g_return_val_if_fail (viewer, -1);
	return MAIL_VIEWER_GET_PRIVATE(viewer)->zoomfactor;
}


gboolean
mail_viewer_is_empty (Mailviewer *viewer)
{
	g_return_val_if_fail (viewer, -1);
	return MAIL_VIEWER_GET_PRIVATE(viewer)->is_empty;
}




void 
mail_viewer_select_all (Mailviewer *viewer)
{
	GtkWidget *gtkhtml;
	g_return_if_fail (viewer);

	if (!(gtkhtml = gtk_bin_get_child(GTK_BIN(viewer))))
		return;

	gtk_html_select_all (GTK_HTML(gtkhtml));
}



gboolean
mail_viewer_copy_selection (Mailviewer *viewer)
{
	GtkWidget *gtkhtml;
	
	g_return_val_if_fail (viewer, FALSE);

	if (!(gtkhtml = gtk_bin_get_child(GTK_BIN(viewer))))
		return FALSE;

	gtk_html_copy (GTK_HTML(gtkhtml));
	
	return TRUE;
}



gboolean
mail_viewer_has_selection (Mailviewer *viewer)
{
	int len;
	gchar *kar;
	GtkWidget *gtkhtml;
	
	g_return_val_if_fail (viewer, FALSE);
	if (!(gtkhtml = gtk_bin_get_child(GTK_BIN(viewer))))
		return FALSE;

	/* it's a inefficient hack... trial and error */
	kar = gtk_html_get_selection_html (GTK_HTML(gtkhtml), &len);
	if (!kar)
		return FALSE;
	g_free(kar);

	return len > 0;
}

gboolean
mail_viewer_search (Mailviewer *viewer, const gchar* txt, gboolean from_start)
{
	GtkWidget	*gtkhtml;

	g_return_val_if_fail (viewer, FALSE);
	g_return_val_if_fail (txt, FALSE);
	
	if (!(gtkhtml = gtk_bin_get_child(GTK_BIN(viewer))))
		return FALSE;
	
	return gtk_html_engine_search (GTK_HTML(gtkhtml), txt,
				       FALSE,	/* case-sensitive? */
				       TRUE,	/* forward? */
				       TRUE);	/* regular? */
}


gboolean
mail_viewer_search_next (Mailviewer *viewer)
{
	GtkWidget	*gtkhtml;

	g_return_val_if_fail (viewer, FALSE);
	
	if (!(gtkhtml = gtk_bin_get_child(GTK_BIN(viewer))))
		return FALSE;

	return gtk_html_engine_search_next (GTK_HTML(gtkhtml));


}

static void
mail_viewer_url_requested  (GtkWidget *widget, const gchar *uri,
			    GtkHTMLStream *stream, 
			    Mailviewer *viewer)
{
	MailviewerPrivate *priv = MAIL_VIEWER_GET_PRIVATE(viewer);
	
	if (priv->show_embedded && g_str_has_prefix (uri, "cid:")) {
		/* +4 ==> skip "cid:" */
		size_t len;
		gchar *img_data = find_cid_image (priv->mime_message, uri + 4, &len);
		if (!img_data) {
			g_warning ("%s not found", uri + 4);
		} else {
			if (len > 0)
				gtk_html_stream_write (stream, img_data, len);
		}
	} else if (priv->show_external) {
		/* do something */
	}
	
	gtk_html_stream_close (stream, GTK_HTML_STREAM_OK);
}



static gchar *
find_cid_image (struct mailmime* mime, const gchar *uri, size_t *len)
{
	switch (mime->mm_type) {
		
	case MAILMIME_SINGLE:	{			
		gchar *data = NULL;
		struct mailmime_single_fields *fields;
		*len =  0;
		
		if (!mime->mm_content_type || !mime->mm_content_type->ct_type || 
		    mime->mm_content_type->ct_type->tp_type != MAILMIME_TYPE_DISCRETE_TYPE)
			return NULL;
		if (mime->mm_content_type->ct_type->tp_data.tp_discrete_type->dt_type !=
		    MAILMIME_DISCRETE_TYPE_IMAGE)
			return NULL;
		if (!mime->mm_mime_fields)
			return NULL;

		/* is this the right thing? */
		fields = mailmime_single_fields_new (mime->mm_mime_fields, mime->mm_content_type);
		if (fields && fields->fld_id &&  g_strcasecmp(fields->fld_id, uri) == 0 &&
		    fields->fld_disposition && fields->fld_disposition->dsp_type &&
		    fields->fld_disposition->dsp_type->dsp_type &&
		    fields->fld_disposition->dsp_type->dsp_type == MAILMIME_DISPOSITION_TYPE_INLINE) {

			int res;
			size_t idx = 0;
			struct mailmime_data *mimedata = mime->mm_data.mm_single;
			res = mailmime_part_parse (mimedata->dt_data.dt_text.dt_data,
						       mimedata->dt_data.dt_text.dt_length,
						       &idx, mimedata->dt_encoding,
						       &data, len);
			if (res != MAILIMF_NO_ERROR) /* data will be NULL in this case */
				g_warning ("some error occured");
			
			return data;
		}
		break;
	}
	
		
	case MAILMIME_MULTIPLE: {
		/* a multipart. check all the parts */
		clistiter *iter;
		clist *parts = mime->mm_data.mm_multipart.mm_mp_list;
		for (iter = clist_begin(parts); iter; iter=clist_next(iter)) {
			gchar *imgdata = find_cid_image ((struct mailmime *)clist_content(iter),
							 uri, len);
			if (imgdata)
				return imgdata;
		}
		break;
	}
	case MAILMIME_MESSAGE:
		return find_cid_image (mime->mm_data.mm_message.mm_msg_mime, uri, len);
	default:
		g_warning ("unexpected mime part type");
	}	
	return NULL;
}



gchar*
find_body_part (struct mailmime* mime, gboolean want_html)
{
	switch (mime->mm_type) {
	case MAILMIME_SINGLE:				
		/* a single mime type: this must be the body then */
		if (!mime->mm_content_type || !mime->mm_content_type->ct_type)
			return NULL;

		if (mime->mm_content_type->ct_type->tp_type != MAILMIME_TYPE_DISCRETE_TYPE)
			return NULL;

		if (mime->mm_content_type->ct_type->tp_data.tp_discrete_type->dt_type !=
		    MAILMIME_DISCRETE_TYPE_TEXT)
			return NULL;

		if (want_html && g_strcasecmp(mime->mm_content_type->ct_subtype,"html") != 0)
			return NULL;
		
		return get_body_utf8 (mime->mm_data.mm_single,
				      mime->mm_content_type);
		
	case MAILMIME_MULTIPLE: {
		/* a multipart. check all the parts */
		clistiter *iter;
		clist *parts = mime->mm_data.mm_multipart.mm_mp_list;

		if (clist_count(parts) == 0)
			g_warning ("invalid mime message");

		for (iter = clist_begin(parts); iter; iter=clist_next(iter)) {
			gchar *body = find_body_part ((struct mailmime *)clist_content(iter),
						      want_html);
			if (body)
				return body;
		}
		break;
	}
	case MAILMIME_MESSAGE:
		/* message in a message? ignore */
		return find_body_part (mime->mm_data.mm_message.mm_msg_mime,
				       want_html);
	default:
		g_warning ("unexpected mime part type");
	}
	
	return NULL;
}

static gchar*
get_body_utf8 (struct mailmime_data *data, struct mailmime_content *ctype)
{
	int res;
	gchar *str, *result;
	size_t len, idx;
	
	g_return_val_if_fail (data, NULL);
	
	if (data->dt_type != MAILMIME_DATA_TEXT) {
		g_warning ("must be MAILMIME_DATA_TEXT");
		return NULL;
	}

	idx = 0;
	str = NULL;
	res = mailmime_part_parse (data->dt_data.dt_text.dt_data,
				   data->dt_data.dt_text.dt_length,
				   &idx,data->dt_encoding,
				   &result,&len);

	if (res != MAILIMF_NO_ERROR) {
		g_warning ("some error occured");
		return NULL;
	}
	
	/* if it's already valid utf8, don't bother to convert */
	/* FIXME: is this assumption always valid?? */
	if (g_utf8_validate(result,len, NULL)) {
		str = g_strndup (result, len);
		mmap_string_unref(result);
		return str;
	}			

	str =  convert_to_utf8 (result,len,
			       mailmime_content_charset_get (ctype));
	mmap_string_unref(result);

	return str;	
}


static gchar *
convert_to_utf8 (const gchar *str, gsize len, const gchar *charset)
{
	GError *err = NULL;
	gchar *utf8str;
	
	utf8str = g_convert_with_fallback (str, len, "UTF-8",
					   charset ? charset : MAIL_VIEWER_DEFAULT_CHARSET,
					   "?",
					   NULL, NULL, &err);
        if (!utf8str) {
                gsize i;
                g_warning ("conversion to utf-8 failed: %s",
			   err ? err->message:"unknown");
                g_error_free (err);

                /* let's hack it instead... */
                utf8str = g_strndup (str,len);
                for (i = 0; i != len; ++i)
                        if (!isascii(str[i]))
                                utf8str[i] = '?';
        }	
	return utf8str;
}
