/**
 * @file main.c
 *
 * This file contains the implementation for 'main()' function of OSSO EMAIL.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 *
 */

#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libintl.h>
#include <locale.h>

#include <gtk/gtk.h>
#include "al_callbacks.h"
#include "email_mgr.h"
#include "manager.h"
#include "editview.h"
#include "ui_al_callbacks.h"
#include "ui_interface.h"
#include "list_view.h"
#include "email_hildon.h"
#include "ui_utils.h"
#include "ui_defs.h"
#include "ui_state_mgmt.h"
#include "email_clipboard.h"

#include "osso-log.h"


extern MainViewInfo *email_main_view;	/*declared in ui_interface.c */

/*
This is the Main function for this application.
*/
int main(int argc, char *argv[])
{
	ULOG_OPEN(APP_NAME " " APP_VERSION);
	ULOG_INFO("LOG START");
	gint ui_startup_mode = 0;
	gchar *ui_startup_input = NULL;

#ifdef ENABLE_NLS
	bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset(PACKAGE, "UTF-8");
	textdomain(PACKAGE);
#endif


	gtk_set_locale();
	setlocale(LC_ALL, "");
	gtk_init(&argc, &argv);
	gnome_vfs_init();
	/*Register signal handler */
	signal(SIGINT, ui_exit);

	/*Engine can send max of two params while launching UI.

	   First argument specifies the action to be taken after start-up:
	   (1)Launching editor in SEND_AS mode
	   (2)Launching a new editor window
	   (3)Launching a New Contact dialog
	   (4)Launching Edit Contacts dialog
	   (5)Opening an existing Inbox message
	   (6)Go to Inbox view
	   (7)Launching Add to Contacts dialog

	   Second argument specifies additional data if required:
	   (1)SEND_AS mode -- Packed MsgInfo structure containing message data
	   (5)Opening Inbox message -- MessageId string
	   (7)Add to Contacts -- One or more contact strings
	   (2)(3)(4)(6) -- Not required
	 */

	if (argc > 1) {
		ui_startup_mode = atoi(argv[1]);
	}
	if (argc > 2) {
		switch (ui_startup_mode) {

		case OSSO_EMAIL_FILE_MGR:
		case OSSO_EMAIL_URL:
		case OSSO_EMAIL_MAIL_TO:
			{
				ui_dbus_unpack_str_base64((guchar *) argv[2], &ui_startup_input);
				break;
			}
		case OSSO_TASKNAV_OPEN_MSG:
			{
				ui_startup_input = argv[2];
				break;
			}
		default:
			break;
		}
	}

	get_ui_context();

	app = HILDON_APP(hildon_app_new());
	hildon_app_set_two_part_title(app, TRUE);
	hildon_app_set_title(app, _("mcen_ap_name"));
	appview = HILDON_APPVIEW(hildon_appview_new(NULL));
#if CHANGED_EMAIL
	if (ui_startup_mode != OSSO_EMAIL_MAIL_TO && ui_startup_mode != OSSO_EMAIL_URL
	    && ui_startup_mode != OSSO_EMAIL_FILE_MGR) {
		/*editor opened in standalone mode from browser/fm. Shouldn't show main view. */
		gtk_widget_show_all(GTK_WIDGET(appview));
		hildon_app_set_appview(app, appview);
		gtk_widget_show(GTK_WIDGET(app));
	}
#else
	gtk_widget_show_all(GTK_WIDGET(appview));
	hildon_app_set_appview(app, appview);
	gtk_widget_show(GTK_WIDGET(app));
#endif
	appview = HILDON_APPVIEW(ui_create_main_window());

	if (appview == NULL) {
		ui_cleanup_memory();
		LOG_CLOSE();
		exit(0);
	}
	if (FALSE == ui_initialize(ui_startup_mode)) {
		ui_cleanup_memory();
		LOG_CLOSE();
		exit(0);
	}

	/*Init clipboard */
	osso_email_clipboard_init();

	switch (ui_startup_mode) {
	case OSSO_EMAIL_FILE_MGR:
	case OSSO_EMAIL_URL:
	case OSSO_EMAIL_MAIL_TO:
		{
#if CHANGED_EMAIL
			/*editor opened in standalone mode from browser/fm. Shouldn't show main view. email_main_view->tags->main_view_hidden is used while closing the editor to decide whether to exit from the email app while editor is being closed. */
			email_main_view->tags->main_view_hidden = TRUE;
			gtk_widget_hide(GTK_WIDGET(appview));
#endif
			ui_dbus_editor_create(ui_startup_input);
			g_free(ui_startup_input);
			break;
		}

	case OSSO_TASKNAV_OPEN_MSG:
		{
			ui_tasknav_open_msg_cb(ui_startup_input);
			break;
		}

	case OSSO_TASKNAV_OPEN_NEW_MSG:
		{
			ui_tasknav_open_msg_cb(NULL);
			break;
		}

	case OSSO_TASKNAV_OPEN_INBOX:
		{
			ui_tasknav_show_inbox_cb();
			break;
		}

	default:
		{
			ULOG_DEBUG_F
			    ("Invalid initialization param received '%d'. Ignoring...\n",
			     ui_startup_mode);
			ui_do_send_recv();
			break;
		}
	}

	gtk_main();
	return 0;
}
