/**
 * @file manager_cb.c
 *
 * This file contains tasknav module related invoking functions for UI.
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#include <stdlib.h>
#include <glib.h>
#include "common.h"
#include "prefs_account.h"
#include "procmsg.h"
#include "manager.h"
#include "folder.h"
#include "al_callbacks.h"
#include "ui_al_callbacks.h"
#include "editview.h"
#include "ui_interface.h"
#include "folder_mgmt.h"
#include "ui_account_store.h"
#include "accountMgmt_callback.h"

static guint idle_event_id = 0;

/**
   This function is called when tasknav requested to UI module to 
   open a message. If no message id is specified open for new message  
   @param msgid id of message to be opened 
   @return none 
*/

void tasknav_open_msg_cb(gchar * msgid)
{
	ui_tasknav_open_msg_cb(msgid);
	return;
}

/**
   This function is called  to open editor window from other applications
   (like send as mail from file manager) 
   @param msginfo contains the message information to be shown in editor 
    window.
   @message body of the message 
   @return none 
*/

void editor_create_cb(MsgInfo * msginfo, gchar * message)
{
	ui_open_editor_in_send_as_mail_to_mode_cb(msginfo, message);
	return;
}



/**
   This function is called when tasknav requested to UI module to 
   open window based on window id
   @param windowid id of thw window to be opened
   @return none 
*/

void tasknav_window_id(gchar * windowid)
{
	ui_tasknav_top_msg_cb(windowid);
	return;
}

/**
   This function is called when tasknav requested to UI module to 
   open inbox 
   @return none 
*/

void tasknav_open_inbox(void)
{
	ui_tasknav_show_inbox_cb();
	return;
}

/**
   This function is called when dbus module receives Disconnected signal
   and wants to close UI application 
   @return none 
*/

void close_application(void)
{
	ui_close_application();
	return;
}

/**
   This function is called when dbus module receives oom signal
   and UI module does the state save operation. 
   @return none 
*/
void oom_signal_recv(void)
{
	ui_tasknav_oom_signal_received();
	return;
}


/**
   This function is called to inform the UI module about change 
   status in mmc
   @param error_code Account module related error
   @return none 
*/

void mmc_hlr_cb(gint error_code)
{
	switch (error_code) {
	case MMC_CONNECTED:
		ULOG_DEBUG_F("mmc connected \n");
		ui_add_archive_folder();
		break;
	case MMC_DISCONNECTED:
		ULOG_DEBUG_F("mmc dis connected \n");
		ui_remove_archive_folder();
		break;
	case MMC_ERROR:
		ULOG_DEBUG_F("mmc error received \n");
		break;
	default:
		ULOG_DEBUG_F("invalid mmc status \n");
	}
}

void folder_msg_headers_cb(gint type, GSList * new_list)
{
	ui_folder_msg_headers_cb(type, new_list);
	return;
}

void folder_set_msg_cb(gint msg_number, gint index, GArray * arguments)
{
	gint i = 0, cur_msg_num = 0, total_msg_count = 0;
	gfloat percent = 0;
	gchar *acc_name = NULL, *msg_id = NULL;
	PrefsAccount *acc_prefs = NULL;

	ULOG_DEBUG_F(" folder_set_msg_cb = %d Index = %d \n", msg_number, index);
	for (i = 0; i < index; i++) {
		switch (g_array_index(arguments, osso_rpc_t, i).type) {
		case DBUS_TYPE_INT32:
			ULOG_DEBUG_F("type %d INT  value = %d \n", DBUS_TYPE_INT32,
				     g_array_index(arguments, osso_rpc_t, i).value.i);
			break;
		case DBUS_TYPE_STRING:
			ULOG_DEBUG_F(" +++ type %d STR  value = %s \n", DBUS_TYPE_STRING,
				     g_array_index(arguments, osso_rpc_t, i).value.s);
			break;
		default:
			ULOG_DEBUG_F(" Type = %d \n", g_array_index(arguments, osso_rpc_t, i).type);
		}
	}

	switch (msg_number) {
	case FOLDER_COMPLETE:
		{
			if (ui_stop_folder_progress_note() == FALSE && idle_event_id > 0) {
				g_source_remove(idle_event_id);
			}
			idle_event_id = 0;
			ui_copy_unfetched_messages_if_required();
			ui_show_note_for_failed_msgs();
			break;
		}
	case FOLDER_DEL_PROGRESS:
		{
			if (idle_event_id == 0) {
				idle_event_id =
				    g_idle_add((GSourceFunc) ui_start_folder_progress_note,
					       GINT_TO_POINTER(ACTION_DELETE));
			}
			break;
		}
	case FOLDER_PROGRESS:
		{
			if (index == 2) {
				cur_msg_num = g_array_index(arguments, osso_rpc_t, 0).value.i;
				total_msg_count = g_array_index(arguments, osso_rpc_t, 1).value.i;
				if (cur_msg_num == 0 && idle_event_id == 0) {
					idle_event_id =
					    g_idle_add((GSourceFunc) ui_start_folder_progress_note,
						       GINT_TO_POINTER(ACTION_PASTE));
				} else {
					percent = (cur_msg_num * 100 / total_msg_count) * 0.01;
					if (percent >= 0 && percent <= 1.0) {
						ui_update_folder_progress_note(percent);
					}
				}
			} else {
				ULOG_DEBUG_F("Invalid settings for FOLDER_PROGRESS option");
			}
			break;
		}
	case FOLDER_MSG_DELETED_ERROR:
		{
			if (index == 1) {
				msg_id = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
				acc_name = ui_get_account_name(msg_id);
				acc_prefs = ui_get_account_detail(acc_name);
				if (acc_prefs->receive_settings.protocol == PROTO_POP3)
					ui_set_msg_cb(msg_number);
				else
					ui_set_error_msg_cb(IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER,
							    msg_id, NULL);

				g_free(acc_name);
				ui_accmgmt_freeprefsaccount(acc_prefs);
			}
			break;
		}
	default:
		{
			ui_set_msg_cb(msg_number);
			break;
		}
	}
}
void
retrv_msgs_status(void)
{
	ui_retrv_complete_cb();
	return;
}
