/**
 * @file message_handler.c
 *
 * This file contains the implementation for message handler operations.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */


#include "ui_common.h"
//AL header files
#include "al_callbacks.h"
#include "utils.h"
#include "folder.h"
#include "defs.h"
#include "engine_errno.h"

//UI_DBUS header files
#include "email_mgr.h"
#include "dbus_utils.h"

//Module specific header files
#include "editview.h"
#include "ui_al_callbacks.h"
#include "ui_interface.h"
#include "ui_state_mgmt.h"
#include "folder_mgmt.h"
#include "message_handler.h"
#include "list_view.h"
#include "ui_error_codes.h"
#include "email_clipboard.h"

#include <hildon-fm/hildon-widgets/hildon-file-selection.h>
#include <hildon-fm/hildon-widgets/hildon-file-system-model.h>
#include <hildon-fm/hildon-widgets/hildon-file-chooser-dialog.h>
#include <gtkfilesystemmemory/gtkfilesystemmemory.h>

//Global Variables
extern MainViewInfo *email_main_view;
/*Helper Functions*/

/**
This is a helper function used to set/unset the UI to marking mode. In
marking mode, all the required menus are disabled. These menus are accessed
from a global list.

@param  marking_mode  Boolean variable to set marking mode ON/OFF
@return void
*/
void ui_invoke_marking_mode(gboolean marking_mode)
{
	GSList *node = NULL;
	gboolean sensitivity = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->marking_mode_widgets == NULL),
		    "Input param check failed", return);

	if (!marking_mode) {
		sensitivity = TRUE;
	}

	node = email_main_view->marking_mode_widgets;
	while (node != NULL) {
		gtk_widget_set_sensitive(node->data, sensitivity);
		node = node->next;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}


/**
This is a 'GtkTreeModelForeachFunc' function used to undim all rows
that were dimmed due to CUT operation in the current folder view.

@param	model		Pointer to the folder view storage model
@param	path		Pointer to the GtkTreePath of current row
@param	iter		Pointer to the GtkTreeIter of current row
@param	data		Pointer to the user data (not used here)
@return boolean		TRUE --> if model is NULL (don't call again)
			FALSE --> Call again with next row
*/
gboolean
ui_undim_all_rows(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((model == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((iter == NULL), "Input param check failed", return TRUE);

	gtk_list_store_set(GTK_LIST_STORE(model), iter, COL_CUT_STATUS, TRUE, -1);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return FALSE;
}

/**
This is a  function used to return the status of the
marked rows in the outbox folder view.

@param  msgId           Pointer to the marked message
@return gboolean TRUE-> when status is sending
		 FALSE->else
*/
static gboolean is_msg_sending(const gchar * msgId)
{
	gint status_bit = -1;

	if (!msgId)
		return FALSE;
	if (email_main_view->folder_manager->currently_displayed_model == OUTBOX_MODEL_NUM) {
		status_bit = g_ascii_digit_value(msgId[strlen(msgId) - 1]);
		if (status_bit == OUTBOX_STATUS_SENDING)
			return FALSE;
	}

	return TRUE;

}


/**
This is a  function used to count the number of
marked rows in the current folder view.

@param	view		Pointer to the folder view storage model
@param	count		Pointer to the user data
@return void
*/
void ui_count_marked_rows(GtkTreeView * view, gint * count)
{
	GtkTreeSelection *selection = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	CHECK_PARAM((view == NULL), "Input param check failed", return);
	CHECK_PARAM((count == NULL), "Input param check failed", return);

	selection = gtk_tree_view_get_selection(view);
	*count = gtk_tree_selection_count_selected_rows(selection);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
}


/**
This is a helper function used to update a given message tag in the given 
folder view. These tags can be - Mark status, In use status, Read status,
Purge status, Fully downloaded status, Body downloaded status, Cut status
and Remote copy status.
Additionally it changes the message icons if needed.

@param	model		Pointer to folder storage model
@param	folderNum	Sequence number of the folder storage model
@param	rowNum		Sequence number of the row to be marked/unmarked
@param	mark		Boolean variable to set row's mark status to ON/OFF
@return gboolean	TRUE-->If success, FALSE-->otherwise.
*/
static gboolean
ui_update_row_tag_in_tree_model(GtkTreeModel * model, gint folderNum,
				const gchar * msgid, rowTag tag, gboolean flag)
{
	GtkTreeIter iter, child_iter;
	GtkTreePath *path = NULL;
	GtkTreeModel *child_model = NULL;
	GtkTreeSelection *selection = NULL;
	gboolean success = FALSE, body_downloaded = FALSE, read_status = FALSE;
	GdkPixbuf *msg_icon = NULL;
	gint priority = -1;
	gint smime_type = SMIME_NONE;
	guint msgid_hash1 = 0, msgid_hash2 = 0;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((model == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((folderNum < 0), "Input param check failed", return FALSE);
	CHECK_PARAM((msgid == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->widgets->messageListView == NULL),
		    "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->icons == NULL), "Input param check failed", return FALSE);

	if (!gtk_tree_model_get_iter_first(model, &iter)) {
		ULOG_DEBUG_F("Storage model empty for model number : %d!\n", folderNum);
		return FALSE;
	}
	//Traverse the rows until message-id is matched.
	msgid_hash1 = g_str_hash(msgid);
	do {
		gtk_tree_model_get(model, &iter, COL_MSGID_HASH, &msgid_hash2,
				   COL_PRIORITY, &priority,
				   COL_SMIME_TYPE, &smime_type, COL_BODY_DOWNLOADED,
				   &body_downloaded, COL_READ_STATUS, &read_status, -1);

		if (msgid_hash1 == msgid_hash2) {
			success = TRUE;
			break;
		}
	}
	while (gtk_tree_model_iter_next(model, &iter));

	if (success) {
		gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT
							       (model), &child_iter, &iter);
		child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(model));

		switch (tag) {
		case IN_USE_STATUS:
			{
				//Change the row selection if the folder is currently selected
				//if (email_main_view->folder_manager->currently_displayed_model == folderNum)
				if (flag) {
					path = gtk_tree_model_get_path(model, &iter);
					selection =
					    gtk_tree_view_get_selection(GTK_TREE_VIEW
									(email_main_view->widgets->
									 messageListView));
					/*Fix bug 23047 , select a single row when next/prev or more then 
					   one viewer is opened. */
					gtk_tree_selection_set_mode(selection,
								    GTK_SELECTION_SINGLE);
					gtk_tree_selection_select_path(selection, path);
					gtk_tree_selection_set_mode(selection,
								    GTK_SELECTION_MULTIPLE);
					gtk_tree_path_free(path);
				}

				gtk_list_store_set(GTK_LIST_STORE(child_model), &child_iter,
						   COL_IN_USE_STATUS, flag, -1);
				break;
			}
		case READ_STATUS:
			{
				msg_icon =
				    ui_get_message_display_icon(body_downloaded, priority, flag,
								smime_type);
				gtk_list_store_set(GTK_LIST_STORE(child_model), &child_iter,
						   COL_READ_STATUS, flag, COL_MSG_ICON, msg_icon,
						   -1);
				break;
			}
		case PURGE_STATUS:
			{
				gtk_list_store_set(GTK_LIST_STORE(child_model), &child_iter,
						   COL_PURGE_ALLOW, flag, -1);
				break;
			}
		case FULLY_DOWNLOADED_STATUS:
			{
				gtk_list_store_set(GTK_LIST_STORE(child_model), &child_iter,
						   COL_FULLY_DOWNLOADED, flag, -1);
				break;
			}
		case BODY_DOWNLOADED_STATUS:
			{
				msg_icon =
				    ui_get_message_display_icon(flag, priority, read_status,
								smime_type);
				gtk_list_store_set(GTK_LIST_STORE(child_model), &child_iter,
						   COL_BODY_DOWNLOADED, flag, COL_MSG_ICON,
						   msg_icon, -1);
				break;
			}
		case CUT_STATUS:
			{
				gtk_list_store_set(GTK_LIST_STORE(child_model), &child_iter,
						   COL_CUT_STATUS, flag, -1);
				break;
			}
		case MARK_BOOL_STATUS:
			{
				gtk_list_store_set(GTK_LIST_STORE(child_model), &child_iter,
						   COL_MARK_STATUS, flag, -1);
				break;
			}
		case ATTACHMENT_STATUS:
			{
				if (flag) {
					gtk_list_store_set(GTK_LIST_STORE(child_model), &child_iter,
							   COL_ATTACHMENT_ICON,
							   email_main_view->icons->attachment_icon,
							   COL_ATTACHMENT_PRESENT, TRUE, -1);
				} else {
					gtk_list_store_set(GTK_LIST_STORE(child_model), &child_iter,
							   COL_ATTACHMENT_ICON, NULL,
							   COL_ATTACHMENT_PRESENT, FALSE, -1);
				}
				break;
			}
		case REMOTE_COPY_STATUS:
			{
				gtk_list_store_set(GTK_LIST_STORE(child_model), &child_iter,
						   COL_REMOTE_COPY, flag, -1);
				break;
			}
		default:
			{
				break;
			}
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return TRUE;
}

/**
This is a helper function used to update a given message tag in the given 
folder view. These tags can be - Mark status, In use status, Read status,
Purge status, Fully downloaded status, Body downloaded status, Cut status
and Remote copy status.

@param	folderNum	Sequence number of the folder storage model
@param	rowNum		Sequence number of the row to be marked/unmarked
@param	mark		Boolean variable to set row's mark status to ON/OFF
@return gboolean	TRUE-->If success, FALSE-->otherwise.
*/
gboolean ui_update_row_tag_in_folder(gint folderNum, const gchar * msgid, rowTag tag, gboolean flag)
{
	GtkTreeModel *model = NULL;
	guint folderCount = 0;
	GSList *node = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((folderNum < 0), "Input param check failed", return FALSE);
	CHECK_PARAM((msgid == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->sorted_model_list == NULL),
		    "Input param check failed", return FALSE);

	//Check if such a folder exists at all!
	folderCount = g_slist_length(email_main_view->sorted_model_list);
	if (folderNum > folderCount) {
		ULOG_DEBUG_F("Folder does not exist!\n");
		return FALSE;
	}

	node = g_slist_nth(email_main_view->sorted_model_list, folderNum);
	if (node == NULL || (model = GTK_TREE_MODEL(node->data)) == NULL) {
		ULOG_DEBUG_F("Folder does not exist!\n");
		return FALSE;
	}

	if (FALSE == ui_update_row_tag_in_tree_model(model, folderNum, msgid, tag, flag)) {
		ULOG_DEBUG_F("Unable to set status for specified row!\n");
		return FALSE;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return TRUE;
}

/**
 This functions gets the GtkTreePaths list for the given list of
 msgids.
 @param marked_msgs is the GSList of msgids on which the operation(move/delete/cut/paste)
 is done.
 @param paths_list is the OUT param containing the list of GtkTreePaths ,to be freed by caller.

*/
static void ui_get_tree_paths(GSList * marked_msgs, GSList ** paths_list)
{
	gint folder_num = -1;
	gchar *folder_name = NULL, *parent_name = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	gchar *msgId = NULL;

	if (ui_get_msg_folder_names(marked_msgs->data, &parent_name, &folder_name) == FALSE) {
		ULOG_DEBUG_F("Unable to get folder name for msgid '%s'\n",
			     (gchar *) marked_msgs->data);
		return;
	}
	folder_num = ui_get_folder_model_number(folder_name, parent_name);
	g_free(parent_name);
	g_free(folder_name);
	if (folder_num == -1) {
		//Unable to resolve folder store number
		ULOG_DEBUG_F("Unable to resolve folder for msg '%s'\n",
			     (gchar *) marked_msgs->data);
		return;
	}

	model = ui_get_sorted_folder_model(folder_num);
	if (!gtk_tree_model_get_iter_first(model, &iter)) {
		ULOG_DEBUG_F("Storage model empty\n");
		return;
	}
	do {
		gtk_tree_model_get(model, &iter, COL_MSG_ID, &msgId, -1);

		if (msgId != NULL &&
		    g_slist_find_custom(marked_msgs, msgId, (GCompareFunc) ui_compare_utf8_strings)
		    != NULL) {
			*paths_list =
			    g_slist_append(*paths_list, gtk_tree_model_get_path(model, &iter));
		}
		g_free(msgId);
	}
	while (gtk_tree_model_iter_next(model, &iter));

}

/**
This is a  function used to obtain list of currently
selected messages. Output is a list of GtkTreeRowReference objects, this list
should be freed by the calling function.

@param  view     Pointer to the current list view
@param	paths_list  Pointer to a GSList* for preparing list of selected rows
@param	undo_operation is the gboolean which says the operation is initiated by undo or not.
@return void
*/
static void ui_get_selected_rows(GtkTreeView * view, GSList ** pathslist, gboolean undo_operation)
{
	GtkTreeSelection *selection = NULL;

	CHECK_PARAM((pathslist == NULL), "Input param check failed", return);
	CHECK_PARAM((view == NULL), "Input param check failed", return);

	if (undo_operation) {
		ui_get_tree_paths(email_main_view->undo->message_id_list, pathslist);
		return;
	}
	selection = gtk_tree_view_get_selection(view);
	*pathslist = (GSList *) gtk_tree_selection_get_selected_rows(selection, NULL);
	return;
}

/**
 This function is used to the get the list of msgids & the smime type 
 of unretrived mails.

 @param view is the pointer the GtkTreeView.
 @return a GSList containg RetrMsgData data structure.Should be freed by calling function.
*/
GSList *ui_get_unretrieved_msg_id_list(GtkTreeView * view)
{
	gboolean body_downloaded = FALSE;
	gchar *msgid = NULL;
	gint smime_type = 0;
	RetrMsgData *retr_msg_data = NULL;
	GList *pathslist = NULL, *temp_paths_list = NULL;
	GSList *msgid_list = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;



	selection = gtk_tree_view_get_selection(view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, &model);
	if (pathslist == NULL) {
		ULOG_DEBUG_L("nothing is selected");
		return NULL;
	}

	/*Get the relevat dat for the selcted rows */
	for (temp_paths_list = pathslist; temp_paths_list != NULL;
	     temp_paths_list = temp_paths_list->next) {
		if (gtk_tree_model_get_iter(model, &iter, (GtkTreePath *) temp_paths_list->data)) {
			gtk_tree_model_get(model, &iter, COL_MSG_ID, &msgid,
					   COL_BODY_DOWNLOADED, &body_downloaded,
					   COL_SMIME_TYPE, &smime_type, -1);
			if (body_downloaded) {
				g_free(msgid);
				continue;
			}
			if (msgid != NULL) {
				retr_msg_data = g_new0(RetrMsgData, 1);
				retr_msg_data->msgid = msgid;
				if (smime_type == SMIME_NONE)
					retr_msg_data->flag = FALSE;
				else
					retr_msg_data->flag = TRUE;
				msgid_list = g_slist_append(msgid_list, retr_msg_data);
			}
		}
	}
	g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(pathslist);
	return msgid_list;
}



/**
This is a function used to obtain message-ids of all not in use msgids


@param  view            Pointer to the  GtkTreeview .
@param operation is a gint specifying the operation such as (sending/resending/cancel sending); 
@param inuse_msg_count is the gint point .
 OUT param conating the count of no.of mesg that are seelcted are in use.
@return  Pointer to a GSList* of msgids that are selected & are not in use.
*/
GSList *ui_get_selected_not_inuse_msgids(GtkTreeView * view, gint operation, gint * inuse_msg_count)
{
	gboolean in_use = FALSE;
	gchar *msgid = NULL;
	GList *pathslist = NULL, *temp_paths_list = NULL;
	GSList *msgid_list = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeModel *model;
	GtkTreeIter iter;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return NULL);
	CHECK_PARAM((email_main_view->tags == NULL), "Input param check failed", return NULL);

	selection = gtk_tree_view_get_selection(view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, &model);
	if (pathslist == NULL) {
		osso_log(LOG_INFO, "nothing is selected");
		return NULL;
	}

	/*Get the relevat dat for the selcted rows */
	for (temp_paths_list = pathslist; temp_paths_list != NULL;
	     temp_paths_list = temp_paths_list->next) {
		if (gtk_tree_model_get_iter(model, &iter, (GtkTreePath *) temp_paths_list->data)) {
			gtk_tree_model_get(model, &iter, COL_MSG_ID, &msgid,
					   COL_IN_USE_STATUS, &in_use, -1);
			if (msgid != NULL) {
				if (operation != OPR_RESEND_MAILS && in_use) {
					g_free(msgid);
					*inuse_msg_count++;
				} else {
					msgid_list = g_slist_append(msgid_list, msgid);
				}
			}
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(pathslist);
	return msgid_list;
}

/**
This is  function used to obtain message-ids of all
seelcted rows in current folder view. Output is a list of MsgId strings, this
list should be freed by the calling function.

@param  view            Pointer to the folder view .
@param  msg_id_info_list Pointer to a GSList* for preparing list of message-ids
@return void
*/
void ui_get_selected_row_msg_ids(GtkTreeView * view, GSList ** msg_id_list)
{
	gboolean body_downloaded = FALSE;
	gchar *msgId = NULL;
	GList *pathslist = NULL, *temp_paths_list = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((msg_id_list == NULL), "Input param check failed", return);

	selection = gtk_tree_view_get_selection(view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, &model);
	if (pathslist == NULL) {
		osso_log(LOG_INFO, "nothing is selected");
		return;
	}

	/*Get the relevat dat for the selcted rows */
	temp_paths_list = pathslist;
	while (temp_paths_list != NULL) {
		if (gtk_tree_model_get_iter(model, &iter, (GtkTreePath *) temp_paths_list->data)) {
			gtk_tree_model_get(model, &iter, COL_MSG_ID, &msgId,
					   COL_BODY_DOWNLOADED, &body_downloaded, -1);
			if (email_main_view->tags->get_only_unretrieved_mails && body_downloaded) {
				g_free(msgId);
				continue;
			}
			if (msgId != NULL) {
				if (is_msg_sending(msgId))
					*msg_id_list = g_slist_append(*msg_id_list, msgId);
			}
		}
		temp_paths_list = temp_paths_list->next;
	}

	g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(pathslist);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a 'GtkTreeModelForeachFunc' function used to obtain message-ids of all
Outbox messages with 'WAITING' status. Output is a list of MsgId strings, this
list should be freed by the calling function.

@param  model           Pointer to the Outbox view storage model
@param  path            Pointer to the GtkTreePath of current row
@param  iter            Pointer to the GtkTreeIter of current row
@param  msg_id_info_list Pointer to a GSList* for preparing list of message-ids
@return boolean         TRUE --> if model is NULL (don't call again)
                        FALSE --> Call again with next row
*/
gboolean
ui_get_outbox_msgids_with_wait_status(GtkTreeModel * model, GtkTreePath * path,
				      GtkTreeIter * iter, GSList ** msg_id_list)
{
	gchar *msg_id = NULL;
	gint outbox_status_bit = -1;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((model == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((iter == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((msg_id_list == NULL), "Input param check failed", return TRUE);

	gtk_tree_model_get(model, iter, COL_MSG_ID, &msg_id, -1);
	outbox_status_bit = g_ascii_digit_value(msg_id[strlen(msg_id) - 1]);

	if (outbox_status_bit == OUTBOX_STATUS_WAITING) {
		if (msg_id != NULL) {
			*msg_id_list = g_slist_append(*msg_id_list, msg_id);
		}
	} else {
		g_free(msg_id);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return FALSE;
}

/**
This is a function used to obtain message-ids of all
unseelcted rows in current folder view. Output is a list of MsgId strings, this
list should be freed by the calling function.

@param  view            Pointer to the folder view .
@param  model           Pointer to the folder view storage model
@param  msg_id_info_list Pointer to a GSList* for preparing list of message-ids
@return void
*/
void ui_get_unselected_row_msg_ids(GtkTreeView * view, GtkTreeModel * model, GSList ** msg_id_list)
{
	gchar *msgId = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeIter iter;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((model == NULL), "Input param check failed", return);
	CHECK_PARAM((msg_id_list == NULL), "Input param check failed", return);



	selection = gtk_tree_view_get_selection(view);
	if (!gtk_tree_model_get_iter_first(model, &iter)) {
		ULOG_DEBUG_F("Storage model empty\n");
		return;
	}
	//Traverse the rows until message-id is matched.
	do {
		if (!gtk_tree_selection_iter_is_selected(selection, &iter)) {
			gtk_tree_model_get(model, &iter, COL_MSG_ID, &msgId, -1);
			if (msgId != NULL) {
				*msg_id_list = g_slist_append(*msg_id_list, msgId);
			}
		}
	}
	while (gtk_tree_model_iter_next(model, &iter));
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a  function used to obtain the count of
selected messages that are unretrieved yet (body not downloaded).

@param  view            Pointer to the folder view .
@param  count           Outparam for storing the count of messages
@return void
*/
void ui_get_unretrieved_selected_msgs_count(GtkTreeView * view, gint * count)
{
	gboolean body_downloaded = FALSE;
	GtkTreeModel *model = NULL;
	GList *pathslist = NULL, *temp_paths_list = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeIter iter;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((count == NULL), "Input param check failed", return);

	selection = gtk_tree_view_get_selection(view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, &model);

	/*Get the relevat data  from the selected rows specifying the unretrieved data */
	temp_paths_list = pathslist;
	while (temp_paths_list != NULL) {
		if (gtk_tree_model_get_iter(model, &iter, (GtkTreePath *) temp_paths_list->data)) {
			gtk_tree_model_get(model, &iter, COL_BODY_DOWNLOADED, &body_downloaded, -1);
			if (!body_downloaded) {
				*count += 1;
			}
		}
		temp_paths_list = temp_paths_list->next;
	}

	g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(pathslist);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a  function used to obtain message-ids of all
rows that have purge option set to ON in current folder view. Output is a
list of MsgId strings, this list should be freed by the calling function.

@param  view            Pointer to the folder view .
@param  msg_id_list     Pointer to a GSList* for preparing list of message-ids
@return void
*/
void ui_get_purgeable_row_msg_ids(GtkTreeView * view, GSList ** msg_id_list)
{
	gboolean purge_allowed = FALSE;
	gchar *msgId = NULL;
	GList *pathslist = NULL, *temp_paths_list = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((view == NULL), "Input param check failed", return);
	CHECK_PARAM((msg_id_list == NULL), "Input param check failed", return);

	selection = gtk_tree_view_get_selection(view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, &model);

	/*Get the relevat data for purging from the selected rows */
	temp_paths_list = pathslist;
	while (temp_paths_list != NULL) {
		if (gtk_tree_model_get_iter(model, &iter, (GtkTreePath *) temp_paths_list->data)) {
			gtk_tree_model_get(model, &iter, COL_MSG_ID, &msgId, COL_PURGE_ALLOW,
					   &purge_allowed, -1);

			if (purge_allowed) {
				if (msgId != NULL) {
					*msg_id_list = g_slist_append(*msg_id_list, msgId);
				}
			} else
				g_free(msgId);
		}
		temp_paths_list = temp_paths_list->next;
	}

	g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(pathslist);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to obtain the row number of a message given
its message-id and its parent folder number.

@param  msgid          message-id string
@param  folderModelNum parent folder number
@return gint           Current row number of the message on success.
                       -1 on failure.
*/
gint ui_get_row_number_of_msg(const gchar * msgid, gint folderModelNum)
{
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	gchar *msgId = NULL;
	gint row_num = -1;
	gboolean success = FALSE;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((msgid == NULL), "Input param check failed", return row_num);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return row_num);
	CHECK_PARAM((email_main_view->sorted_model_list == NULL),
		    "Input param check failed", return row_num);
	CHECK_PARAM((folderModelNum < 0), "Input param check failed", return row_num);
	CHECK_PARAM((folderModelNum >
		     g_slist_length(email_main_view->sorted_model_list) - 1),
		    "Input param check failed", return row_num);

	//Get the folder storage model
	model =
	    GTK_TREE_MODEL(g_slist_nth_data(email_main_view->sorted_model_list, folderModelNum));
	if (!gtk_tree_model_get_iter_first(model, &iter)) {
		ULOG_DEBUG_F("Storage model empty\n");
		return row_num;
	}
	//Traverse the rows until message-id is matched.
	do {
		row_num++;
		gtk_tree_model_get(model, &iter, COL_MSG_ID, &msgId, -1);

		if (msgId != NULL && strncmp(msgid, msgId, MAX_STR_LEN(msgid, msgId)) == 0) {
			ULOG_DEBUG_F("Row found at line %d.\n", row_num);
			success = TRUE;
			g_free(msgId);
			break;
		}
		g_free(msgId);
	}
	while (gtk_tree_model_iter_next(model, &iter));

	if (success == FALSE) {
		ULOG_DEBUG_F("Row not found.\n");
		row_num = -1;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return row_num;
}

/**
This is a helper function used to get the current row information. It is
called when opening the message in a viewer/editor window. Internal fields
of HeaderInfo struct are filled up in this function, these should be freed
by the calling function.

@param  header          Pointer to HeaderInfo structure for storing row data
@return boolean         TRUE --> Success, FALSE --> Failure
*/
gboolean ui_get_row_information(HeaderInfo * header)
{
	GtkTreeModel *model = NULL;
	GtkTreePath *path = NULL;
	GtkTreeIter iter;
	guint folderCount = 0;
	GSList *node = NULL;
	gint numberOfRows = -1;
	gboolean noSubject = FALSE;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((header == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((header->rowNum < 0), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->sorted_model_list == NULL),
		    "Input param check failed", return FALSE);

	//Check if such a folder exists at all!
	folderCount = g_slist_length(email_main_view->sorted_model_list);
	if (header->folderNum > folderCount) {
		ULOG_DEBUG_F("Folder does not exist!\n");
		return FALSE;
	}

	node = g_slist_nth(email_main_view->sorted_model_list, header->folderNum);
	if (node == NULL || (model = GTK_TREE_MODEL(node->data)) == NULL) {
		ULOG_DEBUG_F("Folder does not exist!\n");
		return FALSE;
	}
	//Prepare a new path from the row number
	//Check if a valid row exists for this path and retrieve the row data
	numberOfRows = gtk_tree_model_iter_n_children(model, NULL);
	if (header->rowNum >= numberOfRows) {
		ULOG_DEBUG_F("Row number %d does not exist in current view!\n", header->rowNum);
		return FALSE;
	}

	path = gtk_tree_path_new_from_indices(header->rowNum, -1);
	if (gtk_tree_model_get_iter(model, &iter, path)) {
		gtk_tree_model_get(model, &iter, COL_MSG_ID, &header->msgid,
				   COL_FROM_ADDR, &header->from, COL_TO_ADDR,
				   &header->to, COL_CC_ADDR, &header->cc, COL_BCC_ADDR,
				   &header->bcc, COL_SUBJECT, &header->subject,
				   COL_REPLY_TO, &header->reply_to, COL_DISPLAY_DATE,
				   &header->date, COL_SENT_DATE, &header->sent_date,
				   COL_RECV_DATE, &header->recieve_date,
				   COL_CREATED_DATE, &header->creation_date,
				   COL_PRIORITY, &header->priority, COL_MSG_TYPE,
				   &header->msgType, COL_FULLY_DOWNLOADED,
				   &header->fully_downloaded, COL_BODY_DOWNLOADED,
				   &header->body_downloaded, COL_RECV_PROTOCOL,
				   &header->protocol, COL_READ_RECEIPT,
				   &header->read_receipt, COL_COPY_TO_ADDR,
				   &header->copy_to_own_addr, COL_IN_USE_STATUS,
				   &header->in_use, COL_EDITABLE_STATUS,
				   &header->editable_status, COL_READ_STATUS,
				   &header->read_status, COL_SMIME_TYPE,
				   &header->smime_type, COL_NULL_SUBJECT, &noSubject, -1);
		if (noSubject == TRUE) {
			g_free(header->subject);
			header->subject = NULL;
		}
		header->mimeInfo = NULL;
		header->msgInfo = NULL;

		ULOG_DEBUG_F("Row data : msgid - %s\tfrom - %s\tto - %s\tcc - %s \
      \tsubject - %s\tpriority - %d\t copytoown =%d\tread_recepient = %d\n", header->msgid, header->from, header->to, header->cc, header->subject, header->priority, header->copy_to_own_addr, header->read_receipt);
	} else {
		ULOG_DEBUG_F("Row number %d does not exist in current view!\n", header->rowNum);
		gtk_tree_path_free(path);
		return FALSE;
	}

	gtk_tree_path_free(path);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return TRUE;
}

/**
This is a helper function used to show the 'Delete E-mail Confirmation'
dialog to the user.

@param  msg_count    Number of messages for deletion
@param  device_name  Name of the device
@return gint         0 --> Cancel Delete operation
                     1 --> Delete e-mail from device and server
                     2 --> Delete e-mail from device
*/
static gint ui_delete_email_confirmation_dialog(gint msg_count, const gchar * device_name)
{
	gint deleteType = 0, result = 0;
	GtkWidget *confirmationDialog = NULL;
	GtkWidget *radio1 = NULL;
	GtkWidget *radio2 = NULL;
	GtkWidget *delete_button = NULL;
	GtkWidget *cancel_button = NULL;
	GSList *radio_group = NULL;
	GString *radio_label = NULL;
	const gchar *delete_button_text = NULL;
	gchar *delete_text = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	radio_label = g_string_new(NULL);
	CHECK_PARAM((radio_label == NULL), "Memory allocation failed", return deleteType);
	if (msg_count > 1) {
		delete_text =
		    g_strdup_printf(_("mcen_nc_del_multi_from_phone_and_server"), device_name);
		g_string_printf(radio_label, "%s %s", _("mcen_nc_delete_emails_from"), delete_text);
		g_free(delete_text);
		radio1 = gtk_radio_button_new_with_label(NULL, radio_label->str);
		radio_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radio1));

		delete_text = g_strdup_printf(_("mcen_nc_del_multi_from_phone"), device_name);
		g_string_printf(radio_label, "%s %s", _("mcen_nc_delete_emails_from"), delete_text);
		g_free(delete_text);
		radio2 = gtk_radio_button_new_with_label(radio_group, radio_label->str);
		delete_button_text = _("mcen_bd_multi_email_delete");
	} else {
		delete_text = g_strdup_printf(_("mcen_nc_del_from_phone_and_server"), device_name);
		g_string_printf(radio_label, "%s %s", _("mcen_nc_delete_email_from"), delete_text);
		g_free(delete_text);
		radio1 = gtk_radio_button_new_with_label(NULL, radio_label->str);
		radio_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radio1));

		delete_text = g_strdup_printf(_("mcen_nc_del_from_phone"), device_name);
		g_string_printf(radio_label, "%s %s", _("mcen_nc_delete_email_from"), delete_text);
		g_free(delete_text);
		radio2 = gtk_radio_button_new_with_label(radio_group, radio_label->str);
		delete_button_text = _("mcen_bd_email_delete");
	}
	confirmationDialog =
	    gtk_dialog_new_with_buttons(_("mcen_ti_delete_email_confirmation_title"),
					GTK_WINDOW(app),
					GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, NULL);
	delete_button =
	    gtk_dialog_add_button(GTK_DIALOG(confirmationDialog), delete_button_text,
				  GTK_RESPONSE_ACCEPT);
	cancel_button =
	    gtk_dialog_add_button(GTK_DIALOG(confirmationDialog), _("mcen_bd_cancel"),
				  GTK_RESPONSE_REJECT);
	gtk_dialog_set_has_separator(GTK_DIALOG(confirmationDialog), FALSE);
	gtk_window_set_default_size(GTK_WINDOW(confirmationDialog),
				    ui_predict_dialog_width_based_on_title
				    (confirmationDialog,
				     _("mcen_ti_delete_email_confirmation_title")), -1);

	gtk_widget_show(radio1);
	gtk_widget_show(radio2);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(confirmationDialog)->vbox), radio1, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(confirmationDialog)->vbox), radio2, TRUE, TRUE, 0);

	/*set the appropriate radio button according to previous selection. */
	if (ui_get_delete_email_info()) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio1), TRUE);
	} else {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio2), TRUE);
	}
	g_signal_connect(G_OBJECT(confirmationDialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancel_button);

	result = gtk_dialog_run(GTK_DIALOG(confirmationDialog));
	if (result == GTK_RESPONSE_ACCEPT) {
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio1))) {
			deleteType = 1;
			/*set 1st radio button is active into persistent settings */
			ui_set_delete_email_info(TRUE);
		} else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio2))) {
			deleteType = 2;
			/*set 2nd radio button is active into persistent settings */
			ui_set_delete_email_info(FALSE);
		}
		gtk_widget_destroy(confirmationDialog);
	} else if (result == GTK_RESPONSE_REJECT) {
		deleteType = 0;
		gtk_widget_destroy(confirmationDialog);
	}

	g_string_free(radio_label, TRUE);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return deleteType;
}

/**
This is a helper function used to check the type of messages that user has
selected for deletion, and display appropriate confirmation note.

@param  selectedMsgs    Pointer to the list of selected rows
@param  headers_only    Flag to indicate type of messages selected
@param  remote_copy     Flag to indicate if selected messages have remote
                        copies
@param  download_status_none Flag to indicate if selected messages are not
                             yet downloaded
@param  subject         Pointer to subject string of last selected message
@param  deleteType      Integer pointer to store type of deletion
                        (local/local+remote)
@return boolean         TRUE  --> user has confirmed the action
                        FALSE --> user has denied the action
*/
static gboolean
ui_show_delete_confirmation_note(GSList * selectedMsgs,
				 gboolean headers_only,
				 gboolean remote_copy,
				 gboolean download_status_none,
				 gboolean no_subject, const gchar * subject, gint * deleteType)
{
	GString *confirmationNote = NULL;
	gboolean success = FALSE;
	gint result = -1;
	gchar *delete_note_button_text = NULL;
	gchar *device_name = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((selectedMsgs == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((deleteType == NULL), "Input param check failed", return FALSE);

	confirmationNote = g_string_new(NULL);
	CHECK_PARAM((confirmationNote == NULL), "Memory allocation failed", return FALSE);

	//Check if all the selected mails are type HEADERS_ONLY
	//If so local+server delete is needed
	if (download_status_none) {
		device_name = ui_get_device_name();
		if (g_slist_length(selectedMsgs) == 1) {
			if (no_subject == FALSE) {
				g_string_printf(confirmationNote,
						_("mcen_nc_delete_n_dl_message1"), device_name);
				g_string_append_printf(confirmationNote, "\n%s", subject);
			} else {
				g_string_printf(confirmationNote,
						_("mcen_nc_delete_n_dl_message1"), device_name);
			}
			delete_note_button_text = _("mcen_bd_delete");
		} else {
			g_string_printf(confirmationNote,
					_("mcen_nc_delete_n_dl_messages1"), device_name);
			delete_note_button_text = _("mcen_bd_multi_delete");
		}
		//Display deletion notification
		success =
		    get_user_confirmation(confirmationNote->str,
					  delete_note_button_text,
					  _("mcen_bd_delete_cancel"), CONFIRM_DIALOG);
		*deleteType = 1;
	}
	//Check if all the selected mails do not have a remote copy
	//If so only local delete is needed
	else if (!remote_copy) {
		if (g_slist_length(selectedMsgs) == 1) {
			if (no_subject == FALSE) {
				g_string_printf(confirmationNote, "%s\n%s",
						_("emev_nc_delete_message"), subject);
			} else {
				g_string_printf(confirmationNote, _("emev_nc_delete_message"));
			}
		} else {
			g_string_printf(confirmationNote, _("emev_bd_delete_messages"));
		}

		//Display deletion notification
		success =
		    get_user_confirmation(confirmationNote->str,
					  _("emev_bd_delete_messages_delete"),
					  NULL, CONFIRM_DIALOG);
		*deleteType = 2;
	}
	//Otherwise possible scenarios are :
	//  selection contains combination of headers & downloaded mails, or
	//  some of the mails have remote copies
	//In both cases ask user if he wants to do local or local+server delete
	else {
		device_name = ui_get_device_name();
		result =
		    ui_delete_email_confirmation_dialog(g_slist_length(selectedMsgs), device_name);
		if (result != 0) {
			*deleteType = result;
			success = TRUE;
		}
	}
	g_free(device_name);

	g_string_free(confirmationNote, TRUE);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return success;
}

/**
This is a helper function used to check the type of messages that user has
selected for deletion, and display appropriate confirmation note.

@param	model		Pointer to the folder view storage model
@param	selectedMsgs	Pointer to the list of selected rows
@param	orig_model_num	Folder number of view storage model
@param	deleteType	Integer pointer to store type of deletion
                        (local/local+remote)
@param	undo_operation	Whether current operation is due to UNDO
@param	msg_id_list	Pointer to store the selected message-ids
@param	num_of_msgs_in_use Integer pointer to store number of messages in use
@return boolean		TRUE  --> user has confirmed the action
			FALSE --> user has denied the action
*/
static gboolean
ui_get_delete_operation_confirmation(GtkTreeModel * model,
				     GSList * selectedMsgs,
				     gint orig_model_num, gint * deleteType,
				     gboolean undo_operation,
				     GSList ** msg_id_list, gint * num_of_msgs_in_use)
{
	GSList *node = NULL;
	GtkTreeIter iter;
	gboolean success = TRUE, download_status_none = TRUE, headers_only = TRUE;
	gboolean remote_copy = FALSE, fully_downloaded = FALSE, remote = FALSE;
	gboolean body_downloaded = FALSE;
	gint msg_type = -1, last_row_num = -1;
	gchar *subject = NULL, *msg_id = NULL;
	gboolean no_subject = FALSE, msg_in_use = FALSE;
	GtkTreePath *path = NULL;
	gint *indices = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	CHECK_PARAM((model == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((selectedMsgs == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((deleteType == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((msg_id_list == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((num_of_msgs_in_use == NULL), "Input param check failed", return FALSE);


	//Parse through the selected messages and find their downloaded status
	for (node = selectedMsgs; node != NULL; node = node->next) {
		path = (GtkTreePath *) node->data;

		if (path) {
			g_free(subject);
			if (gtk_tree_model_get_iter(model, &iter, path)) {
				gtk_tree_model_get(model, &iter, COL_FULLY_DOWNLOADED,
						   &fully_downloaded, COL_BODY_DOWNLOADED,
						   &body_downloaded, COL_REMOTE_COPY, &remote,
						   COL_MSG_TYPE, &msg_type, COL_SUBJECT,
						   &subject, COL_NULL_SUBJECT, &no_subject,
						   COL_IN_USE_STATUS, &msg_in_use, COL_MSG_ID,
						   &msg_id, -1);
				if (msg_type != 1) {
					headers_only = FALSE;
				}
				if (fully_downloaded == TRUE || body_downloaded == TRUE) {
					download_status_none = FALSE;
				}
				if (remote) {
					remote_copy = TRUE;
				}
				/*Prepare list of the message_ids.
				   Note that in use messages cannot be deleted. */
				if (!msg_in_use) {
					*msg_id_list = g_slist_append(*msg_id_list, msg_id);
				} else {
					*num_of_msgs_in_use += 1;
					g_free(msg_id);
				}
			}

			/*Find the row number of last selected message */
			indices = gtk_tree_path_get_indices(path);
			if (indices != NULL && indices[0] > last_row_num) {
				last_row_num = indices[0];
			}
		}
	}

	//Check if nothing is selected
	if (g_slist_length(selectedMsgs) == 0) {
		ULOG_DEBUG_F("Nothing to delete!\n");
		success = FALSE;
	}

	/*User confirmation is not needed if this is a UNDO operation */
	else if (!undo_operation) {
		/*If delete operation is done from an opened draft message,
		   take the draft's subject field into consideration */
		if (email_main_view->message_handler->delete_draft) {
			g_free(subject);
			subject = NULL;
			no_subject = TRUE;
			if (email_main_view->message_handler->current_draft_subject != NULL) {
				subject =
				    g_strdup(email_main_view->message_handler->
					     current_draft_subject);
				no_subject = FALSE;
			}
		}
		success =
		    ui_show_delete_confirmation_note(selectedMsgs, headers_only,
						     remote_copy, download_status_none,
						     no_subject, subject, deleteType);
	}

	g_free(subject);

	/*We have to select the next immediate item after the last selected row
	   after completing the delete operation. Store its message-id. */
	if (success) {
		g_free(email_main_view->message_handler->msg_to_be_selected_after_del_opr);
		email_main_view->message_handler->msg_to_be_selected_after_del_opr = NULL;

		if (gtk_tree_model_iter_n_children(model, NULL) > last_row_num + 1) {
			path = gtk_tree_path_new_from_indices(last_row_num + 1, -1);
		} else {
			path = gtk_tree_path_new_from_indices(last_row_num - 1, -1);
		}
		if (gtk_tree_model_get_iter(model, &iter, path)) {
			gtk_tree_model_get(model, &iter, COL_MSG_ID,
					   &email_main_view->message_handler->
					   msg_to_be_selected_after_del_opr, -1);
		}
		gtk_tree_path_free(path);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return success;
}

/**
This is a  function used to get the list of msgids that are retrived &
a list of msgids that are not retrived out of the selected msgs for copying, 

@param	model		Pointer to the folder view storage model
@param	selectedMsgs    Pointer to the list of GtkTreePaths on which copy operation is done.
@param  msg_id_list     is the OUT param conatining the list of msgids on which copy operation to be done.
@param  unfetched_msg_id_list    is the OUT param conatining the list of msgids 
				 which are not retrieved.
@param  num_of_msgs_in_use is the OUT param representing the number of in_use messages 
		        out the selected messages
*/

static void
ui_get_msg_ids_for_copy_operation(GtkTreeModel * model,
				  GSList * selectedMsgs,
				  GSList ** msg_id_list,
				  GSList ** unfetched_msg_id_list, gint * num_of_msgs_in_use)
{
	GSList *node = NULL;
	GtkTreeIter iter;
	gboolean fully_downloaded = FALSE, msg_in_use = FALSE;
	gchar *msg_id = NULL;
	GtkTreePath *path = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	CHECK_PARAM((model == NULL), "Input param check failed", return);
	CHECK_PARAM((selectedMsgs == NULL), "Input param check failed", return);
	CHECK_PARAM((msg_id_list == NULL), "Input param check failed", return);
	CHECK_PARAM((unfetched_msg_id_list == NULL), "Input param check failed", return);
	CHECK_PARAM((num_of_msgs_in_use == NULL), "Input param check failed", return);

	//Parse through the selected messages and find their downloaded status
	for (node = selectedMsgs; node != NULL; node = node->next) {
		path = (GtkTreePath *) node->data;
		if (path) {
			if (gtk_tree_model_get_iter(model, &iter, path)) {
				gtk_tree_model_get(model, &iter, COL_MSG_ID, &msg_id,
						   COL_FULLY_DOWNLOADED, &fully_downloaded,
						   COL_IN_USE_STATUS, &msg_in_use, -1);
				/*Prepare list of the message_ids.
				   Note that in use messages cannot be deleted. */
				if (!msg_in_use) {
					if (fully_downloaded) {
						if (is_msg_sending(msg_id))
							*msg_id_list =
							    g_slist_append(*msg_id_list, msg_id);
					} else {
						*unfetched_msg_id_list =
						    g_slist_append(*unfetched_msg_id_list, msg_id);
					}
				} else {
					*num_of_msgs_in_use += 1;
					g_free(msg_id);
				}
			}
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to check the type of messages that user has
selected for moving, and display appropriate confirmation note.and prepare 
a list of msgids on which move to operation can be done.

@param	model		Pointer to the folder view storage model
@param	paths_list	Pointer to the list of GtkTreePaths on which move operation is done.
@param	orig_model_num  is the source model number when move to operation is done on messages.
@param	undo_operation  is the gboolean which says the operation is initiated by undo or not.
@param  msg_id_list     is the OUT param conatining the list of msgids on which move operation to be done.
@param  num_of_msgs_in_use is the OUT param representing the number of in_use messages 
		        out the selected messages
@return boolean		TRUE  --> user has confirmed the action
			FALSE --> user has denied the action
*/
static gboolean
ui_get_move_operation_confirmation(GtkTreeModel * model, GSList * paths_list,
				   gint orig_model_num,
				   gboolean undo_operation,
				   GSList ** msg_id_list, gint * num_of_msgs_in_use)
{
	GSList *node = NULL;
	GtkTreeIter iter;
	gboolean success = FALSE, download_status_partial = FALSE;
	gboolean download_status_full = FALSE, status = FALSE, msg_in_use = FALSE;
	gboolean remote_copy = FALSE, msgs_remote_status = FALSE;
	GString *confirmationNote = NULL;
	GtkTreePath *path = NULL;
	gchar *msg_id = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	CHECK_PARAM((model == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((paths_list == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((msg_id_list == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((num_of_msgs_in_use == NULL), "Input param check failed", return FALSE);

	//Parse through the selected messages and find their downloaded status
	for (node = paths_list; node != NULL; node = node->next) {
		path = ((GtkTreePath *) node->data);
		if (path) {
			if (gtk_tree_model_get_iter(model, &iter, path)) {
				gtk_tree_model_get(model, &iter, COL_FULLY_DOWNLOADED, &status,
						   COL_IN_USE_STATUS, &msg_in_use, COL_MSG_ID,
						   &msg_id, COL_REMOTE_COPY, &remote_copy, -1);
				if (status) {
					download_status_full = TRUE;
				} else {
					download_status_partial = TRUE;
					msgs_remote_status = TRUE;
				}
				CHECK_PARAM((remote_copy == TRUE), "Remote copy exists",
					    msgs_remote_status = TRUE);
				/*Prepare list of the message_ids.
				   Note that in use messages cannot be moved. */
				if (!msg_in_use) {
					*msg_id_list = g_slist_append(*msg_id_list, msg_id);
				} else {
					*num_of_msgs_in_use += 1;
					g_free(msg_id);
				}
			}
		}
	}

	//Check if nothing is selected
	CHECK_PARAM((g_slist_length(paths_list) == 0), "Nothing to move!", return success);

	/*User confirmation is not needed if :
	   a) if this is UNDO operation
	   b) folder is other than Inbox
	   c) messages do not have any remote copies */
	CHECK_PARAM((undo_operation || msgs_remote_status == FALSE
		     || orig_model_num != INBOX_MODEL_NUM),
		    "No need for user confirmation", return TRUE);

	//Prepare confirmation note text
	confirmationNote = g_string_new(NULL);
	if (g_slist_length(paths_list) == 1) {
		//Check if the selected mail is partially downloaded
		if (download_status_partial) {
			g_string_printf(confirmationNote, _("mcen_nc_move_header"));
			success = get_user_confirmation(confirmationNote->str,
							_("mcen_bd_move_header"),
							_("mcen_bd_move_cancel_header"),
							CONFIRM_DIALOG);
		}
		//Check if the selected mail is fully downloaded
		if (download_status_full) {
			g_string_printf(confirmationNote, _("mcen_nc_move_retrieve"));
			success = get_user_confirmation(confirmationNote->str,
							_("mcen_bd_move_retrieve"),
							_("mcen_bd_move_cancel_header"),
							CONFIRM_DIALOG);
		}
	} else if (g_slist_length(paths_list) > 1) {
		//Check if all the selected mails are only partially downloaded
		if (download_status_partial && !download_status_full) {
			g_string_printf(confirmationNote, _("mcen_nc_move_headers"));
			success = get_user_confirmation(confirmationNote->str,
							_("mcen_bd_move_headers"),
							_("mcen_bd_move_cancel_header"),
							CONFIRM_DIALOG);
		}
		//Check if all the selected mails are only fully downloaded
		if (!download_status_partial && download_status_full) {
			g_string_printf(confirmationNote, _("mcen_nc_move_retrieves"));
			success = get_user_confirmation(confirmationNote->str,
							_("mcen_bd_move_retrieves"),
							_("mcen_bd_move_cancel_header"),
							CONFIRM_DIALOG);
		}
		//Check if the selected mails contain both partially & fully downloaded mails
		if (download_status_partial && download_status_full) {
			g_string_printf(confirmationNote, _("mcen_nc_move_headers"));
			success = get_user_confirmation(confirmationNote->str,
							_("mcen_bd_move_headers"),
							_("mcen_bd_move_cancel_header"),
							CONFIRM_DIALOG);
		}
	}

	g_string_free(confirmationNote, TRUE);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return success;
}

/**
This is a helper function used to check the type of action and display
appropriate confirmation note.

@param	action          Type of action (DELETE, COPY/PASTE, CUT/PASTE, MOVE)
@param	orig_model	Pointer to the folder view storage model
@param	orig_model_num  storage model number of originating folder
@param	paths_list         Pointer to the list of GtkTreePaths  of selected rows
@param	msg_id_list     Pointer to the list of message-ids of selected rows
@param	deleteType	Integer pointer to store type of deletion
                        (local/local+remote)
@return boolean		TRUE  --> user has confirmed the action
			FALSE --> user has denied the action
*/
static gboolean
ui_get_confirmation_for_msghandler_operations(msgHandlingAction action,
					      GtkTreeModel * orig_model,
					      gint orig_model_num,
					      gint dest_model_num,
					      GSList * paths_list,
					      GSList ** msg_id_list,
					      gint * delete_type,
					      const gchar * dest_folder,
					      GSList ** unfetched_msg_id_list,
					      gint * num_of_msgs_in_use, gboolean undo_operation)
{
	gboolean user_confirmation = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	CHECK_PARAM((orig_model == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((paths_list == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((delete_type == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((msg_id_list == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((unfetched_msg_id_list == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((num_of_msgs_in_use == NULL), "Input param check failed", return FALSE);

	//Get user confirmation for MOVE or CUT/PASTE operations.
	//User confirmation is needed only for Inbox folder
	if (action == ACTION_MOVE) {
		user_confirmation =
		    ui_get_move_operation_confirmation(orig_model, paths_list, orig_model_num,
						       undo_operation, msg_id_list,
						       num_of_msgs_in_use);
	}
	//Get user confirmation for DELETE operation
	else if (action == ACTION_DELETE) {
		user_confirmation =
		    ui_get_delete_operation_confirmation(orig_model, paths_list,
							 orig_model_num, delete_type,
							 undo_operation, msg_id_list,
							 num_of_msgs_in_use);
	}
	//User confirmation is not needed for COPY/PASTE operation
	else if (action == ACTION_PASTE) {
		user_confirmation = TRUE;
		ui_get_msg_ids_for_copy_operation(orig_model,
						  paths_list,
						  msg_id_list,
						  unfetched_msg_id_list, num_of_msgs_in_use);
	}
	//Update Undo data
	if (user_confirmation) {
		if (email_main_view->undo->message_id_list != NULL) {
			g_slist_foreach(email_main_view->undo->message_id_list,
					(GFunc) g_free, NULL);
			g_slist_free(email_main_view->undo->message_id_list);
			email_main_view->undo->message_id_list = NULL;
		}
		if (action == ACTION_MOVE) {
			email_main_view->undo->operation = OPER_MOVE;
			email_main_view->undo->dest_folder = orig_model_num;
			email_main_view->undo->orig_folder = dest_model_num;
		} else if (action == ACTION_PASTE) {
			email_main_view->undo->operation = OPER_DELETE;
			email_main_view->undo->orig_folder = dest_model_num;
		} else if (action == ACTION_DELETE) {
			email_main_view->undo->operation = OPER_NONE;
			email_main_view->undo->orig_folder = orig_model_num;
		}
	}
	//Clean the lists on negative confirmation
	if (!user_confirmation) {
		g_slist_foreach(paths_list, (GFunc) gtk_tree_path_free, NULL);
		g_slist_free(paths_list);
		g_slist_foreach(*msg_id_list, (GFunc) g_free, NULL);
		g_slist_free(*msg_id_list);
		*msg_id_list = NULL;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return user_confirmation;
}

/**
This is a helper function used to display "Unable to move" notification.

@param  num_of_msgs_in_use  Number of messages currrently in use
@return void
*/
void ui_show_msgs_in_use_notification(gint operation, gint num_of_msgs_in_use)
{
	GString *notification = NULL;
	gchar *count_str = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	CHECK_PARAM((num_of_msgs_in_use < 0), "Input param check failed", return);

	notification = g_string_new(NULL);

	switch (operation) {
	case ACTION_MOVE:
		{
			if (num_of_msgs_in_use == 1) {
				g_string_printf(notification, _("emev_nc_unabletomove_item"));
				get_user_confirmation(notification->str, NULL, NULL, INFO_BANNER);
			}

			else if (num_of_msgs_in_use > 1) {
				g_string_printf(notification, _("emev_bd_unabletomove_items"),
						num_of_msgs_in_use);
				get_user_confirmation(notification->str, NULL, NULL, INFO_BANNER);
			}
			break;
		}

	case ACTION_DELETE:
		{
			if (num_of_msgs_in_use == 1) {
				gtk_infoprint((GtkWindow *) app,
					      _("mcen_ib_message_unableto_delete"));
			} else if (num_of_msgs_in_use > 1) {
				count_str = g_strdup_printf("%d", num_of_msgs_in_use);
				g_string_printf(notification,
						_("mcen_nc_unable_to_delete_n_messages"),
						count_str);
				get_user_confirmation(notification->str, NULL, NULL, INFO_BANNER);
				g_free(count_str);
			}
			break;
		}

	default:
		{
			break;
		}
	}

	g_string_free(notification, TRUE);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to process each message during message handler
operations.

@param  row_ref        Pointer to the GtkTreeRowReference of current row
@param  origModel      Pointer to the folder view storage model
@param  numOfMsgsInUse Integer pointer to store num of messages currently in use
@param  origFolder     Name of the originating folder
@param  destFolder     Name of the destination folder
@param  action         Type of action
@param  unfetched_msg_id_list   List to store info of partially downloaded msgs
@param  deleteType     Type of deletion (local, local+remote)
@return void
*/
static void
ui_process_current_messages(const gchar * origFolder, const gchar * destFolder,
			    msgHandlingAction action, GSList * msg_id_list, gint deleteType)
{
	gboolean delete_server_copy = FALSE;
	osso_pack_return_t ret_val = OSSO_PACK_OK;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((origFolder == NULL), "Input param check failed", return);
	CHECK_PARAM((msg_id_list == NULL), "Input param check failed", return);

	switch (action) {
	case ACTION_MOVE:
		{
			ret_val = ui_dbus_folder_move_msgs(origFolder, destFolder, msg_id_list);
			break;
		}
	case ACTION_PASTE:
		{
			ret_val = ui_dbus_folder_copy_msgs(origFolder, destFolder, msg_id_list);
			break;
		}
	case ACTION_DELETE:
		{
			if (deleteType == 1) {
				delete_server_copy = TRUE;
			}
			ret_val =
			    ui_dbus_folder_remove_msgs(origFolder, msg_id_list, delete_server_copy);
			break;
		}
	default:
		{
			ULOG_DEBUG_F("Invalid operation");
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function to check whether paste operation is allowed
to the currently selected folder. If not allowed it will show appropriate
info banners.

@param  folder_path  Path of originating folder from which messages were
                     copied/cut.
@param  cut_operation_done  Flag indicating whether previous operation was
                            CUT or COPY.
@param  cut_copy_folder_model  Pointer to store the tree model object of
                               the originating folder.
@param  orig_folder_num  Out param for storing the folder number from which
                         cut/copy operation was done.
*/
static gboolean
ui_proceed_with_paste_operation(const gchar * folder_path,
				gboolean cut_operation_done,
				GtkTreeModel ** cut_copy_folder_model, gint * orig_folder_num)
{
	gboolean ret_val = FALSE;
	gint copy_cut_folder = -1;
	gchar *parent_name = NULL, *folder_name = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((folder_path == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((cut_copy_folder_model == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((orig_folder_num == NULL), "Input param check failed", return FALSE);

	if (ui_get_folder_names_from_path(folder_path, &parent_name, &folder_name) == FALSE) {
		ULOG_DEBUG_F("Folder not available '%s'", folder_path);
		return ret_val;
	}
	copy_cut_folder = ui_get_folder_model_number(folder_name, parent_name);
	*orig_folder_num = copy_cut_folder;
	*cut_copy_folder_model = ui_get_sorted_folder_model(copy_cut_folder);

	if (copy_cut_folder == -1) {
		//Originating folder is set to -1, pasting is not allowed
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_paste"));
	} else if (email_main_view->folder_manager->currently_displayed_model == OUTBOX_MODEL_NUM) {
		//destination model is Outbox, which is not allowed
		if (cut_operation_done == TRUE) {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_move_outbox"));
		} else {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_copy_outbox"));
		}
	} else if (email_main_view->folder_manager->currently_displayed_model == SENT_MODEL_NUM) {
		//destination model is Sent folder, which is not allowed
		if (cut_operation_done == TRUE) {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_move_sentfolder"));
		} else {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_copy_sentfolder"));
		}
	} else if (email_main_view->folder_manager->currently_displayed_model == DRAFT_MODEL_NUM) {
		//destination model is Sent folder, which is not allowed
		if (cut_operation_done == TRUE) {
			gtk_infoprint((GtkWindow *) app,
				      DGETTEXT_HCS("sfil_ib_unable_to_move_selected_items"));
		} else {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_paste"));
		}
	} else if ((gint) email_main_view->folder_manager->currently_displayed_model ==
		   copy_cut_folder) {
		if (cut_operation_done == TRUE) {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_move_samefolder"));
		} else {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_copy_samefolder"));
		}

	} else {
		ret_val = TRUE;
	}

	g_free(parent_name);
	g_free(folder_name);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return ret_val;
}

/**
This is a helper function to dim selected rows during CUT operation.

@param  paths_list      List of pointers to GtkTreePath
@param  folder_num   Sequence number of originating folder
@return void
*/

void ui_dim_selected_rows(GSList * paths_list, gint folder_num)
{
	GSList *node = NULL;
	GtkTreeModel *model = NULL, *child_model = NULL;
	GtkTreeIter iter, child_iter;
	GtkTreePath *path = NULL;
	gchar *msgid = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((paths_list == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets->messageListView == NULL),
		    "Input param check failed", return);

	/*Before dimming the rows, unset folder model from the tree view.
	   Otherwise it will cause flickering of the UI */
	model = ui_get_sorted_folder_model(folder_num);
	gtk_tree_view_set_model(GTK_TREE_VIEW(email_main_view->widgets->messageListView), NULL);
	for (node = paths_list; node != NULL; node = node->next) {
		path = (GtkTreePath *) node->data;
		if (gtk_tree_model_get_iter(model, &iter, path)) {
			gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT(model),
								       &child_iter, &iter);
			child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(model));
			gtk_tree_model_get(model, &iter, COL_MSG_ID, &msgid, -1);
			if (is_msg_sending(msgid))
				gtk_list_store_set(GTK_LIST_STORE(child_model), &child_iter,
						   COL_CUT_STATUS, FALSE, -1);
		}
	}

	/*Reset the folder model back to the tree view */
	model = ui_get_sorted_folder_model(folder_num);
	gtk_tree_view_set_model(GTK_TREE_VIEW(email_main_view->widgets->messageListView), model);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to perform various message handling operations
--> MOVE, CUT, COPY, PASTE, DELETE.

@param	origModelNum	Sequence number of the storage model of originating folder
@param	destModelNum	Sequence number of the storage model of destination folder
@param	action		Type of action
@param	undo_operation	is the gboolean specifing whether invoked as undo operation.
@param	row_ref		Pointer to the GtkTreeRowReference required only if
	MOVE TO/DELETE is invoked from viewer.
@return gboolean
*/
gboolean
ui_manage_selected_items(gint origModelNum, gint destModelNum,
			 msgHandlingAction action, gboolean undo_operation,
			 GtkTreeRowReference * row_ref)
{
	GSList *paths_list = NULL, *unfetched_msg_id_list = NULL;
	GSList *selected_rows = NULL, *msg_id_list = NULL;
	GtkTreeModel *origModel = NULL, *destModel = NULL;
	gchar *origFolder = NULL, *destFolder = NULL;
	gint numOfMsgsInUse = 0, deleteType = 2, num_of_copied_msgs_available = 0;
	gint num_of_unavailable_msgs = 0;
	msgHandlingAction operation = action;
	gboolean userConfirmation = TRUE, cut_operation_done = FALSE, copied = FALSE;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Get the selected row paths
	//Not needed for PASTE action, as the info is captured during CUT/COPY operation
	email_main_view->message_handler->num_of_failed_msgs = 0;
	email_main_view->message_handler->num_of_msgs_in_use = 0;
	switch (action) {
	case ACTION_MOVE:
	case ACTION_DELETE:
		{
			origFolder = ui_get_folder_name(origModelNum);
			origModel = ui_get_sorted_folder_model(origModelNum);
			/*Check if row_ref is provided for move operation
			 * i.e when invoked by viewer.*/
			if (!row_ref)
				ui_get_selected_rows((GtkTreeView *) email_main_view->widgets->
						     messageListView, &paths_list, undo_operation);
			else {
				paths_list = g_slist_append(paths_list,
							    gtk_tree_row_reference_get_path
							    (row_ref));
				gtk_tree_row_reference_free(row_ref);
			}
			break;
		}

	case ACTION_CUT:
	case ACTION_COPY:
		{
			origFolder = ui_get_folder_name(origModelNum);
			origModel = ui_get_sorted_folder_model(origModelNum);
			ui_get_selected_rows((GtkTreeView *) email_main_view->widgets->
					     messageListView, &paths_list, undo_operation);
			if (action == ACTION_COPY) {
				ui_get_msg_ids_for_copy_operation(origModel, paths_list,
								  &selected_rows,
								  &unfetched_msg_id_list,
								  &numOfMsgsInUse);
				if (unfetched_msg_id_list)
					userConfirmation =
					    copy_unretrieved_mails_confirmation
					    (unfetched_msg_id_list);
			} else {
				ui_get_selected_row_msg_ids((GtkTreeView *) email_main_view->
							    widgets->messageListView,
							    &selected_rows);
			}
			//Copy contents to clipboard
			CHECK_PARAM((action == ACTION_CUT), "Cut operation done",
				    cut_operation_done = TRUE);
			if (action == ACTION_CUT || !unfetched_msg_id_list) {
				osso_email_copy_to_clipboard(GTK_WIDGET(app), origFolder,
							     selected_rows, cut_operation_done);
				/*show info banner if sm messages are open*/
                                if(numOfMsgsInUse && action == ACTION_COPY){
                                        if(numOfMsgsInUse != g_slist_length (paths_list))
                                                gtk_infoprint(GTK_WINDOW(app),_("mcen_ib_cannot_copy_some"));
                                        else
                                                gtk_infoprint(GTK_WINDOW(app),_("mcen_ib_unable_to_copy"));
                                        copied = FALSE;
                                }else {
                                        copied = TRUE;
                                }
			}
			CHECK_PARAM((action == ACTION_CUT), "Setting cut folder number",
				    email_main_view->message_handler->cut_folder_num =
				    origModelNum);
			/*Dim the cut rows */
			if (action == ACTION_CUT) {
				ui_dim_selected_rows(paths_list, origModelNum);
				email_main_view->undo->operation = OPER_UNCUT;
			}
			if (userConfirmation && unfetched_msg_id_list) {
				retrieve_mail_contents(unfetched_msg_id_list, origModelNum);
				unfetched_msg_id_list =
				    g_slist_concat(unfetched_msg_id_list, selected_rows);
				email_main_view->message_handler->msgs_to_copy = unfetched_msg_id_list;
                                email_main_view->message_handler->num_of_msgs_in_use = numOfMsgsInUse;
                                copied = FALSE;
			} else {
				g_slist_foreach(selected_rows, (GFunc) g_free, NULL);
				g_slist_free(selected_rows);
			}

			g_free(origFolder);
			g_slist_foreach(paths_list, (GFunc) gtk_tree_path_free, NULL);
			g_slist_free(paths_list);
			ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
			return copied;
		}

	case ACTION_PASTE:
		{
			//Get contents from clipboard
			if (ui_check_clipboard_has_valid_emails
			    (&origFolder, &selected_rows, &cut_operation_done) == FALSE) {
				gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_paste"));
				g_free(origFolder);
				g_slist_foreach(selected_rows, (GFunc) g_free, NULL);
				g_slist_free(selected_rows);
				ULOG_DEBUG_F("No messages are available for pasting!");
				return FALSE;
			}
			//Do necessary checks before pasting data
			if (selected_rows == NULL
			    || ui_proceed_with_paste_operation(origFolder,
							       cut_operation_done,
							       &origModel,
							       &origModelNum) == FALSE) {
				g_free(origFolder);
				if (selected_rows != NULL) {
					g_slist_foreach(selected_rows, (GFunc) g_free, NULL);
					g_slist_free(selected_rows);
				}

				ULOG_DEBUG_F
				    ("Paste operation not allowed or no previous selection to paste!");
				return FALSE;
			}

			if (cut_operation_done) {
				operation = ACTION_MOVE;
				osso_email_cut_paste_from_clipboard_done(GTK_WIDGET(app));
			}
			num_of_unavailable_msgs = g_slist_length(selected_rows);
			ui_get_tree_paths(selected_rows, &paths_list);
			num_of_copied_msgs_available = g_slist_length(paths_list);

			num_of_unavailable_msgs -= num_of_copied_msgs_available;
			if (num_of_unavailable_msgs != 0) {
				gtk_infoprint((GtkWindow *) app, _("mcen_ib_messages_cannot_copy"));
			}
			ULOG_DEBUG_F
			    ("num_of_copied_msgs_available = %d, num_of_unavailable_msgs = %d\n",
			     num_of_copied_msgs_available, num_of_unavailable_msgs);

			g_slist_foreach(selected_rows, (GFunc) g_free, NULL);
			g_slist_free(selected_rows);
			break;
		}
	}

	destFolder = ui_get_folder_name(destModelNum);
	destModel = ui_get_unsorted_folder_model(destModelNum);

	//Show appropriate notifications before proceeding
	userConfirmation =
	    ui_get_confirmation_for_msghandler_operations(operation, origModel,
							  origModelNum,
							  destModelNum, paths_list,
							  &msg_id_list,
							  &deleteType, destFolder,
							  &unfetched_msg_id_list,
							  &numOfMsgsInUse, undo_operation);
	if (!userConfirmation) {
		g_free(origFolder);
		g_free(destFolder);
		ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
		return FALSE;
	}

	/*Copy unfetched messages at a later point if needed */
	if (operation == ACTION_PASTE) {
		email_main_view->message_handler->unfetched_msgs_to_copy = unfetched_msg_id_list;
		email_main_view->message_handler->dest_folder_to_copy = destModelNum;
		if (msg_id_list == NULL) {
			ui_copy_unfetched_messages_if_required();
		}
	}
	//Display notification for failed messages 
	email_main_view->message_handler->current_operation = operation;
	email_main_view->message_handler->num_of_failed_msgs = num_of_unavailable_msgs;
	email_main_view->message_handler->num_of_msgs_in_use = numOfMsgsInUse;
	ULOG_DEBUG_F("numOfMsgsInUse = %d\n", numOfMsgsInUse);

	//if (operation == ACTION_DELETE && msg_id_list == NULL)
	if (msg_id_list == NULL) {
		ui_show_note_for_failed_msgs();
	}
	//Do the message handling operations on selected messages
	ui_process_current_messages(origFolder, destFolder, operation, msg_id_list, deleteType);

	//Clean up memory
	g_free(origFolder);
	g_free(destFolder);

	g_slist_foreach(paths_list, (GFunc) gtk_tree_path_free, NULL);
	g_slist_free(paths_list);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return TRUE;
}

/**
This is a helper function to obtain subject field of a message,
given the message's msg-id and folder number.

@param  msgid      Message-id string
@param  model_num  Folder storage model number
@return gchar*     Subject string (to be freed by the calling function)
*/
gchar *ui_get_msg_subject(const gchar * msgid, gint model_num)
{
	gchar *subject = NULL, *msgId = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	CHECK_PARAM((msgid == NULL), "Input param check failed", return NULL);
	CHECK_PARAM((model_num < 0), "Input param check failed", return NULL);

	model = ui_get_unsorted_folder_model(model_num);
	if (!gtk_tree_model_get_iter_first(model, &iter)) {
		ULOG_DEBUG_F("Storage model empty for model number : %d!\n", model_num);
		return NULL;
	}
	//Traverse the rows until message-id is matched.
	do {
		gtk_tree_model_get(model, &iter, COL_MSG_ID, &msgId, COL_SUBJECT, &subject, -1);

		if (msgId != NULL && strncmp(msgid, msgId, MAX_STR_LEN(msgid, msgId)) == 0) {
			ULOG_DEBUG_F("Row found. \n");
			g_free(msgId);
			break;
		}
		g_free(msgId);
		g_free(subject);
	}
	while (gtk_tree_model_iter_next(model, &iter));

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return subject;
}

/**
This is a helper function used to check whether MOVE operation is allowed.
Operation is not allowed if destination folder is Outbox or Sent folder or
same as originating folder.

@param	destModelNum	Sequence number of the destination folder storage model
@param	window		Pointer to the Move_To_Folder dialog window
@return boolean		TRUE  --> operation is allowed
			FALSE --> operation is not allowed
*/
gboolean ui_is_move_operation_allowed(gint destModelNum, GtkWindow * window)
{
	gboolean success = FALSE;
	GtkWindow *parent = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	CHECK_PARAM((destModelNum < 0), "Input param check failed", return FALSE);

	//Resolve the parent window to display error infoprint
	if (window == NULL) {
		parent = GTK_WINDOW(app);
	} else {
		parent = window;
	}

	//Moving messages to Outbox, Sent or the originating folder is not allowed
	if (destModelNum == OUTBOX_MODEL_NUM) {
		gtk_infoprint(parent, _("mcen_ib_unable_to_move_outbox"));
	} else if (destModelNum == SENT_MODEL_NUM) {
		gtk_infoprint(parent, _("mcen_ib_unable_to_move_sentfolder"));
	} else if (destModelNum == DRAFT_MODEL_NUM) {
		gtk_infoprint(parent, _("sfil_ib_unable_to_move_selected_items"));
	} else if (destModelNum == email_main_view->folder_manager->currently_displayed_model) {
		gtk_infoprint(parent, _("mcen_ib_unable_to_move_samefolder"));
	} else {
		success = TRUE;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return success;
}

/*Signal Handler Functions*/

/**
This is a signal handler function used when OK button in Move_To_Folder
dialog is clicked.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@param	row_ref		Pointer to the GtkTreeRowReference required only if
	MOVE TO is invoked from viewer.
@return gboolean TRUE-->if sucess.
                 FALSE --else.
*/
static gboolean ui_on_ok_move_to_folder_dialog(GtkWidget * widget, gpointer data,
					       GtkTreeRowReference * row_ref)
{
	gint destModelNum = -1, count = 0;
	GtkTreeView *folderView = NULL, *listView = NULL;
	GtkWidget *window = NULL;
	GtkTreeSelection *folderSelection = NULL;
	GtkTreeModel *folderModel = NULL, *listModel = NULL, *child_model = NULL;
	gboolean result = FALSE;
	GtkFilePath *gtk_path = NULL;
	GtkFileSystem *fssystem = NULL;
	GtkTreeIter dest_iter;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the necessary input & global params
	CHECK_PARAM((data == NULL), "Input param check failed", return result);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return result);
	CHECK_PARAM((email_main_view->folder_manager == NULL),
		    "Input param check failed", return result);
	CHECK_PARAM((email_main_view->tags == NULL), "Input param check failed", return result);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return result);
	CHECK_PARAM((email_main_view->widgets->no_msgs_label == NULL),
		    "Input param check failed", return result);

	//Get the userdata pointers
	window = GTK_WIDGET(data);
	folderView = GTK_TREE_VIEW(EMAIL_UI_LOOKUP_OBJECT(window, "tree_view"));
	listView = GTK_TREE_VIEW(EMAIL_UI_LOOKUP_OBJECT(window, "list_view"));
	fssystem = (GtkFileSystem *) (EMAIL_UI_LOOKUP_OBJECT(window, "file_system"));

	if (folderView == NULL || listView == NULL || window == NULL || fssystem == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return result;
	}
	/* Getting the selected iter from path */
	gchar *full_path = gtk_file_chooser_get_current_folder((GtkFileChooser *) window);
	gtk_path = gtk_file_system_filename_to_path(fssystem, full_path);
	if (!gtk_file_system_memory_file_path_to_tree_iter(fssystem, &dest_iter, gtk_path)) {
		ULOG_DEBUG_F("Failed to get destination iter!\n");
		gtk_file_path_free(gtk_path);
		return result;
	}
	/* Getting the values from selected row */
	gtk_tree_model_get(GTK_TREE_MODEL(fssystem), &dest_iter,
			   GTK_FILE_SYSTEM_MEMORY_COLUMN_SIZE, &destModelNum, -1);

	//Get the folder selected
	folderSelection = gtk_tree_view_get_selection(folderView);
	folderModel = gtk_tree_view_get_model(folderView);
	listModel = gtk_tree_view_get_model(listView);
	child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(listModel));
	ui_count_marked_rows(listView, &count);

	if (ui_is_move_operation_allowed(destModelNum, GTK_WINDOW(window))) {
		//All checks are done, proceed with the move operation
		//Append the item to destination model and remove it in originating model
		result =
		    ui_manage_selected_items(email_main_view->folder_manager->
					     currently_displayed_model, destModelNum,
					     ACTION_MOVE, FALSE, row_ref);
		if (result) {
			email_main_view->tags->marking_mode = FALSE;
			ui_invoke_marking_mode(FALSE);
			//If list view is empty, update the appropriate menus & tool bar buttons
			if (gtk_tree_model_iter_n_children(listModel, NULL) == 0) {
				ui_update_folder_specific_widgets(email_main_view->
								  folder_manager->
								  currently_displayed_model);

				//Show 'No messages to show' label
				gtk_widget_show(email_main_view->widgets->no_msgs_label);
			}
		}
	}
	gtk_file_path_free(gtk_path);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return result;
}

/*Window Creation Functions*/

/**
This is a helper function used to create the file system model to be used
for Move_To_Folder dialog.

@param  folderView      Pointer to the tree view object of folder view
@param  listView        Pointer to the tree view object of message view
@param  device_name     Name of the device
@return GtkFileSystem   Pointer to the file system model
*/
static GtkFileSystem *ui_create_move_to_folder_dlg_model(GtkTreeView * folderView,
							 GtkTreeView * listView,
							 const gchar * device_name)
{
	GtkTreeModel *model = NULL;
	GtkTreeStore *memory = NULL;
	GtkFileSystem *fssystem = NULL;
	GtkTreeIter root_iter, iter, main_iter;
	GtkTreeIter child_iter, parent_iter;
	gchar *displayName = NULL, *childFolderName = NULL;
	gint no_of_children = 0, count = 0, model_num = -1;
	GdkPixbuf *folder_icon = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((folderView == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((listView == NULL), "Input param check failed", return FALSE);

	/*Load device icon if required */
	CHECK_PARAM((email_main_view->icons->device_icon == NULL),
		    "Pixbuf not loaded yet",
		    ui_load_icon_pixbuf(&email_main_view->icons->
					device_icon, "qgn_list_filesys_divc_cls", MSG_ICON_SIZE));

	/* This is main tree model of email application where all folders are available */
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	/* Getting the first iter of the main folder model */
	gtk_tree_model_get_iter_first(model, &parent_iter);

	/* Creating GtkFileSystem */
	fssystem = hildon_file_system_model_create_backend("filesystemmemory", FALSE);
	g_assert(fssystem != NULL);
	memory = GTK_TREE_STORE(fssystem);

	/*Adding root level folder to HildonFileSystemModel */
	gtk_tree_store_append(memory, &root_iter, NULL);
	gtk_tree_store_set(memory, &root_iter,
			   GTK_FILE_SYSTEM_MEMORY_COLUMN_ICON,
			   email_main_view->icons->device_icon,
			   GTK_FILE_SYSTEM_MEMORY_COLUMN_NAME, device_name,
			   GTK_FILE_SYSTEM_MEMORY_COLUMN_MIME, "x-directory/normal",
			   GTK_FILE_SYSTEM_MEMORY_COLUMN_SIZE, model_num,
			   GTK_FILE_SYSTEM_MEMORY_COLUMN_IS_FOLDER, TRUE, -1);
	/* Traversing through main folder model and getting required values from that iter.
	 * Appending a row in GtkTreeModel and setting these values in the newly created row.*/
	do {
		gtk_tree_model_get(model, &parent_iter, COL_DISPLAY_NAME, &displayName,
				   COL_MODEL_SEQ_NUM, &model_num, -1);
		if (model_num == OUTBOX_MODEL_NUM ||
		    model_num == SENT_MODEL_NUM || model_num == DRAFT_MODEL_NUM) {
			g_free(displayName);
			continue;
		}
		folder_icon = ui_get_folder_icon(model_num);

		gtk_tree_store_append(memory, &main_iter, &root_iter);
		gtk_tree_store_set(memory, &main_iter,
				   GTK_FILE_SYSTEM_MEMORY_COLUMN_ICON, folder_icon,
				   GTK_FILE_SYSTEM_MEMORY_COLUMN_NAME, displayName,
				   GTK_FILE_SYSTEM_MEMORY_COLUMN_MIME,
				   "x-directory/normal",
				   GTK_FILE_SYSTEM_MEMORY_COLUMN_SIZE, model_num,
				   GTK_FILE_SYSTEM_MEMORY_COLUMN_IS_FOLDER, TRUE, -1);
		no_of_children = gtk_tree_model_iter_n_children(model, &parent_iter);

		if (no_of_children > 0) {
			for (count = 0; count < no_of_children; count++) {
				if ((gtk_tree_model_iter_nth_child(model,
								   &child_iter, &parent_iter,
								   count)) == TRUE) {
					gtk_tree_model_get(model, &child_iter, COL_DISPLAY_NAME,
							   &childFolderName, COL_MODEL_SEQ_NUM,
							   &model_num, -1);
					if (childFolderName != NULL) {
						gtk_tree_store_append(memory, &iter, &main_iter);
						gtk_tree_store_set(memory, &iter,
								   GTK_FILE_SYSTEM_MEMORY_COLUMN_ICON,
								   NULL,
								   GTK_FILE_SYSTEM_MEMORY_COLUMN_NAME,
								   childFolderName,
								   GTK_FILE_SYSTEM_MEMORY_COLUMN_MIME,
								   "x-directory/normal",
								   GTK_FILE_SYSTEM_MEMORY_COLUMN_SIZE,
								   model_num,
								   GTK_FILE_SYSTEM_MEMORY_COLUMN_IS_FOLDER,
								   TRUE, -1);
						g_free(childFolderName);
						childFolderName = NULL;
					}
				}
			}
		}
		g_free(displayName);
		displayName = NULL;
	}
	while (gtk_tree_model_iter_next(model, &parent_iter));

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return fssystem;
}

/**
This is a window creation function used to create the Move_To_Folder dialog.

@param  folderView      Pointer to the tree view object of folder view
@param  listView        Pointer to the tree view object of message view
@param  msg_send_flag   Status of the message if any one in the currently 
                        marked messages is in being sent mode.  
@param	row_ref		Pointer to the GtkTreeRowReference required only if
	MOVE TO is invoked from viewer.
@return boolean         TRUE-->Positive user confirmation and move success
                        FALSE-->Negative user confirmation
*/
gboolean
ui_create_move_to_folder_dialog(GtkTreeView * folderView,
				GtkTreeView * listView, gboolean msg_send_flag,
				GtkTreeRowReference * row_ref)
{
	GtkWidget *MoveToDialog = NULL;
	HildonFileSystemModel *fsmodel = NULL;
	gint response = -1;
	GtkFileSystem *fssystem = NULL;
	gboolean success = FALSE, user_confirmation = TRUE, refresh_dialog = TRUE;
	gchar *device_name = NULL;

	device_name = ui_get_device_name();
	if (device_name == NULL) {
		device_name = g_strdup("/");
	}

	while (success != TRUE) {
		if (refresh_dialog) {
			/* Creating HildonFileSystemModel with GtkFileSystem */
			fssystem =
			    ui_create_move_to_folder_dlg_model(folderView, listView, device_name);
			fsmodel =
			    HILDON_FILE_SYSTEM_MODEL(g_object_new
						     (HILDON_TYPE_FILE_SYSTEM_MODEL,
						      "backend-object", fssystem, "ref-widget",
						      app, "root-dir", device_name,
						      "multi-root", FALSE, NULL));
			/* Creating Move to Folder dialog with this HildonFileSystemModel */
			MoveToDialog =
			    hildon_file_chooser_dialog_new_with_properties((GtkWindow *) app,
									   "action",
									   GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
									   "file-system-model",
									   fsmodel,
									   "show-folder-button",
									   FALSE, NULL);
			if (ossohelp_exists(HELP_ID_MOVETOFOLDERS) == OSSO_OK) {
				gtk_dialog_help_enable(GTK_DIALOG(MoveToDialog));
				g_signal_connect(G_OBJECT(MoveToDialog), "help",
						 G_CALLBACK(ui_on_dialog_help_activate),
						 HELP_ID_MOVETOFOLDERS);
			}
			gtk_window_set_title(GTK_WINDOW(MoveToDialog),
					     _("mcen_ti_moveto_folders_title"));
			gtk_window_set_modal(GTK_WINDOW(MoveToDialog), TRUE);
			gtk_window_set_resizable(GTK_WINDOW(MoveToDialog), FALSE);
			/*Set application name to the window */
			g_object_set_data_full(G_OBJECT(MoveToDialog), WINDOW_NAME,
					       g_strdup(APP_NAME), (GDestroyNotify) g_free);

			if (msg_send_flag == TRUE) {
				gtk_infoprint((GtkWindow *) MoveToDialog,
					      _("mcen_ib_unable_to_cut_mess"));
			}

			gtk_widget_show(MoveToDialog);
			/* Hooking up the HildonFileSystemModel to window */
			EMAIL_UI_HOOKUP_OBJECT_NO_REF(MoveToDialog, folderView, "tree_view");
			EMAIL_UI_HOOKUP_OBJECT_NO_REF(MoveToDialog, listView, "list_view");
			EMAIL_UI_HOOKUP_OBJECT_NO_REF(MoveToDialog, fssystem, "file_system");
		}

		email_main_view->folder_manager->move_to_dialog = MoveToDialog;
		response = gtk_dialog_run(GTK_DIALOG(MoveToDialog));
		if (response == GTK_RESPONSE_OK) {
			success = ui_on_ok_move_to_folder_dialog(NULL, MoveToDialog, row_ref);
			refresh_dialog = FALSE;
		} else if (response == REFRESH_DIALOG_SIGNAL) {
			refresh_dialog = TRUE;
			gtk_widget_destroy(MoveToDialog);
			success = FALSE;
			ULOG_DEBUG_F("Got refresh event on the dialog\n");
		} else {
			user_confirmation = FALSE;
			success = TRUE;
		}
	}
	/*Destroy the Move-to-folder dialog */
	gtk_widget_destroy(MoveToDialog);
	email_main_view->folder_manager->move_to_dialog = NULL;
	g_free(device_name);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return user_confirmation;
}

/*end of file */
