/**
 * @file pop3_cb.c
 *
 * This file contains POP3 module related callback functions for UI.  
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

/* User includes */
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <gtk/gtk.h>
#include "prefs_account.h"
#include "al_callbacks.h"
#include "procmsg.h"
#include "procheader.h"
#include "folder.h"
#include "pop3_msgs.h"
#include "libosso.h"

/*UI header files*/
#include "ui_al_callbacks.h"
#include "editview.h"
#include "ui_interface.h"
#include "accountMgmt_callback.h"
#include "folder_mgmt.h"
#include "message_handler.h"

/* Global variables*/
extern MainViewInfo *email_main_view;	/* Declared in ui_interface.c */

Pop3Callback cb;
PrefsAccount account;

/**
   This function is called to inform the UI module about error occured 
   in pop3 Module 

   @param param_number pop3 module related error
   @return none 
*/

void pop3_param_value_missing_cb(ParamMissing param_number)
{
	ui_param_value_missing_cb(param_number);
	return;
}

/**
   This function is called to inform the UI module about status/error 
   occured  in pop3  Module 

   @param msg_number pop3 module status/error code 
   @param index contains the length of GArray arguments array
   @param arguments GArray of osso_rpc_t elements  which contains the 
    additional information
   @return none 
*/

void pop3_set_msg_cb(gint msg_number, gint index, GArray * arguments)
{
	gint i = 0;
	gint currentMsgNumber = -1, totalMsgNumber = -1, msg_remote_copy = -1;
	gchar *error_info = NULL, *error_info1 = NULL, *account_name = NULL, *msgid = NULL;

	ULOG_DEBUG_F(" pop3_set_msg_cb = %d Index = %d \n", msg_number, index);
	for (i = 0; i < index; i++) {
		switch (g_array_index(arguments, osso_rpc_t, i).type) {
		case DBUS_TYPE_INT32:
			ULOG_DEBUG_F("type %d INT  value = %d \n", DBUS_TYPE_INT32,
				     g_array_index(arguments, osso_rpc_t, i).value.i);
			break;
		case DBUS_TYPE_STRING:
			ULOG_DEBUG_F(" +++ type %d STR  value = %s \n", DBUS_TYPE_STRING,
				     g_array_index(arguments, osso_rpc_t, i).value.s);
			break;
		default:
			ULOG_DEBUG_F(" Type = %d \n", g_array_index(arguments, osso_rpc_t, i).type);
		}
	}

	switch (msg_number) {
	case POP3_MSG_START:
		{
			if (index != 2)
				ui_update_progressbar_text(CONNECTING, NULL);
			if (index >= 1) {
				g_free(email_main_view->account_manager->current_sendrec_account);
				email_main_view->account_manager->current_sendrec_account =
				    g_strdup(g_array_index(arguments, osso_rpc_t, 0).value.s);
				gtk_widget_set_sensitive(email_main_view->menu->email_send_receive,
							 FALSE);
				gtk_widget_set_sensitive(email_main_view->tools->send_receive,
							 FALSE);
				/*Set state_save_disabled flag */
				email_main_view->tags->state_save_disabled = TRUE;
			}
			if (index == 2) {
				msgid = g_strdup(g_array_index(arguments, osso_rpc_t, 1).value.s);
				ui_update_progressbar_text(CONNECTING, msgid);
				g_free(msgid);
				break;
			}
			break;
		}
	case POP3_MSG_COMPLETE:
		{
			ui_send_recv_complete_cb();
			email_main_view->tags->msg_download_in_progress = FALSE;

			/* Edit Account operation stopped Send/Receive */
			if (email_main_view->account_manager->operation_status == EDIT_OP) {
				ui_resume_edit_accout_oper();
			}

			/* Action after deletion of email account  stopped Send/Receive */
			else if (email_main_view->account_manager->operation_status == DELETE_OP) {
				ui_resume_delete_account_oper();
			} else {
				g_free(email_main_view->account_manager->current_sendrec_account);
				email_main_view->account_manager->current_sendrec_account = NULL;
			}
			/*Reset state_save_disabled flag */
			email_main_view->tags->state_save_disabled = FALSE;

			if (email_main_view->account_manager->configured_accounts != NULL) {
				gtk_widget_set_sensitive(email_main_view->menu->email_send_receive,
							 TRUE);
				gtk_widget_set_sensitive(email_main_view->tools->send_receive,
							 TRUE);
			}
			break;
		}
	case POP3_MSG_PROGRESS:
		{
			currentMsgNumber = g_array_index(arguments, osso_rpc_t, 0).value.i;
			totalMsgNumber = g_array_index(arguments, osso_rpc_t, 1).value.i;
			//Call UI callback function
			ui_update_progressbar_text(RECEIVING, NULL);
			ui_update_progressbar_numbers(currentMsgNumber, totalMsgNumber, RECEIVING);
			break;
		}
	case POP3_MSG_AUTH_ERROR:
		{
			error_info = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
			error_info1 = (gchar *) g_array_index(arguments, osso_rpc_t, 1).value.s;

			/*Check if the said account is already being edited/deleted.
			   If so do nothing. */
			if (email_main_view->account_manager->operation_status != NONE_OP) {
				if (error_info1 != NULL
				    && email_main_view->account_manager->current_sendrec_account !=
				    NULL
				    && strcmp(error_info1,
					      email_main_view->account_manager->
					      current_sendrec_account) == 0) {
					ULOG_DEBUG_F("Account is already being edited/deleted\n");
					break;
				}
			}

			ui_set_error_msg_cb(msg_number, error_info, error_info1);
			break;
		}
	case POP3_MSG_CONNECT_ERROR:
		{
			//Call UI callback function
			error_info = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
			ui_set_error_msg_cb(msg_number, error_info, NULL);
			break;
		}
	case POP3_MSG_CONNECT_SUCCESS:
		{
			if (index == 2) {
				msgid = g_strdup(g_array_index(arguments, osso_rpc_t, 1).value.s);
				ui_update_progressbar_text(RECEIVING, msgid);
				g_free(msgid);
			}
			break;
		}
	case POP3_MSG_PASSWD_FAIL:
		{
			account_name = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
			ui_on_mailbox_protected_dialog(account_name, FALSE, FALSE);
			break;
		}
	case POP3_PASSWD_WRONG:
		{
			account_name = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
			ui_on_mailbox_protected_dialog(account_name, FALSE, TRUE);
			break;
		}
	case POP3_MSG_DELETE_ERROR:
		{
			error_info1 = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
			ui_set_error_msg_cb(msg_number, error_info1, NULL);
			break;
		}
	case POP3_MSG_DOWNLOAD:
		{
			if (index == 2) {
				error_info1 =
				    (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
				msg_remote_copy = g_array_index(arguments, osso_rpc_t, 1).value.i;
				ui_update_msg_downloaded_cb(1, 1, msg_remote_copy, error_info1);
			}
			break;
		}
	default:
		{
			//Call UI callback function
			ui_set_msg_cb(msg_number);
			break;
		}
	}

	return;
}

/**
   This function is called to inform the UI module about percentage 
   of mail sent by pop3 Module 

   @param percent percentage of mail received through pop3 module. 
   @return none 
*/

void pop3_set_percentage_cb(gint percent)
{
	ui_update_progressbar_text(RECEIVING, NULL);
	ui_set_percentage_cb(percent);
	return;
}

/**
   This function is called to inform the UI module about the received 
   mail information to outbox.

   @param msgheader Message header information 
   @return none 
*/


void pop3_recv_single_cb(MsgHeader * msgheader)
{
	ui_recv_single_cb(msgheader);
	return;
}

/**
   This function is called to inform the UI module about completion status 
   of received mail throgh pop3 module.

   @param msgheader Message header information
   @return none
*/

void pop3_recv_complete_cb(void)
{
	ui_send_recv_complete_cb();
	email_main_view->tags->msg_download_in_progress = FALSE;

	/* Edit Account operation stopped Send/Receive */
	if (email_main_view->account_manager->operation_status == EDIT_OP) {
		ui_resume_edit_accout_oper();
	}

	/* Action after deletion of email account  stopped Send/Receive */
	else if (email_main_view->account_manager->operation_status == DELETE_OP) {
		ui_resume_delete_account_oper();
	} else {
		g_free(email_main_view->account_manager->current_sendrec_account);
		email_main_view->account_manager->current_sendrec_account = NULL;
	}
	/*Reset state_save_disabled flag */
	email_main_view->tags->state_save_disabled = FALSE;

	if (email_main_view->account_manager->configured_accounts != NULL) {
		gtk_widget_set_sensitive(email_main_view->menu->email_send_receive, TRUE);
		gtk_widget_set_sensitive(email_main_view->tools->send_receive, TRUE);
	}
	return;
}

/**
   This function is called to inform the UI module about the message 
   mime info of the received mail through call back. 

   @param mimeinfo Message mime information
   @param msgid  message id of the received mail 
   @param remote_copy  Flag indicating whether mail has remote copy on server
   @return none
*/

void pop3_recv_body_complete_cb(MimeInfo * mimeinfo, gchar * msgid, gboolean remote_copy)
{
	ui_recv_body_complete_cb(mimeinfo, msgid, remote_copy);
	return;
}
