/**
 * @file security.c
 * 
 * This file contains all  functions for checking the status of a 
 * encryted mail
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/

#include "security.h"
#include "ui_account_store.h"
#include "ui_state_mgmt.h"

GSList *passwd_list = NULL;	/*list for caching the passwords */

static GtkWidget *create_encryptmsg_info(IdStatus idstatus,
					 MimeInfo * mime_info, SecuritySetting * security_setting);
static SecureMailAction
security_status_for_sign(MimeInfo * mime_info, gchar * msgid, MsgFields * msgfields);

static void change_icons(GtkWidget * info_dialog, MimeInfo * mime_info)
{
	GtkWidget *image_sign = NULL;
	GtkWidget *image_encrypt = NULL;
	GtkWidget *text_view = NULL;
	SMimeType smime_type = SMIME_NONE;

	smime_type = mime_info->smime_type;

	image_sign = EMAIL_UI_LOOKUP_OBJECT(info_dialog, "sign_image");
	image_encrypt = EMAIL_UI_LOOKUP_OBJECT(info_dialog, "encrypt_image");
	text_view = EMAIL_UI_LOOKUP_OBJECT(info_dialog, "info_area");

	if (smime_type == SMIME_SIGN || smime_type == SMIME_SIGN_ENCRYPT) {
		if (mime_info->cert_state_recip != VALID) {
			gtk_image_set_from_icon_name((GtkImage *) image_sign,
						     "qgn_viewer_messagin_signed_untrusted",
						     GTK_ICON_SIZE_BUTTON);
		} else {
			gtk_image_set_from_icon_name((GtkImage *) image_sign,
						     "qgn_viewer_messagin_signed",
						     GTK_ICON_SIZE_BUTTON);
		}
	}
	if (smime_type == SMIME_ENCRYPT) {
		if (mime_info->cert_state_sender != VALID) {
			gtk_image_set_from_icon_name((GtkImage *) image_sign,
						     "qgn_viewer_messagin_encrypted_untrusted",
						     GTK_ICON_SIZE_BUTTON);
		} else {
			gtk_image_set_from_icon_name((GtkImage *) image_sign,
						     "qgn_viewer_messagin_encrypted",
						     GTK_ICON_SIZE_BUTTON);
		}
	}
	return;
}

static GtkWidget *change_dialog(GtkWidget * info_dialog, MimeInfo * mime_info,
				SecuritySetting * security_setting, MsgFields * fields)
{

	gtk_widget_destroy(info_dialog);

	set_viewer_security_icon(fields, mime_info);

	return (create_encryptmsg_info(-1, mime_info, security_setting));

}



/**
 Reads the message body if the user clicks on the read button.
 
 @param read is the button that registered the callback.c
 @param window is the Gtkwidget pointer to the dialog
 @return void.
*/
static void on_button_read_clicked(GtkWidget * read, SecuritySetting * security_settings)
{
	GtkWidget *window = NULL;
	SettingType type = ADVANCED_SETTING;

	window = gtk_widget_get_parent(read);
	gtk_dialog_response((GtkDialog *) window, GTK_RESPONSE_OK);
	ui_dbus_set_account_settings(type, security_settings->accSettings);
	ui_init_account_store();

	return;

}

/**
 opens the editTrust settings dialog or the properties window.
 of the certificate upon clicking the editTrust settings/properties.

 @param prop is the button that registered the callback.c
 @param security_settings is the pointer to the SecuritySetting structure.
 
 @return void.
 */
static void on_button_prop_clicked(GtkWidget * prop, SecuritySetting * security_settings)
{
	const gchar *button_text = NULL;
	GtkWidget *window = NULL;
	SettingType type = ADVANCED_SETTING;

	button_text = gtk_button_get_label((GtkButton *) prop);
	window = gtk_widget_get_parent(prop);

	ui_dbus_set_account_settings(type, security_settings->accSettings);
	ui_init_account_store();
	if (g_ascii_strcasecmp(button_text, _("mcen_bd_smime_secnote_properties")) == 0) {
		gtk_dialog_response((GtkDialog *) window, GTK_RESPONSE_HELP);
	} else if (g_ascii_strcasecmp(button_text, _("mcen_bd_smime_secnote_edittrust"))
		   == 0) {
		gtk_dialog_response((GtkDialog *) window, GTK_RESPONSE_YES);
		/*open the CM ui to edit the trust setings of the cert. */
	} else {
		gtk_dialog_response((GtkDialog *) window, GTK_RESPONSE_ACCEPT);
		/*open the CM ui to save the certificate */

	}
	return;
}

/**
 Function to set the appropriate flag if user checks not to show the dialog
 again.
 
 @param checkbutton is the Gtkwidget pointer to the checkbutton 
 @param security_settings is the pointer to the SecuritySetting structure.
 
 @return void.
*/
static void set_confirmation(GtkWidget * checkbutton, SecuritySetting * security_settings)
{
	gboolean togglestate = TRUE;
	PrefsAccount *accountSettings = NULL;

	CHECK_PARAM(security_settings == NULL, "Input param validation failed\n", return);
	CHECK_PARAM(security_settings->accSettings == NULL,
		    "Input param validation failed\n", return);

	accountSettings = security_settings->accSettings;

	togglestate = gtk_toggle_button_get_active((GtkToggleButton *) checkbutton);
	/*set the appropriate flag for not showing the dialog. */
	switch (security_settings->indicationStatus) {
	case INVALID_SENDER:
	case INVALID_RECIP:
		accountSettings->advanced_settings.display_validity = !togglestate;
		break;
	case NOT_TRUSTED:
		accountSettings->advanced_settings.display_trust = !togglestate;
		break;
	default:
		accountSettings->advanced_settings.display_sign_encrypt = !togglestate;
		break;
	}
	return;

}

/**
 This is a helper function to create sign & encryption icons pabased on the
 trust & validitity settings of the certificate.

 @param smime_type is the neum representing the smimetype.
 @param IdStatus is the emnum representing the idstatus
	 i.e-trusted/invalid_sender/invalid_recip etc.
 @param image_sign is the OUT param with the signed image set.
 @param image_encrypt is the OUT param with the encrypt image set.
 
 @return void.
 */
static void
prepare_sign_encrypt_icon(MimeInfo * mime_info, GtkWidget ** image_sign, GtkWidget ** image_encrypt)
{

	gchar *sign_stockid = "qgn_viewer_messagin_signed";
	gchar *encrypt_stockid = "qgn_viewer_messagin_encrypted";
#if SMIME
	SMimeType smime_type = mime_info->smime_type;

	ULOG_DEBUG_L("starts here");
	if (smime_type == SMIME_SIGN || smime_type == SMIME_SIGN_ENCRYPT) {
		if (mime_info->trust == FALSE || mime_info->cert_state_recip != VALID) {
			sign_stockid = "qgn_viewer_messagin_signed_untrusted";
		}
	}
	if (smime_type == SMIME_ENCRYPT || smime_type == SMIME_SIGN_ENCRYPT) {
		if (mime_info->cert_state_sender != VALID) {
			encrypt_stockid = "qgn_viewer_messagin_encrypted_untrusted";
		}
	}
#endif
	*image_sign = gtk_image_new_from_icon_name(sign_stockid, GTK_ICON_SIZE_BUTTON);
	*image_encrypt = gtk_image_new_from_icon_name(encrypt_stockid, GTK_ICON_SIZE_BUTTON);
	ULOG_DEBUG_L("ends here");
	return;
}

/**
 This function sets the confirmation content required based on the smime tye & the certificate state.
 
 @param idstatus is the enum rep the cerificate status.
 @param smime_type is the enum representing the smime type.
 @param textview is the pointer to the GtkWidget.

 @retrun void.
*/
static void set_confirmation_content(IdStatus idstatus, SMimeType smime_type, GtkWidget * textview)
{
	switch (idstatus) {
	case NOT_TRUSTED:
		if (smime_type == SMIME_SIGN) {
			gtk_text_buffer_set_text(gtk_text_view_get_buffer
						 (GTK_TEXT_VIEW(textview)),
						 _("mcen_ia_smime_secnote_2b"), -1);
		}
		if (smime_type == SMIME_SIGN_ENCRYPT) {
			gtk_text_buffer_set_text(gtk_text_view_get_buffer
						 (GTK_TEXT_VIEW(textview)),
						 _("mcen_ia_smime_secnote_2a"), -1);
		}
		break;
	case INVALID_SENDER:
	case INVALID_RECIP:
		switch (smime_type) {
		case SMIME_SIGN:
			gtk_text_buffer_set_text(gtk_text_view_get_buffer
						 (GTK_TEXT_VIEW(textview)),
						 _("mcen_ia_smime_secnote_3b"), -1);
			break;
		case SMIME_ENCRYPT:
			break;
		case SMIME_SIGN_ENCRYPT:
			if (idstatus == INVALID_SENDER) {
				gtk_text_buffer_set_text(gtk_text_view_get_buffer
							 (GTK_TEXT_VIEW(textview)),
							 _("mcen_ia_smime_secnote_3a"), -1);
			}
			break;
		default:
			break;
		}
		break;
	default:
		if (smime_type == SMIME_SIGN) {
			gtk_text_buffer_set_text(gtk_text_view_get_buffer
						 (GTK_TEXT_VIEW(textview)),
						 _("mcen_ia_smime_secnote_1c"), -1);
		}
		if (smime_type == SMIME_ENCRYPT) {
			gtk_text_buffer_set_text(gtk_text_view_get_buffer
						 (GTK_TEXT_VIEW(textview)),
						 _("mcen_ia_smime_secnote_1b"), -1);
		}
		if (smime_type == SMIME_SIGN_ENCRYPT) {
			gtk_text_buffer_set_text(gtk_text_view_get_buffer
						 (GTK_TEXT_VIEW(textview)),
						 _("mcen_ia_smime_secnote_1a"), -1);
		}
		break;
	}
	return;
}

/**
 This function is used to set the title of the information dialog.
 
 @param smime_type is the enum representing the smime_type.
 @param idstatus is the enum representing the idstatus of the certificate.
 @param dialog_security is the pointer to the GtkWidget for the dialog.
 @param image_sign is the GtkWidget representing the signed icon.
 @param image_encrypt is the GtkWidget representing the encryption icon.

 @return void.
*/
static void
set_title(SMimeType smime_type, IdStatus idstatus,
	  GtkWidget * dialog_security, GtkWidget * image_sign, GtkWidget * image_encrypt)
{
	gchar *title = NULL;
	switch (smime_type) {
	case SMIME_SIGN:
		title = _("mail_ti_signed");
		if (idstatus == NOT_TRUSTED) {
			title = _("mail_ti_signed_untrusted");
		}
		gtk_widget_show(image_sign);
		gtk_widget_hide(image_encrypt);
		break;
	case SMIME_ENCRYPT:
		title = _("mail_ti_encrypted");
		gtk_widget_hide(image_sign);
		gtk_widget_show(image_encrypt);
		break;
	case SMIME_SIGN_ENCRYPT:
		title = _("mail_ti_signed_encrypted");
		if (idstatus == NOT_TRUSTED) {
			title = _("mail_ti_signed_encrypted_untrusted");
		}
		gtk_widget_show(image_sign);
		gtk_widget_show(image_encrypt);
		break;
	default:
		break;
	}
	gtk_window_set_title(GTK_WINDOW(dialog_security), title);
	gtk_window_set_default_size(GTK_WINDOW(dialog_security),
				    ui_predict_dialog_width_based_on_title(dialog_security, title),
				    -1);
	return;
}

/**
 This function is used to create the dialog & show the appropriate message if
 the mail has security option set.

 @param idstatus is the enum representing the status of the certificate-
 	 valid/trusted/invalid/not trusted.
 @param mime_info is the pointer to the MimeInfo structure.
 @param accountSettings is the prefs accouint structure pointer.
 @param cert_path is the path os the certificate if its new.

 @return a GtkWidget  pointer to the dialog.
*/
static GtkWidget *create_encryptmsg_info(IdStatus idstatus, MimeInfo * mime_info,
					 SecuritySetting * security_setting)
{

	GtkWidget *dialog_security = NULL;
	GtkWidget *dialog_vbox = NULL;
	GtkWidget *dialog_action_area1 = NULL;
	GtkWidget *table = NULL;
	GtkWidget *image_sign = NULL;
	GtkWidget *image_encrypt = NULL;
	GtkWidget *textview = NULL;
	GtkWidget *button_read = NULL;
	GtkWidget *button_prop = NULL;
	GtkWidget *hbox_confirm = NULL;
	GtkWidget *label_text = NULL;
	GtkWidget *checkbutton2 = NULL;
	PrefsAccount *accountSettings = NULL;
	SMimeType smime_type = SMIME_NONE;

	accountSettings = security_setting->accSettings;
	smime_type = mime_info->smime_type;


	dialog_security = gtk_dialog_new();
	gtk_window_set_transient_for(GTK_WINDOW(dialog_security), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(dialog_security), TRUE);
	dialog_vbox = GTK_DIALOG(dialog_security)->vbox;
	gtk_dialog_set_has_separator((GtkDialog *) dialog_security, FALSE);

	table = gtk_table_new(3, 2, FALSE);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), table, TRUE, TRUE, 0);
	gtk_table_set_col_spacing((GtkTable *) table, 0, 20);
	gtk_table_set_col_spacing((GtkTable *) table, 1, 20);
	prepare_sign_encrypt_icon(mime_info, &image_sign, &image_encrypt);

	gtk_table_attach((GtkTable *) table, image_sign, 0, 1, 0, 1,
			 (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_table_attach((GtkTable *) table, image_encrypt, 0, 1, 1, 2,
			 (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);

	set_title(smime_type, idstatus, dialog_security, image_sign, image_encrypt);

	textview = gtk_text_view_new();
	gtk_text_view_set_editable((GtkTextView *) textview, FALSE);
	gtk_text_view_set_cursor_visible((GtkTextView *) textview, FALSE);
	gtk_text_view_set_wrap_mode((GtkTextView *) textview, GTK_WRAP_WORD_CHAR);
	gtk_widget_show(textview);
	gtk_table_attach_defaults((GtkTable *) table, textview, 1, 2, 0, 2);
	gtk_widget_show(table);

	dialog_action_area1 = GTK_DIALOG(dialog_security)->action_area;
	gtk_widget_show(dialog_action_area1);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1), GTK_BUTTONBOX_END);

	button_read = gtk_button_new_with_mnemonic(_("mcen_bd_smime_secnote_read"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_security), button_read, GTK_RESPONSE_OK);
	gtk_widget_show(button_read);


	switch (idstatus) {
	case INVALID_RECIP:
	case INVALID_SENDER:
		button_prop = gtk_button_new_with_mnemonic(_("mcen_bd_smime_secnote_properties"));
		gtk_widget_show(button_prop);
		gtk_dialog_add_action_widget(GTK_DIALOG(dialog_security), button_prop, 0);
		break;

	case NOT_TRUSTED:
		button_prop = gtk_button_new_with_mnemonic(_("mcen_bd_smime_secnote_edittrust"));
		gtk_widget_show(button_prop);
		gtk_dialog_add_action_widget(GTK_DIALOG(dialog_security), button_prop,
					     GTK_RESPONSE_YES);
		break;
	default:
		break;
	}

	hbox_confirm = gtk_hbox_new(FALSE, 0);
	gtk_widget_show(hbox_confirm);
	gtk_table_attach((GtkTable *) table, hbox_confirm, 1, 2, 2, 3,
			 (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);

	label_text = gtk_label_new(_("mcen_va_smime_secnote_read"));
	gtk_widget_show(label_text);
	gtk_box_pack_start(GTK_BOX(hbox_confirm), label_text, TRUE, TRUE, 0);

	checkbutton2 = gtk_check_button_new_with_mnemonic("");
	gtk_widget_show(checkbutton2);
	gtk_box_pack_start(GTK_BOX(hbox_confirm), checkbutton2, TRUE, TRUE, 0);

	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_security, dialog_security, "security_dialog");
	EMAIL_UI_HOOKUP_OBJECT(dialog_security, checkbutton2, "checkbox");
	EMAIL_UI_HOOKUP_OBJECT(dialog_security, image_sign, "sign_image");
	EMAIL_UI_HOOKUP_OBJECT(dialog_security, image_encrypt, "encrypt_image");
	EMAIL_UI_HOOKUP_OBJECT(dialog_security, textview, "info_area");

	g_signal_connect(G_OBJECT(checkbutton2), "toggled",
			 G_CALLBACK(set_confirmation), security_setting);
	g_signal_connect((gpointer) button_read, "clicked",
			 G_CALLBACK(on_button_read_clicked), security_setting);
	g_signal_connect((gpointer) button_prop, "clicked",
			 G_CALLBACK(on_button_prop_clicked), security_setting);
	if (ossohelp_exists(HELP_ID_SECURITY_DIALOG) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) dialog_security);
		g_signal_connect(G_OBJECT(dialog_security), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_SECURITY_DIALOG);
	}

	set_confirmation_content(idstatus, smime_type, textview);
	return dialog_security;
}

/**
 This function is used to get the account settings for a given account.

 @param msgid id the msgid of the mail.
 @return the pointer to the prefs Account structure.
 */
static PrefsAccount *get_account_settings(const gchar * msgid)
{
	gchar *temp_acc = NULL;
	gchar *current_acc = NULL;
	PrefsAccount *accountSettings = NULL;

	ULOG_DEBUG_L("starts here");
	if (msgid == NULL) {
		return NULL;
	}
	temp_acc = strchr(msgid, MSGID_SEPARATOR);
	if (temp_acc != NULL) {
		current_acc = g_strndup(msgid, strlen(msgid) - strlen(temp_acc));
		if (current_acc != NULL
		    && g_ascii_strncasecmp(current_acc, DEFAULT,
					   MAX_STR_LEN(current_acc, DEFAULT)) != 0) {
			accountSettings = ui_get_account_detail(current_acc);

		}
		g_free(current_acc);
	}
	ULOG_DEBUG_L("ends here");
	return accountSettings;
}

/**
 This is a helper function to check the the validitiy & trustsetting for a
 smime sign/sign_encrypt message.

 @param display_sign_encrypt is the gboolean representing whether a display dialog 
	 has to be shown or not.
 @param mime_info is the MimeInfo structure pointer.
 @return TRUE-if both valid & trusted.
         FALSE-else.
*/
static gboolean check_for_trust_valid(gboolean display_sign_encrypt, MimeInfo * mime_info)
{
	gboolean valid_trusted = FALSE;

	if (mime_info == NULL) {
		return valid_trusted;
	}
	if (display_sign_encrypt == FALSE) {
		return valid_trusted;
	}

	switch (mime_info->smime_type) {
	case SMIME_SIGN:
		if (mime_info->cert_state_recip == VALID && mime_info->trust == TRUE) {
			valid_trusted = TRUE;
		}
		break;
	case SMIME_ENCRYPT:
		if (mime_info->cert_state_sender == VALID && mime_info->trust == TRUE) {
			valid_trusted = TRUE;
		}
		break;
	default:
		valid_trusted = TRUE;
		break;
	}
	return valid_trusted;
}

/**
 This function is used to check sender certificate state.
 @param mime_info is the MimeInfo structure pointer,
        containing the information about the certificate.
 @param security_setting is the SecuritySetting structure pointer.
 @ param display_validity is the falg representing whether warning status has
   to be shown or not.
 @ return SecureMailAction a enum whether to read the mail ,
	   show edit trust settings.
*/
static SecureMailAction
check_sender_cert_state(MimeInfo * mime_info,
			SecuritySetting * security_setting, gboolean display_validity)
{
	SecureMailAction mailAction = READ_MAIL;
	GtkWidget *info_dialog = NULL;
	gint cert_id = 0;
	gint response = GTK_RESPONSE_OK;

	cert_id = mime_info->recip_cert_id;

	if (mime_info->cert_state_sender != VALID && mime_info->smime_type == SMIME_SIGN_ENCRYPT) {
		mailAction = READ_MAIL;
		return mailAction;
		while (info_dialog != NULL) {
			response = gtk_dialog_run((GtkDialog *) info_dialog);
			if (response == GTK_RESPONSE_OK) {
				mailAction = READ_MAIL;
				gtk_widget_destroy(info_dialog);
				return mailAction;
			} else {
				mailAction = SHOW_PROPERTIES;
				/*showing the properties detail dialog */
				certmanui_certificate_details_dialog(info_dialog, TRUE, cert_id);
			}
		}
	}
	return mailAction;
}

/**
 This is a helper function to launch the
	 save certificate dialog if its a new signer certificate.
 
 @param mime_info is the MimeInfo structure pointer,
        containing the information about the certificate.

 @return gboolean-TRUE-successfully imported the certificate.
                  FALSE-else.
*/
static gboolean launch_save_cert_dialog(MimeInfo * mime_info)
{
	gboolean import_successful = TRUE;
	guint cert_id = 0;
	gchar *cert_path = NULL;
	GSList *certid_list = NULL;
	GSList *p12_list = NULL;
	FILE *fp = NULL;

	CHECK_PARAM(mime_info->cert_path == NULL, "no certificte path specified :(", return FALSE);

	fp = fopen(mime_info->cert_path, "r+");
	CHECK_PARAM(fp == NULL, "unable to open file", return FALSE);
	while ((cert_path = get_cert_path(fp)) != NULL) {
		import_successful = certmanui_import_file(app, cert_path, &cert_id, &p12_list);
		if (cert_id == 0 && p12_list != NULL) {
			/*g_new is used for filling the data in the p12_list,so.. */
			cert_id = *((gint *) g_slist_nth_data(p12_list, 0));
		}
		g_slist_foreach(p12_list, (GFunc) g_free, NULL);
		g_slist_free(p12_list);
		unlink(cert_path);
		g_free(cert_path);
		if (import_successful == TRUE) {
			certid_list = g_slist_append(certid_list, GINT_TO_POINTER(cert_id));
		} else {
			break;
		}
	}
	fclose(fp);
	if (import_successful == TRUE) {
		mime_info->saving = FALSE;
		mime_info->trust = get_changed_trust_setting(certid_list);
		mime_info->cert_state_recip = get_changed_cert_state(certid_list);
		mime_info->sender_cert_id = GPOINTER_TO_INT(g_slist_nth_data(certid_list, 0));
	}

	/*The node contents are integers so need not free */
	g_slist_free(certid_list);

	return import_successful;

}

/*Check the whether the digitialid is valid 
 
  @param mime_info is the MimeInfo structure pointer.
  @param accountSettings is the PrefsAccount structure pointer for the current account.
  @param display_validity is the gboolean rep whether the invalid certificate 
  dialog has to be shown or not.
  
  @return the response of the dialog shown i.e READ_MAIl or open properties dialog of the
  CM.
*/
static SecureMailAction
check_for_cert_validity(MimeInfo * mime_info,
			SecuritySetting * security_setting,
			gboolean display_validity, MsgFields * msgfields)
{
	SecureMailAction mailAction = READ_MAIL;
	gint response = GTK_RESPONSE_OK;
	GtkWidget *info_dialog = NULL;
	guint cert_id = 0;
	PrefsAccount *accountSettings = NULL;
	accountSettings = security_setting->accSettings;


	cert_id = mime_info->sender_cert_id;
	if (mime_info->cert_state_recip != VALID) {
		if (display_validity == TRUE) {
			security_setting->indicationStatus = INVALID_SENDER;
			info_dialog =
			    create_encryptmsg_info(INVALID_SENDER, mime_info, security_setting);
		} else {
			mailAction = READ_MAIL;
		}
	}
	while (info_dialog != NULL) {
		response = gtk_dialog_run((GtkDialog *) info_dialog);
		if (response == GTK_RESPONSE_OK) {
			mailAction = READ_MAIL;
			gtk_widget_destroy(info_dialog);
			info_dialog = NULL;
			break;
		}
		/*action to be taken if edittrust settig is clicked */
		else if (response != GTK_RESPONSE_ACCEPT) {
			mailAction = SHOW_PROPERTIES;
			/*showing the properties detail dialog */
			certmanui_certificate_details_dialog(info_dialog, TRUE,
							     mime_info->sender_cert_id);
			continue;
		}

	}
	if (mailAction == READ_MAIL) {
		mailAction = check_sender_cert_state(mime_info, security_setting, display_validity);
	}
	return mailAction;
}

/*Check the security satus of a sign/sign_encrypt mail
 
 @param mime_info is the pointer to the MimeInfo structure pointer.
 @param msgid is the gchar pointer to the msgid of the viewer.
 @return the SecureMailAction enum representing whether to
          READ_MAIl/SHOW_PROPERTIES/EDIT_TRUST_SETTING
 */
static SecureMailAction
security_status_for_sign(MimeInfo * mime_info, gchar * msgid, MsgFields * msgfields)
{
	GtkWidget *info_dialog = NULL;
	gboolean trusted = TRUE;
	gboolean valid_trusted = FALSE;
	gboolean display_sign_encrypt = TRUE;
	gboolean display_validity = TRUE;
	gboolean display_trust = TRUE;
	gboolean import_successful = TRUE;

	gint cert_valid_sender = VALID;
	gint cert_valid_recip = VALID;
	gint response = GTK_RESPONSE_OK;
	SecuritySetting *security_setting = NULL;
	SecureMailAction mailAction = READ_MAIL;
	PrefsAccount *accountSettings = NULL;
	GSList *certid_list = NULL;

	cert_valid_sender = mime_info->cert_state_sender;
	cert_valid_recip = mime_info->cert_state_recip;

	accountSettings = get_account_settings(msgid);
	if (accountSettings != NULL) {
		display_sign_encrypt = accountSettings->advanced_settings.display_sign_encrypt;
		display_validity = accountSettings->advanced_settings.display_validity;
		display_trust = accountSettings->advanced_settings.display_trust;
	}

   /**Check whether the signer certification is available with the CM.
    * if signer verification status is false & its a clear sign message still the
    * message is shown  */
	if (mime_info->signature_verify == FALSE) {
		/*if (mime_info->smime_type == SMIME_CLEAR_SIGN) */
		{
			get_user_confirmation(_("mcen_in_smime_signer_not_available"),
					      NULL, NULL, INFO_BANNER);
			mailAction = READ_MAIL;
			ui_free_prefs_account_structure(accountSettings);
			return mailAction;
		}
	}
/*If its a new signer certificate we lauch the save dialog*/
	if (mime_info->saving == TRUE) {
		import_successful = launch_save_cert_dialog(mime_info);
	}
	security_setting = g_new0(SecuritySetting, 1);
	security_setting->accSettings = accountSettings;

/*if saving was unsucessful we go ahead and read the mesage*/
	if (import_successful == FALSE) {
		info_dialog = create_encryptmsg_info(ENCRYPT, mime_info, security_setting);
		response = gtk_dialog_run((GtkDialog *) info_dialog);
		ui_free_prefs_account_structure(accountSettings);
		g_free(security_setting);
		gtk_widget_destroy(info_dialog);
		return READ_MAIL;
	}

	/*Check the whether the digitialid is valid */
	mailAction =
	    check_for_cert_validity(mime_info, security_setting, display_validity, msgfields);

	trusted = mime_info->trust;
	/*Check the whether the digitial id is is trusted */
	if (trusted == FALSE && display_trust == TRUE) {
		security_setting->indicationStatus = NOT_TRUSTED;
		info_dialog = create_encryptmsg_info(NOT_TRUSTED, mime_info, security_setting);
	}
	while (info_dialog != NULL) {
		response = gtk_dialog_run((GtkDialog *) info_dialog);
		if (response == GTK_RESPONSE_OK) {
			mailAction = READ_MAIL;
			gtk_widget_destroy(info_dialog);
			ui_free_prefs_account_structure(accountSettings);
			g_free(security_setting);
			return mailAction;
		}
		/*if edit trust settings is clicked */
		else if (response == GTK_RESPONSE_YES) {
			mailAction = EDIT_TRUST_SETTINGS;
			/*launch the edit trust setting dialog */
			certmanui_trust_settings(info_dialog, mime_info->sender_cert_id);

			/*get the trust setting after editiing */
			certid_list =
			    g_slist_append(certid_list, GINT_TO_POINTER(mime_info->sender_cert_id));
			trusted = get_changed_trust_setting(certid_list);
			g_slist_free(certid_list);
			certid_list = NULL;

			mime_info->trust = trusted;
			if (trusted == TRUE && mime_info->cert_state_recip == VALID) {
				info_dialog =
				    change_dialog(info_dialog, mime_info, security_setting,
						  msgfields);
			} else if (trusted == TRUE) {
				change_icons(info_dialog, mime_info);
			}
		}
	}
	if (trusted == FALSE && display_trust == FALSE) {
		mailAction = READ_MAIL;
		ui_free_prefs_account_structure(accountSettings);
		g_free(security_setting);
		return mailAction;
	}
	/*Check for the validity & trust settings of the cerificate.
	   display the dialog is everyth is fine */
	valid_trusted = check_for_trust_valid(display_sign_encrypt, mime_info);
	if (valid_trusted == TRUE) {
		security_setting->indicationStatus = ENCRYPT;
		info_dialog = create_encryptmsg_info(ENCRYPT, mime_info, security_setting);
	}
	if (info_dialog != NULL) {
		response = gtk_dialog_run((GtkDialog *) info_dialog);
		if (response == GTK_RESPONSE_OK) {
			mailAction = READ_MAIL;
			gtk_widget_destroy(info_dialog);
		}
	} else {
		mailAction = READ_MAIL;
	}
	ui_free_prefs_account_structure(accountSettings);
	g_free(security_setting);
	return mailAction;
}

/*Check the security status of a encrypt mail
 
 @param mime_info is the pointer to the MimeInfo structure pointer.
 @param msgid is the gchar pointer to the msgid of the viewer.
 @return the SecureMailAction enum representing whether to
          READ_MAIl/SHOW_PROPERTIES.
 */
static SecureMailAction security_status_for_encrypt(MimeInfo * mime_info, const gchar * msgid)
{
	gboolean display_sign_encrypt = TRUE;
	gboolean display_validity = TRUE;
	gboolean display_trust = TRUE;

	gboolean trusted = FALSE;
	gint cert_valid_sender = VALID;
	gint cert_valid_recip = VALID;
	gint response = GTK_RESPONSE_OK;
	SecureMailAction mailAction = READ_MAIL;
	PrefsAccount *accountSettings = NULL;
	GtkWidget *info_dialog = NULL;
	SecuritySetting *security_setting = NULL;

	accountSettings = get_account_settings(msgid);
	if (accountSettings != NULL) {
		display_sign_encrypt = accountSettings->advanced_settings.display_sign_encrypt;
		display_validity = accountSettings->advanced_settings.display_validity;
		display_trust = accountSettings->advanced_settings.display_trust;
	}

	trusted = mime_info->trust;
	cert_valid_sender = mime_info->cert_state_sender;
	cert_valid_recip = mime_info->cert_state_recip;
	security_setting = g_new0(SecuritySetting, 1);
	security_setting->accSettings = accountSettings;

	if (cert_valid_sender != VALID) {
		mailAction = READ_MAIL;
	} else if (display_sign_encrypt == TRUE) {
		security_setting->indicationStatus = ENCRYPT;
		info_dialog = create_encryptmsg_info(ENCRYPT, mime_info, security_setting);
	}
	while (info_dialog != NULL) {
		response = gtk_dialog_run((GtkDialog *) info_dialog);
		if (response == GTK_RESPONSE_OK) {
			mailAction = READ_MAIL;
			gtk_widget_destroy(info_dialog);
			info_dialog = NULL;
		} else {
			mailAction = SHOW_PROPERTIES;
			/*showing the properties detail dialog */
			certmanui_certificate_details_dialog(info_dialog, TRUE,
							     mime_info->recip_cert_id);
		}

	}
	ui_free_prefs_account_structure(accountSettings);
	g_free(security_setting);
	return mailAction;
}


/* Based on the information about the security the appropriate 
  message is poped

 @param msgheader is the HeaderInfo structure pointer.
 @return SecureMailAction is a enum rep whether the mail
	 can be read or some other action such as opening
	 the CM ui for editing trust settings. 
 */
SecureMailAction check_security_status(gchar * msgid, MimeInfo * mime_info, MsgFields * msgfields)
{
	SecureMailAction mailAction = READ_MAIL;

	if (mime_info->smime_type == SMIME_SIGN || mime_info->smime_type == SMIME_SIGN_ENCRYPT) {
      /**Checks the validity & trust settings of the certificates*/
		mailAction = security_status_for_sign(mime_info, msgid, msgfields);
	}
	if (mime_info->smime_type == SMIME_ENCRYPT) {
		/*Checks the validy of the cetificate */
		mailAction = security_status_for_encrypt(mime_info, msgid);
	}
	return mailAction;
}

#if SMIME
/**
 Prepares a GSList of recipient mail ids
 
 @param emailAddress is the gcahr representing a emailid string.
 @param emailid_list is the OUT param containing the email address.
 
 @return void.
*/
static void prepare_mailid_list(const gchar * emailAddress, GSList ** emailid_list)
{
	gchar **mailids = NULL;
	gchar *emailid = NULL;
	gint i = 0;
	RecipientInfo *recipientInfo = NULL;
	if (emailAddress != NULL) {
		mailids = g_strsplit(emailAddress, ADDR_SEPERATOR_CHAR, -1);
		while (mailids[i] != NULL) {
			recipientInfo = g_new0(RecipientInfo, 1);
			emailid = g_utf8_strdown(mailids[i], -1);
			recipientInfo->email_id = ui_parse_from_address(g_strstrip(emailid));
			g_free(emailid);
			emailid = NULL;
			*emailid_list = g_slist_append(*emailid_list, recipientInfo);
			i++;
		}
		g_strfreev(mailids);
	}
}

/*
 This function gets the list of mailids whose certificate are not valid
 
 @param no_cert_id is the GSList of mail ids who have no certificate.
 	This param is not freed here.
 @return a string containing emailids separated with "\n".
*/
static gchar *create_mailid_strings(GSList * no_cert_id)
{
	GString *emailid = NULL;
	gchar *emailidstr = NULL;
	emailid = g_string_new(NULL);
	if (emailid == NULL) {
		return NULL;
	}

	while (no_cert_id != NULL) {
		g_string_append(emailid, (gchar *) no_cert_id->data);
		g_string_append(emailid, CARRIAGE_RETURN);
		no_cert_id = no_cert_id->next;
	}
	emailidstr = g_string_free(emailid, FALSE);
	return emailidstr;
}

/**
 This function frees the Recipient info structure

 @param recipientInfo is the pointer to the RecipientInfo structure.
 @param data user data NULL.

 @return void.
*/

static void free_recipientinfo(RecipientInfo * recipientInfo, gpointer data)
{
	g_free(recipientInfo->email_id);
	g_free(recipientInfo);
	return;
}

/**
 This function is used to fill the certids of the email recipients.
 
 @param emailid_list is the GSList containing the recipient info structures.
   This param is not freed here.
 @param msginfo is the MsgInfo structure.
   This param is not freed here.
   
 @return void.
*/
static void fill_cert_idlist(GSList * emailid_list, MsgInfo * msginfo)
{
	RecipientInfo *recipientInfo = NULL;
	while (emailid_list != NULL) {
		recipientInfo = emailid_list->data;
		if (recipientInfo->cert_id != 0) {
			msginfo->recip_certs_uid =
			    g_slist_append(msginfo->recip_certs_uid,
					   GINT_TO_POINTER(recipientInfo->cert_id));
		}
		emailid_list = emailid_list->next;
	}
}

/** creates the information dialog
 
  @param infonote is the list of mail id string
  @param cer_status is a enum representing whether NO_CERT/INVALID
  	certificate status.
	
  @return GtkWidget pointer to the confirmation dialog.
*/
static GtkWidget *create_user_confirmation(gchar * infonote, gint cert_status)
{
	GtkWidget *smimeinfo_dialog = NULL;
	GtkWidget *dialog_vbox = NULL;
	GtkWidget *vbox = NULL;
	GtkWidget *label_info = NULL;
	GtkWidget *scrolledwindow = NULL;
	GtkWidget *textview_ids = NULL;
	GtkWidget *dialog_action_area = NULL;
	GtkWidget *send = NULL;
	GtkWidget *cancel = NULL;

	smimeinfo_dialog = gtk_dialog_new();
	gtk_dialog_set_has_separator(GTK_DIALOG(smimeinfo_dialog), FALSE);
	gtk_window_set_title(GTK_WINDOW(smimeinfo_dialog), _("mcen_ti_smime_encryption_note"));
	gtk_window_set_transient_for(GTK_WINDOW(smimeinfo_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(smimeinfo_dialog), TRUE);

	dialog_vbox = GTK_DIALOG(smimeinfo_dialog)->vbox;
	gtk_widget_show(dialog_vbox);

	vbox = gtk_vbox_new(FALSE, ZERO_SPACING);
	gtk_widget_show(vbox);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), vbox, TRUE, TRUE, ZERO_PADDING);


	label_info = gtk_label_new(NULL);
	gtk_widget_show(label_info);
	gtk_box_pack_start(GTK_BOX(vbox), label_info, FALSE, FALSE, ZERO_PADDING);

	if (cert_status == NO_CERT) {
		gtk_label_set_text((GtkLabel *) label_info, _("mcen_ia_smime_keys_not_found"));
	} else {
		gtk_label_set_text((GtkLabel *) label_info, _("mcen_ia_smime_keys_not_found"));
	}

	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow);
	gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, ZERO_PADDING);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type((GtkScrolledWindow *)
					    scrolledwindow, GTK_SHADOW_ETCHED_IN);

	textview_ids = gtk_text_view_new();
	gtk_text_view_set_editable((GtkTextView *) textview_ids, FALSE);
	gtk_text_view_set_cursor_visible((GtkTextView *) textview_ids, FALSE);
	gtk_text_view_set_wrap_mode((GtkTextView *) textview_ids, GTK_WRAP_WORD_CHAR);
	gtk_widget_show(textview_ids);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), textview_ids);
	gtk_text_buffer_set_text(gtk_text_view_get_buffer
				 (GTK_TEXT_VIEW(textview_ids)), _(infonote), -1);

	dialog_action_area = GTK_DIALOG(smimeinfo_dialog)->action_area;
	gtk_widget_show(dialog_action_area);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area), GTK_BUTTONBOX_SPREAD);

	send = gtk_button_new_with_mnemonic(_("mcen_bd_smime_send"));
	gtk_widget_show(send);
	gtk_dialog_add_action_widget(GTK_DIALOG(smimeinfo_dialog), send, GTK_RESPONSE_OK);
	GTK_WIDGET_SET_FLAGS(send, GTK_CAN_DEFAULT);

	cancel = gtk_button_new_with_mnemonic(_("mcen_bd_smime_cancel"));
	gtk_widget_show(cancel);
	gtk_dialog_add_action_widget(GTK_DIALOG(smimeinfo_dialog), cancel, GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS(cancel, GTK_CAN_DEFAULT);

	if (ossohelp_exists(HELP_ID_SECURITY_DIALOG) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) smimeinfo_dialog);
		g_signal_connect(G_OBJECT(smimeinfo_dialog), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_SECURITY_DIALOG);
	}
	g_signal_connect(G_OBJECT(smimeinfo_dialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancel);
	ULOG_DEBUG_L("ends here");
	return smimeinfo_dialog;
}

/**
 This function checks the availability of the certificates for the list of mail ids.
 
 @param mailid_list is the GSlist containing the mailid_list.
 
 @return TRUE-if all the recipients have certificates.
         FALSE-else.
*/
static gboolean check_cert_availability(GSList * mailid_list)
{
	GSList *no_cert_id = NULL;
	RecipientInfo *recipientInfo = NULL;
	gchar *emailid = NULL;
	gboolean response = TRUE;
	GtkWidget *dialog = NULL;
	gint num_certificates = 0;
	gint num_no_certids = 0;

	ULOG_DEBUG_L("starts here");
	num_certificates = g_slist_length(mailid_list);
	while (mailid_list != NULL) {
		recipientInfo = mailid_list->data;
		if (recipientInfo->cert_id == 0) {
			no_cert_id = g_slist_append(no_cert_id, g_strdup(recipientInfo->email_id));
		}
		mailid_list = mailid_list->next;
	}

	num_no_certids = g_slist_length(no_cert_id);
	if (num_no_certids == num_certificates) {
		g_slist_foreach(no_cert_id, (GFunc) g_free, NULL);
		g_slist_free(no_cert_id);
		get_user_confirmation(_("mcen_in_smime_no_pub_cert"), NULL, NULL, INFO_BANNER);
		g_slist_foreach(mailid_list, (GFunc) free_recipientinfo, NULL);
		g_slist_free(mailid_list);
		ULOG_DEBUG_L("ends here");
		return FALSE;
	}

	if (no_cert_id != NULL) {
		emailid = create_mailid_strings(no_cert_id);
		g_slist_foreach(no_cert_id, (GFunc) g_free, NULL);
		g_slist_free(no_cert_id);
		/*UI to pop an error message with the list of email IDs.
		   check for validity of recipient certificate else pop up a warning message */
		dialog = create_user_confirmation(emailid, NO_CERT);
		response = gtk_dialog_run((GtkDialog *) dialog);
		if (response == GTK_RESPONSE_CANCEL) {
			gtk_widget_destroy(dialog);
			g_free(emailid);
			g_slist_foreach(mailid_list, (GFunc) free_recipientinfo, NULL);
			g_slist_free(mailid_list);
			ULOG_DEBUG_L("ends here");
			return FALSE;
		} else if (response == GTK_RESPONSE_OK) {
			gtk_widget_destroy(dialog);
			return TRUE;
		}
	}
	ULOG_DEBUG_L("ends here");
	return TRUE;
}

/**
 This function checks the validity of the certificates for the list of mail ids.
 
 @param mailid_list is the GSlist containing the mailid_list.

 @return TRUE-if all the recipients have certificates.
         FALSE-else.
*/
static gboolean check_cert_validity(GSList * mailid_list)
{
	GSList *expired_list = NULL;
	RecipientInfo *recipientInfo = NULL;
	gchar *emailid = NULL;
	gboolean response = TRUE;
	GtkWidget *dialog = NULL;
	while (mailid_list != NULL) {
		recipientInfo = mailid_list->data;
		if (recipientInfo->cert_id != 0 && recipientInfo->cert_valid != VALID) {
			expired_list =
			    g_slist_append(expired_list, g_strdup(recipientInfo->email_id));
		}
		mailid_list = mailid_list->next;
	}
	if (expired_list != NULL) {
		/*UI to pop up error message if atleast one certificate is  expired
		   Check for the common encryption algorithm supported by all  recipients and sender */
		emailid = create_mailid_strings(expired_list);

		dialog = create_user_confirmation(emailid, INVALID_CERT);
		response = gtk_dialog_run((GtkDialog *) dialog);
		g_slist_foreach(expired_list, (GFunc) g_free, NULL);
		g_slist_free(expired_list);
		if (response == GTK_RESPONSE_CANCEL) {
			g_free(emailid);
			g_slist_foreach(mailid_list, (GFunc) free_recipientinfo, NULL);
			g_slist_free(mailid_list);
			gtk_widget_destroy(dialog);
			return FALSE;
		} else if (response == GTK_RESPONSE_OK) {
			gtk_widget_destroy(dialog);
			return TRUE;
		}
	}
	return TRUE;
}


/**
 This function checks for the ids for which the certificates are not
 available & if available what is the status of the certificate if it expired
 or valid.
 
 @param msginfo is the MsgInfo structure.
 	This param is not freed here.
	
 @return gboolean-TRUE-if the recipient has a certificate & user wants to proceed sending
                 FALSE-else.
*/
static gboolean process_recipient_cert(MsgInfo * msg)
{
	GSList *emailid_list = NULL;
	GSList *mailid_list = NULL;
	/*gboolean recipient_cert = TRUE; */
	gboolean response = FALSE, no_common_algorithm = FALSE, proceed = TRUE;
	/*gint common_algorithm = 0; */
	gchar *emailid = NULL;
	/*gchar *infonote = NULL; */

/**Prepare a list of email ids containing in to,cc,bcc fields*/
	prepare_mailid_list(msg->header->to, &emailid_list);
	prepare_mailid_list(msg->header->cc, &emailid_list);
	prepare_mailid_list(msg->header->bcc, &emailid_list);

	get_recipient_certitifcateID(emailid_list);

	/*Check if all recipients have a certificate else pop up an error message */
	mailid_list = emailid_list;
	proceed = check_cert_availability(mailid_list);
	if (proceed == FALSE) {
		return FALSE;
	}
	mailid_list = emailid_list;
	proceed = check_cert_validity(mailid_list);
	if (proceed == FALSE) {
		return FALSE;
	}

	if (no_common_algorithm)
/*UI will pop an error note*/
		/* if (common_algorithm == weak_algorithm) */
	{
		response = get_user_confirmation(_("mcen_in_smime_weak_alg"),
						 _("mail_smime_continue"), NULL, CONFIRM_DIALOG);
		if (!response) {
			g_free(emailid);
			g_slist_foreach(emailid_list, (GFunc) free_recipientinfo, NULL);
			g_slist_free(emailid_list);
			emailid_list = NULL;
			return FALSE;
		}
	}
	mailid_list = emailid_list;
	fill_cert_idlist(mailid_list, msg);
	g_slist_free(emailid_list);
	emailid_list = NULL;
	return TRUE;
}


gchar *get_certificate_password(const guint cert_id)
{
	gchar *password = NULL;
	EVP_PKEY *pkey = NULL;
	gboolean cached = FALSE;
	GConfClient *client = NULL;
	GError *error = NULL;
	GConfValue *gvalue;

	/*check if passwords are cached */
	client = gconf_client_get_default();
	gvalue = gconf_client_get(client, GCONF_REMEMBER_PASSWORDS_KEY, &error);
	if (gvalue == NULL) {
		g_object_unref(client);
		return FALSE;
	}
	cached = gconf_value_get_bool(gvalue);
	if (cached == TRUE) {
		password = get_password(cert_id, passwd_list);
	} else {
		g_slist_foreach(passwd_list, (GFunc) g_free, NULL);
		g_slist_free(passwd_list);
		passwd_list = NULL;
	}
	if (password == NULL) {
		/*pop the password dilaog */
		pkey = certmanui_get_privatekey(app, cert_id, &password, NULL, NULL);
	}

	if (password != NULL) {
		if (cached == TRUE) {
			passwd_list = g_slist_append(passwd_list, g_strdup(password));
		}
	}
	g_object_unref(client);
	gconf_value_free(gvalue);
	return password;
}

/**
 Validations to be done when a mail is sent with security options.
 
 @param msginfo is the MsgInfo structure.
 	This param is not freed here.
	
 @return gboolean-TRUE-if the recipient has a certificate & user wants to proceed sending
                 FALSE-else.
*/
gboolean ui_editor_smime_validations(MsgInfo * msg)
{
	SMimeType smime = SMIME_NONE;
	guint cert_id = 0;
	gchar *email_id = NULL;
	gint cert_state = VALID;
	gboolean success = FALSE;
	PrefsAccount *accSettings = NULL;
	gchar *password = NULL;
	if (msg == NULL) {
		return FALSE;
	}

	smime = msg->settings->smime_type;	/*get the s/mime type as set by user */
/* obtain sender  email ID for the current account */
	if (smime == SMIME_NONE || smime == SMIME_INTERNAL) {
		return TRUE;
	}
	accSettings = ui_get_account_detail(msg->settings->account);
	email_id = g_strdup(accSettings->user_settings.email);
	ui_free_prefs_account_structure(accSettings);

	/*The following function will be in validations.c of S/MIME
	   handler */
	cert_id = get_private_certificate(email_id, &cert_state);
	if (cert_id == 0) {
		get_user_confirmation(_("mcen_in_smime_retrieverror"), NULL, NULL, INFO_BANNER);
		return FALSE;
	}
	/*get the password for the certificate */
	if (smime == SMIME_SIGN || smime == SMIME_SIGN_ENCRYPT) {
		if ((password = get_certificate_password(cert_id)) == NULL) {
			return FALSE;
		} else {
			msg->password = g_strdup(password);
		}
	}

	msg->recip_certs_uid = g_slist_append(msg->recip_certs_uid, GUINT_TO_POINTER(cert_id));
	if (cert_state != VALID) {
		success =
		    get_user_confirmation(_("mcen_cn_smime_invalid_cert"),
					  _("mail_smime_ok"), NULL, CONFIRM_DIALOG);
		if (!success) {
			g_slist_free(msg->recip_certs_uid);
			msg->recip_certs_uid = NULL;
		}
		if (smime == SMIME_SIGN ||
		    ((smime == SMIME_SIGN_ENCRYPT || smime == SMIME_ENCRYPT) && !success)) {
			return success;
		}
	}
	switch (smime) {
	case SMIME_SIGN:
		return TRUE;
	case SMIME_SIGN_ENCRYPT:
	case SMIME_ENCRYPT:
		if (process_recipient_cert(msg) == FALSE) {
			g_slist_free(msg->recip_certs_uid);
			msg->recip_certs_uid = NULL;
			return FALSE;
		}
		break;
	default:
		return TRUE;
	}
	return TRUE;
}
#endif

/**
 This function is used to set the security icon in both editor/viewer
 based on the security type of the mail.
 Except sign/encrypted security is not handled.
 
 @param SMimeType is the enum representing the security option of the amil.
 @param msgFields is the pointer to the MsgFields structure.
 	This paramter is not freed in this function.
 
 @return void.
 */
static void
set_icon(SMimeType mime, MsgFields * fields, gboolean trusted, gint valid, gboolean from_available)
{
	GtkWidget *sign_image = NULL;
	if (from_available != FALSE) {
		sign_image = fields->sign_image;
	} else {
		sign_image = fields->bccencrypt_image;
	}


	if (mime == SMIME_NONE || mime == SMIME_INTERNAL) {
		gtk_widget_hide(fields->sign_image);
		if (fields->encrypt_image != NULL)
			gtk_widget_hide(fields->encrypt_image);
		if (fields->ccencrypt_image)
			gtk_widget_hide(fields->ccencrypt_image);
		gtk_widget_hide(fields->bccencrypt_image);
		return;
	}
#if SMIME
	else if (mime == SMIME_SIGN) {
		if (trusted == TRUE && valid == VALID) {
			gtk_image_set_from_icon_name((GtkImage *) sign_image,
						     "qgn_viewer_messagin_signed",
						     GTK_ICON_SIZE_BUTTON);
		} else {
			gtk_image_set_from_icon_name((GtkImage *) sign_image,
						     "qgn_viewer_messagin_signed_untrusted",
						     GTK_ICON_SIZE_BUTTON);
		}
	} else if (mime == SMIME_ENCRYPT) {
		if (valid == VALID) {
			gtk_image_set_from_icon_name((GtkImage *) sign_image,
						     "qgn_viewer_messagin_encrypted",
						     GTK_ICON_SIZE_BUTTON);
		} else {
			gtk_image_set_from_icon_name((GtkImage *) sign_image,
						     "qgn_viewer_messagin_encrypted_untrusted",
						     GTK_ICON_SIZE_BUTTON);
		}
	}
	if (from_available != FALSE) {
		gtk_widget_show(fields->sign_image);
		gtk_widget_hide(fields->bccencrypt_image);
	} else {
		gtk_widget_show(fields->bccencrypt_image);
		gtk_widget_hide(fields->sign_image);
	}
	if (fields->encrypt_image != NULL)
		gtk_widget_hide(fields->encrypt_image);
	if (fields->ccencrypt_image)
		gtk_widget_hide(fields->ccencrypt_image);
#endif
	return;
}

void set_security_icon(SMimeType mime, MsgFields * fields)
{

	if (mime != SMIME_SIGN_ENCRYPT) {
		set_icon(mime, fields, TRUE, VALID, TRUE);
	}

	else {
		gtk_image_set_from_icon_name((GtkImage *) fields->sign_image,
					     "qgn_viewer_messagin_signed", GTK_ICON_SIZE_BUTTON);
		if (ui_get_cc_field_status() == TRUE) {
			gtk_image_set_from_icon_name((GtkImage *) fields->ccencrypt_image,
						     "qgn_viewer_messagin_encrypted",
						     GTK_ICON_SIZE_BUTTON);
			gtk_widget_show(fields->ccencrypt_image);
			gtk_widget_hide(fields->bccencrypt_image);
			if (fields->encrypt_image != NULL)
				gtk_widget_hide(fields->encrypt_image);
		} else if (ui_get_bcc_field_status() == TRUE) {
			gtk_image_set_from_icon_name((GtkImage *) fields->bccencrypt_image,
						     "qgn_viewer_messagin_encrypted",
						     GTK_ICON_SIZE_BUTTON);
			gtk_widget_show(fields->bccencrypt_image);
			if (fields->ccencrypt_image)
				gtk_widget_hide(fields->ccencrypt_image);
			if (fields->encrypt_image != NULL)
				gtk_widget_hide(fields->encrypt_image);
		} else {
			gtk_image_set_from_icon_name((GtkImage *) fields->encrypt_image,
						     "qgn_viewer_messagin_encrypted",
						     GTK_ICON_SIZE_BUTTON);
			if (fields->encrypt_image != NULL)
				gtk_widget_show(fields->encrypt_image);
			if (fields->ccencrypt_image)
				gtk_widget_hide(fields->ccencrypt_image);
			gtk_widget_hide(fields->bccencrypt_image);

		}
		gtk_widget_show(fields->sign_image);
	}
	return;
}

void set_viewer_security_icon(MsgFields * fields, MimeInfo * mime_info)
{
	GtkWidget *sign_image = NULL;
	GtkWidget *encrypt_image = NULL;

	if (mime_info->smime_type != SMIME_SIGN_ENCRYPT) {
		if (mime_info->smime_type == SMIME_SIGN) {
			set_icon(mime_info->smime_type, fields, mime_info->trust,
				 mime_info->cert_state_recip, fields->from_available);
		} else if (mime_info->smime_type == SMIME_ENCRYPT) {
			set_icon(mime_info->smime_type, fields, mime_info->trust,
				 mime_info->cert_state_sender, fields->from_available);
		} else {
			set_icon(mime_info->smime_type, fields, TRUE, 0, TRUE);
		}
		return;
	}
#if SMIME
	else {
		if (fields->from_available != FALSE) {
			sign_image = fields->sign_image;
			encrypt_image = fields->bccencrypt_image;
		} else {
			sign_image = fields->bccencrypt_image;
			if (fields->cc_available != FALSE) {
				encrypt_image = fields->ccencrypt_image;
			} else {
				encrypt_image = fields->encrypt_image;
			}
		}

		if (mime_info->trust == TRUE && mime_info->cert_state_recip == VALID) {
			gtk_image_set_from_icon_name((GtkImage *) sign_image,
						     "qgn_viewer_messagin_signed",
						     GTK_ICON_SIZE_BUTTON);
		} else {
			gtk_image_set_from_icon_name((GtkImage *) sign_image,
						     "qgn_viewer_messagin_signed_untrusted",
						     GTK_ICON_SIZE_BUTTON);
		}

		if (mime_info->cert_state_sender == VALID) {
			gtk_image_set_from_icon_name((GtkImage *) encrypt_image,
						     "qgn_viewer_messagin_encrypted",
						     GTK_ICON_SIZE_BUTTON);
		} else {
			gtk_image_set_from_icon_name((GtkImage *) encrypt_image,
						     "qgn_viewer_messagin_encrypted_untrusted",
						     GTK_ICON_SIZE_BUTTON);
		}


		gtk_widget_show(fields->bccencrypt_image);

		if (fields->from_available != FALSE)
			gtk_widget_show(fields->sign_image);
		else {
			gtk_widget_hide(fields->sign_image);

			if (fields->cc_available != FALSE) {
				if (fields->encrypt_image != NULL)
					gtk_widget_hide(fields->encrypt_image);
				gtk_widget_show(fields->ccencrypt_image);
			} else {
				if (fields->encrypt_image != NULL)
					gtk_widget_show(fields->encrypt_image);
				if (fields->ccencrypt_image)
					gtk_widget_hide(fields->ccencrypt_image);
			}
		}


	}
#endif
	return;
}
