/**
 * @file smtp_cb.c
 *
 * This file contains SMTP module related callback functions for UI.  
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#include <stdlib.h>
#include <glib.h>
#include "common.h"
#include "prefs_account.h"
#include "procmsg.h"
#include "al_callbacks.h"
#include "ui_al_callbacks.h"
#include "libosso.h"

/*UI header files*/
#include "ui_interface.h"
#include "accountMgmt_callback.h"

/* Global variables*/
extern MainViewInfo *email_main_view;	/* Declared in ui_interface.c */

/**
   This function is called to inform the UI module about error occured 
   in smtp Module 
   @param param_number smtp module related error
   @return none 
*/

void smtp_param_value_missing_cb(ParamMissing param_number)
{
	ui_param_value_missing_cb(param_number);
	return;
}


/**
   This function is called to inform the UI module about status/error 
   occured  in smtp Module 
   @param msg_number smtp module status/error code 
   @param index contains the length of GArray arguments array
   @param arguments GArray of osso_rpc_t elements  which contains the 
    additional information
   @return none 
*/

void smtp_set_msg_cb(gint msg_number, gint index, GArray * arguments)
{
	gchar *arg1 = NULL;
	gchar *arg2 = NULL;

	ULOG_DEBUG_F(" smtp_set_msg_cb = %d Index = %d \n", msg_number, index);

	if( (index >0) && (g_array_index(arguments, osso_rpc_t, 0).type == DBUS_TYPE_STRING)) {
		arg1 = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
		ULOG_DEBUG_F(" %s arg1 %s \n", __FUNCTION__, arg1);
	}
	if( (index >1) && (g_array_index(arguments, osso_rpc_t, 1).type == DBUS_TYPE_STRING)) {
		arg2 = (gchar *) g_array_index(arguments, osso_rpc_t, 1).value.s;
		ULOG_DEBUG_F(" %s arg2 %s \n", __FUNCTION__, arg2);
	}

	switch (msg_number) {
	case SMTP_START:
	case SMTP_AUTHENTICATING:
		{
			if (index != 2)
				ui_update_progressbar_text(CONNECTING, NULL);
			if (index == 1) {
				g_free(email_main_view->account_manager->current_sendrec_account);
				email_main_view->account_manager->current_sendrec_account =
				    g_strdup(g_array_index(arguments, osso_rpc_t, 0).value.s);
				gtk_widget_set_sensitive(email_main_view->menu->email_send_receive,
							 FALSE);
				gtk_widget_set_sensitive(email_main_view->tools->send_receive,
							 FALSE);
				/*Set state_save_disabled flag */
				email_main_view->tags->state_save_disabled = TRUE;
			}
			break;
		}
	case SMTP_SENDING:
		{
			ui_update_progressbar_text(SENDING, NULL);
			break;
		}
	case SMTP_USERID_INVALID:
	case SMTP_PASSWD_INVALID:
	case SMTP_SAVE_COMPLETE:
		{

			ui_set_error_msg_cb(msg_number, arg1, arg2);
			break;
		}
	case SMTP_SERVER_INVALID:
	case SMTP_SESSION_CONNECT_ERROR:
	case SMTP_AUTHTYPE_INVALID:
	case SMTP_AUTHENTICATION_FAIL_ERROR:
	case SMTP_RCPT_LIST_INVALID:
		{
			ui_set_error_msg_cb(msg_number, arg1, arg2);
			break;
		}
	case SMTP_PASSWD_FAIL:
		{
			ui_on_mailbox_protected_dialog(arg1, TRUE, FALSE);
			break;
		}
	case SMTP_PASSWD_WRONG:
		{
			ui_on_mailbox_protected_dialog(arg1, TRUE, TRUE);
			break;
		}
	case SMTP_CHECK_FOR_OUT_MSGS:
		{
			ui_send_outbox_mails_with_wait_status_cb();
			break;
		}
	default:
		{
			ui_set_msg_cb(msg_number);
			break;
		}
	}

	return;
}

/**
   This function is called to inform the UI module about percentage 
   of mail sent by smtp Module 
   @param percent percentage of mail sent by smtp. 
   @return none 
*/


void smtp_set_percentage_cb(gint percent)
{
	ui_set_percentage_cb(percent);
	return;
}


/**
   This function is called to inform the UI module about mails sent 
   in outbox.

   @param total_count Total number of mails exist in outbox
   @param mails_sent Mails sent from Outbox 
   @return none 
*/

void smtp_update_outbox_mails_sent(const gint total_count, const gint mails_sent)
{
	ui_update_progressbar_numbers(mails_sent, total_count, SENDING);
	return;
}

/**
   This function is called to inform the UI module to update the mail
   status(add/delete) to outbox.
   @param msgheader Message header information 
   @param add status about mail added or deleted 
   @param status about message delivery failed 
   @return none 
*/


void smtp_update_folder_view_cb(MsgHeader * msgheader, gboolean add, gboolean status)
{
	ULOG_DEBUG_F(" smtp_update_folder_view_cb status = %d \n", status);
	ui_update_folder_view_cb(msgheader, add, TRUE);
	return;
}

/**
   This function is called to inform the UI module about completion status 
   of send mail 
   @return none
*/

void smtp_send_complete_cb(void)
{
	ui_send_recv_complete_cb();

	/* Edit Account operation stopped Send/Receive */
	if (email_main_view->account_manager->operation_status == EDIT_OP) {
		ui_resume_edit_accout_oper();
	}
	/* Action after deletion of email account  stopped Send/Receive */
	else if (email_main_view->account_manager->operation_status == DELETE_OP) {
		ui_resume_delete_account_oper();
	} else {
		g_free(email_main_view->account_manager->current_sendrec_account);
		email_main_view->account_manager->current_sendrec_account = NULL;
	}
	/*Reset state_save_disabled flag */
	email_main_view->tags->state_save_disabled = FALSE;

	if (email_main_view->account_manager->configured_accounts != NULL) {
		gtk_widget_set_sensitive(email_main_view->menu->email_send_receive, TRUE);
		gtk_widget_set_sensitive(email_main_view->tools->send_receive, TRUE);
	}

	return;
}
