/**
 * @file ui_state_mgmt.c
 *
 * This file contains the implementation for state management 
 * module of OSSO EMAIL.
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#include "ui_common.h"
#include "ui_state_mgmt.h"
#include "editview.h"
#include "ui_al_callbacks.h"
#include "ui_interface.h"
#include "folder_mgmt.h"
#include "message_handler.h"
#include "list_view.h"
#include "email_mgr.h"

/*MainViewInfo object declared in ui_interface.c*/
extern MainViewInfo *email_main_view;

static EmailStateInfo *email_state = NULL;
static EmailPersistentInfo *email_persistent_settings = NULL;

/**
Helper function to create and initialize Email state object.
This object should be freed during application exit.

@param void
@return EmailStateInfo*  Pointer to newly created state object
*/
static EmailStateInfo *ui_initialize_email_state(void)
{
	EmailStateInfo *email_state_info = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	email_state_info = g_new0(EmailStateInfo, 1);

	email_state_info->oom_signal_received = FALSE;
	email_state_info->full_screen_mode = FALSE;
	email_state_info->current_folder = NULL;
	email_state_info->current_msg_id = NULL;
	email_state_info->folderview_scroll_position = 0;
	email_state_info->msgview_scroll_position = 0;
	email_state_info->marked_msgs = NULL;
	email_state_info->open_msgs_list = NULL;
	email_state_info->msgheaders_list = NULL;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return email_state_info;
}

/**
Helper function to create and initialize EmailPersistentInfo object.
This object should be freed during application exit.

@param void
@return EmailPersistentInfo*  Pointer to newly created persistency object
*/
static EmailPersistentInfo *ui_initialize_email_persistent_info(void)
{
	EmailPersistentInfo *persistent_info = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	persistent_info = g_new0(EmailPersistentInfo, 1);

	persistent_info->split_view_mode = TRUE;
	persistent_info->mainview_pane_pos = -1;
	persistent_info->mainview_tool_bar_hidden = FALSE;
	persistent_info->viewer_tool_bar_hidden = FALSE;
	persistent_info->editor_tool_bar_hidden = FALSE;
	persistent_info->mainview_fullscreen_tool_bar_hidden = FALSE;
	persistent_info->viewer_fullscreen_tool_bar_hidden = FALSE;
	persistent_info->editor_fullscreen_tool_bar_hidden = FALSE;
	persistent_info->cc_field_on = FALSE;
	persistent_info->bcc_field_on = FALSE;
	persistent_info->read_receipt = FALSE;
	persistent_info->msg_size_limit = 1000;
	persistent_info->play_sound = FALSE;
	persistent_info->msg_format_type = 0;
	persistent_info->inc_original = TRUE;
	persistent_info->sent_folder_clean_time_stamp = 0;
	persistent_info->sent_folder_clean_option = 0;
	persistent_info->sent_folder_clean_time_value = 0;
	persistent_info->viewing_html_fontname_index = -1;
	persistent_info->viewing_plain_fontname_index = -1;
	persistent_info->editing_html_fontname_index = -1;
	persistent_info->editing_plain_fontname_index = -1;
	persistent_info->delete_servermail_also = TRUE;
	persistent_info->folder_sort_info_list = ui_prepare_folder_sort_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return persistent_info;
}

/**
Helper function to read email persistent settings. To be called only
once during email start-up. It fills the global EmailPersistentInfo
object, which should be freed during application exit.

@param  void
@return boolean  TRUE-->success, FALSE-->failure
*/
gboolean ui_read_persistent_settings(void)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//If settings object is already initialized, return
	CHECK_PARAM((email_persistent_settings != NULL),
		    "Settings already initialized!\n", return FALSE);

	if (ui_dbus_persistentInfo_read(&email_persistent_settings) != OSSO_PACK_OK
	    || email_persistent_settings == NULL) {
		ULOG_DEBUG_F("No prev persistent settings or file read operation failed!\n");
		email_persistent_settings = ui_initialize_email_persistent_info();
		return FALSE;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return TRUE;
}

/**
Helper function to free global Email state object.

@param  void
@return void
*/
void ui_free_email_state_info(void)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_state == NULL), "Email state is NULL", return);

	//Remove previous state settings
	ui_remove_previous_state_settings();

	g_free(email_state);
	email_state = NULL;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to free global Email persistent settings object.

@param  void
@return void
*/
void ui_free_email_persistent_info(void)
{
	GSList *node = NULL;
	FolderSortInfo *sort_info = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Email persistent info is NULL", return);

	//Remove folder sort state settings
	for (node = email_persistent_settings->folder_sort_info_list; node != NULL;
	     node = node->next) {
		sort_info = (FolderSortInfo *) (node->data);
		g_free(sort_info->name);
		g_free(sort_info);
	}
	g_slist_free(email_persistent_settings->folder_sort_info_list);

	g_free(email_persistent_settings);
	email_persistent_settings = NULL;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Function to obtain Email UI's stored state-info from the underlying 
engine. Called every time at startup. If no previous state is found,
default state object is created.

@param  void
@return boolean TRUE-->Got previous state, FALSE-->No previous state
*/
gboolean ui_get_previous_state_info()
{
	EmailStateInfo *prev_email_state = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Get previous state from engine
	ui_dbus_statesave_read(&prev_email_state);
	if (prev_email_state == NULL) {
		ULOG_DEBUG_F("No previous state found!");
		email_state = ui_initialize_email_state();
		ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
		return FALSE;
	} else {
		email_state = prev_email_state;
		ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
		return TRUE;
	}
}

/**
Helper function to get the oom_signal status from the global Email
state object.

@param  void
@return gboolean oom_signal status
*/
gboolean ui_get_oom_signal_status()
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	CHECK_PARAM((email_state == NULL), "Input param check failed", return FALSE);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return email_state->oom_signal_received;
}

/*helper function to set the the state value*/
void ui_unset_oom_signal_status(void)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	CHECK_PARAM((email_state == NULL), "Input param check failed", return);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	email_state->oom_signal_received = FALSE;
}

/**
Helper function to set the oom_signal status into the global Email
state object.

@param  void
@return gboolean oom_signal status
*/
void ui_set_oom_signal_status()
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	CHECK_PARAM((email_state == NULL), "Input param check failed", return);
	email_state->oom_signal_received = TRUE;
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Function to store Email UI's persistent settings. Called every time 
any of the persistent settings change.

@param  void
@return void
*/
void ui_store_persistent_settings(void)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	//Store current settings
	if (ui_dbus_persistentInfo_write(email_persistent_settings) != OSSO_PACK_OK) {
		ULOG_INFO_F("Unable to store email persistent settings\n");
		return;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to restore previous folder sort settings.

@param  folder_sort_info_list  List of FolderInfo structures
@return void
*/
static void ui_restore_folder_sort_settings(GSList * folder_sort_info_list)
{
	GSList *node = NULL;
	FolderSortInfo *sort_info = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((folder_sort_info_list == NULL), "Input param check failed", return);

	for (node = folder_sort_info_list; node != NULL; node = node->next) {
		sort_info = (FolderSortInfo *) (node->data);
		ui_apply_folder_sorting(sort_info);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to update folder sort settings for existing folders,
or to add settings for a new folder.

@param  folder_name  Name of the folder
@param  sort_key     Folder sort key
@param  sort_order   Folder sort order
@return void
*/
void
ui_update_folder_sort_settings(const gchar * folder_name, const guint sort_key,
			       const guint sort_order)
{
	GSList *node = NULL;
	FolderSortInfo *sort_info = NULL;
	gboolean folder_found = FALSE;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((folder_name == NULL), "Input param check failed", return);
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);
	CHECK_PARAM((email_persistent_settings->folder_sort_info_list == NULL),
		    "Input param check failed", return);

	for (node = email_persistent_settings->folder_sort_info_list; node != NULL;
	     node = node->next) {
		sort_info = (FolderSortInfo *) (node->data);
		if (sort_info->name != NULL
		    && strncmp(sort_info->name, folder_name,
			       MAX_STR_LEN(sort_info->name, folder_name)) == 0) {
			sort_info->sort_key = sort_key;
			sort_info->sort_order = sort_order;
			folder_found = TRUE;
			break;
		}
	}

	//Add a new node to the list if necessary
	if (folder_found == FALSE) {
		sort_info = g_new0(FolderSortInfo, 1);

		sort_info->name = g_strdup(folder_name);
		sort_info->sort_key = sort_key;
		sort_info->sort_order = sort_order;
		email_persistent_settings->folder_sort_info_list =
		    g_slist_append(email_persistent_settings->folder_sort_info_list, sort_info);
	}
	//Store UI state
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function which restores persistent settings of Email UI -->
Split view mode, Main view toolbar state, Folder sort settings.

@param  void
@return void
*/
void ui_apply_persistent_settings()
{
	gint pane_width = 0;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets->folder_scrollwindow == NULL),
		    "Input param check failed", return);

	/*Split view / Simple view */
	if (email_persistent_settings->split_view_mode == FALSE) {
		ui_hide_folders(email_main_view->widgets->folder_scrollwindow, FALSE);
	} else {
		/*set the pane position between the folder and message view */
		pane_width = ui_get_folder_pane_width();
		gtk_paned_set_position(GTK_PANED(email_main_view->widgets->folder_msg_pane),
				       pane_width);
	}

	/*Hide toolbar if necessary */
	if (email_persistent_settings->mainview_tool_bar_hidden == TRUE) {
		ui_toggle_toolbar_visibility(FALSE, TRUE);
	}

	if (email_persistent_settings->mainview_fullscreen_tool_bar_hidden == TRUE) {
		ui_toggle_toolbar_visibility(FALSE, FALSE);
	}

	/*Restore folder sort settings */
	ui_restore_folder_sort_settings(email_persistent_settings->folder_sort_info_list);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function which restores previous Email UI state settings -->
Inbox message headers, Folder selection, Scroll positions, Marking
status, Restore openend messages, Message selection.

@param  void
@return void
*/
void ui_initialize_previous_state_settings()
{
	GtkAdjustment *adjustment = NULL;
	gchar *prev_selected_msg_id = NULL;
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_state == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);

	if (email_state->current_msg_id != NULL) {
		prev_selected_msg_id = g_strdup(email_state->current_msg_id);
	}

	/*Restore saved msg headers */
	ui_restore_latest_msg_headers();

	/*Restore folder selection */
	email_main_view->tags->oom_signal_received = TRUE;
	ui_select_row_from_view(email_state->current_folder, VIEW_FOLDER_LIST);

	/*Restore scroll positions */
	adjustment =
	    gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW
						(email_main_view->widgets->folder_scrollwindow));
	gtk_adjustment_set_value(adjustment, email_state->folderview_scroll_position);
	adjustment =
	    gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW
						(email_main_view->widgets->message_scrollwindow));
	gtk_adjustment_set_value(adjustment, email_state->msgview_scroll_position);

	/*Restore marking status */
	ui_restore_marked_msgs(email_state->marked_msgs, FALSE);

	/*Restore opened messages */
	ui_restore_opened_msgs(email_state->open_msgs_list);
	for (node = email_state->open_msgs_list; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) (node->data);
		g_free(open_window->msgid);
		g_free(open_window->oldmsgid);
		g_free(open_window);
	}
	g_slist_free(email_state->open_msgs_list);
	email_state->open_msgs_list = NULL;

	/*Restore message selection */
	ui_select_row_from_view(prev_selected_msg_id, VIEW_MSG_LIST);
	g_free(prev_selected_msg_id);

	email_main_view->tags->oom_signal_received = FALSE;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to fetch message headers from engine, for those
messages that need to be restored back.

@param  open_msgs_list  List of OpenedWindowData structures
@return void
*/
static void ui_fetch_msgs_to_open(const GSList * open_msgs_list)
{
	const GSList *node = NULL;
	GSList *msg_id_list = NULL, *msg_hdr_list = NULL;
	gint folder_num = -1;
	gchar *folder_name = NULL, *parent_name = NULL;
	OpenedWindowData *open_window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	for (node = open_msgs_list; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) (node->data);
		if (open_window->msgid != NULL
		    && strncmp(open_window->msgid, MAIN_VIEW_ID,
			       MAX_STR_LEN(open_window->msgid, MAIN_VIEW_ID)) != 0) {
			if (ui_get_msg_folder_names(open_window->msgid, &parent_name, &folder_name)
			    == FALSE) {
				ULOG_DEBUG_F("Unable to get folder name for msgid '%s'\n",
					     open_window->msgid);
				continue;
			}
			folder_num = ui_get_folder_model_number(folder_name, parent_name);

			//Inbox and Outbox messages are already populated at startup.
			//Prepare a list of remaining messages
			if (folder_num != INBOX_MODEL_NUM && folder_num != OUTBOX_MODEL_NUM) {
				msg_id_list =
				    g_slist_append(msg_id_list, g_strdup(open_window->msgid));
			}

			g_free(parent_name);
			g_free(folder_name);
		}
	}

	//Fetch necessary message headers and add them to respective folders
	if (msg_id_list != NULL) {
		msg_hdr_list = ui_dbus_get_msgheader_list(msg_id_list);
		for (node = msg_hdr_list; node != NULL; node = node->next) {
			ui_update_folder_view_cb((MsgHeader *) node->data, TRUE, TRUE);
		}
		g_slist_free(msg_hdr_list);
		g_slist_foreach(msg_id_list, (GFunc) g_free, NULL);
		g_slist_free(msg_id_list);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to restore previously opened viewer and editor windows.

@param  open_msgs_list  List of OpenedWindowData structures
@return void
*/
void ui_restore_opened_msgs(const GSList * open_msgs_list)
{
	const GSList *node = NULL;
	gint msg_row_num = -1, folder_num = -1;
	gchar *folder_name = NULL, *parent_name = NULL;
	OpenedWindowData *open_window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Fetch any unfetched messages first
	ui_fetch_msgs_to_open(open_msgs_list);

	for (node = open_msgs_list; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) (node->data);
		if (open_window->msgid != NULL
		    && strncmp(open_window->msgid, MAIN_VIEW_ID,
			       MAX_STR_LEN(open_window->msgid, MAIN_VIEW_ID)) != 0) {
			if (ui_get_msg_folder_names(open_window->msgid, &parent_name, &folder_name)
			    == FALSE) {
				ULOG_DEBUG_F("Unable to get folder name for msgid '%s'\n",
					     open_window->msgid);
				continue;
			}
			folder_num = ui_get_folder_model_number(folder_name, parent_name);

			msg_row_num = ui_get_row_number_of_msg(open_window->msgid, folder_num);
			if (msg_row_num == -1) {
				ULOG_DEBUG_F("Message with id '%s' not found\n",
					     open_window->msgid);
			} else {
				ui_open_message(msg_row_num, folder_num, TRUE);
			}

			g_free(parent_name);
			g_free(folder_name);
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to restore marking status of previously marked msgs.

@param  marked_msgs  List of message-id strings
@param  marking_on   Boolean flag to set/unset marking
@return void
*/
void ui_restore_marked_msgs(GSList * marked_msgs, gboolean marking_on)
{
	gint folder_num = -1, count = 0;
	gchar *folder_name = NULL, *parent_name = NULL;
	GtkTreeModel *sorted_model = NULL, *prev_model = NULL;
	GtkTreeIter iter;
	gchar *msgId = NULL;
	GtkTreeSelection *selection = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((marked_msgs == NULL), "Input param check failed", return);

	if (ui_get_msg_folder_names(marked_msgs->data, &parent_name, &folder_name) == FALSE) {
		ULOG_DEBUG_F("Unable to get folder name for msgid '%s'\n",
			     (gchar *) marked_msgs->data);
		return;
	}
	folder_num = ui_get_folder_model_number(folder_name, parent_name);
	if (folder_num == -1) {
		//Unable to resolve folder store number
		ULOG_DEBUG_F("Unable to resolve folder for msg '%s'\n", (gchar *) node->data);
		return;
	}
	g_free(parent_name);
	g_free(folder_name);

	selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->messageListView));
	sorted_model = ui_get_sorted_folder_model(folder_num);
	gtk_tree_selection_unselect_all(selection);
	if (!gtk_tree_model_get_iter_first(sorted_model, &iter)) {
		ULOG_DEBUG_F("Storage model empty\n");
		return;
	}
	/*Check is the model in question is the one which is set, if not set the 
	 * required model & the perform the operation.Later set the previous model back*/
	if (email_main_view->folder_manager->currently_displayed_model != folder_num) {
		prev_model =
		    gtk_tree_view_get_model(GTK_TREE_VIEW
					    (email_main_view->widgets->messageListView));
		gtk_tree_view_set_model(GTK_TREE_VIEW(email_main_view->widgets->messageListView),
					sorted_model);
	}
	//Traverse the rows and set/unset the marking status
	do {
		gtk_tree_model_get(sorted_model, &iter, COL_MSG_ID, &msgId, -1);

		if (msgId != NULL &&
		    g_slist_find_custom(marked_msgs, msgId, (GCompareFunc) ui_compare_utf8_strings)
		    != NULL) {
			gtk_tree_selection_select_iter(selection, &iter);
		}
		g_free(msgId);
	}
	while (gtk_tree_model_iter_next(sorted_model, &iter));

	ui_count_marked_rows(GTK_TREE_VIEW(email_main_view->widgets->messageListView), &count);

	/*Reset the model original model */
	if (prev_model)
		gtk_tree_view_set_model(GTK_TREE_VIEW(email_main_view->widgets->messageListView),
					prev_model);

	//Invoke marking mode
	if (marking_on == TRUE) {
		ui_invoke_marking_mode(TRUE);
		email_main_view->tags->marking_mode = TRUE;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to add back rows corresponding to message headers to
the Inbox folder.

@param  void
@return void
*/
void ui_restore_latest_msg_headers(void)
{
	GtkTreeModel *model = NULL;
	GSList *node = NULL;
	MsgHeader *msg_header = NULL;
	guint msgid_hash = 0;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_state == NULL), "Input param check failed", return);
	CHECK_PARAM((email_state->msgheaders_list == NULL), "Input param check failed", return);

	model = ui_get_unsorted_folder_model(INBOX_MODEL_NUM);
	CHECK_PARAM((model == NULL), "Model missing for Inbox folder!", return);

	for (node = email_state->msgheaders_list; node != NULL; node = node->next) {
		msg_header = (MsgHeader *) (node->data);

		//Check if header is already present in INBOX. If so, ignore it.
		msgid_hash = g_str_hash(msg_header->msgid);
		gtk_tree_model_foreach(model, ui_is_header_already_present, &msgid_hash);
		if (email_main_view->message_handler->is_message_present == TRUE) {
			email_main_view->message_handler->is_message_present = FALSE;
			ULOG_DEBUG_F("Header already present in Inbox!\n");
			ui_free_msg_header(msg_header);
			continue;
		}
		//Append the new row to Inbox
		if (ui_add_listview_header(msg_header, 0, TRUE) == FALSE) {
			ULOG_DEBUG_F
			    ("Adding new header with message-id '%s' to Inbox failed!\n",
			     msg_header->msgid);
		}
		//Free the MsgHeader memory
		ui_free_msg_header(msg_header);
	}

	if (email_main_view->folder_manager->currently_displayed_model == INBOX_MODEL_NUM) {
		ui_update_folder_specific_widgets(INBOX_MODEL_NUM);
	}

	g_slist_free(email_state->msgheaders_list);
	email_state->msgheaders_list = NULL;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to remove all non-persistent state settings from
the global state object. To be called during normal closure or the
Email UI.

@param  void
@return void
*/
void ui_remove_previous_state_settings()
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_state == NULL), "Input param check failed", return);

	email_state->oom_signal_received = FALSE;

	g_free(email_state->current_folder);
	email_state->current_folder = NULL;

	g_free(email_state->current_msg_id);
	email_state->current_msg_id = NULL;

	email_state->folderview_scroll_position = 0;

	email_state->msgview_scroll_position = 0;

	if (email_state->marked_msgs != NULL) {
		g_slist_foreach(email_state->marked_msgs, (GFunc) g_free, NULL);
		g_slist_free(email_state->marked_msgs);
		email_state->marked_msgs = NULL;
	}

	if (email_state->msgheaders_list != NULL) {
		g_slist_foreach(email_state->msgheaders_list, (GFunc) ui_free_msg_header, NULL);
		g_slist_free(email_state->msgheaders_list);
		email_state->msgheaders_list = NULL;
	}

	email_state->open_msgs_list = NULL;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to store message-ids of all marked messages into the
global state object. To be called whenever any message is marked/unmarked.

@param  void
@return void
*/
void ui_store_marked_msgs()
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_state == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);

	if (email_state->marked_msgs != NULL) {
		g_slist_foreach(email_state->marked_msgs, (GFunc) g_free, NULL);
		g_slist_free(email_state->marked_msgs);
		email_state->marked_msgs = NULL;
	}
	ui_get_selected_row_msg_ids((GtkTreeView *) email_main_view->widgets->messageListView,
				    &email_state->marked_msgs);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to store currently selected folder name into the
global state object. To be called whenever folder is changed in main view.

@param  void
@return void
*/
void ui_store_folder_selection()
{
	gchar *current_folder = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_state == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);

	CHECK_PARAM((email_state->current_folder == NULL
		     && email_main_view->folder_manager->currently_displayed_model ==
		     0), "No need to store", return);

	current_folder =
	    ui_get_folder_name(email_main_view->folder_manager->currently_displayed_model);
	CHECK_PARAM((current_folder == NULL), "Unable to get current folder selection", return);

	if (email_state->current_folder != NULL
	    && strncmp(email_state->current_folder, current_folder,
		       MAX_STR_LEN(email_state->current_folder, current_folder)) == 0) {
		ULOG_DEBUG_F("Same folder selected\n");
		g_free(current_folder);
		return;
	}

	g_free(email_state->current_folder);
	email_state->current_folder = g_strdup(current_folder);

	g_free(current_folder);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to store currently selected message-id into the
global state object. To be called whenever message selection gets changed.

@param  current_msg_id  Currently selected message id string
@return void
*/
void ui_store_message_selection(const gchar * current_msg_id)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((current_msg_id == NULL), "Input param check failed", return);
	CHECK_PARAM((email_state == NULL), "Input param check failed", return);

	if (email_state->current_msg_id != NULL
	    && strncmp(email_state->current_msg_id, current_msg_id,
		       MAX_STR_LEN(email_state->current_msg_id, current_msg_id)) == 0) {
		ULOG_DEBUG_F("Same message selected\n");
		return;
	}

	g_free(email_state->current_msg_id);
	email_state->current_msg_id = g_strdup(current_msg_id);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function to obtain currently selected message_id string

@param  void
@return gchar*  Copy of the current message_id.
                To be freed by calling function.
*/
gchar *ui_get_current_message_id()
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_state == NULL), "Input param check failed", return NULL);
	CHECK_PARAM((email_state->current_msg_id == NULL), "Current selection is null",
		    return NULL);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return g_strdup(email_state->current_msg_id);
}

/**
Helper function to store scroll positions of folder view and message view.
To be called when OOM signal is received by the application.

@param  void
@return void
*/
static void ui_store_mainview_scroll_positions()
{
	GtkAdjustment *adjustment = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_state == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);

	adjustment =
	    gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW
						(email_main_view->widgets->folder_scrollwindow));
	email_state->folderview_scroll_position = gtk_adjustment_get_value(adjustment);
	adjustment =
	    gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW
						(email_main_view->widgets->message_scrollwindow));
	email_state->msgview_scroll_position = gtk_adjustment_get_value(adjustment);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to send OOM signal to all opened messages.
To be called when OOM signal is received by the application.

@param  void
@return void
*/
static void ui_openmsgs_send_oom_signal()
{
	GSList *node = NULL;
	OpenedWindowData *open_windows = NULL;
	GtkWidget *open_appview = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*for editor getting the new msgids while state saving */
	for (node = email_main_view->open_windows;
	     node != NULL && node->data != NULL; node = node->next) {
		open_windows = (OpenedWindowData *) node->data;
		open_appview = GTK_WIDGET(open_windows->window_ptr);

		if (strcmp(open_windows->msgid, MAIN_VIEW_ID) == 0) {
			continue;
		}
		if (open_windows->iseditor == TRUE) {
			g_signal_emit_by_name(open_appview, "get_new_msgid");
			ui_flush_gtk_events();
		}
	}

	/*store the current state values of all the opened views */
	for (node = email_main_view->open_windows;
	     node != NULL && node->data != NULL; node = node->next) {
		open_windows = (OpenedWindowData *) node->data;
		open_appview = GTK_WIDGET(open_windows->window_ptr);

		if (strcmp(open_windows->msgid, MAIN_VIEW_ID) == 0) {
			continue;
		}
		if (open_appview != NULL) {
			g_signal_emit_by_name(open_appview, VIEW_GET_STATE_VALUES_SIGNAL);
		}
	}
}

/**
Function to store Email UI's current state-info. Called every time 
UI goes to background.

@param  void
@return void
*/
void ui_store_current_state_info()
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	g_print(" ui_store_current_state_info here \n");
	CHECK_PARAM((email_state == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);

	email_state->oom_signal_received = TRUE;

	//Store scroll positions
	ui_store_mainview_scroll_positions();

	//Fire oom signal to all opened msgs
	ui_openmsgs_send_oom_signal();
	ui_flush_gtk_events();

	/*Store opened messages */
	email_state->open_msgs_list = email_main_view->open_windows;

	//Send current state to engine
	ui_dbus_statesave_write(email_state);

	//email_state->oom_signal_received = FALSE;
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get the toolbar state of a opened msg from the global 
UI state.

@param   msgid    Message-id of opened message
@return  boolean  Tool bar hidden status
*/
gboolean ui_openmsg_get_toolbar_hidden(const gchar * msgid)
{
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;
	gboolean toolbar_hidden = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msgid == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_state == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_state->open_msgs_list == NULL), "Input param check failed",
		    return FALSE);

	for (node = email_state->open_msgs_list; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) (node->data);
		if (open_window->msgid != NULL
		    && strncmp(open_window->msgid, msgid,
			       MAX_STR_LEN(open_window->msgid, msgid)) == 0) {
			toolbar_hidden = open_window->tool_bar_hidden;
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return toolbar_hidden;
}

/**
Helper function to set the toolbar state of a opened msg into the global 
UI state object.

@param   msgid    Message-id of opened message
@param   flag     Tool bar hidden status
@return  void
*/
void ui_openmsg_set_toolbar_hidden(const gchar * msgid, gboolean flag)
{
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msgid == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->open_windows == NULL), "Input param check failed", return);

	for (node = email_main_view->open_windows; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) (node->data);
		if (open_window->msgid != NULL
		    && strncmp(open_window->msgid, msgid,
			       MAX_STR_LEN(open_window->msgid, msgid)) == 0) {
			open_window->tool_bar_hidden = flag;
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get the scroll position of a opened msg from the global 
UI state.

@param   msgid    Message-id of opened message
@return  gint     Scroll position as percentage
*/
guint ui_openmsg_get_scroll_position(const gchar * msgid)
{
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;
	guint scroll_pos = 0;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msgid == NULL), "Input param check failed", return 0);
	CHECK_PARAM((email_state == NULL), "Input param check failed", return 0);
	CHECK_PARAM((email_state->open_msgs_list == NULL), "Input param check failed", return 0);

	for (node = email_state->open_msgs_list; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) (node->data);
		if (open_window->msgid != NULL
		    && strncmp(open_window->msgid, msgid,
			       MAX_STR_LEN(open_window->msgid, msgid)) == 0) {
			scroll_pos = open_window->scroll_position;
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return scroll_pos;
}

/**
Helper function to set the scroll position of a opened msg into the global 
UI state object.

@param   msgid    Message-id of opened message
@param   position Scroll position
@return  void
*/
void ui_openmsg_set_scroll_position(const gchar * msgid, guint position)
{
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msgid == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->open_windows == NULL), "Input param check failed", return);

	for (node = email_main_view->open_windows; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) (node->data);
		if (open_window->msgid != NULL
		    && strncmp(open_window->msgid, msgid,
			       MAX_STR_LEN(open_window->msgid, msgid)) == 0) {
			open_window->scroll_position = position;
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get the zoom state of a opened msg from the global 
UI state.

@param   msgid    Message-id of opened message
@return  guint    Zoom value
*/
guint ui_openmsg_get_zoom_value(const gchar * msgid)
{
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;
	guint zoom_value = 0;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msgid == NULL), "Input param check failed", return 0);
	CHECK_PARAM((email_state == NULL), "Input param check failed", return 0);
	CHECK_PARAM((email_state->open_msgs_list == NULL), "Input param check failed", return 0);

	for (node = email_state->open_msgs_list; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) (node->data);
		if (open_window->msgid != NULL
		    && strncmp(open_window->msgid, msgid,
			       MAX_STR_LEN(open_window->msgid, msgid)) == 0) {
			zoom_value = open_window->zoom_percentage;
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return zoom_value;
}

/**
Helper function to set the zoom state of a opened msg into the global 
UI state object.

@param   msgid    Message-id of opened message
@param   zoom     Zoom value
@return  void
*/
void ui_openmsg_set_zoom_value(const gchar * msgid, guint zoom)
{
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msgid == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->open_windows == NULL), "Input param check failed", return);

	for (node = email_main_view->open_windows; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) (node->data);
		if (open_window->msgid != NULL
		    && strncmp(open_window->msgid, msgid,
			       MAX_STR_LEN(open_window->msgid, msgid)) == 0) {
			open_window->zoom_percentage = zoom;
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get the cursor position of a opened msg from the global 
UI state.

@param   msgid    Message-id of opened message
@param   x        Out param for cursor 'x' co-ordinate
@param   y        Out param for cursor 'y' co-ordinate
@return  void
*/
void ui_openmsg_get_cursor_position(const gchar * msgid, guint * x, guint * y)
{
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msgid == NULL), "Input param check failed", return);
	CHECK_PARAM((x == NULL), "Input param check failed", return);
	CHECK_PARAM((y == NULL), "Input param check failed", return);
	CHECK_PARAM((email_state == NULL), "Input param check failed", return);
	CHECK_PARAM((email_state->open_msgs_list == NULL), "Input param check failed", return);

	*x = 0;
	*y = 0;
	for (node = email_state->open_msgs_list; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) (node->data);
		if (open_window->msgid != NULL
		    && strncmp(open_window->msgid, msgid,
			       MAX_STR_LEN(open_window->msgid, msgid)) == 0) {
			*x = open_window->cursor_x_pos;
			*y = open_window->cursor_y_pos;
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to set the cursor position of a opened msg into the global 
UI state object.

@param   msgid    Message-id of opened message
@param   x        cursor 'x' co-ordinate
@param   y        cursor 'y' co-ordinate
@return void
*/
void ui_openmsg_set_cursor_position(const gchar * msgid, guint x, guint y)
{
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msgid == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->open_windows == NULL), "Input param check failed", return);

	for (node = email_main_view->open_windows; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) (node->data);
		if (open_window->msgid != NULL
		    && strncmp(open_window->msgid, msgid,
			       MAX_STR_LEN(open_window->msgid, msgid)) == 0) {
			open_window->cursor_x_pos = x;
			open_window->cursor_y_pos = y;
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get ON/OFF status for CC field from the global
UI state object.

@param   void
@return  boolean  CC field status
*/
gboolean ui_get_cc_field_status()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return FALSE);
	return email_persistent_settings->cc_field_on;
}

/**
Helper function to set ON/OFF status for CC field into the global
UI state object.

@param   void
@return  boolean  CC field status
*/
void ui_set_cc_field_status(gboolean status)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->cc_field_on = status;

	//Store UI state
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get ON/OFF status for BCC field from the global
UI state object.

@param   void
@return  boolean  BCC field status
*/
gboolean ui_get_bcc_field_status()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return FALSE);
	return email_persistent_settings->bcc_field_on;
}

/**
Helper function to set ON/OFF status for BCC field into the global
UI state object.

@param   void
@return  boolean  BCC field status
*/
void ui_set_bcc_field_status(gboolean status)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->bcc_field_on = status;

	//Store UI state
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to set ON/OFF status for split view mode into the global
UI state object.

@param   void
@return  boolean  split view status
*/
void ui_set_split_view_mode_status(gboolean status)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->split_view_mode = status;

	//Store UI state
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Sets pane position of main UI at the time of its closure into the global UI state object.

@param   hpaned is the mainview horizonatal pane.
@return  void
*/
void ui_set_folder_pane_width(GtkWidget * hpaned)
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);
	CHECK_PARAM((hpaned == NULL), "Input param check failed", return);

	email_persistent_settings->mainview_pane_pos =
	    gtk_paned_get_position(GTK_PANED(email_main_view->widgets->folder_msg_pane));

	/*Store UI state */
	ui_store_persistent_settings();
}

/**
Gets pane position of main UI from the global UI state object. If the persistent settings is NULL or has invalid values, returns the default position for pane.

@return  the pane position.
*/
gint ui_get_folder_pane_width()
{
	gint screen_width = 0;

	if ((email_persistent_settings != NULL)
	    && (email_persistent_settings->mainview_pane_pos != -1)) {
		/*persistent_setting has some valid settings */
		return email_persistent_settings->mainview_pane_pos;
	} else {
		/*get default position */
		gtk_window_get_size(GTK_WINDOW(app), &screen_width, NULL);
		return screen_width * FOLDER_PANE_WIDTH;
	}
}

/**
Helper function to set show/hide status for main view toolbar into the
global UI state object.

@param   void
@return  boolean  toolbar hidden status
*/
void ui_set_toolbar_hidden_status(gboolean status)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->mainview_tool_bar_hidden = status;

	//Store UI state
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to set show/hide status for fullscreen 
main view toolbar into the global UI state object.

@param   void
@return  boolean  toolbar hidden status
*/
void ui_set_fullscreen_toolbar_hidden_status(gboolean status)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->mainview_fullscreen_tool_bar_hidden = status;

	//Store UI state
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get show/hide status for fullscreen mainview toolbar from the
global UI state object.

@param   void
@return  boolean  toolbar hidden status
*/
gboolean ui_get_fullscreen_toolbar_hidden_status()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return FALSE);
	return email_persistent_settings->mainview_fullscreen_tool_bar_hidden;
}

/**
Helper function to get show/hide status for normal 
main view toolbar from the global UI state object.

@param   void
@return  boolean  toolbar hidden status
*/
gboolean ui_get_toolbar_hidden_status()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return FALSE);
	return email_persistent_settings->mainview_tool_bar_hidden;
}



/**
Helper function to set show/hide status for viewer toolbar into the
global UI state object.

@param   void
@return  boolean  toolbar hidden status
*/
void ui_set_viewer_toolbar_hidden_status(gboolean status)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->viewer_tool_bar_hidden = status;

	//Store UI state
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get show/hide status for viewer toolbar from the
global UI state object.

@param   void
@return  boolean  toolbar hidden status
*/
gboolean ui_get_viewer_toolbar_hidden_status()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return FALSE);
	return email_persistent_settings->viewer_tool_bar_hidden;
}

/**
Helper function to set show/hide status for fullscreen viewer toolbar into the
global UI state object.

@param   void
@return  boolean  toolbar hidden status
*/
void ui_set_viewer_fullscreen_toolbar_hidden_status(gboolean status)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->viewer_fullscreen_tool_bar_hidden = status;

	//Store UI state
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get show/hide status for fullscreen viewer toolbar from the
global UI state object.

@param   void
@return  boolean  toolbar hidden status
*/
gboolean ui_get_viewer_fullscreen_toolbar_hidden_status()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return FALSE);
	return email_persistent_settings->viewer_fullscreen_tool_bar_hidden;
}

/**
Helper function to set show/hide status for editor toolbar into the
global UI state object.

@param   void
@return  boolean  toolbar hidden status
*/
void ui_set_editor_toolbar_hidden_status(gboolean status)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->editor_tool_bar_hidden = status;

	//Store UI state
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get show/hide status for editor toolbar from the
global UI state object.

@param   void
@return  boolean  toolbar hidden status
*/
gboolean ui_get_editor_toolbar_hidden_status()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return FALSE);
	return email_persistent_settings->editor_tool_bar_hidden;
}

/**
Helper function to set show/hide status for fullscreen editor toolbar into the
global UI state object.

@param   void
@return  boolean  toolbar hidden status
*/
void ui_set_editor_fullscreen_toolbar_hidden_status(gboolean status)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->editor_fullscreen_tool_bar_hidden = status;

	//Store UI state
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get show/hide status for fullscreen editor toolbar from the
global UI state object.

@param   void
@return  boolean  toolbar hidden status
*/
gboolean ui_get_editor_fullscreen_toolbar_hidden_status()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return FALSE);
	return email_persistent_settings->editor_fullscreen_tool_bar_hidden;
}

/**
Helper function to set sent folder cleaning settings into the global UI
state object.

@param   void
@return  boolean  toolbar hidden status
*/
void
ui_set_sent_folder_cleaning_settings(gint clean_option,
				     gint clean_time_stamp, gint clean_time_value)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->sent_folder_clean_option = clean_option;
	email_persistent_settings->sent_folder_clean_time_stamp = clean_time_stamp;
	email_persistent_settings->sent_folder_clean_time_value = clean_time_value;

	//Store UI state
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get sent_folder_clean_option from the global UI
state object.

@param   void
@return  gint  sent_folder_clean_option value
*/
gint ui_get_sent_folder_clean_option()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return 0);
	return email_persistent_settings->sent_folder_clean_option;
}

/**
Helper function to get sent_folder_clean_time_stamp from the global UI
state object.

@param   void
@return  gint  sent_folder_clean_time_stamp value
*/
gint ui_get_sent_folder_clean_time_stamp()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return 0);
	return email_persistent_settings->sent_folder_clean_time_stamp;
}

/**
Helper function to get sent_folder_clean_time_value from the global UI
state object.

@param   void
@return  gint  sent_folder_clean_time_value value
*/
gint ui_get_sent_folder_clean_time_value()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return 0);
	return email_persistent_settings->sent_folder_clean_time_value;
}

/**
Helper function to set default font settings for editor and viewer 
into the global UI state object.

@param  
@return void
*/
void
ui_set_default_settings(gint viewing_html_fontname_index,
			gint viewing_plain_fontname_index,
			gint editing_html_fontname_index, gint editing_plain_fontname_index)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->viewing_html_fontname_index = viewing_html_fontname_index;
	email_persistent_settings->viewing_plain_fontname_index = viewing_plain_fontname_index;
	email_persistent_settings->editing_html_fontname_index = editing_html_fontname_index;
	email_persistent_settings->editing_plain_fontname_index = editing_plain_fontname_index;

	//Store UI state
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get viewing_html_fontname_index from the global UI
state object.

@param   void
@return  gint  viewing_html_fontname_index
*/
gint ui_get_viewer_html_fontindex()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return 0);
	if (email_persistent_settings->viewing_html_fontname_index >=
            sizeof(font_families) / sizeof(gchar *)
            || email_persistent_settings->viewing_html_fontname_index < 0)
                email_persistent_settings->viewing_html_fontname_index = 0;

	return email_persistent_settings->viewing_html_fontname_index;
}

/**
Helper function to get viewing_plain_fontname_index from the global UI
state object.

@param   void
@return  gint  viewing_plain_fontname_index
*/
gint ui_get_viewer_plain_fontindex()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return 0);
	if (email_persistent_settings->viewing_plain_fontname_index >=
            sizeof(font_families) / sizeof(gchar *)
            || email_persistent_settings->viewing_plain_fontname_index < 0)
                email_persistent_settings->viewing_plain_fontname_index = 0;

	return email_persistent_settings->viewing_plain_fontname_index;
}

/**
Helper function to get editing_html_fontname_index from the global UI
state object.

@param   void
@return  gint  editing_html_fontname_index
*/
gint ui_get_editor_html_fontindex()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return 0);
	if (email_persistent_settings->editing_html_fontname_index >=
            sizeof(font_families) / sizeof(gchar *)
            || email_persistent_settings->editing_html_fontname_index < 0)
                email_persistent_settings->editing_html_fontname_index = 0;

	return email_persistent_settings->editing_html_fontname_index;
}

/**
Helper function to get editing_plain_fontname_index from the global UI
state object.

@param   void
@return  gint  editing_plain_fontname_index
*/
gint ui_get_editor_plain_fontindex()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return 0);
	if (email_persistent_settings->editing_plain_fontname_index >=
            sizeof(font_families) / sizeof(gchar *)
            || email_persistent_settings->editing_plain_fontname_index < 0)
                email_persistent_settings->editing_plain_fontname_index = 0;

	return email_persistent_settings->editing_plain_fontname_index;
}

/**
Helper function to set dialog info of whether to delete mail from both phone and server into the global UI state object.

@param  boolean whether to delete mail from both phone and server 
@return void
*/
void ui_set_delete_email_info(gboolean delete_servermail_also)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->delete_servermail_also = delete_servermail_also;

	/*Store UI state */
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
Helper function to get info for delete email dialog of whether to delete mail from both phone and server from the global UI state object.

@param   void
@return  boolean whether to delete mail from both phone and server 
*/
gboolean ui_get_delete_email_info()
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return TRUE);
	return email_persistent_settings->delete_servermail_also;
}

/**
Gets mail settings info for fields present in settings dialog from the global UI state object.

@param   read_receipt is out param. It represents whether to request for read receipt.
@param   size_limit is out param. It represents size limit of a message.
@param   play_sound is  out param. It represents whether to play sound upon msg arrival.
@param   msg_format_type is out param. It represents html/plain nature of msg
@param   inc_original is out param. It represents whwther to include original msg info while replying a mail.
@return void.
*/
void ui_get_email_settings_info(gboolean * read_receipt, gint * size_limit,
				gboolean * play_sound, gint * msg_format_type,
				gboolean * inc_original)
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);
	if (read_receipt)
		*read_receipt = email_persistent_settings->read_receipt;
	if (size_limit)
		*size_limit = email_persistent_settings->msg_size_limit;
	if (play_sound)
		*play_sound = email_persistent_settings->play_sound;
	if (msg_format_type)
		*msg_format_type = email_persistent_settings->msg_format_type;
	if (inc_original)
		*inc_original = email_persistent_settings->inc_original;
}

/**
Sets mail settings info of fields present in settings dialog into the global UI persistent state object.

@param   read_receipt represents whether to request for read receipt.
@param   size_limit represents size limit of a message.
@param   play_sound represents whether to play sound upon msg arrival.
@param   msg_format_type represents html/plain nature of msg
@param   inc_original represents whether to include original msg info while replying a mail.
@return void.
*/
void ui_set_email_settings_info(gboolean read_receipt, gint size_limit,
				gboolean play_sound, gint msg_format_type, gboolean inc_original)
{
	CHECK_PARAM((email_persistent_settings == NULL), "Input param check failed", return);

	email_persistent_settings->read_receipt = read_receipt;
	email_persistent_settings->msg_size_limit = size_limit;
	email_persistent_settings->play_sound = play_sound;
	email_persistent_settings->msg_format_type = msg_format_type;
	email_persistent_settings->inc_original = inc_original;

	/*Store UI state */
	ui_store_persistent_settings();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}
