/**
 * @file ui_utils.c
 * 
 * This file contains the implementation for utility functions
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#include "email_hildon.h"
#include "ui_utils.h"
#include "html_parser.h"
#include "ui_common.h"
#include "ui_defs.h"
#include "accountMgmt_callback.h"
#include "defs.h"

#include "check_names.h"

#define BT_NAME_PATH "/usr/bin/btname"

#define INVCERT 	_("mcen_cn_smime_invalid_certificate")
#define UNABLE_TOIMPRT 	_("mail_in_unable_to_import_cert")
#define MISSING_10	_("mail_nc_attachments_must_be_retrived")
#define MISSING_11	_("emev_bd_delete_attachements")
#define MISSING_13	_("emev_nc_mailbox_notavailable")
#define MISSING_14	_("emev_nc_mailbox_inuse")
#define MISSING_15	_("mcen_bd_purge_file_remove")
#define MISSING_16	_("emev_bd_retrieve_attachments_cancel")
#define MISSING_18	_("mail_ni_ui_folder_file_error")
#define MISSING_19	_("mail_ni_ui_folder_get_header_error")
#define MISSING_20	_("mail_ni_ui_folder_get_msg_folder_error")
#define MISSING_21	_("mcen_ib_changes_discarded")
#define MISSING_23	_("mcen_ib_readonly_attachment")
#define MISSING_24	_("mcen_ib_username_pw_incorrect")
#define MISSING_25	_("mcen_ib_message_already_deleted")
#define MISSING_26	_("mail_ib_file_operation_failed")
#define MISSING_27	_("mail_ib_ui_folder_copy_msgs_error")
#define MISSING_31	_("mcen_ib_no_mmc_found")
#define MISSING_32	_("mcen_ti_add_email_title")

extern osso_context_t *ui_context;	/*declared in main.c */
extern MainViewInfo *email_main_view;	/* Declared in ui_interface.c */

static guint16 contact_field = 0;
static guint16 field_status = 0;
static guint contact_id = 0;
static GList *contact_search_list = NULL;
static gint current_focus_field_number = 0;
gboolean fired_on_note = FALSE;
static void insert_separator_at_end(GtkTextBuffer * addressBuffer);

/**
		  This is helper function to set the currently focused contact field number. 
		  This would help in focusing the field while adding/deleting a field in contact
		  window.
		  @param field_number stores the field number currently focused.
		  @return nothing.
*/
void ui_set_current_contact_field_number(gint field_number)
{
	if (field_number > 12) {
		return;
	}

	current_focus_field_number = field_number;

	return;
}

/**
  This is helper function to get the currently focused contact field number. 
  This would help in focusing the field while adding/deleting a field in contact
  window.
  @param nothing.
  @return field_number of the currently focused field in contact window.
*/
gint ui_get_current_contact_field_number(void)
{
	return current_focus_field_number;
}

/*
  This is helper function to get the contact id on adding each new contact 
  in contact data base.
  @param nothing.
  @return number contact id.
*/
guint ui_get_contact_id(void)
{
	contact_id = contact_id + 1;
	return contact_id;
}

/**
  This is helper function to set a perticular bit position in contact field 
  variable. Depending on the bit value and the value to be set the bit 
  positon will be set to 1 or 0.This would help to get the current status 
  of fields the are enabled in contact window.
  @param bit_value The bit positon value to set.
  @param value The value to be set.
  @return Nothing.
*/
void set_contact_field_variable(guint16 bit_value, gboolean value)
{
	if (value) {
		if ((contact_field & bit_value) == bit_value) {
			return;	//already set
		} else {
			contact_field += bit_value;
			return;
		}
	} else {
		if ((contact_field & bit_value) == 0) {
			return;	//already unset
		} else {
			contact_field -= bit_value;
			return;
		}
	}
	return;
}

/**
  This is helper function to get bit value in perticular bit position.
  @param value The perticular bit position value from where the value to be retrieved.
  @return TRUE for the bit value 1.
	 FALSE for the bit value 2.
*/
gboolean get_contact_field_variable(guint16 value)
{
	gboolean retval = FALSE;
	if ((contact_field & value) == value) {
		retval = TRUE;
	}
	return retval;
}

/**
  This is helper function to set the perticular bit field status value.
  This would help in getting the current focussed field in contact window.
  @param field_value The field value that to be set.
  @return nothing.
*/
void set_contact_field_status(guint16 field_value)
{
	field_status = field_value;
	return;
}

/**
  This is helper function to set the current field status value.
  @param bit_value This perticular bit value to be set.
  @param value The value to be set.
  @return Nothing.
*/
void set_contact_field_status_variable(guint16 bit_value, gboolean value)
{
	if (value) {
		if ((field_status & bit_value) == bit_value) {
			return;	//already set
		} else {
			field_status += bit_value;
			return;
		}
	} else {
		if ((field_status & bit_value) == 0) {
			return;	//already unset
		} else {
			field_status -= bit_value;
			return;
		}
	}
	return;
}

/**
  This is helper function to get the current field status value.
  @param value The perticular field value from which value to be retrieved.
  @return TRUE If the perticular field value is set, otherwise
			 FALSE.
*/
gboolean get_contact_field_status_variable(guint16 value)
{
	gboolean retval = FALSE;
	if ((field_status & value) == value) {
		retval = TRUE;
	}
	return retval;
}

/**
  This is helper function to get the total field status of the current contact.
  @param Nothing.
  @return field status of the current contact which is 16 bit value.
*/
guint16 get_contact_field_staus(void)
{
	return field_status;
}

/**
  Helper function for printing debug statements to standard output.
  @param  format  Formatting string (format is same as used by printf, etc.)
  @param  ...     variable number of arguments to fit into above format
  @return void
*/
void debug_print(const gchar * format, ...)
{
#ifdef DEBUG
	va_list args;
	gchar buf[BUFFSIZE];

	va_start(args, format);
	g_vsnprintf(buf, sizeof(buf), format, args);
	va_end(args);

	fputs(buf, stdout);
#endif
}

/**
  Helper function to get FontInfo object given its corresponding
  PangoFontDescription object.

  @param  fd        Pointer to PangoFontDescription object
  @return FontInfo* Pointer to FontInfo object
*/
FontInfo *getfontinfo(PangoFontDescription * fd)
{
	FontInfo *tmpfontinfo = NULL;
	gint weight = 0;
	gint size = 0;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	tmpfontinfo = g_new0(FontInfo, 1);
	if (tmpfontinfo == NULL) {
		ULOG_CRIT_F("Unable to allocate memory\n");
		return NULL;
	};
	if (fd == NULL) {
		return tmpfontinfo;
	};

	tmpfontinfo->name = (gchar *) pango_font_description_get_family(fd);
	size = pango_font_description_get_size(fd) / PANGO_SCALE;
	tmpfontinfo->size = g_strdup_printf("%d", size);	//tmpsize->str;
	if (pango_font_description_get_style(fd)) {
		tmpfontinfo->italic = TRUE;
	}
	weight = pango_font_description_get_weight(fd);
	if (weight != PANGO_WEIGHT_NORMAL) {
		tmpfontinfo->bold = TRUE;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return tmpfontinfo;
}

/**
 This is a callback function called where ever zoom is changed.

 @param tag is the GtkTexttag for zoom.
 @param zoominfo is the struct having zoom related info.

 @return void.
*/
static void recalculate_font_size(GtkTextTag * tag, Zoom * zoominfo)
{
	gint fontsize = 0;

	if (tag == NULL) {
		ULOG_DEBUG("Error validating input parameters\n");
		return;
	}
	g_object_get(G_OBJECT(tag), SIZE, &fontsize, NULL);
	if (fontsize > 0) {
		g_object_set(G_OBJECT(tag), SIZE,
			     (gint) ((fontsize * (zoominfo->zoom_factor)) /
				     zoominfo->previous_level), NULL);
	}
	return;
}

/**
 This is a callback function called where ever zoom is changed.

 @param buffer isthe GTkTextBuffer for the body field.
 @param zoominfo Pointer to 'Zoom' object

 @return void.
*/
static void osso_email_zoom_editor(GtkTextBuffer * buffer, Zoom * zoominfo)
{
	if (buffer == NULL) {
		ULOG_DEBUG("error validating input parameters\n");
		return;
	}
	gtk_text_tag_table_foreach(gtk_text_buffer_get_tag_table(buffer), (GtkTextTagTableForeach)
				   recalculate_font_size, zoominfo);
	return;
}

/**set the zoom of current message.

 @param zoominfo is a pointer to structure ZoomInfo.
        This parameter is not freed here.
 @param current_level isthe current zoom level.

 @return void.
*/
void ui_set_zoom(Zoom * zoominfo, gint current_level)
{
	GtkRcStyle *rcstyle = NULL;
	GtkTextBuffer *buf = NULL;
	GtkTextAttributes *values = NULL;
	PangoFontDescription *desc = NULL;
	gchar *zoomfont = NULL;

	/*Zoom viewer */
	if (zoominfo->mail_viewer) {
		mail_viewer_set_zoom(MAIL_VIEWER(zoominfo->mail_viewer),
				     (gdouble) current_level / DEFAULT_ZOOM);
	} else if (zoominfo->msgbody) {
		buf = gtk_text_view_get_buffer(zoominfo->msgbody);
		zoominfo->zoom_factor = current_level;
		osso_email_zoom_editor(buf, zoominfo);
		rcstyle = gtk_widget_get_modifier_style((GtkWidget *) zoominfo->msgbody);
		if (GTK_IS_TEXT_VIEW(zoominfo->msgbody)) {
			ULOG_DEBUG("The zoomfont is %s\n", zoomfont);
			values = gtk_text_view_get_default_attributes(zoominfo->msgbody);
			zoomfont = setzoom(values->font, zoominfo->previous_level, current_level);
			desc = pango_font_description_from_string(zoomfont);
			rcstyle->font_desc = desc;
			gtk_widget_modify_style((GtkWidget *) zoominfo->msgbody, rcstyle);
			gtk_text_attributes_unref(values);
			g_free(zoomfont);
		}
	}
	zoominfo->previous_level = current_level;

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/*
  Helper function for setting zoom value for various fonts.

  @param  fd             Pointer to PangoFontDescription object
  @param  previous_level Previous zoom level
  @param  current_level  Current zoom level
  @return gchar*         String representation of font description
*/
gchar *setzoom(PangoFontDescription * fd, gint previous_level, gint current_level)
{
	double num_of_levels = 0;
	double applicable_size = 0;
	gint fontsize = 0;
	gchar *zoom = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	fontsize = pango_font_description_get_size(fd) / PANGO_SCALE;


	if (current_level < previous_level) {
		num_of_levels = (double) previous_level / (double) current_level;
		applicable_size = (double) fontsize / (double) num_of_levels;
	} else {
		num_of_levels = (double) current_level / (double) previous_level;
		applicable_size = (double) fontsize *(double) num_of_levels;
	}
	ULOG_DEBUG_F("The number of levels %f\n", num_of_levels);
	ULOG_DEBUG_F("The current and applicable size are %d and %f\n", fontsize, applicable_size);

	pango_font_description_set_absolute_size(fd, applicable_size * PANGO_SCALE);
	zoom = pango_font_description_to_string(fd);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return zoom;
}

/**
  Helper function to set initial zoom level for an editor view.

  @param  htflag         Html or Plain-text
  @param  textview_body  Pointer to GtkTextView widget containing mail body
  @param  mail_viewer    Pointer to MailViewer widget used for showing msgbody in viewer. 
  @param  msgid          Message id string
  @param  previous_level Previous zoom level
*/
Zoom *set_initial_zoom_level(GtkTextView * textview_body,
			     GtkWidget * mail_viewer, gchar * msgid, gint previous_level)
{

	Zoom *zoominfo = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	zoominfo = g_new0(Zoom, 1);
	if (zoominfo == NULL) {
		ULOG_CRIT_F("Unable to allocate memory\n");
		return NULL;
	}
	zoominfo->msgbody = textview_body;
	zoominfo->previous_level = previous_level;
	zoominfo->mail_viewer = mail_viewer;
	if (msgid != NULL) {
		zoominfo->msgid = g_strdup(msgid);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return zoominfo;
}

/**
  helper function to set the GtkWidgets in the Zoom structutre
  
  @param zoominfo is the pointer to the Zoom structure.
  @param zoom_menu200 is the pointer to the Gtkwidget-> the 200% zoom menu.
  @param zoom_menu150 is the pointer to the Gtkwidget-> the 150% zoom menu.
  @param zoom_menu100 is the pointer to the Gtkwidget-> the 100% zoom menu.
  @param zoom_menu50 is the pointer to the Gtkwidget-> the 50% zoom menu.
  @return void.
*/
void set_zoominfo_menus(Zoom * zoominfo, gpointer menu)
{
	EditorMenu *editor_menu = NULL;
	ViewerMenu *viewer_menu = NULL;

	if (zoominfo->msgbody) {
		editor_menu = menu;
		zoominfo->zoom_menu200 = editor_menu->zoom_out_max_level;
		zoominfo->zoom_menu150 = editor_menu->zoom_out_level;
		zoominfo->zoom_menu100 = editor_menu->current_zoom_level;
		zoominfo->zoom_menu50 = editor_menu->zoom_in_level;
		zoominfo->zoom_menu80 = editor_menu->zoom_80;
		zoominfo->zoom_menu120 = editor_menu->zoom_120;
	} else {
		viewer_menu = menu;
		zoominfo->zoom_menu200 = viewer_menu->zoom_out_max_level;
		zoominfo->zoom_menu150 = viewer_menu->zoom_out_level;
		zoominfo->zoom_menu100 = viewer_menu->current_zoom_level;
		zoominfo->zoom_menu50 = viewer_menu->zoom_in_level;
		zoominfo->zoom_menu80 = viewer_menu->zoom_80;
		zoominfo->zoom_menu120 = viewer_menu->zoom_120;
	}
	return;
}

/**
  Function to set the zoom level in both editor & viewer based on the inherited value.
 
  @param msgType is the gint representing the mode in which editor is opened.
  @param zoominfo is the pointer to the Zoom structuer.
  @param zoomlevel is the integer value representing the zoom.
  @return void.
*/
void set_inherited_zoom(gint msgType, Zoom * zoominfo, gint zoomlevel)
{
	gint zoomvalue = DEFAULT_ZOOM;


	if (zoomlevel == 0) {
		zoomvalue = DEFAULT_ZOOM;
	} else {
		zoomvalue = zoomlevel;
		if (zoomvalue == ZOOM_OUT) {
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(zoominfo->zoom_menu200),
						       TRUE);
		} else if (zoomvalue == ZOOM_IN) {
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(zoominfo->zoom_menu50),
						       TRUE);
		} else if (zoomvalue == ZOOM_MEDIUM) {
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(zoominfo->zoom_menu150),
						       TRUE);
		} else if (zoomvalue == ZOOM_80) {
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(zoominfo->zoom_menu80),
						       TRUE);
		} else if (zoomvalue == ZOOM_120) {
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(zoominfo->zoom_menu120),
						       TRUE);
		}
	}

	if (zoomvalue != DEFAULT_ZOOM) {
		ui_set_zoom(zoominfo, zoomvalue);
	}
	return;
}



/**
  Helper function to font index given its name.

  @param  fontname  Name of the font
  @return gint      Font index
*/
gint get_index_font_size(gint fontsize)
{
	gint count = 0;
	gint font_sizes[] = { 6, 8, 10, 12, 16, 24, 32 };

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	for (count = 0; count < sizeof(font_sizes) / sizeof(gint); count++) {
		if (font_sizes[count] == fontsize) {
			return count;
		}
	}
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return count;
}

/**
This is GDestroyNotify function called when a complex dialog
is closed. It unsets the 'state_save_disabled' flag.

@param  data  String data attached to the dialog
@return void
*/
void ui_complex_dlg_destroy_notify(gpointer data)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Unset state_save_disabled flag */
	email_main_view->tags->state_save_disabled = FALSE;

	/*Free the data */
	if (data) {
		g_free(data);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
  Helper function to get the top most email window pointer. It gets list of
  top level windows and searches the list until it finds the topmost window
  whose "Window_Name" attribute is set as "Osso Email".

  @param  void
  @return GtkWidget*  Pointer to top most window
*/
GtkWidget *ui_get_top_level_window(void)
{
	GtkWidget *parent = NULL;
	GList *window_list = NULL, *node = NULL;
	gchar *window_name = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	window_list = gtk_window_list_toplevels();
	for (node = g_list_last(window_list); node != NULL; node = node->prev) {
		window_name = g_object_get_data(G_OBJECT(node->data), WINDOW_NAME);
		if (window_name != NULL && strcmp(window_name, APP_NAME) == 0) {
			parent = GTK_WIDGET(node->data);
			break;
		}
	}
	g_list_free(window_list);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return parent;
}

/**
  Helper function to show a confirmation note to the user, which can be
  of two types - Info_Banner with single button or Confirm_Dialog with 
  two buttons.

  In case of Confirm_Dialog -->
  If both button texts are NULL note with "<YES> <NO>" buttons is shown
  If second button text is NULL note with "<first_button> <CANCEL>" buttons is shown
  Otherwise note with "<first_button> <second_button>" buttons is shown.

  In case of Info_Banner <second_button> is redundant, and -->
  If first_button is NULL banner with <OK> button is shown
  Otherwise banner with <first_button> is shown.

  @param  confirmationNote  String to be displayed in the note
  @param  first_button      Name of the first button (in Confirm_Dialog)
  @param  second_button     Name of the second button (in Confirm_Dialog)
  @param  confirmationType  Info_Banner / Confirm_Dialog
  @return boolean           TRUE --> positive confirmation, FALSE --> otherwise
*/
gboolean
get_user_confirmation(const gchar * confirmationNote,
		      const gchar * first_button, const gchar * second_button,
		      gint confirmationType)
{
	gboolean response = TRUE;
	HildonNote *note = NULL;
	GtkWidget *parent = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	if (confirmationNote == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return FALSE;
	}

	parent = ui_get_top_level_window();
	if (parent == NULL) {
		parent = GTK_WIDGET(app);
	}

	switch (confirmationType) {
	case INFO_BANNER:
		note =
		    HILDON_NOTE(hildon_note_new_information(GTK_WINDOW(parent), confirmationNote));
		if (first_button != NULL) {
			hildon_note_set_button_text(note, first_button);
		}
		break;

	case CONFIRM_DIALOG:
		if (first_button != NULL && second_button != NULL) {
			note =
			    HILDON_NOTE(hildon_note_new_confirmation_add_buttons
					(GTK_WINDOW(parent), confirmationNote, first_button,
					 GTK_RESPONSE_ACCEPT, second_button,
					 GTK_RESPONSE_CANCEL, NULL));
		} else {
			note =
			    HILDON_NOTE(hildon_note_new_confirmation
					(GTK_WINDOW(parent), confirmationNote));
			if (first_button != NULL) {
				hildon_note_set_button_text(note, first_button);
			}
		}

		break;

	}

	GTK_WIDGET_SET_FLAGS(note, GTK_CAN_FOCUS);
	GTK_WIDGET_SET_FLAGS(note, GTK_CAN_DEFAULT);
	GTK_WIDGET_SET_FLAGS(note, GTK_RECEIVES_DEFAULT);
	gtk_window_set_transient_for(GTK_WINDOW(note), GTK_WINDOW(parent));
	/*Set application name to the dialog */
	g_object_set_data_full(G_OBJECT(note), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	g_signal_connect(G_OBJECT(note), "key-press-event",
			 G_CALLBACK(ui_on_escape_key_pressed), NULL);

	response = gtk_dialog_run(GTK_DIALOG(note));
	gtk_widget_destroy(GTK_WIDGET(note));
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	if (response == GTK_RESPONSE_CANCEL || response == GTK_RESPONSE_DELETE_EVENT) {
		return FALSE;
	} else {
		return TRUE;
	}
}

/**
  Helper function to free PrefsAccount object.

  @param  ac_prefs  Pointer to PrefsAccount object
  @return void
*/
void ui_free_prefs_account_structure(PrefsAccount * ac_prefs)
{
	ULOG_DEBUG_L("[%s]  starts here...\n", __FUNCTION__);
	if (ac_prefs == NULL) {
		ULOG_DEBUG_F("Account can't be freed ...\n");
		return;
	}
	g_free(ac_prefs->user_settings.account_name);
	g_free(ac_prefs->user_settings.userid);
	g_free(ac_prefs->user_settings.passwd);
	g_free(ac_prefs->user_settings.email);
	g_free(ac_prefs->user_settings.alias_name);

	g_free(ac_prefs->receive_settings.recv_server);
	g_free(ac_prefs->send_settings.smtp_server);
	g_free(ac_prefs->send_settings.username);
	g_free(ac_prefs->send_settings.password);

	g_free(ac_prefs->advanced_settings.signature_file);
	g_free(ac_prefs->advanced_settings.imap_dir);
	g_free(ac_prefs);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
  Helper function to get the name of the account with which given
  message was downloaded.
  Message-id format is  "<account>_<folderpath>_<some unique num>"

  @param  msg_id  Message-id string
  @return gchar*  Account name string, to be freed by calling function.
*/
gchar *ui_get_account_name(const gchar * msg_id)
{
	gchar *temp_str = NULL;
	gchar *account_name = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msg_id == NULL), "Input param check failed", return NULL);
	temp_str = strchr(msg_id, MSGID_SEPARATOR);
	if (temp_str != NULL && (strlen(msg_id) - strlen(temp_str) > 0)) {
		account_name = g_strndup(msg_id, strlen(msg_id) - strlen(temp_str));
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return account_name;
}

/**
  Helper function to NULL check two strings.

  @param  str1  First string
  @param  str2  Second string
  @return gint  0 --> Both are NULL
  1 --> Both are not NULL
  -1 --> Only one of them is NULL
*/
gint ui_null_check_strings(const gchar * str1, const gchar * str2)
{
	gint ret_val = -1;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	if (str1 == NULL && str2 == NULL) {
		ret_val = 0;
	} else if (str1 != NULL && str2 != NULL) {
		ret_val = 1;
	} else {
		ret_val = -1;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return ret_val;
}

/**
  Helper function to parse out parent folder and current folder names
  from message-id string.
  Message-id format is  "<account>_<folderpath>_<some unique num>"
  where <folderpath> format is "<parent folder>_<current folder>"

  @param  msg_id        Message-id string
  @param  parent_folder Pointer for storing parent folder string (out param)
  @param  folder_name   Pointer for storing current folder string (out param)
  @return boolean       TRUE --> parsing success, FALSE --> failure
*/
gboolean ui_get_msg_folder_names(const gchar * msg_id, gchar ** parent_folder, gchar ** folder_name)
{
	gboolean success = TRUE;
	gchar **str_array = NULL;
	guint array_length = 0, i = 0;
	gchar *tmp_str = NULL;
	gchar *under_score_str = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msg_id == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((parent_folder == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((folder_name == NULL), "Input param check failed", return FALSE);

	under_score_str = g_strdup_printf("%c", MSGID_SEPARATOR);
	str_array = g_strsplit(msg_id, under_score_str, MAX_TOKENS);

	tmp_str = str_array[i];
	while (tmp_str != NULL) {
		array_length++;
		i++;
		tmp_str = str_array[i];
	}

	if (array_length != MAX_TOKENS_IN_MSGID && array_length != MAX_TOKENS_IN_MSGID + 1) {
		ULOG_DEBUG_F("Message id parsing failed\n");
		return FALSE;
	}
	//For messages in root level folders, there can be only 2 '_' chars in msg-id
	//For sub-folder messages, one addition '_' char will be present
	if (array_length == MAX_TOKENS_IN_MSGID) {
		//Message is in root level folder
		//No parent folder, folder name is in 2nd token
		*parent_folder = NULL;
		*folder_name = g_strdup(str_array[1]);
	} else if (array_length == MAX_TOKENS_IN_MSGID + 1) {
		//Message is in sub level folder
		//Parent folder is in 2nd token, folder name is in 3rd
		*parent_folder = g_strdup(str_array[1]);
		*folder_name = g_strdup(str_array[2]);
	} else {
		ULOG_DEBUG_F("Message id parsing failed\n");
		success = FALSE;
	}

	g_strfreev(str_array);
	g_free(under_score_str);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return success;
}

/**
  Helper function to get the config directory path. One example is
  "$HOME/apps/email"

  @param  void
  @return gchar* String containing config dir path (not to be freed)
*/
gchar *ui_get_config_dir(void)
{
	gchar *config_dir = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	config_dir = g_strdup_printf("%s", CONFIG_FILE_PATH);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return config_dir;
}

/**
  This is helper function to create text entry with label and internally it will
  be added to a caption control.
  @param label The label name to be set to caption control which is in 
  logical name format.
  @param size_group The size group of caption control.
  @param maxlen This is the maximum lenth of field value.
  @param caption_status The status of the caption control.
  @param field_value The field value to be set.
  @param field_number The field number to be set to current field.
  @param unset_autocap Whether to unset the AUTOCAP property for the field.
  @return Pointer to the newly created caption control info object.
*/
CaptionControlInfo *ui_create_text_entry(gchar * label, GtkSizeGroup * size_group,
					 gint maxlen, gboolean caption_status,
					 gint field_value, gint field_number,
					 gboolean unset_autocap)
{
	GtkWidget *field = NULL;
	GtkWidget *caption_control = NULL;
	CaptionControlInfo *cptn_ctrl_info = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM((label == NULL), "Input param check failed", return NULL);
	CHECK_PARAM((size_group == NULL), "Input param check failed", return NULL);

	/* Creating the entry field */
	field = gtk_entry_new();
	if (unset_autocap) {
		hildon_gtk_entry_set_input_mode(GTK_ENTRY(field), HILDON_GTK_INPUT_MODE_FULL);

	}
	gtk_entry_set_max_length(GTK_ENTRY(field), maxlen);

	/* creating the caption control to the entry widget. */
	caption_control =
	    hildon_caption_new(size_group, label, field, NULL, HILDON_CAPTION_OPTIONAL);

	/* Creating the caption control info structure */
	cptn_ctrl_info = g_new0(CaptionControlInfo, 1);
	if (cptn_ctrl_info == NULL) {
		ULOG_CRIT_F("Memory allocation failure!\n");
		return NULL;
	}

	/* Initialize the structure */
	cptn_ctrl_info->caption_control = NULL;

	/* Fill the structure with the appropriate values */
	cptn_ctrl_info->is_mandatory = caption_status;
	cptn_ctrl_info->field_value = field_value;
	cptn_ctrl_info->field_number = field_number;
	if (caption_status == HILDON_CAPTION_MANDATORY) {
		cptn_ctrl_info->is_shown = TRUE;
	} else {
		cptn_ctrl_info->is_shown = FALSE;
	}
	cptn_ctrl_info->caption_control = caption_control;

	/* Setting the structure to widget */
	g_object_set_data_full(G_OBJECT(field), CAPTION_CONTROL_INFO, cptn_ctrl_info, g_free);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return cptn_ctrl_info;
}

/**
  This is helper function to get the data in conatct field entries.
  If entry is null it will set a dummy val "" in that field.
  @param entry This is Widget pointer from where the field value to be retireved.
  @param field This is the gchar pointer where the field value to be stored.
  @return Nothing.
*/
void ui_get_contact_field_entry(GtkWidget * entry, gchar ** field)
{
	const gchar *buf = NULL;

	ULOG_DEBUG("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);

	if (NULL == entry) {
		return;
	}

	buf = gtk_entry_get_text(GTK_ENTRY(entry));
	if (*field != NULL) {
		g_free(*field);
		*field = NULL;
	}
	if (NULL != buf) {
		*field = g_strndup(buf, (gsize) strlen(buf));
	} else {
		*field = g_strdup(DUMMY_VAL);
	}

	ULOG_DEBUG("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
  This is helper function to set the data in conatct field entries.
  It validates the whether its utf8 encoded string, if not converts to utf8.

  @param entry This is Widget pointer from where the field value to be retireved.
  @param data This is the gchar pointer where the field value to be stored.
  @return Nothing.
*/
void ui_set_contact_field_entry(GtkWidget * entry, const gchar * data)
{
	GError *err = NULL;
	gchar *converted_txt = NULL;
	gint bytes_written = 0;

	ULOG_DEBUG("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);


	if (g_utf8_validate(data, -1, NULL) == FALSE) {
		converted_txt = g_convert(data, strlen(data), "UTF-8", DEFAULT_CHARSET,
					  &bytes_written, NULL, &err);
		if (err) {
			ULOG_DEBUG_F("Character conversion failed with error '%s'", err->message);
			g_error_free(err);
		} else {
			gtk_entry_set_text(GTK_ENTRY(entry), converted_txt);
			g_free(converted_txt);
		}

	} else
		gtk_entry_set_text(GTK_ENTRY(entry), data);

	ULOG_DEBUG("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
  This is helper function to set the search string to list. Internally it will 
  search for the string to be set in the list whether it is already added.
  The newly string is added to first of the list.
  @param search_string The searched string to add to list.
  @return  TRUE If the searched string is found. Otherwise
	  FALSE.
*/
gboolean ui_add_contact_search_string_to_list(gchar * search_string)
{
	gint str_len = 0;
	GList *temp_list = NULL;
	gboolean string_found = TRUE;
	guint no_of_elements = 0;
	GList *node = NULL;
	gchar *search_name = NULL;

	ULOG_DEBUG("[%s][%s]:starts from here :\n", __FILE__, __FUNCTION__);

	if (search_string == NULL) {
		ULOG_DEBUG("[%s][%s]: Input param failed:\n", __FILE__, __FUNCTION__);
		return FALSE;
	}

	search_string = g_strstrip(search_string);
	str_len = (gint) strlen(search_string);
	if (str_len != 0) {
		temp_list = contact_search_list;
		while (temp_list != NULL) {
			if (0 == strncmp(search_string, temp_list->data, strlen(search_string))) {
				string_found = FALSE;
				break;
			}
			temp_list = g_list_next(temp_list);
		}
		if (string_found == TRUE) {
			search_name = g_strdup(search_string);
			contact_search_list =
			    g_list_prepend(contact_search_list, (gpointer) search_name);
			no_of_elements = g_list_length(contact_search_list);
			if (no_of_elements > 5) {
				node = g_list_nth(contact_search_list, 5);
				if (node->data != NULL) {
					g_free(node->data);
				}
				contact_search_list = g_list_delete_link(contact_search_list, node);
			}
		}
	}
	ULOG_DEBUG("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return string_found;
}

/**
  This function will make a copy of the current contact searched list and 
  return.
  @param Nothing.
  @return GSList conatains of searched strings. 
*/
GList *ui_get_contact_search_list(void)
{
	GList *temp_list = NULL;
	ULOG_DEBUG("[%s][%s]:starts from here :\n", __FILE__, __FUNCTION__);

	if (contact_search_list != NULL) {
		temp_list = g_list_copy(contact_search_list);
		ULOG_DEBUG("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
		return temp_list;
	}

	ULOG_DEBUG("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return NULL;
}

/**
  This function will free the conatct searched list and its internal data.
  @param Nothing.
  @return Nothing
*/
void ui_free_contact_search_list(void)
{
	GList *node = NULL;
	ULOG_DEBUG("[%s][%s]:starts from here :\n", __FILE__, __FUNCTION__);

	if (contact_search_list == NULL) {
		ULOG_DEBUG("[%s][%s]:contact search list is free :\n", __FILE__, __FUNCTION__);
		return;
	}

	for (node = contact_search_list; node != NULL; node = node->next) {
		if (node->data != NULL) {
			g_free(node->data);
		}
	}
	g_list_free(contact_search_list);

	ULOG_DEBUG("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);

	return;
}

/**
 This is called to check whether the string under observation
 is already resolved.

 @param start is start position for the address.

 @ return gboolean depending upon the presence of contact tag.
 */
static gboolean check_for_contacttag(GtkTextIter start)
{
	GSList *tag_list = NULL;

	tag_list = gtk_text_iter_get_tags(&start);
	if (tag_list) {
		g_slist_free(tag_list);
		return TRUE;
	} else {
		return FALSE;
	}
}

/**
 This is called to remove extra colons and semi colons.

 @param addressBuffer is recipient field.
 @param start_off is start position to look for separators.
 
 @ return void.
 */
static void remove_address_separators(GtkTextBuffer * addressBuffer, gint start_off)
{
	GtkTextIter start, temp_iter;
	gchar *sep_check = NULL;

	if (addressBuffer == NULL) {
		osso_log(LOG_DEBUG, "[%s][%s] : invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}

	osso_log(LOG_DEBUG, "[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	gtk_text_buffer_get_iter_at_offset(addressBuffer, &start, start_off);
	temp_iter = start;
	gtk_text_iter_forward_char(&temp_iter);
	sep_check = gtk_text_buffer_get_text(addressBuffer, &start, &temp_iter, FALSE);
	while (sep_check != NULL) {
		if ((strcmp(sep_check, ADDR_SEPERATOR_CHAR) == 0)
		    || (strcmp(sep_check, ATTACHMENT_SEPERATOR_CHAR) == 0)
		    || (strcmp(sep_check, HTML_NL) == 0)
		    || (strcmp(sep_check, SPACE) == 0)) {
			gtk_text_buffer_delete(addressBuffer, &start, &temp_iter);
			gtk_text_buffer_get_iter_at_offset(addressBuffer, &start, start_off);
			temp_iter = start;
			gtk_text_iter_forward_char(&temp_iter);
			sep_check = gtk_text_buffer_get_text(addressBuffer,
							     &start, &temp_iter, FALSE);
		} else {
			break;
		}
	}
	osso_log(LOG_DEBUG, "[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	return;

}

/**
 This is called to validate a manually entred recipient address.

 @param addressBuffer is recipient field.
 @param start is start iter for an address.
 @param end is end iter for an address.

 @ return gboolean depending upon the validity of address.
*/
static gboolean
validate_recipient_addr(GtkTextBuffer * addressBuffer,
			GtkTextIter start, GtkTextIter end, gboolean * valid_email)
{
	gchar *address = NULL;
	GtkTextIter l_brace_start, l_brace_end, r_brace_start, r_brace_end, *end_iter = NULL;

	if (addressBuffer == NULL) {
		osso_log(LOG_DEBUG, "[%s][%s] : invalid input :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}

	osso_log(LOG_DEBUG, "[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	address = gtk_text_buffer_get_text(addressBuffer, &start, &end, FALSE);
	if (ui_accmgmt_email_isvalid(address) == TRUE) {
		g_free(address);
		return TRUE;
	}
	g_free(address);
	address = NULL;

	/*">" is the end char the forward seach is retuning false,
	 * so if its end iter we search entire buffer
	 * internal bug fix (650) */
	if (gtk_text_iter_is_end(&end) != TRUE)
		end_iter = &end;

	if ((gtk_text_iter_forward_search(&start, HTML_LT,
					  GTK_TEXT_SEARCH_VISIBLE_ONLY,
					  &l_brace_start, &l_brace_end,
					  end_iter) == TRUE) &&
	    (gtk_text_iter_forward_search
	     (&start, HTML_GT, GTK_TEXT_SEARCH_VISIBLE_ONLY, &r_brace_start,
	      &r_brace_end, end_iter) == TRUE)) {
		gtk_text_iter_forward_char(&l_brace_end);

		/*checking for case where we have a email is as "aaa<xyz@oo.com>qqq" */
		if (gtk_text_iter_compare(&end, &r_brace_end) == 1)
			return FALSE;

		address = gtk_text_buffer_get_text(addressBuffer, &l_brace_end,
						   &r_brace_start, FALSE);
		if (ui_accmgmt_email_isvalid(address) == TRUE) {
			g_free(address);
			return TRUE;
		} else {
			g_free(address);
			*valid_email = FALSE;
			return FALSE;
		}

	}
	return FALSE;
}

/*
 This function inserts the resolved address separated by "," with the tags.
 It also sets the resolved EContact UID as 'econtact-uid' property to the
 newly created tag.

 @param reslovedid  Resolved email address to be inserted into GtkTextBuffer
 @param addressBuffer Pointer to the GtkTextBuffer
 @param start_iter  Start position to be inserted from
 @param uid  EContact UID string

 @return void
*/
static void
insert_contact_with_tag(gchar ** reslovedid, GtkTextBuffer * addressBuffer, GtkTextIter start_iter,
			const gchar * uid)
{
	gchar **split_address = NULL;
	GtkTextIter end, start;
	gint i = 0, offset = 0;
	GString *id = NULL;
	GtkTextTag *tagcontact = NULL;
	CHECK_PARAM(*reslovedid == NULL, "param failed", return);

	tagcontact =
		gtk_text_buffer_create_tag(addressBuffer, NULL, "underline",
					   PANGO_UNDERLINE_SINGLE, "editable", TRUE, NULL);
	g_object_set_data(G_OBJECT(tagcontact), "emailid", GINT_TO_POINTER(EMAIL_ID));
	g_object_set_data_full(G_OBJECT(tagcontact), "econtact-uid",
			       g_strdup(uid), (GDestroyNotify) g_free);

	start = start_iter;
	offset = gtk_text_iter_get_offset(&start);
	split_address = g_strsplit(*reslovedid, ADDR_SEPERATOR_CHAR, -1);
	if (split_address == NULL) {
		gtk_text_buffer_insert_with_tags(addressBuffer, &start,
						 *reslovedid, -1, tagcontact, NULL);
		return;
	}
	id = g_string_new(NULL);
	g_free(*reslovedid);
	*reslovedid = NULL;
	while (split_address[i] != NULL) {
		gtk_text_buffer_insert_with_tags(addressBuffer, &start,
						 split_address[i], -1, tagcontact, NULL);
		g_string_append(id, split_address[i]);
		if (split_address[i + 1] != NULL) {
			offset = offset + g_utf8_strlen(split_address[i], -1);
			gtk_text_buffer_get_iter_at_offset(addressBuffer, &end, offset);
			gtk_text_buffer_insert(addressBuffer, &end, ", ", -1);
			offset += 2;
			gtk_text_buffer_get_iter_at_offset(addressBuffer, &end, offset);
			g_string_append(id, ", ");
		}
		start = end;
		i++;
	}
	g_strfreev(split_address);
	*reslovedid = g_string_free(id, FALSE);
	return;
}

/**
This is called to update progress banner.

@ return void.
*/
void add_progress_fraction()
{
	static gdouble fraction = 0;

	osso_log(LOG_DEBUG, "[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	while (gtk_events_pending()) {
		gtk_main_iteration();
	}
	if (fraction > 0.95) {
		fraction = 0;
	} else {
		fraction += 0.1;
	}
	gtk_banner_set_fraction((GtkWindow *) app, fraction);

	osso_log(LOG_DEBUG, "[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 function to get the end iterator of the email id.

 @param start is the pointer to the GtkTextIter from where the search starts.
 @param end is the pointer to the GtkTextIter OUT param.
 @param addressBuffer is the pointer to the GtkTextBuffer in which the emailid is present.
*/
void get_end_iter_for_emailid(GtkTextIter * start, GtkTextIter * end, GtkTextBuffer * addressBuffer)
{
	GtkTextIter semicostart, semicoend, commastart, commaend;
	gboolean comma_present = FALSE, semico_present = FALSE;
	gint value = 0;

	comma_present = gtk_text_iter_forward_search(start, ADDR_SEPERATOR_CHAR,
						     GTK_TEXT_SEARCH_VISIBLE_ONLY,
						     &commastart, &commaend, NULL);
	semico_present = gtk_text_iter_forward_search(start, ATTACHMENT_SEPERATOR_CHAR,
						      GTK_TEXT_SEARCH_VISIBLE_ONLY,
						      &semicostart, &semicoend, NULL);
	if (semico_present == TRUE && comma_present == TRUE) {
		value = gtk_text_iter_compare(&semicostart, &commastart);
		switch (value) {
		case -1:
			*end = semicostart;
			break;
		case 1:
			*end = commastart;
			break;
		case 0:
			*end = semicostart;
		}
	} else if (comma_present) {
		*end = commastart;
	} else if (semico_present) {
		*end = semicostart;
	} else {
		gtk_text_buffer_get_end_iter(addressBuffer, end);
	}
}


/**
 This is a callback function for doing checknames.

 @param win the widget that registered the callback.
 @param check_names_list is the gslist containing the address buffers for to/cc/bcc.
 @ return void.
 */
void ui_editor_do_check_names(GtkWidget * win, gpointer check_names_list)
{
	GtkTextBuffer *addressBuffer = NULL;
	GSList *check_names_list_local = NULL, *node = NULL;
	GtkTextIter start, end;
	gchar *emailid = NULL, *reslovedid = NULL;
	GtkTextTag *tagcontact = NULL;
	gboolean tagpresent = FALSE;
	check_names_list_local = (GSList *) check_names_list;
	gint start_off = 0;
	gint end_off = 0;
	gint id_len = 0;
	gchar *substring = NULL, *econtact_uid = NULL;
	gboolean valid_emailid = TRUE;

	if (check_names_list == NULL) {
		osso_log(LOG_DEBUG, "[%s][%s] : invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}
	osso_log(LOG_DEBUG, "[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (!open_addressbook()) {
		ULOG_CRIT("Unable to open addressbook");
		osso_log(LOG_CRIT, "Unable to open addressbook");
		gtk_infoprint(GTK_WINDOW(app), _("mail_ib_contacts_file_access_permission"));
		return;
	}

	if (win != NULL) {
		gtk_banner_close((GtkWindow *) app);
		gtk_banner_show_bar((GtkWindow *) app, _("mail_ib_checking_names"));
	}

	while (check_names_list_local != NULL) {
		node = check_names_list_local;
		addressBuffer = (GtkTextBuffer *) node->data;
		tagcontact =
		    gtk_text_buffer_create_tag(addressBuffer, NULL, "underline",
					       PANGO_UNDERLINE_SINGLE, "editable", TRUE, NULL);
		g_object_set_data(G_OBJECT(tagcontact), "emailid", GINT_TO_POINTER(EMAIL_ID));

		/*search in the loop for the separated strings & do check names */
		gtk_text_buffer_get_start_iter(addressBuffer, &start);
		while (gtk_text_iter_is_end(&start) == FALSE) {

			/*get the end iter, i.e where the emailid ends */
			get_end_iter_for_emailid(&start, &end, addressBuffer);

			/*check if the recipient is already resolved */
			tagpresent = check_for_contacttag(start);
			if (tagpresent == TRUE) {
				start = end;
				start_off = gtk_text_iter_get_offset(&start) + 1;
				gtk_text_buffer_get_iter_at_offset(addressBuffer, &start,
								   start_off);
				/*Some address may contain "," but form a part of a resolved address
				 * ex (xyz, abc <abc@www.com>).So we increment the iter one position & check
				 * if the tag is present in the next char. If so, we continue .
				 * else we insert a space after the resolved id .*/
				if (check_for_contacttag(start))
					continue;

				gtk_text_buffer_insert(addressBuffer, &start, SPACE, -1);
				start_off++;
				remove_address_separators(addressBuffer, start_off);
				gtk_text_buffer_get_iter_at_offset(addressBuffer, &start,
								   start_off);
				continue;
			} else {
				emailid =
				    gtk_text_buffer_get_text(addressBuffer, &start, &end, FALSE);
				if (strcmp(emailid, HTML_NL) == 0 || strlen(emailid) == 0) {
					start_off = gtk_text_iter_get_offset(&start);
					gtk_text_buffer_delete(addressBuffer, &start, &end);
					gtk_text_buffer_get_iter_at_offset(addressBuffer, &start,
									   start_off);
					remove_address_separators(addressBuffer, start_off);
					gtk_text_buffer_get_iter_at_offset(addressBuffer, &start,
									   start_off);
					g_free(emailid);
					continue;
				}

				reslovedid = get_resolved_contacts(emailid, &econtact_uid);
				/* insert the resolved recipientwith a tag & remove the name */
				if (reslovedid != NULL) {
					start_off = gtk_text_iter_get_offset(&start);
					end_off = gtk_text_iter_get_offset(&end);
					gtk_text_buffer_delete(addressBuffer, &start, &end);
					gtk_text_buffer_get_iter_at_offset(addressBuffer, &start,
									   start_off);
					insert_contact_with_tag(&reslovedid, addressBuffer, start,
								econtact_uid);
					id_len = g_utf8_strlen(reslovedid, -1);
					start_off = start_off + id_len + 1;
					gtk_text_buffer_get_iter_at_offset(addressBuffer, &start,
									   start_off);
					gtk_text_buffer_insert(addressBuffer, &start, SPACE, -1);
					start_off++;
					remove_address_separators(addressBuffer, start_off);
					gtk_text_buffer_get_iter_at_offset(addressBuffer, &start,
									   start_off);
					g_free(reslovedid);
					g_free(emailid);
				}
				/*popup the error msg if unable to resolve th recipient */
				else {
					g_free(emailid);
					if (validate_recipient_addr
					    (addressBuffer, start, end, &valid_emailid)
					    == TRUE || (win == NULL && valid_emailid == TRUE)) {
						gtk_text_buffer_apply_tag(addressBuffer, tagcontact,
									  &start, &end);
						start = end;
						gtk_text_iter_forward_char(&start);
						start_off = gtk_text_iter_get_offset(&start);
						remove_address_separators(addressBuffer, start_off);
						gtk_text_buffer_get_iter_at_offset(addressBuffer,
										   &start,
										   start_off);
					} else {
						if (win == NULL)
							return;
						gtk_banner_close((GtkWindow *) app);
						gtk_text_buffer_select_range(addressBuffer, &start,
									     &end);
						substring = strchr(emailid, AT_RATE);
						if (valid_emailid == TRUE && substring == NULL) {
							gtk_infoprint((GtkWindow *) app,
								      _
								      ("mcen_nc_no_matching_contacts"));
							email_main_view->tags->
							    check_name_unresolved = TRUE;
						} else {
							gtk_infoprint((GtkWindow *) app,
								      _("mcen_ib_not_valid_email"));
							email_main_view->tags->
							    check_name_unresolved = FALSE;
						}
						return;
					}
				}
			}
			if (win != NULL)
				add_progress_fraction();
		}
		insert_separator_at_end(addressBuffer);
		check_names_list_local = check_names_list_local->next;
	}
	stop_addressbook();

	if (win != NULL)
		gtk_banner_close((GtkWindow *) app);
	osso_log(LOG_DEBUG, "[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is a called to insert a comma at the end in a recipient field.

 @param addressBuffer is recipient field.
 @ return void.
 */
static void insert_separator_at_end(GtkTextBuffer * addressBuffer)
{
	GtkTextIter start, end;
	gchar *text = NULL;

	if (addressBuffer == NULL) {
		osso_log(LOG_DEBUG, "[%s][%s] : invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}
	osso_log(LOG_DEBUG, "[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	gtk_text_buffer_get_end_iter(addressBuffer, &end);
	start = end;
	gtk_text_iter_backward_char(&start);
	text = gtk_text_buffer_get_text(addressBuffer, &start, &end, FALSE);
	if (strlen(text) != 0) {
		if ((strcmp(text, ADDR_SEPERATOR_CHAR) == 0)
		    || (strcmp(text, ATTACHMENT_SEPERATOR_CHAR) == 0)) {
			g_free(text);
			return;
		}
		if (strcmp(text, SPACE) == 0) {
			gtk_text_buffer_delete(addressBuffer, &start, &end);
			g_free(text);
			return;
		}
	}
	g_free(text);
	osso_log(LOG_DEBUG, "[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This is a callback function to invoke address book if recipient field 
   is empty while doing check names
 @param win the widget that registered the callback.
 @param check_names_list is the gslist containing the address buffers for to/cc/bcc.
 @ return void.
 */
void recipient_field_empty_check(GtkWidget * win, EditorInfo * editor_info)
{
	GSList *node = NULL, *check_names_list = NULL;
	GtkTextBuffer *tobuf = NULL, *ccbuf = NULL, *bccbuf = NULL;
	GtkTextIter start, end;
	gchar *textto = NULL, *textcc = NULL, *textbcc = NULL;

	check_names_list = editor_info->addressBuffers;
	if (check_names_list == NULL) {
		osso_log(LOG_DEBUG, "[%s][%s] : invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}
	osso_log(LOG_DEBUG, "[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	node = check_names_list;
	tobuf = g_slist_nth_data(check_names_list, 0);
	ccbuf = g_slist_nth_data(check_names_list, 1);
	bccbuf = g_slist_nth_data(check_names_list, 2);

	gtk_text_buffer_get_bounds(tobuf, &start, &end);
	textto = gtk_text_buffer_get_text(tobuf, &start, &end, FALSE);
	gtk_text_buffer_get_bounds(ccbuf, &start, &end);
	textcc = gtk_text_buffer_get_text(ccbuf, &start, &end, FALSE);
	gtk_text_buffer_get_bounds(bccbuf, &start, &end);
	textbcc = gtk_text_buffer_get_text(bccbuf, &start, &end, FALSE);
	if (strlen(textto) == 0 && strlen(textcc) == 0 && strlen(textbcc) == 0) {
		gtk_widget_grab_focus(editor_info->msgFields->to);
		ui_editor_openAddrBook_cb(NULL, NULL, tobuf);
	}
	g_free(textto);
	g_free(textcc);
	g_free(textbcc);
	osso_log(LOG_DEBUG, "[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This is callback function for help. This function is responsible to get help
 any dialog boxes in the whole application. Depending on the help id help is
 shown for that perticular dialog.
 @param win Widget pointer on which this function is called.
 @param help_id  Depending on the id help is shown.
 @return Nothing.
*/
void ui_on_dialog_help_activate(GtkWidget * win, gchar * help_id)
{
	osso_return_t retval = OSSO_ERROR;
	osso_log(LOG_DEBUG, "[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (help_id == NULL) {
		osso_log(LOG_DEBUG, "[%s][%s] : invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}
	retval = ossohelp_show(ui_context, help_id, OSSO_HELP_SHOW_DIALOG);
	ULOG_DEBUG("ossohelp retval = %d, help_id = %s\n", retval, help_id);

	osso_log(LOG_DEBUG, "[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This is callback function for help. This function is responsible to get help
 any view in the whole application. Depending on the help id help is
 shown for that perticular view.
 @param win Widget pointer on which this function is called.
 @param help_id  Depending on the id help is shown.
 @return Nothing.
*/
void ui_on_view_help_activate(GtkWidget * win, gchar * help_id)
{
	osso_return_t retval = OSSO_ERROR;
	osso_log(LOG_DEBUG, "[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (help_id == NULL) {
		osso_log(LOG_DEBUG, "[%s][%s] : invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}

	/*If help does not exist for this ID, launch the application help view */
	if (ossohelp_exists(help_id) == OSSO_OK) {
		retval = ossohelp_show(ui_context, help_id, 0);
	} else {
		retval = ossohelp_show(ui_context, HELP_ID_DEFAULT, 0);
	}

	ULOG_DEBUG("ossohelp retval = %d, help_id = %s\n", retval, help_id);

	osso_log(LOG_DEBUG, "[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This is a helper function to prepare the destination message_id string
 given the original message_id and the destination folder path.
 Message-id format is  "<account>_<folderpath>_<some unique num>"
 where <folderpath> format is "<parent folder>_<current folder>"

 @param  orig_msg_id       original message_id string
 @param  dest_folder_path  destination folder path
                          (format : "<parent folder>_<current folder>")
 @return gchar *           destination message_id string
*/
static gchar *get_dest_msg_id(const gchar * orig_msg_id, const gchar * dest_folder_path)
{
	gchar **msg_id_tokens = NULL;
	gchar **folder_tokens = NULL;
	gchar *account_name = NULL, *msg_id_num = NULL;
	gchar *dest_folder_parent = NULL, *dest_folder_name = NULL;
	guint msg_id_tokens_len = 0, folder_tokens_len = 0;
	gchar *dest_msg_id = NULL;
	gchar *under_score_str = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((orig_msg_id == NULL), "Input param check failed", return NULL);
	CHECK_PARAM((dest_folder_path == NULL), "Input param check failed", return NULL);

	under_score_str = g_strdup_printf("%c", MSGID_SEPARATOR);
	msg_id_tokens = g_strsplit(orig_msg_id, under_score_str, MAX_TOKENS_IN_MSGID + 1);
	folder_tokens = g_strsplit(dest_folder_path, "/", -1);
	msg_id_tokens_len = g_strv_length(msg_id_tokens);
	folder_tokens_len = g_strv_length(folder_tokens);

	//Do sanity checks
	if (msg_id_tokens_len < MAX_TOKENS_IN_MSGID || folder_tokens_len > 2) {
		g_strfreev(msg_id_tokens);
		g_strfreev(folder_tokens);
		ULOG_DEBUG_F("Parsing failed");
		return NULL;
	}

	account_name = msg_id_tokens[0];
	if (msg_id_tokens[MAX_TOKENS_IN_MSGID] != NULL) {
		msg_id_num = msg_id_tokens[MAX_TOKENS_IN_MSGID];
	} else {
		msg_id_num = msg_id_tokens[MAX_TOKENS_IN_MSGID - 1];
	}

	if (folder_tokens_len == 2) {
		dest_folder_parent = folder_tokens[0];
		dest_folder_name = folder_tokens[1];
	} else {
		dest_folder_parent = NULL;
		dest_folder_name = folder_tokens[0];
	}

	if (dest_folder_parent != NULL) {
		dest_msg_id =
		    g_strjoin(under_score_str, account_name, dest_folder_parent, dest_folder_name,
			      msg_id_num, NULL);
	} else {
		dest_msg_id =
		    g_strjoin(under_score_str, account_name, dest_folder_name, msg_id_num, NULL);
	}

	g_strfreev(msg_id_tokens);
	g_strfreev(folder_tokens);
	g_free(under_score_str);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return dest_msg_id;
}

/**
 This is a helper function to prepare list of destination message_id strings
 given the original message_id strings and the destination folder they have
 to be copied/moved to.
 @param  orig_msg_list     List of original message_id strings
 @param  dest_folder_path  Destination folder path
 @return GSList            List of destination message_id strings
*/
GSList *ui_prepare_dest_msg_ids(const GSList * orig_msg_list, const gchar * dest_folder_path)
{
	gchar *dest_msg_id = NULL;
	const GSList *node = NULL;
	GSList *dest_msg_list = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	for (node = orig_msg_list; node != NULL; node = node->next) {
		dest_msg_id = get_dest_msg_id(node->data, dest_folder_path);
		if (dest_msg_id != NULL) {
			dest_msg_list = g_slist_append(dest_msg_list, dest_msg_id);
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return dest_msg_list;
}

/**
 This is a helper function used to parse FROM 'address' from the format :
 "name<address>". Return value should be freed by the calling function.

 @ param  address         Pointer the the string to be parsed
 @return gchar*          Pointer to the parsed email-address string
*/
gchar *ui_parse_from_address(const gchar * address)
{
	gchar *from = NULL;
	gchar *char_location1 = NULL;
	gchar *char_location2 = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	CHECK_PARAM((address == NULL), "Input param check failed", return NULL);

	char_location1 = strchr(address, LESSER_THAN);
	CHECK_PARAM(char_location1 == NULL, "Input param check failed", return (g_strdup(address)));
	char_location2 = strchr(char_location1, GREATER_THAN);
	if (char_location1 == NULL || char_location2 == NULL
	    || strlen(char_location1) - strlen(char_location2) - 1 <= 0) {
		from = g_strdup(address);
	} else {
		from = g_strndup(char_location1 + 1,
				 (guint) (strlen(char_location1) - strlen(char_location2) - 1));
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return from;
}

/**
 This is a helper function to compare UTF8 strings. Input strings are
 made case insensitive first by using 'g_utf8_casefold' and then compared
 with 'g_utf8_collate'.

 @param  str1  First string to be compared
 @param  str2  Second string to be compared
 @return gint  < 0 if str1 compares before str2,
              0 if they compare equal,
              > 0 if str1 compares after str2.
*/
gint ui_compare_utf8_strings(const gchar * str1, const gchar * str2)
{
	gchar *tmp_str1 = NULL, *tmp_str2 = NULL;
	gint ret_val = -1;

	CHECK_PARAM((str1 == NULL), "Input param check failed", return -1);
	CHECK_PARAM((str2 == NULL), "Input param check failed", return -1);

	tmp_str1 = g_utf8_casefold(str1, -1);
	tmp_str2 = g_utf8_casefold(str2, -1);

	ret_val = g_utf8_collate(tmp_str1, tmp_str2);

	g_free(tmp_str1);
	g_free(tmp_str2);

	return ret_val;
}

/**
  This is signal handler function which will be invoked on escape key pressed for
  the topmost dialog.
  @param widget Pointer to widget on which it is called.
  @param event Pointer to key event.
  @param data Pointer to user data.
  @return If key event is escape key it return TRUE Otherwise
			 FALSE.
*/
gboolean ui_on_escape_key_pressed(GtkWidget * widget, GdkEventKey * event, gpointer data)
{
	GtkWidget *window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	CHECK_PARAM((widget == NULL), "Input param check failed", return FALSE);

	window = (GtkWidget *) data;

	if (event->keyval == GDK_Escape) {
		if (fired_on_note == TRUE) {
			fired_on_note = FALSE;
			return TRUE;
		}
		if (email_main_view->account_manager->esc_sgnl_handled == TRUE) {
			email_main_view->account_manager->esc_sgnl_handled = FALSE;
			return TRUE;
		}
		if (data != NULL) {
			window = (GtkWidget *) data;
			gtk_button_clicked(GTK_BUTTON(window));
		} else {
			fired_on_note = TRUE;
			return FALSE;
		}
		ULOG_DEBUG_L("[%s] : Ends here :\n", __FUNCTION__);
		return TRUE;
	}
	ULOG_DEBUG_L("[%s] : Ends here :\n", __FUNCTION__);
	return FALSE;
}

/**
 *  This is an utility function. This will be used to get the device name .
 *   
 *  @param void
 *  @param It will return the device name. The returned string must  be freed.
 */
gchar *ui_get_device_name()
{
	gchar *deviceName = NULL;
	const gchar *btname = NULL;
	DBusMessage *request_message = NULL;
	DBusMessage *reply_message = NULL;
	DBusError error;
	DBusConnection *conn = NULL;
	DBusMessageIter iter;
	dbus_uint32_t reason = 0;
	dbus_bool_t rv = FALSE;

	dbus_error_init(&error);
	conn = (DBusConnection *) osso_get_sys_dbus_connection(ui_context);

	rv = dbus_bus_start_service_by_name(conn, "com.nokia.btname", 0, &reason, &error);
	if (rv == FALSE) {
		osso_log(LOG_INFO, "Activate service failed for bt name \n");
		return NULL;
	}
	if (dbus_error_is_set(&error)) {
		osso_log(LOG_INFO, "Error Activate btname%s\n", error.message);
		dbus_error_free(&error);
		return NULL;
	}
	/* Create request_message */
	request_message =
	    dbus_message_new_method_call("com.nokia.btname",
					 "/com/nokia/btname/request",
					 "com.nokia.btname.request", "get");
	/* send dbus message to the bt service to get the btname */
	reply_message = dbus_connection_send_with_reply_and_block
	    (conn, request_message, -1, &error);

	/* get the btdevicename from reply_message */
	if (!reply_message) {
		osso_log(LOG_INFO, "Error returned by btname service: %s\n", error.message);
		dbus_error_free(&error);
	} else {

		switch (dbus_message_get_type(reply_message)) {
		case DBUS_MESSAGE_TYPE_ERROR:
			osso_log(LOG_INFO, "error reply from btname service\n");
			break;
		case DBUS_MESSAGE_TYPE_METHOD_RETURN:
			dbus_message_iter_init(reply_message, &iter);
			if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_STRING) {
				osso_log(LOG_INFO,
					 "value returned by btname service is not valid\n");
			}
			dbus_message_iter_get_basic(&iter, &btname);
			if (btname != NULL) {
				deviceName = g_strdup(btname);
			}
			dbus_message_unref(reply_message);
			break;
		default:
			osso_log(LOG_INFO, "unknown msg type for get btname\n");

		}
	}
	dbus_message_unref(request_message);
	return deviceName;
}

/**
This is a helper function to check if a string has a given prefix.
Functionality is similar to "g_str_has_prefix", except that this takes
care of case-insensitive comparision.

@param  str  string to be checked for prefix
@param  pfx  prefix
@return boolean
*/
gboolean has_prefix_insensitive(const gchar * str, const gchar * pfx)
{
	gint i = 0;

	g_assert(str);
	g_assert(pfx);

	for (i = 0; pfx[i]; ++i) {
		if (!str[i])
			return FALSE;
		if (g_ascii_tolower(pfx[i]) != g_ascii_tolower(str[i]))
			return FALSE;
	}
	return TRUE;
}

/**
This is a helper function to predict a given dialog's width based on the
dialog title string. Note that this width is rough estimate only. This
function assumes that dialog title font size is double that of normal text
and margin on either side of title is 50 pixels.

@param  dialog  Pointer to dialog widget
@param  title   Title for the dialog
@return gint    Predicted width for the dialog.
*/
gint ui_predict_dialog_width_based_on_title(GtkWidget * dialog, const gchar * title)
{
	gint width = -1, height = -1;
	PangoLayout *layout = NULL;

	layout = gtk_widget_create_pango_layout(dialog, title);
	pango_layout_get_pixel_size(layout, &width, &height);
	/*Double the width and add the margin of around 50 pixels on either side */
	width = 2 * (width + 50);
	ULOG_DEBUG_F("Predicted dialog title width = %d\n", width);

	g_object_unref(G_OBJECT(layout));

	return width;
}

gchar *ui_get_tmp_path(void)
{
	gchar *tmp_path = NULL;

	tmp_path = g_strdup_printf("%s%s%s%s%s%s%s%s", g_get_home_dir(), GNOME_VFS_URI_PATH_STR,
				   APPLN_PATH, GNOME_VFS_URI_PATH_STR, EMAIL_PATH,
				   GNOME_VFS_URI_PATH_STR, TMP, GNOME_VFS_URI_PATH_STR);

	return tmp_path;
}

gchar *ui_get_tmp_attach_dir(void)
{
	gchar *attach_dir = NULL;
	gchar *tmp_path = NULL;

	tmp_path = ui_get_tmp_path();
	attach_dir = g_strdup_printf("%s%s%s%s", tmp_path, GNOME_VFS_URI_PATH_STR, ATTACH,
				     GNOME_VFS_URI_PATH_STR);
	g_free(tmp_path);
	return attach_dir;
}

gchar *ui_get_tmp_attach_path(void)
{

	gchar *attach_path = NULL;
	gchar *attach_dir = NULL;

	attach_dir = ui_get_tmp_attach_dir();
	attach_path = g_strdup_printf("%s%s%s", attach_dir, GNOME_VFS_URI_PATH_STR, ATTACHM);
	g_free(attach_dir);
	return attach_path;
}

gint make_dir(const gchar * dir)
{
	if (mkdir(dir, S_IRWXU) < 0) {
		return -1;
	}
	if (chmod(dir, S_IRWXU) < 0) {
		return -1;
	}
	return 0;
}

gchar *ui_get_tmp_attach_name()
{
	gchar *name = g_strdup(_("mail_va_attachment")), *str = NULL;

	/*strpping the : is the string so that we just have "Attachment" string */
	if (g_str_has_suffix(name, SUBJECT_SEPERATOR))
		str = g_strndup(name, strlen(name) - strlen(SUBJECT_SEPERATOR));
	else
		str = g_strdup(name);

	g_free(name);
	return str;
}

/**
This is a helper function to prepare correct linguistic string corresponding
to a given size in BYTES. This is used for displaying message/folder size in
'Message details' and 'Folder details' dialogs.

@param  size_in_bytes Size of the message/folder in BYTES
@return Newly allocated string with correct linguistic representation of the size.
        Return value has to be freed by the calling function.
*/
gchar *ui_format_size_value(guint size_in_bytes)
{
	gchar *size_str = NULL;
	guint size_in_mb = size_in_bytes / (1024 * 1024);

	if (size_in_bytes <= 1024) {
		size_str = g_strdup_printf(DGETTEXT_FM("sfil_li_size_kb"), 1);
	} else if (size_in_bytes < 100 * 1024) {
		size_str =
		    g_strdup_printf(DGETTEXT_FM("sfil_li_size_1kb_99kb"), size_in_bytes / 1024);
	} else if (size_in_bytes <= 1024 * 1024) {
		size_str =
		    g_strdup_printf(DGETTEXT_FM("sfil_li_size_100kb_1mb"),
				    (gfloat) size_in_bytes / (1024 * 1024));
	} else if (size_in_bytes <= 10 * 1024 * 1024) {
		size_str =
		    g_strdup_printf(DGETTEXT_FM("sfil_li_size_1mb_10mb"),
				    (gfloat) size_in_bytes / (1024 * 1024));
	} else if (size_in_mb <= 1024) {
		size_str = g_strdup_printf(DGETTEXT_FM("sfil_li_size_10mb_1gb"), size_in_mb);
	} else {
		size_str =
		    g_strdup_printf(DGETTEXT_FM("sfil_li_size_1gb_or_greater"),
				    (gfloat) size_in_mb / 1024);
	}

	return size_str;
}

/**
 This is function gets called when ever a insensitive item is clicked.

@param insensitive_widget is the GtkWidget for which the insensitive_press
 signal is fired.
@param info note is the string to be shown.
@return void.
 */
void show_dimming_note(GtkWidget * insensitive_widget, const gchar * infonote)
{
	gtk_infoprint((GtkWindow *) app, infonote);
	return;
}

/**
 This is a helper funtion to get the path of the message for the msgid & the folder number.

 @param msgid is the gchar pointer to the msgid.
 @param foldernum is the gint representing the folder number.
 @return gchar poiter to the message path.
*/
gchar *ui_get_message_file_path(const gchar * msgid, gint foldernum)
{
	gchar *foldername = NULL;
	gchar *msg_path = NULL;
	gchar *msg_num = NULL;

	foldername = ui_get_folder_name(foldernum);
	if (foldername == NULL)
		return NULL;

	/*If the message is in archive folder ,the path of the message is
	   taken from the mounted mmc path.Rather then the usual apps/email/Mail
	   path.
	 */
	if (ui_compare_utf8_strings(foldername, ARCHIVE) == 0) {
		msg_num = strrchr(msgid, MSGID_SEPARATOR);

		if (msg_num != NULL)
			msg_path = g_strdup_printf("%s%s%s%s%s",
						   MMC_PATH, GNOME_VFS_URI_PATH_STR, R_ARCHIVE,
						   GNOME_VFS_URI_PATH_STR, (msg_num + 1));

		g_free(foldername);
		return msg_path;
	}

	/*Here we get the message path corresponding to the msgid for
	   other then "Archive" folder from apps/email/Mail path.
	 */
	msg_path =
	    g_strdup_printf("%s%s%s%s%s%s%s%s%s%s%s", g_get_home_dir(), GNOME_VFS_URI_PATH_STR,
			    APPLN_PATH, GNOME_VFS_URI_PATH_STR, EMAIL_PATH, GNOME_VFS_URI_PATH_STR,
			    MAIL, GNOME_VFS_URI_PATH_STR, foldername, GNOME_VFS_URI_PATH_STR,
			    msgid);
	g_free(foldername);
	return msg_path;
}

/**
 This is a utility function to get the default plain/html font for the viewer.
 @param html is a gboolean representing whether html/plain mail.
 @return font_name is a gchar representing the font_name
*/
const gchar *ui_get_default_font(gboolean html)
{
	gint index = 0;
	const gchar *font_name = NULL;

	/*getting default font of the viewer */
	if (html != FALSE)
		index = ui_get_viewer_html_fontindex();
	else
		index = ui_get_viewer_plain_fontindex();

	font_name = font_families[index];
	return font_name;
}


/**
This function is used to fill the recipient list on clicking Ok button  
@param recpBuffer is recipient field to be updated
@param recipientlist_email_recipient list of email ids
@param econtact_uid is the UID of the current EContact
@return gboolean
*/
gboolean
ui_update_recipient_buffer(GtkTextBuffer * recpBuffer, GSList * recipientlist_email_recipient,
			   const gchar * econtact_uid)
{
	GtkTextIter iter;
	GtkTextIter start;
	GtkTextIter end;
	gboolean bdataavailable = FALSE;
	GtkTextTag *tagcontact = NULL;
	gchar *emailid = NULL;

	ULOG_DEBUG("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM((recpBuffer == NULL), "Input param check failed:", return FALSE);

	gtk_text_buffer_get_bounds(recpBuffer, &start, &end);
	if (!gtk_text_iter_equal(&start, &end))
		gtk_text_buffer_insert(recpBuffer, &end, SEMICOLON, -1);
	gtk_text_buffer_get_end_iter(recpBuffer, &iter);
	tagcontact =
	    gtk_text_buffer_create_tag(recpBuffer, NULL, "underline",
				       PANGO_UNDERLINE_SINGLE, "editable", TRUE, NULL);
	g_object_set_data(G_OBJECT(tagcontact), "emailid", GINT_TO_POINTER(EMAIL_ID));
	g_object_set_data_full(G_OBJECT(tagcontact), "econtact-uid",
			       g_strdup(econtact_uid), (GDestroyNotify) g_free);
	while (recipientlist_email_recipient != NULL) {
		if ((NULL != (gchar *) (recipientlist_email_recipient->data))
		    && (0 != strlen((gchar *) (recipientlist_email_recipient->data)))) {
			if (TRUE == bdataavailable) {
				gtk_text_buffer_insert(recpBuffer, &iter, SEMICOLON, -1);
			}
			/*inserting the resolved id with tags & other data without
			 * tags*/
			emailid = ui_parse_from_address(recipientlist_email_recipient->data);
			if (ui_accmgmt_email_isvalid(emailid) == TRUE) {
				gtk_text_buffer_insert_with_tags(recpBuffer, &iter, (gchar *)
								 (recipientlist_email_recipient->
								  data), -1, tagcontact, NULL);
			} else {
				gtk_text_buffer_insert(recpBuffer, &iter,
						       (gchar *) (recipientlist_email_recipient->
								  data), -1);
			}
			g_free(emailid);

			emailid = NULL;
			bdataavailable = TRUE;
		}

		recipientlist_email_recipient = g_slist_next(recipientlist_email_recipient);
	}

	recipientlist_email_recipient = g_slist_nth(recipientlist_email_recipient, 0);
	while (NULL != recipientlist_email_recipient) {
		g_free(recipientlist_email_recipient->data);
		recipientlist_email_recipient->data = NULL;
		recipientlist_email_recipient =
		    g_slist_remove(recipientlist_email_recipient,
				   recipientlist_email_recipient->data);
	}
	ULOG_DEBUG("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return TRUE;
}

/**
 * This is a utility function that takes a list of GtkTextBuffers, and checks the presence
 * of 'econtact-uid' property for each of the resolved email addresses. If this property
 * is set, this contact has to be set to recent_group of AB; otherwise this is a new contact.
 *
 * @param addr_buffers  List of GtkTextBuffers
 * @param used_econtact_uids  Out param to store list of used EContact UIDs
 * @param new_emailids  Out param to store list of new email IDs
 * @return void
 * */

void get_recipients_list_to_add_to_addbook(GSList * addr_buffers, GSList ** used_econtact_uids,
                                           GSList ** new_emailids)
{
        GtkTextIter start, end;
        GSList *buff_list = NULL;
        GSList *tags = NULL;
        GtkTextBuffer *text_buffer = NULL;
        GtkTextTag *tag = NULL;
        gchar *emailid = NULL;
        gchar *econtact_uid = NULL;

        if (!used_econtact_uids || !new_emailids)
                return;

        buff_list = addr_buffers;
        while (buff_list) {
                text_buffer = GTK_TEXT_BUFFER(buff_list->data);
                gtk_text_buffer_get_start_iter(text_buffer, &start);
                while (!gtk_text_iter_is_end(&start)) {
                        tags = gtk_text_iter_get_tags(&start);
                        if (tags) {
                                tag = tags->data;
                                end = start;
                                gtk_text_iter_forward_to_tag_toggle(&end, tag);
                                emailid = gtk_text_iter_get_text(&start, &end);

                                econtact_uid = g_object_get_data(G_OBJECT(tag), "econtact-uid");
                                if (econtact_uid) {
                                        *used_econtact_uids =
                                            g_slist_append(*used_econtact_uids,
                                                           g_strdup(econtact_uid));
                                } else {
                                        *new_emailids =
                                            g_slist_append(*new_emailids, g_strdup(emailid));
                                }

                                start = end;
                                g_slist_free(tags);
                        } else
                                gtk_text_iter_forward_char(&start);


                }
                buff_list = buff_list->next;
        }
}

/**
This function is called to check recipient names when enter is pressed.

@param textview is pointer to the recipient field.
@param event is key press event.
@param check_names_list is list of recipient fields.
@return gboolean.
*/
gboolean ui_check_recp_name(GtkWidget * textview, GdkEventKey * event, GSList * check_names_list)
{
	GtkTextBuffer *buf = NULL;
	GtkTextMark *insert = NULL;
	GtkTextIter iter, end_iter;
	gchar *text = NULL;

	ULOG_DEBUG("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM((textview == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((event == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((check_names_list == NULL), "Input param check failed", return FALSE);

	if ((event->keyval != GDK_KP_Enter)) {
		return FALSE;
	}

	buf = gtk_text_view_get_buffer((GtkTextView *) textview);
	insert = gtk_text_buffer_get_insert(buf);
	gtk_text_buffer_get_iter_at_mark(buf, &iter, insert);

	end_iter = iter;
	if (gtk_text_iter_backward_char(&iter) == FALSE) {
		return FALSE;
	}
	text = gtk_text_iter_get_text(&iter, &end_iter);
	if ((strcmp(text, SEMICOLON) == 0) || (strcmp(text, COMMA_S) == 0)
	    || (strcmp(text, SPACE) == 0) || (strcmp(text, CARRIAGE_RETURN) == 0)) {
		return FALSE;
	} else {
		gtk_text_buffer_insert(buf, &end_iter, ATTACHMENT_SEPERATOR_CHAR, ONE);
	}

	ULOG_DEBUG("[%s][%s]:ends here :\n", __FILE__, __FUNCTION__);
	return TRUE;
}


#if 0
get_user_confirmation(_("mcen_fi_folder_settings_setting_on"), NULL, INFO_BANNER);
get_user_confirmation(_("mcen_fi_folder_settings_select_interval"), NULL, INFO_BANNER);
get_user_confirmation(_("mcen_fi_folder_settings_selecttime"), NULL, INFO_BANNER);
get_user_confirmation(_("mcen_li_folder_settings_day"), NULL, INFO_BANNER);
get_user_confirmation(_("mcen_li_folder_settings_week"), NULL, INFO_BANNER);
get_user_confirmation(_("mcen_li_folder_settings_month"), NULL, INFO_BANNER);
get_user_confirmation(_("mcen_nw_deleting"), NULL, INFO_BANNER);
get_user_confirmation(_("mcen_bd_deleting_stop"), NULL, INFO_BANNER);
get_user_confirmation(_("mcen_nc_removesent"), NULL, INFO_BANNER);
get_user_confirmation(_("mcen_ib_already_retrieving"), NULL, INFO_BANNER);
get_user_confirmation(_("mcen_ib_unable_to_cut_mess"), NULL, INFO_BANNER);
get_user_confirmation(_("ckdg_ib_unable_to_mark"), NULL, INFO_BANNER);
get_user_confirmation(_("ckdg_ib_unable_to_unmark"), NULL, INFO_BANNER);
#endif

/* End of file */
