/**
 * @file undo.c
 *
 * This file contains functions for Undo feature in email-editor
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */



// Header file containing all standerd paths.
#include "ui_common.h"

#include "undo.h"
#include "editor.h"
#include "htcallback.h"
#include "callback.h"



/**
This function is called to get tags present in a particular range of text. 
@param start is start of text range.
@param end is end of text range.
@return list of tags.
*/
static GSList *undo_get_tags_in_range(GtkTextIter * start, GtkTextIter * end)
{
	GSList *ret = NULL, *tags = NULL;
	GtkTextIter iter;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (start == NULL || end == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return NULL;
	}

	for (iter = *start; !gtk_text_iter_equal(&iter, end); gtk_text_iter_forward_char(&iter)) {
		tags = gtk_text_iter_get_tags(&iter);
		ret = g_slist_append(ret, tags);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return ret;
}


/**
This function is called to get images present in a particular range of text. 
@param start is start of text range.
@param end is end of text range.
@return list of images.
*/
static GSList *undo_get_images_in_range(GtkTextIter * start, GtkTextIter * end)
{
	GSList *imglist = NULL;
	gunichar ch;
	imgInfo *imginfo = NULL;
	GdkPixbuf *pixbuf = NULL;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (start == NULL || end == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return NULL;
	}
	ch = gtk_text_iter_get_char(start);
	if (ch == 0xFFFC) {
		pixbuf = gtk_text_iter_get_pixbuf(start);
		if (pixbuf != NULL)
			imginfo = g_new0(imgInfo, 1);
		if (imginfo == NULL) {
			return NULL;
		}
		imginfo->img_offset = gtk_text_iter_get_offset(start);
		imginfo->pixbuf = pixbuf;
		imglist = g_slist_append(imglist, imginfo);
		pixbuf = NULL;
	}

	while (gtk_text_iter_forward_find_char(start, predicate, NULL, end)) {
		pixbuf = gtk_text_iter_get_pixbuf(start);
		if (pixbuf == NULL) {
			gtk_text_iter_forward_char(start);
			continue;
		} else {
			imginfo = g_new0(imgInfo, 1);
			if (imginfo == NULL) {
				return NULL;
			}
			imginfo->img_offset = gtk_text_iter_get_offset(start);
			imginfo->pixbuf = pixbuf;
			imglist = g_slist_append(imglist, imginfo);
			gtk_text_iter_forward_char(start);
			pixbuf = NULL;
		}
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return imglist;
}


/**
This function is called to set tags in a range of text which were 
present earlier in that particular range of text. 
@param start is start of text range.
@param end_arg is end of text range.
@param tags is list of tags.
@return void.
*/
static void
undo_set_tags_in_range(GtkTextIter * start, GtkTextIter * end_arg,
		       GSList * tags, EditorInfo * editorinfo)
{
	GtkTextBuffer *buf = NULL;
	GtkTextIter iter, end;
	GSList *tag_list = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (start == NULL || end_arg == NULL || tags == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}
	iter = *start;
	end = iter;
	gtk_text_iter_forward_char(&end);
	buf = gtk_text_iter_get_buffer(&iter);
/*bullets no. handling*/
	g_signal_handlers_block_by_func(G_OBJECT(buf),
					G_CALLBACK(on_text_buffer_apply_tag), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(buf),
					G_CALLBACK(on_text_buffer_remove_tag), editorinfo);

	gtk_text_buffer_remove_all_tags(buf, start, end_arg);

	while (tags != NULL) {
		tag_list = (GSList *) tags->data;
		while (tag_list) {
			gtk_text_buffer_apply_tag(buf, (GtkTextTag *) tag_list->data, &iter, &end);
			tag_list = (GSList *) g_slist_next(tag_list);
		}
		gtk_text_iter_forward_char(&iter);
		gtk_text_iter_forward_char(&end);
		tags = (GSList *) g_slist_next(tags);
	}
	g_signal_handlers_unblock_by_func(G_OBJECT(buf),
					  G_CALLBACK(on_text_buffer_apply_tag), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(buf),
					  G_CALLBACK(on_text_buffer_remove_tag), editorinfo);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called to set images in a range of text which were 
 present earlier in that particular range of text. 
@param start is start of text range.
@param imglist is list of images.
@param editorinfo is pointer to EditorInfo structure.
@return void.
*/
static void
undo_set_images_in_range(GtkTextIter * start, GSList * imglist,
			 EditorInfo * editorinfo, gint action)
{
	GtkTextBuffer *buf = NULL;
	GSList *img_list = NULL;
	imgInfo *imginfo = NULL;
	GtkTextIter end;
	gunichar ch;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (start == NULL || imglist == NULL || editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}
	img_list = imglist;
	buf = gtk_text_iter_get_buffer(start);

	while (img_list != NULL) {
		imginfo = img_list->data;
		gtk_text_buffer_get_iter_at_offset(buf, start, imginfo->img_offset);
		gtk_text_buffer_insert_pixbuf(buf, start, imginfo->pixbuf);
		gtk_text_buffer_get_iter_at_offset(buf, start, imginfo->img_offset + 1);
		gtk_text_buffer_get_iter_at_offset(buf, &end, imginfo->img_offset + 2);
		//removal of junk char of pixbuf
		ch = gtk_text_iter_get_char(start);
               	if (ch == 0xFFFC) {
			gtk_text_buffer_delete(buf, start, &end);
		}
		img_list = (GSList *) g_slist_next(img_list);
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}



/**
This function is called by higher layers to free memory occupied by 
 undo structure.
@param op is a pointer to structure UndoOpInfo.
This paramter is  freed here.
@return void
*/
void undo_free_op(UndoOpInfo * op)
{
	gpointer *tmp;
	int i;
	GSList *imglist = NULL;
	imgInfo *imginfo = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (op == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}
	g_free(op->text);
	g_free(op->prevcolor);
	for (i = 0;; i++) {
		tmp = g_slist_nth_data(op->tag_list, i);
		if (!tmp) {
			break;
		}
		g_slist_free((GSList *) tmp);
	}
	/* Delete elements of op->delete_tag_list ? */
	g_slist_free(op->tag_list);
	for (imglist = op->img_list; imglist != NULL; imglist = imglist->next) {
		imginfo = imglist->data;
		g_free(imginfo);
	}
	g_slist_free(op->img_list);
	g_free(op);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called by higher layers to update the op structure  
 everytime some change relevent to undo happens in editor
@param editorinfo is a pointer to structure EditorInfo.
@param action is action took place in editor.
@param start is start position of the action
@param end is end position of the action.
@return void
*/

static UndoOpInfo *undo_register_new_operation(EditorInfo * editorinfo, int op_type,
					       gint start, gint end)
{
	UndoOpInfo *op = NULL;
	gint length = 0;
	GtkTextIter start_iter, end_iter, insert_iter;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return NULL;
	}

	if (g_queue_get_length(editorinfo->undo_queue) == 5) {
		/* remove the FIRST element in the queue */
		op = g_queue_pop_head(editorinfo->undo_queue);
		undo_free_op(op);
	}

	op = g_new0(UndoOpInfo, 1);

	op->action = op_type;
	op->undobuf = editorinfo->undofield;
	op->start_pos = start;
	op->end_pos = end;
	if (GTK_IS_TEXT_BUFFER(editorinfo->undofield)) {
		gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(editorinfo->undofield),
						 &insert_iter,
						 gtk_text_buffer_get_insert(GTK_TEXT_BUFFER
									    (editorinfo->
									     undofield)));
		op->cursor_pos = gtk_text_iter_get_offset(&insert_iter);
	} else {
		op->cursor_pos = start;
	}
	length = end - start;
	op->prevcolor = g_new0(GdkColor, 1);
	if (op->prevcolor == NULL) {
		return NULL;
	}
	op->prevcolor->pixel = editorinfo->prevcolor->pixel;
	op->prevcolor->red = editorinfo->prevcolor->red;
	op->prevcolor->green = editorinfo->prevcolor->green;
	op->prevcolor->blue = editorinfo->prevcolor->blue;

	if (GTK_IS_TEXT_BUFFER(editorinfo->undofield)) {
		gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &start_iter,
						   start);
		gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &end_iter, end);
		if (length != 0) {
			op->text =
			    gtk_text_buffer_get_slice((GtkTextBuffer *) op->undobuf,
						      &start_iter, &end_iter, TRUE);
		} else {
			op->text = g_strdup("");
		}
		if (op_type != UNDO_INSERT) {
			op->tag_list = undo_get_tags_in_range(&start_iter, &end_iter);
			op->img_list = undo_get_images_in_range(&start_iter, &end_iter);
		} else {
			op->tag_list = NULL;
			op->img_list = NULL;
		}

	} else {
		op->text = gtk_editable_get_chars((GtkEditable *) op->undobuf, start, end);
		op->tag_list = NULL;
	}
	g_queue_push_tail(editorinfo->undo_queue, op);
	gtk_widget_set_sensitive(editorinfo->menu->undo, TRUE);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return op;
}



/**
This function is called by higher layers when any insertion is made in the
 textviews of editor to update undoinfo structure.
@param undobuffer is a pointer to buffer in which text is inserted.
@param iter is a pointer to start of insertion.
@param undotext is a pointer to the text inserted or deleted in the textviews.
@param len is length of the text inserted in the textviews.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
void
undoins(GtkTextBuffer * undobuffer, GtkTextIter * iter, gchar * undotext,
	gint len, EditorInfo * editorinfo)
{
	gint start = 0, end = 0;	/*setting the initial offset to 0i.e begining of the buffer */
	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);

	if (undobuffer != NULL) {
		editorinfo->undofield = (GtkWidget *) undobuffer;
	}
	start = gtk_text_iter_get_offset(iter);
	end = start + len;
	undo_register_new_operation(editorinfo, UNDO_INSERT, start, end);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called to update undo information
        upon paste operation
        
 @param clipboard is a pointer to clipboard.
 @param selection_data is a pointer to data in clipboard.
 @param undosruct is a pointer to structure UndoInfo.
 
 @return void
*/

void undo_paste(GtkTextBuffer * buffer, gint start_offset, EditorInfo * editorinfo)
{
	GtkTextIter end;
	gint end_offset = 0;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (buffer == NULL || editorinfo == NULL) {
		return;
	}
	editorinfo->undofield = (GtkWidget *) buffer;
	gtk_text_buffer_get_iter_at_mark(buffer, &end, gtk_text_buffer_get_insert(buffer));
	end_offset = gtk_text_iter_get_offset(&end);
	undo_register_new_operation(editorinfo, UNDO_INSERT, start_offset, end_offset);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called by higher layers when any insertion of pixbuf is made in the body of editor to update undoinfo structure.
@param undobuffer is a pointer to buffer in which text is inserted.
@param iter is a pointer to start of insertion.
@param pix is a pointer to the pixbuf inserted in the textview.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
/*void
undoinspixbuf (GtkTextBuffer * undobuffer, GtkTextIter * iter,
               GdkPixbuf * pix, EditorInfo * editorinfo)
{
  gint start = 0, end = 0;

  ULOG_DEBUG ("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
  if (pix == NULL || undobuffer == NULL || editorinfo == NULL)
    {
      ULOG_WARN ("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
      return;
    }
  editorinfo->undofield = (GtkWidget *) undobuffer;
  start = gtk_text_iter_get_offset (iter);
  //gtk_text_buffer_get_iter_at_offset(undobuffer, &enditer, (start + len));
  end = start + 1;
  undo_register_new_operation (editorinfo, UNDO_INSERT, start, end);

  ULOG_DEBUG ("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
  return;
}*/

/**
This function is called by higher layers when any insertion of mage is made in the body of editor through FileSelection widget to update undoinfo structure.
@param buffer is a pointer to buffer in which text is inserted.
@param no_of_images is the no. of images inserted.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
void undo_image_insertion(GtkTextBuffer * buffer, gint no_of_images, EditorInfo * editorinfo)
{
	GtkTextIter iter;
	gint start = 0, end = 0;	/*setting the initial offset to 0i.e begining of the buffer */

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (buffer == NULL || editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}

	editorinfo->undofield = (GtkWidget *) buffer;
	gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));

	end = gtk_text_iter_get_offset(&iter);
	start = end - no_of_images;
	/*a new line char is inserted after the insertion of images. So end position is incremented */
	end++;
	undo_register_new_operation(editorinfo, UNDO_INSERT, start, end);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called by higher layers when any insertion is made in the
subject entry field  of to update undoinfo structure.
@param entry is a pointer to text entry in  which text is inserted.
@param undotext is a pointer to the text inserted.
@param len is length of the text inserted.
@param position is a pointer to start of insertion.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
void
undoinssub(GtkEditable * entry, gchar * undotext, gint len,
	   gint * position, EditorInfo * editorinfo)
{
	gint end = 0;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (entry == NULL || editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}

	editorinfo->undofield = (GtkWidget *) entry;
	end = *position + len;
	undo_register_new_operation(editorinfo, UNDO_INSERT, *position, end);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called by higher layers to update undo information
upon deletion of text in message body.
@param undobuffer is a pointer to text buffer in  which text is deleted.
@param delstart is a pointer to start of deletion.
@param delend is a pointer to end of deletion.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
void
undodel(GtkTextBuffer * undobuffer, GtkTextIter * delstart,
	GtkTextIter * delend, EditorInfo * editorinfo)
{
	gint start = 0, end = 0;
	UndoOpInfo *op = NULL;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (undobuffer == NULL || editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}

	editorinfo->undofield = (GtkWidget *) undobuffer;
	start = gtk_text_iter_get_offset(delstart);
	end = gtk_text_iter_get_offset(delend);
	op = undo_register_new_operation(editorinfo, UNDO_DELETE, start, end);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called by higher layers whe any deletion is made in the
subject entry to update undoinfo structure.
@param entry is a pointer to text entry in  which text is deleted.
@param start is of deletion.
@param end is end of deletion.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
void undodelsub(GtkEditable * entry, gint start, gint end, EditorInfo * editorinfo)
{
	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (entry == NULL || editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}

	undo_register_new_operation(editorinfo, UNDO_DELETE, start, end);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This function is called whenever some formatting is changed in the 
 message body.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
UndoOpInfo *update_undoinfo_on_format_change(EditorInfo * editorinfo, GtkWidget * formatitem)
{
	GtkTextBuffer *buffer = NULL;
	gint start = 0, end = 0, linenumber = 0;
	GtkTextIter start_it, end_it;
	UndoOpInfo *op = NULL;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return NULL;
	}
	buffer = gtk_text_view_get_buffer((GtkTextView *) editorinfo->msgFields->body);
	if (buffer != NULL) {
		editorinfo->undofield = (GtkWidget *) buffer;
	}
	//fix may be needed
	if ((formatitem == editorinfo->menu->left)
	    || (formatitem == editorinfo->menu->centred)
	    || (formatitem == editorinfo->menu->right)
	    || (formatitem == editorinfo->menu->bullets)
	    || (formatitem == editorinfo->menu->numbering)
	    || (formatitem == editorinfo->toolbar->bullets)) {
		gtk_text_buffer_get_selection_bounds(buffer, &start_it, &end_it);
		linenumber = gtk_text_iter_get_line(&start_it);
		gtk_text_buffer_get_iter_at_line(buffer, &start_it, linenumber);
		if (!gtk_text_iter_forward_line(&end_it)) {
			gtk_text_iter_forward_sentence_end(&end_it);
		}
	} else if (gtk_text_buffer_get_selection_bounds(buffer, &start_it, &end_it)
		   == FALSE) {
		return NULL;
	}
	start = gtk_text_iter_get_offset(&start_it);
	end = gtk_text_iter_get_offset(&end_it);
	if ((formatitem == editorinfo->menu->bullets)
	    || (formatitem == editorinfo->menu->numbering)
	    || (formatitem == editorinfo->toolbar->bullets)) {
		op = undo_register_new_operation(editorinfo, UNDO_BULLET_CHANGE, start, end);
	} else {
		op = undo_register_new_operation(editorinfo, UNDO_FMT_CHANGE, start, end);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return op;
}



/**
This function is called whenever the message type is changed. 
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
UndoOpInfo *update_undo_on_msg_type_change(EditorInfo * editorinfo)
{
	GtkTextBuffer *buffer = NULL;
	gint start = 0, end = 0;
	GtkTextIter start_it, end_it;
	UndoOpInfo *undo_op = NULL;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return NULL;
	}
	buffer = gtk_text_view_get_buffer((GtkTextView *) editorinfo->msgFields->body);
	if (buffer != NULL) {
		editorinfo->undofield = (GtkWidget *) buffer;
	}

	gtk_text_buffer_get_bounds(buffer, &start_it, &end_it);
	start = gtk_text_iter_get_offset(&start_it);
	end = gtk_text_iter_get_offset(&end_it);
	undo_op = undo_register_new_operation(editorinfo, UNDO_MSG_TYPE_CHANGE, start, end);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return undo_op;
}

/**
This function is called whenever some formatting is changed in the 
 message body.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
void update_undoinfo_on_bg_change(EditorInfo * editorinfo, GtkTextIter start_it, GtkTextIter end_it)
{
	GtkTextBuffer *buffer = NULL;
	gint start = 0, end = 0;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}
	buffer = gtk_text_view_get_buffer((GtkTextView *) editorinfo->msgFields->body);
	editorinfo->undofield = (GtkWidget *) buffer;
	start = gtk_text_iter_get_offset(&start_it);
	end = gtk_text_iter_get_offset(&end_it);
	undo_register_new_operation(editorinfo, UNDO_BG_MOD, start, end);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This function is called to undo insert operation. 
@param op is pointer to UndoOpInfo structure present in the tail of undo queue.
@return void
*/
static void undo_do_as_delete_operation(UndoOpInfo * op)
{
	GtkTextIter start, end;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (op == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}

	if (GTK_IS_TEXT_BUFFER(op->undobuf)) {
		gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &start,
						   op->start_pos);
		gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &end,
						   op->end_pos);
		gtk_text_buffer_delete((GtkTextBuffer *) op->undobuf, &start, &end);
	} else {
		gtk_editable_delete_text((GtkEditable *) op->undobuf, op->start_pos, op->end_pos);
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called to handle bullets & numbering on undo 
  of delete operation. 
@param op is pointer to UndoOpInfo structure present in the tail of undo queue.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
static void undo_handle_bullets_after_insert_operation(UndoOpInfo * op, EditorInfo * editorinfo)
{
	GtkTextIter start_it, end_it;
	GtkTextTag *tag = NULL;
	gint offset = 0;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (op == NULL || editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}
	g_signal_handlers_block_by_func(G_OBJECT(op->undobuf),
					G_CALLBACK(on_text_buffer_delete_range), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(op->undobuf),
					G_CALLBACK(on_text_buffer_insert_text), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(op->undobuf), G_CALLBACK(undoins), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(op->undobuf), G_CALLBACK(undodel), editorinfo);

	gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &start_it, op->start_pos);
	while (gtk_text_iter_get_offset(&start_it) <= op->end_pos) {
		tag = iter_has_bullet_or_number_tag(&start_it);
		if (tag != NULL) {
			end_it = start_it;
			if (!gtk_text_iter_begins_tag(&start_it, tag)) {
				gtk_text_iter_backward_to_tag_toggle(&end_it, tag);
			}
			gtk_text_iter_forward_to_tag_toggle(&end_it, tag);
			offset = gtk_text_iter_get_offset(&end_it);
			set_list_items((GtkTextBuffer *) op->undobuf, &start_it, &end_it, tag,
				       editorinfo);
			gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &start_it,
							   offset);
		}
		if (gtk_text_iter_is_end(&start_it)) {
			break;
		}
		gtk_text_iter_forward_char(&start_it);
	}

	g_signal_handlers_unblock_by_func(G_OBJECT(op->undobuf),
					  G_CALLBACK(on_text_buffer_delete_range), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(op->undobuf),
					  G_CALLBACK(on_text_buffer_insert_text), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(op->undobuf), G_CALLBACK(undoins), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(op->undobuf), G_CALLBACK(undodel), editorinfo);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called to undo delete operation. 
@param op is pointer to UndoOpInfo structure present in the tail of undo queue.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
static void undo_do_as_insert_operation(UndoOpInfo * op, EditorInfo * editorinfo)
{
	GtkTextIter tmp, start, end;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (op == NULL || editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}

	if (!GTK_IS_TEXT_BUFFER(op->undobuf)) {
		gtk_editable_insert_text(GTK_EDITABLE(op->undobuf), op->text,
					 (op->end_pos - op->start_pos), &(op->start_pos));
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return;
	}
	gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &tmp, op->start_pos);

	gtk_text_buffer_insert((GtkTextBuffer *) op->undobuf, &tmp, op->text,
			       (op->end_pos - op->start_pos + 2 * (g_slist_length(op->img_list))));
	//hack for inline image
	gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &tmp, op->start_pos);
	end = tmp;
	gtk_text_iter_forward_chars(&end, (op->end_pos - op->start_pos));
	undo_set_tags_in_range(&tmp, &end, op->tag_list, editorinfo);

	gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &start, op->start_pos);
	undo_set_images_in_range(&start, op->img_list, editorinfo, op->action);

	/* Take care of bullets. */
	undo_handle_bullets_after_insert_operation(op, editorinfo);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called to undo any change in formatting of text. 
@param op is pointer to UndoOpInfo structure present in the tail of undo queue.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
static void undo_do_text_format_change(UndoOpInfo * op, EditorInfo * editorinfo)
{
	GtkTextIter tmp, start, end;
	GtkTextIter start_it, end_it;
	gint previous_zoom = DEFAULT_ZOOM;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (op == NULL || editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}

	if (GTK_IS_TEXT_BUFFER(op->undobuf) == FALSE) {
		return;
	}
	/*If the msg type is changed from plain to HTML and UNDO is done */
	if (editorinfo->emailType == HTML_EDITOR) {
		editorinfo->emailType = PLAIN_EDITOR;
		editorinfo->msgInfo->settings->type = EMAIL_TEXT;
		set_default_font(editorinfo, editorinfo->msgInfo->settings);
		ch_toolbar_insens(NULL, NULL, editorinfo);
		return;
	}
	/*If the msg type is changed from HTML to Plain and UNDO is done */
	g_signal_handlers_block_by_func(G_OBJECT(op->undobuf), (gpointer) undoins, editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(op->undobuf), G_CALLBACK(undodel), editorinfo);
	gtk_text_buffer_get_bounds((GtkTextBuffer *) op->undobuf, &start_it, &end_it);
	gtk_text_buffer_delete((GtkTextBuffer *) op->undobuf, &start_it, &end_it);
	gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &tmp, op->start_pos);

	gtk_text_buffer_insert((GtkTextBuffer *) op->undobuf, &tmp, op->text,
			       (strlen(op->text)));
	//hack for inline image
	gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &tmp, op->start_pos);
	end = tmp;
	gtk_text_iter_forward_chars(&end, (op->end_pos - op->start_pos));

	undo_set_tags_in_range(&tmp, &end, op->tag_list, editorinfo);

	/*Apply the zoom effect and recalculate the font size */
	previous_zoom = editorinfo->zoominfo->previous_level;
	ui_set_zoom(editorinfo->zoominfo, DEFAULT_ZOOM);
	editorinfo->zoominfo->previous_level = previous_zoom;

	gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &start, op->start_pos);
	undo_set_images_in_range(&start, op->img_list, editorinfo, op->action);

	gtk_widget_modify_base(editorinfo->msgFields->body, GTK_STATE_NORMAL, editorinfo->bgcolor);

	editorinfo->emailType = HTML_EDITOR;
	editorinfo->msgInfo->settings->type = EMAIL_HTML;
	ch_toolbar(NULL, NULL, editorinfo);
	set_default_font(editorinfo, editorinfo->msgInfo->settings);

	g_signal_handlers_unblock_by_func(G_OBJECT(op->undobuf), (gpointer) undoins, editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(op->undobuf), G_CALLBACK(undodel), editorinfo);
	//undo_handle_bullets_after_insert_operation(op, editorinfo);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called to undo bulleting /numbering operation. 
@param op is pointer to UndoOpInfo structure present in the tail of undo queue.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
static void undo_bullet_operation(UndoOpInfo * op, EditorInfo * editorinfo)
{
	GtkTextIter start, end;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (op == NULL || editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}

	g_signal_handlers_block_by_func(G_OBJECT(op->undobuf), G_CALLBACK(undoins), editorinfo);
	g_signal_handlers_block_by_func(G_OBJECT(op->undobuf), G_CALLBACK(undodel), editorinfo);
	gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &start, op->del_start);
	gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &end, op->del_end);
	gtk_text_buffer_delete((GtkTextBuffer *) op->undobuf, &start, &end);

	gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &start, op->start_pos);
	gtk_text_buffer_insert((GtkTextBuffer *) op->undobuf, &start, op->text,
			       op->start_pos - op->end_pos);
	gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &start, op->start_pos);
	gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &end, op->end_pos);
	undo_set_tags_in_range(&start, &end, op->tag_list, editorinfo);
	undo_set_images_in_range(&start, op->img_list, editorinfo, op->action);

	g_signal_handlers_unblock_by_func(G_OBJECT(op->undobuf), G_CALLBACK(undoins), editorinfo);
	g_signal_handlers_unblock_by_func(G_OBJECT(op->undobuf), G_CALLBACK(undodel), editorinfo);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called to update undo structure upon bullet/numbering 
 operation. 
@param op is pointer to UndoOpInfo structure present in the tail of undo queue.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
void undo_set_delete_range(UndoOpInfo * op, EditorInfo * editorinfo)
{
	GtkTextBuffer *buffer = NULL;
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	gint linenumber = 0;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (op == NULL || editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(editorinfo->msgFields->body));
	gtk_text_buffer_get_selection_bounds(buffer, &start_iter, &end_iter);
	linenumber = gtk_text_iter_get_line(&start_iter);
	gtk_text_buffer_get_iter_at_line(buffer, &start_iter, linenumber);
	if (!gtk_text_iter_forward_line(&end_iter)) {
		gtk_text_iter_forward_sentence_end(&end_iter);
	}
	op->del_start = gtk_text_iter_get_offset(&start_iter);
	op->del_end = gtk_text_iter_get_offset(&end_iter);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called to undo the last operation present in 
the tail of undo queue. 
@param win is pointer to undo menu option.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
void undo_do_undo(GtkWidget * win, EditorInfo * editorinfo)
{
	UndoOpInfo *op = NULL;
	GtkTextIter start, end, curs;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}

	op = (UndoOpInfo *) g_queue_pop_tail(editorinfo->undo_queue);

	if (!op) {
		return;
	}

	switch (op->action) {
	case UNDO_INSERT:
		g_signal_handlers_block_by_func(G_OBJECT(op->undobuf), (gpointer) undodel,
						editorinfo);
		g_signal_handlers_block_by_func(G_OBJECT(op->undobuf), (gpointer) undodelsub,
						editorinfo);
		undo_do_as_delete_operation(op);
		if (GTK_IS_TEXT_BUFFER(op->undobuf)) {
			gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &curs,
							   op->start_pos);
		} else {
			gtk_editable_set_position((GtkEditable *) op->undobuf, op->start_pos);
		}
		g_signal_handlers_unblock_by_func(G_OBJECT(op->undobuf), (gpointer) undodel,
						  editorinfo);
		g_signal_handlers_unblock_by_func(G_OBJECT(op->undobuf), (gpointer) undodelsub,
						  editorinfo);
		break;

	case UNDO_DELETE:
		g_signal_handlers_block_by_func(G_OBJECT(op->undobuf), (gpointer) undoins,
						editorinfo);
		g_signal_handlers_block_by_func(G_OBJECT(op->undobuf), (gpointer) undoinssub,
						editorinfo);
		undo_do_as_insert_operation(op, editorinfo);
		if (GTK_IS_TEXT_BUFFER(op->undobuf)) {
			gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &curs,
							   op->end_pos);
		} else {
			gtk_editable_set_position((GtkEditable *) op->undobuf, op->end_pos);
		}
		g_signal_handlers_unblock_by_func(G_OBJECT(op->undobuf), (gpointer) undoins,
						  editorinfo);
		g_signal_handlers_unblock_by_func(G_OBJECT(op->undobuf), (gpointer) undoinssub,
						  editorinfo);
		break;

	case UNDO_FMT_CHANGE:
		if (GTK_IS_TEXT_BUFFER(op->undobuf) == FALSE) {
			break;
		}
		gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &start,
						   op->start_pos);
		gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &end,
						   op->end_pos);
		undo_set_tags_in_range(&start, &end, op->tag_list, editorinfo);
		gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &curs,
						   op->end_pos);
		g_signal_handlers_block_by_func(G_OBJECT(editorinfo->menu->left),
						(gpointer) on_format_button_click, editorinfo);
		g_signal_handlers_block_by_func(G_OBJECT(editorinfo->menu->centred),
						(gpointer) on_format_button_click, editorinfo);
		g_signal_handlers_block_by_func(G_OBJECT(editorinfo->menu->right),
						(gpointer) on_format_button_click, editorinfo);
		update_alignment_menu(editorinfo);
		g_signal_handlers_unblock_by_func(G_OBJECT(editorinfo->menu->left),
						  (gpointer) on_format_button_click, editorinfo);
		g_signal_handlers_unblock_by_func(G_OBJECT
						  (editorinfo->menu->centred),
						  (gpointer) on_format_button_click, editorinfo);
		g_signal_handlers_unblock_by_func(G_OBJECT
						  (editorinfo->menu->right),
						  (gpointer) on_format_button_click, editorinfo);
		break;

	case UNDO_BULLET_CHANGE:
		undo_bullet_operation(op, editorinfo);
		gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &curs,
						   op->cursor_pos);
		break;

	case UNDO_MSG_TYPE_CHANGE:
		if (GTK_IS_TEXT_BUFFER(op->undobuf) == FALSE) {
			break;
		}
		undo_do_text_format_change(op, editorinfo);
		gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &curs,
						   op->end_pos);
		break;
	case UNDO_BG_MOD:
		if (GTK_IS_TEXT_BUFFER(op->undobuf) == FALSE) {
			break;
		}
		gtk_widget_modify_base(editorinfo->msgFields->body, GTK_STATE_NORMAL,
				       op->prevcolor);
		editorinfo->bgcolor->pixel = op->prevcolor->pixel;
		editorinfo->bgcolor->red = op->prevcolor->red;
		editorinfo->bgcolor->green = op->prevcolor->green;
		editorinfo->bgcolor->blue = op->prevcolor->blue;
		gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) op->undobuf, &curs,
						   op->end_pos);
		break;

	default:
		g_assert(FALSE);
		break;
	}

	if (GTK_IS_TEXT_BUFFER(op->undobuf)) {
		gtk_text_buffer_place_cursor((GtkTextBuffer *) op->undobuf, &curs);
	}
	if (g_queue_get_length(editorinfo->undo_queue) == 0) {
		gtk_widget_set_sensitive(editorinfo->menu->undo, FALSE);
	}

	refresh_current_attributes((GtkTextView *) editorinfo->msgFields->body, editorinfo);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called by higher layers to free memory occupied by 
 undoinfo structure.
@param editorinfo is a pointer to structure EditorInfo.
@return void
*/
void undo_destroy(EditorInfo * editorinfo)
{
	UndoOpInfo *op = NULL;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);
	if (editorinfo == NULL) {
		ULOG_WARN("[%s][%s]:invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}

	g_free(editorinfo->prevcolor);
	g_free(editorinfo->bgcolor);

	while (!g_queue_is_empty(editorinfo->undo_queue)) {
		/* remove the FIRST element in the queue */
		op = g_queue_pop_head(editorinfo->undo_queue);
		undo_free_op(op);
	}
	g_queue_free(editorinfo->undo_queue);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}
