/**
 * @file viewcallback.c
 *
 * This file contains all callback functions for the viewer.
 *
 *  Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/

/*Editor-Viewer header files*/
#include "editcallback.h"
#include "callback.h"

/*security related header files*/
#include "security.h"

#include "engine_errno.h"
#include <ossoemailinterface.h>
#include <hildon-widgets/hildon-file-system-model.h>
#include "osso-bookmark-interface.h"

/*@null@*/
extern MainViewInfo *email_main_view;	/*declared in ui_interface.c */
extern osso_context_t *ui_context;	/*declared in main.c */

gchar *gnome_vfs_mime_type_from_name(const char *);
static void change_the_icon(GSList * local_attach_names,
			    ViewerInfo * viewer_info, gboolean, gchar *);
static void process_attachment_purging(ViewerInfo * viewer_info, GSList * selected_names);
static gboolean update_mimeinfo_for_last_attachment(ViewerInfo * viewer_info);
static gint ui_get_attachment_path(const MimeInfo * mimeinfo,
				   const gchar * msgid, gint nth_attachment);
static gint
check_for_mhtml_attachment(const MimeInfo * mimeInfo, gint nth_attach, ViewerInfo * viewer_info);

static gboolean check_for_body(MimeInfo * mimeinfo, gboolean first_partflag);
static void root_mimeinfo(MimeInfo * mimeinfo, MimeInfo ** root_mime_info);
static MimeInfo *get_sub_mimeinfo(MimeInfo * mimeInfo, ViewerInfo * viewer_info);

/**
 This is a helper function to free the window pointer from the global list.
 
 @param msgid is a gchar pointer to traverse the global list.
 @returns void.
*/
void ui_update_window_pointers(const gchar * msgid)
{
	OpenedWindowData *open_window = NULL;
	GSList *node = NULL;

	for (node = email_main_view->open_windows; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) node->data;
		if (msgid != NULL && open_window != NULL && open_window->msgid != NULL
		    && strncmp(open_window->msgid, msgid,
			       MAX_STR_LEN(open_window->msgid, msgid)) == 0) {
			email_main_view->open_windows =
			    g_slist_remove_link(email_main_view->open_windows, node);
			g_free(open_window->msgid);
			g_free(open_window);
		}
	}
	return;
}

/**
 This function is called to top the next open window upon closing the current window
 
 @return void.
 */
void ui_show_next_window(void)
{
	OpenedWindowData *open_window = NULL;
	gchar *current_folder_name = NULL;
	gboolean no_view_available = FALSE;
	gboolean is_main_view_next = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (email_main_view->open_windows == NULL) {
		no_view_available = TRUE;
	} else {
		open_window = (OpenedWindowData *) email_main_view->open_windows->data;
		if (open_window->msgid != NULL && strcmp(open_window->msgid, MAIN_VIEW_ID) == 0) {
			/*Next window to show upon closure of current window is the main view. */
			is_main_view_next = TRUE;
			if (g_slist_length(email_main_view->open_windows) == 1) {
				if (email_main_view->tags->main_view_hidden)
					/*editor was opened from browser/fm in standalone mode. Need to close the app. */
					no_view_available = TRUE;
			}
		}
	}

	if (!no_view_available) {
		hildon_app_set_appview(app, (HildonAppView *) open_window->window_ptr);
		if (is_main_view_next) {
			email_main_view->tags->main_view_topped = TRUE;
			ULOG_DEBUG("[%s]:[%d] : Main view topped\n", __FILE__, __LINE__);
			/*Refresh folder title */
			current_folder_name =
			    ui_get_folder_name(email_main_view->folder_manager->
					       currently_displayed_model);
			ui_set_folderview_title(current_folder_name);
			g_free(current_folder_name);
		} else {
			email_main_view->tags->main_view_topped = FALSE;
			ULOG_DEBUG("[%s]:[%d] : Main view not topped\n", __FILE__, __LINE__);
		}
	} else {
		ULOG_DEBUG("[%s]:[%d] : No views available. Application going down\n", __FILE__,
			   __LINE__);
		/*Unset previous email state, its no longer needed now */
		ui_dbus_disconnect_iap_conn();
		ui_dbus_statesave_write(NULL);
		ui_exit(0);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by higher layers for  registering the callback
	for closing the viewer.
 @param win is a pointer to the widget that registered the callback.
 @param viewer_info is a pointer to the ViewerInfo structure.
   viewer_info memory is freed inside this function by calling ui_free_viewerinfo().
 @return void
*/
void ui_viewer_close_cb(GtkWidget * win, ViewerInfo * viewer_info)
{
	gint folderNum = -1;
	gchar *msgid = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (viewer_info == NULL && viewer_info->mail_info == NULL) {
		return;
	}

	folderNum = viewer_info->mail_info->folder_num;
	msgid = viewer_info->msgid;
	/*fix for bug 19576, updateing window pinters properly,
	 * if its email attachment,windows are updated based on unique, msgnum*/
	if (viewer_info->mail_info->email_attachment == FALSE)
		ui_update_window_pointers(viewer_info->msgid);
	else {
		msgid = g_strdup_printf("%d", viewer_info->mail_info->msgnum);
		ui_update_window_pointers(msgid);
		g_free(msgid);
	}

	if (email_main_view->tags->application_closure == TRUE) {
		/*When appliction goes down we simply destroy the appview.
		 * memory is released once the process is killed.*/
		ui_show_next_window();
		gtk_widget_destroy(viewer_info->viewer_appview);
		return;
	}

	if (viewer_info->mail_info->email_attachment == FALSE) {
		/*Unmark the current row in folder view */
		ui_update_row_tag_in_folder(folderNum, msgid, IN_USE_STATUS, FALSE);
		/* Refreashing unread and unsent message count to folders in folder list view */
		ui_add_folder_name_with_unread_unsent_mails(viewer_info->mail_info->folder_num);
	}
	ui_free_viewerinfo(viewer_info);
	if (win != NULL) {
		ui_show_next_window();
	}
	return;
}

/**
  This function prepares the headerinfo structure required for 
  opening the editor from viewer.

  @param viewer_info is a pointer to the ViewerInfo structure.
    NO memory freeing for viewer_info is done.
  @param download_attachments is the flag representing whether
      the attachments have to be downloaded for the message.
  @return a newly allocted HeaderInfo structure pointer.
*/
HeaderInfo *ui_initialise_headerinfo(const ViewerInfo * viewer_info, gboolean download_attachments)
{
	GtkTextBuffer *bufferFrom = NULL;
	GtkTextBuffer *bufferTo = NULL;
	GtkTextBuffer *bufferCc = NULL;
	GtkTextBuffer *bufferSub = NULL;
	GtkTextIter start, end;
	HeaderInfo *msgHeader = NULL;
	MsgFields *msgfields = NULL;

	if (viewer_info == NULL && viewer_info->msg_fields == NULL) {
		return NULL;
	}
	msgfields = viewer_info->msg_fields;
	msgHeader = g_new0(HeaderInfo, 1);
	bufferFrom = gtk_text_view_get_buffer((GtkTextView *) msgfields->from);
	bufferTo = gtk_text_view_get_buffer((GtkTextView *) msgfields->to);
	bufferCc = gtk_text_view_get_buffer((GtkTextView *) msgfields->cc);
	bufferSub = gtk_text_view_get_buffer((GtkTextView *) msgfields->subject);

	/*get the contents from the various text buffers of the viewer */
	gtk_text_buffer_get_bounds(bufferFrom, &start, &end);
	msgHeader->from = gtk_text_buffer_get_text(bufferFrom, &start, &end, FALSE);
	gtk_text_buffer_get_bounds(bufferTo, &start, &end);
	msgHeader->to = gtk_text_buffer_get_text(bufferTo, &start, &end, FALSE);
	gtk_text_buffer_get_bounds(bufferCc, &start, &end);
	msgHeader->cc = gtk_text_buffer_get_text(bufferCc, &start, &end, FALSE);
	gtk_text_buffer_get_bounds(bufferSub, &start, &end);
	msgHeader->subject = gtk_text_buffer_get_text(bufferSub, &start, &end, FALSE);

	msgHeader->msgid = g_strdup(viewer_info->msgid);
	if (viewer_info->reply_to != NULL)
		msgHeader->reply_to = g_strdup(viewer_info->reply_to);
	msgHeader->priority = viewer_info->mail_info->priority;
	msgHeader->smime_type = viewer_info->mail_info->smime_type;
	if (viewer_info->password != NULL)
		msgHeader->password = g_strdup(viewer_info->password);
	msgHeader->certid = viewer_info->certid;
	msgHeader->rowNum = viewer_info->mail_info->row_num;
	msgHeader->folderNum = viewer_info->mail_info->folder_num;
	msgHeader->sent_date = viewer_info->mail_info->date;
	msgHeader->download_attachments = download_attachments;
	msgHeader->original_in_reply = ui_get_original_in_reply_setting(viewer_info->msgid);
	msgHeader->zoomlevel = viewer_info->zoominfo->previous_level;
	if (viewer_info->mail_info->email_attachment == TRUE) {
		msgHeader->mimeInfo = viewer_info->mime_info;
		/*this path is the body path */
		msgHeader->filename = g_strdup(viewer_info->plain_filename);
		/*this path is the entire email attachment  path */
		msgHeader->email_filename = g_strdup(viewer_info->email_attachment_body_path);
		msgHeader->email_attachment = TRUE;
	}
	return msgHeader;

}

/**
 This function is used to get the mail body from the TextView of the
 viewer.
 @param viewer_info is a pointer to the ViewerInfo structure.
    NO memory freeing for viewer_info is done.
    
 @return a newly allocted string containing the body.
*/
gchar *ui_viewer_get_body(const ViewerInfo * viewer_info)
{
	GtkTextBuffer *bufferBody = NULL;
	GtkTextIter start, end;
	MsgFields *msgfields = NULL;
	gchar *message = NULL;

	if (viewer_info == NULL && viewer_info->msg_fields == NULL) {
		return NULL;
	}
	msgfields = viewer_info->msg_fields;
	if (msgfields->body != NULL) {
		bufferBody = gtk_text_view_get_buffer((GtkTextView *) msgfields->body);
		gtk_text_buffer_get_bounds(bufferBody, &start, &end);
		message = gtk_text_buffer_get_text(bufferBody, &start, &end, FALSE);
	}
	return message;
}

/**
 Checks whether attachments are downloaded are not.
 & sets a flag
 
 @param viewer_info is a pointer to the ViewerInfo structure.
    NO memory freeing for viewer_info is done.

 @return gboolean TRUE-if attachments have to be downloaded
                   FALSE-else 
 */
static gboolean set_download_flag(const ViewerInfo * viewer_info)
{
	MailInfo *mailinfo = NULL;
	gboolean download = TRUE;

	if (viewer_info == NULL && viewer_info->mail_info == NULL) {
		return download;
	}
	mailinfo = viewer_info->mail_info;
	if (mailinfo->protocol == PROTO_IMAP4 && mailinfo->recv_type != BODY_ATTCH) {
		download = FALSE;
	}
	return download;
}

/**
 This function is called by higher layers for  registering the callback
	for implementing the reply functionality.
	
 @param win is a pointer to the widget that registered the callback.
 @param viewer_info is a pointer to the ViewerInfo structure.
    No memory freeing for viewer_info is done.
    
 @return void
*/
void ui_viewer_reply_cb(GtkWidget * win, const ViewerInfo * viewer_info)
{
	GtkWidget *replyWindow = NULL;
	HeaderInfo *msgHeader = NULL;
	gchar *body = NULL;
	gboolean download = TRUE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (viewer_info == NULL) {
		return;
	}
	download = set_download_flag(viewer_info);
	msgHeader = ui_initialise_headerinfo(viewer_info, download);
	body = ui_viewer_get_body(viewer_info);
	replyWindow = (GtkWidget *) ui_editor_create(msgHeader, body, REPLY);
	if (replyWindow != NULL) {
		gtk_widget_show(replyWindow);
		hildon_app_set_appview(app, (HildonAppView *) replyWindow);
	}
	ui_freeHeaderInfo(msgHeader);
	g_free(body);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
    This function  checks the message for any extra part if the mimeinfo
    is not of mixed type
    @param mimeinfo    mimeinfo of the message to check for extra parts
    @return   TRUE if message has attachments.
*/
static void
num_of_attachments_downloaded(MimeInfo * mime_info, gint * num_attachments,
			      gboolean * is_downloaded, gboolean first_partflag)
{
	MimeInfo *tempmimeInfo = NULL;
	gboolean extra_part = FALSE;

	for (tempmimeInfo = mime_info; tempmimeInfo != NULL; tempmimeInfo = tempmimeInfo->next) {
		if (tempmimeInfo->content_type != NULL && ((g_ascii_strcasecmp
							    (tempmimeInfo->content_type,
							     MULTIPART_MIXED) == 0)
							   ||
							   (g_ascii_strcasecmp
							    (tempmimeInfo->content_type,
							     MULTIPART_REPORT) == 0)
							   || (extra_part =
							       check_for_body(tempmimeInfo,
									      first_partflag)) ==
							   TRUE)) {
			num_of_attachments_downloaded(tempmimeInfo->children, num_attachments,
						      is_downloaded, first_partflag);
			if (extra_part == FALSE)
				continue;
		}
		if (first_partflag == TRUE) {
			first_partflag = check_for_body(tempmimeInfo, first_partflag);
			if (first_partflag == FALSE)
				continue;
		}
		(*num_attachments)++;
		if (tempmimeInfo->is_downloaded == FALSE) {
			(*is_downloaded) = FALSE;
		}
	}
	return;
}

/**
 Check if the attachments have to be downloaded & shown the appropriate confirmation
 
 @param mime_info is a pointer to the MimeInfo structure.
    No memory freeing for mime_info is done.
 @param msgid id the gchar representing the msgid of the message.
  
 @return gboolean TRUE-if attachments have to be downloaded
                   FALSE-else 
*/
static gboolean
ui_check_attachments_downloaded(const MimeInfo * mime_info,
				const gchar * msgid, gboolean * confirmation_asked)
{
	gboolean is_downloaded = TRUE;
	gboolean download = TRUE;
	gint num_attachments = 0;

	CHECK_PARAM(mime_info == NULL, "Input param check failed\n", return download);
	num_of_attachments_downloaded((MimeInfo *) mime_info, &num_attachments, &is_downloaded,
				      TRUE);
	if (is_downloaded == FALSE) {
		*confirmation_asked = TRUE;
		download =
		    get_user_confirmation(_("mcen_nc_get_msg_txt"), _("mcen_bd_retrieve"),
					  NULL, CONFIRM_DIALOG);

	}
	return download;
}


/**
 This function is called by higher layers for  registering the callback
  for implementing the forward functionality.
 @param win is a pointer to the widget that registered the callback.
 @param viewer_info is a pointer to the ViewerInfo structure.
    No memory freeing for viewer_info is done.
 @return void
*/
void ui_viewer_forward_cb(GtkWidget * win, ViewerInfo * viewer_info)
{
	gboolean download_attachments = TRUE;
	gboolean confirmation_asked = FALSE;
	HeaderInfo *msgHeader = NULL;
	gchar *body = NULL;
	GtkWidget *forwardWindow = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (viewer_info == NULL) {
		return;
	}
	download_attachments =
	    ui_check_attachments_downloaded(viewer_info->mime_info, viewer_info->msgid,
					    &confirmation_asked);
	if (download_attachments == FALSE) {
		return;
	}
	if (confirmation_asked == TRUE) {
		ui_mimeinfo_free(viewer_info->mime_info);
		viewer_info->mime_info = NULL;
		viewer_info->mode = FORWARDING;
		show_progressbar(viewer_info);
		viewer_info->mime_info =
		    ui_dbus_recv_msg_body(viewer_info->msgid, FALSE, TRUE,
					  viewer_info->senderid, viewer_info->receiverid,
					  viewer_info->mail_info->smime_type, FALSE,
					  viewer_info->certid, viewer_info->password, TRUE, FALSE);
	} else {
		msgHeader = ui_initialise_headerinfo(viewer_info, TRUE);
		body = ui_viewer_get_body(viewer_info);
		forwardWindow = GTK_WIDGET(ui_editor_create(msgHeader, body, FORWARD));
		if (forwardWindow != NULL) {
			gtk_widget_show(forwardWindow);
			hildon_app_set_appview(app, (HildonAppView *) forwardWindow);
		}
		ui_freeHeaderInfo(msgHeader);
		g_free(body);
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by hiyher layers for  registering the callback
   for implementing the replyAll functionality.
   
 @param win is a pointer to the widget that registered the callback.
 @param viewer_info is a pointer to the ViewerInfo structure.
    No memory freeing for viewer_info is done.
    
 @return void
*/
/*@unused@*/
void ui_viewer_replyAll_cb(GtkWidget * win, const ViewerInfo * viewer_info)
{
	GtkWidget *replyAllWindow = NULL;
	HeaderInfo *msgHeader = NULL;
	gchar *body = NULL;
	gboolean download = TRUE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (viewer_info == NULL) {
		return;
	}
	download = set_download_flag(viewer_info);
	msgHeader = ui_initialise_headerinfo(viewer_info, download);
	body = ui_viewer_get_body(viewer_info);
	replyAllWindow = GTK_WIDGET(ui_editor_create(msgHeader, body, REPLY_ALL));
	if (replyAllWindow != NULL) {
		hildon_app_set_appview(app, (HildonAppView *) replyAllWindow);
		gtk_widget_show(replyAllWindow);
	}
	ui_freeHeaderInfo(msgHeader);
	g_free(body);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

gboolean check_for_vcard_attachment(const gchar * attachment)
{
	gchar *extension = NULL;
	gchar *str = NULL;
	if (attachment == NULL)
		return FALSE;
	if ((str = strrchr(attachment, EXT_CHAR)) != NULL) {
		extension = g_strdup(str + 1);
		if (ui_compare_utf8_strings(extension, VCF) == 0) {
			g_free(extension);
			return TRUE;
		}
		g_free(extension);
	}
	return FALSE;
}


/**
 This is a helper function to show a temporary name if the
 attachemnt name is not present in mimeinfo.
 @param mime_info is the MimeInfo structure pointer.
 @return gchar pointer to the attcahment name.
*/
static gchar *get_attachment_name(MimeInfo * mime_info)
{
	gchar *temp_name = NULL, *str = NULL;
	static gint random_num = 0;

	CHECK_PARAM(mime_info == NULL, "param invalid", return NULL);

	if (mime_info->filename != NULL) {
		temp_name = g_strdup(mime_info->filename);
		return temp_name;
	}
	/*if attachment name is null we display a temp name */
	if (mime_info->mime_type == MIME_MESSAGE_RFC822)
		temp_name = g_strdup(_("mail_va_no_subject"));
	else {
		if (mime_info->name != NULL) {
			temp_name = g_strdup(mime_info->name);
		} else {
			str = ui_get_tmp_attach_name();
			temp_name = g_strdup_printf("%s %d", str, random_num++);
			g_free(str);
		}
	}
	mime_info->filename = g_strdup(temp_name);
	return temp_name;
}

/**
 helper function to create a attachemnt row in the liststore.
 
  @parma tempmimeInfo is the MimeInfo structure pointer of the attachment.
  @param liststore is the GtkListStore structure pointer.
  @return void.
*/
static void prepare_attachment_row(MimeInfo * tempmimeInfo, GtkListStore * liststore)
{
	gchar *attachment_name = NULL;
	gchar *sizeOfFile = NULL;
	GtkTreeIter iter;
	GtkIconTheme *icontheme = NULL;
	const gchar *mime_string = NULL;
	gchar *icon_string = NULL;
	GdkPixbuf *pixbuf = NULL;
	GError *error = NULL;
	gint num_of_rows = 0;

	icontheme = gtk_icon_theme_get_default();
	attachment_name = get_attachment_name(tempmimeInfo);

	/*if the encoding is base64 then the attachemnt size is adjusted 
	   for the encoded attachment */
	if (tempmimeInfo->encoding_type == ENC_BASE64)
		tempmimeInfo->size = (BASE64_SIZE_ADJUSTMENT * (tempmimeInfo->size));

	/*calculating the size in bytes/kilobytes */
	sizeOfFile = ui_format_size_value(tempmimeInfo->size);

	/*getting the corresponding icon based on the mime type */
	mime_string = (gchar *) gnome_vfs_mime_type_from_name((const char *)
							      attachment_name);
	if (check_for_vcard_attachment(attachment_name) == TRUE)
		mime_string = VCARD;
	prepare_the_icon_string(mime_string, &icon_string, tempmimeInfo->is_downloaded,
				tempmimeInfo->is_purged);
	pixbuf = gtk_icon_theme_load_icon(icontheme, icon_string, ICON_SIZE, 0, &error);

	gtk_list_store_append(liststore, &iter);
	num_of_rows = gtk_tree_model_iter_n_children((GtkTreeModel *) liststore, NULL);
	gtk_list_store_set(liststore, &iter, COL_ICON,
			   pixbuf, COL_ATT_NAME, attachment_name,
			   COL_ATT_SIZE, sizeOfFile, COL_ATT_NUM, num_of_rows - 1, -1);
	g_free(attachment_name);
	g_free(sizeOfFile);
}

/**
 This is a helper function to check whether the part of the mimeinfo
 is the body part or not.
 @param mimeinfo is the MiemiNfo is the mimeInfo pointer.
 @param first_partflag is the flag representing whether it is the first part
        or not.
@return FALSE --> if its a body part.
        TRUE-else,it has attachments
*/
static gboolean check_for_body(MimeInfo * mimeinfo, gboolean first_partflag)
{
	if (first_partflag == TRUE) {
		/* check for body part */
		if ((mimeinfo->mime_type == MIME_TEXT) ||
		    (mimeinfo->mime_type == MIME_TEXT_HTML) ||
		    ((mimeinfo->mime_type == MIME_MULTIPART) &&
		     ((g_ascii_strncasecmp
		       (mimeinfo->content_type, MULTIPART_ALTERNATIVE,
			MAX_STR_LEN(mimeinfo->content_type, MULTIPART_ALTERNATIVE)) == 0)
		      || (g_ascii_strncasecmp(mimeinfo->content_type, MULTIPART_RELATED,
					      MAX_STR_LEN(mimeinfo->content_type,
							  MULTIPART_RELATED)) == 0)))) {
			/* This is body part,so dont add as attachment */
			first_partflag = FALSE;
		}
	}
	return first_partflag;
}

/**
  Helper function to hget the attachemnts from the mimeinfo of the mail.
  @parma imeInfo is the MimeInfo structure pointer of the attachment.
  @param liststore is the GtkListStore structure pointer.
  @param numAttachments is the OUT param rep the number of attachments.
  @param first_partflag is the gboolean representing the whether it is a body part or not.
   
  @return void.
*/
void
display_attachments(MimeInfo * mimeInfo, GtkListStore * liststore, gint * numAttachments,
		    gboolean first_partflag, gboolean email_attachment)
{
	MimeInfo *tempmimeInfo = NULL;
	tempmimeInfo = mimeInfo;
	gboolean extra_part = FALSE;

	for (tempmimeInfo = mimeInfo; tempmimeInfo != NULL; tempmimeInfo = tempmimeInfo->next) {
		if ((tempmimeInfo->content_type != NULL && ((g_ascii_strcasecmp
							     (tempmimeInfo->content_type,
							      MULTIPART_MIXED) == 0)
							    ||
							    (g_ascii_strcasecmp
							     (tempmimeInfo->content_type,
							      MULTIPART_REPORT) == 0)))
		    || (extra_part = check_for_body(tempmimeInfo, first_partflag)) == TRUE) {
			display_attachments(tempmimeInfo->children, liststore, numAttachments,
					    first_partflag, email_attachment);
			if (extra_part == FALSE)
				continue;
		}
		if (first_partflag == TRUE) {
			first_partflag = check_for_body(tempmimeInfo, first_partflag);
			if (first_partflag == FALSE)
				continue;
		}
		(*numAttachments)++;
		if (liststore != NULL) {
			/*for email attachment when mail is downloaded the
			 * attachments are also downloaded*/
			if (email_attachment == TRUE)
				tempmimeInfo->is_downloaded = TRUE;
			prepare_attachment_row(tempmimeInfo, liststore);
		}
	}
}

/**
 This function checks for the attachments for the viewer
 
 @param mimeInfo is the MimeInfo pointer for the current viewer.
    No memory freeing for mimeInfo is done.
 @param view is the Gtkwidget for the attachments.
 
 @return gboolean TRUE-if attachments are present
             else FALSE.
 */
gboolean
ui_viewer_check_attachments(const MimeInfo * mimeinfo,
			    GtkListStore * liststore, GtkWidget * treeview,
			    gboolean email_attachment)
{
	MimeInfo *tempmimeInfo = NULL;
	gboolean status = FALSE;
	GtkIconTheme *icontheme = NULL;
	gint numAttachments = 0;
	GtkTreeIter iter;


	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	icontheme = gtk_icon_theme_get_default();
	CHECK_PARAM(mimeinfo == NULL, "no attachments", return FALSE);

	tempmimeInfo = (MimeInfo *) mimeinfo;
	display_attachments(tempmimeInfo, liststore, &numAttachments, TRUE, email_attachment);

	if (numAttachments != 0) {
		status = TRUE;
		gtk_tree_model_get_iter_first((GtkTreeModel *) liststore, &iter);
	} else
		return status;

	gtk_widget_set_size_request(treeview, -1, ATTA_FIELDS_SIZE * numAttachments);


	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return status;
}

/**
 This is a helper function to activate the appropriate menu item upon attachment
 selection.
 @param viewer_info is a pointer to the ViewerInfo structure.
    No memory freeing for viewer_info is done.
 @result void.
*/
static void ui_activate_attachment_menus(const ViewerInfo * viewer_info)
{
	ViewerMenu *menuitem = NULL;
	ViewerToolbar *toolbutton = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(viewer_info == NULL, "Input param check failed\n", return);
	CHECK_PARAM(viewer_info->menu == NULL, "Input param check failed\n", return);
	CHECK_PARAM(viewer_info->toolbar == NULL, "Input param check failed\n", return);

	menuitem = viewer_info->menu;
	toolbutton = viewer_info->toolbar;
	if (toolbutton->saveAttachment != NULL)
		gtk_widget_set_sensitive(toolbutton->saveAttachment, TRUE);
	if (menuitem->save_attachment != NULL)
		gtk_widget_set_sensitive(menuitem->save_attachment, TRUE);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;

}

/**
 get the next mimeinfo including the sub part.
  given mimeinfo
  
 @param mimeinfo is the MimeInfo structure pointer of the current viewer.
    No memory freeing for mimeinfo is done.calling function shld free the memory.
    
 @return the next mimeInfo structure pointer.
*/
static MimeInfo *mimeinfo_next(MimeInfo * mimeinfo)
{
	if (mimeinfo == NULL) {
		return NULL;
	}
	if (mimeinfo->children) {
		return mimeinfo->children;
	}
	if (mimeinfo->sub) {
		return mimeinfo->sub;
	}
	if (mimeinfo->next) {
		return mimeinfo->next;
	}
	if (mimeinfo->main) {
		mimeinfo = mimeinfo->main;
		if (mimeinfo->next) {
			return mimeinfo->next;
		}
	}
	for (mimeinfo = mimeinfo->parent; mimeinfo != NULL; mimeinfo = mimeinfo->parent) {
		if (mimeinfo->next) {
			return mimeinfo->next;
		}
		if (mimeinfo->main) {
			mimeinfo = mimeinfo->main;
			if (mimeinfo->next) {
				return mimeinfo->next;
			}
		}
	}
	return NULL;
}

/**
 helper function to get the attachment given the attchment seq number.

 @param mimeInfo is the MimeInfo structure pointer.
 @param attachment_seqno is the gint representing the attchment seq number.
 @param first_partflag is the gboolean rep the body part/or not.
 @return the mimeInfo structure pointer of the attachment.
*/
static MimeInfo *get_attachment(MimeInfo * mimeInfo, gint attachment_seqno, gboolean first_partflag)
{
	MimeInfo *mimeinfo = NULL;
	MimeInfo *req_mimeinfo = NULL;
	gint nth_loop = -1;
	gboolean extra_part = FALSE;

	mimeinfo = mimeInfo;

	for (mimeinfo = mimeInfo; mimeinfo != NULL; mimeinfo = mimeinfo->next) {
		if (mimeinfo->content_type != NULL && ((g_ascii_strcasecmp
							(mimeinfo->content_type,
							 MULTIPART_MIXED) == 0)
						       ||
						       (g_ascii_strcasecmp
							(mimeinfo->content_type,
							 MULTIPART_REPORT) == 0)
						       || (extra_part =
							   check_for_body(mimeinfo, first_partflag))
						       == TRUE)) {

			req_mimeinfo =
			    get_attachment(mimeinfo->children, attachment_seqno, first_partflag);
			if (req_mimeinfo != NULL)
				return req_mimeinfo;
			if (extra_part == FALSE)
				continue;
		}
		if (first_partflag == TRUE) {
			first_partflag = check_for_body(mimeinfo, first_partflag);
			if (first_partflag == FALSE)
				continue;
		}
		nth_loop++;
		if (nth_loop == attachment_seqno) {
			req_mimeinfo = mimeinfo;
			return req_mimeinfo;
		}
	}
	return req_mimeinfo;
}

/**
 This function get the required mimeinfo for a given attachment.
 @param nth_attach is the gint representing the attachment sequence number.
 @param mime_info is the MimeInfo structure poinetr.
 @return Mimeinfo structure pointer.
*/
static MimeInfo *get_mimeinfo_for_attachment(const MimeInfo * mime_info, gint attachment_seqno)
{
	MimeInfo *mimeinfo = NULL;

	mimeinfo = get_attachment((MimeInfo *) mime_info, attachment_seqno, TRUE);

	return mimeinfo;
}

/**
  This function is used to check  the attachment type & 
  there by enable or disable the view/save buttons.
 @param viewer_info is a pointer to the ViewerInfo structure.
    No memory freeing for viewer_info is done.
 @return TRUE- if it is not email type attachment.
         FALSE- else 
*/
static gint ui_check_attachment_type(const ViewerInfo * viewer_info)
{

	MsgFields *msgfields = NULL;
	MimeInfo *mimeinfo = NULL;
	MimeInfo *tempMimeinfo = NULL;
	GtkTreeSelection *selection = NULL;
	GList *pathslist = NULL;
	gint email_status = NORMAL_ATTACH;
	gint nth = 0, nth_attach = -1;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(viewer_info == NULL, "Input param check failed\n", return TRUE);
	CHECK_PARAM(viewer_info->msg_fields == NULL, "Input param check failed\n", return TRUE);

	msgfields = viewer_info->msg_fields;
	nth_attach = ui_get_attachment_num(msgfields, 0);
	mimeinfo = viewer_info->mime_info;

	CHECK_PARAM(mimeinfo == NULL, "Input param check failed\n", return TRUE);

	selection = gtk_tree_view_get_selection((GtkTreeView *) viewer_info->msg_fields->view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, NULL);
	while (pathslist != NULL && pathslist->data != NULL) {
		nth_attach = ui_get_attachment_num(viewer_info->msg_fields, nth);
		nth++;
		tempMimeinfo = get_mimeinfo_for_attachment(mimeinfo, nth_attach);
		if (tempMimeinfo != NULL) {
			if (tempMimeinfo->mime_type == MIME_MESSAGE_RFC822) {
				email_status = EMAIL_ATTACH;
				break;
			} else if (tempMimeinfo->is_downloaded == FALSE) {
				email_status = ATTACH_NOTRETRIEVED;
			}
		}
		pathslist = pathslist->next;
	}
	g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(pathslist);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return email_status;
}

/**
 helper function to get check whether its the last attchment to be downloaded.

 @param mime is the MimeInfo structure pointer.
 @param count is the gint OUT param for number of attachments not downloaded.
 @param first_partflag is the gboolean rep the body part/or not.
 @return  void.
*/
static void check_if_last_attachment(MimeInfo * mime, gint * count, gboolean first_partflag)
{
	MimeInfo *tempMimeinfo = NULL;
	gboolean extra_part = FALSE;

	for (tempMimeinfo = mime; tempMimeinfo != NULL; tempMimeinfo = tempMimeinfo->next) {
		if (tempMimeinfo->content_type != NULL && ((g_ascii_strcasecmp
							    (tempMimeinfo->content_type,
							     MULTIPART_MIXED) == 0)
							   ||
							   (g_ascii_strcasecmp
							    (tempMimeinfo->content_type,
							     MULTIPART_REPORT) == 0)
							   || (extra_part =
							       check_for_body(tempMimeinfo,
									      first_partflag)) ==
							   TRUE)) {
			check_if_last_attachment(tempMimeinfo->children, count, first_partflag);
			if (extra_part == FALSE)
				continue;
		}
		/* check for body part &skip the body part */
		if (first_partflag == TRUE) {
			first_partflag = check_for_body(tempMimeinfo, first_partflag);
			if (first_partflag == FALSE)
				continue;
		}

		if (tempMimeinfo->is_downloaded == FALSE) {
			(*count)++;
		}
	}
}


/**
 This function checks if the attachment to be downloaded is the last attachment
 @param mimeinfo is the MimeInfo pointer.
    No memory freeing for mimeinfo is done.
 @return TRUE-if it is lst attachment.
        FALSE-else.
 */
static gboolean last_attachment_tobe_downloaded(const MimeInfo * mimeinfo)
{
	MimeInfo *tempMimeinfo = NULL;
	MimeInfo *root_mime_info = NULL;
	gint count = 0;
	gboolean last_attachment = FALSE;

	root_mimeinfo((MimeInfo *) mimeinfo, &root_mime_info);

	if (root_mime_info != NULL)
		tempMimeinfo = root_mime_info;
	else
		tempMimeinfo = (MimeInfo *) mimeinfo;

	check_if_last_attachment(tempMimeinfo, &count, TRUE);

	if (count == 0) {
		last_attachment = TRUE;
	}
	return last_attachment;
}

/**
 This function is used to check whether the attachments are dowmloaded or not
 and accordingly enable/disable the purge attachments menu.
 
 @param viewer_info is the VieweInfo structure pointer.
 @param selected_attach is the list of selected attachemnts.
 @return void.
 */
static void check_attachment_downloaded(ViewerInfo * viewer_info, GSList * selected_attach)
{
	MimeInfo *tempMimeinfo = NULL;
	GSList *selected_names = NULL;
	GtkTreeSelection *selection = NULL;
	GList *pathslist = NULL;
	gboolean attachment_downloaded = FALSE;
	//gint *indices = NULL;
	gint nth = 0, nth_attach = -1, nth_loop = -1;

	if (viewer_info->mail_info->smime_type != SMIME_NONE) {
		return;
	}
	CHECK_PARAM(viewer_info->mime_info == NULL, "no attachments", return);


	selected_names = selected_attach;
	selection = gtk_tree_view_get_selection((GtkTreeView *) viewer_info->msg_fields->view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, NULL);
	while (selected_names != NULL && selected_names->data != NULL) {
		nth_attach = ui_get_attachment_num(viewer_info->msg_fields, nth);
		nth++;
		tempMimeinfo = get_mimeinfo_for_attachment(viewer_info->mime_info, nth_attach);
		if (tempMimeinfo != NULL && tempMimeinfo->is_downloaded == TRUE) {
			attachment_downloaded = TRUE;
		}
		nth_loop = -1;
		selected_names = selected_names->next;
	}
	g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(pathslist);
	if (attachment_downloaded == TRUE) {
		if (viewer_info->mail_info->folder_num != DRAFT_MODEL_NUM &&
		    viewer_info->mail_info->folder_num != OUTBOX_MODEL_NUM) {
			if (((viewer_info->mail_info->protocol == PROTO_IMAP4)
			     && (viewer_info->mail_info->folder_num == INBOX_MODEL_NUM)) ||
			    (viewer_info->mail_info->folder_num != INBOX_MODEL_NUM)) {
				if (viewer_info->mail_info->smime_type == SMIME_NONE &&
				    viewer_info->mail_info->email_attachment == FALSE)
					gtk_widget_set_sensitive(viewer_info->menu->
								 purge_attachment, TRUE);
				else
					gtk_widget_set_sensitive(viewer_info->menu->
								 purge_attachment, FALSE);
			} else {
				gtk_widget_set_sensitive(viewer_info->menu->purge_attachment,
							 FALSE);
			}
		} else {
			gtk_widget_set_sensitive(viewer_info->menu->purge_attachment, FALSE);
		}

	} else {
		gtk_widget_set_sensitive(viewer_info->menu->purge_attachment, FALSE);
	}

	return;
}

/**
 Function to update the mimeinfo once entire message is downloaded with attachments.
 only called when a email attachment is being viewed.The mime_info points to the sub
 part .
 
 @param viewer_info is the pointer to the ViewerInfo structure pointer.
    No memory freeing for viewer_info is done.
 @param filename is the name of the attachment.
 
 @return void.
*/
void assing_new_mimeinfo(ViewerInfo * viewer_info, const MimeInfo * mime_info)
{
	MimeInfo *new_mimeinfo = NULL;

/*for nested attachments going to the sub part from the root mimeinfo */
	if (viewer_info->mail_info->email_attachment == TRUE) {
		new_mimeinfo = get_sub_mimeinfo((MimeInfo *) mime_info, viewer_info);
		ui_mimeinfo_free(viewer_info->mime_info);
		viewer_info->mime_info = new_mimeinfo;
	}
	return;
}

/**
 This is a callback function for selection of attachments 
 
 @param selection is a GtkTreeSelection that contains the attachments.
 @param viewer_info is a pointer to the ViewerInfo structure.
    No memory freeing for viewer_info is done.
    
 @return GSList of  attachments selected.
 */
GSList *ui_attachment_selection_cb(GtkTreeSelection * eselection, ViewerInfo * viewer_info)
{
	GSList *selected_names = NULL;
	gint not_email = FALSE;
	MsgFields *msgfields = NULL;
	MailInfo *mailinfo = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM(viewer_info == NULL, "Input param check failed\n", return NULL);
	CHECK_PARAM(viewer_info->msg_fields == NULL, "Input param check failed\n", return NULL);

	selected_names = ui_get_selected_attachment_name(viewer_info->msg_fields);
	if (selected_names == NULL) {
		return NULL;
	}
	msgfields = viewer_info->msg_fields;
	mailinfo = viewer_info->mail_info;

	/*check for email type attachment */
	not_email = ui_check_attachment_type(viewer_info);
	if (not_email != EMAIL_ATTACH) {
		ui_activate_attachment_menus(viewer_info);
	} else if (not_email == EMAIL_ATTACH) {
		gtk_widget_set_sensitive(viewer_info->menu->save_attachment, FALSE);
		gtk_widget_set_sensitive(viewer_info->toolbar->saveAttachment, FALSE);
	} else if (not_email == ATTACH_NOTRETRIEVED) {
		gtk_widget_set_sensitive(viewer_info->menu->purge_attachment, FALSE);
	}
	if (not_email == NORMAL_ATTACH) {
		if (viewer_info->mail_info->folder_num != DRAFT_MODEL_NUM &&
		    viewer_info->mail_info->folder_num != OUTBOX_MODEL_NUM) {
			if (((viewer_info->mail_info->protocol == PROTO_IMAP4)
			     && (viewer_info->mail_info->folder_num == INBOX_MODEL_NUM)) ||
			    (viewer_info->mail_info->folder_num != INBOX_MODEL_NUM)) {
				if (viewer_info->mail_info->smime_type == SMIME_NONE &&
				    viewer_info->mail_info->email_attachment == FALSE)
					gtk_widget_set_sensitive(viewer_info->menu->
								 purge_attachment, TRUE);
				else
					gtk_widget_set_sensitive(viewer_info->menu->
								 purge_attachment, FALSE);
			}else {
                                gtk_widget_set_sensitive(viewer_info->menu->
                                                         purge_attachment, FALSE);
                        }
		}
	}
	/*checks the is_downloaded flag & sets the purge attachment menu enabled
	 * is its downloaded.*/
	if (not_email != EMAIL_ATTACH)
		check_attachment_downloaded(viewer_info, selected_names);

	gtk_widget_set_sensitive(viewer_info->menu->view, TRUE);
	gtk_widget_set_sensitive(viewer_info->menu->copy, FALSE);
	g_slist_foreach(selected_names, (GFunc) g_free, NULL);
	g_slist_free(selected_names);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return NULL;
}

/**
 Function prepares the attachinfo structure for forwarding/outbox/draft mails.
 
 @param msgType is a gint representing the mode in which editor is opened.
 @param composeInfo is the MsgInfo pointer returned from the 
 	ui_dbus_recv_msg_attachment call.It is not freed in this function.
 @param tempmimeInfo is the pointer to the MimeInfo of the editor.i
 	It is not freed in this function
 @param MsgInfo is the MsgInfo pointer of the current editor which will be send 
 	to the engine while sending the mail.It is not freed in this function
 
 @result gboolean FALSE,if memory allocation fails or
                                input paramter validation fails.
                  TRUE,else.
*/
static gboolean
ui_editor_prepare_attachinfo(gint msgType, const MsgInfo * composeInfo,
			     gboolean rfc822, MsgInfo * msgInfo)
{
	GSList *attachmentList = NULL;
	AttachInfo *attachmentInfo = NULL;
	AttachInfo *forwardAttachments = NULL;
	gchar *str = NULL;
	static gint random_num = 0;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (msgType == FORWARD_FROM_FOLDER || msgType == FORWARD
	    || msgType == OUTBOX_MAIL || msgType == DRAFT_MAIL) {
		if (composeInfo == NULL && composeInfo->attach_list == NULL) {
			ULOG_DEBUG
			    ("either composeinfo /composeinfo->aattachlist is NULL \n ends here ");
			return FALSE;
		}
		attachmentList = composeInfo->attach_list;
		attachmentInfo = (AttachInfo *) attachmentList->data;
		if (attachmentInfo != NULL) {
			forwardAttachments = g_new0(AttachInfo, 1);
			if (rfc822 == TRUE) {
				forwardAttachments->type = EMAIL;
				forwardAttachments->attachmentname =
				    g_strdup(composeInfo->header->subject);
			} else {
				forwardAttachments->type = attachmentInfo->type;
				if (attachmentInfo->attachmentname != NULL)
					forwardAttachments->attachmentname =
					    g_strdup(attachmentInfo->attachmentname);
				else {
					str = ui_get_tmp_attach_name();
					forwardAttachments->attachmentname =
					    g_strdup_printf("%s %d", str, random_num++);
					g_free(str);
				}
			}
			forwardAttachments->filepath = g_strdup(attachmentInfo->filepath);
			forwardAttachments->email_path = g_strdup(attachmentInfo->email_path);
			forwardAttachments->msgid = g_strdup(attachmentInfo->msgid);
			msgInfo->attach_list =
			    g_slist_append(msgInfo->attach_list, forwardAttachments);
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return TRUE;
}

/**
 This is a helper function to get the size & name of attachments.
 
 @param tempmimeInfo is a MimeInfo structure pointer.It is not freed in this function
 @param composeInfo is the MsgInfo pointer used sending for the mail.
    It is not freed in this function
 @param attachment_view is the Widget where the attachments are placed.

 @return void.
*/
static void
ui_attachment_get_size_name(gboolean rfc822, const MsgInfo * composeInfo, GtkListStore * liststore)
{
	gchar *sizeOfFile = NULL, *str = NULL;
	AttachInfo *attachinfo = NULL;
	static gint random_num = 0;

	attachinfo = composeInfo->attach_list->data;
	if (rfc822 == FALSE) {
		sizeOfFile = ui_file_size(attachinfo->filepath);
		if (attachinfo->attachmentname == NULL) {
			str = ui_get_tmp_attach_name();
			attachinfo->attachmentname = g_strdup_printf("%s %d", str, random_num++);
			g_free(str);
		}
		ui_show_attachments_in_buffer(liststore, attachinfo->attachmentname,
					      attachinfo->filepath, sizeOfFile, FALSE);
	} else if (composeInfo != NULL && composeInfo->header != NULL) {
		sizeOfFile = ui_file_size(attachinfo->email_path);
		if (composeInfo->header->subject == NULL)
			composeInfo->header->subject = g_strdup(_("mail_va_no_subject"));
		ui_show_attachments_in_buffer(liststore,
					      composeInfo->header->subject,
					      composeInfo->header->subject, sizeOfFile, FALSE);
	}


	g_free(sizeOfFile);
	return;
}

/**
 This function is used to prepare the attachments names to be inserted
 in the reply to a mail.

 @param msgType is the mode in which the mail is opened.
 @param ReAttachList is the OUT param containing the attachment names.
 @param tempmimeInfo is the mimeinfo pointer.It is not freed in this function.
    caller has to free this memory.
 
 @return void.
*/
static void
ui_prepare_attachment_list(gint msgType, GSList ** ReAttachList, const MimeInfo * tempmimeInfo)
{
	if (msgType == REPLY_FROM_FOLDER || msgType == REPLY || msgType == REPLY_ALL
	    || msgType == REPLY_ALL_FROM_FOLDER) {
		*ReAttachList = g_slist_append(*ReAttachList, g_strdup(tempmimeInfo->filename));
	}
	return;
}

/**
 This function is used to get update the editor with attachments.

 @param editor is the editor appview pointer.
 @param editor_info is the pointer to the EditorInfo structure pointer..
    It is not freed in this function
 @return void.
*/

void ui_editor_msginfo_thr_cb(GtkWidget * editor, EditorInfo * editor_info)
{
	MsgInfo *msg_info = NULL;
	GSList *msginfo_list = NULL;
	gboolean rfc822 = FALSE;
	msginfo_list = email_main_view->msginfo_list;

	stop_editor_from_recieving_events(editor_info, FALSE);
	gtk_banner_close((GtkWindow *) app);
	if (email_main_view->msginfo_list == NULL && email_main_view->msginfo == NULL)
		return;
	while (msginfo_list != NULL) {
		msg_info = (MsgInfo *) msginfo_list->data;
		if (msg_info->attach_list != NULL && msg_info->attach_list->data != NULL) {
			if (((AttachInfo *) (msg_info->attach_list->data))->type == EMAIL)
				rfc822 = TRUE;
			else
				rfc822 = FALSE;
		} else {
			msginfo_list = msginfo_list->next;
			continue;
		}

		ui_editor_prepare_attachinfo(editor_info->composeType,
					     (MsgInfo *) msginfo_list->data, rfc822,
					     editor_info->msgInfo);
		ui_attachment_get_size_name(rfc822, (MsgInfo *) msginfo_list->data,
					    editor_info->msgFields->liststore);
		msginfo_list = msginfo_list->next;
	}
	/*free the list of msginfos */
	if (email_main_view->msginfo_list != NULL) {
		g_slist_foreach(email_main_view->msginfo_list, (GFunc) ui_freeMem, NULL);
		g_slist_free(email_main_view->msginfo_list);
		email_main_view->msginfo_list = NULL;
	}

	set_attachment_view_size(editor_info->msgInfo->attach_list, editor_info->msgFields);
	show_attachment_field(editor_info->msgFields, TRUE);
	return;

}

/**
 * Helper function to prepare the attachInfo structure for the attachments
 * while forward/draft/outbox mails.
 * @param msgid is teh gchar representing the msgid of the mail.
 * @param mimeinfo is the MiemInfo structure pointer.
 * @param liststore is the GtkListstore pointer.
 * @param msgType is the gint representing the mode in which the mail is opened.
 * @param msgInfo is teh MsgInfo structure pointer of the editor.
 * @param ReAttachList is the OUT param to the list of attachment names.
 * @param mimeinfo_list is the OUT param with the list of attachemnt mimeInfos.
 * @return a enum AttachmentStatus whether
 * NO_ATTACHMNETS
 * ATTACHMENTS_PRESENT
 * MEM_FAILED.
*/
static AttachmentStatus
fill_attachinfo(const gchar * msgid, MimeInfo * mimeinfo, GtkListStore * liststore, gint msgType,
		MsgInfo * msgInfo, GSList ** ReAttachList, GSList ** mimeinfo_list)
{

	if (msgType == FORWARD_FROM_FOLDER || msgType == FORWARD
	    || msgType == OUTBOX_MAIL || msgType == DRAFT_MAIL) {
		/*Checking whether the attachments have to be downloaded if they are
		   not retrived earlier. */
		*mimeinfo_list = g_slist_append(*mimeinfo_list, mimeinfo);
		return ATTACHMENTS_PRESENT;
	} else {
		ui_prepare_attachment_list(msgType, ReAttachList, mimeinfo);
	}
	return NO_ATTACHMENTS;
}

/**
 Helper function to show attachments in editor
 while forward/draft/outbox mails.
 
  @param mimeinfo is the MiemInfo structure pointer.
  @param msgid is teh gchar representing the msgid of the mail.
  @param liststore is the GtkListstore pointer.
  @param msgType is the gint representing the mode in which the mail is opened.
  @param msgInfo is teh MsgInfo structure pointer of the editor.
  @param ReAttachList is the OUT param to the list of attachment names.
  @param mimeinfo_list is the OUT param with the list of attachemnt mimeInfos.
  @return a enum AttachmentStatus whether
  NO_ATTACHMNETS
  ATTACHMENTS_PRESENT
  MEM_ALLOCATION_FAIL
*/
static AttachmentStatus
show_attachments_in_editor(MimeInfo * mime, const gchar * msgid, GtkListStore * liststore,
			   gint msgType, MsgInfo * msgInfo,
			   GSList ** ReAttachList, gboolean first_partflag, GSList ** mimeinfo_list)
{
	MimeInfo *tempmimeInfo = NULL;
	static AttachmentStatus status = NO_ATTACHMENTS;
	gboolean extra_part = FALSE;

	/*re initailising the status value b4 returning */
	if (msgInfo->attach_list == NULL) {
		status = NO_ATTACHMENTS;
	}

	for (tempmimeInfo = mime; tempmimeInfo != NULL; tempmimeInfo = tempmimeInfo->next) {
		if (tempmimeInfo->content_type != NULL && ((g_ascii_strcasecmp
							    (tempmimeInfo->content_type,
							     MULTIPART_MIXED) == 0)
							   ||
							   (g_ascii_strcasecmp
							    (tempmimeInfo->content_type,
							     MULTIPART_REPORT) == 0)
							   || (extra_part =
							       check_for_body(tempmimeInfo,
									      first_partflag)) ==
							   TRUE)) {
			if (show_attachments_in_editor
			    (tempmimeInfo->children, msgid, liststore, msgType, msgInfo,
			     ReAttachList, first_partflag, mimeinfo_list) == MEM_ALLOCATION_FAIL)
				return MEM_ALLOCATION_FAIL;
			if (extra_part == FALSE)
				continue;
		}
		/* check for body part &skip the body part */
		if (first_partflag == TRUE) {
			first_partflag = check_for_body(tempmimeInfo, first_partflag);
			if (first_partflag == FALSE)
				continue;
		}

		status = fill_attachinfo(msgid, tempmimeInfo, liststore, msgType, msgInfo,
					 ReAttachList, mimeinfo_list);
		if (status == MEM_ALLOCATION_FAIL) {
			return status;
		}
	}
	return status;
}

/**
This functions checks for any attachments to be shown in viewer.

 @param mimeInfo is a pointer to the MimeInfo structure.It is not freed in this function
 @param msgid is a gchar pointer to the msgid.
 @param liststore is the  attachments field.
 @param msgInfo is the MsgInfo pointer used for sending.It is not freed in this function
 @param msgType is a gint representing the mode in which editor is opened.
 @param download_attachment is the gboolean to determine whether
	the attachments have to be downloaded or not.
 @param ReAttachList is the OUT param containing 
	list of attachments names if any, while replying to a mail.

 @return ATTACHMENTS_PRESENT  -if attachments are there.
        NO_ATTACHMENTS-if no attachments    FALSE 
        MEM_ALLOCATION_FAIL -if memory allocation fails.
*/
AttachmentStatus
ui_viewer_check_for_attachments(const MimeInfo * mimeinfo, const gchar * msgid,
				GtkListStore * liststore, MsgInfo * msgInfo,
				gint msgType, gboolean download_attachment, GSList ** ReAttachList)
{
	AttachmentStatus status = NO_ATTACHMENTS;
	GSList *mimeinfo_list = NULL, *node = NULL;
	GSList *seqno_list = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM(mimeinfo == NULL, "no attachments", return NO_ATTACHMENTS);

	status =
	    show_attachments_in_editor((MimeInfo *) mimeinfo, msgid, liststore, msgType, msgInfo,
				       ReAttachList, TRUE, &mimeinfo_list);
	node = mimeinfo_list;
	while (node) {
		seqno_list =
		    g_slist_append(seqno_list,
				   GINT_TO_POINTER(email_main_view->tags->window_count * -1));
		node = node->next;
	}
	if (mimeinfo_list != NULL) {
		gtk_banner_show_animation((GtkWindow *) app, _("mail_ib_opening_draft_message"));
		ui_flush_gtk_events();
		ui_dbus_recv_msg_multiple_attachments(mimeinfo_list, seqno_list, msgid);
	}
	g_slist_free(seqno_list);
	g_slist_free(mimeinfo_list);

	return status;
}

/**
 This function is used for toggling the toolbar from editor/viewer.
 
 @param toolbarvisible is the flag representing whether the toolbar is visible or not.
 @param appview is the editor/viewer appview.
 @param app_toolbar is the temporary toolbar pointer for holding the toolbar when it is not
  visible.

 @return void.
*/

void ui_toggle_toolbar(gboolean toolbar_hidden, HildonAppView * appview, GtkWidget * app_toolbar)
{
	if (toolbar_hidden == FALSE) {
		gtk_widget_hide_all(app_toolbar);
	} else if (app_toolbar != NULL) {
		gtk_widget_show_all(app_toolbar);
	}
	return;
}


/**
 This function is called by higher layers for  registering the callback
  to enable/disable the toolbar.
  
 @param win is a pointer to the widget that registered the callback.
 @param editorInfo is a pointer to the editorInfo of the current editor.
   It is not freed in this function

 @return void
*/
void ui_toggle_editor_toolbar_cb(GtkWidget * win, EditorInfo * editorInfo)
{
	HildonAppView *appview = NULL;
	gboolean toolbar_hidden = FALSE;
	gboolean fullscreen_status = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (editorInfo == NULL) {
		return;
	}

	appview = (HildonAppView *) editorInfo->editor_appview;
	toolbar_hidden = ui_get_editor_toolbar_hidden_status();
	fullscreen_status = hildon_appview_get_fullscreen(appview);
	editorInfo->toolbarhidden_normalview = !toolbar_hidden;
	ui_set_editor_toolbar_hidden_status(!toolbar_hidden);

	if (fullscreen_status == 0) {
		ui_toggle_toolbar(toolbar_hidden, appview, (editorInfo->editor_toolbar));
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by higher layers for  registering the callback
  to enable/disable the toolbar.
  
 @param win is a pointer to the widget that registered the callback.
 @param editorInfo is a pointer to the editorInfo of the current editor.
   It is not freed in this function

 @return void
*/
void ui_toggle_editor_fullscreen_toolbar_cb(GtkWidget * win, EditorInfo * editorInfo)
{
	HildonAppView *appview = NULL;
	gboolean toolbar_hidden = FALSE;
	gboolean fullscreen_status = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (editorInfo == NULL) {
		return;
	}
	appview = (HildonAppView *) editorInfo->editor_appview;
	toolbar_hidden = ui_get_editor_fullscreen_toolbar_hidden_status();
	fullscreen_status = hildon_appview_get_fullscreen(appview);
	editorInfo->toolbarhidden_fullscreen = !toolbar_hidden;

	ui_set_editor_fullscreen_toolbar_hidden_status(!toolbar_hidden);
	/*editor is not in fullscreen mode */
	if (fullscreen_status == 0) {
		return;
	}

	ui_toggle_toolbar(toolbar_hidden, appview, (editorInfo->editor_toolbar));
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by higher layers for  registering the callback
    to enable/disable the toolbar.
    
 @param win is a pointer to the widget that registered the callback.
 @param viewerInfo is a pointer to the editorInfo of the current viewer.
    It is not freed in this function
    
 @return void
*/
void ui_toggle_viewer_toolbar_cb(GtkWidget * win, ViewerInfo * viewerInfo)
{
	HildonAppView *appview = NULL;
	gboolean toolbar_hidden = FALSE;
	gboolean fullscreen_status = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (viewerInfo == NULL && viewerInfo->viewer_appview == NULL) {
		return;
	}

	appview = (HildonAppView *) viewerInfo->viewer_appview;
	toolbar_hidden = ui_get_viewer_toolbar_hidden_status();
	fullscreen_status = hildon_appview_get_fullscreen(appview);

	if (fullscreen_status == 0) {
		ui_toggle_toolbar(toolbar_hidden, appview, (viewerInfo->viewer_toolbar));
	}

	toolbar_hidden = !toolbar_hidden;
	ui_set_viewer_toolbar_hidden_status(toolbar_hidden);
	viewerInfo->toolbarhidden_normalview = toolbar_hidden;
	/*bug fix for 19734--> showing progress if present after toggling */
	if (viewerInfo->mail_info->progress_on == TRUE) {
		/*stoping the progress  */
		if (viewerInfo->mail_info->timer_value > 0) {
			g_source_remove(viewerInfo->mail_info->timer_value);
			viewerInfo->mail_info->timer_value = 0;
		}
		show_progressbar(viewerInfo);
		update_progress_bar_text(viewerInfo->viewer_appview, viewerInfo);
	} else {
		hide_progressbar(viewerInfo);
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called by higher layers for  registering the callback
    to enable/disable the toolbar.
    
 @param win is a pointer to the widget that registered the callback.
 @param viewerInfo is a pointer to the editorInfo of the current viewer.
    It is not freed in this function
    
 @return void
*/
void ui_toggle_viewer_fullscreen_toolbar_cb(GtkWidget * win, ViewerInfo * viewerInfo)
{
	HildonAppView *appview = NULL;
	gboolean toolbar_hidden = FALSE;
	gboolean fullscreen_status = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (viewerInfo == NULL && viewerInfo->viewer_appview == NULL) {
		return;
	}

	appview = (HildonAppView *) viewerInfo->viewer_appview;
	toolbar_hidden = ui_get_viewer_fullscreen_toolbar_hidden_status();
	fullscreen_status = hildon_appview_get_fullscreen(appview);

	if (fullscreen_status != FALSE) {
		ui_toggle_toolbar(toolbar_hidden, appview, (viewerInfo->viewer_toolbar));
	}

	toolbar_hidden = !toolbar_hidden;
	ui_set_viewer_fullscreen_toolbar_hidden_status(toolbar_hidden);
	viewerInfo->toolbarhidden_fullscreen = toolbar_hidden;
	/*bug fix for 19734--> showing progress if present after toggling */
	if (viewerInfo->mail_info->progress_on == TRUE) {
		/*stoping the progress  */
		if (viewerInfo->mail_info->timer_value > 0) {
			g_source_remove(viewerInfo->mail_info->timer_value);
			viewerInfo->mail_info->timer_value = 0;
		}
		show_progressbar(viewerInfo);
		update_progress_bar_text(viewerInfo->viewer_appview, viewerInfo);
	} else {
		hide_progressbar(viewerInfo);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}



/**
 This gnome_vfs_xfer callback function  .
 
 @param info is a pointer to the GnomeVFSXferProgressInfo structure.
 @param data is a pointer to the user data.No data freeing is done here.

 @return static int, No specfic fuctionality.just checking the status of operation
*/
/*@unused@*/
static gint xfer_progress_callback(GnomeVFSXferProgressInfo * info, gint * count)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(info == NULL, "Input param check failed\n", return 0);
	ULOG_DEBUG("the status is %d\n", info->status);
	ULOG_DEBUG("the phase is %d\n", info->phase);
	if (info->phase == GNOME_VFS_XFER_PHASE_COMPLETED) {
		(*count)++;
		ULOG_DEBUG("done with trasfering the file \n");
	}

	/*check if error while coping is in progress probably gateway/mmc error */
	if (info->status == GNOME_VFS_XFER_PROGRESS_STATUS_VFSERROR)
		return 0;



	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return 1;
}

/**
 This function is a helper functin for creating the src & target uri list
 needed for saving. 

 @param attachmentPathe is a list of attachment paths.
 @param attachmentname is a list of attachment names.
 @param storageLocation is a gchar pointer to destination location.
 @param src_uri_list is the OUT param for src_uri_list.
 @param dest_uri_list is the OUT param for dest_uri_list.
 
 @return void.
*/

static gboolean
create_target_path(GSList * attachmentPath, GSList * attachmentName,
		   const gchar * storageLocation, GList ** src_uri_list, GList ** dest_uri_list)
{
	GSList *src_paths = NULL;
	GSList *attachment_names = NULL;

	GnomeVFSURI *target_uri = NULL;
	GnomeVFSURI *src_uri = NULL;
	GnomeVFSURI *uri = NULL;
	gint num_of_attachments = 0;
	gboolean files_exists = 0;
	gboolean success = TRUE;

	src_paths = attachmentPath;
	attachment_names = attachmentName;
	num_of_attachments = g_slist_length(src_paths);

	while (src_paths != NULL && attachment_names != NULL) {
		uri = gnome_vfs_uri_new(storageLocation);
		if (num_of_attachments == 1) {
			target_uri = uri;
		} else {
			target_uri = gnome_vfs_uri_append_path(uri,(gchar *) attachment_names->data);
                        gnome_vfs_uri_unref(uri);
		}

		/*create the target uris */
		*dest_uri_list = g_list_append(*dest_uri_list, target_uri);

		/*Source URI to be created */
		src_uri = gnome_vfs_uri_new((gchar *) src_paths->data);
		*src_uri_list = g_list_append(*src_uri_list, src_uri);

		if (gnome_vfs_uri_exists(target_uri) == TRUE) {
			files_exists = TRUE;
		}
		src_paths = src_paths->next;
		attachment_names = attachment_names->next;
	}

	if (files_exists) {
		success =
		    get_user_confirmation(_("emev_nc_replace_files"),
					  _("emev_bd_replace_files_replace"), NULL, CONFIRM_DIALOG);
	}
	if (success == FALSE) {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		g_list_foreach(*src_uri_list, (GFunc) gnome_vfs_uri_unref, NULL);
		g_list_free(*src_uri_list);
		g_list_foreach(*dest_uri_list, (GFunc) gnome_vfs_uri_unref, NULL);
		g_list_free(*dest_uri_list);
		return FALSE;
	}else {
                /*unlink the files with the same name in the target*/
                GList *tmp_list = *dest_uri_list;
                while(tmp_list && tmp_list->data){
                	/*unlink if uri exits*/
                	if(gnome_vfs_uri_exists((GnomeVFSURI *)tmp_list->data))
                        	gnome_vfs_unlink(((GnomeVFSURI *)tmp_list->data)->text);
                        tmp_list = tmp_list->next;
                }
        }

	return TRUE;
}

/**
 Helper function autoname a already existing file.
 @param src_uris is the pointer to GList of src paths.
 @param dest_uris is the pointer to GList of dest paths.
 @param count is the nth node in the dest uri list which has to be renamed.

 @return void.
*/
static void rename_file_paths(GList ** src_uris, GList ** dest_uris, gint count)
{
	static HildonFileSystemModel *shared_model = NULL;
	gchar *uri = NULL;
	GError *error = NULL;
	gchar *ori_uri = NULL;
	GnomeVFSURI *vfsuri = NULL;
	shared_model = g_object_new(HILDON_TYPE_FILE_SYSTEM_MODEL, "ref-widget", app, NULL);


	ori_uri =
	    g_strconcat(((GnomeVFSURI *) g_list_nth_data(*dest_uris, count))->method_string, "://",
			((GnomeVFSURI *) g_list_nth_data(*dest_uris, count))->text, NULL);

	uri = hildon_file_system_model_autoname_uri(shared_model, ori_uri, &error);
	if (uri != NULL) {
		gnome_vfs_uri_unref(((GnomeVFSURI *) g_list_nth_data(*dest_uris, count)));
		g_list_nth(*dest_uris, count)->data = NULL;
		vfsuri = gnome_vfs_uri_new(uri);
		g_list_nth(*dest_uris, count)->data = vfsuri;

	}
	if (error != NULL)
		g_error_free(error);

	*src_uris = g_list_nth(*src_uris, count);
	*dest_uris = g_list_nth(*dest_uris, count);
	return;
}

/**
 This function is a helper functin for saving attachments
 
 @param storageLocation is a pointer to the destination storage location. 
 @param  attachmentPath is a pointer to the attachment location.
   No freeing of attachmentPath list is done.
 @param  attachmentName is a pointer to the name of the attachment location.
  No freeing of attachmentName list is done.
 @param  window is the pointer to the save dialog window.

 @return TRUE if saved successfully else FALSE 
*/

static gboolean
ui_save_attachment(const gchar * storageLocation, GSList * attachmentPath,
		   GSList * attachmentName, GtkWidget * window)
{
	GnomeVFSResult result = GNOME_VFS_OK;
	gboolean success = TRUE;
	GList *src_uris = NULL, *new_dest_uris = NULL, *new_src_uris = NULL;
	GList *dest_uris = NULL;
	gint num_of_attachments = 0;
	gint count = 0;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (storageLocation == NULL || attachmentPath == NULL || attachmentName == NULL) {
		ULOG_DEBUG_F(" ends here ");
		return FALSE;
	}
	success =
	    create_target_path(attachmentPath, attachmentName, storageLocation, &src_uris,
			       &dest_uris);
	if (success == FALSE) {
		ULOG_DEBUG("error while reading or replace confirmation is -ve");
		return FALSE;
	}

	/*showing the progress banner */
	gtk_banner_show_animation(GTK_WINDOW(window), _("mail_va_saving"));
	ui_flush_gtk_events();
	num_of_attachments = g_list_length(src_uris);
	new_dest_uris = dest_uris;
	new_src_uris = src_uris;
	while (1) {
		/*performing the xfer operation on attachments that are 2 be saved */
		result =
		    gnome_vfs_xfer_uri_list(src_uris, dest_uris,
					    GNOME_VFS_XFER_USE_UNIQUE_NAMES,
					    GNOME_VFS_XFER_ERROR_MODE_QUERY,
					    GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
					    (GnomeVFSXferProgressCallback)
					    xfer_progress_callback, &count);
		if (result == GNOME_VFS_ERROR_FILE_EXISTS) {
			/*file already exists then unlink the file, & continue xfer */
			if (num_of_attachments == 1) {
				gnome_vfs_unlink(((GnomeVFSURI *) (dest_uris->data))->text);
				continue;
			}
			/*else autorename the files */
                        /*if the list length is one then set the count(the nth node to be renamed) to 0*/
                        if(g_list_length(dest_uris) == 1)
                                count = 0;
			rename_file_paths(&src_uris, &dest_uris, count);
			count = 0;
			continue;
		}
		dest_uris = new_dest_uris;
		src_uris = new_src_uris;
		switch (result) {
		case GNOME_VFS_ERROR_NOT_FOUND:
			{
				get_user_confirmation(_("emev_ni_attachment_not_found"), NULL, NULL,
						      INFO_BANNER);
				ULOG_INFO_F("Gnome Vfs error : Not found");
				success = FALSE;
				break;
			}
		case GNOME_VFS_ERROR_NO_SPACE:
			{
				get_user_confirmation(DGETTEXT_HCS("sfil_ni_not_enough_memory"),
						      NULL, NULL, INFO_BANNER);
				success = FALSE;
				break;
			}

		case GNOME_VFS_ERROR_READ_ONLY:
			{
				get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL,
						      INFO_BANNER);
				success = FALSE;
				break;
			}
		case GNOME_VFS_ERROR_INTERRUPTED:	/*Error occured while saving in progress */
			{
				get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL,
						      INFO_BANNER);
				gtk_widget_destroy(window);
				success = TRUE;
				break;
			}
		default:
			break;
		}
		break;
	}
	/*Freeing the src & destination uris */
	g_list_foreach(src_uris, (GFunc) gnome_vfs_uri_unref, NULL);
	g_list_foreach(dest_uris, (GFunc) gnome_vfs_uri_unref, NULL);
	g_list_free(src_uris);
	g_list_free(dest_uris);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	if (success)
		gtk_widget_destroy(window);
	return success;
}

/**
 This function is called by  for  registering the callback
    for closing the save attachment dialog.
    
 @param  attachmentPath is a pointer to the attachment location.
   The attachemntPaths list is freed here.
 @param  attachmentName is a pointer to the name of the attachment location.
   The attachementNames list is freed here.
 @param  savedialog is the pointer to the save dialog window.
 
 @return void
*/
void
ui_viewer_cancel_save_attachment_cb(GSList * attachmentNames,
				    GSList * attachmentPaths, GtkWidget * savedialog)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	g_slist_foreach(attachmentNames, (GFunc) g_free, NULL);
	g_slist_foreach(attachmentPaths, (GFunc) gnome_vfs_unlink, NULL);
	g_slist_foreach(attachmentPaths, (GFunc) g_free, NULL);
	g_slist_free(attachmentPaths);
	g_slist_free(attachmentNames);
	if (savedialog != NULL) {
		gtk_widget_destroy(savedialog);
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 Function to run the file saver dialog on some error condition.

 @param  attach_name is a pointer to the name of the attachment .
  This list is  freed either by calling
  ui_viewer_cancel_save_attachment_cb() or ui_viewer_ok_save_attachment_cb().
 @param  srcpath is a pointer to the attachment location.
  This list is  freed either by calling
  ui_viewer_cancel_save_attachment_cb() or ui_viewer_ok_save_attachment_cb().
 @param  saveWindow is the pointer to the save dialog window.
 
 @return void.
*/
static void ui_filesaver_run(GSList * attach_names, GSList * srcpaths, GtkWidget * saveWindow)
{
	gint response = 0;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(saveWindow == NULL, "Input param check failed\n", return);
	response = gtk_dialog_run((GtkDialog *) saveWindow);
	if (response == GTK_RESPONSE_OK) {
		ui_viewer_ok_save_attachment_cb(attach_names, srcpaths,
						(GtkFileChooser *) saveWindow);
	} else if (response == GTK_RESPONSE_CANCEL) {
		ui_viewer_cancel_save_attachment_cb(attach_names, srcpaths, saveWindow);
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is called by the save dialog for registering the ok callback.

@param  attachmentPath is a pointer to the attachment location.
 This list is  freed. 
@param  attachmentName is a pointer to the name of the attachment location.
 This list is  freed. 
@param  savedialog is the pointer to the save dialog window.

@return void 
*/
void
ui_viewer_ok_save_attachment_cb(GSList * attachmentNames,
				GSList * attachmentPaths, GtkFileChooser * savedialog)
{
	const gchar *storageLocation = NULL;
	GSList *filepath_list = NULL;
	filepath_list = gtk_file_chooser_get_uris(savedialog);
	if (filepath_list != NULL && filepath_list->data != NULL) {
		storageLocation = filepath_list->data;
		if (ui_save_attachment
		    (storageLocation, attachmentPaths, attachmentNames, (GtkWidget *) savedialog)) {
			attachmentNames = g_slist_nth(attachmentNames, 0);
			attachmentPaths = g_slist_nth(attachmentPaths, 0);
			g_slist_foreach(attachmentNames, (GFunc) g_free, NULL);
			g_slist_free(attachmentNames);
			g_slist_foreach(attachmentPaths, (GFunc) gnome_vfs_unlink, NULL);
			g_slist_foreach(attachmentPaths, (GFunc) g_free, NULL);
			g_slist_free(attachmentPaths);
		} else {
			ui_filesaver_run(attachmentNames, attachmentPaths,
					 (GtkWidget *) savedialog);
		}
	}
	ui_file_paths_free(filepath_list);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This fuction invokes the hildon file saver dialog for saving the attachment.

 @param  attach_name is a pointer to the name of the attachment .
   This list is freed after the operations are done either in function.
   ui_viewer_ok_save_attachment_cb() or ui_viewer_cancel_save_attachment_cb().
 @param  srcpath is a pointer to the attachment location.
   This list is freed after the operations are done either in function 
   ui_viewer_ok_save_attachment_cb() or ui_viewer_cancel_save_attachment_cb().
   
 @return void. 
*/
static void ui_viewer_create_save_dialog(GSList * attach_names, GSList * srcpaths)
{
	GtkWidget *saveWindow = NULL;
	gint response = 0;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	saveWindow =
	    (GtkWidget *) hildon_file_chooser_dialog_new((GtkWindow *) app,
							 GTK_FILE_CHOOSER_ACTION_SAVE);
	gtk_window_set_title(GTK_WINDOW(saveWindow), _("mcen_ti_save_attachment_title"));
	gtk_window_set_transient_for(GTK_WINDOW(saveWindow), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(saveWindow), TRUE);

	if (ossohelp_exists(HELP_ID_SAVEOBJECTS) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) saveWindow);
		g_signal_connect(G_OBJECT(saveWindow), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_SAVEOBJECTS);
	}
	if (saveWindow != NULL) {
		gtk_file_chooser_set_current_name((GtkFileChooser *) saveWindow,
						  attach_names->data);
		response = gtk_dialog_run((GtkDialog *) saveWindow);
		if (response == GTK_RESPONSE_OK) {
			ui_viewer_ok_save_attachment_cb(attach_names, srcpaths,
							(GtkFileChooser *) saveWindow);
		} else if (response == GTK_RESPONSE_CANCEL) {
			ui_viewer_cancel_save_attachment_cb(attach_names, srcpaths, saveWindow);
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This fuction invokes the hildon file saver dialog for saving the attachment.
  
 @param  attach_name is a pointer to the name of the attachment .
	This list is freed after the operations are done.in functions 
	ui_viewer_ok_save_attachment_cb() or ui_viewer_cancel_save_attachment_cb().
 @param  srcpath is a pointer to the attachment location.
 	This list is freed after the operations are done in functions
	ui_viewer_ok_save_attachment_cb() or ui_viewer_cancel_save_attachment_cb().
 @return void. 
*/
static void
ui_new_create_folder_dialog(GSList * attachmentnames, GSList * srcpaths, gint numof_attachments)
{
	GtkWidget *saveWindow = NULL;
	gint response = 0;
	gchar *str = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	str =
	    g_strdup_printf(DGETTEXT_FM("sfil_va_number_of_objects_attachments"),
			    numof_attachments);
	saveWindow =
	    (GtkWidget *) hildon_file_chooser_dialog_new((GtkWindow *) app,
							 GTK_FILE_CHOOSER_ACTION_SAVE);
	g_object_set((GObject *) saveWindow, "save-multiple", str, NULL);
	g_free(str);

	gtk_window_set_title(GTK_WINDOW(saveWindow), _("mcen_ti_save_attachments_title"));
	gtk_window_set_transient_for(GTK_WINDOW(saveWindow), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(saveWindow), TRUE);

	if (ossohelp_exists(HELP_ID_SAVEOBJECT) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) saveWindow);
		g_signal_connect(G_OBJECT(saveWindow), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_SAVEOBJECT);
	}
	if (saveWindow != NULL) {
		response = gtk_dialog_run((GtkDialog *) saveWindow);
		if (response == GTK_RESPONSE_OK) {
			ui_viewer_ok_save_attachment_cb(attachmentnames, srcpaths,
							(GtkFileChooser *) saveWindow);
		} else if (response == GTK_RESPONSE_CANCEL) {
			ui_viewer_cancel_save_attachment_cb(attachmentnames, srcpaths, saveWindow);
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 launches the dialog for saving non-vcards attachments
 @param  selected_nameslist is the GSlsit representing the non-vcard attachemnt names.
 @param srclist is the GSlist representing the src paths of the attachemnts.
   The input params will be freed after the operations on them in functions
   ui_viewer_create_save_dialog() or ui_new_create_folder_dialog().
 @return void.
*/
static void launch_the_save_dialog(GSList * selected_nameslist, GSList * srclist)
{
	gint numof_attachments = 0;
	numof_attachments = g_slist_length(selected_nameslist);
	if (numof_attachments == 1) {
		ui_viewer_create_save_dialog(selected_nameslist, srclist);
	}
	/*launching multiple save dialog */
	else if (numof_attachments > 1) {
		ui_new_create_folder_dialog(selected_nameslist, srclist, numof_attachments);
	}
	return;
}


/**
 This is a helper function to get the attachment src path from
  the MsgInfo structure pointer.

 @param msg_info is the  MsgInfo structure pointer.
 
 @return gchar representing the attachment src path.
*/
static gchar *get_src_attachment_path(const MsgInfo * msg_info)
{
	gchar *srcpath = NULL;
	AttachInfo *attachInfo = NULL;

	if (msg_info != NULL && msg_info->attach_list != NULL) {
		if (msg_info->attach_list->data != NULL) {
			attachInfo = (AttachInfo *) (msg_info->attach_list->data);
			if (attachInfo->filepath != NULL) {
				srcpath = g_strdup(attachInfo->filepath);
			}
		}
	}
	return srcpath;
}
static void
saving_local_attachments(const GSList * local_attachments, gchar * msgid, ViewerInfo * viewer_info)
{
	MsgInfo *msginfo = NULL;
	GSList *attachment_list = NULL;
	GSList *srclist = NULL, *selected_nameslist = NULL;
	gchar *srcpath = NULL;

	attachment_list = (GSList *) local_attachments;
	if (local_attachments == NULL)
		return;
	/*these will be populated after the attachemnts are retrieved. */
	srclist = viewer_info->selected_attachment_paths;
	selected_nameslist = viewer_info->selected_attachment_names_save;

	while (attachment_list != NULL) {
		msginfo =
		    (MsgInfo *) ui_dbus_recv_msg_attachment((msgid),
							    (MimeInfo *) attachment_list->data);
		if (msginfo != NULL && msginfo->attach_list != NULL) {
			srcpath = get_src_attachment_path(msginfo);
			((MimeInfo *) (attachment_list->data))->is_downloaded = TRUE;
			((MimeInfo *) (attachment_list->data))->is_purged = FALSE;
			if (srcpath != NULL) {
				srclist = g_slist_append(srclist, g_strdup(srcpath));
				selected_nameslist = g_slist_append(selected_nameslist,
								    g_strdup(((MimeInfo
									       *) (attachment_list->
										   data))->
									     filename));
			}
		}
		ui_freeMem(msginfo, NULL);
		msginfo = NULL;
		g_free(srcpath);
		srcpath = NULL;
		attachment_list = attachment_list->next;
	}
	viewer_info->selected_attachment_paths = srclist;
	viewer_info->selected_attachment_names_save = selected_nameslist;

	/*check if there are only local attachments to be saved */
	if (viewer_info->mimeinfo_list == NULL) {
		launch_the_save_dialog(selected_nameslist, srclist);
		viewer_info->selected_attachment_paths = NULL;
		viewer_info->selected_attachment_paths = NULL;
	}
	hide_progressbar(viewer_info);

	return;
}


/**
 This function is used to save the attachments after they are downloaded.
 */
static void
process_for_save_attach(GSList * msginfo_list, ViewerInfo * viewer_info, gboolean last_attachment)
{
	GSList *selected_names = NULL;
	MailInfo *mail_info = NULL;
	gchar *srcpath = NULL;
	MsgInfo *msginfo = NULL;
	MimeInfo *mimeinfo = NULL;

	selected_names = viewer_info->selected_attachment_names;
	mail_info = viewer_info->mail_info;


	hide_progressbar(viewer_info);
	/*enabling & disabling of purge menu item after downloading the attachment */
	if (mail_info->folder_num != DRAFT_MODEL_NUM && mail_info->folder_num != OUTBOX_MODEL_NUM) {
		if (((mail_info->protocol == PROTO_IMAP4)
		     && (mail_info->folder_num == INBOX_MODEL_NUM)) ||
		    (mail_info->folder_num != INBOX_MODEL_NUM)) {
			if (mail_info->smime_type == SMIME_NONE &&
			    viewer_info->mail_info->email_attachment == FALSE)
				gtk_widget_set_sensitive(viewer_info->menu->purge_attachment, TRUE);
			else
				gtk_widget_set_sensitive(viewer_info->menu->purge_attachment,
							 FALSE);
		} else {
			gtk_widget_set_sensitive(viewer_info->menu->purge_attachment, FALSE);
		}
	} else {
		gtk_widget_set_sensitive(viewer_info->menu->purge_attachment, FALSE);
	}

	while (msginfo_list != NULL) {
		msginfo = ((MsgInfo *) msginfo_list->data);
		srcpath = get_src_attachment_path(msginfo);
		mimeinfo = get_mimeinfo_for_attachment(viewer_info->mime_info,
						       msginfo->header->attach_tag);
		viewer_info->selected_attachment_paths =
		    g_slist_append(viewer_info->selected_attachment_paths, g_strdup(srcpath));
		if (mimeinfo != NULL && mimeinfo->filename != NULL) {
			viewer_info->selected_attachment_names_save =
			    g_slist_append(viewer_info->selected_attachment_names_save,
					   g_strdup(mimeinfo->filename));
		}
		g_free(srcpath);
		srcpath = NULL;
		msginfo_list = msginfo_list->next;
	}

	change_the_icon(selected_names, viewer_info, FALSE, NULL);
	saving_local_attachments(viewer_info->local_attachments, viewer_info->msgid, viewer_info);

	launch_the_save_dialog(viewer_info->selected_attachment_names_save,
			       viewer_info->selected_attachment_paths);

	g_slist_foreach(viewer_info->selected_attachment_names, (GFunc) g_free, NULL);
	g_slist_free(viewer_info->selected_attachment_names);
	viewer_info->selected_attachment_names = NULL;
	viewer_info->selected_attachment_paths = NULL;
	viewer_info->selected_attachment_names_save = NULL;
	g_slist_free(viewer_info->mimeinfo_list);
	g_slist_free(viewer_info->seqno_list);
	g_slist_free(viewer_info->local_attachments);
	viewer_info->local_attachments = NULL;
	viewer_info->mimeinfo_list = NULL;
	viewer_info->seqno_list = NULL;

	/*checking if this the last attachment to be downloaded &
	 * accordingly updating the mimeinfo*/
	if (last_attachment == TRUE) {
		update_mimeinfo_for_last_attachment(viewer_info);
	}
	return;
}

/**
 Helper function to prepare the headerinfo for opening the viewer
 
 @param headerInfo is the pointer to the HeaderInfo structure.
   This param is not freed .
 @param composeInfo is the poiter to the MsgInfo structure for which the viewer is to be created.
    This param is not freed .
    
 @return void.
*/
static void ui_fill_headerInfo(HeaderInfo * headerInfo, const MsgInfo * composeInfo)
{
	AttachInfo *attachInfo = NULL;
	CHECK_PARAM(composeInfo == NULL, "Input param check failed\n", return);
	CHECK_PARAM(composeInfo->header == NULL, "Input param check failed\n", return);

	if (composeInfo->attach_list != NULL) {
		attachInfo = (AttachInfo *) g_slist_nth_data(composeInfo->attach_list, 0);
	}
	headerInfo->msgid = g_strdup(composeInfo->header->msgid);
	headerInfo->from = (gchar *) ui_parse_from_address(composeInfo->header->from);
	headerInfo->to = g_strdup(composeInfo->header->to);
	headerInfo->cc = g_strdup(composeInfo->header->cc);
	headerInfo->bcc = g_strdup(composeInfo->header->bcc);
	if (composeInfo->header->replyto != NULL)
		headerInfo->reply_to = g_strdup(composeInfo->header->replyto);
	if (composeInfo->header->subject != NULL)
		headerInfo->subject = g_strdup(composeInfo->header->subject);
	else
		headerInfo->subject = g_strdup(_("mail_va_no_subject"));

	headerInfo->sent_date = composeInfo->header->date_t;
	headerInfo->priority = NORMAL;
	headerInfo->folderNum = NOT_DEFINED;
	headerInfo->rowNum = NOT_DEFINED;
	headerInfo->email_attachment = TRUE;
	if (attachInfo != NULL) {
		if (attachInfo->filepath != NULL)
			headerInfo->filename = g_strdup(attachInfo->filepath);
		if (attachInfo->email_path != NULL)
			headerInfo->email_filename = g_strdup(attachInfo->email_path);
	}

	return;
}

/**
   This function is used to get the parent mimeInfo of a
    emailtype attachment since the mimeInfo of this mail starts from the sub part.
     The parent mimeInfo is assinged to the ViewerInfo structure .
      No memory freeing for viewerInfo & mimeInfo is done.
 
 @param mimeInfo is the MiemInfo structure pointer to the current viewer.
 @param root_mimeInfo is the MiemInfo structure pointer of the root OUT param.
 
 @return void.
*/

static void root_mimeinfo(MimeInfo * mimeinfo, MimeInfo ** root_mime_info)
{
	MimeInfo *mimeInfo = NULL;
	mimeInfo = mimeinfo;
	while (mimeInfo != NULL) {
		if (mimeInfo->parent == NULL && mimeInfo->main == NULL) {
			(*root_mime_info) = mimeInfo;
			return;
		}
		root_mimeinfo(mimeInfo->main, root_mime_info);
		root_mimeinfo(mimeInfo->parent, root_mime_info);
		return;
	}
	return;
}
static MimeInfo *get_sub_mimeinfo(MimeInfo * mimeInfo, ViewerInfo * viewer_info)
{
	MimeInfo *new_mimeinfo = NULL, *required_mimeinfo = NULL, *root_mime_info = NULL;

	new_mimeinfo = mimeInfo;
	/*this mimeinfo is the mimeinfo from sub part */
	required_mimeinfo = viewer_info->mime_info;

	/*get the root mimeinfo of the above sub part */
	root_mimeinfo(viewer_info->mime_info, &root_mime_info);
	while (root_mime_info != NULL && new_mimeinfo != NULL) {
		if (root_mime_info == required_mimeinfo) {
			mimeInfo = new_mimeinfo;
			break;
		}
		root_mime_info = mimeinfo_next(root_mime_info);
		new_mimeinfo = mimeinfo_next(new_mimeinfo);
	}
	return mimeInfo;
}

/**
 This function is called to update the mimeinfo after all the attachemnts are retrieved.

 @param viewer_info is the pointer to the ViewerInfo structure pointer.
 @param headerInfo is the HeaderInfo structure pointer.
 @param last_attachment is the gboolean representing whether the retrieved attachemnt is the 
  last attachment.
 @param filename is the filename of the sub part of the MimeInfo. 
   freed in this function. 
 @return TRUE-if ui_dbus_recv_msg_body returns not NULL;
          FALSE-else.
*/
static gboolean
update_mimeinfo(ViewerInfo * viewer_info, HeaderInfo * headerInfo,
		gboolean last_attachment, gint nth_attachment)
{
	MimeInfo *tempmimeinfo = NULL, *mimeInfo = NULL;

	gboolean value = FALSE;
	ULOG_DEBUG_F("starts");

	if (last_attachment == TRUE) {
		value = update_mimeinfo_for_last_attachment(viewer_info);
		if (value == FALSE)
			return value;
	}
	/*Assing a new mimeinfo for the emailtype attachment which is opened
	   in  viewer. */
	if (last_attachment == TRUE) {
		mimeInfo =
		    ui_dbus_recv_msg_body(viewer_info->msgid, TRUE, TRUE,
					  viewer_info->senderid, viewer_info->receiverid,
					  viewer_info->mail_info->smime_type, FALSE,
					  viewer_info->certid, viewer_info->password, FALSE, TRUE);
	} else {
		mimeInfo =
		    ui_dbus_recv_msg_body(viewer_info->msgid, TRUE, FALSE,
					  viewer_info->senderid, viewer_info->receiverid,
					  viewer_info->mail_info->smime_type, FALSE,
					  viewer_info->certid, viewer_info->password, FALSE, TRUE);
	}
	if (mimeInfo == NULL) {
		return FALSE;
	}

	/*for nested attachments going to the sub part from the root mimeinfo */
	if (viewer_info->mail_info->email_attachment == TRUE) {
		mimeInfo = get_sub_mimeinfo(mimeInfo, viewer_info);
	}

	/*assing the appropriate mimeinfo to the email type attachemnt.
	   which is going to be opened in the viewer. */
	tempmimeinfo = get_mimeinfo_for_attachment(mimeInfo, nth_attachment);
	if (tempmimeinfo != NULL && tempmimeinfo->sub != NULL) {
		headerInfo->mimeInfo = tempmimeinfo->sub;
		headerInfo->smime_type = tempmimeinfo->smime_type;
	}
	ULOG_DEBUG_F("ends");
	return TRUE;
}

/**
 This is a helper function to view a email type attachemnt.
 
 @param msg_info is the MsgInfo structure pointer.
 @param Viewer_info is the ViewerInfo staructuer poiter of the current viewer.
 @param last_attachemnt is gboolean representing whther the attachment being 
  viewed is last attachment.
 @param nth_attach is the gint representing the attachment sequence number.
 
 @return void.
*/
static void
open_viewer(const MsgInfo * msg_info, ViewerInfo * viewer_info,
	    gboolean last_attachment, gint nth_attach)
{
	HeaderInfo *headerInfo = NULL;
	gboolean value = TRUE;
	GSList *selected_names = NULL;
	GtkWidget *viewer = NULL;
	selected_names = viewer_info->selected_attachment_names;
	hide_progressbar(viewer_info);

	headerInfo = g_new0(HeaderInfo, 1);
	ui_fill_headerInfo(headerInfo, msg_info);
	headerInfo->zoomlevel = viewer_info->zoominfo->previous_level;
	if (viewer_info->password != NULL)
		headerInfo->password = g_strdup(viewer_info->password);
	headerInfo->certid = viewer_info->certid;
	/*update mimeInfo if its last attachemnt downloaded */
	value = update_mimeinfo(viewer_info, headerInfo, last_attachment, nth_attach);
	/*if dbus call fails return */
	if (value == FALSE) {
		return;
	}
	/*change the icon after the attachment is retrieved */
	change_the_icon(selected_names, viewer_info, FALSE, headerInfo->subject);
	viewer = ui_viewer_create(headerInfo);
	if (viewer != NULL) {
		gtk_widget_show(viewer);
	}
	if (headerInfo != NULL) {
		ui_freeHeaderInfo(headerInfo);
	}
	return;
}

/**
  Method to invoke an application corresponding to the user selection.
  
 @param window is the viewer appview pointer. 
 @param selection handle to the list of selected items.
 
 @return void
*/
static void invoke_application(GtkWidget * window, const GSList * selection)
{
	gint openresult = 0;
	gchar *path = NULL;
	DBusConnection *con = NULL;

	if (window == NULL) {
		ULOG_ERR("Invalid parameter window");
		return;
	}
	if (selection == NULL) {
		ULOG_ERR("Invalid parameter selection");
		return;
	}
	con = (DBusConnection *) osso_get_dbus_connection(ui_context);
	if (con == NULL) {
		gtk_infoprint((GtkWindow *) app, _("emev_ni_internal_error"));
		return;
	}
	path = g_strconcat("file://", (gchar *) selection->data, NULL);
	openresult = osso_mime_open_file(con, path);
	g_free(path);
	ULOG_DEBUG_F("the result is %d\n", openresult);
	if (openresult != 1) {
		get_user_confirmation(_("mcen_ni_noregistered_viewer"), NULL, NULL, INFO_BANNER);
	}
	return;
}

/**
 This is a helper function to view a non email type attachemnt.
 
 @param Viewer_info is the ViewerInfo staructuer poiter of the current viewer.
 @param srcpath  is the gchar pointer to the attachment path.
 
 @return void.
*/
static void
open_application(ViewerInfo * viewer_info, const gchar * srcpath, gchar * attachment_name)
{
	GSList *viewing_list = NULL;
	gchar *path = NULL;
	const gchar *mime_string = NULL;

	hide_progressbar(viewer_info);

	ULOG_DEBUG_F("the srcpath is %s\n", srcpath);
	mime_string = (gchar *) gnome_vfs_mime_type_from_name(srcpath);
	path = ui_create_filepath_link(srcpath, &attachment_name, TRUE, FALSE);

	/*If moving of file to /var/tmp fails we return without 
	   launching the application. */
	if (path == NULL) {
		if (unlink(srcpath) < 0)
			g_warning("error unlink the file %s\n", srcpath);

		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
		return;
	}

	viewing_list = g_slist_append(viewing_list, path);

	chmod((gchar *) viewing_list->data, S_IRUSR | S_IRGRP | S_IROTH);
	invoke_application(viewer_info->viewer_appview, viewing_list);

	/*change the icon after the attachment is retrieved */
	change_the_icon(viewing_list, viewer_info, FALSE, NULL);

	g_slist_foreach(viewing_list, (GFunc) g_free, NULL);
	g_slist_free(viewing_list);
	return;
}

/**
 Funtion to free all the resources once the attachemnts are recieved.

 @param viewer_info  is the ViewerInfo sturcture pointer to the currentviewer.
 @return void.
*/
static void free_receive_attachment_data(ViewerInfo * viewer_info)
{
	g_slist_foreach(viewer_info->selected_attachment_names, (GFunc) g_free, NULL);
	g_slist_free(viewer_info->selected_attachment_names);
	/*internal mimeinfo pointers r not freed they r only freed 
	   when the viewer is closed */
	g_slist_free(viewer_info->mimeinfo_list);
	g_slist_free(viewer_info->seqno_list);
	viewer_info->mimeinfo_list = NULL;
	viewer_info->seqno_list = NULL;
	viewer_info->selected_attachment_names = NULL;
	hide_progressbar(viewer_info);
	return;
}

/**
   This function is used update the viewer with the mimeinfo we recieve
   through callback from engine.
 
   @param viewer is the pointer to the viewer appview.
   @param viewer_info is the pointer to the ViewerInfo structure pointer.
 
   @return void.
*/
void ui_viewer_get_msginfo_thr_cb(GtkWidget * viewer, ViewerInfo * viewer_info)
{
	MsgInfo *msg_info = NULL;
	MimeInfo *mimeinfo = NULL;
	MailInfo *mail_info = NULL;
	gchar *srcpath = NULL;
	gint nth_loop = -1;
	gboolean last_attachment = FALSE;
	GSList *selected_attachment_names = NULL;
	GSList *msginfo_list = NULL;
	gboolean error = FALSE;

	msginfo_list = email_main_view->msginfo_list;
	selected_attachment_names = viewer_info->selected_attachment_names;
	mail_info = viewer_info->mail_info;

	if (viewer_info->mail_info->progress_on == FALSE) {
		ULOG_INFO("user cancelled receiving of mail\n");
		free_receive_attachment_data(viewer_info);
		g_slist_foreach(email_main_view->msginfo_list, (GFunc) ui_freeMem, NULL);
		g_slist_free(email_main_view->msginfo_list);
		email_main_view->msginfo_list = NULL;
		if (email_main_view->msginfo != NULL)
			ui_freeMem(email_main_view->msginfo, NULL);
		email_main_view->msginfo = NULL;
		return;
	}


/*if error occuredwhile receiving attachment.*/
	if (msginfo_list == NULL && email_main_view->msginfo == NULL) {
		free_receive_attachment_data(viewer_info);
		return;
	}

	while (msginfo_list != NULL) {
		mimeinfo =
		    get_mimeinfo_for_attachment(viewer_info->mime_info,
						((MsgInfo *) (msginfo_list->data))->header->
						attach_tag);
		if (mimeinfo == NULL) {
			error = TRUE;
			break;
		}
		srcpath = get_src_attachment_path((MsgInfo *) msginfo_list->data);
		if (srcpath == NULL) {
			error = TRUE;
			break;
		}
		g_free(srcpath);
		srcpath = NULL;
		msginfo_list = msginfo_list->next;
		mimeinfo->is_downloaded = TRUE;
		mimeinfo->is_purged = FALSE;
	}

/*if error occuredwhile receiving attachment.*/
	if (error) {
		osso_log(LOG_INFO, "The attachment could not be retrieved.");
		free_receive_attachment_data(viewer_info);
		g_slist_foreach(email_main_view->msginfo_list, (GFunc) ui_freeMem, NULL);
		g_slist_free(email_main_view->msginfo_list);
		email_main_view->msginfo_list = NULL;
		return;
	}

	if (email_main_view->msginfo_list != NULL) {
		msginfo_list = email_main_view->msginfo_list;
		msg_info = g_slist_nth_data(msginfo_list, 0);
	} else {
		msg_info = email_main_view->msginfo;
		if (msg_info == NULL) {
			free_receive_attachment_data(viewer_info);
			return;
		}
		mimeinfo =
		    get_mimeinfo_for_attachment(viewer_info->mime_info,
						msg_info->header->attach_tag);
		if (mimeinfo == NULL) {
			free_receive_attachment_data(viewer_info);
			ui_freeMem(msg_info, NULL);
			return;
		}
	}

	/*checking for the srcpath of attachment-i.e whether it is retrieved. */
	srcpath = get_src_attachment_path(msg_info);
	if (srcpath != NULL) {
		mimeinfo->is_downloaded = TRUE;
		mimeinfo->is_purged = FALSE;
	} else {
		osso_log(LOG_INFO, "The attachment could not be retrieved.");
		free_receive_attachment_data(viewer_info);
		g_slist_foreach(email_main_view->msginfo_list, (GFunc) ui_freeMem, NULL);
		g_slist_free(email_main_view->msginfo_list);
		email_main_view->msginfo_list = NULL;
		if (email_main_view->msginfo != NULL)
			ui_freeMem(email_main_view->msginfo, NULL);
		email_main_view->msginfo = NULL;
		return;
	}

	/*checking if this the last attachment to be downloaded */
	last_attachment = last_attachment_tobe_downloaded(viewer_info->mime_info);

	/*viewing a email attachment */
	if (viewer_info->mode == VIEWING_EMAIL) {
		nth_loop = msg_info->header->attach_tag;
		open_viewer(msg_info, viewer_info, last_attachment, nth_loop);
	}

	/*viewing a non email attachment */
	else if (viewer_info->mode == VIEWING_ATTA) {
		open_application(viewer_info, srcpath, (gchar *) selected_attachment_names->data);
		if (last_attachment == TRUE)
			update_mimeinfo_for_last_attachment(viewer_info);
	}

	/*saving of attachments */
	else {
		process_for_save_attach(msginfo_list, viewer_info, last_attachment);
	}
	g_free(srcpath);
	g_slist_foreach(msginfo_list, (GFunc) ui_freeMem, NULL);
	g_slist_free(msginfo_list);
	if (email_main_view->msginfo != NULL)
		ui_freeMem(email_main_view->msginfo, NULL);

	if (viewer_info->mode != SAVING) {
		free_receive_attachment_data(viewer_info);
	}
	email_main_view->msginfo_list = NULL;
	email_main_view->msginfo = NULL;
	return;
}


/**
 This function is called by higher layers for getting the attachment path. 

 @param mimeinfo is a pointer to the mimeInfo structure that contains the attachments.
  It is notr freed inside thsi function.
 @param msgid is the gchar pointer to the msgid.
 @param attnamestr is a gchar pointer to the name  of the attachment.
 @return gchar pointer to the attachment path. 
*/
static gint
ui_get_attachment_path(const MimeInfo * mimeinfo, const gchar * msgid, gint nth_attachment)
{
	MimeInfo *tempMimeinfo = NULL;
	gint atta_type = NON_EMAIL_ATTA;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(mimeinfo == NULL, "no attachments", return ERROR);

	tempMimeinfo = get_mimeinfo_for_attachment(mimeinfo, nth_attachment);
	if (tempMimeinfo != NULL && tempMimeinfo->mime_type != MIME_MESSAGE_RFC822) {
		atta_type = NON_EMAIL_ATTA;

	} else if (tempMimeinfo != NULL && tempMimeinfo->mime_type == MIME_MESSAGE_RFC822) {
		atta_type = EMAIL_ATTA;
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return (atta_type);
}

/**
 Function get the non local attachemnts .
 
 @param nonlocal_attachments is the list of non local attachment mimeinfos.
 @param viewer_info is teh ViewerInfo structure pointer.
 @param atta_seqno_list is the list gint representing the attachment no.s
 @param count is the count of number of non local attachments.
*/
static void
get_nonlocal_attachments(GSList * nonlocal_attachments,
			 ViewerInfo * viewer_info, GSList * atta_seqno_list, gint count)
{
	if (nonlocal_attachments == NULL) {
		return;
	}
	viewer_info->mode = SAVING;
	ui_dbus_recv_msg_multiple_attachments(nonlocal_attachments, atta_seqno_list,
					      viewer_info->msgid);
	if (count > 0)
		show_progressbar(viewer_info);
	else
		viewer_info->mail_info->progress_on = TRUE;
}


/**
 This function is called by higher layers for  registering the callback
 
 @param win is a pointer to the menu item that registered the callback.
 @param viewer_info is the pointer to the ViewerInfo structure.
   This parameter is not freed inside the call.
   
 @return void
*/
void ui_viewer_save_attachment_cb(GtkWidget * win, ViewerInfo * viewer_info)
{
	GSList *selected_names = NULL;
	gint nth_attach = -1;
	gboolean value = TRUE;
	MsgFields *msgfields = NULL;
	MailInfo *mailinfo = NULL;
	gint nth = 0;
	gint count = 0;
	GSList *attachment_name = NULL;
	GList *pathslist = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeSelection *attachment_selection = NULL;
	GtkTreeIter iter;
	GSList *local_attachments = NULL;
	GSList *nonlocal_attachments = NULL;
	GSList *nonlocal_atta_seqno_list = NULL;
	MimeInfo *mimeinfo = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (viewer_info == NULL && viewer_info->msg_fields == NULL) {
		return;
	}
	msgfields = viewer_info->msg_fields;
	mailinfo = viewer_info->mail_info;
	/*checking if the attachment is already in the process of retrieval */
	if (mailinfo->progress_on == TRUE)
		return;


	selected_names = ui_get_selected_attachment_name(msgfields);
	if (selected_names == NULL) {
		attachment_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgfields->view));
		gtk_tree_model_get_iter_first(gtk_tree_view_get_model
					      ((GtkTreeView *) msgfields->view), &iter);
		gtk_tree_selection_select_iter(attachment_selection, &iter);
		selected_names = ui_get_selected_attachment_name(viewer_info->msg_fields);
	}

	viewer_info->selected_attachment_names = selected_names;
	selection = gtk_tree_view_get_selection((GtkTreeView *) msgfields->view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, NULL);
	attachment_name = selected_names;
	while (selected_names != NULL && selected_names->data != NULL) {
		nth_attach = ui_get_attachment_num(viewer_info->msg_fields, nth);
		nth++;
		mimeinfo = get_mimeinfo_for_attachment(viewer_info->mime_info, nth_attach);
		if (mimeinfo != NULL && mimeinfo->is_downloaded == FALSE)
			count++;
		{
			nonlocal_attachments = g_slist_append(nonlocal_attachments, mimeinfo);
			nonlocal_atta_seqno_list =
			    g_slist_append(nonlocal_atta_seqno_list, GINT_TO_POINTER(nth_attach));
		}
		selected_names = selected_names->next;
	}

	if (count == 1) {
		value =
		    get_user_confirmation(_("emev_nc_retrieve_attachment"),
					  _("emev_bd_retrieve_attachment"), NULL, CONFIRM_DIALOG);
	} else if (count != 0) {
		value =
		    get_user_confirmation(_("emev_nc_retrieve_attachments"),
					  _("emev_bd_retrieve_attachments_retrieve"), NULL,
					  CONFIRM_DIALOG);
	}

	if (value == FALSE) {

		gtk_widget_set_sensitive(viewer_info->menu->purge_attachment, FALSE);
		selected_names = viewer_info->selected_attachment_names;
		g_slist_foreach(selected_names, (GFunc) g_free, NULL);
		g_slist_free(selected_names);
		/*the internal nodes i.e mimeinfo pointer not freed,
		 * they will be freed when viewer is closed*/
		g_slist_free(nonlocal_attachments);
		g_slist_free(local_attachments);
		/*its internal nodes are intergers which need not be freed */
		g_slist_free(nonlocal_atta_seqno_list);
		viewer_info->selected_attachment_names = NULL;
	} else {
		viewer_info->local_attachments = local_attachments;
		viewer_info->seqno_list = nonlocal_atta_seqno_list;
		viewer_info->mimeinfo_list = nonlocal_attachments;
		if (viewer_info->mimeinfo_list == NULL) {
			saving_local_attachments(local_attachments, viewer_info->msgid,
						 viewer_info);
			g_slist_free(local_attachments);
			viewer_info->local_attachments = NULL;
		}
		/*retriveing nonloacl attachments */
		else
			get_nonlocal_attachments(nonlocal_attachments, viewer_info,
						 nonlocal_atta_seqno_list, count);
	}


	g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(pathslist);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is used to update the global window pointers.
 Whether they are IN_USE during next & previous message retrival.
 
 @param viewInfo is the HeaderInfo pointer for the next message.
   This param is freed inside this function by calling ui_freeHeaderInfo().
 @param folderModelNum is the number representing the folder.
 @param viewer_info is the ViewerInfo structure pointer  of the current message.
   This param is freed in side this function by calling ui_free_viewerinfo().
 
 @return gboolean TRUE-successful closing of viewer
                  FALSE-else & input parameter validation fails.
*/
static gboolean
ui_update_global_window_tag(HeaderInfo * viewInfo, gint folderModelNum, ViewerInfo * viewer_info)
{
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;

	CHECK_PARAM(viewInfo == NULL, "Input param check failed\n", return FALSE);


	for (node = email_main_view->open_windows; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) node->data;
		if (open_window->msgid == NULL || viewInfo->msgid == NULL
		    || open_window->window_ptr == NULL) {
			ULOG_DEBUG("[%s][%d] : Invalid data in open windows list", __FUNCTION__,
				   __LINE__);
			return FALSE;
		}
		if (strncmp
		    (open_window->msgid, viewInfo->msgid,
		     MAX_STR_LEN(open_window->msgid, viewInfo->msgid)) == 0) {
			email_main_view->open_windows =
			    g_slist_remove_link(email_main_view->open_windows, node);
			email_main_view->open_windows =
			    g_slist_concat(node, email_main_view->open_windows);
			ui_show_next_window();
			ui_update_row_tag_in_folder(folderModelNum, viewInfo->msgid, IN_USE_STATUS,
						    TRUE);
			/* Refreashing unread and unsent message count to folders in folder list view */
			ui_add_folder_name_with_unread_unsent_mails(viewInfo->folderNum);

			g_signal_emit_by_name(hildon_app_get_appview(app), "close");
			ui_flush_gtk_events();

			return TRUE;

		}
	}
	return FALSE;
}

/**
   This function is called by higher layers  for  registering the callback
   for viewing the next message.
   
   @param win is a pointer to the widget that registered the callback.
   @param viewer_info is the ViewerInfo structure pointer  of the current message.
   	This parameter is freed inside this function by calling ui_viewer_close_cb()
   	if the next/prev viewer is successfully opened.
   
   @return void
*/
void ui_view_next_message_cb(GtkWidget * win, ViewerInfo * viewer_info)
{
	gint folderModelNum = -1;
	gint rowNum = -1;
	HeaderInfo *viewInfo = NULL;
	GtkWidget *opened_viewer = NULL;
	GtkWidget *viewer = NULL;
	gboolean reaction = FALSE;
	GtkTreeModel *model = NULL;
	gchar *foldername = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*populate the folder if msg opened after oom signal is recieved */
	if (viewer_info->mail_info->set_prev_settings == TRUE) {
		foldername = ui_get_folder_name(viewer_info->mail_info->folder_num);
		model = ui_get_unsorted_folder_model(viewer_info->mail_info->folder_num);
		gtk_list_store_clear(GTK_LIST_STORE(model));
		ui_populate_folder(foldername, viewer_info->mail_info->folder_num);
		g_free(foldername);
	}
	/*Update the row number of current message */
	viewer_info->mail_info->row_num =
	    ui_get_row_number_of_msg(viewer_info->msgid, viewer_info->mail_info->folder_num);

	rowNum = viewer_info->mail_info->row_num;
	folderModelNum = viewer_info->mail_info->folder_num;

	viewInfo = g_new0(HeaderInfo, 1);
	if (win == viewer_info->menu->next_msg || win == viewer_info->toolbar->nextMsg) {
		viewInfo->rowNum = ++rowNum;
	} else {
		viewInfo->rowNum = --rowNum;
	}
	viewInfo->folderNum = folderModelNum;
	if (ui_get_row_information(viewInfo) == FALSE) {
		ui_freeHeaderInfo(viewInfo);
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_nomore_messages"));
		gtk_widget_set_sensitive(win, FALSE);
		return;
	}


	/*Check if message is already opened.
	   If so simply top the corresponding window */
	if (viewInfo->in_use == TRUE) {
		if (ui_update_global_window_tag(viewInfo, folderModelNum, viewer_info) == TRUE) {
			email_main_view->widgets->opened_viewer = NULL;
		}
	}

	/*Check if message has to be downloaded first */
	if (viewInfo->body_downloaded == FALSE) {
		reaction =
		    get_user_confirmation(_("mcen_nc_get_msg_txt"), _("mcen_bd_retrieve"),
					  NULL, CONFIRM_DIALOG);
		if (reaction == FALSE) {
			ui_freeHeaderInfo(viewInfo);
			return;
		}
	}
	/*keep a reference of the already opened viewer.
	   In the same appview the next message is opened. */
	email_main_view->widgets->opened_viewer = g_object_ref(viewer_info->viewer_appview);
	opened_viewer = email_main_view->widgets->opened_viewer;

	/*Unselect current row in main UI */
	ui_update_row_tag_in_folder(folderModelNum, viewer_info->msgid, IN_USE_STATUS, FALSE);

	viewInfo->headers_hidden = !(viewer_info->mail_info->headers_hidden);
	/*Destroy current viewer window */
	ui_viewer_close_cb(NULL, viewer_info);

	email_main_view->message_handler->currently_selected_row = viewInfo->rowNum;
	gtk_widget_realize(email_main_view->widgets->opened_viewer);
	viewer = GTK_WIDGET(ui_viewer_create(viewInfo));
	if (viewer != NULL) {
		gtk_widget_show(viewer);
		hildon_app_set_appview(app, (HildonAppView *) viewer);
		email_main_view->tags->main_view_topped = FALSE;
	}
	/*The mime_info in this structure is freed with the
	   when the next opened message is closed */
	ui_freeHeaderInfo(viewInfo);
	return;
}

/**
This is a support function for getting the file size.

@param filename is pointer to the path os the file.

@return gchar*--a newly allocated formatted size string.
*/
gchar *ui_file_size(const gchar * fileName)
{
	GnomeVFSFileInfo *info = NULL;
	GnomeVFSResult result;
	gchar *attachmentSize = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (fileName == NULL) {
		return (NULL);
	}
	info = gnome_vfs_file_info_new();
	result = gnome_vfs_get_file_info(fileName, info, GNOME_VFS_FILE_INFO_DEFAULT);
	if (result == GNOME_VFS_OK) {
		ULOG_DEBUG("file properties successfully retrived\n");
		attachmentSize = ui_format_size_value(info->size);
	}
	gnome_vfs_file_info_unref(info);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return (attachmentSize);
}

/*
 This is a support function for reading the message boby.

 @param filename is gchar pointer to the filepath.
 @param buffre_body is the pointer to the GtkTextBuffer.

 @return void.
*/
void ui_read_bodyFile(const gchar * fileName, GtkTextBuffer * buffer_body)
{
	gchar *buffer = NULL;
	GtkTextIter iter;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	ULOG_DEBUG("the filename of boby is %s\n", fileName);

	read_file(fileName, &buffer);
	if (buffer == NULL) {
		g_warning("failed to read %s contents \n", fileName);
		return;
	}
	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(buffer_body), &iter);
	gtk_text_buffer_insert(buffer_body, &iter, (gchar *) buffer, -1);
	g_free(buffer);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is used to get the row reference of the selected message.
 @param row_num is the gint representing the row no.
 @param model is the pointer to the GtkTreeModel.
 @return GtkTreeRowReference pointer.
*/
static GtkTreeRowReference *ui_get_row_reference_for_msg(gint row_num, GtkTreeModel * model)
{
	GtkTreePath *path = NULL;
	GtkTreeRowReference *rowref = NULL;
	path = gtk_tree_path_new_from_indices(row_num, -1);
	rowref = gtk_tree_row_reference_new(model, path);
	return rowref;
}


/**
 This function is called when the user confirms the deletion.
 
 @param win is the widget that invoked the delete message callback.
 @param viewer_info  is the pointer to the ViewerInfo structure of the current viewer.
   This parameter is freed inside this function by calling ui_viewer_close_cb()
   if delete confirmation is true.
   
 @return void
*/
void ui_viewer_delete_message_cb(GtkWidget * win, ViewerInfo * viewer_info)
{
	gint folderNum = -1;
	gint rowNum = -1;
	GtkWidget *window = NULL;
	gboolean reaction = FALSE;
	GtkTreeModel *model = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (viewer_info != NULL && viewer_info->mail_info != NULL) {
		viewer_info->mail_info->row_num =
		    ui_get_row_number_of_msg(viewer_info->msgid,
					     viewer_info->mail_info->folder_num);
		rowNum = viewer_info->mail_info->row_num;
		folderNum = viewer_info->mail_info->folder_num;
		window = viewer_info->viewer_appview;
	}
	if (folderNum < 0 || window == NULL) {
		get_user_confirmation(_("mail_ib_ln_attch_email_del"), NULL, NULL, INFO_BANNER);
		return;
	}

	model = ui_get_unsorted_folder_model(folderNum);

	ui_update_row_tag_in_folder(folderNum, viewer_info->msgid, IN_USE_STATUS, FALSE);
	reaction = ui_manage_selected_items(folderNum, -1, ACTION_DELETE, FALSE,
					    ui_get_row_reference_for_msg(rowNum, model));
	if (reaction == TRUE) {
		ui_viewer_close_cb(win, viewer_info);
	} else {
		ui_update_row_tag_in_folder(folderNum, viewer_info->msgid, IN_USE_STATUS, TRUE);

	}


	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

#if CHANGED_EMAIL
/**
Creates a HeaderInfo structure relevent to the message whose detail is to be shown.
Then launches the message detail dialog.

@param menu message detail menu that is activated.
@param viewer_info  is the pointer to the ViewerInfo structure of the current viewer having info pertaining to a message.
   
@return void
*/
void viewer_message_properties_clicked(GtkWidget * menu, ViewerInfo * viewer_info)
{
	HeaderInfo *headerinfo = NULL;
	MsgHeader *msgheader = NULL;
	GSList *msg_id_list = NULL;
	GSList *msg_header_list = NULL;
	GSList *node = NULL;
	gint msg_size = 0;

	CHECK_PARAM(viewer_info == NULL, "Input param check failed\n", return);

	/*create and fill a header_info structure whose detail is needed. */
	headerinfo = g_new0(HeaderInfo, 1);
	headerinfo->folderNum = viewer_info->mail_info->folder_num;
	headerinfo->rowNum = viewer_info->mail_info->row_num;
	ui_get_row_information(headerinfo);
	msg_id_list = g_slist_append(msg_id_list, headerinfo->msgid);
	msg_header_list = ui_dbus_get_msgheader_list(msg_id_list);
	node = g_slist_nth(msg_header_list, 0);
	if (node != NULL) {
		msgheader = (MsgHeader *) node->data;
		if (msgheader != NULL) {
			msg_size = msgheader->size;
			headerinfo->sent_date = msgheader->date_t;
			headerinfo->recieve_date = msgheader->rx_date;
			headerinfo->creation_date = msgheader->mtime;
			ui_free_msg_header(msgheader);
		}
		ui_create_message_properties_dialog(headerinfo, msg_size);
	} else {
		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
	}

	g_slist_free(msg_id_list);
	g_slist_free(msg_header_list);
	ui_freeHeaderInfo(headerinfo);

	return;
}
#endif
/**
 This function is called when the user confirms the deletion.

 @param win is the widget that invoked the delete message callback.
 @param editorInfo  is the pointer to the ViewerInfo structure of the current viewer.
   This parameter is freed inside this function by calling ui_free_editorinfo()
   if delete confirmation is true.
   
 @return void
*/
void ui_editor_delete_message_cb(GtkWidget * win, EditorInfo * editorInfo)
{
	gint msg_type = 0;
	gint folderNum = -1, row_num = 0;
	GtkWindow *window = NULL;
	gboolean reaction = FALSE;
	gchar *infonote = NULL;
	const gchar *subject = NULL;
	GtkTreeModel *model = NULL;

	ULOG_DEBUG("[%s][%s] :starts here :\n", __FILE__, __FUNCTION__);

	msg_type = editorInfo->composeType;
	window = (GtkWindow *) editorInfo->editor_appview;
	folderNum = editorInfo->folderNum;

	if (folderNum >= 0) {
		model = ui_get_unsorted_folder_model(folderNum);
	}

	if (editorInfo->autosaved == TRUE) {
		ui_get_refresh_folder(folderNum);
	}

	subject = gtk_entry_get_text(GTK_ENTRY(editorInfo->msgFields->subject));
	if (msg_type == OUTBOX_MAIL || msg_type == DRAFT_MAIL) {
		email_main_view->message_handler->delete_draft = TRUE;
		g_free(email_main_view->message_handler->current_draft_subject);
		email_main_view->message_handler->current_draft_subject = NULL;
		if (subject != NULL) {
			email_main_view->message_handler->current_draft_subject = g_strdup(subject);
		}
		ui_update_row_tag_in_folder(folderNum, editorInfo->msgid, IN_USE_STATUS, FALSE);
		reaction = ui_manage_selected_items(folderNum, -1, ACTION_DELETE, FALSE,
						    ui_get_row_reference_for_msg(row_num, model));
		if (reaction != TRUE) {
			ui_update_row_tag_in_folder(folderNum, editorInfo->msgid, IN_USE_STATUS,
						    TRUE);
		}
		email_main_view->message_handler->delete_draft = FALSE;
	} else {
		if (subject != NULL) {
			infonote = g_strdup_printf("%s\n%s", _("emev_nc_delete_message"), subject);
		} else {
			infonote = g_strdup(_("emev_nc_delete_message"));
		}
		reaction =
		    get_user_confirmation(infonote, _("emev_bd_delete_messages_delete"),
					  NULL, CONFIRM_DIALOG);
		g_free(infonote);
	}
	if (reaction == TRUE) {
		ui_update_window_pointers(editorInfo->msgid);
		/* Refreashing unread and unsent message count to folders in folder list view */
		if (folderNum >= 0)
			ui_add_folder_name_with_unread_unsent_mails(editorInfo->folderNum);
		ui_free_editorinfo(editorInfo);
		ui_show_next_window();
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is called when the user confirms the deletion.
 
 @param deletelist is the pointer to the GSList that contains the necessary parameters.

 @return void
*/
void ui_viewer_move_msg_cb(GtkWidget * win, ViewerInfo * viewer_info)
{
	gint folderNum = -1;
	gint rowNum = -1;
	GtkWidget *window = NULL;
	gboolean reaction = FALSE;
	GtkTreeModel *model = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (viewer_info == NULL && viewer_info->mail_info == NULL) {
		return;
	}
	viewer_info->mail_info->row_num =
	    ui_get_row_number_of_msg(viewer_info->msgid, viewer_info->mail_info->folder_num);
	rowNum = viewer_info->mail_info->row_num;
	folderNum = viewer_info->mail_info->folder_num;
	window = viewer_info->viewer_appview;

	if (folderNum < 0 || window == NULL) {
		get_user_confirmation(_("mail_ib_ln_attch_email_del"), NULL, NULL, INFO_BANNER);
		return;
	}

	model = ui_get_unsorted_folder_model(folderNum);

	ui_update_row_tag_in_folder(folderNum, viewer_info->msgid, IN_USE_STATUS, FALSE);

	reaction =
	    ui_create_move_to_folder_dialog(GTK_TREE_VIEW
					    (email_main_view->widgets->folderView),
					    GTK_TREE_VIEW(email_main_view->widgets->
							  messageListView), FALSE,
					    ui_get_row_reference_for_msg(rowNum, model));
	if (reaction == TRUE) {
		ui_viewer_close_cb(win, viewer_info);
	} else {
		ui_update_row_tag_in_folder(folderNum, viewer_info->msgid, IN_USE_STATUS, TRUE);
		ui_update_row_tag_in_folder(folderNum, viewer_info->msgid, MARK_STATUS, FALSE);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This is a helper function to find whether the attachemnt is a mhtml attachemnt.
 
 @param mimeInfo is the MiemInfo structure pointer.
 
 @return MHTML-if mhtml attachemnt.
         NOT_MHTML-else.
	 DECRYPT_ERROR-if decryption fails.
*/
static gint
check_for_mhtml_attachment(const MimeInfo * mimeInfo, gint nth_attach, ViewerInfo * viewer_info)
{
	MimeInfo *partinfo = NULL;
	gint mhtml_attachment = NOT_MHTML;

	for (partinfo = (MimeInfo *) mimeInfo; partinfo != NULL; partinfo = partinfo->children) {
	 /**Checking the content type to be multipart alternative. */
		if (g_ascii_strncasecmp
		    (partinfo->content_type, MULTIPART_ALTERNATIVE,
		     MAX_STR_LEN(partinfo->content_type, MULTIPART_ALTERNATIVE)) == 0
		    || g_ascii_strncasecmp(partinfo->content_type, MULTIPART_RELATED,
					   MAX_STR_LEN(partinfo->content_type,
						       MULTIPART_RELATED)) == 0 ||
		    partinfo->mime_type == MIME_TEXT_HTML) {
			mhtml_attachment = MHTML_ATTACHMENT;
			break;
		} else
		    if (g_ascii_strncasecmp
			(partinfo->content_type, MULTIPART_SIGNED,
			 MAX_STR_LEN(partinfo->content_type, MULTIPART_SIGNED)) == 0
			|| strstr(partinfo->content_type, APP_PKCS7) != NULL) {
			mhtml_attachment = DECRYPT_ERROR;
			break;
		}
	}
	return mhtml_attachment;
}

/**
 This function updates the mimeInfo pointer of the current viewer
 after all the attachemnts are retrieved.
 
 @param viewerInfo is the ViewerInfo structure pointer of current viewer.
 
 @return FALSE-if dbus call fails.
         TRUE-else.
*/
static gboolean update_mimeinfo_for_last_attachment(ViewerInfo * viewer_info)
{

	MimeInfo *mimeInfo = NULL;
	GtkTreeModel *model = NULL;
	guint msgid_hash = 0;

	mimeInfo =
	    ui_dbus_recv_msg_body(viewer_info->msgid, TRUE, TRUE,
				  viewer_info->senderid, viewer_info->receiverid,
				  viewer_info->mail_info->smime_type, FALSE,
				  viewer_info->certid, viewer_info->password, FALSE, TRUE);
	if (mimeInfo == NULL) {
		return FALSE;
	}
	model = ui_get_unsorted_folder_model(viewer_info->mail_info->folder_num);
	msgid_hash = g_str_hash(viewer_info->msgid);

	if (viewer_info->mail_info->email_attachment == FALSE) {
		gtk_tree_model_foreach(model, ui_update_msg_fully_downloaded, &msgid_hash);
		ui_mimeinfo_free(viewer_info->mime_info);
		viewer_info->mime_info = mimeInfo;
	} else {
		assing_new_mimeinfo(viewer_info, mimeInfo);
	}
	return TRUE;

}



/**
 This is call back function for viewing  attached email.

 @param  win is a GtkWidget parameter that initiated the callback.
 @param viewer_info  is the pointer to the ViewerInfo structure of the current viewer.
   This parameter is NOT freed .
  
 @returns void.
*/
void ui_view_attachment_cb(GtkWidget * win, ViewerInfo * viewer_info)
{
	gint atta_type = EMAIL_ATTA;
	GSList *selected_names = NULL;
	gchar *msgid = NULL;
	MimeInfo *tempMimeinfo = NULL;
	gint nth_attach = -1;
	gboolean value = TRUE, last_attachment = FALSE, mhtml_attachment = FALSE;
	MsgFields *msgfields = NULL;
	MailInfo *mailinfo = NULL;
	//GtkTreeSelection *attachment_selection = NULL;
	//GtkTreeIter iter;
	MsgInfo *msginfo = NULL;
	const gchar *font_name = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(viewer_info == NULL, "Input param check failed\n", return);
	CHECK_PARAM(viewer_info->msg_fields == NULL, "Input param check failed\n", return);

	msgfields = viewer_info->msg_fields;
	mailinfo = viewer_info->mail_info;

/*Check if already busy retrieving*/
	if (mailinfo->progress_on == TRUE)
		return;
	selected_names = ui_get_selected_attachment_name(viewer_info->msg_fields);
	if (selected_names == NULL) {
		return;
	}

	if (g_slist_length(selected_names) > 1) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_display_more"));
		g_slist_foreach(selected_names, (GFunc) (g_free), NULL);
		g_slist_free(selected_names);
		return;
	}

	nth_attach = ui_get_attachment_num(msgfields, 0);
	if (nth_attach == -1) {
		g_slist_foreach(selected_names, (GFunc) (g_free), NULL);
		g_slist_free(selected_names);
		return;
	}
	tempMimeinfo = get_mimeinfo_for_attachment(viewer_info->mime_info, nth_attach);

	if (tempMimeinfo == NULL) {
		g_slist_foreach(selected_names, (GFunc) (g_free), NULL);
		g_slist_free(selected_names);
		return;
	}

	if (tempMimeinfo->is_downloaded == FALSE) {
		value =
		    get_user_confirmation(_("emev_nc_retrieve_attachment"),
					  _("emev_bd_retrieve_attachment"), NULL, CONFIRM_DIALOG);
	}
	if (value == FALSE) {
		g_slist_foreach(selected_names, (GFunc) (g_free), NULL);
		g_slist_free(selected_names);
		return;
	}

	msgid = viewer_info->msgid;
	/*viewing other the email type attachments */
	atta_type = ui_get_attachment_path(viewer_info->mime_info, msgid, nth_attach);

	viewer_info->selected_attachment_names = selected_names;
	if (atta_type == EMAIL_ATTA) {
		viewer_info->mode = VIEWING_EMAIL;
	} else {
		viewer_info->mode = VIEWING_ATTA;
		/*viewing  attachment */
		viewer_info->mimeinfo_list =
		    g_slist_append(viewer_info->mimeinfo_list, tempMimeinfo);
		viewer_info->seqno_list =
		    g_slist_append(viewer_info->seqno_list, GINT_TO_POINTER(nth_attach));
		ui_dbus_recv_msg_multiple_attachments(viewer_info->mimeinfo_list,
						      viewer_info->seqno_list, msgid);
		show_progressbar(viewer_info);
		return;
	}

	viewer_info->selected_attachment_names = selected_names;
	/*viewing email attachments */
	tempMimeinfo = get_mimeinfo_for_attachment(viewer_info->mime_info, nth_attach);
	if (tempMimeinfo != NULL && tempMimeinfo->mime_type == MIME_MESSAGE_RFC822) {
		mhtml_attachment =
		    check_for_mhtml_attachment(tempMimeinfo->sub, nth_attach, viewer_info);
		/*for nested smime mails we again get the mimeinfo */
		tempMimeinfo = get_mimeinfo_for_attachment(viewer_info->mime_info, nth_attach);
		if (tempMimeinfo == NULL) {
			g_slist_foreach(selected_names, (GFunc) (g_free), NULL);
			g_slist_free(selected_names);
			viewer_info->selected_attachment_names = NULL;
			return;
		}
		viewer_info->mimeinfo_list =
		    g_slist_append(viewer_info->mimeinfo_list, tempMimeinfo);
		viewer_info->seqno_list =
		    g_slist_append(viewer_info->seqno_list, GINT_TO_POINTER(nth_attach));
		if (mhtml_attachment == DECRYPT_ERROR) {
			get_user_confirmation(_("mcen_in_smime_process_error_decrypt"), NULL, NULL,
					      INFO_BANNER);
		} else if (mhtml_attachment == MHTML_ATTACHMENT) {
			font_name = ui_get_default_font(TRUE);
			if (tempMimeinfo->is_downloaded == TRUE) {
				msginfo =
				    ui_dbus_recv_multipart_attachment(msgid, tempMimeinfo,
								      nth_attach, font_name);
				viewer_info->mail_info->progress_on = TRUE;
				return;

			} else {
				ui_dbus_recv_multipart_attachment(msgid, tempMimeinfo, nth_attach,
								  font_name);
				show_progressbar(viewer_info);
				return;
			}
		} else {
			if (tempMimeinfo->is_downloaded == TRUE) {
				msginfo = ui_dbus_recv_msg_attachment(msgid, tempMimeinfo);
				open_viewer(msginfo, viewer_info, last_attachment, nth_attach);
				ui_freeMem(msginfo, NULL);
				return;
			} else {
				ui_dbus_recv_msg_multiple_attachments(viewer_info->mimeinfo_list,
								      viewer_info->seqno_list,
								      msgid);
				show_progressbar(viewer_info);
				return;
			}
		}
	}
	g_slist_foreach(selected_names, (GFunc) (g_free), NULL);
	g_slist_free(selected_names);
	viewer_info->selected_attachment_names = NULL;
	return;
}


/**
 This function is called to invoke the browser if a url is clicked.
 @param url is the url string to be opened in browser.
 @return void.
 */
void invoke_browser( const gchar * url)
{

	DBusError error;
	DBusConnection *connection = NULL;
	dbus_uint32_t reason = 0;
	dbus_bool_t rv = FALSE;

	dbus_error_init(&error);
	connection = dbus_bus_get(DBUS_BUS_STARTER, &error);

	rv = dbus_bus_start_service_by_name(connection, OSSO_BROWSER_SERVICE, 0, &reason, &error);
	if (rv == FALSE) {
		osso_log(LOG_INFO, "Activate browser service fail\n");
		return;
	}
	if (dbus_error_is_set(&error)) {
		osso_log(LOG_INFO, "Error Activate browser%s\n", error.message);
		dbus_error_free(&error);
		return;
	}
	osso_rpc_run_with_defaults(ui_context, "osso_browser",
				   OSSO_BROWSER_LOAD_URL_REQ, NULL, DBUS_TYPE_STRING,
				   url, DBUS_TYPE_INVALID);
	return;
}
/**
 This function is called to invoke the browser if a url is clicked.
 @param url is the url string to be opened in browser.
 @return void.
 */
void add_to_bookmark(const gchar * url)
{

        DBusError error;
        DBusConnection *connection = NULL;
        dbus_uint32_t reason = 0;
        dbus_bool_t rv = FALSE;

        dbus_error_init(&error);
        connection = dbus_bus_get(DBUS_BUS_STARTER, &error);

        rv = dbus_bus_start_service_by_name(connection, OSSO_BOOKMARK_SERVICE, 0, &reason, &error);
        if (rv == FALSE) {
                osso_log(LOG_INFO, "Activate bookmark service fail\n");
                return;
        }
        if (dbus_error_is_set(&error)) {
                osso_log(LOG_INFO, "Error Activate bookmark%s\n", error.message);
                dbus_error_free(&error);
                return;
        }
        g_print("url is %s\n", url);
        gchar *bookmark_name = g_strdup_printf("%s.bm", "addtobookmark");
        osso_rpc_run_with_defaults(ui_context, "osso_bookmark",
                                   OSSO_BOOKMARK_ADD_BOOKMARK_REQ, NULL, DBUS_TYPE_STRING,
                                   url, DBUS_TYPE_STRING, url, DBUS_TYPE_STRING,
                                   NULL, DBUS_TYPE_STRING, NULL, DBUS_TYPE_INVALID);
        return;
}
/**
 This function is called to get the scroll pos & cursor position while closing
 a mail
 
 @param msgFields is the MsgFields structure pointer.This paramter is not freed in this function.
 @param msgid is the msgid of the current message.

 @return void.
*/
void ui_get_curr_msg_settings(MsgFields * msgFields, const gchar * msgid)
{
	guint scrollpos = 0;
	GtkAdjustment *vadj = NULL;
	GtkTextIter curr_pos;
	GtkTextMark *insert_mark = NULL;
	gint line_num = 0, char_on_line = 0;

	if (msgFields->body == NULL) {
		return;
	}

/*get the scroll position*/
	vadj =
	    gtk_scrolled_window_get_vadjustment((GtkScrolledWindow *) msgFields->scrollwindow_body);
	scrollpos = gtk_adjustment_get_value(vadj);
	ui_openmsg_set_scroll_position(msgid, scrollpos);

/*Get the cursor position*/
	insert_mark =
	    gtk_text_buffer_get_insert(gtk_text_view_get_buffer((GtkTextView *) msgFields->body));
	gtk_text_buffer_get_iter_at_mark(gtk_text_view_get_buffer
					 ((GtkTextView *) msgFields->body), &curr_pos, insert_mark);
	line_num = gtk_text_iter_get_line(&curr_pos);
	char_on_line = gtk_text_iter_get_line_offset(&curr_pos);
	ui_openmsg_set_cursor_position(msgid, line_num, char_on_line);
	return;
}

/**
 This fuction is called to set the cursor position & scroll position
 when the state is restored for a mail.
 
 @param msgFields is the MsgFields structure pointer.this parameter is not freed here.
 @param msgid is the msgid of the current message.
 
 @return void.
*/
void ui_set_saved_msg_values(MsgFields * msgFields, const gchar * msgid)
{
	guint scrollpos = 0;
	GtkAdjustment *vadj = NULL;
	GtkTextIter iter;
	gint line_num = 0, char_on_line = 0;
	if (msgFields->body == NULL) {
		return;
	}
	/*setting the scrollposition. */
	scrollpos = ui_openmsg_get_scroll_position(msgid);
	vadj =
	    gtk_scrolled_window_get_vadjustment((GtkScrolledWindow *) msgFields->scrollwindow_body);
	gtk_adjustment_set_value(vadj, scrollpos);
	gtk_adjustment_value_changed(vadj);

	/*set the cursor position */
	ui_openmsg_get_cursor_position(msgid, &line_num, &char_on_line);
	gtk_widget_grab_focus(msgFields->body);
	gtk_text_buffer_get_start_iter(gtk_text_view_get_buffer
				       ((GtkTextView *) msgFields->body), &iter);
	gtk_text_iter_set_line(&iter, line_num);
	gtk_text_iter_set_line_offset(&iter, char_on_line);
	gtk_text_buffer_place_cursor(gtk_text_view_get_buffer
				     ((GtkTextView *) msgFields->body), &iter);
	return;
}

/**
 Restoring the saved state for the message
 @param viewerInfo is the ViewerInfo  structure pointer.
 This paramter is not freed in this function.

 @return void.
*/
void restore_viewer_saved_state(ViewerInfo * viewerInfo)
{
	gint zoomvalue = DEFAULT_ZOOM;
	/*set the scrollpostion & the cursor position */
	ui_set_saved_msg_values(viewerInfo->msg_fields, viewerInfo->msgid);


	/*setting the zoom state */
	zoomvalue = ui_openmsg_get_zoom_value(viewerInfo->msgid);
	if (zoomvalue == DEFAULT_ZOOM)
		return;

	g_free(viewerInfo->zoominfo);
	viewerInfo->zoominfo = NULL;
	viewerInfo->zoominfo =
	    set_initial_zoom_level((GtkTextView *) viewerInfo->msg_fields->body,
				   viewerInfo->msg_fields->mail_viewer,
				   viewerInfo->msgid, zoomvalue);

	if (zoomvalue == ZOOM_OUT) {
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) viewerInfo->menu->
					       zoom_out_max_level, TRUE);
	} else if (zoomvalue == ZOOM_MEDIUM) {
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) viewerInfo->menu->
					       zoom_out_level, TRUE);
	} else if (zoomvalue == ZOOM_IN) {
		gtk_check_menu_item_set_active((GtkCheckMenuItem *) viewerInfo->menu->
					       zoom_in_level, TRUE);
	}
	ui_set_zoom(viewerInfo->zoominfo, zoomvalue);
	return;
}

/*
 This function is called when a request for the application is made to
 go down,then all the values of the open viewers is obtained &
 the values are stored.
 
 @param viewerapp is the pointer to the opened viewer appview.
 @param viewerInfo is the ViewerInfo  structure pointer.
 This paramter is not freed in this function.
 
 @return void.
*/
void ui_viewer_get_state_values_cb(GtkWidget * viewerapp, ViewerInfo * viewerInfo)
{
	if (viewerInfo == NULL && viewerInfo->msg_fields == NULL && viewerInfo->msgid == NULL) {
		return;
	}
	ui_get_curr_msg_settings(viewerInfo->msg_fields, viewerInfo->msgid);
	return;
}


/**
 This function is used to show different infonote for save/view button/menu
	 is clicked when insensitive.
	 
 @param insensitive_widget is the GtkWidget for which the insensitive_press
	 signal is fired.
 @param fields is the pointer to the MsgFileds structure.
 
 @return void.
 */
void save_view_dimming_note(GtkWidget * insensitive_widget, ViewerInfo * viewer)
{
	GtkWidget *view = NULL;

	/*Updated for S+ reqt.If progress is on, dimmed save/view is clicked,
	 * now info banner should be shown*/
	if (viewer->mail_info->progress_on == TRUE) {
		show_dimming_note(insensitive_widget, _("mail_ib_notavailable_downloading"));
		return;
	}

	/*insensistive press on view menu while not in download state. */
	if (insensitive_widget == viewer->menu->view) {
		show_dimming_note(insensitive_widget, _("mcen_ib_no_attachment_selected"));
		return;
	}

	/*insensistive press on save item while not in download state. */
	view = gtk_window_get_focus((GtkWindow *) app);
	if (GTK_IS_TREE_VIEW(view) == FALSE) {
		show_dimming_note(insensitive_widget, _("mcen_ib_no_attachment_selected"));
	} else {
		show_dimming_note(insensitive_widget, _("mcen_ib_unable_to_save_attach_mail"));
	}
	return;
}

/**
 This function is used to show different infonote for purge button
 is clicked when insensitive.
 
 @param insensitive_widget is the GtkWidget for which the insensitive_press
 	signal is fired.
 @param viewer is the pointer to the ViewerInfo structure.
 
 @return void.
 */
void purge_dimming_note(GtkWidget * insensitive_widget, ViewerInfo * viewer)
{
	GtkWidget *view = NULL;
	gint not_email = ATTACH_NOTRETRIEVED;
	gint nth_attach = 0;
	MimeInfo *mime_info = NULL;
	view = gtk_window_get_focus((GtkWindow *) app);

	if (viewer->mail_info->progress_on == TRUE)
		return;

	if (GTK_IS_TREE_VIEW(view) == FALSE && viewer->mail_info->smime_type == SMIME_NONE
	    && viewer->mail_info->email_attachment == FALSE
	    && viewer->msg_fields->attachments_available) {
		show_dimming_note(insensitive_widget, _("mcen_ib_no_attachment_selected"));
	} else if (viewer->mail_info->smime_type != SMIME_NONE ||
		   !viewer->msg_fields->attachments_available)
		show_dimming_note(insensitive_widget, _("mail_ib_nothing_to_purge"));
	else if (viewer->mail_info->email_attachment == TRUE)
		show_dimming_note(insensitive_widget,
				  _("mail_ib_unable_to_pure_attach_pop_mail_plural"));
	else if (viewer->mail_info->protocol != PROTO_IMAP4) {
		show_dimming_note(insensitive_widget,
				  _("mail_ib_unable_to_pure_attach_pop_mail_singular"));
	} else if (viewer->mail_info->folder_num == INBOX_MODEL_NUM) {
		not_email = ui_check_attachment_type(viewer);
		if (not_email != EMAIL_ATTACH) {
			nth_attach = ui_get_attachment_num(viewer->msg_fields, 0);
			mime_info = get_mimeinfo_for_attachment(viewer->mime_info, nth_attach);
			if (mime_info != NULL && mime_info->is_purged == FALSE) {
				show_dimming_note(insensitive_widget,
						  _("mail_ib_attach_not_local"));
			} else {
				show_dimming_note(insensitive_widget,
						  _("mail_ib_attachment_already_purged"));
			}
		} else {
			show_dimming_note(insensitive_widget, _("mail_ib_ln_attch_email_del"));
		}
	}
	return;
}

void ui_viewer_fontsettings_cb(GtkWidget * fontsettingsmenu, ViewerInfo * viewer)
{

	ui_on_font_setting_activate(fontsettingsmenu);
	return;
}

void ui_viewer_searchemails_cb(GtkWidget * searchmailsmenu)
{
	ui_on_search_emails(searchmailsmenu, NULL);
	return;
}

/**
 This function is used to check whether the attachments are purged or not 
 and accordingly return the value

 @param viewer_info is the VieweInfo structure pointer.
 @param selected_attach is the list of selected attachemnts.
 @return NONE_PURGED if the attachments are not purged.
         SOME_PUGRGED if some of the attachments selected are purged.
	 ALL_PURGED if all the selected attachments are purged.
 */
static gint
check_if_purged(ViewerInfo * viewer_info, GSList * selected_attach,
		GSList ** local_attachments, GSList ** noninbox_attachments)
{
	MimeInfo *tempMimeinfo = NULL;
	GSList *selected_names = NULL;
	GtkTreeSelection *selection = NULL;
	GList *pathslist = NULL;
	gint nth = 0, nth_attach = 0, nth_loop = -1;
	FolderAttachInfo *folderAttachInfo = NULL;
	gint purge_status = NONE_PURGED;

	selected_names = selected_attach;
	selection = gtk_tree_view_get_selection((GtkTreeView *) viewer_info->msg_fields->view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, NULL);

	while (selected_names != NULL && selected_names->data != NULL) {
		nth_attach = ui_get_attachment_num(viewer_info->msg_fields, nth);
		nth++;
		tempMimeinfo = get_mimeinfo_for_attachment(viewer_info->mime_info, nth_attach);
		if (tempMimeinfo != NULL && tempMimeinfo->is_purged == TRUE) {
			if (g_slist_length(*local_attachments) == 0) {
				purge_status = ALL_PURGED;
			} else {
				purge_status = SOME_PURGED;
			}
		} else if (tempMimeinfo != NULL && tempMimeinfo->is_downloaded == TRUE) {
			if (viewer_info->mail_info->folder_num == INBOX_MODEL_NUM &&
				tempMimeinfo->section_number) {
				folderAttachInfo = g_new0(FolderAttachInfo, 1);
				folderAttachInfo->section_number =
				    g_strdup(tempMimeinfo->section_number);
				folderAttachInfo->msgid = g_strdup(viewer_info->msgid);
				*local_attachments =
				    g_slist_append(*local_attachments, folderAttachInfo);
			} else {
				*noninbox_attachments =
				    g_slist_append(*noninbox_attachments,
						   GINT_TO_POINTER(tempMimeinfo->fpos));
			}
			if (purge_status == ALL_PURGED) {
				purge_status = SOME_PURGED;
			} else if (purge_status == NOT_LOCAL)
				purge_status = SOME_NOT_LOCAL;
		} else {
			if (g_slist_length(*noninbox_attachments) == 0
			    && g_slist_length(*local_attachments) == 0
			    && purge_status == NONE_PURGED)
				purge_status = NOT_LOCAL;
			else if (purge_status == NONE_PURGED)
				purge_status = SOME_NOT_LOCAL;
		}
		nth_loop = -1;
		selected_names = selected_names->next;
	}
	return purge_status;
}

/**
 helper funtion to get the purge & download status for attachemnt.
 
 @param mimeinfo is teh MiemInfo is structure poinetr.
 @param purge_status is the gboolen OUT param of purge status.
 @param downloaded is the gboolen OUT param of downloaded status.
 @param protocol is the gint representing the protocol.
 @param first_partflag is the gboolean representing whether body part/not.
 @return void.
*/
static void
get_download_and_purge_status(MimeInfo * mimeinfo, gboolean * purge_status,
			      gboolean * downloaded, gint protocol, gboolean first_partflag)
{
	MimeInfo *mimeInfo = NULL;
	gboolean extra_part = FALSE;

	for (mimeInfo = mimeinfo; mimeInfo != NULL; mimeInfo = mimeInfo->next) {
		if (mimeInfo->content_type != NULL && ((g_ascii_strcasecmp
							(mimeInfo->content_type,
							 MULTIPART_MIXED) == 0)
						       ||
						       (g_ascii_strcasecmp
							(mimeInfo->content_type,
							 MULTIPART_REPORT) == 0)
						       || (extra_part =
							   check_for_body(mimeInfo, first_partflag)
							   == TRUE))) {
			get_download_and_purge_status(mimeInfo->children, purge_status,
						      downloaded, protocol, first_partflag);
			if (extra_part == FALSE)
				continue;
		}
		/* check for body part &skip the body part */
		if (first_partflag == TRUE) {
			first_partflag = check_for_body(mimeInfo, first_partflag);
			if (first_partflag == FALSE)
				continue;
		}
		if (mimeInfo->is_downloaded == TRUE) {
			if (protocol == PROTO_IMAP4) {
				(*purge_status) = TRUE;
			}
		} else {
			(*downloaded) = FALSE;
		}
	}
}

/**
 Function to update the purge & downloaded falgs after purging.

 @param viewer_info is the viewerInfo structure pointer.
 @return void.
*/
static void update_purged_downloaded_falgs(ViewerInfo * viewer_info)
{
	gboolean purge_status = FALSE;
	gboolean downloaded = TRUE;
	MimeInfo *mimeinfo = NULL;

	CHECK_PARAM(viewer_info == NULL, "input param failed", return);
	CHECK_PARAM(viewer_info->mail_info == NULL, "input param failed", return);
	CHECK_PARAM(viewer_info->mime_info == NULL, "no attachments", return);
	mimeinfo = viewer_info->mime_info;

	get_download_and_purge_status(mimeinfo, &purge_status, &downloaded,
				      viewer_info->mail_info->protocol, TRUE);


	if (viewer_info->mail_info->email_attachment == FALSE) {
		ui_update_row_tag_in_folder(viewer_info->mail_info->folder_num,
					    viewer_info->msgid, PURGE_STATUS, purge_status);
		ui_update_row_tag_in_folder(viewer_info->mail_info->folder_num,
					    viewer_info->msgid, FULLY_DOWNLOADED_STATUS,
					    downloaded);
	}
	return;
}

/*
 This function changes the attachment icon after its purged or
 retrived.

 @param local_attach_names is the list of attachment names.
 @param viewer_info is the ViewerInfo structure pointer.
 @return void.
 */
static void
change_the_icon(GSList * local_attach_names, ViewerInfo * viewer_info,
		gboolean purged, gchar * attachment_name)
{
	GSList *attachment_names = NULL;
	MimeInfo *tempMimeinfo = NULL;
	GList *pathslist = NULL;
	GtkTreeSelection *selection = NULL;
	gint nth = 0, nth_loop = -1;
	GtkTreeIter iter;
	MsgFields *msgfields = NULL;
	const gchar *mime_string = NULL;
	gchar *icon_string = NULL;
	GdkPixbuf *pixbuf = NULL;
	GError *error = NULL;
	GtkIconTheme *icontheme = NULL;
	gint nth_attach = 0;

	msgfields = viewer_info->msg_fields;
	attachment_names = local_attach_names;
	icontheme = gtk_icon_theme_get_default();

	selection = gtk_tree_view_get_selection((GtkTreeView *) msgfields->view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, NULL);

	while (attachment_names != NULL && attachment_names->data != NULL) {
		nth_attach = ui_get_attachment_num(viewer_info->msg_fields, nth);
		nth++;
		tempMimeinfo = get_mimeinfo_for_attachment(viewer_info->mime_info, nth_attach);

		/*bug fix for 17486,skipping the already purged attachment */
		if (tempMimeinfo != NULL && purged == TRUE
		    && (tempMimeinfo->is_purged == TRUE || tempMimeinfo->is_downloaded == FALSE)) {
			continue;
		}
		if (tempMimeinfo != NULL) {
			mime_string = (gchar *) gnome_vfs_mime_type_from_name((const char *)
									      attachment_names->
									      data);
			if (check_for_vcard_attachment((const char *) attachment_names->data)
			    == TRUE) {
				mime_string = VCARD;
			}
			if (purged == TRUE) {
				tempMimeinfo->is_purged = TRUE;
				tempMimeinfo->is_downloaded = FALSE;
			}
			if (attachment_name != NULL) {
				tempMimeinfo->is_downloaded = TRUE;
				tempMimeinfo->is_purged = FALSE;
			}
			prepare_the_icon_string(mime_string, &icon_string,
						tempMimeinfo->is_downloaded,
						tempMimeinfo->is_purged);
			pixbuf =
			    gtk_icon_theme_load_icon(icontheme, icon_string, ICON_SIZE, 0, &error);
			gtk_tree_model_get_iter((GtkTreeModel *) msgfields->liststore,
						&iter,
						(GtkTreePath *) g_list_nth_data(pathslist,
										nth - 1));
			gtk_list_store_set((GtkListStore *) msgfields->liststore, &iter,
					   COL_ICON, pixbuf, -1);
			/*changing the attcahment name once its downloaded. */
			if (attachment_name != NULL) {
				gtk_list_store_set((GtkListStore *) msgfields->liststore,
						   &iter, COL_ATT_NAME, attachment_name, -1);
			}

			nth_loop = -1;
		}
		attachment_names = attachment_names->next;
	}
	update_purged_downloaded_falgs(viewer_info);
	g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(pathslist);
	return;

}

/**
 helper funtion to get the correction mimeinfo corresponding to attachment.
 
 @param mimeinfo is teh MimeInfo is structure pointer.
 @param attachment_name is the gchar pointer to the name of attachemnt after download.
 @param first_partflag is the gboolean representing whether body part/not.
 @return MimeInfo is structure pointer.
*/
static MimeInfo *get_mimeinfo_for_updation(MimeInfo * mimeinfo, gchar * attachment_name,
					   gboolean first_partflag)
{
	MimeInfo *tempMimeinfo = NULL;
	MimeInfo *mime = NULL;
	gboolean extra_part = FALSE;



	for (tempMimeinfo = mimeinfo; tempMimeinfo != NULL; tempMimeinfo = tempMimeinfo->next) {
		if (tempMimeinfo->content_type != NULL
		    && ((g_ascii_strcasecmp(tempMimeinfo->content_type, MULTIPART_MIXED) == 0)
			|| (g_ascii_strcasecmp(tempMimeinfo->content_type, MULTIPART_REPORT) == 0)
			|| (extra_part = check_for_body(tempMimeinfo, first_partflag)) == TRUE)) {
			mime =
			    get_mimeinfo_for_updation(tempMimeinfo->children,
						      attachment_name, first_partflag);
			if (mime != NULL)
				return mimeinfo;
			if (extra_part == FALSE)
				continue;
		}
		/* check for body part &skip the body part */
		if (first_partflag == TRUE) {
			first_partflag = check_for_body(tempMimeinfo, first_partflag);
			if (first_partflag == FALSE)
				continue;
		}

		if ((tempMimeinfo->filename != NULL
		     && strncmp(attachment_name, tempMimeinfo->filename,
				MAX_STR_LEN(attachment_name, tempMimeinfo->filename)) == 0)) {
			return tempMimeinfo;
		}
	}
	return mime;
}



/**
 This function is used to update the icon of the attachments
 after purging is done from list_view.
 
 @param model is the GtkTreeModel os the attachment field.
 @param path is the GtkTreePath of the attachment field.
 @param iter is the GtkTreeIter of the attachment field.
 @param viewer_info is the pointer to the ViewerInfo structuer pointer.

 @return gboolean TRUE-if call was successful
                  FALSE-else .
 */

static gboolean
update_icon(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, ViewerInfo * viewer_info)
{
	gchar *attachment_name = NULL;
	const gchar *mime_string = NULL;
	MimeInfo *mimeinfo = NULL;
	gchar *icon_string = NULL;
	GdkPixbuf *pixbuf = NULL;
	GError *error = NULL;
	GtkIconTheme *icontheme = NULL;

	mimeinfo = viewer_info->mime_info;
	icontheme = gtk_icon_theme_get_default();
	gtk_tree_model_get((GtkTreeModel *) model, iter, 1, &attachment_name, -1);
	mime_string = (gchar *) gnome_vfs_mime_type_from_name((const char *) attachment_name);

	mimeinfo = get_mimeinfo_for_updation(viewer_info->mime_info, attachment_name, TRUE);

	if (mimeinfo != NULL) {
		prepare_the_icon_string(mime_string, &icon_string, mimeinfo->is_downloaded,
					mimeinfo->is_purged);
		pixbuf = gtk_icon_theme_load_icon(icontheme, icon_string, ICON_SIZE, 0, &error);
		gtk_list_store_set((GtkListStore *) model, iter, COL_ICON, pixbuf, -1);
	}
	g_free(attachment_name);
	return TRUE;
}

void change_purged_icon(GtkWidget * wid, ViewerInfo * viewer)
{

	if (viewer == NULL) {
		return;
	}
	ui_mimeinfo_free(viewer->mime_info);
	viewer->mime_info =
	    ui_dbus_recv_msg_body(viewer->msgid, TRUE, FALSE, viewer->senderid,
				  viewer->receiverid, viewer->mail_info->smime_type,
				  FALSE, viewer->certid, viewer->password, FALSE, TRUE);
	gtk_tree_model_foreach((GtkTreeModel *) viewer->msg_fields->liststore,
			       (GtkTreeModelForeachFunc) update_icon, viewer);

	return;
}

/**
  helper function the fee the folderattachinfo structure used for purging.
  @param folderattachinfo is the FolderAttachInfo structure pointer.
  @return void.
*/
static void free_folderattachinfo(FolderAttachInfo * folderattachinfo)
{
	if (folderattachinfo != NULL) {
		g_free(folderattachinfo->msgid);
		g_free(folderattachinfo->section_number);
		g_free(folderattachinfo);
	}
	return;
}

/**
 helper funtion to get the correction mimeinfo corresponding to attachment.
 
 @param mimeinfo is teh MimeInfo is structure pointer.
 @param purged_secid is the gchar pointer to the secid  of attachment .
 @param first_partflag is the gboolean representing whether body part/not.
 @return MimeInfo is structure pointer.
*/
static MimeInfo *get_purged_mimeinfo(MimeInfo * mimeinfo, gchar * purged_secid,
				     gboolean first_partflag)
{
	MimeInfo *tempMimeinfo = NULL;
	MimeInfo *mime = NULL;
	gboolean extra_part = FALSE;

	for (tempMimeinfo = mimeinfo; tempMimeinfo != NULL; tempMimeinfo = tempMimeinfo->next) {
		if (tempMimeinfo->content_type != NULL
		    && ((g_ascii_strcasecmp(tempMimeinfo->content_type, MULTIPART_MIXED) == 0)
			|| (g_ascii_strcasecmp(tempMimeinfo->content_type, MULTIPART_REPORT) == 0)
			|| (extra_part = check_for_body(tempMimeinfo, first_partflag)) == TRUE)) {
			if ((mime =
			     get_purged_mimeinfo(tempMimeinfo->children, purged_secid,
						 first_partflag)) != NULL)
				return mime;
			if (extra_part == FALSE)
				continue;
		}
/* check for body part &skip the body part*/
		if (first_partflag == TRUE) {
			first_partflag = check_for_body(tempMimeinfo, first_partflag);
			if (first_partflag == FALSE)
				continue;
		}
		if ((tempMimeinfo->section_number != NULL &&
		     strncmp(purged_secid, tempMimeinfo->section_number,
			     MAX_STR_LEN(purged_secid, tempMimeinfo->section_number)) == 0)) {
			return tempMimeinfo;
		}
	}
	return mime;

}

/**
 This function checks the returned sections numbers of the successfull purged attachments.
 & prepares a list of attachment names that were purged.

 @param purged_list is the GSlist of sectionids returned after purging.
 @param viewer_info  is the pointer to the ViewerInfo structure of the current viewer.

 @return GSList of the successful purged attachments.
*/
static GSList *get_successful_purge_list(GSList * purged_list, const ViewerInfo * viewer_info)
{
	MimeInfo *mimeinfo = NULL;
	GSList *purged_attachments = NULL;
	GSList *purged_secid_list = NULL;

	purged_secid_list = purged_list;

	while (purged_secid_list != NULL && purged_secid_list->data != NULL) {
		mimeinfo =
		    get_purged_mimeinfo(viewer_info->mime_info,
					(gchar *) purged_secid_list->data, TRUE);
		if (mimeinfo != NULL) {
			purged_attachments =
			    g_slist_append(purged_attachments, g_strdup(mimeinfo->filename));
		}
		purged_secid_list = purged_secid_list->next;
	}
	return purged_attachments;
}

void ui_viewer_purge_attachment_cb(GtkWidget * purgemenu, ViewerInfo * viewer_info)
{

	MsgFields *msgfields = NULL;
	MailInfo *mailinfo = NULL;
	GSList *selected_names = NULL;
	GtkTreeSelection *attachment_selection = NULL;
	GtkTreeIter iter;

	msgfields = viewer_info->msg_fields;
	mailinfo = viewer_info->mail_info;
	selected_names = ui_get_selected_attachment_name(msgfields);

	if (selected_names == NULL) {
		attachment_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgfields->view));
		gtk_tree_model_get_iter_first(gtk_tree_view_get_model
					      ((GtkTreeView *) msgfields->view), &iter);
		gtk_tree_selection_select_iter(attachment_selection, &iter);
		selected_names = ui_get_selected_attachment_name(viewer_info->msg_fields);
	}
	/*the selected attachments are checked for downloaded/already
	 * purged accordingly purging is proceeded.*/
	process_attachment_purging(viewer_info, selected_names);

	g_slist_foreach(selected_names, (GFunc) g_free, NULL);
	g_slist_free(selected_names);
	return;
}

/**
 This function checks if the selected attachments are downloaded/already
 purged. based on that purging progresses.

 @param viewer_info is the ViewerInfo structure pointer.
 @param selected_names is the list of attachments names.
 
 @return void.
*/
static void process_attachment_purging(ViewerInfo * viewer_info, GSList * selected_names)
{
	GSList *local_attach_names = NULL;
	GSList *noninbox_attachments = NULL;
	GSList *purged_secid_list = NULL;
	GSList *purged_list = NULL;
	gint purge_status = NONE_PURGED, num_of_rows = 0;
	gboolean response = FALSE;
	gchar *infonote = NULL;
	gchar *num = NULL;
	gboolean success = TRUE;
	gint engine_errno = 0;
	MimeInfo *mimeInfo = NULL;

	ULOG_DEBUG_F("starts here");
	purge_status =
	    check_if_purged(viewer_info, selected_names, &local_attach_names,
			    &noninbox_attachments);
	if (purge_status == NOT_LOCAL) {
		gtk_infoprint((GtkWindow *) app, _("mail_ib_attach_not_local"));
		return;
	}

	if (purge_status == NONE_PURGED) {
		if (g_slist_length(local_attach_names) == 1
		    || g_slist_length(noninbox_attachments) == 1) {
			infonote =
			    g_strdup_printf(_("mcen_nc_purge_file_text"),
					    (gchar *) selected_names->data);
		} else {
			num = g_strdup_printf("%d", g_slist_length(selected_names));
			infonote = g_strdup_printf(_("mcen_nc_purge_files_text"), num);
			g_free(num);
		}
		response =
		    get_user_confirmation(infonote, _("mcen_bd_purge_file_remove"), NULL,
					  CONFIRM_DIALOG);
		g_free(infonote);
		if (response == FALSE) {
			g_slist_foreach(local_attach_names, (GFunc) free_folderattachinfo, NULL);
			g_slist_free(local_attach_names);
			/*the data inside the list are intergers,need not free them */
			g_slist_free(noninbox_attachments);
			return;
		}
	}
	switch (purge_status) {
	case NONE_PURGED:
		if (viewer_info->mail_info->folder_num == INBOX_MODEL_NUM && local_attach_names) {
			purged_secid_list = ui_dbus_folder_remove_attachment(local_attach_names);
		} else {
			success =
			    ui_dbus_rem_msg_attachments(viewer_info->msgid, noninbox_attachments);
		}
		break;
	case ALL_PURGED:
		gtk_infoprint((GtkWindow *) app, _("mail_ib_attachment_already_purged"));
		break;
	case SOME_PURGED:
	case SOME_NOT_LOCAL:
		response =
		    get_user_confirmation(_("mcen_nc_some_attachments_not_local"),
					  _("mcen_nc_some_attachments_not_local_yes"),
					  _("mcen_nc_some_attachments_not_local_no"),
					  CONFIRM_DIALOG);
		if (response == FALSE) {
			g_slist_foreach(local_attach_names, (GFunc) free_folderattachinfo, NULL);
			g_slist_free(local_attach_names);
			/*its internal contents are integers need not free them */
			g_slist_free(noninbox_attachments);
			return;
		}
		if (viewer_info->mail_info->folder_num == INBOX_MODEL_NUM) {
			/*purge the local attachemnts */
			purged_secid_list = ui_dbus_folder_remove_attachment(local_attach_names);
		} else {
			success =
			    ui_dbus_rem_msg_attachments(viewer_info->msgid, noninbox_attachments);
		}
		break;
	}
	if (response == TRUE
	    && (purge_status == NONE_PURGED || purge_status == SOME_PURGED
		|| purge_status == SOME_NOT_LOCAL)) {

		if (viewer_info->mail_info->folder_num == INBOX_MODEL_NUM && local_attach_names) {
			if (purged_secid_list == NULL ||
			    g_slist_length(purged_secid_list) !=
			    g_slist_length(local_attach_names)) {
				engine_errno = ui_dbus_get_engine_errno();
				if (engine_errno != ENGINE_ERROR_DEVICE_MEMORY_FULL)
					get_user_confirmation(_
							      ("mail_ib_unable_to_pure_attach_pop_mail_plural"),
							      NULL, NULL, INFO_BANNER);
			} else if (purged_secid_list != NULL
				   && (g_slist_length(purged_secid_list) !=
				       g_slist_length(local_attach_names))) {
				engine_errno = ui_dbus_get_engine_errno();
				if (engine_errno != ENGINE_ERROR_DEVICE_MEMORY_FULL)
					get_user_confirmation(_("emev_ni_internal_error"),
							      NULL, NULL, INFO_BANNER);
			} else {
				purged_list =
				    get_successful_purge_list(purged_secid_list, viewer_info);
				/*change the icon */
				change_the_icon(purged_list, viewer_info, TRUE, NULL);
				g_slist_foreach(purged_list, (GFunc) g_free, NULL);
				g_slist_free(purged_list);
				success = 0;
			}
		} else {
			if (success != 1) {
				/*remove the attachemnt row */
				ui_remove_attachment_from_list(viewer_info->msg_fields,
							       viewer_info->menu->purge_attachment);
			} else {
				get_user_confirmation(_("emev_ni_internal_error"), NULL,
						      NULL, INFO_BANNER);
			}
		}
		
		 /*update the mimeinfo if purging was success*/
                if (success != 1) {
                        /*as the attcahment is purged update dthe mimeinfo corresponding
                           * to msgid */
                        mimeInfo = ui_dbus_recv_msg_body(viewer_info->msgid, TRUE, FALSE,
                                                         viewer_info->senderid,
                                                         viewer_info->receiverid,
                                                         viewer_info->mail_info->smime_type, FALSE,
                                                         viewer_info->certid, viewer_info->password,
                                                         FALSE, TRUE);
                        if (mimeInfo) {
                                ui_mimeinfo_free(viewer_info->mime_info);
                                viewer_info->mime_info = mimeInfo;
                        }
                }
		/*changing the attachment icon in list view & dimming purge/save menu items */
		num_of_rows =
		    gtk_tree_model_iter_n_children((GtkTreeModel *) viewer_info->
						   msg_fields->liststore, NULL);
		if (num_of_rows == 0) {
			ui_update_row_tag_in_folder(viewer_info->mail_info->folder_num,
						    viewer_info->msgid, PURGE_STATUS, FALSE);
			ui_update_row_tag_in_folder(viewer_info->mail_info->folder_num,
						    viewer_info->msgid, ATTACHMENT_STATUS, FALSE);
			gtk_widget_set_sensitive(viewer_info->menu->save_attachment, FALSE);
			gtk_widget_set_sensitive(viewer_info->menu->purge_attachment, FALSE);
			gtk_widget_set_sensitive(viewer_info->toolbar->saveAttachment, FALSE);
			gtk_widget_set_sensitive(viewer_info->menu->view, FALSE);
			viewer_info->msg_fields->attachments_available = FALSE;
		}
		set_header_field_heigth(viewer_info);
		/*freeing the various list */
		if (purged_secid_list != NULL)
			g_slist_free(purged_secid_list);
		if (local_attach_names != NULL) {
			g_slist_foreach(local_attach_names, (GFunc) free_folderattachinfo, NULL);
			g_slist_free(local_attach_names);
		}
		if (noninbox_attachments != NULL)
			g_slist_free(noninbox_attachments);

	}
	ULOG_DEBUG_F("ends here");
	return;
}

/**
 updates the toolbar pixels values on toggling from normal/fullscreen view
 @param viewer_info is the ViewInfo structure pointer
 @return void.
*/

void update_toolbar_view(ViewerInfo * viewer_info)
{
	ViewerToolbar *toolbar = NULL;
	gint margin_btw_buttons = 0;
	gint left_margin = 0, right_margin = 0;
	toolbar = viewer_info->toolbar;


	if (email_main_view->tags->full_screen_mode == TRUE) {
		left_margin = FULL_SCREEN_TOOLBAR_LEFT_MARGIN;
		right_margin = FULL_SCREEN_TOOLBAR_RIGHT_MARGIN;
		margin_btw_buttons = FULL_SCREEN_TOOLBAR_MARGIN_BTW_BUTTONS;
	} else {
		left_margin = TOOLBAR_LEFT_MARGIN;
		right_margin = TOOLBAR_RIGHT_MARGIN;
		margin_btw_buttons = TOOLBAR_MARGIN_BTW_BUTTONS;
	}

	gtk_widget_set_size_request(toolbar->left_space, left_margin, TOOLBAR_BUTTON_HEIGHT);
	gtk_widget_set_size_request(toolbar->reply_space, margin_btw_buttons, TOOLBAR_BUTTON_HEIGHT);
	gtk_widget_set_size_request(toolbar->replyAll_space, margin_btw_buttons, TOOLBAR_BUTTON_HEIGHT);
	gtk_widget_set_size_request(toolbar->resend_space, margin_btw_buttons, TOOLBAR_BUTTON_HEIGHT);
	gtk_widget_set_size_request(toolbar->forward_space, margin_btw_buttons, TOOLBAR_BUTTON_HEIGHT);
	gtk_widget_set_size_request(toolbar->saveAttachment_space, margin_btw_buttons,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_widget_set_size_request(toolbar->moveTo_space, margin_btw_buttons, TOOLBAR_BUTTON_HEIGHT);
	gtk_widget_set_size_request(toolbar->delete_msg_space, margin_btw_buttons, TOOLBAR_BUTTON_HEIGHT);
	gtk_widget_set_size_request(toolbar->prevMsg_space, margin_btw_buttons, TOOLBAR_BUTTON_HEIGHT);
	gtk_widget_set_size_request(toolbar->nextMsg_space, margin_btw_buttons, TOOLBAR_BUTTON_HEIGHT);
	return;
}

void ui_viewer_toggle_to_fullscreen(GtkWidget * wid, ViewerInfo * viewer)
{

	gboolean fullscreen = FALSE;

	if (HILDON_IS_APPVIEW(wid) == FALSE) {
		fullscreen = gtk_check_menu_item_get_active((GtkCheckMenuItem *) wid);
		hildon_appview_set_fullscreen((HildonAppView *) viewer->viewer_appview, fullscreen);
		email_main_view->tags->full_screen_mode = fullscreen;
	} else {

		fullscreen = email_main_view->tags->full_screen_mode;
	}
	/*modify the toobar view on normal/fullscreen views */
	update_toolbar_view(viewer);
	if (fullscreen == FALSE) {
		ui_toggle_toolbar(!(viewer->toolbarhidden_normalview),
				  (HildonAppView *) viewer->viewer_appview,
				  (viewer->viewer_toolbar));
	} else {
		ui_toggle_toolbar(!(viewer->toolbarhidden_fullscreen),
				  (HildonAppView *) viewer->viewer_appview,
				  (viewer->viewer_toolbar));
	}
	if (viewer->mail_info->progress_on == TRUE) {
		/*stoping the progress  */
		if (viewer->mail_info->timer_value > 0) {
			g_source_remove(viewer->mail_info->timer_value);
			viewer->mail_info->timer_value = 0;
		}
		show_progressbar(viewer);
		update_progress_bar_text(viewer->viewer_appview, viewer);
	} else {
		hide_progressbar(viewer);
	}
	return;
}

void ui_editor_toggle_to_fullscreen(GtkWidget * wid, EditorInfo * editor)
{
	gboolean fullscreen = FALSE;

	if (HILDON_IS_APPVIEW(wid) == FALSE) {
		fullscreen = gtk_check_menu_item_get_active((GtkCheckMenuItem *) wid);
		hildon_appview_set_fullscreen((HildonAppView *) editor->editor_appview, fullscreen);
		email_main_view->tags->full_screen_mode = fullscreen;
	} else {
		fullscreen = email_main_view->tags->full_screen_mode;
	}

	if (fullscreen == FALSE) {
		ui_toggle_toolbar(!(editor->toolbarhidden_normalview),
				  (HildonAppView *) editor->editor_appview,
				  (editor->editor_toolbar));
	} else {
		ui_toggle_toolbar(!(editor->toolbarhidden_fullscreen),
				  (HildonAppView *) editor->editor_appview,
				  (editor->editor_toolbar));
	}
	return;
}

void ui_viewer_launch_cm_cb(GtkWidget * wid, ViewerInfo * viewer)
{
	MimeInfo *mime_info = NULL;

	ULOG_DEBUG_F("starts here");
	CHECK_PARAM(viewer == NULL, "input param failed", return);
	CHECK_PARAM(viewer->mime_info == NULL, "input param failed", return);

	mime_info = viewer->mime_info;
	switch (mime_info->smime_type) {
	case SMIME_SIGN:
	case SMIME_SIGN_ENCRYPT:
		ui_viewer_signer_cert_properties(NULL, NULL, viewer);
		break;
	case SMIME_ENCRYPT:
		ui_viewer_recip_cert_properties(NULL, NULL, viewer);
		break;
	default:
		break;
	}
	ULOG_DEBUG_F("ends here");
	return;
}

gboolean ui_viewer_signer_cert_properties(GtkWidget * wid, GdkEvent * event, ViewerInfo * viewer)
{
	MimeInfo *mime_info = NULL;

	ULOG_DEBUG_F("starts here");
	CHECK_PARAM(viewer == NULL, "input param failed", return FALSE);
	CHECK_PARAM(viewer->mime_info == NULL, "input param failed", return FALSE);

	if (viewer->mail_info->email_attachment == TRUE) {
		mime_info = viewer->mime_info->main;
	} else {
		mime_info = viewer->mime_info;
	}
	if (mime_info->saving == TRUE) {
		get_user_confirmation(_("mail_ib_its_new_signer_cert_no_properties"),
				      NULL, NULL, INFO_BANNER);
		return FALSE;
	}
	if (mime_info->smime_type == SMIME_SIGN || mime_info->smime_type == SMIME_SIGN_ENCRYPT) {
		certmanui_certificate_details_dialog(app, TRUE, mime_info->sender_cert_id);
	}
	if (mime_info->smime_type == SMIME_ENCRYPT) {
		certmanui_certificate_details_dialog(app, TRUE, mime_info->recip_cert_id);
	}
	ULOG_DEBUG_F("ends here");
	return FALSE;
}

gboolean ui_viewer_recip_cert_properties(GtkWidget * wid, GdkEvent * event, ViewerInfo * viewer)
{
	MimeInfo *mime_info = NULL;

	ULOG_DEBUG_F("starts here");
	CHECK_PARAM(viewer == NULL, "input param failed", return FALSE);
	CHECK_PARAM(viewer->mime_info == NULL, "input param failed", return FALSE);

	if (viewer->mail_info->email_attachment == TRUE) {
		mime_info = viewer->mime_info->main;
	} else {
		mime_info = viewer->mime_info;
	}
	certmanui_certificate_details_dialog(app, TRUE, mime_info->recip_cert_id);
	ULOG_DEBUG_F("ends here");
	return FALSE;
}

/*
Funtion to check the appropriate zoom menus after
zooming with hardware keys.

@param zoominfo is the pointer to the Zoom structure .
*/
static void set_zoom_menu(Zoom * zoominfo)
{
	CHECK_PARAM(zoominfo == NULL, "input param failed", return);

	GtkWidget *menu = NULL;
	switch (zoominfo->previous_level) {
	case DEFAULT_ZOOM:
		menu = zoominfo->zoom_menu100;
		break;
	case ZOOM_MEDIUM:
		menu = zoominfo->zoom_menu150;
		break;
	case ZOOM_OUT:
		menu = zoominfo->zoom_menu200;
		break;
	case ZOOM_IN:
		menu = zoominfo->zoom_menu50;
		break;
	case ZOOM_80:
		menu = zoominfo->zoom_menu80;
		break;
	case ZOOM_120:
		menu = zoominfo->zoom_menu120;
		break;
	default:
		return;
	}
	g_signal_handlers_block_by_func(G_OBJECT(menu), (gpointer) on_zoom_activate, zoominfo);
	gtk_check_menu_item_set_active((GtkCheckMenuItem *) menu, TRUE);
	g_signal_handlers_unblock_by_func(G_OBJECT(menu), (gpointer) on_zoom_activate, zoominfo);
	return;
}

/**
 This function is used get the next level of zoom when increasing/decreasing the zoom using
 Hardware keys.
 @param current_level is the value of the current zoom level.
 @param increase is the gboolena representing whether we need to
        increase or decrease the zoom level.
 @return the next zoom level value,
         or -1, if max value is reached while increasing the zoom or
                if min value is reached while decreasing the zoom.
*/
static gint get_next_zoom_value(gint current_level, gboolean increase)
{
	gint zoomvalue[] = { 50, 80, 100, 120, 150, 200 };
	gint i = 0, size = sizeof(zoomvalue) / sizeof(gint);

	for (i = 0; i < size; i++) {
		if (current_level == zoomvalue[i]) {
			if (increase && (i + 1) < size)
				return (zoomvalue[i + 1]);
			else if (!increase && ((i - 1) >= 0))
				return (zoomvalue[i - 1]);

			return -1;
		}
	}
	return -1;
}

/*
zooming the viewer with hardware keys.
 increasing the zoom

@param appview is the viewer appview.
@param viewer_info is the pointer to the ViewerInfo structure pointer.

@return .

*/
void increase_zoom_view(GtkWidget * appview, Zoom * zoominfo)
{
	gint zoomvalue = 0;
	GtkWidget *focused_wid = NULL;

	CHECK_PARAM(zoominfo == NULL, "invalid param", return);

	focused_wid = gtk_window_get_focus((GtkWindow *) app);

	if (GTK_IS_TEXT_VIEW(focused_wid) == TRUE) {
		if ((zoominfo->msgbody != NULL
		     && gtk_widget_is_focus((GtkWidget *) (zoominfo->msgbody)) == FALSE)
		    || zoominfo->msgbody == NULL) {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_zoom"));
			return;
		}
	} else if (GTK_IS_EDITABLE(focused_wid) || GTK_IS_TREE_VIEW(focused_wid)) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_zoom"));
		return;
	}

	if (zoominfo->msgbody)
		zoomvalue = zoominfo->previous_level + 50;
	else
		zoomvalue = get_next_zoom_value(zoominfo->previous_level, TRUE);
	if (zoomvalue == -1 || zoomvalue > 200) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_maxzoom_level"));
		return;
	}

	ui_set_zoom(zoominfo, zoomvalue);
	set_zoom_menu(zoominfo);
	return;
}

/*
zooming the viewer with hardware keys.
 decreasing the zoom

@param appview is the viewer appview.
@param viewer_info is the pointer to the ViewerInfo structure pointer.

@return .

*/
void decrease_zoom_view(GtkWidget * appview, Zoom * zoominfo)
{
	gint zoomvalue = 0;
	GtkWidget *focused_wid = NULL;

	CHECK_PARAM(zoominfo == NULL, "invalid param", return);
	ULOG_DEBUG_F("recieved the signal\n");


	focused_wid = gtk_window_get_focus((GtkWindow *) app);
	if (GTK_IS_TEXT_VIEW(focused_wid) == TRUE) {
		if ((zoominfo->msgbody != NULL
		     && gtk_widget_is_focus((GtkWidget *) (zoominfo->msgbody)) == FALSE)
		    || zoominfo->msgbody == NULL) {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_zoom"));
			return;
		}
	} else if (GTK_IS_EDITABLE(focused_wid) || GTK_IS_TREE_VIEW(focused_wid)) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_zoom"));
		return;
	}

	if (zoominfo->msgbody)
		zoomvalue = zoominfo->previous_level - 50;
	else
		zoomvalue = get_next_zoom_value(zoominfo->previous_level, FALSE);
	if (zoomvalue == -1 || zoomvalue < 50) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_minzoom_level"));
		return;
	}

	ui_set_zoom(zoominfo, zoomvalue);
	set_zoom_menu(zoominfo);
	return;
}

/*end of file */
