/**
 * @file folder_mgmt.c
 *
 * This file contains the implementation for folder management module of EMAIL.
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

/* Header file containing all standard paths.*/
#include "ui_common.h"

#include "al_callbacks.h"
#include "utils.h"
#include "folder.h"
#include "defs.h"
#include "editview.h"

/*UI_DBUS user defined header files*/
#include "email_mgr.h"
#include "dbus_utils.h"

/*UI related user defined header files.*/
#include "ui_al_callbacks.h"
#include "ui_interface.h"
#include "folder_mgmt.h"
#include "message_handler.h"
#include "list_view.h"
#include "callback.h"
#include "editcallback.h"
#include "accountMgmt_callback.h"


/* Global variables*/
extern MainViewInfo *email_main_view;	/* Declared in ui_interface.c */

static GtkWidget *ui_create_new_folder_dialog(gpointer data);
static void ui_create_rename_folder_dialog(GtkWindow * window,
					   GtkTreeModel * model,
					   GtkTreeIter * iter, gint model_num,
					   gchar * old_name,
					   GtkTreeModel * manage_model, GtkTreeIter * manage_iter);
/*Helper Functions*/

/**
This is a 'GtkTreeModelForeachFunc' function used to get the list of message
ids of old messages from sent folder.
@param  model           Pointer to the sent folder storage model
@param  path            Pointer to the GtkTreePath of current row
@param  iter            Pointer to the GtkTreeIter of current row
@param  msg_id_list     Pointer to the GSList to store the message ids.
@return boolean         If input param check fails -> TRUE (don't call again)
                        Otherwise -> FALSE (Call again with next row)
*/
static gboolean
ui_get_old_sent_items(GtkTreeModel * model, GtkTreePath * path,
		      GtkTreeIter * iter, GSList ** msg_id_list)
{
	gchar *msg_id = NULL;
	gint sent_time = INIT_VALUE;
	gint sent_folder_clean_time_stamp = INIT_VALUE, sent_folder_setting_value = INIT_VALUE;
	gint sent_folder_clean_option = INIT_VALUE;
	GDate *cur_date = NULL, *old_date = NULL, *msg_date = NULL;
	gint day_count = INIT_VALUE, msg_old_by_num_of_days = INIT_VALUE;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((model == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((iter == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((msg_id_list == NULL), "Input param check failed", return TRUE);

	sent_folder_clean_time_stamp = ui_get_sent_folder_clean_time_stamp();
	sent_folder_setting_value = ui_get_sent_folder_clean_time_value();
	sent_folder_clean_option = ui_get_sent_folder_clean_option();
	cur_date = g_date_new();
	old_date = g_date_new();
	g_date_set_time(cur_date, time(NULL));
	g_date_set_time(old_date, time(NULL));
#if CHANGED_EMAIL
	day_count = sent_folder_setting_value;
#else
	if (sent_folder_clean_option == (DAYS)) {
		day_count = sent_folder_setting_value;
	} else if (sent_folder_clean_option == WEEKS) {
		day_count = sent_folder_setting_value * 7;
	} else if (sent_folder_clean_option == MONTHS) {
		g_date_subtract_months(old_date, sent_folder_setting_value);
		day_count = g_date_days_between(old_date, cur_date);
	}
#endif

	gtk_tree_model_get(model, iter, COL_MSG_ID, &msg_id, COL_SENT_DATE, &sent_time, -1);
	msg_date = g_date_new();
	g_date_set_time(msg_date, sent_time);
	msg_old_by_num_of_days = g_date_days_between(msg_date, cur_date);

	if (msg_old_by_num_of_days < 0) {
		ULOG_DEBUG_F("Sent message time stamp is in future!\n");
	} else if (msg_old_by_num_of_days > day_count) {
		*msg_id_list = g_slist_append(*msg_id_list, g_strdup(msg_id));
	} else if (msg_old_by_num_of_days == day_count) {
		ULOG_DEBUG_F("Day count is matching. Check with time stamp\n");
		if ((sent_time % SECONDS_IN_A_DAY) <
		    (sent_folder_clean_time_stamp % SECONDS_IN_A_DAY)) {
			*msg_id_list = g_slist_append(*msg_id_list, g_strdup(msg_id));
		}
	}

	g_free(msg_id);
	g_date_free(cur_date);
	g_date_free(msg_date);
	g_date_free(old_date);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return FALSE;
}

/**
This is a helper function used to clean Sent folder.
@param  show_note  Boolean flag indicating whether to ask user's confirmation
@return void
*/
void ui_empty_sent_folder(gboolean show_note)
{
	gint response = -1;
	GtkTreeModel *model = NULL;
	GSList *msg_id_list = NULL;
	gboolean user_confirmation = TRUE;
	gint sent_folder_clean_option = INIT_VALUE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (email_main_view == NULL || email_main_view->sorted_model_list == NULL
	    || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	sent_folder_clean_option = ui_get_sent_folder_clean_option();
	CHECK_PARAM((sent_folder_clean_option == DO_NOT_EMPTY),
		    "Sent folder cleaning is OFF", return);

	model = ui_get_unsorted_folder_model(SENT_MODEL_NUM);
	gtk_tree_model_foreach(model, (GtkTreeModelForeachFunc)
			       ui_get_old_sent_items, &msg_id_list);

	/*Proceed to clean only if there are old msgs in Sent folder */
	if (msg_id_list != NULL) {
		if (show_note) {
			user_confirmation =
			    get_user_confirmation(_("mcen_nc_removesent"),
						  _("mcen_bd_folder_settings_ok"),
						  NULL, CONFIRM_DIALOG);
		}

		if (user_confirmation == TRUE) {
			/*Call AL function to clean Sent folder */
			response = ui_dbus_folder_remove_msgs(SENT_DIR, msg_id_list, FALSE);
			if (response != 0) {
				ULOG_DEBUG("[%s][%d] : Cleaning of Sent folder failed!\n",
					   __FUNCTION__, __LINE__);
				ui_set_msg_cb(response);
			}
		} else {
			ULOG_DEBUG("[%s][%d] : User selected not to empty Sent folder!\n",
				   __FUNCTION__, __LINE__);
		}
		g_slist_foreach(msg_id_list, (GFunc) g_free, NULL);
		g_slist_free(msg_id_list);
	}

	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a 'GtkTreeModelForeachFunc' function used to select a row from
the message tree view.
@param  model           Pointer to the folder view storage model
@param  path            Pointer to the GtkTreePath of current row
@param  iter            Pointer to the GtkTreeIter of current row
@param  data            Pointer to message_id hash value
@return boolean         Folder found -> TRUE (don't call again)
                        Otherwise -> FALSE (Call again with next row)
*/
static gboolean
ui_select_msg_foreach_func(GtkTreeModel * model, GtkTreePath * path,
			   GtkTreeIter * iter, gpointer data)
{
	GtkTreeSelection *selection = NULL;
	GtkTreeIter sorted_iter;
	GtkTreeModel *sorted_model = NULL;
	guint msgid_hash1 = 0, msgid_hash2 = 0;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((model == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((data == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((iter == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((email_main_view->widgets->messageListView == NULL),
		    "Input param check failed", return TRUE);

	msgid_hash2 = *(guint *) data;
	gtk_tree_model_get(model, iter, COL_MSGID_HASH, &msgid_hash1, -1);
	if (msgid_hash1 == msgid_hash2) {
		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW
							(email_main_view->widgets->
							 messageListView));
		sorted_model = gtk_tree_view_get_model(GTK_TREE_VIEW
						       (email_main_view->widgets->messageListView));
		gtk_tree_model_sort_convert_child_iter_to_iter(GTK_TREE_MODEL_SORT
							       (sorted_model), &sorted_iter, iter);
		gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
		gtk_tree_selection_select_iter(selection, &sorted_iter);
		gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return TRUE;
	} else {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a helper function used to select a row from the folder tree view
or message tree view.
@param  row_identifier  String to identify the given row : 'folder name' in
                        case of folder view, 'msg-id' for message view.
@param  view_type       folder view / message view
@return void
*/
void ui_select_row_from_view(gchar * row_identifier, gint view_type)
{
	GtkTreeModel *model = NULL;
	gchar *folder_name = NULL, *parent_name = NULL;
	PopupFolderMenu *folder_popup_menu = NULL;
	gint folder_model_num = 0;
	guint msgid_hash = 0;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((row_identifier == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);

	switch (view_type) {
	case VIEW_FOLDER_LIST:
		{
			if (ui_get_folder_names_from_path
			    (row_identifier, &parent_name, &folder_name) == FALSE) {
				ULOG_DEBUG("[%s][%d] : Folder path is invalid!\n",
					   __FUNCTION__, __LINE__);
				return;
			}
			folder_popup_menu =
			    ui_get_folder_activated_structure(parent_name, folder_name);
			if (folder_popup_menu != NULL) {
				/*Populate folder if required, INBOX & OUTBOX are already populated */
				folder_model_num =
				    ui_get_folder_model_number(folder_name, parent_name);
				if (folder_model_num != INBOX_MODEL_NUM
				    && folder_model_num != OUTBOX_MODEL_NUM) {
					ui_populate_folder(row_identifier, folder_model_num);
				}

				/*Select the folder by activating corresponding menu in pop-up
				   folder selector */
				if (!gtk_check_menu_item_get_active
				    (GTK_CHECK_MENU_ITEM(folder_popup_menu->menu_item))) {
					gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
								       (folder_popup_menu->
									menu_item), TRUE);
				}
			} else {
				ULOG_DEBUG("[%s][%d] : Folder is unavailable!\n", __FUNCTION__,
					   __LINE__);
			}

			g_free(parent_name);
			g_free(folder_name);
			break;
		}
	case VIEW_MSG_LIST:
		{
			model = ui_get_unsorted_folder_model(email_main_view->folder_manager->
							     currently_displayed_model);
			msgid_hash = g_str_hash(row_identifier);
			gtk_tree_model_foreach(model, (GtkTreeModelForeachFunc)
					       ui_select_msg_foreach_func, &msgid_hash);
			break;
		}
	default:
		break;
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is used to check whether given child name is already present in
the given parent folder, other wise if parent folder not exists it checks for
folder name in the root level.
@param parent_name  To store parent folder name.
@param new_folder_name  To store the folder name which will be newly created.
@return boolean         A folder already exists with such a name -> TRUE
                        Otherwise -> FALSE
*/
static gboolean
ui_check_folder_name_exists(const gchar * parent_name, const gchar * new_folder_name)
{
	GtkTreeModel *model = NULL;
	GtkTreeIter parent_iter;
	GtkTreeIter child_iter;
	gchar *parent_folder_name = NULL;
	gchar *child_folder_name = NULL;
	const gchar *root_folder_name = NULL;
	gint no_of_children = 0;
	gint count = 0;
	gboolean retval = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM((new_folder_name == NULL), "Input param check failed", return FALSE);

	model = email_main_view->folder_manager->folder_view_model;

	CHECK_PARAM((model == NULL), "Storage model empty for folder view!", return FALSE);
	retval = gtk_tree_model_get_iter_first(model, &parent_iter);
	CHECK_PARAM((retval == FALSE), "Storage model empty for folder view!", return FALSE);

	root_folder_name = parent_name;
	CHECK_PARAM((parent_name == NULL), "Parent folder is null",
		    root_folder_name = new_folder_name);

	/*Traverse the rows until model_num is matched, select the row if found and
	   compare with parent folder name. */
	do {
		gtk_tree_model_get(model, &parent_iter, COL_FOLDER_NAME, &parent_folder_name, -1);
		if (parent_folder_name != NULL
		    && ui_compare_utf8_strings(parent_folder_name, root_folder_name) == 0) {
			if (parent_name != NULL) {
				no_of_children =
				    gtk_tree_model_iter_n_children(model, &parent_iter);
				for (count = 0; count < no_of_children; count++) {
					if ((gtk_tree_model_iter_nth_child(model,
									   &child_iter,
									   &parent_iter,
									   count)) == TRUE) {
						gtk_tree_model_get(model, &child_iter,
								   COL_FOLDER_NAME,
								   &child_folder_name, -1);
						if (child_folder_name != NULL
						    &&
						    ui_compare_utf8_strings
						    (child_folder_name, new_folder_name) == 0) {
							g_free(parent_folder_name);
							g_free(child_folder_name);
							return TRUE;
						}
					}
					g_free(child_folder_name);
				}
			} else {
				g_free(parent_folder_name);
				return TRUE;
			}
		}
		g_free(parent_folder_name);
		parent_folder_name = NULL;
	}
	while (gtk_tree_model_iter_next(model, &parent_iter));
	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return FALSE;
}

/**
This is a helper function to check if the folder name given by user
during new-folder or rename-folder operations is same as any of the
mandatory folders.
@param  folder_name  Name for the folder to be created/renamed
@return gboolean     TRUE --> Folder name is allowed, FALSE --> Not allowed
*/
static gboolean ui_is_folder_name_allowed(const gchar * folder_name)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM((folder_name == NULL), "Input param check failed", return FALSE);

	if (ui_compare_utf8_strings(folder_name, _("mcen_me_folder_inbox")) == 0) {
		ULOG_DEBUG("Folder name can't be same as inbox folder\n");
		return FALSE;
	}
	if (ui_compare_utf8_strings(folder_name, _("mcen_me_folder_outbox")) == 0) {
		ULOG_DEBUG("Folder name can't be same as outbox folder\n");
		return FALSE;
	}
	if (ui_compare_utf8_strings(folder_name, _("mcen_me_folder_sent")) == 0) {
		ULOG_DEBUG("Folder name can't be same as sent folder\n");
		return FALSE;
	}
	if (ui_compare_utf8_strings(folder_name, _("mcen_me_folder_drafts")) == 0) {
		ULOG_DEBUG("Folder name can't be same as drafts folder\n");
		return FALSE;
	}
	if (ui_compare_utf8_strings(folder_name, _("mcen_me_folder_archive")) == 0) {
		ULOG_DEBUG("Folder name can't be same as archive folder\n");
		return FALSE;
	}
	if (ui_compare_utf8_strings(folder_name, INBOX_DIR) == 0) {
		ULOG_DEBUG("Folder name can't be same as inbox folder\n");
		return FALSE;
	}
	if (ui_compare_utf8_strings(folder_name, OUTBOX_DIR) == 0) {
		ULOG_DEBUG("Folder name can't be same as outbox folder\n");
		return FALSE;
	}
	if (ui_compare_utf8_strings(folder_name, SENT_DIR) == 0) {
		ULOG_DEBUG("Folder name can't be same as sent folder\n");
		return FALSE;
	}
	if (ui_compare_utf8_strings(folder_name, DRAFT_DIR) == 0) {
		ULOG_DEBUG("Folder name can't be same as drafts folder\n");
		return FALSE;
	}
	if (ui_compare_utf8_strings(folder_name, ARCHIVE) == 0) {
		ULOG_DEBUG("Folder name can't be same as archive folder\n");
		return FALSE;
	}

	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return TRUE;
}

/**
This is a helper function to find the validity of a folder name.
'_' '#' and '/' characters are not permitted in a valid folder name.
@param  folder_name    Pointer to the name string
@param  parent_window  Pointer to new/rename folder dialog
@param  entry          Pointer to GtkEntry field
@return gboolean
*/
static gboolean
ui_is_folder_name_valid(const gchar * folder_name, GtkWindow * parent_window, GtkWidget * entry)
{
	gchar *char_location1 = NULL;
	gchar *char_location2 = NULL;
	gchar *char_location3 = NULL;
	gchar *error_note = NULL;
	gboolean valid = FALSE;
	gint bufLen = 0;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((folder_name == NULL), "Input param check failed!", return valid);
	CHECK_PARAM((parent_window == NULL), "Input param check failed!", return valid);
	CHECK_PARAM((entry == NULL), "Input param check failed!", return valid);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed!", return valid);
	CHECK_PARAM((email_main_view->folder_manager == NULL),
		    "Input param check failed!", return valid);

	bufLen = g_utf8_strlen(folder_name, -1);

	/*Check if folder name is empty */
	if (bufLen == 0) {
		gtk_infoprint(parent_window, _("mcen_ib_enter_name"));
		gtk_widget_grab_focus(GTK_WIDGET(entry));
	}
	/*Check if folder name exceeds the size limit */
	else if (bufLen > 20) {
		gtk_infoprint(parent_window, _("mcen_ib_maxchar_reached"));
	}
	/*Check if folder name contains illegal characters */
	else {
		char_location1 = g_utf8_strchr(folder_name, bufLen, MSGID_SEPARATOR);
		char_location2 = g_utf8_strchr(folder_name, bufLen, FORWD_SLASH);
		char_location3 = g_utf8_strchr(folder_name, bufLen, DOT);
		if ((char_location1 == NULL && char_location2 == NULL) && char_location3 == NULL) {
			valid = TRUE;
		} else {
			error_note =
			    g_strdup_printf(_("mcen_ib_illegal_chars_error"),
					    FOLDER_NAME_RESTRICTED_CHARS);
			gtk_infoprint(parent_window, error_note);
			/*Focus the text */
			gtk_widget_grab_focus(GTK_WIDGET(entry));
			gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
			g_free(error_note);
		}
	}

	if (valid == TRUE) {
		/* Check if it is named after any mandatory folder */
		if (ui_is_folder_name_allowed(folder_name) == FALSE) {
			gtk_infoprint(parent_window, _("mcen_ib_folder_exists"));
			/*Focus the text */
			gtk_widget_grab_focus(GTK_WIDGET(entry));
			gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
			email_main_view->folder_manager->is_folder_name_taken = FALSE;
			valid = FALSE;
		}
		/*Check if any folder exists already with the new name */
		else if (ui_check_folder_name_exists
			 (email_main_view->folder_manager->parent_folder_name,
			  folder_name) == TRUE) {
			gtk_infoprint(parent_window, _("mcen_ib_folder_exists"));
			/*Focus the text */
			gtk_widget_grab_focus(GTK_WIDGET(entry));
			gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
			email_main_view->folder_manager->is_folder_name_taken = FALSE;
			valid = FALSE;
		}
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return valid;
}

/**
This function is used to delete all messages in a given folder.
@param model_num The model number of the folder.
@param folder_name To hold the folder name.
@return gboolean SUCCESS on deleting of all the mails otherwise
                 FAILURE
*/
gboolean ui_delete_all_msgs_from_folder(gint model_num, gchar * folder_name)
{
	gint num_of_msgs_in_folder = -1, response = -1;
	GtkTreeModel *model = NULL;
	gboolean success = FALSE;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model_num < 0 || folder_name == NULL
	    || email_main_view == NULL || email_main_view->sorted_model_list == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return success;
	}

	model = GTK_TREE_MODEL(g_slist_nth_data(email_main_view->sorted_model_list, model_num));
	if (model == NULL) {
		ULOG_DEBUG("[%s][%d] : Folder model missing!\n", __FUNCTION__, __LINE__);
		return success;
	}
	num_of_msgs_in_folder = gtk_tree_model_iter_n_children(model, NULL);
	if (num_of_msgs_in_folder > 0) {
		/*Call AL function to delete all folder messages first */
		response = ui_dbus_folder_remove_all_msg(folder_name, TRUE);
		if (response != 0) {
			ULOG_DEBUG
			    ("[%s][%d] :'ui_dbus_folder_remove_all_msg'call failed!\n",
			     __FUNCTION__, __LINE__);
			ui_set_msg_cb(response);
		} else {
			success = TRUE;
		}
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return success;
}

/**
This function is used to rename a user defined folder.
@param model The folder tree model.
@param iter To hold the iterator the folder to be renamed.
@param model_num The model number of the folder to be renamed.
@param old_name To hold the old name of the folder.
@param new_name To hold the new name of the folder.
@return void.
*/
static void
ui_rename_folder(GtkTreeModel * model, GtkTreeIter * iter, gint model_num,
		 gchar * old_name, gchar * new_name,
		 GtkTreeModel * manage_model, GtkTreeIter * manage_iter)
{
	gint response = -1;
	gchar *old_folder_name = NULL;
	gchar *new_folder_name = NULL;
	gchar *display_name = NULL;
	GtkTreeIter child_iter;
	gboolean sub_folder_present = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model == NULL || iter == NULL || old_name == NULL || new_name == NULL
	    || email_main_view == NULL || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}
	if (email_main_view->folder_manager->parent_folder_name != NULL) {
		new_folder_name = g_strdup_printf("%s/%s",
						  email_main_view->folder_manager->
						  parent_folder_name, new_name);
		old_folder_name =
		    g_strdup_printf("%s/%s",
				    email_main_view->folder_manager->parent_folder_name, old_name);
	} else {
		new_folder_name = g_strdup(new_name);
		old_folder_name = g_strdup(old_name);
	}
	response = ui_dbus_folder_rename(old_folder_name, new_folder_name);

	/*Call AL function for deleting the folder */
	if (response != 0) {
		ULOG_DEBUG
		    ("[%s][%d] : Unable to rename the folder '%s'!\n",
		     __FUNCTION__, __LINE__, old_name);
		ui_set_msg_cb(response);
	} else {
		gtk_tree_store_set(GTK_TREE_STORE(model), iter, COL_FOLDER_NAME,
				   new_name, COL_DISPLAY_NAME, new_name, -1);

		/*Rename corresponding folder menu item */
		ui_on_rename_folder_menu(old_folder_name, new_folder_name);

		/*If folder view in main screen is displaying this folder,
		   modify the folder title bar label */
		if (model_num == email_main_view->folder_manager->currently_displayed_model) {
			ui_set_folderview_title(new_folder_name);
		}
		/* Add unread unsent message count to folder in folder list view. */
		ui_add_folder_name_with_unread_unsent_mails(model_num);

		if (email_main_view->tags->manage_folder_dlg == TRUE) {
			gtk_tree_model_get(model, iter, COL_DISPLAY_NAME, &display_name, -1);
			gtk_tree_store_set(GTK_TREE_STORE(manage_model), manage_iter,
					   COL_MANAGE_DISPLAY_NAME, display_name, -1);
			g_free(display_name);
		}
		g_free(new_folder_name);
		g_free(old_folder_name);
		new_folder_name = NULL;
		old_folder_name = NULL;

		/*Rename the parent name for sub-folders and corresponding folder menu 
		   items if required */
		sub_folder_present = gtk_tree_model_iter_children(model, &child_iter, iter);
		while (sub_folder_present) {
			gtk_tree_model_get(model, &child_iter, COL_FOLDER_NAME, &display_name, -1);

			new_folder_name = g_strdup_printf("%s/%s", new_name, display_name);
			old_folder_name = g_strdup_printf("%s/%s", old_name, display_name);
			ui_on_rename_folder_menu(old_folder_name, new_folder_name);
			g_free(new_folder_name);
			g_free(old_folder_name);
			g_free(display_name);
			new_folder_name = NULL;
			old_folder_name = NULL;

			gtk_tree_store_set(GTK_TREE_STORE(model), &child_iter,
					   COL_PARENT_NAME, new_name, -1);

			sub_folder_present = gtk_tree_model_iter_next(model, &child_iter);
		}
	}
	g_free(new_folder_name);
	g_free(old_folder_name);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a helper function to select a specific row manage folder view, that
corresponds to a given model sequence number.
@param	view		Pointer to GtkTreeView of manage folder view
@param	folder_num	Model sequence number of required folder to be selected
@return void
*/
static void ui_select_manage_folder_from_view(GtkTreeView * manage_view, gint folder_num)
{
	GtkTreeModel *model = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeIter iter, child_iter, parent_iter;
	gint model_num = -1, no_of_children1 = 0, count1 = 0;
	gint count2 = 0, no_of_children2 = 0;
	gboolean seq_found = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (manage_view == NULL || folder_num < 0) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}
	gtk_tree_view_expand_all(manage_view);
	model = gtk_tree_view_get_model(manage_view);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(manage_view));

	if (model == NULL || !gtk_tree_model_get_iter_first(model, &iter)) {
		ULOG_DEBUG("[%s][%d] : Storage model empty for folder view!\n",
			   __FUNCTION__, __LINE__);
		return;
	}

	/*Traverse the rows until model_num is matched, select the row if found. */
	do {
		no_of_children1 = gtk_tree_model_iter_n_children(model, &iter);
		if (no_of_children1 > 0) {
			for (count1 = 0; count1 < no_of_children1; count1++) {
				if ((gtk_tree_model_iter_nth_child(model,
								   &parent_iter, &iter,
								   count1)) == TRUE) {
					gtk_tree_model_get(model, &parent_iter,
							   COL_MANAGE_MODEL_SEQ_NUM,
							   &model_num, -1);
					no_of_children2 =
					    gtk_tree_model_iter_n_children(model, &parent_iter);
					if (folder_num == model_num) {
						gtk_tree_selection_select_iter(selection,
									       &parent_iter);
						ULOG_DEBUG
						    ("[%s][%d] : Row found. Selection done.\n",
						     __FUNCTION__, __LINE__);
						seq_found = TRUE;
						break;
					} else if (no_of_children2 > 0) {
						for (count2 = 0; count2 < no_of_children2; count2++) {
							if ((gtk_tree_model_iter_nth_child
							     (model, &child_iter, &parent_iter,
							      count2)) == TRUE) {
								gtk_tree_model_get(model,
										   &child_iter,
										   COL_MANAGE_MODEL_SEQ_NUM,
										   &model_num, -1);
								if (folder_num == model_num) {
									gtk_tree_selection_select_iter
									    (selection,
									     &child_iter);
									ULOG_DEBUG
									    ("[%s][%d] : Row found. Selection done.\n",
									     __FUNCTION__,
									     __LINE__);
									seq_found = TRUE;
									break;
								}
							}
						}
					}
				}
			}
		}
	}
	while (gtk_tree_model_iter_next(model, &iter) && !seq_found);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/*
This function used to delete the selected folder.
@param view                Tree veiw of the folder list store.
@param model               Folder model which contains all existing folders.
@param iter                Iter of the folder to be deleted.
@param model_num           Model number of the folder to be deleted.
@param folder_name         Actual folder name of the folder to be deleted.
@param folder_display_name Display folder name of the folder to be deleted.
@param manage_view         Pointer to manage folder tree view.
@param manage_iter         Pointer to iter pointing to manage folder tree view.
*/

static void
ui_delete_folder(GtkTreeView * view, GtkTreeModel * model, GtkTreeIter * iter,
		 gint model_num, gchar * folder_name,
		 gchar * folder_display_name, GtkTreeView * manage_view, GtkTreeIter * manage_iter)
{
	GString *message = NULL;
	gint response = -1;
	GtkTreeModel *child_model = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeIter first_iter, child_iter;
	GtkTreeModel *manage_folder_model = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (view == NULL || model == NULL || iter == NULL || folder_name == NULL
	    || folder_display_name == NULL || email_main_view == NULL
	    || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	/*Get user confirmation first */
	message = g_string_new(NULL);
	g_string_printf(message, _("mcen_nc_delete_folder_text"), folder_display_name);
	if (get_user_confirmation
	    (message->str, _("mcen_bd_delete_folder"), NULL, CONFIRM_DIALOG) == TRUE) {
		/*Call AL function for deleting the folder */
		response = ui_dbus_folder_delete(folder_name);
		if (response != 0) {
			ULOG_DEBUG("[%s][%d] : 'ui_dbus_folder_delete' call failed!\n",
				   __FUNCTION__, __LINE__);
			ui_set_msg_cb(response);
		} else {
			child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(model));
			gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT
								       (model), &child_iter, iter);
			if (email_main_view->tags->manage_folder_dlg == TRUE) {
				manage_folder_model =
				    gtk_tree_view_get_model(GTK_TREE_VIEW(manage_view));
				gtk_tree_store_remove(GTK_TREE_STORE(manage_folder_model),
						      manage_iter);
				ui_select_manage_folder_from_view(manage_view, INBOX_MODEL_NUM);
			}
			gtk_tree_store_remove(GTK_TREE_STORE(child_model), &child_iter);

			/*If folder view in main screen is showing this folder,
			   set it to point to Inbox folder */
			if (model_num == email_main_view->folder_manager->currently_displayed_model) {
				selection =
				    gtk_tree_view_get_selection(GTK_TREE_VIEW
								(email_main_view->
								 widgets->folderView));
				gtk_tree_model_get_iter_first(model, &first_iter);
				gtk_tree_selection_select_iter(selection, &first_iter);
			}

			/*Destroy the corresponding folder view store */
			ui_delete_listview_model(model_num);

			//Select first (Inbox) row
			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
			gtk_tree_model_get_iter_first(model, &first_iter);
			gtk_tree_selection_select_iter(selection, &first_iter);

			ui_delete_folder_from_menu(folder_name);
		}
	}

	g_string_free(message, TRUE);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a helper function to select a specific row from folder view, that
corresponds to a given model sequence number.
@param	view		Pointer to GtkTreeView of folder view
@param	folder_num	Model sequence number of required folder to be selected
@return void
*/
static void ui_select_folder_from_view(GtkTreeView * view, gint folder_num)
{
	GtkTreeModel *model = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeIter iter;
	gint model_num = -1;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (view == NULL || folder_num < 0) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	model = gtk_tree_view_get_model(view);
	selection = gtk_tree_view_get_selection(view);

	if (model == NULL || !gtk_tree_model_get_iter_first(model, &iter)) {
		ULOG_DEBUG("[%s][%d] : Storage model empty for folder view!\n",
			   __FUNCTION__, __LINE__);
		return;
	}

	/*Traverse the rows until model_num is matched, select the row if found. */
	do {
		gtk_tree_model_get(model, &iter, COL_MODEL_SEQ_NUM, &model_num, -1);

		if (folder_num == model_num) {
			gtk_tree_selection_select_iter(selection, &iter);
			ULOG_DEBUG("[%s][%d] : Row found. Selection done.\n", __FUNCTION__,
				   __LINE__);
			break;
		}
	}
	while (gtk_tree_model_iter_next(model, &iter));

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a callback function that gets called whenever selection changes in a
folder view. It checks if current selected folder is mandatory folder and dims
'Rename' and 'Delete' buttons.
@param	selection	Pointer to GtkTreeSelection object of folder view
@param	data		Pointer to user data containing required buttons
@return void
*/
static void ui_view_selection_changed_cb(GtkTreeSelection * selection, gpointer data)
{
	GtkWidget *rename_folder = NULL, *delete_folder = NULL, *new_folder = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	gboolean user_created = FALSE;
	gboolean sub_folder = FALSE;
	gint model_num = -1;
	gboolean ret_val = FALSE, is_archive = FALSE;
	GtkWidget *tree_view = NULL;
	gchar *selected_folder_name = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM((selection == NULL), "Input param check failed!", return);
	CHECK_PARAM((data == NULL), "Input param check failed!", return);

	rename_folder = EMAIL_UI_LOOKUP_OBJECT(data, "Rename");
	delete_folder = EMAIL_UI_LOOKUP_OBJECT(data, "Delete");
	tree_view = EMAIL_UI_LOOKUP_OBJECT(data, "tree_view");
	new_folder = EMAIL_UI_LOOKUP_OBJECT(data, "NewFolder");

	CHECK_PARAM((rename_folder == NULL), " Hook Up objects are missing!", return);
	CHECK_PARAM((delete_folder == NULL), " Hook Up objects are missing!", return);
	CHECK_PARAM((tree_view == NULL), " Hook Up objects are missing!", return);

	ret_val = gtk_tree_selection_get_selected(selection, &model, &iter);
	if (ret_val == FALSE) {
		ULOG_DEBUG("[%s][%d] : Nothing is selected!\n", __FUNCTION__, __LINE__);
		return;
	}

	if (model == NULL) {
		ULOG_DEBUG("[%s][%d] : No model set for folder view!\n", __FUNCTION__, __LINE__);
		return;
	}

	gtk_tree_model_get(model, &iter, COL_MANAGE_MODEL_SEQ_NUM, &model_num,
			   COL_MANAGE_USER_CREATED, &user_created,
			   COL_MANAGE_IS_SUBFOLDER, &sub_folder, -1);

	selected_folder_name = ui_get_folder_name(model_num);
	g_free(email_main_view->folder_manager->parent_folder_name);
	email_main_view->folder_manager->parent_folder_name = g_strdup(selected_folder_name);

	gtk_widget_set_sensitive(new_folder, TRUE);

	if (user_created == FALSE) {
		gtk_widget_set_sensitive(rename_folder, FALSE);
		gtk_widget_set_sensitive(delete_folder, FALSE);
		if (ui_compare_utf8_strings(selected_folder_name, ARCHIVE) == 0) {
			is_archive = TRUE;
			gtk_widget_set_sensitive(new_folder, FALSE);
		}
		if (model_num == SP_DEVICE_MODEL_NUM || is_archive) {
			g_signal_connect(delete_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 DGETTEXT_HCS("ckct_ib_unable_to_delete"));
			g_signal_connect(rename_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 DGETTEXT_HCS("ckdg_ib_unable_to_rename"));
			g_signal_connect(new_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 _("mail_in_ui_folder_create_error"));
		}
	} else {
		gtk_widget_set_sensitive(rename_folder, TRUE);
		gtk_widget_set_sensitive(delete_folder, TRUE);

		if (sub_folder == TRUE) {
			gtk_widget_set_sensitive(new_folder, FALSE);
			g_signal_handlers_disconnect_by_func(new_folder, show_dimming_note,
							     _
							     ("mail_ib_unable_to_create_folder_outbox"));
			g_signal_handlers_disconnect_by_func(new_folder, show_dimming_note,
							     _
							     ("mail_ib_unable_to_create_folder_drafts"));
			g_signal_handlers_disconnect_by_func(new_folder, show_dimming_note,
							     _
							     ("mail_ib_unable_to_create_folder_sent"));
			g_signal_handlers_disconnect_by_func(new_folder, show_dimming_note,
							     _("mail_in_ui_folder_create_error"));
		}
	}

	switch (model_num) {
	case INBOX_MODEL_NUM:
		{
			g_signal_connect(delete_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 _("mcen_ib_unable_to_delete_inbox"));
			g_signal_connect(rename_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 _("mcen_ib_unable_to_rename_inbox"));
			break;
		}
	case OUTBOX_MODEL_NUM:
		{
			gtk_widget_set_sensitive(new_folder, FALSE);
			g_signal_connect(new_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 _("mail_ib_unable_to_create_folder_outbox"));
			g_signal_connect(delete_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 _("mcen_ib_unable_to_delete_outbox"));
			g_signal_connect(rename_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 _("mcen_ib_unable_to_rename_outbox"));
			break;
		}
	case DRAFT_MODEL_NUM:
		{
			gtk_widget_set_sensitive(new_folder, FALSE);
			g_signal_connect(new_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 _("mail_ib_unable_to_create_folder_drafts"));
			g_signal_connect(delete_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 _("mcen_ib_unable_to_delete_drafts"));
			g_signal_connect(rename_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 _("mcen_ib_unable_to_rename_drafts"));
			break;
		}
	case SENT_MODEL_NUM:
		{
			gtk_widget_set_sensitive(new_folder, FALSE);
			g_signal_connect(new_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 _("mail_ib_unable_to_create_folder_sent"));
			g_signal_connect(delete_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 _("mcen_ib_unable_to_delete_sent"));
			g_signal_connect(rename_folder, "insensitive_press",
					 G_CALLBACK(show_dimming_note),
					 _("mcen_ib_unable_to_rename_sent"));
			break;
		}
	case MEMORY_CARD_MODEL_NUM:
		{
			gtk_widget_set_sensitive(new_folder, FALSE);
			break;
		}
	default:
		break;
	}
	g_free(selected_folder_name);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is used to create parent folder menu items and sub folder menu
items. On every menu item is created, the folder info will be stored in a
structure and appended to the global list.
@param parent_folder  Pointer to parent folder name.
@param parent_folder  Pointer to sub folder name.
@param user_created    Boolean flag to indicate whether the folder is user created.
@param folder_list_store  Pointer to storage model of folder view.
@return void.
*/
static void
ui_add_new_popupmenu_item(gchar * parent_folder, gchar * folder_name,
			  gboolean user_created, GtkTreeStore * folder_list_store)
{
	GtkWidget *new_menu_item = NULL;
	gint folderPosition = 0;
	PopupFolderMenu *popup_folder_menu = NULL;
	gchar *child_name = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM((folder_list_store == NULL), "Input param check failed!", return);
	CHECK_PARAM((folder_name == NULL), "Input param check failed!", return);

	if (parent_folder != NULL) {
		child_name = g_strdup_printf("%s%s", INDENT, folder_name);
	} else {
		child_name = g_strdup(folder_name);
	}

	new_menu_item = gtk_radio_menu_item_new_with_label
	    (email_main_view->menu->popup_folder_menu_group, child_name);
	g_free(child_name);

	popup_folder_menu = g_new0(PopupFolderMenu, 1);
	CHECK_PARAM((popup_folder_menu == NULL), "Memory allocation failure!", return);
	popup_folder_menu->parent_folder = NULL;
	popup_folder_menu->folder_name = NULL;
	popup_folder_menu->menu_item = NULL;
	if (parent_folder != NULL) {
		popup_folder_menu->parent_folder = g_strdup(parent_folder);
	}
	popup_folder_menu->folder_name = g_strdup(folder_name);
	popup_folder_menu->menu_item = new_menu_item;
	email_main_view->folder_manager->folder_menu_items = g_slist_append
	    (email_main_view->folder_manager->folder_menu_items, popup_folder_menu);

	email_main_view->menu->popup_folder_menu_group =
	    gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(new_menu_item));

	folderPosition = ui_get_folder_row_position_from_view(folder_name, parent_folder);
	gtk_menu_shell_insert(GTK_MENU_SHELL
			      (email_main_view->menu->folders_menu),
			      new_menu_item, (folderPosition + STD_FOLDER_MENUS));

	if (strncmp(folder_name, INBOX_DIR, MAX_STR_LEN(folder_name, INBOX_DIR)) == 0) {
		email_main_view->menu->folder_inbox = new_menu_item;
	}

	g_signal_connect((gpointer) new_menu_item, "activate",
			 G_CALLBACK(ui_on_folder_menu_activated), popup_folder_menu);
	gtk_widget_show(new_menu_item);
	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a helper function to obtain the display name for mandatory folders.
It returns the display name of the folders that are being added to the
folder view, given the folder's storage model number.
@param  folder_num  Folder storage model number
@param  folder_name Name of the folder as stored by underlying engine
@return Returns gchar* folder display name Other wise NULL.
*/
const gchar *ui_get_folder_display_name(gint folder_num, const gchar * folder_name)
{
	const gchar *folder_display_name = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	switch (folder_num) {
	case INBOX_MODEL_NUM:
		{
			folder_display_name = _("mcen_me_folder_inbox");
			break;
		}
	case OUTBOX_MODEL_NUM:
		{
			folder_display_name = _("mcen_me_folder_outbox");
			break;
		}
	case DRAFT_MODEL_NUM:
		{
			folder_display_name = _("mcen_me_folder_drafts");
			break;
		}
	case SENT_MODEL_NUM:
		{
			folder_display_name = _("mcen_me_folder_sent");
			break;
		}
	default:
		{
			folder_display_name = folder_name;
			break;
		}
	}

	if (folder_num == email_main_view->tags->archive_model_num) {
		folder_display_name = _("mcen_me_folder_archive");
	}

	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return folder_display_name;
}

/**
This is a helper function for loading icon pixbufs from the current
icon_theme.

@param  icon       Pixbuf pointer for loading the icon
@param  icon_name  Name of the icon
@param  icon_size  Size of the icon
@return void
*/
void ui_load_icon_pixbuf(GdkPixbuf ** icon, const gchar * icon_name, gint icon_size)
{
	GError *error = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM((icon == NULL), "Input param check failed!", return);
	CHECK_PARAM((email_main_view->icons->icon_theme == NULL),
		    "Input param check failed!", return);

	/*Check if pixbuf is already loaded */
	if (*icon != NULL) {
		ULOG_DEBUG_F("Icon already loaded");
		return;
	}

	*icon = gtk_icon_theme_load_icon(email_main_view->icons->icon_theme,
					 icon_name, icon_size, GTK_ICON_LOOKUP_NO_SVG, &error);
	if (error != NULL) {
		ULOG_DEBUG_F("Error while loading icon '%s' : %s", icon_name, error->message);
		g_error_free(error);
		return;
	}

	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a helper function to get folder icon for displaying in the folder
view.

@param  folder_num  Sequence number of the folder storage model
@return GdkPixbuf*  Pointer to the icon pixbuf for the given folder
*/
GdkPixbuf *ui_get_folder_icon(gint folder_num)
{
	GdkPixbuf *folder_icon = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM((email_main_view->icons == NULL), "Input param check failed!", return NULL);

	if (folder_num == email_main_view->tags->archive_model_num) {
		if (email_main_view->icons->mmc_archive_icon == NULL) {
			ui_load_icon_pixbuf(&email_main_view->icons->mmc_archive_icon,
					    "qgn_list_gene_mmc", ARCHIVE_ICON_SIZE);
		}
		ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
		return email_main_view->icons->mmc_archive_icon;
	}

	switch (folder_num) {
	case INBOX_MODEL_NUM:
		{
			if (email_main_view->icons->inbox_icon == NULL) {
				ui_load_icon_pixbuf(&email_main_view->icons->inbox_icon,
						    "qgn_list_messagin_inbox", INBOX_ICON_SIZE);
			}
			folder_icon = email_main_view->icons->inbox_icon;
			break;
		}
	case OUTBOX_MODEL_NUM:
		{
			if (email_main_view->icons->outbox_icon == NULL) {
				ui_load_icon_pixbuf(&email_main_view->icons->outbox_icon,
						    "qgn_list_messagin_outbox", OUTBOX_ICON_SIZE);
			}
			folder_icon = email_main_view->icons->outbox_icon;
			break;
		}
	case DRAFT_MODEL_NUM:
		{
			if (email_main_view->icons->drafts_icon == NULL) {
				ui_load_icon_pixbuf(&email_main_view->icons->drafts_icon,
						    "qgn_list_messagin_drafts", DRAFTS_ICON_SIZE);
			}
			folder_icon = email_main_view->icons->drafts_icon;
			break;
		}
	case SENT_MODEL_NUM:
		{
			if (email_main_view->icons->sent_folder_icon == NULL) {
				ui_load_icon_pixbuf(&email_main_view->icons->sent_folder_icon,
						    "qgn_list_messagin_sent",
						    SENT_FOLDER_ICON_SIZE);
			}
			folder_icon = email_main_view->icons->sent_folder_icon;
			break;
		}
	default:
		{
			if (email_main_view->icons->user_folder_icon == NULL) {
				ui_load_icon_pixbuf(&email_main_view->icons->user_folder_icon,
						    "qgn_list_gene_fldr_cls",
						    USER_FOLDER_ICON_SIZE);
			}
			folder_icon = email_main_view->icons->user_folder_icon;
			break;
		}
	}

	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return folder_icon;
}

/**
This is a helper function used to add a new folder row into the folder view.
@param	folder_list_store Pointer to storage model of folder view
@param	folder_path	Pointer to character string containing new folder name
@param	user_created	Boolean variable to indicate if the new folder is user-created
@return void
*/
void
ui_add_new_folder(GtkTreeStore * folder_list_store, gchar * folder_path,
		  gboolean user_created, gint unread_unsent_count)
{
	gchar *parent_folder = NULL;
	gchar *child_folder = NULL;
	GtkTreeIter iter;
	sortingField sorting_column = SORT_BY_DATE;
	sortingOrder sort_order = ORDER_DESCENDING;
	GtkTreeIter parent_iter;
	guint folder_num = 0;
	const gchar *folder_display_name = NULL;
	GdkPixbuf *folder_icon = NULL;
	gchar *display_name = NULL;
	gboolean populated = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((folder_list_store == NULL), "Input param check failed!", return);
	CHECK_PARAM((folder_path == NULL), "Input param check failed!", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed!", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed!", return);
	CHECK_PARAM((email_main_view->menu == NULL), "Input param check failed!", return);
	CHECK_PARAM((email_main_view->menu->folders_menu == NULL),
		    "Input param check failed!", return);

	/* This is added for sub folder implementation */
	if (ui_get_folder_names_from_path(folder_path, &parent_folder, &child_folder)) {
		if (parent_folder != NULL) {
			if (ui_get_folder_iter(parent_folder, &parent_iter) == FALSE) {
				ULOG_DEBUG("[%s][%d] : Failed to get at iter \n",
					   __FUNCTION__, __LINE__);
				g_free(parent_folder);
				g_free(child_folder);
				return;
			}
			gtk_tree_store_append(folder_list_store, &iter, &parent_iter);
		} else {
			gtk_tree_store_append(folder_list_store, &iter, NULL);
		}
	} else {
		ULOG_DEBUG("[%s][%d] : Failed to get the parent folder \n", __FUNCTION__, __LINE__);
		return;
	}

	if (ui_compare_utf8_strings(folder_path, INBOX_DIR) == 0) {
		folder_num = INBOX_MODEL_NUM;
		populated = TRUE;
	} else if (ui_compare_utf8_strings(folder_path, OUTBOX_DIR) == 0) {
		folder_num = OUTBOX_MODEL_NUM;
		populated = TRUE;
	} else if (ui_compare_utf8_strings(folder_path, SENT_DIR) == 0) {
		folder_num = SENT_MODEL_NUM;
	} else if (ui_compare_utf8_strings(folder_path, DRAFT_DIR) == 0) {
		folder_num = DRAFT_MODEL_NUM;
	} else {
		folder_num = email_main_view->folder_manager->model_seq_num;
		email_main_view->folder_manager->model_seq_num++;
	}

	if (ui_compare_utf8_strings(folder_path, ARCHIVE) == 0) {
		email_main_view->tags->archive_model_num = folder_num;
	}

	/*Get folder display name - Display name for mandatory folders should be
	   localizable. */
	folder_display_name = ui_get_folder_display_name(folder_num, child_folder);

	if (unread_unsent_count != DEFAULT_UNREAD_UNSENT_COUNT) {
		if (unread_unsent_count != 0) {
			display_name =
			    g_strdup_printf("%s (%d)", folder_display_name, unread_unsent_count);
		} else {
			display_name = g_strdup_printf("%s", folder_display_name);
		}
	} else {
		display_name = g_strdup_printf("%s", folder_display_name);
	}

	/*Get folder icon */
	folder_icon = ui_get_folder_icon(folder_num);

	/*Add entry for the new folder in the FolderView */
	gtk_tree_store_set(folder_list_store, &iter, COL_FOLDER_ICON,
			   folder_icon, COL_FOLDER_NAME, child_folder,
			   COL_MODEL_SEQ_NUM, folder_num,
			   COL_USER_CREATED, user_created, COL_SORT_BY,
			   sorting_column, COL_SORT_ORDER, sort_order,
			   COL_PARENT_NAME, parent_folder, COL_DISPLAY_NAME,
			   display_name, COL_UNREAD_UNSENT_COUNT, unread_unsent_count,
			   COL_FOLDER_POPULATED, populated, -1);

	gtk_tree_view_expand_all(GTK_TREE_VIEW(email_main_view->widgets->folderView));

	/*Prepare and store new model for the new folder */
	ui_create_listview_model();

	ui_add_new_popupmenu_item(parent_folder, child_folder, user_created, folder_list_store);

	/*Add folder settings to UI state info object */
	ui_update_folder_sort_settings(folder_path, SORT_BY_DATE, ORDER_DESCENDING);

	g_free(parent_folder);
	g_free(child_folder);
	g_free(display_name);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a 'GtkTreeModelForeachFunc' function used to check whether a message header is
already present in current folder view. If already present, it updates a global variable.
@param	model		Pointer to the folder view storage model
@param	path		Pointer to the GtkTreePath of current row
@param	iter		Pointer to the GtkTreeIter of current row
@param	data		Pointer to string hash of new header's message-id
@return boolean 	A folder already exists with such a name -> TRUE (don't call again)
			Otherwise -> FALSE (Call again with next row)
*/
gboolean
ui_is_header_already_present(GtkTreeModel * model, GtkTreePath * path,
			     GtkTreeIter * iter, gpointer data)
{
	guint msgid_hash1 = 0;
	guint msgid_hash2 = *(guint *) data;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	gtk_tree_model_get(model, iter, COL_MSGID_HASH, &msgid_hash1, -1);
	if (msgid_hash1 == msgid_hash2) {
		/*Update global flag */
		email_main_view->message_handler->is_message_present = TRUE;
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return TRUE;
	} else {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a 'GtkTreeModelForeachFunc' function used to update the 'action after retrieval'
tag of a message.
@param	model		Pointer to the folder view storage model
@param	path		Pointer to the GtkTreePath of current row
@param	iter		Pointer to the GtkTreeIter of current row
@param	data		Pointer to string hash of message_id
@return boolean 	Found message with given msg_id -> TRUE (don't call again)
			Otherwise -> FALSE (Call again with next row)
*/
gboolean
ui_update_action_after_retrieval(GtkTreeModel * model, GtkTreePath * path,
				 GtkTreeIter * iter, gpointer data)
{
	guint msgid_hash1 = 0, msgid_hash2 = 0;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model == NULL || iter == NULL || data == NULL
	    || email_main_view == NULL || email_main_view->message_handler == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return TRUE;
	}

	msgid_hash2 = *(guint *) data;
	gtk_tree_model_get(model, iter, COL_MSGID_HASH, &msgid_hash1, -1);
	if (msgid_hash1 == msgid_hash2) {
		gtk_list_store_set(GTK_LIST_STORE(model), iter,
				   COL_ACTION_AFTER_RETRIEVAL,
				   email_main_view->message_handler->
				   set_action_after_retrieval, -1);

		ULOG_DEBUG
		    ("[%s][%s] : Action after retrieval tag for message with id = '%s' set to %d\n",
		     __FILE__, __FUNCTION__, msgid,
		     email_main_view->message_handler->set_action_after_retrieval);
		return TRUE;
	} else {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a 'GtkTreeModelForeachFunc' function used to get the 'action after retrieval'
tag of a message.
@param	model		Pointer to the folder view storage model
@param	path		Pointer to the GtkTreePath of current row
@param	iter		Pointer to the GtkTreeIter of current row
@param	data		Pointer to string hash of message_id
@return boolean 	Found message with given msg_id -> TRUE (don't call again)
			Otherwise -> FALSE (Call again with next row)
*/
gboolean
ui_get_action_after_retrieval(GtkTreeModel * model, GtkTreePath * path,
			      GtkTreeIter * iter, gpointer data)
{
	guint msgid_hash1 = 0, msgid_hash2 = 0;
	gint action = -1;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model == NULL || iter == NULL || data == NULL
	    || email_main_view == NULL || email_main_view->message_handler == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return TRUE;
	}

	msgid_hash2 = *(guint *) data;
	gtk_tree_model_get(model, iter, COL_MSGID_HASH, &msgid_hash1,
			   COL_ACTION_AFTER_RETRIEVAL, &action, -1);
	if (msgid_hash1 == msgid_hash2) {
		email_main_view->message_handler->get_action_after_retrieval = action;

		ULOG_DEBUG
		    ("[%s][%s] : Action after retrieval tag for message with id = '%s' is %d\n",
		     __FILE__, __FUNCTION__, msgid, action);
		return TRUE;
	} else {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a 'GtkTreeModelForeachFunc' function used to update the 'fully downloaded' tag
of a message to TRUE.
@param	model		Pointer to the folder view storage model
@param	path		Pointer to the GtkTreePath of current row
@param	iter		Pointer to the GtkTreeIter of current row
@param	data		Pointer to string hash of message_id
@return boolean 	Found message with given msg_id -> TRUE (don't call again)
			Otherwise -> FALSE (Call again with next row)
*/
gboolean
ui_update_msg_fully_downloaded(GtkTreeModel * model, GtkTreePath * path,
			       GtkTreeIter * iter, gpointer data)
{
	guint msgid_hash1 = 0, msgid_hash2 = 0;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model == NULL || iter == NULL || data == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return TRUE;
	}

	msgid_hash2 = *(guint *) data;
	gtk_tree_model_get(model, iter, COL_MSGID_HASH, &msgid_hash1, -1);
	if (msgid_hash1 == msgid_hash2) {
		gtk_list_store_set(GTK_LIST_STORE(model), iter, COL_FULLY_DOWNLOADED, TRUE, -1);

		ULOG_DEBUG
		    ("[%s][%s] : Fully downloaded tag for message with id = '%s' set to TRUE.\n",
		     __FILE__, __FUNCTION__, msgid);
		return TRUE;
	} else {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a 'GtkTreeModelForeachFunc' function used to update the 'body downloaded' tag
of a message to TRUE.
@param	model		Pointer to the folder view storage model
@param	path		Pointer to the GtkTreePath of current row
@param	iter		Pointer to the GtkTreeIter of current row
@param	data		Pointer to string hash of message_id
@return boolean 	Found message with given msg_id -> TRUE (don't call again)
			Otherwise -> FALSE (Call again with next row)
*/
gboolean
ui_update_msg_body_downloaded(GtkTreeModel * model, GtkTreePath * path,
			      GtkTreeIter * iter, gpointer data)
{
	guint msgid_hash1 = 0, msgid_hash2 = 0;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model == NULL || iter == NULL || data == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return TRUE;
	}

	msgid_hash2 = *(guint *) data;
	gtk_tree_model_get(model, iter, COL_MSGID_HASH, &msgid_hash1, -1);
	if (msgid_hash1 == msgid_hash2) {
		gtk_list_store_set(GTK_LIST_STORE(model), iter, COL_BODY_DOWNLOADED, TRUE, -1);

		ULOG_DEBUG
		    ("[%s][%s] : Body downloaded tag for message with id = '%s' set to TRUE.\n",
		     __FILE__, __FUNCTION__, msgid);
		return TRUE;
	} else {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a 'GtkTreeModelForeachFunc' function used to update the message
listview icon of a message to dustbin icon.
@param	model		Pointer to the folder view storage model
@param	path		Pointer to the GtkTreePath of current row
@param	iter		Pointer to the GtkTreeIter of current row
@param	data		Pointer to the character string containing message-id
@return boolean 	Found message with given msg_id -> TRUE (don't call again)
			Otherwise -> FALSE (Call again with next row)
*/
gboolean
ui_update_msg_dustbin_icon(GtkTreeModel * model, GtkTreePath * path,
			   GtkTreeIter * iter, gpointer data)
{
	guint msgid_hash1 = 0, msgid_hash2 = 0;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model == NULL || iter == NULL || data == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return TRUE;
	}

	if (email_main_view->icons->dustbin_icon == NULL) {
		ui_load_icon_pixbuf(&email_main_view->icons->dustbin_icon,
				    "qgn_toolb_messagin_delete", MSG_ICON_SIZE);
	}

	msgid_hash2 = *(guint *) data;
	gtk_tree_model_get(model, iter, COL_MSGID_HASH, &msgid_hash1, -1);
	if (msgid_hash1 == msgid_hash2) {
		gtk_list_store_set(GTK_LIST_STORE(model), iter, COL_MSG_ICON,
				   email_main_view->icons->dustbin_icon, -1);

		ULOG_DEBUG
		    ("[%s][%s] : List view icon for message with id = '%s' set to Dustbin.\n",
		     __FILE__, __FUNCTION__, msgid);
		return TRUE;
	} else {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a 'GtkTreeModelForeachFunc' function used to show 'attachment icon' for a
specific row in folder view.
@param	model		Pointer to the folder view storage model
@param	path		Pointer to the GtkTreePath of current row
@param	iter		Pointer to the GtkTreeIter of current row
@param	data		Pointer to the character string containing message-id
@return boolean 	Found message with given msg_id -> TRUE (don't call again)
			Otherwise -> FALSE (Call again with next row)
*/
gboolean
ui_update_msg_attachment_icon(GtkTreeModel * model, GtkTreePath * path,
			      GtkTreeIter * iter, gpointer data)
{
	guint msgid_hash1 = 0, msgid_hash2 = 0;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model == NULL || iter == NULL || data == NULL || email_main_view == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return TRUE;
	}

	if (email_main_view->icons->attachment_icon == NULL) {
		ui_load_icon_pixbuf(&email_main_view->icons->attachment_icon,
				    "qgn_list_gene_attacpap", MSG_ICON_SIZE);
	}

	msgid_hash2 = *(guint *) data;
	gtk_tree_model_get(model, iter, COL_MSGID_HASH, &msgid_hash1, -1);
	if (msgid_hash1 == msgid_hash2) {
		gtk_list_store_set(GTK_LIST_STORE(model), iter, COL_ATTACHMENT_ICON,
				   email_main_view->icons->attachment_icon,
				   COL_ATTACHMENT_PRESENT, TRUE, -1);
		ULOG_DEBUG
		    ("[%s][%s] : Enabled attachment icon for message with id = '%s'.\n",
		     __FILE__, __FUNCTION__, msgid);

		return TRUE;
	} else {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a 'GtkTreeModelForeachFunc' function used to find sort order
of a folder.  If found, it updates a global variable.
@param	model	Pointer to the folder view storage model
@param	path	Pointer to the GtkTreePath of current row
@param	iter	Pointer to the GtkTreeIter of current row
@param	data	Pointer to the storage model sequence number
@return boolean	A folder is found corresponding to the sequence number -> TRUE
		Otherwise -> FALSE (Call again with next row)
*/
gboolean
ui_get_folder_sort_order(GtkTreeModel * model, GtkTreePath * path,
			 GtkTreeIter * iter, gpointer data)
{
	gint model_num = -1;
	guint sort_order = 0;
	gint *folder_num = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model == NULL || iter == NULL || data == NULL
	    || email_main_view == NULL || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return TRUE;
	}

	folder_num = (gint *) data;

	gtk_tree_model_get(model, iter, COL_MODEL_SEQ_NUM, &model_num,
			   COL_SORT_ORDER, &sort_order, -1);

	if (*folder_num == model_num) {
		email_main_view->folder_manager->is_folder_present = TRUE;
		email_main_view->folder_manager->folder_sort_order = sort_order;
		ULOG_DEBUG("[%s][%s] : Folder found. sort_order = %d\n", __FILE__,
			   __FUNCTION__, sort_order);
		return TRUE;
	} else {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a 'GtkTreeModelForeachFunc' function used to find unread/unsent
msgs count of a folder.  If found, it updates a global variable.
@param	model	Pointer to the folder view storage model
@param	path	Pointer to the GtkTreePath of current row
@param	iter	Pointer to the GtkTreeIter of current row
@param	data	Pointer to the storage model sequence number
@return boolean	A folder is found corresponding to the sequence number -> TRUE
		Otherwise -> FALSE (Call again with next row)
*/
static gboolean
ui_get_folder_msg_count(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
	gint model_num = -1, msg_count = -1;
	gint *folder_num = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model == NULL || iter == NULL || data == NULL
	    || email_main_view == NULL || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return TRUE;
	}

	folder_num = (gint *) data;

	gtk_tree_model_get(model, iter, COL_MODEL_SEQ_NUM, &model_num,
			   COL_UNREAD_UNSENT_COUNT, &msg_count, -1);

	if (*folder_num == model_num) {
		email_main_view->folder_manager->is_folder_present = TRUE;
		email_main_view->folder_manager->folder_msg_count = msg_count;
		ULOG_DEBUG("[%s][%s] : Folder found. Msg count = %d\n", __FILE__,
			   __FUNCTION__, msg_count);
		return TRUE;
	} else {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a 'GtkTreeModelForeachFunc' function used to reset unread/unsent
msgs count of a folder.
@param	model	Pointer to the folder view storage model
@param	path	Pointer to the GtkTreePath of current row
@param	iter	Pointer to the GtkTreeIter of current row
@param	data	Pointer to the storage model sequence number
@return boolean	A folder is found corresponding to the sequence number -> TRUE
		Otherwise -> FALSE (Call again with next row)
*/
gboolean
ui_reset_folder_msg_count(GtkTreeModel * model, GtkTreePath * path,
			  GtkTreeIter * iter, gpointer data)
{
	gint model_num = -1;
	gint *folder_num = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model == NULL || iter == NULL || data == NULL
	    || email_main_view == NULL || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return TRUE;
	}

	folder_num = (gint *) data;

	gtk_tree_model_get(model, iter, COL_MODEL_SEQ_NUM, &model_num, -1);

	if (*folder_num == model_num) {
		gtk_tree_store_set(GTK_TREE_STORE(model), iter, COL_UNREAD_UNSENT_COUNT, -1, -1);
		ULOG_DEBUG("[%s][%s] : Folder found. Msg count reset\n", __FILE__, __FUNCTION__);
		return TRUE;
	} else {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a 'GtkTreeModelForeachFunc' function used to find the folder name corresponding
to a given storage model sequence number.If found, it updates a global variable.
@param	model		Pointer to the folder view storage model
@param	path		Pointer to the GtkTreePath of current row
@param	iter		Pointer to the GtkTreeIter of current row
@param	data		Pointer to the storage model sequence number
@return boolean 	A folder is found corresponding to the sequence number -> TRUE
			Otherwise -> FALSE (Call again with next row)
*/
static gboolean
ui_find_folder(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
	gint model_num = -1;
	gchar *buf = NULL;
	gint *folder_num = NULL;

	gchar *parent_name = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model == NULL || iter == NULL || data == NULL
	    || email_main_view == NULL || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return TRUE;
	}

	folder_num = (gint *) data;

	gtk_tree_model_get(model, iter, COL_FOLDER_NAME, &buf, COL_MODEL_SEQ_NUM,
			   &model_num, COL_PARENT_NAME, &parent_name, -1);

	if (*folder_num == model_num && buf != NULL) {
		email_main_view->folder_manager->is_folder_present = TRUE;

		/*Clean up previous memory if necessary */
		if (email_main_view->folder_manager->searched_folder_name != NULL)
			g_free(email_main_view->folder_manager->searched_folder_name);

		if (parent_name != NULL) {
			/*Creating sub-folder path */
			email_main_view->folder_manager->searched_folder_name =
			    g_strdup_printf("%s/%s", parent_name, buf);
			g_free(parent_name);
		} else {
			/*Store the folder name */
			email_main_view->folder_manager->searched_folder_name = g_strdup(buf);
		}

		g_free(buf);
		ULOG_DEBUG("[%s][%s] : Folder found. Name = %s\n", __FILE__,
			   __FUNCTION__, email_main_view->folder_manager->searched_folder_name);
		return TRUE;
	} else {
		g_free(buf);
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a helper function used to obtain the folder name corresponding to a given storage
model sequence number.
@param	model_num	Storage model sequence number
@return gchar*		Success -> Pointer to the character string containing the folder name
			Failure -> NULL
*/
gchar *ui_get_folder_name(guint model_num)
{
	gchar *folder_name = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model_num >= g_slist_length(email_main_view->sorted_model_list)
	    || email_main_view == NULL || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return folder_name;
	}

	email_main_view->folder_manager->is_folder_present = FALSE;

	/*Check if any folder exists already with the above name */
	gtk_tree_model_foreach(email_main_view->folder_manager->folder_view_model,
			       ui_find_folder, &model_num);
	if (email_main_view->folder_manager->is_folder_present) {
		folder_name = g_strdup(email_main_view->folder_manager->searched_folder_name);
	}
	ULOG_DEBUG("[%s][%d] : Folder name = %s\n", __FUNCTION__, __LINE__, folder_name);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return folder_name;
}

/**
This is a helper function used to obtain the display string to be shown in
New folder dialog. If a folder name 'New folder' exists, then the name will
increment as per file autonaming in Word and Sketch (e.g., New fodler (01),
New folder (02), etc.)
@param parent_name Folder name that to be checked. 	
@return Returns gchar*	New folder name that can be created Otherwise NULL.
*/
static gchar *ui_get_new_folder_display_string(gchar * parent_name)
{
	gchar *new_folder_str = NULL;
	gint folder_count = 1;
	gboolean is_folder_name_taken = FALSE;
	gchar *temp_str = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (email_main_view == NULL || email_main_view->folder_manager == NULL
	    || email_main_view->folder_manager->folder_view_model == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return new_folder_str;
	}

	is_folder_name_taken = FALSE;
	new_folder_str = g_strdup_printf(_("mcen_ia_default_folder_name"));
	if (ui_check_folder_name_exists(parent_name, new_folder_str) == TRUE) {
		is_folder_name_taken = TRUE;
		while (is_folder_name_taken) {
			is_folder_name_taken = FALSE;
			g_free(new_folder_str);
			new_folder_str = NULL;
			temp_str = g_strdup_printf("%d", folder_count);
			new_folder_str =
			    g_strdup_printf(_("mcen_ia_default_folder_name_s"), temp_str);
			g_free(temp_str);

			if (ui_check_folder_name_exists(parent_name, new_folder_str) == TRUE) {
				is_folder_name_taken = TRUE;
			} else {
				is_folder_name_taken = FALSE;
			}

			folder_count++;
		}
	}

	ULOG_DEBUG("[%s][%s] : New folder display string is %s\n", __FILE__,
		   __FUNCTION__, display_str);
	return new_folder_str;
}

/**
This is a 'GtkTreeModelForeachFunc' function used to update the sort settings
of a folder.
@param  model   Pointer to the folder view storage model
@param  path    Pointer to the GtkTreePath of current row
@param  iter    Pointer to the GtkTreeIter of current row
@param  data    Pointer to FolderSortInfo object
@return boolean Param check failed, or success -> TRUE (don't call again)
                Otherwise -> FALSE (Call again with next row)
*/
static gboolean
ui_apply_folder_sorting_foreach_func(GtkTreeModel * model, GtkTreePath * path,
				     GtkTreeIter * iter, gpointer data)
{
	gchar *current_parent_name = NULL, *current_folder_name = NULL;
	gchar *current_folder_path = NULL;
	FolderSortInfo *sort_info = NULL;
	gint sort_column = -1, model_num = -1;
	GtkSortType sort_order;
	GtkTreeModel *folder_model = NULL;
	GtkTreeSortable *sortable = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (data != NULL) {
		sort_info = (FolderSortInfo *) data;
	}

	/*Check the necessary input & global params */
	CHECK_PARAM((sort_info == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((sort_info->name == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((model == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((iter == NULL), "Input param check failed", return TRUE);

	switch (sort_info->sort_key) {
	case SORT_BY_ADDRESS:
		sort_column = COL_SENDER_RECIPIENT;
		break;
	case SORT_BY_SUBJECT:
		sort_column = COL_SUBJECT;
		break;
	case SORT_BY_SIZE:
		sort_column = COL_SIZE;
		break;
	case SORT_BY_PRIORITY:
		sort_column = COL_PRIORITY;
		break;
	case SORT_BY_READ_STATUS:
		sort_column = COL_READ_STATUS;
		break;
	case SORT_BY_ATTACHMENT:
		sort_column = COL_ATTACHMENT_PRESENT;
		break;
	case SORT_BY_DATE:
	default:
		sort_column = COL_DISPLAY_DATE;
		break;
	}

	switch (sort_info->sort_order) {
	case ORDER_ASCENDING:
		sort_order = GTK_SORT_ASCENDING;
		break;
	case ORDER_DESCENDING:
	default:
		sort_order = GTK_SORT_DESCENDING;
		break;
	}

	gtk_tree_model_get(model, iter, COL_FOLDER_NAME, &current_folder_name,
			   COL_PARENT_NAME, &current_parent_name, COL_MODEL_SEQ_NUM,
			   &model_num, -1);
	if (current_parent_name == NULL) {
		current_folder_path = g_strdup(current_folder_name);
	} else {
		current_folder_path =
		    g_strdup_printf("%s/%s", current_parent_name, current_folder_name);
	}

	/*TO DO : Parse out parent/child names from folder name */
	if (current_folder_path != NULL
	    && strncmp(sort_info->name, current_folder_path,
		       MAX_STR_LEN(sort_info->name, current_folder_path)) == 0) {
		gtk_tree_store_set(GTK_TREE_STORE(model), iter, COL_SORT_BY,
				   sort_info->sort_key, COL_SORT_ORDER, sort_order, -1);
		folder_model = ui_get_sorted_folder_model(model_num);
		sortable = GTK_TREE_SORTABLE(folder_model);
		gtk_tree_sortable_set_sort_column_id(sortable, sort_column, sort_order);

		g_free(current_folder_path);
		g_free(current_parent_name);
		g_free(current_folder_name);
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return TRUE;
	} else {
		g_free(current_folder_path);
		g_free(current_parent_name);
		g_free(current_folder_name);
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This is a helper function used to update the sort settings of a folder.
It is called during initialization after previous UI state is read.
@param  sort_info  Pointer to FolderSortInfo object
@return void
*/
void ui_apply_folder_sorting(FolderSortInfo * sort_info)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM((sort_info == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager->folder_view_model == NULL),
		    "Input param check failed", return);

	gtk_tree_model_foreach(email_main_view->folder_manager->folder_view_model,
			       (GtkTreeModelForeachFunc)
			       ui_apply_folder_sorting_foreach_func, sort_info);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a 'GtkTreeModelForeachFunc' function used to capture the sort settings
of all folders.
@param  model   Pointer to the folder view storage model
@param  path    Pointer to the GtkTreePath of current row
@param  iter    Pointer to the GtkTreeIter of current row
@param  data    Pointer to the list for storing FolderSortInfo objects
@return boolean Param check failed, or success -> TRUE (don't call again)
                Otherwise -> FALSE (Call again with next row)
*/
static gboolean
ui_prepare_folder_sort_settings_foreach_func(GtkTreeModel * model,
					     GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
	FolderSortInfo *sort_info = NULL;
	GSList **folder_sort_info_list = NULL;
	gchar *folder_name = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (data != NULL) {
		folder_sort_info_list = (GSList **) data;
	}

	/*Check the necessary input & global params */
	CHECK_PARAM((folder_sort_info_list == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((model == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((iter == NULL), "Input param check failed", return TRUE);

	sort_info = g_new0(FolderSortInfo, 1);
	if (sort_info == NULL) {
		ULOG_CRIT_F("Memory allocation failure!\n");
		return TRUE;
	}

	gtk_tree_model_get(model, iter, COL_FOLDER_NAME, &folder_name,
			   COL_SORT_BY, &sort_info->sort_key, COL_SORT_ORDER,
			   &sort_info->sort_order, -1);
	sort_info->name = g_strdup(folder_name);

	*folder_sort_info_list = g_slist_append(*folder_sort_info_list, sort_info);

	g_free(folder_name);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return FALSE;
}

/**
This is a helper function used to capture the default sort settings of
all folders. It is called during initialization.
@param  void
@return GSList*  Pointer to list containing FolderSortInfo objects 
                 Otherwise NULL.
*/
GSList *ui_prepare_folder_sort_settings()
{
	GSList *folder_sort_info_list = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	gtk_tree_model_foreach(email_main_view->folder_manager->folder_view_model,
			       (GtkTreeModelForeachFunc)
			       ui_prepare_folder_sort_settings_foreach_func,
			       &folder_sort_info_list);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return folder_sort_info_list;
}

/*Signal Handler Functions*/
/**
This is a signal handler function used when OK button in Sort_Messages Dialog is clicked.
It gets the new sorting preferences and applies them to the current folder view.
@param	sort_key   Integer value of sorting key to be applied
@param	sort_order GTK_SORT_ASCENDING/GTK_SORT_DESCENDING
@return void
*/
void ui_on_ok_sort_messages_dialog(gint sort_key, GtkSortType sort_order)
{
	GSList *node = NULL;
	gint sort_column = -1;
	gint model_num = -1;
	GtkTreeIter iter, child_iter;
	GtkTreeModel *model = NULL, *child_model = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeSortable *sortable = NULL;
	gchar *folder_name = NULL;
	gchar *parent_name = NULL;
	gchar *folder_path = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (email_main_view == NULL || email_main_view->sorted_model_list == NULL
	    || email_main_view->widgets == NULL || email_main_view->widgets->folderView == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	switch (sort_key) {
	case SORT_BY_ADDRESS:
		sort_column = COL_SENDER_RECIPIENT;
		break;
	case SORT_BY_DATE:
		sort_column = COL_DISPLAY_DATE;
		break;
	case SORT_BY_SUBJECT:
		sort_column = COL_SUBJECT;
		break;
	case SORT_BY_SIZE:
		sort_column = COL_SIZE;
		break;
	case SORT_BY_PRIORITY:
		sort_column = COL_PRIORITY;
		break;
	case SORT_BY_READ_STATUS:
		sort_column = COL_READ_STATUS;
		break;
	case SORT_BY_ATTACHMENT:
		sort_column = COL_ATTACHMENT_PRESENT;
		break;
	default:
		break;
	}

	selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	if (model == NULL) {
		ULOG_DEBUG("[%s][%d] : No model set for folder view!\n", __FUNCTION__, __LINE__);
		return;
	}

	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, COL_MODEL_SEQ_NUM, &model_num,
				   COL_FOLDER_NAME, &folder_name, COL_PARENT_NAME,
				   &parent_name, -1);

		/*Set the new sorting preferences for that particular folderview */
		gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT
							       (model), &child_iter, &iter);
		child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(model));
		gtk_tree_store_set(GTK_TREE_STORE(child_model), &child_iter,
				   COL_SORT_BY, sort_key, COL_SORT_ORDER, sort_order, -1);
		/*Add folder settings to UI state info object */
		if (parent_name == NULL) {
			folder_path = g_strdup(folder_name);
		} else {
			folder_path = g_strdup_printf("%s/%s", parent_name, folder_name);
		}
		ui_update_folder_sort_settings(folder_path, sort_key, sort_order);

		node = g_slist_nth(email_main_view->sorted_model_list, (guint) model_num);
		sortable = GTK_TREE_SORTABLE(node->data);
		/*Update the new sorting preferences in corresponding message view model */
		g_object_set_data(G_OBJECT(sortable), "sort_order", GINT_TO_POINTER(sort_order));
		gtk_banner_show_animation(GTK_WINDOW(app), DGETTEXT_HCS("ckdg_pb_updating"));
		ui_flush_gtk_events();

		gtk_tree_sortable_set_sort_column_id(sortable, sort_column, sort_order);
		gtk_banner_close(GTK_WINDOW(app));

		g_free(folder_name);
		g_free(parent_name);
		g_free(folder_path);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a signal handler function used when NEW button is clicked in Manage_Folders and
Move_To Dialogs. It opens the New_Folder Dialog for creating a new user folder.
@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
void ui_on_new_manage_folders_dialog(GtkWidget * widget, gpointer data)
{
	GtkWidget *new_folder_dialog = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (widget == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	new_folder_dialog = GTK_WIDGET(ui_create_new_folder_dialog(data));
	if (new_folder_dialog == NULL) {
		ULOG_DEBUG("[%s][%d] : Creation of New_Folder Dialog failed!\n",
			   __FUNCTION__, __LINE__);
	} else {
		gtk_widget_show(new_folder_dialog);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a signal handler function used when RENAME button is clicked in the Manage_Folders
Dialog. It opens the Rename_Folder Dialog, checks the uniqueness of the new folder name,
before renaming the user created folder.
@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
void ui_on_rename_manage_folders_dialog(GtkWidget * widget, gpointer userdata)
{
	gchar *folder_name = NULL;
	gboolean user_created = FALSE;
	GtkTreeView *folder_view = NULL;
	GtkTreeView *manage_view = NULL;
	GtkWindow *window = NULL;
	GtkTreeModel *folder_model = NULL, *child_model = NULL, *manage_model = NULL;
	GtkTreeIter folder_iter, child_iter, manage_iter;
	GtkTreeSelection *selection = NULL;
	gint model_num = -1;
	GString *message = NULL;
	gchar *parent_folder_name = NULL;
	gchar *manage_path_str = NULL, *folder_path_str = NULL;
	GtkTreePath *folder_path = NULL;
	GtkWidget *manage_folders = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (widget == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}
	folder_view = GTK_TREE_VIEW(email_main_view->widgets->folderView);
	folder_model = gtk_tree_view_get_model(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	CHECK_PARAM((folder_view == NULL), "Folder Tree view is missing!", return);

	/* If the this signal is invoked from manage folder dialog */
	if (email_main_view->tags->manage_folder_dlg == TRUE) {
		manage_folders = GTK_WIDGET(userdata);
		manage_view = GTK_TREE_VIEW(EMAIL_UI_LOOKUP_OBJECT(manage_folders, "tree_view"));

		CHECK_PARAM((manage_view == NULL), "Folder Tree view is missing!", return);
		selection = gtk_tree_view_get_selection(manage_view);
		if (gtk_tree_selection_get_selected(selection, &manage_model, &manage_iter)) {
			manage_path_str =
			    gtk_tree_model_get_string_from_iter(manage_model, &manage_iter);
			folder_path_str = g_strdup_printf("%s", &manage_path_str[CHILD_PATH]);
			folder_path = gtk_tree_path_new_from_string(folder_path_str);
			if (gtk_tree_model_get_iter(folder_model, &folder_iter, folder_path)
			    == TRUE) {
				gtk_tree_model_get(folder_model, &folder_iter,
						   COL_FOLDER_NAME, &folder_name,
						   COL_MODEL_SEQ_NUM, &model_num,
						   COL_USER_CREATED, &user_created,
						   COL_PARENT_NAME, &parent_folder_name, -1);
			}
			g_free(manage_path_str);
			g_free(folder_path_str);
			gtk_tree_path_free(folder_path);
		}
	} else {		/* This is from main view */

		selection = gtk_tree_view_get_selection(folder_view);
		if (gtk_tree_selection_get_selected(selection, &folder_model, &folder_iter)) {
			gtk_tree_model_get(folder_model, &folder_iter, COL_FOLDER_NAME,
					   &folder_name, COL_MODEL_SEQ_NUM, &model_num,
					   COL_USER_CREATED, &user_created, COL_PARENT_NAME,
					   &parent_folder_name, -1);
		}
	}
	child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(folder_model));
	gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT
						       (folder_model), &child_iter, &folder_iter);
	message = g_string_new(NULL);
	CHECK_PARAM((message == NULL), "Memory allocation failed", return);

	if (user_created == FALSE) {
		if (model_num == INBOX_MODEL_NUM) {
			g_string_printf(message, _("mcen_ib_unable_to_rename_inbox"));
		} else if (model_num == OUTBOX_MODEL_NUM) {
			g_string_printf(message, _("mcen_ib_unable_to_rename_outbox"));
		} else if (model_num == SENT_MODEL_NUM) {
			g_string_printf(message, _("mcen_ib_unable_to_rename_sent"));
		} else if (model_num == DRAFT_MODEL_NUM) {
			g_string_printf(message, _("mcen_ib_unable_to_rename_drafts"));
		}
		get_user_confirmation(message->str, NULL, NULL, INFO_BANNER);
	} else {
		/*Create and run Rename Folder dialog */
		g_free(email_main_view->folder_manager->parent_folder_name);
		email_main_view->folder_manager->parent_folder_name = g_strdup(parent_folder_name);
		ui_create_rename_folder_dialog(window, child_model, &child_iter, model_num,
					       folder_name, manage_model, &manage_iter);

	}
	if (email_main_view->tags->manage_folder_dlg == TRUE) {
		ui_view_selection_changed_cb(selection, manage_folders);
	}
	g_free(folder_name);
	g_free(parent_folder_name);

	g_string_free(message, TRUE);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a signal handler function used when RENAME menu item is clicked in
folders sub menu.  It displays an information banner stating unable to rename 
the mandatory folders.
@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
void ui_on_folder_rename_menu_item_dimmed(GtkWidget * widget, gpointer userdata)
{
	gboolean user_created = FALSE;
	GtkTreeView *view = NULL;
	GtkTreeModel *model = NULL, *child_model = NULL;
	GtkTreeIter iter, child_iter;
	GtkTreeSelection *selection = NULL;
	gint model_num = -1;
	GString *message = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (widget == NULL || userdata == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	view = GTK_TREE_VIEW(userdata);

	selection = gtk_tree_view_get_selection(view);

	message = g_string_new(NULL);
	CHECK_PARAM((message == NULL), "Memory allocation failed", return);

	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(model));
		gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT
							       (model), &child_iter, &iter);
		gtk_tree_model_get(model, &iter, COL_MODEL_SEQ_NUM, &model_num,
				   COL_USER_CREATED, &user_created, -1);
		if (user_created == FALSE) {
			if (model_num == INBOX_MODEL_NUM) {
				g_string_printf(message, _("mcen_ib_unable_to_rename_inbox"));
			} else if (model_num == OUTBOX_MODEL_NUM) {
				g_string_printf(message, _("mcen_ib_unable_to_rename_outbox"));
			} else if (model_num == SENT_MODEL_NUM) {
				g_string_printf(message, _("mcen_ib_unable_to_rename_sent"));
			} else if (model_num == DRAFT_MODEL_NUM) {
				g_string_printf(message, _("mcen_ib_unable_to_rename_drafts"));
			} else if (model_num == email_main_view->tags->archive_model_num) {
				g_string_printf(message, DGETTEXT_HCS("ckdg_ib_unable_to_rename"));
			}
			show_dimming_note(widget, message->str);
		}
	}

	g_string_free(message, TRUE);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a signal handler function used when DELETE button is clicked in Manage_Folders
Dialog. It allows deletion of any user created folder.
@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
void ui_on_delete_manage_folders_dialog(GtkWidget * widget, gpointer userdata)
{
	GtkTreeIter folder_iter;
	GtkTreeIter manage_iter;
	gchar *folder_name = NULL;
	gboolean user_created = FALSE;
	GtkTreeView *folder_view = NULL;
	GtkTreeView *manage_view = NULL;
	gint model_num = -1, parent_model_num = -1;
	GtkTreeModel *folder_model = NULL;
	GtkTreeModel *manage_model = NULL;
	GString *message = NULL;
	GtkTreeSelection *selection = NULL;
	gchar *parent_folder_name = NULL;
	gchar *local_folder_path = NULL, *folder_display_path = NULL;
	gchar *manage_path_str = NULL, *folder_path_str = NULL;
	GtkTreePath *folder_path = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((widget == NULL), "Input param check failed!", return);

	folder_view = GTK_TREE_VIEW(email_main_view->widgets->folderView);
	folder_model = gtk_tree_view_get_model(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	CHECK_PARAM((folder_view == NULL), "Folder Tree view is missing!", return);
	/* If this is invoked from manage folder dialog */
	if (email_main_view->tags->manage_folder_dlg == TRUE) {
		manage_view = GTK_TREE_VIEW(userdata);
		CHECK_PARAM((manage_view == NULL), "Folder Tree view is missing!", return);
		selection = gtk_tree_view_get_selection(manage_view);
		if (gtk_tree_selection_get_selected(selection, &manage_model, &manage_iter)) {
			manage_path_str =
			    gtk_tree_model_get_string_from_iter(manage_model, &manage_iter);
			folder_path_str = g_strdup_printf("%s", &manage_path_str[CHILD_PATH]);
			folder_path = gtk_tree_path_new_from_string(folder_path_str);
			if (gtk_tree_model_get_iter(folder_model, &folder_iter, folder_path)
			    == TRUE) {
				gtk_tree_model_get(folder_model, &folder_iter,
						   COL_FOLDER_NAME, &folder_name,
						   COL_MODEL_SEQ_NUM, &model_num,
						   COL_USER_CREATED, &user_created,
						   COL_PARENT_NAME, &parent_folder_name, -1);
			}
			g_free(manage_path_str);
			g_free(folder_path_str);
			gtk_tree_path_free(folder_path);
		}
	} else {
		selection = gtk_tree_view_get_selection(folder_view);
		if (gtk_tree_selection_get_selected(selection, &folder_model, &folder_iter)) {
			gtk_tree_model_get(folder_model, &folder_iter, COL_FOLDER_NAME,
					   &folder_name, COL_MODEL_SEQ_NUM, &model_num,
					   COL_USER_CREATED, &user_created, COL_PARENT_NAME,
					   &parent_folder_name, -1);
		}
	}
	message = g_string_new(NULL);
	CHECK_PARAM((message == NULL), "Memory allocation failed", return);

	if (user_created == FALSE) {
		if (model_num == INBOX_MODEL_NUM) {
			g_string_printf(message, _("mcen_ib_unable_to_delete_inbox"));
		} else if (model_num == OUTBOX_MODEL_NUM) {
			g_string_printf(message, _("mcen_ib_unable_to_delete_outbox"));
		} else if (model_num == SENT_MODEL_NUM) {
			g_string_printf(message, _("mcen_ib_unable_to_delete_sent"));
		} else if (model_num == DRAFT_MODEL_NUM) {
			g_string_printf(message, _("mcen_ib_unable_to_delete_drafts"));
		}
		get_user_confirmation(message->str, NULL, NULL, INFO_BANNER);
	} else {
		if (parent_folder_name != NULL) {
			/*Creating sub-folder paths */
			local_folder_path =
			    g_strdup_printf("%s/%s", parent_folder_name, folder_name);
			parent_model_num = ui_get_folder_model_number(parent_folder_name, NULL);
			folder_display_path =
			    g_strdup_printf("%s/%s",
					    ui_get_folder_display_name(parent_model_num,
								       parent_folder_name),
					    folder_name);
			ui_delete_folder(folder_view, folder_model, &folder_iter, model_num,
					 local_folder_path, folder_display_path, manage_view,
					 &manage_iter);
			g_free(local_folder_path);
			g_free(folder_display_path);
			g_free(parent_folder_name);
		} else {
			ui_delete_folder(folder_view, folder_model, &folder_iter, model_num,
					 folder_name, folder_name, manage_view, &manage_iter);
		}
	}
	g_free(folder_name);
	g_string_free(message, TRUE);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a signal handler function used when DELETE menu item is clicked in
folders sub menu.  It displays an information banner stating unable to delete 
the mandatory folders.
@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
void ui_on_folder_delete_menu_item_dimmed(GtkWidget * widget, gpointer userdata)
{
	GtkTreeIter iter;
	gboolean user_created = FALSE;
	GtkTreeView *view = NULL;
	gint model_num = -1;
	GtkTreeModel *model = NULL;
	GString *message = NULL;
	GtkTreeSelection *selection = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((widget == NULL), "Input param check failed!", return);
	CHECK_PARAM((userdata == NULL), "Input param check failed!", return);

	view = GTK_TREE_VIEW(userdata);

	CHECK_PARAM((view == NULL), "Folder Tree view is missing!", return);

	message = g_string_new(NULL);
	CHECK_PARAM((message == NULL), "Memory allocation failed", return);

	selection = gtk_tree_view_get_selection(view);

	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, COL_MODEL_SEQ_NUM, &model_num,
				   COL_USER_CREATED, &user_created, -1);
		if (user_created == FALSE) {
			if (model_num == INBOX_MODEL_NUM) {
				g_string_printf(message, _("mcen_ib_unable_to_delete_inbox"));
			} else if (model_num == OUTBOX_MODEL_NUM) {
				g_string_printf(message, _("mcen_ib_unable_to_delete_outbox"));
			} else if (model_num == SENT_MODEL_NUM) {
				g_string_printf(message, _("mcen_ib_unable_to_delete_sent"));
			} else if (model_num == DRAFT_MODEL_NUM) {
				g_string_printf(message, _("mcen_ib_unable_to_delete_drafts"));
			} else if (model_num == email_main_view->tags->archive_model_num) {
				g_string_printf(message, DGETTEXT_HCS("ckct_ib_unable_to_delete"));
			}
			show_dimming_note(widget, message->str);
		}
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/*
This is callback function which will be called on new folder creation from
manage folder dialog.
@param tree_view Manage folder dialog folder view.
@param seq_num   Model seq number of the newly created folder.
@param new_name  Name of the folder which is newly created.
*/
static void
ui_add_new_folder_to_manage_folders(GtkTreeView * tree_view, gint seq_num, gchar * new_name)
{
	GtkTreeModel *model = NULL;
	GtkTreeModel *manage_folder_model = NULL;
	GtkTreeIter iter, child_iter, manage_parent_iter, manage_child_iter;
	gchar *new_path_str = NULL, *path_str = NULL;
	gint no_of_children = 0, count = 0, model_seq_num = -1;
	GtkTreePath *new_path = NULL;
	gboolean sub_folder = FALSE, seq_found = FALSE;
	gint parent_count = 0, position = 0;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (tree_view == NULL || new_name == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	gtk_tree_model_get_iter_first(model, &iter);
	do {
		no_of_children = 0;
		gtk_tree_model_get(model, &iter, COL_MODEL_SEQ_NUM, &model_seq_num, -1);
		if (seq_num == model_seq_num) {
			path_str = g_strdup(ROOT_PATH);
			sub_folder = FALSE;
			seq_found = TRUE;
			break;
		} else {
			no_of_children = gtk_tree_model_iter_n_children(model, &iter);

			if (no_of_children > 0) {
				for (count = 0; count < no_of_children; count++) {
					if ((gtk_tree_model_iter_nth_child
					     (model, &child_iter, &iter, count)) == TRUE) {
						gtk_tree_model_get(model, &child_iter,
								   COL_MODEL_SEQ_NUM,
								   &model_seq_num, -1);
						if (model_seq_num == seq_num) {
							path_str =
							    gtk_tree_model_get_string_from_iter
							    (model, &iter);
							sub_folder = TRUE;
							seq_found = TRUE;
							break;
						}
					}
				}
			}
		}
		parent_count++;
	}
	while (gtk_tree_model_iter_next(model, &iter) && !seq_found);

	if (strncmp(path_str, ROOT_PATH, MAX_STR_LEN(path_str, ROOT_PATH)) == 0
	    && no_of_children == 0) {
		new_path_str = g_strdup(ROOT_PATH);
		position = parent_count;
	} else {
		new_path_str = g_strdup_printf("%s:%s", ROOT_PATH, path_str);
		position = count;
	}
	new_path = gtk_tree_path_new_from_string(new_path_str);

	manage_folder_model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree_view));

	if (gtk_tree_model_get_iter(manage_folder_model, &manage_parent_iter, new_path) == TRUE) {
		gtk_tree_store_insert(GTK_TREE_STORE(manage_folder_model),
				      &manage_child_iter, &manage_parent_iter, position);
		gtk_tree_store_set(GTK_TREE_STORE(manage_folder_model),
				   &manage_child_iter, COL_MANAGE_MODEL_SEQ_NUM,
				   model_seq_num, COL_MANAGE_USER_CREATED, TRUE,
				   COL_MANAGE_FOLDER_ICON,
				   email_main_view->icons->user_folder_icon,
				   COL_MANAGE_DISPLAY_NAME, new_name,
				   COL_MANAGE_IS_SUBFOLDER, sub_folder, -1);
	}
	g_free(new_path_str);
	g_free(path_str);
	gtk_tree_path_free(new_path);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a signal handler function used OK button in New_Folder dialog is clicked. It
checks the uniqueness of the new folder name, before creating the new user folder.
@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_ok_new_folder_dialog(GtkWidget * widget, gpointer userdata)
{
	gchar *new_name = NULL, *folder_path = NULL;
	GtkEntry *entry = NULL;
	GtkTreeView *view = NULL;
	GtkTreeModel *folder_list_store = NULL;
	GtkWindow *window = NULL;
	gint response = -1;
	GString *tempFolderPath = NULL;
	gchar *local_folder_path = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((widget == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);

	window = GTK_WINDOW(userdata);
	if (email_main_view->tags->manage_folder_dlg == TRUE) {
		view = GTK_TREE_VIEW(EMAIL_UI_LOOKUP_OBJECT(window, "manage_folders_view"));
	}
	folder_list_store = email_main_view->folder_manager->folder_view_model;
	entry = GTK_ENTRY(EMAIL_UI_LOOKUP_OBJECT(window, "name_entry"));

	CHECK_PARAM((entry == NULL), "Lookup widgets missing!", return);

	/*Get the new folder name */
	new_name = g_strdup(gtk_entry_get_text(entry));
	new_name = g_strstrip(new_name);
	/*Check if folder name is valid. If not return back to the user.
	   If valid, create the new folder, destroy the dialog and return. */
	if (ui_is_folder_name_valid(new_name, window, GTK_WIDGET(entry)) == TRUE) {
		/*Create folder path */
		folder_path =
		    g_strdup_printf("%s%c%s%c%s", STANDARD_PATH, G_DIR_SEPARATOR,
				    FOLDER_DIR_PATH, G_DIR_SEPARATOR, MAIL);

		/* This is added for sub folder implementation */
		if (email_main_view->folder_manager->parent_folder_name != NULL) {
			tempFolderPath = g_string_new(NULL);
			g_string_append_printf(tempFolderPath, "%s/%s",
					       email_main_view->folder_manager->
					       parent_folder_name, new_name);
			local_folder_path = g_strdup(tempFolderPath->str);
			g_string_free(tempFolderPath, TRUE);
			/*Call AL function to create the folder */
			response = ui_dbus_folder_create(local_folder_path, folder_path, F_LOCAL);
		} else {
			/*Call AL function to create the folder */
			response = ui_dbus_folder_create(new_name, folder_path, F_LOCAL);
		}
		if (0 != response) {
			ULOG_DEBUG("[%s][%d] : Unable to create new folder '%s'!\n",
				   __FUNCTION__, __LINE__, new_name);
			ui_set_msg_cb(response);
		} else {
			if (email_main_view->folder_manager->parent_folder_name != NULL) {
				ui_add_new_folder(GTK_TREE_STORE(folder_list_store),
						  local_folder_path, TRUE, -1);
				g_free(local_folder_path);
				/*g_free (email_main_view->folder_manager->parent_folder_name);
				   email_main_view->folder_manager->parent_folder_name = NULL; */
			} else {
				ui_add_new_folder(GTK_TREE_STORE(folder_list_store),
						  new_name, TRUE, -1);
			}
			// add new folder to manage folder dialog.
			if (email_main_view->tags->manage_folder_dlg == TRUE) {
				ui_add_new_folder_to_manage_folders(view,
								    email_main_view->
								    folder_manager->
								    model_seq_num - 1, new_name);
			}
		}
		/*Change selection in the tree view */
		ui_select_folder_from_view(GTK_TREE_VIEW
					   (email_main_view->widgets->folderView),
					   email_main_view->folder_manager->model_seq_num - 1);
		ui_select_manage_folder_from_view(GTK_TREE_VIEW(view),
						  email_main_view->folder_manager->
						  model_seq_num - 1);

		gtk_widget_destroy(GTK_WIDGET(window));
	}

	g_free(new_name);
	g_free(folder_path);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a signal handler function used when CLOSE button in New_Folder Dialog is
clicked. It closes the dialog.
@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_close_new_folder_dialog(GtkWidget * widget, gpointer data)
{
	GtkWidget *new_folder = NULL;
	GtkWidget *manage_folders = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeView *treeview = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (widget == NULL || data == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}
	new_folder = GTK_WIDGET(data);
	/*g_free (email_main_view->folder_manager->parent_folder_name);
	   email_main_view->folder_manager->parent_folder_name = NULL; */
	if (email_main_view->tags->manage_folder_dlg == TRUE) {
		manage_folders = GTK_WIDGET(EMAIL_UI_LOOKUP_OBJECT(new_folder, "Manage_folders"));
		treeview = GTK_TREE_VIEW(EMAIL_UI_LOOKUP_OBJECT(manage_folders, "tree_view"));
		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
		ui_view_selection_changed_cb(selection, manage_folders);
	}
	gtk_widget_destroy(new_folder);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This is a signal handler function used when check button is toggled in
Folder_Settings Dialog. This will enable and disable the corresponding
widgets in Folder_Settings Dialog.
@param  widget          Pointer to the widget on which this function is called
@param  data            Pointer to the user data
@return void
*/
static void ui_on_toggle_folder_settings_dialog(GtkWidget * widget, gpointer data)
{
	GtkWidget *combo_button = NULL;
	GtkWidget *spin_button = NULL;
	GtkWidget *window = NULL;
	GtkWidget *check_box = NULL;
	gboolean status = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((widget == NULL), "Input param check failed", return);
	CHECK_PARAM((data == NULL), "Input param check failed", return);

	window = GTK_WIDGET(data);
	combo_button = GTK_WIDGET(EMAIL_UI_LOOKUP_OBJECT(window, "time_combo"));
	spin_button = GTK_WIDGET(EMAIL_UI_LOOKUP_OBJECT(window, "spin_button"));
	check_box = GTK_WIDGET(EMAIL_UI_LOOKUP_OBJECT(window, "check_button"));
	CHECK_PARAM((combo_button == NULL), "Lookup widgets missing!", return);
	CHECK_PARAM((spin_button == NULL), "Lookup widgets missing!", return);
	CHECK_PARAM((check_box == NULL), "Lookup widgets missing!", return);

	status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_box));
	if (status) {
		gtk_widget_set_sensitive(combo_button, TRUE);
		gtk_widget_set_sensitive(spin_button, TRUE);
	} else {
		gtk_widget_set_sensitive(combo_button, FALSE);
		gtk_widget_set_sensitive(spin_button, FALSE);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a signal handler function used when OK button in Folder_Settings
Dialog is clicked. It saves the user settings, checks if there are any
old sent messages to be removed and asks the user whether to delete them.

@param  widget  Pointer to the widget on which this function is called
@param  data    Pointer to the user data
@return void
*/
static void ui_on_ok_folder_settings_dialog(GtkWidget * widget, gpointer data)
{
	GtkWidget *combo_button = NULL;
	GtkWidget *spin_button = NULL;
	GtkWidget *window = NULL;
	GtkWidget *check_box = NULL;
	gint sent_folder_clean_option = -1, sent_folder_clean_time_stamp = -1;
	gint sent_folder_setting_value = -1;
	gint combo_value = -1;
	time_t cur_time;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((widget == NULL), "Input param check failed", return);
	CHECK_PARAM((data == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);

	window = GTK_WIDGET(data);
	combo_button = GTK_WIDGET(EMAIL_UI_LOOKUP_OBJECT(window, "time_combo"));
	spin_button = GTK_WIDGET(EMAIL_UI_LOOKUP_OBJECT(window, "spin_button"));
	check_box = GTK_WIDGET(EMAIL_UI_LOOKUP_OBJECT(window, "check_button"));

	CHECK_PARAM((combo_button == NULL), "Lookup widgets missing!", return);
	CHECK_PARAM((spin_button == NULL), "Lookup widgets missing!", return);
	CHECK_PARAM((check_box == NULL), "Lookup widgets missing!", return);

	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_box))) {
		sent_folder_setting_value =
		    hildon_number_editor_get_value((HildonNumberEditor *) spin_button);
		combo_value = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_button));
		if (combo_value == 0) {
			sent_folder_clean_option = DAYS;
		} else if (combo_value == 1) {
			sent_folder_clean_option = WEEKS;
		} else {
			sent_folder_clean_option = MONTHS;
		}
	} else {
		sent_folder_clean_option = DO_NOT_EMPTY;
	}

	ULOG_DEBUG
	    ("[%s][%d] : sent_folder_clean_option is %d\n",
	     __FUNCTION__, __LINE__, sent_folder_clean_option);

	/*Calculate current time_stamp when cleaning operation is done */
	time(&cur_time);
	sent_folder_clean_time_stamp = cur_time;
	ULOG_DEBUG
	    ("[%s][%d] : Time stamp for cleaning operation = %d seconds\n",
	     __FUNCTION__, __LINE__, sent_folder_clean_time_stamp);

	/*Store the settings */
	ui_set_sent_folder_cleaning_settings(sent_folder_clean_option,
					     sent_folder_clean_time_stamp,
					     sent_folder_setting_value);

	gtk_widget_destroy(window);

	if (sent_folder_clean_option != DO_NOT_EMPTY) {
		ui_empty_sent_folder(TRUE);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a signal handler function used when CLOSE button in Folder_Settings Dialog is
clicked. It closes the dialog.
@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_destroy_folder_dialog(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (widget == NULL || data == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	gtk_widget_destroy(GTK_WIDGET(data));
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a signal handler function used when CLOSE button in Manage_Folders Dialog is
clicked. It closes the dialog.
@param  widget          Pointer to the widget on which this function is called
@param  data            Pointer to the user data
@return void
*/
static void ui_on_close_manage_folders_dialog(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (widget == NULL || data == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	gtk_widget_destroy(GTK_WIDGET(data));
	email_main_view->tags->manage_folder_dlg = FALSE;
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/* 
  This is helper function to create tree view model for manage folder dialog.
  @param void.
  @return Pointer to tree view widget.  
*/
static GtkWidget *ui_create_manage_view_and_model()
{
	GtkTreeViewColumn *tree_view_column = NULL;
	GtkTreeModel *model = NULL;
	GtkCellRenderer *renderer1 = NULL;
	GtkCellRenderer *renderer2 = NULL;
	GtkWidget *view = NULL;
	GtkTreeStore *manageListStore = NULL;
	GtkTreeIter iter1;
	GtkTreeIter parent_iter, child_iter, iter, parent_child_iter;
	gchar *displayName = NULL, *childFolderName = NULL, *parentFolderName = NULL;
	gint no_of_children = 0, count = 0, seq_num = -1;
	gboolean userCreated = FALSE;
	gchar *device_name = NULL;
	GdkPixbuf *folder_icon = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	view = gtk_tree_view_new();

	tree_view_column = gtk_tree_view_column_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), tree_view_column);
	renderer1 = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(tree_view_column, renderer1, TRUE);
	gtk_tree_view_column_add_attribute(tree_view_column, renderer1,
					   ATTRIBUTE_PIXBUF, COL_MANAGE_FOLDER_ICON);

	tree_view_column = gtk_tree_view_column_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), tree_view_column);
	renderer2 = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(tree_view_column, renderer2, TRUE);
	gtk_tree_view_column_add_attribute(tree_view_column, renderer2,
					   ATTRIBUTE_TEXT, COL_MANAGE_DISPLAY_NAME);
	manageListStore =
	    gtk_tree_store_new(NUM_MANAGE_VIEW_COLS, G_TYPE_UINT, G_TYPE_BOOLEAN,
			       GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_BOOLEAN);

	device_name = ui_get_device_name();

	CHECK_PARAM((email_main_view->icons->device_icon == NULL),
		    "Pixbuf not loaded yet",
		    ui_load_icon_pixbuf(&email_main_view->icons->
					device_icon, "qgn_list_filesys_divc_cls", MSG_ICON_SIZE));

	/* Inserting main 'Root' row in manage tree view */
	gtk_tree_store_append(manageListStore, &iter1, NULL);
	gtk_tree_store_set(GTK_TREE_STORE(manageListStore), &iter1,
			   COL_MANAGE_MODEL_SEQ_NUM, SP_DEVICE_MODEL_NUM,
			   COL_MANAGE_USER_CREATED, FALSE,
			   COL_MANAGE_FOLDER_ICON,
			   email_main_view->icons->device_icon,
			   COL_MANAGE_DISPLAY_NAME, device_name,
			   COL_MANAGE_IS_SUBFOLDER, FALSE, -1);

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	gtk_tree_model_get_iter_first(model, &parent_iter);
	do {
		/* Traversing in main folder view to get each row informantion and 
		 * inserting to manage folder view */
		gtk_tree_model_get(model, &parent_iter, COL_DISPLAY_NAME, &displayName,
				   COL_MODEL_SEQ_NUM, &seq_num,
				   COL_USER_CREATED, &userCreated, COL_FOLDER_ICON,
				   &folder_icon, -1);
		gtk_tree_store_append(manageListStore, &child_iter, &iter1);
		gtk_tree_store_set(GTK_TREE_STORE(manageListStore), &child_iter,
				   COL_MANAGE_MODEL_SEQ_NUM, seq_num,
				   COL_MANAGE_USER_CREATED, userCreated,
				   COL_MANAGE_FOLDER_ICON, folder_icon,
				   COL_MANAGE_DISPLAY_NAME, displayName,
				   COL_MANAGE_IS_SUBFOLDER, FALSE, -1);
		no_of_children = gtk_tree_model_iter_n_children(model, &parent_iter);

		if (no_of_children > 0) {
			/* This is for sub folder */
			for (count = 0; count < no_of_children; count++) {
				if ((gtk_tree_model_iter_nth_child(model,
								   &parent_child_iter,
								   &parent_iter, count)) == TRUE) {
					gtk_tree_model_get(model, &parent_child_iter,
							   COL_DISPLAY_NAME,
							   &childFolderName,
							   COL_MODEL_SEQ_NUM, &seq_num,
							   COL_USER_CREATED, &userCreated,
							   COL_FOLDER_ICON, &folder_icon, -1);
					if (childFolderName != NULL) {
						gtk_tree_store_append(manageListStore,
								      &iter, &child_iter);
						gtk_tree_store_set(manageListStore,
								   &iter,
								   COL_MANAGE_MODEL_SEQ_NUM,
								   seq_num,
								   COL_MANAGE_USER_CREATED,
								   userCreated,
								   COL_MANAGE_FOLDER_ICON,
								   folder_icon,
								   COL_MANAGE_DISPLAY_NAME,
								   childFolderName,
								   COL_MANAGE_IS_SUBFOLDER,
								   TRUE, -1);
						g_free(parentFolderName);
						parentFolderName = NULL;
						g_free(childFolderName);
						childFolderName = NULL;
					}
				}
			}
		}
		g_free(displayName);
		displayName = NULL;
	}
	while (gtk_tree_model_iter_next(model, &parent_iter));

	gtk_tree_view_set_model(GTK_TREE_VIEW(view), GTK_TREE_MODEL(manageListStore));
	g_free(device_name);

	ULOG_DEBUG_L("[%s] : Ends here :\n", __FUNCTION__);
	return view;
}


/*Window Creation Functions*/
/**
This is a window creation function used to create the Manage_Folders Dialog.
@param  main_folder_view        Pointer to the tree view object of folder view
@return GtkWidget*      Pointer to the newly created dialog
*/
GtkWidget *ui_create_manage_folders_dialog(GtkTreeView * main_folder_view)
{
	GtkWidget *manage_folders = NULL;
	GtkWidget *vbox1 = NULL;
	GtkWidget *scrolledwindow1 = NULL;
	GtkWidget *treeview1 = NULL;
	GtkWidget *button1 = NULL, *button2 = NULL, *button3 = NULL, *button4 = NULL;
	GtkTreeSelection *selection = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (main_folder_view == NULL
	    || email_main_view == NULL || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return manage_folders;
	}

	manage_folders = gtk_dialog_new();
	gtk_window_set_default_size(GTK_WINDOW(manage_folders), 420, 250);
	gtk_dialog_set_has_separator(GTK_DIALOG(manage_folders), FALSE);

	gtk_window_set_title(GTK_WINDOW(manage_folders), _("mcen_ti_manage_folders_title"));
	gtk_window_set_position(GTK_WINDOW(manage_folders), GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(manage_folders), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(manage_folders), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(manage_folders), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(manage_folders), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_window_set_resizable(GTK_WINDOW(manage_folders), TRUE);
	vbox1 = GTK_DIALOG(manage_folders)->vbox;
	gtk_widget_show(vbox1);

	scrolledwindow1 = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow1);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolledwindow1, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow1),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	treeview1 = ui_create_manage_view_and_model();
	gtk_widget_show(treeview1);
	gtk_container_add(GTK_CONTAINER(scrolledwindow1), treeview1);

	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview1), FALSE);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview1));

	/*Set the main folder view's model to display the folders */
	gtk_tree_view_expand_all(GTK_TREE_VIEW(treeview1));
	/*Select currently opened folder */
	ui_select_manage_folder_from_view(GTK_TREE_VIEW(treeview1),
					  (gint) email_main_view->folder_manager->
					  currently_displayed_model);

	button1 = gtk_button_new_with_mnemonic(_("mcen_bd_manage_folders_new"));
	gtk_widget_show(button1);
	gtk_dialog_add_action_widget(GTK_DIALOG(manage_folders), button1, 1);
	GTK_WIDGET_SET_FLAGS(button1, GTK_CAN_DEFAULT);

	button2 = gtk_button_new_with_mnemonic(_("mcen_bd_manage_folders_rename"));
	gtk_widget_show(button2);
	gtk_dialog_add_action_widget(GTK_DIALOG(manage_folders), button2, 2);
	GTK_WIDGET_SET_FLAGS(button2, GTK_CAN_DEFAULT);

	button3 = gtk_button_new_with_mnemonic(_("mcen_bd_manage_folders_delete"));
	gtk_widget_show(button3);
	gtk_dialog_add_action_widget(GTK_DIALOG(manage_folders), button3, 3);
	GTK_WIDGET_SET_FLAGS(button3, GTK_CAN_DEFAULT);

	button4 = gtk_button_new_with_mnemonic(_("mcen_bd_manage_folders_close"));
	gtk_widget_show(button4);
	gtk_dialog_add_action_widget(GTK_DIALOG(manage_folders), button4, 4);
	GTK_WIDGET_SET_FLAGS(button4, GTK_CAN_DEFAULT);
	gtk_widget_grab_focus(button4);

	/* Callback functions */
	g_signal_connect(G_OBJECT(button1), "clicked",
			 G_CALLBACK(ui_on_new_manage_folders_dialog), manage_folders);

	g_signal_connect(G_OBJECT(button3), "clicked",
			 G_CALLBACK(ui_on_delete_manage_folders_dialog), treeview1);

	g_signal_connect(G_OBJECT(button2), "clicked",
			 G_CALLBACK(ui_on_rename_manage_folders_dialog), manage_folders);
	g_signal_connect(G_OBJECT(button4), "clicked",
			 G_CALLBACK(ui_on_close_manage_folders_dialog), manage_folders);

	g_signal_connect(selection, "changed",
			 G_CALLBACK(ui_view_selection_changed_cb), manage_folders);

	g_signal_connect(G_OBJECT(manage_folders), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), button4);

	/* Checking whether help content exist or not. */
	if (ossohelp_exists(HELP_ID_MANAGEFOLDERS) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) manage_folders);
		g_signal_connect(G_OBJECT(manage_folders), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_MANAGEFOLDERS);
	}

	/*Hookup needed child widgets */
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(manage_folders, treeview1, "tree_view");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(manage_folders, button2, "Rename");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(manage_folders, button3, "Delete");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(manage_folders, button1, "NewFolder");

	ui_view_selection_changed_cb(selection, manage_folders);
	email_main_view->tags->manage_folder_dlg = TRUE;
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return manage_folders;
}


/**
This is a signal handler function called whenever text is inserted in the
entry field of New folder dialog or Rename folder dialog. It ensures that
not more than 20 chars are inserted into the field, and when the limit is
reached it shows up a info banner.

@param  entry     Pointer to the entry widget
@param  text      String that has been inserted
@param  len       Length of the inserted string
@param  position  Position of current insertion
@param  dialog    Pointer to the New folder dialog
@return void
*/
static void
ui_on_insert_text_new_folder_dialog(GtkEditable * entry, gchar * text,
				    gint len, gint * position, GtkWindow * dialog)
{

	gchar *cur_text = NULL;
	gchar *new_text = NULL;

	ULOG_DEBUG("[%s][%s] : start here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM(entry == NULL, "Input param check failed", return);
	CHECK_PARAM(dialog == NULL, "Input param check failed", return);

	/*With HWR this signal is being generated twice for each insertion (Not sure why).
	   Once with empty 'text' param and again with correct param.
	   Adding a check for this scenario. */
	if (text == NULL || strlen(text) == 0 || position == NULL) {
		ULOG_DEBUG_F("Input param check failed");
		return;
	}

	cur_text = (gchar *) gtk_entry_get_text(GTK_ENTRY(entry));

	if (cur_text != NULL && g_utf8_strlen(cur_text, -1) > MAX_FOLDER_NAME_LEN) {
		new_text = g_strndup(cur_text, MAX_FOLDER_NAME_LEN);
		gtk_entry_set_text(GTK_ENTRY(entry), new_text);
		g_free(new_text);
		gtk_infoprint(dialog, _("mcen_ib_maxchar_reached"));
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This is a window creation function used to create the New_Folder Dialog.
@param	data Pointer to user data.
@return GtkWidget*	Pointer to the newly created dialog
*/
static GtkWidget *ui_create_new_folder_dialog(gpointer data)
{
	GtkWidget *new_folder_dialog = NULL;
	GtkWidget *dialog_vbox1 = NULL;
	GtkWidget *caption_control = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *entry1 = NULL;
	GtkWidget *dialog_action_area1 = NULL;
	GtkWidget *cancelbutton1 = NULL;
	GtkWidget *okbutton1 = NULL;
	gchar *new_folder_display_str = NULL;
	GtkWidget *manage_folders = NULL;
	GtkTreeView *manage_folders_view = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (email_main_view->tags->manage_folder_dlg == TRUE) {
		manage_folders = GTK_WIDGET(data);
		manage_folders_view =
		    GTK_TREE_VIEW(EMAIL_UI_LOOKUP_OBJECT(manage_folders, "tree_view"));
	}

	new_folder_dialog = gtk_dialog_new();

	gtk_window_set_title(GTK_WINDOW(new_folder_dialog), _("mcen_ti_new_folder"));
	gtk_window_set_position(GTK_WINDOW(new_folder_dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(new_folder_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(new_folder_dialog), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(new_folder_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(new_folder_dialog), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_window_set_resizable(GTK_WINDOW(new_folder_dialog), TRUE);
	gtk_dialog_set_has_separator(GTK_DIALOG(new_folder_dialog), FALSE);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(new_folder_dialog), TRUE);

	dialog_vbox1 = GTK_DIALOG(new_folder_dialog)->vbox;
	gtk_widget_show(dialog_vbox1);

	entry1 = gtk_entry_new();
	gtk_widget_show(entry1);
	new_folder_display_str =
	    ui_get_new_folder_display_string(email_main_view->folder_manager->parent_folder_name);
	gtk_entry_set_text(GTK_ENTRY(entry1), new_folder_display_str);
	gtk_editable_select_region(GTK_EDITABLE(entry1), 0, -1);
	g_free(new_folder_display_str);

	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_new_folder_name"), entry1, NULL,
			       HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox1), caption_control, TRUE, TRUE, 0);
	gtk_widget_show(caption_control);

	dialog_action_area1 = GTK_DIALOG(new_folder_dialog)->action_area;
	gtk_widget_show(dialog_action_area1);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1), GTK_BUTTONBOX_END);

	okbutton1 = gtk_button_new_with_mnemonic(_("mcen_bd_new_folder_ok"));
	gtk_widget_show(okbutton1);
	gtk_dialog_add_action_widget(GTK_DIALOG(new_folder_dialog), okbutton1, GTK_RESPONSE_OK);
	GTK_WIDGET_SET_FLAGS(okbutton1, GTK_CAN_DEFAULT);

	cancelbutton1 = gtk_button_new_with_mnemonic(_("mcen_bd_new_folder_cancel"));
	gtk_widget_show(cancelbutton1);
	gtk_dialog_add_action_widget(GTK_DIALOG(new_folder_dialog), cancelbutton1,
				     GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS(cancelbutton1, GTK_CAN_DEFAULT);

	/*Signal Handlers */
	g_signal_connect(G_OBJECT(cancelbutton1), "clicked",
			 G_CALLBACK(ui_on_close_new_folder_dialog), new_folder_dialog);

	g_signal_connect(G_OBJECT(okbutton1), "clicked",
			 G_CALLBACK(ui_on_ok_new_folder_dialog), new_folder_dialog);

	if (ossohelp_exists(HELP_ID_NEWFOLDER) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) new_folder_dialog);
		g_signal_connect(G_OBJECT(new_folder_dialog), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_NEWFOLDER);
	}

	g_signal_connect(G_OBJECT(new_folder_dialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancelbutton1);

	g_signal_connect_after(G_OBJECT(entry1), "insert-text",
			       G_CALLBACK(ui_on_insert_text_new_folder_dialog), new_folder_dialog);

	EMAIL_UI_HOOKUP_OBJECT_NO_REF(new_folder_dialog, entry1, "name_entry");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(new_folder_dialog, manage_folders, "Manage_folders");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(new_folder_dialog, manage_folders_view,
				      "manage_folders_view");

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return new_folder_dialog;
}

/**
This is a window creation function used to create the Rename Folder Dialog.
@param  window	   Pointer to the widget on which this function is called
@param	model	   Pointer to the folder view storage model
@param	iter	   Pointer to the GtkTreeIter of current row
@param  model_num   To hold model sequence number.
@param	old_name   Pointer to gchar    to hold old folder name.
@param manage_model Pointer to manage folder dialog tree model.
@param manage_iter Pointer to iter pointing to manage folder tree view.
@return void
*/
void
ui_create_rename_folder_dialog(GtkWindow * window, GtkTreeModel * model,
			       GtkTreeIter * iter, gint model_num,
			       gchar * old_name, GtkTreeModel * manage_model,
			       GtkTreeIter * manage_iter)
{
	gchar *new_name = NULL;
	gboolean dialog_exit = FALSE;
	GtkWidget *rename_folder_dialog = NULL;
	GtkWidget *entry = NULL;
	GtkWidget *caption_control = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *ok_button = NULL;
	GtkWidget *cancel_button = NULL;
	GtkWidget *dialog_vbox1 = NULL;
	GtkWidget *dialog_action_area = NULL;
	gint result = -1;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (model == NULL || iter == NULL || old_name == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	/*Create Rename Folder dialog */
	rename_folder_dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(rename_folder_dialog), _("ckdg_ti_rename_folder"));

	gtk_window_set_position(GTK_WINDOW(rename_folder_dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(rename_folder_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(rename_folder_dialog), TRUE);

	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(rename_folder_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(rename_folder_dialog), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_window_set_resizable(GTK_WINDOW(rename_folder_dialog), TRUE);
	gtk_dialog_set_has_separator(GTK_DIALOG(rename_folder_dialog), FALSE);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(rename_folder_dialog), TRUE);

	dialog_vbox1 = GTK_DIALOG(rename_folder_dialog)->vbox;
	gtk_widget_show(dialog_vbox1);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	entry = gtk_entry_new();
	gtk_widget_show(entry);
	gtk_entry_set_text(GTK_ENTRY(entry), old_name);
	gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);

	caption_control =
	    hildon_caption_new(size_group, _("ckdg_fi_rename_name"), entry, NULL,
			       HILDON_CAPTION_OPTIONAL);
	gtk_widget_show(caption_control);
	gtk_box_pack_start(GTK_BOX(dialog_vbox1), caption_control, TRUE, TRUE, 0);

	/* Getting the dialog action area for password dialog */
	dialog_action_area = GTK_DIALOG(rename_folder_dialog)->action_area;
	gtk_widget_show(dialog_action_area);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area), GTK_BUTTONBOX_END);
	/* Creating ok button */
	ok_button =
	    gtk_dialog_add_button(GTK_DIALOG(rename_folder_dialog),
				  _("ckdg_bd_rename_object_dialog_ok"), GTK_RESPONSE_ACCEPT);

	/* Creating cancel button */
	cancel_button =
	    gtk_dialog_add_button(GTK_DIALOG(rename_folder_dialog),
				  _("ckdg_bd_rename_object_dialog_cancel"), GTK_RESPONSE_REJECT);

	if (ossohelp_exists(HELP_ID_RENAMEFOLDER) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) rename_folder_dialog);
		g_signal_connect(G_OBJECT(rename_folder_dialog), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_RENAMEFOLDER);
	}
	g_signal_connect(G_OBJECT(rename_folder_dialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancel_button);
	g_signal_connect_after(G_OBJECT(entry), "insert-text",
			       G_CALLBACK(ui_on_insert_text_new_folder_dialog),
			       rename_folder_dialog);

	while (dialog_exit != TRUE) {
		result = gtk_dialog_run(GTK_DIALOG(rename_folder_dialog));
		switch (result) {
		case GTK_RESPONSE_ACCEPT:
			/*Get the new folder name */
			new_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
			new_name = g_strstrip(new_name);

			/*Check if old name and new name are same.If so do nothing */
			if (strncmp(old_name, new_name, MAX_STR_LEN(old_name, new_name)) == 0) {
				dialog_exit = TRUE;
			}
			/*Check if folder name is valid. If valid, rename the folder and
			   exit the dialog. Otherwise run dialog again. */
			else if (ui_is_folder_name_valid
				 (new_name, GTK_WINDOW(rename_folder_dialog), entry) == TRUE) {
				ui_rename_folder(model, iter, model_num, old_name, new_name,
						 manage_model, manage_iter);
				dialog_exit = TRUE;
			}
			g_free(new_name);
			new_name = NULL;
			break;

		case GTK_RESPONSE_REJECT:
			dialog_exit = TRUE;
			break;

		default:
			break;
		}
	}
	gtk_widget_destroy(rename_folder_dialog);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a window creation function used to create the Sort_Messages Dialog.
@param	main_folder_view	Pointer to the tree view object of folder view
@return void
*/
void ui_create_sort_messages_dialog(GtkTreeView * main_folder_view)
{
	HildonSortDialog *sort_dialog = NULL;
	gint response_id = -1, default_sort_index = -1;
	gint sort_key = -1, current_sort_key = -1;
	GtkSortType sort_order, current_sort_order;
	GtkTreeModel *folder_list_store = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeIter iter;

	/*Check the input params */
	if (main_folder_view == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	sort_dialog = HILDON_SORT_DIALOG(hildon_sort_dialog_new(GTK_WINDOW(app)));
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(sort_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(sort_dialog), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);

	default_sort_index =
	    hildon_sort_dialog_add_sort_key(sort_dialog, _("mcen_li_sort_sender_recipient"));
	hildon_sort_dialog_add_sort_key(sort_dialog, _("mcen_li_sort_date"));
	hildon_sort_dialog_add_sort_key(sort_dialog, _("mcen_li_sort_subject"));
	hildon_sort_dialog_add_sort_key(sort_dialog, _("mcen_li_sort_size"));
	hildon_sort_dialog_add_sort_key(sort_dialog, _("mcen_li_sort_priority"));

	/*Get the current sorting choices of the selected folder and display them */
	folder_list_store = gtk_tree_view_get_model(main_folder_view);
	selection = gtk_tree_view_get_selection(main_folder_view);
	if (gtk_tree_selection_get_selected(selection, &folder_list_store, &iter)) {
		gtk_tree_model_get(folder_list_store, &iter, COL_SORT_BY,
				   &current_sort_key, COL_SORT_ORDER, &current_sort_order, -1);
		/*if the current sort order is by msgicon /attachment.
		 * set the sort order to date.
		 * Once ui spec fixes in sort dialog this will be removed*/
		if (current_sort_key == SORT_BY_READ_STATUS
		    || current_sort_key == SORT_BY_ATTACHMENT)
			current_sort_key = SORT_BY_DATE;

		hildon_sort_dialog_set_sort_key(sort_dialog, current_sort_key);
		hildon_sort_dialog_set_sort_order(sort_dialog, current_sort_order);
	}

	gtk_widget_show(GTK_WIDGET(sort_dialog));

	if (ossohelp_exists(HELP_ID_SORT) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) sort_dialog);
		g_signal_connect(G_OBJECT(sort_dialog), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_SORT);
	}

	response_id = gtk_dialog_run(GTK_DIALOG(sort_dialog));
	if (response_id == GTK_RESPONSE_OK) {
		sort_key = hildon_sort_dialog_get_sort_key(sort_dialog);
		sort_order = hildon_sort_dialog_get_sort_order(sort_dialog);
		ULOG_DEBUG("[%s][%s] : Sort key = %d, Sort order = %d\n",
			   __FILE__, __FUNCTION__, sort_key, sort_order);
		ui_on_ok_sort_messages_dialog(sort_key, sort_order);
	}

	gtk_widget_destroy(GTK_WIDGET(sort_dialog));
	return;
}

/**
This is a window creation function used to create the Folder_Settings Dialog.
@param	void
@return GtkWidget*	Pointer to the newly created dialog Otherwise NULL.
*/
GtkWidget *ui_create_folder_settings_dialog()
{
	GtkWidget *dialog = NULL;
	GtkWidget *vbox = NULL;
	GtkWidget *label = NULL;
	GtkWidget *dialog_action_area = NULL;
	GtkWidget *cancelbutton = NULL;
	GtkWidget *okbutton = NULL;
	gint sent_folder_clean_option = -1, sent_folder_clean_time_stamp = -1;
	gint sent_folder_setting_value = -1;
	GtkWidget *check_box = NULL;
	GtkWidget *check_box_cptn_ctrl = NULL;
	GtkSizeGroup *size_group1 = NULL;
	GtkWidget *spinbutton = NULL;
	GtkWidget *cap_contr_sizelimit = NULL;
	GtkWidget *select_time_combo = NULL;
	GtkWidget *select_time_cap_contr = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (email_main_view == NULL || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return dialog;
	}

	dialog = gtk_dialog_new();
	gtk_window_set_default_size(GTK_WINDOW(dialog), 450, 220);
	gtk_window_set_title(GTK_WINDOW(dialog), _("mcen_ti_folder_settings"));
	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(dialog), WINDOW_NAME, g_strdup(APP_NAME),
			       (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(dialog), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_window_set_resizable(GTK_WINDOW(dialog), TRUE);
	gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog), TRUE);

	vbox = GTK_DIALOG(dialog)->vbox;
	gtk_widget_show(vbox);

	size_group1 = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	label = gtk_label_new(_("mcen_ia_folder_settings_info"));
	gtk_misc_set_alignment((GtkMisc *) label, 0, 0);
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, ZERO);

	/* Creating check box */
	check_box = gtk_check_button_new();
	gtk_widget_show(check_box);
	check_box_cptn_ctrl =
	    hildon_caption_new(size_group1, _("mcen_fi_folder_settings_setting_on"),
			       check_box, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), check_box_cptn_ctrl, TRUE, TRUE, ZERO);
	gtk_widget_show(check_box_cptn_ctrl);

	/* Creating spin button */
	spinbutton = hildon_number_editor_new(FOL_SETT_MIN, FOL_SETT_MAX);
	hildon_number_editor_set_value((HildonNumberEditor *) spinbutton, 6);
	gtk_widget_show(spinbutton);
	gtk_widget_set_sensitive(spinbutton, FALSE);

	cap_contr_sizelimit = hildon_caption_new(size_group1,
						 _
						 ("mcen_fi_folder_settings_select_interval"),
						 spinbutton, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), cap_contr_sizelimit, TRUE, TRUE, ZERO);
	gtk_widget_show(cap_contr_sizelimit);

	/* Creating Combo box */
	select_time_combo = gtk_combo_box_new_text();
	gtk_widget_show(select_time_combo);
	gtk_widget_set_sensitive(select_time_combo, FALSE);
	gtk_combo_box_append_text(GTK_COMBO_BOX(select_time_combo),
				  _("mcen_li_folder_settings_day"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(select_time_combo),
				  _("mcen_li_folder_settings_week"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(select_time_combo),
				  _("mcen_li_folder_settings_month"));
	gtk_combo_box_set_active(GTK_COMBO_BOX(select_time_combo), 2);
	select_time_cap_contr = hildon_caption_new(size_group1,
						   _
						   ("mcen_fi_folder_settings_selecttime"),
						   select_time_combo, NULL,
						   HILDON_CAPTION_OPTIONAL);
	gtk_widget_show(select_time_cap_contr);
	gtk_box_pack_start(GTK_BOX(vbox), select_time_cap_contr, TRUE, TRUE, ZERO);

	/* Getting dialog action area */
	dialog_action_area = GTK_DIALOG(dialog)->action_area;
	gtk_widget_show(dialog_action_area);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area), GTK_BUTTONBOX_END);

	/* Creating buttons */
	okbutton = gtk_button_new_with_mnemonic(_("mcen_bd_folder_settings_ok"));
	gtk_widget_show(okbutton);
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog), okbutton, GTK_RESPONSE_OK);
	GTK_WIDGET_SET_FLAGS(okbutton, GTK_CAN_DEFAULT);

	cancelbutton = gtk_button_new_with_mnemonic(_("mcen_bd_folder_settings_cancel"));
	gtk_widget_show(cancelbutton);
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog), cancelbutton, GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS(cancelbutton, GTK_CAN_DEFAULT);

	/* Hooking up the widgets */
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog, spinbutton, "spin_button");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog, select_time_combo, "time_combo");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog, check_box, "check_button");

	/* Connecting signals to widgets */
	g_signal_connect(G_OBJECT(check_box), "toggled",
			 G_CALLBACK(ui_on_toggle_folder_settings_dialog), dialog);
	g_signal_connect(G_OBJECT(okbutton), "clicked",
			 G_CALLBACK(ui_on_ok_folder_settings_dialog), dialog);
	g_signal_connect(G_OBJECT(cancelbutton), "clicked",
			 G_CALLBACK(ui_destroy_folder_dialog), dialog);
	g_signal_connect(G_OBJECT(dialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancelbutton);
	if (ossohelp_exists(HELP_ID_FOLDERSETTINGS) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) dialog);
		g_signal_connect(G_OBJECT(dialog), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_FOLDERSETTINGS);
	}

	sent_folder_clean_option = ui_get_sent_folder_clean_option();
	sent_folder_clean_time_stamp = ui_get_sent_folder_clean_time_stamp();
	sent_folder_setting_value = ui_get_sent_folder_clean_time_value();

	if (sent_folder_clean_option == DO_NOT_EMPTY) {
		gtk_toggle_button_set_active((GtkToggleButton *) check_box, FALSE);
	} else {
		gtk_toggle_button_set_active((GtkToggleButton *) check_box, TRUE);
		hildon_number_editor_set_value((HildonNumberEditor *) spinbutton,
					       sent_folder_setting_value);
		if (sent_folder_clean_option == DAYS) {
			gtk_combo_box_set_active(GTK_COMBO_BOX(select_time_combo), 0);
		} else if (sent_folder_clean_option == WEEKS) {
			gtk_combo_box_set_active(GTK_COMBO_BOX(select_time_combo), 1);
		} else {
			gtk_combo_box_set_active(GTK_COMBO_BOX(select_time_combo), 2);
		}
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return dialog;
}

/**
Calculate the position of the current selected folder row in the folder view.
@param1  folder_name  holds the folder name of the folder to retrieve the postion.
@return  int  the position of the folder in the folder list.
*/
gint ui_get_folder_row_position_from_view(gchar * folder_name, gchar * parentFolder)
{
	GtkTreeModel *model = NULL;
	GtkTreeIter parent_iter;
	GtkTreeIter child_iter;
	gchar *parent_folder_name = NULL;
	gchar *child_folder_name = NULL;
	const gchar *root_folder_name = NULL;
	gint no_of_children = 0;
	gint count = 0;
	gint position = 0;
	gboolean row_found = FALSE;
	gboolean retval = FALSE;

	ULOG_DEBUG("[%s][%s] : Starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM((folder_name == NULL), "Input param check failed", return position);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return position);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed",
		    return position);
	CHECK_PARAM((email_main_view->widgets->folderView == NULL),
		    "Input param check failed", return position);

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	retval = gtk_tree_model_get_iter_first(model, &parent_iter);
	CHECK_PARAM((retval == FALSE), "Storage model is empty", return position);

	root_folder_name = parentFolder;
	CHECK_PARAM((parentFolder == NULL), "Parent folder is null",
		    root_folder_name = folder_name);

	do {
		gtk_tree_model_get(model, &parent_iter, COL_FOLDER_NAME, &parent_folder_name, -1);
		if (parent_folder_name != NULL
		    && strncmp(parent_folder_name, root_folder_name,
			       MAX_STR_LEN(parent_folder_name, root_folder_name)) == 0) {
			if (parentFolder != NULL) {
				position++;
				no_of_children =
				    gtk_tree_model_iter_n_children(model, &parent_iter);
				for (count = 0; count < no_of_children; count++) {
					if ((gtk_tree_model_iter_nth_child(model,
									   &child_iter,
									   &parent_iter,
									   count)) == TRUE) {
						gtk_tree_model_get(model, &child_iter,
								   COL_FOLDER_NAME,
								   &child_folder_name, -1);
						if (child_folder_name != NULL
						    && strncmp(child_folder_name,
							       folder_name,
							       MAX_STR_LEN
							       (child_folder_name,
								folder_name)) == 0) {
							g_free(child_folder_name);
							row_found = TRUE;
							break;
						} else {
							position++;
						}
					}
					g_free(child_folder_name);
				}
			} else {
				row_found = TRUE;
				g_free(parent_folder_name);
				break;
			}
		} else {
			no_of_children = gtk_tree_model_iter_n_children(model, &parent_iter);
			position = position + no_of_children;
			position++;
		}
		g_free(parent_folder_name);
		parent_folder_name = NULL;
	}
	while (!row_found && gtk_tree_model_iter_next(model, &parent_iter));

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return position;
}

/**
This is a 'GtkTreeModelForeachFunc' function used to populte the current
selected folder. Depending on the parent folder and child folder the iter
row will be selected.
@param	model		Pointer to the folder view storage model
@param	path		Pointer to the GtkTreePath of current row
@param	iter		Pointer to the GtkTreeIter of current row
@param	data		Pointer to the structure containig folder information.
@return boolean 	If the folder is selected -> TRUE (don't call again)
			Otherwise -> FALSE (Call again with next row)
*/
static gboolean
ui_folder_menu_activated_for_each(GtkTreeModel * model, GtkTreePath * path,
				  GtkTreeIter * iter, gpointer data)
{
	gchar *local_folder_name = NULL;
	PopupFolderMenu *local_menu = NULL;
	GtkTreeSelection *selection = NULL;
	gchar *parent_name = NULL;
	GtkTreeModelSort *sorted_model = NULL;
	GtkTreeIter sorted_iter, parent_iter;
	GtkTreePath *parent_path = NULL;
	gboolean retval = FALSE;
	gint str_check = -1;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM((model == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((data == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((email_main_view->folder_manager == NULL),
		    "Input param check failed", return TRUE);

	local_menu = (PopupFolderMenu *) data;
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW
						(email_main_view->widgets->folderView));
	sorted_model =
	    GTK_TREE_MODEL_SORT(gtk_tree_view_get_model
				(GTK_TREE_VIEW(email_main_view->widgets->folderView)));

	gtk_tree_model_get(model, iter, COL_FOLDER_NAME, &local_folder_name,
			   COL_PARENT_NAME, &parent_name, -1);

	/*Make sure that both parent strings are either NULL or not NULL */
	str_check = ui_null_check_strings(local_menu->parent_folder, parent_name);

	if (str_check == 0) {
		if (strncmp(local_menu->folder_name, local_folder_name,
			    MAX_STR_LEN(local_menu->folder_name, local_folder_name)) == 0) {
			gtk_tree_model_sort_convert_child_iter_to_iter(sorted_model,
								       &sorted_iter, iter);
			gtk_tree_selection_select_iter(selection, &sorted_iter);
			g_free(local_folder_name);
			g_free(parent_name);
			return TRUE;
		}
	} else if (str_check == 1) {
		if (local_menu->folder_name != NULL && local_folder_name != NULL) {
			if (strncmp(local_menu->parent_folder, parent_name,
				    MAX_STR_LEN(local_menu->parent_folder, parent_name)) == 0) {
				if (strncmp(local_menu->folder_name, local_folder_name,
					    MAX_STR_LEN(local_menu->folder_name,
							local_folder_name)) == 0) {
					gtk_tree_model_sort_convert_child_iter_to_iter
					    (sorted_model, &sorted_iter, iter);
					retval = gtk_tree_model_iter_parent(GTK_TREE_MODEL
									    (sorted_model),
									    &parent_iter,
									    &sorted_iter);
					CHECK_PARAM((retval == FALSE),
						    "Unable to obtain parent iter!", return TRUE);
					parent_path =
					    gtk_tree_model_get_path(GTK_TREE_MODEL
								    (sorted_model), &parent_iter);
					if (parent_path != NULL) {
						gtk_tree_view_expand_row(GTK_TREE_VIEW
									 (email_main_view->
									  widgets->
									  folderView),
									 parent_path, TRUE);
						gtk_tree_path_free(parent_path);
					}
					gtk_tree_selection_select_iter(selection, &sorted_iter);
					g_free(local_folder_name);
					g_free(parent_name);
					return TRUE;
				}
			}
		}
	}

	g_free(local_folder_name);
	g_free(parent_name);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return FALSE;
}

/**
This is a callback function that gets called whenever menu item is selected
in the folder popup menu.
@param  widget          Pointer to the widget on which this function is called
@param  data            Pointer to the user data
@return void
*/
void ui_on_folder_menu_activated(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG("[%s][%s] : Starts here :\n", __FILE__, __FUNCTION__);

	if (widget == NULL || email_main_view == NULL
	    || email_main_view->widgets == NULL
	    || email_main_view->widgets->folderView == NULL || data == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	gtk_tree_model_foreach(email_main_view->folder_manager->folder_view_model,
			       ui_folder_menu_activated_for_each, data);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
When ever a folder is selected in the folder view, subsequently that folder
will be toggled in the folder pop up menu. 
@param  folderMenu holds the folder structure currently which is selected
                   in the folder list view.
@return void
*/
void ui_select_folder_menu_item(PopupFolderMenu * folderMenu)
{
	ULOG_DEBUG("[%s][%s] : Starts here :\n", __FILE__, __FUNCTION__);

	if (folderMenu == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}
	if (folderMenu->menu_item != NULL) {
		if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(folderMenu->menu_item))) {
			g_signal_handlers_block_by_func(G_OBJECT(folderMenu->menu_item),
							(gpointer) ui_on_folder_menu_activated,
							folderMenu);
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
						       (folderMenu->menu_item), TRUE);
			g_signal_handlers_unblock_by_func(G_OBJECT(folderMenu->menu_item),
							  (gpointer)
							  ui_on_folder_menu_activated, folderMenu);
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This routine is used to count the number of unread rows
in the current folder view.
@param  model           Pointer to the folder view storage model
@param  path            Pointer to the GtkTreePath of current row
@param  iter            Pointer to the GtkTreeIter of current row
@param  data            Pointer to the user data
@return boolean         TRUE --> if model is NULL (don't call again)
			FALSE --> Call again with next row
*/

gboolean
ui_count_unread_rows(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
	gint *count = NULL;
	gboolean row_is_read = FALSE;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (model == NULL || iter == NULL || data == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return TRUE;
	}

	count = (gint *) data;
	gtk_tree_model_get(model, iter, COL_READ_STATUS, &row_is_read, -1);
	if (row_is_read) {
		*count = *count + 1;
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return FALSE;
}

/*
This routine will support to display unread or unsent messages on popup
folder menu. The count will be displayed in braces besides to the folder
name.
@param widget  pointer to the widget on which this function is called.
@param data    pointer to the user data.
@return void. 
*/
void ui_on_folder_menu_select_activated(GtkWidget * widget, gpointer data)
{
	PopupFolderMenu *local_data = NULL;
	GSList *temp_list = NULL;
	gchar *temp_str = NULL;
	gchar *folder_path = NULL;
	gchar *folder_name = NULL;

	ULOG_DEBUG("[%s][%s] : Starts here :\n", __FILE__, __FUNCTION__);

	if (email_main_view == NULL
	    || email_main_view->folder_manager == NULL
	    || email_main_view->folder_manager->folder_menu_items == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}

	temp_list = email_main_view->folder_manager->folder_menu_items;
	for (; temp_list != NULL; temp_list = temp_list->next) {
		local_data = (PopupFolderMenu *) temp_list->data;
		temp_str = NULL;
		if (local_data->parent_folder == NULL && local_data->folder_name != NULL) {
			folder_path = g_strdup(local_data->folder_name);
			folder_name = ui_folder_name_with_unread_mails(folder_path);
		} else if (local_data->parent_folder != NULL && local_data->folder_name != NULL) {
			folder_path = g_strdup_printf("%s/%s", local_data->parent_folder,
						      local_data->folder_name);
			temp_str = ui_folder_name_with_unread_mails(folder_path);
			folder_name = g_strdup_printf("%s%s", INDENT, temp_str);
			g_free(temp_str);
		}
		gtk_label_set_text(GTK_LABEL
				   (GTK_BIN(GTK_MENU_ITEM(local_data->menu_item))->
				    child), folder_name);
		g_free(folder_path);
		g_free(folder_name);
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This routine is used to get the folder model sequence number.
@param folder_name   pointer to the folder name to get model number.
@param parent_folder Pointer to parent folder name to the current folder name.
@return integer     model sequence number Other wise -1.
*/
gint ui_get_folder_model_number(const gchar * folder_name, const gchar * parent_folder)
{
	GtkTreeModel *model = NULL;
	GtkTreeIter parent_iter;
	GtkTreeIter child_iter;
	gchar *parent_folder_name = NULL;
	gchar *child_folder_name = NULL;
	const gchar *search_folder_name = NULL;
	gint no_of_children = 0;
	gint count = 0;
	gint model_Seq_Num = -1;
	gboolean retval = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM((folder_name == NULL), "Input param check failed", return model_Seq_Num);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return model_Seq_Num);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed",
		    return model_Seq_Num);
	CHECK_PARAM((email_main_view->widgets->folderView == NULL),
		    "Input param check failed", return model_Seq_Num);

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	CHECK_PARAM((model == NULL), "Storage model empty for folder view!", return model_Seq_Num);
	retval = gtk_tree_model_get_iter_first(model, &parent_iter);
	CHECK_PARAM((retval == FALSE), "Storage model empty for folder view!",
		    return model_Seq_Num);

	if (parent_folder == NULL) {
		search_folder_name = folder_name;
	} else {
		search_folder_name = parent_folder;
	}
	do {
		gtk_tree_model_get(model, &parent_iter, COL_FOLDER_NAME,
				   &parent_folder_name, COL_MODEL_SEQ_NUM, &model_Seq_Num, -1);
		if (parent_folder_name != NULL
		    && strncmp(parent_folder_name, search_folder_name,
			       MAX_STR_LEN(parent_folder_name, search_folder_name)) == 0) {
			if (parent_folder != NULL) {
				no_of_children =
				    gtk_tree_model_iter_n_children(model, &parent_iter);
				for (count = 0; count < no_of_children; count++) {
					if ((gtk_tree_model_iter_nth_child(model,
									   &child_iter,
									   &parent_iter,
									   count)) == TRUE) {
						model_Seq_Num = -1;
						gtk_tree_model_get(model, &child_iter,
								   COL_FOLDER_NAME,
								   &child_folder_name,
								   COL_MODEL_SEQ_NUM,
								   &model_Seq_Num, -1);
						if (child_folder_name != NULL
						    && strncmp(child_folder_name,
							       folder_name,
							       MAX_STR_LEN
							       (child_folder_name,
								folder_name)) == 0) {
							g_free(parent_folder_name);
							g_free(child_folder_name);
							return model_Seq_Num;
						}
					}
					g_free(child_folder_name);
				}
			} else {
				g_free(parent_folder_name);
				return model_Seq_Num;
			}
		}
		g_free(parent_folder_name);
		parent_folder_name = NULL;
	}
	while (gtk_tree_model_iter_next(model, &parent_iter));
	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return model_Seq_Num;
}

/**
This routine is used to get the sorted folder storage model corresponding
to a folder number.
@param  model_number  folder model sequence number
@return GtkTreeModel  Pointer to sorted folder model Otherwise NULL.
*/
GtkTreeModel *ui_get_sorted_folder_model(gint model_number)
{
	GtkTreeModel *model = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model_number < 0 || email_main_view == NULL
	    || email_main_view->sorted_model_list == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return NULL;
	}

	model = GTK_TREE_MODEL(g_slist_nth_data(email_main_view->sorted_model_list, model_number));

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return model;
}

/**
This routine is used to get the unsorted folder storage model corresponding
to a folder number.
@param  model_number  folder model sequence number
@return GtkTreeModel  Pointer to unsorted folder model
*/
GtkTreeModel *ui_get_unsorted_folder_model(gint model_number)
{
	GtkTreeModel *model = NULL;
	GtkTreeModel *unsorted_model = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model_number < 0 || email_main_view == NULL
	    || email_main_view->sorted_model_list == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return NULL;
	}

	model = GTK_TREE_MODEL(g_slist_nth_data(email_main_view->sorted_model_list, model_number));
	if (model == NULL) {
		ULOG_DEBUG("[%s][%d] : Storage model missing!\n", __FUNCTION__, __LINE__);
		return NULL;
	}

	unsorted_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(model));

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return unsorted_model;
}

/**
This routine is used to change the folder name in the folder popup menu.
@param current_folder_name  pointer to current folder name
@param newFolderName      pointer to new folder name
@return integer  On Success 0  Otherwise -1.
*/
gint ui_on_rename_folder_menu(gchar * current_folder_name, gchar * newFolderName)
{
	gchar *current_parent_folder = NULL;
	gchar *current_child_folder = NULL;
	gchar *new_parent_folder = NULL;
	gchar *new_child_folder = NULL;
	PopupFolderMenu *local_data = NULL;

	ULOG_DEBUG("[%s][%s] : Starts here :\n", __FILE__, __FUNCTION__);

	if (current_folder_name == NULL || newFolderName == NULL
	    || email_main_view == NULL || email_main_view->menu == NULL
	    || email_main_view->menu->popup_folder_menu_group == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return -1;
	}
	if (ui_get_folder_names_from_path
	    (current_folder_name, &current_parent_folder, &current_child_folder) == FALSE) {
		return -1;
	}
	if (ui_get_folder_names_from_path(newFolderName, &new_parent_folder,
					  &new_child_folder) == FALSE) {
		return -1;
	}
	local_data = ui_get_folder_activated_structure(current_parent_folder, current_child_folder);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(GTK_MENU_ITEM(local_data->menu_item))->child),
			   new_child_folder);
	g_free(local_data->folder_name);
	local_data->folder_name = g_strdup(new_child_folder);
	g_free(local_data->parent_folder);
	local_data->parent_folder = g_strdup(new_parent_folder);

	g_free(current_parent_folder);
	g_free(current_child_folder);
	g_free(new_parent_folder);
	g_free(new_child_folder);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return 0;
}

/**
This routine is used to delete a folder from the folder popup menu.
@param folder_name   pointer to the folder name
@return void
*/
void ui_delete_folder_from_menu(const gchar * folder_name)
{
	gchar *parent_folder = NULL;
	gchar *child_folder = NULL;
	PopupFolderMenu *local_data = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (folder_name == NULL || email_main_view == NULL
	    || email_main_view->menu == NULL
	    || email_main_view->menu->popup_folder_menu_group == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return;
	}
	if (ui_get_folder_names_from_path(folder_name, &parent_folder, &child_folder) == FALSE) {
		return;
	}
	local_data = ui_get_folder_activated_structure(parent_folder, child_folder);

	g_free(local_data->parent_folder);
	g_free(local_data->folder_name);
	gtk_widget_destroy(GTK_WIDGET(local_data->menu_item));
	email_main_view->folder_manager->folder_menu_items =
	    g_slist_remove(email_main_view->folder_manager->folder_menu_items, local_data);
	g_free(local_data);
	g_free(parent_folder);
	g_free(child_folder);

	email_main_view->menu->popup_folder_menu_group = gtk_radio_menu_item_get_group
	    (GTK_RADIO_MENU_ITEM(email_main_view->menu->folder_inbox));

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

//Drag and Drop signal handlers
/**
This is call back function which will be called in drag and drop scenario.
This routine will called at receiver end to paste the data.
@param treeview The GtkWidget on which it is called.
@param drag_context To hold the drag context to perfom operations.
@param x The x co-ordinate value of the cursor where it is droped.
@param y The y co-ordinate value of the cursor where it is droped.
@param data The data that is to be moved from source to destination.
@param info The information of the drag operation.
@param event_time The event of the time when it is droped.
@param user_data To hold the user data to be carried.
@return void.
*/
void
drag_data_received_handler(GtkWidget * treeview,
			   GdkDragContext * drag_context, gint x, gint y,
			   GtkSelectionData * data, guint info,
			   guint event_time, gpointer user_data)
{
	GtkTreeModel *folder_model = NULL;
	GtkTreeIter iter;
	gint current_model_num = 0, count = 0;
	gchar *current_folder_name = NULL;
	GtkTreePath *path = NULL;
	gboolean ret_val = FALSE, retval = FALSE;

	ULOG_DEBUG("[%s][%s] : Starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM((treeview == NULL), "Input param check failed!", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed!", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed!", return);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed!", return);
	CHECK_PARAM((email_main_view->widgets->messageListView == NULL),
		    "Input param check failed!", return);

	folder_model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
	ret_val = gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(treeview), x,
						y, &path, NULL, NULL, NULL);
	CHECK_PARAM((ret_val == FALSE), "No folder path at drop place!", return);

	ui_count_marked_rows(GTK_TREE_VIEW(treeview), &count);
	if (count == 0) {
		get_user_confirmation(_("mcen_ib_nothing_to_move"), NULL, NULL, INFO_BANNER);
		return;
	}

	if (email_main_view->folder_manager->currently_displayed_model == OUTBOX_MODEL_NUM) {
		retval = ui_unselect_outbox_msg_being_sent();
		if (retval == TRUE) {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_cut_mess"));
		}
	}

	if (gtk_tree_model_get_iter(folder_model, &iter, path)) {
		gtk_tree_model_get(folder_model, &iter, COL_MODEL_SEQ_NUM,
				   &current_model_num, COL_FOLDER_NAME, &current_folder_name, -1);
		ULOG_DEBUG("[%s][%d] : Folder at drop place = %s\n", __FUNCTION__, __LINE__,
			   current_folder_name);
		g_free(current_folder_name);

		if (ui_is_move_operation_allowed(current_model_num, NULL)) {
			ret_val =
			    ui_manage_selected_items(email_main_view->folder_manager->
						     currently_displayed_model,
						     current_model_num, ACTION_MOVE, FALSE, NULL);
			if (!ret_val) {
				ULOG_DEBUG("[%s][%d] : Move operation failed!\n",
					   __FUNCTION__, __LINE__);
			}
		}
	}

	/*Stop signal emission */
	g_signal_stop_emission_by_name(email_main_view->widgets->folderView, "drag_data_received");

	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is call back function which will be called in drag and drop scenario.
This routine will called at start of drag operation. Doesn't do much, but
it is mandatory to have this function defined.
@param treeview Pointer to tree view on which the drag pop up menu called.
@param drag_context Pointer to drag context.
@param user_data Pointer to user data.
@return void.
*/
void drag_begin_handler(GtkWidget * treeview, GdkDragContext * drag_context, gpointer user_data)
{
	ULOG_DEBUG("[%s][%s] : Starts here :\n", __FILE__, __FUNCTION__);

	/*Block tree selection functions */
	g_signal_handlers_block_by_func(G_OBJECT
					(email_main_view->widgets->messageListView),
					(gpointer) ui_messageview_on_row_selected, NULL);

	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is call back function which will be called in drag and drop scenario.
This routine will called to obtain data to be pasted.
@param treeview Pointer to tree view on which the drag pop up menu called.
@param drag_context Pointer to drag context.
@param data Pointer to drag selection data.
@param info Drag information.
@param time Time of the drag operation is called.
@param user_data Pointer to user data.
@return void.
*/
void
drag_data_get_handler(GtkWidget * treeview,
		      GdkDragContext * drag_context, GtkSelectionData * data,
		      guint info, guint time, gpointer user_data)
{
	ULOG_DEBUG("[%s][%s] : Starts here :\n", __FILE__, __FUNCTION__);

	gtk_selection_data_set(data, gdk_atom_intern("text/html", TRUE), 0, NULL, 0);

	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is call back function which will be called in drag and drop scenario.
This routine will called at end of drag operation. Doesn't do much, but it
is mandatory to have this function defined.
@param treeview Pointer to tree view on which the drag pop up menu called.
@param drag_context Pointer to drag context.
@param user_data Pointer to user data.
@return void.
*/
void drag_end_handler(GtkWidget * treeview, GdkDragContext * drag_context, gpointer user_data)
{
	ULOG_DEBUG("[%s][%s] : Starts here :\n", __FILE__, __FUNCTION__);

	/*select the folder having currently displayed messages. */
	ui_select_folder_from_view(GTK_TREE_VIEW(email_main_view->widgets->folderView),
				   email_main_view->folder_manager->currently_displayed_model);
	/*Unblock tree selection functions */
	g_signal_handlers_unblock_by_func(G_OBJECT
					  (email_main_view->widgets->
					   messageListView),
					  (gpointer) ui_messageview_on_row_selected, NULL);

	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 * This is call back function which will be called while drag is going on.
 * It selects a folder depending upon the stylus position.
 * @param treeview Pointer to tree view on which the drag destination.
 * @param drag_context Pointer to drag context.
 * @param x is x-coordinate of stylus tip.
 * @param y is y-coordinate of stylus tip.
 * @param time is time-stamp of drag.
 * @param user_data Pointer to user data, NULL here.
 * @return TRUE if callback is executed.
 * */
gboolean
drag_motion_handler(GtkWidget * view, GdkDragContext * drag_context, gint x, gint y,
		    guint time, gpointer user_data)
{
	GtkTreePath *path = NULL;
	GtkTreeSelection *selection = NULL;

	gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(view), x, y, &path, NULL, NULL, NULL);
	if (path) {
		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
		/*ui_folderview_selection_changed_cb will get fired upon selecting a path in the folder view. */
		g_signal_handlers_block_by_func(G_OBJECT(selection),
						(gpointer) ui_folderview_selection_changed_cb,
						NULL);
		gtk_tree_selection_select_path(selection, path);
		g_signal_handlers_unblock_by_func(G_OBJECT(selection),
						  (gpointer) ui_folderview_selection_changed_cb,
						  NULL);
		gtk_tree_path_free(path);
	}
	return TRUE;
}


/**
Helper function to obtain the unread/unsent count of a folder, given its
storage model number.
@param  model_num  Folder storage model sequence number.
@return gint      Unread/unsent count Otherwise -1.
*/
static gint ui_get_folder_unread_unsent_count(guint model_num)
{
	gint msg_count = -1;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the necessary input & global params */
	if (model_num >= g_slist_length(email_main_view->sorted_model_list)
	    || email_main_view == NULL || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return msg_count;
	}

	email_main_view->folder_manager->is_folder_present = FALSE;

	/*Check if any folder exists already with the above name */
	gtk_tree_model_foreach(email_main_view->folder_manager->folder_view_model,
			       ui_get_folder_msg_count, &model_num);
	if (email_main_view->folder_manager->is_folder_present) {
		msg_count = email_main_view->folder_manager->folder_msg_count;
	}
	ULOG_DEBUG("[%s][%d] : Folder unread/unsent count = %d\n", __FUNCTION__,
		   __LINE__, msg_count);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return msg_count;
}

/*
This function is used to set unread unsent messages count to folder in braces
along with folder name in folder list view.
@param model_num Model number of the folder.
@return void.
*/
void ui_add_folder_name_with_unread_unsent_mails(gint model_num)
{
	GtkTreeModel *folder_model = NULL;
	GtkTreeModel *message_model = NULL;
	gint count = 0, model_Seq_Num = -1, no_of_children = 0;
	gchar *local_display_name = NULL;
	gchar *folder_name = NULL;
	const gchar *display_name = NULL;
	GtkTreeIter parent_iter;
	GtkTreeIter child_iter;
	gboolean retval = FALSE;
	gboolean row_found = FALSE;
	gboolean child_flag = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	folder_model = email_main_view->folder_manager->folder_view_model;
	retval = gtk_tree_model_get_iter_first(folder_model, &parent_iter);
	CHECK_PARAM((retval == FALSE), "Storage model empty for folder view!", return);

	/* Traverse through tree model until the model number is matched */
	do {
		g_free(folder_name);
		gtk_tree_model_get(folder_model, &parent_iter, COL_FOLDER_NAME,
				   &folder_name, COL_MODEL_SEQ_NUM, &model_Seq_Num, -1);
		if (model_num == model_Seq_Num) {
			row_found = TRUE;
			break;
		} else {
			no_of_children = gtk_tree_model_iter_n_children(folder_model, &parent_iter);
			if (no_of_children > 0) {
				for (count = 0; count < no_of_children; count++) {
					g_free(folder_name);
					folder_name = NULL;
					if ((gtk_tree_model_iter_nth_child(folder_model,
									   &child_iter,
									   &parent_iter,
									   count)) == TRUE) {
						model_Seq_Num = -1;
						gtk_tree_model_get(folder_model,
								   &child_iter,
								   COL_FOLDER_NAME,
								   &folder_name,
								   COL_MODEL_SEQ_NUM,
								   &model_Seq_Num, -1);
						if (model_num == model_Seq_Num) {
							row_found = TRUE;
							child_flag = TRUE;
							break;
						}
					}
				}
			}
		}
	}
	while (!row_found && gtk_tree_model_iter_next(folder_model, &parent_iter));

	if (folder_name != NULL) {
		display_name = ui_get_folder_display_name(model_num, folder_name);

		/*Check if count is already available.
		   If not calculate the count from the folder storage model. */
		count = ui_get_folder_unread_unsent_count(model_num);

		if (count == DEFAULT_UNREAD_UNSENT_COUNT || count == 0) {
			message_model = ui_get_unsorted_folder_model(model_num);
			/*If folder is OUTBOX or DRAFTS, total num of msgs is UNSENT count */
			if (model_num == OUTBOX_MODEL_NUM || model_num == DRAFT_MODEL_NUM) {
				count = gtk_tree_model_iter_n_children(message_model, NULL);
			}
			/*For other folders get the unread msgs count */
			else {
				count = 0;
				gtk_tree_model_foreach(message_model, (GtkTreeModelForeachFunc)
						       ui_count_unread_rows, &count);
			}
		}
		if (count != 0) {
			local_display_name = g_strdup_printf("%s (%d)", display_name, count);
		} else {
			local_display_name = g_strdup_printf("%s", display_name);
		}

		if (child_flag == FALSE) {
			gtk_tree_store_set(GTK_TREE_STORE(folder_model), &parent_iter,
					   COL_DISPLAY_NAME, local_display_name, -1);
		} else {
			gtk_tree_store_set(GTK_TREE_STORE(folder_model), &child_iter,
					   COL_DISPLAY_NAME, local_display_name, -1);
		}

		g_free(local_display_name);
	}
	g_free(folder_name);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is used to get the folder name with unread masseges count in
brases for a given folder.
@param folderpath To hold the folder path.
@return pointer to gchar  folder name with unread messges count.
*/
gchar *ui_folder_name_with_unread_mails(const gchar * folderpath)
{
	GtkTreeModel *model = NULL;
	gint count = 0;
	gchar *ret_str = NULL;
	const gchar *folder_display_name = NULL;
	gint model_num = 0;
	GString *local_folder_name = NULL;
	gchar *parent_folder = NULL, *child_folder = NULL;

	if (folderpath == NULL) {
		ULOG_DEBUG("[%s][%d] : Input param check failed!\n", __FUNCTION__, __LINE__);
		return NULL;
	}

	if (ui_get_folder_names_from_path(folderpath, &parent_folder, &child_folder) == TRUE) {
		model_num = ui_get_folder_model_number(child_folder, parent_folder);
	}

	if (model_num == INVALID_MODEL_NUM) {
		ULOG_DEBUG("[%s][%s] : Invalid Model Number :\n", __FILE__, __FUNCTION__);
		g_free(parent_folder);
		g_free(child_folder);
		return NULL;
	}
	model = ui_get_unsorted_folder_model(model_num);
	folder_display_name = ui_get_folder_display_name(model_num, child_folder);

	/*Check if count is already available.
	   If not calculate the count from the folder storage model. */
	count = ui_get_folder_unread_unsent_count(model_num);

	if (count == DEFAULT_UNREAD_UNSENT_COUNT) {
		/*If folder is OUTBOX or DRAFTS, total num of msgs is UNSENT count */
		if (model_num == OUTBOX_MODEL_NUM || model_num == DRAFT_MODEL_NUM) {
			count = gtk_tree_model_iter_n_children(model, NULL);
		}
		/*For other folders get the unread msgs count */
		else {
			count = 0;
			gtk_tree_model_foreach(model, (GtkTreeModelForeachFunc)
					       ui_count_unread_rows, &count);
		}
	}

	local_folder_name = g_string_new(NULL);
	CHECK_PARAM((local_folder_name == NULL), "Memory allocation failed", return NULL);
	if (count != 0) {
		g_string_printf(local_folder_name, "%s (%d)", folder_display_name, count);
	} else {
		g_string_printf(local_folder_name, "%s", folder_display_name);
	}
	ret_str = g_strdup(local_folder_name->str);
	g_string_free(local_folder_name, TRUE);
	g_free(parent_folder);
	g_free(child_folder);
	return ret_str;
}

/**
This is a helper function which checks if Inbox view is currently displayed,
and if required sets Inbox storage model to the message listview.
This is called during send-receive operation on receiving any new mails.
@param  model_num         Sequence number of the folder storage model
@param  set_sorted_model  Whether to set sorted or unsorted model to listview
@return void
*/
void ui_set_folder_model_to_msgview(gint model_num, gboolean set_sorted_model)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets->messageListView == NULL),
		    "Input param check failed", return);

	if (email_main_view->folder_manager->currently_displayed_model == model_num) {
		if (gtk_tree_view_get_model
		    (GTK_TREE_VIEW(email_main_view->widgets->messageListView)) == NULL) {
			if (set_sorted_model) {
				gtk_tree_view_set_model(GTK_TREE_VIEW
							(email_main_view->widgets->
							 messageListView),
							ui_get_sorted_folder_model(model_num));
			} else {
				gtk_tree_view_set_model(GTK_TREE_VIEW
							(email_main_view->widgets->
							 messageListView),
							ui_get_unsorted_folder_model(model_num));
			}

			ULOG_DEBUG("[%s][%d] : View is empty. Current model set!\n",
				   __FUNCTION__, __LINE__);

			/*Hide 'No messages to show' label */
			gtk_widget_hide(email_main_view->widgets->no_msgs_label);
			gtk_widget_show(email_main_view->widgets->message_scrollwindow);
		}
	}

	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a helper function which checks if current view is not Inbox view
and sets Inbox storage model to the message listview.
This is called during send-receive operation so that on receiving any new
mails, Inbox view is displayed.
@param void
@return void
*/
void ui_display_inbox_folder_view(void)
{
	GtkTreeIter first_iter;
	GtkTreeSelection *selection = NULL;
	GtkTreeModel *model = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets->folderView == NULL),
		    "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets->messageListView == NULL),
		    "Input param check failed", return);
	CHECK_PARAM((email_main_view->menu == NULL), "Input param check failed", return);

	selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	if (email_main_view->folder_manager->currently_displayed_model == INBOX_MODEL_NUM) {
		ULOG_DEBUG("[%s][%s] : Inbox view already selected\n", __FILE__, __FUNCTION__);
		/*Refresh the current view */
		ui_folderview_selection_changed_cb(selection, NULL);
	} else {

		model = gtk_tree_view_get_model(GTK_TREE_VIEW
						(email_main_view->widgets->folderView));
		gtk_tree_model_get_iter_first(model, &first_iter);
		gtk_tree_selection_select_iter(selection, &first_iter);
	}

	if (email_main_view->tags->main_view_topped) {
		gtk_widget_grab_focus(email_main_view->widgets->messageListView);
	}

	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a helper function to parse out parent and sub folder names in the given folder path.
This is called when creating sub folders to any parent folder.
@param path           To store the folder path
@param parent_folder  This is out param store the parent folder name.
@param sub_folder     This is out param store the child folder name.
@return gboolean    return TRUE for success.
		           FALSE for failure.
*/
gboolean
ui_get_folder_names_from_path(const gchar * path, gchar ** parent_folder, gchar ** sub_folder)
{
	gchar **local_folder_name = NULL;
	CHECK_PARAM((path == NULL), "Input param check failed", return FALSE);

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (g_utf8_strchr(path, g_utf8_strlen(path, -1), '/') != NULL) {
		local_folder_name = g_strsplit(path, "/", 2);
		if (g_utf8_strchr
		    (local_folder_name[1], g_utf8_strlen(local_folder_name[1], -1), '/') != NULL) {
			g_strfreev(local_folder_name);
			local_folder_name = NULL;
			return FALSE;
		} else {
			*parent_folder = g_strdup(local_folder_name[0]);
			*sub_folder = g_strdup(local_folder_name[1]);
			g_strfreev(local_folder_name);
			local_folder_name = NULL;
			return TRUE;
		}
	} else {
		*parent_folder = NULL;
		*sub_folder = g_strdup(path);
		return TRUE;
	}
	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
}

/**
This is a helper function which checks if given folder has any parent folder,
and return a pointer to the parent folder's row in folder view.
@param folder_name  To store the current folder name.
@return iter        Pointer to parent iter if exisiting, otherwise NULL.
*/
gboolean ui_get_folder_iter(const gchar * folder_name, GtkTreeIter * iter)
{
	GtkTreeModel *model = NULL;
	gchar *local_folder_name = NULL;

	CHECK_PARAM((folder_name == NULL), "Input param check failed", return FALSE);
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	model = email_main_view->folder_manager->folder_view_model;
	if (gtk_tree_model_get_iter_first(model, iter) == FALSE) {
		ULOG_DEBUG("[%s][%s] : Faild at get iter :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
	do {
		gtk_tree_model_get(model, iter, COL_FOLDER_NAME, &local_folder_name, -1);

		if (local_folder_name != NULL
		    && strncmp(local_folder_name, folder_name,
			       MAX_STR_LEN(local_folder_name, folder_name)) == 0) {
			g_free(local_folder_name);
			return TRUE;
		}
		g_free(local_folder_name);
	}
	while (gtk_tree_model_iter_next(model, iter));

	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return FALSE;
}


/**
This is a helper function which checks if given folder has any parent folder,
and return the parent folder name.
@param folder_name  To store the current folder name.
@return parent_folder_name   Pointer to parent folder name if exisiting, otherwise NULL.
*/
gchar *ui_get_parent_folder_name(const gchar * folder_name)
{
	GtkTreeIter child_iter;
	GtkTreeIter parent_iter;
	GtkTreeModel *model = NULL;
	gchar *parent_folder_name = NULL;

	CHECK_PARAM((folder_name == NULL), "Input param check failed", return NULL);
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	model = email_main_view->folder_manager->folder_view_model;
	if (ui_get_folder_iter(folder_name, &child_iter) == TRUE) {
		if (gtk_tree_model_iter_parent(model, &parent_iter, &child_iter) == TRUE) {
			gtk_tree_model_get(model, &parent_iter, COL_FOLDER_NAME,
					   &parent_folder_name, -1);
			return parent_folder_name;
		}
	}
	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return NULL;
}

/**
This function is used to free the global folder_menu_items single linked list
and it contents.
@return Nothing.
*/
void ui_free_folder_menu_items_list(void)
{
	GSList *local_list = NULL;
	PopupFolderMenu *popup_folder_menu = NULL;

	ULOG_DEBUG("[%s][%s] : Starts here!:\n", __FILE__, __FUNCTION__);

	if (email_main_view->folder_manager->folder_menu_items == NULL) {
		ULOG_DEBUG("[%s][%s] : Input param failed!:\n", __FILE__, __FUNCTION__);
		return;
	}

	local_list = email_main_view->folder_manager->folder_menu_items;
	while (local_list != NULL) {
		popup_folder_menu = (PopupFolderMenu *) local_list->data;
		g_free(popup_folder_menu->parent_folder);
		g_free(popup_folder_menu->folder_name);
		g_free(popup_folder_menu);
		local_list = g_slist_next(local_list);
	}
	g_slist_free(email_main_view->folder_manager->folder_menu_items);
	email_main_view->folder_manager->folder_menu_items = NULL;

	ULOG_DEBUG("[%s][%s] : Ends here!:\n", __FILE__, __FUNCTION__);
}

/**
This function is used to get the folder information structure
by checking with parenet folder name and child folder name in the global list.
@param parent_name pointer to parent folder name.
@param child_name pointer to child folder name.
@return PopupFolderMenu Pointer to folder info structure Otherwise NULL.
*/
PopupFolderMenu *ui_get_folder_activated_structure(gchar * parent_name, gchar * child_name)
{
	PopupFolderMenu *local_data = NULL;
	GSList *temp_list = NULL;
	gint success = -1;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM((child_name == NULL), "Input param check failed", return NULL);

	temp_list = email_main_view->folder_manager->folder_menu_items;
	for (; temp_list != NULL; temp_list = temp_list->next) {
		local_data = (PopupFolderMenu *) temp_list->data;
		if (local_data->parent_folder == NULL && parent_name == NULL) {
			success = strncmp(local_data->folder_name, child_name,
					  MAX_STR_LEN(local_data->folder_name, child_name));
			CHECK_PARAM((success == 0), "Folder struct found", return local_data);
		} else if (local_data->parent_folder != NULL && parent_name != NULL) {
			if (local_data->folder_name != NULL && child_name != NULL) {
				if (strncmp(local_data->parent_folder, parent_name,
					    MAX_STR_LEN(local_data->parent_folder,
							parent_name)) == 0) {
					success =
					    strncmp(local_data->folder_name,
						    child_name,
						    MAX_STR_LEN(local_data->
								folder_name, child_name));
					CHECK_PARAM((success == 0), "Folder struct found",
						    return local_data);
				}
			}
		}
	}

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return NULL;
}

/*
This function is called by the engine when ever the MMC card is pluged-in the
device to add the archive folder in the folder list.
@return nothing.
*/
void ui_add_archive_folder(void)
{
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	gchar *folder_name = NULL;
	gboolean retval = FALSE;

	ULOG_DEBUG("[%s][%s] : Starts here \n", __FILE__, __FUNCTION__);

	model = email_main_view->folder_manager->folder_view_model;

	CHECK_PARAM((model == NULL), "Storage model empty for folder view!", return);
	retval = gtk_tree_model_get_iter_first(model, &iter);
	CHECK_PARAM((retval == FALSE), "Storage model empty for folder view!", return);
	do {
		gtk_tree_model_get(model, &iter, COL_FOLDER_NAME, &folder_name, -1);
		if (folder_name != NULL && ui_compare_utf8_strings(folder_name, ARCHIVE) == 0) {
			g_free(folder_name);
			ULOG_DEBUG("[%s][%d] : Folder exists with name '%s'!\n",
				   __FUNCTION__, __LINE__, folder_name);
			return;
		}
		g_free(folder_name);
		folder_name = NULL;
	}
	while (gtk_tree_model_iter_next(model, &iter));

	ui_add_new_folder(GTK_TREE_STORE(email_main_view->folder_manager->
					 folder_view_model), ARCHIVE, FALSE, -1);

	/*Add the row to Move to Dialog also if required */
	if (email_main_view->folder_manager->move_to_dialog != NULL) {
		gtk_dialog_response(GTK_DIALOG
				    (email_main_view->folder_manager->move_to_dialog),
				    REFRESH_DIALOG_SIGNAL);
	}

	ULOG_DEBUG("[%s][%s] : ends here \n", __FILE__, __FUNCTION__);
	return;
}

/*
This function is called by the engine when ever the MMC card is removed from
the device to remove the archive folder from the folder list.
@return nothing.
*/
void ui_remove_archive_folder(void)
{
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	gchar *folder_name = NULL;
	gboolean retval = FALSE;
	gint modelnum = -1;

	ULOG_DEBUG("[%s][%s] : Starts here \n", __FILE__, __FUNCTION__);

	model = email_main_view->folder_manager->folder_view_model;

	CHECK_PARAM((model == NULL), "Storage model empty for folder view!", return);
	retval = gtk_tree_model_get_iter_first(model, &iter);
	CHECK_PARAM((retval == FALSE), "Storage model empty for folder view!", return);
	do {
		gtk_tree_model_get(model, &iter, COL_FOLDER_NAME, &folder_name,
				   COL_MODEL_SEQ_NUM, &modelnum, -1);
		if (folder_name != NULL && ui_compare_utf8_strings(folder_name, ARCHIVE) == 0) {
			/*Display Inbox folder view */
			ui_display_inbox_folder_view();

			/*Remove the row from folder view */
			gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
			ui_delete_folder_from_menu(folder_name);
			g_free(folder_name);

			/*Destroy the corresponding folder view store */
			ui_delete_listview_model(modelnum);

			/*Remove the row from Move to Dialog also if required */
			if (email_main_view->folder_manager->move_to_dialog != NULL) {
				gtk_dialog_response(GTK_DIALOG
						    (email_main_view->folder_manager->
						     move_to_dialog), REFRESH_DIALOG_SIGNAL);
			}
			return;
		}
		g_free(folder_name);
		folder_name = NULL;
	}
	while (gtk_tree_model_iter_next(model, &iter));

	return;
}

/**
This is a window creation function used to create the folder properties dialog.
@param win pointer to the widget on which this function is called.
@param data pointer to the user data.
@return viod
*/
void ui_create_folder_properties_dialog(GtkWidget * win, gpointer data)
{
	GtkWidget *prop_dialog = NULL;
	GtkWidget *dialog_vbox1 = NULL;
	GtkWidget *caption_control = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *name_label_data = NULL;
	GtkWidget *noofmessages_label_data = NULL;
	GtkWidget *size_label_data = NULL;
	GtkWidget *dialog_action_area = NULL;
#if RETAINED_EMAIL
	GtkWidget *folder_setting_button = NULL;
#endif
	GtkWidget *close_button = NULL;
	GtkTreeModel *folder_model = NULL;
	gchar *current_folder_name = NULL;
	gchar *parent_folder = NULL, *sub_folder = NULL;
	gint num_of_msgs_in_folder = -1;
	gchar *num_of_msgs = NULL;
	gint size_of_folder = 0;
	gchar *size_of_folder_str = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	prop_dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(prop_dialog), _("mcen_ti_folder_properties"));
	gtk_window_set_position(GTK_WINDOW(prop_dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(prop_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(prop_dialog), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(prop_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	gtk_window_set_resizable(GTK_WINDOW(prop_dialog), TRUE);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(prop_dialog), TRUE);
	gtk_dialog_set_has_separator(GTK_DIALOG(prop_dialog), FALSE);

	dialog_vbox1 = GTK_DIALOG(prop_dialog)->vbox;
	gtk_widget_show(dialog_vbox1);

	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	current_folder_name = ui_get_folder_name(email_main_view->folder_manager->
						 currently_displayed_model);
	if (ui_get_folder_names_from_path(current_folder_name, &parent_folder,
					  &sub_folder) == FALSE) {
		ULOG_DEBUG("[%s][%s] : failed to folder name:\n", __FILE__, __FUNCTION__);
		return;
	} else {
		name_label_data =
		    gtk_label_new(ui_get_folder_display_name
				  (email_main_view->folder_manager->
				   currently_displayed_model, sub_folder));
		gtk_misc_set_alignment((GtkMisc *) name_label_data, 0, 0);
		g_free(parent_folder);
	}
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_folder_properties_foldername"),
			       name_label_data, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox1), caption_control, TRUE, TRUE, 0);
	gtk_widget_show_all(caption_control);

	folder_model =
	    ui_get_sorted_folder_model(email_main_view->folder_manager->currently_displayed_model);
	num_of_msgs_in_folder = gtk_tree_model_iter_n_children(folder_model, NULL);
	num_of_msgs = g_strdup_printf("%d", num_of_msgs_in_folder);
	noofmessages_label_data = gtk_label_new(num_of_msgs);
	gtk_misc_set_alignment((GtkMisc *) noofmessages_label_data, 0, 0);
	g_free(num_of_msgs);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_folder_properties_messages"),
			       noofmessages_label_data, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox1), caption_control, TRUE, TRUE, 0);
	gtk_widget_show_all(caption_control);

	size_of_folder = ui_dbus_folder_get_size(current_folder_name);
	if (size_of_folder < 0) {
		ULOG_DEBUG_F("Illegal folder size obtained '%d'", size_of_folder);
		size_of_folder = 0;
	}
	size_of_folder_str = ui_format_size_value(size_of_folder);
	size_label_data = gtk_label_new(size_of_folder_str);
	gtk_misc_set_alignment((GtkMisc *) size_label_data, 0, 0);
	g_free(size_of_folder_str);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_folder_properties_size"),
			       size_label_data, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox1), caption_control, TRUE, TRUE, 0);
	gtk_widget_show_all(caption_control);

	dialog_action_area = GTK_DIALOG(prop_dialog)->action_area;
	gtk_widget_show(dialog_action_area);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area), GTK_BUTTONBOX_END);

#if RETAINED_EMAIL
	folder_setting_button =
	    gtk_button_new_with_mnemonic(_("mcen_bd_folder_properties_settings"));
	gtk_widget_show(folder_setting_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(prop_dialog),
				     folder_setting_button, GTK_RESPONSE_OK);
	GTK_WIDGET_SET_FLAGS(folder_setting_button, GTK_CAN_DEFAULT);

	if (sub_folder != NULL && strncmp(sub_folder, SENT_DIR,
					  MAX_STR_LEN(sub_folder, SENT_DIR)) == 0) {
		gtk_widget_set_sensitive(folder_setting_button, TRUE);
	} else {
		gtk_widget_set_sensitive(folder_setting_button, FALSE);
	}
#endif

	close_button = gtk_button_new_with_mnemonic(_("mcen_bd_folder_properties_close"));
	gtk_widget_show(close_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(prop_dialog), close_button, GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS(close_button, GTK_CAN_DEFAULT);

#if RETAINED_EMAIL
	g_signal_connect(G_OBJECT(folder_setting_button), "clicked",
			 G_CALLBACK(ui_on_folder_settings_activate), NULL);
#endif
	g_signal_connect(G_OBJECT(close_button), "clicked",
			 G_CALLBACK(ui_destroy_folder_dialog), prop_dialog);
#if RETAINED_EMAIL
	g_signal_connect(folder_setting_button, "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mail_ib_no_settings_inbox"));
#endif
	if (ossohelp_exists(HELP_ID_FOLDERPROPERTIES) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) prop_dialog);
		g_signal_connect(G_OBJECT(prop_dialog), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_FOLDERPROPERTIES);
	}
	g_signal_connect(G_OBJECT(prop_dialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), close_button);
	gtk_widget_show(GTK_WIDGET(prop_dialog));

	g_free(sub_folder);
	g_free(current_folder_name);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is helper function to get date and time in string format.
@param msg_date This is time structure which contains date and time seconds format.
@return date and time in string format Otherwise NULL.
*/
static gchar *ui_get_message_date_time_string(time_t msg_date)
{
	gchar *date_str = NULL;
	gchar *time_str = NULL;
	gchar *display_str = NULL;
	gboolean retval = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	/*If date is not set, return a blank string */
	if (msg_date == 0) {
		display_str = g_strdup(" ");
		return display_str;
	}
	date_str = g_new0(gchar, MAX_DATE_LEN);
	time_str = g_new0(gchar, MAX_DATE_LEN);
	if (date_str == NULL || time_str == NULL) {
		ULOG_CRIT_F("[%s][%s] : Memory allocation failure!\n", __FILE__, __FUNCTION__);
		g_free(date_str);
		g_free(time_str);
		return NULL;
	}
	retval = strftime(time_str, MAX_DATE_LEN, "%X", localtime(&msg_date));
	if (retval == FALSE) {
		ULOG_DEBUG("[%s][%s] : Unable to collect date values!\n", __FILE__, __FUNCTION__);
		return NULL;
	}
	retval = strftime(date_str, MAX_DATE_LEN, "%x", localtime(&msg_date));
	if (retval == FALSE) {
		ULOG_DEBUG("[%s][%s] : Unable to collect date values!\n", __FILE__, __FUNCTION__);
		return NULL;
	}
	display_str = g_strdup_printf("%s %s", date_str, time_str);
	g_free(date_str);
	g_free(time_str);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return display_str;
}

/**
This is helper function to create label field in the message properties dialog.
@param table Pointer to table in which created label should be inserted.
@param label_name Pointer to label name by which field label is created.
@param data_name Pointer to data name by which data label is created.
@param left value to be attached to left side of the label.
@param right value to be attached to right side of the label.
@param top value to be attached to top side of the label.
@param bottom value to ve attached to bottom side of the label.
@return void.
*/
static void
ui_add_data_field_label_to_table(GtkTable * table, gchar * label_name,
				 gchar * data_name, guint left, guint right,
				 guint * top, guint * bottom)
{
	GtkWidget *field_label = NULL, *data_label = NULL;
	guint data_left = 0, data_right = 0;

	ULOG_DEBUG("[%s][%s] : Starts here :\n", __FILE__, __FUNCTION__);

	if (table == NULL || label_name == NULL) {
		ULOG_DEBUG("[%s][%s] : Input param check failed :\n", __FILE__, __FUNCTION__);
		return;
	}
	data_left = left + 1;
	data_right = right + 1;

	field_label = gtk_label_new(label_name);
	gtk_widget_show(field_label);
	gtk_label_set_line_wrap((GtkLabel *) field_label, TRUE);
	gtk_misc_set_alignment(GTK_MISC(field_label), FIELD_X_ALIGN, FIELD_Y_ALIGN);
	gtk_misc_set_padding(GTK_MISC(field_label), X_PADDING, Y_PADDING);
	gtk_table_attach(table, field_label, left, right, (*top), (*bottom),
			 (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_label_set_justify(GTK_LABEL(field_label), GTK_JUSTIFY_FILL);

	data_label = gtk_label_new(data_name);
	gtk_widget_show(data_label);
	gtk_label_set_line_wrap((GtkLabel *) data_label, TRUE);
	gtk_misc_set_alignment(GTK_MISC(data_label), DATA_X_ALIGN, DATA_Y_ALIGN);
	gtk_misc_set_padding(GTK_MISC(data_label), X_PADDING, Y_PADDING);
	gtk_table_attach(table, data_label, data_left, data_right, (*top), (*bottom),
			 GTK_FILL, (GtkAttachOptions) 0, 0, 0);
	(*top)++;
	(*bottom)++;
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This is a window creation function used to create the message properties dialog.
@param	headerinfo This is a structure contains total messages information.
@param  msg_size   This is integer type holds message size information.
@return viod
*/
void ui_create_message_properties_dialog(HeaderInfo * headerinfo, gint msg_size)
{
	GtkWidget *prop_dialog = NULL;
	GtkWidget *dialog_vbox1 = NULL;
	GtkWidget *vbox1 = NULL;
	gint response_id = -1;
	gchar *size_str = NULL;
	GtkWidget *close_button = NULL;
	GtkWidget *scrolledwindow = NULL;
	GtkTable *table = NULL;
	gchar *date_time_str = NULL;
	guint left = 0, right = 1, top = 0, bottom = 1;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM((headerinfo == NULL), "Input param check failed", return);

	prop_dialog = gtk_dialog_new();
	gtk_window_set_default_size(GTK_WINDOW(prop_dialog), 400, 220);
	gtk_window_set_title(GTK_WINDOW(prop_dialog), _("mcen_ti_message_properties"));
	gtk_window_set_transient_for(GTK_WINDOW(prop_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(prop_dialog), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(prop_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	gtk_window_set_resizable(GTK_WINDOW(prop_dialog), TRUE);
	gtk_dialog_set_has_separator(GTK_DIALOG(prop_dialog), FALSE);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(prop_dialog), TRUE);

	vbox1 = GTK_DIALOG(prop_dialog)->vbox;
	gtk_widget_show(vbox1);

	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolledwindow, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	dialog_vbox1 = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(dialog_vbox1);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolledwindow), dialog_vbox1);
	gtk_viewport_set_shadow_type((GtkViewport *)
				     ((GtkBin *) (scrolledwindow))->child, GTK_SHADOW_NONE);
	table = GTK_TABLE(gtk_table_new(10, 2, FALSE));
	gtk_widget_show((GtkWidget *) table);
	gtk_container_add(GTK_CONTAINER(dialog_vbox1), GTK_WIDGET(table));
	gtk_table_set_homogeneous(GTK_TABLE(table), FALSE);

	if (headerinfo->folderNum == DRAFT_MODEL_NUM
	    && !email_main_view->account_manager->configured_accounts)
		ui_add_data_field_label_to_table(table, _("mcen_fi_message_properties_from"), NULL,
						 left, right, &top, &bottom);
	else
		ui_add_data_field_label_to_table(table, _("mcen_fi_message_properties_from"),
						 headerinfo->from, left, right, &top, &bottom);

	ui_add_data_field_label_to_table(table,
					 _("mcen_fi_message_properties_subject"),
					 headerinfo->subject, left, right, &top, &bottom);

	if (headerinfo->folderNum != OUTBOX_MODEL_NUM
	    && headerinfo->folderNum != SENT_MODEL_NUM && headerinfo->folderNum != DRAFT_MODEL_NUM)
	{
		date_time_str = ui_get_message_date_time_string(headerinfo->recieve_date);
		ui_add_data_field_label_to_table(table,
						 _
						 ("mcen_fi_message_properties_received"),
						 date_time_str, left, right, &top, &bottom);
		g_free(date_time_str);
		date_time_str = NULL;
		/*If recieve_date is not set, ignore the sent_date value as well */
		if (headerinfo->recieve_date == 0)
			headerinfo->sent_date = 0;
	}

	if (headerinfo->folderNum == OUTBOX_MODEL_NUM || headerinfo->folderNum == DRAFT_MODEL_NUM) {
		date_time_str = ui_get_message_date_time_string(headerinfo->creation_date);
		ui_add_data_field_label_to_table(table,
						 _("mcen_fi_message_properties_created"),
						 date_time_str, left, right, &top, &bottom);
		g_free(date_time_str);
		date_time_str = NULL;
	}

	if (headerinfo->folderNum != OUTBOX_MODEL_NUM && headerinfo->folderNum != DRAFT_MODEL_NUM) {
		date_time_str = ui_get_message_date_time_string(headerinfo->sent_date);
		ui_add_data_field_label_to_table(table,
						 _("mcen_fi_message_properties_sent"),
						 date_time_str, left, right, &top, &bottom);
		g_free(date_time_str);
		date_time_str = NULL;
	}
	ui_add_data_field_label_to_table(table, _("mcen_fi_message_properties_to"),
					 headerinfo->to, left, right, &top, &bottom);
	if (headerinfo->cc != NULL) {
		ui_add_data_field_label_to_table(table,
						 _("mcen_fi_message_properties_cc"),
						 headerinfo->cc, left, right, &top, &bottom);
	}

	if (msg_size > 0) {
		size_str = ui_format_size_value(msg_size);
	} else {
		size_str = g_strdup(_("mcen_va_message_properties_size_noinfo"));
	}
	ui_add_data_field_label_to_table(table, _("mcen_fi_message_properties_size"),
					 size_str, left, right, &top, &bottom);
	g_free(size_str);

	close_button = gtk_button_new_with_mnemonic(_("mcen_bd_message_properties"));
	gtk_widget_show(close_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(prop_dialog), close_button, GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS(close_button, GTK_CAN_DEFAULT);

	if (ossohelp_exists(HELP_ID_MESSAGEPROPERTIES) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) prop_dialog);
		g_signal_connect(G_OBJECT(prop_dialog), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_MESSAGEPROPERTIES);
	}
	g_signal_connect(G_OBJECT(prop_dialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), close_button);
	gtk_widget_show(GTK_WIDGET(prop_dialog));
	response_id = gtk_dialog_run(GTK_DIALOG(prop_dialog));

	gtk_widget_destroy(GTK_WIDGET(prop_dialog));
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/*
This is signal handler function, which will be invoked on message details in
main menu.
@param win pointer to the widget on which this function is called.
@param data pointer to user data. 
@return void.
*/
void ui_on_message_properties_clicked(GtkWidget * win, gpointer data)
{
	HeaderInfo *headerinfo = NULL;
	MsgHeader *msgheader = NULL;
	GSList *msg_id_list = NULL;
	GSList *msg_header_list = NULL;
	GSList *node = NULL;
	gint msg_size = 0;
	GtkTreeModel *list_model = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check if no messages are present in list view */
	list_model =
	    gtk_tree_view_get_model(GTK_TREE_VIEW(email_main_view->widgets->messageListView));
	if (list_model == NULL) {
		ULOG_DEBUG_F("No model set for current folder view!");
		return;
	}

	/*Check if marking mode is ON */
	if (email_main_view->tags->marking_mode) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_select_oneitem_only"));
		ULOG_DEBUG_F("Command not available in marking mode");
		return;
	}

	headerinfo = g_new0(HeaderInfo, 1);

	if (headerinfo == NULL) {
		ULOG_CRIT_F("[%s][%d] : Memory allocation failure!\n", __FUNCTION__, __LINE__);
		return;
	}
	headerinfo->folderNum = email_main_view->folder_manager->currently_displayed_model;
	headerinfo->rowNum = email_main_view->message_handler->currently_selected_row;

	ui_get_row_information(headerinfo);
	if (headerinfo->fully_downloaded == TRUE || headerinfo->body_downloaded == TRUE) {
		msg_id_list = g_slist_append(msg_id_list, headerinfo->msgid);
		msg_header_list = ui_dbus_get_msgheader_list(msg_id_list);
		node = g_slist_nth(msg_header_list, 0);
		if (node != NULL) {
			msgheader = (MsgHeader *) node->data;
			if (msgheader != NULL) {
				msg_size = msgheader->size;
				headerinfo->sent_date = msgheader->date_t;
				headerinfo->recieve_date = msgheader->rx_date;
				headerinfo->creation_date = msgheader->mtime;
			}
		} else {
			g_slist_free(msg_id_list);
			g_slist_free(msg_header_list);
			get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
			return;
		}
		g_slist_free(msg_id_list);
		g_slist_free(msg_header_list);
	} else {
		ULOG_DEBUG("[%s][%d] : Download flags are FALSE\n", __FUNCTION__, __LINE__);
		msg_size = 0;
	}

	ui_create_message_properties_dialog(headerinfo, msg_size);
	ui_freeHeaderInfo(headerinfo);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/* end of file */
