/**
 * @file ui_interface.c
 *
 * This file contains the implementation for main view module.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#define DBUS_API_SUBJECT_TO_CHANGE


#include<dbus/dbus.h>
#include<dbus/dbus-glib.h>
#include<dbus/dbus-glib-lowlevel.h>

/*AL header files*/
#include "al_callbacks.h"
#include "utils.h"
#include "folder.h"
#include "smtp_errorcodes.h"
#include "pop3_msgs.h"
#include "imap.h"
#include "defs.h"
#include "engine_errno.h"
#include "smime_security.h"
#include "al_support.h"
#include "email_gconf.h"
/*UI_DBUS header files*/
#include "email_mgr.h"
#include "dbus_utils.h"

/*Module specific header files*/
#include "ui_common.h"
#include "editview.h"
#include "ui_al_callbacks.h"
#include "ui_interface.h"
#include "ui_error_codes.h"

/*Folder Management header files*/
#include "folder_mgmt.h"
#include "message_handler.h"
#include "list_view.h"

/*Account Management header files*/
#include "ui_account_store.h"
#include "accountMgmt_callback.h"
#include "accountMgmt.h"


/*Signature management header files*/
#include "enablesignatures_callbacks.h"


/*Editor-Viewer header files*/
#include "callback.h"
#include "editcallback.h"
#include  "security.h"
#include "email_clipboard.h"

#include "dbus_utils_def.h"
#include "osso-helplib.h"

#include "contact_utils.h"
#include "email_contacts.h"

/*Global variables*/

/* This global variable is freed in ui_cleanup_memory function*/
MainViewInfo *email_main_view = NULL;
/* Variable to store OSSO Context for UI process */
osso_context_t *ui_context;
/* Flag to indicate whether start-up is complete */
gboolean startup_complete = FALSE;
/* Flag to indicate if external events are to be ignored.
   This is needed when UI is busy with listview updation */
gboolean ignore_ext_events = FALSE;
/*context sensitive popup menu for send recieve*/
static GtkWidget *send_recv_popup_menu = NULL;

extern gboolean fired_on_note;
static void ui_on_toolbar1_activate(GtkWidget * widget, gpointer data);
static void ui_on_hide_folders1_activate(GtkWidget * widget, gpointer data);
static void ui_on_tap_and_hold_folder_popup(GtkWidget * menu, gint * x,
					    gint * y, gboolean * push_in, GtkWidget * widget);
static void ui_on_tap_and_hold_message_popup(GtkWidget * menu, gint * x,
					     gint * y, gboolean * push_in, GtkWidget * widget);
static void ui_update_folder_popmenu_specific_widgets(gint modelNum, gboolean is_subfolder);
static void osso_email_autosave_cb(gpointer data);
static gboolean ui_get_outbox_msg_currently_being_sent(gchar ** msg_id);
static void ui_on_fullscreen_toolbar1_activate(GtkWidget * widget, gpointer data);
static void close_viewer_progress(void);
static void free_retrmsgid_info(RetrMsgData * rtr_msgid_data);
/*
 *
 *  This function is added for auto save functionality.
 */
static void osso_email_autosave_cb(gpointer data)
{
	ui_store_marked_msgs();
	ui_store_current_state_info();
	return;
}

/**
 * This is a helper function to get  UI context from libosso
 *
 * @param  void
 * @return void
 */

void get_ui_context()
{
	ui_context = osso_initialize(APP_NAME, APP_VERSION, TRUE, NULL);
	osso_application_set_autosave_cb(ui_context, osso_email_autosave_cb, NULL);
}

/**
This is a helper function to close UI application when disconnect signal is
received from email engine.

@param	void
@return void
*/
void ui_close_application(void)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
	ui_exit(0);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}


/**
This is a helper function to close UI application in normal way.

@param	signal	Integer value of the signal received
@return void
*/
void ui_exit(int signal)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	ULOG_DEBUG_F("Received signal is %d. Cleaning up global memory!\n", signal);
	ui_cleanup_memory();

	if (gnome_vfs_initialized() == TRUE) {
		gnome_vfs_shutdown();
	}

	gtk_clipboard_set_can_store(gtk_clipboard_get(GDK_NONE), NULL, 0);
	gtk_clipboard_store(gtk_clipboard_get(GDK_NONE));

	if (gtk_main_level() > 0) {
		gtk_main_quit();
	}
	osso_deinitialize(ui_context);
	shutdown_addressbook();
	LOG_CLOSE();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	exit(0);
}

/**
This is a helper function that is called to free up global memory when closing
the UI application.

@param	void
@return void
*/
void ui_cleanup_memory(void)
{
	GSList *node = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	ui_free_folder_menu_items_list();

	/*destroy the context sensitive popup send recieve menu */
	if (send_recv_popup_menu)
		gtk_widget_destroy(send_recv_popup_menu);


	/*Destroy main window, this will destroy all internal widgets, menus and
	   toolbar items */
	gtk_widget_destroy(GTK_WIDGET(app));

	/*Clean up folder list stores */
	for (node = email_main_view->sorted_model_list; node != NULL; node = node->next) {
		if (node->data != NULL) {
			gtk_list_store_clear(GTK_LIST_STORE
					     (gtk_tree_model_sort_get_model
					      (GTK_TREE_MODEL_SORT(node->data))));
			g_object_unref(node->data);
		}
	}
	g_slist_free(email_main_view->sorted_model_list);

	gtk_tree_store_clear(GTK_TREE_STORE(email_main_view->folder_manager->folder_view_model));
	g_object_unref(email_main_view->folder_manager->folder_view_model);

	/*Free Main view structure and its internals */
	g_free(email_main_view->widgets);
	g_free(email_main_view->menu);
	g_free(email_main_view->tools);
	g_free(email_main_view->message_popup);
	g_free(email_main_view->folder_popup);
	g_free(email_main_view->tags->reply_indicator);
	g_free(email_main_view->tags->fwd_indicator);
	g_free(email_main_view->tags);
	g_free(email_main_view->folder_manager->searched_folder_name);
	g_free(email_main_view->folder_manager->parent_folder_name);
	g_free(email_main_view->folder_manager);
	g_free(email_main_view->message_handler->current_draft_subject);
	g_free(email_main_view->message_handler->msg_to_be_selected_after_del_opr);
	g_free(email_main_view->message_handler);

	g_slist_foreach(email_main_view->account_manager->configured_accounts, (GFunc) g_free,
			NULL);
	g_slist_free(email_main_view->account_manager->configured_accounts);
	g_free(email_main_view->account_manager->default_account);
	g_free(email_main_view->account_manager->current_sendrec_account);
	g_free(email_main_view->account_manager->error_info1);
	g_free(email_main_view->account_manager->error_info2);
	g_free(email_main_view->account_manager->alias_name);
	g_free(email_main_view->account_manager);

	if (email_main_view->undo->message_id_list != NULL) {
		g_slist_foreach(email_main_view->undo->message_id_list, (GFunc) g_free, NULL);
		g_slist_free(email_main_view->undo->message_id_list);
	}
	g_free(email_main_view->undo);


	CHECK_PARAM((email_main_view->icons->inbox_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->inbox_icon));
	CHECK_PARAM((email_main_view->icons->outbox_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->outbox_icon));
	CHECK_PARAM((email_main_view->icons->drafts_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->drafts_icon));
	CHECK_PARAM((email_main_view->icons->sent_folder_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->sent_folder_icon));
	CHECK_PARAM((email_main_view->icons->user_folder_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->user_folder_icon));
	CHECK_PARAM((email_main_view->icons->mmc_archive_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->mmc_archive_icon));
	CHECK_PARAM((email_main_view->icons->device_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->device_icon));

	CHECK_PARAM((email_main_view->icons->hdr_low_unread_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->hdr_low_unread_icon));
	CHECK_PARAM((email_main_view->icons->hdr_high_unread_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->hdr_high_unread_icon));
	CHECK_PARAM((email_main_view->icons->hdr_unread_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->hdr_unread_icon));
	CHECK_PARAM((email_main_view->icons->hdr_low_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->hdr_low_icon));
	CHECK_PARAM((email_main_view->icons->hdr_high_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->hdr_high_icon));
	CHECK_PARAM((email_main_view->icons->hdr_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->hdr_icon));

	CHECK_PARAM((email_main_view->icons->hdr_signed_low_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->hdr_signed_low_icon));
	CHECK_PARAM((email_main_view->icons->hdr_signed_high_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->hdr_signed_high_icon));
	CHECK_PARAM((email_main_view->icons->hdr_signed_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->hdr_signed_icon));
	CHECK_PARAM((email_main_view->icons->hdr_unread_signed_low_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->hdr_unread_signed_low_icon));
	CHECK_PARAM((email_main_view->icons->hdr_unread_signed_high_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->hdr_unread_signed_high_icon));
	CHECK_PARAM((email_main_view->icons->hdr_unread_signed_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->hdr_unread_signed_icon));

	CHECK_PARAM((email_main_view->icons->hdr_encrypted_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->hdr_encrypted_icon));
	CHECK_PARAM((email_main_view->icons->hdr_encrypted_high_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->hdr_encrypted_high_icon));
	CHECK_PARAM((email_main_view->icons->hdr_encrypted_low_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->hdr_encrypted_low_icon));
	CHECK_PARAM((email_main_view->icons->hdr_unread_encrypted_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->hdr_unread_encrypted_icon));
	CHECK_PARAM((email_main_view->icons->hdr_unread_encrypted_high_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->hdr_unread_encrypted_high_icon));
	CHECK_PARAM((email_main_view->icons->hdr_unread_encrypted_low_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->hdr_unread_encrypted_low_icon));

	CHECK_PARAM((email_main_view->icons->mail_low_unread_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->mail_low_unread_icon));
	CHECK_PARAM((email_main_view->icons->mail_high_unread_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_high_unread_icon));
	CHECK_PARAM((email_main_view->icons->mail_unread_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_unread_icon));
	CHECK_PARAM((email_main_view->icons->mail_low_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_low_icon));
	CHECK_PARAM((email_main_view->icons->mail_high_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_high_icon));
	CHECK_PARAM((email_main_view->icons->mail_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_icon));

	CHECK_PARAM((email_main_view->icons->mail_signed_unread_low_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_signed_unread_low_icon));
	CHECK_PARAM((email_main_view->icons->mail_signed_unread_high_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_signed_unread_high_icon));
	CHECK_PARAM((email_main_view->icons->mail_signed_unread_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_signed_unread_icon));
	CHECK_PARAM((email_main_view->icons->mail_signed_read_low_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_signed_read_low_icon));
	CHECK_PARAM((email_main_view->icons->mail_signed_read_high_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_signed_read_high_icon));
	CHECK_PARAM((email_main_view->icons->mail_signed_read_icon != NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_signed_read_icon));

	CHECK_PARAM((email_main_view->icons->mail_encrypted_unread_low_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_encrypted_unread_low_icon));
	CHECK_PARAM((email_main_view->icons->mail_encrypted_unread_high_icon !=
		     NULL), "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_encrypted_unread_high_icon));
	CHECK_PARAM((email_main_view->icons->mail_encrypted_unread_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_encrypted_unread_icon));
	CHECK_PARAM((email_main_view->icons->mail_encrypted_read_high_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_encrypted_read_high_icon));
	CHECK_PARAM((email_main_view->icons->mail_encrypted_read_low_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_encrypted_read_low_icon));
	CHECK_PARAM((email_main_view->icons->mail_encrypted_read_icon != NULL),
		    "Unref the icon",
		    g_object_unref(email_main_view->icons->mail_encrypted_read_icon));

	CHECK_PARAM((email_main_view->icons->attachment_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->attachment_icon));
	CHECK_PARAM((email_main_view->icons->check_msg_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->check_msg_icon));
	CHECK_PARAM((email_main_view->icons->dustbin_icon != NULL),
		    "Unref the icon", g_object_unref(email_main_view->icons->dustbin_icon));

	g_free(email_main_view->icons);

	g_slist_free(email_main_view->marking_mode_widgets);
	g_slist_free(email_main_view->account_menu_list);

	/*Free email state objects */
	ui_free_email_state_info();
	ui_free_email_persistent_info();

	/*Deinitalize certman applet */
	if (startup_complete) {
		certmanui_deinit();
	}

	g_free(email_main_view);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/*Helper Functions*/

/**
This is a helper function to show dimming notes for some of the menu items
based on the context. Two different types of dimming notes are shown -
(a) If there are no messages available in current list view.
(b) If marking mode is on due to marking of one or more messages.

@param	widget	Pointer to the widget on which this function is called
@param	data	Pointer to the user data (not used here)
@return void
*/
static void ui_insensitive_command_accessed(GtkWidget * widget, gpointer data)
{
	GtkTreeSelection *selection = NULL;
	GtkTreeModel *model = NULL;
	gint num_of_msgs_in_cur_view = -1;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((widget == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->menu == NULL), "Input param check failed", return);

	model =
	    ui_get_sorted_folder_model(email_main_view->folder_manager->currently_displayed_model);
	num_of_msgs_in_cur_view = gtk_tree_model_iter_n_children(model, NULL);

	/*if message is being sent dont show dimming note */
	if (widget == email_main_view->message_popup->send ||
	    widget == email_main_view->menu->message_send ||
	    widget == email_main_view->tools->send) {
		if ((email_main_view->folder_manager->currently_displayed_model == OUTBOX_MODEL_NUM)
		    && email_main_view->tags->progress_bar_available
		    && email_main_view->tags->current_mode == SENDING) {
			return;
		}
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_send"));

	}

	selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->messageListView));

	if ((gtk_tree_selection_count_selected_rows(selection) > 1) && num_of_msgs_in_cur_view > 0) {
		if (widget == email_main_view->menu->message_details) {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_select_oneitem_only"));
		} else {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_select_one_message"));
		}
	} else if (widget == email_main_view->menu->message_forward
		   || widget == email_main_view->tools->forward) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_fw"));
	} else if (widget == email_main_view->menu->message_reply
		   || widget == email_main_view->menu->message_reply_to_all
		   || widget == email_main_view->tools->reply
		   || widget == email_main_view->tools->reply_to_all) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_reply"));
	} else if (widget == email_main_view->menu->message_resend
		   || widget == email_main_view->tools->resend) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_resend"));
	} else if (widget == email_main_view->message_popup->open
		   || widget == email_main_view->menu->message_open) {
		if ((email_main_view->folder_manager->currently_displayed_model ==
		     OUTBOX_MODEL_NUM) && email_main_view->tags->progress_bar_available) {
			/*Dont show a not when message is being sent */
			return;
		} else {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_open"));
		}
	} else if (widget == email_main_view->menu->message_details) {
		gtk_infoprint((GtkWindow *) app, _("mail_ib_no_details_inbox"));
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function to show dimming notes for cut and move to operations
based on the context. Two different types of dimming notes are shown -
(a) If there are no messages available in current list view.
(b) If marking mode is on due to marking of one or more messages.

@param  widget  Pointer to the widget on which this function is called
@param  data    Pointer to the user data (not used here)
@return void
*/
static void ui_move_to_cut_insensitive_command_accessed(GtkWidget * widget, gpointer data)
{
	GtkTreeModel *model = NULL;
	GtkTreeView *view = NULL;
	gint num_of_msgs = -1;
	GtkTreeSelection *selection = NULL;
	gboolean retval = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((widget == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->menu == NULL), "Input param check failed", return);

	model =
	    ui_get_sorted_folder_model(email_main_view->folder_manager->currently_displayed_model);
	num_of_msgs = gtk_tree_model_iter_n_children(model, NULL);

	if (widget == email_main_view->menu->edit_cut) {
		/*Fix for #25174.Unable to move messages while being sent info note
		   is displayed when CUT option is selected for a mail that is being sent.
		   Instead Unable to Cut should be displayed. */
		if (num_of_msgs == 0) {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_cut"));
		} else if (email_main_view->folder_manager->currently_displayed_model ==
			   OUTBOX_MODEL_NUM) {
			retval = ui_get_outbox_msg_currently_being_sent(NULL);
			view = GTK_TREE_VIEW(email_main_view->widgets->messageListView);
			selection = gtk_tree_view_get_selection(view);
			num_of_msgs = gtk_tree_selection_count_selected_rows(selection);
			if ((retval) && (num_of_msgs == 1)) {
				gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_cut"));
			}
		}
	} else
	    if (widget == email_main_view->menu->edit_move_to
		|| widget == email_main_view->tools->move_to
		|| widget == email_main_view->message_popup->move_to) {
		if (num_of_msgs == 0) {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_move"));
		} else if (email_main_view->folder_manager->currently_displayed_model ==
			   OUTBOX_MODEL_NUM) {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_cut_mess"));
		}
	}
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when a account sub-menu is clicked under
Send & Receive main menu.

@param	widget	Pointer to the send_receive_account sub-menu widget on which
		this function is called
@param	data	Pointer to the character string containing account name
@return void
*/
static void ui_on_send_receive_account_activate(GtkWidget * widget, gpointer data)
{
	gchar *account_selected = NULL;
	gboolean user_confirmation = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	ULOG_DEBUG_F("data = %s\n", (gchar *) data);

	if (data == NULL) {
		/*Check if there are no accounts configured */
		if (email_main_view->account_manager->configured_accounts == NULL) {
			user_confirmation =
			    get_user_confirmation(_("mcen_nc_no_email_acnts_defined"),
						  _("mcen_bd_define"), NULL, CONFIRM_DIALOG);

			/*Return if a new account has not been successfully added */
			if (user_confirmation) {
				ui_on_new_e_mail_account1_activate(GTK_WIDGET(app), NULL);
			}
			return;
		}
		if (email_main_view->account_manager->default_account == NULL) {
			get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
			return;
		}
		account_selected = g_strdup(email_main_view->account_manager->default_account);
	} else {
		account_selected = g_strdup(data);
	}

	ULOG_DEBUG_F("Selected account name : %s\n", account_selected);

	/*Call AL functions for Send & Receive */
	if ((strncmp(account_selected, _("mcen_me_inbox_sendandreceive_all"),
		     MAX_STR_LEN(account_selected, _("mcen_me_inbox_sendandreceive_all"))) == 0) ||
	    (strncmp(account_selected, _("mcen_me_toolbar_sendreceive_all"),
		     MAX_STR_LEN(account_selected, _("mcen_me_toolbar_sendreceive_all"))) == 0)) {
		ui_dbus_send_recv_mail(NULL);
	} else {
		ui_dbus_send_recv_mail(account_selected);
	}

	g_free(account_selected);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
 This function is used set the set the default account with bold text in the menu.
 @param menu_item is the pointer to the GtkWidget , a menu.
 @param account_name is the const gchar pointer to the account name.

*/
static void show_default_account_with_bold(GtkWidget * item, const gchar * account_name)
{
	gchar *markup = NULL;

	if (!email_main_view->account_manager->default_account) {
		return;
	}
	/*get the account name from the menu label */
	if (!account_name) {
		account_name = (gchar *) gtk_label_get_text(GTK_LABEL(GTK_BIN(item)->child));
	}

	/*check if default if so,show the default account with bold */
	if (ui_compare_utf8_strings(email_main_view->account_manager->default_account,
				    account_name) == 0) {
		markup = g_strdup_printf("<b>%s</b>", account_name);
		gtk_label_set_markup(GTK_LABEL(GTK_BIN(item)->child), markup);
		g_free(markup);
	} else {
		gtk_label_set_text(GTK_LABEL(GTK_BIN(item)->child), account_name);
	}
}

/**
 This function is used set the default account with bold text in the menu.
*/
static void bold_default_account()
{
	if (!email_main_view->account_menu_list)
		return;

	g_slist_foreach(email_main_view->account_menu_list, (GFunc) show_default_account_with_bold,
			NULL);
	return;
}

/**
 This is used to popup the context sensitive menu for the send receive button.
 @param send_recive is the send_recieve button.
 @return void.
*/
static void context_send_recieve_menu_cb(GtkWidget * send_recive)
{
	GtkWidget *menu_item = NULL;
	gint no_accounts = 0;
	gchar *account = NULL;

	/*destroy the popup menu if already created */
	if (send_recv_popup_menu)
		gtk_widget_destroy(send_recv_popup_menu);

	/*if email account are not there or onle one account is there
	 * need not show contect sensive menu*/
	if (!email_main_view->account_manager->configured_accounts ||
	    g_slist_length(email_main_view->account_manager->configured_accounts) == 1)
		return;

	send_recv_popup_menu = gtk_menu_new();

	/*Create menu for "all" & a separator */
	menu_item = gtk_menu_item_new_with_label(_("mcen_me_toolbar_sendreceive_all"));
	gtk_menu_shell_prepend(GTK_MENU_SHELL(send_recv_popup_menu), menu_item);
	g_signal_connect(G_OBJECT(menu_item), "activate",
			 G_CALLBACK(ui_on_send_receive_account_activate),
			 _("mcen_me_toolbar_sendreceive_all"));
	menu_item = gtk_menu_item_new();
	gtk_menu_shell_prepend(GTK_MENU_SHELL(send_recv_popup_menu), menu_item);

	/*create menu for default account with bold text */
	menu_item = gtk_menu_item_new_with_label(email_main_view->account_manager->default_account);

	/*Show the default account in bold */
	account = g_strdup_printf(_("mcen_me_toolbar_sendreceive_default"),
				  email_main_view->account_manager->default_account);
	show_default_account_with_bold(menu_item, account);
	g_free(account);
	account = NULL;

	gtk_menu_shell_prepend(GTK_MENU_SHELL(send_recv_popup_menu), menu_item);
	g_signal_connect(G_OBJECT(menu_item), "activate",
			 G_CALLBACK(ui_on_send_receive_account_activate),
			 email_main_view->account_manager->default_account);

	/*Append all the remaining configured accounts */
	no_accounts = g_slist_length(email_main_view->account_manager->configured_accounts);
	no_accounts--;
	while (no_accounts != -1) {

		/*Check if the account being appened is the default account.
		 * if so skip to next as default is already appened.*/
		if (ui_compare_utf8_strings(email_main_view->account_manager->default_account,
					    (gchar
					     *) (g_slist_nth_data(email_main_view->account_manager->
								  configured_accounts,
								  no_accounts)))) {
			account = g_strdup_printf(_("mcen_me_toolbar_sendreceive_mailbox_n"),
						  (gchar
						   *) (g_slist_nth_data(email_main_view->
									account_manager->
									configured_accounts,
									no_accounts)));
			menu_item = gtk_menu_item_new_with_label(account);
			g_free(account);
			account = NULL;
			gtk_menu_shell_prepend(GTK_MENU_SHELL(send_recv_popup_menu), menu_item);
			g_signal_connect(G_OBJECT(menu_item), "activate",
					 G_CALLBACK(ui_on_send_receive_account_activate),
					 g_slist_nth_data(email_main_view->account_manager->
							  configured_accounts, no_accounts));
		}
		no_accounts--;
	}
	gtk_widget_show_all(send_recv_popup_menu);
	gtk_menu_popup(GTK_MENU(send_recv_popup_menu), NULL, NULL, NULL, NULL, 0,
		       gtk_get_current_event_time());
	return;
}




/**
This is a signal handler function used when 'Send & receive' menu is 
clicked in insensitive mode.

@param	widget	Pointer to the widget on which this function is called
@param	data	Pointer to the user data (not used here)
@return void
*/
static void ui_on_send_receive_insensitive_click(GtkWidget * widget, gpointer data)
{
	gboolean user_confirmation = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((widget == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->account_manager == NULL), "Input param check failed", return);

	/*Collapse the appview menu if required */
	if (GTK_WIDGET_VISIBLE(hildon_appview_get_menu(appview))) {
		_hildon_appview_toggle_menu(appview, GDK_CURRENT_TIME);
	}

	/*Check if there are no accounts configured */
	if (email_main_view->account_manager->configured_accounts == NULL) {
		user_confirmation =
		    get_user_confirmation(_("mcen_nc_no_email_acnts_defined"),
					  _("mcen_bd_define"), NULL, CONFIRM_DIALOG);

		/*Return if a new account has not been successfully added */
		if (user_confirmation) {
			ui_on_new_e_mail_account1_activate(GTK_WIDGET(app), NULL);
		}
	} else {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_already_retrieving"));
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to add a new account sub-menu under Send &
Receive main menu. Note that sub-menu should be created only when there
are more than one accounts. If there is only one account clicking on the
'Send & Receive' main menu should activate the operation on default account.

@param  accName  Pointer to the character string containing account name
@param  app_init Flag indicating whether UI initialization is happening
@return void
*/
static void ui_send_receive_menu_add_account(gchar * accName, gboolean app_init)
{
	GtkWidget *item = NULL;
	gint num_of_accounts = -1;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((accName == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->account_manager->configured_accounts == NULL),
		    "Input param check failed", return);

	num_of_accounts = g_slist_length(email_main_view->account_manager->configured_accounts);

	if (num_of_accounts == 1) {
		gtk_widget_set_sensitive(email_main_view->menu->email_send_receive, TRUE);
		gtk_widget_set_sensitive(email_main_view->tools->send_receive, TRUE);
		if (email_main_view->tags->send_receive_sig_handler != 0) {
			g_signal_handler_disconnect(email_main_view->menu->email_send_receive,
						    email_main_view->tags->
						    send_receive_sig_handler);
			email_main_view->tags->send_receive_sig_handler = 0;
		}
		email_main_view->tags->send_receive_sig_handler =
		    g_signal_connect((gpointer) email_main_view->menu->email_send_receive,
				     "activate", G_CALLBACK(ui_on_send_receive_account_activate),
				     accName);
		ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
		return;
	}

	if (email_main_view->menu->email_send_receive_menu == NULL) {
		ui_prepare_send_receive_menu();
		if (app_init) {
			ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
			return;
		}
	}

	item = gtk_menu_item_new_with_label(accName);
	gtk_widget_show(item);
	gtk_container_add(GTK_CONTAINER(email_main_view->menu->email_send_receive_menu), item);
	g_signal_connect((gpointer) item, "activate",
			 G_CALLBACK(ui_on_send_receive_account_activate), accName);

	/*Append the account menu items into global list */
	email_main_view->account_menu_list =
	    g_slist_append(email_main_view->account_menu_list, item);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to remove a account sub-menu under Send &
Receive main menu. Note that sub-menu should be available only when there
are more than one accounts. If there is only one account clicking on the
'Send & Receive' main menu should activate the operation on default account.

@param	accSeqNum	Sequence number of the account sub-menu to be removed
@return void
*/
static void ui_send_receive_menu_remove_account(guint accSeqNum)
{
	GSList *node = NULL;
	gint num_of_menu_items = -1;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((email_main_view->account_menu_list == NULL), "Input param check failed",
		    return);
	CHECK_PARAM((accSeqNum >= g_slist_length(email_main_view->account_menu_list)),
		    "Input param check failed", return);

	num_of_menu_items = g_slist_length(email_main_view->account_menu_list);
	ULOG_DEBUG_F("Num of elements in menu list before deletion = %d\n", num_of_menu_items);

	if (num_of_menu_items == 2) {
		gtk_widget_destroy(email_main_view->menu->email_send_receive_menu);
		email_main_view->menu->email_send_receive_menu = NULL;
		g_slist_free(email_main_view->account_menu_list);
		email_main_view->account_menu_list = NULL;
	} else {
		node = g_slist_nth(email_main_view->account_menu_list, accSeqNum);
		if (node != NULL && node->data != NULL) {
			email_main_view->account_menu_list =
			    g_slist_remove_link(email_main_view->account_menu_list, node);
			gtk_widget_destroy(GTK_WIDGET(node->data));
			ULOG_DEBUG_F
			    ("Num of elements in menu list after deletion = %d\n",
			     g_slist_length(email_main_view->account_menu_list));
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to prepare sub-menus under Send & Receive main
menu during initialization of the UI. Note that sub-menu should be available
only when there are more than one accounts. If there is only one account
clicking on the 'Send & Receive' main menu should activate the operation on
default account.

@param  void
@return void
*/
void ui_prepare_send_receive_menu(void)
{
	GtkWidget *all = NULL;
	GtkWidget *first_account_menu = NULL;
	GtkWidget *separator = NULL;
	gchar *first_account_name = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	if (g_slist_length(email_main_view->account_manager->configured_accounts) < 2) {
		return;
	}

	/*If required, disconnect signal handler attached to Send_Receive
	   main menu */
	gtk_widget_set_sensitive(email_main_view->menu->email_send_receive, TRUE);
	gtk_widget_set_sensitive(email_main_view->tools->send_receive, TRUE);
	if (email_main_view->tags->send_receive_sig_handler != 0) {
		g_signal_handler_disconnect(email_main_view->menu->email_send_receive,
					    email_main_view->tags->send_receive_sig_handler);
		email_main_view->tags->send_receive_sig_handler = 0;
	}

	email_main_view->menu->email_send_receive_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM
				  (email_main_view->menu->email_send_receive),
				  email_main_view->menu->email_send_receive_menu);

	all = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_sendandreceive_all"));
	gtk_widget_show(all);
	gtk_container_add(GTK_CONTAINER(email_main_view->menu->email_send_receive_menu), all);

	g_signal_connect((gpointer) all, "activate",
			 G_CALLBACK(ui_on_send_receive_account_activate),
			 _("mcen_me_inbox_sendandreceive_all"));

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(email_main_view->menu->email_send_receive_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	first_account_name = (gchar *) email_main_view->account_manager->configured_accounts->data;
	first_account_menu = gtk_menu_item_new_with_label(first_account_name);
	gtk_widget_show(first_account_menu);
	gtk_container_add(GTK_CONTAINER
			  (email_main_view->menu->email_send_receive_menu), first_account_menu);
	g_signal_connect((gpointer) first_account_menu, "activate",
			 G_CALLBACK(ui_on_send_receive_account_activate), first_account_name);
	email_main_view->account_menu_list =
	    g_slist_append(email_main_view->account_menu_list, first_account_menu);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This function will be called by Account Management module whenever a new
account is created. Note that sub-menu should be available only when there
are more than one accounts. If there is only one account clicking on the
'Send & Receive' main menu should activate the operation on default account.

@param	accName 	Pointer to the character string containing account name
@return void
*/
void ui_add_mail_account(const gchar * accName)
{
	gchar *new_account = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (accName == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Append account name into global account_names_list */
	new_account = g_strdup(accName);
	email_main_view->account_manager->configured_accounts =
	    g_slist_append(email_main_view->account_manager->configured_accounts, new_account);

	/*Add a corresponding menu item in Send & Receive menu */
	ui_send_receive_menu_add_account(new_account, FALSE);

	/*If required, disconnect signal handler attached to Send_Receive
	   main menu */
	if (g_slist_length(email_main_view->account_manager->configured_accounts) == 2) {
		gtk_widget_set_sensitive(email_main_view->menu->email_send_receive, TRUE);
		gtk_widget_set_sensitive(email_main_view->tools->send_receive, TRUE);
		if (email_main_view->tags->send_receive_sig_handler != 0) {
			g_signal_handler_disconnect(email_main_view->menu->
						    email_send_receive,
						    email_main_view->tags->
						    send_receive_sig_handler);
			email_main_view->tags->send_receive_sig_handler = 0;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This function will be called by Account Management module whenever an
account is removed. Note that sub-menu should be available only when there
are more than one accounts. If there is only one account clicking on the
'Send & Receive' main menu should activate the operation on default account.

@param	accName 	Pointer to the character string containing account name
@return void
*/
void ui_remove_mail_account(gchar * accName)
{
	GSList *node = NULL;
	guint acc_seq_num = 0;
	gboolean acc_found = FALSE;
	gint num_of_accounts = -1;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (accName == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Get the sequence number of the account menu */
	node = email_main_view->account_manager->configured_accounts;
	while (node != NULL && node->data != NULL) {
		if (strncmp(node->data, accName, MAX_STR_LEN(node->data, accName)) == 0) {
			acc_found = TRUE;
			ULOG_DEBUG_F("Found account name. Seq num = %d\n", acc_seq_num);
			break;
		}
		node = node->next;
		acc_seq_num++;
	}

	if (acc_found) {
		/*Remove account name from global account_names_list */
		num_of_accounts =
		    g_slist_length(email_main_view->account_manager->configured_accounts);
		ULOG_DEBUG_F("Num of elements in names list before deletion = %d\n",
			     num_of_accounts);
		node =
		    g_slist_nth(email_main_view->account_manager->configured_accounts, acc_seq_num);
		email_main_view->account_manager->configured_accounts =
		    g_slist_remove_link(email_main_view->account_manager->configured_accounts,
					node);
		g_free(node->data);
		num_of_accounts =
		    g_slist_length(email_main_view->account_manager->configured_accounts);
		ULOG_DEBUG_F("Num of elements in names list after deletion = %d\n",
			     num_of_accounts);

		/*Remove the corresponding menu item from Send & Receive menu */
		ui_send_receive_menu_remove_account(acc_seq_num);

		if (num_of_accounts == 1) {
			if (email_main_view->tags->send_receive_sig_handler != 0) {
				g_signal_handler_disconnect(email_main_view->menu->
							    email_send_receive,
							    email_main_view->tags->
							    send_receive_sig_handler);
				email_main_view->tags->send_receive_sig_handler = 0;
			}
			email_main_view->tags->send_receive_sig_handler =
			    g_signal_connect((gpointer) email_main_view->menu->
					     email_send_receive, "activate",
					     G_CALLBACK
					     (ui_on_send_receive_account_activate),
					     email_main_view->account_manager->configured_accounts->
					     data);
		} else if (num_of_accounts == 0) {
			gtk_widget_set_sensitive(email_main_view->menu->email_send_receive, FALSE);
			gtk_widget_set_sensitive(email_main_view->tools->send_receive, FALSE);
		}

		/*Refresh required folders */
		ui_refresh_required_folders();
	} else {
		ULOG_DEBUG_F("Account does not exist!\n");
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function which checks if any GTK events are pending,
and if so runs the GTK_MAIN_ITERATION loop to flush out the pending events.

@param  void
@return void
*/
void ui_flush_gtk_events(void)
{
	while (gtk_events_pending()) {
		gtk_main_iteration();
	}

	return;
}

/**
This is a helper function for initializing configured accounts during start-up.
In limited init, only account settings are read and validated. In full init,
Send-receive menu items handling is also done.

@param	full_init  Boolean flag representing full initialization.
@return boolean    TRUE --> success, FALSE --> failure
*/
static gboolean ui_initialize_account_settings(gboolean full_init)
{
	GSList *node = NULL;
	gchar *acc_name = NULL;
	GSList *acc_list = NULL;
	PrefsAccount *account_settings;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	acc_list = ui_dbus_get_account_list();
	if (acc_list == NULL) {
		ULOG_DEBUG_F("Account list is NULL!\n");
		if (full_init) {
			gtk_widget_set_sensitive(email_main_view->menu->email_send_receive, FALSE);
			gtk_widget_set_sensitive(email_main_view->tools->send_receive, FALSE);
		}
	} else {
		email_main_view->account_manager->configured_accounts = acc_list;
		ui_init_account_store();
	}

	for (node = email_main_view->account_manager->configured_accounts;
	     node != NULL; node = node->next) {
		if (node->data == NULL) {
			ULOG_DEBUG_F("Invalid account list received! Initialization failed!\n");
			get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
			return FALSE;
		}
		ULOG_DEBUG_F("Got an account name during initialization : %s\n",
			     (gchar *) node->data);

		if (full_init) {
			ui_send_receive_menu_add_account(node->data, TRUE);
		}

		/*Store the default account name */
		acc_name = node->data;
		account_settings = ui_get_account_detail(acc_name);
		if (account_settings == NULL) {
			ULOG_DEBUG_F
			    ("Unable to get settings for account '%s'! Initialization failed!\n",
			     acc_name);
			get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
			return FALSE;
		} else if (!ui_accMgmt_validate_account(account_settings)) {
			ULOG_DEBUG_F("Invalid settings for account %s! Initialization failed!\n",
				     acc_name);
			get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
			ui_free_prefs_account_structure(account_settings);
			return FALSE;
		} else if (account_settings->user_settings.is_default == TRUE) {
			ULOG_DEBUG_F("Default account is '%s'\n", acc_name);
			email_main_view->account_manager->default_account = g_strdup(acc_name);
		}
		ui_free_prefs_account_structure(account_settings);
	}

	if (email_main_view->account_manager->default_account == NULL
	    && email_main_view->account_manager->configured_accounts != NULL) {
		ULOG_DEBUG_F("No account is set as default! Making the first one as default!");
		email_main_view->account_manager->default_account =
		    g_strdup(email_main_view->account_manager->configured_accounts->data);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return TRUE;
}

/**
Helper function to add a folder into the folder_view during application
initialization.

@param  folder  Folder object containing folder info
@return void
*/
static void ui_add_folder_to_view(const Folder * folder)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((folder == NULL), "Input param check failed\n", return);
	CHECK_PARAM((folder->name == NULL), "Input param check failed\n", return);

	if (ui_compare_utf8_strings(folder->name, INBOX_DIR) == 0) {
		ui_add_new_folder(GTK_TREE_STORE
				  (email_main_view->folder_manager->
				   folder_view_model), INBOX_DIR, FALSE, -1);
	} else if (ui_compare_utf8_strings(folder->name, OUTBOX_DIR) == 0) {
		ui_add_new_folder(GTK_TREE_STORE
				  (email_main_view->folder_manager->
				   folder_view_model), OUTBOX_DIR, FALSE, folder->total);
	} else if (ui_compare_utf8_strings(folder->name, DRAFT_DIR) == 0) {
		ui_add_new_folder(GTK_TREE_STORE
				  (email_main_view->folder_manager->
				   folder_view_model), DRAFT_DIR, FALSE, folder->total);
	} else if (ui_compare_utf8_strings(folder->name, SENT_DIR) == 0) {
		ui_add_new_folder(GTK_TREE_STORE
				  (email_main_view->folder_manager->
				   folder_view_model), SENT_DIR, FALSE, folder->unread);
	}

	else if (ui_compare_utf8_strings(folder->name, ARCHIVE) == 0) {
		ui_add_new_folder(GTK_TREE_STORE
				  (email_main_view->folder_manager->
				   folder_view_model), ARCHIVE, FALSE, folder->unread);
	} else {
		ui_add_new_folder(GTK_TREE_STORE
				  (email_main_view->folder_manager->
				   folder_view_model), folder->name, TRUE, folder->unread);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function for initializing and populating folders during
start-up.

@param	void
@return boolean    TRUE --> success, FALSE --> failure
*/
static gboolean ui_initialize_folder_settings()
{
	GSList *node = NULL, *folderList = NULL;
	Folder *folder = NULL;
	PopupFolderMenu *folder_menu_structure = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Get the folder list */
	ui_dbus_folder_get_list(&folderList);
	if (folderList == NULL) {
		ULOG_DEBUG_F("Folder list is NULL! Initialization failed!\n");
		ui_set_msg_cb(ui_dbus_get_engine_errno());
		return FALSE;
	}

	/*Mandatory folders are added already. Add remaining folders */
	for (node = folderList; node != NULL; node = node->next) {
		folder = (Folder *) node->data;
		if (folder == NULL || folder->name == NULL) {
			ULOG_DEBUG_F("Invalid folder list received! Initialization failed!\n");
			get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
			return FALSE;
		}
		ULOG_DEBUG_F("Got a folder name during initialization : %s\n", folder->name);

		ui_add_folder_to_view(folder);

		g_free(folder->name);
		g_free(folder->path);
		g_free(folder);
	}
	g_slist_free(folderList);

	/*Select Inbox menu item */
	folder_menu_structure = ui_get_folder_activated_structure(NULL, INBOX_DIR);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(folder_menu_structure->menu_item), TRUE);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return TRUE;
}

/**
This is a helper function called by the main UI for initializing configured
accounts, folders and populating folders during start-up.

@param  startup_mode  Mode of application start-up
@return boolean       TRUE --> success, FALSE --> failure
*/
gboolean ui_initialize(gint startup_mode)
{
	gboolean full_init = TRUE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Initialize UI Dbus which will register with engine */
	ui_dbus_init();
	ui_dbus_is_ready();
	ui_dbus_is_engine_ready();


	/*Get the configured account settings */
	if (ui_initialize_account_settings(full_init) == FALSE) {
		ULOG_DEBUG_F("Account settings initialization failed!\n");
		return FALSE;
	}

	/*Get the folder settings */
	if (ui_initialize_folder_settings() == FALSE) {
		ULOG_DEBUG_F("Folder settings initialization failed!\n");
		return FALSE;
	}

	if (ui_read_persistent_settings() == TRUE) {
		ui_apply_persistent_settings();
	}

	/*No need to populate all the folders, populate just Inbox and Outbox */
	ui_populate_folder(INBOX_DIR, INBOX_MODEL_NUM);
	ui_populate_folder(OUTBOX_DIR, OUTBOX_MODEL_NUM);

	ui_set_folderview_title(INBOX_DIR);
	ui_add_folder_name_with_unread_unsent_mails(INBOX_MODEL_NUM);
	ui_add_folder_name_with_unread_unsent_mails(OUTBOX_MODEL_NUM);

	ui_flush_gtk_events();

	/*Apply persistent & previous state settings if required */
	if (ui_get_previous_state_info() == TRUE) {
		email_main_view->tags->prev_oom_status = ui_get_oom_signal_status();
		/*Initialize previous state settings, only if application was
		   closed due to oom-signal during last life-cycle */
		if (email_main_view->tags->prev_oom_status == TRUE) {
			ui_initialize_previous_state_settings();
		}
		ui_set_oom_signal_status();
		/*Unset previous email state, its no longer needed now */
		ui_dbus_statesave_write(NULL);
	}
	/*Initialize certman applet */
	if (ui_context != NULL) {
		if (!certmanui_init(ui_context)) {
			ULOG_DEBUG_F("Cannot initialize certman ui");
			return FALSE;
		}
	}

	startup_complete = TRUE;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return TRUE;
}

/**
This is a helper function to perform send-receive operation on default account
during application start-up. Send-receive is required only if application was
closed normally during previous life-cycle.

@param  void
@return void
*/
void ui_do_send_recv(void)
{
	gboolean user_confirmation = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->account_manager == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->tags == NULL), "Input param check failed", return);

	if (email_main_view->tags->prev_oom_status == FALSE) {
		/*Check if there are no accounts configured */
		if (email_main_view->account_manager->configured_accounts == NULL) {
			user_confirmation =
			    get_user_confirmation(_("mcen_nc_no_email_acnts_defined"),
						  _("mcen_bd_define"), NULL, CONFIRM_DIALOG);

			/*Return if a new account has not been successfully added */
			if (user_confirmation) {
				ui_on_new_e_mail_account1_activate(GTK_WIDGET(app), NULL);
			}
		} else if (email_main_view->account_manager->default_account != NULL) {
			ui_dbus_send_recv_mail(email_main_view->account_manager->default_account);
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to populate a folder with the residing message
headers.

@param	folderName	Pointer to the character string containing account name
@param	folderNum	Sequence number of the folder storage model to be
			populated
@return void
*/
void ui_populate_folder(const gchar * folderName, guint folderNum)
{
	GSList *node = NULL;
	GSList *msg_headers = NULL;
	gint count = 0;
	gboolean more_hdrs_available = TRUE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (folderName == NULL || folderNum >= g_slist_length(email_main_view->sorted_model_list)) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Reset folder message count as it is not needed any more */
	gtk_tree_model_foreach(email_main_view->folder_manager->folder_view_model,
			       ui_reset_folder_msg_count, &folderNum);

	/*Do not handle external events while we are busy populating */
	ignore_ext_events = TRUE;

	/*Call AL function to obtain message headers */
	while (more_hdrs_available) {
		msg_headers = ui_dbus_populate_folder(folderName, count);
		if (msg_headers == NULL) {
			ULOG_DEBUG_F("Msg Header list during populate_folder is NULL!\n");
			more_hdrs_available = FALSE;
			continue;
		}

		node = msg_headers;
		while (node) {
			MsgHeader *header = (MsgHeader *) (node->data);
			if (header == NULL) {
				ULOG_DEBUG_F("Invalid header obtained while populating \
		       folder - %s\n", folderName);
			} else {
				/*Append a new row to current folder view */
				if (ui_add_listview_header(header, folderNum, FALSE) == FALSE) {
					ULOG_DEBUG_F("Header info is not valid!\n");
				}
				ui_free_msg_header(header);
			}

			node = node->next;
		}
		g_slist_free(msg_headers);

		count += NUM_MAILS;
		/*Set unsorted model to the view */
		ui_set_folder_model_to_msgview(folderNum, FALSE);
		/*Flush Gtk events */
		ui_flush_gtk_events();
	}

	/*We are done populating */
	ignore_ext_events = FALSE;

	if (email_main_view->folder_manager->currently_displayed_model == folderNum) {
		/*Set sorted model to the view */
		if (count > 0) {
			gtk_tree_view_set_model(GTK_TREE_VIEW
						(email_main_view->widgets->messageListView),
						ui_get_sorted_folder_model(folderNum));
		}

		ui_update_folder_specific_widgets(folderNum);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a 'GtkTreeModelForeachFunc' function used to populate each folder
present in the folder view.

@param	model		Pointer to the folder view storage model
@param	path		Pointer to the GtkTreePath of current row
@param	iter		Pointer to the GtkTreeIter of current row
@param	data		Pointer to the user data (not used here)
@return boolean 	TRUE --> if model is NULL (don't call again)
			FALSE --> Call again with next row
*/
gboolean
ui_populate_all_folders(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
	gchar *folder_name = NULL;
	gchar *parent_folder = NULL;
	gchar *folder_path = NULL;
	guint modelNum = 0;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (model == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return TRUE;
	}

	gtk_tree_model_get(model, iter, COL_FOLDER_NAME, &folder_name,
			   COL_MODEL_SEQ_NUM, &modelNum, COL_PARENT_NAME, &parent_folder, -1);
	ULOG_DEBUG_F("Folder = %s\tModelNum = %d\n", folder_name, modelNum);

	if (parent_folder == NULL) {
		folder_path = g_strdup(folder_name);
	} else {
		folder_path = g_strdup_printf("%s/%s", parent_folder, folder_name);
		g_free(parent_folder);
	}

	if (folder_path != NULL) {
		ui_populate_folder(folder_path, modelNum);
		g_free(folder_path);
		g_free(folder_name);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return FALSE;
}

/**
This is a signal handler function used when message tree view obtains
focus. It calls 'ui_messageview_selection_changed_cb' to update required
menu items.

@param  widget  Pointer to the widget on which this function is called
@param  event   Pointer to the focus event structure
@param  data    Pointer to the user data (not used)
@return boolean Always FALSE so that this signal is propogated further.
*/
static gboolean ui_on_messageview_focussed(GtkWidget * widget, GdkEventFocus * event, gpointer data)
{
	GtkTreeSelection *selection = NULL;

	selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->messageListView));
	ui_messageview_selection_changed_cb(selection, NULL);

	if (widget != NULL) {
		email_main_view->tags->messageview_has_focus = TRUE;
		email_main_view->tags->folderview_has_focus = FALSE;
	}

	return FALSE;
}

/**
This is a signal handler function used when folder tree view obtains
focus. It dims/undims required menu items.

@param  widget  Pointer to the widget on which this function is called
@param  event   Pointer to the focus event structure
@param  data    Pointer to the user data (not used)
@return boolean Always FALSE so that this signal is propogated further.
*/
static gboolean ui_on_folderview_focussed(GtkWidget * widget, GdkEventFocus * event, gpointer data)
{

	if (widget != NULL) {
		email_main_view->tags->folderview_has_focus = TRUE;
		email_main_view->tags->messageview_has_focus = FALSE;
	}

	return FALSE;
}

/**
This is a helper function used to populate INBOX folder.

@param	void
@return void
*/
void ui_populate_inbox(void)
{
	GtkTreeModel *model = NULL;
	GtkTreeSelection *selection = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	model =
	    gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT
					  (email_main_view->sorted_model_list->data));
	if (model == NULL) {
		ULOG_DEBUG_F("Model missing for Inbox folder!\n");
		return;
	}

	/*Unlink the tree model if Inbox folder is currently displayed */
	if (email_main_view->folder_manager->currently_displayed_model == INBOX_MODEL_NUM) {
		gtk_tree_view_set_model(GTK_TREE_VIEW(email_main_view->widgets->messageListView),
					NULL);
	}

	/*Clean Inbox */
	gtk_list_store_clear(GTK_LIST_STORE(model));

	/*Populate Inbox */
	ui_populate_folder(INBOX_DIR, INBOX_MODEL_NUM);

	/*Refresh the current view if it is Inbox */
	if (email_main_view->folder_manager->currently_displayed_model == INBOX_MODEL_NUM) {
		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW
							(email_main_view->widgets->folderView));
		ui_folderview_selection_changed_cb(selection, NULL);

		if (gtk_tree_model_iter_n_children(model, NULL) != 0) {
			gtk_widget_grab_focus(email_main_view->widgets->messageListView);
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to top already opened message view, given its
message-id.

@param  msg_id          Pointer to the message id string
@return boolean         TRUE --> success, FALSE --> failure
*/
gboolean ui_top_opened_message(const gchar * msg_id)
{
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;
	gboolean window_found = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	if (msg_id == NULL || email_main_view == NULL
	    || email_main_view->open_windows == NULL || email_main_view->tags == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return window_found;
	}

	for (node = email_main_view->open_windows; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) node->data;
		if (open_window == NULL || open_window->msgid == NULL
		    || open_window->window_ptr == NULL) {
			ULOG_DEBUG_F("Invalid data in open windows list");
		} else if (strncmp(open_window->msgid, msg_id,
				   MAX_STR_LEN(open_window->msgid, msg_id)) == 0) {
			hildon_app_set_appview(app, HILDON_APPVIEW(open_window->window_ptr));
			email_main_view->tags->main_view_topped = FALSE;
			email_main_view->open_windows =
			    g_slist_remove_link(email_main_view->open_windows, node);
			email_main_view->open_windows =
			    g_slist_concat(node, email_main_view->open_windows);
			ULOG_DEBUG_F("Message with msgid = '%s' topped\n", msg_id);
			window_found = TRUE;
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return window_found;
}

/**
This is a helper function to fire "Include original in reply" confirmation
note the the user.

@param  msgid   Pointer to the message-id string
@return boolean TRUE --> positive confirmation, FALSE --> otherwise
*/
static gboolean ui_get_original_in_reply_confirmation(const gchar * msgid)
{
	gboolean confirmation = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msgid == NULL),
		    "ui_get_original_in_reply_confirmation : Input param check failed",
		    return FALSE);

	confirmation =
	    get_user_confirmation(_("emev_nc_include_original"),
				  _("emev_bd_include_original_yes"),
				  _("emev_bd_include_original_no"), CONFIRM_DIALOG);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return confirmation;
}

/**
This is a helper function used to invoke editor window from main UI in
various modes.

@param	view  Pointer to the GtkTreeView of the current folder view
@param	mode  Mode of invoking editor --> New Email, Reply, ReplyToAll, Forward
@return void
*/
static void ui_open_editor(GtkTreeView * view, gint mode)
{
	GtkWidget *composer = NULL;
	GtkTreePath *path = NULL;
	gint *indices = NULL;
	GtkTreeModel *model = NULL, *child_model = NULL;
	GtkTreeSelection *selection = NULL;
	HeaderInfo *header = NULL;
	gchar *receiverid = NULL, *senderid = NULL, *password = NULL;
	gboolean msg_to_be_downloaded = FALSE;
	guint certid = 0, msgid_hash = 0;
	gint cert_state = VALID;
	GList *paths_list = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((view == NULL), "ui_open_editor : Input param check failed!", return);

	model = gtk_tree_view_get_model(view);
	CHECK_PARAM((model == NULL), "ui_open_editor : No model set for current folder view!",
		    return);

	selection = gtk_tree_view_get_selection(view);
	if (gtk_tree_selection_count_selected_rows(selection) > 1) {
		/*if more than one row or no row is selected.we return */
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_select_one_message"));
		return;
	}


	paths_list = gtk_tree_selection_get_selected_rows(selection, NULL);
	if (paths_list == NULL) {
		/*if  no row is selected.we return */
		return;
	}


	/*If a row is selected get its row seq number and open the message */
	path = (GtkTreePath *) g_list_nth_data(paths_list, 0);
	indices = gtk_tree_path_get_indices(path);
	if (indices == NULL) {
		g_list_foreach(paths_list, (GFunc) gtk_tree_path_free, NULL);
		g_list_free(paths_list);
		return;
	}

	/*Get the current row info */
	header = g_new0(HeaderInfo, 1);
	header->folderNum = email_main_view->folder_manager->currently_displayed_model;
	header->rowNum = indices[0];
	ui_get_row_information(header);

	g_list_foreach(paths_list, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(paths_list);

	/*Check if message is already opened from Drafts or Outbox
	   If so simply top the corresponding window */
	if ((mode == OUTBOX_MAIL || mode == DRAFT_MAIL) && header->in_use) {
		if (ui_top_opened_message(header->msgid) == FALSE) {
			ULOG_DEBUG_F("Unable to top opened message!\n");
		}
		ui_freeHeaderInfo(header);
		return;
	}

	/*Show 'Message not local' confirmation if required */
	header->download_attachments = TRUE;
	if (mode == FORWARD_FROM_FOLDER && !header->fully_downloaded) {
		if (get_user_confirmation(_("mcen_nc_get_msg_txt"),
					  _("mcen_bd_retrieve"), NULL, CONFIRM_DIALOG) == FALSE) {
			/*No need to forward the mail, just return */
			ui_freeHeaderInfo(header);
			return;
		}
		msg_to_be_downloaded = TRUE;
	}

	/*If replying to a partially downloaded message,
	   Show "Include original message in reply confirmation note" if necessary */
	header->original_in_reply = ui_get_original_in_reply_setting(header->msgid);
	if (mode == REPLY_FROM_FOLDER || mode == REPLY_ALL_FROM_FOLDER) {
		if (!header->fully_downloaded && header->msgType == HEADERS
		    && header->original_in_reply) {
			header->original_in_reply =
			    ui_get_original_in_reply_confirmation(header->msgid);
			msg_to_be_downloaded = header->original_in_reply;
		}
	}

	/*Check if message has to be downloaded first */
	if (!header->fully_downloaded && header->protocol == PROTO_POP3 && msg_to_be_downloaded) {
		receiverid = get_receiversid(header->msgid);
		senderid = ui_parse_from_address(header->from);
		if (header->smime_type != SMIME_NONE
		    && header->smime_type != SMIME_INTERNAL && header->smime_type != SMIME_SIGN) {
			certid = get_private_certificate(receiverid, &cert_state);
			if (certid == 0) {
				get_user_confirmation(_("mcen_in_smime_cannot_decrypt"),
						      NULL, NULL, INFO_BANNER);
				email_main_view->message_handler->
				    set_action_after_retrieval = NO_ACTION_AFTER_RETRIEVAL;
				return;
			}

			password = get_certificate_password(certid);
			if (password == NULL)
				return;
		}
		/*For pop3 headers only message, this call is made with callback
		   flag set to FALSE. It will return NULL, hence nothing to free. */
		ui_dbus_recv_msg_body(header->msgid, FALSE, FALSE,
				      senderid, receiverid, header->smime_type,
				      (header->read_status), certid, password, TRUE, FALSE);

		/*Set the 'action after retrieval' tag for this message */
		email_main_view->message_handler->set_action_after_retrieval =
		    NO_ACTION_AFTER_RETRIEVAL;
		switch (mode) {
		case REPLY_FROM_FOLDER:
			email_main_view->message_handler->set_action_after_retrieval =
			    REPLY_AFTER_RETRIEVAL;
			break;

		case FORWARD_FROM_FOLDER:
			email_main_view->message_handler->set_action_after_retrieval =
			    FORWARD_AFTER_RETRIEVAL;
			break;

		case REPLY_ALL_FROM_FOLDER:
			email_main_view->message_handler->set_action_after_retrieval =
			    REPLY_ALL_AFTER_RETRIEVAL;
			break;
		default:
			ULOG_DEBUG_F("No mode is selected!\n");
			break;
		}

		child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(model));
		msgid_hash = g_str_hash(header->msgid);
		gtk_tree_model_foreach(child_model, ui_update_action_after_retrieval, &msgid_hash);

		g_free(receiverid);
		g_free(senderid);
		g_free(password);
		ui_freeHeaderInfo(header);
		ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
		return;
	}
	/*Call the composer window */
	composer = GTK_WIDGET(ui_editor_create(header, NULL, mode));
	if (composer == NULL) {
		ULOG_DEBUG_F("Creation of Email Editor failed!\n");
	} else {
		hildon_app_set_appview(app, (HildonAppView *) composer);
		gtk_widget_show(composer);
		email_main_view->tags->main_view_topped = FALSE;
	}

	/*Free allocated memory */
	ui_freeHeaderInfo(header);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;

}

/**
This is helper function to update main view toolbar items for a
given folder. If required it updates the toolbar to 'download'
state, for displaying message send-receive progress info.

@param  modelNum  Sequence number of the folder storage model
@return void
*/
static void ui_update_folder_specific_tool_items(guint model_num)
{
	GtkTreeModel *list_model = NULL;
	gboolean sensitivity = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	list_model = ui_get_sorted_folder_model(model_num);

	/*Dim required tool bar buttons if the model is empty */
	if (list_model != NULL && gtk_tree_model_iter_n_children(list_model, NULL) != 0) {
		sensitivity = TRUE;
	}
	gtk_widget_set_sensitive(email_main_view->tools->reply, sensitivity);
	gtk_widget_set_sensitive(email_main_view->tools->reply_to_all, sensitivity);
	gtk_widget_set_sensitive(email_main_view->tools->forward, sensitivity);
	gtk_widget_set_sensitive(email_main_view->tools->delete_msg, sensitivity);
	gtk_widget_set_sensitive(email_main_view->tools->send, sensitivity);
	gtk_widget_set_sensitive(email_main_view->tools->resend, sensitivity);
	gtk_widget_set_sensitive(email_main_view->tools->cancel_sending, sensitivity);
	gtk_widget_set_sensitive(email_main_view->tools->move_to, sensitivity);

	/*First hide all the tool buttons */
	gtk_widget_hide(email_main_view->tools->send);
	gtk_widget_hide(email_main_view->tools->send_space);
	gtk_widget_hide(email_main_view->tools->reply);
	gtk_widget_hide(email_main_view->tools->reply_space);
	gtk_widget_hide(email_main_view->tools->reply_to_all);
	gtk_widget_hide(email_main_view->tools->reply_to_all_space);
	gtk_widget_hide(email_main_view->tools->forward);
	gtk_widget_hide(email_main_view->tools->forward_space);
	gtk_widget_hide(email_main_view->tools->delete_msg);
	gtk_widget_hide(email_main_view->tools->delete_msg_space);
	gtk_widget_hide(email_main_view->tools->resend);
	gtk_widget_hide(email_main_view->tools->resend_space);
	gtk_widget_hide(email_main_view->tools->cancel_sending);
	gtk_widget_hide(email_main_view->tools->cancel_sending_space);
	gtk_widget_hide(email_main_view->tools->move_to);
	gtk_widget_hide(email_main_view->tools->move_to_space);
	gtk_widget_hide(email_main_view->tools->send_receive);
	gtk_widget_hide(email_main_view->tools->send_receive_space);
	gtk_widget_hide(email_main_view->tools->separator);
	gtk_widget_hide(email_main_view->tools->progress_item);
	gtk_widget_hide(email_main_view->tools->progress_bar_button);

	/*Show-Hide appropriate toolbar buttons
	   depending upon whether the toolar needs to be in 'download' state */
	if (email_main_view->tags->progress_bar_available) {
		gtk_widget_show(email_main_view->tools->separator);
		gtk_widget_show_all(email_main_view->tools->progress_bar_button);
		gtk_widget_show_all(email_main_view->tools->progress_item);
	} else {
		gtk_widget_show(email_main_view->tools->send_receive);
	}
	switch (model_num) {
	case OUTBOX_MODEL_NUM:	/*OUTBOX folder */
		{
			gtk_widget_show(email_main_view->tools->send);
			gtk_widget_show(email_main_view->tools->send_space);
			gtk_widget_show(email_main_view->tools->cancel_sending);
			gtk_widget_show(email_main_view->tools->cancel_sending_space);
			gtk_widget_show(email_main_view->tools->move_to);
			gtk_widget_show(email_main_view->tools->move_to_space);
			if (!email_main_view->tags->progress_bar_available) {
				gtk_widget_show(email_main_view->tools->delete_msg);
				gtk_widget_show(email_main_view->tools->delete_msg_space);
			}
			ui_on_messageview_focussed(NULL, NULL, NULL);
			break;
		}
	case SENT_MODEL_NUM:	/*SENT folder */
		{
			gtk_widget_show(email_main_view->tools->forward);
			gtk_widget_show(email_main_view->tools->forward_space);
			gtk_widget_show(email_main_view->tools->resend);
			gtk_widget_show(email_main_view->tools->resend_space);
			gtk_widget_show(email_main_view->tools->move_to);
			gtk_widget_show(email_main_view->tools->move_to_space);
			if (!email_main_view->tags->progress_bar_available) {
				gtk_widget_show(email_main_view->tools->delete_msg);
				gtk_widget_show(email_main_view->tools->delete_msg_space);
			}
			break;
		}
	case DRAFT_MODEL_NUM:	/*DRAFTS folder */
		{
			gtk_widget_show(email_main_view->tools->move_to);
			gtk_widget_show(email_main_view->tools->move_to_space);
			gtk_widget_show(email_main_view->tools->delete_msg);
			gtk_widget_show(email_main_view->tools->delete_msg_space);
			break;
		}
	default:		/*INBOX folder or any user created folder */
		{
			gtk_widget_show(email_main_view->tools->reply);
			gtk_widget_show(email_main_view->tools->reply_space);
			gtk_widget_show(email_main_view->tools->reply_to_all);
			gtk_widget_show(email_main_view->tools->reply_to_all_space);
			gtk_widget_show(email_main_view->tools->forward);
			gtk_widget_show(email_main_view->tools->forward_space);
			if (!email_main_view->tags->progress_bar_available) {
				gtk_widget_show(email_main_view->tools->delete_msg);
				gtk_widget_show(email_main_view->tools->delete_msg_space);
				gtk_widget_show(email_main_view->tools->move_to);
				gtk_widget_show(email_main_view->tools->move_to_space);
			}
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a 'GtkTreeModelForeachFunc' function used to check the current selected
folder is a sub folder.

@param  model           Pointer to the folder view storage model
@param  path            Pointer to the GtkTreePath of current row
@param  iter            Pointer to the GtkTreeIter of current row
@param  data            Pointer to the user data
@return boolean         TRUE --> If sub folder is found;Do not call again.
                        FALSE --> Call again on next row.
*/
static gboolean
ui_is_sub_folder(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
	gint *model_num = NULL;
	gint model_seq_num = -1;
	GtkTreeIter parent_iter;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	CHECK_PARAM((model == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((iter == NULL), "Input param check failed", return TRUE);
	CHECK_PARAM((data == NULL), "Input param check failed", return TRUE);

	model_num = (gint *) data;
	gtk_tree_model_get(model, iter, COL_MODEL_SEQ_NUM, &model_seq_num, -1);
	if ((*model_num) == model_seq_num) {
		if (gtk_tree_model_iter_parent(model, &parent_iter, iter) == TRUE) {
			email_main_view->tags->is_sub_folder = TRUE;
			return TRUE;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return FALSE;
}

/**
This is helper function to update main view menu and toolbar items for a
given folder.

@param  modelNum  Sequence number of the folder storage model
@return void
*/
void ui_update_folder_specific_widgets(guint modelNum)
{
	GtkTreeModel *list_model = NULL;
	GtkTreeModel *folder_model = NULL;
	gint num_of_msgs = 0;
	gboolean sensitivity = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	list_model = ui_get_sorted_folder_model(modelNum);

	folder_model = email_main_view->folder_manager->folder_view_model;
	email_main_view->tags->is_sub_folder = FALSE;
	gtk_tree_model_foreach(folder_model,
			       (GtkTreeModelForeachFunc) ui_is_sub_folder, (gpointer) & modelNum);
	if(list_model)
		num_of_msgs = gtk_tree_model_iter_n_children(list_model, NULL);
	
	/*Dim required menus if the model is empty */
	if (list_model != NULL && num_of_msgs != 0) {
		sensitivity = TRUE;
	}
	gtk_widget_set_sensitive(email_main_view->menu->message_reply, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->message_reply_to_all, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->message_forward, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->edit_move_to, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->message_send, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->message_resend, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->message_cancel_sending, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->message_delete, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->message_details, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->message_open, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->folders_new_folder, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->folders_folder_settings, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->folders_details, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->edit_cut, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->edit_copy, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->view_sort, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->email_purge_attachments, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->edit_select_mark_all, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->edit_select_clear_all_marks, sensitivity);

	if (num_of_msgs == 0) {
		gtk_widget_set_sensitive(email_main_view->menu->email_retrieve_mail_contents,
					 FALSE);
	}

	/*Update the appropriate menu items */
	switch (modelNum) {
	case OUTBOX_MODEL_NUM:	/*OUTBOX folder */
		{
			gtk_widget_show(email_main_view->menu->message_send);
			gtk_widget_show(email_main_view->menu->message_cancel_sending);
			gtk_widget_hide(email_main_view->menu->message_reply);
			gtk_widget_hide(email_main_view->menu->message_reply_to_all);
			gtk_widget_hide(email_main_view->menu->message_resend);
			gtk_widget_hide(email_main_view->menu->message_forward);

			gtk_widget_set_sensitive(email_main_view->menu->edit_paste, FALSE);

			gtk_widget_set_sensitive(email_main_view->menu->folders_new_folder, FALSE);
			gtk_widget_set_sensitive(email_main_view->menu->folders_folder_settings,
						 FALSE);
			gtk_widget_set_sensitive(email_main_view->menu->folders_details, TRUE);

			gtk_widget_set_sensitive(email_main_view->menu->email_purge_attachments,
						 FALSE);
			ui_on_messageview_focussed(NULL, NULL, NULL);

			break;
		}
	case SENT_MODEL_NUM:	/*SENT folder */
		{
			gtk_widget_show(email_main_view->menu->message_forward);
			gtk_widget_hide(email_main_view->menu->message_reply);
			gtk_widget_hide(email_main_view->menu->message_reply_to_all);
			gtk_widget_hide(email_main_view->menu->message_send);
			gtk_widget_hide(email_main_view->menu->message_cancel_sending);
			gtk_widget_show(email_main_view->menu->message_resend);

			gtk_widget_set_sensitive(email_main_view->menu->edit_paste, FALSE);

			gtk_widget_set_sensitive(email_main_view->menu->folders_new_folder, FALSE);
			gtk_widget_set_sensitive(email_main_view->menu->folders_folder_settings,
						 TRUE);
			gtk_widget_set_sensitive(email_main_view->menu->folders_details, TRUE);

			if (num_of_msgs > 0) {
				gtk_widget_set_sensitive(email_main_view->menu->
							 email_purge_attachments, TRUE);
			}

			break;
		}
	case DRAFT_MODEL_NUM:	/*DRAFTS folder */
		{
			gtk_widget_show(email_main_view->menu->message_send);
			gtk_widget_hide(email_main_view->menu->message_reply);
			gtk_widget_hide(email_main_view->menu->message_reply_to_all);
			gtk_widget_hide(email_main_view->menu->message_forward);
			gtk_widget_hide(email_main_view->menu->message_resend);
			gtk_widget_hide(email_main_view->menu->message_cancel_sending);

			gtk_widget_set_sensitive(email_main_view->menu->edit_paste, FALSE);

			gtk_widget_set_sensitive(email_main_view->menu->folders_new_folder, FALSE);
			gtk_widget_set_sensitive(email_main_view->menu->folders_folder_settings,
						 FALSE);
			gtk_widget_set_sensitive(email_main_view->menu->folders_details, TRUE);

			gtk_widget_set_sensitive(email_main_view->menu->email_purge_attachments,
						 FALSE);

			break;
		}
	default:		/*INBOX folder or any user created folder */
		{
			gtk_widget_show(email_main_view->menu->message_reply);
			gtk_widget_show(email_main_view->menu->message_reply_to_all);
			gtk_widget_show(email_main_view->menu->message_forward);
			gtk_widget_hide(email_main_view->menu->message_resend);
			gtk_widget_hide(email_main_view->menu->message_send);
			gtk_widget_hide(email_main_view->menu->message_cancel_sending);

			gtk_widget_set_sensitive(email_main_view->menu->edit_paste, TRUE);

			gtk_widget_set_sensitive(email_main_view->menu->folders_folder_settings,
						 FALSE);
			gtk_widget_set_sensitive(email_main_view->menu->folders_details, TRUE);

			if (email_main_view->tags->is_sub_folder == TRUE
			    || modelNum == email_main_view->tags->archive_model_num) {
				gtk_widget_set_sensitive(email_main_view->menu->folders_new_folder,
							 FALSE);
			} else {
				gtk_widget_set_sensitive(email_main_view->menu->folders_new_folder,
							 TRUE);
			}

			break;
		}
	}

	/*Update tool bar items */
	ui_update_folder_specific_tool_items(modelNum);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function to send selected mails for following operations -
sending draft mails, sending outbox mails, resending sent mails.
It get the message-ids of all marked messages in current view and fire
appropriate call to engine based on the operation type. If none are marked,
it sends the currently selected message. If no messages are selected it
simply returns.

@param  operation_type  Type of operation - as described above
@param  view            Pointer to the current message list view
@return void
*/
static void ui_send_selected_mails(guint operation_type, GtkTreeView * view)
{
	GSList *msg_id_list = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeSelection *selection = NULL;
	gint in_use_msgs_count = 0;
	gboolean user_confirmation = FALSE;
	gchar *notification = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (view == NULL || email_main_view == NULL || email_main_view->account_manager == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Check if there are no accounts configured
	   And launch new account configuration dialog if required */
	if (email_main_view->account_manager->configured_accounts == NULL) {
		user_confirmation =
		    get_user_confirmation(_("mcen_nc_no_email_acnts_defined"),
					  _("mcen_bd_define"), NULL, CONFIRM_DIALOG);

		/*Return if a new account has not been successfully added */
		if (user_confirmation) {
			switch (operation_type) {
			case OPR_SEND_OUTBOX_MAILS:
			case OPR_SEND_DRAFT_MAILS:
				email_main_view->account_manager->action_after_acct_creation =
				    SEND_MSGS;
				break;
			case OPR_RESEND_MAILS:
				email_main_view->account_manager->action_after_acct_creation =
				    RESEND_MSGS;
				break;
			}
			ui_on_new_e_mail_account1_activate(GTK_WIDGET(app), NULL);
		} else {
			ui_on_cancel_send_activate(NULL, NULL);
		}
		ULOG_DEBUG_F("No accounts defined yet! Unable to proceed\n");
		return;
	}

	model = gtk_tree_view_get_model(view);
	if (model == NULL) {
		ULOG_DEBUG_F("No storage model for current view!\n");
		return;
	}


	/*Check if atleast one selection is made in listview */
	selection = gtk_tree_view_get_selection(view);
	if (gtk_tree_selection_count_selected_rows(selection) < 1) {
		get_user_confirmation(_("mcen_ib_nothing_to_send"), NULL, NULL, INFO_BANNER);
		ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
		return;
	}


	/*Get the message-ids for all selected messages */
	msg_id_list = ui_get_selected_not_inuse_msgids(view, operation_type, &in_use_msgs_count);


	if (msg_id_list != NULL) {
		/*Call AL function for sending all selected message */
		ui_dbus_send_selected_mails(operation_type, msg_id_list);
		g_slist_foreach(msg_id_list, (GFunc) g_free, NULL);
		g_slist_free(msg_id_list);
	}

	/*Display appropriate notification for unsent messages */
	if (in_use_msgs_count > 0) {
		if (in_use_msgs_count == 1) {
			notification = _("emev_nc_unabletomove_item");
		} else {
			notification = _("emev_bd_unabletomove_items");
		}
		get_user_confirmation(notification, NULL, NULL, INFO_BANNER);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to hide/ unhide the Folder view.

@param	folder_view     Pointer to the folder view widget
@return boolean         Split view mode ON/OFF
*/
gboolean ui_hide_folders(GtkWidget * folder_view, gboolean user_action)
{
	gboolean split_view_mode = FALSE;
	gint pane_width = 0;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	if (folder_view == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return FALSE;
	}

	if (GTK_WIDGET_VISIBLE(folder_view)) {
		/*set the pane position into the persistent settings */
		ui_set_folder_pane_width(email_main_view->widgets->folder_msg_pane);
		gtk_widget_hide(folder_view);
		gtk_widget_show(email_main_view->widgets->message_scrollwindow);
		split_view_mode = FALSE;

		if (!user_action) {
			g_signal_handlers_block_by_func(email_main_view->menu->
							view_folders,
							ui_on_hide_folders1_activate,
							email_main_view->widgets->
							folder_scrollwindow);
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
						       (email_main_view->menu->view_folders),
						       FALSE);
			g_signal_handlers_unblock_by_func(email_main_view->menu->view_folders,
							  ui_on_hide_folders1_activate,
							  email_main_view->widgets->
							  folder_scrollwindow);
		}
	} else {
		gtk_widget_show(folder_view);
		split_view_mode = TRUE;
		/*set the pane position */
		pane_width = ui_get_folder_pane_width();
		gtk_paned_set_position(GTK_PANED(email_main_view->widgets->folder_msg_pane),
				       pane_width);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return split_view_mode;
}

/**
This is a helper function used to hide/ unhide the main view toolbar.

@param	void
@return void
*/
void ui_toggle_toolbar_visibility(gboolean user_action, gboolean show_toolbar)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*If its not user action that means its from presistent settings */
	if (!user_action) {
		email_main_view->tags->show_tool_bar = !(ui_get_toolbar_hidden_status());
		email_main_view->tags->show_fullscreen_tool_bar =
		    !(ui_get_fullscreen_toolbar_hidden_status());

		g_signal_handlers_block_by_func(email_main_view->menu->
						view_show_toolbar_normal_screen,
						ui_on_toolbar1_activate, NULL);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
					       (email_main_view->menu->
						view_show_toolbar_normal_screen),
					       email_main_view->tags->show_tool_bar);
		g_signal_handlers_unblock_by_func(email_main_view->menu->
						  view_show_toolbar_normal_screen,
						  ui_on_toolbar1_activate, NULL);
		g_signal_handlers_block_by_func(email_main_view->menu->
						view_show_toolbar_full_screen,
						ui_on_fullscreen_toolbar1_activate, NULL);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
					       (email_main_view->menu->
						view_show_toolbar_full_screen),
					       email_main_view->tags->show_fullscreen_tool_bar);
		g_signal_handlers_unblock_by_func(email_main_view->menu->
						  view_show_toolbar_full_screen,
						  ui_on_toolbar1_activate, NULL);
		if (show_toolbar == TRUE)	/*means for normal view */
			gtk_widget_hide(email_main_view->tools->toolbar);
		return;

	}

	if (show_toolbar == TRUE)
		gtk_widget_hide(email_main_view->tools->toolbar);
	else
		gtk_widget_show(email_main_view->tools->toolbar);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/*Signal Handling Functions*/

/**
This is a signal handler function used when Hide_Folders menu is clicked.
It will hide/unhide the Folders menu.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_hide_folders1_activate(GtkWidget * widget, gpointer data)
{
	gboolean split_view_mode = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (data == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	split_view_mode = ui_hide_folders(GTK_WIDGET(data), TRUE);

	/*Update UI state object */
	ui_set_split_view_mode_status(split_view_mode);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Toolbar menu is clicked.
It will hide/unhide Tool bar from the current view.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data (not used)
@return void
*/
static void ui_on_toolbar1_activate(GtkWidget * widget, gpointer data)
{
	gboolean toolbar_hidden = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (widget == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	email_main_view->tags->show_tool_bar =
	    gtk_check_menu_item_get_active((GtkCheckMenuItem *) widget);
	if (email_main_view->tags->full_screen_mode == FALSE) {
		ui_toggle_toolbar_visibility(TRUE, !(email_main_view->tags->show_tool_bar));
	}

	/*Update UI state object */
	if (email_main_view->tags->show_tool_bar == FALSE) {
		toolbar_hidden = TRUE;
	}
	ui_set_toolbar_hidden_status(!(email_main_view->tags->show_tool_bar));

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Toolbar menu is clicked.
It will hide/unhide Tool bar from the current view.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data (not used)
@return void
*/
static void ui_on_fullscreen_toolbar1_activate(GtkWidget * widget, gpointer data)
{
	gboolean toolbar_hidden = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (widget == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	email_main_view->tags->show_fullscreen_tool_bar =
	    gtk_check_menu_item_get_active((GtkCheckMenuItem *) widget);
	ULOG_DEBUG_F("the toolbar visibility for fullscreen b4 is %d\n",
		     email_main_view->tags->show_fullscreen_tool_bar);
	if (email_main_view->tags->full_screen_mode != FALSE) {
		ui_toggle_toolbar_visibility(TRUE,
					     !(email_main_view->tags->show_fullscreen_tool_bar));
	}


	/*Update UI state object */
	if (email_main_view->tags->show_fullscreen_tool_bar == FALSE) {
		toolbar_hidden = TRUE;
	}
	email_main_view->tags->show_fullscreen_tool_bar =
	    !email_main_view->tags->show_fullscreen_tool_bar;
	ui_set_fullscreen_toolbar_hidden_status(toolbar_hidden);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function which checks whether clipboard has valid
emails that can be pasted.

@param  orig_folder  Out param for storing the originating folder of the
                     messages to be pasted.
@param  msg_id_list  Out param for storing the list of msg-ids for pasting.
@param  cut_operation_done  Out param for storing if CUT operation was done.
@return boolean  TRUE --> Clipboard has email content, and atleast one of the
                          emails is available for pasting.
                 FALSE --> Otherwise.
*/
gboolean
ui_check_clipboard_has_valid_emails(gchar ** orig_folder,
				    GSList ** msg_id_list, gboolean * cut_operation_done)
{
	gchar *parent_name = NULL, *folder_name = NULL;
	gboolean enable_paste_opr = FALSE;
	GSList *node = NULL;
	gint copy_cut_folder = -1;
	guint msgid_hash = 0;
	GtkTreeModel *cut_copy_folder_model = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((orig_folder == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((msg_id_list == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((cut_operation_done == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->message_handler == NULL),
		    "Input param check failed", return FALSE);

	if (osso_email_clipboard_has_emails(GTK_WIDGET(app)) == TRUE) {
		//Get contents from clipboard
		osso_email_get_list_from_clipboard(GTK_WIDGET(app), orig_folder,
						   msg_id_list, cut_operation_done);
		if (ui_get_folder_names_from_path(*orig_folder, &parent_name, &folder_name) == TRUE) {
			copy_cut_folder = ui_get_folder_model_number(folder_name, parent_name);
			cut_copy_folder_model = ui_get_unsorted_folder_model(copy_cut_folder);
		}

		if (*msg_id_list != NULL || cut_copy_folder_model != NULL) {
			for (node = *msg_id_list; node != NULL; node = node->next) {
				email_main_view->message_handler->is_message_present = FALSE;
				msgid_hash = g_str_hash(node->data);
				gtk_tree_model_foreach(cut_copy_folder_model,
						       ui_is_header_already_present, &msgid_hash);
				if (email_main_view->message_handler->is_message_present == TRUE) {
					ULOG_DEBUG_F("Msg '%s' is available", (gchar *) node->data);
					enable_paste_opr = TRUE;
					break;
				}
			}
		}
	}

	g_free(parent_name);
	g_free(folder_name);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return enable_paste_opr;
}

/**
This is a helper function to check if paste operation is allowed for
the current context. Paste operation is never allowed for Outbox, Drafts
and Sent folders.

@param  void
@return boolean
*/
static gboolean ui_is_paste_operation_allowed()
{
	gint current_folder_num = -1;
	gchar *orig_folder = NULL;
	GSList *msg_id_list = NULL;
	gboolean cut_operation_done = FALSE, opr_allowed = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	current_folder_num = email_main_view->folder_manager->currently_displayed_model;

	if (ui_check_clipboard_has_valid_emails
	    (&orig_folder, &msg_id_list, &cut_operation_done) == TRUE) {
		if (current_folder_num != OUTBOX_MODEL_NUM &&
		    current_folder_num != SENT_MODEL_NUM && current_folder_num != DRAFT_MODEL_NUM) {
			opr_allowed = TRUE;
		}
	}

	if (msg_id_list != NULL) {
		g_slist_foreach(msg_id_list, (GFunc) g_free, NULL);
		g_slist_free(msg_id_list);
	}
	g_free(orig_folder);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return opr_allowed;
}

/**
This is a signal handler function used when Edit menu is clicked. It will
(1)Check the clipboard content type and disable/enable 'Paste' menu.
(2)Check if previous operation is undoable and disable/enable 'Undo' menu.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_edit_activate(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Handle PASTE menu */
	if (ui_is_paste_operation_allowed() == TRUE) {
		gtk_widget_set_sensitive(email_main_view->menu->edit_paste, TRUE);
	} else {
		gtk_widget_set_sensitive(email_main_view->menu->edit_paste, FALSE);
	}

	/*Handle UNDO menu */
	if (email_main_view->undo->operation != OPER_NONE) {
		gtk_widget_set_sensitive(email_main_view->menu->edit_undo, TRUE);
	} else {
		gtk_widget_set_sensitive(email_main_view->menu->edit_undo, FALSE);
	}

	/*Handle UNMARK menu */
	if (email_main_view->tags->marking_mode == FALSE) {
		gtk_widget_set_sensitive(email_main_view->menu->edit_select_clear_all_marks, FALSE);
	} else {
		gtk_widget_set_sensitive(email_main_view->menu->edit_select_clear_all_marks, TRUE);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function called when 'E-mail' menu is clicked.
If marking mode is ON, it analyzes marked messages and enables 'Retrieve
e-mail contents' menu accordingly.

@param	widget	Pointer to the widget on which this function is called
@param	data	Pointer to the user data
@return void
*/
static void ui_on_email_activate(GtkWidget * widget, gpointer data)
{
	gint unretrieved_msgs_count = 0;
	GtkTreeModel *model = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->menu == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->tags == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->menu->email_retrieve_mail_contents == NULL),
		    "Input param check failed", return);

	/*Update appropriate menus */
	model =
	    ui_get_sorted_folder_model(email_main_view->folder_manager->currently_displayed_model);
	CHECK_PARAM((model == NULL), "Storage model is NULL for current folder view", return);

	ui_get_unretrieved_selected_msgs_count((GtkTreeView *) email_main_view->widgets->
					       messageListView, &unretrieved_msgs_count);
	if (unretrieved_msgs_count > 0) {
		gtk_widget_set_sensitive(email_main_view->menu->email_retrieve_mail_contents, TRUE);
	} else {
		gtk_widget_set_sensitive(email_main_view->menu->email_retrieve_mail_contents,
					 FALSE);
	}


	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}



/**
This is a signal handler function used when Copy menu is clicked. It will
store the row information of all the selected messages for further operation.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_copy1_activate(GtkWidget * widget, gpointer data)
{
	GtkTreeView *list_view = NULL;
	GtkTreeModel *list_model = NULL;
	GtkTreeModel *child_model = NULL;
	GtkTreeSelection *selection = NULL;
	gboolean copied = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	/*Check the necessary input and global params */
	if (widget == NULL || email_main_view == NULL
	    || email_main_view->widgets == NULL
	    || email_main_view->widgets->folderView == NULL
	    || email_main_view->widgets->messageListView == NULL
	    || email_main_view->message_handler == NULL
	    || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Get the number of marked rows */
	list_view = GTK_TREE_VIEW(email_main_view->widgets->messageListView);
	list_model = gtk_tree_view_get_model(list_view);
	if (list_model == NULL) {
		ULOG_DEBUG_F("No model set for current folder view!\n");
		return;
	}

	child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(list_model));

	/*Check if atleast one selection is made in listview */
	selection = gtk_tree_view_get_selection(list_view);
	if (gtk_tree_selection_count_selected_rows(selection) < 1) {
		get_user_confirmation(_("mcen_ib_unable_to_copy"), NULL, NULL, INFO_BANNER);
		return;
	}
	
	g_slist_foreach(email_main_view->message_handler->msgs_to_copy,
                                                        (GFunc) g_free, NULL);
        g_slist_free(email_main_view->message_handler->msgs_to_copy);
        email_main_view->message_handler->msgs_to_copy = NULL;

	copied = ui_manage_selected_items((gint) email_main_view->folder_manager->
					  currently_displayed_model, -1, ACTION_COPY, FALSE, NULL);


	/*Show 'copied' banner */
	if (copied) {
		gtk_confirmation_banner_with_icon_name((GtkWindow *) app,
						       _("mail_ib_copied_to_clipboard"),
						       "qgn_note_infoprint");
	}


	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
   This function shows the appropriate confirmation note for copying unfetched mails
   @param unfetched_msg_id_lis is the GSList of the unfetched msgid list.
   @return gboolean TRUE--> if user confirms the retrival
                    FALSE other wise.
*/
gboolean copy_unretrieved_mails_confirmation(const GSList * unfetched_msg_id_list)
{
	gint num_of_msgs = 0;
	gboolean userConfirmation = TRUE;

	if (!unfetched_msg_id_list)
		return userConfirmation;

	num_of_msgs = g_slist_length((GSList *) unfetched_msg_id_list);
	if (num_of_msgs > 1) {
		userConfirmation =
		    get_user_confirmation(_("mcen_nc_get_multi_msg_txt"),
					  _("mcen_bd_retrieve"), NULL, CONFIRM_DIALOG);
	} else {
		userConfirmation =
		    get_user_confirmation(_("mcen_nc_get_msg_txt"), _("mcen_bd_retrieves"),
					  NULL, CONFIRM_DIALOG);
	}

	return userConfirmation;
}

/**
   This function is used to retrieved the mail contents of unfetched mails
   on userconfirmation for copying.
   @param unfetched_msg_id_lis is the GSList of the unfetched msgid list.
   @param foldernum is the gint representing the model number from which the messages are copied.
*/
void retrieve_mail_contents(const GSList * unfetched_msg_id_list, gint foldernum)
{
	GSList *unretrieved_msgids = (GSList *) unfetched_msg_id_list;
        RetrMsgData *retr_msg_data = NULL;
        GSList *retr_list = NULL;

        /*prepare the info for unretrieved msg list */
        while (unretrieved_msgids != NULL) {
                retr_msg_data = g_new0(RetrMsgData, 1);
                retr_msg_data->msgid = g_strdup(unretrieved_msgids->data);
                retr_msg_data->flag = TRUE;
                retr_list = g_slist_append(retr_list, retr_msg_data);
                unretrieved_msgids = unretrieved_msgids->next;
        }
        ui_dbus_retrieve_msgs(retr_list, TRUE);
        g_slist_foreach(retr_list, (GFunc) free_retrmsgid_info, NULL);
        g_slist_free(retr_list);

}

/**
This is a signal handler function used when Paste menu is clicked. It gets the
row information collected during Cut/Copy operation and pastes them into the
current folder.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_paste1_activate(GtkWidget * widget, gpointer data)
{
	GSList *node = NULL;
	GtkTreeView *view = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	/*Check the necessary input and global params */
	if (widget == NULL || email_main_view == NULL
	    || email_main_view->widgets == NULL
	    || email_main_view->widgets->messageListView == NULL
	    || email_main_view->sorted_model_list == NULL
	    || email_main_view->message_handler == NULL
	    || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	view = GTK_TREE_VIEW(email_main_view->widgets->messageListView);

	ui_manage_selected_items(-1,
				 (gint) email_main_view->folder_manager->
				 currently_displayed_model, ACTION_PASTE, FALSE, NULL);

	/*Set the appropriate model to the list view, if not set */
	node =
	    g_slist_nth(email_main_view->sorted_model_list,
			email_main_view->folder_manager->currently_displayed_model);
	if (gtk_tree_view_get_model(view) == NULL
	    && gtk_tree_model_iter_n_children(GTK_TREE_MODEL(node->data), NULL) != 0) {
		gtk_tree_view_set_model(view, GTK_TREE_MODEL(node->data));
		ULOG_DEBUG_F("View is empty. Current model set!\n");
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
 This function is used to clear  the selection.
 Dont know whats the explixt need of clear all. if one taps on any row the selection
 anyways goes .why then clear all?
 
 @param view is the pointer to the GtkTreeView widget.
 @return void.
*/
static void ui_clear_selection(GtkTreeView * view)
{
	GtkTreeSelection *selection = NULL;
	GtkTreeIter iter;
	GtkTreeModel *model = NULL;

	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_unselect_all(selection);

	if (!(model = gtk_tree_view_get_model(view)))
		return;

	if (gtk_tree_model_get_iter_first(model, &iter))
		gtk_tree_selection_select_iter(selection, &iter);

	return;
}

/**
 This function is used to select all the rows.

 @param view is the pointer to the GtkTreeView widget.
 @return void.
*/
static void ui_selection_all(GtkTreeView * view)
{
	GtkTreeSelection *selection = NULL;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_select_all(selection);
	return;
}

/**
This is a signal handler function used when Mark-All menu is clicked. It marks
all the currently selected rows in the current folder view.

@param	widget		Pointer to the widget on which this function is called
@param	view		Pointer to the current list view
@return void
*/
static void ui_on_mark_all1_activate(GtkWidget * widget, GtkTreeView * view)
{
	GtkTreeModel *model = NULL;
	GtkTreeModel *child_model = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (widget == NULL || view == NULL || email_main_view == NULL
	    || email_main_view->menu == NULL
	    || email_main_view->menu->edit_select_clear_all_marks == NULL
	    || email_main_view->tags == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	model = gtk_tree_view_get_model(view);
	if (model == NULL) {
		ULOG_DEBUG_F("No model set for current folder view!\n");
		return;
	}
	child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(model));

	/*Mark all rows */
	ui_selection_all(GTK_TREE_VIEW(email_main_view->widgets->messageListView));

	gtk_widget_set_sensitive(widget, FALSE);
	gtk_widget_set_sensitive(email_main_view->menu->edit_select_clear_all_marks, TRUE);

	/*Set the marking mode */
	if (email_main_view->tags->marking_mode == FALSE) {
		ULOG_DEBUG_F("Invoking marking mode\n");
		email_main_view->tags->marking_mode = TRUE;
		ui_invoke_marking_mode(TRUE);
	}

	/*Update the global state object */
	ui_store_marked_msgs();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Clear-All-Marks menu is clicked.
It unmarks all the currently selected rows in the current folder view.

@param	widget		Pointer to the widget on which this function is called
@param	view		Pointer to the current list view
@return void
*/
static void ui_on_clear_all_marks1_activate(GtkWidget * widget, GtkTreeView * view)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	/*Check the necessary input and global params */
	if (view == NULL || email_main_view == NULL
	    || email_main_view->menu == NULL
	    || email_main_view->menu->edit_select_mark_all == NULL
	    || email_main_view->menu->edit_select_clear_all_marks == NULL
	    || email_main_view->tags == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Clear all marks */
	ui_clear_selection(GTK_TREE_VIEW(email_main_view->widgets->messageListView));

	/*Update menu items */
	gtk_widget_set_sensitive(email_main_view->menu->edit_select_clear_all_marks, FALSE);
	gtk_widget_set_sensitive(email_main_view->menu->edit_select_mark_all, TRUE);

	/*Set the marking mode */
	if (email_main_view->tags->marking_mode == TRUE) {
		ULOG_DEBUG_F("Revoking marking mode\n");
		email_main_view->tags->marking_mode = FALSE;
		ui_invoke_marking_mode(FALSE);

		/*Update the global state object */
		ui_store_marked_msgs();
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to get the message id of the first selected message
that is being sent & unseelct the row.
@param msg_id OUT param containing the Pointer to mesage id which is currently selected 
	and status is being sent.
@return TRUE  If any one message in the currently seelcted messages is
              being sent.
        FALSE None of the message is being sent in the currently selected mesages.
*/
static gboolean ui_get_outbox_msg_currently_being_sent(gchar ** msg_id)
{
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	GtkTreeSelection *selection = NULL;
	gchar *current_msg_id = NULL;
	gboolean retval = FALSE;
	gint outbox_status_bit = -1;

	model = ui_get_sorted_folder_model(OUTBOX_MODEL_NUM);
	retval = gtk_tree_model_get_iter_first(model, &iter);
	CHECK_PARAM((retval == FALSE), "Storage model is empty!", return retval);
	selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->messageListView));
	/* Traverse the model for getting the message id which is marked and status is
	 * being sent */
	do {
		if (gtk_tree_selection_iter_is_selected(selection, &iter)) {
			gtk_tree_model_get(model, &iter, COL_MSG_ID, &current_msg_id, -1);
			outbox_status_bit =
			    g_ascii_digit_value(current_msg_id[strlen(current_msg_id) - 1]);

			if (outbox_status_bit == OUTBOX_STATUS_SENDING) {
				if ((msg_id) != NULL) {
					(*msg_id) = current_msg_id;
					gtk_tree_selection_unselect_iter(selection, &iter);
				}
				return TRUE;
			}
			g_free(current_msg_id);
			current_msg_id = NULL;
		}
	}
	while (gtk_tree_model_iter_next(model, &iter));

	return FALSE;
}

/**
This is helper function, this will be called while cut and move operations
to unselect the selected message which is in sending, if required.
@param void.
@return TRUE  If any one message in the currently marked messages is
              being sent.
        FALSE None of the message is being sent in the currently marked mesages.
*/
gboolean ui_unselect_outbox_msg_being_sent()
{
	gchar *msg_id = NULL;
	gboolean retval = FALSE;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	if (email_main_view->folder_manager->currently_displayed_model == OUTBOX_MODEL_NUM) {
		/* Get the message id which is being sent and marked */
		retval = ui_get_outbox_msg_currently_being_sent(&msg_id);
		if (retval == TRUE) {
			/* Uncheck the mark status of the message which is being sent */
			ui_update_row_tag_in_folder(email_main_view->folder_manager->
						    currently_displayed_model, msg_id,
						    MARK_BOOL_STATUS, FALSE);
			g_free(msg_id);
		}
	}
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return retval;
}

/**
This is a signal handler function used when Cut menu is clicked. It will store
the row information of all the selected messages for further operation.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_cut1_activate(GtkWidget * widget, gpointer data)
{
	GtkTreeView *list_view = NULL;
	GtkTreeModel *list_model = NULL;
	GtkTreeModel *child_model = NULL;
	GtkTreeSelection *selection = NULL;
	GtkAdjustment *adjustment = NULL;
	guint scroll_position = 0;
	gint num_of_msgs = 0;
	gboolean retval = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (widget == NULL || email_main_view == NULL
	    || email_main_view->widgets == NULL
	    || email_main_view->widgets->folderView == NULL
	    || email_main_view->widgets->messageListView == NULL
	    || email_main_view->message_handler == NULL
	    || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	/*Get the number of marked rows */
	list_view = GTK_TREE_VIEW(email_main_view->widgets->messageListView);
	list_model = gtk_tree_view_get_model(list_view);
	if (list_model == NULL) {
		ULOG_DEBUG_F("No model set for current folder view!\n");
		return;
	}

	child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(list_model));

	/*Check if atleast one selection is made in listview */
	selection = gtk_tree_view_get_selection(list_view);
	if (gtk_tree_selection_count_selected_rows(selection) < 1) {
		get_user_confirmation(_("mcen_ib_nothing_to_cut"), NULL, NULL, INFO_BANNER);
		return;
	}

	/*Get current scroll position */
	adjustment =
	    gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW
						(email_main_view->widgets->message_scrollwindow));
	scroll_position = gtk_adjustment_get_value(adjustment);

	/* Check whether any one message is being sent in currently marked
	 * messages, then display infoprint.*/
	if (email_main_view->folder_manager->currently_displayed_model == OUTBOX_MODEL_NUM) {
		num_of_msgs = gtk_tree_model_iter_n_children(list_model, NULL);
		retval = ui_get_outbox_msg_currently_being_sent(NULL);
		if (num_of_msgs > 1 && retval) {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_cut_mess"));
		}
	}

	/*Proceed to cut the selected messages */
	ui_manage_selected_items((gint) email_main_view->folder_manager->
				 currently_displayed_model, -1, ACTION_CUT, FALSE, NULL);

	/*Restore old scroll position */
	gtk_adjustment_set_value(adjustment, scroll_position);

	/*Show 'copied' banner if required */
	if (!retval) {
		gtk_confirmation_banner_with_icon_name((GtkWindow *) app,
						       _("mail_ib_copied_to_clipboard"),
						       "qgn_note_infoprint");
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Delete menu is clicked from the
main UI. It deletes the currently marked/selected rows from the current
folder view.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_delete1_activate(GtkWidget * widget, gpointer data)
{
	GtkTreeView *list_view = NULL;
	gint current_row_num = -1;
	GtkTreeModel *list_model = NULL;
	GtkTreeModel *child_model = NULL;
	GtkTreeSelection *selection = NULL;
	gboolean result = FALSE;
	gint signal_type = -1;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (widget == NULL || data == NULL || email_main_view == NULL
	    || email_main_view->widgets == NULL
	    || email_main_view->widgets->no_msgs_label == NULL
	    || email_main_view->widgets->messageListView == NULL
	    || email_main_view->message_handler == NULL
	    || email_main_view->folder_manager == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	signal_type = GPOINTER_TO_INT(data);

	list_view = GTK_TREE_VIEW(email_main_view->widgets->messageListView);

	/*Get the number of marked rows */
	list_model = gtk_tree_view_get_model(list_view);
	if (list_model == NULL) {
		ULOG_DEBUG_F("No model set for current folder view!\n");
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_del"));
		return;
	}

	if (signal_type == INSENSITIVE_SIGNAL_NUM
	    && gtk_tree_model_iter_n_children(list_model, NULL) != 0) {
		if (email_main_view->tags->is_msg_being_sent) {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_delete"));
		} else {
			gtk_infoprint((GtkWindow *) app, _("mcen_ib_message_unableto_delete"));
		}
		return;
	}

	child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(list_model));

	/*Check if atleast one selection is made in listview */
	selection = gtk_tree_view_get_selection(list_view);
	if (gtk_tree_selection_count_selected_rows(selection) < 1) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_del"));
		return;
	}

	current_row_num = email_main_view->message_handler->currently_selected_row;
	result =
	    ui_manage_selected_items((gint) email_main_view->folder_manager->
				     currently_displayed_model, -1, ACTION_DELETE, FALSE, NULL);

	/*If folder view is empty, update the appropriate menus & tool bar buttons */
	if (gtk_tree_model_iter_n_children(list_model, NULL) == 0) {
		ui_update_folder_specific_widgets((guint) email_main_view->
						  folder_manager->currently_displayed_model);

		/*Show 'No messages to show' label */
		gtk_widget_show(email_main_view->widgets->no_msgs_label);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Manage-Folders menu is clicked.
It opens the 'Manage Folders' dialog.

@param  widget          Pointer to the widget on which this function is called
@param  data            Pointer to the user data
@return void
*/
static void ui_on_manage_folders1_activate(GtkWidget * widget, gpointer data)
{
	GtkWidget *manage_folders = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (widget == NULL || data == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Call the Manage_Folders dialog */
	manage_folders = GTK_WIDGET(ui_create_manage_folders_dialog(GTK_TREE_VIEW(data)));
	if (manage_folders == NULL) {
		ULOG_DEBUG_F("Creation of Manage_Folders dialog failed!\n");
	} else {
		gtk_widget_show(manage_folders);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when the Move-To menu is clicked.
It opens the 'Move To Folder' dialog.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_move_to1_activate(GtkWidget * widget, gpointer data)
{
	/*GtkWidget *moveToFolderDialog = NULL; */
	GtkTreeView *folder_View = NULL;
	GtkTreeView *listView = NULL;
	GtkTreeModel *list_model = NULL;
	GtkTreeModel *child_model = NULL;
	GtkTreeSelection *selection = NULL;
	gboolean retval = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (widget == NULL || email_main_view == NULL
	    || email_main_view->widgets == NULL
	    || email_main_view->widgets->folderView == NULL
	    || email_main_view->widgets->messageListView == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	folder_View = GTK_TREE_VIEW(email_main_view->widgets->folderView);
	listView = GTK_TREE_VIEW(email_main_view->widgets->messageListView);

	/*Get the number of marked rows */
	list_model = gtk_tree_view_get_model(listView);
	if (list_model == NULL) {
		ULOG_DEBUG_F("No model set for current folder view!\n");
		return;
	}

	child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(list_model));

	/*Check if atleast one selection is made in listview */
	selection = gtk_tree_view_get_selection(listView);
	if (gtk_tree_selection_count_selected_rows(selection) < 1) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_move"));
		return;
	}

	/* Check whether any one message is being sent in currently marked
	 * messages.*/
	retval = ui_unselect_outbox_msg_being_sent();

	/*Call the Move_To_Folder dialog */
	ui_create_move_to_folder_dialog(folder_View, listView, retval, NULL);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used Sort menu is clicked. It opens the
'Sort Messages' Dialog.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_sort1_activate(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (widget == NULL || data == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	/*Call the Manage_Folders dialog */
	ui_create_sort_messages_dialog(GTK_TREE_VIEW(data));

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Folder-Settings menu is clicked.
It opens the 'Folder Settings' Dialog.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data (not used here)
@return void
*/
void ui_on_folder_settings_activate(GtkWidget * widget, gpointer data)
{
	GtkWidget *folder_settings_dialog = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (widget == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Call the Folder_Settings dialog */
	folder_settings_dialog = GTK_WIDGET(ui_create_folder_settings_dialog());
	if (folder_settings_dialog == NULL) {
		ULOG_DEBUG_F("Creation of Folder_Settings dialog failed!\n");
	} else {
		gtk_widget_show(folder_settings_dialog);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Account-Settings menu is clicked.
It opens the 'E-mail Accounts' Dialog.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data (not used here)
@return void
*/
static void ui_on_account_settings1_activate(GtkWidget * widget, gpointer data)
{
	GtkWidget *window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (widget == NULL || email_main_view == NULL || email_main_view->account_manager == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	/*Call the Email_Accounts_Setup dialog, if no accounts are configured
	   Else call the Email_Accounts dialog */
	if (email_main_view->account_manager->configured_accounts == NULL) {
		window = ui_accmgmt_createAccountsetupDialog(FALSE);
	} else {
		window =
		    ui_accmgmt_createemailaccountsedit(email_main_view->account_manager->
						       configured_accounts);
	}
	if (window == NULL) {
		ULOG_DEBUG_F("Creation of Email_Accounts dialog failed!\n");
	} else {
		gtk_widget_show(window);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when New-Email-Account menu is clicked.
It opens the 'E-mail Account Set-up' Dialog.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data (not used here)
@return void
*/
void ui_on_new_e_mail_account1_activate(GtkWidget * widget, gpointer data)
{
	GtkWidget *window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (widget == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	/*Call the Email_Accounts_Setup dialog */
	window = ui_accmgmt_createAccountsetupDialog(FALSE);
	if (window == NULL) {
		ULOG_DEBUG_F("Creation of Email_Accounts_Setup dialog failed!\n");
	} else {
		gtk_widget_show(window);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to open a new viewer window for a message
corresponding to a particular row in a particular folder.

@param  row_num     Row number of the message to be opened
@param  folder_num  Folder number of the message to be opened
@param set_prev_settings is the flag determining whether
        the opened mail should retain the previous settimgs or not.
@param failed_mail is flag for opening the mail where the send operation fails.
        after poping the error.
@return void
*/
void ui_open_message(gint row_num, gint folder_num, gboolean set_prev_settings)
{
	GtkWidget *viewer = NULL, *composer = NULL;
	HeaderInfo *header = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Get the current row info */
	header = g_new0(HeaderInfo, 1);
	header->folderNum = folder_num;
	header->rowNum = row_num;
	header->set_prev_settings = set_prev_settings;

	ui_get_row_information(header);

	/*Check if message is already opened.
	   If so simply top the corresponding window */
	if (header->in_use) {
		if (ui_top_opened_message(header->msgid) == FALSE) {
			ULOG_DEBUG_F("Unable to top opened message!\n");
		}
		ui_freeHeaderInfo(header);
		return;
	}

	/*Check if message is editable, if so open in editor */
	if (header->editable_status
	    && (folder_num == DRAFT_MODEL_NUM || folder_num == OUTBOX_MODEL_NUM)) {
		if (set_prev_settings) {
			/*Call the composer window */
			composer = GTK_WIDGET(ui_editor_create(header, NULL, DRAFT_MAIL));
			if (composer == NULL) {
				ULOG_DEBUG_F("Creation of Email Editor failed!\n");
			} else {
				hildon_app_set_appview(app, (HildonAppView *) composer);
				gtk_widget_show(composer);
				email_main_view->tags->main_view_topped = FALSE;
			}
		} else {
			ui_open_editor(GTK_TREE_VIEW(email_main_view->widgets->messageListView),
				       DRAFT_MAIL);
		}
		ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
		ui_freeHeaderInfo(header);
		return;
	}

	/*Call the viewer window */
	viewer = ui_viewer_create(header);
	if (viewer == NULL) {
		ULOG_DEBUG_F("Creation of Email_Viewer failed!\n");
		email_main_view->tags->main_view_topped = TRUE;
	} else {
		hildon_app_set_appview(app, HILDON_APPVIEW(viewer));
		gtk_widget_show(viewer);
		email_main_view->tags->main_view_topped = FALSE;
	}

	/*The internal mime_info is freed when the  opened message is closed */
	ui_freeHeaderInfo(header);

	return;
}

/**
This is a signal handler function used when Open menu is clicked. It opens a new
viewer/editor window for the currently selected message.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_open1_activate(GtkWidget * widget, gpointer data)
{
	GtkTreePath *path = NULL;
	gint *indices = NULL;
	GtkTreeView *view = NULL;
	GtkTreeIter iter;
	GtkTreeModel *model = NULL;
	GtkTreeSelection *selection = NULL;
	gchar *msgid = NULL, *tmp_msgid = NULL;
	GList *paths_list = NULL;
	gboolean retval = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (widget == NULL || data == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	view = (GtkTreeView *) data;

	model = gtk_tree_view_get_model(view);
	if (model == NULL) {
		ULOG_DEBUG_F("No model set for current folder view!\n");
		return;
	}

	selection = gtk_tree_view_get_selection(view);
	if (gtk_tree_selection_count_selected_rows(selection) > 1) {
		/*if more than one row or no row is selected.we return */
		return;
	}

	paths_list = gtk_tree_selection_get_selected_rows(selection, NULL);
	if (paths_list == NULL) {
		/*if  no row is selected.we return */
		return;
	}
	/*If a row is selected get its row seq number and open the message */
	path = (GtkTreePath *) g_list_nth_data(paths_list, 0);
	indices = gtk_tree_path_get_indices(path);
	if (indices == NULL) {
		ULOG_DEBUG_F("Unable to obtain row number of the currently selected row!\n");
		g_list_foreach(paths_list, (GFunc) gtk_tree_path_free, NULL);
		g_list_free(paths_list);
		return;
	} else {
		ULOG_DEBUG_F("Row number of the currently selected row = %d\n", indices[0]);
		email_main_view->message_handler->currently_selected_row = indices[0];
	}

	/*If opening from DRAFTS or OUTBOX open in editor */
	/*fix for bug #26361 when a message in sending status in outbox is clicked,editor is
	   opened which caused the bug
	   when message is in sending status the editor is not opened */
	if (email_main_view->folder_manager->currently_displayed_model == OUTBOX_MODEL_NUM) {
		gtk_tree_model_get_iter((GtkTreeModel *) model, &iter, path);
		gtk_tree_model_get(model, &iter, COL_MSG_ID, &msgid, -1);
		retval = ui_get_outbox_msg_currently_being_sent(&tmp_msgid);
		if (!retval || (retval && (ui_compare_utf8_strings(tmp_msgid, msgid) != 0))) {
			ui_open_editor(view, OUTBOX_MAIL);
		}
		g_free(msgid);
		g_free(tmp_msgid);
		g_list_foreach(paths_list, (GFunc) gtk_tree_path_free, NULL);
		g_list_free(paths_list);
		return;
	}

	ui_open_message(email_main_view->message_handler->currently_selected_row,
			email_main_view->folder_manager->currently_displayed_model, FALSE);
	g_list_foreach(paths_list, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(paths_list);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
 * This function opens the address book application
 *
 * @param  widget          Pointer to the widget on which this function is called
 * @param  data            Pointer to the user data (not used here)
 * @return void
 * */


static void ui_on_addressbook_activate(GtkWidget * widget, gpointer data)
{
	osso_return_t retval = OSSO_ERROR;

	retval = osso_rpc_run_with_defaults(ui_context, "osso_addressbook", "top_application", NULL,
					    DBUS_TYPE_INVALID);

	if (retval != OSSO_OK) {
		osso_log(LOG_ERR, "Error in opening address book application\n");
	}

	return;
}


/**
This is a signal handler function used when the signature menu is clicked. It
opens the 'E-mail signature' Dialog.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data (not used here)
@return void
*/
void ui_on_signatures1_activate(GtkWidget * widget, gpointer data)
{
	GtkWidget *signature_settings = NULL;
	gboolean user_confirmation = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (email_main_view == NULL || email_main_view->account_manager == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	/*Check if there are no accounts configured
	   And launch new account configuration dialog if required */
	if (email_main_view->account_manager->configured_accounts == NULL) {
		user_confirmation =
		    get_user_confirmation(_("mcen_nc_no_email_acnts_defined"),
					  _("mcen_bd_define"), NULL, CONFIRM_DIALOG);

		/*Return if a new account has not been successfully added */
		if (user_confirmation) {
			email_main_view->account_manager->action_after_acct_creation =
			    INVOKE_SIGNATURES_DIALOG;
			ui_on_new_e_mail_account1_activate(widget, NULL);
		}
		ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
		return;
	}

	/*Call the Email_Signature dialog */
	signature_settings =
	    create_enablesignature_window(email_main_view->account_manager->configured_accounts);
	if (signature_settings == NULL) {
		ULOG_DEBUG_F("Creation of Email_Signature dialog failed!\n");
	} else {
		gtk_widget_show(signature_settings);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}


/**
This is a signal handler function used when New-Email menu is clicked. It opens
a new editor window for message composing.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data (not used here)
@return void
*/
void ui_on_new_email_activate(GtkWidget * widget, gpointer data)
{
	HildonAppView *editorview = NULL;
	gboolean confirmation = FALSE;
	GtkWidget *account_setup_win = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (email_main_view == NULL || email_main_view->tags == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Check if accounts are defined. */
	if ((email_main_view->account_manager->configured_accounts) == NULL) {
		confirmation =
		    get_user_confirmation(_("mcen_nc_no_email_acnts_defined"),
					  _("mcen_bd_yes"), _("mcen_bd_no"), CONFIRM_DIALOG);
		if (confirmation == TRUE) {
			email_main_view->account_manager->action_after_acct_creation = NEW_EMAILS;
			account_setup_win =
			    (GtkWidget *) ui_accmgmt_createAccountsetupDialog(FALSE);
			if (account_setup_win != NULL) {
				gtk_widget_show(account_setup_win);
			}
			ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
			return;
		}
	}

	/*Call the composer window */
	editorview = HILDON_APPVIEW(ui_editor_create(NULL, NULL, NEW_MAIL));
	if (editorview == NULL) {
		ULOG_DEBUG_F("Creation of New_Email_Composer failed!\n");
	} else {
		hildon_app_set_appview(app, editorview);
		gtk_widget_show(GTK_WIDGET(editorview));
		email_main_view->tags->main_view_topped = FALSE;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Forward menu is clicked. It opens
a new editor window for forwarding the currently selected message.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
void ui_on_forward1_activate(GtkWidget * widget, gpointer data)
{
	GtkTreeView *view = NULL;
	gboolean confirmation = FALSE;
	GtkWidget *account_setup_win = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (data == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	/*Check if accounts are defined. */
	if ((email_main_view->account_manager->configured_accounts) == NULL) {
		confirmation =
		    get_user_confirmation(_("mcen_nc_no_email_acnts_defined"), NULL, NULL,
					  CONFIRM_DIALOG);
		if (confirmation == TRUE) {
			email_main_view->account_manager->action_after_acct_creation = FORWARD_MSGS;
			account_setup_win =
			    (GtkWidget *) ui_accmgmt_createAccountsetupDialog(FALSE);
			if (account_setup_win != NULL) {
				gtk_widget_show(account_setup_win);
			}
			ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
			return;
		}
	}

	/*Open the Editor window */
	view = (GtkTreeView *) data;
	ui_open_editor(view, FORWARD_FROM_FOLDER);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Reply menu is clicked. It opens
a new editor window for replying to the currently selected message.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_reply1_activate(GtkWidget * widget, gpointer data)
{
	GtkTreeView *view = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (widget == NULL || data == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	/*Open the Editor window */
	view = (GtkTreeView *) data;
	ui_open_editor(view, REPLY_FROM_FOLDER);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Reply-To-All menu is clicked. It
opens a new editor window for replying to all in the currently selected message.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_reply_to_all1_activate(GtkWidget * widget, gpointer data)
{
	GtkTreeView *view = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (widget == NULL || data == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	/*Open the Editor window */
	view = (GtkTreeView *) data;
	ui_open_editor(view, REPLY_ALL_FROM_FOLDER);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
 This function is used to free the RetrMsgData structure.
 
 @param rtr_msgid_data is the pointer the RetrMsgData structure.
 @return void.
*/
static void free_retrmsgid_info(RetrMsgData * rtr_msgid_data)
{
	g_free(rtr_msgid_data->msgid);
	g_free(rtr_msgid_data);
}

/**
This is a signal handler function used when Retrieve-Mail-Contents menu is
clicked. It retrieves the contents of all selected e-mail headers.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_retrieve_mail_contents1_activate(GtkWidget * widget, gpointer data)
{
	GtkTreeView *view = NULL;
	GtkTreeModel *sorted_model = NULL;
	GSList *retr_list = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (widget == NULL || data == NULL || email_main_view == NULL
	    || email_main_view->tags == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	view = GTK_TREE_VIEW(data);
	sorted_model = gtk_tree_view_get_model(view);
	if (sorted_model == NULL) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_select_headers"));
		ULOG_DEBUG_F("No storage model for current view!\n");
		return;
	}


	email_main_view->tags->get_only_unretrieved_mails = TRUE;
	/*Get the message-ids for all unretrieved selected messages */
	retr_list = ui_get_unretrieved_msg_id_list(view);
	email_main_view->tags->get_only_unretrieved_mails = FALSE;

	/* Retrieve the contents */
	ui_dbus_retrieve_msgs(retr_list, FALSE);

	g_slist_foreach(retr_list, (GFunc) free_retrmsgid_info, NULL);
	g_slist_free(retr_list);


	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Resend menu is clicked. It is
active only in Sent folder view. It resends the currently selected messages.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
void ui_on_resend_activate(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (data == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	ui_send_selected_mails(OPR_RESEND_MAILS, GTK_TREE_VIEW(data));

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Cancel sending menu is clicked.
It is active only in Outbox folder view. It cancels sending of the currently
selected messages.

@param  widget          Pointer to the widget on which this function is called
@param  data            Pointer to the user data
@return void
*/
void ui_on_cancel_send_activate(GtkWidget * widget, gpointer data)
{
	GtkTreeView *view = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeSelection *selection = NULL;
	gint in_use_msgs_count = 0;
	GSList *msg_id_list = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (email_main_view == NULL || email_main_view->account_manager == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	view = GTK_TREE_VIEW(email_main_view->widgets->messageListView);
	model = gtk_tree_view_get_model(view);
	if (model == NULL) {
		ULOG_DEBUG_F("No storage model for current view!\n");
		return;
	}

	/*Check if atleast one selection is made in listview */
	selection = gtk_tree_view_get_selection(view);
	if (gtk_tree_selection_count_selected_rows(selection) < 1) {
		get_user_confirmation(_("mcen_ib_nothing_to_send"), NULL, NULL, INFO_BANNER);
		ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
		return;		//Do nothing
	}


	/*Get the message-ids for all selected messages */
	msg_id_list = ui_get_selected_not_inuse_msgids(view, -1, &in_use_msgs_count);

	if (msg_id_list != NULL) {
		/*Call AL function for cancelling all selected messages */
		ui_dbus_cancel_send_mail(msg_id_list);
		g_slist_foreach(msg_id_list, (GFunc) g_free, NULL);
		g_slist_free(msg_id_list);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Send menu is clicked. It is active
only in Outbox or Drafts folder view. It sends the currently selected messages.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
void ui_on_send_activate(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (data == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	if (email_main_view->folder_manager->currently_displayed_model == DRAFT_MODEL_NUM) {
		ui_send_selected_mails(OPR_SEND_DRAFT_MAILS, GTK_TREE_VIEW(data));
	} else {
		ui_send_selected_mails(OPR_SEND_OUTBOX_MAILS, GTK_TREE_VIEW(data));
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Cancel button in the progress bar
is clicked. It is active only when progress bar is visible. It cancels the
current send/receive operation.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
void ui_on_cancel_clicked(GtkWidget * widget, gpointer data)
{
	gboolean send_cancel_req_on_normal_connection = TRUE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (email_main_view == NULL || email_main_view->tags == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	if (email_main_view->tags->msg_download_in_progress) {
		send_cancel_req_on_normal_connection = FALSE;
	}

	/*Call AL function for cancelling retrieve operations */
	switch (email_main_view->tags->current_mode) {
	case RECEIVING:
		{
			ui_dbus_cancel_recv_mail(email_main_view->account_manager->
						 current_sendrec_account,
						 send_cancel_req_on_normal_connection);
			break;
		}
	case SENDING:
		{
			ui_dbus_cancel_send_mail(NULL);
			break;
		}
	default:
		{
			/*This could be connecting state. Call both the cancel operations */
			ui_dbus_cancel_recv_mail(email_main_view->account_manager->
						 current_sendrec_account,
						 send_cancel_req_on_normal_connection);
			ui_dbus_cancel_send_mail(NULL);
			break;
		}
	}

	return;
}

/**
This is a signal handler function used when 'Close window' menu is activated.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
static void ui_on_close_window_activate(GtkWidget * widget, gpointer data)
{
	HildonAppView *view = NULL;
	OpenedWindowData *open_window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (email_main_view == NULL
	    || email_main_view->tags == NULL || email_main_view->open_windows == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	if (email_main_view->tags->main_view_topped == FALSE) {
		view = hildon_app_get_appview(app);
		g_signal_emit_by_name(view, "close");
	} else {
		/*If there are opened views beneath,
		   just hide main view and top the below view */
		if (g_slist_length(email_main_view->open_windows) > 1) {
			open_window = (OpenedWindowData *) email_main_view->open_windows->data;
			if (strcmp(open_window->msgid, MAIN_VIEW_ID) != 0) {
				ULOG_DEBUG_F("Current view is not Main view\n");
			} else {
				/*Unregister main view */
				hildon_app_unregister_view(app, appview);
				email_main_view->open_windows =
				    g_slist_remove(email_main_view->open_windows, open_window);
				g_free(open_window->msgid);
				g_free(open_window->oldmsgid);
				g_free(open_window);
				email_main_view->tags->main_view_hidden = TRUE;
				open_window =
				    (OpenedWindowData *) email_main_view->open_windows->data;
				hildon_app_set_appview(app,
						       (HildonAppView *) open_window->window_ptr);
				email_main_view->tags->main_view_topped = FALSE;
				ULOG_DEBUG_F("Message '%s' topped\n", open_window->msgid);
			}
		}
		/*If only main view is existing, close the application */
		else {
			/*Unregister the appview b4 destroying */
			ui_dbus_disconnect_iap_conn();
			hildon_app_unregister_view(app, appview);
			gtk_widget_destroy(GTK_WIDGET(appview));
			/*Unset previous email state, its no longer needed now */
			ui_dbus_statesave_write(NULL);
			ui_exit(0);
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when 'Close all windows' menu is
activated.

@param	widget		Pointer to the widget on which this function is called
@param	data		Pointer to the user data
@return void
*/
void ui_on_close_all_windows_activate(GtkWidget * widget, gpointer data)
{
	gboolean confirmation = TRUE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (email_main_view == NULL || email_main_view->open_windows == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*set the pane position into the persistent settings */
	ui_set_folder_pane_width(email_main_view->widgets->folder_msg_pane);
	/*Throw appropriate notification */
	if (g_slist_length(email_main_view->open_windows) > 1) {
		confirmation =
		    get_user_confirmation(_("emev_nc_close_windows"),
					  _("emev_bd_close_windows_yes"),
					  _("emev_bd_close_windows_no"), CONFIRM_DIALOG);
	}

	if (confirmation == TRUE) {
		email_main_view->tags->application_closure = TRUE;
		ui_dbus_disconnect_iap_conn();
		ui_close_all_windows_cb();
		gtk_widget_destroy(GTK_WIDGET(appview));
		/*Unset previous email state, its no longer needed now */
		ui_dbus_statesave_write(NULL);
		ui_exit(0);
	} else {		/*Remove only the current view */

		ui_on_close_window_activate(NULL, NULL);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when 'Full screen' menu is activated.

@param  widget          Pointer to the widget on which this function is called
@param  data            Pointer to the user data
@return void
*/
static void ui_on_full_screen_activate(GtkWidget * widget, gpointer data)
{
	gboolean ret_val = FALSE;
	gboolean toolbar_visibility = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (email_main_view == NULL || email_main_view->tags == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	ret_val = hildon_appview_get_fullscreen((HildonAppView *) appview);

	email_main_view->tags->show_tool_bar =
	    gtk_check_menu_item_get_active((GtkCheckMenuItem *) email_main_view->menu->
					   view_show_toolbar_normal_screen);
	email_main_view->tags->show_fullscreen_tool_bar =
	    gtk_check_menu_item_get_active((GtkCheckMenuItem *) email_main_view->menu->
					   view_show_toolbar_full_screen);

	if (widget == NULL) {
		if (ret_val != FALSE) {
			if (data != NULL)
				toolbar_visibility = !(email_main_view->tags->show_tool_bar);
			else
				toolbar_visibility =
				    !(email_main_view->tags->show_fullscreen_tool_bar);

			ui_toggle_toolbar_visibility(TRUE, toolbar_visibility);
		} else {
			if (data != NULL)
				toolbar_visibility =
				    !(email_main_view->tags->show_fullscreen_tool_bar);
			else
				toolbar_visibility = !(email_main_view->tags->show_tool_bar);
			ui_toggle_toolbar_visibility(TRUE, toolbar_visibility);
		}
		return;
	}


	if (widget != NULL && HILDON_IS_APPVIEW(widget) == FALSE) {
		email_main_view->tags->full_screen_mode = gtk_check_menu_item_get_active
		    ((GtkCheckMenuItem *) widget);
	}

	if (ret_val == FALSE) {
		hildon_appview_set_fullscreen((HildonAppView *) data, TRUE);
		ui_toggle_toolbar_visibility(TRUE,
					     !(email_main_view->tags->show_fullscreen_tool_bar));
	} else {
		hildon_appview_set_fullscreen((HildonAppView *) data, FALSE);
		ui_toggle_toolbar_visibility(TRUE, !(email_main_view->tags->show_tool_bar));
	}


	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when 'View' menu is activated.

@param  widget          Pointer to the widget on which this function is called
@param  data            Pointer to the user data
@return void
*/
static void ui_on_view_menu_select_activated(GtkWidget * widget, gpointer data)
{
	gboolean ret_val = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	ret_val = hildon_appview_get_fullscreen((HildonAppView *) appview);

	g_signal_handlers_block_by_func(G_OBJECT
					(email_main_view->menu->view_full_screen),
					(gpointer) ui_on_full_screen_activate, appview);

	if (ret_val != FALSE) {
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
					       (email_main_view->menu->view_full_screen), TRUE);
	} else {
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
					       (email_main_view->menu->view_full_screen), FALSE);
	}
	g_signal_handlers_unblock_by_func(G_OBJECT
					  (email_main_view->menu->view_full_screen),
					  (gpointer) ui_on_full_screen_activate, appview);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when 'Undo' menu is activated.

@param  widget Pointer to the widget on which this function is called
@param  data   Pointer to the user data
@return void
*/
static void ui_on_undo_activate(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((widget == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->undo == NULL), "Input param check failed", return);
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	if (email_main_view->undo->operation != OPER_UNCUT &&
	    email_main_view->undo->message_id_list == NULL) {
		ULOG_DEBUG_F("Nothing to Undo!");
		email_main_view->undo->operation = OPER_NONE;
		return;
	}

	switch (email_main_view->undo->operation) {
	case OPER_DELETE:
		{
			ui_manage_selected_items(email_main_view->undo->orig_folder, -1,
						 ACTION_DELETE, TRUE, NULL);
			break;
		}
	case OPER_MOVE:
		{
			ui_manage_selected_items(email_main_view->undo->orig_folder,
						 email_main_view->undo->dest_folder, ACTION_MOVE,
						 TRUE, NULL);
			break;
		}
	case OPER_UNCUT:
		{
			ui_undim_cut_messages();
			break;
		}
	default:
		{
			break;
		}
	}

	g_slist_foreach(email_main_view->undo->message_id_list, (GFunc) g_free, NULL);
	g_slist_free(email_main_view->undo->message_id_list);
	email_main_view->undo->message_id_list = NULL;
	email_main_view->undo->operation = OPER_NONE;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when 'Purge Attachments' menu is
activated.

@param  widget Pointer to the widget on which this function is called
@param  view   Pointer to the message listview widget
@return void
*/
static void ui_on_purge_attachments_activate(GtkWidget * widget, GtkTreeView * view)
{
	GSList *message_id_list = NULL, *temp_message_id_list = NULL;
	gboolean user_confirmation = FALSE;
	GSList *failed_msg_list = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((widget == NULL), "Input param check failed", return);
	CHECK_PARAM((view == NULL), "Input param check failed", return);
	CHECK_PARAM((ignore_ext_events == TRUE), "External events are blocked", return);

	user_confirmation =
	    get_user_confirmation(_("mcen_nc_purge_file_text_inbox"),
				  _("mcen_bd_purge_file_remove"), NULL, CONFIRM_DIALOG);

	CHECK_PARAM((user_confirmation == FALSE), "User cancelled the request", return);

	/*get the list of msgids of messages for which purging is allowed from
	   the selected messages */
	ui_get_purgeable_row_msg_ids((GtkTreeView *) email_main_view->widgets->
				     messageListView, &message_id_list);

	if (message_id_list == NULL) {
		return;
	}

	/* fire the dbus call to remove the retrived attachments for the
	   msgid list */
	failed_msg_list = ui_dbus_folder_remove_all_attachment(message_id_list);

	/*Check if purging fails */
	if (failed_msg_list != NULL) {
		/*Freeing the failed msgid list */
		g_slist_foreach(failed_msg_list, (GFunc) g_free, NULL);
		g_slist_free(failed_msg_list);
		osso_log(LOG_DEBUG, "Error occurred while trying to purge attachments");
	} else {
		for (temp_message_id_list = message_id_list; temp_message_id_list != NULL;
		     temp_message_id_list = temp_message_id_list->next) {

			/*Update corresponding flags on success */
			ui_update_row_tag_in_folder(email_main_view->folder_manager->
						    currently_displayed_model,
						    temp_message_id_list->data, PURGE_STATUS,
						    FALSE);
			ui_update_row_tag_in_folder(email_main_view->folder_manager->
						    currently_displayed_model,
						    temp_message_id_list->data,
						    FULLY_DOWNLOADED_STATUS, FALSE);
			if (email_main_view->folder_manager->currently_displayed_model !=
			    INBOX_MODEL_NUM) {
				ui_update_row_tag_in_folder(email_main_view->folder_manager->
							    currently_displayed_model,
							    temp_message_id_list->data,
							    ATTACHMENT_STATUS, FALSE);
			}
		}
		ui_on_messageview_focussed(NULL, NULL, NULL);
	}


	/*Freeing the msgid list */
	g_slist_foreach(message_id_list, (GFunc) g_free, NULL);
	g_slist_free(message_id_list);


	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function to show dimming notes when 'Purge attachments'
menu is accessed in insensitive mode.

@param	widget	Pointer to the widget on which this function is called
@param	data	Pointer to the tree view widget
@return void
*/
static void ui_insensitive_purge_menu_accessed(GtkWidget * widget, gpointer data)
{
	GtkTreeSelection *selection = NULL;
	GtkTreeModel *folder_model = NULL;
	GtkTreeIter iter;
	gint protocol = -1;
	gboolean attachment_present = FALSE;
	GList *pathslist = NULL, *temp_pathslist = NULL;
	GtkTreePath *path = NULL;
	gboolean pop_protocol = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((data == NULL), "Input param check failed", return);

	if (email_main_view->tags->marking_mode) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_select_one_message"));
	} else {
		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(data));
		if (gtk_tree_selection_count_selected_rows(selection) == 0) {
			gtk_infoprint((GtkWindow *) app, _("mail_ib_nothing_to_purge"));
			ULOG_DEBUG_F("Nothing is selected or no model set for folder view!");
			return;
		}

		pathslist = gtk_tree_selection_get_selected_rows(selection, &folder_model);
		temp_pathslist = pathslist;

		for (temp_pathslist = pathslist;
		     temp_pathslist != NULL && temp_pathslist->data != NULL;
		     temp_pathslist = temp_pathslist->next) {
			path = (GtkTreePath *) temp_pathslist->data;
			gtk_tree_model_get_iter((GtkTreeModel *) folder_model, &iter, path);

			gtk_tree_model_get(folder_model, &iter, COL_RECV_PROTOCOL,
					   &protocol, COL_ATTACHMENT_PRESENT,
					   &attachment_present, -1);

			if (protocol == PROTO_POP3 && attachment_present) {
				gtk_infoprint((GtkWindow *) app,
					      _("mail_ib_unable_to_pure_attach_pop_mail_singular"));
				pop_protocol = TRUE;
				break;
			}
		}
		if (pop_protocol == FALSE)
			gtk_infoprint((GtkWindow *) app, _("mail_ib_nothing_to_purge"));
		g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
		g_list_free(pathslist);

		ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
		return;
	}
}

/**
This is a signal handler function used when a row is selected from the message
view.

@param	treeView	Pointer to the GtkTreeView of the message view
@param	path		Pointer to the GtkTreePath of current row
@param	column		Pointer to the GtkTreeViewcolumn of current row
@param	data		Pointer to the user data
@return void
*/
void
ui_messageview_on_row_selected(GtkTreeView * treeView, GtkTreePath * path,
			       GtkTreeViewColumn * column, gpointer data)
{
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	gboolean cut_status = FALSE;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (treeView == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}
	model = gtk_tree_view_get_model(treeView);
	gtk_tree_model_get_iter(model, &iter, path);
	gtk_tree_model_get(model, &iter, COL_CUT_STATUS, &cut_status, -1);

	if (!cut_status)
		return;

	g_signal_handlers_block_by_func(G_OBJECT(treeView), (gpointer)
					ui_messageview_on_row_selected, NULL);
	ui_on_open1_activate(GTK_WIDGET(treeView), treeView);
	g_signal_handlers_unblock_by_func(G_OBJECT(treeView), (gpointer)
					  ui_messageview_on_row_selected, NULL);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function to dim/undim delete,open,send menus based on current
selection of the messages.

@param  msg_in_use  Boolean flag indicating if currently selected msg is in use
@param  marked_rows Total number of marked rows
@return void
*/
static void ui_toggle_menus_based_on_selection(gboolean msg_in_use, gint marked_rows)
{
	gboolean retval = FALSE, sensitivity = TRUE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->tools == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);

	if (email_main_view->folder_manager->currently_displayed_model == OUTBOX_MODEL_NUM) {
		retval = ui_get_outbox_msg_currently_being_sent(NULL);
		if (retval != FALSE)
			email_main_view->tags->is_msg_being_sent = TRUE;
	}

	/* Checking whether the currently selected message is being sent or nothing is selected */
	if ((marked_rows == 0) ||( marked_rows == 1 && retval)){
		sensitivity = FALSE;
	}

	gtk_widget_set_sensitive(email_main_view->menu->message_open, sensitivity);
	gtk_widget_set_sensitive(email_main_view->message_popup->open, sensitivity);

	/* Checking whether the currently selected message is being sent or in use */
	if ((marked_rows == 0 || marked_rows == 1) && (msg_in_use || retval)) {
		sensitivity = FALSE;
	}
	else {
		sensitivity = TRUE;
	}

	gtk_widget_set_sensitive(email_main_view->menu->message_delete, sensitivity);
	gtk_widget_set_sensitive(email_main_view->tools->delete_msg, sensitivity);
	gtk_widget_set_sensitive(email_main_view->message_popup->delete_msg, sensitivity);
	gtk_widget_set_sensitive(email_main_view->message_popup->send, sensitivity);
	gtk_widget_set_sensitive(email_main_view->menu->message_send, sensitivity);
	gtk_widget_set_sensitive(email_main_view->tools->send, sensitivity);
	if (marked_rows > 1)
		gtk_widget_set_sensitive(email_main_view->menu->message_open, FALSE);
                
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
 This is a helper function to dim menu items if multiple rows are selceted.
 @paran rows_selected is the gint representing the no of rows selected.
 @return void.
*/
static void ui_dim_undim_marking_mode_widgets(gint rows_selected)
{
	/*if more then one message is selected ,dim the reply/replyall/forward menu-tool
	   buttons */
	if (rows_selected > 1) {
		email_main_view->tags->marking_mode = TRUE;
	} else {
		email_main_view->tags->marking_mode = FALSE;
	}
	ui_invoke_marking_mode(email_main_view->tags->marking_mode);

	/*diming the folders_folder_settings menu */
	if (!email_main_view->tags->marking_mode &&
	    email_main_view->folder_manager->currently_displayed_model == SENT_MODEL_NUM)
		gtk_widget_set_sensitive(email_main_view->menu->folders_folder_settings, TRUE);
	else
		gtk_widget_set_sensitive(email_main_view->menu->folders_folder_settings, FALSE);


}

/**
This function checks whether any of selected message in the outbox 
is having status other than suspended. Depending on this, the item 'cancel_sending' is enabled/disabled.

@return TRUE  If any one message in the currently seelcted messages is
              not in suspended status.
        FALSE All the currently selected messages are in  suspended status.
*/
static gboolean ui_is_outbox_msg_cancel_allowed()
{
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	GtkTreeSelection *selection = NULL;
	gchar *current_msg_id = NULL;
	gint outbox_status_bit = -1;
	GList *pathslist = NULL, *tmplist = NULL;
	GtkTreePath *path = NULL;
	gboolean retval = FALSE;

	model = ui_get_sorted_folder_model(OUTBOX_MODEL_NUM);
	selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->messageListView));

	pathslist = gtk_tree_selection_get_selected_rows(selection, &model);
	if (pathslist == NULL) {
		ULOG_DEBUG_L("nothing is selected");
		return FALSE;
	}
	for (tmplist = pathslist; tmplist != NULL; tmplist = tmplist->next) {
		path = (GtkTreePath *) pathslist->data;
		gtk_tree_model_get_iter((GtkTreeModel *) model, &iter, path);
		gtk_tree_model_get(model, &iter, COL_MSG_ID, &current_msg_id, -1);
		outbox_status_bit = g_ascii_digit_value(current_msg_id[strlen(current_msg_id) - 1]);
		g_free(current_msg_id);
		current_msg_id = NULL;
		if (outbox_status_bit != OUTBOX_STATUS_SUSPENDED) {
			/*A msg is found to be in non suspended mode. */
			retval = TRUE;
			break;
		}
	}

	g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(pathslist);

	return retval;
}


/**
Set the cut, move to and cancel_sending menu items and toolbar items
to enabled or disabled state depending on the message selection change in outbox.
@return void.
*/
static void ui_set_sensitivity_for_outbox_widgets()
{
	gboolean set_sensitive = FALSE;
	GtkTreeView *view = NULL;
	GtkTreeSelection *selection = NULL;
	gint num_of_msgs = 0;
	ULOG_DEBUG_L("[%s] : Starts here :\n", __FUNCTION__);

	gtk_widget_set_sensitive(email_main_view->menu->edit_cut, TRUE);
	gtk_widget_set_sensitive(email_main_view->menu->edit_move_to, TRUE);
	gtk_widget_set_sensitive(email_main_view->tools->move_to, TRUE);
	gtk_widget_set_sensitive(email_main_view->message_popup->move_to, TRUE);

	if (email_main_view->folder_manager->currently_displayed_model != OUTBOX_MODEL_NUM) {
		return;
	}
	/* Checking whether the marked message is being sent */
	view = GTK_TREE_VIEW(email_main_view->widgets->messageListView);
	selection = gtk_tree_view_get_selection(view);
	num_of_msgs = gtk_tree_selection_count_selected_rows(selection);
	set_sensitive = ui_get_outbox_msg_currently_being_sent(NULL);
	/* #fix 26174 */
	if (((set_sensitive) && (num_of_msgs == 1)) || (num_of_msgs == 0))
		gtk_widget_set_sensitive(email_main_view->menu->edit_cut, !set_sensitive);
	else if ((set_sensitive) && (num_of_msgs > 1))
		gtk_widget_set_sensitive(email_main_view->menu->edit_cut, set_sensitive);
	gtk_widget_set_sensitive(email_main_view->menu->edit_move_to, !set_sensitive);
	gtk_widget_set_sensitive(email_main_view->tools->move_to, !set_sensitive);
	gtk_widget_set_sensitive(email_main_view->message_popup->move_to, !set_sensitive);

	/*Check whether 'cancel_sending' has to be enabled */
	set_sensitive = ui_is_outbox_msg_cancel_allowed();
	gtk_widget_set_sensitive(email_main_view->menu->message_cancel_sending, set_sensitive);
	gtk_widget_set_sensitive(email_main_view->message_popup->cancel_sending, set_sensitive);
	gtk_widget_set_sensitive(email_main_view->tools->cancel_sending, set_sensitive);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used whenever selection changes in message
list view.

@param  selection  Pointer to GtkTreeSelection of message list view
@param  data       Pointer to user data (not used here)
@return void
*/
void ui_messageview_selection_changed_cb(GtkTreeSelection * selection, gpointer data)
{
	GtkTreeModel *folder_model = NULL;
	GtkTreeIter iter;
	gint *indices = NULL;
	gboolean purge_allow = FALSE;
	GtkTreePath *path = NULL;
	gint marked_rows = 0, total_rows = -1;
	gboolean msg_in_use = FALSE;
	gboolean msg_in_use_flag = FALSE;
	GList *pathslist = NULL, *temp_paths_list = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	CHECK_PARAM((selection == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->message_handler == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->menu == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->menu->edit_select_clear_all_marks == NULL),
		    "Input param check failed", return);
	CHECK_PARAM((email_main_view->menu->email_retrieve_mail_contents == NULL),
		    "Input param check failed", return);


	pathslist = gtk_tree_selection_get_selected_rows(selection, &folder_model);
	if (!pathslist) {
		ULOG_DEBUG_F("Nothing is selected!\n");
		return;
	}

	/*Number of selected rows */
	marked_rows = gtk_tree_selection_count_selected_rows(selection);

	if (folder_model == NULL) {
		g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
		g_list_free(pathslist);
		ULOG_DEBUG_F("No model set for message view!\n");
		return;
	}

	for (temp_paths_list = pathslist; pathslist != NULL && pathslist->data != NULL;
	     pathslist = pathslist->next) {

		path = (GtkTreePath *) pathslist->data;
		indices = gtk_tree_path_get_indices(path);
		gtk_tree_model_get_iter((GtkTreeModel *) folder_model, &iter, path);

		gtk_tree_model_get(folder_model, &iter,
				   COL_PURGE_ALLOW, &purge_allow,
				   COL_IN_USE_STATUS, &msg_in_use, -1);
		if (msg_in_use != FALSE)
			msg_in_use_flag = TRUE;

		/*storing the selected row numbers */
		email_main_view->message_handler->currently_selected_row = indices[0];
	}

	/*Update menus according to message tags */
	total_rows = gtk_tree_model_iter_n_children(folder_model, NULL);
	if (marked_rows < total_rows) {
		gtk_widget_set_sensitive(email_main_view->menu->edit_select_mark_all, TRUE);
	} else {
		gtk_widget_set_sensitive(email_main_view->menu->edit_select_mark_all, FALSE);
	}


	ui_set_sensitivity_for_outbox_widgets();

	/*Dimming Reply/ReplyAll/forward etc menu & toolbuttons if more then one message is selected */
	ui_dim_undim_marking_mode_widgets(marked_rows);

	if (marked_rows == 1 && purge_allow) {
		gtk_widget_set_sensitive(email_main_view->menu->email_purge_attachments, TRUE);
	} else {
		gtk_widget_set_sensitive(email_main_view->menu->email_purge_attachments, FALSE);
	}
	email_main_view->tags->is_msg_being_sent = FALSE;
	ui_toggle_menus_based_on_selection(msg_in_use_flag, marked_rows);

	g_list_foreach(temp_paths_list, (GFunc) gtk_tree_path_free, NULL);
	g_list_free(temp_paths_list);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function which sets the folder title to the main view.

@param  folder_name  Name of current folder which is on display
@return void
*/
void ui_set_folderview_title(const gchar * folder_name)
{
	gchar *folder_title = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	CHECK_PARAM((folder_name == NULL), "Input param check failed", return);

	folder_title = ui_folder_name_with_unread_mails(folder_name);
	if (folder_title != NULL) {
		hildon_appview_set_title(appview, folder_title);
	} else {
		hildon_appview_set_title(appview, folder_name);
	}

	g_free(folder_title);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function to undim messages that were dimmed due to CUT
operation.

@param  void
@return void
*/
void ui_undim_cut_messages(void)
{
	GtkTreeModel *model = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->message_handler == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->message_handler->cut_folder_num == -1),
		    "No previous CUT operation registered", return);

	model = ui_get_unsorted_folder_model(email_main_view->message_handler->cut_folder_num);
	CHECK_PARAM((model == NULL), "Originating folder of CUT operation missing", return);

	gtk_tree_model_foreach(model, ui_undim_all_rows, NULL);
	email_main_view->message_handler->cut_folder_num = -1;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to set the folder_populated flag
in the folder_view.
@param model_num Model number of the folder.
@param flag is the gboolean representing the value of folder_populated
*/
static void ui_set_is_folder_populated(gint model_num, gboolean flag)
{
	GtkTreeModel *folder_model = NULL;
	GtkTreeIter parent_iter;
	gboolean retval = FALSE;
	gint model_Seq_Num = -1;


	folder_model = email_main_view->folder_manager->folder_view_model;
	retval = gtk_tree_model_get_iter_first(folder_model, &parent_iter);
	CHECK_PARAM((retval == FALSE), "Storage model empty for folder view!", return);

	do {
		gtk_tree_model_get(folder_model, &parent_iter, COL_MODEL_SEQ_NUM,
				   &model_Seq_Num, -1);
		if (model_num == model_Seq_Num) {
			gtk_tree_store_set(GTK_TREE_STORE(folder_model), &parent_iter,
					   COL_FOLDER_POPULATED, flag, -1);
		}
	}
	while (gtk_tree_model_iter_next(folder_model, &parent_iter));
	return;
}

/**
This is a signal handler function used whenever selection changes in folder
list view.

@param  selection  Pointer to GtkTreeSelection of folder list view
@param  data       Pointer to user data (not used here)
@return void
*/
void ui_folderview_selection_changed_cb(GtkTreeSelection * selection, gpointer data)
{
	GtkTreeModel *folder_model = NULL;
	GtkTreeModel *list_model = NULL;
	GSList *model_node = NULL;
	gboolean is_sub_folder = FALSE;
	GtkTreeIter iter;
	gint current_mode_num = 0;
	gchar *current_folder_name = NULL;
	GtkTreeSelection *msg_selection = NULL;
	gchar *current_parent_name = NULL;
	PopupFolderMenu *folder_menu_structure = NULL;
	gchar *folder_path = NULL;
	gchar *title = NULL;
	gboolean folder_populated = FALSE;
	static gboolean sent_folder_cleaning_check_done = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	CHECK_PARAM((selection == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->message_handler == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->sorted_model_list == NULL), "Input param check failed",
		    return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets->messageListView == NULL),
		    "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets->no_msgs_label == NULL),
		    "Input param check failed", return);
	CHECK_PARAM((email_main_view->menu == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->menu->edit_select_mark_all == NULL),
		    "Input param check failed", return);
	CHECK_PARAM((email_main_view->menu->edit_select_clear_all_marks == NULL),
		    "Input param check failed", return);

	if (!gtk_tree_selection_get_selected(selection, &folder_model, &iter)) {
		ULOG_DEBUG_F("Nothing is selected!\n");
		return;
	}

	if (folder_model == NULL) {
		ULOG_DEBUG_F("No model set for folder view!\n");
		return;
	}

	/*Get folder data from folderListView */
	gtk_tree_model_get(folder_model, &iter, COL_MODEL_SEQ_NUM, &current_mode_num,
			   COL_FOLDER_NAME, &current_folder_name,
			   COL_PARENT_NAME, &current_parent_name, COL_FOLDER_POPULATED,
			   &folder_populated, -1);
	model_node = g_slist_nth(email_main_view->sorted_model_list, (guint) current_mode_num);
	list_model = GTK_TREE_MODEL(model_node->data);
	ULOG_DEBUG_F("Previous model : %d\tCurrent model : %d\n",
		     email_main_view->folder_manager->currently_displayed_model, current_mode_num);

	email_main_view->folder_manager->currently_displayed_model = (guint) current_mode_num;

	/*Inbox & Outbox - populated at start-up and updated dynamically.
	   Drafts - repopulate with latest message headers every time.
	   Others - populated for first time only and updated dynamically.
	   Not needed when folder is selected during state restoration. */
	/*Do not populate if UI is already busy with list view updations */

	msg_selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->messageListView));
	if (email_main_view->tags->oom_signal_received == FALSE
	    && current_mode_num != INBOX_MODEL_NUM
	    && current_mode_num != OUTBOX_MODEL_NUM && !ignore_ext_events) {
		g_signal_handlers_block_by_func(G_OBJECT(msg_selection), (gpointer)
						ui_messageview_selection_changed_cb, NULL);
		if (!folder_populated || (gtk_tree_model_iter_n_children(list_model, NULL) == 0)
		    || current_mode_num == DRAFT_MODEL_NUM) {
			gtk_list_store_clear(GTK_LIST_STORE
					     (gtk_tree_model_sort_get_model
					      (GTK_TREE_MODEL_SORT(list_model))));
			if (current_parent_name != NULL) {
				is_sub_folder = TRUE;
				folder_path =
				    g_strdup_printf("%s/%s", current_parent_name,
						    current_folder_name);
				ui_populate_folder(folder_path, (guint) current_mode_num);
				ui_set_is_folder_populated(current_mode_num, TRUE);
			} else {
				ui_populate_folder(current_folder_name, (guint) current_mode_num);
				ui_set_is_folder_populated(current_mode_num, TRUE);
			}
		}
		g_signal_handlers_unblock_by_func(G_OBJECT(msg_selection), (gpointer)
						  ui_messageview_selection_changed_cb, NULL);
	}

	/*Set corresponding list model to the listView
	   Show or Hide 'No messages to show' label */
	if (gtk_tree_model_iter_n_children(list_model, NULL) != 0) {
		gtk_tree_view_set_model(GTK_TREE_VIEW
					(email_main_view->widgets->messageListView), list_model);
		/*Highlight the first row */
		gtk_tree_model_get_iter_first(list_model, &iter);
		gtk_tree_selection_select_iter(msg_selection, &iter);

		gtk_widget_show(email_main_view->widgets->message_scrollwindow);
		gtk_widget_hide(email_main_view->widgets->no_msgs_label);
	} else {
		gtk_tree_view_set_model(GTK_TREE_VIEW(email_main_view->widgets->messageListView),
					NULL);
		gtk_widget_hide(email_main_view->widgets->message_scrollwindow);
		gtk_widget_show(email_main_view->widgets->no_msgs_label);
	}

	/*Clear all marked items in previous list view */
	ui_clear_selection(GTK_TREE_VIEW(email_main_view->widgets->messageListView));
	email_main_view->tags->marking_mode = FALSE;

	/*Modify the title bar label */
	if (current_parent_name != NULL) {
		ui_set_folderview_title(folder_path);
		g_free(folder_path);
	} else {
		if (ui_compare_utf8_strings(current_folder_name, ARCHIVE) != 0
		    && is_sub_folder == FALSE) {
			g_free(email_main_view->folder_manager->parent_folder_name);
			email_main_view->folder_manager->parent_folder_name =
			    g_strdup(current_folder_name);
		}
		ui_set_folderview_title(current_folder_name);
	}
	/* Add unread unsent message count to folder in folder list view. */
	if (current_mode_num != SENT_MODEL_NUM) {
		ui_add_folder_name_with_unread_unsent_mails(current_mode_num);
	}

	/*Update the appropriate menus & tool bar buttons */
	ui_update_folder_specific_widgets((guint) current_mode_num);

	ui_on_folderview_focussed(NULL, NULL, NULL);
	ui_on_messageview_focussed(NULL, NULL, NULL);

	folder_menu_structure =
	    ui_get_folder_activated_structure(current_parent_name, current_folder_name);
	ui_select_folder_menu_item(folder_menu_structure);

	/*Update the global state object */
	ui_store_folder_selection();


	/*change the title of the column "from" for draft/outbox/sent folder
	 * to "To"*/
	if (current_mode_num == OUTBOX_MODEL_NUM || current_mode_num == DRAFT_MODEL_NUM ||
	    current_mode_num == SENT_MODEL_NUM)
		title = _("mcen_ia_column_to");
	else
		title = _("mcen_ia_column_from");
	gtk_tree_view_column_set_title(gtk_tree_view_get_column(GTK_TREE_VIEW
								(email_main_view->widgets->
								 messageListView),
								COLUMN_SEND_RECIP), title);
	/*Update the title of the date/time column to show status for outbox folder */
	if (current_mode_num == OUTBOX_MODEL_NUM)
		title = _("mcen_ia_column_messagestatus");
	else
		title = _("mcen_ia_column_date");

	gtk_tree_view_column_set_title(gtk_tree_view_get_column(GTK_TREE_VIEW
								(email_main_view->widgets->
								 messageListView),
								COLUMN_DATE_STATUS), title);


	/*If Sent folder is being populated for first time,
	   clean up old items if required */
	if (current_mode_num == SENT_MODEL_NUM && !sent_folder_cleaning_check_done) {
		ui_empty_sent_folder(FALSE);
		sent_folder_cleaning_check_done = TRUE;
	}

	g_free(current_folder_name);
	g_free(current_parent_name);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function to refresh Inbox, Outbox and the currrently displayed
folder if required. This function should be called whenever drastic file
operations are done by engine like in the case of account deletion and renaming
- where all message files related to that account are deleted or renamed.

@param  void
@return void
*/
void ui_refresh_required_folders()
{
	GtkTreeModel *list_model = NULL;
	GtkTreeSelection *selection = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (email_main_view == NULL
	    || email_main_view->sorted_model_list == NULL
	    || email_main_view->widgets == NULL || email_main_view->widgets->folderView == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Refresh Inbox */
	email_main_view->message_handler->store_all_headers = TRUE;
	ui_populate_inbox();
	email_main_view->message_handler->store_all_headers = FALSE;

	/*Clear and Refresh Outbox */
	list_model = ui_get_unsorted_folder_model(OUTBOX_MODEL_NUM);
	gtk_list_store_clear(GTK_LIST_STORE(list_model));
	ui_populate_folder(OUTBOX_DIR, OUTBOX_MODEL_NUM);

	/*Refresh the current view */
	selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	ui_folderview_selection_changed_cb(selection, NULL);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function which gets called whenever main view
is topped from the task navigator.

@param  hildon_app  Pointer to the HildonApp object on which signal is called
@param  data        Pointer to user data (not used)
@return void
*/
static void ui_mainview_topped_cb(HildonApp * hildon_app, gpointer data)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	ui_tasknav_top_msg_cb(NULL);
	ui_on_messageview_focussed(NULL, NULL, NULL);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used whenever window manager's close
option('X' icon on top right hand corner of any window) is clicked.

@param  widget     Pointer to window widget
@param  data       Pointer to user data (not used here)
@return gboolean   Always return TRUE so that gtk doesn't destroy the whole
                   window, which is the default behaviour
*/
static gboolean ui_destroy_current_view(GtkWidget * widget, gpointer data)
{
	HildonAppView *view = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (widget == NULL || email_main_view == NULL || email_main_view->tags == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return TRUE;
	}

	if (email_main_view->tags->main_view_topped == FALSE) {
		ULOG_DEBUG_F("Destroying current view.\n");
		view = hildon_app_get_appview(app);
		g_signal_emit_by_name(view, "close");
	} else {
		ULOG_DEBUG_F("Destroying all views.\n");
		ui_on_close_all_windows_activate(widget, NULL);
	}
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return TRUE;
}

/**
 This function sets the fullscreen state of all the open windows.
 @return void,
*/
static void ui_toggle_fullscreen(void)
{
	GtkWidget *open_appview = NULL;
	gboolean full_screen_state = TRUE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM(email_main_view->tags == NULL, "param check failed", return);

	/*set the full_screen_mode tag */
	open_appview = (GtkWidget *) hildon_app_get_appview(app);

	full_screen_state = hildon_appview_get_fullscreen((HildonAppView *) open_appview);
	if (full_screen_state == 0) {
		email_main_view->tags->full_screen_mode = 1;
	} else {
		email_main_view->tags->full_screen_mode = 0;
	}
	/*signal to toggle the toolbar status once the fullscreen is toggled */
	if (email_main_view->tags->main_view_topped == FALSE) {
		g_signal_emit_by_name(open_appview, "toggle_fullscreen");
	} else {
		ui_on_full_screen_activate(NULL, appview);
	}


	return;
}

/**
 This is help function to take appropriate action for harwre key events.
 @param keyval is the event->keyval.
 @param viewer is the appview pointer of current view .
 @return gboolean TRUE.-to stop the event from propogation.
                  FALSE-else.
*/
static gboolean action_for_key_values(guint keyval, HildonAppView * viewer)
{

	HildonAppView *view = NULL;
	gboolean stop_event_propogation = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	switch (keyval) {
	case GDK_F7:
	case GDK_F8:
		CHECK_PARAM((email_main_view->tags->main_view_topped == TRUE),
			    "Ignoring zooming hardkeys for main view", return FALSE);
		ULOG_DEBUG_F("zooming current view.\n");
		if (viewer == NULL)
			view = hildon_app_get_appview(app);
		else
			view = viewer;
		if (keyval == GDK_F7) {
			g_signal_emit_by_name(view, "increase_zoom");
		} else if (keyval == GDK_F8) {
			g_signal_emit_by_name(view, "decrease_zoom");
		}
		break;

	case GDK_F6:
		ui_toggle_fullscreen();
		break;

	case GDK_Left:
	case GDK_Right:
		CHECK_PARAM((email_main_view->tags->main_view_topped == FALSE),
			    "Ignoring left/right hardkeys for other views", return FALSE);
		if (email_main_view->tags->folderview_has_focus == TRUE) {
			if (keyval == GDK_Right
			    && GTK_WIDGET_VISIBLE(email_main_view->widgets->no_msgs_label) ==
			    FALSE) {
				gtk_widget_grab_focus(email_main_view->widgets->messageListView);
			}
			stop_event_propogation = TRUE;
		} else if (email_main_view->tags->messageview_has_focus == TRUE) {
			if (keyval == GDK_Left
			    && GTK_WIDGET_VISIBLE(email_main_view->widgets->folder_scrollwindow) ==
			    TRUE) {
				gtk_widget_grab_focus(email_main_view->widgets->folderView);
			}
			stop_event_propogation = TRUE;
		}
		break;

	default:
		break;
	}
	return stop_event_propogation;
}

/**
This function is used to
        1.zoom the editor/viewer by hardware keys
            i.e GDK_f7/GDK_F8 for increase /decrease of zoom.
        2.switch to fullscreen i.e GDK_F6

@param widget is the main app that recieved the signal.
@param event is the key-press event.
@param data is the user define data.

@retrun TRUE-to stop the propagation of the event,
        FASLE-else.
*/
static gboolean ui_handle_key_events_cb(GtkWidget * widget, GdkEventKey * event, gpointer data)
{
	gboolean stop_event_propogation = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	CHECK_PARAM((widget == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->tags == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return FALSE);

	stop_event_propogation = action_for_key_values(event->keyval, NULL);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return stop_event_propogation;
}

/**
This function is used to hadle the GDK_Esc on the current view

@param widget is the main app that recieved the signal.
@param event is the key-press event.
@param data is the user define data.

@retrun TRUE-to stop the propagation of the event,
        FASLE-else.
*/
static gboolean
ui_on_current_view_escape_key_pressed(GtkWidget * widget, GdkEventKey * event, gpointer data)
{
	HildonAppView *view = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input and global params */
	if (widget == NULL || email_main_view->tags == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return FALSE;
	}

	if (event->keyval == GDK_Escape && email_main_view->tags->main_view_topped == FALSE) {
		if (fired_on_note == TRUE) {
			fired_on_note = FALSE;
			return TRUE;
		}
		ULOG_DEBUG_F("Destroying current view.\n");
		view = hildon_app_get_appview(app);
		g_signal_emit_by_name(view, "close");
		return TRUE;
	}
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return FALSE;
}

/**
This is a signal handler function called when UI goes to background.
It saves current UI state and sets the killable flag, so that UI can
be killed by task navigator if needed. Note that this will be done only
if there are no complex dialogs (any dialog that needs user input)
open when UI goes to background.

@param  widget     Pointer to window widget
@param  data       Pointer to HildonApp widget
*/
void ui_topmost_status_lost(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	if (!email_main_view->tags->state_save_disabled) {
		ULOG_DEBUG_F("Saving state and setting killable flag");
		ui_store_marked_msgs();
		ui_store_current_state_info();
		hildon_app_set_killable(HILDON_APP(data), TRUE);
	} else {
		ULOG_DEBUG_F("Complex modal dialog is open or send-recv is happening.\
		      State save is not done");
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function called when UI comes to foreground.
It resets current UI state to NULL and resets the killable flag, so
that UI cannot be killed by task navigator.

@param  widget     Pointer to window widget
@param  data       Pointer to HildonApp widget
*/
void ui_topmost_status_acquired(GtkWidget * widget, gpointer data)
{
	static gboolean ui_topped_first_time = TRUE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Take care that we do not over write the state object
	   when UI comes up for the first time */
	if (!ui_topped_first_time) {
		ui_dbus_statesave_write(NULL);
	}
	ui_topped_first_time = FALSE;

	hildon_app_set_killable(HILDON_APP(data), FALSE);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a signal handler function used when Search_Emails menu is clicked.
It will invoke Global Search application with searching option set to emails.

@param  widget  Pointer to the widget on which this function is called
@param  data    Pointer to the user data (not used)
@return void
*/
void ui_on_search_emails(GtkWidget * widget, gpointer data)
{
	osso_return_t retval = OSSO_ERROR;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	retval =
	    osso_rpc_run_with_defaults(ui_context, "osso_global_search",
				       "search_email", NULL, DBUS_TYPE_INVALID);

	if (retval != OSSO_OK) {
		show_dimming_note(widget, "Error invoking search application");
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is callback function which will be invoked from help from main menu.
Help text will be shown depending on the folder which is currently displayed.
@param widget widget pointer on which it is called.
@param data Pointer to user data.
@return Nothing.
*/
static void ui_on_help(GtkWidget * widget, gpointer data)
{
	guint current_model = 0;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	current_model = email_main_view->folder_manager->currently_displayed_model;

	if (current_model == email_main_view->tags->archive_model_num) {
		ui_on_view_help_activate(widget, HELP_ID_ARCHIVE);
		return;
	} else {
		switch (current_model) {
		case INBOX_MODEL_NUM:
			ui_on_view_help_activate(widget, HELP_ID_INBOX);
			break;

		case OUTBOX_MODEL_NUM:
			ui_on_view_help_activate(widget, HELP_ID_OUTBOX);
			break;

		case DRAFT_MODEL_NUM:
			ui_on_view_help_activate(widget, HELP_ID_DRAFTS);
			break;

		case SENT_MODEL_NUM:
			ui_on_view_help_activate(widget, HELP_ID_SENT);
			break;

		default:
			ui_on_view_help_activate(widget, HELP_ID_USERFOLDER);
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is callback function which will be invoked from help from popup menu.
Help text will be shown depending on the folder for which popup is selected.
@param widget widget pointer on which it is called.
@param data Pointer to user data.
@return Nothing.
*/
static void ui_on_popup_help_activate(GtkWidget * widget, gpointer data)
{
	guint current_model = 0;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	current_model = email_main_view->folder_manager->currently_popup_model;

	if (current_model == email_main_view->tags->archive_model_num) {
		ui_on_view_help_activate(widget, HELP_ID_ARCHIVE);
		return;
	} else {
		switch (current_model) {
		case INBOX_MODEL_NUM:
			ui_on_view_help_activate(widget, HELP_ID_INBOX);
			break;

		case OUTBOX_MODEL_NUM:
			ui_on_view_help_activate(widget, HELP_ID_OUTBOX);
			break;

		case DRAFT_MODEL_NUM:
			ui_on_view_help_activate(widget, HELP_ID_DRAFTS);
			break;

		case SENT_MODEL_NUM:
			ui_on_view_help_activate(widget, HELP_ID_SENT);
			break;

		default:
			ui_on_view_help_activate(widget, HELP_ID_USERFOLDER);
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is callback function which will be invoked upon insensitive press on items of manage folder menu items.
@param widget widget pointer on which it is called.
@param main_view is pointer to MainViewInfo struct.
@return Nothing.
*/
static void manage_folder_items_dimming_note(GtkWidget * menu, MainViewInfo * main_view)
{
	guint current_model = 0;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	current_model = email_main_view->folder_manager->currently_displayed_model;

	switch (current_model) {
	case INBOX_MODEL_NUM:
		break;

	case OUTBOX_MODEL_NUM:
		if ((menu == main_view->menu->folders_new_folder)
		    || (menu == main_view->folder_popup->new_folder))
			gtk_infoprint((GtkWindow *) app,
				      _("mail_ib_unable_to_create_folder_outbox"));
		break;

	case DRAFT_MODEL_NUM:
		if ((menu == main_view->menu->folders_new_folder)
		    || (menu == main_view->folder_popup->new_folder))
			gtk_infoprint((GtkWindow *) app,
				      _("mail_ib_unable_to_create_folder_drafts"));
		break;

	case SENT_MODEL_NUM:
		if ((menu == main_view->menu->folders_new_folder)
		    || (menu == main_view->folder_popup->new_folder))
			gtk_infoprint((GtkWindow *) app, _("mail_ib_unable_to_create_folder_sent"));
		break;

	default:
		break;
	}

	if (current_model == email_main_view->tags->archive_model_num) {
		if ((menu == main_view->menu->folders_new_folder)
		    || (menu == main_view->folder_popup->new_folder))
			gtk_infoprint((GtkWindow *) app, _("mail_in_ui_folder_create_error"));

	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to construct a new MainViewInfo object.
This object has to be freed when application is exiting.

@param  void
@return MainViewInfo*  Pointer to newly created MainViewInfo structure
*/
static MainViewInfo *ui_construct_main_view_info()
{
	MainViewInfo *main_view = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	main_view = g_new0(MainViewInfo, 1);
	main_view->widgets = g_new0(MainViewWidgets, 1);
	main_view->menu = g_new0(MainViewMenu, 1);
	main_view->tools = g_new0(MainViewToolbar, 1);
	main_view->message_popup = g_new0(MessageViewPopupMenu, 1);
	main_view->folder_popup = g_new0(FolderViewPopupMenu, 1);
	main_view->tags = g_new0(MainViewTags, 1);
	main_view->folder_manager = g_new0(FolderManager, 1);
	main_view->message_handler = g_new0(MessageHandler, 1);
	main_view->account_manager = g_new0(AccountManager, 1);
	main_view->undo = g_new0(MainViewUndoInfo, 1);
	main_view->icons = g_new0(MainViewIcons, 1);

	/*Initialize all required variables */
	main_view->marking_mode_widgets = NULL;
	main_view->sorted_model_list = NULL;
	main_view->account_menu_list = NULL;
	main_view->open_windows = NULL;

	main_view->menu->popup_folder_menu_group = NULL;
	main_view->tools->fullscreen_toolbar = NULL;

	main_view->tags->show_tool_bar = !(ui_get_toolbar_hidden_status());
	main_view->tags->show_fullscreen_tool_bar = !(ui_get_fullscreen_toolbar_hidden_status());
	main_view->tags->marking_mode = FALSE;
	main_view->tags->progress_bar_available = FALSE;
	main_view->tags->get_only_unretrieved_mails = FALSE;
	main_view->tags->full_screen_mode = FALSE;
	main_view->tags->main_view_topped = TRUE;
	main_view->tags->main_view_hidden = FALSE;
	main_view->tags->oom_signal_received = FALSE;
	main_view->tags->prev_oom_status = FALSE;
	main_view->tags->application_closure = FALSE;
	main_view->tags->folderview_has_focus = FALSE;
	main_view->tags->messageview_has_focus = FALSE;
	main_view->tags->is_msg_being_sent = FALSE;
	main_view->tags->msg_download_in_progress = FALSE;
	main_view->tags->state_save_disabled = FALSE;
	main_view->tags->current_mode = 0;
	main_view->tags->send_receive_sig_handler = 0;
	main_view->tags->window_count = 0;
	main_view->tags->current_msg_count = 0;
	main_view->tags->total_msg_count = 0;
	main_view->tags->archive_model_num = DEFAULT_ARCHIVE_MODEL_NUM;
	main_view->tags->reply_indicator = g_strconcat(_("mail_va_re"), SUBJECT_SEPERATOR, NULL);
	main_view->tags->fwd_indicator = g_strconcat(_("mail_va_fw"), SUBJECT_SEPERATOR, NULL);
	main_view->tags->reply_offset = strlen(main_view->tags->reply_indicator);
	main_view->tags->fwd_offset = strlen(main_view->tags->fwd_indicator);
	main_view->tags->normal_screen_width = 0;
	main_view->tags->check_name_unresolved = TRUE;

	main_view->folder_manager->model_seq_num = DEFAULT_MODEL_NUM;
	main_view->folder_manager->currently_displayed_model = 0;
	main_view->folder_manager->folder_msg_count = DEFAULT_MSG_COUNT;
	main_view->folder_manager->folder_sort_order = 0;
	main_view->folder_manager->is_folder_name_taken = FALSE;
	main_view->folder_manager->is_folder_present = FALSE;
	main_view->folder_manager->searched_folder_name = NULL;
	main_view->folder_manager->parent_folder_name = NULL;
	main_view->folder_manager->folder_view_model = NULL;
	main_view->folder_manager->folder_progress_bar = NULL;
	main_view->folder_manager->folder_progress_note = NULL;
	main_view->folder_manager->move_to_dialog = NULL;

	main_view->message_handler->currently_selected_row = DEFAULT_SELECTED_ROW;
	main_view->message_handler->set_action_after_retrieval = NO_ACTION_AFTER_RETRIEVAL;
	main_view->message_handler->get_action_after_retrieval = NO_ACTION_AFTER_RETRIEVAL;
	main_view->message_handler->is_message_present = FALSE;
	main_view->message_handler->store_all_headers = FALSE;
	main_view->message_handler->unfetched_msgs_to_copy = NULL;
	main_view->message_handler->current_draft_subject = NULL;
	main_view->message_handler->msg_to_be_selected_after_del_opr = NULL;
	main_view->message_handler->delete_draft = FALSE;
	main_view->message_handler->cut_folder_num = DEFAULT_ORIG_FOLDER;

	main_view->account_manager->configured_accounts = NULL;
	main_view->account_manager->default_account = NULL;
	main_view->account_manager->current_sendrec_account = NULL;
	main_view->account_manager->action_after_acct_creation = NO_ACTION;
	main_view->account_manager->error_info1 = NULL;
	main_view->account_manager->error_info2 = NULL;
	main_view->account_manager->operation_status = NONE_OP;
	main_view->account_manager->edit_account_dialog = NULL;
	main_view->account_manager->esc_sgnl_handled = FALSE;
	main_view->account_manager->alias_name = NULL;



	main_view->undo->operation = OPER_NONE;
	main_view->undo->message_id_list = NULL;
	main_view->undo->orig_folder = DEFAULT_ORIG_FOLDER;
	main_view->undo->dest_folder = DEFAULT_DEST_FOLDER;

	main_view->icons->icon_theme = gtk_icon_theme_get_default();

	main_view->icons->inbox_icon = NULL;
	main_view->icons->outbox_icon = NULL;
	main_view->icons->drafts_icon = NULL;
	main_view->icons->sent_folder_icon = NULL;
	main_view->icons->user_folder_icon = NULL;
	main_view->icons->mmc_archive_icon = NULL;
	main_view->icons->device_icon = NULL;

	main_view->icons->hdr_low_unread_icon = NULL;
	main_view->icons->hdr_high_unread_icon = NULL;
	main_view->icons->hdr_unread_icon = NULL;
	main_view->icons->hdr_low_icon = NULL;
	main_view->icons->hdr_high_icon = NULL;
	main_view->icons->hdr_icon = NULL;

	main_view->icons->hdr_signed_low_icon = NULL;
	main_view->icons->hdr_signed_high_icon = NULL;
	main_view->icons->hdr_signed_icon = NULL;
	main_view->icons->hdr_unread_signed_low_icon = NULL;
	main_view->icons->hdr_unread_signed_high_icon = NULL;
	main_view->icons->hdr_unread_signed_icon = NULL;

	main_view->icons->hdr_encrypted_icon = NULL;
	main_view->icons->hdr_encrypted_high_icon = NULL;
	main_view->icons->hdr_encrypted_low_icon = NULL;
	main_view->icons->hdr_unread_encrypted_icon = NULL;
	main_view->icons->hdr_unread_encrypted_high_icon = NULL;
	main_view->icons->hdr_unread_encrypted_low_icon = NULL;

	main_view->icons->mail_low_unread_icon = NULL;
	main_view->icons->mail_high_unread_icon = NULL;
	main_view->icons->mail_unread_icon = NULL;
	main_view->icons->mail_low_icon = NULL;
	main_view->icons->mail_high_icon = NULL;
	main_view->icons->mail_icon = NULL;

	main_view->icons->mail_signed_unread_low_icon = NULL;
	main_view->icons->mail_signed_unread_high_icon = NULL;
	main_view->icons->mail_signed_unread_icon = NULL;
	main_view->icons->mail_signed_read_low_icon = NULL;
	main_view->icons->mail_signed_read_high_icon = NULL;
	main_view->icons->mail_signed_read_icon = NULL;

	main_view->icons->mail_encrypted_unread_low_icon = NULL;
	main_view->icons->mail_encrypted_unread_high_icon = NULL;
	main_view->icons->mail_encrypted_unread_icon = NULL;
	main_view->icons->mail_encrypted_read_high_icon = NULL;
	main_view->icons->mail_encrypted_read_low_icon = NULL;
	main_view->icons->mail_encrypted_read_icon = NULL;

	main_view->icons->attachment_icon = NULL;
	main_view->icons->check_msg_icon = NULL;
	main_view->icons->dustbin_icon = NULL;

	font_families[0] = "Sans";
        font_families[1] = "Serif";
        font_families[2] = "Monospace";
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return main_view;
}

/**
This is helper function to create and initialize 'Message' menu sub items of
email main view.

@param  main_menu  Pointer to MainViewMenu object
@return void
*/
static void ui_initialize_main_view_message_menu(MainViewMenu * main_menu)
{
	GtkMenu *menubar = NULL;
#if CHANGED_EMAIL
	GtkWidget *menu_separator = NULL;
#endif

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	menubar = hildon_appview_get_menu(appview);

	main_menu->message = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_message"));
	gtk_container_add(GTK_CONTAINER(menubar), main_menu->message);

	main_menu->message_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menu->message), main_menu->message_menu);

	main_menu->message_new_email = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_newemail"));
	gtk_widget_show(main_menu->message_new_email);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->message_new_email);

	main_menu->message_open = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_open"));
	gtk_widget_show(main_menu->message_open);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->message_open);

	main_menu->message_separator1 = gtk_menu_item_new();
	gtk_widget_show(main_menu->message_separator1);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->message_separator1);
	gtk_widget_set_sensitive(main_menu->message_separator1, FALSE);

	main_menu->message_reply = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_reply"));
	gtk_widget_show(main_menu->message_reply);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->message_reply);

	main_menu->message_reply_to_all =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_replytoall"));
	gtk_widget_show(main_menu->message_reply_to_all);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->message_reply_to_all);

	main_menu->message_resend = gtk_menu_item_new_with_mnemonic(_("mcen_me_sent_resend"));
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->message_resend);

	main_menu->message_forward = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_forward"));
	gtk_widget_show(main_menu->message_forward);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->message_forward);

	main_menu->message_cancel_sending =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_outbox_cancelsend"));
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu),
			  main_menu->message_cancel_sending);

	main_menu->message_send = gtk_menu_item_new_with_mnemonic(_("mcen_me_outbox_send"));
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->message_send);

	main_menu->message_separator2 = gtk_menu_item_new();
	gtk_widget_show(main_menu->message_separator2);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->message_separator2);
	gtk_widget_set_sensitive(main_menu->message_separator2, FALSE);

	main_menu->message_delete = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_delete"));
	gtk_widget_show(main_menu->message_delete);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->message_delete);

	main_menu->message_separator3 = gtk_menu_item_new();
	gtk_widget_show(main_menu->message_separator3);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->message_separator3);
	gtk_widget_set_sensitive(main_menu->message_separator3, FALSE);
#if CHANGED_EMAIL
	main_menu->email_send_receive =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_senandreceive"));
	gtk_widget_show(main_menu->email_send_receive);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->email_send_receive);

	main_menu->email_retrieve_mail_contents =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_retrieve_contents"));
	gtk_widget_show(main_menu->email_retrieve_mail_contents);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu),
			  main_menu->email_retrieve_mail_contents);

	main_menu->email_purge_attachments =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_purge_attachments"));
	gtk_widget_show(main_menu->email_purge_attachments);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu),
			  main_menu->email_purge_attachments);

	menu_separator = gtk_menu_item_new();
	gtk_widget_show(menu_separator);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), menu_separator);
	gtk_widget_set_sensitive(menu_separator, FALSE);
#endif
	main_menu->message_details =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_messagedetails"));
	gtk_widget_show(main_menu->message_details);
	gtk_container_add(GTK_CONTAINER(main_menu->message_menu), main_menu->message_details);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to create and initialize 'Edit' menu sub items of
email main view.

@param  main_menu  Pointer to MainViewMenu structure
@return void
*/
static void ui_initialize_main_view_edit_menu(MainViewMenu * main_menu)
{
	GtkMenu *menubar = NULL;
#if CHANGED_EMAIL
	GtkWidget *separator = NULL;
#endif
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	menubar = hildon_appview_get_menu(appview);

	main_menu->edit = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_edit"));
	gtk_container_add(GTK_CONTAINER(menubar), main_menu->edit);

	main_menu->edit_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menu->edit), main_menu->edit_menu);

	main_menu->edit_undo = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_undo"));
	gtk_widget_show(main_menu->edit_undo);
	gtk_container_add(GTK_CONTAINER(main_menu->edit_menu), main_menu->edit_undo);

	main_menu->edit_separator1 = gtk_menu_item_new();
	gtk_widget_show(main_menu->edit_separator1);
	gtk_container_add(GTK_CONTAINER(main_menu->edit_menu), main_menu->edit_separator1);
	gtk_widget_set_sensitive(main_menu->edit_separator1, FALSE);

	main_menu->edit_cut = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_cut"));
	gtk_widget_show(main_menu->edit_cut);
	gtk_container_add(GTK_CONTAINER(main_menu->edit_menu), main_menu->edit_cut);

	main_menu->edit_copy = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_copy"));
	gtk_widget_show(main_menu->edit_copy);
	gtk_container_add(GTK_CONTAINER(main_menu->edit_menu), main_menu->edit_copy);

	main_menu->edit_paste = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_paste"));
	gtk_widget_show(main_menu->edit_paste);
	gtk_container_add(GTK_CONTAINER(main_menu->edit_menu), main_menu->edit_paste);
#if CHANGED_EMAIL
	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(main_menu->edit_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

#endif
	main_menu->edit_select = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_select"));
	gtk_widget_show(main_menu->edit_select);
	gtk_container_add(GTK_CONTAINER(main_menu->edit_menu), main_menu->edit_select);

	main_menu->edit_select_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menu->edit_select),
				  main_menu->edit_select_menu);

	main_menu->edit_select_mark_all =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_markall"));
	gtk_widget_show(main_menu->edit_select_mark_all);
	gtk_container_add(GTK_CONTAINER(main_menu->edit_select_menu),
			  main_menu->edit_select_mark_all);

	main_menu->edit_select_clear_all_marks =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_clearmarks"));
	gtk_widget_show(main_menu->edit_select_clear_all_marks);
	gtk_container_add(GTK_CONTAINER(main_menu->edit_select_menu),
			  main_menu->edit_select_clear_all_marks);

	main_menu->edit_separator2 = gtk_menu_item_new();
	gtk_widget_show(main_menu->edit_separator2);
	gtk_container_add(GTK_CONTAINER(main_menu->edit_menu), main_menu->edit_separator2);
	gtk_widget_set_sensitive(main_menu->edit_separator2, FALSE);

	main_menu->edit_move_to = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_moveto"));
	gtk_widget_show(main_menu->edit_move_to);
	gtk_container_add(GTK_CONTAINER(main_menu->edit_menu), main_menu->edit_move_to);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to create and initialize 'Folders' menu sub items of
email main view.

@param  main_menu  Pointer to MainViewMenu object
@return void
*/
static void ui_initialize_main_view_folders_menu(MainViewMenu * main_menu)
{
	GtkMenu *menubar = NULL;
	GtkWidget *manage_folder_separator = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	menubar = hildon_appview_get_menu(appview);
	main_menu->folders = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_folders"));
	gtk_container_add(GTK_CONTAINER(menubar), main_menu->folders);

	main_menu->folders_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menu->folders), main_menu->folders_menu);

	main_menu->folders_new_folder =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_newfolder"));
	gtk_widget_show(main_menu->folders_new_folder);
	gtk_container_add(GTK_CONTAINER(main_menu->folders_menu), main_menu->folders_new_folder);

	main_menu->folders_manage_folders =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_manage_folders"));
	gtk_widget_show(main_menu->folders_manage_folders);
	gtk_container_add(GTK_CONTAINER(main_menu->folders_menu),
			  main_menu->folders_manage_folders);

	manage_folder_separator = gtk_menu_item_new();
	gtk_widget_show(manage_folder_separator);
	gtk_container_add(GTK_CONTAINER(main_menu->folders_menu), manage_folder_separator);
	gtk_widget_set_sensitive(manage_folder_separator, FALSE);

	main_menu->folders_details =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_folderdetails"));
	gtk_widget_show(main_menu->folders_details);
	gtk_container_add(GTK_CONTAINER(main_menu->folders_menu), main_menu->folders_details);
#if RETAINED_EMAIL
	main_menu->folders_folder_settings =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_foldersettings"));
	gtk_widget_show(main_menu->folders_folder_settings);
	gtk_container_add(GTK_CONTAINER(main_menu->folders_menu),
			  main_menu->folders_folder_settings);
#endif
	main_menu->folders_separator = gtk_menu_item_new();
	gtk_widget_show(main_menu->folders_separator);
	gtk_container_add(GTK_CONTAINER(main_menu->folders_menu), main_menu->folders_separator);
	gtk_widget_set_sensitive(main_menu->folders_separator, FALSE);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to create and initialize 'Screen' menu sub items of
email main view.

@param  main_menu  Pointer to MainViewMenu object
@return void
*/
static void ui_initialize_main_view_screen_menu(MainViewMenu * main_menu)
{
	GtkMenu *menubar = NULL;
#if CHANGED_EMAIL
	GtkWidget *separator = NULL;
#endif
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	menubar = hildon_appview_get_menu(appview);

	main_menu->view = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_view"));
	gtk_container_add(GTK_CONTAINER(menubar), main_menu->view);

	main_menu->view_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menu->view), main_menu->view_menu);

	main_menu->view_sort = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_sort"));
	gtk_widget_show(main_menu->view_sort);
	gtk_container_add(GTK_CONTAINER(main_menu->view_menu), main_menu->view_sort);

	main_menu->view_separator = gtk_menu_item_new();
	gtk_widget_show(main_menu->view_separator);
	gtk_container_add(GTK_CONTAINER(main_menu->view_menu), main_menu->view_separator);
	gtk_widget_set_sensitive(main_menu->view_separator, FALSE);
#if RETAINED_EMAIL
	main_menu->view_show_toolbar = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_toolbar"));
	gtk_widget_show(main_menu->view_show_toolbar);
	gtk_container_add(GTK_CONTAINER(main_menu->view_menu), main_menu->view_show_toolbar);

	main_menu->view_show_toolbar_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menu->view_show_toolbar),
				  main_menu->view_show_toolbar_menu);

	main_menu->view_show_toolbar_normal_screen =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_inbox_normalview"));
	gtk_widget_show(main_menu->view_show_toolbar_normal_screen);
	gtk_container_add(GTK_CONTAINER(main_menu->view_show_toolbar_menu),
			  main_menu->view_show_toolbar_normal_screen);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				       (main_menu->view_show_toolbar_normal_screen), TRUE);

	main_menu->view_show_toolbar_full_screen =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_inbox_optimizedview"));
	gtk_widget_show(main_menu->view_show_toolbar_full_screen);
	gtk_container_add(GTK_CONTAINER(main_menu->view_show_toolbar_menu),
			  main_menu->view_show_toolbar_full_screen);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				       (main_menu->view_show_toolbar_full_screen),
				       !ui_get_fullscreen_toolbar_hidden_status());
#endif
	main_menu->view_folders =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_inbox_hidefolders"));
	gtk_widget_show(main_menu->view_folders);
	gtk_container_add(GTK_CONTAINER(main_menu->view_menu), main_menu->view_folders);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(main_menu->view_folders), TRUE);

	main_menu->view_full_screen =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_inbox_fullscreen"));
	gtk_widget_show(main_menu->view_full_screen);
	gtk_container_add(GTK_CONTAINER(main_menu->view_menu), main_menu->view_full_screen);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(main_menu->view_full_screen), FALSE);

#if CHANGED_EMAIL
	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(main_menu->view_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	main_menu->view_show_toolbar = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_toolbar"));
	gtk_widget_show(main_menu->view_show_toolbar);
	gtk_container_add(GTK_CONTAINER(main_menu->view_menu), main_menu->view_show_toolbar);

	main_menu->view_show_toolbar_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menu->view_show_toolbar),
				  main_menu->view_show_toolbar_menu);

	main_menu->view_show_toolbar_normal_screen =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_inbox_normalview"));
	gtk_widget_show(main_menu->view_show_toolbar_normal_screen);
	gtk_container_add(GTK_CONTAINER(main_menu->view_show_toolbar_menu),
			  main_menu->view_show_toolbar_normal_screen);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				       (main_menu->view_show_toolbar_normal_screen), TRUE);

	main_menu->view_show_toolbar_full_screen =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_inbox_optimizedview"));
	gtk_widget_show(main_menu->view_show_toolbar_full_screen);
	gtk_container_add(GTK_CONTAINER(main_menu->view_show_toolbar_menu),
			  main_menu->view_show_toolbar_full_screen);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				       (main_menu->view_show_toolbar_full_screen),
				       !ui_get_fullscreen_toolbar_hidden_status());
#endif
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

#if RETAINED_EMAIL
/**
This is helper function to create and initialize 'Email' menu sub items of
email main view.

@param  main_menu  Pointer to MainViewMenu object
@return void
*/
static void ui_initialize_main_view_email_menu(MainViewMenu * main_menu)
{
	GtkMenu *menubar = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	menubar = hildon_appview_get_menu(appview);

	main_menu->email = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_email"));
	gtk_container_add(GTK_CONTAINER(menubar), main_menu->email);

	main_menu->email_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menu->email), main_menu->email_menu);

	main_menu->email_send_receive =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_senandreceive"));
	gtk_widget_show(main_menu->email_send_receive);
	gtk_container_add(GTK_CONTAINER(main_menu->email_menu), main_menu->email_send_receive);

	main_menu->email_retrieve_mail_contents =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_retrieve_contents"));
	gtk_widget_show(main_menu->email_retrieve_mail_contents);
	gtk_container_add(GTK_CONTAINER(main_menu->email_menu),
			  main_menu->email_retrieve_mail_contents);

	main_menu->email_purge_attachments =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_purge_attachments"));
	gtk_widget_show(main_menu->email_purge_attachments);
	gtk_container_add(GTK_CONTAINER(main_menu->email_menu), main_menu->email_purge_attachments);


	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}
#endif
#if CHANGED_EMAIL
/**
This is helper function to create and initialize 'account' menu sub items of
email main view.

@param  main_menu  Pointer to MainViewMenu object
@return void
*/
static void ui_initialize_main_view_account_menu(MainViewMenu * main_menu)
{
	GtkMenu *menubar = NULL;
	GtkWidget *separator = NULL;

	menubar = hildon_appview_get_menu(appview);
	main_menu->email = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_accounts"));
	gtk_container_add(GTK_CONTAINER(menubar), main_menu->email);

	main_menu->email_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menu->email), main_menu->email_menu);

	main_menu->tools_new_e_mail_account =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_newaccount"));
	gtk_widget_show(main_menu->tools_new_e_mail_account);
	gtk_container_add(GTK_CONTAINER(main_menu->email_menu),
			  main_menu->tools_new_e_mail_account);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(main_menu->email_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	main_menu->tools_account_settings =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_accountsettings"));
	gtk_widget_show(main_menu->tools_account_settings);
	gtk_container_add(GTK_CONTAINER(main_menu->email_menu), main_menu->tools_account_settings);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}
#endif

/**
This is helper function to create and initialize 'Tools' menu sub items of
email main view.

@param  main_menu  Pointer to MainViewMenu object
@return void
*/
static void ui_initialize_main_view_tools_menu(MainViewMenu * main_menu)
{
	GtkMenu *menubar = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	menubar = hildon_appview_get_menu(appview);

	main_menu->tools = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_tools"));
	gtk_container_add(GTK_CONTAINER(menubar), main_menu->tools);

	main_menu->tools_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menu->tools), main_menu->tools_menu);

#if RETAINED_EMAIL
	main_menu->tools_addressbook =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_open_addressbook"));
	gtk_widget_show(main_menu->tools_addressbook);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_addressbook);


	main_menu->tools_separator1 = gtk_menu_item_new();
	gtk_widget_show(main_menu->tools_separator1);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_separator1);
	gtk_widget_set_sensitive(main_menu->tools_separator1, FALSE);

	main_menu->tools_new_e_mail_account =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_newaccount"));
	gtk_widget_show(main_menu->tools_new_e_mail_account);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu),
			  main_menu->tools_new_e_mail_account);

	main_menu->tools_account_settings =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_accountsettings"));
	gtk_widget_show(main_menu->tools_account_settings);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_account_settings);

	main_menu->tools_options = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_options"));
	gtk_widget_show(main_menu->tools_options);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_options);

#else

	main_menu->tools_options = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_options"));
	gtk_widget_show(main_menu->tools_options);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_options);

	main_menu->tools_separator1 = gtk_menu_item_new();
	gtk_widget_show(main_menu->tools_separator1);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_separator1);
	gtk_widget_set_sensitive(main_menu->tools_separator1, FALSE);

	main_menu->tools_addressbook =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_open_addressbook"));
	gtk_widget_show(main_menu->tools_addressbook);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_addressbook);

	main_menu->tools_signatures =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_signatures"));
	gtk_widget_show(main_menu->tools_signatures);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_signatures);

	main_menu->tools_font = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_fonts"));
	gtk_widget_show(main_menu->tools_font);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_font);
#endif
	main_menu->tools_separator4 = gtk_menu_item_new();
	gtk_widget_show(main_menu->tools_separator4);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_separator4);
	gtk_widget_set_sensitive(main_menu->tools_separator4, FALSE);

	main_menu->tools_search_e_mails =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_searchemails"));
	gtk_widget_show(main_menu->tools_search_e_mails);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_search_e_mails);

	main_menu->tools_separator5 = gtk_menu_item_new();
	gtk_widget_show(main_menu->tools_separator5);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_separator5);
	gtk_widget_set_sensitive(main_menu->tools_separator5, FALSE);

	main_menu->tools_help = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_help"));
	gtk_widget_show(main_menu->tools_help);
	gtk_container_add(GTK_CONTAINER(main_menu->tools_menu), main_menu->tools_help);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to create and initialize 'Close' menu sub items of
email main view.

@param  main_menu  Pointer to MainViewMenu object
@return void
*/
static void ui_initialize_main_view_close_menu(MainViewMenu * main_menu)
{
	GtkMenu *menubar = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	menubar = hildon_appview_get_menu(appview);

	main_menu->close = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_close"));
	gtk_container_add(GTK_CONTAINER(menubar), main_menu->close);

	main_menu->close_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menu->close), main_menu->close_menu);

	main_menu->close_window = gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_close_window"));
	gtk_widget_show(main_menu->close_window);
	gtk_container_add(GTK_CONTAINER(main_menu->close_menu), main_menu->close_window);

	main_menu->close_all_windows =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_inbox_close_windows"));
	gtk_widget_show(main_menu->close_all_windows);
	gtk_container_add(GTK_CONTAINER(main_menu->close_menu), main_menu->close_all_windows);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to create and initialize message view popup menu items
of email main view.

@param  message_popup  Pointer to MessageViewPopupMenu object
@return void
*/
static void ui_initialize_message_view_popup_menu(MessageViewPopupMenu * message_popup)
{
#if CHANGED_EMAIL
	GtkWidget *separator = NULL;
#endif
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	message_popup->pop_up_menu = gtk_menu_new();

	message_popup->open = gtk_menu_item_new_with_label(_("mcen_me_inbox_open"));
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), message_popup->open);

#if CHANGED_EMAIL
	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);
#endif

	message_popup->reply = gtk_menu_item_new_with_label(_("mcen_me_inbox_reply"));
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), message_popup->reply);

	message_popup->reply_to_all = gtk_menu_item_new_with_label(_("mcen_me_inbox_replytoall"));
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), message_popup->reply_to_all);

	message_popup->resend = gtk_menu_item_new_with_label(_("mcen_me_sent_resend"));
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), message_popup->resend);

	message_popup->forward = gtk_menu_item_new_with_label(_("mcen_me_inbox_forward"));
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), message_popup->forward);

	message_popup->cancel_sending =
	    gtk_menu_item_new_with_label(_("mcen_me_outbox_cancelsend"));
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), message_popup->cancel_sending);

	message_popup->send = gtk_menu_item_new_with_label(_("mcen_me_outbox_send"));
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), message_popup->send);

#if CHANGED_EMAIL
	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);
#endif
	message_popup->copy = gtk_menu_item_new_with_label(_("mcen_me_inbox_copy"));
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), message_popup->copy);

	message_popup->paste = gtk_menu_item_new_with_label(_("mcen_me_inbox_paste"));
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), message_popup->paste);

	message_popup->move_to = gtk_menu_item_new_with_label(_("mcen_me_inbox_moveto"));
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), message_popup->move_to);

#if CHANGED_EMAIL
	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);
#endif
	message_popup->delete_msg = gtk_menu_item_new_with_label(_("mcen_me_inbox_delete"));
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), message_popup->delete_msg);

#if CHANGED_EMAIL
	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);
#endif
	message_popup->help = gtk_menu_item_new_with_label(_("mcen_me_inbox_help"));
	gtk_container_add(GTK_CONTAINER(message_popup->pop_up_menu), message_popup->help);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to connect signals to message view popup menu items
of email main view.

@param  main_view  Pointer to MainViewInfo object
@return void
*/
static void ui_connect_message_view_popup_menu_signals(MainViewInfo * main_view)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	g_signal_connect(main_view->message_popup->open, "activate",
			 G_CALLBACK(ui_on_open1_activate), main_view->widgets->messageListView);
	g_signal_connect(main_view->message_popup->open, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->message_popup->reply, "activate",
			 G_CALLBACK(ui_on_reply1_activate), main_view->widgets->messageListView);
	g_signal_connect(main_view->message_popup->reply, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->message_popup->reply_to_all, "activate",
			 G_CALLBACK(ui_on_reply_to_all1_activate),
			 main_view->widgets->messageListView);
	g_signal_connect(main_view->message_popup->reply_to_all, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->message_popup->forward, "activate",
			 G_CALLBACK(ui_on_forward1_activate), main_view->widgets->messageListView);
	g_signal_connect(main_view->message_popup->forward, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);

	g_signal_connect(main_view->message_popup->copy, "activate",
			 G_CALLBACK(ui_on_copy1_activate), NULL);
	g_signal_connect(main_view->message_popup->move_to, "activate",
			 G_CALLBACK(ui_on_move_to1_activate), NULL);
	g_signal_connect(main_view->message_popup->move_to, "insensitive_press",
			 G_CALLBACK(ui_move_to_cut_insensitive_command_accessed), NULL);

	g_signal_connect(main_view->message_popup->paste, "activate",
			 G_CALLBACK(ui_on_paste1_activate), NULL);
	g_signal_connect(main_view->message_popup->paste, "insensitive_press",
			 G_CALLBACK(ui_on_paste1_activate), NULL);

	g_signal_connect(main_view->message_popup->delete_msg, "activate",
			 G_CALLBACK(ui_on_delete1_activate), GINT_TO_POINTER(SENSITIVE_SIGNAL_NUM));
	g_signal_connect(main_view->message_popup->delete_msg, "insensitive_press",
			 G_CALLBACK(ui_on_delete1_activate),
			 GINT_TO_POINTER(INSENSITIVE_SIGNAL_NUM));
	g_signal_connect(main_view->message_popup->resend, "activate",
			 G_CALLBACK(ui_on_resend_activate), main_view->widgets->messageListView);
	g_signal_connect(main_view->message_popup->resend, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->message_popup->cancel_sending, "activate",
			 G_CALLBACK(ui_on_cancel_send_activate), NULL);
	g_signal_connect(main_view->message_popup->cancel_sending, "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_nothing_to_cancel"));
	g_signal_connect(main_view->message_popup->send, "activate",
			 G_CALLBACK(ui_on_send_activate), main_view->widgets->messageListView);
	g_signal_connect(main_view->message_popup->send, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->message_popup->help, "activate", G_CALLBACK(ui_on_help), NULL);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to create and initialize folder view popup menu items
of email main view.

@param  folder_popup  Pointer to FolderViewPopupMenu object
@return void
*/
static void ui_initialize_folder_view_popup_menu(FolderViewPopupMenu * folder_popup)
{
#if CHANGED_EMAIL
	GtkWidget *separator = NULL;
#endif
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	folder_popup->pop_up_menu = gtk_menu_new();

	folder_popup->open = gtk_menu_item_new_with_label(_("mcen_me_inbox_open"));
	gtk_widget_show(folder_popup->open);
	gtk_container_add(GTK_CONTAINER(folder_popup->pop_up_menu), folder_popup->open);

	folder_popup->new_folder = gtk_menu_item_new_with_label(_("mcen_me_inbox_newfolder"));
	gtk_widget_show(folder_popup->new_folder);
	gtk_container_add(GTK_CONTAINER(folder_popup->pop_up_menu), folder_popup->new_folder);

#if CHANGED_EMAIL
	folder_popup->rename_folder = gtk_menu_item_new_with_label(_("mcen_me_user_renamefolder"));
	gtk_widget_show(folder_popup->rename_folder);
	gtk_container_add(GTK_CONTAINER(folder_popup->pop_up_menu), folder_popup->rename_folder);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(folder_popup->pop_up_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);
#endif
	folder_popup->paste = gtk_menu_item_new_with_label(_("mcen_me_inbox_paste"));
	gtk_widget_show(folder_popup->paste);
	gtk_container_add(GTK_CONTAINER(folder_popup->pop_up_menu), folder_popup->paste);

#if RETAINED_EMAIL
	folder_popup->rename_folder = gtk_menu_item_new_with_label(_("mcen_me_user_renamefolder"));
	gtk_widget_show(folder_popup->rename_folder);
	gtk_container_add(GTK_CONTAINER(folder_popup->pop_up_menu), folder_popup->rename_folder);
#else
	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(folder_popup->pop_up_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);
#endif

	folder_popup->delete_msg = gtk_menu_item_new_with_label(_("mcen_me_user_deletefolder"));
	gtk_widget_show(folder_popup->delete_msg);
	gtk_container_add(GTK_CONTAINER(folder_popup->pop_up_menu), folder_popup->delete_msg);

#if CHANGED_EMAIL
	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(folder_popup->pop_up_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);
#endif
	folder_popup->search = gtk_menu_item_new_with_label(_("mcen_me_inbox_searchemails"));
	gtk_widget_show(folder_popup->search);
	gtk_container_add(GTK_CONTAINER(folder_popup->pop_up_menu), folder_popup->search);

	folder_popup->help = gtk_menu_item_new_with_label(_("mcen_me_inbox_help"));
	gtk_widget_show(folder_popup->help);
	gtk_container_add(GTK_CONTAINER(folder_popup->pop_up_menu), folder_popup->help);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to connect signals to folder view popup menu items
of email main view.

@param  main_view  Pointer to MainViewInfo object
@return void
*/
static void ui_connect_folder_view_popup_menu_signals(MainViewInfo * main_view)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	g_signal_connect(main_view->folder_popup->open, "activate",
			 G_CALLBACK(ui_folder_menu_item_open_activate),
			 main_view->widgets->folderView);

	g_signal_connect(main_view->folder_popup->new_folder, "activate",
			 G_CALLBACK(ui_on_new_manage_folders_dialog), NULL);
	g_signal_connect(main_view->folder_popup->new_folder, "insensitive_press",
			 G_CALLBACK(manage_folder_items_dimming_note), main_view);

	g_signal_connect(main_view->folder_popup->paste, "activate",
			 G_CALLBACK(ui_folder_menu_item_paste_activate),
			 main_view->widgets->folderView);
	g_signal_connect(main_view->folder_popup->paste, "insensitive_press",
			 G_CALLBACK(ui_folder_menu_item_paste_activate),
			 main_view->widgets->folderView);

	g_signal_connect(main_view->folder_popup->rename_folder, "activate",
			 G_CALLBACK(ui_folder_menu_item_rename_folder_activate),
			 main_view->widgets->folderView);

	g_signal_connect(main_view->folder_popup->delete_msg, "activate",
			 G_CALLBACK(ui_folder_menu_item_delete_activate),
			 main_view->widgets->folderView);

	g_signal_connect(main_view->folder_popup->search, "activate",
			 G_CALLBACK(ui_on_search_emails), NULL);

	g_signal_connect(main_view->folder_popup->help, "activate",
			 G_CALLBACK(ui_on_popup_help_activate), NULL);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to connect signals to menu items of email main view.

@param  main_view  Pointer to MainViewInfo object
@return void
*/
static void ui_connect_main_view_menu_signals(MainViewInfo * main_view)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	g_signal_connect(main_view->menu->message_new_email, "activate",
			 G_CALLBACK(ui_on_new_email_activate), NULL);
	g_signal_connect(main_view->menu->message_open, "activate",
			 G_CALLBACK(ui_on_open1_activate), main_view->widgets->messageListView);
	g_signal_connect(main_view->menu->message_open, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->menu->message_reply, "activate",
			 G_CALLBACK(ui_on_reply1_activate), main_view->widgets->messageListView);
	g_signal_connect(main_view->menu->message_reply, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->menu->message_resend, "activate",
			 G_CALLBACK(ui_on_resend_activate), main_view->widgets->messageListView);
	g_signal_connect(main_view->menu->message_resend, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->menu->message_reply_to_all, "activate",
			 G_CALLBACK(ui_on_reply_to_all1_activate),
			 main_view->widgets->messageListView);
	g_signal_connect(main_view->menu->message_reply_to_all, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->menu->message_forward, "activate",
			 G_CALLBACK(ui_on_forward1_activate), main_view->widgets->messageListView);
	g_signal_connect(main_view->menu->message_forward, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->menu->message_cancel_sending, "activate",
			 G_CALLBACK(ui_on_cancel_send_activate), NULL);
	g_signal_connect(main_view->menu->message_cancel_sending, "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_nothing_to_cancel"));
	g_signal_connect(main_view->menu->message_send, "activate", G_CALLBACK(ui_on_send_activate),
			 main_view->widgets->messageListView);
	g_signal_connect(main_view->menu->message_send, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);

	g_signal_connect(main_view->menu->message_delete, "activate",
			 G_CALLBACK(ui_on_delete1_activate), GINT_TO_POINTER(SENSITIVE_SIGNAL_NUM));
	g_signal_connect(main_view->menu->message_delete, "insensitive_press",
			 G_CALLBACK(ui_on_delete1_activate),
			 GINT_TO_POINTER(INSENSITIVE_SIGNAL_NUM));

	g_signal_connect(main_view->menu->message_details, "activate",
			 G_CALLBACK(ui_on_message_properties_clicked), NULL);
	g_signal_connect(main_view->menu->message_details, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);

	g_signal_connect(main_view->menu->edit, "activate", G_CALLBACK(ui_on_edit_activate), NULL);
	g_signal_connect(main_view->menu->edit_cut, "activate", G_CALLBACK(ui_on_cut1_activate),
			 NULL);
	g_signal_connect(main_view->menu->edit_cut, "insensitive_press",
			 G_CALLBACK(ui_move_to_cut_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->menu->edit_copy, "activate", G_CALLBACK(ui_on_copy1_activate),
			 NULL);
	g_signal_connect(main_view->menu->edit_copy, "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_nothing_to_copy"));
	g_signal_connect(main_view->menu->edit_paste, "activate", G_CALLBACK(ui_on_paste1_activate),
			 NULL);
	g_signal_connect(main_view->menu->edit_paste, "insensitive_press",
			 G_CALLBACK(ui_on_paste1_activate), NULL);
	g_signal_connect(main_view->menu->edit_select_mark_all, "activate",
			 G_CALLBACK(ui_on_mark_all1_activate), main_view->widgets->messageListView);
	g_signal_connect(main_view->menu->edit_select_mark_all, "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mail_ib_nothing_to_mark"));
	g_signal_connect(main_view->menu->edit_select_clear_all_marks, "activate",
			 G_CALLBACK(ui_on_clear_all_marks1_activate),
			 main_view->widgets->messageListView);
	g_signal_connect(main_view->menu->edit_select_clear_all_marks, "insensitive_press",
			 G_CALLBACK(show_dimming_note), DGETTEXT_HCS("ckdg_ib_no_marks_to_clear"));
	g_signal_connect(main_view->menu->edit_undo, "activate", G_CALLBACK(ui_on_undo_activate),
			 NULL);
	g_signal_connect(main_view->menu->edit_undo, "insensitive_press",
			 G_CALLBACK(show_dimming_note), DGETTEXT_HCS("ecoc_ib_nothing_to_undo"));

	g_signal_connect(main_view->menu->edit_move_to, "activate",
			 G_CALLBACK(ui_on_move_to1_activate), NULL);
	g_signal_connect(main_view->menu->edit_move_to, "insensitive_press",
			 G_CALLBACK(ui_move_to_cut_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->menu->folders, "activate",
			 G_CALLBACK(ui_on_folder_menu_select_activated), NULL);
	g_signal_connect(main_view->menu->folders_new_folder, "activate",
			 G_CALLBACK(ui_folder_menu_item_new_folder_activate),
			 main_view->widgets->folderView);
	g_signal_connect(main_view->menu->folders_new_folder, "insensitive_press",
			 G_CALLBACK(manage_folder_items_dimming_note), main_view);
	g_signal_connect(main_view->menu->folders_manage_folders, "activate",
			 G_CALLBACK(ui_on_manage_folders1_activate),
			 main_view->widgets->folderView);
	g_signal_connect(main_view->menu->folders_details, "activate",
			 G_CALLBACK(ui_create_folder_properties_dialog), NULL);
	g_signal_connect(main_view->menu->folders_details, "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mail_ib_no_details_inbox"));
	g_signal_connect(main_view->menu->folders_folder_settings, "activate",
			 G_CALLBACK(ui_on_folder_settings_activate), NULL);
	g_signal_connect(main_view->menu->folders_folder_settings, "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mail_ib_no_settings_inbox"));

	g_signal_connect(main_view->menu->view, "activate",
			 G_CALLBACK(ui_on_view_menu_select_activated), appview);
	g_signal_connect(main_view->menu->view_sort, "activate",
			 G_CALLBACK(ui_on_sort1_activate), main_view->widgets->folderView);
	g_signal_connect(main_view->menu->view_sort, "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_nothing_to_sort"));
	g_signal_connect(main_view->menu->view_folders, "activate",
			 G_CALLBACK(ui_on_hide_folders1_activate),
			 main_view->widgets->folder_scrollwindow);
	g_signal_connect(main_view->menu->view_full_screen, "activate",
			 G_CALLBACK(ui_on_full_screen_activate), appview);

	g_signal_connect(main_view->menu->view_show_toolbar_normal_screen,
			 "activate", G_CALLBACK(ui_on_toolbar1_activate), NULL);
	g_signal_connect(main_view->menu->view_show_toolbar_full_screen, "activate",
			 G_CALLBACK(ui_on_fullscreen_toolbar1_activate), appview);
#if CHANGED_EMAIL
	g_signal_connect(main_view->menu->message, "activate", G_CALLBACK(ui_on_email_activate),
			 NULL);
#else
	g_signal_connect(main_view->menu->email, "activate", G_CALLBACK(ui_on_email_activate),
			 NULL);
#endif
	g_signal_connect(main_view->menu->email_send_receive, "insensitive_press",
			 G_CALLBACK(ui_on_send_receive_insensitive_click), NULL);
	g_signal_connect(main_view->menu->email_send_receive, "activate",
			 G_CALLBACK(bold_default_account), NULL);
	g_signal_connect(main_view->menu->email_retrieve_mail_contents, "activate",
			 G_CALLBACK(ui_on_retrieve_mail_contents1_activate),
			 main_view->widgets->messageListView);
	g_signal_connect(main_view->menu->email_retrieve_mail_contents, "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_select_headers"));
	g_signal_connect(main_view->menu->email_purge_attachments, "activate",
			 G_CALLBACK(ui_on_purge_attachments_activate),
			 main_view->widgets->messageListView);
	g_signal_connect(main_view->menu->email_purge_attachments, "insensitive_press",
			 G_CALLBACK(ui_insensitive_purge_menu_accessed),
			 main_view->widgets->messageListView);
	g_signal_connect(main_view->menu->tools_addressbook, "activate",
			 G_CALLBACK(ui_on_addressbook_activate), NULL);

	g_signal_connect(main_view->menu->tools_new_e_mail_account, "activate",
			 G_CALLBACK(ui_on_new_e_mail_account1_activate), NULL);
	g_signal_connect(main_view->menu->tools_account_settings, "activate",
			 G_CALLBACK(ui_on_account_settings1_activate), NULL);
	g_signal_connect(main_view->menu->tools_options, "activate",
			 G_CALLBACK(ui_on_tools_options_activate), NULL);
#if CHANGED_EMAIL
	g_signal_connect(main_view->menu->tools_font, "activate",
			 G_CALLBACK(ui_on_font_setting_activate), NULL);
	g_signal_connect(main_view->menu->tools_signatures, "activate",
			 G_CALLBACK(ui_on_signatures1_activate), NULL);
#endif
	g_signal_connect(main_view->menu->tools_search_e_mails, "activate",
			 G_CALLBACK(ui_on_search_emails), NULL);
	g_signal_connect(main_view->menu->tools_help, "activate", G_CALLBACK(ui_on_help), NULL);

	g_signal_connect(main_view->menu->close_window, "activate",
			 G_CALLBACK(ui_on_close_all_windows_activate), NULL);
	g_signal_connect(main_view->menu->close_all_windows, "activate",
			 G_CALLBACK(ui_on_close_all_windows_activate), app);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to create and initialize all menu items
of email main view.

@param  main_view  Pointer to MainViewInfo object
@return void
*/
static void ui_initialize_main_view_menu_items(MainViewInfo * main_view)
{
	GtkMenu *menubar = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	ui_initialize_main_view_message_menu(main_view->menu);
	ui_initialize_main_view_edit_menu(main_view->menu);
#if CHANGED_EMAIL
	ui_initialize_main_view_screen_menu(main_view->menu);
	ui_initialize_main_view_folders_menu(main_view->menu);
	ui_initialize_main_view_account_menu(main_view->menu);
#else
	ui_initialize_main_view_folders_menu(main_view->menu);
	ui_initialize_main_view_screen_menu(main_view->menu);
	ui_initialize_main_view_email_menu(main_view->menu);
#endif
	ui_initialize_main_view_tools_menu(main_view->menu);
	ui_initialize_main_view_close_menu(main_view->menu);
	ui_initialize_message_view_popup_menu(main_view->message_popup);
	ui_initialize_folder_view_popup_menu(main_view->folder_popup);

	menubar = hildon_appview_get_menu(appview);
	gtk_widget_show_all(GTK_WIDGET(menubar));

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to create and initialize toolbar items
of email main view.

@param  tools  Pointer to MainViewToolbar object
@return void
*/
static void ui_initialize_main_view_toolbar(MainViewToolbar * tools)
{
	GtkWidget *image_new = NULL;
	GtkWidget *image_resend = NULL;
	GtkWidget *image_reply = NULL;
	GtkWidget *image_reply_all = NULL;
	GtkWidget *image_send = NULL;
	GtkWidget *image_move_to = NULL;
	GtkWidget *image_delete = NULL;
	GtkWidget *image_forward = NULL;
	GtkWidget *image_snd_rcv = NULL;
	GtkWidget *image_cancel_send = NULL;
	GtkWidget *progressbar_alignment = NULL;
	GtkWidget *progress_bar_box = NULL;
	GtkWidget *prog_stop_image = NULL;
	GtkWidget *prog_animation = NULL;
	GtkIconInfo *prog_anim_info = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	tools->toolbar = gtk_toolbar_new();
	tools->fullscreen_toolbar = NULL;
	gtk_toolbar_set_style(GTK_TOOLBAR(tools->toolbar), GTK_ORIENTATION_HORIZONTAL);

	/*left margin space toolitem */
	tools->left_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(tools->left_space, TOOLBAR_BUTTON_WIDTH / 2,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->left_space), -1);

	/*new msg toolitem */
	image_new = gtk_image_new_from_icon_name("qgn_toolb_messagin_new", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(image_new);
	tools->new_msg = GTK_WIDGET(gtk_tool_button_new(image_new, NULL));
	gtk_widget_show(tools->new_msg);
	gtk_widget_set_size_request(tools->new_msg, TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->new_msg), -1);

	/*new msg space toolitem */
	tools->new_msg_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(tools->new_msg_space, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->new_msg_space), -1);

	/*resend toolitem */
	image_resend =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_resend", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(image_resend);
	tools->resend = GTK_WIDGET(gtk_tool_button_new(image_resend, NULL));
	gtk_widget_set_size_request(tools->resend, TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->resend), -1);

	/*resend space toolitem */
	tools->resend_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(tools->resend_space, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->resend_space), -1);

	/*cancel sending toolitem */
	image_cancel_send =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_cancelsend", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(image_cancel_send);
	tools->cancel_sending = GTK_WIDGET(gtk_tool_button_new(image_cancel_send, NULL));
	gtk_widget_set_size_request(tools->cancel_sending, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->cancel_sending), -1);

	/*cancel sending toolitem */
	tools->cancel_sending_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(tools->cancel_sending_space, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->cancel_sending_space),
			   -1);

	/*send tooitem */
	image_send = gtk_image_new_from_icon_name("qgn_toolb_messagin_send", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(image_send);
	tools->send = GTK_WIDGET(gtk_tool_button_new(image_send, NULL));
	gtk_widget_set_size_request(tools->send, TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->send), -1);

	/*send space toolitem */
	tools->send_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(tools->send_space, TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->send_space), -1);

	/*reply toolitem */
	image_reply =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_reply", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(image_reply);
	tools->reply = GTK_WIDGET(gtk_tool_button_new(image_reply, NULL));
	gtk_widget_show(tools->reply);
	gtk_widget_set_size_request(tools->reply, TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->reply), -1);

	/*reply space toolitem */
	tools->reply_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(tools->reply_space, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->reply_space), -1);

	/*reply all toolitem */
	image_reply_all =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_replytoall", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(image_reply_all);
	tools->reply_to_all = GTK_WIDGET(gtk_tool_button_new(image_reply_all, NULL));
	gtk_widget_show(tools->reply_to_all);
	gtk_widget_set_size_request(tools->reply_to_all, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->reply_to_all), -1);

	/*reply all toolitem */
	tools->reply_to_all_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(tools->reply_to_all_space, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->reply_to_all_space),
			   -1);

	/*forward toolitem */
	image_forward =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_forward", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(image_forward);
	tools->forward = GTK_WIDGET(gtk_tool_button_new(image_forward, NULL));
	gtk_widget_show(tools->forward);
	gtk_widget_set_size_request(tools->forward, TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->forward), -1);

	/*forward toolitem */
	tools->forward_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(tools->forward_space, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->forward_space), -1);

	/*move to toolitem */
	image_move_to =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_moveto", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(image_move_to);
	tools->move_to = GTK_WIDGET(gtk_tool_button_new(image_move_to, NULL));
	gtk_widget_show(tools->move_to);
	gtk_widget_set_size_request(tools->move_to, TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->move_to), -1);

	/*move to space toolitem */
	tools->move_to_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(tools->move_to_space, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->move_to_space), -1);

	/*Delete toolitem */
	image_delete =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_delete", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(image_delete);
	tools->delete_msg = GTK_WIDGET(gtk_tool_button_new(image_delete, NULL));
	gtk_widget_show(tools->delete_msg);
	gtk_widget_set_size_request(tools->delete_msg, TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->delete_msg), -1);


	/*Delete space toolitem */
	tools->delete_msg_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(tools->delete_msg_space, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->delete_msg_space), -1);

	/*Send/recv toolitem */
	image_snd_rcv =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_sendreceive", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(image_snd_rcv);
	tools->send_receive = GTK_WIDGET(gtk_tool_button_new(image_snd_rcv, NULL));
	gtk_widget_show(tools->send_receive);
	gtk_widget_set_size_request(tools->send_receive, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->send_receive), -1);
	gtk_widget_tap_and_hold_setup(GTK_WIDGET(tools->send_receive), NULL, NULL,
				      GTK_TAP_AND_HOLD_NONE);

	/*send/recv space toolitem */
	tools->send_receive_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(tools->send_receive_space, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->send_receive_space),
			   -1);

	/*progress separator */
	tools->separator = GTK_WIDGET(gtk_separator_tool_item_new());
	gtk_widget_show(tools->separator);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->separator), -1);

	/*progress separator space */
	/*tools->separator_space = GTK_WIDGET(gtk_tool_item_new());
	   gtk_widget_set_size_request(tools->separator_space, TOOLBAR_BUTTON_WIDTH, 
	   TOOLBAR_BUTTON_HEIGHT);
	   gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar),GTK_TOOL_ITEM(tools->separator_space),-1); */

	/*progress bar */
	progress_bar_box = gtk_hbox_new(FALSE, HILDON_MARGIN_DEFAULT);
	prog_stop_image = gtk_image_new_from_icon_name("qgn_toolb_gene_stop", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(prog_stop_image);
	tools->progress_bar_button = GTK_WIDGET(gtk_tool_button_new(prog_stop_image, NULL));
	gtk_widget_set_size_request(tools->progress_bar_button, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->progress_bar_button),
			   -1);

	/*stop button space */
	/*tools->progress_bar_button_space = GTK_WIDGET(gtk_tool_item_new());
	   gtk_widget_set_size_request(tools->progress_bar_button_space, TOOLBAR_BUTTON_WIDTH, 
	   TOOLBAR_BUTTON_HEIGHT);
	   gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar),GTK_TOOL_ITEM(tools->progress_bar_button_space),-1); */

	/*progressbar */
	progressbar_alignment = gtk_alignment_new(XALIGN, XALIGN, 1, XSPACE);
	gtk_widget_show(progressbar_alignment);
	gtk_box_pack_start(GTK_BOX(progress_bar_box), progressbar_alignment, TRUE, TRUE, 0);

	tools->progressbar = gtk_progress_bar_new();
	gtk_progress_bar_set_ellipsize(GTK_PROGRESS_BAR(tools->progressbar), PANGO_ELLIPSIZE_END);
	gtk_container_add(GTK_CONTAINER(progressbar_alignment), tools->progressbar);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(tools->progressbar), _("mcen_me_connecting"));

	/*animation icon */
	prog_anim_info =
	    gtk_icon_theme_lookup_icon(email_main_view->icons->icon_theme,
				       "qgn_indi_pball_a", HILDON_ICON_SIZE_26,
				       GTK_ICON_LOOKUP_NO_SVG);
	if (prog_anim_info) {
		prog_animation =
		    gtk_image_new_from_file(gtk_icon_info_get_filename(prog_anim_info));
		gtk_icon_info_free(prog_anim_info);
		gtk_box_pack_start(GTK_BOX(progress_bar_box), prog_animation, FALSE, FALSE, 0);
	}

	tools->progress_item = GTK_WIDGET(gtk_tool_item_new());
	gtk_container_add(GTK_CONTAINER(tools->progress_item), progress_bar_box);
	gtk_widget_show_all(tools->progress_item);
	gtk_widget_show_all(progress_bar_box);
	gtk_toolbar_insert(GTK_TOOLBAR(tools->toolbar), GTK_TOOL_ITEM(tools->progress_item), -1);
	gtk_tool_item_set_expand(GTK_TOOL_ITEM(tools->progress_item), TRUE);
	gtk_box_pack_start(GTK_BOX(appview->vbox), tools->toolbar, FALSE, FALSE, 0);
	gtk_widget_show_all(tools->toolbar);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to connect signals to toolbar items of email main view.

@param  main_view  Pointer to MainViewInfo object
@return void
*/
static void ui_connect_main_view_toolbar_signals(MainViewInfo * main_view)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	g_signal_connect(main_view->tools->new_msg, "clicked",
			 G_CALLBACK(ui_on_new_email_activate), NULL);
	g_signal_connect(main_view->tools->reply, "clicked",
			 G_CALLBACK(ui_on_reply1_activate), main_view->widgets->messageListView);
	g_signal_connect(main_view->tools->reply, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->tools->reply_to_all, "clicked",
			 G_CALLBACK(ui_on_reply_to_all1_activate),
			 main_view->widgets->messageListView);
	g_signal_connect(main_view->tools->reply_to_all, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->tools->forward, "clicked", G_CALLBACK(ui_on_forward1_activate),
			 main_view->widgets->messageListView);
	g_signal_connect(main_view->tools->forward, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->tools->send, "clicked", G_CALLBACK(ui_on_send_activate),
			 main_view->widgets->messageListView);
	g_signal_connect(main_view->tools->send, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->tools->resend, "clicked", G_CALLBACK(ui_on_resend_activate),
			 main_view->widgets->messageListView);
	g_signal_connect(main_view->tools->resend, "insensitive_press",
			 G_CALLBACK(ui_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->tools->send_receive, "clicked",
			 G_CALLBACK(ui_on_send_receive_account_activate), NULL);
	g_signal_connect(main_view->tools->send_receive, "insensitive_press",
			 G_CALLBACK(ui_on_send_receive_insensitive_click), NULL);
	g_signal_connect_after(GTK_BIN(main_view->tools->send_receive)->child, "tap-and-hold",
			       G_CALLBACK(context_send_recieve_menu_cb), NULL);
	g_signal_connect(main_view->tools->move_to, "clicked", G_CALLBACK(ui_on_move_to1_activate),
			 NULL);
	g_signal_connect(main_view->tools->move_to, "insensitive_press",
			 G_CALLBACK(ui_move_to_cut_insensitive_command_accessed), NULL);
	g_signal_connect(main_view->tools->delete_msg, "clicked",
			 G_CALLBACK(ui_on_delete1_activate), GINT_TO_POINTER(SENSITIVE_SIGNAL_NUM));
	g_signal_connect(main_view->tools->delete_msg, "insensitive_press",
			 G_CALLBACK(ui_on_delete1_activate),
			 GINT_TO_POINTER(INSENSITIVE_SIGNAL_NUM));
	g_signal_connect(main_view->tools->cancel_sending, "clicked",
			 G_CALLBACK(ui_on_cancel_send_activate), NULL);
	g_signal_connect(main_view->tools->cancel_sending, "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_nothing_to_cancel"));
	g_signal_connect(main_view->tools->progress_bar_button, "clicked",
			 G_CALLBACK(ui_on_cancel_clicked), NULL);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to create and initialize all data widgets, and
prepare skeleton of email main view.

@param  widgets  Pointer to MainViewWidgets object
@return void
*/
static void ui_initialize_main_view_widgets(MainViewWidgets * widgets)
{
	GtkWidget *vbox1 = NULL;
	GtkWidget *vbox2 = NULL;
	GtkTargetEntry target_entries = { "text/html", GTK_TARGET_SAME_APP, 0 };
	guint ntargets = 1;
	gint screen_width = 0, screen_height = 0, folder_pane_width = 0;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	vbox1 = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox1);
	gtk_container_add(GTK_CONTAINER(appview), vbox1);

	widgets->folder_msg_pane = gtk_hpaned_new();
	gtk_widget_show(widgets->folder_msg_pane);
	gtk_box_pack_start(GTK_BOX(vbox1), widgets->folder_msg_pane, TRUE, TRUE, 0);

	widgets->folder_scrollwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(widgets->folder_scrollwindow);
	gtk_paned_add1(GTK_PANED(widgets->folder_msg_pane), widgets->folder_scrollwindow);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (widgets->folder_scrollwindow),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_set_border_width(GTK_CONTAINER(widgets->folder_scrollwindow),
				       HILDON_MARGIN_DEFAULT);
	gtk_window_get_size(GTK_WINDOW(app), &screen_width, &screen_height);
	folder_pane_width = (screen_width * FOLDER_PANE_WIDTH);
	gtk_paned_set_position(GTK_PANED(widgets->folder_msg_pane), folder_pane_width);
	email_main_view->tags->normal_screen_width = screen_width;

	widgets->folderView = ui_create_view_and_model(VIEW_FOLDER_LIST, TRUE);
	gtk_widget_show(widgets->folderView);
	gtk_container_add(GTK_CONTAINER(widgets->folder_scrollwindow), widgets->folderView);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(widgets->folderView), FALSE);

	vbox2 = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox2);
	gtk_paned_add2(GTK_PANED(widgets->folder_msg_pane), vbox2);

	widgets->no_msgs_label = gtk_label_new(_("mcen_ia_nomessages"));
	gtk_widget_show(widgets->no_msgs_label);
	gtk_box_pack_start(GTK_BOX(vbox2), widgets->no_msgs_label, FALSE, FALSE, 0);
	gtk_label_set_justify(GTK_LABEL(widgets->no_msgs_label), GTK_JUSTIFY_LEFT);

	widgets->message_scrollwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_box_pack_start(GTK_BOX(vbox2), widgets->message_scrollwindow, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (widgets->message_scrollwindow),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_set_border_width(GTK_CONTAINER(widgets->message_scrollwindow),
				       HILDON_MARGIN_DEFAULT);

	widgets->messageListView = ui_create_view_and_model(VIEW_MSG_LIST, TRUE);
	gtk_widget_show(widgets->messageListView);
	gtk_container_add(GTK_CONTAINER(widgets->message_scrollwindow), widgets->messageListView);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(widgets->messageListView), TRUE);

	gtk_drag_dest_set(widgets->folderView, GTK_DEST_DEFAULT_ALL,
			  &target_entries, ntargets, GDK_ACTION_COPY);

	gtk_drag_source_set(widgets->messageListView, GDK_BUTTON1_MASK,
			    &target_entries, ntargets, GDK_ACTION_COPY);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to connect signals to data widgets of email main
view.

@param  main_view  Pointer to MainViewInfo object
@return void
*/
static void ui_connect_main_view_widget_signals(MainViewInfo * main_view)
{
	GtkTreeSelection *folder_selection = NULL;
	GtkTreeSelection *message_selection = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	folder_selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(main_view->widgets->folderView));
	message_selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(main_view->widgets->messageListView));
	g_signal_connect(folder_selection, "changed",
			 G_CALLBACK(ui_folderview_selection_changed_cb), NULL);
	g_signal_connect(message_selection, "changed",
			 G_CALLBACK(ui_messageview_selection_changed_cb), NULL);
	g_signal_connect(main_view->widgets->messageListView, "row-activated",
			 G_CALLBACK(ui_messageview_on_row_selected), NULL);

	g_signal_connect(main_view->widgets->messageListView, "button-press-event",
			 (GCallback) ui_on_popup_message_button_pressed,
			 main_view->message_popup->pop_up_menu);
	g_signal_connect(main_view->widgets->folderView, "button-press-event",
			 (GCallback) ui_on_popup_folder_button_pressed,
			 main_view->folder_popup->pop_up_menu);
	gtk_widget_tap_and_hold_setup(main_view->widgets->folderView,
				      main_view->folder_popup->pop_up_menu, (GtkCallback)
				      ui_on_tap_and_hold_folder_popup, GTK_TAP_AND_HOLD_PASS_PRESS);
	/*Use GTK_TAP_AND_HOLD_NONE option instead of GTK_TAP_AND_HOLD_PASS_PRESS */
	gtk_widget_tap_and_hold_setup(main_view->widgets->messageListView,
				      main_view->message_popup->pop_up_menu, (GtkCallback)
				      ui_on_tap_and_hold_message_popup, GTK_TAP_AND_HOLD_NONE);
	g_signal_connect(G_OBJECT(appview), "switched_to", G_CALLBACK(ui_mainview_topped_cb), NULL);
	g_signal_connect(G_OBJECT(app), "delete_event", G_CALLBACK(ui_destroy_current_view), NULL);
	g_signal_connect(G_OBJECT(app), "key_press_event", G_CALLBACK(ui_handle_key_events_cb),
			 NULL);

	g_signal_connect(G_OBJECT(app), "key-press-event",
			 G_CALLBACK(ui_on_current_view_escape_key_pressed), NULL);

	g_signal_connect(G_OBJECT(app), "topmost_status_lose", G_CALLBACK(ui_topmost_status_lost),
			 app);
	g_signal_connect(G_OBJECT(app), "topmost_status_acquire",
			 G_CALLBACK(ui_topmost_status_acquired), app);

	gtk_signal_connect((GtkObject *) main_view->widgets->folderView,
			   "drag_data_received", (GtkSignalFunc) drag_data_received_handler, NULL);

	gtk_signal_connect((GtkObject *) main_view->widgets->messageListView,
			   "drag_begin", (GtkSignalFunc) drag_begin_handler, NULL);

	gtk_signal_connect((GtkObject *) main_view->widgets->messageListView,
			   "drag_data_get", (GtkSignalFunc) drag_data_get_handler, NULL);

	gtk_signal_connect((GtkObject *) main_view->widgets->messageListView,
			   "drag_end", (GtkSignalFunc) drag_end_handler, NULL);
	gtk_signal_connect((GtkObject *) main_view->widgets->folderView,
			   "drag_motion", (GtkSignalFunc) drag_motion_handler, NULL);

	g_signal_connect(main_view->widgets->folderView, "focus-in-event",
			 (GCallback) ui_on_folderview_focussed, NULL);
	g_signal_connect(main_view->widgets->messageListView, "focus-in-event",
			 (GCallback) ui_on_messageview_focussed, NULL);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
}

/**
This is helper function to store all menu items, toolbar items that are to
disabled/enabled when marking mode is ON/OFF.

@param  main_view  Pointer to MainViewInfo object
@return void
*/
static void ui_store_marking_mode_widgets(MainViewInfo * main_view)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->menu->message_open);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->menu->message_reply);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->menu->message_reply_to_all);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->menu->message_forward);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->menu->message_details);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->menu->edit_paste);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets,
			   main_view->menu->folders_folder_settings);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->menu->view_sort);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets,
			   main_view->menu->email_purge_attachments);

	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->tools->reply);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->tools->reply_to_all);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->tools->forward);

	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->message_popup->open);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->message_popup->reply);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->message_popup->reply_to_all);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->message_popup->forward);
	main_view->marking_mode_widgets =
	    g_slist_append(main_view->marking_mode_widgets, main_view->message_popup->paste);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/*Main Window*/

/**
This is a window creation function used to create the main UI window.

@param	void
@return HildonAppView*  Pointer to the newly created HildonAppView structure
*/
HildonAppView *ui_create_main_window(void)
{
	OpenedWindowData *open_window = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Set full screen short cut key */
	hildon_appview_set_fullscreen_key_allowed(appview, TRUE);

	/*Create MainViewInfo object */
	email_main_view = ui_construct_main_view_info();
	if (email_main_view == NULL) {
		ULOG_CRIT_F("Memory allocation failure!\n");
		return NULL;
	}

	/*Allocate memory for open_window object */
	open_window = g_new0(OpenedWindowData, 1);

	/*Initialize internal widgets, menus and toolbar items */
	ui_initialize_main_view_menu_items(email_main_view);
	ui_initialize_main_view_widgets(email_main_view->widgets);
	ui_initialize_main_view_toolbar(email_main_view->tools);

	hildon_appview_set_title(appview, _("mcen_me_folder_inbox"));

	/*Connect appropriate signal handlers to all the required widgets */
	ui_connect_main_view_widget_signals(email_main_view);
	ui_connect_main_view_menu_signals(email_main_view);
	ui_connect_main_view_toolbar_signals(email_main_view);
	ui_connect_message_view_popup_menu_signals(email_main_view);
	ui_connect_folder_view_popup_menu_signals(email_main_view);

	/*Store all the menus that are to be hidden in marking mode */
	ui_store_marking_mode_widgets(email_main_view);

	/*Store main view data in global list with a dummy msg-id */
	open_window->msgid = g_strdup(MAIN_VIEW_ID);
	open_window->oldmsgid = NULL;
	open_window->window_ptr = (gpointer) appview;
	open_window->msgNum = email_main_view->tags->window_count;
	open_window->iseditor = FALSE;
	email_main_view->open_windows = g_slist_append(email_main_view->open_windows, open_window);

	/*Update Inbox specific menus, toolbar items */
	ui_update_folder_specific_widgets(INBOX_MODEL_NUM);

	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(app), WINDOW_NAME, g_strdup(APP_NAME),
			       (GDestroyNotify) g_free);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return appview;
}

/*Callback functions called by underlying engine*/

/**
This is a callback function called by the engine process to initmate any missing
params in configuration settings or in data passed by the UI process.

@param	param_number	Integer value of the missing parameter
@return void
*/
void ui_param_value_missing_cb(ParamMissing param_number)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	if (param_number != 0) {
		ui_set_msg_cb(param_number);
	} else {
		ULOG_DEBUG_F("param_number is 0!\n");
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a callback function called by the engine process to intimate percentage of
send/receive operation currently executed.

@param	percent 	Integer value of percentage
@return void
*/
void ui_set_percentage_cb(gint percent)
{
	gfloat fraction = percent * 0.01;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	ULOG_DEBUG_F("Percent = %1.2f\n", fraction);

	/*Update the progress bar */
	if (email_main_view->tags->progress_bar_available == TRUE) {
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR
					      (email_main_view->tools->progressbar), fraction);
	} else {
		ULOG_DEBUG_F("Progress bar unavailable!\n");
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function which is called when ever unread message is added to
a folder. This will increament the unread unsent count if it is not populated yet.
@param model_num Model number of the folder.
@return void.
*/
void ui_add_unread_count_to_folder(gint model_num)
{
	GtkTreeModel *folder_model = NULL;
	GtkTreeIter parent_iter;
	gboolean retval = FALSE;
	gint model_Seq_Num = -1, msg_count = -1;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	folder_model = email_main_view->folder_manager->folder_view_model;
	retval = gtk_tree_model_get_iter_first(folder_model, &parent_iter);
	CHECK_PARAM((retval == FALSE), "Storage model empty for folder view!", return);

	do {
		gtk_tree_model_get(folder_model, &parent_iter, COL_MODEL_SEQ_NUM,
				   &model_Seq_Num, COL_UNREAD_UNSENT_COUNT, &msg_count, -1);
		if (model_num == model_Seq_Num) {
			/* folder already populated */
			if (msg_count == -1) {
				return;
			}
			/* Folder is not populated so increament the unread count */
			else {
				msg_count = msg_count + 1;
				gtk_tree_store_set(GTK_TREE_STORE(folder_model), &parent_iter,
						   COL_UNREAD_UNSENT_COUNT, msg_count, -1);
				return;
			}
		}
	}
	while (gtk_tree_model_iter_next(folder_model, &parent_iter));

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
  This is helper function is used to check where a given folder is populated
  earlier or not.if not it returns the folder path.
  @param model_num Model number of the folder.
  @return gchar a valid folder path if the folder has to be populated else NULL..
*/
static gchar *ui_get_is_folder_populated(gint model_num)
{
	GtkTreeModel *folder_model = NULL;
	GtkTreeIter parent_iter;
	gboolean retval = FALSE, folder_populated = FALSE;
	gchar *folder_path = NULL, *folder_name = NULL, *parent_folder = NULL;
	gint model_Seq_Num = -1;


	folder_model = email_main_view->folder_manager->folder_view_model;
	retval = gtk_tree_model_get_iter_first(folder_model, &parent_iter);
	CHECK_PARAM((retval == FALSE), "Storage model empty for folder view!", return NULL);

	do {
		gtk_tree_model_get(folder_model, &parent_iter, COL_MODEL_SEQ_NUM,
				   &model_Seq_Num, COL_FOLDER_POPULATED, &folder_populated, -1);
		if (model_num == model_Seq_Num && !folder_populated) {
			gtk_tree_model_get(folder_model, &parent_iter,
					   COL_FOLDER_NAME, &folder_name,
					   COL_PARENT_NAME, &parent_folder, -1);
			if (!parent_folder) {
				folder_path = g_strdup(folder_name);
			} else {
				folder_path = g_strdup_printf("%s/%s", parent_folder, folder_name);
			}
			g_free(parent_folder);
			g_free(folder_name);

			return folder_path;
		}
	}
	while (gtk_tree_model_iter_next(folder_model, &parent_iter));

	return NULL;
}

/**
This is a callback function called by the engine process whenever a message
is added or removed from Outbox/Draft folder. Memory created for MsgHeader
object by the underlying Dbus layer has to be freed here.

@param	msgheader	Pointer to structure containing message header info
@param	add		Flag to indicate whether to add or remove message
@return void
*/
void ui_update_folder_view_cb(MsgHeader * msgheader, gboolean add, gboolean free_mem)
{
	gchar *parent_folder = NULL, *folder_name = NULL, *folder_path = NULL;
	gint model_num = -1;
	guint msgid_hash = 0;
	GtkTreeModel *model = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (msgheader == NULL || msgheader->msgid == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Get the folder name from msgid */
	if (ui_get_msg_folder_names(msgheader->msgid, &parent_folder, &folder_name)
	    == FALSE) {
		ui_free_msg_header(msgheader);
		ULOG_DEBUG_F("Unable to resolve folder from msg-id\n");
		return;
	}

	if (parent_folder == NULL) {
		folder_path = g_strdup(folder_name);
	} else {
		folder_path = g_strdup_printf("%s/%s", parent_folder, folder_name);
	}
	model_num = ui_get_folder_model_number(folder_name, parent_folder);
	email_main_view->message_handler->is_message_present = FALSE;
	/*Add row to Outbox if needed */
	if (add) {
		/*Check if header is already present in folder. If so, ignore it. */
		model = ui_get_unsorted_folder_model(model_num);
		msgid_hash = g_str_hash(msgheader->msgid);
		gtk_tree_model_foreach(model, ui_is_header_already_present, &msgid_hash);
		if (email_main_view->message_handler->is_message_present == TRUE) {
			email_main_view->message_handler->is_message_present = FALSE;
			ULOG_DEBUG_F("Message already present in Folder!\n");
		} else {
			msgheader->msgtag.fullydownloaded = TRUE;
			msgheader->msgtag.bodydownloaded = TRUE;
			msgheader->msgtag.remote_copy = FALSE;
			if (ui_add_listview_header(msgheader, model_num, TRUE) == FALSE) {
				ULOG_DEBUG_F("Header info is not valid!\n");
			} else {
				ULOG_DEBUG_F("Header added successfully to Outbox!\n");
			}
			if ((msgheader->flags.perm_flags & READ_STATUS_BIT) == READ_STATUS_BIT) {
				ui_add_unread_count_to_folder(model_num);
			}
		}
	} else {
		ui_remove_listview_header(msgheader->msgid, model_num);
	}

	/*Update view title, folder menu items, tool buttons if required */
	if (email_main_view->folder_manager->currently_displayed_model == model_num) {
		ui_update_folder_specific_widgets(model_num);
		ui_set_folderview_title(folder_path);
		/*Invoke marking mode if necessary */
		if (email_main_view->tags->marking_mode == TRUE) {
			ULOG_DEBUG_F("Invoking marking mode\n");
			ui_invoke_marking_mode(TRUE);
		}
	}
	/* Add unread unsent message count to folder in folder list view. */
	if (model_num != SENT_MODEL_NUM) {
		ui_add_folder_name_with_unread_unsent_mails(model_num);
	}

	if (free_mem) {
		ui_free_msg_header(msgheader);
	}
	g_free(parent_folder);
	g_free(folder_name);
	g_free(folder_path);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a callback function called by the engine process to delete a bunch
of messages from UI listviews. Memory created for message id list by the
underlying Dbus layer has to be freed here.

@param  msg_id_list  List of message id strings
@return void
*/
void ui_remove_messages_cb(GSList * msg_id_list)
{
	GSList *node = NULL;
	gchar *parent_folder = NULL, *folder_name = NULL, *folder_path = NULL;
	gint model_num = -1;
	gboolean current_view_modified = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM((msg_id_list == NULL), "Input param check failed", return);

	for (node = msg_id_list; node != NULL; node = node->next) {
		/*Get the folder name from msgid */
		if (ui_get_msg_folder_names(node->data, &parent_folder, &folder_name)
		    == FALSE) {
			ULOG_DEBUG_F("Unable to resolve folder from msg-id\n");
			continue;
		}

		model_num = ui_get_folder_model_number(folder_name, parent_folder);
		ui_remove_listview_header(node->data, model_num);
		if (email_main_view->folder_manager->currently_displayed_model == model_num) {
			current_view_modified = TRUE;
		}

		g_free(parent_folder);
		g_free(folder_name);
	}

	/*Update view title, folder menu items, tool buttons if required */
	if (current_view_modified) {
		ui_update_folder_specific_widgets(email_main_view->folder_manager->
						  currently_displayed_model);
		folder_path =
		    ui_get_folder_name(email_main_view->folder_manager->currently_displayed_model);
		ui_set_folderview_title(folder_path);
		g_free(folder_path);
		/* Add unread unsent message count to folder in folder list view. */
		if (email_main_view->folder_manager->currently_displayed_model != SENT_MODEL_NUM) {
			ui_add_folder_name_with_unread_unsent_mails(email_main_view->
								    folder_manager->
								    currently_displayed_model);
		}
	}

	g_slist_foreach(msg_id_list, (GFunc) g_free, NULL);
	g_slist_free(msg_id_list);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a callback function called by the engine process to initimate that
Send/Receive operation is complete. Send/Receive Progress bar is closed
here.

@param	void
@return void
*/
void ui_send_recv_complete_cb(void)
{
	gint errorno = 0;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Close the progress bar */
	if (email_main_view->tags->progress_bar_available == TRUE) {
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(email_main_view->tools->progressbar),
					      1.0);
		email_main_view->tags->progress_bar_available = FALSE;
		/*Update tool bar items */
		ui_update_folder_specific_tool_items(email_main_view->folder_manager->
						     currently_displayed_model);
	} else {
		ULOG_DEBUG_F("Progress bar unavailable!\n");
	}

	/*Update message specific action items */
	ui_on_messageview_focussed(NULL, NULL, NULL);

	errorno = ui_dbus_get_engine_errno();
	if (errorno == ENGINE_ERROR_CONNECT_SERVER || errorno == ENGINE_ERROR_FILE_ERROR)
		close_viewer_progress();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
}

/**
This is a callback function called by the engine process to give the message
attachment for a requested attachment.

@param	msginfo	Pointer to MsgInfo structure containing the attachemnt info. 
@return void
*/
void ui_recv_msg_attachment_cb(MsgInfo * msginfo)
{
	GSList *node = NULL;
	OpenedWindowData *open_windows = NULL;
	GtkWidget *viewer = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check if msginfo is null, if so close the 
	   progress bar & return.
	 */
	if (msginfo == NULL) {
		close_viewer_progress();
		return;
	}

	email_main_view->msginfo = msginfo;
	email_main_view->msginfo_list = NULL;

	for (node = email_main_view->open_windows;
	     node != NULL && node->data != NULL; node = node->next) {
		open_windows = (OpenedWindowData *) node->data;
		viewer = GTK_WIDGET(open_windows->window_ptr);
		if ((open_windows->oldmsgid != NULL &&
		     strcmp(open_windows->oldmsgid, msginfo->header->msgid) == 0) &&
		    (msginfo->header->attach_tag >= 0)) {
			g_signal_emit_by_name(viewer, "update_msginfo");
			return;
		} else if ((msginfo->header->attach_tag <= 0) &&
			   (abs(msginfo->header->attach_tag) == open_windows->msgNum)) {
			ULOG_DEBUG_F("recieving attachments in editor\n");
			gtk_banner_close((GtkWindow *) app);
			g_signal_emit_by_name(viewer, "update_msginfo");
			return;
		}
	}
	return;
}

/**
 This function is used to hadle the error cases occuring while downloading the amil.
 either pop the certificate pass word dialog /close the opened message.

 @param msgid is the gchar pointer to the msgid.
 @return void.
*/
static void action_for_error(gchar * msgid)
{
	gint errorNum = 0;
	HeaderInfo *header = NULL;
	gint msg_row_num = 0;
	GtkWidget *viewer = NULL;
	GSList *node = NULL;
	OpenedWindowData *open_windows = NULL;
	gchar *receiverid = NULL;
	gchar *senderid = NULL;
	gchar *password = NULL;
	gint cert_state = 0;
	guint certid = 0;

	errorNum = ui_dbus_get_engine_errno();
	/*pop the pass word dialog */
	if (errorNum == SMIME_INTERNAL_PASS_ERROR) {
		msg_row_num = ui_get_row_number_of_msg(msgid, INBOX_MODEL_NUM);
		header = g_new0(HeaderInfo, 1);
		header->folderNum = INBOX_MODEL_NUM;
		header->rowNum = msg_row_num;
		header->set_prev_settings = FALSE;
		ui_get_row_information(header);

		receiverid = get_receiversid(header->msgid);
		senderid = ui_parse_from_address(header->from);
		certid = get_private_certificate(receiverid, &cert_state);
		if (certid == 0) {
			get_user_confirmation(_("mcen_in_smime_cannot_decrypt"), NULL, NULL,
					      INFO_BANNER);
		} else {
			password = get_certificate_password(certid);
			if (password != NULL) {
				ui_dbus_recv_msg_body(msgid, FALSE, FALSE, senderid, receiverid,
						      SMIME_ENCRYPT, (header->read_status),
						      certid, password, FALSE, TRUE);
			}
		}
		g_free(receiverid);
		g_free(senderid);
		g_free(password);
		ui_freeHeaderInfo(header);
	}
	if (errorNum != SMIME_INTERNAL_PASS_ERROR || password == NULL) {
		/*close the opening of message */
		for (node = email_main_view->open_windows;
		     node != NULL && node->data != NULL; node = node->next) {
			open_windows = (OpenedWindowData *) node->data;
			viewer = GTK_WIDGET(open_windows->window_ptr);
			if (open_windows->oldmsgid != NULL &&
			    open_windows->oldmsgid != NULL
			    && strcmp(open_windows->oldmsgid, msgid) == 0) {
				g_signal_emit_by_name(viewer, "close");
			}
		}
		ui_set_msg_cb(errorNum);
	}

	return;
}

/**
This is a callback function called by the engine process to give the message
body info for a requested message. Memory created for MimeInfo object and
message-id string by the underlying Dbus layer has to be freed here.

@param  mimeinfo     Pointer to structure containing message body info
@param  msgid        Pointer to message-id string
@param  remote_copy  Flag indicating whether mail has remote copy on server
@return void
*/
void ui_recv_body_complete_cb(MimeInfo * mimeinfo, gchar * msgid, gboolean remote_copy)
{
	GtkWidget *viewer = NULL, *composer = NULL;
	GtkTreeModel *model = NULL;
	gint msg_row_num = 0;
	guint msgid_hash = 0;
	GSList *node = NULL;
	HeaderInfo *header = NULL;
	OpenedWindowData *open_windows = NULL;
	gint num_attachments = 0;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (mimeinfo == NULL || msgid == NULL) {
		if (mimeinfo == NULL && msgid != NULL) {
			action_for_error(msgid);
		}
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Check if message is present in listview. Body might have been retrieved
	   as a result of opening the message from task naviagor. If so just open
	   the message. */
	if (ui_add_msg_if_not_present(msgid) == TRUE) {
		ui_tasknav_open_msg_cb(msgid);
		ULOG_DEBUG_F("Message opened from task navigator");
		return;
	}

	/*Update the necessary tags for this message */
	model = ui_get_unsorted_folder_model(INBOX_MODEL_NUM);
	ui_update_row_tag_in_folder(INBOX_MODEL_NUM, msgid, FULLY_DOWNLOADED_STATUS, TRUE);
	ui_update_row_tag_in_folder(INBOX_MODEL_NUM, msgid, BODY_DOWNLOADED_STATUS, TRUE);
	ui_update_row_tag_in_folder(INBOX_MODEL_NUM, msgid, REMOTE_COPY_STATUS, remote_copy);

	ui_on_messageview_focussed(NULL, NULL, NULL);

	/*Get the 'action after retrieval' tag for this message */
	email_main_view->message_handler->get_action_after_retrieval = NO_ACTION_AFTER_RETRIEVAL;
	msgid_hash = g_str_hash(msgid);
	gtk_tree_model_foreach(model, ui_get_action_after_retrieval, &msgid_hash);

	/*Update attachment icon info if required */
	display_attachments(mimeinfo, NULL, &num_attachments, TRUE, FALSE);
	if (num_attachments > 0) {
		gtk_tree_model_foreach(model, ui_update_msg_attachment_icon, &msgid_hash);
	}

	msg_row_num = ui_get_row_number_of_msg(msgid, INBOX_MODEL_NUM);
	header = g_new0(HeaderInfo, 1);
	header->folderNum = INBOX_MODEL_NUM;
	header->rowNum = msg_row_num;
	header->set_prev_settings = FALSE;
	ui_get_row_information(header);
	header->original_in_reply = TRUE;

	switch (email_main_view->message_handler->get_action_after_retrieval) {
	case OPEN_AFTER_RETRIEVAL:
		{
			email_main_view->mimeinfo = mimeinfo;
			for (node = email_main_view->open_windows;
			     node != NULL && node->data != NULL; node = node->next) {
				open_windows = (OpenedWindowData *) node->data;
				viewer = GTK_WIDGET(open_windows->window_ptr);
				if (open_windows->oldmsgid != NULL
				    && strcmp(open_windows->oldmsgid, msgid) == 0) {
					g_signal_emit_by_name(viewer, "update_mimeinfo");
					ui_flush_gtk_events();
					break;
				}
			}
		}
		break;

	case REPLY_AFTER_RETRIEVAL:
		{
			/*Call the composer window */
			composer = GTK_WIDGET(ui_editor_create(header, NULL, REPLY_FROM_FOLDER));
			if (composer == NULL) {
				ULOG_DEBUG_F("Creation of Email_Viewer failed!\n");
			} else {
				hildon_app_set_appview(app, (HildonAppView *) composer);
				gtk_widget_show(composer);
				email_main_view->tags->main_view_topped = FALSE;
			}
		}
		break;

	case REPLY_ALL_AFTER_RETRIEVAL:
		{
			/*Call the composer window */
			composer =
			    GTK_WIDGET(ui_editor_create(header, NULL, REPLY_ALL_FROM_FOLDER));
			if (composer == NULL) {
				ULOG_DEBUG_F("Creation of Email_Viewer failed!\n");
			} else {
				hildon_app_set_appview(app, (HildonAppView *) composer);
				gtk_widget_show(composer);
				email_main_view->tags->main_view_topped = FALSE;
			}
		}
		break;

	case FORWARD_AFTER_RETRIEVAL:
		{
			/*Call the composer window */
			composer = GTK_WIDGET(ui_editor_create(header, NULL, FORWARD_FROM_FOLDER));
			if (composer == NULL) {
				ULOG_DEBUG_F("Creation of Email_Viewer failed!\n");
			} else {
				hildon_app_set_appview(app, (HildonAppView *) composer);
				gtk_widget_show(composer);
				email_main_view->tags->main_view_topped = FALSE;
			}
		}
		break;

	default:
		{
			email_main_view->mimeinfo = mimeinfo;
			for (node = email_main_view->open_windows;
			     node != NULL && node->data != NULL; node = node->next) {
				open_windows = (OpenedWindowData *) node->data;
				viewer = GTK_WIDGET(open_windows->window_ptr);
				if (open_windows->oldmsgid != NULL
				    && strcmp(open_windows->oldmsgid, msgid) == 0) {
					g_signal_emit_by_name(viewer, "update_mimeinfo");
					ui_flush_gtk_events();
					break;
				}
			}
		}
		break;
	}
	/*Resetting the action_after_retrieval flag after the message is retrieved */
	email_main_view->message_handler->set_action_after_retrieval = NO_ACTION_AFTER_RETRIEVAL;
	gtk_tree_model_foreach(model, ui_update_action_after_retrieval, &msgid_hash);

        /*update the seen flag for the message */
        if (header->folderNum == INBOX_MODEL_NUM &&
            header->read_status && header->protocol == PROTO_IMAP4)
                ui_dbus_update_seen_flag(header->msgid);


	/*Free allocated memory */
	ui_freeHeaderInfo(header);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a callback function called by the engine process to give a list of
message headers obtained from receive operation on an IMAP account. Memory
created for the MsgInfo list by the underlying Dbus layer has to be freed here.

@param	new_list	List of MsgInfo objects
@return void
*/
void ui_recv_msg_attachments(GSList * new_list)
{
	GSList *node = NULL;
	MsgInfo *msg_info = NULL;
	OpenedWindowData *open_windows = NULL;
	GtkWidget *viewer = NULL;

	/*Check Whether the msginfo list is NULL &
	   return if the list is  NULL */
	if (new_list == NULL) {
		close_viewer_progress();
		return;
	}

	msg_info = (MsgInfo *) (new_list->data);
	email_main_view->msginfo_list = new_list;
	email_main_view->msginfo = NULL;

	for (node = email_main_view->open_windows;
	     node != NULL && node->data != NULL; node = node->next) {
		open_windows = (OpenedWindowData *) node->data;
		viewer = GTK_WIDGET(open_windows->window_ptr);
		if (open_windows->oldmsgid != NULL &&
		    strcmp(open_windows->oldmsgid, msg_info->header->msgid) == 0
		    && msg_info->header->attach_tag >= 0) {
			msg_info = (MsgInfo *) (new_list->data);
			email_main_view->msginfo_list = new_list;
			email_main_view->msginfo = NULL;
			g_signal_emit_by_name(viewer, "update_msginfo");
			return;
		}
		/*check added if retrival is requested by editor */
		else if ((msg_info->header->attach_tag < 0) &&
			 (abs(msg_info->header->attach_tag) == open_windows->msgNum)) {
			ULOG_DEBUG_F("recieving attachments in editor\n");
			g_signal_emit_by_name(viewer, "update_msginfo");
			return;
		}

	}
	return;
}

/**
This is a callback function called by the engine process to give a list of
message headers obtained from receive operation on an IMAP account. Memory
created for the MsgInfo list by the underlying Dbus layer has to be freed here.

@param	new_list	List of MsgInfo objects
@return void
*/
void ui_imap_msg_headers_cb(GSList * new_list)
{
	GSList *node = NULL;
	MsgInfo *msg_info = NULL;
	gint totalMsgNumber = 0, currentMsgNumber = 0;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (new_list == NULL) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Unset model for the inbox view so that UI is not slowed down */
	if (email_main_view->folder_manager->currently_displayed_model == INBOX_MODEL_NUM) {
		gtk_tree_view_set_model(GTK_TREE_VIEW(email_main_view->widgets->messageListView),
					NULL);
		gtk_widget_hide(email_main_view->widgets->message_scrollwindow);
	}

	for (node = new_list; node != NULL; node = node->next) {
		msg_info = (MsgInfo *) (node->data);
		if (msg_info == NULL || msg_info->header == NULL || msg_info->header->msgid == NULL) {
			ULOG_DEBUG_F("Invalid header obtained.\n");
			continue;
		}

		/*Append the new row to Inbox */
		if (ui_add_listview_header(msg_info->header, INBOX_MODEL_NUM, FALSE) == FALSE) {
			ULOG_DEBUG_F
			    ("Adding new header with message-id '%s' to Inbox failed!\n",
			     msg_info->header->msgid);
		}
		/*Get the cur no. of mails retieved */
		if (!osso_email_gconf_get_int(EMAIL_RECV_MAILS_PROGRESS_CUR, &currentMsgNumber))
			osso_log(LOG_DEBUG,
				 "Error getting the value of no. of emails retrieved uptil now ");
		/*Get the total no. of mails to retieved */
		if (!osso_email_gconf_get_int(EMAIL_RECV_MAILS_PROGRESS_TOTAL, &totalMsgNumber))
			osso_log(LOG_DEBUG, "Error getting the value  ");
		ui_update_progressbar_numbers(currentMsgNumber, totalMsgNumber, RECEIVING);

	}

	/*Set the Inbox model to the list view, if required */
	ui_set_folder_model_to_msgview(INBOX_MODEL_NUM, TRUE);

	/*Display Inbox folder view */
	ui_display_inbox_folder_view();

	if (email_main_view->folder_manager->currently_displayed_model == INBOX_MODEL_NUM) {
		ui_update_folder_specific_widgets(INBOX_MODEL_NUM);
	}

	/*Free memory */
	for (node = new_list; node != NULL; node = node->next) {
		ui_freeMem(node->data, NULL);
	}
	g_slist_free(new_list);

	ui_flush_gtk_events();
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}
/**
 This function checks whether the given msgid is retrieved or not.
 @param msgid is the msgid to be checked if retrieved or not.
 @param model is the GtkTreeModel pointer.
 @return TRUe ->if retrieved
         FALSE ->else
*/
static gboolean
check_if_retrieved(gchar *msgid, GtkTreeModel *model)
{
        GtkTreeIter iter;
        gchar *msgId = NULL;
        gboolean body_downloaded = FALSE, in_use = FALSE;

        if (!gtk_tree_model_get_iter_first(model, &iter)) {
                                   ULOG_DEBUG_F("Storage model empty\n");
                return FALSE;
          }
     do {
            gtk_tree_model_get(model, &iter, COL_MSG_ID, &msgId, COL_BODY_DOWNLOADED, &body_downloaded,
                            COL_IN_USE_STATUS, &in_use, -1);
            if (msgId != NULL && (strncmp(msgid, msgId, MAX_STR_LEN(msgid, msgId)) == 0)){
                    g_free(msgId);
                    if(body_downloaded && in_use == FALSE)
                        return TRUE;
                    else
                        return FALSE;
              }
     }
        while (gtk_tree_model_iter_next(model, &iter));
     return FALSE;
}
/**
 This callback indiacted the retrieval of mails is done for copy operation.
 the retrieved mails are copied to clipboard & "copied" banner is shown
*/
void
ui_retrv_complete_cb(void)
{
        GSList *ret_list = NULL;
        GSList *tmp_list = NULL;
        GtkTreeModel *model = NULL;
        gchar *parent_name = NULL, *folder_name = NULL;
        gint foldernum = 0, count = 0;

        if(!email_main_view->message_handler->msgs_to_copy){
                return;
        }

        /*get the model for the msgid*/
        if (ui_get_msg_folder_names((gchar *)email_main_view->message_handler->msgs_to_copy->data,
                                &parent_name, &folder_name) == FALSE) {
             ULOG_DEBUG_F("Unable to get folder name for msgid \n");
             return;
        }
        foldernum = ui_get_folder_model_number(folder_name, parent_name);
        g_free(folder_name);
        g_free(parent_name);
        model = GTK_TREE_MODEL(g_slist_nth_data(email_main_view->sorted_model_list, foldernum));
        tmp_list = email_main_view->message_handler->msgs_to_copy;

        /*check if retrieval was unsuccessful , and append to list to be copied*/
        for(; tmp_list != NULL && tmp_list->data != NULL; tmp_list = tmp_list->next){
                if(check_if_retrieved(tmp_list->data,model)){
                        ret_list = g_slist_append(ret_list,tmp_list->data);
                }else {
                        /*if for some message is opened or if retrieval failed*/
                        count++;
                }
	
        }
        /*copy the retrieved list to clipboard*/
         folder_name = ui_get_folder_name(foldernum);
        osso_email_copy_to_clipboard(GTK_WIDGET(app), folder_name,
                                      ret_list, FALSE );
        /*show the appropriate banner*/
        if(count == 0 && email_main_view->message_handler->num_of_msgs_in_use == 0)
                gtk_confirmation_banner_with_icon_name((GtkWindow *) app,
                              _("mail_ib_copied_to_clipboard"), "qgn_note_infoprint");
        else if((count != g_slist_length(email_main_view->message_handler->msgs_to_copy)) ||
                        email_main_view->message_handler->num_of_msgs_in_use)
                gtk_infoprint(GTK_WINDOW(app),_("mcen_ib_cannot_copy_some"));
        else
                gtk_infoprint(GTK_WINDOW(app),_("mcen_ib_unable_to_copy"));

        /*free mem*/
        g_slist_free(ret_list);
        g_slist_foreach(email_main_view->message_handler->msgs_to_copy,(GFunc)g_free,NULL);
        g_slist_free(email_main_view->message_handler->msgs_to_copy);
        email_main_view->message_handler->msgs_to_copy = NULL;
        email_main_view->message_handler->num_of_msgs_in_use = 0;
}

/**
This is a callback function called by the engine process to give a list of
message headers during message handling operations. Actions to be taken can
be: add the headers, remove the headers, update download status for the
headers, or mark the messages for deletion (dustbin icon).

@param  type            Type of action
@param	new_list	List of MsgHeader objects
@return void
*/
void ui_folder_msg_headers_cb(gint type, GSList * new_list)
{
	GSList *node = NULL;
	MsgHeader *msg_header = NULL;
	GtkTreeModel *model = NULL;
	gint count = 0;
	guint msgid_hash = 0;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((new_list == NULL), "Input param check failed", return);

	model = ui_get_unsorted_folder_model(INBOX_MODEL_NUM);

	/*Do not handle external events while we are busy populating */
	ignore_ext_events = TRUE;

	for (node = new_list; node != NULL; node = node->next) {
		msg_header = (MsgHeader *) (node->data);
		if (msg_header == NULL || msg_header->msgid == NULL) {
			ULOG_DEBUG_F("Invalid header obtained.\n");
			continue;
		}

		switch (type) {
		case LIST_FOLDER_ADD:
			{
				/*Append the new row to corresponding folder model */
				ui_update_folder_view_cb(msg_header, TRUE, FALSE);
				/*Add message-id to the undo list */
				email_main_view->undo->message_id_list =
				    g_slist_append(email_main_view->undo->message_id_list,
						   g_strdup(msg_header->msgid));
				break;
			}
		case LIST_FOLDER_DEL:
			{
				/*Append the new row to corresponding folder model */
				ui_update_folder_view_cb(msg_header, FALSE, FALSE);
				break;
			}
		case LIST_FOLDER_UPDATE:
			{
				/*Update download status flags for the message */
				ui_update_msg_downloaded_cb(msg_header->msgtag.
							    fullydownloaded,
							    msg_header->msgtag.bodydownloaded,
							    -1, msg_header->msgid);
				break;
			}
		case LIST_FOLDER_BIN:
			{
				/*Update list view icon for the message to dustbin */
				msgid_hash = g_str_hash(msg_header->msgid);
				gtk_tree_model_foreach(model, ui_update_msg_dustbin_icon,
						       &msgid_hash);
				break;
			}
		default:
			{
				ULOG_DEBUG_F("Invalid list type obtained");
				break;
			}
		}

		/*Refresh the UI after adding/removing every 25 msgs */
		count++;
		if (count % 25 == 0) {
			ui_flush_gtk_events();
		}
	}

	/*We have to select the next immediate item after the last selected row
	   after completing the delete operation. */
	if (type == LIST_FOLDER_DEL &&
	    email_main_view->message_handler->msg_to_be_selected_after_del_opr) {
		ui_select_row_from_view(email_main_view->message_handler->
					msg_to_be_selected_after_del_opr, VIEW_MSG_LIST);
		g_free(email_main_view->message_handler->msg_to_be_selected_after_del_opr);
		email_main_view->message_handler->msg_to_be_selected_after_del_opr = NULL;
	}

	/*We are done populating */
	ignore_ext_events = FALSE;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to launch a Progress banner with cancel note,
when doing folder operations like Move, Cut/Copy-Paste, Delete.
Note that this has to be called with "g_idle_add".

@param  data     Pointer to integer indicating type of operation
@return boolean  FALSE so that it will run only once
*/
gboolean ui_start_folder_progress_note(gpointer data)
{
	gboolean response = FALSE;
	gint action = -1;
	gchar *progress_label = DGETTEXT_HCS("ckct_nw_pasting");
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((data == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->folder_manager == NULL),
		    "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->folder_manager->folder_progress_note != NULL),
		    "Progress note already started", return FALSE);

	action = GPOINTER_TO_INT(data);
	if (action == ACTION_DELETE) {
		progress_label = _("mcen_nw_deleting");
	}

	email_main_view->folder_manager->folder_progress_bar = gtk_progress_bar_new();
	email_main_view->folder_manager->folder_progress_note =
	    hildon_note_new_cancel_with_progress_bar(GTK_WINDOW(app), progress_label,
						     GTK_PROGRESS_BAR
						     (email_main_view->folder_manager->
						      folder_progress_bar));
	if (action == ACTION_DELETE) {
		hildon_note_set_button_text(HILDON_NOTE
					    (email_main_view->folder_manager->
					     folder_progress_note), _("mcen_bd_deleting_stop"));
	}
	response =
	    gtk_dialog_run(GTK_DIALOG(email_main_view->folder_manager->folder_progress_note));

	if (response == GTK_RESPONSE_CANCEL) {
		ULOG_DEBUG_F("Cancelling the folder operation");
		ui_dbus_folder_cancel();
	} else if (response == OPR_COMPLETE_SIGNAL) {
		ULOG_DEBUG_F("Operation completed");
	}

	gtk_widget_destroy(email_main_view->folder_manager->folder_progress_note);
	email_main_view->folder_manager->folder_progress_note = NULL;
	email_main_view->folder_manager->folder_progress_bar = NULL;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return FALSE;
}

/**
This is helper function to stop the Progress banner with cancel note,
when doing folder operations like Move, Cut/Copy-Paste.

@param  void
@return void
*/
gboolean ui_stop_folder_progress_note(void)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->folder_manager == NULL),
		    "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->folder_manager->folder_progress_note == NULL),
		    "Progress note already stopped", return FALSE);

	gtk_dialog_response(GTK_DIALOG
			    (email_main_view->folder_manager->folder_progress_note),
			    OPR_COMPLETE_SIGNAL);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return TRUE;
}

/**
This is helper function to update the Progress banner with cancel note,
when doing folder operations like Move, Cut/Copy-Paste.

@param  percent  Percentage of operation completed
@return void
*/
void ui_update_folder_progress_note(gfloat percent)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager->folder_progress_note == NULL),
		    "Progress note already stopped", return);
	CHECK_PARAM((email_main_view->folder_manager->folder_progress_bar == NULL),
		    "Progress bar unavailable", return);

	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR
				      (email_main_view->folder_manager->folder_progress_bar),
				      percent);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to fire copy operations to the engine,
for messages that are unfetched.

@param  void
@return void
*/
void ui_copy_unfetched_messages_if_required(void)
{
	gboolean userConfirmation = FALSE;
	gchar *origFolder = NULL, *destFolder = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	//Check the input params
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->message_handler == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->message_handler->unfetched_msgs_to_copy ==
		     NULL), "Nothing to copy", return);

	//Show 'Message not local' confirmation
	userConfirmation =
	    copy_unretrieved_mails_confirmation(email_main_view->message_handler->
						unfetched_msgs_to_copy);

	if (userConfirmation) {
		origFolder = ui_get_folder_name(INBOX_MODEL_NUM);
		destFolder =
		    ui_get_folder_name(email_main_view->message_handler->dest_folder_to_copy);
		ui_dbus_folder_copy_msgs(origFolder, destFolder,
					 email_main_view->message_handler->unfetched_msgs_to_copy);
		g_free(origFolder);
		g_free(destFolder);
	}

	g_slist_foreach(email_main_view->message_handler->unfetched_msgs_to_copy, (GFunc) g_free,
			NULL);
	g_slist_free(email_main_view->message_handler->unfetched_msgs_to_copy);
	email_main_view->message_handler->unfetched_msgs_to_copy = NULL;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function for showing error notes to the user during
message handling operations like delete, paste, etc.

@param  void
@return void
*/
void ui_show_note_for_failed_msgs(void)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	switch (email_main_view->message_handler->current_operation) {
	case ACTION_PASTE:
		{
			if (email_main_view->message_handler->num_of_failed_msgs > 0) {
				gtk_infoprint((GtkWindow *) app, _("mcen_ib_messages_cannot_copy"));
				email_main_view->message_handler->num_of_failed_msgs = 0;
			}
			if (email_main_view->message_handler->num_of_msgs_in_use > 0) {
				if (email_main_view->message_handler->num_of_failed_msgs != 0)
					gtk_infoprint((GtkWindow *) app,
						      _("mcen_ib_cannot_copy_some"));
				else
					gtk_infoprint((GtkWindow *) app,
						      _("mail_ib_ui_folder_copy_msgs_error"));
				email_main_view->message_handler->num_of_msgs_in_use = 0;
			}
			break;
		}
	case ACTION_DELETE:
	case ACTION_MOVE:
		{
			if (email_main_view->message_handler->num_of_msgs_in_use > 0) {
				ui_show_msgs_in_use_notification(email_main_view->message_handler->
								 current_operation,
								 email_main_view->message_handler->
								 num_of_msgs_in_use);
				email_main_view->message_handler->num_of_msgs_in_use = 0;
			}
			break;
		}
	default:
		{
			break;
		}
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a callback function called by the engine process to update the
latest status of a message after user tries to retrieve it.

@param  msg_fully_downloaded  'Fully downloaded' status of the message
@param  msg_body_downloaded   'Body downloaded' status of the message
@param  msg_remote_copy       'Remote copy' status of the message
@param  msg_id                Message-id of the message
@return void
*/
void
ui_update_msg_downloaded_cb(gint msg_fully_downloaded,
			    gint msg_body_downloaded, gint msg_remote_copy, gchar * msg_id)
{
	GtkTreeModel *model = NULL;
	gchar *current_msg_id = NULL;
	gboolean fully_downloaded = FALSE, body_downloaded = FALSE, remote_copy = FALSE;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((msg_id == NULL), "Input param check failed", return);

	model = ui_get_unsorted_folder_model(INBOX_MODEL_NUM);

	if (msg_fully_downloaded != 0) {
		fully_downloaded = TRUE;
	}
	if (msg_body_downloaded != 0) {
		body_downloaded = TRUE;
	}
	ui_update_row_tag_in_folder(INBOX_MODEL_NUM, msg_id, FULLY_DOWNLOADED_STATUS,
				    fully_downloaded);
	ui_update_row_tag_in_folder(INBOX_MODEL_NUM, msg_id, BODY_DOWNLOADED_STATUS,
				    body_downloaded);
	if (msg_remote_copy != -1) {
		if (msg_remote_copy != 0) {
			remote_copy = TRUE;
		}
		ui_update_row_tag_in_folder(INBOX_MODEL_NUM, msg_id, REMOTE_COPY_STATUS,
					    remote_copy);
	}

	/* Refresh menus if this is currently selected message */
	current_msg_id = ui_get_current_message_id();
	if (ui_compare_utf8_strings(current_msg_id, msg_id) == 0) {
		ui_on_messageview_focussed(NULL, NULL, NULL);
	}
	/*show the recieving progress*/
        if(email_main_view->tags->progress_bar_available){
                ui_update_progressbar_text(RECEIVING, NULL);
        }

	g_free(current_msg_id);
	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a callback function called by the engine process when IMAP
synchronization error occurs. Error note is shown to the user and
send-receive operation is done on the specified account.

@param  account    Name of the account
@param  show_note  Flag indicating whether to show error note
@return void
*/
void ui_handle_sync_error_cb(gchar * account, gboolean show_note)
{
	gchar *infonote = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((account == NULL), "Input param check failed", return);

	/*IMAP Synchronization error occured while processing mails. */
	if (show_note) {
		infonote = g_strdup_printf(_("mail_ib_imap_server_sync_error"), account);
		get_user_confirmation(infonote, NULL, NULL, INFO_BANNER);
		g_free(infonote);
	}

	/*Refresh Inbox */
	ui_populate_inbox();

	/*Perform Send-Receive operation on account */
	ui_on_send_receive_account_activate(NULL, account);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a callback function called by the e-mail manager process to open
an editor window during 'Send as' & 'mail to' operations. Memory created
for MsgInfo object and msg_body string by the underlying Dbus layer has
to be freed here.

@param  msg_info        Pointer to MsgInfo structure
@param  msg_body        Pointer to message body string
@return void
*/
void ui_open_editor_in_send_as_mail_to_mode_cb(MsgInfo * msg_info, gchar * msg_body)
{
	GtkWidget *composer = NULL;
	HeaderInfo *header = NULL;
	OpenedWindowData *open_window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Top Email application first */
	gtk_window_present(GTK_WINDOW(app));

	/*Check if modal dialogs are open. If so do nothing */
	if (gtk_window_get_transient_for(GTK_WINDOW(ui_get_top_level_window())) != NULL) {
		ULOG_DEBUG_F("Modal dialog is open. Topping email and returning\n");
		/*Register current view with HildonApp again */
		if (email_main_view->open_windows != NULL) {
			open_window = (OpenedWindowData *) email_main_view->open_windows->data;
			hildon_app_unregister_view(app, HILDON_APPVIEW(open_window->window_ptr));
			hildon_app_register_view(app, HILDON_APPVIEW(open_window->window_ptr));
		}
		ui_freeMem(msg_info, msg_body);
		return;
	}

	header = g_new0(HeaderInfo, 1);
	header->msgid = NULL;
	header->to = NULL;
	header->cc = NULL;
	header->bcc = NULL;
	header->from = NULL;
	header->subject = NULL;
	header->mimeInfo = NULL;
	header->msgInfo = msg_info;

	/*Call the composer window */
	composer = GTK_WIDGET(ui_editor_create(header, msg_body, SEND_AS));
	if (composer == NULL) {
		ULOG_DEBUG_F("Creation of Email_Editor failed!\n");
	} else {
		hildon_app_set_appview(app, HILDON_APPVIEW(composer));
		gtk_widget_show(composer);
		email_main_view->tags->main_view_topped = FALSE;
	}

	/*Free allocated memory */
	ui_freeMem(msg_info, msg_body);
	g_free(header);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function to launch a new composer window. To be used as a
GSourceFunc to be called when main loop is idle.

@param  void
@return boolean  FALSE so that it will run only once
*/
static gboolean ui_launch_new_composer_window(void)
{
	ui_on_new_email_activate(NULL, NULL);
	return FALSE;
}

/**
This is a callback function called by the e-mail manager process to open
an Inbox message upon request from TaskNavigator. It also is used to open
a new compose window. Memory created for msgid string by the underlying
Dbus layer has to be freed here.

@param  msgid  Pointer to the message id string
@return void
*/
void ui_tasknav_open_msg_cb(gchar * msgid)
{
	gint msg_row_num = -1, model_num = -1;
	MimeInfo *mime_info = NULL;
	gchar *parent_folder = NULL, *folder_name = NULL, *folder_path = NULL;
	OpenedWindowData *open_window = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Top Email application first */
	gtk_window_present(GTK_WINDOW(app));

	/*Check if modal dialogs are open. If so do nothing */
	if (gtk_window_get_transient_for(GTK_WINDOW(ui_get_top_level_window())) != NULL) {
		ULOG_DEBUG_F("Modal dialog is open. Topping email and returning\n");
		/*Register current view with HildonApp again */
		if (email_main_view->open_windows != NULL) {
			open_window = (OpenedWindowData *) email_main_view->open_windows->data;
			hildon_app_unregister_view(app, HILDON_APPVIEW(open_window->window_ptr));
			hildon_app_register_view(app, HILDON_APPVIEW(open_window->window_ptr));
		}
		return;
	}

	if (msgid == NULL) {
		g_idle_add((GSourceFunc) ui_launch_new_composer_window, NULL);
		ULOG_DEBUG_F("New compose window launched!\n");
		return;
	}

	if (ui_get_msg_folder_names(msgid, &parent_folder, &folder_name) == FALSE) {
		ULOG_DEBUG_F("Unable to resolve folder from msg-id\n");
		return;
	}

	model_num = ui_get_folder_model_number(folder_name, parent_folder);
	g_free(parent_folder);
	g_free(folder_name);

	/*populate the folder is not already populated */
	if (email_main_view->tags->oom_signal_received == FALSE
	    && model_num != INBOX_MODEL_NUM
	    && model_num != OUTBOX_MODEL_NUM && !ignore_ext_events) {
		folder_path = ui_get_is_folder_populated(model_num);
		if (folder_path) {
			ui_populate_folder(folder_path, model_num);
			ui_set_is_folder_populated(model_num, TRUE);
			g_free(folder_path);
		}
	}

	msg_row_num = ui_get_row_number_of_msg(msgid, model_num);
	if (msg_row_num == -1) {
		ULOG_DEBUG_F("Msg not available. Trying to download the message.");
		mime_info =
		    ui_dbus_recv_msg_body(msgid, FALSE, FALSE, NULL, NULL, SMIME_NONE,
					  TRUE, 0, NULL, FALSE, TRUE);
		if (mime_info != NULL) {
			if (ui_add_msg_if_not_present(msgid) == TRUE) {
				msg_row_num = ui_get_row_number_of_msg(msgid, model_num);
			}
			ui_mimeinfo_free(mime_info);
		} else {
			ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
			return;
		}
	}

	if (msg_row_num == -1) {
		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
		ULOG_DEBUG_F("Unable to find message with this ID in Inbox\n");
	} else {
		ui_open_message(msg_row_num, model_num, FALSE);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to update view sorting order when email
views are switched by task navigator. Memory allocated for msgid string
will be freed here.

@param  msgid  Pointer to the message id string
@return void
*/
void ui_tasknav_top_msg_cb(gchar * msgid)
{
	GSList *node = NULL;
	OpenedWindowData *open_window = NULL;
	gboolean window_found = FALSE;

	ULOG_DEBUG_F("ui_tasknav_top_msg_cb called with data: %s\n", msgid);

	if (msgid == NULL || strlen(msgid) == 0) {
		if (email_main_view->tags->main_view_hidden == TRUE) {
			open_window = g_new0(OpenedWindowData, 1);
			open_window->msgid = g_strdup(MAIN_VIEW_ID);
			open_window->oldmsgid = NULL;
			open_window->window_ptr = (gpointer) appview;
			open_window->msgNum = 0;
			open_window->iseditor = FALSE;
			email_main_view->open_windows =
			    g_slist_append(email_main_view->open_windows, open_window);

			hildon_app_register_view(app, appview);
		}
		email_main_view->tags->main_view_hidden = FALSE;
		email_main_view->tags->main_view_topped = TRUE;
		msgid = g_strdup(MAIN_VIEW_ID);
	} else {
		email_main_view->tags->main_view_topped = FALSE;
	}

	for (node = email_main_view->open_windows; node != NULL; node = node->next) {
		open_window = (OpenedWindowData *) node->data;
		if (open_window->msgid == NULL || open_window->window_ptr == NULL) {
			ULOG_DEBUG_F("Invalid data in open windows list");
		} else
		    if (strncmp(open_window->msgid, msgid, MAX_STR_LEN(open_window->msgid, msgid))
			== 0) {
			hildon_app_set_appview(app, HILDON_APPVIEW(open_window->window_ptr));
			email_main_view->open_windows =
			    g_slist_remove_link(email_main_view->open_windows, node);
			email_main_view->open_windows =
			    g_slist_concat(node, email_main_view->open_windows);
			ULOG_DEBUG_F("Message with msgid = '%s' topped\n", msgid);
			window_found = TRUE;

			/*set the fullscreen property */
			hildon_appview_set_fullscreen(HILDON_APPVIEW
						      (open_window->window_ptr),
						      email_main_view->tags->full_screen_mode);
			/*signal to toggle the toolbar status once the fullscreen is toggled */
			if (email_main_view->tags->main_view_topped == FALSE) {
				g_signal_emit_by_name(HILDON_APPVIEW(open_window->window_ptr),
						      "toggle_fullscreen");
			} else {
				ui_on_full_screen_activate(NULL, NULL);
			}
			break;
		}
	}

	if (window_found == FALSE) {
		ULOG_DEBUG_F("No message found with msgid = '%s'!\n", msgid);
	}

	g_free(msgid);
	return;
}

/**
This is a callback function called by the e-mail manager process to top
Email UI's main view and display Inbox folder view.

@param  void
@return void
*/
void ui_tasknav_show_inbox_cb(void)
{
	GtkTreeModel *model = NULL;
	GtkTreeIter first_iter;
	GtkTreeSelection *selection = NULL;
	OpenedWindowData *open_window = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->widgets->folderView == NULL), "Input param check failed",
		    return);

	/*Top email application */
	gtk_window_present(GTK_WINDOW(app));

	/*Check if modal dialogs are open. If so do nothing */
	if (gtk_window_get_transient_for(GTK_WINDOW(ui_get_top_level_window())) != NULL) {
		ULOG_DEBUG_F("Modal dialog is open. Topping email and returning\n");
		/*Register current view with HildonApp again */
		if (email_main_view->open_windows != NULL) {
			open_window = (OpenedWindowData *) email_main_view->open_windows->data;
			hildon_app_unregister_view(app, HILDON_APPVIEW(open_window->window_ptr));
			hildon_app_register_view(app, HILDON_APPVIEW(open_window->window_ptr));
		}
		return;
	}

	/*Top main UI first */
	ui_tasknav_top_msg_cb(NULL);
	hildon_app_notify_view_changed(app, appview);

	/*Display Inbox folderview by selecting first row in folder tree view */
	selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	gtk_tree_model_get_iter_first(model, &first_iter);
	gtk_tree_selection_select_iter(selection, &first_iter);
	gtk_widget_grab_focus(email_main_view->widgets->folderView);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a callback function called by the e-mail manager process to notify
that OOM signal is received.

@param  void
@return void
*/
void ui_tasknav_oom_signal_received(void)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	osso_log(LOG_INFO, "oom signal recieved");

	email_main_view->tags->oom_signal_received = TRUE;
	ui_store_marked_msgs();
	ui_store_current_state_info();
	ui_close_all_windows_cb();
	gtk_widget_destroy(GTK_WIDGET(appview));

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
 This is used to open the mail which has caused the 
 send error.
 @param msgid is the gchar pointer to the messageid which has to be opened.

 @return void.
*/
static void ui_open_failed_outbox_mail(const gchar * msgid)
{
	gint msg_row_num = 0;
	HeaderInfo *header = NULL;
	GtkWidget *composer = NULL;

	ULOG_DEBUG_L("[%s] : starts here :", __FUNCTION__);
	CHECK_PARAM(msgid == NULL, "Input param check failed", return);

	msg_row_num = ui_get_row_number_of_msg(msgid, OUTBOX_MODEL_NUM);
	if (msg_row_num == -1) {
		ULOG_INFO_F("Outbox does not have a message with msgid %s", msgid);
		return;
	}
	header = g_new0(HeaderInfo, 1);
	header->folderNum = OUTBOX_MODEL_NUM;
	header->rowNum = msg_row_num;
	header->set_prev_settings = FALSE;

	ui_get_row_information(header);
	/*Call the composer window */
	composer = GTK_WIDGET(ui_editor_create(header, NULL, FAILED_MAIL));
	if (composer == NULL) {
		ULOG_DEBUG_F("Creation of Email Editor failed!\n");
	} else {
		hildon_app_set_appview(app, (HildonAppView *) composer);
		gtk_widget_show(composer);
		email_main_view->tags->main_view_topped = FALSE;
	}
	ui_freeHeaderInfo(header);

	ULOG_DEBUG_L("[%s] : ends here :", __FUNCTION__);
	return;
}

static void close_viewer_progress(void)
{
	GSList *tempNode = NULL;
	OpenedWindowData *open_windows = NULL;
	GtkWidget *viewer = NULL;
	for (tempNode = email_main_view->open_windows; tempNode != NULL; tempNode = tempNode->next) {
		open_windows = (OpenedWindowData *) (tempNode->data);
		if ((open_windows->iseditor == FALSE
		     && strcmp(open_windows->msgid, MAIN_VIEW_ID) != 0)) {
			viewer = (GtkWidget *) open_windows->window_ptr;
			g_signal_emit_by_name(viewer, "cancel_progress");
		}
	}
	return;
}


/**
This function is used to cloe the editor after saving.
@param msgid is the msgid corresponding to the editor that has to be close.
@return void.
*/
static void close_open_view(const gchar * msgid, gboolean is_editor)
{

	GSList *tempNode = NULL;
	OpenedWindowData *open_windows = NULL;
	GtkWidget *editor = NULL;

	gtk_banner_close((GtkWindow *) app);
	if (is_editor == TRUE && ui_get_oom_signal_status() == TRUE) {
		return;
	}


	for (tempNode = email_main_view->open_windows; tempNode != NULL; tempNode = tempNode->next) {
		open_windows = (OpenedWindowData *) (tempNode->data);
		if ((is_editor && strcmp(open_windows->msgid, msgid) == 0) ||
		    (is_editor == FALSE && open_windows->oldmsgid != NULL &&
		     strcmp(open_windows->oldmsgid, msgid) == 0)) {
			editor = (GtkWidget *) open_windows->window_ptr;
			g_signal_emit_by_name(editor, "close");
			break;
		}
	}
	return;
}

/**
This is a callback function called by the engine process to initimate any
error that occured during its processing.

@param  msg_number  Error number
@param  info        String giving information abt the account/server for
                    which the error occured.
@return void
*/
void ui_set_error_msg_cb(gint msg_number, const gchar * error_info1, const gchar * error_info2)
{
	gchar *infonote = NULL;
	gchar *msg_subject = NULL;
	gchar *folder_name = NULL, *parent_name = NULL;
	const gchar *folder_display_name = NULL;
	gint foldernum = DRAFT_MODEL_NUM, foldernum1 = DRAFT_MODEL_NUM;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	gtk_banner_close((GtkWindow *) app);
	ui_flush_gtk_events();

	switch (msg_number) {
	case SMTP_SESSION_CONNECT_ERROR:
	case SMTP_SERVER_INVALID:
		infonote = g_strdup_printf(_("emev_ib_ui_smtp_server_invalid"), error_info1);
		break;

	case SMTP_AUTHTYPE_INVALID:
	case SMTP_AUTHENTICATION_FAIL_ERROR:
		infonote =
		    g_strdup_printf(_("emev_ni_ui_smtp_authentication_fail_error"), error_info1);
		break;

	case SMTP_USERID_INVALID:
		infonote =
		    g_strdup_printf(_("emev_ni_ui_smtp_userid_invalid"), error_info2, error_info1);
		break;

	case SMTP_PASSWD_INVALID:
		infonote =
		    g_strdup_printf(_("emev_ni_ui_smtp_passwd_invalid"), error_info2, error_info1);
		break;

	case SMTP_RCPT_LIST_INVALID:
		infonote = g_strdup(_("mcen_ib_not_valid_email"));
		break;

	case SMTP_SAVE_COMPLETE:
		if (email_main_view->tags->application_closure == TRUE) {
			break;
		}
		ULOG_DEBUG_F("*********save call back has come**");
		gtk_banner_close((GtkWindow *) app);
		/*get the folder num from the folder name */
		if (ui_get_msg_folder_names(error_info1, &parent_name, &folder_name) == FALSE) {
			ULOG_DEBUG_F("Unable to get folder name for msgid \n");
			break;
		}
		foldernum = ui_get_folder_model_number(folder_name, parent_name);
		folder_display_name = ui_get_folder_display_name(foldernum, folder_name);
		close_open_view(error_info1, TRUE);
		/*Refresh the current view */
		ui_get_refresh_folder(foldernum);
		g_free(parent_name);
		g_free(folder_name);
		if (foldernum == OUTBOX_MODEL_NUM) {
			if (error_info2 == NULL) {
				ULOG_DEBUG("engine is busy message sent to outbox");
				return;
			}
			if (ui_get_msg_folder_names(error_info2, &parent_name, &folder_name) ==
			    FALSE) {
				ULOG_DEBUG_F("Unable to get folder name for msgid ");
				break;
			}
			foldernum1 = ui_get_folder_model_number(folder_name, parent_name);
			g_free(parent_name);
			g_free(folder_name);
			if (foldernum != OUTBOX_MODEL_NUM) {
				ULOG_DEBUG("engine is busy message sent to outbox");
				return;
			}
		}
		infonote = g_strdup_printf(_("mail_va_saved_to_drafts"), folder_display_name, NULL);
		break;
	case POP3_MSG_CONNECT_ERROR:
		infonote = g_strdup_printf(_("emev_ni_ui_pop3_msg_connect_error"), error_info1);
		break;

	case IMAP_CONNECT_SERVER_ERROR:
	case IMAP_AUTHFAIL:
		infonote = g_strdup_printf(_("emev_ni_ui_imap_connect_server_error"), error_info1);
		break;

	case POP3_MSG_AUTH_ERROR:
		infonote =
		    g_strdup_printf(_("emev_ib_ui_pop3_msg_auth_error"), error_info1, error_info2);
		break;

	case IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER:
	case IMAP_MSG_FETCH_ERROR:
		msg_subject = ui_get_msg_subject(error_info1, INBOX_MODEL_NUM);
		infonote =
		    g_strdup_printf(_("emev_ni_ui_imap_message_not_available_in_server"),
				    msg_subject);
		g_free(msg_subject);
		/*closing the viewer, if open on error */
		close_open_view(error_info1, FALSE);
		break;

	case POP3_MSG_DELETE_ERROR:
		infonote = g_strdup(_("emev_ni_ui_pop3_msg_recv_error"));
		break;

	default:
		ULOG_DEBUG_L("[%s] : No message number found:\n", __FUNCTION__);
		break;
	}


	/*Take additional action if needed */
	switch (msg_number) {
	case IMAP_CONNECT_SERVER_ERROR:
	case IMAP_AUTHFAIL:
	case SMTP_SERVER_INVALID:
	case POP3_MSG_AUTH_ERROR:
		{
			if (email_main_view->account_manager->is_mailbox_opened == FALSE) {
				get_user_confirmation(infonote, NULL, NULL, INFO_BANNER);
				if (msg_number == IMAP_CONNECT_SERVER_ERROR) {
					ui_accmgmt_display_dialog_for_feilds((gchar *) error_info2,
									     ACCOUNT_DIALOG3);
				} else if (msg_number == IMAP_AUTHFAIL
					   || msg_number == POP3_MSG_AUTH_ERROR) {
					ui_accmgmt_display_dialog_for_feilds((gchar *) error_info2,
									     ACCOUNT_DIALOG2);
				} else if (msg_number == SMTP_SERVER_INVALID) {
					email_main_view->account_manager->
					    action_after_acct_creation = SEND_MSGS;
					ui_accmgmt_display_dialog_for_feilds((gchar *) error_info2,
									     SMTPSERVER_DIALOG);
				}
			} else {
				email_main_view->account_manager->action_after_acct_creation =
				    MAILBOX_ERROR;
				email_main_view->account_manager->mailbox_error_code = msg_number;
				g_free(email_main_view->account_manager->error_info1);
				g_free(email_main_view->account_manager->error_info2);
				email_main_view->account_manager->error_info1 =
				    g_strdup(error_info1);
				email_main_view->account_manager->error_info2 =
				    g_strdup(error_info2);
			}
		}
		break;
	case SMTP_RCPT_LIST_INVALID:
		get_user_confirmation(infonote, NULL, NULL, INFO_BANNER);
		/*open the editor which has a invalid address */
		ui_open_failed_outbox_mail(error_info1);
		break;
	case IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER:
	case POP3_MSG_DELETE_ERROR:
		get_user_confirmation(infonote, NULL, NULL, INFO_BANNER);
		ui_remove_listview_header(error_info1, INBOX_MODEL_NUM);
		if (email_main_view->folder_manager->currently_displayed_model == INBOX_MODEL_NUM) {
			ui_add_folder_name_with_unread_unsent_mails(INBOX_MODEL_NUM);
			ui_set_folderview_title(INBOX_DIR);
		}

		break;
	case SMTP_SAVE_COMPLETE:
		if (ui_get_oom_signal_status() == FALSE) {
			gtk_infoprint((GtkWindow *) app, infonote);
		}
		break;
	default:
		get_user_confirmation(infonote, NULL, NULL, INFO_BANNER);
		ULOG_DEBUG_L("[%s] : No message number found:\n", __FUNCTION__);
		break;
	}
	g_free(infonote);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}


/**
This is a callback function called by the engine process to initimate any
error that occured during its processing.

@param	msg_number	Error number
@return void
*/
void ui_set_msg_cb(gint msg_number)
{
	gchar *infonote = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	gtk_banner_close((GtkWindow *) app);
	ui_flush_gtk_events();
	switch (msg_number) {
	case SMTP_EMAIL_ADDRESS_NULL:
		get_user_confirmation(_("mcen_ib_not_valid_email"), NULL, NULL, INFO_BANNER);
		break;

	case SMTP_INVALID_EMAIL_ADDRESS:
		get_user_confirmation(_("mail_usr_not_available_on_server_dev"), NULL, NULL,
				      INFO_BANNER);
		break;

	case SMTP_BAD_RESPONSE:
	case SMTP_MSG_HDR_NULL:
	case SMTP_INTERNAL_ERROR:
	case SMTP_CHARSET_NULL:
	case SMTP_ACCOUNT_NULL:
	case SMTP_ATTACH_LIST_INVALID:
	case SMTP_ACCOUNT_NAME_INVALID:
	case SMTP_ATTACH_LIST_NULL:
	case SMTP_SUBJECT_IS_EMPTY:
	case SMTP_MESSAGE_NULL:
	case SMTP_MESSAGE_INFO_NULL:
	case SMTP_SEND_ERROR:
		gtk_infoprint(GTK_WINDOW(app), _("emev_ib_ui_smtp_send_error"));
		break;

	case POP3_MSG_LOCK_ERROR:
		get_user_confirmation(_("emev_ib_ui_pop3_msg_lock_error"), NULL, NULL, INFO_BANNER);
		break;

	case POP3_MSG_RECV_ERROR:
		get_user_confirmation(_("emev_ni_ui_pop3_msg_recv_error"), NULL, NULL, INFO_BANNER);
		break;

	case POP3_MSG_RECV_CANCEL:
		gtk_infoprint((GtkWindow *) app, _("emev_ib_ui_pop3_msg_recv_cancel"));
		ui_flush_gtk_events();
		break;

	case POP3_MSG_IO_ERROR:
		get_user_confirmation(_("emev_ui_pop3_msg_io_error"), NULL, NULL, INFO_BANNER);
		break;

	case POP3_MSG_EOF:
		get_user_confirmation(_("emev_ni_ui_pop3_msg_eof"), NULL, NULL, INFO_BANNER);
		break;
	case POP3_CONNECTION_FLIGHT_MODE:
	case IMAP_CONNECTION_FLIGHT_MODE:
	case SMTP_FLIGHT_MODE_ERROR:
		get_user_confirmation(DGETTEXT_FM("sfil_ib_no_connections_flightmode"), NULL, NULL,
				                                      INFO_BANNER);
		break;

	case IMAP_ERROR:
	case POP3_MSG_LIST_ERROR:
	case POP3_MSG_ID_ERROR:
	case POP3_MSG_NOSUPPORT_ERROR:
	case IMAP_INVALID_MSGID_ERROR:
	case IMAP_NO_HEADERS_DOWNLOADED:
	case SMIME_CANNOT_SAVE_CERTS_ERROR:
		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
		break;

	case POP3_MSG_SIZELIMIT_ERROR:
		get_user_confirmation(_("emev_ni_ui_pop3_msg_sizelimit_error"), NULL, NULL,
				      INFO_BANNER);
		break;

	case IMAP_MSG_SIZE_EXCEED_ERROR:

		get_user_confirmation(_("emev_ni_ui_imap_msg_size_exceed_error"), NULL, NULL,
				      INFO_BANNER);
		break;

	case IMAP_UID_FETCH_ERROR:
		get_user_confirmation(_("emev_ib_ui_imap_uid_fetch_error"), NULL, NULL,
				      INFO_BANNER);
		break;

	case IMAP_INBOX_SELECT_ERROR:
		get_user_confirmation(_("emev_ui_imap_inbox_select_error"), NULL, NULL,
				      INFO_BANNER);
		break;


	case IMAP_FLAG_SET_ERROR:
	case IMAP_EXPUNGE_ERROR:
		gtk_infoprint((GtkWindow *) app, _("emev_ib_ui_imap_flag_set_error"));
		break;


	case IMAP_SERVER_NOT_SUPPORT_ERROR:
		get_user_confirmation(_("emev_ib_ui_imap_server_not_support_error"),
				      NULL, NULL, INFO_BANNER);
		break;

	case IMAP_EMAIL_ATTACH_NOT_AVAILABLE_ERROR:
		get_user_confirmation(_("emev_ni_attachment_not_found"), NULL, NULL, INFO_BANNER);
		break;

	case FOLDER_CREATE_ERROR:
		get_user_confirmation(_("mail_in_ui_folder_create_error"), NULL, NULL, INFO_BANNER);
		break;

	case FOLDER_DELETE_ERROR:
		get_user_confirmation(_("mail_in_ui_folder_delete_error"), NULL, NULL, INFO_BANNER);
		break;

	case FOLDER_RENAME_ERROR:
		get_user_confirmation(_("sfil_ib_renaming_not_allowed"), NULL, NULL, INFO_BANNER);
		break;

	case FOLDER_FILE_OPEN_ERROR:
		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
		break;

	case FOLDER_FILE_WRITE_ERROR:
		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
		break;

	case FOLDER_READ_ERROR:
		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
		break;

	case FOLDER_REMOVE_MSG_ERROR:
		get_user_confirmation(DGETTEXT_HCS("ckct_ib_unable_to_delete"), NULL, NULL,
				      INFO_BANNER);
		break;

	case FOLDER_REMOVE_MSGS_ERROR:
		get_user_confirmation(DGETTEXT_HCS("sfil_ib_unable_to_delete_selected_items"), NULL,
				      NULL, INFO_BANNER);
		break;

	case FOLDER_MOVE_MSG_ERROR:
		get_user_confirmation(DGETTEXT_HCS("sfil_ib_unable_to_move_selected_items"), NULL,
				      NULL, INFO_BANNER);
		break;

	case FOLDER_REMOVE_ALL_MSGS_ERROR:
		get_user_confirmation(DGETTEXT_HCS("sfil_ib_unable_to_delete_selected_items"), NULL,
				      NULL, INFO_BANNER);
		break;

	case FOLDER_REMOVE_SERVER_MSGS_ERROR:
		gtk_infoprint((GtkWindow *) app, _("emev_ib_ui_imap_flag_set_error"));
		break;

	case FOLDER_MOVE_MSGS_ERROR:
		get_user_confirmation(DGETTEXT_HCS("sfil_ib_unable_to_move_selected_items"), NULL,
				      NULL, INFO_BANNER);
		break;

	case FOLDER_INVALID_PATH_ERROR:
	case FOLDER_POPULATE_ERROR:
	case ENGINE_ERROR_INVALID_FILE_CONTENT:
		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
		break;

	case FOLDER_COPY_MSG_ERROR:
		get_user_confirmation(_("mcen_ib_unable_to_copy"), NULL, NULL, INFO_BANNER);
		break;

	case FOLDER_COPY_MSGS_ERROR:
		get_user_confirmation(_("mcen_ib_cannot_copy_some"), NULL, NULL, INFO_BANNER);
		break;

	case FOLDER_DEL_MOVE_MSG_ERROR:
	case FOLDER_DEL_COPY_MSG_ERROR:
	case FOLDER_MSG_DELETED_ERROR:
		get_user_confirmation(_("emev_ni_ui_pop3_msg_recv_error"), NULL, NULL, INFO_BANNER);
		break;


	case SSL_INVALID_SSL_METHOD:
	case SSL_LOAD_CIPHER_LIST_FAILED:
	case SSL_CIPHER_KEY_STRENGTH_NOT_SUPPORTED:
	case SSL_SERVER_CERT_NULL:
	case SSL_SERVER_CERT_INVALID:
	case SSL_SERVER_CERT_EXPIRED:
	case SSL_CIPHER_NOT_SUPPORTED:
	case SSL_INIT_FAILED:
	case SSL_INTERNAL_ERROR:
		get_user_confirmation(_("mail_ni_ssl_certificate_error"), NULL, NULL, INFO_BANNER);
		break;

	case SSL_SOCKET_CONNECTION_FAILED:
	case SSL_SERVER_NOT_SUPPORTED:
	case POP3_MSG_SSL_ERROR:
	case POP3_MSG_SOCKET_ERROR:
	case IMAP_STARTTLS_ERROR:
		get_user_confirmation(_("emev_ui_pop3_msg_socket_error"), NULL, NULL, INFO_BANNER);
		break;

	case ENGINE_ERROR_INVALID_FOLDER:
	case ENGINE_ERROR_MSG_FILE_DOES_NOT_EXIST:
		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
		ULOG_DEBUG_F("invalid folder or message file does not exits\n");
		break;

	case ENGINE_ERROR_FOLDER_MESSAGE_DEL_LIST_FAILED:
		ULOG_DEBUG_F("message del list failed \n");
		break;

	case ENGINE_ERROR_INVALID_MSGID:
		ULOG_DEBUG_F("invalid msgid \n");
		break;
	case ENGINE_ERROR_DEVICE_MEMORY_FULL:
		get_user_confirmation(DGETTEXT_KE("cerm_device_memory_full"), NULL, NULL,
				      INFO_BANNER);
		close_viewer_progress();

		break;
	case ENGINE_ERROR_NO_MEM:
	case ENGINE_ERROR_MIME_INVALID:
	case ENGINE_ERROR_FILE_ERROR:
	case ENGINE_ERROR_NO_FILE_PERM:
	case ENGINE_ERROR_NO_FOLDER_PERM:
	case ENGINE_ERROR_PARSE_FAIL:
	case FOLDER_NO_PERMS:
	case FOLDER_DOESNOT_EXIST_ERROR:
	case SMTP_FILE_WRITE_ERROR:
	case SMTP_WRITE_HEADERS_ERROR:
	case SMTP_FILE_POINTER_NULL:
	case SMTP_FILE_CLOSE_ERROR:
	case SMTP_FILE_NAME_NULL:
	case SMTP_FILE_OPEN_ERROR:
	case SMTP_COPYTO_SENT_FOLDER_ERROR:
	case SMTP_COPYTO_OUT_FOLDER_ERROR:
	case SMTP_COPYTO_DRAFT_FOLDER_ERROR:
	case ENGINE_ERROR_FOLDER_INIT:
		get_user_confirmation(_("emev_ni_internal_error"), NULL, NULL, INFO_BANNER);
		break;

	case ENGINE_ERROR_FILE_STAT_FAILED:
		ULOG_DEBUG_F("stat failed \n");
		break;

	case ENGINE_ERROR_NULL_SUB_MIME:
		ULOG_DEBUG_F("invalid RFC 822 mime information \n");
		break;

	case ENGINE_ERROR_NULL_PART_INFO:
		ULOG_DEBUG_F("invalid mime info information \n");
		break;

	case ENGINE_ERROR_INVALID_PARAM:
		ULOG_DEBUG_F("invalid parameter \n");
		break;

		/*smime related error codes */
	case SMIME_NO_CERTIFICATE_FOR_SIGNING:
		get_user_confirmation(_("mcen_in_smime_retrieverror"), NULL, NULL, INFO_BANNER);
		break;

	case SMIME_NO_PRIVATE_KEY_FOR_SIGNING:
		get_user_confirmation(_("mcen_in_smime_key_retrieverror"), NULL, NULL, INFO_BANNER);
		break;

	case SMIME_UNABLE_TO_RETRIEVE_CERTIFICATE:
		get_user_confirmation(_("mcen_in_smime_retrieverror"), NULL, NULL, INFO_BANNER);
		break;

	case SMIME_SIGNER_CERT_NOT_AVAILABLE_ERROR:
		get_user_confirmation(_("mcen_in_smime_signer_not_available"), NULL, NULL,
				      INFO_BANNER);
		break;

	case SMIME_SIGNING_PROCESSING_ERROR:
	case SMIME_UNABLE_TO_SIGN:
		get_user_confirmation(_("mcen_in_smime_sign_error"), NULL, NULL, INFO_BANNER);
		break;

	case SMIME_ENCRYPT_PROCESSING_ERROR:
		get_user_confirmation(_("mcen_in_smime_unable_encrypt"), NULL, NULL, INFO_BANNER);
		break;

	case SMIME_NO_PRIVATE_CERT_FOR_SIGNING:
		get_user_confirmation(_("mcen_in_smime_retrieverror"), NULL, NULL, INFO_BANNER);
		break;

	case SMIME_UNKNOWN_ENCRYPTION_ALGO:
		get_user_confirmation(_("mcen_cn_smime_common_alg_error"), NULL, NULL, INFO_BANNER);
		break;

	case SMIME_NO_CERT_FOR_ENCRYPTION:
		get_user_confirmation(_("mcen_in_smime_retrieverror"), NULL, NULL, INFO_BANNER);
		break;

	case SMIME_DECRYPT_PROCESSING_ERROR:
	case SMIME_UNABLE_TO_DECRYPT:
		get_user_confirmation(_("mcen_in_smime_process_error_decrypt"), NULL, NULL,
				      INFO_BANNER);
		break;

	case SMIME_SIGNATURE_VERIF_FAILED:
		get_user_confirmation(_("mcen_in_smime_verify_failure"), NULL, NULL, INFO_BANNER);
		break;

	case SMIME_UNABLE_TO_ENCRYPT:
		get_user_confirmation(_("mcen_in_smime_unable_encrypt"), NULL, NULL, INFO_BANNER);
		break;

	case SMIME_VERIF_PROCESSING_ERROR:
		get_user_confirmation(_("mcen_in_smime_process_error_sig"), NULL, NULL,
				      INFO_BANNER);
		break;

	case SMIME_NO_CERT_FOR_DECRYPTION_ERROR:
		get_user_confirmation(_("mcen_in_smime_cannot_decrypt"), NULL, NULL, INFO_BANNER);
		break;

	}
	if (infonote != NULL) {
		get_user_confirmation(infonote, NULL, NULL, INFO_BANNER);
		g_free(infonote);
	}


	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to update the text of the progress bar.

@param	mode     Mode of current operation - CONNECTING/SENDING/RECEIVING
@param	msgid    is the gchar pointer to the message

@return void
*/
void ui_update_progressbar_text(progress_bar_mode mode, gchar * msgid)
{
	gchar *displayText = NULL, *cur_num_str = NULL, *total_num_str = NULL;
	GSList *node = NULL;
	OpenedWindowData *open_windows = NULL;
	GtkWidget *viewer = NULL;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	if (msgid != NULL) {
		email_main_view->tags->current_mode = mode;
		email_main_view->tags->msg_download_in_progress = TRUE;
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR
					  (email_main_view->tools->progressbar),
					  _("mail_me_receiving"));

		for (node = email_main_view->open_windows;
		     node != NULL && node->data != NULL; node = node->next) {
			open_windows = (OpenedWindowData *) node->data;
			viewer = GTK_WIDGET(open_windows->window_ptr);
			if (open_windows->iseditor == FALSE && open_windows->oldmsgid != NULL
			    && strcmp(open_windows->oldmsgid, msgid) == 0) {
				g_signal_emit_by_name(viewer, "update_progressbar_text");
				return;
			}
		}
		return;
	}

	/*Show the progress bar if the mode is CONNECTING */
	if (email_main_view->tags->progress_bar_available == FALSE) {
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(email_main_view->tools->progressbar),
					      0.0);
		email_main_view->tags->progress_bar_available = TRUE;
		/*Update tool bar items */
		ui_update_folder_specific_tool_items(email_main_view->folder_manager->
						     currently_displayed_model);
	}

	if (mode == CONNECTING) {
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR
					  (email_main_view->tools->progressbar),
					  _("mcen_me_connecting"));
	} else if (mode == SENDING) {
		cur_num_str = g_strdup_printf("%d", email_main_view->tags->current_msg_count);
		total_num_str = g_strdup_printf("%d", email_main_view->tags->total_msg_count);
		displayText = g_strdup_printf(_("mcen_me_sending"), cur_num_str, total_num_str);

		gtk_progress_bar_set_text(GTK_PROGRESS_BAR
					  (email_main_view->tools->progressbar), displayText);
		g_free(displayText);
		g_free(cur_num_str);
		g_free(total_num_str);
	 } else if (mode == RECEIVING) {
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR
					  (email_main_view->tools->progressbar),
					  _("mail_me_receiving"));
	}

	email_main_view->tags->current_mode = mode;

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a helper function used to update the message info in the progress bar.

@param	currentMsgNumber Current number of the message being sent/retrieved
@param	totalMsgNumber	 Total number of messages to be sent/retrieved
@return void
*/
void
ui_update_progressbar_numbers(gint currentMsgNumber, gint totalMsgNumber, progress_bar_mode mode)
{
	gchar *displayText = NULL, *cur_num_str = NULL, *total_num_str = NULL;

	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the input params */
	if (currentMsgNumber < 0 || totalMsgNumber < 0) {
		ULOG_DEBUG_F("Input param check failed!\n");
		return;
	}

	/*Store the numbers */
	email_main_view->tags->current_msg_count = currentMsgNumber;
	email_main_view->tags->total_msg_count = totalMsgNumber;

	/*Update the progress bar */
	if (email_main_view->tags->progress_bar_available == TRUE) {
		cur_num_str = g_strdup_printf("%d", currentMsgNumber);
		total_num_str = g_strdup_printf("%d", totalMsgNumber);
		if (mode == RECEIVING) {
			displayText =
			    g_strdup_printf(_("mcen_me_receiving"), cur_num_str, total_num_str);
		} else if (mode == SENDING) {
			displayText =
			    g_strdup_printf(_("mcen_me_sending"), cur_num_str, total_num_str);
		}
		if (displayText != NULL) {
			gtk_progress_bar_set_text(GTK_PROGRESS_BAR
						  (email_main_view->tools->progressbar),
						  displayText);
			g_free(displayText);
		}
		g_free(cur_num_str);
		g_free(total_num_str);
	} else {
		ULOG_DEBUG_F("Progress bar unavailable!\n");
	}
#if 0
	/*close the progress bar if recieving is complete */
	if (currentMsgNumber >= totalMsgNumber) {
		ui_send_recv_complete_cb();
	}
#endif

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This routine will update the popup menu for folder list view.
Depend on the folder on which it is activated the popup menu will be updated.

@param modelNum  holds the folder specific model number.
@return nothing.
*/
static void ui_update_folder_popmenu_specific_widgets(gint modelNum, gboolean is_subfolder)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	gtk_widget_set_sensitive(email_main_view->folder_popup->open, TRUE);
	gtk_widget_set_sensitive(email_main_view->folder_popup->new_folder, TRUE);
	gtk_widget_set_sensitive(email_main_view->folder_popup->paste, TRUE);
	gtk_widget_set_sensitive(email_main_view->folder_popup->rename_folder, TRUE);
	gtk_widget_set_sensitive(email_main_view->folder_popup->delete_msg, TRUE);
	gtk_widget_set_sensitive(email_main_view->folder_popup->search, TRUE);
	gtk_widget_set_sensitive(email_main_view->folder_popup->help, TRUE);

	gtk_widget_show(email_main_view->folder_popup->open);
	gtk_widget_show(email_main_view->folder_popup->new_folder);
	gtk_widget_show(email_main_view->folder_popup->paste);
	gtk_widget_show(email_main_view->folder_popup->rename_folder);
	gtk_widget_show(email_main_view->folder_popup->delete_msg);
	gtk_widget_show(email_main_view->folder_popup->search);
	gtk_widget_show(email_main_view->folder_popup->help);

	/*Archive folder */
	if (modelNum == email_main_view->tags->archive_model_num) {
		gtk_widget_show(email_main_view->folder_popup->open);
		gtk_widget_show(email_main_view->folder_popup->new_folder);
		gtk_widget_set_sensitive(email_main_view->folder_popup->new_folder, FALSE);
		gtk_widget_show(email_main_view->folder_popup->paste);
#if CHANGED_EMAIL
		gtk_widget_set_sensitive(email_main_view->folder_popup->rename_folder, FALSE);
		gtk_widget_set_sensitive(email_main_view->folder_popup->delete_msg, FALSE);
#else
		gtk_widget_hide(email_main_view->folder_popup->rename_folder);
		gtk_widget_hide(email_main_view->folder_popup->delete_msg);
#endif
	} else if (modelNum == -1) {
		gtk_widget_set_sensitive(email_main_view->folder_popup->open, FALSE);
		gtk_widget_set_sensitive(email_main_view->folder_popup->paste, FALSE);
		gtk_widget_set_sensitive(email_main_view->folder_popup->rename_folder, FALSE);
		gtk_widget_set_sensitive(email_main_view->folder_popup->delete_msg, FALSE);
	} else {
		switch (modelNum) {
		case OUTBOX_MODEL_NUM:	/*OUTBOX folder */
			{
				gtk_widget_show(email_main_view->folder_popup->open);
				gtk_widget_show(email_main_view->folder_popup->search);
				gtk_widget_show(email_main_view->folder_popup->help);
				gtk_widget_show(email_main_view->folder_popup->new_folder);
				gtk_widget_set_sensitive(email_main_view->folder_popup->new_folder,
							 FALSE);
#if CHANGED_EMAIL
				gtk_widget_set_sensitive(email_main_view->folder_popup->paste,
							 FALSE);
				gtk_widget_set_sensitive(email_main_view->folder_popup->
							 rename_folder, FALSE);
				gtk_widget_set_sensitive(email_main_view->folder_popup->delete_msg,
							 FALSE);
#else
				gtk_widget_hide(email_main_view->folder_popup->paste);
				gtk_widget_hide(email_main_view->folder_popup->rename_folder);
				gtk_widget_hide(email_main_view->folder_popup->delete_msg);
#endif
				break;
			}
		case SENT_MODEL_NUM:	/*SENT folder */
			{
				gtk_widget_show(email_main_view->folder_popup->open);
				gtk_widget_show(email_main_view->folder_popup->search);
				gtk_widget_show(email_main_view->folder_popup->help);
				gtk_widget_show(email_main_view->folder_popup->new_folder);
				gtk_widget_set_sensitive(email_main_view->folder_popup->new_folder,
							 FALSE);
#if CHANGED_EMAIL
				gtk_widget_set_sensitive(email_main_view->folder_popup->paste,
							 FALSE);
				gtk_widget_set_sensitive(email_main_view->folder_popup->
							 rename_folder, FALSE);
				gtk_widget_set_sensitive(email_main_view->folder_popup->delete_msg,
							 FALSE);
#else
				gtk_widget_hide(email_main_view->folder_popup->paste);
				gtk_widget_hide(email_main_view->folder_popup->rename_folder);
				gtk_widget_hide(email_main_view->folder_popup->delete_msg);
#endif

				break;
			}
		case DRAFT_MODEL_NUM:	/*DRAFTS folder */
			{
				gtk_widget_show(email_main_view->folder_popup->open);
				gtk_widget_show(email_main_view->folder_popup->new_folder);
				gtk_widget_set_sensitive(email_main_view->folder_popup->new_folder,
							 FALSE);
				gtk_widget_show(email_main_view->folder_popup->paste);
				gtk_widget_set_sensitive(email_main_view->folder_popup->paste,
							 FALSE);
				gtk_widget_show(email_main_view->folder_popup->search);
				gtk_widget_show(email_main_view->folder_popup->help);
#if CHANGED_EMAIL
				gtk_widget_set_sensitive(email_main_view->folder_popup->
							 rename_folder, FALSE);
				gtk_widget_set_sensitive(email_main_view->folder_popup->delete_msg,
							 FALSE);
#else
				gtk_widget_hide(email_main_view->folder_popup->rename_folder);
				gtk_widget_hide(email_main_view->folder_popup->delete_msg);
#endif

				break;
			}
		case INBOX_MODEL_NUM:	/*INBOX folder */
			{
				gtk_widget_show(email_main_view->folder_popup->open);
				gtk_widget_show(email_main_view->folder_popup->new_folder);
				gtk_widget_show(email_main_view->folder_popup->paste);
				gtk_widget_show(email_main_view->folder_popup->search);
				gtk_widget_show(email_main_view->folder_popup->help);
#if CHANGED_EMAIL
				gtk_widget_set_sensitive(email_main_view->folder_popup->
							 rename_folder, FALSE);
				gtk_widget_set_sensitive(email_main_view->folder_popup->delete_msg,
							 FALSE);
#else
				gtk_widget_hide(email_main_view->folder_popup->rename_folder);
				gtk_widget_hide(email_main_view->folder_popup->delete_msg);
#endif
				break;
			}
		default:	/*User Created folders. */
			{
				if (is_subfolder == TRUE) {
#if CHANGED_EMAIL
					gtk_widget_show(email_main_view->folder_popup->new_folder);
					gtk_widget_set_sensitive(email_main_view->folder_popup->
								 new_folder, FALSE);
#else
					gtk_widget_hide(email_main_view->folder_popup->new_folder);
#endif
				} else {
					gtk_widget_show(email_main_view->folder_popup->new_folder);
				}
				gtk_widget_show(email_main_view->folder_popup->open);
				gtk_widget_show(email_main_view->folder_popup->paste);
				gtk_widget_show(email_main_view->folder_popup->rename_folder);
				gtk_widget_show(email_main_view->folder_popup->delete_msg);
				gtk_widget_show(email_main_view->folder_popup->search);
				gtk_widget_show(email_main_view->folder_popup->help);
			}
		}
	}

	/*Additional handling for PASTE menu
	   Dim it if pasting is not supported */
	if (ui_is_paste_operation_allowed() == TRUE) {
		gtk_widget_set_sensitive(email_main_view->folder_popup->paste, TRUE);
	} else {
		gtk_widget_set_sensitive(email_main_view->folder_popup->paste, FALSE);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is helper function to get invoke folder popup menu. This function is called
when user right clicks or uses tap and hold functionality on folder view.
@param void.
@return TRUE If successfully popup menu is launhed.
        FALSE otherewise.
*/
static gboolean ui_popup_folder_popup_menu()
{
	GtkTreeSelection *selection = NULL;
	GtkTreePath *path = NULL;
	GtkTreeModel *folder_model = NULL;
	gint model_num = -1;
	GtkTreeIter iter;
	gchar *parent_name = NULL;
	gchar *folder_name = NULL;
	gboolean is_subfolder = FALSE;
	gboolean retVal = FALSE;

	selection =
	    gtk_tree_view_get_selection(GTK_TREE_VIEW(email_main_view->widgets->folderView));
	retVal =
	    gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(email_main_view->widgets->folderView),
					  email_main_view->tags->current_click_x_pos,
					  email_main_view->tags->current_click_y_pos, &path, NULL,
					  NULL, NULL);
	g_free(email_main_view->folder_manager->parent_folder_name);
	email_main_view->folder_manager->parent_folder_name = NULL;
	if (retVal) {
		g_signal_handlers_block_by_func(G_OBJECT(selection), (gpointer)
						ui_folderview_selection_changed_cb, NULL);
		gtk_tree_selection_select_path(selection, path);

		if (!gtk_tree_selection_get_selected(selection, &folder_model, &iter)) {
			ULOG_DEBUG_F("Nothing is selected!\n");
			return FALSE;
		}

		if (folder_model == NULL) {
			ULOG_DEBUG_F("No model set for folder view!\n");
			return FALSE;
		}

		/*Get folder data from folderListView */
		gtk_tree_model_get(folder_model, &iter, COL_MODEL_SEQ_NUM,
				   &model_num, COL_PARENT_NAME, &parent_name,
				   COL_FOLDER_NAME, &folder_name, -1);
		if (parent_name == NULL) {
			email_main_view->folder_manager->parent_folder_name = g_strdup(folder_name);
		} else {
			is_subfolder = TRUE;
		}
		g_free(parent_name);
		g_free(folder_name);
		ui_update_folder_popmenu_specific_widgets(model_num, is_subfolder);

		g_signal_handlers_unblock_by_func(G_OBJECT(selection), (gpointer)
						  ui_folderview_selection_changed_cb, NULL);
		gtk_tree_path_free(path);
	} else {
		ui_update_folder_popmenu_specific_widgets(model_num, is_subfolder);
	}
	email_main_view->folder_manager->currently_popup_model = model_num;
	return TRUE;
}


/**
This is call back routine. On every right click on the folder view this
routine will be called.

@param treeview   pointer to widget to which this routine is called.
@param event      pointer to the event it is called.
@param data       pointer to the user data.
@return           on SUCCESS return TRUE
                  on FAILURE return FALSE.
*/
gboolean
ui_on_popup_folder_button_pressed(GtkWidget * treeview, GdkEventButton * event, gpointer userdata)
{
	gboolean retVal = FALSE;

	email_main_view->tags->current_click_x_pos = event->x;
	email_main_view->tags->current_click_y_pos = event->y;

	if (event->type == GDK_BUTTON_PRESS && event->button == ON_RIGHT_CLICK_PRESSED) {
		retVal = ui_popup_folder_popup_menu();

		if (retVal == TRUE) {
			gtk_menu_popup(GTK_MENU(userdata), NULL, NULL, NULL, NULL,
				       (event != NULL) ? event->button : 0,
				       gdk_event_get_time((GdkEvent *) event));

			return TRUE;
		}
	}

	return FALSE;
}

/**
This is a callback routine which gets called whenever tap-and-hold event
occurs on folder view.
@return void
*/
static void
ui_on_tap_and_hold_folder_popup(GtkWidget * menu, gint * x, gint * y,
				gboolean * push_in, GtkWidget * widget)
{
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);
	*push_in = TRUE;
	ui_popup_folder_popup_menu();

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is a callback routine which gets called whenever tap-and-hold event
occurs on message view.
@return void
*/
static void
ui_on_tap_and_hold_message_popup(GtkWidget * menu, gint * x, gint * y,
				 gboolean * push_in, GtkWidget * widget)
{
	GtkTreeSelection *selection = NULL;
	GtkTreePath *path = NULL;
	gboolean retVal = FALSE;
	GtkRequisition req;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	CHECK_PARAM((menu == NULL), "Input param check failed", return);
	CHECK_PARAM((widget == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->tags == NULL), "Input param check failed", return);
	CHECK_PARAM((email_main_view->folder_manager == NULL), "Input param check failed", return);

	if(!GTK_IS_TREE_VIEW(widget)) {
		ULOG_DEBUG_F("Widget is not treeview! Hiding the popup menu\n");
		gtk_widget_hide(menu);
		return;
	}

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
	retVal =
	    gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget),
					  email_main_view->tags->current_click_x_pos,
					  email_main_view->tags->current_click_y_pos,
					  &path, NULL, NULL, NULL);
	if (retVal) {
		if (!gtk_tree_selection_path_is_selected(selection, path)) {
			gtk_tree_selection_select_path(selection, path);
		}
		ui_update_message_popmenu_specific_widgets(email_main_view->
							   folder_manager->
							   currently_displayed_model);
		gtk_tree_path_free(path);
	} else {
		gtk_widget_hide(menu);
	}

	if (email_main_view->tags->marking_mode == TRUE) {
		ULOG_DEBUG_F("Invoking marking mode\n");
		ui_invoke_marking_mode(TRUE);
	}

	/*Push in the menu first. And adjust the x co-ordinate if required */
	*push_in = TRUE;

	gtk_widget_size_request(menu, &req);
	if (*x + req.width > FULLSCREEN_WIDTH) {
		*x = *x - req.width;
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is call back routine. On every right click on the message view this
routine will be called.

@param treeview   pointer to widget to which this routine is called.
@param event      pointer to the event it is called.
@param data       pointer to the user data.
@return           on SUCCESS return TRUE
                  on FAILURE return FALSE.
*/
gboolean
ui_on_popup_message_button_pressed(GtkWidget * treeview, GdkEventButton * event, gpointer userdata)
{
	GtkTreeSelection *selection = NULL;
	GtkTreePath *path = NULL;
	gint model_num = -1;
	gboolean retVal = FALSE;

	CHECK_PARAM((treeview == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((event == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view == NULL), "Input param check failed", return FALSE);
	CHECK_PARAM((email_main_view->tags == NULL), "Input param check failed", return FALSE);

	email_main_view->tags->current_click_x_pos = event->x;
	email_main_view->tags->current_click_y_pos = event->y;

	if (event->type == GDK_BUTTON_PRESS && event->button == THREE) {
		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
		retVal =
		    gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(treeview), event->x,
						  event->y, &path, NULL, NULL, NULL);
		if (retVal) {
			gtk_tree_selection_select_path(selection, path);
			ui_update_message_popmenu_specific_widgets(email_main_view->
								   folder_manager->
								   currently_displayed_model);
			gtk_tree_path_free(path);
		} else {
			ui_update_message_popmenu_specific_widgets(model_num);
		}
		gtk_menu_popup(GTK_MENU(userdata), NULL, NULL, NULL, NULL,
			       (event != NULL) ? event->button : 0,
			       gdk_event_get_time((GdkEvent *) event));

		if (email_main_view->tags->marking_mode == TRUE) {
			ULOG_DEBUG_F("Invoking marking mode\n");
			ui_invoke_marking_mode(TRUE);
		}
		return TRUE;
	}

	return FALSE;
}

/**
This routine will update the popup menu for message list view.
Depend on the message on which folder it is activated the popup
menu will be updated.

@param modelNum  holds the folder specific model number.
@return nothing.
*/

void ui_update_message_popmenu_specific_widgets(gint modelNum)
{
	gint folder_num = -1;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

#if RETAINED_EMAIL
	/* Get move_to to normol position for all folders */
	gtk_menu_reorder_child((GtkMenu *) email_main_view->message_popup->
			       pop_up_menu, email_main_view->message_popup->move_to, NINE);
#endif
	gtk_widget_show(email_main_view->message_popup->open);
	gtk_widget_show(email_main_view->message_popup->reply);
	gtk_widget_show(email_main_view->message_popup->reply_to_all);
	gtk_widget_show(email_main_view->message_popup->forward);
	gtk_widget_show(email_main_view->message_popup->copy);
	gtk_widget_show(email_main_view->message_popup->paste);
	gtk_widget_show(email_main_view->message_popup->move_to);
	gtk_widget_show(email_main_view->message_popup->delete_msg);
	gtk_widget_show(email_main_view->message_popup->resend);
	gtk_widget_show(email_main_view->message_popup->cancel_sending);
	gtk_widget_show(email_main_view->message_popup->send);
	gtk_widget_show(email_main_view->message_popup->help);
	gtk_widget_set_sensitive(email_main_view->message_popup->copy, TRUE);
	gtk_widget_set_sensitive(email_main_view->message_popup->paste, TRUE);


	if (modelNum == -1) {
		folder_num = email_main_view->folder_manager->currently_displayed_model;
		gtk_widget_set_sensitive(email_main_view->message_popup->open, FALSE);
		gtk_widget_set_sensitive(email_main_view->message_popup->reply, FALSE);
		gtk_widget_set_sensitive(email_main_view->message_popup->reply_to_all, FALSE);
		gtk_widget_set_sensitive(email_main_view->message_popup->forward, FALSE);
		gtk_widget_set_sensitive(email_main_view->message_popup->copy, FALSE);
		gtk_widget_set_sensitive(email_main_view->message_popup->paste, TRUE);
		gtk_widget_set_sensitive(email_main_view->message_popup->move_to, FALSE);
		gtk_widget_set_sensitive(email_main_view->message_popup->delete_msg, FALSE);
		gtk_widget_set_sensitive(email_main_view->message_popup->resend, FALSE);
		gtk_widget_set_sensitive(email_main_view->message_popup->cancel_sending, FALSE);
		gtk_widget_set_sensitive(email_main_view->message_popup->send, FALSE);
		gtk_widget_set_sensitive(email_main_view->message_popup->help, FALSE);
	} else {
		folder_num = modelNum;
	}
	switch (folder_num) {
	case OUTBOX_MODEL_NUM:	/*OUTBOX folder */
		{
			gtk_widget_show(email_main_view->message_popup->open);
			gtk_widget_hide(email_main_view->message_popup->reply);
			gtk_widget_hide(email_main_view->message_popup->reply_to_all);
			gtk_widget_hide(email_main_view->message_popup->forward);
#if CHANGED_EMAIL
			gtk_widget_set_sensitive(email_main_view->message_popup->copy, FALSE);
			gtk_widget_set_sensitive(email_main_view->message_popup->paste, FALSE);
#else
			gtk_widget_hide(email_main_view->message_popup->copy);
			gtk_widget_hide(email_main_view->message_popup->paste);
#endif
			gtk_widget_show(email_main_view->message_popup->cancel_sending);
			gtk_widget_show(email_main_view->message_popup->send);
			gtk_widget_show(email_main_view->message_popup->move_to);
			gtk_widget_show(email_main_view->message_popup->delete_msg);
			gtk_widget_show(email_main_view->message_popup->help);
			gtk_widget_hide(email_main_view->message_popup->resend);
			ui_on_messageview_focussed(NULL, NULL, NULL);
		}
		break;
	case SENT_MODEL_NUM:	/*SENT folder */
		{
			gtk_widget_show(email_main_view->message_popup->open);
			gtk_widget_show(email_main_view->message_popup->resend);
			gtk_widget_hide(email_main_view->message_popup->reply);
			gtk_widget_hide(email_main_view->message_popup->reply_to_all);
			gtk_widget_show(email_main_view->message_popup->forward);
			gtk_widget_show(email_main_view->message_popup->copy);
#if CHANGED_EMAIL
			gtk_widget_set_sensitive(email_main_view->message_popup->paste, FALSE);
#else
			gtk_widget_hide(email_main_view->message_popup->paste);
#endif
			gtk_widget_show(email_main_view->message_popup->move_to);
			gtk_widget_show(email_main_view->message_popup->delete_msg);
			gtk_widget_show(email_main_view->message_popup->help);
			gtk_widget_hide(email_main_view->message_popup->cancel_sending);
			gtk_widget_hide(email_main_view->message_popup->send);
		}
		break;
	case DRAFT_MODEL_NUM:	/*DRAFTS folder */
		{
			gtk_widget_hide(email_main_view->message_popup->reply);
			gtk_widget_hide(email_main_view->message_popup->reply_to_all);
			gtk_widget_hide(email_main_view->message_popup->forward);
			gtk_widget_hide(email_main_view->message_popup->resend);
			gtk_widget_hide(email_main_view->message_popup->cancel_sending);
#if CHANGED_EMAIL
			gtk_widget_show(email_main_view->message_popup->send);
#else
			gtk_widget_hide(email_main_view->message_popup->send);

			/* Exceptional for drafts folder reopsitioning the move folder two 
			 * positions back. */
			gtk_menu_reorder_child((GtkMenu *) email_main_view->message_popup->
					       pop_up_menu, email_main_view->message_popup->move_to,
					       SEVEN);
#endif
			gtk_widget_show(email_main_view->message_popup->open);
			gtk_widget_show(email_main_view->message_popup->move_to);
			gtk_widget_show(email_main_view->message_popup->copy);
			gtk_widget_show(email_main_view->message_popup->paste);
			gtk_widget_show(email_main_view->message_popup->delete_msg);
			gtk_widget_show(email_main_view->message_popup->help);
		}
		break;
	case INBOX_MODEL_NUM:	/*INBOX folder */
		{
			gtk_widget_show(email_main_view->message_popup->open);
			gtk_widget_show(email_main_view->message_popup->reply);
			gtk_widget_show(email_main_view->message_popup->reply_to_all);
			gtk_widget_show(email_main_view->message_popup->forward);
			gtk_widget_show(email_main_view->message_popup->copy);
			gtk_widget_show(email_main_view->message_popup->paste);
			gtk_widget_show(email_main_view->message_popup->move_to);
			gtk_widget_show(email_main_view->message_popup->delete_msg);
			gtk_widget_show(email_main_view->message_popup->help);
			gtk_widget_hide(email_main_view->message_popup->resend);
			gtk_widget_hide(email_main_view->message_popup->cancel_sending);
			gtk_widget_hide(email_main_view->message_popup->send);
		}
		break;
	default:
		{
#if RETAINED_EMAIL
			/*Reopsitioning the move folder two positions back. */
			gtk_menu_reorder_child((GtkMenu *) email_main_view->message_popup->
					       pop_up_menu, email_main_view->message_popup->move_to,
					       SEVEN);
#endif
			gtk_widget_show(email_main_view->message_popup->open);
			gtk_widget_show(email_main_view->message_popup->reply);
			gtk_widget_show(email_main_view->message_popup->reply_to_all);
			gtk_widget_show(email_main_view->message_popup->forward);
			gtk_widget_show(email_main_view->message_popup->move_to);
			gtk_widget_show(email_main_view->message_popup->copy);
			gtk_widget_show(email_main_view->message_popup->paste);
			gtk_widget_show(email_main_view->message_popup->delete_msg);
			gtk_widget_show(email_main_view->message_popup->help);
			gtk_widget_hide(email_main_view->message_popup->resend);
			gtk_widget_hide(email_main_view->message_popup->cancel_sending);
			gtk_widget_hide(email_main_view->message_popup->send);
		}
		break;
	}

	/*Additional handling for PASTE menu
	   Dim it if pasting is not supported */
	if (ui_is_paste_operation_allowed() == TRUE) {
		gtk_widget_set_sensitive(email_main_view->message_popup->paste, TRUE);
	} else {
		gtk_widget_set_sensitive(email_main_view->message_popup->paste, FALSE);
	}

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is call back routine. This will be invoked for opening the folders
in the folder list view from popup menu.

@param widget  The widget pointer points to which this routine will be called.
@param data    pointer to the user data.
@return nothing.
*/
void ui_folder_menu_item_open_activate(GtkWidget * widget, gpointer data)
{
	GtkTreeSelection *selection = NULL;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(data));
	ui_folderview_selection_changed_cb(selection, NULL);

	return;
}

/**
This is call back routine. This will be invoked to create new folder
in the folder list view from popup menu.

@param widget  The widget pointer points to which this routine will be called.
@param data    pointer to the user data.
@return nothing.
*/
void ui_folder_menu_item_new_folder_activate(GtkWidget * widget, gpointer data)
{
	GtkTreeSelection *selection = NULL;
	GtkTreeModel *folder_model = NULL;
	GtkTreeIter iter;
	ULOG_DEBUG_L("[%s] : starts here :\n", __FUNCTION__);

	/*Check the necessary input & global params */
	CHECK_PARAM((data == NULL), "Input param check failed", return);

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(data));
	if (!gtk_tree_selection_get_selected(selection, &folder_model, &iter)) {
		ULOG_DEBUG_F("Nothing is selected!\n");
		return;
	}

	if (folder_model == NULL) {
		ULOG_DEBUG_F("No model set for folder view!\n");
		return;
	}

	/*Get folder data from folderListView */
	g_free(email_main_view->folder_manager->parent_folder_name);
	gtk_tree_model_get(folder_model, &iter, COL_FOLDER_NAME,
			   &email_main_view->folder_manager->parent_folder_name, -1);

	ui_on_new_manage_folders_dialog(widget, NULL);

	ULOG_DEBUG_L("[%s] : ends here :\n", __FUNCTION__);
	return;
}

/**
This is call back routine. This will be invoked to paste any selected item
to the folder in the folder list view from popup menu.

@param widget  The widget pointer points to which this routine will be called.
@param data    pointer to the user data.
@return nothing.
*/

void ui_folder_menu_item_paste_activate(GtkWidget * widget, gpointer data)
{
	ui_on_paste1_activate(widget, NULL);
	return;
}

/**
This is call back routine. This will be invoked to rename the folder
in the folder list view from popup menu.

@param widget  The widget pointer points to which this routine will be called.
@param data    pointer to the user data.
@return nothing.
*/
void ui_folder_menu_item_rename_folder_activate(GtkWidget * widget, gpointer data)
{
	ui_on_rename_manage_folders_dialog(widget, NULL);
	return;
}

/**
This is call back routine. This will be invoked to delete any folder
in the folder list view from popup menu.

@param widget  The widget pointer points to which this routine will be called.
@param data    pointer to the user data.
@return nothing.
*/
void ui_folder_menu_item_delete_activate(GtkWidget * widget, gpointer data)
{
	ui_on_delete_manage_folders_dialog(widget, NULL);
	return;
}

/**
This is a callback function called by email engine to initiate the sending
of any pending Outbox messages with WAITING status.

@param  void
@return void
*/
void ui_send_outbox_mails_with_wait_status_cb()
{
	GSList *msg_id_list = NULL;
	GtkTreeModel *model = NULL;
	ULOG_DEBUG_L("[%s] : starts here :", __FUNCTION__);

	model = ui_get_unsorted_folder_model(OUTBOX_MODEL_NUM);
	gtk_tree_model_foreach(model, (GtkTreeModelForeachFunc)
			       ui_get_outbox_msgids_with_wait_status, &msg_id_list);

	if (msg_id_list != NULL) {
		ULOG_DEBUG_F("Number of msgs being sent = %d", g_slist_length(msg_id_list));
		ui_dbus_send_selected_mails(OPR_SEND_OUTBOX_MAILS, msg_id_list);
		g_slist_foreach(msg_id_list, (GFunc) g_free, NULL);
		g_slist_free(msg_id_list);
	}

	ULOG_DEBUG_L("[%s] : ends here :", __FUNCTION__);
	return;
}

/*end of file */
